from typing import Literal, TypeAlias

DefIscriptDict: dict[str, int] = {
    "Scourge": 0,
    "Unknown1": 1,
    "Scourge Death": 2,
    "Scourge Explosion": 3,
    "Broodling": 4,
    "Broodling Remnants": 5,
    "Infested Terran": 6,
    "Infested Terran Explosion": 7,
    "Guardian Cocoon": 8,
    "Defiler": 9,
    "Defiler Remnants": 10,
    "Drone": 11,
    "Drone Remnants": 12,
    "Egg": 13,
    "Egg Remnants": 14,
    "Guardian": 15,
    "Guardian Birth": 16,
    "Guardian Death": 17,
    "Hydralisk": 18,
    "Hydralisk Remnants": 19,
    "Infested Kerrigan": 20,
    "Larva": 21,
    "Larva Remnants": 22,
    "Mutalisk": 23,
    "Mutalisk Death": 24,
    "Overlord": 25,
    "Overlord Death": 26,
    "Queen": 27,
    "Queen Death": 28,
    "Ultralisk": 29,
    "Ultralisk Remnants": 30,
    "Zergling": 31,
    "Zergling Remnants": 32,
    "Zerg Air Death Explosion": 33,
    "Zerg Building Explosion": 34,
    "Unknown35": 35,
    "Zerg Birth": 36,
    "Egg Spawn": 37,
    "Cerebrate": 38,
    "Infested Command Center": 39,
    "Spawning Pool": 40,
    "Evolution Chamber": 41,
    "Creep Colony": 42,
    "Hatchery": 43,
    "Hive": 44,
    "Lair": 45,
    "Sunken Colony": 46,
    "Mature Chrysalis": 47,
    "Greater Spire": 48,
    "Defiler Mound": 49,
    "Queen's Nest": 50,
    "Nydus Canal": 51,
    "Overmind With Shell": 52,
    "Overmind Remnants": 53,
    "Overmind Without Shell": 54,
    "Ultralisk Cavern": 55,
    "Extractor": 56,
    "Hydralisk Den": 57,
    "Spire": 58,
    "Spore Colony": 59,
    "Infested Command Center Overlay": 60,
    "Zerg Construction (Small)": 61,
    "Zerg Construction (Medium)": 62,
    "Zerg Building Morph": 63,
    "Zerg Construction (Large)": 64,
    "Zerg Building Spawn": 65,
    "Battlecruiser": 66,
    "Civilian": 67,
    "Dropship": 68,
    "Firebat": 69,
    "Ghost": 70,
    "Ghost Remnants": 71,
    "Ghost Death": 72,
    "Nuke Beam": 73,
    "Nuke Target Dot": 74,
    "Goliath (Base)": 75,
    "Goliath (Turret)": 76,
    "Sarah Kerrigan": 77,
    "Marine": 78,
    "Marine Remnants": 79,
    "Marine Death": 80,
    "Scanner Sweep": 81,
    "Wraith": 82,
    "Wraith Afterburners": 83,
    "SCV": 84,
    "Unknown85": 85,
    "Vulture": 86,
    "Spider Mine": 87,
    "Science Vessel (Base)": 88,
    "Science Vessel (Turret)": 89,
    "Siege Tank (Tank) Base": 90,
    "Siege Tank (Tank) Turret": 91,
    "Siege Tank (Siege) Base": 92,
    "Siege Tank (Siege) Turret": 93,
    "Academy": 94,
    "Academy Overlay": 95,
    "Barracks": 96,
    "Armory": 97,
    "Armory Overlay": 98,
    "Comsat Station": 99,
    "Comsat Connector": 100,
    "Comsat Overlay": 101,
    "Command Center": 102,
    "Command Center Overlay": 103,
    "Crashed Battlecruiser": 104,
    "Supply Depot": 105,
    "Supply Depot Overlay": 106,
    "Control Tower": 107,
    "Control Tower Connector": 108,
    "Control Tower Overlay": 109,
    "Unknown110": 110,
    "Factory": 111,
    "Factory Overlay": 112,
    "Covert Ops": 113,
    "Covert Ops Connector": 114,
    "Covert Ops Overlay": 115,
    "Ion Cannon": 116,
    "Machine Shop": 117,
    "Machine Shop Connector": 118,
    "Missile Turret (Base)": 119,
    "Missile Turret (Turret)": 120,
    "Physics Lab": 121,
    "Physics Lab Connector": 122,
    "Bunker": 123,
    "Bunker Overlay": 124,
    "Refinery": 125,
    "Science Facility": 126,
    "Science Facility Overlay": 127,
    "Nuclear Silo": 128,
    "Nuclear Silo Connector": 129,
    "Nuclear Silo Overlay": 130,
    "Nuclear Missile": 131,
    "Unknown132": 132,
    "Nuke Explosion": 133,
    "Starport": 134,
    "Starport Overlay": 135,
    "Engineering Bay": 136,
    "Engineering Bay Overlay": 137,
    "Terran Construction (Large)": 138,
    "Terran Construction (Medium)": 139,
    "Terran Construction (Small)": 140,
    "Addon Construction": 141,
    "Explosion (Small)": 142,
    "Explosion (Medium)": 143,
    "Explosion (Large)": 144,
    "Building Rubble Header": 145,
    "Arbiter": 146,
    "Archon Energy": 147,
    "Archon Being": 148,
    "Archon Swirl": 149,
    "Unknown150": 150,
    "Carrier": 151,
    "Dark Templar (Hero)": 152,
    "Dragoon": 153,
    "Dragoon Remnants": 154,
    "Interceptor": 155,
    "Probe": 156,
    "Shuttle": 157,
    "High Templar": 158,
    "Reaver": 159,
    "Scarab": 160,
    "Scout": 161,
    "Scout Engines": 162,
    "Zealot": 163,
    "Zealot Death": 164,
    "Observer": 165,
    "Templar Archives": 166,
    "Assimilator": 167,
    "Observatory": 168,
    "Unknown169": 169,
    "Citadel Of Adun": 170,
    "Forge": 171,
    "Forge Overlay": 172,
    "Gateway": 173,
    "Cybernetics Core": 174,
    "Cybrnetics Core Overlay": 175,
    "Khaydarin Crystal Formation": 176,
    "Nexus": 177,
    "Nexus Overlay": 178,
    "Photon Cannon": 179,
    "Arbiter Tribunal": 180,
    "Pylon": 181,
    "Robotics Facility": 182,
    "Shield Battery": 183,
    "Shield Battery Overlay": 184,
    "Stargate": 185,
    "Stargate Overlay": 186,
    "Stasis Cell/Prison": 187,
    "Robotics Support Bay": 188,
    "Temple": 189,
    "Fleet Beacon": 190,
    "Warp Anchor": 191,
    "Warp Flash Header": 192,
    "Warp Texture": 193,
    "Unknown194": 194,
    "Unknown195": 195,
    "Unknown196": 196,
    "Unknown197": 197,
    "Ragnasaur": 198,
    "Rhynadon": 199,
    "Bengalaas": 200,
    "Vespene Geyser": 201,
    "Vespene Geyser2": 202,
    "Vespene Geyser Shadow": 203,
    "Mineral Field Type1": 204,
    "Mineral Field Type2": 205,
    "Mineral Field Type3": 206,
    "Unknown207": 207,
    "Zerg Beacon Overlay": 208,
    "Terran Beacon Overlay": 209,
    "Protoss Beacon Overlay": 210,
    "Zerg Beacon": 211,
    "Protoss Beacon": 212,
    "Terran Beacon": 213,
    "Unknown214": 214,
    "Powerups Shadow Header": 215,
    "Flag": 216,
    "Psi Emitter": 217,
    "Data Disk": 218,
    "Crystals Shadows": 219,
    "Young Chrysalis": 220,
    "Ore Chunk": 221,
    "Ore Chunk2": 222,
    "Gas Sac": 223,
    "Gas Sac2": 224,
    "Gas Orb": 225,
    "Gas Orb2": 226,
    "Gas Tank": 227,
    "Gas Tank2": 228,
    "Archon Overlay": 229,
    "Particle Beam Hit": 230,
    "Dual Photon Blaster Hit": 231,
    "Anti-Matter Missile": 232,
    "Pulse Cannon": 233,
    "Phase Disruptor": 234,
    "STA/STS Photon Cannon Overlay": 235,
    "Psionic Storm": 236,
    "Fusion Cutter Hit": 237,
    "Gauss Rifle Hit": 238,
    "Gemini Missiles": 239,
    "Longbolt Missile": 240,
    "C-10 Canister Rifle Hit": 241,
    "Fragmentation Grenade": 242,
    "ATA/ATS Laser Battery/Burst Lasers": 243,
    "Unknown244": 244,
    "Lockdown Hit": 245,
    "Arclite Shock Cannon Hit": 246,
    "Yamato Gun": 247,
    "Yamato Gun Trail": 248,
    "Lockdown/EMP Shockwave Missile": 249,
    "Siege Tank (Tank) Turret Overlay": 250,
    "Siege Tank (Siege) Turret Overlay": 251,
    "Science Vessel Overlay": 252,
    "Hallucination Hit": 253,
    "Unknown254": 254,
    "Unknown255": 255,
    "Needle Spines Hit": 256,
    "Venom (Unused)": 257,
    "Subterranean Tentacle": 258,
    "Venom Hit (Unused)": 259,
    "Acid Spore": 260,
    "Acid Spore Hit": 261,
    "Guardian Attack Overlay": 262,
    "Unknown263": 263,
    "Glave Wurm": 264,
    "Glave Wurm Hit": 265,
    "Seeker Spores": 266,
    "Queen Spell Holder": 267,
    "Psionic Shockwave Hit": 268,
    "Glave Wurm Trail": 269,
    "Seeker Spores Overlay": 270,
    "Acid Spray": 271,
    "Unknown272": 272,
    "Longbolt/Halo/Gemini Missiles Trail": 273,
    "Burowing Dust": 274,
    "Shadow Header": 275,
    "Shield Overlay": 276,
    "Unknown277": 277,
    "Double Explosion": 278,
    "Nuclear Missile Death": 279,
    "Spider Mine Explosion": 280,
    "Vespene Geyser Smokes": 281,
    "Unknown282": 282,
    "Fragmentation Grenade Hit": 283,
    "Grenade Shot Smoke": 284,
    "Phase Disruptor/Anti-Matter Missile Hit": 285,
    "Scarab/Anti-Matter Missile Overlay": 286,
    "Scarab Hit": 287,
    "Cursor Marker": 288,
    "Circle Marker": 289,
    "Carrier Engines": 290,
    "Engines/Glow Header": 291,
    "White Circle": 292,
    "Battlecruiser Attack Overlay": 293,
    "ATA/ATS Laser Battery/Burst Lasers Hit": 294,
    "Unknown295": 295,
    "Plague Cloud": 296,
    "Plague Overlay": 297,
    "Consume": 298,
    "Dark Swarm": 299,
    "Defensive Matrix Overlay": 300,
    "Defensive Matrix Hit": 301,
    "Ensnare": 302,
    "Ensnare Overlay": 303,
    "Irradiate": 304,
    "Recall Field": 305,
    "Stasis Field Overlay": 306,
    "Stasis Field Hit": 307,
    "Recharge Shields (Large)": 308,
    "Recharge Shields (Small)": 309,
    "High Templar Glow": 310,
    "Needle Spines Overlay": 311,
    "Flamethrower": 312,
    "Gemini Missiles Explosion": 313,
    "Yamato Gun Overlay": 314,
    "Yamato Gun Hit": 315,
    "Unknown316": 316,
    "Psionic Storm Part Variant1": 317,
    "Psionic Storm Part Variant2": 318,
    "EMP Shockwave Hit (Part1)": 319,
    "EMP Shockwave Hit (Part2)": 320,
    "Hallucination Death": 321,
    "Flames (Small)": 322,
    "Unknown323": 323,
    "Bleeding (Small) Variant1": 324,
    "Bleeding (Small) Variant2": 325,
    "Flames (Large)": 326,
    "Unknown327": 327,
    "Bleeding (Large) Variant1": 328,
    "Bleeding (Large) Variant2": 329,
    "Dust Variant1": 330,
    "Dust Variant2": 331,
    "Confirm Circle": 332,
    "Psi Field Type1": 333,
    "Psi Field Type2": 334,
    "Start Location": 335,
    "Doodad Header": 336,
    "Doodad Header2": 337,
    "Space Platform Doodad": 338,
    "Space Platform Doodad2": 339,
    "Installation Doodad": 340,
    "Installation Doodad2": 341,
    "Installation Right Wall Fans": 342,
    "Installation Left Wall Fans": 343,
    "Installation Gear": 344,
    "Floor Missile Trap": 345,
    "Floor Missile Trap Turret": 346,
    "Floor Gun Trap": 347,
    "Wall Missile Trap Type1": 348,
    "Wall Missile Trap Typet2": 349,
    "Wall Flame Trap Type1": 350,
    "Wall Flame Trap Type2": 351,
    "Map Revealer": 352,
    "Lurker Egg": 353,
    "Lurker": 354,
    "Unknown355": 355,
    "Lurker Remnants": 356,
    "Devourer": 357,
    "Devourer Birth": 358,
    "Devourer Death": 359,
    "Medic": 360,
    "Medic Remnants": 361,
    "Valkyrie": 362,
    "Unknown363": 363,
    "Unknown364": 364,
    "Dark Archon Energy": 365,
    "Dark Archon Being": 366,
    "Dark Archon Swirl": 367,
    "Dark Archon Death": 368,
    "Corsair": 369,
    "Corsair Engines": 370,
    "Unknown371": 371,
    "Dark Templar (Unit)": 372,
    "Neutron Flare": 373,
    "Disruption Web": 374,
    "Scantid": 375,
    "Kakaru": 376,
    "Ursadon": 377,
    "Halo Rocket": 378,
    "Optical Flare": 379,
    "Subterranean Spines": 380,
    "Corrosive Acid": 381,
    "Corrosive Acid Hit": 382,
    "Acid Spores (1) Overlay": 383,
    "Acid Spores (2-3) Overlay": 384,
    "Acid Spores (4-5) Overlay": 385,
    "Acid Spores (6-9) Overlay": 386,
    "Ice Doodad": 387,
    "Doodad Shadows Header (BW)": 388,
    "Restoration Hit": 389,
    "Mind Control Hit": 390,
    "Optical Flare Hit": 391,
    "Feedback": 392,
    "Maelstrom Overlay": 393,
    "Unknown394": 394,
    "Unknown395": 395,
    "Desert Doodad": 396,
    "Desert Doodad2": 397,
    "Desert Doodad3": 398,
    "Desert Doodad Overlay": 399,
    "Desert Doodad4": 400,
    "Twilight Doodad": 401,
    "Twilight Doodad2": 402,
    "Twilight Doodad3": 403,
    "Twilight Doodad4": 404,
    "Overmind Cocoon": 405,
    "Warp Gate": 406,
    "Psi Disrupter": 407,
    "Power Generator": 408,
    "Warp Gate Overlay": 409,
    "Xel'Naga Temple": 410,
    "Maelstrom Hit": 411,
}

DefaultIscript: TypeAlias = Literal[
    "Scourge",
    "Unknown1",
    "Scourge Death",
    "Scourge Explosion",
    "Broodling",
    "Broodling Remnants",
    "Infested Terran",
    "Infested Terran Explosion",
    "Guardian Cocoon",
    "Defiler",
    "Defiler Remnants",
    "Drone",
    "Drone Remnants",
    "Egg",
    "Egg Remnants",
    "Guardian",
    "Guardian Birth",
    "Guardian Death",
    "Hydralisk",
    "Hydralisk Remnants",
    "Infested Kerrigan",
    "Larva",
    "Larva Remnants",
    "Mutalisk",
    "Mutalisk Death",
    "Overlord",
    "Overlord Death",
    "Queen",
    "Queen Death",
    "Ultralisk",
    "Ultralisk Remnants",
    "Zergling",
    "Zergling Remnants",
    "Zerg Air Death Explosion",
    "Zerg Building Explosion",
    "Unknown35",
    "Zerg Birth",
    "Egg Spawn",
    "Cerebrate",
    "Infested Command Center",
    "Spawning Pool",
    "Evolution Chamber",
    "Creep Colony",
    "Hatchery",
    "Hive",
    "Lair",
    "Sunken Colony",
    "Mature Chrysalis",
    "Greater Spire",
    "Defiler Mound",
    "Queen's Nest",
    "Nydus Canal",
    "Overmind With Shell",
    "Overmind Remnants",
    "Overmind Without Shell",
    "Ultralisk Cavern",
    "Extractor",
    "Hydralisk Den",
    "Spire",
    "Spore Colony",
    "Infested Command Center Overlay",
    "Zerg Construction (Small)",
    "Zerg Construction (Medium)",
    "Zerg Building Morph",
    "Zerg Construction (Large)",
    "Zerg Building Spawn",
    "Battlecruiser",
    "Civilian",
    "Dropship",
    "Firebat",
    "Ghost",
    "Ghost Remnants",
    "Ghost Death",
    "Nuke Beam",
    "Nuke Target Dot",
    "Goliath (Base)",
    "Goliath (Turret)",
    "Sarah Kerrigan",
    "Marine",
    "Marine Remnants",
    "Marine Death",
    "Scanner Sweep",
    "Wraith",
    "Wraith Afterburners",
    "SCV",
    "Unknown85",
    "Vulture",
    "Spider Mine",
    "Science Vessel (Base)",
    "Science Vessel (Turret)",
    "Siege Tank (Tank) Base",
    "Siege Tank (Tank) Turret",
    "Siege Tank (Siege) Base",
    "Siege Tank (Siege) Turret",
    "Academy",
    "Academy Overlay",
    "Barracks",
    "Armory",
    "Armory Overlay",
    "Comsat Station",
    "Comsat Connector",
    "Comsat Overlay",
    "Command Center",
    "Command Center Overlay",
    "Crashed Battlecruiser",
    "Supply Depot",
    "Supply Depot Overlay",
    "Control Tower",
    "Control Tower Connector",
    "Control Tower Overlay",
    "Unknown110",
    "Factory",
    "Factory Overlay",
    "Covert Ops",
    "Covert Ops Connector",
    "Covert Ops Overlay",
    "Ion Cannon",
    "Machine Shop",
    "Machine Shop Connector",
    "Missile Turret (Base)",
    "Missile Turret (Turret)",
    "Physics Lab",
    "Physics Lab Connector",
    "Bunker",
    "Bunker Overlay",
    "Refinery",
    "Science Facility",
    "Science Facility Overlay",
    "Nuclear Silo",
    "Nuclear Silo Connector",
    "Nuclear Silo Overlay",
    "Nuclear Missile",
    "Unknown132",
    "Nuke Explosion",
    "Starport",
    "Starport Overlay",
    "Engineering Bay",
    "Engineering Bay Overlay",
    "Terran Construction (Large)",
    "Terran Construction (Medium)",
    "Terran Construction (Small)",
    "Addon Construction",
    "Explosion (Small)",
    "Explosion (Medium)",
    "Explosion (Large)",
    "Building Rubble Header",
    "Arbiter",
    "Archon Energy",
    "Archon Being",
    "Archon Swirl",
    "Unknown150",
    "Carrier",
    "Dark Templar (Hero)",
    "Dragoon",
    "Dragoon Remnants",
    "Interceptor",
    "Probe",
    "Shuttle",
    "High Templar",
    "Reaver",
    "Scarab",
    "Scout",
    "Scout Engines",
    "Zealot",
    "Zealot Death",
    "Observer",
    "Templar Archives",
    "Assimilator",
    "Observatory",
    "Unknown169",
    "Citadel Of Adun",
    "Forge",
    "Forge Overlay",
    "Gateway",
    "Cybernetics Core",
    "Cybrnetics Core Overlay",
    "Khaydarin Crystal Formation",
    "Nexus",
    "Nexus Overlay",
    "Photon Cannon",
    "Arbiter Tribunal",
    "Pylon",
    "Robotics Facility",
    "Shield Battery",
    "Shield Battery Overlay",
    "Stargate",
    "Stargate Overlay",
    "Stasis Cell/Prison",
    "Robotics Support Bay",
    "Temple",
    "Fleet Beacon",
    "Warp Anchor",
    "Warp Flash Header",
    "Warp Texture",
    "Unknown194",
    "Unknown195",
    "Unknown196",
    "Unknown197",
    "Ragnasaur",
    "Rhynadon",
    "Bengalaas",
    "Vespene Geyser",
    "Vespene Geyser2",
    "Vespene Geyser Shadow",
    "Mineral Field Type1",
    "Mineral Field Type2",
    "Mineral Field Type3",
    "Unknown207",
    "Zerg Beacon Overlay",
    "Terran Beacon Overlay",
    "Protoss Beacon Overlay",
    "Zerg Beacon",
    "Protoss Beacon",
    "Terran Beacon",
    "Unknown214",
    "Powerups Shadow Header",
    "Flag",
    "Psi Emitter",
    "Data Disk",
    "Crystals Shadows",
    "Young Chrysalis",
    "Ore Chunk",
    "Ore Chunk2",
    "Gas Sac",
    "Gas Sac2",
    "Gas Orb",
    "Gas Orb2",
    "Gas Tank",
    "Gas Tank2",
    "Archon Overlay",
    "Particle Beam Hit",
    "Dual Photon Blaster Hit",
    "Anti-Matter Missile",
    "Pulse Cannon",
    "Phase Disruptor",
    "STA/STS Photon Cannon Overlay",
    "Psionic Storm",
    "Fusion Cutter Hit",
    "Gauss Rifle Hit",
    "Gemini Missiles",
    "Longbolt Missile",
    "C-10 Canister Rifle Hit",
    "Fragmentation Grenade",
    "ATA/ATS Laser Battery/Burst Lasers",
    "Unknown244",
    "Lockdown Hit",
    "Arclite Shock Cannon Hit",
    "Yamato Gun",
    "Yamato Gun Trail",
    "Lockdown/EMP Shockwave Missile",
    "Siege Tank (Tank) Turret Overlay",
    "Siege Tank (Siege) Turret Overlay",
    "Science Vessel Overlay",
    "Hallucination Hit",
    "Unknown254",
    "Unknown255",
    "Needle Spines Hit",
    "Venom (Unused)",
    "Subterranean Tentacle",
    "Venom Hit (Unused)",
    "Acid Spore",
    "Acid Spore Hit",
    "Guardian Attack Overlay",
    "Unknown263",
    "Glave Wurm",
    "Glave Wurm Hit",
    "Seeker Spores",
    "Queen Spell Holder",
    "Psionic Shockwave Hit",
    "Glave Wurm Trail",
    "Seeker Spores Overlay",
    "Acid Spray",
    "Unknown272",
    "Longbolt/Halo/Gemini Missiles Trail",
    "Burowing Dust",
    "Shadow Header",
    "Shield Overlay",
    "Unknown277",
    "Double Explosion",
    "Nuclear Missile Death",
    "Spider Mine Explosion",
    "Vespene Geyser Smokes",
    "Unknown282",
    "Fragmentation Grenade Hit",
    "Grenade Shot Smoke",
    "Phase Disruptor/Anti-Matter Missile Hit",
    "Scarab/Anti-Matter Missile Overlay",
    "Scarab Hit",
    "Cursor Marker",
    "Circle Marker",
    "Carrier Engines",
    "Engines/Glow Header",
    "White Circle",
    "Battlecruiser Attack Overlay",
    "ATA/ATS Laser Battery/Burst Lasers Hit",
    "Unknown295",
    "Plague Cloud",
    "Plague Overlay",
    "Consume",
    "Dark Swarm",
    "Defensive Matrix Overlay",
    "Defensive Matrix Hit",
    "Ensnare",
    "Ensnare Overlay",
    "Irradiate",
    "Recall Field",
    "Stasis Field Overlay",
    "Stasis Field Hit",
    "Recharge Shields (Large)",
    "Recharge Shields (Small)",
    "High Templar Glow",
    "Needle Spines Overlay",
    "Flamethrower",
    "Gemini Missiles Explosion",
    "Yamato Gun Overlay",
    "Yamato Gun Hit",
    "Unknown316",
    "Psionic Storm Part Variant1",
    "Psionic Storm Part Variant2",
    "EMP Shockwave Hit (Part1)",
    "EMP Shockwave Hit (Part2)",
    "Hallucination Death",
    "Flames (Small)",
    "Unknown323",
    "Bleeding (Small) Variant1",
    "Bleeding (Small) Variant2",
    "Flames (Large)",
    "Unknown327",
    "Bleeding (Large) Variant1",
    "Bleeding (Large) Variant2",
    "Dust Variant1",
    "Dust Variant2",
    "Confirm Circle",
    "Psi Field Type1",
    "Psi Field Type2",
    "Start Location",
    "Doodad Header",
    "Doodad Header2",
    "Space Platform Doodad",
    "Space Platform Doodad2",
    "Installation Doodad",
    "Installation Doodad2",
    "Installation Right Wall Fans",
    "Installation Left Wall Fans",
    "Installation Gear",
    "Floor Missile Trap",
    "Floor Missile Trap Turret",
    "Floor Gun Trap",
    "Wall Missile Trap Type1",
    "Wall Missile Trap Typet2",
    "Wall Flame Trap Type1",
    "Wall Flame Trap Type2",
    "Map Revealer",
    "Lurker Egg",
    "Lurker",
    "Unknown355",
    "Lurker Remnants",
    "Devourer",
    "Devourer Birth",
    "Devourer Death",
    "Medic",
    "Medic Remnants",
    "Valkyrie",
    "Unknown363",
    "Unknown364",
    "Dark Archon Energy",
    "Dark Archon Being",
    "Dark Archon Swirl",
    "Dark Archon Death",
    "Corsair",
    "Corsair Engines",
    "Unknown371",
    "Dark Templar (Unit)",
    "Neutron Flare",
    "Disruption Web",
    "Scantid",
    "Kakaru",
    "Ursadon",
    "Halo Rocket",
    "Optical Flare",
    "Subterranean Spines",
    "Corrosive Acid",
    "Corrosive Acid Hit",
    "Acid Spores (1) Overlay",
    "Acid Spores (2-3) Overlay",
    "Acid Spores (4-5) Overlay",
    "Acid Spores (6-9) Overlay",
    "Ice Doodad",
    "Doodad Shadows Header (BW)",
    "Restoration Hit",
    "Mind Control Hit",
    "Optical Flare Hit",
    "Feedback",
    "Maelstrom Overlay",
    "Unknown394",
    "Unknown395",
    "Desert Doodad",
    "Desert Doodad2",
    "Desert Doodad3",
    "Desert Doodad Overlay",
    "Desert Doodad4",
    "Twilight Doodad",
    "Twilight Doodad2",
    "Twilight Doodad3",
    "Twilight Doodad4",
    "Overmind Cocoon",
    "Warp Gate",
    "Psi Disrupter",
    "Power Generator",
    "Warp Gate Overlay",
    "Xel'Naga Temple",
    "Maelstrom Hit",
]
