from typing import Literal, TypeAlias

DefImageDict: dict[str, int] = {
    "Scourge": 0,
    "Scourge Shadow": 1,
    "Scourge Birth": 2,
    "Scourge Death": 3,
    "Scourge Explosion": 4,
    "Broodling": 5,
    "Broodling Shadow": 6,
    "Broodling Remnants": 7,
    "Infested Terran": 8,
    "Infested Terran Shadow": 9,
    "Infested Terran Explosion": 10,
    "Guardian Cocoon": 11,
    "Guardian Cocoon Shadow": 12,
    "Defiler": 13,
    "Defiler Shadow": 14,
    "Defiler Birth": 15,
    "Defiler Remnants": 16,
    "Drone": 17,
    "Drone Shadow": 18,
    "Drone Birth": 19,
    "Drone Remnants": 20,
    "Egg": 21,
    "Egg Shadow": 22,
    "Egg Spawn": 23,
    "Egg Remnants": 24,
    "Guardian": 25,
    "Guardian Shadow": 26,
    "Guardian Birth": 27,
    "Guardian Death": 28,
    "Hydralisk": 29,
    "Hydralisk Shadow": 30,
    "Hydralisk Birth": 31,
    "Hydralisk Remnants": 32,
    "Infested Kerrigan": 33,
    "Infested Kerrigan Shadow": 34,
    "Needle Spines": 35,
    "Larva": 36,
    "Larva Remnants": 37,
    "Mutalisk": 38,
    "Mutalisk Shadow": 39,
    "Mutalisk Birth": 40,
    "Mutalisk Death": 41,
    "Overlord": 42,
    "Overlord Shadow": 43,
    "Overlord Birth": 44,
    "Overlord Death": 45,
    "Queen": 46,
    "Queen Shadow": 47,
    "Queen Death": 48,
    "Queen Birth": 49,
    "Ultralisk": 50,
    "Ultralisk Shadow": 51,
    "Ultralisk Birth": 52,
    "Ultralisk Remnants": 53,
    "Zergling": 54,
    "Zergling Shadow": 55,
    "Zergling Birth": 56,
    "Zergling Remnants": 57,
    "Zerg Air Death Explosion (Large)": 58,
    "Zerg Air Death Explosion (Small)": 59,
    "Zerg Building Explosion": 60,
    "Cerebrate": 61,
    "Cerebrate Shadow": 62,
    "Infested Command Center": 63,
    "Spawning Pool": 64,
    "Spawning Pool Shadow": 65,
    "Evolution Chamber": 66,
    "Evolution Chamber Shadow": 67,
    "Creep Colony": 68,
    "Creep Colony Shadow": 69,
    "Hatchery": 70,
    "Hatchery Shadow": 71,
    "Hive": 72,
    "Hive Shadow": 73,
    "Lair": 74,
    "Lair Shadow": 75,
    "Sunken Colony": 76,
    "Sunken Colony Shadow": 77,
    "Mature Chrysalis": 78,
    "Mature Chrysalis Shadow": 79,
    "Greater Spire": 80,
    "Greater Spire Shadow": 81,
    "Defiler Mound": 82,
    "Defiler Mound Shadow": 83,
    "Queen's Nest": 84,
    "Queen's Nest Shadow": 85,
    "Nydus Canal": 86,
    "Nydus Canal Shadow": 87,
    "Overmind With Shell": 88,
    "Overmind Remnants": 89,
    "Overmind Without Shell": 90,
    "Ultralisk Cavern": 91,
    "Ultralisk Cavern Shadow": 92,
    "Extractor": 93,
    "Extractor Shadow": 94,
    "Hydralisk Den": 95,
    "Hydralisk Den Shadow": 96,
    "Spire": 97,
    "Spire Shadow": 98,
    "Spore Colony": 99,
    "Spore Colony Shadow": 100,
    "Infested Command Center Overlay": 101,
    "Zerg Construction (Large)": 102,
    "Zerg Building Morph": 103,
    "Zerg Construction (Medium)": 104,
    "Zerg Construction (Small)": 105,
    "Zerg Building Construction Shadow": 106,
    "Zerg Building Spawn (Small)": 107,
    "Zerg Building Spawn (Medium)": 108,
    "Zerg Building Spawn (Large)": 109,
    "Zerg Building Rubble (Small)": 110,
    "Zerg Building Rubble (Large)": 111,
    "Carrier": 112,
    "Carrier Shadow": 113,
    "Carrier Engines": 114,
    "Carrier Warp Flash": 115,
    "Interceptor": 116,
    "Interceptor Shadow": 117,
    "Shuttle": 118,
    "Shuttle Shadow": 119,
    "Shuttle Engines": 120,
    "Shuttle Warp Flash": 121,
    "Dragoon": 122,
    "Dragoon Shadow": 123,
    "Dragoon Remnants": 124,
    "Dragoon Warp Flash": 125,
    "High Templar": 126,
    "High Templar Shadow": 127,
    "High Templar Warp Flash": 128,
    "Dark Templar (Hero)": 129,
    "Arbiter": 130,
    "Arbiter Shadow": 131,
    "Arbiter Engines": 132,
    "Arbiter Warp Flash": 133,
    "Archon Energy": 134,
    "Archon Being": 135,
    "Archon Swirl": 136,
    "Probe": 137,
    "Probe Shadow": 138,
    "Probe Warp Flash": 139,
    "Scout": 140,
    "Scout Shadow": 141,
    "Scout Engines": 142,
    "Scout Warp Flash": 143,
    "Reaver": 144,
    "Reaver Shadow": 145,
    "Reaver Warp Flash": 146,
    "Scarab": 147,
    "Observer": 148,
    "Observer Shadow": 149,
    "Observer Warp Flash": 150,
    "Zealot": 151,
    "Zealot Shadow": 152,
    "Zealot Death": 153,
    "Zealot Warp Flash": 154,
    "Templar Archives": 155,
    "Templar Archives Warp Flash": 156,
    "Templar Archives Shadow": 157,
    "Assimilator": 158,
    "Assimilator Warp Flash": 159,
    "Assimilator Shadow": 160,
    "Observatory": 161,
    "Observatory Warp Flash": 162,
    "Observatory Shadow": 163,
    "Citadel of Adun": 164,
    "Citadel of Adun Warp Flash": 165,
    "Citadel of Adun Shadow": 166,
    "Forge": 167,
    "Forge Overlay": 168,
    "Forge Warp Flash": 169,
    "Forge Shadow": 170,
    "Gateway": 171,
    "Gateway Warp Flash": 172,
    "Gateway Shadow": 173,
    "Cybernetics Core": 174,
    "Cybernetics Core Warp Flash": 175,
    "Cybernetics Core Overlay": 176,
    "Cybernetics Core Shadow": 177,
    "Khaydarin Crystal Formation": 178,
    "Nexus": 179,
    "Nexus Warp Flash": 180,
    "Nexus Overlay": 181,
    "Nexus Shadow": 182,
    "Photon Cannon": 183,
    "Photon Cannon Shadow": 184,
    "Photon Cannon Warp Flash": 185,
    "Arbiter Tribunal": 186,
    "Arbiter Tribunal Warp Flash": 187,
    "Arbiter Tribunal Shadow": 188,
    "Pylon": 189,
    "Pylon Warp Flash": 190,
    "Pylon Shadow": 191,
    "Robotics Facility": 192,
    "Robotics Facility Warp Flash": 193,
    "Robotics Facility Shadow": 194,
    "Shield Battery": 195,
    "Shield Battery Overlay": 196,
    "Shileld Battery Warp Flash": 197,
    "Shield Battery Shadow": 198,
    "Stargate": 199,
    "Stargate Overlay": 200,
    "Stargate Warp Flash": 201,
    "Stargate Shadow": 202,
    "Stasis Cell/Prison": 203,
    "Robotics Support Bay": 204,
    "Robotics Support Bay Warp Flash": 205,
    "Robotics Support Bay Shadow": 206,
    "Protoss Temple": 207,
    "Fleet Beacon": 208,
    "Fleet Beacon Warp Flash": 209,
    "Warp Texture": 210,
    "Warp Anchor": 211,
    "Fleet Beacon Shadow": 212,
    "Explosion1 (Small)": 213,
    "Explosion1 (Medium)": 214,
    "Explosion (Large)": 215,
    "Protoss Building Rubble (Small)": 216,
    "Protoss Building Rubble (Large)": 217,
    "Battlecruiser": 218,
    "Battlecruiser Shadow": 219,
    "Battlecruiser Engines": 220,
    "Civilian": 221,
    "Civilian Shadow": 222,
    "Dropship": 223,
    "Dropship Shadow": 224,
    "Dropship Engines": 225,
    "Firebat": 226,
    "Firebat Shadow": 227,
    "Ghost": 228,
    "Ghost Shadow": 229,
    "Ghost Remnants": 230,
    "Ghost Death": 231,
    "Nuke Beam": 232,
    "Nuke Target Dot": 233,
    "Goliath Base": 234,
    "Goliath Turret": 235,
    "Goliath Shadow": 236,
    "Sarah Kerrigan": 237,
    "Sarah Kerrigan Shadow": 238,
    "Marine": 239,
    "Marine Shadow": 240,
    "Marine Remnants": 241,
    "Marine Death": 242,
    "Wraith": 243,
    "Wraith Shadow": 244,
    "Wraith Engines": 245,
    "Scanner Sweep": 246,
    "SCV": 247,
    "SCV Shadow": 248,
    "SCV Glow": 249,
    "Siege Tank (Tank) Base": 250,
    "Siege Tank (Tank) Turret": 251,
    "Siege Tank (Tank) Base Shadow": 252,
    "Siege Tank (Siege) Base": 253,
    "Siege Tank (Siege) Turret": 254,
    "Siege Tank (Siege) Base Shadow": 255,
    "Vulture": 256,
    "Vulture Shadow": 257,
    "Spider Mine": 258,
    "Spider Mine Shadow": 259,
    "Science Vessel (Base)": 260,
    "Science Vessel (Turret)": 261,
    "Science Vessel Shadow": 262,
    "Terran Academy": 263,
    "Academy Overlay": 264,
    "Academy Shadow": 265,
    "Barracks": 266,
    "Barracks Shadow": 267,
    "Armory": 268,
    "Armory Overlay": 269,
    "Armory Shadow": 270,
    "Comsat Station": 271,
    "Comsat Station Connector": 272,
    "Comsat Station Overlay": 273,
    "Comsat Station Shadow": 274,
    "Command Center": 275,
    "Command Center Overlay": 276,
    "Command Center Shadow": 277,
    "Supply Depot": 278,
    "Supply Depot Overlay": 279,
    "Supply Depot Shadow": 280,
    "Control Tower": 281,
    "Control Tower Connector": 282,
    "Control Tower Overlay": 283,
    "Control Tower Shadow": 284,
    "Factory": 285,
    "Factory Overlay": 286,
    "Factory Shadow": 287,
    "Covert Ops": 288,
    "Covert Ops Connector": 289,
    "Covert Ops Overlay": 290,
    "Covert Ops Shadow": 291,
    "Ion Cannon": 292,
    "Machine Shop": 293,
    "Machine Shop Connector": 294,
    "Machine Shop Shadow": 295,
    "Missile Turret (Base)": 296,
    "Missile Turret (Turret)": 297,
    "Missile Turret (Base) Shadow": 298,
    "Crashed Batlecruiser": 299,
    "Crashed Battlecruiser Shadow": 300,
    "Physics Lab": 301,
    "Physics Lab Connector": 302,
    "Physics Lab Shadow": 303,
    "Bunker": 304,
    "Bunker Shadow": 305,
    "Bunker Overlay": 306,
    "Refinery": 307,
    "Refinery Shadow": 308,
    "Science Facility": 309,
    "Science Facility Overlay": 310,
    "Science Facility Shadow": 311,
    "Nuclear Silo": 312,
    "Nuclear Silo Connector": 313,
    "Nuclear Silo Overlay": 314,
    "Nuclear Silo Shadow": 315,
    "Nuclear Missile": 316,
    "Nuclear Missile Shadow": 317,
    "Nuke Hit": 318,
    "Starport": 319,
    "Starport Overlay": 320,
    "Starport Shadow": 321,
    "Engineering Bay": 322,
    "Engineering Bay Overlay": 323,
    "Engineering Bay Shadow": 324,
    "Terran Construction (Large)": 325,
    "Terran Construction (Large) Shadow": 326,
    "Terran Construction (Medium)": 327,
    "Terran Construction (Medium) Shadow": 328,
    "Addon Construction": 329,
    "Terran Construction (Small)": 330,
    "Terran Construction (Small) Shadow": 331,
    "Explosion2 (Small)": 332,
    "Explosion2 (Medium)": 333,
    "Building Explosion (Large)": 334,
    "Terran Building Rubble (Small)": 335,
    "Terran Building Rubble (Large)": 336,
    "Dark Swarm": 337,
    "Ragnasaur (Ash)": 338,
    "Ragnasaur (Ash) Shadow": 339,
    "Rhynadon (Badlands)": 340,
    "Rhynadon (Badlands) Shadow": 341,
    "Bengalaas (Jungle)": 342,
    "Bengalaas (Jungle) Shadow": 343,
    "Vespene Geyser": 344,
    "Vespene Geyser2": 345,
    "Vespene Geyser Shadow": 346,
    "Mineral Field Type1": 347,
    "Mineral Field Type1 Shadow": 348,
    "Mineral Field Type2": 349,
    "Mineral Field Type2 Shadow": 350,
    "Mineral Field Type3": 351,
    "Mineral Field Type3 Shadow": 352,
    "Independent Starport (Unused)": 353,
    "Zerg Beacon": 354,
    "Zerg Beacon Overlay": 355,
    "Terran Beacon": 356,
    "Terran Beacon Overlay": 357,
    "Protoss Beacon": 358,
    "Protoss Beacon Overlay": 359,
    "Magna Pulse (Unused)": 360,
    "Lockdown Field (Small)": 361,
    "Lockdown Field (Medium)": 362,
    "Lockdown Field (Large)": 363,
    "Stasis Field Hit": 364,
    "Stasis Field (Small)": 365,
    "Stasis Field (Medium)": 366,
    "Stasis Field (Large)": 367,
    "Recharge Shields (Small)": 368,
    "Recharge Shields (Medium)": 369,
    "Recharge Shields (Large)": 370,
    "Defensive Matrix Front (Small)": 371,
    "Defensive Matrix Front (Medium)": 372,
    "Defensive Matrix Front (Large)": 373,
    "Defensive Matrix Back (Small)": 374,
    "Defensive Matrix Back (Medium)": 375,
    "Defensive Matrix Back (Large)": 376,
    "Defensive Matrix Hit (Small)": 377,
    "Defensive Matrix Hit (Medium)": 378,
    "Defensive Matrix Hit (Large)": 379,
    "Irradiate (Small)": 380,
    "Irradiate (Medium)": 381,
    "Irradiate (Large)": 382,
    "Ensnare Cloud": 383,
    "Ensnare Overlay (Small)": 384,
    "Ensnare Overlay (Medium)": 385,
    "Ensnare Overlay (Large)": 386,
    "Plague Cloud": 387,
    "Plague Overlay (Small)": 388,
    "Plague Overlay (Medium)": 389,
    "Plague Overlay (Large)": 390,
    "Recall Field": 391,
    "Flag": 392,
    "Young Chrysalis": 393,
    "Psi Emitter": 394,
    "Data Disc": 395,
    "Khaydarin Crystal": 396,
    "Mineral Chunk Type1": 397,
    "Mineral Chunk Type2": 398,
    "Protoss Gas Orb Type1": 399,
    "Protoss Gas Orb Type2": 400,
    "Zerg Gas Sac Type1": 401,
    "Zerg Gas Sac Type2": 402,
    "Terran Gas Tank Type1": 403,
    "Terran Gas Tank Type2": 404,
    "Mineral Chunk Shadow": 405,
    "Protoss Gas Orb Shadow": 406,
    "Zerg Gas Sac Shadow": 407,
    "Terran Gas Tank Shadow": 408,
    "Data Disk Shadow (Ground)": 409,
    "Data Disk Shadow (Carried)": 410,
    "Flag Shadow (Ground)": 411,
    "Flag Shadow (Carried)": 412,
    "Crystal Shadow (Ground)": 413,
    "Crystal Shadow (Carried)": 414,
    "Young Chrysalis Shadow (Ground)": 415,
    "Young Chrysalis Shadow (Carried)": 416,
    "Psi Emitter Shadow (Ground)": 417,
    "Psi Emitter Shadow (Carried)": 418,
    "Acid Spray (Unused)": 419,
    "Plasma Drip (Unused)": 420,
    "FlameThrower": 421,
    "Longbolt/Gemini Missiles Trail": 422,
    "Burrowing Dust": 423,
    "Shield Overlay": 424,
    "Small Explosion (Unused)": 425,
    "Double Explosion": 426,
    "Phase Disruptor Hit": 427,
    "Nuclear Missile Death": 428,
    "Spider Mine Death": 429,
    "Vespene Geyser Smoke1": 430,
    "Vespene Geyser Smoke2": 431,
    "Vespene Geyser Smoke3": 432,
    "Vespene Geyser Smoke4": 433,
    "Vespene Geyser Smoke5": 434,
    "Vespene Geyser Smoke1 Overlay": 435,
    "Vespene Geyser Smoke2 Overlay": 436,
    "Vespene Geyser Smoke3 Overlay": 437,
    "Vespene Geyser Smoke4 Overlay": 438,
    "Vespene Geyser Smoke5 Overlay": 439,
    "Fragmentation Grenade Hit": 440,
    "Grenade Shot Smoke": 441,
    "Anti-Matter Missile Hit": 442,
    "Scarab/Anti-Matter Missile Overlay": 443,
    "Scarab Hit": 444,
    "Cursor Marker": 445,
    "Battlecruiser Attack Overlay": 446,
    "Burst Lasers Hit": 447,
    "Burst Lasers Overlay (Unused)": 448,
    "High Templar Glow": 449,
    "Flames1 Type1 (Small)": 450,
    "Flames1 Type2 (Small)": 451,
    "Flames1 Type3 (Small)": 452,
    "Flames2 Type3 (Small)": 453,
    "Flames3 Type3 (Small)": 454,
    "Flames4 Type3 (Small)": 455,
    "Flames5 Type3 (Small)": 456,
    "Flames6 Type3 (Small)": 457,
    "Bleeding Variant1 Type1 (Small)": 458,
    "Bleeding Variant1 Type2 (Small)": 459,
    "Bleeding Variant1 Type3 (Small)": 460,
    "Bleeding Variant1 Type4 (Small)": 461,
    "Bleeding Variant2 Type1 (Small)": 462,
    "Bleeding Variant2 Type2 (Small)": 463,
    "Bleeding Variant2 Type3 (Small)": 464,
    "Bleeding Variant2 Type4 (Small)": 465,
    "Flames2 Type1 (Small)": 466,
    "Flames2 Type2 (Small)": 467,
    "Flames7 Type3 (Small)": 468,
    "Flames3 Type1 (Small)": 469,
    "Flames3 Type2 (Small)": 470,
    "Flames8 Type3 (Small)": 471,
    "Flames1 Type1 (Large)": 472,
    "Flames1 Type2 (Large)": 473,
    "Flames1 Type3 (Large)": 474,
    "Flames2 Type3 (Large)": 475,
    "Flames3 Type3 (Large)": 476,
    "Flames4 Type3 (Large)": 477,
    "Flames5 Type3 (Large)": 478,
    "Flames6 Type3 (Large)": 479,
    "Bleeding Variant1 Type1 (Large)": 480,
    "Bleeding Variant1 Type2 (Large)": 481,
    "Bleeding Variant1 Type3 (Large)": 482,
    "Bleeding Variant1 Type4 (Large)": 483,
    "Bleeding Variant2 Type1 (Large)": 484,
    "Bleeding Variant2 Type3 (Large)": 485,
    "Bleeding Variant3 Type3 (Large)": 486,
    "Bleeding Variant2 Type4 (Large)": 487,
    "Flames2 Type1 (Large)": 488,
    "Flames2 Type2 (Large)": 489,
    "Flames7 Type3 (Large)": 490,
    "Flames3 Type1 (Large)": 491,
    "Flames3 Type2 (Large)": 492,
    "Flames8 Type3 (Large)": 493,
    "Building Landing Dust Type1": 494,
    "Building Landing Dust Type2": 495,
    "Building Landing Dust Type3": 496,
    "Building Landing Dust Type4": 497,
    "Building Landing Dust Type5": 498,
    "Building Lifting Dust Type1": 499,
    "Building Lifting Dust Type2": 500,
    "Building Lifting Dust Type3": 501,
    "Building Lifting Dust Type4": 502,
    "White Circle": 503,
    "Needle Spine Hit": 504,
    "Plasma Drip Hit (Unused)": 505,
    "Sunken Colony Tentacle": 506,
    "Venom (Unused Zerg Weapon)": 507,
    "Venom Hit (Unused)": 508,
    "Acid Spore": 509,
    "Acid Spore Hit": 510,
    "Glave Wurm": 511,
    "Glave Wurm/Seeker Spores Hit": 512,
    "Glave Wurm Trail": 513,
    "Seeker Spores Overlay": 514,
    "Seeker Spores": 515,
    "Queen Spell Holder": 516,
    "Consume": 517,
    "Guardian Attack Overlay": 518,
    "Dual Photon Blasters Hit": 519,
    "Particle Beam Hit": 520,
    "Anti-Matter Missile": 521,
    "Pulse Cannon": 522,
    "Phase Disruptor": 523,
    "STA/STS Photon Cannon Overlay": 524,
    "Psionic Storm": 525,
    "Fusion Cutter Hit": 526,
    "Gauss Rifle Hit": 527,
    "Gemini Missiles": 528,
    "Lockdown/LongBolt/Hellfire Missile": 529,
    "Gemini Missiles Explosion": 530,
    "C-10 Canister Rifle Hit": 531,
    "Fragmentation Grenade": 532,
    "Arclite Shock Cannon Hit": 533,
    "ATS/ATA Laser Battery": 534,
    "Burst Lasers": 535,
    "Siege Tank(Tank) Turret Attack Overlay": 536,
    "Siege Tank(Siege) Turret Attack Overlay": 537,
    "Science Vessel Overlay (Part1)": 538,
    "Science Vessel Overlay (Part2)": 539,
    "Science Vessel Overlay (Part3)": 540,
    "Yamato Gun": 541,
    "Yamato Gun Trail": 542,
    "Yamato Gun Overlay": 543,
    "Yamato Gun Hit": 544,
    "Hallucination Hit": 545,
    "Scanner Sweep Hit": 546,
    "Archon Birth (Unused)": 547,
    "Psionic Shockwave Hit": 548,
    "Archon Beam": 549,
    "Psionic Storm Part1": 550,
    "Psionic Storm Part2": 551,
    "Psionic Storm Part3": 552,
    "Psionic Storm Part4": 553,
    "EMP Shockwave Missile": 554,
    "EMP Shockwave Hit (Part1)": 555,
    "EMP Shockwave Hit (Part2)": 556,
    "Hallucination Death1": 557,
    "Hallucination Death2": 558,
    "Hallucination Death3": 559,
    "Circle Marker1": 560,
    "Selection Circle (22pixels)": 561,
    "Selection Circle (32pixels)": 562,
    "Selection Circle (48pixels)": 563,
    "Selection Circle (62pixels)": 564,
    "Selection Circle (72pixels)": 565,
    "Selection Circle (94pixels)": 566,
    "Selection Circle (110pixels)": 567,
    "Selection Circle (122pixels)": 568,
    "Selection Circle (146pixels)": 569,
    "Selection Circle (224pixels)": 570,
    "Selection Circle Dashed (22pixels)": 571,
    "Selection Circle Dashed (32pixels)": 572,
    "Selection Circle Dashed (48pixels)": 573,
    "Selection Circle Dashed (62pixels)": 574,
    "Selection Circle Dashed (72pixels)": 575,
    "Selection Circle Dashed (94pixels)": 576,
    "Selection Circle Dashed (110pixels)": 577,
    "Selection Circle Dashed (122pixels)": 578,
    "Selection Circle Dashed (146pixels)": 579,
    "Selection Circle Dashed (224pixels)": 580,
    "Circle Marker2": 581,
    "Map Revealer": 582,
    "Circle Marker3": 583,
    "Psi Field1 (Right Upper)": 584,
    "Psi Field1 (Right Lower)": 585,
    "Psi Field2 (Right Upper)": 586,
    "Psi Field2 (Right Lower)": 587,
    "Start Location": 588,
    "2/38 Ash": 589,
    "2/38 Ash Shadow": 590,
    "2/39 Ash": 591,
    "2/39 Ash Shadow": 592,
    "2/41 Ash": 593,
    "2/41 Ash Shadow": 594,
    "2/40 Ash": 595,
    "2/40 Ash Shadow": 596,
    "2/42 Ash": 597,
    "2/42 Ash Shadow": 598,
    "2/43 Ash": 599,
    "2/44 Ash": 600,
    "2/1 Ash": 601,
    "2/4 Ash": 602,
    "2/5 Ash": 603,
    "2/30 Ash": 604,
    "2/28 Ash": 605,
    "2/29 Ash": 606,
    "4/1 Ash": 607,
    "4/2 Ash": 608,
    "4/3 Ash": 609,
    "4/56 Jungle": 610,
    "4/56 Jungle Shadow": 611,
    "4/57 Jungle": 612,
    "4/57 Jungle Shadow": 613,
    "4/58 Jungle": 614,
    "4/58 Jungle Shadow": 615,
    "4/59 Jungle": 616,
    "4/59 Jungle Shadow": 617,
    "9/5 Jungle": 618,
    "9/5 Jungle Shadow": 619,
    "9/6 Jungle": 620,
    "9/6 Jungle Shadow": 621,
    "9/7 Jungle": 622,
    "9/7 Jungle Shadow": 623,
    "4/51 Jungle": 624,
    "4/51 Jungle Shadow": 625,
    "4/52 Jungle": 626,
    "4/52 Jungle Shadow": 627,
    "4/54 Jungle": 628,
    "4/54 Jungle Shadow": 629,
    "4/53 Jungle": 630,
    "4/53 Jungle Shadow": 631,
    "9/1 Jungle": 632,
    "9/1 Jungle Shadow": 633,
    "9/2 Jungle": 634,
    "9/2 Jungle Shadow": 635,
    "9/3 Jungle": 636,
    "9/3 Jungle Shadow": 637,
    "9/4 Jungle": 638,
    "9/4 Jungle Shadow": 639,
    "4/12 Jungle": 640,
    "4/13 Jungle": 641,
    "4/1 Jungle": 642,
    "4/3 Jungle": 643,
    "4/2 Jungle": 644,
    "4/5 Jungle": 645,
    "4/4 Jungle": 646,
    "4/9 Jungle": 647,
    "4/10 Jungle": 648,
    "5/5 Jungle": 649,
    "5/7 Jungle": 650,
    "5/6 Jungle": 651,
    "5/9 Jungle": 652,
    "5/8 Jungle": 653,
    "4/6 Jungle": 654,
    "4/7 Jungle": 655,
    "4/17 Jungle": 656,
    "13/4 Jungle": 657,
    "11/5 Jungle": 658,
    "11/6 Jungle": 659,
    "11/7 Jungle": 660,
    "11/8 Jungle": 661,
    "11/10 Jungle": 662,
    "11/11 Jungle": 663,
    "11/12 Jungle": 664,
    "7/4 Platform": 665,
    "7/4 Platform Shadow": 666,
    "7/5 Platform": 667,
    "7/5 Platform Shadow": 668,
    "7/6 Platform": 669,
    "7/6 Platform Shadow": 670,
    "7/1 Platform": 671,
    "7/1 Platform Shadow": 672,
    "7/2 Platform": 673,
    "7/2 Platform Shadow": 674,
    "7/3 Platform": 675,
    "7/3 Platform Shadow": 676,
    "7/9 Platform": 677,
    "7/10 Platform": 678,
    "7/8 Platform": 679,
    "7/7 Platform": 680,
    "7/26 Platform": 681,
    "7/24 Platform": 682,
    "7/28 Platform": 683,
    "7/27 Platform": 684,
    "7/25 Platform": 685,
    "7/29 Platform": 686,
    "7/30 Platform": 687,
    "7/31 Platform": 688,
    "12/1 Platform": 689,
    "9/27 Platform": 690,
    "5/54 Badlands": 691,
    "5/54 Badlands Shadow": 692,
    "5/55 Badlands": 693,
    "5/55 Badlands Shadow": 694,
    "5/56 Badlands": 695,
    "5/56 Badlands Shadow": 696,
    "5/57 Badlands": 697,
    "5/57 Badlands Shadow": 698,
    "6/16 Badlands": 699,
    "6/17 Badlands": 700,
    "6/20 Badlands": 701,
    "6/21 Badlands": 702,
    "5/10 Badlands": 703,
    "5/50 Badlands": 704,
    "5/50 Badlands Shadow": 705,
    "5/52 Badlands": 706,
    "5/52 Badlands Shadow": 707,
    "5/53 Badlands": 708,
    "5/53 Badlands Shadow": 709,
    "5/51 Badlands": 710,
    "5/51 Badlands Shadow": 711,
    "6/3 Badlands": 712,
    "11/3 Badlands": 713,
    "11/8 Badlands": 714,
    "11/6 Badlands": 715,
    "11/7 Badlands": 716,
    "11/9 Badlands": 717,
    "11/10 Badlands": 718,
    "11/11 Badlands": 719,
    "11/12 Badlands": 720,
    "11/13 Badlands": 721,
    "11/14 Badlands": 722,
    "1/13 Badlands": 723,
    "1/9 Badlands": 724,
    "1/11 Badlands": 725,
    "1/14 Badlands": 726,
    "1/10 Badlands": 727,
    "1/12 Badlands": 728,
    "1/15 Badlands": 729,
    "1/7 Badlands": 730,
    "1/5 Badlands": 731,
    "1/16 Badlands": 732,
    "1/8 Badlands": 733,
    "1/6 Badlands": 734,
    "Floor Gun Trap": 735,
    "Floor Missile Trap": 736,
    "Floor Missile Trap Turret": 737,
    "Wall Missile Trap": 738,
    "Wall Missile Trap2": 739,
    "Wall Flame Trap": 740,
    "Wall Flame Trap2": 741,
    "Left Upper Level Door": 742,
    "Right Upper Level Door": 743,
    "4/15 Installation1": 744,
    "4/15 Installation2": 745,
    "3/9 Installation": 746,
    "3/10 Installation": 747,
    "3/11 Installation": 748,
    "3/12 Installation": 749,
    "Substructure Left Door": 750,
    "Substructure Right Door": 751,
    "3/1 Installation": 752,
    "3/2 Installation": 753,
    "Substructure Opening Hole": 754,
    "7/21 Twilight": 755,
    "Unknown Twilight": 756,
    "7/13 Twilight": 757,
    "7/14 Twilight": 758,
    "7/16 Twilight": 759,
    "7/15 Twilight": 760,
    "7/19 Twilight": 761,
    "7/20 Twilight": 762,
    "7/17 Twilight": 763,
    "6/1 Twilight": 764,
    "6/2 Twilight": 765,
    "6/3 Twilight": 766,
    "6/4 Twilight": 767,
    "6/5 Twilight": 768,
    "8/3 Twilight1": 769,
    "8/3 Twilight2": 770,
    "9/29 Ice": 771,
    "9/29 Ice Shadow": 772,
    "9/28 Ice": 773,
    "9/28 Ice Shadow": 774,
    "12/38 Ice": 775,
    "12/38 Ice Shadow": 776,
    "12/37 Ice": 777,
    "12/37 Ice Shadow": 778,
    "12/33 Ice1": 779,
    "12/33 Ice1 Shadow": 780,
    "9/21 Ice": 781,
    "9/21 Ice Shadow": 782,
    "9/15 Ice": 783,
    "9/15 Ice Shadow": 784,
    "9/16 Ice": 785,
    "9/16 Ice Shadow": 786,
    "Unknown787": 787,
    "Unknown788": 788,
    "12/9 Ice1": 789,
    "12/10 Ice1": 790,
    "9/24 Ice": 791,
    "9/24 Ice Shadow": 792,
    "9/23 Ice": 793,
    "9/23 Ice Shadow": 794,
    "Unknown795": 795,
    "Unknown Ice Shadow": 796,
    "12/7 Ice": 797,
    "12/7 Ice Shadow": 798,
    "12/8 Ice": 799,
    "12/8 Ice Shadow": 800,
    "12/9 Ice2": 801,
    "12/9 Ice2 Shadow": 802,
    "12/10 Ice2": 803,
    "12/10 Ice2 Shadow": 804,
    "12/40 Ice": 805,
    "12/40 Ice Shadow": 806,
    "12/41 Ice": 807,
    "12/41 Ice Shadow": 808,
    "12/42 Ice": 809,
    "12/42 Ice Shadow": 810,
    "12/5 Ice": 811,
    "12/5 Ice Shadow": 812,
    "12/6 Ice": 813,
    "12/6 Ice Shadow": 814,
    "12/36 Ice": 815,
    "12/36 Ice Shadow": 816,
    "12/32 Ice": 817,
    "12/32 Ice Shadow": 818,
    "12/33 Ice2": 819,
    "12/33 Ice2 Shadow": 820,
    "12/34 Ice": 821,
    "12/34 Ice Shadow": 822,
    "12/24 Ice1": 823,
    "12/24 Ice1 Shadow": 824,
    "12/25 Ice1": 825,
    "12/25 Ice1 Shadow": 826,
    "12/30 Ice1": 827,
    "12/30 Ice1 Shadow": 828,
    "12/31 Ice": 829,
    "12/31 Ice Shadow": 830,
    "12/20 Ice": 831,
    "12/30 Ice2": 832,
    "12/30 Ice2 Shadow": 833,
    "9/22 Ice": 834,
    "9/22 Ice Shadow": 835,
    "12/24 Ice2": 836,
    "12/24 Ice2 Shadow": 837,
    "12/25 Ice2": 838,
    "12/25 Ice2 Shadow": 839,
    "Unknown840": 840,
    "4/1 Ice": 841,
    "6/1 Ice": 842,
    "5/6 Ice": 843,
    "5/6 Ice Shadow": 844,
    "5/7 Ice": 845,
    "5/7 Ice Shadow": 846,
    "5/8 Ice": 847,
    "5/8 Ice Shadow": 848,
    "5/9 Ice": 849,
    "5/9 Ice Shadow": 850,
    "10/10 Desert1": 851,
    "10/12 Desert1": 852,
    "10/12 Desert1 Shadow": 853,
    "10/8 Desert1": 854,
    "10/8 Desert1 Shadow": 855,
    "10/9 Desert1": 856,
    "10/9 Desert1 Shadow": 857,
    "6/10 Desert": 858,
    "6/10 Desert Shadow": 859,
    "6/13 Desert1": 860,
    "6/13 Desert1 Shadow": 861,
    "Unknown Desert": 862,
    "Unknown Desert Shadow": 863,
    "10/12 Desert2": 864,
    "10/12 Desert2 Shadow": 865,
    "10/9 Desert2": 866,
    "10/9 Desert2 Shadow": 867,
    "10/10 Desert2": 868,
    "10/10 Desert2 Shadow": 869,
    "10/11 Desert": 870,
    "10/11 Desert Shadow": 871,
    "10/14 Desert": 872,
    "10/14 Desert Shadow": 873,
    "10/41 Desert": 874,
    "10/41 Desert Shadow": 875,
    "10/39 Desert": 876,
    "1/39 Desert Shadow": 877,
    "10/8 Desert2": 878,
    "10/8 Desert2 Shadow": 879,
    "10/6 Desert": 880,
    "10/7 Desert": 881,
    "10/7 Desert Shadow": 882,
    "4/6 Desert": 883,
    "4/6 Desert Shadow": 884,
    "4/11 Desert": 885,
    "4/11 Desert Shadow": 886,
    "4/10 Desert": 887,
    "4/10 Desert Shadow": 888,
    "4/9 Desert": 889,
    "4/7 Desert": 890,
    "4/7 Desert Shadow": 891,
    "4/12 Desert": 892,
    "4/12 Desert Shadow": 893,
    "4/8 Desert": 894,
    "4/13 Desert": 895,
    "4/13 Desert Shadow": 896,
    "4/17 Desert": 897,
    "4/15 Desert1": 898,
    "4/15 Desert1 Shadow": 899,
    "10/23 Desert": 900,
    "10/23 Desert Shadow": 901,
    "10/5 Desert": 902,
    "10/5 Desert Shadow": 903,
    "6/13 Desert2": 904,
    "6/13 Desert2 Shadow": 905,
    "6/20 Desert": 906,
    "4/15 Desert2": 907,
    "4/15 Desert2 Shadow": 908,
    "8/23 Desert": 909,
    "8/23 Desert Shadow": 910,
    "12/1 Desert Overlay": 911,
    "11/3 Desert": 912,
    "Unknown913": 913,
    "Lurker Egg": 914,
    "Devourer": 915,
    "Devourer Shadow": 916,
    "Devourer Birth": 917,
    "Devourer Death": 918,
    "Lurker Birth": 919,
    "Lurker Remnants": 920,
    "Lurker": 921,
    "Lurker Shadow": 922,
    "Overmind Cocoon": 923,
    "Overmind Cocoon Shadow": 924,
    "Dark Archon Energy": 925,
    "Dark Archon Being": 926,
    "Dark Archon Swirl": 927,
    "Dark Archon Death": 928,
    "Corsair": 929,
    "Corsair Shadow": 930,
    "Corsair Engines": 931,
    "Neutron Flare Overlay (Unused)": 932,
    "Dark Templar (Unit)": 933,
    "Warp Gate": 934,
    "Warp Gate Shadow": 935,
    "Warp Gate Overlay": 936,
    "Xel'Naga Temple": 937,
    "Xel'Naga Temple Shadow": 938,
    "Valkyrie": 939,
    "Valkyrie Shadow": 940,
    "Valkyrie Engines": 941,
    "Valkyrie Engines2 (Unused)": 942,
    "Valkyrie Afterburners (Unused)": 943,
    "Medic": 944,
    "Medic Shadow": 945,
    "Medic Remnants": 946,
    "Psi Disrupter": 947,
    "Psi Disrupter Shadow": 948,
    "Power Generator": 949,
    "Power Generator Shadow": 950,
    "Disruption Web": 951,
    "Scantid (Desert)": 952,
    "Scantid (Desert) Shadow": 953,
    "Kakaru (Twilight)": 954,
    "Kakaru (Twilight) Shadow": 955,
    "Ursadon (Ice)": 956,
    "Ursadon (Ice) Shadow": 957,
    "Uraj": 958,
    "Khalis": 959,
    "Halo Rockets Trail": 960,
    "Subterranean Spines": 961,
    "Corrosive Acid Shot": 962,
    "Corrosive Acid Hit": 963,
    "Neutron Flare": 964,
    "Halo Rockets": 965,
    "Optical Flare Grenade": 966,
    "Restoration Hit (Small)": 967,
    "Restoration Hit (Medium)": 968,
    "Restoration Hit (Large)": 969,
    "Heal (Small)": 970,
    "Heal (Medium)": 971,
    "Heal (Large)": 972,
    "Mind Control Hit (Small)": 973,
    "Mind Control Hit (Medium)": 974,
    "Mind Control Hit (Large)": 975,
    "Optical Flare Hit (Small)": 976,
    "Optical Flare Hit (Medium)": 977,
    "Optical Flare Hit (Large)": 978,
    "Feedback (Small)": 979,
    "Feedback (Medium)": 980,
    "Feedback Hit (Large)": 981,
    "Maelstrom Overlay (Small)": 982,
    "Maelstrom Overlay (Medium)": 983,
    "Maelstrom Overlay (Large)": 984,
    "Subterranean Spines Hit": 985,
    "Acid Spores (1) Overlay (Small)": 986,
    "Acid Spores (2-3) Overlay (Small)": 987,
    "Acid Spores (4-5) Overlay (Small)": 988,
    "Acid Spores (6-9) Overlay (Small)": 989,
    "Acid Spores (1) Overlay (Medium)": 990,
    "Acid Spores (2-3) Overlay (Medium)": 991,
    "Acid Spores (4-5) Overlay (Medium)": 992,
    "Acid Spores (6-9) Overlay (Medium)": 993,
    "Acid Spores (1) Overlay (Large)": 994,
    "Acid Spores (2-3) Overlay (Large)": 995,
    "Acid Spores (4-5) Overlay (Large)": 996,
    "Acid Spores (6-9) Overlay (Large)": 997,
    "Maelstrom Hit": 998,
}

DefaultImage: TypeAlias = Literal[
    "Scourge",
    "Scourge Shadow",
    "Scourge Birth",
    "Scourge Death",
    "Scourge Explosion",
    "Broodling",
    "Broodling Shadow",
    "Broodling Remnants",
    "Infested Terran",
    "Infested Terran Shadow",
    "Infested Terran Explosion",
    "Guardian Cocoon",
    "Guardian Cocoon Shadow",
    "Defiler",
    "Defiler Shadow",
    "Defiler Birth",
    "Defiler Remnants",
    "Drone",
    "Drone Shadow",
    "Drone Birth",
    "Drone Remnants",
    "Egg",
    "Egg Shadow",
    "Egg Spawn",
    "Egg Remnants",
    "Guardian",
    "Guardian Shadow",
    "Guardian Birth",
    "Guardian Death",
    "Hydralisk",
    "Hydralisk Shadow",
    "Hydralisk Birth",
    "Hydralisk Remnants",
    "Infested Kerrigan",
    "Infested Kerrigan Shadow",
    "Needle Spines",
    "Larva",
    "Larva Remnants",
    "Mutalisk",
    "Mutalisk Shadow",
    "Mutalisk Birth",
    "Mutalisk Death",
    "Overlord",
    "Overlord Shadow",
    "Overlord Birth",
    "Overlord Death",
    "Queen",
    "Queen Shadow",
    "Queen Death",
    "Queen Birth",
    "Ultralisk",
    "Ultralisk Shadow",
    "Ultralisk Birth",
    "Ultralisk Remnants",
    "Zergling",
    "Zergling Shadow",
    "Zergling Birth",
    "Zergling Remnants",
    "Zerg Air Death Explosion (Large)",
    "Zerg Air Death Explosion (Small)",
    "Zerg Building Explosion",
    "Cerebrate",
    "Cerebrate Shadow",
    "Infested Command Center",
    "Spawning Pool",
    "Spawning Pool Shadow",
    "Evolution Chamber",
    "Evolution Chamber Shadow",
    "Creep Colony",
    "Creep Colony Shadow",
    "Hatchery",
    "Hatchery Shadow",
    "Hive",
    "Hive Shadow",
    "Lair",
    "Lair Shadow",
    "Sunken Colony",
    "Sunken Colony Shadow",
    "Mature Chrysalis",
    "Mature Chrysalis Shadow",
    "Greater Spire",
    "Greater Spire Shadow",
    "Defiler Mound",
    "Defiler Mound Shadow",
    "Queen's Nest",
    "Queen's Nest Shadow",
    "Nydus Canal",
    "Nydus Canal Shadow",
    "Overmind With Shell",
    "Overmind Remnants",
    "Overmind Without Shell",
    "Ultralisk Cavern",
    "Ultralisk Cavern Shadow",
    "Extractor",
    "Extractor Shadow",
    "Hydralisk Den",
    "Hydralisk Den Shadow",
    "Spire",
    "Spire Shadow",
    "Spore Colony",
    "Spore Colony Shadow",
    "Infested Command Center Overlay",
    "Zerg Construction (Large)",
    "Zerg Building Morph",
    "Zerg Construction (Medium)",
    "Zerg Construction (Small)",
    "Zerg Building Construction Shadow",
    "Zerg Building Spawn (Small)",
    "Zerg Building Spawn (Medium)",
    "Zerg Building Spawn (Large)",
    "Zerg Building Rubble (Small)",
    "Zerg Building Rubble (Large)",
    "Carrier",
    "Carrier Shadow",
    "Carrier Engines",
    "Carrier Warp Flash",
    "Interceptor",
    "Interceptor Shadow",
    "Shuttle",
    "Shuttle Shadow",
    "Shuttle Engines",
    "Shuttle Warp Flash",
    "Dragoon",
    "Dragoon Shadow",
    "Dragoon Remnants",
    "Dragoon Warp Flash",
    "High Templar",
    "High Templar Shadow",
    "High Templar Warp Flash",
    "Dark Templar (Hero)",
    "Arbiter",
    "Arbiter Shadow",
    "Arbiter Engines",
    "Arbiter Warp Flash",
    "Archon Energy",
    "Archon Being",
    "Archon Swirl",
    "Probe",
    "Probe Shadow",
    "Probe Warp Flash",
    "Scout",
    "Scout Shadow",
    "Scout Engines",
    "Scout Warp Flash",
    "Reaver",
    "Reaver Shadow",
    "Reaver Warp Flash",
    "Scarab",
    "Observer",
    "Observer Shadow",
    "Observer Warp Flash",
    "Zealot",
    "Zealot Shadow",
    "Zealot Death",
    "Zealot Warp Flash",
    "Templar Archives",
    "Templar Archives Warp Flash",
    "Templar Archives Shadow",
    "Assimilator",
    "Assimilator Warp Flash",
    "Assimilator Shadow",
    "Observatory",
    "Observatory Warp Flash",
    "Observatory Shadow",
    "Citadel of Adun",
    "Citadel of Adun Warp Flash",
    "Citadel of Adun Shadow",
    "Forge",
    "Forge Overlay",
    "Forge Warp Flash",
    "Forge Shadow",
    "Gateway",
    "Gateway Warp Flash",
    "Gateway Shadow",
    "Cybernetics Core",
    "Cybernetics Core Warp Flash",
    "Cybernetics Core Overlay",
    "Cybernetics Core Shadow",
    "Khaydarin Crystal Formation",
    "Nexus",
    "Nexus Warp Flash",
    "Nexus Overlay",
    "Nexus Shadow",
    "Photon Cannon",
    "Photon Cannon Shadow",
    "Photon Cannon Warp Flash",
    "Arbiter Tribunal",
    "Arbiter Tribunal Warp Flash",
    "Arbiter Tribunal Shadow",
    "Pylon",
    "Pylon Warp Flash",
    "Pylon Shadow",
    "Robotics Facility",
    "Robotics Facility Warp Flash",
    "Robotics Facility Shadow",
    "Shield Battery",
    "Shield Battery Overlay",
    "Shileld Battery Warp Flash",
    "Shield Battery Shadow",
    "Stargate",
    "Stargate Overlay",
    "Stargate Warp Flash",
    "Stargate Shadow",
    "Stasis Cell/Prison",
    "Robotics Support Bay",
    "Robotics Support Bay Warp Flash",
    "Robotics Support Bay Shadow",
    "Protoss Temple",
    "Fleet Beacon",
    "Fleet Beacon Warp Flash",
    "Warp Texture",
    "Warp Anchor",
    "Fleet Beacon Shadow",
    "Explosion1 (Small)",
    "Explosion1 (Medium)",
    "Explosion (Large)",
    "Protoss Building Rubble (Small)",
    "Protoss Building Rubble (Large)",
    "Battlecruiser",
    "Battlecruiser Shadow",
    "Battlecruiser Engines",
    "Civilian",
    "Civilian Shadow",
    "Dropship",
    "Dropship Shadow",
    "Dropship Engines",
    "Firebat",
    "Firebat Shadow",
    "Ghost",
    "Ghost Shadow",
    "Ghost Remnants",
    "Ghost Death",
    "Nuke Beam",
    "Nuke Target Dot",
    "Goliath Base",
    "Goliath Turret",
    "Goliath Shadow",
    "Sarah Kerrigan",
    "Sarah Kerrigan Shadow",
    "Marine",
    "Marine Shadow",
    "Marine Remnants",
    "Marine Death",
    "Wraith",
    "Wraith Shadow",
    "Wraith Engines",
    "Scanner Sweep",
    "SCV",
    "SCV Shadow",
    "SCV Glow",
    "Siege Tank (Tank) Base",
    "Siege Tank (Tank) Turret",
    "Siege Tank (Tank) Base Shadow",
    "Siege Tank (Siege) Base",
    "Siege Tank (Siege) Turret",
    "Siege Tank (Siege) Base Shadow",
    "Vulture",
    "Vulture Shadow",
    "Spider Mine",
    "Spider Mine Shadow",
    "Science Vessel (Base)",
    "Science Vessel (Turret)",
    "Science Vessel Shadow",
    "Terran Academy",
    "Academy Overlay",
    "Academy Shadow",
    "Barracks",
    "Barracks Shadow",
    "Armory",
    "Armory Overlay",
    "Armory Shadow",
    "Comsat Station",
    "Comsat Station Connector",
    "Comsat Station Overlay",
    "Comsat Station Shadow",
    "Command Center",
    "Command Center Overlay",
    "Command Center Shadow",
    "Supply Depot",
    "Supply Depot Overlay",
    "Supply Depot Shadow",
    "Control Tower",
    "Control Tower Connector",
    "Control Tower Overlay",
    "Control Tower Shadow",
    "Factory",
    "Factory Overlay",
    "Factory Shadow",
    "Covert Ops",
    "Covert Ops Connector",
    "Covert Ops Overlay",
    "Covert Ops Shadow",
    "Ion Cannon",
    "Machine Shop",
    "Machine Shop Connector",
    "Machine Shop Shadow",
    "Missile Turret (Base)",
    "Missile Turret (Turret)",
    "Missile Turret (Base) Shadow",
    "Crashed Batlecruiser",
    "Crashed Battlecruiser Shadow",
    "Physics Lab",
    "Physics Lab Connector",
    "Physics Lab Shadow",
    "Bunker",
    "Bunker Shadow",
    "Bunker Overlay",
    "Refinery",
    "Refinery Shadow",
    "Science Facility",
    "Science Facility Overlay",
    "Science Facility Shadow",
    "Nuclear Silo",
    "Nuclear Silo Connector",
    "Nuclear Silo Overlay",
    "Nuclear Silo Shadow",
    "Nuclear Missile",
    "Nuclear Missile Shadow",
    "Nuke Hit",
    "Starport",
    "Starport Overlay",
    "Starport Shadow",
    "Engineering Bay",
    "Engineering Bay Overlay",
    "Engineering Bay Shadow",
    "Terran Construction (Large)",
    "Terran Construction (Large) Shadow",
    "Terran Construction (Medium)",
    "Terran Construction (Medium) Shadow",
    "Addon Construction",
    "Terran Construction (Small)",
    "Terran Construction (Small) Shadow",
    "Explosion2 (Small)",
    "Explosion2 (Medium)",
    "Building Explosion (Large)",
    "Terran Building Rubble (Small)",
    "Terran Building Rubble (Large)",
    "Dark Swarm",
    "Ragnasaur (Ash)",
    "Ragnasaur (Ash) Shadow",
    "Rhynadon (Badlands)",
    "Rhynadon (Badlands) Shadow",
    "Bengalaas (Jungle)",
    "Bengalaas (Jungle) Shadow",
    "Vespene Geyser",
    "Vespene Geyser2",
    "Vespene Geyser Shadow",
    "Mineral Field Type1",
    "Mineral Field Type1 Shadow",
    "Mineral Field Type2",
    "Mineral Field Type2 Shadow",
    "Mineral Field Type3",
    "Mineral Field Type3 Shadow",
    "Independent Starport (Unused)",
    "Zerg Beacon",
    "Zerg Beacon Overlay",
    "Terran Beacon",
    "Terran Beacon Overlay",
    "Protoss Beacon",
    "Protoss Beacon Overlay",
    "Magna Pulse (Unused)",
    "Lockdown Field (Small)",
    "Lockdown Field (Medium)",
    "Lockdown Field (Large)",
    "Stasis Field Hit",
    "Stasis Field (Small)",
    "Stasis Field (Medium)",
    "Stasis Field (Large)",
    "Recharge Shields (Small)",
    "Recharge Shields (Medium)",
    "Recharge Shields (Large)",
    "Defensive Matrix Front (Small)",
    "Defensive Matrix Front (Medium)",
    "Defensive Matrix Front (Large)",
    "Defensive Matrix Back (Small)",
    "Defensive Matrix Back (Medium)",
    "Defensive Matrix Back (Large)",
    "Defensive Matrix Hit (Small)",
    "Defensive Matrix Hit (Medium)",
    "Defensive Matrix Hit (Large)",
    "Irradiate (Small)",
    "Irradiate (Medium)",
    "Irradiate (Large)",
    "Ensnare Cloud",
    "Ensnare Overlay (Small)",
    "Ensnare Overlay (Medium)",
    "Ensnare Overlay (Large)",
    "Plague Cloud",
    "Plague Overlay (Small)",
    "Plague Overlay (Medium)",
    "Plague Overlay (Large)",
    "Recall Field",
    "Flag",
    "Young Chrysalis",
    "Psi Emitter",
    "Data Disc",
    "Khaydarin Crystal",
    "Mineral Chunk Type1",
    "Mineral Chunk Type2",
    "Protoss Gas Orb Type1",
    "Protoss Gas Orb Type2",
    "Zerg Gas Sac Type1",
    "Zerg Gas Sac Type2",
    "Terran Gas Tank Type1",
    "Terran Gas Tank Type2",
    "Mineral Chunk Shadow",
    "Protoss Gas Orb Shadow",
    "Zerg Gas Sac Shadow",
    "Terran Gas Tank Shadow",
    "Data Disk Shadow (Ground)",
    "Data Disk Shadow (Carried)",
    "Flag Shadow (Ground)",
    "Flag Shadow (Carried)",
    "Crystal Shadow (Ground)",
    "Crystal Shadow (Carried)",
    "Young Chrysalis Shadow (Ground)",
    "Young Chrysalis Shadow (Carried)",
    "Psi Emitter Shadow (Ground)",
    "Psi Emitter Shadow (Carried)",
    "Acid Spray (Unused)",
    "Plasma Drip (Unused)",
    "FlameThrower",
    "Longbolt/Gemini Missiles Trail",
    "Burrowing Dust",
    "Shield Overlay",
    "Small Explosion (Unused)",
    "Double Explosion",
    "Phase Disruptor Hit",
    "Nuclear Missile Death",
    "Spider Mine Death",
    "Vespene Geyser Smoke1",
    "Vespene Geyser Smoke2",
    "Vespene Geyser Smoke3",
    "Vespene Geyser Smoke4",
    "Vespene Geyser Smoke5",
    "Vespene Geyser Smoke1 Overlay",
    "Vespene Geyser Smoke2 Overlay",
    "Vespene Geyser Smoke3 Overlay",
    "Vespene Geyser Smoke4 Overlay",
    "Vespene Geyser Smoke5 Overlay",
    "Fragmentation Grenade Hit",
    "Grenade Shot Smoke",
    "Anti-Matter Missile Hit",
    "Scarab/Anti-Matter Missile Overlay",
    "Scarab Hit",
    "Cursor Marker",
    "Battlecruiser Attack Overlay",
    "Burst Lasers Hit",
    "Burst Lasers Overlay (Unused)",
    "High Templar Glow",
    "Flames1 Type1 (Small)",
    "Flames1 Type2 (Small)",
    "Flames1 Type3 (Small)",
    "Flames2 Type3 (Small)",
    "Flames3 Type3 (Small)",
    "Flames4 Type3 (Small)",
    "Flames5 Type3 (Small)",
    "Flames6 Type3 (Small)",
    "Bleeding Variant1 Type1 (Small)",
    "Bleeding Variant1 Type2 (Small)",
    "Bleeding Variant1 Type3 (Small)",
    "Bleeding Variant1 Type4 (Small)",
    "Bleeding Variant2 Type1 (Small)",
    "Bleeding Variant2 Type2 (Small)",
    "Bleeding Variant2 Type3 (Small)",
    "Bleeding Variant2 Type4 (Small)",
    "Flames2 Type1 (Small)",
    "Flames2 Type2 (Small)",
    "Flames7 Type3 (Small)",
    "Flames3 Type1 (Small)",
    "Flames3 Type2 (Small)",
    "Flames8 Type3 (Small)",
    "Flames1 Type1 (Large)",
    "Flames1 Type2 (Large)",
    "Flames1 Type3 (Large)",
    "Flames2 Type3 (Large)",
    "Flames3 Type3 (Large)",
    "Flames4 Type3 (Large)",
    "Flames5 Type3 (Large)",
    "Flames6 Type3 (Large)",
    "Bleeding Variant1 Type1 (Large)",
    "Bleeding Variant1 Type2 (Large)",
    "Bleeding Variant1 Type3 (Large)",
    "Bleeding Variant1 Type4 (Large)",
    "Bleeding Variant2 Type1 (Large)",
    "Bleeding Variant2 Type3 (Large)",
    "Bleeding Variant3 Type3 (Large)",
    "Bleeding Variant2 Type4 (Large)",
    "Flames2 Type1 (Large)",
    "Flames2 Type2 (Large)",
    "Flames7 Type3 (Large)",
    "Flames3 Type1 (Large)",
    "Flames3 Type2 (Large)",
    "Flames8 Type3 (Large)",
    "Building Landing Dust Type1",
    "Building Landing Dust Type2",
    "Building Landing Dust Type3",
    "Building Landing Dust Type4",
    "Building Landing Dust Type5",
    "Building Lifting Dust Type1",
    "Building Lifting Dust Type2",
    "Building Lifting Dust Type3",
    "Building Lifting Dust Type4",
    "White Circle",
    "Needle Spine Hit",
    "Plasma Drip Hit (Unused)",
    "Sunken Colony Tentacle",
    "Venom (Unused Zerg Weapon)",
    "Venom Hit (Unused)",
    "Acid Spore",
    "Acid Spore Hit",
    "Glave Wurm",
    "Glave Wurm/Seeker Spores Hit",
    "Glave Wurm Trail",
    "Seeker Spores Overlay",
    "Seeker Spores",
    "Queen Spell Holder",
    "Consume",
    "Guardian Attack Overlay",
    "Dual Photon Blasters Hit",
    "Particle Beam Hit",
    "Anti-Matter Missile",
    "Pulse Cannon",
    "Phase Disruptor",
    "STA/STS Photon Cannon Overlay",
    "Psionic Storm",
    "Fusion Cutter Hit",
    "Gauss Rifle Hit",
    "Gemini Missiles",
    "Lockdown/LongBolt/Hellfire Missile",
    "Gemini Missiles Explosion",
    "C-10 Canister Rifle Hit",
    "Fragmentation Grenade",
    "Arclite Shock Cannon Hit",
    "ATS/ATA Laser Battery",
    "Burst Lasers",
    "Siege Tank(Tank) Turret Attack Overlay",
    "Siege Tank(Siege) Turret Attack Overlay",
    "Science Vessel Overlay (Part1)",
    "Science Vessel Overlay (Part2)",
    "Science Vessel Overlay (Part3)",
    "Yamato Gun",
    "Yamato Gun Trail",
    "Yamato Gun Overlay",
    "Yamato Gun Hit",
    "Hallucination Hit",
    "Scanner Sweep Hit",
    "Archon Birth (Unused)",
    "Psionic Shockwave Hit",
    "Archon Beam",
    "Psionic Storm Part1",
    "Psionic Storm Part2",
    "Psionic Storm Part3",
    "Psionic Storm Part4",
    "EMP Shockwave Missile",
    "EMP Shockwave Hit (Part1)",
    "EMP Shockwave Hit (Part2)",
    "Hallucination Death1",
    "Hallucination Death2",
    "Hallucination Death3",
    "Circle Marker1",
    "Selection Circle (22pixels)",
    "Selection Circle (32pixels)",
    "Selection Circle (48pixels)",
    "Selection Circle (62pixels)",
    "Selection Circle (72pixels)",
    "Selection Circle (94pixels)",
    "Selection Circle (110pixels)",
    "Selection Circle (122pixels)",
    "Selection Circle (146pixels)",
    "Selection Circle (224pixels)",
    "Selection Circle Dashed (22pixels)",
    "Selection Circle Dashed (32pixels)",
    "Selection Circle Dashed (48pixels)",
    "Selection Circle Dashed (62pixels)",
    "Selection Circle Dashed (72pixels)",
    "Selection Circle Dashed (94pixels)",
    "Selection Circle Dashed (110pixels)",
    "Selection Circle Dashed (122pixels)",
    "Selection Circle Dashed (146pixels)",
    "Selection Circle Dashed (224pixels)",
    "Circle Marker2",
    "Map Revealer",
    "Circle Marker3",
    "Psi Field1 (Right Upper)",
    "Psi Field1 (Right Lower)",
    "Psi Field2 (Right Upper)",
    "Psi Field2 (Right Lower)",
    "Start Location",
    "2/38 Ash",
    "2/38 Ash Shadow",
    "2/39 Ash",
    "2/39 Ash Shadow",
    "2/41 Ash",
    "2/41 Ash Shadow",
    "2/40 Ash",
    "2/40 Ash Shadow",
    "2/42 Ash",
    "2/42 Ash Shadow",
    "2/43 Ash",
    "2/44 Ash",
    "2/1 Ash",
    "2/4 Ash",
    "2/5 Ash",
    "2/30 Ash",
    "2/28 Ash",
    "2/29 Ash",
    "4/1 Ash",
    "4/2 Ash",
    "4/3 Ash",
    "4/56 Jungle",
    "4/56 Jungle Shadow",
    "4/57 Jungle",
    "4/57 Jungle Shadow",
    "4/58 Jungle",
    "4/58 Jungle Shadow",
    "4/59 Jungle",
    "4/59 Jungle Shadow",
    "9/5 Jungle",
    "9/5 Jungle Shadow",
    "9/6 Jungle",
    "9/6 Jungle Shadow",
    "9/7 Jungle",
    "9/7 Jungle Shadow",
    "4/51 Jungle",
    "4/51 Jungle Shadow",
    "4/52 Jungle",
    "4/52 Jungle Shadow",
    "4/54 Jungle",
    "4/54 Jungle Shadow",
    "4/53 Jungle",
    "4/53 Jungle Shadow",
    "9/1 Jungle",
    "9/1 Jungle Shadow",
    "9/2 Jungle",
    "9/2 Jungle Shadow",
    "9/3 Jungle",
    "9/3 Jungle Shadow",
    "9/4 Jungle",
    "9/4 Jungle Shadow",
    "4/12 Jungle",
    "4/13 Jungle",
    "4/1 Jungle",
    "4/3 Jungle",
    "4/2 Jungle",
    "4/5 Jungle",
    "4/4 Jungle",
    "4/9 Jungle",
    "4/10 Jungle",
    "5/5 Jungle",
    "5/7 Jungle",
    "5/6 Jungle",
    "5/9 Jungle",
    "5/8 Jungle",
    "4/6 Jungle",
    "4/7 Jungle",
    "4/17 Jungle",
    "13/4 Jungle",
    "11/5 Jungle",
    "11/6 Jungle",
    "11/7 Jungle",
    "11/8 Jungle",
    "11/10 Jungle",
    "11/11 Jungle",
    "11/12 Jungle",
    "7/4 Platform",
    "7/4 Platform Shadow",
    "7/5 Platform",
    "7/5 Platform Shadow",
    "7/6 Platform",
    "7/6 Platform Shadow",
    "7/1 Platform",
    "7/1 Platform Shadow",
    "7/2 Platform",
    "7/2 Platform Shadow",
    "7/3 Platform",
    "7/3 Platform Shadow",
    "7/9 Platform",
    "7/10 Platform",
    "7/8 Platform",
    "7/7 Platform",
    "7/26 Platform",
    "7/24 Platform",
    "7/28 Platform",
    "7/27 Platform",
    "7/25 Platform",
    "7/29 Platform",
    "7/30 Platform",
    "7/31 Platform",
    "12/1 Platform",
    "9/27 Platform",
    "5/54 Badlands",
    "5/54 Badlands Shadow",
    "5/55 Badlands",
    "5/55 Badlands Shadow",
    "5/56 Badlands",
    "5/56 Badlands Shadow",
    "5/57 Badlands",
    "5/57 Badlands Shadow",
    "6/16 Badlands",
    "6/17 Badlands",
    "6/20 Badlands",
    "6/21 Badlands",
    "5/10 Badlands",
    "5/50 Badlands",
    "5/50 Badlands Shadow",
    "5/52 Badlands",
    "5/52 Badlands Shadow",
    "5/53 Badlands",
    "5/53 Badlands Shadow",
    "5/51 Badlands",
    "5/51 Badlands Shadow",
    "6/3 Badlands",
    "11/3 Badlands",
    "11/8 Badlands",
    "11/6 Badlands",
    "11/7 Badlands",
    "11/9 Badlands",
    "11/10 Badlands",
    "11/11 Badlands",
    "11/12 Badlands",
    "11/13 Badlands",
    "11/14 Badlands",
    "1/13 Badlands",
    "1/9 Badlands",
    "1/11 Badlands",
    "1/14 Badlands",
    "1/10 Badlands",
    "1/12 Badlands",
    "1/15 Badlands",
    "1/7 Badlands",
    "1/5 Badlands",
    "1/16 Badlands",
    "1/8 Badlands",
    "1/6 Badlands",
    "Floor Gun Trap",
    "Floor Missile Trap",
    "Floor Missile Trap Turret",
    "Wall Missile Trap",
    "Wall Missile Trap2",
    "Wall Flame Trap",
    "Wall Flame Trap2",
    "Left Upper Level Door",
    "Right Upper Level Door",
    "4/15 Installation1",
    "4/15 Installation2",
    "3/9 Installation",
    "3/10 Installation",
    "3/11 Installation",
    "3/12 Installation",
    "Substructure Left Door",
    "Substructure Right Door",
    "3/1 Installation",
    "3/2 Installation",
    "Substructure Opening Hole",
    "7/21 Twilight",
    "Unknown Twilight",
    "7/13 Twilight",
    "7/14 Twilight",
    "7/16 Twilight",
    "7/15 Twilight",
    "7/19 Twilight",
    "7/20 Twilight",
    "7/17 Twilight",
    "6/1 Twilight",
    "6/2 Twilight",
    "6/3 Twilight",
    "6/4 Twilight",
    "6/5 Twilight",
    "8/3 Twilight1",
    "8/3 Twilight2",
    "9/29 Ice",
    "9/29 Ice Shadow",
    "9/28 Ice",
    "9/28 Ice Shadow",
    "12/38 Ice",
    "12/38 Ice Shadow",
    "12/37 Ice",
    "12/37 Ice Shadow",
    "12/33 Ice1",
    "12/33 Ice1 Shadow",
    "9/21 Ice",
    "9/21 Ice Shadow",
    "9/15 Ice",
    "9/15 Ice Shadow",
    "9/16 Ice",
    "9/16 Ice Shadow",
    "Unknown787",
    "Unknown788",
    "12/9 Ice1",
    "12/10 Ice1",
    "9/24 Ice",
    "9/24 Ice Shadow",
    "9/23 Ice",
    "9/23 Ice Shadow",
    "Unknown795",
    "Unknown Ice Shadow",
    "12/7 Ice",
    "12/7 Ice Shadow",
    "12/8 Ice",
    "12/8 Ice Shadow",
    "12/9 Ice2",
    "12/9 Ice2 Shadow",
    "12/10 Ice2",
    "12/10 Ice2 Shadow",
    "12/40 Ice",
    "12/40 Ice Shadow",
    "12/41 Ice",
    "12/41 Ice Shadow",
    "12/42 Ice",
    "12/42 Ice Shadow",
    "12/5 Ice",
    "12/5 Ice Shadow",
    "12/6 Ice",
    "12/6 Ice Shadow",
    "12/36 Ice",
    "12/36 Ice Shadow",
    "12/32 Ice",
    "12/32 Ice Shadow",
    "12/33 Ice2",
    "12/33 Ice2 Shadow",
    "12/34 Ice",
    "12/34 Ice Shadow",
    "12/24 Ice1",
    "12/24 Ice1 Shadow",
    "12/25 Ice1",
    "12/25 Ice1 Shadow",
    "12/30 Ice1",
    "12/30 Ice1 Shadow",
    "12/31 Ice",
    "12/31 Ice Shadow",
    "12/20 Ice",
    "12/30 Ice2",
    "12/30 Ice2 Shadow",
    "9/22 Ice",
    "9/22 Ice Shadow",
    "12/24 Ice2",
    "12/24 Ice2 Shadow",
    "12/25 Ice2",
    "12/25 Ice2 Shadow",
    "Unknown840",
    "4/1 Ice",
    "6/1 Ice",
    "5/6 Ice",
    "5/6 Ice Shadow",
    "5/7 Ice",
    "5/7 Ice Shadow",
    "5/8 Ice",
    "5/8 Ice Shadow",
    "5/9 Ice",
    "5/9 Ice Shadow",
    "10/10 Desert1",
    "10/12 Desert1",
    "10/12 Desert1 Shadow",
    "10/8 Desert1",
    "10/8 Desert1 Shadow",
    "10/9 Desert1",
    "10/9 Desert1 Shadow",
    "6/10 Desert",
    "6/10 Desert Shadow",
    "6/13 Desert1",
    "6/13 Desert1 Shadow",
    "Unknown Desert",
    "Unknown Desert Shadow",
    "10/12 Desert2",
    "10/12 Desert2 Shadow",
    "10/9 Desert2",
    "10/9 Desert2 Shadow",
    "10/10 Desert2",
    "10/10 Desert2 Shadow",
    "10/11 Desert",
    "10/11 Desert Shadow",
    "10/14 Desert",
    "10/14 Desert Shadow",
    "10/41 Desert",
    "10/41 Desert Shadow",
    "10/39 Desert",
    "1/39 Desert Shadow",
    "10/8 Desert2",
    "10/8 Desert2 Shadow",
    "10/6 Desert",
    "10/7 Desert",
    "10/7 Desert Shadow",
    "4/6 Desert",
    "4/6 Desert Shadow",
    "4/11 Desert",
    "4/11 Desert Shadow",
    "4/10 Desert",
    "4/10 Desert Shadow",
    "4/9 Desert",
    "4/7 Desert",
    "4/7 Desert Shadow",
    "4/12 Desert",
    "4/12 Desert Shadow",
    "4/8 Desert",
    "4/13 Desert",
    "4/13 Desert Shadow",
    "4/17 Desert",
    "4/15 Desert1",
    "4/15 Desert1 Shadow",
    "10/23 Desert",
    "10/23 Desert Shadow",
    "10/5 Desert",
    "10/5 Desert Shadow",
    "6/13 Desert2",
    "6/13 Desert2 Shadow",
    "6/20 Desert",
    "4/15 Desert2",
    "4/15 Desert2 Shadow",
    "8/23 Desert",
    "8/23 Desert Shadow",
    "12/1 Desert Overlay",
    "11/3 Desert",
    "Unknown913",
    "Lurker Egg",
    "Devourer",
    "Devourer Shadow",
    "Devourer Birth",
    "Devourer Death",
    "Lurker Birth",
    "Lurker Remnants",
    "Lurker",
    "Lurker Shadow",
    "Overmind Cocoon",
    "Overmind Cocoon Shadow",
    "Dark Archon Energy",
    "Dark Archon Being",
    "Dark Archon Swirl",
    "Dark Archon Death",
    "Corsair",
    "Corsair Shadow",
    "Corsair Engines",
    "Neutron Flare Overlay (Unused)",
    "Dark Templar (Unit)",
    "Warp Gate",
    "Warp Gate Shadow",
    "Warp Gate Overlay",
    "Xel'Naga Temple",
    "Xel'Naga Temple Shadow",
    "Valkyrie",
    "Valkyrie Shadow",
    "Valkyrie Engines",
    "Valkyrie Engines2 (Unused)",
    "Valkyrie Afterburners (Unused)",
    "Medic",
    "Medic Shadow",
    "Medic Remnants",
    "Psi Disrupter",
    "Psi Disrupter Shadow",
    "Power Generator",
    "Power Generator Shadow",
    "Disruption Web",
    "Scantid (Desert)",
    "Scantid (Desert) Shadow",
    "Kakaru (Twilight)",
    "Kakaru (Twilight) Shadow",
    "Ursadon (Ice)",
    "Ursadon (Ice) Shadow",
    "Uraj",
    "Khalis",
    "Halo Rockets Trail",
    "Subterranean Spines",
    "Corrosive Acid Shot",
    "Corrosive Acid Hit",
    "Neutron Flare",
    "Halo Rockets",
    "Optical Flare Grenade",
    "Restoration Hit (Small)",
    "Restoration Hit (Medium)",
    "Restoration Hit (Large)",
    "Heal (Small)",
    "Heal (Medium)",
    "Heal (Large)",
    "Mind Control Hit (Small)",
    "Mind Control Hit (Medium)",
    "Mind Control Hit (Large)",
    "Optical Flare Hit (Small)",
    "Optical Flare Hit (Medium)",
    "Optical Flare Hit (Large)",
    "Feedback (Small)",
    "Feedback (Medium)",
    "Feedback Hit (Large)",
    "Maelstrom Overlay (Small)",
    "Maelstrom Overlay (Medium)",
    "Maelstrom Overlay (Large)",
    "Subterranean Spines Hit",
    "Acid Spores (1) Overlay (Small)",
    "Acid Spores (2-3) Overlay (Small)",
    "Acid Spores (4-5) Overlay (Small)",
    "Acid Spores (6-9) Overlay (Small)",
    "Acid Spores (1) Overlay (Medium)",
    "Acid Spores (2-3) Overlay (Medium)",
    "Acid Spores (4-5) Overlay (Medium)",
    "Acid Spores (6-9) Overlay (Medium)",
    "Acid Spores (1) Overlay (Large)",
    "Acid Spores (2-3) Overlay (Large)",
    "Acid Spores (4-5) Overlay (Large)",
    "Acid Spores (6-9) Overlay (Large)",
    "Maelstrom Hit",
]
