; Storm definition file with alternate Storm.dll names
LIBRARY      "Storm"

EXPORTS
    StormCloseArchive      @252      ; 0x0FC
    StormCloseFile         @253      ; 0x0FD
    StormDestroy           @262      ; 0x106
    StormGetFileArchive    @264      ; 0x108
    StormGetFileSize       @265      ; 0x109
    StormOpenArchive       @266      ; 0x10A
    StormOpenFile          @267      ; 0x10B
    StormOpenFileEx        @268      ; 0x10C
    StormReadFile          @269      ; 0x10D
    StormSetBasePath       @270      ; 0x10E
    StormSetFilePointer    @271      ; 0x10F
    StormSetLocale         @272      ; 0x110
    StormGetBasePath       @273      ; 0x111
    StormGetArchiveName    @275      ; 0x113
    StormGetFileName       @276      ; 0x114

;   StormSetLastError      @465      ; 0x

    StormCompress          @551      ; 0x227
    StormDecompress        @552      ; 0x228
    