// This is a source file for WDK build of StormLib
// It is copied to the root folder during the build process

#include "src\adpcm\adpcm.cpp"
#include "src\huffman\huff.cpp"
#include "src\sparse\sparse.cpp"

#include "src\FileStream.cpp"
#include "src\SBaseCommon.cpp"
#include "src\SBaseDumpData.cpp"
#include "src\SBaseFileTable.cpp"
#include "src\SBaseSubTypes.cpp"
#include "src\SCompression.cpp"
#include "src\SFileAddFile.cpp"
#include "src\SFileAttributes.cpp"
#include "src\SFileCompactArchive.cpp"
#include "src\SFileCreateArchive.cpp"
#include "src\SFileExtractFile.cpp"
#include "src\SFileFindFile.cpp"
#include "src\SFileGetFileInfo.cpp"
#include "src\SFileListFile.cpp"
#include "src\SFileOpenArchive.cpp"
#include "src\SFileOpenFileEx.cpp"
#include "src\SFilePatchArchives.cpp"
#include "src\SFileReadFile.cpp"
#include "src\SFileVerify.cpp"
