from typing import Literal, TypeAlias

DefSpriteDict: dict[str, int] = {
    "2/38 Ash": 0,
    "2/39 Ash": 1,
    "2/41 Ash": 2,
    "2/40 Ash": 3,
    "2/42 Ash": 4,
    "2/43 Ash": 5,
    "2/44 Ash": 6,
    "2/1 Ash": 7,
    "2/4 Ash": 8,
    "2/5 Ash": 9,
    "2/30 Ash": 10,
    "2/28 Ash": 11,
    "2/29 Ash": 12,
    "4/1 Ash": 13,
    "4/2 Ash": 14,
    "4/3 Ash": 15,
    "4/56 Jungle": 16,
    "4/57 Jungle": 17,
    "4/58 Jungle": 18,
    "4/59 Jungle": 19,
    "9/5 Jungle": 20,
    "9/6 Jungle": 21,
    "9/7 Jungle": 22,
    "4/51 Jungle": 23,
    "4/52 Jungle": 24,
    "4/54 Jungle": 25,
    "4/53 Jungle": 26,
    "9/1 Jungle": 27,
    "9/2 Jungle": 28,
    "9/3 Jungle": 29,
    "9/4 Jungle": 30,
    "4/12 Jungle": 31,
    "4/13 Jungle": 32,
    "4/1 Jungle": 33,
    "4/3 Jungle": 34,
    "4/2 Jungle": 35,
    "4/5 Jungle": 36,
    "4/4 Jungle": 37,
    "4/9 Jungle": 38,
    "4/10 Jungle": 39,
    "5/5 Jungle": 40,
    "5/7 Jungle": 41,
    "5/6 Jungle": 42,
    "5/9 Jungle": 43,
    "5/8 Jungle": 44,
    "4/6 Jungle": 45,
    "4/7 Jungle": 46,
    "4/17 Jungle": 47,
    "13/4 Jungle": 48,
    "11/5 Jungle": 49,
    "11/6 Jungle": 50,
    "11/7 Jungle": 51,
    "11/8 Jungle": 52,
    "11/10 Jungle": 53,
    "11/11 Jungle": 54,
    "11/12 Jungle": 55,
    "7/4 Platform": 56,
    "7/5 Platform": 57,
    "7/6 Platform": 58,
    "7/1 Platform": 59,
    "7/2 Platform": 60,
    "7/3 Platform": 61,
    "7/9 Platform": 62,
    "7/10 Platform": 63,
    "7/8 Platform": 64,
    "7/7 Platform": 65,
    "7/26 Platform": 66,
    "7/24 Platform": 67,
    "7/28 Platform": 68,
    "7/27 Platform": 69,
    "7/25 Platform": 70,
    "7/29 Platform": 71,
    "7/30 Platform": 72,
    "7/31 Platform": 73,
    "12/1 Platform": 74,
    "9/27 Platform": 75,
    "5/54 Badlands": 76,
    "5/55 Badlands": 77,
    "5/56 Badlands": 78,
    "5/57 Badlands": 79,
    "6/16 Badlands": 80,
    "6/17 Badlands": 81,
    "6/20 Badlands": 82,
    "6/21 Badlands": 83,
    "5/10 Badlands": 84,
    "5/50 Badlands": 85,
    "5/52 Badlands": 86,
    "5/53 Badlands": 87,
    "5/51 Badlands": 88,
    "6/3 Badlands": 89,
    "11/3 Badlands": 90,
    "11/8 Badlands": 91,
    "11/6 Badlands": 92,
    "11/7 Badlands": 93,
    "11/9 Badlands": 94,
    "11/10 Badlands": 95,
    "11/11 Badlands": 96,
    "11/12 Badlands": 97,
    "11/13 Badlands": 98,
    "11/14 Badlands": 99,
    "1/13 Badlands": 100,
    "1/9 Badlands": 101,
    "1/11 Badlands": 102,
    "1/14 Badlands": 103,
    "1/10 Badlands": 104,
    "1/12 Badlands": 105,
    "1/15 Badlands": 106,
    "1/7 Badlands": 107,
    "1/5 Badlands": 108,
    "1/16 Badlands": 109,
    "1/8 Badlands": 110,
    "1/6 Badlands1": 111,
    "1/6 Badlands2": 112,
    "1/6 Badlands3": 113,
    "1/6 Badlands4": 114,
    "1/6 Badlands5": 115,
    "1/6 Badlands6": 116,
    "1/6 Badlands7": 117,
    "1/6 Badlands8": 118,
    "4/15 Installation1": 119,
    "4/15 Installation2": 120,
    "3/9 Installation": 121,
    "3/10 Installation": 122,
    "3/11 Installation": 123,
    "3/12 Installation": 124,
    "1/6 Badlands9": 125,
    "1/6 Badlands10": 126,
    "3/1 Installation": 127,
    "3/2 Installation": 128,
    "1/6 Badlands11": 129,
    "Scourge": 130,
    "Scourge Death": 131,
    "Scourge Explosion": 132,
    "Broodling": 133,
    "Broodling Remnants": 134,
    "Infested Terran": 135,
    "Infested Terran Explosion": 136,
    "Guardian Cocoon": 137,
    "Defiler": 138,
    "Defiler Remnants": 139,
    "Drone": 140,
    "Drone Remnants": 141,
    "Egg": 142,
    "Egg Remnants": 143,
    "Guardian": 144,
    "Guardian Death": 145,
    "Hydralisk": 146,
    "Hydralisk Remnants": 147,
    "Infested Kerrigan": 148,
    "Larva": 149,
    "Larva Remnants": 150,
    "Mutalisk": 151,
    "Mutalisk Death": 152,
    "Overlord": 153,
    "Overlord Death": 154,
    "Queen": 155,
    "Queen Death": 156,
    "Ultralisk": 157,
    "Ultralisk Remnants": 158,
    "Zergling": 159,
    "Zergling Remnants": 160,
    "Cerebrate": 161,
    "Infested Command Center": 162,
    "Spawning Pool": 163,
    "Mature Chrysalis": 164,
    "Evolution Chamber": 165,
    "Creep Colony": 166,
    "Hatchery": 167,
    "Hive": 168,
    "Lair": 169,
    "Sunken Colony": 170,
    "Greater Spire": 171,
    "Defiler Mound": 172,
    "Queen's Nest": 173,
    "Nydus Canal": 174,
    "Overmind With Shell": 175,
    "Overmind Without Shell": 176,
    "Ultralisk Cavern": 177,
    "Extractor": 178,
    "Hydralisk Den": 179,
    "Spire": 180,
    "Spore Colony": 181,
    "Zerg Building Spawn (Small)": 182,
    "Zerg Building Spawn (Medium)": 183,
    "Zerg Building Spawn (Large)": 184,
    "Zerg Building Explosion": 185,
    "Zerg Building Rubble (Small)": 186,
    "Zerg Building Rubble (Large)": 187,
    "Arbiter": 188,
    "Archon Energy": 189,
    "Carrier": 190,
    "Dragoon": 191,
    "Dragoon Remnants": 192,
    "Interceptor": 193,
    "Probe": 194,
    "Scout": 195,
    "Shuttle": 196,
    "High Templar": 197,
    "Dark Templar (Hero)": 198,
    "Reaver": 199,
    "Scarab": 200,
    "Zealot": 201,
    "Observer": 202,
    "Templar Archives": 203,
    "Assimilator": 204,
    "Observatory": 205,
    "Citadel of Adun": 206,
    "Forge": 207,
    "Gateway": 208,
    "Cybernetics Core": 209,
    "Khaydarin Crystal Formation": 210,
    "Nexus": 211,
    "Photon Cannon": 212,
    "Arbiter Tribunal": 213,
    "Pylon": 214,
    "Robotics Facility": 215,
    "Shield Battery": 216,
    "Stargate": 217,
    "Stasis Cell/Prison": 218,
    "Robotics Support Bay": 219,
    "Protoss Temple": 220,
    "Fleet Beacon": 221,
    "Explosion (Large)": 222,
    "Protoss Building Rubble (Small)": 223,
    "Protoss Building Rubble (Large)": 224,
    "Battlecruiser": 225,
    "Civilian": 226,
    "Dropship": 227,
    "Firebat": 228,
    "Ghost": 229,
    "Ghost Remnants": 230,
    "Nuke Target Dot": 231,
    "Goliath Base": 232,
    "Goliath Turret": 233,
    "Sarah Kerrigan": 234,
    "Marine": 235,
    "Marine Remnants": 236,
    "Scanner Sweep": 237,
    "Wraith": 238,
    "SCV": 239,
    "Siege Tank (Tank) Base": 240,
    "Siege Tank (Tank) Turret": 241,
    "Siege Tank (Siege) Base": 242,
    "Siege Tank (Siege) Turret": 243,
    "Vulture": 244,
    "Spider Mine": 245,
    "Science Vessel (Base)": 246,
    "Science Vessel (Turret)": 247,
    "Terran Academy": 248,
    "Barracks": 249,
    "Armory": 250,
    "Comsat Station": 251,
    "Command Center": 252,
    "Supply Depot": 253,
    "Control Tower": 254,
    "Factory": 255,
    "Covert Ops": 256,
    "Ion Cannon": 257,
    "Machine Shop": 258,
    "Missile Turret (Base)": 259,
    "Crashed Batlecruiser": 260,
    "Physics Lab": 261,
    "Bunker": 262,
    "Refinery": 263,
    "Science Facility": 264,
    "Nuclear Silo": 265,
    "Nuclear Missile": 266,
    "Nuke Hit": 267,
    "Starport": 268,
    "Engineering Bay": 269,
    "Terran Construction (Large)": 270,
    "Terran Construction (Small)": 271,
    "Building Explosion (Large)": 272,
    "Terran Building Rubble (Small)": 273,
    "Terran Building Rubble (Large)": 274,
    "Vespene Geyser": 275,
    "Ragnasaur (Ash)": 276,
    "Rhynadon (Badlands)": 277,
    "Bengalaas (Jungle)": 278,
    "Mineral Field Type1": 279,
    "Mineral Field Type2": 280,
    "Mineral Field Type3": 281,
    "Independent Starport (Unused)": 282,
    "Zerg Beacon": 283,
    "Terran Beacon": 284,
    "Protoss Beacon": 285,
    "Dark Swarm": 286,
    "Flag": 287,
    "Young Chrysalis": 288,
    "Psi Emitter": 289,
    "Data Disc": 290,
    "Khaydarin Crystal": 291,
    "Mineral Chunk Type1": 292,
    "Mineral Chunk Type2": 293,
    "Protoss Gas Orb Type1": 294,
    "Protoss Gas Orb Type2": 295,
    "Zerg Gas Sac Type1": 296,
    "Zerg Gas Sac Type2": 297,
    "Terran Gas Tank Type1": 298,
    "Terran Gas Tank Type2": 299,
    "White Circle (Invisible)": 300,
    "Start Location": 301,
    "Map Revealer": 302,
    "Floor Gun Trap": 303,
    "Wall Missile Trap": 304,
    "Wall Missile Trap2": 305,
    "Wall Flame Trap": 306,
    "Wall Flame Trap2": 307,
    "Floor Missile Trap": 308,
    "Longbolt/Gemini Missiles Trail": 309,
    "Grenade Shot Smoke": 310,
    "Vespene Geyser Smoke1": 311,
    "Vespene Geyser Smoke2": 312,
    "Vespene Geyser Smoke3": 313,
    "Vespene Geyser Smoke4": 314,
    "Vespene Geyser Smoke5": 315,
    "Small Explosion (Unused)": 316,
    "Double Explosion": 317,
    "Cursor Marker": 318,
    "Egg Spawn": 319,
    "High Templar Glow": 320,
    "Psi Field (Right Upper)": 321,
    "Burrowing Dust": 322,
    "Building Landing Dust Type1": 323,
    "Building Landing Dust Type2": 324,
    "Building Landing Dust Type3": 325,
    "Building Landing Dust Type4": 326,
    "Building Landing Dust Type5": 327,
    "Building Lifting Dust Type1": 328,
    "Building Lifting Dust Type2": 329,
    "Building Lifting Dust Type3": 330,
    "Building Lifting Dust Type4": 331,
    "Needle Spines": 332,
    "Dual Photon Blasters Hit": 333,
    "Particle Beam Hit": 334,
    "Anti-Matter Missile": 335,
    "Pulse Cannon": 336,
    "Phase Disruptor": 337,
    "STA/STS Photon Cannon Overlay": 338,
    "Psionic Storm": 339,
    "Fusion Cutter Hit": 340,
    "Gauss Rifle Hit": 341,
    "Gemini Missiles": 342,
    "Fragmentation Grenade": 343,
    "Magna Pulse (Unused)": 344,
    "Lockdown/LongBolt/Hellfire Missile": 345,
    "C-10 Canister Rifle Hit": 346,
    "ATS/ATA Laser Battery": 347,
    "Burst Lasers": 348,
    "Arclite Shock Cannon Hit": 349,
    "Yamato Gun": 350,
    "Yamato Gun Trail": 351,
    "EMP Shockwave Missile": 352,
    "Needle Spine Hit": 353,
    "Plasma Drip Hit (Unused)": 354,
    "Sunken Colony Tentacle": 355,
    "Venom (Unused Zerg Weapon)": 356,
    "Acid Spore": 357,
    "Glave Wurm": 358,
    "Seeker Spores": 359,
    "Queen Spell Holder": 360,
    "Stasis Field Hit": 361,
    "Plague Cloud": 362,
    "Consume": 363,
    "Ensnare": 364,
    "Glave Wurm/Seeker Spores Hit": 365,
    "Psionic Shockwave Hit": 366,
    "Glave Wurm Trail": 367,
    "Seeker Spores Overlay": 368,
    "Phase Disruptor (Unused)": 369,
    "White Circle": 370,
    "Acid Spray (Unused)": 371,
    "Plasma Drip (Unused)": 372,
    "Scarab/Anti-Matter Missile Overlay": 373,
    "Hallucination Death1": 374,
    "Hallucination Death2": 375,
    "Hallucination Death3": 376,
    "Bunker Overlay": 377,
    "FlameThrower": 378,
    "Recall Field": 379,
    "Scanner Sweep Hit": 380,
    "Left Upper Level Door": 381,
    "Right Upper Level Door": 382,
    "Substructure Left Door": 383,
    "Substructure Right Door": 384,
    "Substructure Opening Hole": 385,
    "7/13 Twilight": 386,
    "7/14 Twilight": 387,
    "7/16 Twilight": 388,
    "7/15 Twilight": 389,
    "7/19 Twilight": 390,
    "7/20 Twilight": 391,
    "7/21 Twilight": 392,
    "Unknown Twilight": 393,
    "7/17 Twilight": 394,
    "6/1 Twilight": 395,
    "6/2 Twilight": 396,
    "6/3 Twilight": 397,
    "6/4 Twilight": 398,
    "6/5 Twilight": 399,
    "8/3 Twilight": 400,
    "8/4 Twilight": 401,
    "9/29 Ice": 402,
    "9/28 Ice": 403,
    "12/38 Ice": 404,
    "12/37 Ice": 405,
    "12/33 Ice": 406,
    "9/21 Ice": 407,
    "9/15 Ice": 408,
    "9/16 Ice": 409,
    "Unknown410": 410,
    "Unknown411": 411,
    "12/9 Ice1": 412,
    "12/10 Ice": 413,
    "9/24 Ice": 414,
    "9/23 Ice": 415,
    "Unknown416": 416,
    "12/7 Ice": 417,
    "12/8 Ice": 418,
    "12/9 Ice2": 419,
    "12/11 Ice": 420,
    "12/40 Ice": 421,
    "12/41 Ice": 422,
    "12/42 Ice": 423,
    "12/5 Ice": 424,
    "12/6 Ice": 425,
    "12/36 Ice": 426,
    "12/32 Ice": 427,
    "12/35 Ice": 428,
    "12/34 Ice": 429,
    "12/24 Ice": 430,
    "12/25 Ice": 431,
    "9/22 Ice": 432,
    "12/31 Ice": 433,
    "12/20 Ice": 434,
    "12/26 Ice": 435,
    "12/27 Ice": 436,
    "12/30 Ice": 437,
    "12/39 Ice": 438,
    "Unknown439": 439,
    "4/1 Ice": 440,
    "6/1 Ice": 441,
    "5/6 Ice": 442,
    "5/7 Ice": 443,
    "5/8 Ice": 444,
    "5/9 Ice": 445,
    "10/10 Desert1": 446,
    "10/12 Desert1": 447,
    "10/8 Desert": 448,
    "10/9 Desert1": 449,
    "6/10 Desert": 450,
    "6/13 Desert": 451,
    "Unknown Desert": 452,
    "10/12 Desert2": 453,
    "10/9 Desert2": 454,
    "10/10 Desert2": 455,
    "10/11 Desert": 456,
    "10/14 Desert": 457,
    "10/41 Desert": 458,
    "10/39 Desert": 459,
    "10/9 Desert": 460,
    "10/6 Desert": 461,
    "10/7 Desert": 462,
    "4/6 Desert": 463,
    "4/11 Desert": 464,
    "4/10 Desert": 465,
    "4/9 Desert": 466,
    "4/7 Desert": 467,
    "4/12 Desert": 468,
    "4/8 Desert": 469,
    "4/13 Desert": 470,
    "6/14 Desert": 471,
    "4/17 Desert": 472,
    "6/20 Desert": 473,
    "4/15 Desert1": 474,
    "4/15 Desert2": 475,
    "10/23 Desert": 476,
    "8/23 Desert": 477,
    "10/5 Desert": 478,
    "12/1 Desert Overlay": 479,
    "11/3 Desert": 480,
    "Lurker Egg": 481,
    "Devourer": 482,
    "Devourer Death": 483,
    "Lurker Remnants": 484,
    "Lurker": 485,
    "Dark Archon Energy": 486,
    "Corsair": 487,
    "Dark Templar (Unit)": 488,
    "Medic": 489,
    "Medic Remnants": 490,
    "Valkyrie": 491,
    "Scantid (Desert)": 492,
    "Kakaru (Twilight)": 493,
    "Ursadon (Ice)": 494,
    "Overmind Cocoon": 495,
    "Power Generator": 496,
    "Xel'Naga Temple": 497,
    "Psi Disrupter": 498,
    "Warp Gate": 499,
    "Feedback Hit (Small)": 500,
    "Feedback Hit (Medium)": 501,
    "Feedback Hit (Large)": 502,
    "Disruption Web": 503,
    "White Circle2": 504,
    "Halo Rockets Trail": 505,
    "Neutron Flare": 506,
    "Neutron Flare Overlay (Unused)": 507,
    "Optical Flare Grenade": 508,
    "Halo Rockets": 509,
    "Subterranean Spines Hit": 510,
    "Subterranean Spines": 511,
    "Corrosive Acid Shot": 512,
    "Corrosive Acid Hit": 513,
    "Maelstrom Hit": 514,
    "Uraj": 515,
    "Khalis": 516,
}

DefaultSprite: TypeAlias = Literal[
    "2/38 Ash",
    "2/39 Ash",
    "2/41 Ash",
    "2/40 Ash",
    "2/42 Ash",
    "2/43 Ash",
    "2/44 Ash",
    "2/1 Ash",
    "2/4 Ash",
    "2/5 Ash",
    "2/30 Ash",
    "2/28 Ash",
    "2/29 Ash",
    "4/1 Ash",
    "4/2 Ash",
    "4/3 Ash",
    "4/56 Jungle",
    "4/57 Jungle",
    "4/58 Jungle",
    "4/59 Jungle",
    "9/5 Jungle",
    "9/6 Jungle",
    "9/7 Jungle",
    "4/51 Jungle",
    "4/52 Jungle",
    "4/54 Jungle",
    "4/53 Jungle",
    "9/1 Jungle",
    "9/2 Jungle",
    "9/3 Jungle",
    "9/4 Jungle",
    "4/12 Jungle",
    "4/13 Jungle",
    "4/1 Jungle",
    "4/3 Jungle",
    "4/2 Jungle",
    "4/5 Jungle",
    "4/4 Jungle",
    "4/9 Jungle",
    "4/10 Jungle",
    "5/5 Jungle",
    "5/7 Jungle",
    "5/6 Jungle",
    "5/9 Jungle",
    "5/8 Jungle",
    "4/6 Jungle",
    "4/7 Jungle",
    "4/17 Jungle",
    "13/4 Jungle",
    "11/5 Jungle",
    "11/6 Jungle",
    "11/7 Jungle",
    "11/8 Jungle",
    "11/10 Jungle",
    "11/11 Jungle",
    "11/12 Jungle",
    "7/4 Platform",
    "7/5 Platform",
    "7/6 Platform",
    "7/1 Platform",
    "7/2 Platform",
    "7/3 Platform",
    "7/9 Platform",
    "7/10 Platform",
    "7/8 Platform",
    "7/7 Platform",
    "7/26 Platform",
    "7/24 Platform",
    "7/28 Platform",
    "7/27 Platform",
    "7/25 Platform",
    "7/29 Platform",
    "7/30 Platform",
    "7/31 Platform",
    "12/1 Platform",
    "9/27 Platform",
    "5/54 Badlands",
    "5/55 Badlands",
    "5/56 Badlands",
    "5/57 Badlands",
    "6/16 Badlands",
    "6/17 Badlands",
    "6/20 Badlands",
    "6/21 Badlands",
    "5/10 Badlands",
    "5/50 Badlands",
    "5/52 Badlands",
    "5/53 Badlands",
    "5/51 Badlands",
    "6/3 Badlands",
    "11/3 Badlands",
    "11/8 Badlands",
    "11/6 Badlands",
    "11/7 Badlands",
    "11/9 Badlands",
    "11/10 Badlands",
    "11/11 Badlands",
    "11/12 Badlands",
    "11/13 Badlands",
    "11/14 Badlands",
    "1/13 Badlands",
    "1/9 Badlands",
    "1/11 Badlands",
    "1/14 Badlands",
    "1/10 Badlands",
    "1/12 Badlands",
    "1/15 Badlands",
    "1/7 Badlands",
    "1/5 Badlands",
    "1/16 Badlands",
    "1/8 Badlands",
    "1/6 Badlands1",
    "1/6 Badlands2",
    "1/6 Badlands3",
    "1/6 Badlands4",
    "1/6 Badlands5",
    "1/6 Badlands6",
    "1/6 Badlands7",
    "1/6 Badlands8",
    "4/15 Installation1",
    "4/15 Installation2",
    "3/9 Installation",
    "3/10 Installation",
    "3/11 Installation",
    "3/12 Installation",
    "1/6 Badlands9",
    "1/6 Badlands10",
    "3/1 Installation",
    "3/2 Installation",
    "1/6 Badlands11",
    "Scourge",
    "Scourge Death",
    "Scourge Explosion",
    "Broodling",
    "Broodling Remnants",
    "Infested Terran",
    "Infested Terran Explosion",
    "Guardian Cocoon",
    "Defiler",
    "Defiler Remnants",
    "Drone",
    "Drone Remnants",
    "Egg",
    "Egg Remnants",
    "Guardian",
    "Guardian Death",
    "Hydralisk",
    "Hydralisk Remnants",
    "Infested Kerrigan",
    "Larva",
    "Larva Remnants",
    "Mutalisk",
    "Mutalisk Death",
    "Overlord",
    "Overlord Death",
    "Queen",
    "Queen Death",
    "Ultralisk",
    "Ultralisk Remnants",
    "Zergling",
    "Zergling Remnants",
    "Cerebrate",
    "Infested Command Center",
    "Spawning Pool",
    "Mature Chrysalis",
    "Evolution Chamber",
    "Creep Colony",
    "Hatchery",
    "Hive",
    "Lair",
    "Sunken Colony",
    "Greater Spire",
    "Defiler Mound",
    "Queen's Nest",
    "Nydus Canal",
    "Overmind With Shell",
    "Overmind Without Shell",
    "Ultralisk Cavern",
    "Extractor",
    "Hydralisk Den",
    "Spire",
    "Spore Colony",
    "Zerg Building Spawn (Small)",
    "Zerg Building Spawn (Medium)",
    "Zerg Building Spawn (Large)",
    "Zerg Building Explosion",
    "Zerg Building Rubble (Small)",
    "Zerg Building Rubble (Large)",
    "Arbiter",
    "Archon Energy",
    "Carrier",
    "Dragoon",
    "Dragoon Remnants",
    "Interceptor",
    "Probe",
    "Scout",
    "Shuttle",
    "High Templar",
    "Dark Templar (Hero)",
    "Reaver",
    "Scarab",
    "Zealot",
    "Observer",
    "Templar Archives",
    "Assimilator",
    "Observatory",
    "Citadel of Adun",
    "Forge",
    "Gateway",
    "Cybernetics Core",
    "Khaydarin Crystal Formation",
    "Nexus",
    "Photon Cannon",
    "Arbiter Tribunal",
    "Pylon",
    "Robotics Facility",
    "Shield Battery",
    "Stargate",
    "Stasis Cell/Prison",
    "Robotics Support Bay",
    "Protoss Temple",
    "Fleet Beacon",
    "Explosion (Large)",
    "Protoss Building Rubble (Small)",
    "Protoss Building Rubble (Large)",
    "Battlecruiser",
    "Civilian",
    "Dropship",
    "Firebat",
    "Ghost",
    "Ghost Remnants",
    "Nuke Target Dot",
    "Goliath Base",
    "Goliath Turret",
    "Sarah Kerrigan",
    "Marine",
    "Marine Remnants",
    "Scanner Sweep",
    "Wraith",
    "SCV",
    "Siege Tank (Tank) Base",
    "Siege Tank (Tank) Turret",
    "Siege Tank (Siege) Base",
    "Siege Tank (Siege) Turret",
    "Vulture",
    "Spider Mine",
    "Science Vessel (Base)",
    "Science Vessel (Turret)",
    "Terran Academy",
    "Barracks",
    "Armory",
    "Comsat Station",
    "Command Center",
    "Supply Depot",
    "Control Tower",
    "Factory",
    "Covert Ops",
    "Ion Cannon",
    "Machine Shop",
    "Missile Turret (Base)",
    "Crashed Batlecruiser",
    "Physics Lab",
    "Bunker",
    "Refinery",
    "Science Facility",
    "Nuclear Silo",
    "Nuclear Missile",
    "Nuke Hit",
    "Starport",
    "Engineering Bay",
    "Terran Construction (Large)",
    "Terran Construction (Small)",
    "Building Explosion (Large)",
    "Terran Building Rubble (Small)",
    "Terran Building Rubble (Large)",
    "Vespene Geyser",
    "Ragnasaur (Ash)",
    "Rhynadon (Badlands)",
    "Bengalaas (Jungle)",
    "Mineral Field Type1",
    "Mineral Field Type2",
    "Mineral Field Type3",
    "Independent Starport (Unused)",
    "Zerg Beacon",
    "Terran Beacon",
    "Protoss Beacon",
    "Dark Swarm",
    "Flag",
    "Young Chrysalis",
    "Psi Emitter",
    "Data Disc",
    "Khaydarin Crystal",
    "Mineral Chunk Type1",
    "Mineral Chunk Type2",
    "Protoss Gas Orb Type1",
    "Protoss Gas Orb Type2",
    "Zerg Gas Sac Type1",
    "Zerg Gas Sac Type2",
    "Terran Gas Tank Type1",
    "Terran Gas Tank Type2",
    "White Circle (Invisible)",
    "Start Location",
    "Map Revealer",
    "Floor Gun Trap",
    "Wall Missile Trap",
    "Wall Missile Trap2",
    "Wall Flame Trap",
    "Wall Flame Trap2",
    "Floor Missile Trap",
    "Longbolt/Gemini Missiles Trail",
    "Grenade Shot Smoke",
    "Vespene Geyser Smoke1",
    "Vespene Geyser Smoke2",
    "Vespene Geyser Smoke3",
    "Vespene Geyser Smoke4",
    "Vespene Geyser Smoke5",
    "Small Explosion (Unused)",
    "Double Explosion",
    "Cursor Marker",
    "Egg Spawn",
    "High Templar Glow",
    "Psi Field (Right Upper)",
    "Burrowing Dust",
    "Building Landing Dust Type1",
    "Building Landing Dust Type2",
    "Building Landing Dust Type3",
    "Building Landing Dust Type4",
    "Building Landing Dust Type5",
    "Building Lifting Dust Type1",
    "Building Lifting Dust Type2",
    "Building Lifting Dust Type3",
    "Building Lifting Dust Type4",
    "Needle Spines",
    "Dual Photon Blasters Hit",
    "Particle Beam Hit",
    "Anti-Matter Missile",
    "Pulse Cannon",
    "Phase Disruptor",
    "STA/STS Photon Cannon Overlay",
    "Psionic Storm",
    "Fusion Cutter Hit",
    "Gauss Rifle Hit",
    "Gemini Missiles",
    "Fragmentation Grenade",
    "Magna Pulse (Unused)",
    "Lockdown/LongBolt/Hellfire Missile",
    "C-10 Canister Rifle Hit",
    "ATS/ATA Laser Battery",
    "Burst Lasers",
    "Arclite Shock Cannon Hit",
    "Yamato Gun",
    "Yamato Gun Trail",
    "EMP Shockwave Missile",
    "Needle Spine Hit",
    "Plasma Drip Hit (Unused)",
    "Sunken Colony Tentacle",
    "Venom (Unused Zerg Weapon)",
    "Acid Spore",
    "Glave Wurm",
    "Seeker Spores",
    "Queen Spell Holder",
    "Stasis Field Hit",
    "Plague Cloud",
    "Consume",
    "Ensnare",
    "Glave Wurm/Seeker Spores Hit",
    "Psionic Shockwave Hit",
    "Glave Wurm Trail",
    "Seeker Spores Overlay",
    "Phase Disruptor (Unused)",
    "White Circle",
    "Acid Spray (Unused)",
    "Plasma Drip (Unused)",
    "Scarab/Anti-Matter Missile Overlay",
    "Hallucination Death1",
    "Hallucination Death2",
    "Hallucination Death3",
    "Bunker Overlay",
    "FlameThrower",
    "Recall Field",
    "Scanner Sweep Hit",
    "Left Upper Level Door",
    "Right Upper Level Door",
    "Substructure Left Door",
    "Substructure Right Door",
    "Substructure Opening Hole",
    "7/13 Twilight",
    "7/14 Twilight",
    "7/16 Twilight",
    "7/15 Twilight",
    "7/19 Twilight",
    "7/20 Twilight",
    "7/21 Twilight",
    "Unknown Twilight",
    "7/17 Twilight",
    "6/1 Twilight",
    "6/2 Twilight",
    "6/3 Twilight",
    "6/4 Twilight",
    "6/5 Twilight",
    "8/3 Twilight",
    "8/4 Twilight",
    "9/29 Ice",
    "9/28 Ice",
    "12/38 Ice",
    "12/37 Ice",
    "12/33 Ice",
    "9/21 Ice",
    "9/15 Ice",
    "9/16 Ice",
    "Unknown410",
    "Unknown411",
    "12/9 Ice1",
    "12/10 Ice",
    "9/24 Ice",
    "9/23 Ice",
    "Unknown416",
    "12/7 Ice",
    "12/8 Ice",
    "12/9 Ice2",
    "12/11 Ice",
    "12/40 Ice",
    "12/41 Ice",
    "12/42 Ice",
    "12/5 Ice",
    "12/6 Ice",
    "12/36 Ice",
    "12/32 Ice",
    "12/35 Ice",
    "12/34 Ice",
    "12/24 Ice",
    "12/25 Ice",
    "9/22 Ice",
    "12/31 Ice",
    "12/20 Ice",
    "12/26 Ice",
    "12/27 Ice",
    "12/30 Ice",
    "12/39 Ice",
    "Unknown439",
    "4/1 Ice",
    "6/1 Ice",
    "5/6 Ice",
    "5/7 Ice",
    "5/8 Ice",
    "5/9 Ice",
    "10/10 Desert1",
    "10/12 Desert1",
    "10/8 Desert",
    "10/9 Desert1",
    "6/10 Desert",
    "6/13 Desert",
    "Unknown Desert",
    "10/12 Desert2",
    "10/9 Desert2",
    "10/10 Desert2",
    "10/11 Desert",
    "10/14 Desert",
    "10/41 Desert",
    "10/39 Desert",
    "10/9 Desert",
    "10/6 Desert",
    "10/7 Desert",
    "4/6 Desert",
    "4/11 Desert",
    "4/10 Desert",
    "4/9 Desert",
    "4/7 Desert",
    "4/12 Desert",
    "4/8 Desert",
    "4/13 Desert",
    "6/14 Desert",
    "4/17 Desert",
    "6/20 Desert",
    "4/15 Desert1",
    "4/15 Desert2",
    "10/23 Desert",
    "8/23 Desert",
    "10/5 Desert",
    "12/1 Desert Overlay",
    "11/3 Desert",
    "Lurker Egg",
    "Devourer",
    "Devourer Death",
    "Lurker Remnants",
    "Lurker",
    "Dark Archon Energy",
    "Corsair",
    "Dark Templar (Unit)",
    "Medic",
    "Medic Remnants",
    "Valkyrie",
    "Scantid (Desert)",
    "Kakaru (Twilight)",
    "Ursadon (Ice)",
    "Overmind Cocoon",
    "Power Generator",
    "Xel'Naga Temple",
    "Psi Disrupter",
    "Warp Gate",
    "Feedback Hit (Small)",
    "Feedback Hit (Medium)",
    "Feedback Hit (Large)",
    "Disruption Web",
    "White Circle2",
    "Halo Rockets Trail",
    "Neutron Flare",
    "Neutron Flare Overlay (Unused)",
    "Optical Flare Grenade",
    "Halo Rockets",
    "Subterranean Spines Hit",
    "Subterranean Spines",
    "Corrosive Acid Shot",
    "Corrosive Acid Hit",
    "Maelstrom Hit",
    "Uraj",
    "Khalis",
]
