// This is a source file for WDK build of StormLib
// It is copied to the root folder during the build process

#include "src\lzma\C\LzFind.c"
#include "src\lzma\C\LzFindMt.c"
#include "src\lzma\C\LzmaDec.c"
#include "src\lzma\C\LzmaEnc.c"
#include "src\lzma\C\Threads.c"
