# Copyright 2023 by Armoha.
# All rights reserved.
# This file is part of EUD python library (eudplib),
# and is released under "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

from collections.abc import Iterator

def generate_linetable(
    data: bytes, linetable: bytes, positions: Iterator
) -> bytes: ...
