# Copyright 2014 by trgk.
# All rights reserved.
# This file is part of EUD python library (eudplib),
# and is released under "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

from .runtrigtrg import EUDLoopTrigger
from .trigtrg import (
    Accumulate,
    Action,
    Always,
    Briefing,
    Bring,
    CenterView,
    Command,
    CommandLeast,
    CommandLeastAt,
    CommandMost,
    CommandMostAt,
    Comment,
    Condition,
    CountdownTimer,
    CreateUnit,
    CreateUnitWithProperties,
    Deaths,
    DeathsX,
    Defeat,
    DisplayText,
    Draw,
    ElapsedTime,
    GiveUnits,
    HighestScore,
    Kills,
    KillUnit,
    KillUnitAt,
    LeaderBoardComputerPlayers,
    LeaderBoardControl,
    LeaderBoardControlAt,
    LeaderBoardGoalControl,
    LeaderBoardGoalControlAt,
    LeaderBoardGoalKills,
    LeaderBoardGoalResources,
    LeaderBoardGoalScore,
    LeaderBoardGreed,
    LeaderBoardKills,
    LeaderBoardResources,
    LeaderBoardScore,
    LeastKills,
    LeastResources,
    LowestScore,
    Memory,
    MemoryX,
    MinimapPing,
    ModifyUnitEnergy,
    ModifyUnitHangarCount,
    ModifyUnitHitPoints,
    ModifyUnitResourceAmount,
    ModifyUnitShields,
    MostKills,
    MostResources,
    MoveLocation,
    MoveUnit,
    MuteUnitSpeech,
    Never,
    NoAction,
    NoCondition,
    Opponents,
    Order,
    PauseGame,
    PauseTimer,
    PlayWAV,
    PreserveTrigger,
    RemoveUnit,
    RemoveUnitAt,
    RunAIScript,
    RunAIScriptAt,
    Score,
    SetAllianceStatus,
    SetCountdownTimer,
    SetDeaths,
    SetDeathsX,
    SetDoodadState,
    SetInvincibility,
    SetMemory,
    SetMemoryX,
    SetMissionObjectives,
    SetNextScenario,
    SetResources,
    SetScore,
    SetSwitch,
    Switch,
    TalkingPortrait,
    Transmission,
    Trigger,
    UnMuteUnitSpeech,
    UnpauseGame,
    UnpauseTimer,
    Victory,
    Wait,
)

__all__ = [
    "EUDLoopTrigger",
    "Accumulate",
    "Action",
    "Always",
    "Briefing",
    "Bring",
    "CenterView",
    "Command",
    "CommandLeast",
    "CommandLeastAt",
    "CommandMost",
    "CommandMostAt",
    "Comment",
    "Condition",
    "CountdownTimer",
    "CreateUnit",
    "CreateUnitWithProperties",
    "Deaths",
    "DeathsX",
    "Defeat",
    "DisplayText",
    "Draw",
    "ElapsedTime",
    "GiveUnits",
    "HighestScore",
    "Kills",
    "KillUnit",
    "KillUnitAt",
    "LeaderBoardComputerPlayers",
    "LeaderBoardControl",
    "LeaderBoardControlAt",
    "LeaderBoardGoalControl",
    "LeaderBoardGoalControlAt",
    "LeaderBoardGoalKills",
    "LeaderBoardGoalResources",
    "LeaderBoardGoalScore",
    "LeaderBoardGreed",
    "LeaderBoardKills",
    "LeaderBoardResources",
    "LeaderBoardScore",
    "LeastKills",
    "LeastResources",
    "LowestScore",
    "Memory",
    "MemoryX",
    "MinimapPing",
    "ModifyUnitEnergy",
    "ModifyUnitHangarCount",
    "ModifyUnitHitPoints",
    "ModifyUnitResourceAmount",
    "ModifyUnitShields",
    "MostKills",
    "MostResources",
    "MoveLocation",
    "MoveUnit",
    "MuteUnitSpeech",
    "Never",
    "NoAction",
    "NoCondition",
    "Opponents",
    "Order",
    "PauseGame",
    "PauseTimer",
    "PlayWAV",
    "PreserveTrigger",
    "RemoveUnit",
    "RemoveUnitAt",
    "RunAIScript",
    "RunAIScriptAt",
    "Score",
    "SetAllianceStatus",
    "SetCountdownTimer",
    "SetDeaths",
    "SetDeathsX",
    "SetDoodadState",
    "SetInvincibility",
    "SetMemory",
    "SetMemoryX",
    "SetMissionObjectives",
    "SetNextScenario",
    "SetResources",
    "SetScore",
    "SetSwitch",
    "Switch",
    "TalkingPortrait",
    "Transmission",
    "Trigger",
    "UnMuteUnitSpeech",
    "UnpauseGame",
    "UnpauseTimer",
    "Victory",
    "Wait",
]
