from typing import Literal, TypeAlias

DefTechDict: dict[str, int] = {
    "Stim Packs": 0,
    "Lockdown": 1,
    "EMP Shockwave": 2,
    "Spider Mines": 3,
    "Scanner Sweep": 4,
    "Tank Siege Mode": 5,
    "Defensive Matrix": 6,
    "Irradiate": 7,
    "Yamato Gun": 8,
    "Cloaking Field": 9,
    "Personnel Cloaking": 10,
    "Burrowing": 11,
    "Infestation": 12,
    "Spawn Broodlings": 13,
    "Dark Swarm": 14,
    "Plague": 15,
    "Consume": 16,
    "Ensnare": 17,
    "Parasite": 18,
    "Psionic Storm": 19,
    "Hallucination": 20,
    "Recall": 21,
    "Stasis Field": 22,
    "Archon Warp": 23,
    "Restoration": 24,
    "Disruption Web": 25,
    "Unused26": 26,
    "Mind Control": 27,
    "Dark Archon Meld": 28,
    "Feedback": 29,
    "Optical Flare": 30,
    "Maelstrom": 31,
    "Lurker Aspect": 32,
    "Unused33": 33,
    "Healing": 34,
    "Unused35": 35,
    "Unused36": 36,
    "Unused37": 37,
    "Unused38": 38,
    "Unused39": 39,
    "Unused40": 40,
    "Unused41": 41,
    "Unused42": 42,
    "Unused43": 43,
}

DefaultTech: TypeAlias = Literal[
    "Stim Packs",
    "Lockdown",
    "EMP Shockwave",
    "Spider Mines",
    "Scanner Sweep",
    "Tank Siege Mode",
    "Defensive Matrix",
    "Irradiate",
    "Yamato Gun",
    "Cloaking Field",
    "Personnel Cloaking",
    "Burrowing",
    "Infestation",
    "Spawn Broodlings",
    "Dark Swarm",
    "Plague",
    "Consume",
    "Ensnare",
    "Parasite",
    "Psionic Storm",
    "Hallucination",
    "Recall",
    "Stasis Field",
    "Archon Warp",
    "Restoration",
    "Disruption Web",
    "Unused26",
    "Mind Control",
    "Dark Archon Meld",
    "Feedback",
    "Optical Flare",
    "Maelstrom",
    "Lurker Aspect",
    "Unused33",
    "Healing",
    "Unused35",
    "Unused36",
    "Unused37",
    "Unused38",
    "Unused39",
    "Unused40",
    "Unused41",
    "Unused42",
    "Unused43",
]
