//
// This compound source is here for reducing the long build time
// of the entire StormLib library
//

#pragma warning(disable:4242)            // der_decode_ia5_string.c(84) : warning C4242: '=' : conversion from 'int' to 'unsigned char', possible loss of data

#include "libtomcrypt/src/hashes/hash_memory.c"
#include "libtomcrypt/src/hashes/md5.c"
#include "libtomcrypt/src/hashes/sha1.c"
#include "libtomcrypt/src/hashes/sha256.c"
#include "libtomcrypt/src/math/multi.c"
#include "libtomcrypt/src/math/rand_prime.c"
#include "libtomcrypt/src/misc/base64_decode.c"
#include "libtomcrypt/src/misc/crypt_argchk.c"
#include "libtomcrypt/src/misc/crypt_find_hash.c"
#include "libtomcrypt/src/misc/crypt_find_prng.c"
#include "libtomcrypt/src/misc/crypt_hash_descriptor.c"
#include "libtomcrypt/src/misc/crypt_hash_is_valid.c"
#include "libtomcrypt/src/misc/crypt_libc.c"
#include "libtomcrypt/src/misc/crypt_ltc_mp_descriptor.c"
#include "libtomcrypt/src/misc/crypt_prng_descriptor.c"
#include "libtomcrypt/src/misc/crypt_prng_is_valid.c"
#include "libtomcrypt/src/misc/crypt_register_hash.c"
#include "libtomcrypt/src/misc/crypt_register_prng.c"
#include "libtomcrypt/src/misc/zeromem.c"
#include "libtomcrypt/src/pk/asn1/der_decode_bit_string.c"
#include "libtomcrypt/src/pk/asn1/der_decode_boolean.c"
#include "libtomcrypt/src/pk/asn1/der_decode_choice.c"
#include "libtomcrypt/src/pk/asn1/der_decode_ia5_string.c"
#include "libtomcrypt/src/pk/asn1/der_decode_integer.c"
#include "libtomcrypt/src/pk/asn1/der_decode_object_identifier.c"
#include "libtomcrypt/src/pk/asn1/der_decode_octet_string.c"
#include "libtomcrypt/src/pk/asn1/der_decode_printable_string.c"
#include "libtomcrypt/src/pk/asn1/der_decode_sequence_ex.c"
#include "libtomcrypt/src/pk/asn1/der_decode_sequence_flexi.c"
#include "libtomcrypt/src/pk/asn1/der_decode_sequence_multi.c"
#include "libtomcrypt/src/pk/asn1/der_decode_short_integer.c"
#include "libtomcrypt/src/pk/asn1/der_decode_utctime.c"
#include "libtomcrypt/src/pk/asn1/der_decode_utf8_string.c"
#include "libtomcrypt/src/pk/asn1/der_encode_bit_string.c"
#include "libtomcrypt/src/pk/asn1/der_encode_boolean.c"
#include "libtomcrypt/src/pk/asn1/der_encode_ia5_string.c"
#include "libtomcrypt/src/pk/asn1/der_encode_integer.c"
#include "libtomcrypt/src/pk/asn1/der_encode_object_identifier.c"
#include "libtomcrypt/src/pk/asn1/der_encode_octet_string.c"
#include "libtomcrypt/src/pk/asn1/der_encode_printable_string.c"
#include "libtomcrypt/src/pk/asn1/der_encode_sequence_ex.c"
#include "libtomcrypt/src/pk/asn1/der_encode_sequence_multi.c"
#include "libtomcrypt/src/pk/asn1/der_encode_set.c"
#include "libtomcrypt/src/pk/asn1/der_encode_setof.c"
#include "libtomcrypt/src/pk/asn1/der_encode_short_integer.c"
#include "libtomcrypt/src/pk/asn1/der_encode_utctime.c"
#include "libtomcrypt/src/pk/asn1/der_encode_utf8_string.c"
#include "libtomcrypt/src/pk/asn1/der_length_bit_string.c"
#include "libtomcrypt/src/pk/asn1/der_length_boolean.c"
#include "libtomcrypt/src/pk/asn1/der_length_ia5_string.c"
#include "libtomcrypt/src/pk/asn1/der_length_integer.c"
#include "libtomcrypt/src/pk/asn1/der_length_object_identifier.c"
#include "libtomcrypt/src/pk/asn1/der_length_octet_string.c"
#include "libtomcrypt/src/pk/asn1/der_length_printable_string.c"
#include "libtomcrypt/src/pk/asn1/der_length_sequence.c"
#include "libtomcrypt/src/pk/asn1/der_length_short_integer.c"
#include "libtomcrypt/src/pk/asn1/der_length_utctime.c"
#include "libtomcrypt/src/pk/asn1/der_length_utf8_string.c"
#include "libtomcrypt/src/pk/asn1/der_sequence_free.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_map.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_mul2add.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_mulmod.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_points.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_projective_add_point.c"
#include "libtomcrypt/src/pk/ecc/ltc_ecc_projective_dbl_point.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_mgf1.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_oaep_decode.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_pss_decode.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_pss_encode.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_v1_5_decode.c"
#include "libtomcrypt/src/pk/pkcs1/pkcs_1_v1_5_encode.c"
#include "libtomcrypt/src/pk/rsa/rsa_exptmod.c"
#include "libtomcrypt/src/pk/rsa/rsa_free.c"
#include "libtomcrypt/src/pk/rsa/rsa_import.c"
#include "libtomcrypt/src/pk/rsa/rsa_make_key.c"
#include "libtomcrypt/src/pk/rsa/rsa_sign_hash.c"
#include "libtomcrypt/src/pk/rsa/rsa_verify_hash.c"
#include "libtomcrypt/src/pk/rsa/rsa_verify_simple.c"
