# Copyright 2014 by trgk.
# All rights reserved.
# This file is part of EUD python library (eudplib),
# and is released under "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

# Just run _eudvsupport. This won't be visible in user side.
from . import _eudvsupport  # FIXME: remove hacky way
