#define IFX                              1
#define ELSE                             2
#define ASSIGN                           3
#define QMARK                            4
#define COMMA                            5
#define LOR                              6
#define LAND                             7
#define LNOT                             8
#define EQ                               9
#define LE                              10
#define LT                              11
#define GE                              12
#define GT                              13
#define NE                              14
#define BITOR                           15
#define BITXOR                          16
#define BITAND                          17
#define LSHIFT                          18
#define RSHIFT                          19
#define PLUS                            20
#define MINUS                           21
#define DIVIDE                          22
#define MULTIPLY                        23
#define MOD                             24
#define UNARY                           25
#define BITNOT                          26
#define LPAREN                          27
#define LSQBRACKET                      28
#define PERIOD                          29
#define SEMICOLON                       30
#define IMPORT                          31
#define NAME                            32
#define AS                              33
#define COLON                           34
#define FUNCTION                        35
#define RPAREN                          36
#define OBJECT                          37
#define LBRACKET                        38
#define EXTENDS                         39
#define VAR                             40
#define RBRACKET                        41
#define NUMBER                          42
#define RSQBRACKET                      43
#define SUBSCRIPT                       44
#define STRING                          45
#define KILLS                           46
#define MEMBER                          47
#define FUNCCALL                        48
#define L2V                             49
#define MAPSTRING                       50
#define UNIT                            51
#define SWITCH                          52
#define LOCATION                        53
#define STATTXTTBL                      54
#define VARRAY                          55
#define LIST                            56
#define STATIC                          57
#define CONST                           58
#define INC                             59
#define DEC                             60
#define IADD                            61
#define ISUB                            62
#define IMUL                            63
#define IDIV                            64
#define IMOD                            65
#define ILSH                            66
#define IRSH                            67
#define IBND                            68
#define IBOR                            69
#define IBXR                            70
#define ONCE                            71
#define IF                              72
#define EPDSWITCH                       73
#define SWITCHCASE                      74
#define CASE                            75
#define DEFAULT                         76
#define WHILE                           77
#define FOR                             78
#define FOREACH                         79
#define CONTINUE                        80
#define BREAK                           81
#define RETURN                          82
#define CONDITIONNAME                   83
#define ACTIONNAME                      84
#define ACTIONALLPNAME                  85
