# ruff: noqa: E501

cp949_table = [
    ((0x81, 0x41), (0xAC02)),  # HANGUL SYLLABLE KIYEOK A SSANGKIYEOK
    ((0x81, 0x42), (0xAC03)),  # HANGUL SYLLABLE KIYEOK A KIYEOKSIOS
    ((0x81, 0x43), (0xAC05)),  # HANGUL SYLLABLE KIYEOK A NIEUNCIEUC
    ((0x81, 0x44), (0xAC06)),  # HANGUL SYLLABLE KIYEOK A NIEUNHIEUH
    ((0x81, 0x45), (0xAC0B)),  # HANGUL SYLLABLE KIYEOK A RIEULPIEUP
    ((0x81, 0x46), (0xAC0C)),  # HANGUL SYLLABLE KIYEOK A RIEULSIOS
    ((0x81, 0x47), (0xAC0D)),  # HANGUL SYLLABLE KIYEOK A RIEULTHIEUTH
    ((0x81, 0x48), (0xAC0E)),  # HANGUL SYLLABLE KIYEOK A RIEULPHIEUPH
    ((0x81, 0x49), (0xAC0F)),  # HANGUL SYLLABLE KIYEOK A RIEULHIEUH
    ((0x81, 0x4A), (0xAC18)),  # HANGUL SYLLABLE KIYEOK A KHIEUKH
    ((0x81, 0x4B), (0xAC1E)),  # HANGUL SYLLABLE KIYEOK AE SSANGKIYEOK
    ((0x81, 0x4C), (0xAC1F)),  # HANGUL SYLLABLE KIYEOK AE KIYEOKSIOS
    ((0x81, 0x4D), (0xAC21)),  # HANGUL SYLLABLE KIYEOK AE NIEUNCIEUC
    ((0x81, 0x4E), (0xAC22)),  # HANGUL SYLLABLE KIYEOK AE NIEUNHIEUH
    ((0x81, 0x4F), (0xAC23)),  # HANGUL SYLLABLE KIYEOK AE TIKEUT
    ((0x81, 0x50), (0xAC25)),  # HANGUL SYLLABLE KIYEOK AE RIEULKIYEOK
    ((0x81, 0x51), (0xAC26)),  # HANGUL SYLLABLE KIYEOK AE RIEULMIEUM
    ((0x81, 0x52), (0xAC27)),  # HANGUL SYLLABLE KIYEOK AE RIEULPIEUP
    ((0x81, 0x53), (0xAC28)),  # HANGUL SYLLABLE KIYEOK AE RIEULSIOS
    ((0x81, 0x54), (0xAC29)),  # HANGUL SYLLABLE KIYEOK AE RIEULTHIEUTH
    ((0x81, 0x55), (0xAC2A)),  # HANGUL SYLLABLE KIYEOK AE RIEULPHIEUPH
    ((0x81, 0x56), (0xAC2B)),  # HANGUL SYLLABLE KIYEOK AE RIEULHIEUH
    ((0x81, 0x57), (0xAC2E)),  # HANGUL SYLLABLE KIYEOK AE PIEUPSIOS
    ((0x81, 0x58), (0xAC32)),  # HANGUL SYLLABLE KIYEOK AE CIEUC
    ((0x81, 0x59), (0xAC33)),  # HANGUL SYLLABLE KIYEOK AE CHIEUCH
    ((0x81, 0x5A), (0xAC34)),  # HANGUL SYLLABLE KIYEOK AE KHIEUKH
    ((0x81, 0x61), (0xAC35)),  # HANGUL SYLLABLE KIYEOK AE THIEUTH
    ((0x81, 0x62), (0xAC36)),  # HANGUL SYLLABLE KIYEOK AE PHIEUPH
    ((0x81, 0x63), (0xAC37)),  # HANGUL SYLLABLE KIYEOK AE HIEUH
    ((0x81, 0x64), (0xAC3A)),  # HANGUL SYLLABLE KIYEOK YA SSANGKIYEOK
    ((0x81, 0x65), (0xAC3B)),  # HANGUL SYLLABLE KIYEOK YA KIYEOKSIOS
    ((0x81, 0x66), (0xAC3D)),  # HANGUL SYLLABLE KIYEOK YA NIEUNCIEUC
    ((0x81, 0x67), (0xAC3E)),  # HANGUL SYLLABLE KIYEOK YA NIEUNHIEUH
    ((0x81, 0x68), (0xAC3F)),  # HANGUL SYLLABLE KIYEOK YA TIKEUT
    ((0x81, 0x69), (0xAC41)),  # HANGUL SYLLABLE KIYEOK YA RIEULKIYEOK
    ((0x81, 0x6A), (0xAC42)),  # HANGUL SYLLABLE KIYEOK YA RIEULMIEUM
    ((0x81, 0x6B), (0xAC43)),  # HANGUL SYLLABLE KIYEOK YA RIEULPIEUP
    ((0x81, 0x6C), (0xAC44)),  # HANGUL SYLLABLE KIYEOK YA RIEULSIOS
    ((0x81, 0x6D), (0xAC45)),  # HANGUL SYLLABLE KIYEOK YA RIEULTHIEUTH
    ((0x81, 0x6E), (0xAC46)),  # HANGUL SYLLABLE KIYEOK YA RIEULPHIEUPH
    ((0x81, 0x6F), (0xAC47)),  # HANGUL SYLLABLE KIYEOK YA RIEULHIEUH
    ((0x81, 0x70), (0xAC48)),  # HANGUL SYLLABLE KIYEOK YA MIEUM
    ((0x81, 0x71), (0xAC49)),  # HANGUL SYLLABLE KIYEOK YA PIEUP
    ((0x81, 0x72), (0xAC4A)),  # HANGUL SYLLABLE KIYEOK YA PIEUPSIOS
    ((0x81, 0x73), (0xAC4C)),  # HANGUL SYLLABLE KIYEOK YA SSANGSIOS
    ((0x81, 0x74), (0xAC4E)),  # HANGUL SYLLABLE KIYEOK YA CIEUC
    ((0x81, 0x75), (0xAC4F)),  # HANGUL SYLLABLE KIYEOK YA CHIEUCH
    ((0x81, 0x76), (0xAC50)),  # HANGUL SYLLABLE KIYEOK YA KHIEUKH
    ((0x81, 0x77), (0xAC51)),  # HANGUL SYLLABLE KIYEOK YA THIEUTH
    ((0x81, 0x78), (0xAC52)),  # HANGUL SYLLABLE KIYEOK YA PHIEUPH
    ((0x81, 0x79), (0xAC53)),  # HANGUL SYLLABLE KIYEOK YA HIEUH
    ((0x81, 0x7A), (0xAC55)),  # HANGUL SYLLABLE KIYEOK YAE KIYEOK
    ((0x81, 0x81), (0xAC56)),  # HANGUL SYLLABLE KIYEOK YAE SSANGKIYEOK
    ((0x81, 0x82), (0xAC57)),  # HANGUL SYLLABLE KIYEOK YAE KIYEOKSIOS
    ((0x81, 0x83), (0xAC59)),  # HANGUL SYLLABLE KIYEOK YAE NIEUNCIEUC
    ((0x81, 0x84), (0xAC5A)),  # HANGUL SYLLABLE KIYEOK YAE NIEUNHIEUH
    ((0x81, 0x85), (0xAC5B)),  # HANGUL SYLLABLE KIYEOK YAE TIKEUT
    ((0x81, 0x86), (0xAC5D)),  # HANGUL SYLLABLE KIYEOK YAE RIEULKIYEOK
    ((0x81, 0x87), (0xAC5E)),  # HANGUL SYLLABLE KIYEOK YAE RIEULMIEUM
    ((0x81, 0x88), (0xAC5F)),  # HANGUL SYLLABLE KIYEOK YAE RIEULPIEUP
    ((0x81, 0x89), (0xAC60)),  # HANGUL SYLLABLE KIYEOK YAE RIEULSIOS
    ((0x81, 0x8A), (0xAC61)),  # HANGUL SYLLABLE KIYEOK YAE RIEULTHIEUTH
    ((0x81, 0x8B), (0xAC62)),  # HANGUL SYLLABLE KIYEOK YAE RIEULPHIEUPH
    ((0x81, 0x8C), (0xAC63)),  # HANGUL SYLLABLE KIYEOK YAE RIEULHIEUH
    ((0x81, 0x8D), (0xAC64)),  # HANGUL SYLLABLE KIYEOK YAE MIEUM
    ((0x81, 0x8E), (0xAC65)),  # HANGUL SYLLABLE KIYEOK YAE PIEUP
    ((0x81, 0x8F), (0xAC66)),  # HANGUL SYLLABLE KIYEOK YAE PIEUPSIOS
    ((0x81, 0x90), (0xAC67)),  # HANGUL SYLLABLE KIYEOK YAE SIOS
    ((0x81, 0x91), (0xAC68)),  # HANGUL SYLLABLE KIYEOK YAE SSANGSIOS
    ((0x81, 0x92), (0xAC69)),  # HANGUL SYLLABLE KIYEOK YAE IEUNG
    ((0x81, 0x93), (0xAC6A)),  # HANGUL SYLLABLE KIYEOK YAE CIEUC
    ((0x81, 0x94), (0xAC6B)),  # HANGUL SYLLABLE KIYEOK YAE CHIEUCH
    ((0x81, 0x95), (0xAC6C)),  # HANGUL SYLLABLE KIYEOK YAE KHIEUKH
    ((0x81, 0x96), (0xAC6D)),  # HANGUL SYLLABLE KIYEOK YAE THIEUTH
    ((0x81, 0x97), (0xAC6E)),  # HANGUL SYLLABLE KIYEOK YAE PHIEUPH
    ((0x81, 0x98), (0xAC6F)),  # HANGUL SYLLABLE KIYEOK YAE HIEUH
    ((0x81, 0x99), (0xAC72)),  # HANGUL SYLLABLE KIYEOK EO SSANGKIYEOK
    ((0x81, 0x9A), (0xAC73)),  # HANGUL SYLLABLE KIYEOK EO KIYEOKSIOS
    ((0x81, 0x9B), (0xAC75)),  # HANGUL SYLLABLE KIYEOK EO NIEUNCIEUC
    ((0x81, 0x9C), (0xAC76)),  # HANGUL SYLLABLE KIYEOK EO NIEUNHIEUH
    ((0x81, 0x9D), (0xAC79)),  # HANGUL SYLLABLE KIYEOK EO RIEULKIYEOK
    ((0x81, 0x9E), (0xAC7B)),  # HANGUL SYLLABLE KIYEOK EO RIEULPIEUP
    ((0x81, 0x9F), (0xAC7C)),  # HANGUL SYLLABLE KIYEOK EO RIEULSIOS
    ((0x81, 0xA0), (0xAC7D)),  # HANGUL SYLLABLE KIYEOK EO RIEULTHIEUTH
    ((0x81, 0xA1), (0xAC7E)),  # HANGUL SYLLABLE KIYEOK EO RIEULPHIEUPH
    ((0x81, 0xA2), (0xAC7F)),  # HANGUL SYLLABLE KIYEOK EO RIEULHIEUH
    ((0x81, 0xA3), (0xAC82)),  # HANGUL SYLLABLE KIYEOK EO PIEUPSIOS
    ((0x81, 0xA4), (0xAC87)),  # HANGUL SYLLABLE KIYEOK EO CHIEUCH
    ((0x81, 0xA5), (0xAC88)),  # HANGUL SYLLABLE KIYEOK EO KHIEUKH
    ((0x81, 0xA6), (0xAC8D)),  # HANGUL SYLLABLE KIYEOK E KIYEOK
    ((0x81, 0xA7), (0xAC8E)),  # HANGUL SYLLABLE KIYEOK E SSANGKIYEOK
    ((0x81, 0xA8), (0xAC8F)),  # HANGUL SYLLABLE KIYEOK E KIYEOKSIOS
    ((0x81, 0xA9), (0xAC91)),  # HANGUL SYLLABLE KIYEOK E NIEUNCIEUC
    ((0x81, 0xAA), (0xAC92)),  # HANGUL SYLLABLE KIYEOK E NIEUNHIEUH
    ((0x81, 0xAB), (0xAC93)),  # HANGUL SYLLABLE KIYEOK E TIKEUT
    ((0x81, 0xAC), (0xAC95)),  # HANGUL SYLLABLE KIYEOK E RIEULKIYEOK
    ((0x81, 0xAD), (0xAC96)),  # HANGUL SYLLABLE KIYEOK E RIEULMIEUM
    ((0x81, 0xAE), (0xAC97)),  # HANGUL SYLLABLE KIYEOK E RIEULPIEUP
    ((0x81, 0xAF), (0xAC98)),  # HANGUL SYLLABLE KIYEOK E RIEULSIOS
    ((0x81, 0xB0), (0xAC99)),  # HANGUL SYLLABLE KIYEOK E RIEULTHIEUTH
    ((0x81, 0xB1), (0xAC9A)),  # HANGUL SYLLABLE KIYEOK E RIEULPHIEUPH
    ((0x81, 0xB2), (0xAC9B)),  # HANGUL SYLLABLE KIYEOK E RIEULHIEUH
    ((0x81, 0xB3), (0xAC9E)),  # HANGUL SYLLABLE KIYEOK E PIEUPSIOS
    ((0x81, 0xB4), (0xACA2)),  # HANGUL SYLLABLE KIYEOK E CIEUC
    ((0x81, 0xB5), (0xACA3)),  # HANGUL SYLLABLE KIYEOK E CHIEUCH
    ((0x81, 0xB6), (0xACA4)),  # HANGUL SYLLABLE KIYEOK E KHIEUKH
    ((0x81, 0xB7), (0xACA5)),  # HANGUL SYLLABLE KIYEOK E THIEUTH
    ((0x81, 0xB8), (0xACA6)),  # HANGUL SYLLABLE KIYEOK E PHIEUPH
    ((0x81, 0xB9), (0xACA7)),  # HANGUL SYLLABLE KIYEOK E HIEUH
    ((0x81, 0xBA), (0xACAB)),  # HANGUL SYLLABLE KIYEOK YEO KIYEOKSIOS
    ((0x81, 0xBB), (0xACAD)),  # HANGUL SYLLABLE KIYEOK YEO NIEUNCIEUC
    ((0x81, 0xBC), (0xACAE)),  # HANGUL SYLLABLE KIYEOK YEO NIEUNHIEUH
    ((0x81, 0xBD), (0xACB1)),  # HANGUL SYLLABLE KIYEOK YEO RIEULKIYEOK
    ((0x81, 0xBE), (0xACB2)),  # HANGUL SYLLABLE KIYEOK YEO RIEULMIEUM
    ((0x81, 0xBF), (0xACB3)),  # HANGUL SYLLABLE KIYEOK YEO RIEULPIEUP
    ((0x81, 0xC0), (0xACB4)),  # HANGUL SYLLABLE KIYEOK YEO RIEULSIOS
    ((0x81, 0xC1), (0xACB5)),  # HANGUL SYLLABLE KIYEOK YEO RIEULTHIEUTH
    ((0x81, 0xC2), (0xACB6)),  # HANGUL SYLLABLE KIYEOK YEO RIEULPHIEUPH
    ((0x81, 0xC3), (0xACB7)),  # HANGUL SYLLABLE KIYEOK YEO RIEULHIEUH
    ((0x81, 0xC4), (0xACBA)),  # HANGUL SYLLABLE KIYEOK YEO PIEUPSIOS
    ((0x81, 0xC5), (0xACBE)),  # HANGUL SYLLABLE KIYEOK YEO CIEUC
    ((0x81, 0xC6), (0xACBF)),  # HANGUL SYLLABLE KIYEOK YEO CHIEUCH
    ((0x81, 0xC7), (0xACC0)),  # HANGUL SYLLABLE KIYEOK YEO KHIEUKH
    ((0x81, 0xC8), (0xACC2)),  # HANGUL SYLLABLE KIYEOK YEO PHIEUPH
    ((0x81, 0xC9), (0xACC3)),  # HANGUL SYLLABLE KIYEOK YEO HIEUH
    ((0x81, 0xCA), (0xACC5)),  # HANGUL SYLLABLE KIYEOK YE KIYEOK
    ((0x81, 0xCB), (0xACC6)),  # HANGUL SYLLABLE KIYEOK YE SSANGKIYEOK
    ((0x81, 0xCC), (0xACC7)),  # HANGUL SYLLABLE KIYEOK YE KIYEOKSIOS
    ((0x81, 0xCD), (0xACC9)),  # HANGUL SYLLABLE KIYEOK YE NIEUNCIEUC
    ((0x81, 0xCE), (0xACCA)),  # HANGUL SYLLABLE KIYEOK YE NIEUNHIEUH
    ((0x81, 0xCF), (0xACCB)),  # HANGUL SYLLABLE KIYEOK YE TIKEUT
    ((0x81, 0xD0), (0xACCD)),  # HANGUL SYLLABLE KIYEOK YE RIEULKIYEOK
    ((0x81, 0xD1), (0xACCE)),  # HANGUL SYLLABLE KIYEOK YE RIEULMIEUM
    ((0x81, 0xD2), (0xACCF)),  # HANGUL SYLLABLE KIYEOK YE RIEULPIEUP
    ((0x81, 0xD3), (0xACD0)),  # HANGUL SYLLABLE KIYEOK YE RIEULSIOS
    ((0x81, 0xD4), (0xACD1)),  # HANGUL SYLLABLE KIYEOK YE RIEULTHIEUTH
    ((0x81, 0xD5), (0xACD2)),  # HANGUL SYLLABLE KIYEOK YE RIEULPHIEUPH
    ((0x81, 0xD6), (0xACD3)),  # HANGUL SYLLABLE KIYEOK YE RIEULHIEUH
    ((0x81, 0xD7), (0xACD4)),  # HANGUL SYLLABLE KIYEOK YE MIEUM
    ((0x81, 0xD8), (0xACD6)),  # HANGUL SYLLABLE KIYEOK YE PIEUPSIOS
    ((0x81, 0xD9), (0xACD8)),  # HANGUL SYLLABLE KIYEOK YE SSANGSIOS
    ((0x81, 0xDA), (0xACD9)),  # HANGUL SYLLABLE KIYEOK YE IEUNG
    ((0x81, 0xDB), (0xACDA)),  # HANGUL SYLLABLE KIYEOK YE CIEUC
    ((0x81, 0xDC), (0xACDB)),  # HANGUL SYLLABLE KIYEOK YE CHIEUCH
    ((0x81, 0xDD), (0xACDC)),  # HANGUL SYLLABLE KIYEOK YE KHIEUKH
    ((0x81, 0xDE), (0xACDD)),  # HANGUL SYLLABLE KIYEOK YE THIEUTH
    ((0x81, 0xDF), (0xACDE)),  # HANGUL SYLLABLE KIYEOK YE PHIEUPH
    ((0x81, 0xE0), (0xACDF)),  # HANGUL SYLLABLE KIYEOK YE HIEUH
    ((0x81, 0xE1), (0xACE2)),  # HANGUL SYLLABLE KIYEOK O SSANGKIYEOK
    ((0x81, 0xE2), (0xACE3)),  # HANGUL SYLLABLE KIYEOK O KIYEOKSIOS
    ((0x81, 0xE3), (0xACE5)),  # HANGUL SYLLABLE KIYEOK O NIEUNCIEUC
    ((0x81, 0xE4), (0xACE6)),  # HANGUL SYLLABLE KIYEOK O NIEUNHIEUH
    ((0x81, 0xE5), (0xACE9)),  # HANGUL SYLLABLE KIYEOK O RIEULKIYEOK
    ((0x81, 0xE6), (0xACEB)),  # HANGUL SYLLABLE KIYEOK O RIEULPIEUP
    ((0x81, 0xE7), (0xACED)),  # HANGUL SYLLABLE KIYEOK O RIEULTHIEUTH
    ((0x81, 0xE8), (0xACEE)),  # HANGUL SYLLABLE KIYEOK O RIEULPHIEUPH
    ((0x81, 0xE9), (0xACF2)),  # HANGUL SYLLABLE KIYEOK O PIEUPSIOS
    ((0x81, 0xEA), (0xACF4)),  # HANGUL SYLLABLE KIYEOK O SSANGSIOS
    ((0x81, 0xEB), (0xACF7)),  # HANGUL SYLLABLE KIYEOK O CHIEUCH
    ((0x81, 0xEC), (0xACF8)),  # HANGUL SYLLABLE KIYEOK O KHIEUKH
    ((0x81, 0xED), (0xACF9)),  # HANGUL SYLLABLE KIYEOK O THIEUTH
    ((0x81, 0xEE), (0xACFA)),  # HANGUL SYLLABLE KIYEOK O PHIEUPH
    ((0x81, 0xEF), (0xACFB)),  # HANGUL SYLLABLE KIYEOK O HIEUH
    ((0x81, 0xF0), (0xACFE)),  # HANGUL SYLLABLE KIYEOK WA SSANGKIYEOK
    ((0x81, 0xF1), (0xACFF)),  # HANGUL SYLLABLE KIYEOK WA KIYEOKSIOS
    ((0x81, 0xF2), (0xAD01)),  # HANGUL SYLLABLE KIYEOK WA NIEUNCIEUC
    ((0x81, 0xF3), (0xAD02)),  # HANGUL SYLLABLE KIYEOK WA NIEUNHIEUH
    ((0x81, 0xF4), (0xAD03)),  # HANGUL SYLLABLE KIYEOK WA TIKEUT
    ((0x81, 0xF5), (0xAD05)),  # HANGUL SYLLABLE KIYEOK WA RIEULKIYEOK
    ((0x81, 0xF6), (0xAD07)),  # HANGUL SYLLABLE KIYEOK WA RIEULPIEUP
    ((0x81, 0xF7), (0xAD08)),  # HANGUL SYLLABLE KIYEOK WA RIEULSIOS
    ((0x81, 0xF8), (0xAD09)),  # HANGUL SYLLABLE KIYEOK WA RIEULTHIEUTH
    ((0x81, 0xF9), (0xAD0A)),  # HANGUL SYLLABLE KIYEOK WA RIEULPHIEUPH
    ((0x81, 0xFA), (0xAD0B)),  # HANGUL SYLLABLE KIYEOK WA RIEULHIEUH
    ((0x81, 0xFB), (0xAD0E)),  # HANGUL SYLLABLE KIYEOK WA PIEUPSIOS
    ((0x81, 0xFC), (0xAD10)),  # HANGUL SYLLABLE KIYEOK WA SSANGSIOS
    ((0x81, 0xFD), (0xAD12)),  # HANGUL SYLLABLE KIYEOK WA CIEUC
    ((0x81, 0xFE), (0xAD13)),  # HANGUL SYLLABLE KIYEOK WA CHIEUCH
    ((0x82, 0x41), (0xAD14)),  # HANGUL SYLLABLE KIYEOK WA KHIEUKH
    ((0x82, 0x42), (0xAD15)),  # HANGUL SYLLABLE KIYEOK WA THIEUTH
    ((0x82, 0x43), (0xAD16)),  # HANGUL SYLLABLE KIYEOK WA PHIEUPH
    ((0x82, 0x44), (0xAD17)),  # HANGUL SYLLABLE KIYEOK WA HIEUH
    ((0x82, 0x45), (0xAD19)),  # HANGUL SYLLABLE KIYEOK WAE KIYEOK
    ((0x82, 0x46), (0xAD1A)),  # HANGUL SYLLABLE KIYEOK WAE SSANGKIYEOK
    ((0x82, 0x47), (0xAD1B)),  # HANGUL SYLLABLE KIYEOK WAE KIYEOKSIOS
    ((0x82, 0x48), (0xAD1D)),  # HANGUL SYLLABLE KIYEOK WAE NIEUNCIEUC
    ((0x82, 0x49), (0xAD1E)),  # HANGUL SYLLABLE KIYEOK WAE NIEUNHIEUH
    ((0x82, 0x4A), (0xAD1F)),  # HANGUL SYLLABLE KIYEOK WAE TIKEUT
    ((0x82, 0x4B), (0xAD21)),  # HANGUL SYLLABLE KIYEOK WAE RIEULKIYEOK
    ((0x82, 0x4C), (0xAD22)),  # HANGUL SYLLABLE KIYEOK WAE RIEULMIEUM
    ((0x82, 0x4D), (0xAD23)),  # HANGUL SYLLABLE KIYEOK WAE RIEULPIEUP
    ((0x82, 0x4E), (0xAD24)),  # HANGUL SYLLABLE KIYEOK WAE RIEULSIOS
    ((0x82, 0x4F), (0xAD25)),  # HANGUL SYLLABLE KIYEOK WAE RIEULTHIEUTH
    ((0x82, 0x50), (0xAD26)),  # HANGUL SYLLABLE KIYEOK WAE RIEULPHIEUPH
    ((0x82, 0x51), (0xAD27)),  # HANGUL SYLLABLE KIYEOK WAE RIEULHIEUH
    ((0x82, 0x52), (0xAD28)),  # HANGUL SYLLABLE KIYEOK WAE MIEUM
    ((0x82, 0x53), (0xAD2A)),  # HANGUL SYLLABLE KIYEOK WAE PIEUPSIOS
    ((0x82, 0x54), (0xAD2B)),  # HANGUL SYLLABLE KIYEOK WAE SIOS
    ((0x82, 0x55), (0xAD2E)),  # HANGUL SYLLABLE KIYEOK WAE CIEUC
    ((0x82, 0x56), (0xAD2F)),  # HANGUL SYLLABLE KIYEOK WAE CHIEUCH
    ((0x82, 0x57), (0xAD30)),  # HANGUL SYLLABLE KIYEOK WAE KHIEUKH
    ((0x82, 0x58), (0xAD31)),  # HANGUL SYLLABLE KIYEOK WAE THIEUTH
    ((0x82, 0x59), (0xAD32)),  # HANGUL SYLLABLE KIYEOK WAE PHIEUPH
    ((0x82, 0x5A), (0xAD33)),  # HANGUL SYLLABLE KIYEOK WAE HIEUH
    ((0x82, 0x61), (0xAD36)),  # HANGUL SYLLABLE KIYEOK OE SSANGKIYEOK
    ((0x82, 0x62), (0xAD37)),  # HANGUL SYLLABLE KIYEOK OE KIYEOKSIOS
    ((0x82, 0x63), (0xAD39)),  # HANGUL SYLLABLE KIYEOK OE NIEUNCIEUC
    ((0x82, 0x64), (0xAD3A)),  # HANGUL SYLLABLE KIYEOK OE NIEUNHIEUH
    ((0x82, 0x65), (0xAD3B)),  # HANGUL SYLLABLE KIYEOK OE TIKEUT
    ((0x82, 0x66), (0xAD3D)),  # HANGUL SYLLABLE KIYEOK OE RIEULKIYEOK
    ((0x82, 0x67), (0xAD3E)),  # HANGUL SYLLABLE KIYEOK OE RIEULMIEUM
    ((0x82, 0x68), (0xAD3F)),  # HANGUL SYLLABLE KIYEOK OE RIEULPIEUP
    ((0x82, 0x69), (0xAD40)),  # HANGUL SYLLABLE KIYEOK OE RIEULSIOS
    ((0x82, 0x6A), (0xAD41)),  # HANGUL SYLLABLE KIYEOK OE RIEULTHIEUTH
    ((0x82, 0x6B), (0xAD42)),  # HANGUL SYLLABLE KIYEOK OE RIEULPHIEUPH
    ((0x82, 0x6C), (0xAD43)),  # HANGUL SYLLABLE KIYEOK OE RIEULHIEUH
    ((0x82, 0x6D), (0xAD46)),  # HANGUL SYLLABLE KIYEOK OE PIEUPSIOS
    ((0x82, 0x6E), (0xAD48)),  # HANGUL SYLLABLE KIYEOK OE SSANGSIOS
    ((0x82, 0x6F), (0xAD4A)),  # HANGUL SYLLABLE KIYEOK OE CIEUC
    ((0x82, 0x70), (0xAD4B)),  # HANGUL SYLLABLE KIYEOK OE CHIEUCH
    ((0x82, 0x71), (0xAD4C)),  # HANGUL SYLLABLE KIYEOK OE KHIEUKH
    ((0x82, 0x72), (0xAD4D)),  # HANGUL SYLLABLE KIYEOK OE THIEUTH
    ((0x82, 0x73), (0xAD4E)),  # HANGUL SYLLABLE KIYEOK OE PHIEUPH
    ((0x82, 0x74), (0xAD4F)),  # HANGUL SYLLABLE KIYEOK OE HIEUH
    ((0x82, 0x75), (0xAD51)),  # HANGUL SYLLABLE KIYEOK YO KIYEOK
    ((0x82, 0x76), (0xAD52)),  # HANGUL SYLLABLE KIYEOK YO SSANGKIYEOK
    ((0x82, 0x77), (0xAD53)),  # HANGUL SYLLABLE KIYEOK YO KIYEOKSIOS
    ((0x82, 0x78), (0xAD55)),  # HANGUL SYLLABLE KIYEOK YO NIEUNCIEUC
    ((0x82, 0x79), (0xAD56)),  # HANGUL SYLLABLE KIYEOK YO NIEUNHIEUH
    ((0x82, 0x7A), (0xAD57)),  # HANGUL SYLLABLE KIYEOK YO TIKEUT
    ((0x82, 0x81), (0xAD59)),  # HANGUL SYLLABLE KIYEOK YO RIEULKIYEOK
    ((0x82, 0x82), (0xAD5A)),  # HANGUL SYLLABLE KIYEOK YO RIEULMIEUM
    ((0x82, 0x83), (0xAD5B)),  # HANGUL SYLLABLE KIYEOK YO RIEULPIEUP
    ((0x82, 0x84), (0xAD5C)),  # HANGUL SYLLABLE KIYEOK YO RIEULSIOS
    ((0x82, 0x85), (0xAD5D)),  # HANGUL SYLLABLE KIYEOK YO RIEULTHIEUTH
    ((0x82, 0x86), (0xAD5E)),  # HANGUL SYLLABLE KIYEOK YO RIEULPHIEUPH
    ((0x82, 0x87), (0xAD5F)),  # HANGUL SYLLABLE KIYEOK YO RIEULHIEUH
    ((0x82, 0x88), (0xAD60)),  # HANGUL SYLLABLE KIYEOK YO MIEUM
    ((0x82, 0x89), (0xAD62)),  # HANGUL SYLLABLE KIYEOK YO PIEUPSIOS
    ((0x82, 0x8A), (0xAD64)),  # HANGUL SYLLABLE KIYEOK YO SSANGSIOS
    ((0x82, 0x8B), (0xAD65)),  # HANGUL SYLLABLE KIYEOK YO IEUNG
    ((0x82, 0x8C), (0xAD66)),  # HANGUL SYLLABLE KIYEOK YO CIEUC
    ((0x82, 0x8D), (0xAD67)),  # HANGUL SYLLABLE KIYEOK YO CHIEUCH
    ((0x82, 0x8E), (0xAD68)),  # HANGUL SYLLABLE KIYEOK YO KHIEUKH
    ((0x82, 0x8F), (0xAD69)),  # HANGUL SYLLABLE KIYEOK YO THIEUTH
    ((0x82, 0x90), (0xAD6A)),  # HANGUL SYLLABLE KIYEOK YO PHIEUPH
    ((0x82, 0x91), (0xAD6B)),  # HANGUL SYLLABLE KIYEOK YO HIEUH
    ((0x82, 0x92), (0xAD6E)),  # HANGUL SYLLABLE KIYEOK U SSANGKIYEOK
    ((0x82, 0x93), (0xAD6F)),  # HANGUL SYLLABLE KIYEOK U KIYEOKSIOS
    ((0x82, 0x94), (0xAD71)),  # HANGUL SYLLABLE KIYEOK U NIEUNCIEUC
    ((0x82, 0x95), (0xAD72)),  # HANGUL SYLLABLE KIYEOK U NIEUNHIEUH
    ((0x82, 0x96), (0xAD77)),  # HANGUL SYLLABLE KIYEOK U RIEULPIEUP
    ((0x82, 0x97), (0xAD78)),  # HANGUL SYLLABLE KIYEOK U RIEULSIOS
    ((0x82, 0x98), (0xAD79)),  # HANGUL SYLLABLE KIYEOK U RIEULTHIEUTH
    ((0x82, 0x99), (0xAD7A)),  # HANGUL SYLLABLE KIYEOK U RIEULPHIEUPH
    ((0x82, 0x9A), (0xAD7E)),  # HANGUL SYLLABLE KIYEOK U PIEUPSIOS
    ((0x82, 0x9B), (0xAD80)),  # HANGUL SYLLABLE KIYEOK U SSANGSIOS
    ((0x82, 0x9C), (0xAD83)),  # HANGUL SYLLABLE KIYEOK U CHIEUCH
    ((0x82, 0x9D), (0xAD84)),  # HANGUL SYLLABLE KIYEOK U KHIEUKH
    ((0x82, 0x9E), (0xAD85)),  # HANGUL SYLLABLE KIYEOK U THIEUTH
    ((0x82, 0x9F), (0xAD86)),  # HANGUL SYLLABLE KIYEOK U PHIEUPH
    ((0x82, 0xA0), (0xAD87)),  # HANGUL SYLLABLE KIYEOK U HIEUH
    ((0x82, 0xA1), (0xAD8A)),  # HANGUL SYLLABLE KIYEOK WEO SSANGKIYEOK
    ((0x82, 0xA2), (0xAD8B)),  # HANGUL SYLLABLE KIYEOK WEO KIYEOKSIOS
    ((0x82, 0xA3), (0xAD8D)),  # HANGUL SYLLABLE KIYEOK WEO NIEUNCIEUC
    ((0x82, 0xA4), (0xAD8E)),  # HANGUL SYLLABLE KIYEOK WEO NIEUNHIEUH
    ((0x82, 0xA5), (0xAD8F)),  # HANGUL SYLLABLE KIYEOK WEO TIKEUT
    ((0x82, 0xA6), (0xAD91)),  # HANGUL SYLLABLE KIYEOK WEO RIEULKIYEOK
    ((0x82, 0xA7), (0xAD92)),  # HANGUL SYLLABLE KIYEOK WEO RIEULMIEUM
    ((0x82, 0xA8), (0xAD93)),  # HANGUL SYLLABLE KIYEOK WEO RIEULPIEUP
    ((0x82, 0xA9), (0xAD94)),  # HANGUL SYLLABLE KIYEOK WEO RIEULSIOS
    ((0x82, 0xAA), (0xAD95)),  # HANGUL SYLLABLE KIYEOK WEO RIEULTHIEUTH
    ((0x82, 0xAB), (0xAD96)),  # HANGUL SYLLABLE KIYEOK WEO RIEULPHIEUPH
    ((0x82, 0xAC), (0xAD97)),  # HANGUL SYLLABLE KIYEOK WEO RIEULHIEUH
    ((0x82, 0xAD), (0xAD98)),  # HANGUL SYLLABLE KIYEOK WEO MIEUM
    ((0x82, 0xAE), (0xAD99)),  # HANGUL SYLLABLE KIYEOK WEO PIEUP
    ((0x82, 0xAF), (0xAD9A)),  # HANGUL SYLLABLE KIYEOK WEO PIEUPSIOS
    ((0x82, 0xB0), (0xAD9B)),  # HANGUL SYLLABLE KIYEOK WEO SIOS
    ((0x82, 0xB1), (0xAD9E)),  # HANGUL SYLLABLE KIYEOK WEO CIEUC
    ((0x82, 0xB2), (0xAD9F)),  # HANGUL SYLLABLE KIYEOK WEO CHIEUCH
    ((0x82, 0xB3), (0xADA0)),  # HANGUL SYLLABLE KIYEOK WEO KHIEUKH
    ((0x82, 0xB4), (0xADA1)),  # HANGUL SYLLABLE KIYEOK WEO THIEUTH
    ((0x82, 0xB5), (0xADA2)),  # HANGUL SYLLABLE KIYEOK WEO PHIEUPH
    ((0x82, 0xB6), (0xADA3)),  # HANGUL SYLLABLE KIYEOK WEO HIEUH
    ((0x82, 0xB7), (0xADA5)),  # HANGUL SYLLABLE KIYEOK WE KIYEOK
    ((0x82, 0xB8), (0xADA6)),  # HANGUL SYLLABLE KIYEOK WE SSANGKIYEOK
    ((0x82, 0xB9), (0xADA7)),  # HANGUL SYLLABLE KIYEOK WE KIYEOKSIOS
    ((0x82, 0xBA), (0xADA8)),  # HANGUL SYLLABLE KIYEOK WE NIEUN
    ((0x82, 0xBB), (0xADA9)),  # HANGUL SYLLABLE KIYEOK WE NIEUNCIEUC
    ((0x82, 0xBC), (0xADAA)),  # HANGUL SYLLABLE KIYEOK WE NIEUNHIEUH
    ((0x82, 0xBD), (0xADAB)),  # HANGUL SYLLABLE KIYEOK WE TIKEUT
    ((0x82, 0xBE), (0xADAC)),  # HANGUL SYLLABLE KIYEOK WE RIEUL
    ((0x82, 0xBF), (0xADAD)),  # HANGUL SYLLABLE KIYEOK WE RIEULKIYEOK
    ((0x82, 0xC0), (0xADAE)),  # HANGUL SYLLABLE KIYEOK WE RIEULMIEUM
    ((0x82, 0xC1), (0xADAF)),  # HANGUL SYLLABLE KIYEOK WE RIEULPIEUP
    ((0x82, 0xC2), (0xADB0)),  # HANGUL SYLLABLE KIYEOK WE RIEULSIOS
    ((0x82, 0xC3), (0xADB1)),  # HANGUL SYLLABLE KIYEOK WE RIEULTHIEUTH
    ((0x82, 0xC4), (0xADB2)),  # HANGUL SYLLABLE KIYEOK WE RIEULPHIEUPH
    ((0x82, 0xC5), (0xADB3)),  # HANGUL SYLLABLE KIYEOK WE RIEULHIEUH
    ((0x82, 0xC6), (0xADB4)),  # HANGUL SYLLABLE KIYEOK WE MIEUM
    ((0x82, 0xC7), (0xADB5)),  # HANGUL SYLLABLE KIYEOK WE PIEUP
    ((0x82, 0xC8), (0xADB6)),  # HANGUL SYLLABLE KIYEOK WE PIEUPSIOS
    ((0x82, 0xC9), (0xADB8)),  # HANGUL SYLLABLE KIYEOK WE SSANGSIOS
    ((0x82, 0xCA), (0xADB9)),  # HANGUL SYLLABLE KIYEOK WE IEUNG
    ((0x82, 0xCB), (0xADBA)),  # HANGUL SYLLABLE KIYEOK WE CIEUC
    ((0x82, 0xCC), (0xADBB)),  # HANGUL SYLLABLE KIYEOK WE CHIEUCH
    ((0x82, 0xCD), (0xADBC)),  # HANGUL SYLLABLE KIYEOK WE KHIEUKH
    ((0x82, 0xCE), (0xADBD)),  # HANGUL SYLLABLE KIYEOK WE THIEUTH
    ((0x82, 0xCF), (0xADBE)),  # HANGUL SYLLABLE KIYEOK WE PHIEUPH
    ((0x82, 0xD0), (0xADBF)),  # HANGUL SYLLABLE KIYEOK WE HIEUH
    ((0x82, 0xD1), (0xADC2)),  # HANGUL SYLLABLE KIYEOK WI SSANGKIYEOK
    ((0x82, 0xD2), (0xADC3)),  # HANGUL SYLLABLE KIYEOK WI KIYEOKSIOS
    ((0x82, 0xD3), (0xADC5)),  # HANGUL SYLLABLE KIYEOK WI NIEUNCIEUC
    ((0x82, 0xD4), (0xADC6)),  # HANGUL SYLLABLE KIYEOK WI NIEUNHIEUH
    ((0x82, 0xD5), (0xADC7)),  # HANGUL SYLLABLE KIYEOK WI TIKEUT
    ((0x82, 0xD6), (0xADC9)),  # HANGUL SYLLABLE KIYEOK WI RIEULKIYEOK
    ((0x82, 0xD7), (0xADCA)),  # HANGUL SYLLABLE KIYEOK WI RIEULMIEUM
    ((0x82, 0xD8), (0xADCB)),  # HANGUL SYLLABLE KIYEOK WI RIEULPIEUP
    ((0x82, 0xD9), (0xADCC)),  # HANGUL SYLLABLE KIYEOK WI RIEULSIOS
    ((0x82, 0xDA), (0xADCD)),  # HANGUL SYLLABLE KIYEOK WI RIEULTHIEUTH
    ((0x82, 0xDB), (0xADCE)),  # HANGUL SYLLABLE KIYEOK WI RIEULPHIEUPH
    ((0x82, 0xDC), (0xADCF)),  # HANGUL SYLLABLE KIYEOK WI RIEULHIEUH
    ((0x82, 0xDD), (0xADD2)),  # HANGUL SYLLABLE KIYEOK WI PIEUPSIOS
    ((0x82, 0xDE), (0xADD4)),  # HANGUL SYLLABLE KIYEOK WI SSANGSIOS
    ((0x82, 0xDF), (0xADD5)),  # HANGUL SYLLABLE KIYEOK WI IEUNG
    ((0x82, 0xE0), (0xADD6)),  # HANGUL SYLLABLE KIYEOK WI CIEUC
    ((0x82, 0xE1), (0xADD7)),  # HANGUL SYLLABLE KIYEOK WI CHIEUCH
    ((0x82, 0xE2), (0xADD8)),  # HANGUL SYLLABLE KIYEOK WI KHIEUKH
    ((0x82, 0xE3), (0xADD9)),  # HANGUL SYLLABLE KIYEOK WI THIEUTH
    ((0x82, 0xE4), (0xADDA)),  # HANGUL SYLLABLE KIYEOK WI PHIEUPH
    ((0x82, 0xE5), (0xADDB)),  # HANGUL SYLLABLE KIYEOK WI HIEUH
    ((0x82, 0xE6), (0xADDD)),  # HANGUL SYLLABLE KIYEOK YU KIYEOK
    ((0x82, 0xE7), (0xADDE)),  # HANGUL SYLLABLE KIYEOK YU SSANGKIYEOK
    ((0x82, 0xE8), (0xADDF)),  # HANGUL SYLLABLE KIYEOK YU KIYEOKSIOS
    ((0x82, 0xE9), (0xADE1)),  # HANGUL SYLLABLE KIYEOK YU NIEUNCIEUC
    ((0x82, 0xEA), (0xADE2)),  # HANGUL SYLLABLE KIYEOK YU NIEUNHIEUH
    ((0x82, 0xEB), (0xADE3)),  # HANGUL SYLLABLE KIYEOK YU TIKEUT
    ((0x82, 0xEC), (0xADE5)),  # HANGUL SYLLABLE KIYEOK YU RIEULKIYEOK
    ((0x82, 0xED), (0xADE6)),  # HANGUL SYLLABLE KIYEOK YU RIEULMIEUM
    ((0x82, 0xEE), (0xADE7)),  # HANGUL SYLLABLE KIYEOK YU RIEULPIEUP
    ((0x82, 0xEF), (0xADE8)),  # HANGUL SYLLABLE KIYEOK YU RIEULSIOS
    ((0x82, 0xF0), (0xADE9)),  # HANGUL SYLLABLE KIYEOK YU RIEULTHIEUTH
    ((0x82, 0xF1), (0xADEA)),  # HANGUL SYLLABLE KIYEOK YU RIEULPHIEUPH
    ((0x82, 0xF2), (0xADEB)),  # HANGUL SYLLABLE KIYEOK YU RIEULHIEUH
    ((0x82, 0xF3), (0xADEC)),  # HANGUL SYLLABLE KIYEOK YU MIEUM
    ((0x82, 0xF4), (0xADED)),  # HANGUL SYLLABLE KIYEOK YU PIEUP
    ((0x82, 0xF5), (0xADEE)),  # HANGUL SYLLABLE KIYEOK YU PIEUPSIOS
    ((0x82, 0xF6), (0xADEF)),  # HANGUL SYLLABLE KIYEOK YU SIOS
    ((0x82, 0xF7), (0xADF0)),  # HANGUL SYLLABLE KIYEOK YU SSANGSIOS
    ((0x82, 0xF8), (0xADF1)),  # HANGUL SYLLABLE KIYEOK YU IEUNG
    ((0x82, 0xF9), (0xADF2)),  # HANGUL SYLLABLE KIYEOK YU CIEUC
    ((0x82, 0xFA), (0xADF3)),  # HANGUL SYLLABLE KIYEOK YU CHIEUCH
    ((0x82, 0xFB), (0xADF4)),  # HANGUL SYLLABLE KIYEOK YU KHIEUKH
    ((0x82, 0xFC), (0xADF5)),  # HANGUL SYLLABLE KIYEOK YU THIEUTH
    ((0x82, 0xFD), (0xADF6)),  # HANGUL SYLLABLE KIYEOK YU PHIEUPH
    ((0x82, 0xFE), (0xADF7)),  # HANGUL SYLLABLE KIYEOK YU HIEUH
    ((0x83, 0x41), (0xADFA)),  # HANGUL SYLLABLE KIYEOK EU SSANGKIYEOK
    ((0x83, 0x42), (0xADFB)),  # HANGUL SYLLABLE KIYEOK EU KIYEOKSIOS
    ((0x83, 0x43), (0xADFD)),  # HANGUL SYLLABLE KIYEOK EU NIEUNCIEUC
    ((0x83, 0x44), (0xADFE)),  # HANGUL SYLLABLE KIYEOK EU NIEUNHIEUH
    ((0x83, 0x45), (0xAE02)),  # HANGUL SYLLABLE KIYEOK EU RIEULMIEUM
    ((0x83, 0x46), (0xAE03)),  # HANGUL SYLLABLE KIYEOK EU RIEULPIEUP
    ((0x83, 0x47), (0xAE04)),  # HANGUL SYLLABLE KIYEOK EU RIEULSIOS
    ((0x83, 0x48), (0xAE05)),  # HANGUL SYLLABLE KIYEOK EU RIEULTHIEUTH
    ((0x83, 0x49), (0xAE06)),  # HANGUL SYLLABLE KIYEOK EU RIEULPHIEUPH
    ((0x83, 0x4A), (0xAE07)),  # HANGUL SYLLABLE KIYEOK EU RIEULHIEUH
    ((0x83, 0x4B), (0xAE0A)),  # HANGUL SYLLABLE KIYEOK EU PIEUPSIOS
    ((0x83, 0x4C), (0xAE0C)),  # HANGUL SYLLABLE KIYEOK EU SSANGSIOS
    ((0x83, 0x4D), (0xAE0E)),  # HANGUL SYLLABLE KIYEOK EU CIEUC
    ((0x83, 0x4E), (0xAE0F)),  # HANGUL SYLLABLE KIYEOK EU CHIEUCH
    ((0x83, 0x4F), (0xAE10)),  # HANGUL SYLLABLE KIYEOK EU KHIEUKH
    ((0x83, 0x50), (0xAE11)),  # HANGUL SYLLABLE KIYEOK EU THIEUTH
    ((0x83, 0x51), (0xAE12)),  # HANGUL SYLLABLE KIYEOK EU PHIEUPH
    ((0x83, 0x52), (0xAE13)),  # HANGUL SYLLABLE KIYEOK EU HIEUH
    ((0x83, 0x53), (0xAE15)),  # HANGUL SYLLABLE KIYEOK YI KIYEOK
    ((0x83, 0x54), (0xAE16)),  # HANGUL SYLLABLE KIYEOK YI SSANGKIYEOK
    ((0x83, 0x55), (0xAE17)),  # HANGUL SYLLABLE KIYEOK YI KIYEOKSIOS
    ((0x83, 0x56), (0xAE18)),  # HANGUL SYLLABLE KIYEOK YI NIEUN
    ((0x83, 0x57), (0xAE19)),  # HANGUL SYLLABLE KIYEOK YI NIEUNCIEUC
    ((0x83, 0x58), (0xAE1A)),  # HANGUL SYLLABLE KIYEOK YI NIEUNHIEUH
    ((0x83, 0x59), (0xAE1B)),  # HANGUL SYLLABLE KIYEOK YI TIKEUT
    ((0x83, 0x5A), (0xAE1C)),  # HANGUL SYLLABLE KIYEOK YI RIEUL
    ((0x83, 0x61), (0xAE1D)),  # HANGUL SYLLABLE KIYEOK YI RIEULKIYEOK
    ((0x83, 0x62), (0xAE1E)),  # HANGUL SYLLABLE KIYEOK YI RIEULMIEUM
    ((0x83, 0x63), (0xAE1F)),  # HANGUL SYLLABLE KIYEOK YI RIEULPIEUP
    ((0x83, 0x64), (0xAE20)),  # HANGUL SYLLABLE KIYEOK YI RIEULSIOS
    ((0x83, 0x65), (0xAE21)),  # HANGUL SYLLABLE KIYEOK YI RIEULTHIEUTH
    ((0x83, 0x66), (0xAE22)),  # HANGUL SYLLABLE KIYEOK YI RIEULPHIEUPH
    ((0x83, 0x67), (0xAE23)),  # HANGUL SYLLABLE KIYEOK YI RIEULHIEUH
    ((0x83, 0x68), (0xAE24)),  # HANGUL SYLLABLE KIYEOK YI MIEUM
    ((0x83, 0x69), (0xAE25)),  # HANGUL SYLLABLE KIYEOK YI PIEUP
    ((0x83, 0x6A), (0xAE26)),  # HANGUL SYLLABLE KIYEOK YI PIEUPSIOS
    ((0x83, 0x6B), (0xAE27)),  # HANGUL SYLLABLE KIYEOK YI SIOS
    ((0x83, 0x6C), (0xAE28)),  # HANGUL SYLLABLE KIYEOK YI SSANGSIOS
    ((0x83, 0x6D), (0xAE29)),  # HANGUL SYLLABLE KIYEOK YI IEUNG
    ((0x83, 0x6E), (0xAE2A)),  # HANGUL SYLLABLE KIYEOK YI CIEUC
    ((0x83, 0x6F), (0xAE2B)),  # HANGUL SYLLABLE KIYEOK YI CHIEUCH
    ((0x83, 0x70), (0xAE2C)),  # HANGUL SYLLABLE KIYEOK YI KHIEUKH
    ((0x83, 0x71), (0xAE2D)),  # HANGUL SYLLABLE KIYEOK YI THIEUTH
    ((0x83, 0x72), (0xAE2E)),  # HANGUL SYLLABLE KIYEOK YI PHIEUPH
    ((0x83, 0x73), (0xAE2F)),  # HANGUL SYLLABLE KIYEOK YI HIEUH
    ((0x83, 0x74), (0xAE32)),  # HANGUL SYLLABLE KIYEOK I SSANGKIYEOK
    ((0x83, 0x75), (0xAE33)),  # HANGUL SYLLABLE KIYEOK I KIYEOKSIOS
    ((0x83, 0x76), (0xAE35)),  # HANGUL SYLLABLE KIYEOK I NIEUNCIEUC
    ((0x83, 0x77), (0xAE36)),  # HANGUL SYLLABLE KIYEOK I NIEUNHIEUH
    ((0x83, 0x78), (0xAE39)),  # HANGUL SYLLABLE KIYEOK I RIEULKIYEOK
    ((0x83, 0x79), (0xAE3B)),  # HANGUL SYLLABLE KIYEOK I RIEULPIEUP
    ((0x83, 0x7A), (0xAE3C)),  # HANGUL SYLLABLE KIYEOK I RIEULSIOS
    ((0x83, 0x81), (0xAE3D)),  # HANGUL SYLLABLE KIYEOK I RIEULTHIEUTH
    ((0x83, 0x82), (0xAE3E)),  # HANGUL SYLLABLE KIYEOK I RIEULPHIEUPH
    ((0x83, 0x83), (0xAE3F)),  # HANGUL SYLLABLE KIYEOK I RIEULHIEUH
    ((0x83, 0x84), (0xAE42)),  # HANGUL SYLLABLE KIYEOK I PIEUPSIOS
    ((0x83, 0x85), (0xAE44)),  # HANGUL SYLLABLE KIYEOK I SSANGSIOS
    ((0x83, 0x86), (0xAE47)),  # HANGUL SYLLABLE KIYEOK I CHIEUCH
    ((0x83, 0x87), (0xAE48)),  # HANGUL SYLLABLE KIYEOK I KHIEUKH
    ((0x83, 0x88), (0xAE49)),  # HANGUL SYLLABLE KIYEOK I THIEUTH
    ((0x83, 0x89), (0xAE4B)),  # HANGUL SYLLABLE KIYEOK I HIEUH
    ((0x83, 0x8A), (0xAE4F)),  # HANGUL SYLLABLE SSANGKIYEOK A KIYEOKSIOS
    ((0x83, 0x8B), (0xAE51)),  # HANGUL SYLLABLE SSANGKIYEOK A NIEUNCIEUC
    ((0x83, 0x8C), (0xAE52)),  # HANGUL SYLLABLE SSANGKIYEOK A NIEUNHIEUH
    ((0x83, 0x8D), (0xAE53)),  # HANGUL SYLLABLE SSANGKIYEOK A TIKEUT
    ((0x83, 0x8E), (0xAE55)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULKIYEOK
    ((0x83, 0x8F), (0xAE57)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULPIEUP
    ((0x83, 0x90), (0xAE58)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULSIOS
    ((0x83, 0x91), (0xAE59)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULTHIEUTH
    ((0x83, 0x92), (0xAE5A)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULPHIEUPH
    ((0x83, 0x93), (0xAE5B)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULHIEUH
    ((0x83, 0x94), (0xAE5E)),  # HANGUL SYLLABLE SSANGKIYEOK A PIEUPSIOS
    ((0x83, 0x95), (0xAE62)),  # HANGUL SYLLABLE SSANGKIYEOK A CIEUC
    ((0x83, 0x96), (0xAE63)),  # HANGUL SYLLABLE SSANGKIYEOK A CHIEUCH
    ((0x83, 0x97), (0xAE64)),  # HANGUL SYLLABLE SSANGKIYEOK A KHIEUKH
    ((0x83, 0x98), (0xAE66)),  # HANGUL SYLLABLE SSANGKIYEOK A PHIEUPH
    ((0x83, 0x99), (0xAE67)),  # HANGUL SYLLABLE SSANGKIYEOK A HIEUH
    ((0x83, 0x9A), (0xAE6A)),  # HANGUL SYLLABLE SSANGKIYEOK AE SSANGKIYEOK
    ((0x83, 0x9B), (0xAE6B)),  # HANGUL SYLLABLE SSANGKIYEOK AE KIYEOKSIOS
    ((0x83, 0x9C), (0xAE6D)),  # HANGUL SYLLABLE SSANGKIYEOK AE NIEUNCIEUC
    ((0x83, 0x9D), (0xAE6E)),  # HANGUL SYLLABLE SSANGKIYEOK AE NIEUNHIEUH
    ((0x83, 0x9E), (0xAE6F)),  # HANGUL SYLLABLE SSANGKIYEOK AE TIKEUT
    ((0x83, 0x9F), (0xAE71)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULKIYEOK
    ((0x83, 0xA0), (0xAE72)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULMIEUM
    ((0x83, 0xA1), (0xAE73)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULPIEUP
    ((0x83, 0xA2), (0xAE74)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULSIOS
    ((0x83, 0xA3), (0xAE75)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULTHIEUTH
    ((0x83, 0xA4), (0xAE76)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULPHIEUPH
    ((0x83, 0xA5), (0xAE77)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEULHIEUH
    ((0x83, 0xA6), (0xAE7A)),  # HANGUL SYLLABLE SSANGKIYEOK AE PIEUPSIOS
    ((0x83, 0xA7), (0xAE7E)),  # HANGUL SYLLABLE SSANGKIYEOK AE CIEUC
    ((0x83, 0xA8), (0xAE7F)),  # HANGUL SYLLABLE SSANGKIYEOK AE CHIEUCH
    ((0x83, 0xA9), (0xAE80)),  # HANGUL SYLLABLE SSANGKIYEOK AE KHIEUKH
    ((0x83, 0xAA), (0xAE81)),  # HANGUL SYLLABLE SSANGKIYEOK AE THIEUTH
    ((0x83, 0xAB), (0xAE82)),  # HANGUL SYLLABLE SSANGKIYEOK AE PHIEUPH
    ((0x83, 0xAC), (0xAE83)),  # HANGUL SYLLABLE SSANGKIYEOK AE HIEUH
    ((0x83, 0xAD), (0xAE86)),  # HANGUL SYLLABLE SSANGKIYEOK YA SSANGKIYEOK
    ((0x83, 0xAE), (0xAE87)),  # HANGUL SYLLABLE SSANGKIYEOK YA KIYEOKSIOS
    ((0x83, 0xAF), (0xAE88)),  # HANGUL SYLLABLE SSANGKIYEOK YA NIEUN
    ((0x83, 0xB0), (0xAE89)),  # HANGUL SYLLABLE SSANGKIYEOK YA NIEUNCIEUC
    ((0x83, 0xB1), (0xAE8A)),  # HANGUL SYLLABLE SSANGKIYEOK YA NIEUNHIEUH
    ((0x83, 0xB2), (0xAE8B)),  # HANGUL SYLLABLE SSANGKIYEOK YA TIKEUT
    ((0x83, 0xB3), (0xAE8D)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULKIYEOK
    ((0x83, 0xB4), (0xAE8E)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULMIEUM
    ((0x83, 0xB5), (0xAE8F)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULPIEUP
    ((0x83, 0xB6), (0xAE90)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULSIOS
    ((0x83, 0xB7), (0xAE91)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULTHIEUTH
    ((0x83, 0xB8), (0xAE92)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULPHIEUPH
    ((0x83, 0xB9), (0xAE93)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEULHIEUH
    ((0x83, 0xBA), (0xAE94)),  # HANGUL SYLLABLE SSANGKIYEOK YA MIEUM
    ((0x83, 0xBB), (0xAE95)),  # HANGUL SYLLABLE SSANGKIYEOK YA PIEUP
    ((0x83, 0xBC), (0xAE96)),  # HANGUL SYLLABLE SSANGKIYEOK YA PIEUPSIOS
    ((0x83, 0xBD), (0xAE97)),  # HANGUL SYLLABLE SSANGKIYEOK YA SIOS
    ((0x83, 0xBE), (0xAE98)),  # HANGUL SYLLABLE SSANGKIYEOK YA SSANGSIOS
    ((0x83, 0xBF), (0xAE99)),  # HANGUL SYLLABLE SSANGKIYEOK YA IEUNG
    ((0x83, 0xC0), (0xAE9A)),  # HANGUL SYLLABLE SSANGKIYEOK YA CIEUC
    ((0x83, 0xC1), (0xAE9B)),  # HANGUL SYLLABLE SSANGKIYEOK YA CHIEUCH
    ((0x83, 0xC2), (0xAE9C)),  # HANGUL SYLLABLE SSANGKIYEOK YA KHIEUKH
    ((0x83, 0xC3), (0xAE9D)),  # HANGUL SYLLABLE SSANGKIYEOK YA THIEUTH
    ((0x83, 0xC4), (0xAE9E)),  # HANGUL SYLLABLE SSANGKIYEOK YA PHIEUPH
    ((0x83, 0xC5), (0xAE9F)),  # HANGUL SYLLABLE SSANGKIYEOK YA HIEUH
    ((0x83, 0xC6), (0xAEA0)),  # HANGUL SYLLABLE SSANGKIYEOK YAE
    ((0x83, 0xC7), (0xAEA1)),  # HANGUL SYLLABLE SSANGKIYEOK YAE KIYEOK
    ((0x83, 0xC8), (0xAEA2)),  # HANGUL SYLLABLE SSANGKIYEOK YAE SSANGKIYEOK
    ((0x83, 0xC9), (0xAEA3)),  # HANGUL SYLLABLE SSANGKIYEOK YAE KIYEOKSIOS
    ((0x83, 0xCA), (0xAEA4)),  # HANGUL SYLLABLE SSANGKIYEOK YAE NIEUN
    ((0x83, 0xCB), (0xAEA5)),  # HANGUL SYLLABLE SSANGKIYEOK YAE NIEUNCIEUC
    ((0x83, 0xCC), (0xAEA6)),  # HANGUL SYLLABLE SSANGKIYEOK YAE NIEUNHIEUH
    ((0x83, 0xCD), (0xAEA7)),  # HANGUL SYLLABLE SSANGKIYEOK YAE TIKEUT
    ((0x83, 0xCE), (0xAEA8)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEUL
    ((0x83, 0xCF), (0xAEA9)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULKIYEOK
    ((0x83, 0xD0), (0xAEAA)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULMIEUM
    ((0x83, 0xD1), (0xAEAB)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULPIEUP
    ((0x83, 0xD2), (0xAEAC)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULSIOS
    ((0x83, 0xD3), (0xAEAD)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULTHIEUTH
    ((0x83, 0xD4), (0xAEAE)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULPHIEUPH
    ((0x83, 0xD5), (0xAEAF)),  # HANGUL SYLLABLE SSANGKIYEOK YAE RIEULHIEUH
    ((0x83, 0xD6), (0xAEB0)),  # HANGUL SYLLABLE SSANGKIYEOK YAE MIEUM
    ((0x83, 0xD7), (0xAEB1)),  # HANGUL SYLLABLE SSANGKIYEOK YAE PIEUP
    ((0x83, 0xD8), (0xAEB2)),  # HANGUL SYLLABLE SSANGKIYEOK YAE PIEUPSIOS
    ((0x83, 0xD9), (0xAEB3)),  # HANGUL SYLLABLE SSANGKIYEOK YAE SIOS
    ((0x83, 0xDA), (0xAEB4)),  # HANGUL SYLLABLE SSANGKIYEOK YAE SSANGSIOS
    ((0x83, 0xDB), (0xAEB5)),  # HANGUL SYLLABLE SSANGKIYEOK YAE IEUNG
    ((0x83, 0xDC), (0xAEB6)),  # HANGUL SYLLABLE SSANGKIYEOK YAE CIEUC
    ((0x83, 0xDD), (0xAEB7)),  # HANGUL SYLLABLE SSANGKIYEOK YAE CHIEUCH
    ((0x83, 0xDE), (0xAEB8)),  # HANGUL SYLLABLE SSANGKIYEOK YAE KHIEUKH
    ((0x83, 0xDF), (0xAEB9)),  # HANGUL SYLLABLE SSANGKIYEOK YAE THIEUTH
    ((0x83, 0xE0), (0xAEBA)),  # HANGUL SYLLABLE SSANGKIYEOK YAE PHIEUPH
    ((0x83, 0xE1), (0xAEBB)),  # HANGUL SYLLABLE SSANGKIYEOK YAE HIEUH
    ((0x83, 0xE2), (0xAEBF)),  # HANGUL SYLLABLE SSANGKIYEOK EO KIYEOKSIOS
    ((0x83, 0xE3), (0xAEC1)),  # HANGUL SYLLABLE SSANGKIYEOK EO NIEUNCIEUC
    ((0x83, 0xE4), (0xAEC2)),  # HANGUL SYLLABLE SSANGKIYEOK EO NIEUNHIEUH
    ((0x83, 0xE5), (0xAEC3)),  # HANGUL SYLLABLE SSANGKIYEOK EO TIKEUT
    ((0x83, 0xE6), (0xAEC5)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULKIYEOK
    ((0x83, 0xE7), (0xAEC6)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULMIEUM
    ((0x83, 0xE8), (0xAEC7)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULPIEUP
    ((0x83, 0xE9), (0xAEC8)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULSIOS
    ((0x83, 0xEA), (0xAEC9)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULTHIEUTH
    ((0x83, 0xEB), (0xAECA)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULPHIEUPH
    ((0x83, 0xEC), (0xAECB)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEULHIEUH
    ((0x83, 0xED), (0xAECE)),  # HANGUL SYLLABLE SSANGKIYEOK EO PIEUPSIOS
    ((0x83, 0xEE), (0xAED2)),  # HANGUL SYLLABLE SSANGKIYEOK EO CIEUC
    ((0x83, 0xEF), (0xAED3)),  # HANGUL SYLLABLE SSANGKIYEOK EO CHIEUCH
    ((0x83, 0xF0), (0xAED4)),  # HANGUL SYLLABLE SSANGKIYEOK EO KHIEUKH
    ((0x83, 0xF1), (0xAED5)),  # HANGUL SYLLABLE SSANGKIYEOK EO THIEUTH
    ((0x83, 0xF2), (0xAED6)),  # HANGUL SYLLABLE SSANGKIYEOK EO PHIEUPH
    ((0x83, 0xF3), (0xAED7)),  # HANGUL SYLLABLE SSANGKIYEOK EO HIEUH
    ((0x83, 0xF4), (0xAEDA)),  # HANGUL SYLLABLE SSANGKIYEOK E SSANGKIYEOK
    ((0x83, 0xF5), (0xAEDB)),  # HANGUL SYLLABLE SSANGKIYEOK E KIYEOKSIOS
    ((0x83, 0xF6), (0xAEDD)),  # HANGUL SYLLABLE SSANGKIYEOK E NIEUNCIEUC
    ((0x83, 0xF7), (0xAEDE)),  # HANGUL SYLLABLE SSANGKIYEOK E NIEUNHIEUH
    ((0x83, 0xF8), (0xAEDF)),  # HANGUL SYLLABLE SSANGKIYEOK E TIKEUT
    ((0x83, 0xF9), (0xAEE0)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEUL
    ((0x83, 0xFA), (0xAEE1)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULKIYEOK
    ((0x83, 0xFB), (0xAEE2)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULMIEUM
    ((0x83, 0xFC), (0xAEE3)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULPIEUP
    ((0x83, 0xFD), (0xAEE4)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULSIOS
    ((0x83, 0xFE), (0xAEE5)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULTHIEUTH
    ((0x84, 0x41), (0xAEE6)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULPHIEUPH
    ((0x84, 0x42), (0xAEE7)),  # HANGUL SYLLABLE SSANGKIYEOK E RIEULHIEUH
    ((0x84, 0x43), (0xAEE9)),  # HANGUL SYLLABLE SSANGKIYEOK E PIEUP
    ((0x84, 0x44), (0xAEEA)),  # HANGUL SYLLABLE SSANGKIYEOK E PIEUPSIOS
    ((0x84, 0x45), (0xAEEC)),  # HANGUL SYLLABLE SSANGKIYEOK E SSANGSIOS
    ((0x84, 0x46), (0xAEEE)),  # HANGUL SYLLABLE SSANGKIYEOK E CIEUC
    ((0x84, 0x47), (0xAEEF)),  # HANGUL SYLLABLE SSANGKIYEOK E CHIEUCH
    ((0x84, 0x48), (0xAEF0)),  # HANGUL SYLLABLE SSANGKIYEOK E KHIEUKH
    ((0x84, 0x49), (0xAEF1)),  # HANGUL SYLLABLE SSANGKIYEOK E THIEUTH
    ((0x84, 0x4A), (0xAEF2)),  # HANGUL SYLLABLE SSANGKIYEOK E PHIEUPH
    ((0x84, 0x4B), (0xAEF3)),  # HANGUL SYLLABLE SSANGKIYEOK E HIEUH
    ((0x84, 0x4C), (0xAEF5)),  # HANGUL SYLLABLE SSANGKIYEOK YEO KIYEOK
    ((0x84, 0x4D), (0xAEF6)),  # HANGUL SYLLABLE SSANGKIYEOK YEO SSANGKIYEOK
    ((0x84, 0x4E), (0xAEF7)),  # HANGUL SYLLABLE SSANGKIYEOK YEO KIYEOKSIOS
    ((0x84, 0x4F), (0xAEF9)),  # HANGUL SYLLABLE SSANGKIYEOK YEO NIEUNCIEUC
    ((0x84, 0x50), (0xAEFA)),  # HANGUL SYLLABLE SSANGKIYEOK YEO NIEUNHIEUH
    ((0x84, 0x51), (0xAEFB)),  # HANGUL SYLLABLE SSANGKIYEOK YEO TIKEUT
    ((0x84, 0x52), (0xAEFD)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULKIYEOK
    ((0x84, 0x53), (0xAEFE)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULMIEUM
    ((0x84, 0x54), (0xAEFF)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULPIEUP
    ((0x84, 0x55), (0xAF00)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULSIOS
    ((0x84, 0x56), (0xAF01)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULTHIEUTH
    ((0x84, 0x57), (0xAF02)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULPHIEUPH
    ((0x84, 0x58), (0xAF03)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEULHIEUH
    ((0x84, 0x59), (0xAF04)),  # HANGUL SYLLABLE SSANGKIYEOK YEO MIEUM
    ((0x84, 0x5A), (0xAF05)),  # HANGUL SYLLABLE SSANGKIYEOK YEO PIEUP
    ((0x84, 0x61), (0xAF06)),  # HANGUL SYLLABLE SSANGKIYEOK YEO PIEUPSIOS
    ((0x84, 0x62), (0xAF09)),  # HANGUL SYLLABLE SSANGKIYEOK YEO IEUNG
    ((0x84, 0x63), (0xAF0A)),  # HANGUL SYLLABLE SSANGKIYEOK YEO CIEUC
    ((0x84, 0x64), (0xAF0B)),  # HANGUL SYLLABLE SSANGKIYEOK YEO CHIEUCH
    ((0x84, 0x65), (0xAF0C)),  # HANGUL SYLLABLE SSANGKIYEOK YEO KHIEUKH
    ((0x84, 0x66), (0xAF0E)),  # HANGUL SYLLABLE SSANGKIYEOK YEO PHIEUPH
    ((0x84, 0x67), (0xAF0F)),  # HANGUL SYLLABLE SSANGKIYEOK YEO HIEUH
    ((0x84, 0x68), (0xAF11)),  # HANGUL SYLLABLE SSANGKIYEOK YE KIYEOK
    ((0x84, 0x69), (0xAF12)),  # HANGUL SYLLABLE SSANGKIYEOK YE SSANGKIYEOK
    ((0x84, 0x6A), (0xAF13)),  # HANGUL SYLLABLE SSANGKIYEOK YE KIYEOKSIOS
    ((0x84, 0x6B), (0xAF14)),  # HANGUL SYLLABLE SSANGKIYEOK YE NIEUN
    ((0x84, 0x6C), (0xAF15)),  # HANGUL SYLLABLE SSANGKIYEOK YE NIEUNCIEUC
    ((0x84, 0x6D), (0xAF16)),  # HANGUL SYLLABLE SSANGKIYEOK YE NIEUNHIEUH
    ((0x84, 0x6E), (0xAF17)),  # HANGUL SYLLABLE SSANGKIYEOK YE TIKEUT
    ((0x84, 0x6F), (0xAF18)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEUL
    ((0x84, 0x70), (0xAF19)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULKIYEOK
    ((0x84, 0x71), (0xAF1A)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULMIEUM
    ((0x84, 0x72), (0xAF1B)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULPIEUP
    ((0x84, 0x73), (0xAF1C)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULSIOS
    ((0x84, 0x74), (0xAF1D)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULTHIEUTH
    ((0x84, 0x75), (0xAF1E)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULPHIEUPH
    ((0x84, 0x76), (0xAF1F)),  # HANGUL SYLLABLE SSANGKIYEOK YE RIEULHIEUH
    ((0x84, 0x77), (0xAF20)),  # HANGUL SYLLABLE SSANGKIYEOK YE MIEUM
    ((0x84, 0x78), (0xAF21)),  # HANGUL SYLLABLE SSANGKIYEOK YE PIEUP
    ((0x84, 0x79), (0xAF22)),  # HANGUL SYLLABLE SSANGKIYEOK YE PIEUPSIOS
    ((0x84, 0x7A), (0xAF23)),  # HANGUL SYLLABLE SSANGKIYEOK YE SIOS
    ((0x84, 0x81), (0xAF24)),  # HANGUL SYLLABLE SSANGKIYEOK YE SSANGSIOS
    ((0x84, 0x82), (0xAF25)),  # HANGUL SYLLABLE SSANGKIYEOK YE IEUNG
    ((0x84, 0x83), (0xAF26)),  # HANGUL SYLLABLE SSANGKIYEOK YE CIEUC
    ((0x84, 0x84), (0xAF27)),  # HANGUL SYLLABLE SSANGKIYEOK YE CHIEUCH
    ((0x84, 0x85), (0xAF28)),  # HANGUL SYLLABLE SSANGKIYEOK YE KHIEUKH
    ((0x84, 0x86), (0xAF29)),  # HANGUL SYLLABLE SSANGKIYEOK YE THIEUTH
    ((0x84, 0x87), (0xAF2A)),  # HANGUL SYLLABLE SSANGKIYEOK YE PHIEUPH
    ((0x84, 0x88), (0xAF2B)),  # HANGUL SYLLABLE SSANGKIYEOK YE HIEUH
    ((0x84, 0x89), (0xAF2E)),  # HANGUL SYLLABLE SSANGKIYEOK O SSANGKIYEOK
    ((0x84, 0x8A), (0xAF2F)),  # HANGUL SYLLABLE SSANGKIYEOK O KIYEOKSIOS
    ((0x84, 0x8B), (0xAF31)),  # HANGUL SYLLABLE SSANGKIYEOK O NIEUNCIEUC
    ((0x84, 0x8C), (0xAF33)),  # HANGUL SYLLABLE SSANGKIYEOK O TIKEUT
    ((0x84, 0x8D), (0xAF35)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULKIYEOK
    ((0x84, 0x8E), (0xAF36)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULMIEUM
    ((0x84, 0x8F), (0xAF37)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULPIEUP
    ((0x84, 0x90), (0xAF38)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULSIOS
    ((0x84, 0x91), (0xAF39)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULTHIEUTH
    ((0x84, 0x92), (0xAF3A)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULPHIEUPH
    ((0x84, 0x93), (0xAF3B)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEULHIEUH
    ((0x84, 0x94), (0xAF3E)),  # HANGUL SYLLABLE SSANGKIYEOK O PIEUPSIOS
    ((0x84, 0x95), (0xAF40)),  # HANGUL SYLLABLE SSANGKIYEOK O SSANGSIOS
    ((0x84, 0x96), (0xAF44)),  # HANGUL SYLLABLE SSANGKIYEOK O KHIEUKH
    ((0x84, 0x97), (0xAF45)),  # HANGUL SYLLABLE SSANGKIYEOK O THIEUTH
    ((0x84, 0x98), (0xAF46)),  # HANGUL SYLLABLE SSANGKIYEOK O PHIEUPH
    ((0x84, 0x99), (0xAF47)),  # HANGUL SYLLABLE SSANGKIYEOK O HIEUH
    ((0x84, 0x9A), (0xAF4A)),  # HANGUL SYLLABLE SSANGKIYEOK WA SSANGKIYEOK
    ((0x84, 0x9B), (0xAF4B)),  # HANGUL SYLLABLE SSANGKIYEOK WA KIYEOKSIOS
    ((0x84, 0x9C), (0xAF4C)),  # HANGUL SYLLABLE SSANGKIYEOK WA NIEUN
    ((0x84, 0x9D), (0xAF4D)),  # HANGUL SYLLABLE SSANGKIYEOK WA NIEUNCIEUC
    ((0x84, 0x9E), (0xAF4E)),  # HANGUL SYLLABLE SSANGKIYEOK WA NIEUNHIEUH
    ((0x84, 0x9F), (0xAF4F)),  # HANGUL SYLLABLE SSANGKIYEOK WA TIKEUT
    ((0x84, 0xA0), (0xAF51)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULKIYEOK
    ((0x84, 0xA1), (0xAF52)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULMIEUM
    ((0x84, 0xA2), (0xAF53)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULPIEUP
    ((0x84, 0xA3), (0xAF54)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULSIOS
    ((0x84, 0xA4), (0xAF55)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULTHIEUTH
    ((0x84, 0xA5), (0xAF56)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULPHIEUPH
    ((0x84, 0xA6), (0xAF57)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEULHIEUH
    ((0x84, 0xA7), (0xAF58)),  # HANGUL SYLLABLE SSANGKIYEOK WA MIEUM
    ((0x84, 0xA8), (0xAF59)),  # HANGUL SYLLABLE SSANGKIYEOK WA PIEUP
    ((0x84, 0xA9), (0xAF5A)),  # HANGUL SYLLABLE SSANGKIYEOK WA PIEUPSIOS
    ((0x84, 0xAA), (0xAF5B)),  # HANGUL SYLLABLE SSANGKIYEOK WA SIOS
    ((0x84, 0xAB), (0xAF5E)),  # HANGUL SYLLABLE SSANGKIYEOK WA CIEUC
    ((0x84, 0xAC), (0xAF5F)),  # HANGUL SYLLABLE SSANGKIYEOK WA CHIEUCH
    ((0x84, 0xAD), (0xAF60)),  # HANGUL SYLLABLE SSANGKIYEOK WA KHIEUKH
    ((0x84, 0xAE), (0xAF61)),  # HANGUL SYLLABLE SSANGKIYEOK WA THIEUTH
    ((0x84, 0xAF), (0xAF62)),  # HANGUL SYLLABLE SSANGKIYEOK WA PHIEUPH
    ((0x84, 0xB0), (0xAF63)),  # HANGUL SYLLABLE SSANGKIYEOK WA HIEUH
    ((0x84, 0xB1), (0xAF66)),  # HANGUL SYLLABLE SSANGKIYEOK WAE SSANGKIYEOK
    ((0x84, 0xB2), (0xAF67)),  # HANGUL SYLLABLE SSANGKIYEOK WAE KIYEOKSIOS
    ((0x84, 0xB3), (0xAF68)),  # HANGUL SYLLABLE SSANGKIYEOK WAE NIEUN
    ((0x84, 0xB4), (0xAF69)),  # HANGUL SYLLABLE SSANGKIYEOK WAE NIEUNCIEUC
    ((0x84, 0xB5), (0xAF6A)),  # HANGUL SYLLABLE SSANGKIYEOK WAE NIEUNHIEUH
    ((0x84, 0xB6), (0xAF6B)),  # HANGUL SYLLABLE SSANGKIYEOK WAE TIKEUT
    ((0x84, 0xB7), (0xAF6C)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEUL
    ((0x84, 0xB8), (0xAF6D)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULKIYEOK
    ((0x84, 0xB9), (0xAF6E)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULMIEUM
    ((0x84, 0xBA), (0xAF6F)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULPIEUP
    ((0x84, 0xBB), (0xAF70)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULSIOS
    ((0x84, 0xBC), (0xAF71)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULTHIEUTH
    ((0x84, 0xBD), (0xAF72)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULPHIEUPH
    ((0x84, 0xBE), (0xAF73)),  # HANGUL SYLLABLE SSANGKIYEOK WAE RIEULHIEUH
    ((0x84, 0xBF), (0xAF74)),  # HANGUL SYLLABLE SSANGKIYEOK WAE MIEUM
    ((0x84, 0xC0), (0xAF75)),  # HANGUL SYLLABLE SSANGKIYEOK WAE PIEUP
    ((0x84, 0xC1), (0xAF76)),  # HANGUL SYLLABLE SSANGKIYEOK WAE PIEUPSIOS
    ((0x84, 0xC2), (0xAF77)),  # HANGUL SYLLABLE SSANGKIYEOK WAE SIOS
    ((0x84, 0xC3), (0xAF78)),  # HANGUL SYLLABLE SSANGKIYEOK WAE SSANGSIOS
    ((0x84, 0xC4), (0xAF7A)),  # HANGUL SYLLABLE SSANGKIYEOK WAE CIEUC
    ((0x84, 0xC5), (0xAF7B)),  # HANGUL SYLLABLE SSANGKIYEOK WAE CHIEUCH
    ((0x84, 0xC6), (0xAF7C)),  # HANGUL SYLLABLE SSANGKIYEOK WAE KHIEUKH
    ((0x84, 0xC7), (0xAF7D)),  # HANGUL SYLLABLE SSANGKIYEOK WAE THIEUTH
    ((0x84, 0xC8), (0xAF7E)),  # HANGUL SYLLABLE SSANGKIYEOK WAE PHIEUPH
    ((0x84, 0xC9), (0xAF7F)),  # HANGUL SYLLABLE SSANGKIYEOK WAE HIEUH
    ((0x84, 0xCA), (0xAF81)),  # HANGUL SYLLABLE SSANGKIYEOK OE KIYEOK
    ((0x84, 0xCB), (0xAF82)),  # HANGUL SYLLABLE SSANGKIYEOK OE SSANGKIYEOK
    ((0x84, 0xCC), (0xAF83)),  # HANGUL SYLLABLE SSANGKIYEOK OE KIYEOKSIOS
    ((0x84, 0xCD), (0xAF85)),  # HANGUL SYLLABLE SSANGKIYEOK OE NIEUNCIEUC
    ((0x84, 0xCE), (0xAF86)),  # HANGUL SYLLABLE SSANGKIYEOK OE NIEUNHIEUH
    ((0x84, 0xCF), (0xAF87)),  # HANGUL SYLLABLE SSANGKIYEOK OE TIKEUT
    ((0x84, 0xD0), (0xAF89)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULKIYEOK
    ((0x84, 0xD1), (0xAF8A)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULMIEUM
    ((0x84, 0xD2), (0xAF8B)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULPIEUP
    ((0x84, 0xD3), (0xAF8C)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULSIOS
    ((0x84, 0xD4), (0xAF8D)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULTHIEUTH
    ((0x84, 0xD5), (0xAF8E)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULPHIEUPH
    ((0x84, 0xD6), (0xAF8F)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEULHIEUH
    ((0x84, 0xD7), (0xAF92)),  # HANGUL SYLLABLE SSANGKIYEOK OE PIEUPSIOS
    ((0x84, 0xD8), (0xAF93)),  # HANGUL SYLLABLE SSANGKIYEOK OE SIOS
    ((0x84, 0xD9), (0xAF94)),  # HANGUL SYLLABLE SSANGKIYEOK OE SSANGSIOS
    ((0x84, 0xDA), (0xAF96)),  # HANGUL SYLLABLE SSANGKIYEOK OE CIEUC
    ((0x84, 0xDB), (0xAF97)),  # HANGUL SYLLABLE SSANGKIYEOK OE CHIEUCH
    ((0x84, 0xDC), (0xAF98)),  # HANGUL SYLLABLE SSANGKIYEOK OE KHIEUKH
    ((0x84, 0xDD), (0xAF99)),  # HANGUL SYLLABLE SSANGKIYEOK OE THIEUTH
    ((0x84, 0xDE), (0xAF9A)),  # HANGUL SYLLABLE SSANGKIYEOK OE PHIEUPH
    ((0x84, 0xDF), (0xAF9B)),  # HANGUL SYLLABLE SSANGKIYEOK OE HIEUH
    ((0x84, 0xE0), (0xAF9D)),  # HANGUL SYLLABLE SSANGKIYEOK YO KIYEOK
    ((0x84, 0xE1), (0xAF9E)),  # HANGUL SYLLABLE SSANGKIYEOK YO SSANGKIYEOK
    ((0x84, 0xE2), (0xAF9F)),  # HANGUL SYLLABLE SSANGKIYEOK YO KIYEOKSIOS
    ((0x84, 0xE3), (0xAFA0)),  # HANGUL SYLLABLE SSANGKIYEOK YO NIEUN
    ((0x84, 0xE4), (0xAFA1)),  # HANGUL SYLLABLE SSANGKIYEOK YO NIEUNCIEUC
    ((0x84, 0xE5), (0xAFA2)),  # HANGUL SYLLABLE SSANGKIYEOK YO NIEUNHIEUH
    ((0x84, 0xE6), (0xAFA3)),  # HANGUL SYLLABLE SSANGKIYEOK YO TIKEUT
    ((0x84, 0xE7), (0xAFA4)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEUL
    ((0x84, 0xE8), (0xAFA5)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULKIYEOK
    ((0x84, 0xE9), (0xAFA6)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULMIEUM
    ((0x84, 0xEA), (0xAFA7)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULPIEUP
    ((0x84, 0xEB), (0xAFA8)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULSIOS
    ((0x84, 0xEC), (0xAFA9)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULTHIEUTH
    ((0x84, 0xED), (0xAFAA)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULPHIEUPH
    ((0x84, 0xEE), (0xAFAB)),  # HANGUL SYLLABLE SSANGKIYEOK YO RIEULHIEUH
    ((0x84, 0xEF), (0xAFAC)),  # HANGUL SYLLABLE SSANGKIYEOK YO MIEUM
    ((0x84, 0xF0), (0xAFAD)),  # HANGUL SYLLABLE SSANGKIYEOK YO PIEUP
    ((0x84, 0xF1), (0xAFAE)),  # HANGUL SYLLABLE SSANGKIYEOK YO PIEUPSIOS
    ((0x84, 0xF2), (0xAFAF)),  # HANGUL SYLLABLE SSANGKIYEOK YO SIOS
    ((0x84, 0xF3), (0xAFB0)),  # HANGUL SYLLABLE SSANGKIYEOK YO SSANGSIOS
    ((0x84, 0xF4), (0xAFB1)),  # HANGUL SYLLABLE SSANGKIYEOK YO IEUNG
    ((0x84, 0xF5), (0xAFB2)),  # HANGUL SYLLABLE SSANGKIYEOK YO CIEUC
    ((0x84, 0xF6), (0xAFB3)),  # HANGUL SYLLABLE SSANGKIYEOK YO CHIEUCH
    ((0x84, 0xF7), (0xAFB4)),  # HANGUL SYLLABLE SSANGKIYEOK YO KHIEUKH
    ((0x84, 0xF8), (0xAFB5)),  # HANGUL SYLLABLE SSANGKIYEOK YO THIEUTH
    ((0x84, 0xF9), (0xAFB6)),  # HANGUL SYLLABLE SSANGKIYEOK YO PHIEUPH
    ((0x84, 0xFA), (0xAFB7)),  # HANGUL SYLLABLE SSANGKIYEOK YO HIEUH
    ((0x84, 0xFB), (0xAFBA)),  # HANGUL SYLLABLE SSANGKIYEOK U SSANGKIYEOK
    ((0x84, 0xFC), (0xAFBB)),  # HANGUL SYLLABLE SSANGKIYEOK U KIYEOKSIOS
    ((0x84, 0xFD), (0xAFBD)),  # HANGUL SYLLABLE SSANGKIYEOK U NIEUNCIEUC
    ((0x84, 0xFE), (0xAFBE)),  # HANGUL SYLLABLE SSANGKIYEOK U NIEUNHIEUH
    ((0x85, 0x41), (0xAFBF)),  # HANGUL SYLLABLE SSANGKIYEOK U TIKEUT
    ((0x85, 0x42), (0xAFC1)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULKIYEOK
    ((0x85, 0x43), (0xAFC2)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULMIEUM
    ((0x85, 0x44), (0xAFC3)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULPIEUP
    ((0x85, 0x45), (0xAFC4)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULSIOS
    ((0x85, 0x46), (0xAFC5)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULTHIEUTH
    ((0x85, 0x47), (0xAFC6)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULPHIEUPH
    ((0x85, 0x48), (0xAFCA)),  # HANGUL SYLLABLE SSANGKIYEOK U PIEUPSIOS
    ((0x85, 0x49), (0xAFCC)),  # HANGUL SYLLABLE SSANGKIYEOK U SSANGSIOS
    ((0x85, 0x4A), (0xAFCF)),  # HANGUL SYLLABLE SSANGKIYEOK U CHIEUCH
    ((0x85, 0x4B), (0xAFD0)),  # HANGUL SYLLABLE SSANGKIYEOK U KHIEUKH
    ((0x85, 0x4C), (0xAFD1)),  # HANGUL SYLLABLE SSANGKIYEOK U THIEUTH
    ((0x85, 0x4D), (0xAFD2)),  # HANGUL SYLLABLE SSANGKIYEOK U PHIEUPH
    ((0x85, 0x4E), (0xAFD3)),  # HANGUL SYLLABLE SSANGKIYEOK U HIEUH
    ((0x85, 0x4F), (0xAFD5)),  # HANGUL SYLLABLE SSANGKIYEOK WEO KIYEOK
    ((0x85, 0x50), (0xAFD6)),  # HANGUL SYLLABLE SSANGKIYEOK WEO SSANGKIYEOK
    ((0x85, 0x51), (0xAFD7)),  # HANGUL SYLLABLE SSANGKIYEOK WEO KIYEOKSIOS
    ((0x85, 0x52), (0xAFD8)),  # HANGUL SYLLABLE SSANGKIYEOK WEO NIEUN
    ((0x85, 0x53), (0xAFD9)),  # HANGUL SYLLABLE SSANGKIYEOK WEO NIEUNCIEUC
    ((0x85, 0x54), (0xAFDA)),  # HANGUL SYLLABLE SSANGKIYEOK WEO NIEUNHIEUH
    ((0x85, 0x55), (0xAFDB)),  # HANGUL SYLLABLE SSANGKIYEOK WEO TIKEUT
    ((0x85, 0x56), (0xAFDD)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULKIYEOK
    ((0x85, 0x57), (0xAFDE)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULMIEUM
    ((0x85, 0x58), (0xAFDF)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULPIEUP
    ((0x85, 0x59), (0xAFE0)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULSIOS
    ((0x85, 0x5A), (0xAFE1)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULTHIEUTH
    ((0x85, 0x61), (0xAFE2)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULPHIEUPH
    ((0x85, 0x62), (0xAFE3)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEULHIEUH
    ((0x85, 0x63), (0xAFE4)),  # HANGUL SYLLABLE SSANGKIYEOK WEO MIEUM
    ((0x85, 0x64), (0xAFE5)),  # HANGUL SYLLABLE SSANGKIYEOK WEO PIEUP
    ((0x85, 0x65), (0xAFE6)),  # HANGUL SYLLABLE SSANGKIYEOK WEO PIEUPSIOS
    ((0x85, 0x66), (0xAFE7)),  # HANGUL SYLLABLE SSANGKIYEOK WEO SIOS
    ((0x85, 0x67), (0xAFEA)),  # HANGUL SYLLABLE SSANGKIYEOK WEO CIEUC
    ((0x85, 0x68), (0xAFEB)),  # HANGUL SYLLABLE SSANGKIYEOK WEO CHIEUCH
    ((0x85, 0x69), (0xAFEC)),  # HANGUL SYLLABLE SSANGKIYEOK WEO KHIEUKH
    ((0x85, 0x6A), (0xAFED)),  # HANGUL SYLLABLE SSANGKIYEOK WEO THIEUTH
    ((0x85, 0x6B), (0xAFEE)),  # HANGUL SYLLABLE SSANGKIYEOK WEO PHIEUPH
    ((0x85, 0x6C), (0xAFEF)),  # HANGUL SYLLABLE SSANGKIYEOK WEO HIEUH
    ((0x85, 0x6D), (0xAFF2)),  # HANGUL SYLLABLE SSANGKIYEOK WE SSANGKIYEOK
    ((0x85, 0x6E), (0xAFF3)),  # HANGUL SYLLABLE SSANGKIYEOK WE KIYEOKSIOS
    ((0x85, 0x6F), (0xAFF5)),  # HANGUL SYLLABLE SSANGKIYEOK WE NIEUNCIEUC
    ((0x85, 0x70), (0xAFF6)),  # HANGUL SYLLABLE SSANGKIYEOK WE NIEUNHIEUH
    ((0x85, 0x71), (0xAFF7)),  # HANGUL SYLLABLE SSANGKIYEOK WE TIKEUT
    ((0x85, 0x72), (0xAFF9)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULKIYEOK
    ((0x85, 0x73), (0xAFFA)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULMIEUM
    ((0x85, 0x74), (0xAFFB)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULPIEUP
    ((0x85, 0x75), (0xAFFC)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULSIOS
    ((0x85, 0x76), (0xAFFD)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULTHIEUTH
    ((0x85, 0x77), (0xAFFE)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULPHIEUPH
    ((0x85, 0x78), (0xAFFF)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEULHIEUH
    ((0x85, 0x79), (0xB002)),  # HANGUL SYLLABLE SSANGKIYEOK WE PIEUPSIOS
    ((0x85, 0x7A), (0xB003)),  # HANGUL SYLLABLE SSANGKIYEOK WE SIOS
    ((0x85, 0x81), (0xB005)),  # HANGUL SYLLABLE SSANGKIYEOK WE IEUNG
    ((0x85, 0x82), (0xB006)),  # HANGUL SYLLABLE SSANGKIYEOK WE CIEUC
    ((0x85, 0x83), (0xB007)),  # HANGUL SYLLABLE SSANGKIYEOK WE CHIEUCH
    ((0x85, 0x84), (0xB008)),  # HANGUL SYLLABLE SSANGKIYEOK WE KHIEUKH
    ((0x85, 0x85), (0xB009)),  # HANGUL SYLLABLE SSANGKIYEOK WE THIEUTH
    ((0x85, 0x86), (0xB00A)),  # HANGUL SYLLABLE SSANGKIYEOK WE PHIEUPH
    ((0x85, 0x87), (0xB00B)),  # HANGUL SYLLABLE SSANGKIYEOK WE HIEUH
    ((0x85, 0x88), (0xB00D)),  # HANGUL SYLLABLE SSANGKIYEOK WI KIYEOK
    ((0x85, 0x89), (0xB00E)),  # HANGUL SYLLABLE SSANGKIYEOK WI SSANGKIYEOK
    ((0x85, 0x8A), (0xB00F)),  # HANGUL SYLLABLE SSANGKIYEOK WI KIYEOKSIOS
    ((0x85, 0x8B), (0xB011)),  # HANGUL SYLLABLE SSANGKIYEOK WI NIEUNCIEUC
    ((0x85, 0x8C), (0xB012)),  # HANGUL SYLLABLE SSANGKIYEOK WI NIEUNHIEUH
    ((0x85, 0x8D), (0xB013)),  # HANGUL SYLLABLE SSANGKIYEOK WI TIKEUT
    ((0x85, 0x8E), (0xB015)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULKIYEOK
    ((0x85, 0x8F), (0xB016)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULMIEUM
    ((0x85, 0x90), (0xB017)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULPIEUP
    ((0x85, 0x91), (0xB018)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULSIOS
    ((0x85, 0x92), (0xB019)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULTHIEUTH
    ((0x85, 0x93), (0xB01A)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULPHIEUPH
    ((0x85, 0x94), (0xB01B)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEULHIEUH
    ((0x85, 0x95), (0xB01E)),  # HANGUL SYLLABLE SSANGKIYEOK WI PIEUPSIOS
    ((0x85, 0x96), (0xB01F)),  # HANGUL SYLLABLE SSANGKIYEOK WI SIOS
    ((0x85, 0x97), (0xB020)),  # HANGUL SYLLABLE SSANGKIYEOK WI SSANGSIOS
    ((0x85, 0x98), (0xB021)),  # HANGUL SYLLABLE SSANGKIYEOK WI IEUNG
    ((0x85, 0x99), (0xB022)),  # HANGUL SYLLABLE SSANGKIYEOK WI CIEUC
    ((0x85, 0x9A), (0xB023)),  # HANGUL SYLLABLE SSANGKIYEOK WI CHIEUCH
    ((0x85, 0x9B), (0xB024)),  # HANGUL SYLLABLE SSANGKIYEOK WI KHIEUKH
    ((0x85, 0x9C), (0xB025)),  # HANGUL SYLLABLE SSANGKIYEOK WI THIEUTH
    ((0x85, 0x9D), (0xB026)),  # HANGUL SYLLABLE SSANGKIYEOK WI PHIEUPH
    ((0x85, 0x9E), (0xB027)),  # HANGUL SYLLABLE SSANGKIYEOK WI HIEUH
    ((0x85, 0x9F), (0xB029)),  # HANGUL SYLLABLE SSANGKIYEOK YU KIYEOK
    ((0x85, 0xA0), (0xB02A)),  # HANGUL SYLLABLE SSANGKIYEOK YU SSANGKIYEOK
    ((0x85, 0xA1), (0xB02B)),  # HANGUL SYLLABLE SSANGKIYEOK YU KIYEOKSIOS
    ((0x85, 0xA2), (0xB02C)),  # HANGUL SYLLABLE SSANGKIYEOK YU NIEUN
    ((0x85, 0xA3), (0xB02D)),  # HANGUL SYLLABLE SSANGKIYEOK YU NIEUNCIEUC
    ((0x85, 0xA4), (0xB02E)),  # HANGUL SYLLABLE SSANGKIYEOK YU NIEUNHIEUH
    ((0x85, 0xA5), (0xB02F)),  # HANGUL SYLLABLE SSANGKIYEOK YU TIKEUT
    ((0x85, 0xA6), (0xB030)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEUL
    ((0x85, 0xA7), (0xB031)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULKIYEOK
    ((0x85, 0xA8), (0xB032)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULMIEUM
    ((0x85, 0xA9), (0xB033)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULPIEUP
    ((0x85, 0xAA), (0xB034)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULSIOS
    ((0x85, 0xAB), (0xB035)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULTHIEUTH
    ((0x85, 0xAC), (0xB036)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULPHIEUPH
    ((0x85, 0xAD), (0xB037)),  # HANGUL SYLLABLE SSANGKIYEOK YU RIEULHIEUH
    ((0x85, 0xAE), (0xB038)),  # HANGUL SYLLABLE SSANGKIYEOK YU MIEUM
    ((0x85, 0xAF), (0xB039)),  # HANGUL SYLLABLE SSANGKIYEOK YU PIEUP
    ((0x85, 0xB0), (0xB03A)),  # HANGUL SYLLABLE SSANGKIYEOK YU PIEUPSIOS
    ((0x85, 0xB1), (0xB03B)),  # HANGUL SYLLABLE SSANGKIYEOK YU SIOS
    ((0x85, 0xB2), (0xB03C)),  # HANGUL SYLLABLE SSANGKIYEOK YU SSANGSIOS
    ((0x85, 0xB3), (0xB03D)),  # HANGUL SYLLABLE SSANGKIYEOK YU IEUNG
    ((0x85, 0xB4), (0xB03E)),  # HANGUL SYLLABLE SSANGKIYEOK YU CIEUC
    ((0x85, 0xB5), (0xB03F)),  # HANGUL SYLLABLE SSANGKIYEOK YU CHIEUCH
    ((0x85, 0xB6), (0xB040)),  # HANGUL SYLLABLE SSANGKIYEOK YU KHIEUKH
    ((0x85, 0xB7), (0xB041)),  # HANGUL SYLLABLE SSANGKIYEOK YU THIEUTH
    ((0x85, 0xB8), (0xB042)),  # HANGUL SYLLABLE SSANGKIYEOK YU PHIEUPH
    ((0x85, 0xB9), (0xB043)),  # HANGUL SYLLABLE SSANGKIYEOK YU HIEUH
    ((0x85, 0xBA), (0xB046)),  # HANGUL SYLLABLE SSANGKIYEOK EU SSANGKIYEOK
    ((0x85, 0xBB), (0xB047)),  # HANGUL SYLLABLE SSANGKIYEOK EU KIYEOKSIOS
    ((0x85, 0xBC), (0xB049)),  # HANGUL SYLLABLE SSANGKIYEOK EU NIEUNCIEUC
    ((0x85, 0xBD), (0xB04B)),  # HANGUL SYLLABLE SSANGKIYEOK EU TIKEUT
    ((0x85, 0xBE), (0xB04D)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULKIYEOK
    ((0x85, 0xBF), (0xB04F)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULPIEUP
    ((0x85, 0xC0), (0xB050)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULSIOS
    ((0x85, 0xC1), (0xB051)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULTHIEUTH
    ((0x85, 0xC2), (0xB052)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULPHIEUPH
    ((0x85, 0xC3), (0xB056)),  # HANGUL SYLLABLE SSANGKIYEOK EU PIEUPSIOS
    ((0x85, 0xC4), (0xB058)),  # HANGUL SYLLABLE SSANGKIYEOK EU SSANGSIOS
    ((0x85, 0xC5), (0xB05A)),  # HANGUL SYLLABLE SSANGKIYEOK EU CIEUC
    ((0x85, 0xC6), (0xB05B)),  # HANGUL SYLLABLE SSANGKIYEOK EU CHIEUCH
    ((0x85, 0xC7), (0xB05C)),  # HANGUL SYLLABLE SSANGKIYEOK EU KHIEUKH
    ((0x85, 0xC8), (0xB05E)),  # HANGUL SYLLABLE SSANGKIYEOK EU PHIEUPH
    ((0x85, 0xC9), (0xB05F)),  # HANGUL SYLLABLE SSANGKIYEOK EU HIEUH
    ((0x85, 0xCA), (0xB060)),  # HANGUL SYLLABLE SSANGKIYEOK YI
    ((0x85, 0xCB), (0xB061)),  # HANGUL SYLLABLE SSANGKIYEOK YI KIYEOK
    ((0x85, 0xCC), (0xB062)),  # HANGUL SYLLABLE SSANGKIYEOK YI SSANGKIYEOK
    ((0x85, 0xCD), (0xB063)),  # HANGUL SYLLABLE SSANGKIYEOK YI KIYEOKSIOS
    ((0x85, 0xCE), (0xB064)),  # HANGUL SYLLABLE SSANGKIYEOK YI NIEUN
    ((0x85, 0xCF), (0xB065)),  # HANGUL SYLLABLE SSANGKIYEOK YI NIEUNCIEUC
    ((0x85, 0xD0), (0xB066)),  # HANGUL SYLLABLE SSANGKIYEOK YI NIEUNHIEUH
    ((0x85, 0xD1), (0xB067)),  # HANGUL SYLLABLE SSANGKIYEOK YI TIKEUT
    ((0x85, 0xD2), (0xB068)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEUL
    ((0x85, 0xD3), (0xB069)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULKIYEOK
    ((0x85, 0xD4), (0xB06A)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULMIEUM
    ((0x85, 0xD5), (0xB06B)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULPIEUP
    ((0x85, 0xD6), (0xB06C)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULSIOS
    ((0x85, 0xD7), (0xB06D)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULTHIEUTH
    ((0x85, 0xD8), (0xB06E)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULPHIEUPH
    ((0x85, 0xD9), (0xB06F)),  # HANGUL SYLLABLE SSANGKIYEOK YI RIEULHIEUH
    ((0x85, 0xDA), (0xB070)),  # HANGUL SYLLABLE SSANGKIYEOK YI MIEUM
    ((0x85, 0xDB), (0xB071)),  # HANGUL SYLLABLE SSANGKIYEOK YI PIEUP
    ((0x85, 0xDC), (0xB072)),  # HANGUL SYLLABLE SSANGKIYEOK YI PIEUPSIOS
    ((0x85, 0xDD), (0xB073)),  # HANGUL SYLLABLE SSANGKIYEOK YI SIOS
    ((0x85, 0xDE), (0xB074)),  # HANGUL SYLLABLE SSANGKIYEOK YI SSANGSIOS
    ((0x85, 0xDF), (0xB075)),  # HANGUL SYLLABLE SSANGKIYEOK YI IEUNG
    ((0x85, 0xE0), (0xB076)),  # HANGUL SYLLABLE SSANGKIYEOK YI CIEUC
    ((0x85, 0xE1), (0xB077)),  # HANGUL SYLLABLE SSANGKIYEOK YI CHIEUCH
    ((0x85, 0xE2), (0xB078)),  # HANGUL SYLLABLE SSANGKIYEOK YI KHIEUKH
    ((0x85, 0xE3), (0xB079)),  # HANGUL SYLLABLE SSANGKIYEOK YI THIEUTH
    ((0x85, 0xE4), (0xB07A)),  # HANGUL SYLLABLE SSANGKIYEOK YI PHIEUPH
    ((0x85, 0xE5), (0xB07B)),  # HANGUL SYLLABLE SSANGKIYEOK YI HIEUH
    ((0x85, 0xE6), (0xB07E)),  # HANGUL SYLLABLE SSANGKIYEOK I SSANGKIYEOK
    ((0x85, 0xE7), (0xB07F)),  # HANGUL SYLLABLE SSANGKIYEOK I KIYEOKSIOS
    ((0x85, 0xE8), (0xB081)),  # HANGUL SYLLABLE SSANGKIYEOK I NIEUNCIEUC
    ((0x85, 0xE9), (0xB082)),  # HANGUL SYLLABLE SSANGKIYEOK I NIEUNHIEUH
    ((0x85, 0xEA), (0xB083)),  # HANGUL SYLLABLE SSANGKIYEOK I TIKEUT
    ((0x85, 0xEB), (0xB085)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULKIYEOK
    ((0x85, 0xEC), (0xB086)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULMIEUM
    ((0x85, 0xED), (0xB087)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULPIEUP
    ((0x85, 0xEE), (0xB088)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULSIOS
    ((0x85, 0xEF), (0xB089)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULTHIEUTH
    ((0x85, 0xF0), (0xB08A)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULPHIEUPH
    ((0x85, 0xF1), (0xB08B)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEULHIEUH
    ((0x85, 0xF2), (0xB08E)),  # HANGUL SYLLABLE SSANGKIYEOK I PIEUPSIOS
    ((0x85, 0xF3), (0xB090)),  # HANGUL SYLLABLE SSANGKIYEOK I SSANGSIOS
    ((0x85, 0xF4), (0xB092)),  # HANGUL SYLLABLE SSANGKIYEOK I CIEUC
    ((0x85, 0xF5), (0xB093)),  # HANGUL SYLLABLE SSANGKIYEOK I CHIEUCH
    ((0x85, 0xF6), (0xB094)),  # HANGUL SYLLABLE SSANGKIYEOK I KHIEUKH
    ((0x85, 0xF7), (0xB095)),  # HANGUL SYLLABLE SSANGKIYEOK I THIEUTH
    ((0x85, 0xF8), (0xB096)),  # HANGUL SYLLABLE SSANGKIYEOK I PHIEUPH
    ((0x85, 0xF9), (0xB097)),  # HANGUL SYLLABLE SSANGKIYEOK I HIEUH
    ((0x85, 0xFA), (0xB09B)),  # HANGUL SYLLABLE NIEUN A KIYEOKSIOS
    ((0x85, 0xFB), (0xB09D)),  # HANGUL SYLLABLE NIEUN A NIEUNCIEUC
    ((0x85, 0xFC), (0xB09E)),  # HANGUL SYLLABLE NIEUN A NIEUNHIEUH
    ((0x85, 0xFD), (0xB0A3)),  # HANGUL SYLLABLE NIEUN A RIEULPIEUP
    ((0x85, 0xFE), (0xB0A4)),  # HANGUL SYLLABLE NIEUN A RIEULSIOS
    ((0x86, 0x41), (0xB0A5)),  # HANGUL SYLLABLE NIEUN A RIEULTHIEUTH
    ((0x86, 0x42), (0xB0A6)),  # HANGUL SYLLABLE NIEUN A RIEULPHIEUPH
    ((0x86, 0x43), (0xB0A7)),  # HANGUL SYLLABLE NIEUN A RIEULHIEUH
    ((0x86, 0x44), (0xB0AA)),  # HANGUL SYLLABLE NIEUN A PIEUPSIOS
    ((0x86, 0x45), (0xB0B0)),  # HANGUL SYLLABLE NIEUN A KHIEUKH
    ((0x86, 0x46), (0xB0B2)),  # HANGUL SYLLABLE NIEUN A PHIEUPH
    ((0x86, 0x47), (0xB0B6)),  # HANGUL SYLLABLE NIEUN AE SSANGKIYEOK
    ((0x86, 0x48), (0xB0B7)),  # HANGUL SYLLABLE NIEUN AE KIYEOKSIOS
    ((0x86, 0x49), (0xB0B9)),  # HANGUL SYLLABLE NIEUN AE NIEUNCIEUC
    ((0x86, 0x4A), (0xB0BA)),  # HANGUL SYLLABLE NIEUN AE NIEUNHIEUH
    ((0x86, 0x4B), (0xB0BB)),  # HANGUL SYLLABLE NIEUN AE TIKEUT
    ((0x86, 0x4C), (0xB0BD)),  # HANGUL SYLLABLE NIEUN AE RIEULKIYEOK
    ((0x86, 0x4D), (0xB0BE)),  # HANGUL SYLLABLE NIEUN AE RIEULMIEUM
    ((0x86, 0x4E), (0xB0BF)),  # HANGUL SYLLABLE NIEUN AE RIEULPIEUP
    ((0x86, 0x4F), (0xB0C0)),  # HANGUL SYLLABLE NIEUN AE RIEULSIOS
    ((0x86, 0x50), (0xB0C1)),  # HANGUL SYLLABLE NIEUN AE RIEULTHIEUTH
    ((0x86, 0x51), (0xB0C2)),  # HANGUL SYLLABLE NIEUN AE RIEULPHIEUPH
    ((0x86, 0x52), (0xB0C3)),  # HANGUL SYLLABLE NIEUN AE RIEULHIEUH
    ((0x86, 0x53), (0xB0C6)),  # HANGUL SYLLABLE NIEUN AE PIEUPSIOS
    ((0x86, 0x54), (0xB0CA)),  # HANGUL SYLLABLE NIEUN AE CIEUC
    ((0x86, 0x55), (0xB0CB)),  # HANGUL SYLLABLE NIEUN AE CHIEUCH
    ((0x86, 0x56), (0xB0CC)),  # HANGUL SYLLABLE NIEUN AE KHIEUKH
    ((0x86, 0x57), (0xB0CD)),  # HANGUL SYLLABLE NIEUN AE THIEUTH
    ((0x86, 0x58), (0xB0CE)),  # HANGUL SYLLABLE NIEUN AE PHIEUPH
    ((0x86, 0x59), (0xB0CF)),  # HANGUL SYLLABLE NIEUN AE HIEUH
    ((0x86, 0x5A), (0xB0D2)),  # HANGUL SYLLABLE NIEUN YA SSANGKIYEOK
    ((0x86, 0x61), (0xB0D3)),  # HANGUL SYLLABLE NIEUN YA KIYEOKSIOS
    ((0x86, 0x62), (0xB0D5)),  # HANGUL SYLLABLE NIEUN YA NIEUNCIEUC
    ((0x86, 0x63), (0xB0D6)),  # HANGUL SYLLABLE NIEUN YA NIEUNHIEUH
    ((0x86, 0x64), (0xB0D7)),  # HANGUL SYLLABLE NIEUN YA TIKEUT
    ((0x86, 0x65), (0xB0D9)),  # HANGUL SYLLABLE NIEUN YA RIEULKIYEOK
    ((0x86, 0x66), (0xB0DA)),  # HANGUL SYLLABLE NIEUN YA RIEULMIEUM
    ((0x86, 0x67), (0xB0DB)),  # HANGUL SYLLABLE NIEUN YA RIEULPIEUP
    ((0x86, 0x68), (0xB0DC)),  # HANGUL SYLLABLE NIEUN YA RIEULSIOS
    ((0x86, 0x69), (0xB0DD)),  # HANGUL SYLLABLE NIEUN YA RIEULTHIEUTH
    ((0x86, 0x6A), (0xB0DE)),  # HANGUL SYLLABLE NIEUN YA RIEULPHIEUPH
    ((0x86, 0x6B), (0xB0DF)),  # HANGUL SYLLABLE NIEUN YA RIEULHIEUH
    ((0x86, 0x6C), (0xB0E1)),  # HANGUL SYLLABLE NIEUN YA PIEUP
    ((0x86, 0x6D), (0xB0E2)),  # HANGUL SYLLABLE NIEUN YA PIEUPSIOS
    ((0x86, 0x6E), (0xB0E3)),  # HANGUL SYLLABLE NIEUN YA SIOS
    ((0x86, 0x6F), (0xB0E4)),  # HANGUL SYLLABLE NIEUN YA SSANGSIOS
    ((0x86, 0x70), (0xB0E6)),  # HANGUL SYLLABLE NIEUN YA CIEUC
    ((0x86, 0x71), (0xB0E7)),  # HANGUL SYLLABLE NIEUN YA CHIEUCH
    ((0x86, 0x72), (0xB0E8)),  # HANGUL SYLLABLE NIEUN YA KHIEUKH
    ((0x86, 0x73), (0xB0E9)),  # HANGUL SYLLABLE NIEUN YA THIEUTH
    ((0x86, 0x74), (0xB0EA)),  # HANGUL SYLLABLE NIEUN YA PHIEUPH
    ((0x86, 0x75), (0xB0EB)),  # HANGUL SYLLABLE NIEUN YA HIEUH
    ((0x86, 0x76), (0xB0EC)),  # HANGUL SYLLABLE NIEUN YAE
    ((0x86, 0x77), (0xB0ED)),  # HANGUL SYLLABLE NIEUN YAE KIYEOK
    ((0x86, 0x78), (0xB0EE)),  # HANGUL SYLLABLE NIEUN YAE SSANGKIYEOK
    ((0x86, 0x79), (0xB0EF)),  # HANGUL SYLLABLE NIEUN YAE KIYEOKSIOS
    ((0x86, 0x7A), (0xB0F0)),  # HANGUL SYLLABLE NIEUN YAE NIEUN
    ((0x86, 0x81), (0xB0F1)),  # HANGUL SYLLABLE NIEUN YAE NIEUNCIEUC
    ((0x86, 0x82), (0xB0F2)),  # HANGUL SYLLABLE NIEUN YAE NIEUNHIEUH
    ((0x86, 0x83), (0xB0F3)),  # HANGUL SYLLABLE NIEUN YAE TIKEUT
    ((0x86, 0x84), (0xB0F4)),  # HANGUL SYLLABLE NIEUN YAE RIEUL
    ((0x86, 0x85), (0xB0F5)),  # HANGUL SYLLABLE NIEUN YAE RIEULKIYEOK
    ((0x86, 0x86), (0xB0F6)),  # HANGUL SYLLABLE NIEUN YAE RIEULMIEUM
    ((0x86, 0x87), (0xB0F7)),  # HANGUL SYLLABLE NIEUN YAE RIEULPIEUP
    ((0x86, 0x88), (0xB0F8)),  # HANGUL SYLLABLE NIEUN YAE RIEULSIOS
    ((0x86, 0x89), (0xB0F9)),  # HANGUL SYLLABLE NIEUN YAE RIEULTHIEUTH
    ((0x86, 0x8A), (0xB0FA)),  # HANGUL SYLLABLE NIEUN YAE RIEULPHIEUPH
    ((0x86, 0x8B), (0xB0FB)),  # HANGUL SYLLABLE NIEUN YAE RIEULHIEUH
    ((0x86, 0x8C), (0xB0FC)),  # HANGUL SYLLABLE NIEUN YAE MIEUM
    ((0x86, 0x8D), (0xB0FD)),  # HANGUL SYLLABLE NIEUN YAE PIEUP
    ((0x86, 0x8E), (0xB0FE)),  # HANGUL SYLLABLE NIEUN YAE PIEUPSIOS
    ((0x86, 0x8F), (0xB0FF)),  # HANGUL SYLLABLE NIEUN YAE SIOS
    ((0x86, 0x90), (0xB100)),  # HANGUL SYLLABLE NIEUN YAE SSANGSIOS
    ((0x86, 0x91), (0xB101)),  # HANGUL SYLLABLE NIEUN YAE IEUNG
    ((0x86, 0x92), (0xB102)),  # HANGUL SYLLABLE NIEUN YAE CIEUC
    ((0x86, 0x93), (0xB103)),  # HANGUL SYLLABLE NIEUN YAE CHIEUCH
    ((0x86, 0x94), (0xB104)),  # HANGUL SYLLABLE NIEUN YAE KHIEUKH
    ((0x86, 0x95), (0xB105)),  # HANGUL SYLLABLE NIEUN YAE THIEUTH
    ((0x86, 0x96), (0xB106)),  # HANGUL SYLLABLE NIEUN YAE PHIEUPH
    ((0x86, 0x97), (0xB107)),  # HANGUL SYLLABLE NIEUN YAE HIEUH
    ((0x86, 0x98), (0xB10A)),  # HANGUL SYLLABLE NIEUN EO SSANGKIYEOK
    ((0x86, 0x99), (0xB10D)),  # HANGUL SYLLABLE NIEUN EO NIEUNCIEUC
    ((0x86, 0x9A), (0xB10E)),  # HANGUL SYLLABLE NIEUN EO NIEUNHIEUH
    ((0x86, 0x9B), (0xB10F)),  # HANGUL SYLLABLE NIEUN EO TIKEUT
    ((0x86, 0x9C), (0xB111)),  # HANGUL SYLLABLE NIEUN EO RIEULKIYEOK
    ((0x86, 0x9D), (0xB114)),  # HANGUL SYLLABLE NIEUN EO RIEULSIOS
    ((0x86, 0x9E), (0xB115)),  # HANGUL SYLLABLE NIEUN EO RIEULTHIEUTH
    ((0x86, 0x9F), (0xB116)),  # HANGUL SYLLABLE NIEUN EO RIEULPHIEUPH
    ((0x86, 0xA0), (0xB117)),  # HANGUL SYLLABLE NIEUN EO RIEULHIEUH
    ((0x86, 0xA1), (0xB11A)),  # HANGUL SYLLABLE NIEUN EO PIEUPSIOS
    ((0x86, 0xA2), (0xB11E)),  # HANGUL SYLLABLE NIEUN EO CIEUC
    ((0x86, 0xA3), (0xB11F)),  # HANGUL SYLLABLE NIEUN EO CHIEUCH
    ((0x86, 0xA4), (0xB120)),  # HANGUL SYLLABLE NIEUN EO KHIEUKH
    ((0x86, 0xA5), (0xB121)),  # HANGUL SYLLABLE NIEUN EO THIEUTH
    ((0x86, 0xA6), (0xB122)),  # HANGUL SYLLABLE NIEUN EO PHIEUPH
    ((0x86, 0xA7), (0xB126)),  # HANGUL SYLLABLE NIEUN E SSANGKIYEOK
    ((0x86, 0xA8), (0xB127)),  # HANGUL SYLLABLE NIEUN E KIYEOKSIOS
    ((0x86, 0xA9), (0xB129)),  # HANGUL SYLLABLE NIEUN E NIEUNCIEUC
    ((0x86, 0xAA), (0xB12A)),  # HANGUL SYLLABLE NIEUN E NIEUNHIEUH
    ((0x86, 0xAB), (0xB12B)),  # HANGUL SYLLABLE NIEUN E TIKEUT
    ((0x86, 0xAC), (0xB12D)),  # HANGUL SYLLABLE NIEUN E RIEULKIYEOK
    ((0x86, 0xAD), (0xB12E)),  # HANGUL SYLLABLE NIEUN E RIEULMIEUM
    ((0x86, 0xAE), (0xB12F)),  # HANGUL SYLLABLE NIEUN E RIEULPIEUP
    ((0x86, 0xAF), (0xB130)),  # HANGUL SYLLABLE NIEUN E RIEULSIOS
    ((0x86, 0xB0), (0xB131)),  # HANGUL SYLLABLE NIEUN E RIEULTHIEUTH
    ((0x86, 0xB1), (0xB132)),  # HANGUL SYLLABLE NIEUN E RIEULPHIEUPH
    ((0x86, 0xB2), (0xB133)),  # HANGUL SYLLABLE NIEUN E RIEULHIEUH
    ((0x86, 0xB3), (0xB136)),  # HANGUL SYLLABLE NIEUN E PIEUPSIOS
    ((0x86, 0xB4), (0xB13A)),  # HANGUL SYLLABLE NIEUN E CIEUC
    ((0x86, 0xB5), (0xB13B)),  # HANGUL SYLLABLE NIEUN E CHIEUCH
    ((0x86, 0xB6), (0xB13C)),  # HANGUL SYLLABLE NIEUN E KHIEUKH
    ((0x86, 0xB7), (0xB13D)),  # HANGUL SYLLABLE NIEUN E THIEUTH
    ((0x86, 0xB8), (0xB13E)),  # HANGUL SYLLABLE NIEUN E PHIEUPH
    ((0x86, 0xB9), (0xB13F)),  # HANGUL SYLLABLE NIEUN E HIEUH
    ((0x86, 0xBA), (0xB142)),  # HANGUL SYLLABLE NIEUN YEO SSANGKIYEOK
    ((0x86, 0xBB), (0xB143)),  # HANGUL SYLLABLE NIEUN YEO KIYEOKSIOS
    ((0x86, 0xBC), (0xB145)),  # HANGUL SYLLABLE NIEUN YEO NIEUNCIEUC
    ((0x86, 0xBD), (0xB146)),  # HANGUL SYLLABLE NIEUN YEO NIEUNHIEUH
    ((0x86, 0xBE), (0xB147)),  # HANGUL SYLLABLE NIEUN YEO TIKEUT
    ((0x86, 0xBF), (0xB149)),  # HANGUL SYLLABLE NIEUN YEO RIEULKIYEOK
    ((0x86, 0xC0), (0xB14A)),  # HANGUL SYLLABLE NIEUN YEO RIEULMIEUM
    ((0x86, 0xC1), (0xB14B)),  # HANGUL SYLLABLE NIEUN YEO RIEULPIEUP
    ((0x86, 0xC2), (0xB14C)),  # HANGUL SYLLABLE NIEUN YEO RIEULSIOS
    ((0x86, 0xC3), (0xB14D)),  # HANGUL SYLLABLE NIEUN YEO RIEULTHIEUTH
    ((0x86, 0xC4), (0xB14E)),  # HANGUL SYLLABLE NIEUN YEO RIEULPHIEUPH
    ((0x86, 0xC5), (0xB14F)),  # HANGUL SYLLABLE NIEUN YEO RIEULHIEUH
    ((0x86, 0xC6), (0xB152)),  # HANGUL SYLLABLE NIEUN YEO PIEUPSIOS
    ((0x86, 0xC7), (0xB153)),  # HANGUL SYLLABLE NIEUN YEO SIOS
    ((0x86, 0xC8), (0xB156)),  # HANGUL SYLLABLE NIEUN YEO CIEUC
    ((0x86, 0xC9), (0xB157)),  # HANGUL SYLLABLE NIEUN YEO CHIEUCH
    ((0x86, 0xCA), (0xB159)),  # HANGUL SYLLABLE NIEUN YEO THIEUTH
    ((0x86, 0xCB), (0xB15A)),  # HANGUL SYLLABLE NIEUN YEO PHIEUPH
    ((0x86, 0xCC), (0xB15B)),  # HANGUL SYLLABLE NIEUN YEO HIEUH
    ((0x86, 0xCD), (0xB15D)),  # HANGUL SYLLABLE NIEUN YE KIYEOK
    ((0x86, 0xCE), (0xB15E)),  # HANGUL SYLLABLE NIEUN YE SSANGKIYEOK
    ((0x86, 0xCF), (0xB15F)),  # HANGUL SYLLABLE NIEUN YE KIYEOKSIOS
    ((0x86, 0xD0), (0xB161)),  # HANGUL SYLLABLE NIEUN YE NIEUNCIEUC
    ((0x86, 0xD1), (0xB162)),  # HANGUL SYLLABLE NIEUN YE NIEUNHIEUH
    ((0x86, 0xD2), (0xB163)),  # HANGUL SYLLABLE NIEUN YE TIKEUT
    ((0x86, 0xD3), (0xB164)),  # HANGUL SYLLABLE NIEUN YE RIEUL
    ((0x86, 0xD4), (0xB165)),  # HANGUL SYLLABLE NIEUN YE RIEULKIYEOK
    ((0x86, 0xD5), (0xB166)),  # HANGUL SYLLABLE NIEUN YE RIEULMIEUM
    ((0x86, 0xD6), (0xB167)),  # HANGUL SYLLABLE NIEUN YE RIEULPIEUP
    ((0x86, 0xD7), (0xB168)),  # HANGUL SYLLABLE NIEUN YE RIEULSIOS
    ((0x86, 0xD8), (0xB169)),  # HANGUL SYLLABLE NIEUN YE RIEULTHIEUTH
    ((0x86, 0xD9), (0xB16A)),  # HANGUL SYLLABLE NIEUN YE RIEULPHIEUPH
    ((0x86, 0xDA), (0xB16B)),  # HANGUL SYLLABLE NIEUN YE RIEULHIEUH
    ((0x86, 0xDB), (0xB16C)),  # HANGUL SYLLABLE NIEUN YE MIEUM
    ((0x86, 0xDC), (0xB16D)),  # HANGUL SYLLABLE NIEUN YE PIEUP
    ((0x86, 0xDD), (0xB16E)),  # HANGUL SYLLABLE NIEUN YE PIEUPSIOS
    ((0x86, 0xDE), (0xB16F)),  # HANGUL SYLLABLE NIEUN YE SIOS
    ((0x86, 0xDF), (0xB170)),  # HANGUL SYLLABLE NIEUN YE SSANGSIOS
    ((0x86, 0xE0), (0xB171)),  # HANGUL SYLLABLE NIEUN YE IEUNG
    ((0x86, 0xE1), (0xB172)),  # HANGUL SYLLABLE NIEUN YE CIEUC
    ((0x86, 0xE2), (0xB173)),  # HANGUL SYLLABLE NIEUN YE CHIEUCH
    ((0x86, 0xE3), (0xB174)),  # HANGUL SYLLABLE NIEUN YE KHIEUKH
    ((0x86, 0xE4), (0xB175)),  # HANGUL SYLLABLE NIEUN YE THIEUTH
    ((0x86, 0xE5), (0xB176)),  # HANGUL SYLLABLE NIEUN YE PHIEUPH
    ((0x86, 0xE6), (0xB177)),  # HANGUL SYLLABLE NIEUN YE HIEUH
    ((0x86, 0xE7), (0xB17A)),  # HANGUL SYLLABLE NIEUN O SSANGKIYEOK
    ((0x86, 0xE8), (0xB17B)),  # HANGUL SYLLABLE NIEUN O KIYEOKSIOS
    ((0x86, 0xE9), (0xB17D)),  # HANGUL SYLLABLE NIEUN O NIEUNCIEUC
    ((0x86, 0xEA), (0xB17E)),  # HANGUL SYLLABLE NIEUN O NIEUNHIEUH
    ((0x86, 0xEB), (0xB17F)),  # HANGUL SYLLABLE NIEUN O TIKEUT
    ((0x86, 0xEC), (0xB181)),  # HANGUL SYLLABLE NIEUN O RIEULKIYEOK
    ((0x86, 0xED), (0xB183)),  # HANGUL SYLLABLE NIEUN O RIEULPIEUP
    ((0x86, 0xEE), (0xB184)),  # HANGUL SYLLABLE NIEUN O RIEULSIOS
    ((0x86, 0xEF), (0xB185)),  # HANGUL SYLLABLE NIEUN O RIEULTHIEUTH
    ((0x86, 0xF0), (0xB186)),  # HANGUL SYLLABLE NIEUN O RIEULPHIEUPH
    ((0x86, 0xF1), (0xB187)),  # HANGUL SYLLABLE NIEUN O RIEULHIEUH
    ((0x86, 0xF2), (0xB18A)),  # HANGUL SYLLABLE NIEUN O PIEUPSIOS
    ((0x86, 0xF3), (0xB18C)),  # HANGUL SYLLABLE NIEUN O SSANGSIOS
    ((0x86, 0xF4), (0xB18E)),  # HANGUL SYLLABLE NIEUN O CIEUC
    ((0x86, 0xF5), (0xB18F)),  # HANGUL SYLLABLE NIEUN O CHIEUCH
    ((0x86, 0xF6), (0xB190)),  # HANGUL SYLLABLE NIEUN O KHIEUKH
    ((0x86, 0xF7), (0xB191)),  # HANGUL SYLLABLE NIEUN O THIEUTH
    ((0x86, 0xF8), (0xB195)),  # HANGUL SYLLABLE NIEUN WA KIYEOK
    ((0x86, 0xF9), (0xB196)),  # HANGUL SYLLABLE NIEUN WA SSANGKIYEOK
    ((0x86, 0xFA), (0xB197)),  # HANGUL SYLLABLE NIEUN WA KIYEOKSIOS
    ((0x86, 0xFB), (0xB199)),  # HANGUL SYLLABLE NIEUN WA NIEUNCIEUC
    ((0x86, 0xFC), (0xB19A)),  # HANGUL SYLLABLE NIEUN WA NIEUNHIEUH
    ((0x86, 0xFD), (0xB19B)),  # HANGUL SYLLABLE NIEUN WA TIKEUT
    ((0x86, 0xFE), (0xB19D)),  # HANGUL SYLLABLE NIEUN WA RIEULKIYEOK
    ((0x87, 0x41), (0xB19E)),  # HANGUL SYLLABLE NIEUN WA RIEULMIEUM
    ((0x87, 0x42), (0xB19F)),  # HANGUL SYLLABLE NIEUN WA RIEULPIEUP
    ((0x87, 0x43), (0xB1A0)),  # HANGUL SYLLABLE NIEUN WA RIEULSIOS
    ((0x87, 0x44), (0xB1A1)),  # HANGUL SYLLABLE NIEUN WA RIEULTHIEUTH
    ((0x87, 0x45), (0xB1A2)),  # HANGUL SYLLABLE NIEUN WA RIEULPHIEUPH
    ((0x87, 0x46), (0xB1A3)),  # HANGUL SYLLABLE NIEUN WA RIEULHIEUH
    ((0x87, 0x47), (0xB1A4)),  # HANGUL SYLLABLE NIEUN WA MIEUM
    ((0x87, 0x48), (0xB1A5)),  # HANGUL SYLLABLE NIEUN WA PIEUP
    ((0x87, 0x49), (0xB1A6)),  # HANGUL SYLLABLE NIEUN WA PIEUPSIOS
    ((0x87, 0x4A), (0xB1A7)),  # HANGUL SYLLABLE NIEUN WA SIOS
    ((0x87, 0x4B), (0xB1A9)),  # HANGUL SYLLABLE NIEUN WA IEUNG
    ((0x87, 0x4C), (0xB1AA)),  # HANGUL SYLLABLE NIEUN WA CIEUC
    ((0x87, 0x4D), (0xB1AB)),  # HANGUL SYLLABLE NIEUN WA CHIEUCH
    ((0x87, 0x4E), (0xB1AC)),  # HANGUL SYLLABLE NIEUN WA KHIEUKH
    ((0x87, 0x4F), (0xB1AD)),  # HANGUL SYLLABLE NIEUN WA THIEUTH
    ((0x87, 0x50), (0xB1AE)),  # HANGUL SYLLABLE NIEUN WA PHIEUPH
    ((0x87, 0x51), (0xB1AF)),  # HANGUL SYLLABLE NIEUN WA HIEUH
    ((0x87, 0x52), (0xB1B0)),  # HANGUL SYLLABLE NIEUN WAE
    ((0x87, 0x53), (0xB1B1)),  # HANGUL SYLLABLE NIEUN WAE KIYEOK
    ((0x87, 0x54), (0xB1B2)),  # HANGUL SYLLABLE NIEUN WAE SSANGKIYEOK
    ((0x87, 0x55), (0xB1B3)),  # HANGUL SYLLABLE NIEUN WAE KIYEOKSIOS
    ((0x87, 0x56), (0xB1B4)),  # HANGUL SYLLABLE NIEUN WAE NIEUN
    ((0x87, 0x57), (0xB1B5)),  # HANGUL SYLLABLE NIEUN WAE NIEUNCIEUC
    ((0x87, 0x58), (0xB1B6)),  # HANGUL SYLLABLE NIEUN WAE NIEUNHIEUH
    ((0x87, 0x59), (0xB1B7)),  # HANGUL SYLLABLE NIEUN WAE TIKEUT
    ((0x87, 0x5A), (0xB1B8)),  # HANGUL SYLLABLE NIEUN WAE RIEUL
    ((0x87, 0x61), (0xB1B9)),  # HANGUL SYLLABLE NIEUN WAE RIEULKIYEOK
    ((0x87, 0x62), (0xB1BA)),  # HANGUL SYLLABLE NIEUN WAE RIEULMIEUM
    ((0x87, 0x63), (0xB1BB)),  # HANGUL SYLLABLE NIEUN WAE RIEULPIEUP
    ((0x87, 0x64), (0xB1BC)),  # HANGUL SYLLABLE NIEUN WAE RIEULSIOS
    ((0x87, 0x65), (0xB1BD)),  # HANGUL SYLLABLE NIEUN WAE RIEULTHIEUTH
    ((0x87, 0x66), (0xB1BE)),  # HANGUL SYLLABLE NIEUN WAE RIEULPHIEUPH
    ((0x87, 0x67), (0xB1BF)),  # HANGUL SYLLABLE NIEUN WAE RIEULHIEUH
    ((0x87, 0x68), (0xB1C0)),  # HANGUL SYLLABLE NIEUN WAE MIEUM
    ((0x87, 0x69), (0xB1C1)),  # HANGUL SYLLABLE NIEUN WAE PIEUP
    ((0x87, 0x6A), (0xB1C2)),  # HANGUL SYLLABLE NIEUN WAE PIEUPSIOS
    ((0x87, 0x6B), (0xB1C3)),  # HANGUL SYLLABLE NIEUN WAE SIOS
    ((0x87, 0x6C), (0xB1C4)),  # HANGUL SYLLABLE NIEUN WAE SSANGSIOS
    ((0x87, 0x6D), (0xB1C5)),  # HANGUL SYLLABLE NIEUN WAE IEUNG
    ((0x87, 0x6E), (0xB1C6)),  # HANGUL SYLLABLE NIEUN WAE CIEUC
    ((0x87, 0x6F), (0xB1C7)),  # HANGUL SYLLABLE NIEUN WAE CHIEUCH
    ((0x87, 0x70), (0xB1C8)),  # HANGUL SYLLABLE NIEUN WAE KHIEUKH
    ((0x87, 0x71), (0xB1C9)),  # HANGUL SYLLABLE NIEUN WAE THIEUTH
    ((0x87, 0x72), (0xB1CA)),  # HANGUL SYLLABLE NIEUN WAE PHIEUPH
    ((0x87, 0x73), (0xB1CB)),  # HANGUL SYLLABLE NIEUN WAE HIEUH
    ((0x87, 0x74), (0xB1CD)),  # HANGUL SYLLABLE NIEUN OE KIYEOK
    ((0x87, 0x75), (0xB1CE)),  # HANGUL SYLLABLE NIEUN OE SSANGKIYEOK
    ((0x87, 0x76), (0xB1CF)),  # HANGUL SYLLABLE NIEUN OE KIYEOKSIOS
    ((0x87, 0x77), (0xB1D1)),  # HANGUL SYLLABLE NIEUN OE NIEUNCIEUC
    ((0x87, 0x78), (0xB1D2)),  # HANGUL SYLLABLE NIEUN OE NIEUNHIEUH
    ((0x87, 0x79), (0xB1D3)),  # HANGUL SYLLABLE NIEUN OE TIKEUT
    ((0x87, 0x7A), (0xB1D5)),  # HANGUL SYLLABLE NIEUN OE RIEULKIYEOK
    ((0x87, 0x81), (0xB1D6)),  # HANGUL SYLLABLE NIEUN OE RIEULMIEUM
    ((0x87, 0x82), (0xB1D7)),  # HANGUL SYLLABLE NIEUN OE RIEULPIEUP
    ((0x87, 0x83), (0xB1D8)),  # HANGUL SYLLABLE NIEUN OE RIEULSIOS
    ((0x87, 0x84), (0xB1D9)),  # HANGUL SYLLABLE NIEUN OE RIEULTHIEUTH
    ((0x87, 0x85), (0xB1DA)),  # HANGUL SYLLABLE NIEUN OE RIEULPHIEUPH
    ((0x87, 0x86), (0xB1DB)),  # HANGUL SYLLABLE NIEUN OE RIEULHIEUH
    ((0x87, 0x87), (0xB1DE)),  # HANGUL SYLLABLE NIEUN OE PIEUPSIOS
    ((0x87, 0x88), (0xB1E0)),  # HANGUL SYLLABLE NIEUN OE SSANGSIOS
    ((0x87, 0x89), (0xB1E1)),  # HANGUL SYLLABLE NIEUN OE IEUNG
    ((0x87, 0x8A), (0xB1E2)),  # HANGUL SYLLABLE NIEUN OE CIEUC
    ((0x87, 0x8B), (0xB1E3)),  # HANGUL SYLLABLE NIEUN OE CHIEUCH
    ((0x87, 0x8C), (0xB1E4)),  # HANGUL SYLLABLE NIEUN OE KHIEUKH
    ((0x87, 0x8D), (0xB1E5)),  # HANGUL SYLLABLE NIEUN OE THIEUTH
    ((0x87, 0x8E), (0xB1E6)),  # HANGUL SYLLABLE NIEUN OE PHIEUPH
    ((0x87, 0x8F), (0xB1E7)),  # HANGUL SYLLABLE NIEUN OE HIEUH
    ((0x87, 0x90), (0xB1EA)),  # HANGUL SYLLABLE NIEUN YO SSANGKIYEOK
    ((0x87, 0x91), (0xB1EB)),  # HANGUL SYLLABLE NIEUN YO KIYEOKSIOS
    ((0x87, 0x92), (0xB1ED)),  # HANGUL SYLLABLE NIEUN YO NIEUNCIEUC
    ((0x87, 0x93), (0xB1EE)),  # HANGUL SYLLABLE NIEUN YO NIEUNHIEUH
    ((0x87, 0x94), (0xB1EF)),  # HANGUL SYLLABLE NIEUN YO TIKEUT
    ((0x87, 0x95), (0xB1F1)),  # HANGUL SYLLABLE NIEUN YO RIEULKIYEOK
    ((0x87, 0x96), (0xB1F2)),  # HANGUL SYLLABLE NIEUN YO RIEULMIEUM
    ((0x87, 0x97), (0xB1F3)),  # HANGUL SYLLABLE NIEUN YO RIEULPIEUP
    ((0x87, 0x98), (0xB1F4)),  # HANGUL SYLLABLE NIEUN YO RIEULSIOS
    ((0x87, 0x99), (0xB1F5)),  # HANGUL SYLLABLE NIEUN YO RIEULTHIEUTH
    ((0x87, 0x9A), (0xB1F6)),  # HANGUL SYLLABLE NIEUN YO RIEULPHIEUPH
    ((0x87, 0x9B), (0xB1F7)),  # HANGUL SYLLABLE NIEUN YO RIEULHIEUH
    ((0x87, 0x9C), (0xB1F8)),  # HANGUL SYLLABLE NIEUN YO MIEUM
    ((0x87, 0x9D), (0xB1FA)),  # HANGUL SYLLABLE NIEUN YO PIEUPSIOS
    ((0x87, 0x9E), (0xB1FC)),  # HANGUL SYLLABLE NIEUN YO SSANGSIOS
    ((0x87, 0x9F), (0xB1FE)),  # HANGUL SYLLABLE NIEUN YO CIEUC
    ((0x87, 0xA0), (0xB1FF)),  # HANGUL SYLLABLE NIEUN YO CHIEUCH
    ((0x87, 0xA1), (0xB200)),  # HANGUL SYLLABLE NIEUN YO KHIEUKH
    ((0x87, 0xA2), (0xB201)),  # HANGUL SYLLABLE NIEUN YO THIEUTH
    ((0x87, 0xA3), (0xB202)),  # HANGUL SYLLABLE NIEUN YO PHIEUPH
    ((0x87, 0xA4), (0xB203)),  # HANGUL SYLLABLE NIEUN YO HIEUH
    ((0x87, 0xA5), (0xB206)),  # HANGUL SYLLABLE NIEUN U SSANGKIYEOK
    ((0x87, 0xA6), (0xB207)),  # HANGUL SYLLABLE NIEUN U KIYEOKSIOS
    ((0x87, 0xA7), (0xB209)),  # HANGUL SYLLABLE NIEUN U NIEUNCIEUC
    ((0x87, 0xA8), (0xB20A)),  # HANGUL SYLLABLE NIEUN U NIEUNHIEUH
    ((0x87, 0xA9), (0xB20D)),  # HANGUL SYLLABLE NIEUN U RIEULKIYEOK
    ((0x87, 0xAA), (0xB20E)),  # HANGUL SYLLABLE NIEUN U RIEULMIEUM
    ((0x87, 0xAB), (0xB20F)),  # HANGUL SYLLABLE NIEUN U RIEULPIEUP
    ((0x87, 0xAC), (0xB210)),  # HANGUL SYLLABLE NIEUN U RIEULSIOS
    ((0x87, 0xAD), (0xB211)),  # HANGUL SYLLABLE NIEUN U RIEULTHIEUTH
    ((0x87, 0xAE), (0xB212)),  # HANGUL SYLLABLE NIEUN U RIEULPHIEUPH
    ((0x87, 0xAF), (0xB213)),  # HANGUL SYLLABLE NIEUN U RIEULHIEUH
    ((0x87, 0xB0), (0xB216)),  # HANGUL SYLLABLE NIEUN U PIEUPSIOS
    ((0x87, 0xB1), (0xB218)),  # HANGUL SYLLABLE NIEUN U SSANGSIOS
    ((0x87, 0xB2), (0xB21A)),  # HANGUL SYLLABLE NIEUN U CIEUC
    ((0x87, 0xB3), (0xB21B)),  # HANGUL SYLLABLE NIEUN U CHIEUCH
    ((0x87, 0xB4), (0xB21C)),  # HANGUL SYLLABLE NIEUN U KHIEUKH
    ((0x87, 0xB5), (0xB21D)),  # HANGUL SYLLABLE NIEUN U THIEUTH
    ((0x87, 0xB6), (0xB21E)),  # HANGUL SYLLABLE NIEUN U PHIEUPH
    ((0x87, 0xB7), (0xB21F)),  # HANGUL SYLLABLE NIEUN U HIEUH
    ((0x87, 0xB8), (0xB221)),  # HANGUL SYLLABLE NIEUN WEO KIYEOK
    ((0x87, 0xB9), (0xB222)),  # HANGUL SYLLABLE NIEUN WEO SSANGKIYEOK
    ((0x87, 0xBA), (0xB223)),  # HANGUL SYLLABLE NIEUN WEO KIYEOKSIOS
    ((0x87, 0xBB), (0xB224)),  # HANGUL SYLLABLE NIEUN WEO NIEUN
    ((0x87, 0xBC), (0xB225)),  # HANGUL SYLLABLE NIEUN WEO NIEUNCIEUC
    ((0x87, 0xBD), (0xB226)),  # HANGUL SYLLABLE NIEUN WEO NIEUNHIEUH
    ((0x87, 0xBE), (0xB227)),  # HANGUL SYLLABLE NIEUN WEO TIKEUT
    ((0x87, 0xBF), (0xB228)),  # HANGUL SYLLABLE NIEUN WEO RIEUL
    ((0x87, 0xC0), (0xB229)),  # HANGUL SYLLABLE NIEUN WEO RIEULKIYEOK
    ((0x87, 0xC1), (0xB22A)),  # HANGUL SYLLABLE NIEUN WEO RIEULMIEUM
    ((0x87, 0xC2), (0xB22B)),  # HANGUL SYLLABLE NIEUN WEO RIEULPIEUP
    ((0x87, 0xC3), (0xB22C)),  # HANGUL SYLLABLE NIEUN WEO RIEULSIOS
    ((0x87, 0xC4), (0xB22D)),  # HANGUL SYLLABLE NIEUN WEO RIEULTHIEUTH
    ((0x87, 0xC5), (0xB22E)),  # HANGUL SYLLABLE NIEUN WEO RIEULPHIEUPH
    ((0x87, 0xC6), (0xB22F)),  # HANGUL SYLLABLE NIEUN WEO RIEULHIEUH
    ((0x87, 0xC7), (0xB230)),  # HANGUL SYLLABLE NIEUN WEO MIEUM
    ((0x87, 0xC8), (0xB231)),  # HANGUL SYLLABLE NIEUN WEO PIEUP
    ((0x87, 0xC9), (0xB232)),  # HANGUL SYLLABLE NIEUN WEO PIEUPSIOS
    ((0x87, 0xCA), (0xB233)),  # HANGUL SYLLABLE NIEUN WEO SIOS
    ((0x87, 0xCB), (0xB235)),  # HANGUL SYLLABLE NIEUN WEO IEUNG
    ((0x87, 0xCC), (0xB236)),  # HANGUL SYLLABLE NIEUN WEO CIEUC
    ((0x87, 0xCD), (0xB237)),  # HANGUL SYLLABLE NIEUN WEO CHIEUCH
    ((0x87, 0xCE), (0xB238)),  # HANGUL SYLLABLE NIEUN WEO KHIEUKH
    ((0x87, 0xCF), (0xB239)),  # HANGUL SYLLABLE NIEUN WEO THIEUTH
    ((0x87, 0xD0), (0xB23A)),  # HANGUL SYLLABLE NIEUN WEO PHIEUPH
    ((0x87, 0xD1), (0xB23B)),  # HANGUL SYLLABLE NIEUN WEO HIEUH
    ((0x87, 0xD2), (0xB23D)),  # HANGUL SYLLABLE NIEUN WE KIYEOK
    ((0x87, 0xD3), (0xB23E)),  # HANGUL SYLLABLE NIEUN WE SSANGKIYEOK
    ((0x87, 0xD4), (0xB23F)),  # HANGUL SYLLABLE NIEUN WE KIYEOKSIOS
    ((0x87, 0xD5), (0xB240)),  # HANGUL SYLLABLE NIEUN WE NIEUN
    ((0x87, 0xD6), (0xB241)),  # HANGUL SYLLABLE NIEUN WE NIEUNCIEUC
    ((0x87, 0xD7), (0xB242)),  # HANGUL SYLLABLE NIEUN WE NIEUNHIEUH
    ((0x87, 0xD8), (0xB243)),  # HANGUL SYLLABLE NIEUN WE TIKEUT
    ((0x87, 0xD9), (0xB244)),  # HANGUL SYLLABLE NIEUN WE RIEUL
    ((0x87, 0xDA), (0xB245)),  # HANGUL SYLLABLE NIEUN WE RIEULKIYEOK
    ((0x87, 0xDB), (0xB246)),  # HANGUL SYLLABLE NIEUN WE RIEULMIEUM
    ((0x87, 0xDC), (0xB247)),  # HANGUL SYLLABLE NIEUN WE RIEULPIEUP
    ((0x87, 0xDD), (0xB248)),  # HANGUL SYLLABLE NIEUN WE RIEULSIOS
    ((0x87, 0xDE), (0xB249)),  # HANGUL SYLLABLE NIEUN WE RIEULTHIEUTH
    ((0x87, 0xDF), (0xB24A)),  # HANGUL SYLLABLE NIEUN WE RIEULPHIEUPH
    ((0x87, 0xE0), (0xB24B)),  # HANGUL SYLLABLE NIEUN WE RIEULHIEUH
    ((0x87, 0xE1), (0xB24C)),  # HANGUL SYLLABLE NIEUN WE MIEUM
    ((0x87, 0xE2), (0xB24D)),  # HANGUL SYLLABLE NIEUN WE PIEUP
    ((0x87, 0xE3), (0xB24E)),  # HANGUL SYLLABLE NIEUN WE PIEUPSIOS
    ((0x87, 0xE4), (0xB24F)),  # HANGUL SYLLABLE NIEUN WE SIOS
    ((0x87, 0xE5), (0xB250)),  # HANGUL SYLLABLE NIEUN WE SSANGSIOS
    ((0x87, 0xE6), (0xB251)),  # HANGUL SYLLABLE NIEUN WE IEUNG
    ((0x87, 0xE7), (0xB252)),  # HANGUL SYLLABLE NIEUN WE CIEUC
    ((0x87, 0xE8), (0xB253)),  # HANGUL SYLLABLE NIEUN WE CHIEUCH
    ((0x87, 0xE9), (0xB254)),  # HANGUL SYLLABLE NIEUN WE KHIEUKH
    ((0x87, 0xEA), (0xB255)),  # HANGUL SYLLABLE NIEUN WE THIEUTH
    ((0x87, 0xEB), (0xB256)),  # HANGUL SYLLABLE NIEUN WE PHIEUPH
    ((0x87, 0xEC), (0xB257)),  # HANGUL SYLLABLE NIEUN WE HIEUH
    ((0x87, 0xED), (0xB259)),  # HANGUL SYLLABLE NIEUN WI KIYEOK
    ((0x87, 0xEE), (0xB25A)),  # HANGUL SYLLABLE NIEUN WI SSANGKIYEOK
    ((0x87, 0xEF), (0xB25B)),  # HANGUL SYLLABLE NIEUN WI KIYEOKSIOS
    ((0x87, 0xF0), (0xB25D)),  # HANGUL SYLLABLE NIEUN WI NIEUNCIEUC
    ((0x87, 0xF1), (0xB25E)),  # HANGUL SYLLABLE NIEUN WI NIEUNHIEUH
    ((0x87, 0xF2), (0xB25F)),  # HANGUL SYLLABLE NIEUN WI TIKEUT
    ((0x87, 0xF3), (0xB261)),  # HANGUL SYLLABLE NIEUN WI RIEULKIYEOK
    ((0x87, 0xF4), (0xB262)),  # HANGUL SYLLABLE NIEUN WI RIEULMIEUM
    ((0x87, 0xF5), (0xB263)),  # HANGUL SYLLABLE NIEUN WI RIEULPIEUP
    ((0x87, 0xF6), (0xB264)),  # HANGUL SYLLABLE NIEUN WI RIEULSIOS
    ((0x87, 0xF7), (0xB265)),  # HANGUL SYLLABLE NIEUN WI RIEULTHIEUTH
    ((0x87, 0xF8), (0xB266)),  # HANGUL SYLLABLE NIEUN WI RIEULPHIEUPH
    ((0x87, 0xF9), (0xB267)),  # HANGUL SYLLABLE NIEUN WI RIEULHIEUH
    ((0x87, 0xFA), (0xB26A)),  # HANGUL SYLLABLE NIEUN WI PIEUPSIOS
    ((0x87, 0xFB), (0xB26B)),  # HANGUL SYLLABLE NIEUN WI SIOS
    ((0x87, 0xFC), (0xB26C)),  # HANGUL SYLLABLE NIEUN WI SSANGSIOS
    ((0x87, 0xFD), (0xB26D)),  # HANGUL SYLLABLE NIEUN WI IEUNG
    ((0x87, 0xFE), (0xB26E)),  # HANGUL SYLLABLE NIEUN WI CIEUC
    ((0x88, 0x41), (0xB26F)),  # HANGUL SYLLABLE NIEUN WI CHIEUCH
    ((0x88, 0x42), (0xB270)),  # HANGUL SYLLABLE NIEUN WI KHIEUKH
    ((0x88, 0x43), (0xB271)),  # HANGUL SYLLABLE NIEUN WI THIEUTH
    ((0x88, 0x44), (0xB272)),  # HANGUL SYLLABLE NIEUN WI PHIEUPH
    ((0x88, 0x45), (0xB273)),  # HANGUL SYLLABLE NIEUN WI HIEUH
    ((0x88, 0x46), (0xB276)),  # HANGUL SYLLABLE NIEUN YU SSANGKIYEOK
    ((0x88, 0x47), (0xB277)),  # HANGUL SYLLABLE NIEUN YU KIYEOKSIOS
    ((0x88, 0x48), (0xB278)),  # HANGUL SYLLABLE NIEUN YU NIEUN
    ((0x88, 0x49), (0xB279)),  # HANGUL SYLLABLE NIEUN YU NIEUNCIEUC
    ((0x88, 0x4A), (0xB27A)),  # HANGUL SYLLABLE NIEUN YU NIEUNHIEUH
    ((0x88, 0x4B), (0xB27B)),  # HANGUL SYLLABLE NIEUN YU TIKEUT
    ((0x88, 0x4C), (0xB27D)),  # HANGUL SYLLABLE NIEUN YU RIEULKIYEOK
    ((0x88, 0x4D), (0xB27E)),  # HANGUL SYLLABLE NIEUN YU RIEULMIEUM
    ((0x88, 0x4E), (0xB27F)),  # HANGUL SYLLABLE NIEUN YU RIEULPIEUP
    ((0x88, 0x4F), (0xB280)),  # HANGUL SYLLABLE NIEUN YU RIEULSIOS
    ((0x88, 0x50), (0xB281)),  # HANGUL SYLLABLE NIEUN YU RIEULTHIEUTH
    ((0x88, 0x51), (0xB282)),  # HANGUL SYLLABLE NIEUN YU RIEULPHIEUPH
    ((0x88, 0x52), (0xB283)),  # HANGUL SYLLABLE NIEUN YU RIEULHIEUH
    ((0x88, 0x53), (0xB286)),  # HANGUL SYLLABLE NIEUN YU PIEUPSIOS
    ((0x88, 0x54), (0xB287)),  # HANGUL SYLLABLE NIEUN YU SIOS
    ((0x88, 0x55), (0xB288)),  # HANGUL SYLLABLE NIEUN YU SSANGSIOS
    ((0x88, 0x56), (0xB28A)),  # HANGUL SYLLABLE NIEUN YU CIEUC
    ((0x88, 0x57), (0xB28B)),  # HANGUL SYLLABLE NIEUN YU CHIEUCH
    ((0x88, 0x58), (0xB28C)),  # HANGUL SYLLABLE NIEUN YU KHIEUKH
    ((0x88, 0x59), (0xB28D)),  # HANGUL SYLLABLE NIEUN YU THIEUTH
    ((0x88, 0x5A), (0xB28E)),  # HANGUL SYLLABLE NIEUN YU PHIEUPH
    ((0x88, 0x61), (0xB28F)),  # HANGUL SYLLABLE NIEUN YU HIEUH
    ((0x88, 0x62), (0xB292)),  # HANGUL SYLLABLE NIEUN EU SSANGKIYEOK
    ((0x88, 0x63), (0xB293)),  # HANGUL SYLLABLE NIEUN EU KIYEOKSIOS
    ((0x88, 0x64), (0xB295)),  # HANGUL SYLLABLE NIEUN EU NIEUNCIEUC
    ((0x88, 0x65), (0xB296)),  # HANGUL SYLLABLE NIEUN EU NIEUNHIEUH
    ((0x88, 0x66), (0xB297)),  # HANGUL SYLLABLE NIEUN EU TIKEUT
    ((0x88, 0x67), (0xB29B)),  # HANGUL SYLLABLE NIEUN EU RIEULPIEUP
    ((0x88, 0x68), (0xB29C)),  # HANGUL SYLLABLE NIEUN EU RIEULSIOS
    ((0x88, 0x69), (0xB29D)),  # HANGUL SYLLABLE NIEUN EU RIEULTHIEUTH
    ((0x88, 0x6A), (0xB29E)),  # HANGUL SYLLABLE NIEUN EU RIEULPHIEUPH
    ((0x88, 0x6B), (0xB29F)),  # HANGUL SYLLABLE NIEUN EU RIEULHIEUH
    ((0x88, 0x6C), (0xB2A2)),  # HANGUL SYLLABLE NIEUN EU PIEUPSIOS
    ((0x88, 0x6D), (0xB2A4)),  # HANGUL SYLLABLE NIEUN EU SSANGSIOS
    ((0x88, 0x6E), (0xB2A7)),  # HANGUL SYLLABLE NIEUN EU CHIEUCH
    ((0x88, 0x6F), (0xB2A8)),  # HANGUL SYLLABLE NIEUN EU KHIEUKH
    ((0x88, 0x70), (0xB2A9)),  # HANGUL SYLLABLE NIEUN EU THIEUTH
    ((0x88, 0x71), (0xB2AB)),  # HANGUL SYLLABLE NIEUN EU HIEUH
    ((0x88, 0x72), (0xB2AD)),  # HANGUL SYLLABLE NIEUN YI KIYEOK
    ((0x88, 0x73), (0xB2AE)),  # HANGUL SYLLABLE NIEUN YI SSANGKIYEOK
    ((0x88, 0x74), (0xB2AF)),  # HANGUL SYLLABLE NIEUN YI KIYEOKSIOS
    ((0x88, 0x75), (0xB2B1)),  # HANGUL SYLLABLE NIEUN YI NIEUNCIEUC
    ((0x88, 0x76), (0xB2B2)),  # HANGUL SYLLABLE NIEUN YI NIEUNHIEUH
    ((0x88, 0x77), (0xB2B3)),  # HANGUL SYLLABLE NIEUN YI TIKEUT
    ((0x88, 0x78), (0xB2B5)),  # HANGUL SYLLABLE NIEUN YI RIEULKIYEOK
    ((0x88, 0x79), (0xB2B6)),  # HANGUL SYLLABLE NIEUN YI RIEULMIEUM
    ((0x88, 0x7A), (0xB2B7)),  # HANGUL SYLLABLE NIEUN YI RIEULPIEUP
    ((0x88, 0x81), (0xB2B8)),  # HANGUL SYLLABLE NIEUN YI RIEULSIOS
    ((0x88, 0x82), (0xB2B9)),  # HANGUL SYLLABLE NIEUN YI RIEULTHIEUTH
    ((0x88, 0x83), (0xB2BA)),  # HANGUL SYLLABLE NIEUN YI RIEULPHIEUPH
    ((0x88, 0x84), (0xB2BB)),  # HANGUL SYLLABLE NIEUN YI RIEULHIEUH
    ((0x88, 0x85), (0xB2BC)),  # HANGUL SYLLABLE NIEUN YI MIEUM
    ((0x88, 0x86), (0xB2BD)),  # HANGUL SYLLABLE NIEUN YI PIEUP
    ((0x88, 0x87), (0xB2BE)),  # HANGUL SYLLABLE NIEUN YI PIEUPSIOS
    ((0x88, 0x88), (0xB2BF)),  # HANGUL SYLLABLE NIEUN YI SIOS
    ((0x88, 0x89), (0xB2C0)),  # HANGUL SYLLABLE NIEUN YI SSANGSIOS
    ((0x88, 0x8A), (0xB2C1)),  # HANGUL SYLLABLE NIEUN YI IEUNG
    ((0x88, 0x8B), (0xB2C2)),  # HANGUL SYLLABLE NIEUN YI CIEUC
    ((0x88, 0x8C), (0xB2C3)),  # HANGUL SYLLABLE NIEUN YI CHIEUCH
    ((0x88, 0x8D), (0xB2C4)),  # HANGUL SYLLABLE NIEUN YI KHIEUKH
    ((0x88, 0x8E), (0xB2C5)),  # HANGUL SYLLABLE NIEUN YI THIEUTH
    ((0x88, 0x8F), (0xB2C6)),  # HANGUL SYLLABLE NIEUN YI PHIEUPH
    ((0x88, 0x90), (0xB2C7)),  # HANGUL SYLLABLE NIEUN YI HIEUH
    ((0x88, 0x91), (0xB2CA)),  # HANGUL SYLLABLE NIEUN I SSANGKIYEOK
    ((0x88, 0x92), (0xB2CB)),  # HANGUL SYLLABLE NIEUN I KIYEOKSIOS
    ((0x88, 0x93), (0xB2CD)),  # HANGUL SYLLABLE NIEUN I NIEUNCIEUC
    ((0x88, 0x94), (0xB2CE)),  # HANGUL SYLLABLE NIEUN I NIEUNHIEUH
    ((0x88, 0x95), (0xB2CF)),  # HANGUL SYLLABLE NIEUN I TIKEUT
    ((0x88, 0x96), (0xB2D1)),  # HANGUL SYLLABLE NIEUN I RIEULKIYEOK
    ((0x88, 0x97), (0xB2D3)),  # HANGUL SYLLABLE NIEUN I RIEULPIEUP
    ((0x88, 0x98), (0xB2D4)),  # HANGUL SYLLABLE NIEUN I RIEULSIOS
    ((0x88, 0x99), (0xB2D5)),  # HANGUL SYLLABLE NIEUN I RIEULTHIEUTH
    ((0x88, 0x9A), (0xB2D6)),  # HANGUL SYLLABLE NIEUN I RIEULPHIEUPH
    ((0x88, 0x9B), (0xB2D7)),  # HANGUL SYLLABLE NIEUN I RIEULHIEUH
    ((0x88, 0x9C), (0xB2DA)),  # HANGUL SYLLABLE NIEUN I PIEUPSIOS
    ((0x88, 0x9D), (0xB2DC)),  # HANGUL SYLLABLE NIEUN I SSANGSIOS
    ((0x88, 0x9E), (0xB2DE)),  # HANGUL SYLLABLE NIEUN I CIEUC
    ((0x88, 0x9F), (0xB2DF)),  # HANGUL SYLLABLE NIEUN I CHIEUCH
    ((0x88, 0xA0), (0xB2E0)),  # HANGUL SYLLABLE NIEUN I KHIEUKH
    ((0x88, 0xA1), (0xB2E1)),  # HANGUL SYLLABLE NIEUN I THIEUTH
    ((0x88, 0xA2), (0xB2E3)),  # HANGUL SYLLABLE NIEUN I HIEUH
    ((0x88, 0xA3), (0xB2E7)),  # HANGUL SYLLABLE TIKEUT A KIYEOKSIOS
    ((0x88, 0xA4), (0xB2E9)),  # HANGUL SYLLABLE TIKEUT A NIEUNCIEUC
    ((0x88, 0xA5), (0xB2EA)),  # HANGUL SYLLABLE TIKEUT A NIEUNHIEUH
    ((0x88, 0xA6), (0xB2F0)),  # HANGUL SYLLABLE TIKEUT A RIEULSIOS
    ((0x88, 0xA7), (0xB2F1)),  # HANGUL SYLLABLE TIKEUT A RIEULTHIEUTH
    ((0x88, 0xA8), (0xB2F2)),  # HANGUL SYLLABLE TIKEUT A RIEULPHIEUPH
    ((0x88, 0xA9), (0xB2F6)),  # HANGUL SYLLABLE TIKEUT A PIEUPSIOS
    ((0x88, 0xAA), (0xB2FC)),  # HANGUL SYLLABLE TIKEUT A KHIEUKH
    ((0x88, 0xAB), (0xB2FD)),  # HANGUL SYLLABLE TIKEUT A THIEUTH
    ((0x88, 0xAC), (0xB2FE)),  # HANGUL SYLLABLE TIKEUT A PHIEUPH
    ((0x88, 0xAD), (0xB302)),  # HANGUL SYLLABLE TIKEUT AE SSANGKIYEOK
    ((0x88, 0xAE), (0xB303)),  # HANGUL SYLLABLE TIKEUT AE KIYEOKSIOS
    ((0x88, 0xAF), (0xB305)),  # HANGUL SYLLABLE TIKEUT AE NIEUNCIEUC
    ((0x88, 0xB0), (0xB306)),  # HANGUL SYLLABLE TIKEUT AE NIEUNHIEUH
    ((0x88, 0xB1), (0xB307)),  # HANGUL SYLLABLE TIKEUT AE TIKEUT
    ((0x88, 0xB2), (0xB309)),  # HANGUL SYLLABLE TIKEUT AE RIEULKIYEOK
    ((0x88, 0xB3), (0xB30A)),  # HANGUL SYLLABLE TIKEUT AE RIEULMIEUM
    ((0x88, 0xB4), (0xB30B)),  # HANGUL SYLLABLE TIKEUT AE RIEULPIEUP
    ((0x88, 0xB5), (0xB30C)),  # HANGUL SYLLABLE TIKEUT AE RIEULSIOS
    ((0x88, 0xB6), (0xB30D)),  # HANGUL SYLLABLE TIKEUT AE RIEULTHIEUTH
    ((0x88, 0xB7), (0xB30E)),  # HANGUL SYLLABLE TIKEUT AE RIEULPHIEUPH
    ((0x88, 0xB8), (0xB30F)),  # HANGUL SYLLABLE TIKEUT AE RIEULHIEUH
    ((0x88, 0xB9), (0xB312)),  # HANGUL SYLLABLE TIKEUT AE PIEUPSIOS
    ((0x88, 0xBA), (0xB316)),  # HANGUL SYLLABLE TIKEUT AE CIEUC
    ((0x88, 0xBB), (0xB317)),  # HANGUL SYLLABLE TIKEUT AE CHIEUCH
    ((0x88, 0xBC), (0xB318)),  # HANGUL SYLLABLE TIKEUT AE KHIEUKH
    ((0x88, 0xBD), (0xB319)),  # HANGUL SYLLABLE TIKEUT AE THIEUTH
    ((0x88, 0xBE), (0xB31A)),  # HANGUL SYLLABLE TIKEUT AE PHIEUPH
    ((0x88, 0xBF), (0xB31B)),  # HANGUL SYLLABLE TIKEUT AE HIEUH
    ((0x88, 0xC0), (0xB31D)),  # HANGUL SYLLABLE TIKEUT YA KIYEOK
    ((0x88, 0xC1), (0xB31E)),  # HANGUL SYLLABLE TIKEUT YA SSANGKIYEOK
    ((0x88, 0xC2), (0xB31F)),  # HANGUL SYLLABLE TIKEUT YA KIYEOKSIOS
    ((0x88, 0xC3), (0xB320)),  # HANGUL SYLLABLE TIKEUT YA NIEUN
    ((0x88, 0xC4), (0xB321)),  # HANGUL SYLLABLE TIKEUT YA NIEUNCIEUC
    ((0x88, 0xC5), (0xB322)),  # HANGUL SYLLABLE TIKEUT YA NIEUNHIEUH
    ((0x88, 0xC6), (0xB323)),  # HANGUL SYLLABLE TIKEUT YA TIKEUT
    ((0x88, 0xC7), (0xB324)),  # HANGUL SYLLABLE TIKEUT YA RIEUL
    ((0x88, 0xC8), (0xB325)),  # HANGUL SYLLABLE TIKEUT YA RIEULKIYEOK
    ((0x88, 0xC9), (0xB326)),  # HANGUL SYLLABLE TIKEUT YA RIEULMIEUM
    ((0x88, 0xCA), (0xB327)),  # HANGUL SYLLABLE TIKEUT YA RIEULPIEUP
    ((0x88, 0xCB), (0xB328)),  # HANGUL SYLLABLE TIKEUT YA RIEULSIOS
    ((0x88, 0xCC), (0xB329)),  # HANGUL SYLLABLE TIKEUT YA RIEULTHIEUTH
    ((0x88, 0xCD), (0xB32A)),  # HANGUL SYLLABLE TIKEUT YA RIEULPHIEUPH
    ((0x88, 0xCE), (0xB32B)),  # HANGUL SYLLABLE TIKEUT YA RIEULHIEUH
    ((0x88, 0xCF), (0xB32C)),  # HANGUL SYLLABLE TIKEUT YA MIEUM
    ((0x88, 0xD0), (0xB32D)),  # HANGUL SYLLABLE TIKEUT YA PIEUP
    ((0x88, 0xD1), (0xB32E)),  # HANGUL SYLLABLE TIKEUT YA PIEUPSIOS
    ((0x88, 0xD2), (0xB32F)),  # HANGUL SYLLABLE TIKEUT YA SIOS
    ((0x88, 0xD3), (0xB330)),  # HANGUL SYLLABLE TIKEUT YA SSANGSIOS
    ((0x88, 0xD4), (0xB331)),  # HANGUL SYLLABLE TIKEUT YA IEUNG
    ((0x88, 0xD5), (0xB332)),  # HANGUL SYLLABLE TIKEUT YA CIEUC
    ((0x88, 0xD6), (0xB333)),  # HANGUL SYLLABLE TIKEUT YA CHIEUCH
    ((0x88, 0xD7), (0xB334)),  # HANGUL SYLLABLE TIKEUT YA KHIEUKH
    ((0x88, 0xD8), (0xB335)),  # HANGUL SYLLABLE TIKEUT YA THIEUTH
    ((0x88, 0xD9), (0xB336)),  # HANGUL SYLLABLE TIKEUT YA PHIEUPH
    ((0x88, 0xDA), (0xB337)),  # HANGUL SYLLABLE TIKEUT YA HIEUH
    ((0x88, 0xDB), (0xB338)),  # HANGUL SYLLABLE TIKEUT YAE
    ((0x88, 0xDC), (0xB339)),  # HANGUL SYLLABLE TIKEUT YAE KIYEOK
    ((0x88, 0xDD), (0xB33A)),  # HANGUL SYLLABLE TIKEUT YAE SSANGKIYEOK
    ((0x88, 0xDE), (0xB33B)),  # HANGUL SYLLABLE TIKEUT YAE KIYEOKSIOS
    ((0x88, 0xDF), (0xB33C)),  # HANGUL SYLLABLE TIKEUT YAE NIEUN
    ((0x88, 0xE0), (0xB33D)),  # HANGUL SYLLABLE TIKEUT YAE NIEUNCIEUC
    ((0x88, 0xE1), (0xB33E)),  # HANGUL SYLLABLE TIKEUT YAE NIEUNHIEUH
    ((0x88, 0xE2), (0xB33F)),  # HANGUL SYLLABLE TIKEUT YAE TIKEUT
    ((0x88, 0xE3), (0xB340)),  # HANGUL SYLLABLE TIKEUT YAE RIEUL
    ((0x88, 0xE4), (0xB341)),  # HANGUL SYLLABLE TIKEUT YAE RIEULKIYEOK
    ((0x88, 0xE5), (0xB342)),  # HANGUL SYLLABLE TIKEUT YAE RIEULMIEUM
    ((0x88, 0xE6), (0xB343)),  # HANGUL SYLLABLE TIKEUT YAE RIEULPIEUP
    ((0x88, 0xE7), (0xB344)),  # HANGUL SYLLABLE TIKEUT YAE RIEULSIOS
    ((0x88, 0xE8), (0xB345)),  # HANGUL SYLLABLE TIKEUT YAE RIEULTHIEUTH
    ((0x88, 0xE9), (0xB346)),  # HANGUL SYLLABLE TIKEUT YAE RIEULPHIEUPH
    ((0x88, 0xEA), (0xB347)),  # HANGUL SYLLABLE TIKEUT YAE RIEULHIEUH
    ((0x88, 0xEB), (0xB348)),  # HANGUL SYLLABLE TIKEUT YAE MIEUM
    ((0x88, 0xEC), (0xB349)),  # HANGUL SYLLABLE TIKEUT YAE PIEUP
    ((0x88, 0xED), (0xB34A)),  # HANGUL SYLLABLE TIKEUT YAE PIEUPSIOS
    ((0x88, 0xEE), (0xB34B)),  # HANGUL SYLLABLE TIKEUT YAE SIOS
    ((0x88, 0xEF), (0xB34C)),  # HANGUL SYLLABLE TIKEUT YAE SSANGSIOS
    ((0x88, 0xF0), (0xB34D)),  # HANGUL SYLLABLE TIKEUT YAE IEUNG
    ((0x88, 0xF1), (0xB34E)),  # HANGUL SYLLABLE TIKEUT YAE CIEUC
    ((0x88, 0xF2), (0xB34F)),  # HANGUL SYLLABLE TIKEUT YAE CHIEUCH
    ((0x88, 0xF3), (0xB350)),  # HANGUL SYLLABLE TIKEUT YAE KHIEUKH
    ((0x88, 0xF4), (0xB351)),  # HANGUL SYLLABLE TIKEUT YAE THIEUTH
    ((0x88, 0xF5), (0xB352)),  # HANGUL SYLLABLE TIKEUT YAE PHIEUPH
    ((0x88, 0xF6), (0xB353)),  # HANGUL SYLLABLE TIKEUT YAE HIEUH
    ((0x88, 0xF7), (0xB357)),  # HANGUL SYLLABLE TIKEUT EO KIYEOKSIOS
    ((0x88, 0xF8), (0xB359)),  # HANGUL SYLLABLE TIKEUT EO NIEUNCIEUC
    ((0x88, 0xF9), (0xB35A)),  # HANGUL SYLLABLE TIKEUT EO NIEUNHIEUH
    ((0x88, 0xFA), (0xB35D)),  # HANGUL SYLLABLE TIKEUT EO RIEULKIYEOK
    ((0x88, 0xFB), (0xB360)),  # HANGUL SYLLABLE TIKEUT EO RIEULSIOS
    ((0x88, 0xFC), (0xB361)),  # HANGUL SYLLABLE TIKEUT EO RIEULTHIEUTH
    ((0x88, 0xFD), (0xB362)),  # HANGUL SYLLABLE TIKEUT EO RIEULPHIEUPH
    ((0x88, 0xFE), (0xB363)),  # HANGUL SYLLABLE TIKEUT EO RIEULHIEUH
    ((0x89, 0x41), (0xB366)),  # HANGUL SYLLABLE TIKEUT EO PIEUPSIOS
    ((0x89, 0x42), (0xB368)),  # HANGUL SYLLABLE TIKEUT EO SSANGSIOS
    ((0x89, 0x43), (0xB36A)),  # HANGUL SYLLABLE TIKEUT EO CIEUC
    ((0x89, 0x44), (0xB36C)),  # HANGUL SYLLABLE TIKEUT EO KHIEUKH
    ((0x89, 0x45), (0xB36D)),  # HANGUL SYLLABLE TIKEUT EO THIEUTH
    ((0x89, 0x46), (0xB36F)),  # HANGUL SYLLABLE TIKEUT EO HIEUH
    ((0x89, 0x47), (0xB372)),  # HANGUL SYLLABLE TIKEUT E SSANGKIYEOK
    ((0x89, 0x48), (0xB373)),  # HANGUL SYLLABLE TIKEUT E KIYEOKSIOS
    ((0x89, 0x49), (0xB375)),  # HANGUL SYLLABLE TIKEUT E NIEUNCIEUC
    ((0x89, 0x4A), (0xB376)),  # HANGUL SYLLABLE TIKEUT E NIEUNHIEUH
    ((0x89, 0x4B), (0xB377)),  # HANGUL SYLLABLE TIKEUT E TIKEUT
    ((0x89, 0x4C), (0xB379)),  # HANGUL SYLLABLE TIKEUT E RIEULKIYEOK
    ((0x89, 0x4D), (0xB37A)),  # HANGUL SYLLABLE TIKEUT E RIEULMIEUM
    ((0x89, 0x4E), (0xB37B)),  # HANGUL SYLLABLE TIKEUT E RIEULPIEUP
    ((0x89, 0x4F), (0xB37C)),  # HANGUL SYLLABLE TIKEUT E RIEULSIOS
    ((0x89, 0x50), (0xB37D)),  # HANGUL SYLLABLE TIKEUT E RIEULTHIEUTH
    ((0x89, 0x51), (0xB37E)),  # HANGUL SYLLABLE TIKEUT E RIEULPHIEUPH
    ((0x89, 0x52), (0xB37F)),  # HANGUL SYLLABLE TIKEUT E RIEULHIEUH
    ((0x89, 0x53), (0xB382)),  # HANGUL SYLLABLE TIKEUT E PIEUPSIOS
    ((0x89, 0x54), (0xB386)),  # HANGUL SYLLABLE TIKEUT E CIEUC
    ((0x89, 0x55), (0xB387)),  # HANGUL SYLLABLE TIKEUT E CHIEUCH
    ((0x89, 0x56), (0xB388)),  # HANGUL SYLLABLE TIKEUT E KHIEUKH
    ((0x89, 0x57), (0xB389)),  # HANGUL SYLLABLE TIKEUT E THIEUTH
    ((0x89, 0x58), (0xB38A)),  # HANGUL SYLLABLE TIKEUT E PHIEUPH
    ((0x89, 0x59), (0xB38B)),  # HANGUL SYLLABLE TIKEUT E HIEUH
    ((0x89, 0x5A), (0xB38D)),  # HANGUL SYLLABLE TIKEUT YEO KIYEOK
    ((0x89, 0x61), (0xB38E)),  # HANGUL SYLLABLE TIKEUT YEO SSANGKIYEOK
    ((0x89, 0x62), (0xB38F)),  # HANGUL SYLLABLE TIKEUT YEO KIYEOKSIOS
    ((0x89, 0x63), (0xB391)),  # HANGUL SYLLABLE TIKEUT YEO NIEUNCIEUC
    ((0x89, 0x64), (0xB392)),  # HANGUL SYLLABLE TIKEUT YEO NIEUNHIEUH
    ((0x89, 0x65), (0xB393)),  # HANGUL SYLLABLE TIKEUT YEO TIKEUT
    ((0x89, 0x66), (0xB395)),  # HANGUL SYLLABLE TIKEUT YEO RIEULKIYEOK
    ((0x89, 0x67), (0xB396)),  # HANGUL SYLLABLE TIKEUT YEO RIEULMIEUM
    ((0x89, 0x68), (0xB397)),  # HANGUL SYLLABLE TIKEUT YEO RIEULPIEUP
    ((0x89, 0x69), (0xB398)),  # HANGUL SYLLABLE TIKEUT YEO RIEULSIOS
    ((0x89, 0x6A), (0xB399)),  # HANGUL SYLLABLE TIKEUT YEO RIEULTHIEUTH
    ((0x89, 0x6B), (0xB39A)),  # HANGUL SYLLABLE TIKEUT YEO RIEULPHIEUPH
    ((0x89, 0x6C), (0xB39B)),  # HANGUL SYLLABLE TIKEUT YEO RIEULHIEUH
    ((0x89, 0x6D), (0xB39C)),  # HANGUL SYLLABLE TIKEUT YEO MIEUM
    ((0x89, 0x6E), (0xB39D)),  # HANGUL SYLLABLE TIKEUT YEO PIEUP
    ((0x89, 0x6F), (0xB39E)),  # HANGUL SYLLABLE TIKEUT YEO PIEUPSIOS
    ((0x89, 0x70), (0xB39F)),  # HANGUL SYLLABLE TIKEUT YEO SIOS
    ((0x89, 0x71), (0xB3A2)),  # HANGUL SYLLABLE TIKEUT YEO CIEUC
    ((0x89, 0x72), (0xB3A3)),  # HANGUL SYLLABLE TIKEUT YEO CHIEUCH
    ((0x89, 0x73), (0xB3A4)),  # HANGUL SYLLABLE TIKEUT YEO KHIEUKH
    ((0x89, 0x74), (0xB3A5)),  # HANGUL SYLLABLE TIKEUT YEO THIEUTH
    ((0x89, 0x75), (0xB3A6)),  # HANGUL SYLLABLE TIKEUT YEO PHIEUPH
    ((0x89, 0x76), (0xB3A7)),  # HANGUL SYLLABLE TIKEUT YEO HIEUH
    ((0x89, 0x77), (0xB3A9)),  # HANGUL SYLLABLE TIKEUT YE KIYEOK
    ((0x89, 0x78), (0xB3AA)),  # HANGUL SYLLABLE TIKEUT YE SSANGKIYEOK
    ((0x89, 0x79), (0xB3AB)),  # HANGUL SYLLABLE TIKEUT YE KIYEOKSIOS
    ((0x89, 0x7A), (0xB3AD)),  # HANGUL SYLLABLE TIKEUT YE NIEUNCIEUC
    ((0x89, 0x81), (0xB3AE)),  # HANGUL SYLLABLE TIKEUT YE NIEUNHIEUH
    ((0x89, 0x82), (0xB3AF)),  # HANGUL SYLLABLE TIKEUT YE TIKEUT
    ((0x89, 0x83), (0xB3B0)),  # HANGUL SYLLABLE TIKEUT YE RIEUL
    ((0x89, 0x84), (0xB3B1)),  # HANGUL SYLLABLE TIKEUT YE RIEULKIYEOK
    ((0x89, 0x85), (0xB3B2)),  # HANGUL SYLLABLE TIKEUT YE RIEULMIEUM
    ((0x89, 0x86), (0xB3B3)),  # HANGUL SYLLABLE TIKEUT YE RIEULPIEUP
    ((0x89, 0x87), (0xB3B4)),  # HANGUL SYLLABLE TIKEUT YE RIEULSIOS
    ((0x89, 0x88), (0xB3B5)),  # HANGUL SYLLABLE TIKEUT YE RIEULTHIEUTH
    ((0x89, 0x89), (0xB3B6)),  # HANGUL SYLLABLE TIKEUT YE RIEULPHIEUPH
    ((0x89, 0x8A), (0xB3B7)),  # HANGUL SYLLABLE TIKEUT YE RIEULHIEUH
    ((0x89, 0x8B), (0xB3B8)),  # HANGUL SYLLABLE TIKEUT YE MIEUM
    ((0x89, 0x8C), (0xB3B9)),  # HANGUL SYLLABLE TIKEUT YE PIEUP
    ((0x89, 0x8D), (0xB3BA)),  # HANGUL SYLLABLE TIKEUT YE PIEUPSIOS
    ((0x89, 0x8E), (0xB3BB)),  # HANGUL SYLLABLE TIKEUT YE SIOS
    ((0x89, 0x8F), (0xB3BC)),  # HANGUL SYLLABLE TIKEUT YE SSANGSIOS
    ((0x89, 0x90), (0xB3BD)),  # HANGUL SYLLABLE TIKEUT YE IEUNG
    ((0x89, 0x91), (0xB3BE)),  # HANGUL SYLLABLE TIKEUT YE CIEUC
    ((0x89, 0x92), (0xB3BF)),  # HANGUL SYLLABLE TIKEUT YE CHIEUCH
    ((0x89, 0x93), (0xB3C0)),  # HANGUL SYLLABLE TIKEUT YE KHIEUKH
    ((0x89, 0x94), (0xB3C1)),  # HANGUL SYLLABLE TIKEUT YE THIEUTH
    ((0x89, 0x95), (0xB3C2)),  # HANGUL SYLLABLE TIKEUT YE PHIEUPH
    ((0x89, 0x96), (0xB3C3)),  # HANGUL SYLLABLE TIKEUT YE HIEUH
    ((0x89, 0x97), (0xB3C6)),  # HANGUL SYLLABLE TIKEUT O SSANGKIYEOK
    ((0x89, 0x98), (0xB3C7)),  # HANGUL SYLLABLE TIKEUT O KIYEOKSIOS
    ((0x89, 0x99), (0xB3C9)),  # HANGUL SYLLABLE TIKEUT O NIEUNCIEUC
    ((0x89, 0x9A), (0xB3CA)),  # HANGUL SYLLABLE TIKEUT O NIEUNHIEUH
    ((0x89, 0x9B), (0xB3CD)),  # HANGUL SYLLABLE TIKEUT O RIEULKIYEOK
    ((0x89, 0x9C), (0xB3CF)),  # HANGUL SYLLABLE TIKEUT O RIEULPIEUP
    ((0x89, 0x9D), (0xB3D1)),  # HANGUL SYLLABLE TIKEUT O RIEULTHIEUTH
    ((0x89, 0x9E), (0xB3D2)),  # HANGUL SYLLABLE TIKEUT O RIEULPHIEUPH
    ((0x89, 0x9F), (0xB3D3)),  # HANGUL SYLLABLE TIKEUT O RIEULHIEUH
    ((0x89, 0xA0), (0xB3D6)),  # HANGUL SYLLABLE TIKEUT O PIEUPSIOS
    ((0x89, 0xA1), (0xB3D8)),  # HANGUL SYLLABLE TIKEUT O SSANGSIOS
    ((0x89, 0xA2), (0xB3DA)),  # HANGUL SYLLABLE TIKEUT O CIEUC
    ((0x89, 0xA3), (0xB3DC)),  # HANGUL SYLLABLE TIKEUT O KHIEUKH
    ((0x89, 0xA4), (0xB3DE)),  # HANGUL SYLLABLE TIKEUT O PHIEUPH
    ((0x89, 0xA5), (0xB3DF)),  # HANGUL SYLLABLE TIKEUT O HIEUH
    ((0x89, 0xA6), (0xB3E1)),  # HANGUL SYLLABLE TIKEUT WA KIYEOK
    ((0x89, 0xA7), (0xB3E2)),  # HANGUL SYLLABLE TIKEUT WA SSANGKIYEOK
    ((0x89, 0xA8), (0xB3E3)),  # HANGUL SYLLABLE TIKEUT WA KIYEOKSIOS
    ((0x89, 0xA9), (0xB3E5)),  # HANGUL SYLLABLE TIKEUT WA NIEUNCIEUC
    ((0x89, 0xAA), (0xB3E6)),  # HANGUL SYLLABLE TIKEUT WA NIEUNHIEUH
    ((0x89, 0xAB), (0xB3E7)),  # HANGUL SYLLABLE TIKEUT WA TIKEUT
    ((0x89, 0xAC), (0xB3E9)),  # HANGUL SYLLABLE TIKEUT WA RIEULKIYEOK
    ((0x89, 0xAD), (0xB3EA)),  # HANGUL SYLLABLE TIKEUT WA RIEULMIEUM
    ((0x89, 0xAE), (0xB3EB)),  # HANGUL SYLLABLE TIKEUT WA RIEULPIEUP
    ((0x89, 0xAF), (0xB3EC)),  # HANGUL SYLLABLE TIKEUT WA RIEULSIOS
    ((0x89, 0xB0), (0xB3ED)),  # HANGUL SYLLABLE TIKEUT WA RIEULTHIEUTH
    ((0x89, 0xB1), (0xB3EE)),  # HANGUL SYLLABLE TIKEUT WA RIEULPHIEUPH
    ((0x89, 0xB2), (0xB3EF)),  # HANGUL SYLLABLE TIKEUT WA RIEULHIEUH
    ((0x89, 0xB3), (0xB3F0)),  # HANGUL SYLLABLE TIKEUT WA MIEUM
    ((0x89, 0xB4), (0xB3F1)),  # HANGUL SYLLABLE TIKEUT WA PIEUP
    ((0x89, 0xB5), (0xB3F2)),  # HANGUL SYLLABLE TIKEUT WA PIEUPSIOS
    ((0x89, 0xB6), (0xB3F3)),  # HANGUL SYLLABLE TIKEUT WA SIOS
    ((0x89, 0xB7), (0xB3F4)),  # HANGUL SYLLABLE TIKEUT WA SSANGSIOS
    ((0x89, 0xB8), (0xB3F5)),  # HANGUL SYLLABLE TIKEUT WA IEUNG
    ((0x89, 0xB9), (0xB3F6)),  # HANGUL SYLLABLE TIKEUT WA CIEUC
    ((0x89, 0xBA), (0xB3F7)),  # HANGUL SYLLABLE TIKEUT WA CHIEUCH
    ((0x89, 0xBB), (0xB3F8)),  # HANGUL SYLLABLE TIKEUT WA KHIEUKH
    ((0x89, 0xBC), (0xB3F9)),  # HANGUL SYLLABLE TIKEUT WA THIEUTH
    ((0x89, 0xBD), (0xB3FA)),  # HANGUL SYLLABLE TIKEUT WA PHIEUPH
    ((0x89, 0xBE), (0xB3FB)),  # HANGUL SYLLABLE TIKEUT WA HIEUH
    ((0x89, 0xBF), (0xB3FD)),  # HANGUL SYLLABLE TIKEUT WAE KIYEOK
    ((0x89, 0xC0), (0xB3FE)),  # HANGUL SYLLABLE TIKEUT WAE SSANGKIYEOK
    ((0x89, 0xC1), (0xB3FF)),  # HANGUL SYLLABLE TIKEUT WAE KIYEOKSIOS
    ((0x89, 0xC2), (0xB400)),  # HANGUL SYLLABLE TIKEUT WAE NIEUN
    ((0x89, 0xC3), (0xB401)),  # HANGUL SYLLABLE TIKEUT WAE NIEUNCIEUC
    ((0x89, 0xC4), (0xB402)),  # HANGUL SYLLABLE TIKEUT WAE NIEUNHIEUH
    ((0x89, 0xC5), (0xB403)),  # HANGUL SYLLABLE TIKEUT WAE TIKEUT
    ((0x89, 0xC6), (0xB404)),  # HANGUL SYLLABLE TIKEUT WAE RIEUL
    ((0x89, 0xC7), (0xB405)),  # HANGUL SYLLABLE TIKEUT WAE RIEULKIYEOK
    ((0x89, 0xC8), (0xB406)),  # HANGUL SYLLABLE TIKEUT WAE RIEULMIEUM
    ((0x89, 0xC9), (0xB407)),  # HANGUL SYLLABLE TIKEUT WAE RIEULPIEUP
    ((0x89, 0xCA), (0xB408)),  # HANGUL SYLLABLE TIKEUT WAE RIEULSIOS
    ((0x89, 0xCB), (0xB409)),  # HANGUL SYLLABLE TIKEUT WAE RIEULTHIEUTH
    ((0x89, 0xCC), (0xB40A)),  # HANGUL SYLLABLE TIKEUT WAE RIEULPHIEUPH
    ((0x89, 0xCD), (0xB40B)),  # HANGUL SYLLABLE TIKEUT WAE RIEULHIEUH
    ((0x89, 0xCE), (0xB40C)),  # HANGUL SYLLABLE TIKEUT WAE MIEUM
    ((0x89, 0xCF), (0xB40D)),  # HANGUL SYLLABLE TIKEUT WAE PIEUP
    ((0x89, 0xD0), (0xB40E)),  # HANGUL SYLLABLE TIKEUT WAE PIEUPSIOS
    ((0x89, 0xD1), (0xB40F)),  # HANGUL SYLLABLE TIKEUT WAE SIOS
    ((0x89, 0xD2), (0xB411)),  # HANGUL SYLLABLE TIKEUT WAE IEUNG
    ((0x89, 0xD3), (0xB412)),  # HANGUL SYLLABLE TIKEUT WAE CIEUC
    ((0x89, 0xD4), (0xB413)),  # HANGUL SYLLABLE TIKEUT WAE CHIEUCH
    ((0x89, 0xD5), (0xB414)),  # HANGUL SYLLABLE TIKEUT WAE KHIEUKH
    ((0x89, 0xD6), (0xB415)),  # HANGUL SYLLABLE TIKEUT WAE THIEUTH
    ((0x89, 0xD7), (0xB416)),  # HANGUL SYLLABLE TIKEUT WAE PHIEUPH
    ((0x89, 0xD8), (0xB417)),  # HANGUL SYLLABLE TIKEUT WAE HIEUH
    ((0x89, 0xD9), (0xB419)),  # HANGUL SYLLABLE TIKEUT OE KIYEOK
    ((0x89, 0xDA), (0xB41A)),  # HANGUL SYLLABLE TIKEUT OE SSANGKIYEOK
    ((0x89, 0xDB), (0xB41B)),  # HANGUL SYLLABLE TIKEUT OE KIYEOKSIOS
    ((0x89, 0xDC), (0xB41D)),  # HANGUL SYLLABLE TIKEUT OE NIEUNCIEUC
    ((0x89, 0xDD), (0xB41E)),  # HANGUL SYLLABLE TIKEUT OE NIEUNHIEUH
    ((0x89, 0xDE), (0xB41F)),  # HANGUL SYLLABLE TIKEUT OE TIKEUT
    ((0x89, 0xDF), (0xB421)),  # HANGUL SYLLABLE TIKEUT OE RIEULKIYEOK
    ((0x89, 0xE0), (0xB422)),  # HANGUL SYLLABLE TIKEUT OE RIEULMIEUM
    ((0x89, 0xE1), (0xB423)),  # HANGUL SYLLABLE TIKEUT OE RIEULPIEUP
    ((0x89, 0xE2), (0xB424)),  # HANGUL SYLLABLE TIKEUT OE RIEULSIOS
    ((0x89, 0xE3), (0xB425)),  # HANGUL SYLLABLE TIKEUT OE RIEULTHIEUTH
    ((0x89, 0xE4), (0xB426)),  # HANGUL SYLLABLE TIKEUT OE RIEULPHIEUPH
    ((0x89, 0xE5), (0xB427)),  # HANGUL SYLLABLE TIKEUT OE RIEULHIEUH
    ((0x89, 0xE6), (0xB42A)),  # HANGUL SYLLABLE TIKEUT OE PIEUPSIOS
    ((0x89, 0xE7), (0xB42C)),  # HANGUL SYLLABLE TIKEUT OE SSANGSIOS
    ((0x89, 0xE8), (0xB42D)),  # HANGUL SYLLABLE TIKEUT OE IEUNG
    ((0x89, 0xE9), (0xB42E)),  # HANGUL SYLLABLE TIKEUT OE CIEUC
    ((0x89, 0xEA), (0xB42F)),  # HANGUL SYLLABLE TIKEUT OE CHIEUCH
    ((0x89, 0xEB), (0xB430)),  # HANGUL SYLLABLE TIKEUT OE KHIEUKH
    ((0x89, 0xEC), (0xB431)),  # HANGUL SYLLABLE TIKEUT OE THIEUTH
    ((0x89, 0xED), (0xB432)),  # HANGUL SYLLABLE TIKEUT OE PHIEUPH
    ((0x89, 0xEE), (0xB433)),  # HANGUL SYLLABLE TIKEUT OE HIEUH
    ((0x89, 0xEF), (0xB435)),  # HANGUL SYLLABLE TIKEUT YO KIYEOK
    ((0x89, 0xF0), (0xB436)),  # HANGUL SYLLABLE TIKEUT YO SSANGKIYEOK
    ((0x89, 0xF1), (0xB437)),  # HANGUL SYLLABLE TIKEUT YO KIYEOKSIOS
    ((0x89, 0xF2), (0xB438)),  # HANGUL SYLLABLE TIKEUT YO NIEUN
    ((0x89, 0xF3), (0xB439)),  # HANGUL SYLLABLE TIKEUT YO NIEUNCIEUC
    ((0x89, 0xF4), (0xB43A)),  # HANGUL SYLLABLE TIKEUT YO NIEUNHIEUH
    ((0x89, 0xF5), (0xB43B)),  # HANGUL SYLLABLE TIKEUT YO TIKEUT
    ((0x89, 0xF6), (0xB43C)),  # HANGUL SYLLABLE TIKEUT YO RIEUL
    ((0x89, 0xF7), (0xB43D)),  # HANGUL SYLLABLE TIKEUT YO RIEULKIYEOK
    ((0x89, 0xF8), (0xB43E)),  # HANGUL SYLLABLE TIKEUT YO RIEULMIEUM
    ((0x89, 0xF9), (0xB43F)),  # HANGUL SYLLABLE TIKEUT YO RIEULPIEUP
    ((0x89, 0xFA), (0xB440)),  # HANGUL SYLLABLE TIKEUT YO RIEULSIOS
    ((0x89, 0xFB), (0xB441)),  # HANGUL SYLLABLE TIKEUT YO RIEULTHIEUTH
    ((0x89, 0xFC), (0xB442)),  # HANGUL SYLLABLE TIKEUT YO RIEULPHIEUPH
    ((0x89, 0xFD), (0xB443)),  # HANGUL SYLLABLE TIKEUT YO RIEULHIEUH
    ((0x89, 0xFE), (0xB444)),  # HANGUL SYLLABLE TIKEUT YO MIEUM
    ((0x8A, 0x41), (0xB445)),  # HANGUL SYLLABLE TIKEUT YO PIEUP
    ((0x8A, 0x42), (0xB446)),  # HANGUL SYLLABLE TIKEUT YO PIEUPSIOS
    ((0x8A, 0x43), (0xB447)),  # HANGUL SYLLABLE TIKEUT YO SIOS
    ((0x8A, 0x44), (0xB448)),  # HANGUL SYLLABLE TIKEUT YO SSANGSIOS
    ((0x8A, 0x45), (0xB449)),  # HANGUL SYLLABLE TIKEUT YO IEUNG
    ((0x8A, 0x46), (0xB44A)),  # HANGUL SYLLABLE TIKEUT YO CIEUC
    ((0x8A, 0x47), (0xB44B)),  # HANGUL SYLLABLE TIKEUT YO CHIEUCH
    ((0x8A, 0x48), (0xB44C)),  # HANGUL SYLLABLE TIKEUT YO KHIEUKH
    ((0x8A, 0x49), (0xB44D)),  # HANGUL SYLLABLE TIKEUT YO THIEUTH
    ((0x8A, 0x4A), (0xB44E)),  # HANGUL SYLLABLE TIKEUT YO PHIEUPH
    ((0x8A, 0x4B), (0xB44F)),  # HANGUL SYLLABLE TIKEUT YO HIEUH
    ((0x8A, 0x4C), (0xB452)),  # HANGUL SYLLABLE TIKEUT U SSANGKIYEOK
    ((0x8A, 0x4D), (0xB453)),  # HANGUL SYLLABLE TIKEUT U KIYEOKSIOS
    ((0x8A, 0x4E), (0xB455)),  # HANGUL SYLLABLE TIKEUT U NIEUNCIEUC
    ((0x8A, 0x4F), (0xB456)),  # HANGUL SYLLABLE TIKEUT U NIEUNHIEUH
    ((0x8A, 0x50), (0xB457)),  # HANGUL SYLLABLE TIKEUT U TIKEUT
    ((0x8A, 0x51), (0xB459)),  # HANGUL SYLLABLE TIKEUT U RIEULKIYEOK
    ((0x8A, 0x52), (0xB45A)),  # HANGUL SYLLABLE TIKEUT U RIEULMIEUM
    ((0x8A, 0x53), (0xB45B)),  # HANGUL SYLLABLE TIKEUT U RIEULPIEUP
    ((0x8A, 0x54), (0xB45C)),  # HANGUL SYLLABLE TIKEUT U RIEULSIOS
    ((0x8A, 0x55), (0xB45D)),  # HANGUL SYLLABLE TIKEUT U RIEULTHIEUTH
    ((0x8A, 0x56), (0xB45E)),  # HANGUL SYLLABLE TIKEUT U RIEULPHIEUPH
    ((0x8A, 0x57), (0xB45F)),  # HANGUL SYLLABLE TIKEUT U RIEULHIEUH
    ((0x8A, 0x58), (0xB462)),  # HANGUL SYLLABLE TIKEUT U PIEUPSIOS
    ((0x8A, 0x59), (0xB464)),  # HANGUL SYLLABLE TIKEUT U SSANGSIOS
    ((0x8A, 0x5A), (0xB466)),  # HANGUL SYLLABLE TIKEUT U CIEUC
    ((0x8A, 0x61), (0xB467)),  # HANGUL SYLLABLE TIKEUT U CHIEUCH
    ((0x8A, 0x62), (0xB468)),  # HANGUL SYLLABLE TIKEUT U KHIEUKH
    ((0x8A, 0x63), (0xB469)),  # HANGUL SYLLABLE TIKEUT U THIEUTH
    ((0x8A, 0x64), (0xB46A)),  # HANGUL SYLLABLE TIKEUT U PHIEUPH
    ((0x8A, 0x65), (0xB46B)),  # HANGUL SYLLABLE TIKEUT U HIEUH
    ((0x8A, 0x66), (0xB46D)),  # HANGUL SYLLABLE TIKEUT WEO KIYEOK
    ((0x8A, 0x67), (0xB46E)),  # HANGUL SYLLABLE TIKEUT WEO SSANGKIYEOK
    ((0x8A, 0x68), (0xB46F)),  # HANGUL SYLLABLE TIKEUT WEO KIYEOKSIOS
    ((0x8A, 0x69), (0xB470)),  # HANGUL SYLLABLE TIKEUT WEO NIEUN
    ((0x8A, 0x6A), (0xB471)),  # HANGUL SYLLABLE TIKEUT WEO NIEUNCIEUC
    ((0x8A, 0x6B), (0xB472)),  # HANGUL SYLLABLE TIKEUT WEO NIEUNHIEUH
    ((0x8A, 0x6C), (0xB473)),  # HANGUL SYLLABLE TIKEUT WEO TIKEUT
    ((0x8A, 0x6D), (0xB474)),  # HANGUL SYLLABLE TIKEUT WEO RIEUL
    ((0x8A, 0x6E), (0xB475)),  # HANGUL SYLLABLE TIKEUT WEO RIEULKIYEOK
    ((0x8A, 0x6F), (0xB476)),  # HANGUL SYLLABLE TIKEUT WEO RIEULMIEUM
    ((0x8A, 0x70), (0xB477)),  # HANGUL SYLLABLE TIKEUT WEO RIEULPIEUP
    ((0x8A, 0x71), (0xB478)),  # HANGUL SYLLABLE TIKEUT WEO RIEULSIOS
    ((0x8A, 0x72), (0xB479)),  # HANGUL SYLLABLE TIKEUT WEO RIEULTHIEUTH
    ((0x8A, 0x73), (0xB47A)),  # HANGUL SYLLABLE TIKEUT WEO RIEULPHIEUPH
    ((0x8A, 0x74), (0xB47B)),  # HANGUL SYLLABLE TIKEUT WEO RIEULHIEUH
    ((0x8A, 0x75), (0xB47C)),  # HANGUL SYLLABLE TIKEUT WEO MIEUM
    ((0x8A, 0x76), (0xB47D)),  # HANGUL SYLLABLE TIKEUT WEO PIEUP
    ((0x8A, 0x77), (0xB47E)),  # HANGUL SYLLABLE TIKEUT WEO PIEUPSIOS
    ((0x8A, 0x78), (0xB47F)),  # HANGUL SYLLABLE TIKEUT WEO SIOS
    ((0x8A, 0x79), (0xB481)),  # HANGUL SYLLABLE TIKEUT WEO IEUNG
    ((0x8A, 0x7A), (0xB482)),  # HANGUL SYLLABLE TIKEUT WEO CIEUC
    ((0x8A, 0x81), (0xB483)),  # HANGUL SYLLABLE TIKEUT WEO CHIEUCH
    ((0x8A, 0x82), (0xB484)),  # HANGUL SYLLABLE TIKEUT WEO KHIEUKH
    ((0x8A, 0x83), (0xB485)),  # HANGUL SYLLABLE TIKEUT WEO THIEUTH
    ((0x8A, 0x84), (0xB486)),  # HANGUL SYLLABLE TIKEUT WEO PHIEUPH
    ((0x8A, 0x85), (0xB487)),  # HANGUL SYLLABLE TIKEUT WEO HIEUH
    ((0x8A, 0x86), (0xB489)),  # HANGUL SYLLABLE TIKEUT WE KIYEOK
    ((0x8A, 0x87), (0xB48A)),  # HANGUL SYLLABLE TIKEUT WE SSANGKIYEOK
    ((0x8A, 0x88), (0xB48B)),  # HANGUL SYLLABLE TIKEUT WE KIYEOKSIOS
    ((0x8A, 0x89), (0xB48C)),  # HANGUL SYLLABLE TIKEUT WE NIEUN
    ((0x8A, 0x8A), (0xB48D)),  # HANGUL SYLLABLE TIKEUT WE NIEUNCIEUC
    ((0x8A, 0x8B), (0xB48E)),  # HANGUL SYLLABLE TIKEUT WE NIEUNHIEUH
    ((0x8A, 0x8C), (0xB48F)),  # HANGUL SYLLABLE TIKEUT WE TIKEUT
    ((0x8A, 0x8D), (0xB490)),  # HANGUL SYLLABLE TIKEUT WE RIEUL
    ((0x8A, 0x8E), (0xB491)),  # HANGUL SYLLABLE TIKEUT WE RIEULKIYEOK
    ((0x8A, 0x8F), (0xB492)),  # HANGUL SYLLABLE TIKEUT WE RIEULMIEUM
    ((0x8A, 0x90), (0xB493)),  # HANGUL SYLLABLE TIKEUT WE RIEULPIEUP
    ((0x8A, 0x91), (0xB494)),  # HANGUL SYLLABLE TIKEUT WE RIEULSIOS
    ((0x8A, 0x92), (0xB495)),  # HANGUL SYLLABLE TIKEUT WE RIEULTHIEUTH
    ((0x8A, 0x93), (0xB496)),  # HANGUL SYLLABLE TIKEUT WE RIEULPHIEUPH
    ((0x8A, 0x94), (0xB497)),  # HANGUL SYLLABLE TIKEUT WE RIEULHIEUH
    ((0x8A, 0x95), (0xB498)),  # HANGUL SYLLABLE TIKEUT WE MIEUM
    ((0x8A, 0x96), (0xB499)),  # HANGUL SYLLABLE TIKEUT WE PIEUP
    ((0x8A, 0x97), (0xB49A)),  # HANGUL SYLLABLE TIKEUT WE PIEUPSIOS
    ((0x8A, 0x98), (0xB49B)),  # HANGUL SYLLABLE TIKEUT WE SIOS
    ((0x8A, 0x99), (0xB49C)),  # HANGUL SYLLABLE TIKEUT WE SSANGSIOS
    ((0x8A, 0x9A), (0xB49E)),  # HANGUL SYLLABLE TIKEUT WE CIEUC
    ((0x8A, 0x9B), (0xB49F)),  # HANGUL SYLLABLE TIKEUT WE CHIEUCH
    ((0x8A, 0x9C), (0xB4A0)),  # HANGUL SYLLABLE TIKEUT WE KHIEUKH
    ((0x8A, 0x9D), (0xB4A1)),  # HANGUL SYLLABLE TIKEUT WE THIEUTH
    ((0x8A, 0x9E), (0xB4A2)),  # HANGUL SYLLABLE TIKEUT WE PHIEUPH
    ((0x8A, 0x9F), (0xB4A3)),  # HANGUL SYLLABLE TIKEUT WE HIEUH
    ((0x8A, 0xA0), (0xB4A5)),  # HANGUL SYLLABLE TIKEUT WI KIYEOK
    ((0x8A, 0xA1), (0xB4A6)),  # HANGUL SYLLABLE TIKEUT WI SSANGKIYEOK
    ((0x8A, 0xA2), (0xB4A7)),  # HANGUL SYLLABLE TIKEUT WI KIYEOKSIOS
    ((0x8A, 0xA3), (0xB4A9)),  # HANGUL SYLLABLE TIKEUT WI NIEUNCIEUC
    ((0x8A, 0xA4), (0xB4AA)),  # HANGUL SYLLABLE TIKEUT WI NIEUNHIEUH
    ((0x8A, 0xA5), (0xB4AB)),  # HANGUL SYLLABLE TIKEUT WI TIKEUT
    ((0x8A, 0xA6), (0xB4AD)),  # HANGUL SYLLABLE TIKEUT WI RIEULKIYEOK
    ((0x8A, 0xA7), (0xB4AE)),  # HANGUL SYLLABLE TIKEUT WI RIEULMIEUM
    ((0x8A, 0xA8), (0xB4AF)),  # HANGUL SYLLABLE TIKEUT WI RIEULPIEUP
    ((0x8A, 0xA9), (0xB4B0)),  # HANGUL SYLLABLE TIKEUT WI RIEULSIOS
    ((0x8A, 0xAA), (0xB4B1)),  # HANGUL SYLLABLE TIKEUT WI RIEULTHIEUTH
    ((0x8A, 0xAB), (0xB4B2)),  # HANGUL SYLLABLE TIKEUT WI RIEULPHIEUPH
    ((0x8A, 0xAC), (0xB4B3)),  # HANGUL SYLLABLE TIKEUT WI RIEULHIEUH
    ((0x8A, 0xAD), (0xB4B4)),  # HANGUL SYLLABLE TIKEUT WI MIEUM
    ((0x8A, 0xAE), (0xB4B6)),  # HANGUL SYLLABLE TIKEUT WI PIEUPSIOS
    ((0x8A, 0xAF), (0xB4B8)),  # HANGUL SYLLABLE TIKEUT WI SSANGSIOS
    ((0x8A, 0xB0), (0xB4BA)),  # HANGUL SYLLABLE TIKEUT WI CIEUC
    ((0x8A, 0xB1), (0xB4BB)),  # HANGUL SYLLABLE TIKEUT WI CHIEUCH
    ((0x8A, 0xB2), (0xB4BC)),  # HANGUL SYLLABLE TIKEUT WI KHIEUKH
    ((0x8A, 0xB3), (0xB4BD)),  # HANGUL SYLLABLE TIKEUT WI THIEUTH
    ((0x8A, 0xB4), (0xB4BE)),  # HANGUL SYLLABLE TIKEUT WI PHIEUPH
    ((0x8A, 0xB5), (0xB4BF)),  # HANGUL SYLLABLE TIKEUT WI HIEUH
    ((0x8A, 0xB6), (0xB4C1)),  # HANGUL SYLLABLE TIKEUT YU KIYEOK
    ((0x8A, 0xB7), (0xB4C2)),  # HANGUL SYLLABLE TIKEUT YU SSANGKIYEOK
    ((0x8A, 0xB8), (0xB4C3)),  # HANGUL SYLLABLE TIKEUT YU KIYEOKSIOS
    ((0x8A, 0xB9), (0xB4C5)),  # HANGUL SYLLABLE TIKEUT YU NIEUNCIEUC
    ((0x8A, 0xBA), (0xB4C6)),  # HANGUL SYLLABLE TIKEUT YU NIEUNHIEUH
    ((0x8A, 0xBB), (0xB4C7)),  # HANGUL SYLLABLE TIKEUT YU TIKEUT
    ((0x8A, 0xBC), (0xB4C9)),  # HANGUL SYLLABLE TIKEUT YU RIEULKIYEOK
    ((0x8A, 0xBD), (0xB4CA)),  # HANGUL SYLLABLE TIKEUT YU RIEULMIEUM
    ((0x8A, 0xBE), (0xB4CB)),  # HANGUL SYLLABLE TIKEUT YU RIEULPIEUP
    ((0x8A, 0xBF), (0xB4CC)),  # HANGUL SYLLABLE TIKEUT YU RIEULSIOS
    ((0x8A, 0xC0), (0xB4CD)),  # HANGUL SYLLABLE TIKEUT YU RIEULTHIEUTH
    ((0x8A, 0xC1), (0xB4CE)),  # HANGUL SYLLABLE TIKEUT YU RIEULPHIEUPH
    ((0x8A, 0xC2), (0xB4CF)),  # HANGUL SYLLABLE TIKEUT YU RIEULHIEUH
    ((0x8A, 0xC3), (0xB4D1)),  # HANGUL SYLLABLE TIKEUT YU PIEUP
    ((0x8A, 0xC4), (0xB4D2)),  # HANGUL SYLLABLE TIKEUT YU PIEUPSIOS
    ((0x8A, 0xC5), (0xB4D3)),  # HANGUL SYLLABLE TIKEUT YU SIOS
    ((0x8A, 0xC6), (0xB4D4)),  # HANGUL SYLLABLE TIKEUT YU SSANGSIOS
    ((0x8A, 0xC7), (0xB4D6)),  # HANGUL SYLLABLE TIKEUT YU CIEUC
    ((0x8A, 0xC8), (0xB4D7)),  # HANGUL SYLLABLE TIKEUT YU CHIEUCH
    ((0x8A, 0xC9), (0xB4D8)),  # HANGUL SYLLABLE TIKEUT YU KHIEUKH
    ((0x8A, 0xCA), (0xB4D9)),  # HANGUL SYLLABLE TIKEUT YU THIEUTH
    ((0x8A, 0xCB), (0xB4DA)),  # HANGUL SYLLABLE TIKEUT YU PHIEUPH
    ((0x8A, 0xCC), (0xB4DB)),  # HANGUL SYLLABLE TIKEUT YU HIEUH
    ((0x8A, 0xCD), (0xB4DE)),  # HANGUL SYLLABLE TIKEUT EU SSANGKIYEOK
    ((0x8A, 0xCE), (0xB4DF)),  # HANGUL SYLLABLE TIKEUT EU KIYEOKSIOS
    ((0x8A, 0xCF), (0xB4E1)),  # HANGUL SYLLABLE TIKEUT EU NIEUNCIEUC
    ((0x8A, 0xD0), (0xB4E2)),  # HANGUL SYLLABLE TIKEUT EU NIEUNHIEUH
    ((0x8A, 0xD1), (0xB4E5)),  # HANGUL SYLLABLE TIKEUT EU RIEULKIYEOK
    ((0x8A, 0xD2), (0xB4E7)),  # HANGUL SYLLABLE TIKEUT EU RIEULPIEUP
    ((0x8A, 0xD3), (0xB4E8)),  # HANGUL SYLLABLE TIKEUT EU RIEULSIOS
    ((0x8A, 0xD4), (0xB4E9)),  # HANGUL SYLLABLE TIKEUT EU RIEULTHIEUTH
    ((0x8A, 0xD5), (0xB4EA)),  # HANGUL SYLLABLE TIKEUT EU RIEULPHIEUPH
    ((0x8A, 0xD6), (0xB4EB)),  # HANGUL SYLLABLE TIKEUT EU RIEULHIEUH
    ((0x8A, 0xD7), (0xB4EE)),  # HANGUL SYLLABLE TIKEUT EU PIEUPSIOS
    ((0x8A, 0xD8), (0xB4F0)),  # HANGUL SYLLABLE TIKEUT EU SSANGSIOS
    ((0x8A, 0xD9), (0xB4F2)),  # HANGUL SYLLABLE TIKEUT EU CIEUC
    ((0x8A, 0xDA), (0xB4F3)),  # HANGUL SYLLABLE TIKEUT EU CHIEUCH
    ((0x8A, 0xDB), (0xB4F4)),  # HANGUL SYLLABLE TIKEUT EU KHIEUKH
    ((0x8A, 0xDC), (0xB4F5)),  # HANGUL SYLLABLE TIKEUT EU THIEUTH
    ((0x8A, 0xDD), (0xB4F6)),  # HANGUL SYLLABLE TIKEUT EU PHIEUPH
    ((0x8A, 0xDE), (0xB4F7)),  # HANGUL SYLLABLE TIKEUT EU HIEUH
    ((0x8A, 0xDF), (0xB4F9)),  # HANGUL SYLLABLE TIKEUT YI KIYEOK
    ((0x8A, 0xE0), (0xB4FA)),  # HANGUL SYLLABLE TIKEUT YI SSANGKIYEOK
    ((0x8A, 0xE1), (0xB4FB)),  # HANGUL SYLLABLE TIKEUT YI KIYEOKSIOS
    ((0x8A, 0xE2), (0xB4FC)),  # HANGUL SYLLABLE TIKEUT YI NIEUN
    ((0x8A, 0xE3), (0xB4FD)),  # HANGUL SYLLABLE TIKEUT YI NIEUNCIEUC
    ((0x8A, 0xE4), (0xB4FE)),  # HANGUL SYLLABLE TIKEUT YI NIEUNHIEUH
    ((0x8A, 0xE5), (0xB4FF)),  # HANGUL SYLLABLE TIKEUT YI TIKEUT
    ((0x8A, 0xE6), (0xB500)),  # HANGUL SYLLABLE TIKEUT YI RIEUL
    ((0x8A, 0xE7), (0xB501)),  # HANGUL SYLLABLE TIKEUT YI RIEULKIYEOK
    ((0x8A, 0xE8), (0xB502)),  # HANGUL SYLLABLE TIKEUT YI RIEULMIEUM
    ((0x8A, 0xE9), (0xB503)),  # HANGUL SYLLABLE TIKEUT YI RIEULPIEUP
    ((0x8A, 0xEA), (0xB504)),  # HANGUL SYLLABLE TIKEUT YI RIEULSIOS
    ((0x8A, 0xEB), (0xB505)),  # HANGUL SYLLABLE TIKEUT YI RIEULTHIEUTH
    ((0x8A, 0xEC), (0xB506)),  # HANGUL SYLLABLE TIKEUT YI RIEULPHIEUPH
    ((0x8A, 0xED), (0xB507)),  # HANGUL SYLLABLE TIKEUT YI RIEULHIEUH
    ((0x8A, 0xEE), (0xB508)),  # HANGUL SYLLABLE TIKEUT YI MIEUM
    ((0x8A, 0xEF), (0xB509)),  # HANGUL SYLLABLE TIKEUT YI PIEUP
    ((0x8A, 0xF0), (0xB50A)),  # HANGUL SYLLABLE TIKEUT YI PIEUPSIOS
    ((0x8A, 0xF1), (0xB50B)),  # HANGUL SYLLABLE TIKEUT YI SIOS
    ((0x8A, 0xF2), (0xB50C)),  # HANGUL SYLLABLE TIKEUT YI SSANGSIOS
    ((0x8A, 0xF3), (0xB50D)),  # HANGUL SYLLABLE TIKEUT YI IEUNG
    ((0x8A, 0xF4), (0xB50E)),  # HANGUL SYLLABLE TIKEUT YI CIEUC
    ((0x8A, 0xF5), (0xB50F)),  # HANGUL SYLLABLE TIKEUT YI CHIEUCH
    ((0x8A, 0xF6), (0xB510)),  # HANGUL SYLLABLE TIKEUT YI KHIEUKH
    ((0x8A, 0xF7), (0xB511)),  # HANGUL SYLLABLE TIKEUT YI THIEUTH
    ((0x8A, 0xF8), (0xB512)),  # HANGUL SYLLABLE TIKEUT YI PHIEUPH
    ((0x8A, 0xF9), (0xB513)),  # HANGUL SYLLABLE TIKEUT YI HIEUH
    ((0x8A, 0xFA), (0xB516)),  # HANGUL SYLLABLE TIKEUT I SSANGKIYEOK
    ((0x8A, 0xFB), (0xB517)),  # HANGUL SYLLABLE TIKEUT I KIYEOKSIOS
    ((0x8A, 0xFC), (0xB519)),  # HANGUL SYLLABLE TIKEUT I NIEUNCIEUC
    ((0x8A, 0xFD), (0xB51A)),  # HANGUL SYLLABLE TIKEUT I NIEUNHIEUH
    ((0x8A, 0xFE), (0xB51D)),  # HANGUL SYLLABLE TIKEUT I RIEULKIYEOK
    ((0x8B, 0x41), (0xB51E)),  # HANGUL SYLLABLE TIKEUT I RIEULMIEUM
    ((0x8B, 0x42), (0xB51F)),  # HANGUL SYLLABLE TIKEUT I RIEULPIEUP
    ((0x8B, 0x43), (0xB520)),  # HANGUL SYLLABLE TIKEUT I RIEULSIOS
    ((0x8B, 0x44), (0xB521)),  # HANGUL SYLLABLE TIKEUT I RIEULTHIEUTH
    ((0x8B, 0x45), (0xB522)),  # HANGUL SYLLABLE TIKEUT I RIEULPHIEUPH
    ((0x8B, 0x46), (0xB523)),  # HANGUL SYLLABLE TIKEUT I RIEULHIEUH
    ((0x8B, 0x47), (0xB526)),  # HANGUL SYLLABLE TIKEUT I PIEUPSIOS
    ((0x8B, 0x48), (0xB52B)),  # HANGUL SYLLABLE TIKEUT I CHIEUCH
    ((0x8B, 0x49), (0xB52C)),  # HANGUL SYLLABLE TIKEUT I KHIEUKH
    ((0x8B, 0x4A), (0xB52D)),  # HANGUL SYLLABLE TIKEUT I THIEUTH
    ((0x8B, 0x4B), (0xB52E)),  # HANGUL SYLLABLE TIKEUT I PHIEUPH
    ((0x8B, 0x4C), (0xB52F)),  # HANGUL SYLLABLE TIKEUT I HIEUH
    ((0x8B, 0x4D), (0xB532)),  # HANGUL SYLLABLE SSANGTIKEUT A SSANGKIYEOK
    ((0x8B, 0x4E), (0xB533)),  # HANGUL SYLLABLE SSANGTIKEUT A KIYEOKSIOS
    ((0x8B, 0x4F), (0xB535)),  # HANGUL SYLLABLE SSANGTIKEUT A NIEUNCIEUC
    ((0x8B, 0x50), (0xB536)),  # HANGUL SYLLABLE SSANGTIKEUT A NIEUNHIEUH
    ((0x8B, 0x51), (0xB537)),  # HANGUL SYLLABLE SSANGTIKEUT A TIKEUT
    ((0x8B, 0x52), (0xB539)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULKIYEOK
    ((0x8B, 0x53), (0xB53A)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULMIEUM
    ((0x8B, 0x54), (0xB53B)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULPIEUP
    ((0x8B, 0x55), (0xB53C)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULSIOS
    ((0x8B, 0x56), (0xB53D)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULTHIEUTH
    ((0x8B, 0x57), (0xB53E)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULPHIEUPH
    ((0x8B, 0x58), (0xB53F)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEULHIEUH
    ((0x8B, 0x59), (0xB542)),  # HANGUL SYLLABLE SSANGTIKEUT A PIEUPSIOS
    ((0x8B, 0x5A), (0xB546)),  # HANGUL SYLLABLE SSANGTIKEUT A CIEUC
    ((0x8B, 0x61), (0xB547)),  # HANGUL SYLLABLE SSANGTIKEUT A CHIEUCH
    ((0x8B, 0x62), (0xB548)),  # HANGUL SYLLABLE SSANGTIKEUT A KHIEUKH
    ((0x8B, 0x63), (0xB549)),  # HANGUL SYLLABLE SSANGTIKEUT A THIEUTH
    ((0x8B, 0x64), (0xB54A)),  # HANGUL SYLLABLE SSANGTIKEUT A PHIEUPH
    ((0x8B, 0x65), (0xB54E)),  # HANGUL SYLLABLE SSANGTIKEUT AE SSANGKIYEOK
    ((0x8B, 0x66), (0xB54F)),  # HANGUL SYLLABLE SSANGTIKEUT AE KIYEOKSIOS
    ((0x8B, 0x67), (0xB551)),  # HANGUL SYLLABLE SSANGTIKEUT AE NIEUNCIEUC
    ((0x8B, 0x68), (0xB552)),  # HANGUL SYLLABLE SSANGTIKEUT AE NIEUNHIEUH
    ((0x8B, 0x69), (0xB553)),  # HANGUL SYLLABLE SSANGTIKEUT AE TIKEUT
    ((0x8B, 0x6A), (0xB555)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULKIYEOK
    ((0x8B, 0x6B), (0xB556)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULMIEUM
    ((0x8B, 0x6C), (0xB557)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULPIEUP
    ((0x8B, 0x6D), (0xB558)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULSIOS
    ((0x8B, 0x6E), (0xB559)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULTHIEUTH
    ((0x8B, 0x6F), (0xB55A)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULPHIEUPH
    ((0x8B, 0x70), (0xB55B)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEULHIEUH
    ((0x8B, 0x71), (0xB55E)),  # HANGUL SYLLABLE SSANGTIKEUT AE PIEUPSIOS
    ((0x8B, 0x72), (0xB562)),  # HANGUL SYLLABLE SSANGTIKEUT AE CIEUC
    ((0x8B, 0x73), (0xB563)),  # HANGUL SYLLABLE SSANGTIKEUT AE CHIEUCH
    ((0x8B, 0x74), (0xB564)),  # HANGUL SYLLABLE SSANGTIKEUT AE KHIEUKH
    ((0x8B, 0x75), (0xB565)),  # HANGUL SYLLABLE SSANGTIKEUT AE THIEUTH
    ((0x8B, 0x76), (0xB566)),  # HANGUL SYLLABLE SSANGTIKEUT AE PHIEUPH
    ((0x8B, 0x77), (0xB567)),  # HANGUL SYLLABLE SSANGTIKEUT AE HIEUH
    ((0x8B, 0x78), (0xB568)),  # HANGUL SYLLABLE SSANGTIKEUT YA
    ((0x8B, 0x79), (0xB569)),  # HANGUL SYLLABLE SSANGTIKEUT YA KIYEOK
    ((0x8B, 0x7A), (0xB56A)),  # HANGUL SYLLABLE SSANGTIKEUT YA SSANGKIYEOK
    ((0x8B, 0x81), (0xB56B)),  # HANGUL SYLLABLE SSANGTIKEUT YA KIYEOKSIOS
    ((0x8B, 0x82), (0xB56C)),  # HANGUL SYLLABLE SSANGTIKEUT YA NIEUN
    ((0x8B, 0x83), (0xB56D)),  # HANGUL SYLLABLE SSANGTIKEUT YA NIEUNCIEUC
    ((0x8B, 0x84), (0xB56E)),  # HANGUL SYLLABLE SSANGTIKEUT YA NIEUNHIEUH
    ((0x8B, 0x85), (0xB56F)),  # HANGUL SYLLABLE SSANGTIKEUT YA TIKEUT
    ((0x8B, 0x86), (0xB570)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEUL
    ((0x8B, 0x87), (0xB571)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULKIYEOK
    ((0x8B, 0x88), (0xB572)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULMIEUM
    ((0x8B, 0x89), (0xB573)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULPIEUP
    ((0x8B, 0x8A), (0xB574)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULSIOS
    ((0x8B, 0x8B), (0xB575)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULTHIEUTH
    ((0x8B, 0x8C), (0xB576)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULPHIEUPH
    ((0x8B, 0x8D), (0xB577)),  # HANGUL SYLLABLE SSANGTIKEUT YA RIEULHIEUH
    ((0x8B, 0x8E), (0xB578)),  # HANGUL SYLLABLE SSANGTIKEUT YA MIEUM
    ((0x8B, 0x8F), (0xB579)),  # HANGUL SYLLABLE SSANGTIKEUT YA PIEUP
    ((0x8B, 0x90), (0xB57A)),  # HANGUL SYLLABLE SSANGTIKEUT YA PIEUPSIOS
    ((0x8B, 0x91), (0xB57B)),  # HANGUL SYLLABLE SSANGTIKEUT YA SIOS
    ((0x8B, 0x92), (0xB57C)),  # HANGUL SYLLABLE SSANGTIKEUT YA SSANGSIOS
    ((0x8B, 0x93), (0xB57D)),  # HANGUL SYLLABLE SSANGTIKEUT YA IEUNG
    ((0x8B, 0x94), (0xB57E)),  # HANGUL SYLLABLE SSANGTIKEUT YA CIEUC
    ((0x8B, 0x95), (0xB57F)),  # HANGUL SYLLABLE SSANGTIKEUT YA CHIEUCH
    ((0x8B, 0x96), (0xB580)),  # HANGUL SYLLABLE SSANGTIKEUT YA KHIEUKH
    ((0x8B, 0x97), (0xB581)),  # HANGUL SYLLABLE SSANGTIKEUT YA THIEUTH
    ((0x8B, 0x98), (0xB582)),  # HANGUL SYLLABLE SSANGTIKEUT YA PHIEUPH
    ((0x8B, 0x99), (0xB583)),  # HANGUL SYLLABLE SSANGTIKEUT YA HIEUH
    ((0x8B, 0x9A), (0xB584)),  # HANGUL SYLLABLE SSANGTIKEUT YAE
    ((0x8B, 0x9B), (0xB585)),  # HANGUL SYLLABLE SSANGTIKEUT YAE KIYEOK
    ((0x8B, 0x9C), (0xB586)),  # HANGUL SYLLABLE SSANGTIKEUT YAE SSANGKIYEOK
    ((0x8B, 0x9D), (0xB587)),  # HANGUL SYLLABLE SSANGTIKEUT YAE KIYEOKSIOS
    ((0x8B, 0x9E), (0xB588)),  # HANGUL SYLLABLE SSANGTIKEUT YAE NIEUN
    ((0x8B, 0x9F), (0xB589)),  # HANGUL SYLLABLE SSANGTIKEUT YAE NIEUNCIEUC
    ((0x8B, 0xA0), (0xB58A)),  # HANGUL SYLLABLE SSANGTIKEUT YAE NIEUNHIEUH
    ((0x8B, 0xA1), (0xB58B)),  # HANGUL SYLLABLE SSANGTIKEUT YAE TIKEUT
    ((0x8B, 0xA2), (0xB58C)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEUL
    ((0x8B, 0xA3), (0xB58D)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULKIYEOK
    ((0x8B, 0xA4), (0xB58E)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULMIEUM
    ((0x8B, 0xA5), (0xB58F)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULPIEUP
    ((0x8B, 0xA6), (0xB590)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULSIOS
    ((0x8B, 0xA7), (0xB591)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULTHIEUTH
    ((0x8B, 0xA8), (0xB592)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULPHIEUPH
    ((0x8B, 0xA9), (0xB593)),  # HANGUL SYLLABLE SSANGTIKEUT YAE RIEULHIEUH
    ((0x8B, 0xAA), (0xB594)),  # HANGUL SYLLABLE SSANGTIKEUT YAE MIEUM
    ((0x8B, 0xAB), (0xB595)),  # HANGUL SYLLABLE SSANGTIKEUT YAE PIEUP
    ((0x8B, 0xAC), (0xB596)),  # HANGUL SYLLABLE SSANGTIKEUT YAE PIEUPSIOS
    ((0x8B, 0xAD), (0xB597)),  # HANGUL SYLLABLE SSANGTIKEUT YAE SIOS
    ((0x8B, 0xAE), (0xB598)),  # HANGUL SYLLABLE SSANGTIKEUT YAE SSANGSIOS
    ((0x8B, 0xAF), (0xB599)),  # HANGUL SYLLABLE SSANGTIKEUT YAE IEUNG
    ((0x8B, 0xB0), (0xB59A)),  # HANGUL SYLLABLE SSANGTIKEUT YAE CIEUC
    ((0x8B, 0xB1), (0xB59B)),  # HANGUL SYLLABLE SSANGTIKEUT YAE CHIEUCH
    ((0x8B, 0xB2), (0xB59C)),  # HANGUL SYLLABLE SSANGTIKEUT YAE KHIEUKH
    ((0x8B, 0xB3), (0xB59D)),  # HANGUL SYLLABLE SSANGTIKEUT YAE THIEUTH
    ((0x8B, 0xB4), (0xB59E)),  # HANGUL SYLLABLE SSANGTIKEUT YAE PHIEUPH
    ((0x8B, 0xB5), (0xB59F)),  # HANGUL SYLLABLE SSANGTIKEUT YAE HIEUH
    ((0x8B, 0xB6), (0xB5A2)),  # HANGUL SYLLABLE SSANGTIKEUT EO SSANGKIYEOK
    ((0x8B, 0xB7), (0xB5A3)),  # HANGUL SYLLABLE SSANGTIKEUT EO KIYEOKSIOS
    ((0x8B, 0xB8), (0xB5A5)),  # HANGUL SYLLABLE SSANGTIKEUT EO NIEUNCIEUC
    ((0x8B, 0xB9), (0xB5A6)),  # HANGUL SYLLABLE SSANGTIKEUT EO NIEUNHIEUH
    ((0x8B, 0xBA), (0xB5A7)),  # HANGUL SYLLABLE SSANGTIKEUT EO TIKEUT
    ((0x8B, 0xBB), (0xB5A9)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULKIYEOK
    ((0x8B, 0xBC), (0xB5AC)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULSIOS
    ((0x8B, 0xBD), (0xB5AD)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULTHIEUTH
    ((0x8B, 0xBE), (0xB5AE)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULPHIEUPH
    ((0x8B, 0xBF), (0xB5AF)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULHIEUH
    ((0x8B, 0xC0), (0xB5B2)),  # HANGUL SYLLABLE SSANGTIKEUT EO PIEUPSIOS
    ((0x8B, 0xC1), (0xB5B6)),  # HANGUL SYLLABLE SSANGTIKEUT EO CIEUC
    ((0x8B, 0xC2), (0xB5B7)),  # HANGUL SYLLABLE SSANGTIKEUT EO CHIEUCH
    ((0x8B, 0xC3), (0xB5B8)),  # HANGUL SYLLABLE SSANGTIKEUT EO KHIEUKH
    ((0x8B, 0xC4), (0xB5B9)),  # HANGUL SYLLABLE SSANGTIKEUT EO THIEUTH
    ((0x8B, 0xC5), (0xB5BA)),  # HANGUL SYLLABLE SSANGTIKEUT EO PHIEUPH
    ((0x8B, 0xC6), (0xB5BE)),  # HANGUL SYLLABLE SSANGTIKEUT E SSANGKIYEOK
    ((0x8B, 0xC7), (0xB5BF)),  # HANGUL SYLLABLE SSANGTIKEUT E KIYEOKSIOS
    ((0x8B, 0xC8), (0xB5C1)),  # HANGUL SYLLABLE SSANGTIKEUT E NIEUNCIEUC
    ((0x8B, 0xC9), (0xB5C2)),  # HANGUL SYLLABLE SSANGTIKEUT E NIEUNHIEUH
    ((0x8B, 0xCA), (0xB5C3)),  # HANGUL SYLLABLE SSANGTIKEUT E TIKEUT
    ((0x8B, 0xCB), (0xB5C5)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULKIYEOK
    ((0x8B, 0xCC), (0xB5C6)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULMIEUM
    ((0x8B, 0xCD), (0xB5C7)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULPIEUP
    ((0x8B, 0xCE), (0xB5C8)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULSIOS
    ((0x8B, 0xCF), (0xB5C9)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULTHIEUTH
    ((0x8B, 0xD0), (0xB5CA)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULPHIEUPH
    ((0x8B, 0xD1), (0xB5CB)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEULHIEUH
    ((0x8B, 0xD2), (0xB5CE)),  # HANGUL SYLLABLE SSANGTIKEUT E PIEUPSIOS
    ((0x8B, 0xD3), (0xB5D2)),  # HANGUL SYLLABLE SSANGTIKEUT E CIEUC
    ((0x8B, 0xD4), (0xB5D3)),  # HANGUL SYLLABLE SSANGTIKEUT E CHIEUCH
    ((0x8B, 0xD5), (0xB5D4)),  # HANGUL SYLLABLE SSANGTIKEUT E KHIEUKH
    ((0x8B, 0xD6), (0xB5D5)),  # HANGUL SYLLABLE SSANGTIKEUT E THIEUTH
    ((0x8B, 0xD7), (0xB5D6)),  # HANGUL SYLLABLE SSANGTIKEUT E PHIEUPH
    ((0x8B, 0xD8), (0xB5D7)),  # HANGUL SYLLABLE SSANGTIKEUT E HIEUH
    ((0x8B, 0xD9), (0xB5D9)),  # HANGUL SYLLABLE SSANGTIKEUT YEO KIYEOK
    ((0x8B, 0xDA), (0xB5DA)),  # HANGUL SYLLABLE SSANGTIKEUT YEO SSANGKIYEOK
    ((0x8B, 0xDB), (0xB5DB)),  # HANGUL SYLLABLE SSANGTIKEUT YEO KIYEOKSIOS
    ((0x8B, 0xDC), (0xB5DC)),  # HANGUL SYLLABLE SSANGTIKEUT YEO NIEUN
    ((0x8B, 0xDD), (0xB5DD)),  # HANGUL SYLLABLE SSANGTIKEUT YEO NIEUNCIEUC
    ((0x8B, 0xDE), (0xB5DE)),  # HANGUL SYLLABLE SSANGTIKEUT YEO NIEUNHIEUH
    ((0x8B, 0xDF), (0xB5DF)),  # HANGUL SYLLABLE SSANGTIKEUT YEO TIKEUT
    ((0x8B, 0xE0), (0xB5E0)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEUL
    ((0x8B, 0xE1), (0xB5E1)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULKIYEOK
    ((0x8B, 0xE2), (0xB5E2)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULMIEUM
    ((0x8B, 0xE3), (0xB5E3)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULPIEUP
    ((0x8B, 0xE4), (0xB5E4)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULSIOS
    ((0x8B, 0xE5), (0xB5E5)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULTHIEUTH
    ((0x8B, 0xE6), (0xB5E6)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULPHIEUPH
    ((0x8B, 0xE7), (0xB5E7)),  # HANGUL SYLLABLE SSANGTIKEUT YEO RIEULHIEUH
    ((0x8B, 0xE8), (0xB5E8)),  # HANGUL SYLLABLE SSANGTIKEUT YEO MIEUM
    ((0x8B, 0xE9), (0xB5E9)),  # HANGUL SYLLABLE SSANGTIKEUT YEO PIEUP
    ((0x8B, 0xEA), (0xB5EA)),  # HANGUL SYLLABLE SSANGTIKEUT YEO PIEUPSIOS
    ((0x8B, 0xEB), (0xB5EB)),  # HANGUL SYLLABLE SSANGTIKEUT YEO SIOS
    ((0x8B, 0xEC), (0xB5ED)),  # HANGUL SYLLABLE SSANGTIKEUT YEO IEUNG
    ((0x8B, 0xED), (0xB5EE)),  # HANGUL SYLLABLE SSANGTIKEUT YEO CIEUC
    ((0x8B, 0xEE), (0xB5EF)),  # HANGUL SYLLABLE SSANGTIKEUT YEO CHIEUCH
    ((0x8B, 0xEF), (0xB5F0)),  # HANGUL SYLLABLE SSANGTIKEUT YEO KHIEUKH
    ((0x8B, 0xF0), (0xB5F1)),  # HANGUL SYLLABLE SSANGTIKEUT YEO THIEUTH
    ((0x8B, 0xF1), (0xB5F2)),  # HANGUL SYLLABLE SSANGTIKEUT YEO PHIEUPH
    ((0x8B, 0xF2), (0xB5F3)),  # HANGUL SYLLABLE SSANGTIKEUT YEO HIEUH
    ((0x8B, 0xF3), (0xB5F4)),  # HANGUL SYLLABLE SSANGTIKEUT YE
    ((0x8B, 0xF4), (0xB5F5)),  # HANGUL SYLLABLE SSANGTIKEUT YE KIYEOK
    ((0x8B, 0xF5), (0xB5F6)),  # HANGUL SYLLABLE SSANGTIKEUT YE SSANGKIYEOK
    ((0x8B, 0xF6), (0xB5F7)),  # HANGUL SYLLABLE SSANGTIKEUT YE KIYEOKSIOS
    ((0x8B, 0xF7), (0xB5F8)),  # HANGUL SYLLABLE SSANGTIKEUT YE NIEUN
    ((0x8B, 0xF8), (0xB5F9)),  # HANGUL SYLLABLE SSANGTIKEUT YE NIEUNCIEUC
    ((0x8B, 0xF9), (0xB5FA)),  # HANGUL SYLLABLE SSANGTIKEUT YE NIEUNHIEUH
    ((0x8B, 0xFA), (0xB5FB)),  # HANGUL SYLLABLE SSANGTIKEUT YE TIKEUT
    ((0x8B, 0xFB), (0xB5FC)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEUL
    ((0x8B, 0xFC), (0xB5FD)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULKIYEOK
    ((0x8B, 0xFD), (0xB5FE)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULMIEUM
    ((0x8B, 0xFE), (0xB5FF)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULPIEUP
    ((0x8C, 0x41), (0xB600)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULSIOS
    ((0x8C, 0x42), (0xB601)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULTHIEUTH
    ((0x8C, 0x43), (0xB602)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULPHIEUPH
    ((0x8C, 0x44), (0xB603)),  # HANGUL SYLLABLE SSANGTIKEUT YE RIEULHIEUH
    ((0x8C, 0x45), (0xB604)),  # HANGUL SYLLABLE SSANGTIKEUT YE MIEUM
    ((0x8C, 0x46), (0xB605)),  # HANGUL SYLLABLE SSANGTIKEUT YE PIEUP
    ((0x8C, 0x47), (0xB606)),  # HANGUL SYLLABLE SSANGTIKEUT YE PIEUPSIOS
    ((0x8C, 0x48), (0xB607)),  # HANGUL SYLLABLE SSANGTIKEUT YE SIOS
    ((0x8C, 0x49), (0xB608)),  # HANGUL SYLLABLE SSANGTIKEUT YE SSANGSIOS
    ((0x8C, 0x4A), (0xB609)),  # HANGUL SYLLABLE SSANGTIKEUT YE IEUNG
    ((0x8C, 0x4B), (0xB60A)),  # HANGUL SYLLABLE SSANGTIKEUT YE CIEUC
    ((0x8C, 0x4C), (0xB60B)),  # HANGUL SYLLABLE SSANGTIKEUT YE CHIEUCH
    ((0x8C, 0x4D), (0xB60C)),  # HANGUL SYLLABLE SSANGTIKEUT YE KHIEUKH
    ((0x8C, 0x4E), (0xB60D)),  # HANGUL SYLLABLE SSANGTIKEUT YE THIEUTH
    ((0x8C, 0x4F), (0xB60E)),  # HANGUL SYLLABLE SSANGTIKEUT YE PHIEUPH
    ((0x8C, 0x50), (0xB60F)),  # HANGUL SYLLABLE SSANGTIKEUT YE HIEUH
    ((0x8C, 0x51), (0xB612)),  # HANGUL SYLLABLE SSANGTIKEUT O SSANGKIYEOK
    ((0x8C, 0x52), (0xB613)),  # HANGUL SYLLABLE SSANGTIKEUT O KIYEOKSIOS
    ((0x8C, 0x53), (0xB615)),  # HANGUL SYLLABLE SSANGTIKEUT O NIEUNCIEUC
    ((0x8C, 0x54), (0xB616)),  # HANGUL SYLLABLE SSANGTIKEUT O NIEUNHIEUH
    ((0x8C, 0x55), (0xB617)),  # HANGUL SYLLABLE SSANGTIKEUT O TIKEUT
    ((0x8C, 0x56), (0xB619)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULKIYEOK
    ((0x8C, 0x57), (0xB61A)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULMIEUM
    ((0x8C, 0x58), (0xB61B)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULPIEUP
    ((0x8C, 0x59), (0xB61C)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULSIOS
    ((0x8C, 0x5A), (0xB61D)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULTHIEUTH
    ((0x8C, 0x61), (0xB61E)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULPHIEUPH
    ((0x8C, 0x62), (0xB61F)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEULHIEUH
    ((0x8C, 0x63), (0xB620)),  # HANGUL SYLLABLE SSANGTIKEUT O MIEUM
    ((0x8C, 0x64), (0xB621)),  # HANGUL SYLLABLE SSANGTIKEUT O PIEUP
    ((0x8C, 0x65), (0xB622)),  # HANGUL SYLLABLE SSANGTIKEUT O PIEUPSIOS
    ((0x8C, 0x66), (0xB623)),  # HANGUL SYLLABLE SSANGTIKEUT O SIOS
    ((0x8C, 0x67), (0xB624)),  # HANGUL SYLLABLE SSANGTIKEUT O SSANGSIOS
    ((0x8C, 0x68), (0xB626)),  # HANGUL SYLLABLE SSANGTIKEUT O CIEUC
    ((0x8C, 0x69), (0xB627)),  # HANGUL SYLLABLE SSANGTIKEUT O CHIEUCH
    ((0x8C, 0x6A), (0xB628)),  # HANGUL SYLLABLE SSANGTIKEUT O KHIEUKH
    ((0x8C, 0x6B), (0xB629)),  # HANGUL SYLLABLE SSANGTIKEUT O THIEUTH
    ((0x8C, 0x6C), (0xB62A)),  # HANGUL SYLLABLE SSANGTIKEUT O PHIEUPH
    ((0x8C, 0x6D), (0xB62B)),  # HANGUL SYLLABLE SSANGTIKEUT O HIEUH
    ((0x8C, 0x6E), (0xB62D)),  # HANGUL SYLLABLE SSANGTIKEUT WA KIYEOK
    ((0x8C, 0x6F), (0xB62E)),  # HANGUL SYLLABLE SSANGTIKEUT WA SSANGKIYEOK
    ((0x8C, 0x70), (0xB62F)),  # HANGUL SYLLABLE SSANGTIKEUT WA KIYEOKSIOS
    ((0x8C, 0x71), (0xB630)),  # HANGUL SYLLABLE SSANGTIKEUT WA NIEUN
    ((0x8C, 0x72), (0xB631)),  # HANGUL SYLLABLE SSANGTIKEUT WA NIEUNCIEUC
    ((0x8C, 0x73), (0xB632)),  # HANGUL SYLLABLE SSANGTIKEUT WA NIEUNHIEUH
    ((0x8C, 0x74), (0xB633)),  # HANGUL SYLLABLE SSANGTIKEUT WA TIKEUT
    ((0x8C, 0x75), (0xB635)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULKIYEOK
    ((0x8C, 0x76), (0xB636)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULMIEUM
    ((0x8C, 0x77), (0xB637)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULPIEUP
    ((0x8C, 0x78), (0xB638)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULSIOS
    ((0x8C, 0x79), (0xB639)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULTHIEUTH
    ((0x8C, 0x7A), (0xB63A)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULPHIEUPH
    ((0x8C, 0x81), (0xB63B)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEULHIEUH
    ((0x8C, 0x82), (0xB63C)),  # HANGUL SYLLABLE SSANGTIKEUT WA MIEUM
    ((0x8C, 0x83), (0xB63D)),  # HANGUL SYLLABLE SSANGTIKEUT WA PIEUP
    ((0x8C, 0x84), (0xB63E)),  # HANGUL SYLLABLE SSANGTIKEUT WA PIEUPSIOS
    ((0x8C, 0x85), (0xB63F)),  # HANGUL SYLLABLE SSANGTIKEUT WA SIOS
    ((0x8C, 0x86), (0xB640)),  # HANGUL SYLLABLE SSANGTIKEUT WA SSANGSIOS
    ((0x8C, 0x87), (0xB641)),  # HANGUL SYLLABLE SSANGTIKEUT WA IEUNG
    ((0x8C, 0x88), (0xB642)),  # HANGUL SYLLABLE SSANGTIKEUT WA CIEUC
    ((0x8C, 0x89), (0xB643)),  # HANGUL SYLLABLE SSANGTIKEUT WA CHIEUCH
    ((0x8C, 0x8A), (0xB644)),  # HANGUL SYLLABLE SSANGTIKEUT WA KHIEUKH
    ((0x8C, 0x8B), (0xB645)),  # HANGUL SYLLABLE SSANGTIKEUT WA THIEUTH
    ((0x8C, 0x8C), (0xB646)),  # HANGUL SYLLABLE SSANGTIKEUT WA PHIEUPH
    ((0x8C, 0x8D), (0xB647)),  # HANGUL SYLLABLE SSANGTIKEUT WA HIEUH
    ((0x8C, 0x8E), (0xB649)),  # HANGUL SYLLABLE SSANGTIKEUT WAE KIYEOK
    ((0x8C, 0x8F), (0xB64A)),  # HANGUL SYLLABLE SSANGTIKEUT WAE SSANGKIYEOK
    ((0x8C, 0x90), (0xB64B)),  # HANGUL SYLLABLE SSANGTIKEUT WAE KIYEOKSIOS
    ((0x8C, 0x91), (0xB64C)),  # HANGUL SYLLABLE SSANGTIKEUT WAE NIEUN
    ((0x8C, 0x92), (0xB64D)),  # HANGUL SYLLABLE SSANGTIKEUT WAE NIEUNCIEUC
    ((0x8C, 0x93), (0xB64E)),  # HANGUL SYLLABLE SSANGTIKEUT WAE NIEUNHIEUH
    ((0x8C, 0x94), (0xB64F)),  # HANGUL SYLLABLE SSANGTIKEUT WAE TIKEUT
    ((0x8C, 0x95), (0xB650)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEUL
    ((0x8C, 0x96), (0xB651)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULKIYEOK
    ((0x8C, 0x97), (0xB652)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULMIEUM
    ((0x8C, 0x98), (0xB653)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULPIEUP
    ((0x8C, 0x99), (0xB654)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULSIOS
    ((0x8C, 0x9A), (0xB655)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULTHIEUTH
    ((0x8C, 0x9B), (0xB656)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULPHIEUPH
    ((0x8C, 0x9C), (0xB657)),  # HANGUL SYLLABLE SSANGTIKEUT WAE RIEULHIEUH
    ((0x8C, 0x9D), (0xB658)),  # HANGUL SYLLABLE SSANGTIKEUT WAE MIEUM
    ((0x8C, 0x9E), (0xB659)),  # HANGUL SYLLABLE SSANGTIKEUT WAE PIEUP
    ((0x8C, 0x9F), (0xB65A)),  # HANGUL SYLLABLE SSANGTIKEUT WAE PIEUPSIOS
    ((0x8C, 0xA0), (0xB65B)),  # HANGUL SYLLABLE SSANGTIKEUT WAE SIOS
    ((0x8C, 0xA1), (0xB65C)),  # HANGUL SYLLABLE SSANGTIKEUT WAE SSANGSIOS
    ((0x8C, 0xA2), (0xB65D)),  # HANGUL SYLLABLE SSANGTIKEUT WAE IEUNG
    ((0x8C, 0xA3), (0xB65E)),  # HANGUL SYLLABLE SSANGTIKEUT WAE CIEUC
    ((0x8C, 0xA4), (0xB65F)),  # HANGUL SYLLABLE SSANGTIKEUT WAE CHIEUCH
    ((0x8C, 0xA5), (0xB660)),  # HANGUL SYLLABLE SSANGTIKEUT WAE KHIEUKH
    ((0x8C, 0xA6), (0xB661)),  # HANGUL SYLLABLE SSANGTIKEUT WAE THIEUTH
    ((0x8C, 0xA7), (0xB662)),  # HANGUL SYLLABLE SSANGTIKEUT WAE PHIEUPH
    ((0x8C, 0xA8), (0xB663)),  # HANGUL SYLLABLE SSANGTIKEUT WAE HIEUH
    ((0x8C, 0xA9), (0xB665)),  # HANGUL SYLLABLE SSANGTIKEUT OE KIYEOK
    ((0x8C, 0xAA), (0xB666)),  # HANGUL SYLLABLE SSANGTIKEUT OE SSANGKIYEOK
    ((0x8C, 0xAB), (0xB667)),  # HANGUL SYLLABLE SSANGTIKEUT OE KIYEOKSIOS
    ((0x8C, 0xAC), (0xB669)),  # HANGUL SYLLABLE SSANGTIKEUT OE NIEUNCIEUC
    ((0x8C, 0xAD), (0xB66A)),  # HANGUL SYLLABLE SSANGTIKEUT OE NIEUNHIEUH
    ((0x8C, 0xAE), (0xB66B)),  # HANGUL SYLLABLE SSANGTIKEUT OE TIKEUT
    ((0x8C, 0xAF), (0xB66C)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEUL
    ((0x8C, 0xB0), (0xB66D)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULKIYEOK
    ((0x8C, 0xB1), (0xB66E)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULMIEUM
    ((0x8C, 0xB2), (0xB66F)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULPIEUP
    ((0x8C, 0xB3), (0xB670)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULSIOS
    ((0x8C, 0xB4), (0xB671)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULTHIEUTH
    ((0x8C, 0xB5), (0xB672)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULPHIEUPH
    ((0x8C, 0xB6), (0xB673)),  # HANGUL SYLLABLE SSANGTIKEUT OE RIEULHIEUH
    ((0x8C, 0xB7), (0xB674)),  # HANGUL SYLLABLE SSANGTIKEUT OE MIEUM
    ((0x8C, 0xB8), (0xB675)),  # HANGUL SYLLABLE SSANGTIKEUT OE PIEUP
    ((0x8C, 0xB9), (0xB676)),  # HANGUL SYLLABLE SSANGTIKEUT OE PIEUPSIOS
    ((0x8C, 0xBA), (0xB677)),  # HANGUL SYLLABLE SSANGTIKEUT OE SIOS
    ((0x8C, 0xBB), (0xB678)),  # HANGUL SYLLABLE SSANGTIKEUT OE SSANGSIOS
    ((0x8C, 0xBC), (0xB679)),  # HANGUL SYLLABLE SSANGTIKEUT OE IEUNG
    ((0x8C, 0xBD), (0xB67A)),  # HANGUL SYLLABLE SSANGTIKEUT OE CIEUC
    ((0x8C, 0xBE), (0xB67B)),  # HANGUL SYLLABLE SSANGTIKEUT OE CHIEUCH
    ((0x8C, 0xBF), (0xB67C)),  # HANGUL SYLLABLE SSANGTIKEUT OE KHIEUKH
    ((0x8C, 0xC0), (0xB67D)),  # HANGUL SYLLABLE SSANGTIKEUT OE THIEUTH
    ((0x8C, 0xC1), (0xB67E)),  # HANGUL SYLLABLE SSANGTIKEUT OE PHIEUPH
    ((0x8C, 0xC2), (0xB67F)),  # HANGUL SYLLABLE SSANGTIKEUT OE HIEUH
    ((0x8C, 0xC3), (0xB680)),  # HANGUL SYLLABLE SSANGTIKEUT YO
    ((0x8C, 0xC4), (0xB681)),  # HANGUL SYLLABLE SSANGTIKEUT YO KIYEOK
    ((0x8C, 0xC5), (0xB682)),  # HANGUL SYLLABLE SSANGTIKEUT YO SSANGKIYEOK
    ((0x8C, 0xC6), (0xB683)),  # HANGUL SYLLABLE SSANGTIKEUT YO KIYEOKSIOS
    ((0x8C, 0xC7), (0xB684)),  # HANGUL SYLLABLE SSANGTIKEUT YO NIEUN
    ((0x8C, 0xC8), (0xB685)),  # HANGUL SYLLABLE SSANGTIKEUT YO NIEUNCIEUC
    ((0x8C, 0xC9), (0xB686)),  # HANGUL SYLLABLE SSANGTIKEUT YO NIEUNHIEUH
    ((0x8C, 0xCA), (0xB687)),  # HANGUL SYLLABLE SSANGTIKEUT YO TIKEUT
    ((0x8C, 0xCB), (0xB688)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEUL
    ((0x8C, 0xCC), (0xB689)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULKIYEOK
    ((0x8C, 0xCD), (0xB68A)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULMIEUM
    ((0x8C, 0xCE), (0xB68B)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULPIEUP
    ((0x8C, 0xCF), (0xB68C)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULSIOS
    ((0x8C, 0xD0), (0xB68D)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULTHIEUTH
    ((0x8C, 0xD1), (0xB68E)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULPHIEUPH
    ((0x8C, 0xD2), (0xB68F)),  # HANGUL SYLLABLE SSANGTIKEUT YO RIEULHIEUH
    ((0x8C, 0xD3), (0xB690)),  # HANGUL SYLLABLE SSANGTIKEUT YO MIEUM
    ((0x8C, 0xD4), (0xB691)),  # HANGUL SYLLABLE SSANGTIKEUT YO PIEUP
    ((0x8C, 0xD5), (0xB692)),  # HANGUL SYLLABLE SSANGTIKEUT YO PIEUPSIOS
    ((0x8C, 0xD6), (0xB693)),  # HANGUL SYLLABLE SSANGTIKEUT YO SIOS
    ((0x8C, 0xD7), (0xB694)),  # HANGUL SYLLABLE SSANGTIKEUT YO SSANGSIOS
    ((0x8C, 0xD8), (0xB695)),  # HANGUL SYLLABLE SSANGTIKEUT YO IEUNG
    ((0x8C, 0xD9), (0xB696)),  # HANGUL SYLLABLE SSANGTIKEUT YO CIEUC
    ((0x8C, 0xDA), (0xB697)),  # HANGUL SYLLABLE SSANGTIKEUT YO CHIEUCH
    ((0x8C, 0xDB), (0xB698)),  # HANGUL SYLLABLE SSANGTIKEUT YO KHIEUKH
    ((0x8C, 0xDC), (0xB699)),  # HANGUL SYLLABLE SSANGTIKEUT YO THIEUTH
    ((0x8C, 0xDD), (0xB69A)),  # HANGUL SYLLABLE SSANGTIKEUT YO PHIEUPH
    ((0x8C, 0xDE), (0xB69B)),  # HANGUL SYLLABLE SSANGTIKEUT YO HIEUH
    ((0x8C, 0xDF), (0xB69E)),  # HANGUL SYLLABLE SSANGTIKEUT U SSANGKIYEOK
    ((0x8C, 0xE0), (0xB69F)),  # HANGUL SYLLABLE SSANGTIKEUT U KIYEOKSIOS
    ((0x8C, 0xE1), (0xB6A1)),  # HANGUL SYLLABLE SSANGTIKEUT U NIEUNCIEUC
    ((0x8C, 0xE2), (0xB6A2)),  # HANGUL SYLLABLE SSANGTIKEUT U NIEUNHIEUH
    ((0x8C, 0xE3), (0xB6A3)),  # HANGUL SYLLABLE SSANGTIKEUT U TIKEUT
    ((0x8C, 0xE4), (0xB6A5)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULKIYEOK
    ((0x8C, 0xE5), (0xB6A6)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULMIEUM
    ((0x8C, 0xE6), (0xB6A7)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULPIEUP
    ((0x8C, 0xE7), (0xB6A8)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULSIOS
    ((0x8C, 0xE8), (0xB6A9)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULTHIEUTH
    ((0x8C, 0xE9), (0xB6AA)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULPHIEUPH
    ((0x8C, 0xEA), (0xB6AD)),  # HANGUL SYLLABLE SSANGTIKEUT U PIEUP
    ((0x8C, 0xEB), (0xB6AE)),  # HANGUL SYLLABLE SSANGTIKEUT U PIEUPSIOS
    ((0x8C, 0xEC), (0xB6AF)),  # HANGUL SYLLABLE SSANGTIKEUT U SIOS
    ((0x8C, 0xED), (0xB6B0)),  # HANGUL SYLLABLE SSANGTIKEUT U SSANGSIOS
    ((0x8C, 0xEE), (0xB6B2)),  # HANGUL SYLLABLE SSANGTIKEUT U CIEUC
    ((0x8C, 0xEF), (0xB6B3)),  # HANGUL SYLLABLE SSANGTIKEUT U CHIEUCH
    ((0x8C, 0xF0), (0xB6B4)),  # HANGUL SYLLABLE SSANGTIKEUT U KHIEUKH
    ((0x8C, 0xF1), (0xB6B5)),  # HANGUL SYLLABLE SSANGTIKEUT U THIEUTH
    ((0x8C, 0xF2), (0xB6B6)),  # HANGUL SYLLABLE SSANGTIKEUT U PHIEUPH
    ((0x8C, 0xF3), (0xB6B7)),  # HANGUL SYLLABLE SSANGTIKEUT U HIEUH
    ((0x8C, 0xF4), (0xB6B8)),  # HANGUL SYLLABLE SSANGTIKEUT WEO
    ((0x8C, 0xF5), (0xB6B9)),  # HANGUL SYLLABLE SSANGTIKEUT WEO KIYEOK
    ((0x8C, 0xF6), (0xB6BA)),  # HANGUL SYLLABLE SSANGTIKEUT WEO SSANGKIYEOK
    ((0x8C, 0xF7), (0xB6BB)),  # HANGUL SYLLABLE SSANGTIKEUT WEO KIYEOKSIOS
    ((0x8C, 0xF8), (0xB6BC)),  # HANGUL SYLLABLE SSANGTIKEUT WEO NIEUN
    ((0x8C, 0xF9), (0xB6BD)),  # HANGUL SYLLABLE SSANGTIKEUT WEO NIEUNCIEUC
    ((0x8C, 0xFA), (0xB6BE)),  # HANGUL SYLLABLE SSANGTIKEUT WEO NIEUNHIEUH
    ((0x8C, 0xFB), (0xB6BF)),  # HANGUL SYLLABLE SSANGTIKEUT WEO TIKEUT
    ((0x8C, 0xFC), (0xB6C0)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEUL
    ((0x8C, 0xFD), (0xB6C1)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULKIYEOK
    ((0x8C, 0xFE), (0xB6C2)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULMIEUM
    ((0x8D, 0x41), (0xB6C3)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULPIEUP
    ((0x8D, 0x42), (0xB6C4)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULSIOS
    ((0x8D, 0x43), (0xB6C5)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULTHIEUTH
    ((0x8D, 0x44), (0xB6C6)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULPHIEUPH
    ((0x8D, 0x45), (0xB6C7)),  # HANGUL SYLLABLE SSANGTIKEUT WEO RIEULHIEUH
    ((0x8D, 0x46), (0xB6C8)),  # HANGUL SYLLABLE SSANGTIKEUT WEO MIEUM
    ((0x8D, 0x47), (0xB6C9)),  # HANGUL SYLLABLE SSANGTIKEUT WEO PIEUP
    ((0x8D, 0x48), (0xB6CA)),  # HANGUL SYLLABLE SSANGTIKEUT WEO PIEUPSIOS
    ((0x8D, 0x49), (0xB6CB)),  # HANGUL SYLLABLE SSANGTIKEUT WEO SIOS
    ((0x8D, 0x4A), (0xB6CC)),  # HANGUL SYLLABLE SSANGTIKEUT WEO SSANGSIOS
    ((0x8D, 0x4B), (0xB6CD)),  # HANGUL SYLLABLE SSANGTIKEUT WEO IEUNG
    ((0x8D, 0x4C), (0xB6CE)),  # HANGUL SYLLABLE SSANGTIKEUT WEO CIEUC
    ((0x8D, 0x4D), (0xB6CF)),  # HANGUL SYLLABLE SSANGTIKEUT WEO CHIEUCH
    ((0x8D, 0x4E), (0xB6D0)),  # HANGUL SYLLABLE SSANGTIKEUT WEO KHIEUKH
    ((0x8D, 0x4F), (0xB6D1)),  # HANGUL SYLLABLE SSANGTIKEUT WEO THIEUTH
    ((0x8D, 0x50), (0xB6D2)),  # HANGUL SYLLABLE SSANGTIKEUT WEO PHIEUPH
    ((0x8D, 0x51), (0xB6D3)),  # HANGUL SYLLABLE SSANGTIKEUT WEO HIEUH
    ((0x8D, 0x52), (0xB6D5)),  # HANGUL SYLLABLE SSANGTIKEUT WE KIYEOK
    ((0x8D, 0x53), (0xB6D6)),  # HANGUL SYLLABLE SSANGTIKEUT WE SSANGKIYEOK
    ((0x8D, 0x54), (0xB6D7)),  # HANGUL SYLLABLE SSANGTIKEUT WE KIYEOKSIOS
    ((0x8D, 0x55), (0xB6D8)),  # HANGUL SYLLABLE SSANGTIKEUT WE NIEUN
    ((0x8D, 0x56), (0xB6D9)),  # HANGUL SYLLABLE SSANGTIKEUT WE NIEUNCIEUC
    ((0x8D, 0x57), (0xB6DA)),  # HANGUL SYLLABLE SSANGTIKEUT WE NIEUNHIEUH
    ((0x8D, 0x58), (0xB6DB)),  # HANGUL SYLLABLE SSANGTIKEUT WE TIKEUT
    ((0x8D, 0x59), (0xB6DC)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEUL
    ((0x8D, 0x5A), (0xB6DD)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULKIYEOK
    ((0x8D, 0x61), (0xB6DE)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULMIEUM
    ((0x8D, 0x62), (0xB6DF)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULPIEUP
    ((0x8D, 0x63), (0xB6E0)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULSIOS
    ((0x8D, 0x64), (0xB6E1)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULTHIEUTH
    ((0x8D, 0x65), (0xB6E2)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULPHIEUPH
    ((0x8D, 0x66), (0xB6E3)),  # HANGUL SYLLABLE SSANGTIKEUT WE RIEULHIEUH
    ((0x8D, 0x67), (0xB6E4)),  # HANGUL SYLLABLE SSANGTIKEUT WE MIEUM
    ((0x8D, 0x68), (0xB6E5)),  # HANGUL SYLLABLE SSANGTIKEUT WE PIEUP
    ((0x8D, 0x69), (0xB6E6)),  # HANGUL SYLLABLE SSANGTIKEUT WE PIEUPSIOS
    ((0x8D, 0x6A), (0xB6E7)),  # HANGUL SYLLABLE SSANGTIKEUT WE SIOS
    ((0x8D, 0x6B), (0xB6E8)),  # HANGUL SYLLABLE SSANGTIKEUT WE SSANGSIOS
    ((0x8D, 0x6C), (0xB6E9)),  # HANGUL SYLLABLE SSANGTIKEUT WE IEUNG
    ((0x8D, 0x6D), (0xB6EA)),  # HANGUL SYLLABLE SSANGTIKEUT WE CIEUC
    ((0x8D, 0x6E), (0xB6EB)),  # HANGUL SYLLABLE SSANGTIKEUT WE CHIEUCH
    ((0x8D, 0x6F), (0xB6EC)),  # HANGUL SYLLABLE SSANGTIKEUT WE KHIEUKH
    ((0x8D, 0x70), (0xB6ED)),  # HANGUL SYLLABLE SSANGTIKEUT WE THIEUTH
    ((0x8D, 0x71), (0xB6EE)),  # HANGUL SYLLABLE SSANGTIKEUT WE PHIEUPH
    ((0x8D, 0x72), (0xB6EF)),  # HANGUL SYLLABLE SSANGTIKEUT WE HIEUH
    ((0x8D, 0x73), (0xB6F1)),  # HANGUL SYLLABLE SSANGTIKEUT WI KIYEOK
    ((0x8D, 0x74), (0xB6F2)),  # HANGUL SYLLABLE SSANGTIKEUT WI SSANGKIYEOK
    ((0x8D, 0x75), (0xB6F3)),  # HANGUL SYLLABLE SSANGTIKEUT WI KIYEOKSIOS
    ((0x8D, 0x76), (0xB6F5)),  # HANGUL SYLLABLE SSANGTIKEUT WI NIEUNCIEUC
    ((0x8D, 0x77), (0xB6F6)),  # HANGUL SYLLABLE SSANGTIKEUT WI NIEUNHIEUH
    ((0x8D, 0x78), (0xB6F7)),  # HANGUL SYLLABLE SSANGTIKEUT WI TIKEUT
    ((0x8D, 0x79), (0xB6F9)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULKIYEOK
    ((0x8D, 0x7A), (0xB6FA)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULMIEUM
    ((0x8D, 0x81), (0xB6FB)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULPIEUP
    ((0x8D, 0x82), (0xB6FC)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULSIOS
    ((0x8D, 0x83), (0xB6FD)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULTHIEUTH
    ((0x8D, 0x84), (0xB6FE)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULPHIEUPH
    ((0x8D, 0x85), (0xB6FF)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEULHIEUH
    ((0x8D, 0x86), (0xB702)),  # HANGUL SYLLABLE SSANGTIKEUT WI PIEUPSIOS
    ((0x8D, 0x87), (0xB703)),  # HANGUL SYLLABLE SSANGTIKEUT WI SIOS
    ((0x8D, 0x88), (0xB704)),  # HANGUL SYLLABLE SSANGTIKEUT WI SSANGSIOS
    ((0x8D, 0x89), (0xB706)),  # HANGUL SYLLABLE SSANGTIKEUT WI CIEUC
    ((0x8D, 0x8A), (0xB707)),  # HANGUL SYLLABLE SSANGTIKEUT WI CHIEUCH
    ((0x8D, 0x8B), (0xB708)),  # HANGUL SYLLABLE SSANGTIKEUT WI KHIEUKH
    ((0x8D, 0x8C), (0xB709)),  # HANGUL SYLLABLE SSANGTIKEUT WI THIEUTH
    ((0x8D, 0x8D), (0xB70A)),  # HANGUL SYLLABLE SSANGTIKEUT WI PHIEUPH
    ((0x8D, 0x8E), (0xB70B)),  # HANGUL SYLLABLE SSANGTIKEUT WI HIEUH
    ((0x8D, 0x8F), (0xB70C)),  # HANGUL SYLLABLE SSANGTIKEUT YU
    ((0x8D, 0x90), (0xB70D)),  # HANGUL SYLLABLE SSANGTIKEUT YU KIYEOK
    ((0x8D, 0x91), (0xB70E)),  # HANGUL SYLLABLE SSANGTIKEUT YU SSANGKIYEOK
    ((0x8D, 0x92), (0xB70F)),  # HANGUL SYLLABLE SSANGTIKEUT YU KIYEOKSIOS
    ((0x8D, 0x93), (0xB710)),  # HANGUL SYLLABLE SSANGTIKEUT YU NIEUN
    ((0x8D, 0x94), (0xB711)),  # HANGUL SYLLABLE SSANGTIKEUT YU NIEUNCIEUC
    ((0x8D, 0x95), (0xB712)),  # HANGUL SYLLABLE SSANGTIKEUT YU NIEUNHIEUH
    ((0x8D, 0x96), (0xB713)),  # HANGUL SYLLABLE SSANGTIKEUT YU TIKEUT
    ((0x8D, 0x97), (0xB714)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEUL
    ((0x8D, 0x98), (0xB715)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULKIYEOK
    ((0x8D, 0x99), (0xB716)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULMIEUM
    ((0x8D, 0x9A), (0xB717)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULPIEUP
    ((0x8D, 0x9B), (0xB718)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULSIOS
    ((0x8D, 0x9C), (0xB719)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULTHIEUTH
    ((0x8D, 0x9D), (0xB71A)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULPHIEUPH
    ((0x8D, 0x9E), (0xB71B)),  # HANGUL SYLLABLE SSANGTIKEUT YU RIEULHIEUH
    ((0x8D, 0x9F), (0xB71C)),  # HANGUL SYLLABLE SSANGTIKEUT YU MIEUM
    ((0x8D, 0xA0), (0xB71D)),  # HANGUL SYLLABLE SSANGTIKEUT YU PIEUP
    ((0x8D, 0xA1), (0xB71E)),  # HANGUL SYLLABLE SSANGTIKEUT YU PIEUPSIOS
    ((0x8D, 0xA2), (0xB71F)),  # HANGUL SYLLABLE SSANGTIKEUT YU SIOS
    ((0x8D, 0xA3), (0xB720)),  # HANGUL SYLLABLE SSANGTIKEUT YU SSANGSIOS
    ((0x8D, 0xA4), (0xB721)),  # HANGUL SYLLABLE SSANGTIKEUT YU IEUNG
    ((0x8D, 0xA5), (0xB722)),  # HANGUL SYLLABLE SSANGTIKEUT YU CIEUC
    ((0x8D, 0xA6), (0xB723)),  # HANGUL SYLLABLE SSANGTIKEUT YU CHIEUCH
    ((0x8D, 0xA7), (0xB724)),  # HANGUL SYLLABLE SSANGTIKEUT YU KHIEUKH
    ((0x8D, 0xA8), (0xB725)),  # HANGUL SYLLABLE SSANGTIKEUT YU THIEUTH
    ((0x8D, 0xA9), (0xB726)),  # HANGUL SYLLABLE SSANGTIKEUT YU PHIEUPH
    ((0x8D, 0xAA), (0xB727)),  # HANGUL SYLLABLE SSANGTIKEUT YU HIEUH
    ((0x8D, 0xAB), (0xB72A)),  # HANGUL SYLLABLE SSANGTIKEUT EU SSANGKIYEOK
    ((0x8D, 0xAC), (0xB72B)),  # HANGUL SYLLABLE SSANGTIKEUT EU KIYEOKSIOS
    ((0x8D, 0xAD), (0xB72D)),  # HANGUL SYLLABLE SSANGTIKEUT EU NIEUNCIEUC
    ((0x8D, 0xAE), (0xB72E)),  # HANGUL SYLLABLE SSANGTIKEUT EU NIEUNHIEUH
    ((0x8D, 0xAF), (0xB731)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULKIYEOK
    ((0x8D, 0xB0), (0xB732)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULMIEUM
    ((0x8D, 0xB1), (0xB733)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULPIEUP
    ((0x8D, 0xB2), (0xB734)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULSIOS
    ((0x8D, 0xB3), (0xB735)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULTHIEUTH
    ((0x8D, 0xB4), (0xB736)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULPHIEUPH
    ((0x8D, 0xB5), (0xB737)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEULHIEUH
    ((0x8D, 0xB6), (0xB73A)),  # HANGUL SYLLABLE SSANGTIKEUT EU PIEUPSIOS
    ((0x8D, 0xB7), (0xB73C)),  # HANGUL SYLLABLE SSANGTIKEUT EU SSANGSIOS
    ((0x8D, 0xB8), (0xB73D)),  # HANGUL SYLLABLE SSANGTIKEUT EU IEUNG
    ((0x8D, 0xB9), (0xB73E)),  # HANGUL SYLLABLE SSANGTIKEUT EU CIEUC
    ((0x8D, 0xBA), (0xB73F)),  # HANGUL SYLLABLE SSANGTIKEUT EU CHIEUCH
    ((0x8D, 0xBB), (0xB740)),  # HANGUL SYLLABLE SSANGTIKEUT EU KHIEUKH
    ((0x8D, 0xBC), (0xB741)),  # HANGUL SYLLABLE SSANGTIKEUT EU THIEUTH
    ((0x8D, 0xBD), (0xB742)),  # HANGUL SYLLABLE SSANGTIKEUT EU PHIEUPH
    ((0x8D, 0xBE), (0xB743)),  # HANGUL SYLLABLE SSANGTIKEUT EU HIEUH
    ((0x8D, 0xBF), (0xB745)),  # HANGUL SYLLABLE SSANGTIKEUT YI KIYEOK
    ((0x8D, 0xC0), (0xB746)),  # HANGUL SYLLABLE SSANGTIKEUT YI SSANGKIYEOK
    ((0x8D, 0xC1), (0xB747)),  # HANGUL SYLLABLE SSANGTIKEUT YI KIYEOKSIOS
    ((0x8D, 0xC2), (0xB749)),  # HANGUL SYLLABLE SSANGTIKEUT YI NIEUNCIEUC
    ((0x8D, 0xC3), (0xB74A)),  # HANGUL SYLLABLE SSANGTIKEUT YI NIEUNHIEUH
    ((0x8D, 0xC4), (0xB74B)),  # HANGUL SYLLABLE SSANGTIKEUT YI TIKEUT
    ((0x8D, 0xC5), (0xB74D)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULKIYEOK
    ((0x8D, 0xC6), (0xB74E)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULMIEUM
    ((0x8D, 0xC7), (0xB74F)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULPIEUP
    ((0x8D, 0xC8), (0xB750)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULSIOS
    ((0x8D, 0xC9), (0xB751)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULTHIEUTH
    ((0x8D, 0xCA), (0xB752)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULPHIEUPH
    ((0x8D, 0xCB), (0xB753)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEULHIEUH
    ((0x8D, 0xCC), (0xB756)),  # HANGUL SYLLABLE SSANGTIKEUT YI PIEUPSIOS
    ((0x8D, 0xCD), (0xB757)),  # HANGUL SYLLABLE SSANGTIKEUT YI SIOS
    ((0x8D, 0xCE), (0xB758)),  # HANGUL SYLLABLE SSANGTIKEUT YI SSANGSIOS
    ((0x8D, 0xCF), (0xB759)),  # HANGUL SYLLABLE SSANGTIKEUT YI IEUNG
    ((0x8D, 0xD0), (0xB75A)),  # HANGUL SYLLABLE SSANGTIKEUT YI CIEUC
    ((0x8D, 0xD1), (0xB75B)),  # HANGUL SYLLABLE SSANGTIKEUT YI CHIEUCH
    ((0x8D, 0xD2), (0xB75C)),  # HANGUL SYLLABLE SSANGTIKEUT YI KHIEUKH
    ((0x8D, 0xD3), (0xB75D)),  # HANGUL SYLLABLE SSANGTIKEUT YI THIEUTH
    ((0x8D, 0xD4), (0xB75E)),  # HANGUL SYLLABLE SSANGTIKEUT YI PHIEUPH
    ((0x8D, 0xD5), (0xB75F)),  # HANGUL SYLLABLE SSANGTIKEUT YI HIEUH
    ((0x8D, 0xD6), (0xB761)),  # HANGUL SYLLABLE SSANGTIKEUT I KIYEOK
    ((0x8D, 0xD7), (0xB762)),  # HANGUL SYLLABLE SSANGTIKEUT I SSANGKIYEOK
    ((0x8D, 0xD8), (0xB763)),  # HANGUL SYLLABLE SSANGTIKEUT I KIYEOKSIOS
    ((0x8D, 0xD9), (0xB765)),  # HANGUL SYLLABLE SSANGTIKEUT I NIEUNCIEUC
    ((0x8D, 0xDA), (0xB766)),  # HANGUL SYLLABLE SSANGTIKEUT I NIEUNHIEUH
    ((0x8D, 0xDB), (0xB767)),  # HANGUL SYLLABLE SSANGTIKEUT I TIKEUT
    ((0x8D, 0xDC), (0xB769)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULKIYEOK
    ((0x8D, 0xDD), (0xB76A)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULMIEUM
    ((0x8D, 0xDE), (0xB76B)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULPIEUP
    ((0x8D, 0xDF), (0xB76C)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULSIOS
    ((0x8D, 0xE0), (0xB76D)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULTHIEUTH
    ((0x8D, 0xE1), (0xB76E)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULPHIEUPH
    ((0x8D, 0xE2), (0xB76F)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEULHIEUH
    ((0x8D, 0xE3), (0xB772)),  # HANGUL SYLLABLE SSANGTIKEUT I PIEUPSIOS
    ((0x8D, 0xE4), (0xB774)),  # HANGUL SYLLABLE SSANGTIKEUT I SSANGSIOS
    ((0x8D, 0xE5), (0xB776)),  # HANGUL SYLLABLE SSANGTIKEUT I CIEUC
    ((0x8D, 0xE6), (0xB777)),  # HANGUL SYLLABLE SSANGTIKEUT I CHIEUCH
    ((0x8D, 0xE7), (0xB778)),  # HANGUL SYLLABLE SSANGTIKEUT I KHIEUKH
    ((0x8D, 0xE8), (0xB779)),  # HANGUL SYLLABLE SSANGTIKEUT I THIEUTH
    ((0x8D, 0xE9), (0xB77A)),  # HANGUL SYLLABLE SSANGTIKEUT I PHIEUPH
    ((0x8D, 0xEA), (0xB77B)),  # HANGUL SYLLABLE SSANGTIKEUT I HIEUH
    ((0x8D, 0xEB), (0xB77E)),  # HANGUL SYLLABLE RIEUL A SSANGKIYEOK
    ((0x8D, 0xEC), (0xB77F)),  # HANGUL SYLLABLE RIEUL A KIYEOKSIOS
    ((0x8D, 0xED), (0xB781)),  # HANGUL SYLLABLE RIEUL A NIEUNCIEUC
    ((0x8D, 0xEE), (0xB782)),  # HANGUL SYLLABLE RIEUL A NIEUNHIEUH
    ((0x8D, 0xEF), (0xB783)),  # HANGUL SYLLABLE RIEUL A TIKEUT
    ((0x8D, 0xF0), (0xB785)),  # HANGUL SYLLABLE RIEUL A RIEULKIYEOK
    ((0x8D, 0xF1), (0xB786)),  # HANGUL SYLLABLE RIEUL A RIEULMIEUM
    ((0x8D, 0xF2), (0xB787)),  # HANGUL SYLLABLE RIEUL A RIEULPIEUP
    ((0x8D, 0xF3), (0xB788)),  # HANGUL SYLLABLE RIEUL A RIEULSIOS
    ((0x8D, 0xF4), (0xB789)),  # HANGUL SYLLABLE RIEUL A RIEULTHIEUTH
    ((0x8D, 0xF5), (0xB78A)),  # HANGUL SYLLABLE RIEUL A RIEULPHIEUPH
    ((0x8D, 0xF6), (0xB78B)),  # HANGUL SYLLABLE RIEUL A RIEULHIEUH
    ((0x8D, 0xF7), (0xB78E)),  # HANGUL SYLLABLE RIEUL A PIEUPSIOS
    ((0x8D, 0xF8), (0xB793)),  # HANGUL SYLLABLE RIEUL A CHIEUCH
    ((0x8D, 0xF9), (0xB794)),  # HANGUL SYLLABLE RIEUL A KHIEUKH
    ((0x8D, 0xFA), (0xB795)),  # HANGUL SYLLABLE RIEUL A THIEUTH
    ((0x8D, 0xFB), (0xB79A)),  # HANGUL SYLLABLE RIEUL AE SSANGKIYEOK
    ((0x8D, 0xFC), (0xB79B)),  # HANGUL SYLLABLE RIEUL AE KIYEOKSIOS
    ((0x8D, 0xFD), (0xB79D)),  # HANGUL SYLLABLE RIEUL AE NIEUNCIEUC
    ((0x8D, 0xFE), (0xB79E)),  # HANGUL SYLLABLE RIEUL AE NIEUNHIEUH
    ((0x8E, 0x41), (0xB79F)),  # HANGUL SYLLABLE RIEUL AE TIKEUT
    ((0x8E, 0x42), (0xB7A1)),  # HANGUL SYLLABLE RIEUL AE RIEULKIYEOK
    ((0x8E, 0x43), (0xB7A2)),  # HANGUL SYLLABLE RIEUL AE RIEULMIEUM
    ((0x8E, 0x44), (0xB7A3)),  # HANGUL SYLLABLE RIEUL AE RIEULPIEUP
    ((0x8E, 0x45), (0xB7A4)),  # HANGUL SYLLABLE RIEUL AE RIEULSIOS
    ((0x8E, 0x46), (0xB7A5)),  # HANGUL SYLLABLE RIEUL AE RIEULTHIEUTH
    ((0x8E, 0x47), (0xB7A6)),  # HANGUL SYLLABLE RIEUL AE RIEULPHIEUPH
    ((0x8E, 0x48), (0xB7A7)),  # HANGUL SYLLABLE RIEUL AE RIEULHIEUH
    ((0x8E, 0x49), (0xB7AA)),  # HANGUL SYLLABLE RIEUL AE PIEUPSIOS
    ((0x8E, 0x4A), (0xB7AE)),  # HANGUL SYLLABLE RIEUL AE CIEUC
    ((0x8E, 0x4B), (0xB7AF)),  # HANGUL SYLLABLE RIEUL AE CHIEUCH
    ((0x8E, 0x4C), (0xB7B0)),  # HANGUL SYLLABLE RIEUL AE KHIEUKH
    ((0x8E, 0x4D), (0xB7B1)),  # HANGUL SYLLABLE RIEUL AE THIEUTH
    ((0x8E, 0x4E), (0xB7B2)),  # HANGUL SYLLABLE RIEUL AE PHIEUPH
    ((0x8E, 0x4F), (0xB7B3)),  # HANGUL SYLLABLE RIEUL AE HIEUH
    ((0x8E, 0x50), (0xB7B6)),  # HANGUL SYLLABLE RIEUL YA SSANGKIYEOK
    ((0x8E, 0x51), (0xB7B7)),  # HANGUL SYLLABLE RIEUL YA KIYEOKSIOS
    ((0x8E, 0x52), (0xB7B9)),  # HANGUL SYLLABLE RIEUL YA NIEUNCIEUC
    ((0x8E, 0x53), (0xB7BA)),  # HANGUL SYLLABLE RIEUL YA NIEUNHIEUH
    ((0x8E, 0x54), (0xB7BB)),  # HANGUL SYLLABLE RIEUL YA TIKEUT
    ((0x8E, 0x55), (0xB7BC)),  # HANGUL SYLLABLE RIEUL YA RIEUL
    ((0x8E, 0x56), (0xB7BD)),  # HANGUL SYLLABLE RIEUL YA RIEULKIYEOK
    ((0x8E, 0x57), (0xB7BE)),  # HANGUL SYLLABLE RIEUL YA RIEULMIEUM
    ((0x8E, 0x58), (0xB7BF)),  # HANGUL SYLLABLE RIEUL YA RIEULPIEUP
    ((0x8E, 0x59), (0xB7C0)),  # HANGUL SYLLABLE RIEUL YA RIEULSIOS
    ((0x8E, 0x5A), (0xB7C1)),  # HANGUL SYLLABLE RIEUL YA RIEULTHIEUTH
    ((0x8E, 0x61), (0xB7C2)),  # HANGUL SYLLABLE RIEUL YA RIEULPHIEUPH
    ((0x8E, 0x62), (0xB7C3)),  # HANGUL SYLLABLE RIEUL YA RIEULHIEUH
    ((0x8E, 0x63), (0xB7C4)),  # HANGUL SYLLABLE RIEUL YA MIEUM
    ((0x8E, 0x64), (0xB7C5)),  # HANGUL SYLLABLE RIEUL YA PIEUP
    ((0x8E, 0x65), (0xB7C6)),  # HANGUL SYLLABLE RIEUL YA PIEUPSIOS
    ((0x8E, 0x66), (0xB7C8)),  # HANGUL SYLLABLE RIEUL YA SSANGSIOS
    ((0x8E, 0x67), (0xB7CA)),  # HANGUL SYLLABLE RIEUL YA CIEUC
    ((0x8E, 0x68), (0xB7CB)),  # HANGUL SYLLABLE RIEUL YA CHIEUCH
    ((0x8E, 0x69), (0xB7CC)),  # HANGUL SYLLABLE RIEUL YA KHIEUKH
    ((0x8E, 0x6A), (0xB7CD)),  # HANGUL SYLLABLE RIEUL YA THIEUTH
    ((0x8E, 0x6B), (0xB7CE)),  # HANGUL SYLLABLE RIEUL YA PHIEUPH
    ((0x8E, 0x6C), (0xB7CF)),  # HANGUL SYLLABLE RIEUL YA HIEUH
    ((0x8E, 0x6D), (0xB7D0)),  # HANGUL SYLLABLE RIEUL YAE
    ((0x8E, 0x6E), (0xB7D1)),  # HANGUL SYLLABLE RIEUL YAE KIYEOK
    ((0x8E, 0x6F), (0xB7D2)),  # HANGUL SYLLABLE RIEUL YAE SSANGKIYEOK
    ((0x8E, 0x70), (0xB7D3)),  # HANGUL SYLLABLE RIEUL YAE KIYEOKSIOS
    ((0x8E, 0x71), (0xB7D4)),  # HANGUL SYLLABLE RIEUL YAE NIEUN
    ((0x8E, 0x72), (0xB7D5)),  # HANGUL SYLLABLE RIEUL YAE NIEUNCIEUC
    ((0x8E, 0x73), (0xB7D6)),  # HANGUL SYLLABLE RIEUL YAE NIEUNHIEUH
    ((0x8E, 0x74), (0xB7D7)),  # HANGUL SYLLABLE RIEUL YAE TIKEUT
    ((0x8E, 0x75), (0xB7D8)),  # HANGUL SYLLABLE RIEUL YAE RIEUL
    ((0x8E, 0x76), (0xB7D9)),  # HANGUL SYLLABLE RIEUL YAE RIEULKIYEOK
    ((0x8E, 0x77), (0xB7DA)),  # HANGUL SYLLABLE RIEUL YAE RIEULMIEUM
    ((0x8E, 0x78), (0xB7DB)),  # HANGUL SYLLABLE RIEUL YAE RIEULPIEUP
    ((0x8E, 0x79), (0xB7DC)),  # HANGUL SYLLABLE RIEUL YAE RIEULSIOS
    ((0x8E, 0x7A), (0xB7DD)),  # HANGUL SYLLABLE RIEUL YAE RIEULTHIEUTH
    ((0x8E, 0x81), (0xB7DE)),  # HANGUL SYLLABLE RIEUL YAE RIEULPHIEUPH
    ((0x8E, 0x82), (0xB7DF)),  # HANGUL SYLLABLE RIEUL YAE RIEULHIEUH
    ((0x8E, 0x83), (0xB7E0)),  # HANGUL SYLLABLE RIEUL YAE MIEUM
    ((0x8E, 0x84), (0xB7E1)),  # HANGUL SYLLABLE RIEUL YAE PIEUP
    ((0x8E, 0x85), (0xB7E2)),  # HANGUL SYLLABLE RIEUL YAE PIEUPSIOS
    ((0x8E, 0x86), (0xB7E3)),  # HANGUL SYLLABLE RIEUL YAE SIOS
    ((0x8E, 0x87), (0xB7E4)),  # HANGUL SYLLABLE RIEUL YAE SSANGSIOS
    ((0x8E, 0x88), (0xB7E5)),  # HANGUL SYLLABLE RIEUL YAE IEUNG
    ((0x8E, 0x89), (0xB7E6)),  # HANGUL SYLLABLE RIEUL YAE CIEUC
    ((0x8E, 0x8A), (0xB7E7)),  # HANGUL SYLLABLE RIEUL YAE CHIEUCH
    ((0x8E, 0x8B), (0xB7E8)),  # HANGUL SYLLABLE RIEUL YAE KHIEUKH
    ((0x8E, 0x8C), (0xB7E9)),  # HANGUL SYLLABLE RIEUL YAE THIEUTH
    ((0x8E, 0x8D), (0xB7EA)),  # HANGUL SYLLABLE RIEUL YAE PHIEUPH
    ((0x8E, 0x8E), (0xB7EB)),  # HANGUL SYLLABLE RIEUL YAE HIEUH
    ((0x8E, 0x8F), (0xB7EE)),  # HANGUL SYLLABLE RIEUL EO SSANGKIYEOK
    ((0x8E, 0x90), (0xB7EF)),  # HANGUL SYLLABLE RIEUL EO KIYEOKSIOS
    ((0x8E, 0x91), (0xB7F1)),  # HANGUL SYLLABLE RIEUL EO NIEUNCIEUC
    ((0x8E, 0x92), (0xB7F2)),  # HANGUL SYLLABLE RIEUL EO NIEUNHIEUH
    ((0x8E, 0x93), (0xB7F3)),  # HANGUL SYLLABLE RIEUL EO TIKEUT
    ((0x8E, 0x94), (0xB7F5)),  # HANGUL SYLLABLE RIEUL EO RIEULKIYEOK
    ((0x8E, 0x95), (0xB7F6)),  # HANGUL SYLLABLE RIEUL EO RIEULMIEUM
    ((0x8E, 0x96), (0xB7F7)),  # HANGUL SYLLABLE RIEUL EO RIEULPIEUP
    ((0x8E, 0x97), (0xB7F8)),  # HANGUL SYLLABLE RIEUL EO RIEULSIOS
    ((0x8E, 0x98), (0xB7F9)),  # HANGUL SYLLABLE RIEUL EO RIEULTHIEUTH
    ((0x8E, 0x99), (0xB7FA)),  # HANGUL SYLLABLE RIEUL EO RIEULPHIEUPH
    ((0x8E, 0x9A), (0xB7FB)),  # HANGUL SYLLABLE RIEUL EO RIEULHIEUH
    ((0x8E, 0x9B), (0xB7FE)),  # HANGUL SYLLABLE RIEUL EO PIEUPSIOS
    ((0x8E, 0x9C), (0xB802)),  # HANGUL SYLLABLE RIEUL EO CIEUC
    ((0x8E, 0x9D), (0xB803)),  # HANGUL SYLLABLE RIEUL EO CHIEUCH
    ((0x8E, 0x9E), (0xB804)),  # HANGUL SYLLABLE RIEUL EO KHIEUKH
    ((0x8E, 0x9F), (0xB805)),  # HANGUL SYLLABLE RIEUL EO THIEUTH
    ((0x8E, 0xA0), (0xB806)),  # HANGUL SYLLABLE RIEUL EO PHIEUPH
    ((0x8E, 0xA1), (0xB80A)),  # HANGUL SYLLABLE RIEUL E SSANGKIYEOK
    ((0x8E, 0xA2), (0xB80B)),  # HANGUL SYLLABLE RIEUL E KIYEOKSIOS
    ((0x8E, 0xA3), (0xB80D)),  # HANGUL SYLLABLE RIEUL E NIEUNCIEUC
    ((0x8E, 0xA4), (0xB80E)),  # HANGUL SYLLABLE RIEUL E NIEUNHIEUH
    ((0x8E, 0xA5), (0xB80F)),  # HANGUL SYLLABLE RIEUL E TIKEUT
    ((0x8E, 0xA6), (0xB811)),  # HANGUL SYLLABLE RIEUL E RIEULKIYEOK
    ((0x8E, 0xA7), (0xB812)),  # HANGUL SYLLABLE RIEUL E RIEULMIEUM
    ((0x8E, 0xA8), (0xB813)),  # HANGUL SYLLABLE RIEUL E RIEULPIEUP
    ((0x8E, 0xA9), (0xB814)),  # HANGUL SYLLABLE RIEUL E RIEULSIOS
    ((0x8E, 0xAA), (0xB815)),  # HANGUL SYLLABLE RIEUL E RIEULTHIEUTH
    ((0x8E, 0xAB), (0xB816)),  # HANGUL SYLLABLE RIEUL E RIEULPHIEUPH
    ((0x8E, 0xAC), (0xB817)),  # HANGUL SYLLABLE RIEUL E RIEULHIEUH
    ((0x8E, 0xAD), (0xB81A)),  # HANGUL SYLLABLE RIEUL E PIEUPSIOS
    ((0x8E, 0xAE), (0xB81C)),  # HANGUL SYLLABLE RIEUL E SSANGSIOS
    ((0x8E, 0xAF), (0xB81E)),  # HANGUL SYLLABLE RIEUL E CIEUC
    ((0x8E, 0xB0), (0xB81F)),  # HANGUL SYLLABLE RIEUL E CHIEUCH
    ((0x8E, 0xB1), (0xB820)),  # HANGUL SYLLABLE RIEUL E KHIEUKH
    ((0x8E, 0xB2), (0xB821)),  # HANGUL SYLLABLE RIEUL E THIEUTH
    ((0x8E, 0xB3), (0xB822)),  # HANGUL SYLLABLE RIEUL E PHIEUPH
    ((0x8E, 0xB4), (0xB823)),  # HANGUL SYLLABLE RIEUL E HIEUH
    ((0x8E, 0xB5), (0xB826)),  # HANGUL SYLLABLE RIEUL YEO SSANGKIYEOK
    ((0x8E, 0xB6), (0xB827)),  # HANGUL SYLLABLE RIEUL YEO KIYEOKSIOS
    ((0x8E, 0xB7), (0xB829)),  # HANGUL SYLLABLE RIEUL YEO NIEUNCIEUC
    ((0x8E, 0xB8), (0xB82A)),  # HANGUL SYLLABLE RIEUL YEO NIEUNHIEUH
    ((0x8E, 0xB9), (0xB82B)),  # HANGUL SYLLABLE RIEUL YEO TIKEUT
    ((0x8E, 0xBA), (0xB82D)),  # HANGUL SYLLABLE RIEUL YEO RIEULKIYEOK
    ((0x8E, 0xBB), (0xB82E)),  # HANGUL SYLLABLE RIEUL YEO RIEULMIEUM
    ((0x8E, 0xBC), (0xB82F)),  # HANGUL SYLLABLE RIEUL YEO RIEULPIEUP
    ((0x8E, 0xBD), (0xB830)),  # HANGUL SYLLABLE RIEUL YEO RIEULSIOS
    ((0x8E, 0xBE), (0xB831)),  # HANGUL SYLLABLE RIEUL YEO RIEULTHIEUTH
    ((0x8E, 0xBF), (0xB832)),  # HANGUL SYLLABLE RIEUL YEO RIEULPHIEUPH
    ((0x8E, 0xC0), (0xB833)),  # HANGUL SYLLABLE RIEUL YEO RIEULHIEUH
    ((0x8E, 0xC1), (0xB836)),  # HANGUL SYLLABLE RIEUL YEO PIEUPSIOS
    ((0x8E, 0xC2), (0xB83A)),  # HANGUL SYLLABLE RIEUL YEO CIEUC
    ((0x8E, 0xC3), (0xB83B)),  # HANGUL SYLLABLE RIEUL YEO CHIEUCH
    ((0x8E, 0xC4), (0xB83C)),  # HANGUL SYLLABLE RIEUL YEO KHIEUKH
    ((0x8E, 0xC5), (0xB83D)),  # HANGUL SYLLABLE RIEUL YEO THIEUTH
    ((0x8E, 0xC6), (0xB83E)),  # HANGUL SYLLABLE RIEUL YEO PHIEUPH
    ((0x8E, 0xC7), (0xB83F)),  # HANGUL SYLLABLE RIEUL YEO HIEUH
    ((0x8E, 0xC8), (0xB841)),  # HANGUL SYLLABLE RIEUL YE KIYEOK
    ((0x8E, 0xC9), (0xB842)),  # HANGUL SYLLABLE RIEUL YE SSANGKIYEOK
    ((0x8E, 0xCA), (0xB843)),  # HANGUL SYLLABLE RIEUL YE KIYEOKSIOS
    ((0x8E, 0xCB), (0xB845)),  # HANGUL SYLLABLE RIEUL YE NIEUNCIEUC
    ((0x8E, 0xCC), (0xB846)),  # HANGUL SYLLABLE RIEUL YE NIEUNHIEUH
    ((0x8E, 0xCD), (0xB847)),  # HANGUL SYLLABLE RIEUL YE TIKEUT
    ((0x8E, 0xCE), (0xB848)),  # HANGUL SYLLABLE RIEUL YE RIEUL
    ((0x8E, 0xCF), (0xB849)),  # HANGUL SYLLABLE RIEUL YE RIEULKIYEOK
    ((0x8E, 0xD0), (0xB84A)),  # HANGUL SYLLABLE RIEUL YE RIEULMIEUM
    ((0x8E, 0xD1), (0xB84B)),  # HANGUL SYLLABLE RIEUL YE RIEULPIEUP
    ((0x8E, 0xD2), (0xB84C)),  # HANGUL SYLLABLE RIEUL YE RIEULSIOS
    ((0x8E, 0xD3), (0xB84D)),  # HANGUL SYLLABLE RIEUL YE RIEULTHIEUTH
    ((0x8E, 0xD4), (0xB84E)),  # HANGUL SYLLABLE RIEUL YE RIEULPHIEUPH
    ((0x8E, 0xD5), (0xB84F)),  # HANGUL SYLLABLE RIEUL YE RIEULHIEUH
    ((0x8E, 0xD6), (0xB850)),  # HANGUL SYLLABLE RIEUL YE MIEUM
    ((0x8E, 0xD7), (0xB852)),  # HANGUL SYLLABLE RIEUL YE PIEUPSIOS
    ((0x8E, 0xD8), (0xB854)),  # HANGUL SYLLABLE RIEUL YE SSANGSIOS
    ((0x8E, 0xD9), (0xB855)),  # HANGUL SYLLABLE RIEUL YE IEUNG
    ((0x8E, 0xDA), (0xB856)),  # HANGUL SYLLABLE RIEUL YE CIEUC
    ((0x8E, 0xDB), (0xB857)),  # HANGUL SYLLABLE RIEUL YE CHIEUCH
    ((0x8E, 0xDC), (0xB858)),  # HANGUL SYLLABLE RIEUL YE KHIEUKH
    ((0x8E, 0xDD), (0xB859)),  # HANGUL SYLLABLE RIEUL YE THIEUTH
    ((0x8E, 0xDE), (0xB85A)),  # HANGUL SYLLABLE RIEUL YE PHIEUPH
    ((0x8E, 0xDF), (0xB85B)),  # HANGUL SYLLABLE RIEUL YE HIEUH
    ((0x8E, 0xE0), (0xB85E)),  # HANGUL SYLLABLE RIEUL O SSANGKIYEOK
    ((0x8E, 0xE1), (0xB85F)),  # HANGUL SYLLABLE RIEUL O KIYEOKSIOS
    ((0x8E, 0xE2), (0xB861)),  # HANGUL SYLLABLE RIEUL O NIEUNCIEUC
    ((0x8E, 0xE3), (0xB862)),  # HANGUL SYLLABLE RIEUL O NIEUNHIEUH
    ((0x8E, 0xE4), (0xB863)),  # HANGUL SYLLABLE RIEUL O TIKEUT
    ((0x8E, 0xE5), (0xB865)),  # HANGUL SYLLABLE RIEUL O RIEULKIYEOK
    ((0x8E, 0xE6), (0xB866)),  # HANGUL SYLLABLE RIEUL O RIEULMIEUM
    ((0x8E, 0xE7), (0xB867)),  # HANGUL SYLLABLE RIEUL O RIEULPIEUP
    ((0x8E, 0xE8), (0xB868)),  # HANGUL SYLLABLE RIEUL O RIEULSIOS
    ((0x8E, 0xE9), (0xB869)),  # HANGUL SYLLABLE RIEUL O RIEULTHIEUTH
    ((0x8E, 0xEA), (0xB86A)),  # HANGUL SYLLABLE RIEUL O RIEULPHIEUPH
    ((0x8E, 0xEB), (0xB86B)),  # HANGUL SYLLABLE RIEUL O RIEULHIEUH
    ((0x8E, 0xEC), (0xB86E)),  # HANGUL SYLLABLE RIEUL O PIEUPSIOS
    ((0x8E, 0xED), (0xB870)),  # HANGUL SYLLABLE RIEUL O SSANGSIOS
    ((0x8E, 0xEE), (0xB872)),  # HANGUL SYLLABLE RIEUL O CIEUC
    ((0x8E, 0xEF), (0xB873)),  # HANGUL SYLLABLE RIEUL O CHIEUCH
    ((0x8E, 0xF0), (0xB874)),  # HANGUL SYLLABLE RIEUL O KHIEUKH
    ((0x8E, 0xF1), (0xB875)),  # HANGUL SYLLABLE RIEUL O THIEUTH
    ((0x8E, 0xF2), (0xB876)),  # HANGUL SYLLABLE RIEUL O PHIEUPH
    ((0x8E, 0xF3), (0xB877)),  # HANGUL SYLLABLE RIEUL O HIEUH
    ((0x8E, 0xF4), (0xB879)),  # HANGUL SYLLABLE RIEUL WA KIYEOK
    ((0x8E, 0xF5), (0xB87A)),  # HANGUL SYLLABLE RIEUL WA SSANGKIYEOK
    ((0x8E, 0xF6), (0xB87B)),  # HANGUL SYLLABLE RIEUL WA KIYEOKSIOS
    ((0x8E, 0xF7), (0xB87D)),  # HANGUL SYLLABLE RIEUL WA NIEUNCIEUC
    ((0x8E, 0xF8), (0xB87E)),  # HANGUL SYLLABLE RIEUL WA NIEUNHIEUH
    ((0x8E, 0xF9), (0xB87F)),  # HANGUL SYLLABLE RIEUL WA TIKEUT
    ((0x8E, 0xFA), (0xB880)),  # HANGUL SYLLABLE RIEUL WA RIEUL
    ((0x8E, 0xFB), (0xB881)),  # HANGUL SYLLABLE RIEUL WA RIEULKIYEOK
    ((0x8E, 0xFC), (0xB882)),  # HANGUL SYLLABLE RIEUL WA RIEULMIEUM
    ((0x8E, 0xFD), (0xB883)),  # HANGUL SYLLABLE RIEUL WA RIEULPIEUP
    ((0x8E, 0xFE), (0xB884)),  # HANGUL SYLLABLE RIEUL WA RIEULSIOS
    ((0x8F, 0x41), (0xB885)),  # HANGUL SYLLABLE RIEUL WA RIEULTHIEUTH
    ((0x8F, 0x42), (0xB886)),  # HANGUL SYLLABLE RIEUL WA RIEULPHIEUPH
    ((0x8F, 0x43), (0xB887)),  # HANGUL SYLLABLE RIEUL WA RIEULHIEUH
    ((0x8F, 0x44), (0xB888)),  # HANGUL SYLLABLE RIEUL WA MIEUM
    ((0x8F, 0x45), (0xB889)),  # HANGUL SYLLABLE RIEUL WA PIEUP
    ((0x8F, 0x46), (0xB88A)),  # HANGUL SYLLABLE RIEUL WA PIEUPSIOS
    ((0x8F, 0x47), (0xB88B)),  # HANGUL SYLLABLE RIEUL WA SIOS
    ((0x8F, 0x48), (0xB88C)),  # HANGUL SYLLABLE RIEUL WA SSANGSIOS
    ((0x8F, 0x49), (0xB88E)),  # HANGUL SYLLABLE RIEUL WA CIEUC
    ((0x8F, 0x4A), (0xB88F)),  # HANGUL SYLLABLE RIEUL WA CHIEUCH
    ((0x8F, 0x4B), (0xB890)),  # HANGUL SYLLABLE RIEUL WA KHIEUKH
    ((0x8F, 0x4C), (0xB891)),  # HANGUL SYLLABLE RIEUL WA THIEUTH
    ((0x8F, 0x4D), (0xB892)),  # HANGUL SYLLABLE RIEUL WA PHIEUPH
    ((0x8F, 0x4E), (0xB893)),  # HANGUL SYLLABLE RIEUL WA HIEUH
    ((0x8F, 0x4F), (0xB894)),  # HANGUL SYLLABLE RIEUL WAE
    ((0x8F, 0x50), (0xB895)),  # HANGUL SYLLABLE RIEUL WAE KIYEOK
    ((0x8F, 0x51), (0xB896)),  # HANGUL SYLLABLE RIEUL WAE SSANGKIYEOK
    ((0x8F, 0x52), (0xB897)),  # HANGUL SYLLABLE RIEUL WAE KIYEOKSIOS
    ((0x8F, 0x53), (0xB898)),  # HANGUL SYLLABLE RIEUL WAE NIEUN
    ((0x8F, 0x54), (0xB899)),  # HANGUL SYLLABLE RIEUL WAE NIEUNCIEUC
    ((0x8F, 0x55), (0xB89A)),  # HANGUL SYLLABLE RIEUL WAE NIEUNHIEUH
    ((0x8F, 0x56), (0xB89B)),  # HANGUL SYLLABLE RIEUL WAE TIKEUT
    ((0x8F, 0x57), (0xB89C)),  # HANGUL SYLLABLE RIEUL WAE RIEUL
    ((0x8F, 0x58), (0xB89D)),  # HANGUL SYLLABLE RIEUL WAE RIEULKIYEOK
    ((0x8F, 0x59), (0xB89E)),  # HANGUL SYLLABLE RIEUL WAE RIEULMIEUM
    ((0x8F, 0x5A), (0xB89F)),  # HANGUL SYLLABLE RIEUL WAE RIEULPIEUP
    ((0x8F, 0x61), (0xB8A0)),  # HANGUL SYLLABLE RIEUL WAE RIEULSIOS
    ((0x8F, 0x62), (0xB8A1)),  # HANGUL SYLLABLE RIEUL WAE RIEULTHIEUTH
    ((0x8F, 0x63), (0xB8A2)),  # HANGUL SYLLABLE RIEUL WAE RIEULPHIEUPH
    ((0x8F, 0x64), (0xB8A3)),  # HANGUL SYLLABLE RIEUL WAE RIEULHIEUH
    ((0x8F, 0x65), (0xB8A4)),  # HANGUL SYLLABLE RIEUL WAE MIEUM
    ((0x8F, 0x66), (0xB8A5)),  # HANGUL SYLLABLE RIEUL WAE PIEUP
    ((0x8F, 0x67), (0xB8A6)),  # HANGUL SYLLABLE RIEUL WAE PIEUPSIOS
    ((0x8F, 0x68), (0xB8A7)),  # HANGUL SYLLABLE RIEUL WAE SIOS
    ((0x8F, 0x69), (0xB8A9)),  # HANGUL SYLLABLE RIEUL WAE IEUNG
    ((0x8F, 0x6A), (0xB8AA)),  # HANGUL SYLLABLE RIEUL WAE CIEUC
    ((0x8F, 0x6B), (0xB8AB)),  # HANGUL SYLLABLE RIEUL WAE CHIEUCH
    ((0x8F, 0x6C), (0xB8AC)),  # HANGUL SYLLABLE RIEUL WAE KHIEUKH
    ((0x8F, 0x6D), (0xB8AD)),  # HANGUL SYLLABLE RIEUL WAE THIEUTH
    ((0x8F, 0x6E), (0xB8AE)),  # HANGUL SYLLABLE RIEUL WAE PHIEUPH
    ((0x8F, 0x6F), (0xB8AF)),  # HANGUL SYLLABLE RIEUL WAE HIEUH
    ((0x8F, 0x70), (0xB8B1)),  # HANGUL SYLLABLE RIEUL OE KIYEOK
    ((0x8F, 0x71), (0xB8B2)),  # HANGUL SYLLABLE RIEUL OE SSANGKIYEOK
    ((0x8F, 0x72), (0xB8B3)),  # HANGUL SYLLABLE RIEUL OE KIYEOKSIOS
    ((0x8F, 0x73), (0xB8B5)),  # HANGUL SYLLABLE RIEUL OE NIEUNCIEUC
    ((0x8F, 0x74), (0xB8B6)),  # HANGUL SYLLABLE RIEUL OE NIEUNHIEUH
    ((0x8F, 0x75), (0xB8B7)),  # HANGUL SYLLABLE RIEUL OE TIKEUT
    ((0x8F, 0x76), (0xB8B9)),  # HANGUL SYLLABLE RIEUL OE RIEULKIYEOK
    ((0x8F, 0x77), (0xB8BA)),  # HANGUL SYLLABLE RIEUL OE RIEULMIEUM
    ((0x8F, 0x78), (0xB8BB)),  # HANGUL SYLLABLE RIEUL OE RIEULPIEUP
    ((0x8F, 0x79), (0xB8BC)),  # HANGUL SYLLABLE RIEUL OE RIEULSIOS
    ((0x8F, 0x7A), (0xB8BD)),  # HANGUL SYLLABLE RIEUL OE RIEULTHIEUTH
    ((0x8F, 0x81), (0xB8BE)),  # HANGUL SYLLABLE RIEUL OE RIEULPHIEUPH
    ((0x8F, 0x82), (0xB8BF)),  # HANGUL SYLLABLE RIEUL OE RIEULHIEUH
    ((0x8F, 0x83), (0xB8C2)),  # HANGUL SYLLABLE RIEUL OE PIEUPSIOS
    ((0x8F, 0x84), (0xB8C4)),  # HANGUL SYLLABLE RIEUL OE SSANGSIOS
    ((0x8F, 0x85), (0xB8C6)),  # HANGUL SYLLABLE RIEUL OE CIEUC
    ((0x8F, 0x86), (0xB8C7)),  # HANGUL SYLLABLE RIEUL OE CHIEUCH
    ((0x8F, 0x87), (0xB8C8)),  # HANGUL SYLLABLE RIEUL OE KHIEUKH
    ((0x8F, 0x88), (0xB8C9)),  # HANGUL SYLLABLE RIEUL OE THIEUTH
    ((0x8F, 0x89), (0xB8CA)),  # HANGUL SYLLABLE RIEUL OE PHIEUPH
    ((0x8F, 0x8A), (0xB8CB)),  # HANGUL SYLLABLE RIEUL OE HIEUH
    ((0x8F, 0x8B), (0xB8CD)),  # HANGUL SYLLABLE RIEUL YO KIYEOK
    ((0x8F, 0x8C), (0xB8CE)),  # HANGUL SYLLABLE RIEUL YO SSANGKIYEOK
    ((0x8F, 0x8D), (0xB8CF)),  # HANGUL SYLLABLE RIEUL YO KIYEOKSIOS
    ((0x8F, 0x8E), (0xB8D1)),  # HANGUL SYLLABLE RIEUL YO NIEUNCIEUC
    ((0x8F, 0x8F), (0xB8D2)),  # HANGUL SYLLABLE RIEUL YO NIEUNHIEUH
    ((0x8F, 0x90), (0xB8D3)),  # HANGUL SYLLABLE RIEUL YO TIKEUT
    ((0x8F, 0x91), (0xB8D5)),  # HANGUL SYLLABLE RIEUL YO RIEULKIYEOK
    ((0x8F, 0x92), (0xB8D6)),  # HANGUL SYLLABLE RIEUL YO RIEULMIEUM
    ((0x8F, 0x93), (0xB8D7)),  # HANGUL SYLLABLE RIEUL YO RIEULPIEUP
    ((0x8F, 0x94), (0xB8D8)),  # HANGUL SYLLABLE RIEUL YO RIEULSIOS
    ((0x8F, 0x95), (0xB8D9)),  # HANGUL SYLLABLE RIEUL YO RIEULTHIEUTH
    ((0x8F, 0x96), (0xB8DA)),  # HANGUL SYLLABLE RIEUL YO RIEULPHIEUPH
    ((0x8F, 0x97), (0xB8DB)),  # HANGUL SYLLABLE RIEUL YO RIEULHIEUH
    ((0x8F, 0x98), (0xB8DC)),  # HANGUL SYLLABLE RIEUL YO MIEUM
    ((0x8F, 0x99), (0xB8DE)),  # HANGUL SYLLABLE RIEUL YO PIEUPSIOS
    ((0x8F, 0x9A), (0xB8E0)),  # HANGUL SYLLABLE RIEUL YO SSANGSIOS
    ((0x8F, 0x9B), (0xB8E2)),  # HANGUL SYLLABLE RIEUL YO CIEUC
    ((0x8F, 0x9C), (0xB8E3)),  # HANGUL SYLLABLE RIEUL YO CHIEUCH
    ((0x8F, 0x9D), (0xB8E4)),  # HANGUL SYLLABLE RIEUL YO KHIEUKH
    ((0x8F, 0x9E), (0xB8E5)),  # HANGUL SYLLABLE RIEUL YO THIEUTH
    ((0x8F, 0x9F), (0xB8E6)),  # HANGUL SYLLABLE RIEUL YO PHIEUPH
    ((0x8F, 0xA0), (0xB8E7)),  # HANGUL SYLLABLE RIEUL YO HIEUH
    ((0x8F, 0xA1), (0xB8EA)),  # HANGUL SYLLABLE RIEUL U SSANGKIYEOK
    ((0x8F, 0xA2), (0xB8EB)),  # HANGUL SYLLABLE RIEUL U KIYEOKSIOS
    ((0x8F, 0xA3), (0xB8ED)),  # HANGUL SYLLABLE RIEUL U NIEUNCIEUC
    ((0x8F, 0xA4), (0xB8EE)),  # HANGUL SYLLABLE RIEUL U NIEUNHIEUH
    ((0x8F, 0xA5), (0xB8EF)),  # HANGUL SYLLABLE RIEUL U TIKEUT
    ((0x8F, 0xA6), (0xB8F1)),  # HANGUL SYLLABLE RIEUL U RIEULKIYEOK
    ((0x8F, 0xA7), (0xB8F2)),  # HANGUL SYLLABLE RIEUL U RIEULMIEUM
    ((0x8F, 0xA8), (0xB8F3)),  # HANGUL SYLLABLE RIEUL U RIEULPIEUP
    ((0x8F, 0xA9), (0xB8F4)),  # HANGUL SYLLABLE RIEUL U RIEULSIOS
    ((0x8F, 0xAA), (0xB8F5)),  # HANGUL SYLLABLE RIEUL U RIEULTHIEUTH
    ((0x8F, 0xAB), (0xB8F6)),  # HANGUL SYLLABLE RIEUL U RIEULPHIEUPH
    ((0x8F, 0xAC), (0xB8F7)),  # HANGUL SYLLABLE RIEUL U RIEULHIEUH
    ((0x8F, 0xAD), (0xB8FA)),  # HANGUL SYLLABLE RIEUL U PIEUPSIOS
    ((0x8F, 0xAE), (0xB8FC)),  # HANGUL SYLLABLE RIEUL U SSANGSIOS
    ((0x8F, 0xAF), (0xB8FE)),  # HANGUL SYLLABLE RIEUL U CIEUC
    ((0x8F, 0xB0), (0xB8FF)),  # HANGUL SYLLABLE RIEUL U CHIEUCH
    ((0x8F, 0xB1), (0xB900)),  # HANGUL SYLLABLE RIEUL U KHIEUKH
    ((0x8F, 0xB2), (0xB901)),  # HANGUL SYLLABLE RIEUL U THIEUTH
    ((0x8F, 0xB3), (0xB902)),  # HANGUL SYLLABLE RIEUL U PHIEUPH
    ((0x8F, 0xB4), (0xB903)),  # HANGUL SYLLABLE RIEUL U HIEUH
    ((0x8F, 0xB5), (0xB905)),  # HANGUL SYLLABLE RIEUL WEO KIYEOK
    ((0x8F, 0xB6), (0xB906)),  # HANGUL SYLLABLE RIEUL WEO SSANGKIYEOK
    ((0x8F, 0xB7), (0xB907)),  # HANGUL SYLLABLE RIEUL WEO KIYEOKSIOS
    ((0x8F, 0xB8), (0xB908)),  # HANGUL SYLLABLE RIEUL WEO NIEUN
    ((0x8F, 0xB9), (0xB909)),  # HANGUL SYLLABLE RIEUL WEO NIEUNCIEUC
    ((0x8F, 0xBA), (0xB90A)),  # HANGUL SYLLABLE RIEUL WEO NIEUNHIEUH
    ((0x8F, 0xBB), (0xB90B)),  # HANGUL SYLLABLE RIEUL WEO TIKEUT
    ((0x8F, 0xBC), (0xB90C)),  # HANGUL SYLLABLE RIEUL WEO RIEUL
    ((0x8F, 0xBD), (0xB90D)),  # HANGUL SYLLABLE RIEUL WEO RIEULKIYEOK
    ((0x8F, 0xBE), (0xB90E)),  # HANGUL SYLLABLE RIEUL WEO RIEULMIEUM
    ((0x8F, 0xBF), (0xB90F)),  # HANGUL SYLLABLE RIEUL WEO RIEULPIEUP
    ((0x8F, 0xC0), (0xB910)),  # HANGUL SYLLABLE RIEUL WEO RIEULSIOS
    ((0x8F, 0xC1), (0xB911)),  # HANGUL SYLLABLE RIEUL WEO RIEULTHIEUTH
    ((0x8F, 0xC2), (0xB912)),  # HANGUL SYLLABLE RIEUL WEO RIEULPHIEUPH
    ((0x8F, 0xC3), (0xB913)),  # HANGUL SYLLABLE RIEUL WEO RIEULHIEUH
    ((0x8F, 0xC4), (0xB914)),  # HANGUL SYLLABLE RIEUL WEO MIEUM
    ((0x8F, 0xC5), (0xB915)),  # HANGUL SYLLABLE RIEUL WEO PIEUP
    ((0x8F, 0xC6), (0xB916)),  # HANGUL SYLLABLE RIEUL WEO PIEUPSIOS
    ((0x8F, 0xC7), (0xB917)),  # HANGUL SYLLABLE RIEUL WEO SIOS
    ((0x8F, 0xC8), (0xB919)),  # HANGUL SYLLABLE RIEUL WEO IEUNG
    ((0x8F, 0xC9), (0xB91A)),  # HANGUL SYLLABLE RIEUL WEO CIEUC
    ((0x8F, 0xCA), (0xB91B)),  # HANGUL SYLLABLE RIEUL WEO CHIEUCH
    ((0x8F, 0xCB), (0xB91C)),  # HANGUL SYLLABLE RIEUL WEO KHIEUKH
    ((0x8F, 0xCC), (0xB91D)),  # HANGUL SYLLABLE RIEUL WEO THIEUTH
    ((0x8F, 0xCD), (0xB91E)),  # HANGUL SYLLABLE RIEUL WEO PHIEUPH
    ((0x8F, 0xCE), (0xB91F)),  # HANGUL SYLLABLE RIEUL WEO HIEUH
    ((0x8F, 0xCF), (0xB921)),  # HANGUL SYLLABLE RIEUL WE KIYEOK
    ((0x8F, 0xD0), (0xB922)),  # HANGUL SYLLABLE RIEUL WE SSANGKIYEOK
    ((0x8F, 0xD1), (0xB923)),  # HANGUL SYLLABLE RIEUL WE KIYEOKSIOS
    ((0x8F, 0xD2), (0xB924)),  # HANGUL SYLLABLE RIEUL WE NIEUN
    ((0x8F, 0xD3), (0xB925)),  # HANGUL SYLLABLE RIEUL WE NIEUNCIEUC
    ((0x8F, 0xD4), (0xB926)),  # HANGUL SYLLABLE RIEUL WE NIEUNHIEUH
    ((0x8F, 0xD5), (0xB927)),  # HANGUL SYLLABLE RIEUL WE TIKEUT
    ((0x8F, 0xD6), (0xB928)),  # HANGUL SYLLABLE RIEUL WE RIEUL
    ((0x8F, 0xD7), (0xB929)),  # HANGUL SYLLABLE RIEUL WE RIEULKIYEOK
    ((0x8F, 0xD8), (0xB92A)),  # HANGUL SYLLABLE RIEUL WE RIEULMIEUM
    ((0x8F, 0xD9), (0xB92B)),  # HANGUL SYLLABLE RIEUL WE RIEULPIEUP
    ((0x8F, 0xDA), (0xB92C)),  # HANGUL SYLLABLE RIEUL WE RIEULSIOS
    ((0x8F, 0xDB), (0xB92D)),  # HANGUL SYLLABLE RIEUL WE RIEULTHIEUTH
    ((0x8F, 0xDC), (0xB92E)),  # HANGUL SYLLABLE RIEUL WE RIEULPHIEUPH
    ((0x8F, 0xDD), (0xB92F)),  # HANGUL SYLLABLE RIEUL WE RIEULHIEUH
    ((0x8F, 0xDE), (0xB930)),  # HANGUL SYLLABLE RIEUL WE MIEUM
    ((0x8F, 0xDF), (0xB931)),  # HANGUL SYLLABLE RIEUL WE PIEUP
    ((0x8F, 0xE0), (0xB932)),  # HANGUL SYLLABLE RIEUL WE PIEUPSIOS
    ((0x8F, 0xE1), (0xB933)),  # HANGUL SYLLABLE RIEUL WE SIOS
    ((0x8F, 0xE2), (0xB934)),  # HANGUL SYLLABLE RIEUL WE SSANGSIOS
    ((0x8F, 0xE3), (0xB935)),  # HANGUL SYLLABLE RIEUL WE IEUNG
    ((0x8F, 0xE4), (0xB936)),  # HANGUL SYLLABLE RIEUL WE CIEUC
    ((0x8F, 0xE5), (0xB937)),  # HANGUL SYLLABLE RIEUL WE CHIEUCH
    ((0x8F, 0xE6), (0xB938)),  # HANGUL SYLLABLE RIEUL WE KHIEUKH
    ((0x8F, 0xE7), (0xB939)),  # HANGUL SYLLABLE RIEUL WE THIEUTH
    ((0x8F, 0xE8), (0xB93A)),  # HANGUL SYLLABLE RIEUL WE PHIEUPH
    ((0x8F, 0xE9), (0xB93B)),  # HANGUL SYLLABLE RIEUL WE HIEUH
    ((0x8F, 0xEA), (0xB93E)),  # HANGUL SYLLABLE RIEUL WI SSANGKIYEOK
    ((0x8F, 0xEB), (0xB93F)),  # HANGUL SYLLABLE RIEUL WI KIYEOKSIOS
    ((0x8F, 0xEC), (0xB941)),  # HANGUL SYLLABLE RIEUL WI NIEUNCIEUC
    ((0x8F, 0xED), (0xB942)),  # HANGUL SYLLABLE RIEUL WI NIEUNHIEUH
    ((0x8F, 0xEE), (0xB943)),  # HANGUL SYLLABLE RIEUL WI TIKEUT
    ((0x8F, 0xEF), (0xB945)),  # HANGUL SYLLABLE RIEUL WI RIEULKIYEOK
    ((0x8F, 0xF0), (0xB946)),  # HANGUL SYLLABLE RIEUL WI RIEULMIEUM
    ((0x8F, 0xF1), (0xB947)),  # HANGUL SYLLABLE RIEUL WI RIEULPIEUP
    ((0x8F, 0xF2), (0xB948)),  # HANGUL SYLLABLE RIEUL WI RIEULSIOS
    ((0x8F, 0xF3), (0xB949)),  # HANGUL SYLLABLE RIEUL WI RIEULTHIEUTH
    ((0x8F, 0xF4), (0xB94A)),  # HANGUL SYLLABLE RIEUL WI RIEULPHIEUPH
    ((0x8F, 0xF5), (0xB94B)),  # HANGUL SYLLABLE RIEUL WI RIEULHIEUH
    ((0x8F, 0xF6), (0xB94D)),  # HANGUL SYLLABLE RIEUL WI PIEUP
    ((0x8F, 0xF7), (0xB94E)),  # HANGUL SYLLABLE RIEUL WI PIEUPSIOS
    ((0x8F, 0xF8), (0xB950)),  # HANGUL SYLLABLE RIEUL WI SSANGSIOS
    ((0x8F, 0xF9), (0xB952)),  # HANGUL SYLLABLE RIEUL WI CIEUC
    ((0x8F, 0xFA), (0xB953)),  # HANGUL SYLLABLE RIEUL WI CHIEUCH
    ((0x8F, 0xFB), (0xB954)),  # HANGUL SYLLABLE RIEUL WI KHIEUKH
    ((0x8F, 0xFC), (0xB955)),  # HANGUL SYLLABLE RIEUL WI THIEUTH
    ((0x8F, 0xFD), (0xB956)),  # HANGUL SYLLABLE RIEUL WI PHIEUPH
    ((0x8F, 0xFE), (0xB957)),  # HANGUL SYLLABLE RIEUL WI HIEUH
    ((0x90, 0x41), (0xB95A)),  # HANGUL SYLLABLE RIEUL YU SSANGKIYEOK
    ((0x90, 0x42), (0xB95B)),  # HANGUL SYLLABLE RIEUL YU KIYEOKSIOS
    ((0x90, 0x43), (0xB95D)),  # HANGUL SYLLABLE RIEUL YU NIEUNCIEUC
    ((0x90, 0x44), (0xB95E)),  # HANGUL SYLLABLE RIEUL YU NIEUNHIEUH
    ((0x90, 0x45), (0xB95F)),  # HANGUL SYLLABLE RIEUL YU TIKEUT
    ((0x90, 0x46), (0xB961)),  # HANGUL SYLLABLE RIEUL YU RIEULKIYEOK
    ((0x90, 0x47), (0xB962)),  # HANGUL SYLLABLE RIEUL YU RIEULMIEUM
    ((0x90, 0x48), (0xB963)),  # HANGUL SYLLABLE RIEUL YU RIEULPIEUP
    ((0x90, 0x49), (0xB964)),  # HANGUL SYLLABLE RIEUL YU RIEULSIOS
    ((0x90, 0x4A), (0xB965)),  # HANGUL SYLLABLE RIEUL YU RIEULTHIEUTH
    ((0x90, 0x4B), (0xB966)),  # HANGUL SYLLABLE RIEUL YU RIEULPHIEUPH
    ((0x90, 0x4C), (0xB967)),  # HANGUL SYLLABLE RIEUL YU RIEULHIEUH
    ((0x90, 0x4D), (0xB96A)),  # HANGUL SYLLABLE RIEUL YU PIEUPSIOS
    ((0x90, 0x4E), (0xB96C)),  # HANGUL SYLLABLE RIEUL YU SSANGSIOS
    ((0x90, 0x4F), (0xB96E)),  # HANGUL SYLLABLE RIEUL YU CIEUC
    ((0x90, 0x50), (0xB96F)),  # HANGUL SYLLABLE RIEUL YU CHIEUCH
    ((0x90, 0x51), (0xB970)),  # HANGUL SYLLABLE RIEUL YU KHIEUKH
    ((0x90, 0x52), (0xB971)),  # HANGUL SYLLABLE RIEUL YU THIEUTH
    ((0x90, 0x53), (0xB972)),  # HANGUL SYLLABLE RIEUL YU PHIEUPH
    ((0x90, 0x54), (0xB973)),  # HANGUL SYLLABLE RIEUL YU HIEUH
    ((0x90, 0x55), (0xB976)),  # HANGUL SYLLABLE RIEUL EU SSANGKIYEOK
    ((0x90, 0x56), (0xB977)),  # HANGUL SYLLABLE RIEUL EU KIYEOKSIOS
    ((0x90, 0x57), (0xB979)),  # HANGUL SYLLABLE RIEUL EU NIEUNCIEUC
    ((0x90, 0x58), (0xB97A)),  # HANGUL SYLLABLE RIEUL EU NIEUNHIEUH
    ((0x90, 0x59), (0xB97B)),  # HANGUL SYLLABLE RIEUL EU TIKEUT
    ((0x90, 0x5A), (0xB97D)),  # HANGUL SYLLABLE RIEUL EU RIEULKIYEOK
    ((0x90, 0x61), (0xB97E)),  # HANGUL SYLLABLE RIEUL EU RIEULMIEUM
    ((0x90, 0x62), (0xB97F)),  # HANGUL SYLLABLE RIEUL EU RIEULPIEUP
    ((0x90, 0x63), (0xB980)),  # HANGUL SYLLABLE RIEUL EU RIEULSIOS
    ((0x90, 0x64), (0xB981)),  # HANGUL SYLLABLE RIEUL EU RIEULTHIEUTH
    ((0x90, 0x65), (0xB982)),  # HANGUL SYLLABLE RIEUL EU RIEULPHIEUPH
    ((0x90, 0x66), (0xB983)),  # HANGUL SYLLABLE RIEUL EU RIEULHIEUH
    ((0x90, 0x67), (0xB986)),  # HANGUL SYLLABLE RIEUL EU PIEUPSIOS
    ((0x90, 0x68), (0xB988)),  # HANGUL SYLLABLE RIEUL EU SSANGSIOS
    ((0x90, 0x69), (0xB98B)),  # HANGUL SYLLABLE RIEUL EU CHIEUCH
    ((0x90, 0x6A), (0xB98C)),  # HANGUL SYLLABLE RIEUL EU KHIEUKH
    ((0x90, 0x6B), (0xB98F)),  # HANGUL SYLLABLE RIEUL EU HIEUH
    ((0x90, 0x6C), (0xB990)),  # HANGUL SYLLABLE RIEUL YI
    ((0x90, 0x6D), (0xB991)),  # HANGUL SYLLABLE RIEUL YI KIYEOK
    ((0x90, 0x6E), (0xB992)),  # HANGUL SYLLABLE RIEUL YI SSANGKIYEOK
    ((0x90, 0x6F), (0xB993)),  # HANGUL SYLLABLE RIEUL YI KIYEOKSIOS
    ((0x90, 0x70), (0xB994)),  # HANGUL SYLLABLE RIEUL YI NIEUN
    ((0x90, 0x71), (0xB995)),  # HANGUL SYLLABLE RIEUL YI NIEUNCIEUC
    ((0x90, 0x72), (0xB996)),  # HANGUL SYLLABLE RIEUL YI NIEUNHIEUH
    ((0x90, 0x73), (0xB997)),  # HANGUL SYLLABLE RIEUL YI TIKEUT
    ((0x90, 0x74), (0xB998)),  # HANGUL SYLLABLE RIEUL YI RIEUL
    ((0x90, 0x75), (0xB999)),  # HANGUL SYLLABLE RIEUL YI RIEULKIYEOK
    ((0x90, 0x76), (0xB99A)),  # HANGUL SYLLABLE RIEUL YI RIEULMIEUM
    ((0x90, 0x77), (0xB99B)),  # HANGUL SYLLABLE RIEUL YI RIEULPIEUP
    ((0x90, 0x78), (0xB99C)),  # HANGUL SYLLABLE RIEUL YI RIEULSIOS
    ((0x90, 0x79), (0xB99D)),  # HANGUL SYLLABLE RIEUL YI RIEULTHIEUTH
    ((0x90, 0x7A), (0xB99E)),  # HANGUL SYLLABLE RIEUL YI RIEULPHIEUPH
    ((0x90, 0x81), (0xB99F)),  # HANGUL SYLLABLE RIEUL YI RIEULHIEUH
    ((0x90, 0x82), (0xB9A0)),  # HANGUL SYLLABLE RIEUL YI MIEUM
    ((0x90, 0x83), (0xB9A1)),  # HANGUL SYLLABLE RIEUL YI PIEUP
    ((0x90, 0x84), (0xB9A2)),  # HANGUL SYLLABLE RIEUL YI PIEUPSIOS
    ((0x90, 0x85), (0xB9A3)),  # HANGUL SYLLABLE RIEUL YI SIOS
    ((0x90, 0x86), (0xB9A4)),  # HANGUL SYLLABLE RIEUL YI SSANGSIOS
    ((0x90, 0x87), (0xB9A5)),  # HANGUL SYLLABLE RIEUL YI IEUNG
    ((0x90, 0x88), (0xB9A6)),  # HANGUL SYLLABLE RIEUL YI CIEUC
    ((0x90, 0x89), (0xB9A7)),  # HANGUL SYLLABLE RIEUL YI CHIEUCH
    ((0x90, 0x8A), (0xB9A8)),  # HANGUL SYLLABLE RIEUL YI KHIEUKH
    ((0x90, 0x8B), (0xB9A9)),  # HANGUL SYLLABLE RIEUL YI THIEUTH
    ((0x90, 0x8C), (0xB9AA)),  # HANGUL SYLLABLE RIEUL YI PHIEUPH
    ((0x90, 0x8D), (0xB9AB)),  # HANGUL SYLLABLE RIEUL YI HIEUH
    ((0x90, 0x8E), (0xB9AE)),  # HANGUL SYLLABLE RIEUL I SSANGKIYEOK
    ((0x90, 0x8F), (0xB9AF)),  # HANGUL SYLLABLE RIEUL I KIYEOKSIOS
    ((0x90, 0x90), (0xB9B1)),  # HANGUL SYLLABLE RIEUL I NIEUNCIEUC
    ((0x90, 0x91), (0xB9B2)),  # HANGUL SYLLABLE RIEUL I NIEUNHIEUH
    ((0x90, 0x92), (0xB9B3)),  # HANGUL SYLLABLE RIEUL I TIKEUT
    ((0x90, 0x93), (0xB9B5)),  # HANGUL SYLLABLE RIEUL I RIEULKIYEOK
    ((0x90, 0x94), (0xB9B6)),  # HANGUL SYLLABLE RIEUL I RIEULMIEUM
    ((0x90, 0x95), (0xB9B7)),  # HANGUL SYLLABLE RIEUL I RIEULPIEUP
    ((0x90, 0x96), (0xB9B8)),  # HANGUL SYLLABLE RIEUL I RIEULSIOS
    ((0x90, 0x97), (0xB9B9)),  # HANGUL SYLLABLE RIEUL I RIEULTHIEUTH
    ((0x90, 0x98), (0xB9BA)),  # HANGUL SYLLABLE RIEUL I RIEULPHIEUPH
    ((0x90, 0x99), (0xB9BB)),  # HANGUL SYLLABLE RIEUL I RIEULHIEUH
    ((0x90, 0x9A), (0xB9BE)),  # HANGUL SYLLABLE RIEUL I PIEUPSIOS
    ((0x90, 0x9B), (0xB9C0)),  # HANGUL SYLLABLE RIEUL I SSANGSIOS
    ((0x90, 0x9C), (0xB9C2)),  # HANGUL SYLLABLE RIEUL I CIEUC
    ((0x90, 0x9D), (0xB9C3)),  # HANGUL SYLLABLE RIEUL I CHIEUCH
    ((0x90, 0x9E), (0xB9C4)),  # HANGUL SYLLABLE RIEUL I KHIEUKH
    ((0x90, 0x9F), (0xB9C5)),  # HANGUL SYLLABLE RIEUL I THIEUTH
    ((0x90, 0xA0), (0xB9C6)),  # HANGUL SYLLABLE RIEUL I PHIEUPH
    ((0x90, 0xA1), (0xB9C7)),  # HANGUL SYLLABLE RIEUL I HIEUH
    ((0x90, 0xA2), (0xB9CA)),  # HANGUL SYLLABLE MIEUM A SSANGKIYEOK
    ((0x90, 0xA3), (0xB9CB)),  # HANGUL SYLLABLE MIEUM A KIYEOKSIOS
    ((0x90, 0xA4), (0xB9CD)),  # HANGUL SYLLABLE MIEUM A NIEUNCIEUC
    ((0x90, 0xA5), (0xB9D3)),  # HANGUL SYLLABLE MIEUM A RIEULPIEUP
    ((0x90, 0xA6), (0xB9D4)),  # HANGUL SYLLABLE MIEUM A RIEULSIOS
    ((0x90, 0xA7), (0xB9D5)),  # HANGUL SYLLABLE MIEUM A RIEULTHIEUTH
    ((0x90, 0xA8), (0xB9D6)),  # HANGUL SYLLABLE MIEUM A RIEULPHIEUPH
    ((0x90, 0xA9), (0xB9D7)),  # HANGUL SYLLABLE MIEUM A RIEULHIEUH
    ((0x90, 0xAA), (0xB9DA)),  # HANGUL SYLLABLE MIEUM A PIEUPSIOS
    ((0x90, 0xAB), (0xB9DC)),  # HANGUL SYLLABLE MIEUM A SSANGSIOS
    ((0x90, 0xAC), (0xB9DF)),  # HANGUL SYLLABLE MIEUM A CHIEUCH
    ((0x90, 0xAD), (0xB9E0)),  # HANGUL SYLLABLE MIEUM A KHIEUKH
    ((0x90, 0xAE), (0xB9E2)),  # HANGUL SYLLABLE MIEUM A PHIEUPH
    ((0x90, 0xAF), (0xB9E6)),  # HANGUL SYLLABLE MIEUM AE SSANGKIYEOK
    ((0x90, 0xB0), (0xB9E7)),  # HANGUL SYLLABLE MIEUM AE KIYEOKSIOS
    ((0x90, 0xB1), (0xB9E9)),  # HANGUL SYLLABLE MIEUM AE NIEUNCIEUC
    ((0x90, 0xB2), (0xB9EA)),  # HANGUL SYLLABLE MIEUM AE NIEUNHIEUH
    ((0x90, 0xB3), (0xB9EB)),  # HANGUL SYLLABLE MIEUM AE TIKEUT
    ((0x90, 0xB4), (0xB9ED)),  # HANGUL SYLLABLE MIEUM AE RIEULKIYEOK
    ((0x90, 0xB5), (0xB9EE)),  # HANGUL SYLLABLE MIEUM AE RIEULMIEUM
    ((0x90, 0xB6), (0xB9EF)),  # HANGUL SYLLABLE MIEUM AE RIEULPIEUP
    ((0x90, 0xB7), (0xB9F0)),  # HANGUL SYLLABLE MIEUM AE RIEULSIOS
    ((0x90, 0xB8), (0xB9F1)),  # HANGUL SYLLABLE MIEUM AE RIEULTHIEUTH
    ((0x90, 0xB9), (0xB9F2)),  # HANGUL SYLLABLE MIEUM AE RIEULPHIEUPH
    ((0x90, 0xBA), (0xB9F3)),  # HANGUL SYLLABLE MIEUM AE RIEULHIEUH
    ((0x90, 0xBB), (0xB9F6)),  # HANGUL SYLLABLE MIEUM AE PIEUPSIOS
    ((0x90, 0xBC), (0xB9FB)),  # HANGUL SYLLABLE MIEUM AE CHIEUCH
    ((0x90, 0xBD), (0xB9FC)),  # HANGUL SYLLABLE MIEUM AE KHIEUKH
    ((0x90, 0xBE), (0xB9FD)),  # HANGUL SYLLABLE MIEUM AE THIEUTH
    ((0x90, 0xBF), (0xB9FE)),  # HANGUL SYLLABLE MIEUM AE PHIEUPH
    ((0x90, 0xC0), (0xB9FF)),  # HANGUL SYLLABLE MIEUM AE HIEUH
    ((0x90, 0xC1), (0xBA02)),  # HANGUL SYLLABLE MIEUM YA SSANGKIYEOK
    ((0x90, 0xC2), (0xBA03)),  # HANGUL SYLLABLE MIEUM YA KIYEOKSIOS
    ((0x90, 0xC3), (0xBA04)),  # HANGUL SYLLABLE MIEUM YA NIEUN
    ((0x90, 0xC4), (0xBA05)),  # HANGUL SYLLABLE MIEUM YA NIEUNCIEUC
    ((0x90, 0xC5), (0xBA06)),  # HANGUL SYLLABLE MIEUM YA NIEUNHIEUH
    ((0x90, 0xC6), (0xBA07)),  # HANGUL SYLLABLE MIEUM YA TIKEUT
    ((0x90, 0xC7), (0xBA09)),  # HANGUL SYLLABLE MIEUM YA RIEULKIYEOK
    ((0x90, 0xC8), (0xBA0A)),  # HANGUL SYLLABLE MIEUM YA RIEULMIEUM
    ((0x90, 0xC9), (0xBA0B)),  # HANGUL SYLLABLE MIEUM YA RIEULPIEUP
    ((0x90, 0xCA), (0xBA0C)),  # HANGUL SYLLABLE MIEUM YA RIEULSIOS
    ((0x90, 0xCB), (0xBA0D)),  # HANGUL SYLLABLE MIEUM YA RIEULTHIEUTH
    ((0x90, 0xCC), (0xBA0E)),  # HANGUL SYLLABLE MIEUM YA RIEULPHIEUPH
    ((0x90, 0xCD), (0xBA0F)),  # HANGUL SYLLABLE MIEUM YA RIEULHIEUH
    ((0x90, 0xCE), (0xBA10)),  # HANGUL SYLLABLE MIEUM YA MIEUM
    ((0x90, 0xCF), (0xBA11)),  # HANGUL SYLLABLE MIEUM YA PIEUP
    ((0x90, 0xD0), (0xBA12)),  # HANGUL SYLLABLE MIEUM YA PIEUPSIOS
    ((0x90, 0xD1), (0xBA13)),  # HANGUL SYLLABLE MIEUM YA SIOS
    ((0x90, 0xD2), (0xBA14)),  # HANGUL SYLLABLE MIEUM YA SSANGSIOS
    ((0x90, 0xD3), (0xBA16)),  # HANGUL SYLLABLE MIEUM YA CIEUC
    ((0x90, 0xD4), (0xBA17)),  # HANGUL SYLLABLE MIEUM YA CHIEUCH
    ((0x90, 0xD5), (0xBA18)),  # HANGUL SYLLABLE MIEUM YA KHIEUKH
    ((0x90, 0xD6), (0xBA19)),  # HANGUL SYLLABLE MIEUM YA THIEUTH
    ((0x90, 0xD7), (0xBA1A)),  # HANGUL SYLLABLE MIEUM YA PHIEUPH
    ((0x90, 0xD8), (0xBA1B)),  # HANGUL SYLLABLE MIEUM YA HIEUH
    ((0x90, 0xD9), (0xBA1C)),  # HANGUL SYLLABLE MIEUM YAE
    ((0x90, 0xDA), (0xBA1D)),  # HANGUL SYLLABLE MIEUM YAE KIYEOK
    ((0x90, 0xDB), (0xBA1E)),  # HANGUL SYLLABLE MIEUM YAE SSANGKIYEOK
    ((0x90, 0xDC), (0xBA1F)),  # HANGUL SYLLABLE MIEUM YAE KIYEOKSIOS
    ((0x90, 0xDD), (0xBA20)),  # HANGUL SYLLABLE MIEUM YAE NIEUN
    ((0x90, 0xDE), (0xBA21)),  # HANGUL SYLLABLE MIEUM YAE NIEUNCIEUC
    ((0x90, 0xDF), (0xBA22)),  # HANGUL SYLLABLE MIEUM YAE NIEUNHIEUH
    ((0x90, 0xE0), (0xBA23)),  # HANGUL SYLLABLE MIEUM YAE TIKEUT
    ((0x90, 0xE1), (0xBA24)),  # HANGUL SYLLABLE MIEUM YAE RIEUL
    ((0x90, 0xE2), (0xBA25)),  # HANGUL SYLLABLE MIEUM YAE RIEULKIYEOK
    ((0x90, 0xE3), (0xBA26)),  # HANGUL SYLLABLE MIEUM YAE RIEULMIEUM
    ((0x90, 0xE4), (0xBA27)),  # HANGUL SYLLABLE MIEUM YAE RIEULPIEUP
    ((0x90, 0xE5), (0xBA28)),  # HANGUL SYLLABLE MIEUM YAE RIEULSIOS
    ((0x90, 0xE6), (0xBA29)),  # HANGUL SYLLABLE MIEUM YAE RIEULTHIEUTH
    ((0x90, 0xE7), (0xBA2A)),  # HANGUL SYLLABLE MIEUM YAE RIEULPHIEUPH
    ((0x90, 0xE8), (0xBA2B)),  # HANGUL SYLLABLE MIEUM YAE RIEULHIEUH
    ((0x90, 0xE9), (0xBA2C)),  # HANGUL SYLLABLE MIEUM YAE MIEUM
    ((0x90, 0xEA), (0xBA2D)),  # HANGUL SYLLABLE MIEUM YAE PIEUP
    ((0x90, 0xEB), (0xBA2E)),  # HANGUL SYLLABLE MIEUM YAE PIEUPSIOS
    ((0x90, 0xEC), (0xBA2F)),  # HANGUL SYLLABLE MIEUM YAE SIOS
    ((0x90, 0xED), (0xBA30)),  # HANGUL SYLLABLE MIEUM YAE SSANGSIOS
    ((0x90, 0xEE), (0xBA31)),  # HANGUL SYLLABLE MIEUM YAE IEUNG
    ((0x90, 0xEF), (0xBA32)),  # HANGUL SYLLABLE MIEUM YAE CIEUC
    ((0x90, 0xF0), (0xBA33)),  # HANGUL SYLLABLE MIEUM YAE CHIEUCH
    ((0x90, 0xF1), (0xBA34)),  # HANGUL SYLLABLE MIEUM YAE KHIEUKH
    ((0x90, 0xF2), (0xBA35)),  # HANGUL SYLLABLE MIEUM YAE THIEUTH
    ((0x90, 0xF3), (0xBA36)),  # HANGUL SYLLABLE MIEUM YAE PHIEUPH
    ((0x90, 0xF4), (0xBA37)),  # HANGUL SYLLABLE MIEUM YAE HIEUH
    ((0x90, 0xF5), (0xBA3A)),  # HANGUL SYLLABLE MIEUM EO SSANGKIYEOK
    ((0x90, 0xF6), (0xBA3B)),  # HANGUL SYLLABLE MIEUM EO KIYEOKSIOS
    ((0x90, 0xF7), (0xBA3D)),  # HANGUL SYLLABLE MIEUM EO NIEUNCIEUC
    ((0x90, 0xF8), (0xBA3E)),  # HANGUL SYLLABLE MIEUM EO NIEUNHIEUH
    ((0x90, 0xF9), (0xBA3F)),  # HANGUL SYLLABLE MIEUM EO TIKEUT
    ((0x90, 0xFA), (0xBA41)),  # HANGUL SYLLABLE MIEUM EO RIEULKIYEOK
    ((0x90, 0xFB), (0xBA43)),  # HANGUL SYLLABLE MIEUM EO RIEULPIEUP
    ((0x90, 0xFC), (0xBA44)),  # HANGUL SYLLABLE MIEUM EO RIEULSIOS
    ((0x90, 0xFD), (0xBA45)),  # HANGUL SYLLABLE MIEUM EO RIEULTHIEUTH
    ((0x90, 0xFE), (0xBA46)),  # HANGUL SYLLABLE MIEUM EO RIEULPHIEUPH
    ((0x91, 0x41), (0xBA47)),  # HANGUL SYLLABLE MIEUM EO RIEULHIEUH
    ((0x91, 0x42), (0xBA4A)),  # HANGUL SYLLABLE MIEUM EO PIEUPSIOS
    ((0x91, 0x43), (0xBA4C)),  # HANGUL SYLLABLE MIEUM EO SSANGSIOS
    ((0x91, 0x44), (0xBA4F)),  # HANGUL SYLLABLE MIEUM EO CHIEUCH
    ((0x91, 0x45), (0xBA50)),  # HANGUL SYLLABLE MIEUM EO KHIEUKH
    ((0x91, 0x46), (0xBA51)),  # HANGUL SYLLABLE MIEUM EO THIEUTH
    ((0x91, 0x47), (0xBA52)),  # HANGUL SYLLABLE MIEUM EO PHIEUPH
    ((0x91, 0x48), (0xBA56)),  # HANGUL SYLLABLE MIEUM E SSANGKIYEOK
    ((0x91, 0x49), (0xBA57)),  # HANGUL SYLLABLE MIEUM E KIYEOKSIOS
    ((0x91, 0x4A), (0xBA59)),  # HANGUL SYLLABLE MIEUM E NIEUNCIEUC
    ((0x91, 0x4B), (0xBA5A)),  # HANGUL SYLLABLE MIEUM E NIEUNHIEUH
    ((0x91, 0x4C), (0xBA5B)),  # HANGUL SYLLABLE MIEUM E TIKEUT
    ((0x91, 0x4D), (0xBA5D)),  # HANGUL SYLLABLE MIEUM E RIEULKIYEOK
    ((0x91, 0x4E), (0xBA5E)),  # HANGUL SYLLABLE MIEUM E RIEULMIEUM
    ((0x91, 0x4F), (0xBA5F)),  # HANGUL SYLLABLE MIEUM E RIEULPIEUP
    ((0x91, 0x50), (0xBA60)),  # HANGUL SYLLABLE MIEUM E RIEULSIOS
    ((0x91, 0x51), (0xBA61)),  # HANGUL SYLLABLE MIEUM E RIEULTHIEUTH
    ((0x91, 0x52), (0xBA62)),  # HANGUL SYLLABLE MIEUM E RIEULPHIEUPH
    ((0x91, 0x53), (0xBA63)),  # HANGUL SYLLABLE MIEUM E RIEULHIEUH
    ((0x91, 0x54), (0xBA66)),  # HANGUL SYLLABLE MIEUM E PIEUPSIOS
    ((0x91, 0x55), (0xBA6A)),  # HANGUL SYLLABLE MIEUM E CIEUC
    ((0x91, 0x56), (0xBA6B)),  # HANGUL SYLLABLE MIEUM E CHIEUCH
    ((0x91, 0x57), (0xBA6C)),  # HANGUL SYLLABLE MIEUM E KHIEUKH
    ((0x91, 0x58), (0xBA6D)),  # HANGUL SYLLABLE MIEUM E THIEUTH
    ((0x91, 0x59), (0xBA6E)),  # HANGUL SYLLABLE MIEUM E PHIEUPH
    ((0x91, 0x5A), (0xBA6F)),  # HANGUL SYLLABLE MIEUM E HIEUH
    ((0x91, 0x61), (0xBA72)),  # HANGUL SYLLABLE MIEUM YEO SSANGKIYEOK
    ((0x91, 0x62), (0xBA73)),  # HANGUL SYLLABLE MIEUM YEO KIYEOKSIOS
    ((0x91, 0x63), (0xBA75)),  # HANGUL SYLLABLE MIEUM YEO NIEUNCIEUC
    ((0x91, 0x64), (0xBA76)),  # HANGUL SYLLABLE MIEUM YEO NIEUNHIEUH
    ((0x91, 0x65), (0xBA77)),  # HANGUL SYLLABLE MIEUM YEO TIKEUT
    ((0x91, 0x66), (0xBA79)),  # HANGUL SYLLABLE MIEUM YEO RIEULKIYEOK
    ((0x91, 0x67), (0xBA7A)),  # HANGUL SYLLABLE MIEUM YEO RIEULMIEUM
    ((0x91, 0x68), (0xBA7B)),  # HANGUL SYLLABLE MIEUM YEO RIEULPIEUP
    ((0x91, 0x69), (0xBA7C)),  # HANGUL SYLLABLE MIEUM YEO RIEULSIOS
    ((0x91, 0x6A), (0xBA7D)),  # HANGUL SYLLABLE MIEUM YEO RIEULTHIEUTH
    ((0x91, 0x6B), (0xBA7E)),  # HANGUL SYLLABLE MIEUM YEO RIEULPHIEUPH
    ((0x91, 0x6C), (0xBA7F)),  # HANGUL SYLLABLE MIEUM YEO RIEULHIEUH
    ((0x91, 0x6D), (0xBA80)),  # HANGUL SYLLABLE MIEUM YEO MIEUM
    ((0x91, 0x6E), (0xBA81)),  # HANGUL SYLLABLE MIEUM YEO PIEUP
    ((0x91, 0x6F), (0xBA82)),  # HANGUL SYLLABLE MIEUM YEO PIEUPSIOS
    ((0x91, 0x70), (0xBA86)),  # HANGUL SYLLABLE MIEUM YEO CIEUC
    ((0x91, 0x71), (0xBA88)),  # HANGUL SYLLABLE MIEUM YEO KHIEUKH
    ((0x91, 0x72), (0xBA89)),  # HANGUL SYLLABLE MIEUM YEO THIEUTH
    ((0x91, 0x73), (0xBA8A)),  # HANGUL SYLLABLE MIEUM YEO PHIEUPH
    ((0x91, 0x74), (0xBA8B)),  # HANGUL SYLLABLE MIEUM YEO HIEUH
    ((0x91, 0x75), (0xBA8D)),  # HANGUL SYLLABLE MIEUM YE KIYEOK
    ((0x91, 0x76), (0xBA8E)),  # HANGUL SYLLABLE MIEUM YE SSANGKIYEOK
    ((0x91, 0x77), (0xBA8F)),  # HANGUL SYLLABLE MIEUM YE KIYEOKSIOS
    ((0x91, 0x78), (0xBA90)),  # HANGUL SYLLABLE MIEUM YE NIEUN
    ((0x91, 0x79), (0xBA91)),  # HANGUL SYLLABLE MIEUM YE NIEUNCIEUC
    ((0x91, 0x7A), (0xBA92)),  # HANGUL SYLLABLE MIEUM YE NIEUNHIEUH
    ((0x91, 0x81), (0xBA93)),  # HANGUL SYLLABLE MIEUM YE TIKEUT
    ((0x91, 0x82), (0xBA94)),  # HANGUL SYLLABLE MIEUM YE RIEUL
    ((0x91, 0x83), (0xBA95)),  # HANGUL SYLLABLE MIEUM YE RIEULKIYEOK
    ((0x91, 0x84), (0xBA96)),  # HANGUL SYLLABLE MIEUM YE RIEULMIEUM
    ((0x91, 0x85), (0xBA97)),  # HANGUL SYLLABLE MIEUM YE RIEULPIEUP
    ((0x91, 0x86), (0xBA98)),  # HANGUL SYLLABLE MIEUM YE RIEULSIOS
    ((0x91, 0x87), (0xBA99)),  # HANGUL SYLLABLE MIEUM YE RIEULTHIEUTH
    ((0x91, 0x88), (0xBA9A)),  # HANGUL SYLLABLE MIEUM YE RIEULPHIEUPH
    ((0x91, 0x89), (0xBA9B)),  # HANGUL SYLLABLE MIEUM YE RIEULHIEUH
    ((0x91, 0x8A), (0xBA9C)),  # HANGUL SYLLABLE MIEUM YE MIEUM
    ((0x91, 0x8B), (0xBA9D)),  # HANGUL SYLLABLE MIEUM YE PIEUP
    ((0x91, 0x8C), (0xBA9E)),  # HANGUL SYLLABLE MIEUM YE PIEUPSIOS
    ((0x91, 0x8D), (0xBA9F)),  # HANGUL SYLLABLE MIEUM YE SIOS
    ((0x91, 0x8E), (0xBAA0)),  # HANGUL SYLLABLE MIEUM YE SSANGSIOS
    ((0x91, 0x8F), (0xBAA1)),  # HANGUL SYLLABLE MIEUM YE IEUNG
    ((0x91, 0x90), (0xBAA2)),  # HANGUL SYLLABLE MIEUM YE CIEUC
    ((0x91, 0x91), (0xBAA3)),  # HANGUL SYLLABLE MIEUM YE CHIEUCH
    ((0x91, 0x92), (0xBAA4)),  # HANGUL SYLLABLE MIEUM YE KHIEUKH
    ((0x91, 0x93), (0xBAA5)),  # HANGUL SYLLABLE MIEUM YE THIEUTH
    ((0x91, 0x94), (0xBAA6)),  # HANGUL SYLLABLE MIEUM YE PHIEUPH
    ((0x91, 0x95), (0xBAA7)),  # HANGUL SYLLABLE MIEUM YE HIEUH
    ((0x91, 0x96), (0xBAAA)),  # HANGUL SYLLABLE MIEUM O SSANGKIYEOK
    ((0x91, 0x97), (0xBAAD)),  # HANGUL SYLLABLE MIEUM O NIEUNCIEUC
    ((0x91, 0x98), (0xBAAE)),  # HANGUL SYLLABLE MIEUM O NIEUNHIEUH
    ((0x91, 0x99), (0xBAAF)),  # HANGUL SYLLABLE MIEUM O TIKEUT
    ((0x91, 0x9A), (0xBAB1)),  # HANGUL SYLLABLE MIEUM O RIEULKIYEOK
    ((0x91, 0x9B), (0xBAB3)),  # HANGUL SYLLABLE MIEUM O RIEULPIEUP
    ((0x91, 0x9C), (0xBAB4)),  # HANGUL SYLLABLE MIEUM O RIEULSIOS
    ((0x91, 0x9D), (0xBAB5)),  # HANGUL SYLLABLE MIEUM O RIEULTHIEUTH
    ((0x91, 0x9E), (0xBAB6)),  # HANGUL SYLLABLE MIEUM O RIEULPHIEUPH
    ((0x91, 0x9F), (0xBAB7)),  # HANGUL SYLLABLE MIEUM O RIEULHIEUH
    ((0x91, 0xA0), (0xBABA)),  # HANGUL SYLLABLE MIEUM O PIEUPSIOS
    ((0x91, 0xA1), (0xBABC)),  # HANGUL SYLLABLE MIEUM O SSANGSIOS
    ((0x91, 0xA2), (0xBABE)),  # HANGUL SYLLABLE MIEUM O CIEUC
    ((0x91, 0xA3), (0xBABF)),  # HANGUL SYLLABLE MIEUM O CHIEUCH
    ((0x91, 0xA4), (0xBAC0)),  # HANGUL SYLLABLE MIEUM O KHIEUKH
    ((0x91, 0xA5), (0xBAC1)),  # HANGUL SYLLABLE MIEUM O THIEUTH
    ((0x91, 0xA6), (0xBAC2)),  # HANGUL SYLLABLE MIEUM O PHIEUPH
    ((0x91, 0xA7), (0xBAC3)),  # HANGUL SYLLABLE MIEUM O HIEUH
    ((0x91, 0xA8), (0xBAC5)),  # HANGUL SYLLABLE MIEUM WA KIYEOK
    ((0x91, 0xA9), (0xBAC6)),  # HANGUL SYLLABLE MIEUM WA SSANGKIYEOK
    ((0x91, 0xAA), (0xBAC7)),  # HANGUL SYLLABLE MIEUM WA KIYEOKSIOS
    ((0x91, 0xAB), (0xBAC9)),  # HANGUL SYLLABLE MIEUM WA NIEUNCIEUC
    ((0x91, 0xAC), (0xBACA)),  # HANGUL SYLLABLE MIEUM WA NIEUNHIEUH
    ((0x91, 0xAD), (0xBACB)),  # HANGUL SYLLABLE MIEUM WA TIKEUT
    ((0x91, 0xAE), (0xBACC)),  # HANGUL SYLLABLE MIEUM WA RIEUL
    ((0x91, 0xAF), (0xBACD)),  # HANGUL SYLLABLE MIEUM WA RIEULKIYEOK
    ((0x91, 0xB0), (0xBACE)),  # HANGUL SYLLABLE MIEUM WA RIEULMIEUM
    ((0x91, 0xB1), (0xBACF)),  # HANGUL SYLLABLE MIEUM WA RIEULPIEUP
    ((0x91, 0xB2), (0xBAD0)),  # HANGUL SYLLABLE MIEUM WA RIEULSIOS
    ((0x91, 0xB3), (0xBAD1)),  # HANGUL SYLLABLE MIEUM WA RIEULTHIEUTH
    ((0x91, 0xB4), (0xBAD2)),  # HANGUL SYLLABLE MIEUM WA RIEULPHIEUPH
    ((0x91, 0xB5), (0xBAD3)),  # HANGUL SYLLABLE MIEUM WA RIEULHIEUH
    ((0x91, 0xB6), (0xBAD4)),  # HANGUL SYLLABLE MIEUM WA MIEUM
    ((0x91, 0xB7), (0xBAD5)),  # HANGUL SYLLABLE MIEUM WA PIEUP
    ((0x91, 0xB8), (0xBAD6)),  # HANGUL SYLLABLE MIEUM WA PIEUPSIOS
    ((0x91, 0xB9), (0xBAD7)),  # HANGUL SYLLABLE MIEUM WA SIOS
    ((0x91, 0xBA), (0xBADA)),  # HANGUL SYLLABLE MIEUM WA CIEUC
    ((0x91, 0xBB), (0xBADB)),  # HANGUL SYLLABLE MIEUM WA CHIEUCH
    ((0x91, 0xBC), (0xBADC)),  # HANGUL SYLLABLE MIEUM WA KHIEUKH
    ((0x91, 0xBD), (0xBADD)),  # HANGUL SYLLABLE MIEUM WA THIEUTH
    ((0x91, 0xBE), (0xBADE)),  # HANGUL SYLLABLE MIEUM WA PHIEUPH
    ((0x91, 0xBF), (0xBADF)),  # HANGUL SYLLABLE MIEUM WA HIEUH
    ((0x91, 0xC0), (0xBAE0)),  # HANGUL SYLLABLE MIEUM WAE
    ((0x91, 0xC1), (0xBAE1)),  # HANGUL SYLLABLE MIEUM WAE KIYEOK
    ((0x91, 0xC2), (0xBAE2)),  # HANGUL SYLLABLE MIEUM WAE SSANGKIYEOK
    ((0x91, 0xC3), (0xBAE3)),  # HANGUL SYLLABLE MIEUM WAE KIYEOKSIOS
    ((0x91, 0xC4), (0xBAE4)),  # HANGUL SYLLABLE MIEUM WAE NIEUN
    ((0x91, 0xC5), (0xBAE5)),  # HANGUL SYLLABLE MIEUM WAE NIEUNCIEUC
    ((0x91, 0xC6), (0xBAE6)),  # HANGUL SYLLABLE MIEUM WAE NIEUNHIEUH
    ((0x91, 0xC7), (0xBAE7)),  # HANGUL SYLLABLE MIEUM WAE TIKEUT
    ((0x91, 0xC8), (0xBAE8)),  # HANGUL SYLLABLE MIEUM WAE RIEUL
    ((0x91, 0xC9), (0xBAE9)),  # HANGUL SYLLABLE MIEUM WAE RIEULKIYEOK
    ((0x91, 0xCA), (0xBAEA)),  # HANGUL SYLLABLE MIEUM WAE RIEULMIEUM
    ((0x91, 0xCB), (0xBAEB)),  # HANGUL SYLLABLE MIEUM WAE RIEULPIEUP
    ((0x91, 0xCC), (0xBAEC)),  # HANGUL SYLLABLE MIEUM WAE RIEULSIOS
    ((0x91, 0xCD), (0xBAED)),  # HANGUL SYLLABLE MIEUM WAE RIEULTHIEUTH
    ((0x91, 0xCE), (0xBAEE)),  # HANGUL SYLLABLE MIEUM WAE RIEULPHIEUPH
    ((0x91, 0xCF), (0xBAEF)),  # HANGUL SYLLABLE MIEUM WAE RIEULHIEUH
    ((0x91, 0xD0), (0xBAF0)),  # HANGUL SYLLABLE MIEUM WAE MIEUM
    ((0x91, 0xD1), (0xBAF1)),  # HANGUL SYLLABLE MIEUM WAE PIEUP
    ((0x91, 0xD2), (0xBAF2)),  # HANGUL SYLLABLE MIEUM WAE PIEUPSIOS
    ((0x91, 0xD3), (0xBAF3)),  # HANGUL SYLLABLE MIEUM WAE SIOS
    ((0x91, 0xD4), (0xBAF4)),  # HANGUL SYLLABLE MIEUM WAE SSANGSIOS
    ((0x91, 0xD5), (0xBAF5)),  # HANGUL SYLLABLE MIEUM WAE IEUNG
    ((0x91, 0xD6), (0xBAF6)),  # HANGUL SYLLABLE MIEUM WAE CIEUC
    ((0x91, 0xD7), (0xBAF7)),  # HANGUL SYLLABLE MIEUM WAE CHIEUCH
    ((0x91, 0xD8), (0xBAF8)),  # HANGUL SYLLABLE MIEUM WAE KHIEUKH
    ((0x91, 0xD9), (0xBAF9)),  # HANGUL SYLLABLE MIEUM WAE THIEUTH
    ((0x91, 0xDA), (0xBAFA)),  # HANGUL SYLLABLE MIEUM WAE PHIEUPH
    ((0x91, 0xDB), (0xBAFB)),  # HANGUL SYLLABLE MIEUM WAE HIEUH
    ((0x91, 0xDC), (0xBAFD)),  # HANGUL SYLLABLE MIEUM OE KIYEOK
    ((0x91, 0xDD), (0xBAFE)),  # HANGUL SYLLABLE MIEUM OE SSANGKIYEOK
    ((0x91, 0xDE), (0xBAFF)),  # HANGUL SYLLABLE MIEUM OE KIYEOKSIOS
    ((0x91, 0xDF), (0xBB01)),  # HANGUL SYLLABLE MIEUM OE NIEUNCIEUC
    ((0x91, 0xE0), (0xBB02)),  # HANGUL SYLLABLE MIEUM OE NIEUNHIEUH
    ((0x91, 0xE1), (0xBB03)),  # HANGUL SYLLABLE MIEUM OE TIKEUT
    ((0x91, 0xE2), (0xBB05)),  # HANGUL SYLLABLE MIEUM OE RIEULKIYEOK
    ((0x91, 0xE3), (0xBB06)),  # HANGUL SYLLABLE MIEUM OE RIEULMIEUM
    ((0x91, 0xE4), (0xBB07)),  # HANGUL SYLLABLE MIEUM OE RIEULPIEUP
    ((0x91, 0xE5), (0xBB08)),  # HANGUL SYLLABLE MIEUM OE RIEULSIOS
    ((0x91, 0xE6), (0xBB09)),  # HANGUL SYLLABLE MIEUM OE RIEULTHIEUTH
    ((0x91, 0xE7), (0xBB0A)),  # HANGUL SYLLABLE MIEUM OE RIEULPHIEUPH
    ((0x91, 0xE8), (0xBB0B)),  # HANGUL SYLLABLE MIEUM OE RIEULHIEUH
    ((0x91, 0xE9), (0xBB0C)),  # HANGUL SYLLABLE MIEUM OE MIEUM
    ((0x91, 0xEA), (0xBB0E)),  # HANGUL SYLLABLE MIEUM OE PIEUPSIOS
    ((0x91, 0xEB), (0xBB10)),  # HANGUL SYLLABLE MIEUM OE SSANGSIOS
    ((0x91, 0xEC), (0xBB12)),  # HANGUL SYLLABLE MIEUM OE CIEUC
    ((0x91, 0xED), (0xBB13)),  # HANGUL SYLLABLE MIEUM OE CHIEUCH
    ((0x91, 0xEE), (0xBB14)),  # HANGUL SYLLABLE MIEUM OE KHIEUKH
    ((0x91, 0xEF), (0xBB15)),  # HANGUL SYLLABLE MIEUM OE THIEUTH
    ((0x91, 0xF0), (0xBB16)),  # HANGUL SYLLABLE MIEUM OE PHIEUPH
    ((0x91, 0xF1), (0xBB17)),  # HANGUL SYLLABLE MIEUM OE HIEUH
    ((0x91, 0xF2), (0xBB19)),  # HANGUL SYLLABLE MIEUM YO KIYEOK
    ((0x91, 0xF3), (0xBB1A)),  # HANGUL SYLLABLE MIEUM YO SSANGKIYEOK
    ((0x91, 0xF4), (0xBB1B)),  # HANGUL SYLLABLE MIEUM YO KIYEOKSIOS
    ((0x91, 0xF5), (0xBB1D)),  # HANGUL SYLLABLE MIEUM YO NIEUNCIEUC
    ((0x91, 0xF6), (0xBB1E)),  # HANGUL SYLLABLE MIEUM YO NIEUNHIEUH
    ((0x91, 0xF7), (0xBB1F)),  # HANGUL SYLLABLE MIEUM YO TIKEUT
    ((0x91, 0xF8), (0xBB21)),  # HANGUL SYLLABLE MIEUM YO RIEULKIYEOK
    ((0x91, 0xF9), (0xBB22)),  # HANGUL SYLLABLE MIEUM YO RIEULMIEUM
    ((0x91, 0xFA), (0xBB23)),  # HANGUL SYLLABLE MIEUM YO RIEULPIEUP
    ((0x91, 0xFB), (0xBB24)),  # HANGUL SYLLABLE MIEUM YO RIEULSIOS
    ((0x91, 0xFC), (0xBB25)),  # HANGUL SYLLABLE MIEUM YO RIEULTHIEUTH
    ((0x91, 0xFD), (0xBB26)),  # HANGUL SYLLABLE MIEUM YO RIEULPHIEUPH
    ((0x91, 0xFE), (0xBB27)),  # HANGUL SYLLABLE MIEUM YO RIEULHIEUH
    ((0x92, 0x41), (0xBB28)),  # HANGUL SYLLABLE MIEUM YO MIEUM
    ((0x92, 0x42), (0xBB2A)),  # HANGUL SYLLABLE MIEUM YO PIEUPSIOS
    ((0x92, 0x43), (0xBB2C)),  # HANGUL SYLLABLE MIEUM YO SSANGSIOS
    ((0x92, 0x44), (0xBB2D)),  # HANGUL SYLLABLE MIEUM YO IEUNG
    ((0x92, 0x45), (0xBB2E)),  # HANGUL SYLLABLE MIEUM YO CIEUC
    ((0x92, 0x46), (0xBB2F)),  # HANGUL SYLLABLE MIEUM YO CHIEUCH
    ((0x92, 0x47), (0xBB30)),  # HANGUL SYLLABLE MIEUM YO KHIEUKH
    ((0x92, 0x48), (0xBB31)),  # HANGUL SYLLABLE MIEUM YO THIEUTH
    ((0x92, 0x49), (0xBB32)),  # HANGUL SYLLABLE MIEUM YO PHIEUPH
    ((0x92, 0x4A), (0xBB33)),  # HANGUL SYLLABLE MIEUM YO HIEUH
    ((0x92, 0x4B), (0xBB37)),  # HANGUL SYLLABLE MIEUM U KIYEOKSIOS
    ((0x92, 0x4C), (0xBB39)),  # HANGUL SYLLABLE MIEUM U NIEUNCIEUC
    ((0x92, 0x4D), (0xBB3A)),  # HANGUL SYLLABLE MIEUM U NIEUNHIEUH
    ((0x92, 0x4E), (0xBB3F)),  # HANGUL SYLLABLE MIEUM U RIEULPIEUP
    ((0x92, 0x4F), (0xBB40)),  # HANGUL SYLLABLE MIEUM U RIEULSIOS
    ((0x92, 0x50), (0xBB41)),  # HANGUL SYLLABLE MIEUM U RIEULTHIEUTH
    ((0x92, 0x51), (0xBB42)),  # HANGUL SYLLABLE MIEUM U RIEULPHIEUPH
    ((0x92, 0x52), (0xBB43)),  # HANGUL SYLLABLE MIEUM U RIEULHIEUH
    ((0x92, 0x53), (0xBB46)),  # HANGUL SYLLABLE MIEUM U PIEUPSIOS
    ((0x92, 0x54), (0xBB48)),  # HANGUL SYLLABLE MIEUM U SSANGSIOS
    ((0x92, 0x55), (0xBB4A)),  # HANGUL SYLLABLE MIEUM U CIEUC
    ((0x92, 0x56), (0xBB4B)),  # HANGUL SYLLABLE MIEUM U CHIEUCH
    ((0x92, 0x57), (0xBB4C)),  # HANGUL SYLLABLE MIEUM U KHIEUKH
    ((0x92, 0x58), (0xBB4E)),  # HANGUL SYLLABLE MIEUM U PHIEUPH
    ((0x92, 0x59), (0xBB51)),  # HANGUL SYLLABLE MIEUM WEO KIYEOK
    ((0x92, 0x5A), (0xBB52)),  # HANGUL SYLLABLE MIEUM WEO SSANGKIYEOK
    ((0x92, 0x61), (0xBB53)),  # HANGUL SYLLABLE MIEUM WEO KIYEOKSIOS
    ((0x92, 0x62), (0xBB55)),  # HANGUL SYLLABLE MIEUM WEO NIEUNCIEUC
    ((0x92, 0x63), (0xBB56)),  # HANGUL SYLLABLE MIEUM WEO NIEUNHIEUH
    ((0x92, 0x64), (0xBB57)),  # HANGUL SYLLABLE MIEUM WEO TIKEUT
    ((0x92, 0x65), (0xBB59)),  # HANGUL SYLLABLE MIEUM WEO RIEULKIYEOK
    ((0x92, 0x66), (0xBB5A)),  # HANGUL SYLLABLE MIEUM WEO RIEULMIEUM
    ((0x92, 0x67), (0xBB5B)),  # HANGUL SYLLABLE MIEUM WEO RIEULPIEUP
    ((0x92, 0x68), (0xBB5C)),  # HANGUL SYLLABLE MIEUM WEO RIEULSIOS
    ((0x92, 0x69), (0xBB5D)),  # HANGUL SYLLABLE MIEUM WEO RIEULTHIEUTH
    ((0x92, 0x6A), (0xBB5E)),  # HANGUL SYLLABLE MIEUM WEO RIEULPHIEUPH
    ((0x92, 0x6B), (0xBB5F)),  # HANGUL SYLLABLE MIEUM WEO RIEULHIEUH
    ((0x92, 0x6C), (0xBB60)),  # HANGUL SYLLABLE MIEUM WEO MIEUM
    ((0x92, 0x6D), (0xBB62)),  # HANGUL SYLLABLE MIEUM WEO PIEUPSIOS
    ((0x92, 0x6E), (0xBB64)),  # HANGUL SYLLABLE MIEUM WEO SSANGSIOS
    ((0x92, 0x6F), (0xBB65)),  # HANGUL SYLLABLE MIEUM WEO IEUNG
    ((0x92, 0x70), (0xBB66)),  # HANGUL SYLLABLE MIEUM WEO CIEUC
    ((0x92, 0x71), (0xBB67)),  # HANGUL SYLLABLE MIEUM WEO CHIEUCH
    ((0x92, 0x72), (0xBB68)),  # HANGUL SYLLABLE MIEUM WEO KHIEUKH
    ((0x92, 0x73), (0xBB69)),  # HANGUL SYLLABLE MIEUM WEO THIEUTH
    ((0x92, 0x74), (0xBB6A)),  # HANGUL SYLLABLE MIEUM WEO PHIEUPH
    ((0x92, 0x75), (0xBB6B)),  # HANGUL SYLLABLE MIEUM WEO HIEUH
    ((0x92, 0x76), (0xBB6D)),  # HANGUL SYLLABLE MIEUM WE KIYEOK
    ((0x92, 0x77), (0xBB6E)),  # HANGUL SYLLABLE MIEUM WE SSANGKIYEOK
    ((0x92, 0x78), (0xBB6F)),  # HANGUL SYLLABLE MIEUM WE KIYEOKSIOS
    ((0x92, 0x79), (0xBB70)),  # HANGUL SYLLABLE MIEUM WE NIEUN
    ((0x92, 0x7A), (0xBB71)),  # HANGUL SYLLABLE MIEUM WE NIEUNCIEUC
    ((0x92, 0x81), (0xBB72)),  # HANGUL SYLLABLE MIEUM WE NIEUNHIEUH
    ((0x92, 0x82), (0xBB73)),  # HANGUL SYLLABLE MIEUM WE TIKEUT
    ((0x92, 0x83), (0xBB74)),  # HANGUL SYLLABLE MIEUM WE RIEUL
    ((0x92, 0x84), (0xBB75)),  # HANGUL SYLLABLE MIEUM WE RIEULKIYEOK
    ((0x92, 0x85), (0xBB76)),  # HANGUL SYLLABLE MIEUM WE RIEULMIEUM
    ((0x92, 0x86), (0xBB77)),  # HANGUL SYLLABLE MIEUM WE RIEULPIEUP
    ((0x92, 0x87), (0xBB78)),  # HANGUL SYLLABLE MIEUM WE RIEULSIOS
    ((0x92, 0x88), (0xBB79)),  # HANGUL SYLLABLE MIEUM WE RIEULTHIEUTH
    ((0x92, 0x89), (0xBB7A)),  # HANGUL SYLLABLE MIEUM WE RIEULPHIEUPH
    ((0x92, 0x8A), (0xBB7B)),  # HANGUL SYLLABLE MIEUM WE RIEULHIEUH
    ((0x92, 0x8B), (0xBB7C)),  # HANGUL SYLLABLE MIEUM WE MIEUM
    ((0x92, 0x8C), (0xBB7D)),  # HANGUL SYLLABLE MIEUM WE PIEUP
    ((0x92, 0x8D), (0xBB7E)),  # HANGUL SYLLABLE MIEUM WE PIEUPSIOS
    ((0x92, 0x8E), (0xBB7F)),  # HANGUL SYLLABLE MIEUM WE SIOS
    ((0x92, 0x8F), (0xBB80)),  # HANGUL SYLLABLE MIEUM WE SSANGSIOS
    ((0x92, 0x90), (0xBB81)),  # HANGUL SYLLABLE MIEUM WE IEUNG
    ((0x92, 0x91), (0xBB82)),  # HANGUL SYLLABLE MIEUM WE CIEUC
    ((0x92, 0x92), (0xBB83)),  # HANGUL SYLLABLE MIEUM WE CHIEUCH
    ((0x92, 0x93), (0xBB84)),  # HANGUL SYLLABLE MIEUM WE KHIEUKH
    ((0x92, 0x94), (0xBB85)),  # HANGUL SYLLABLE MIEUM WE THIEUTH
    ((0x92, 0x95), (0xBB86)),  # HANGUL SYLLABLE MIEUM WE PHIEUPH
    ((0x92, 0x96), (0xBB87)),  # HANGUL SYLLABLE MIEUM WE HIEUH
    ((0x92, 0x97), (0xBB89)),  # HANGUL SYLLABLE MIEUM WI KIYEOK
    ((0x92, 0x98), (0xBB8A)),  # HANGUL SYLLABLE MIEUM WI SSANGKIYEOK
    ((0x92, 0x99), (0xBB8B)),  # HANGUL SYLLABLE MIEUM WI KIYEOKSIOS
    ((0x92, 0x9A), (0xBB8D)),  # HANGUL SYLLABLE MIEUM WI NIEUNCIEUC
    ((0x92, 0x9B), (0xBB8E)),  # HANGUL SYLLABLE MIEUM WI NIEUNHIEUH
    ((0x92, 0x9C), (0xBB8F)),  # HANGUL SYLLABLE MIEUM WI TIKEUT
    ((0x92, 0x9D), (0xBB91)),  # HANGUL SYLLABLE MIEUM WI RIEULKIYEOK
    ((0x92, 0x9E), (0xBB92)),  # HANGUL SYLLABLE MIEUM WI RIEULMIEUM
    ((0x92, 0x9F), (0xBB93)),  # HANGUL SYLLABLE MIEUM WI RIEULPIEUP
    ((0x92, 0xA0), (0xBB94)),  # HANGUL SYLLABLE MIEUM WI RIEULSIOS
    ((0x92, 0xA1), (0xBB95)),  # HANGUL SYLLABLE MIEUM WI RIEULTHIEUTH
    ((0x92, 0xA2), (0xBB96)),  # HANGUL SYLLABLE MIEUM WI RIEULPHIEUPH
    ((0x92, 0xA3), (0xBB97)),  # HANGUL SYLLABLE MIEUM WI RIEULHIEUH
    ((0x92, 0xA4), (0xBB98)),  # HANGUL SYLLABLE MIEUM WI MIEUM
    ((0x92, 0xA5), (0xBB99)),  # HANGUL SYLLABLE MIEUM WI PIEUP
    ((0x92, 0xA6), (0xBB9A)),  # HANGUL SYLLABLE MIEUM WI PIEUPSIOS
    ((0x92, 0xA7), (0xBB9B)),  # HANGUL SYLLABLE MIEUM WI SIOS
    ((0x92, 0xA8), (0xBB9C)),  # HANGUL SYLLABLE MIEUM WI SSANGSIOS
    ((0x92, 0xA9), (0xBB9D)),  # HANGUL SYLLABLE MIEUM WI IEUNG
    ((0x92, 0xAA), (0xBB9E)),  # HANGUL SYLLABLE MIEUM WI CIEUC
    ((0x92, 0xAB), (0xBB9F)),  # HANGUL SYLLABLE MIEUM WI CHIEUCH
    ((0x92, 0xAC), (0xBBA0)),  # HANGUL SYLLABLE MIEUM WI KHIEUKH
    ((0x92, 0xAD), (0xBBA1)),  # HANGUL SYLLABLE MIEUM WI THIEUTH
    ((0x92, 0xAE), (0xBBA2)),  # HANGUL SYLLABLE MIEUM WI PHIEUPH
    ((0x92, 0xAF), (0xBBA3)),  # HANGUL SYLLABLE MIEUM WI HIEUH
    ((0x92, 0xB0), (0xBBA5)),  # HANGUL SYLLABLE MIEUM YU KIYEOK
    ((0x92, 0xB1), (0xBBA6)),  # HANGUL SYLLABLE MIEUM YU SSANGKIYEOK
    ((0x92, 0xB2), (0xBBA7)),  # HANGUL SYLLABLE MIEUM YU KIYEOKSIOS
    ((0x92, 0xB3), (0xBBA9)),  # HANGUL SYLLABLE MIEUM YU NIEUNCIEUC
    ((0x92, 0xB4), (0xBBAA)),  # HANGUL SYLLABLE MIEUM YU NIEUNHIEUH
    ((0x92, 0xB5), (0xBBAB)),  # HANGUL SYLLABLE MIEUM YU TIKEUT
    ((0x92, 0xB6), (0xBBAD)),  # HANGUL SYLLABLE MIEUM YU RIEULKIYEOK
    ((0x92, 0xB7), (0xBBAE)),  # HANGUL SYLLABLE MIEUM YU RIEULMIEUM
    ((0x92, 0xB8), (0xBBAF)),  # HANGUL SYLLABLE MIEUM YU RIEULPIEUP
    ((0x92, 0xB9), (0xBBB0)),  # HANGUL SYLLABLE MIEUM YU RIEULSIOS
    ((0x92, 0xBA), (0xBBB1)),  # HANGUL SYLLABLE MIEUM YU RIEULTHIEUTH
    ((0x92, 0xBB), (0xBBB2)),  # HANGUL SYLLABLE MIEUM YU RIEULPHIEUPH
    ((0x92, 0xBC), (0xBBB3)),  # HANGUL SYLLABLE MIEUM YU RIEULHIEUH
    ((0x92, 0xBD), (0xBBB5)),  # HANGUL SYLLABLE MIEUM YU PIEUP
    ((0x92, 0xBE), (0xBBB6)),  # HANGUL SYLLABLE MIEUM YU PIEUPSIOS
    ((0x92, 0xBF), (0xBBB8)),  # HANGUL SYLLABLE MIEUM YU SSANGSIOS
    ((0x92, 0xC0), (0xBBB9)),  # HANGUL SYLLABLE MIEUM YU IEUNG
    ((0x92, 0xC1), (0xBBBA)),  # HANGUL SYLLABLE MIEUM YU CIEUC
    ((0x92, 0xC2), (0xBBBB)),  # HANGUL SYLLABLE MIEUM YU CHIEUCH
    ((0x92, 0xC3), (0xBBBC)),  # HANGUL SYLLABLE MIEUM YU KHIEUKH
    ((0x92, 0xC4), (0xBBBD)),  # HANGUL SYLLABLE MIEUM YU THIEUTH
    ((0x92, 0xC5), (0xBBBE)),  # HANGUL SYLLABLE MIEUM YU PHIEUPH
    ((0x92, 0xC6), (0xBBBF)),  # HANGUL SYLLABLE MIEUM YU HIEUH
    ((0x92, 0xC7), (0xBBC1)),  # HANGUL SYLLABLE MIEUM EU KIYEOK
    ((0x92, 0xC8), (0xBBC2)),  # HANGUL SYLLABLE MIEUM EU SSANGKIYEOK
    ((0x92, 0xC9), (0xBBC3)),  # HANGUL SYLLABLE MIEUM EU KIYEOKSIOS
    ((0x92, 0xCA), (0xBBC5)),  # HANGUL SYLLABLE MIEUM EU NIEUNCIEUC
    ((0x92, 0xCB), (0xBBC6)),  # HANGUL SYLLABLE MIEUM EU NIEUNHIEUH
    ((0x92, 0xCC), (0xBBC7)),  # HANGUL SYLLABLE MIEUM EU TIKEUT
    ((0x92, 0xCD), (0xBBC9)),  # HANGUL SYLLABLE MIEUM EU RIEULKIYEOK
    ((0x92, 0xCE), (0xBBCA)),  # HANGUL SYLLABLE MIEUM EU RIEULMIEUM
    ((0x92, 0xCF), (0xBBCB)),  # HANGUL SYLLABLE MIEUM EU RIEULPIEUP
    ((0x92, 0xD0), (0xBBCC)),  # HANGUL SYLLABLE MIEUM EU RIEULSIOS
    ((0x92, 0xD1), (0xBBCD)),  # HANGUL SYLLABLE MIEUM EU RIEULTHIEUTH
    ((0x92, 0xD2), (0xBBCE)),  # HANGUL SYLLABLE MIEUM EU RIEULPHIEUPH
    ((0x92, 0xD3), (0xBBCF)),  # HANGUL SYLLABLE MIEUM EU RIEULHIEUH
    ((0x92, 0xD4), (0xBBD1)),  # HANGUL SYLLABLE MIEUM EU PIEUP
    ((0x92, 0xD5), (0xBBD2)),  # HANGUL SYLLABLE MIEUM EU PIEUPSIOS
    ((0x92, 0xD6), (0xBBD4)),  # HANGUL SYLLABLE MIEUM EU SSANGSIOS
    ((0x92, 0xD7), (0xBBD5)),  # HANGUL SYLLABLE MIEUM EU IEUNG
    ((0x92, 0xD8), (0xBBD6)),  # HANGUL SYLLABLE MIEUM EU CIEUC
    ((0x92, 0xD9), (0xBBD7)),  # HANGUL SYLLABLE MIEUM EU CHIEUCH
    ((0x92, 0xDA), (0xBBD8)),  # HANGUL SYLLABLE MIEUM EU KHIEUKH
    ((0x92, 0xDB), (0xBBD9)),  # HANGUL SYLLABLE MIEUM EU THIEUTH
    ((0x92, 0xDC), (0xBBDA)),  # HANGUL SYLLABLE MIEUM EU PHIEUPH
    ((0x92, 0xDD), (0xBBDB)),  # HANGUL SYLLABLE MIEUM EU HIEUH
    ((0x92, 0xDE), (0xBBDC)),  # HANGUL SYLLABLE MIEUM YI
    ((0x92, 0xDF), (0xBBDD)),  # HANGUL SYLLABLE MIEUM YI KIYEOK
    ((0x92, 0xE0), (0xBBDE)),  # HANGUL SYLLABLE MIEUM YI SSANGKIYEOK
    ((0x92, 0xE1), (0xBBDF)),  # HANGUL SYLLABLE MIEUM YI KIYEOKSIOS
    ((0x92, 0xE2), (0xBBE0)),  # HANGUL SYLLABLE MIEUM YI NIEUN
    ((0x92, 0xE3), (0xBBE1)),  # HANGUL SYLLABLE MIEUM YI NIEUNCIEUC
    ((0x92, 0xE4), (0xBBE2)),  # HANGUL SYLLABLE MIEUM YI NIEUNHIEUH
    ((0x92, 0xE5), (0xBBE3)),  # HANGUL SYLLABLE MIEUM YI TIKEUT
    ((0x92, 0xE6), (0xBBE4)),  # HANGUL SYLLABLE MIEUM YI RIEUL
    ((0x92, 0xE7), (0xBBE5)),  # HANGUL SYLLABLE MIEUM YI RIEULKIYEOK
    ((0x92, 0xE8), (0xBBE6)),  # HANGUL SYLLABLE MIEUM YI RIEULMIEUM
    ((0x92, 0xE9), (0xBBE7)),  # HANGUL SYLLABLE MIEUM YI RIEULPIEUP
    ((0x92, 0xEA), (0xBBE8)),  # HANGUL SYLLABLE MIEUM YI RIEULSIOS
    ((0x92, 0xEB), (0xBBE9)),  # HANGUL SYLLABLE MIEUM YI RIEULTHIEUTH
    ((0x92, 0xEC), (0xBBEA)),  # HANGUL SYLLABLE MIEUM YI RIEULPHIEUPH
    ((0x92, 0xED), (0xBBEB)),  # HANGUL SYLLABLE MIEUM YI RIEULHIEUH
    ((0x92, 0xEE), (0xBBEC)),  # HANGUL SYLLABLE MIEUM YI MIEUM
    ((0x92, 0xEF), (0xBBED)),  # HANGUL SYLLABLE MIEUM YI PIEUP
    ((0x92, 0xF0), (0xBBEE)),  # HANGUL SYLLABLE MIEUM YI PIEUPSIOS
    ((0x92, 0xF1), (0xBBEF)),  # HANGUL SYLLABLE MIEUM YI SIOS
    ((0x92, 0xF2), (0xBBF0)),  # HANGUL SYLLABLE MIEUM YI SSANGSIOS
    ((0x92, 0xF3), (0xBBF1)),  # HANGUL SYLLABLE MIEUM YI IEUNG
    ((0x92, 0xF4), (0xBBF2)),  # HANGUL SYLLABLE MIEUM YI CIEUC
    ((0x92, 0xF5), (0xBBF3)),  # HANGUL SYLLABLE MIEUM YI CHIEUCH
    ((0x92, 0xF6), (0xBBF4)),  # HANGUL SYLLABLE MIEUM YI KHIEUKH
    ((0x92, 0xF7), (0xBBF5)),  # HANGUL SYLLABLE MIEUM YI THIEUTH
    ((0x92, 0xF8), (0xBBF6)),  # HANGUL SYLLABLE MIEUM YI PHIEUPH
    ((0x92, 0xF9), (0xBBF7)),  # HANGUL SYLLABLE MIEUM YI HIEUH
    ((0x92, 0xFA), (0xBBFA)),  # HANGUL SYLLABLE MIEUM I SSANGKIYEOK
    ((0x92, 0xFB), (0xBBFB)),  # HANGUL SYLLABLE MIEUM I KIYEOKSIOS
    ((0x92, 0xFC), (0xBBFD)),  # HANGUL SYLLABLE MIEUM I NIEUNCIEUC
    ((0x92, 0xFD), (0xBBFE)),  # HANGUL SYLLABLE MIEUM I NIEUNHIEUH
    ((0x92, 0xFE), (0xBC01)),  # HANGUL SYLLABLE MIEUM I RIEULKIYEOK
    ((0x93, 0x41), (0xBC03)),  # HANGUL SYLLABLE MIEUM I RIEULPIEUP
    ((0x93, 0x42), (0xBC04)),  # HANGUL SYLLABLE MIEUM I RIEULSIOS
    ((0x93, 0x43), (0xBC05)),  # HANGUL SYLLABLE MIEUM I RIEULTHIEUTH
    ((0x93, 0x44), (0xBC06)),  # HANGUL SYLLABLE MIEUM I RIEULPHIEUPH
    ((0x93, 0x45), (0xBC07)),  # HANGUL SYLLABLE MIEUM I RIEULHIEUH
    ((0x93, 0x46), (0xBC0A)),  # HANGUL SYLLABLE MIEUM I PIEUPSIOS
    ((0x93, 0x47), (0xBC0E)),  # HANGUL SYLLABLE MIEUM I CIEUC
    ((0x93, 0x48), (0xBC10)),  # HANGUL SYLLABLE MIEUM I KHIEUKH
    ((0x93, 0x49), (0xBC12)),  # HANGUL SYLLABLE MIEUM I PHIEUPH
    ((0x93, 0x4A), (0xBC13)),  # HANGUL SYLLABLE MIEUM I HIEUH
    ((0x93, 0x4B), (0xBC19)),  # HANGUL SYLLABLE PIEUP A NIEUNCIEUC
    ((0x93, 0x4C), (0xBC1A)),  # HANGUL SYLLABLE PIEUP A NIEUNHIEUH
    ((0x93, 0x4D), (0xBC20)),  # HANGUL SYLLABLE PIEUP A RIEULSIOS
    ((0x93, 0x4E), (0xBC21)),  # HANGUL SYLLABLE PIEUP A RIEULTHIEUTH
    ((0x93, 0x4F), (0xBC22)),  # HANGUL SYLLABLE PIEUP A RIEULPHIEUPH
    ((0x93, 0x50), (0xBC23)),  # HANGUL SYLLABLE PIEUP A RIEULHIEUH
    ((0x93, 0x51), (0xBC26)),  # HANGUL SYLLABLE PIEUP A PIEUPSIOS
    ((0x93, 0x52), (0xBC28)),  # HANGUL SYLLABLE PIEUP A SSANGSIOS
    ((0x93, 0x53), (0xBC2A)),  # HANGUL SYLLABLE PIEUP A CIEUC
    ((0x93, 0x54), (0xBC2B)),  # HANGUL SYLLABLE PIEUP A CHIEUCH
    ((0x93, 0x55), (0xBC2C)),  # HANGUL SYLLABLE PIEUP A KHIEUKH
    ((0x93, 0x56), (0xBC2E)),  # HANGUL SYLLABLE PIEUP A PHIEUPH
    ((0x93, 0x57), (0xBC2F)),  # HANGUL SYLLABLE PIEUP A HIEUH
    ((0x93, 0x58), (0xBC32)),  # HANGUL SYLLABLE PIEUP AE SSANGKIYEOK
    ((0x93, 0x59), (0xBC33)),  # HANGUL SYLLABLE PIEUP AE KIYEOKSIOS
    ((0x93, 0x5A), (0xBC35)),  # HANGUL SYLLABLE PIEUP AE NIEUNCIEUC
    ((0x93, 0x61), (0xBC36)),  # HANGUL SYLLABLE PIEUP AE NIEUNHIEUH
    ((0x93, 0x62), (0xBC37)),  # HANGUL SYLLABLE PIEUP AE TIKEUT
    ((0x93, 0x63), (0xBC39)),  # HANGUL SYLLABLE PIEUP AE RIEULKIYEOK
    ((0x93, 0x64), (0xBC3A)),  # HANGUL SYLLABLE PIEUP AE RIEULMIEUM
    ((0x93, 0x65), (0xBC3B)),  # HANGUL SYLLABLE PIEUP AE RIEULPIEUP
    ((0x93, 0x66), (0xBC3C)),  # HANGUL SYLLABLE PIEUP AE RIEULSIOS
    ((0x93, 0x67), (0xBC3D)),  # HANGUL SYLLABLE PIEUP AE RIEULTHIEUTH
    ((0x93, 0x68), (0xBC3E)),  # HANGUL SYLLABLE PIEUP AE RIEULPHIEUPH
    ((0x93, 0x69), (0xBC3F)),  # HANGUL SYLLABLE PIEUP AE RIEULHIEUH
    ((0x93, 0x6A), (0xBC42)),  # HANGUL SYLLABLE PIEUP AE PIEUPSIOS
    ((0x93, 0x6B), (0xBC46)),  # HANGUL SYLLABLE PIEUP AE CIEUC
    ((0x93, 0x6C), (0xBC47)),  # HANGUL SYLLABLE PIEUP AE CHIEUCH
    ((0x93, 0x6D), (0xBC48)),  # HANGUL SYLLABLE PIEUP AE KHIEUKH
    ((0x93, 0x6E), (0xBC4A)),  # HANGUL SYLLABLE PIEUP AE PHIEUPH
    ((0x93, 0x6F), (0xBC4B)),  # HANGUL SYLLABLE PIEUP AE HIEUH
    ((0x93, 0x70), (0xBC4E)),  # HANGUL SYLLABLE PIEUP YA SSANGKIYEOK
    ((0x93, 0x71), (0xBC4F)),  # HANGUL SYLLABLE PIEUP YA KIYEOKSIOS
    ((0x93, 0x72), (0xBC51)),  # HANGUL SYLLABLE PIEUP YA NIEUNCIEUC
    ((0x93, 0x73), (0xBC52)),  # HANGUL SYLLABLE PIEUP YA NIEUNHIEUH
    ((0x93, 0x74), (0xBC53)),  # HANGUL SYLLABLE PIEUP YA TIKEUT
    ((0x93, 0x75), (0xBC54)),  # HANGUL SYLLABLE PIEUP YA RIEUL
    ((0x93, 0x76), (0xBC55)),  # HANGUL SYLLABLE PIEUP YA RIEULKIYEOK
    ((0x93, 0x77), (0xBC56)),  # HANGUL SYLLABLE PIEUP YA RIEULMIEUM
    ((0x93, 0x78), (0xBC57)),  # HANGUL SYLLABLE PIEUP YA RIEULPIEUP
    ((0x93, 0x79), (0xBC58)),  # HANGUL SYLLABLE PIEUP YA RIEULSIOS
    ((0x93, 0x7A), (0xBC59)),  # HANGUL SYLLABLE PIEUP YA RIEULTHIEUTH
    ((0x93, 0x81), (0xBC5A)),  # HANGUL SYLLABLE PIEUP YA RIEULPHIEUPH
    ((0x93, 0x82), (0xBC5B)),  # HANGUL SYLLABLE PIEUP YA RIEULHIEUH
    ((0x93, 0x83), (0xBC5C)),  # HANGUL SYLLABLE PIEUP YA MIEUM
    ((0x93, 0x84), (0xBC5E)),  # HANGUL SYLLABLE PIEUP YA PIEUPSIOS
    ((0x93, 0x85), (0xBC5F)),  # HANGUL SYLLABLE PIEUP YA SIOS
    ((0x93, 0x86), (0xBC60)),  # HANGUL SYLLABLE PIEUP YA SSANGSIOS
    ((0x93, 0x87), (0xBC61)),  # HANGUL SYLLABLE PIEUP YA IEUNG
    ((0x93, 0x88), (0xBC62)),  # HANGUL SYLLABLE PIEUP YA CIEUC
    ((0x93, 0x89), (0xBC63)),  # HANGUL SYLLABLE PIEUP YA CHIEUCH
    ((0x93, 0x8A), (0xBC64)),  # HANGUL SYLLABLE PIEUP YA KHIEUKH
    ((0x93, 0x8B), (0xBC65)),  # HANGUL SYLLABLE PIEUP YA THIEUTH
    ((0x93, 0x8C), (0xBC66)),  # HANGUL SYLLABLE PIEUP YA PHIEUPH
    ((0x93, 0x8D), (0xBC67)),  # HANGUL SYLLABLE PIEUP YA HIEUH
    ((0x93, 0x8E), (0xBC68)),  # HANGUL SYLLABLE PIEUP YAE
    ((0x93, 0x8F), (0xBC69)),  # HANGUL SYLLABLE PIEUP YAE KIYEOK
    ((0x93, 0x90), (0xBC6A)),  # HANGUL SYLLABLE PIEUP YAE SSANGKIYEOK
    ((0x93, 0x91), (0xBC6B)),  # HANGUL SYLLABLE PIEUP YAE KIYEOKSIOS
    ((0x93, 0x92), (0xBC6C)),  # HANGUL SYLLABLE PIEUP YAE NIEUN
    ((0x93, 0x93), (0xBC6D)),  # HANGUL SYLLABLE PIEUP YAE NIEUNCIEUC
    ((0x93, 0x94), (0xBC6E)),  # HANGUL SYLLABLE PIEUP YAE NIEUNHIEUH
    ((0x93, 0x95), (0xBC6F)),  # HANGUL SYLLABLE PIEUP YAE TIKEUT
    ((0x93, 0x96), (0xBC70)),  # HANGUL SYLLABLE PIEUP YAE RIEUL
    ((0x93, 0x97), (0xBC71)),  # HANGUL SYLLABLE PIEUP YAE RIEULKIYEOK
    ((0x93, 0x98), (0xBC72)),  # HANGUL SYLLABLE PIEUP YAE RIEULMIEUM
    ((0x93, 0x99), (0xBC73)),  # HANGUL SYLLABLE PIEUP YAE RIEULPIEUP
    ((0x93, 0x9A), (0xBC74)),  # HANGUL SYLLABLE PIEUP YAE RIEULSIOS
    ((0x93, 0x9B), (0xBC75)),  # HANGUL SYLLABLE PIEUP YAE RIEULTHIEUTH
    ((0x93, 0x9C), (0xBC76)),  # HANGUL SYLLABLE PIEUP YAE RIEULPHIEUPH
    ((0x93, 0x9D), (0xBC77)),  # HANGUL SYLLABLE PIEUP YAE RIEULHIEUH
    ((0x93, 0x9E), (0xBC78)),  # HANGUL SYLLABLE PIEUP YAE MIEUM
    ((0x93, 0x9F), (0xBC79)),  # HANGUL SYLLABLE PIEUP YAE PIEUP
    ((0x93, 0xA0), (0xBC7A)),  # HANGUL SYLLABLE PIEUP YAE PIEUPSIOS
    ((0x93, 0xA1), (0xBC7B)),  # HANGUL SYLLABLE PIEUP YAE SIOS
    ((0x93, 0xA2), (0xBC7C)),  # HANGUL SYLLABLE PIEUP YAE SSANGSIOS
    ((0x93, 0xA3), (0xBC7D)),  # HANGUL SYLLABLE PIEUP YAE IEUNG
    ((0x93, 0xA4), (0xBC7E)),  # HANGUL SYLLABLE PIEUP YAE CIEUC
    ((0x93, 0xA5), (0xBC7F)),  # HANGUL SYLLABLE PIEUP YAE CHIEUCH
    ((0x93, 0xA6), (0xBC80)),  # HANGUL SYLLABLE PIEUP YAE KHIEUKH
    ((0x93, 0xA7), (0xBC81)),  # HANGUL SYLLABLE PIEUP YAE THIEUTH
    ((0x93, 0xA8), (0xBC82)),  # HANGUL SYLLABLE PIEUP YAE PHIEUPH
    ((0x93, 0xA9), (0xBC83)),  # HANGUL SYLLABLE PIEUP YAE HIEUH
    ((0x93, 0xAA), (0xBC86)),  # HANGUL SYLLABLE PIEUP EO SSANGKIYEOK
    ((0x93, 0xAB), (0xBC87)),  # HANGUL SYLLABLE PIEUP EO KIYEOKSIOS
    ((0x93, 0xAC), (0xBC89)),  # HANGUL SYLLABLE PIEUP EO NIEUNCIEUC
    ((0x93, 0xAD), (0xBC8A)),  # HANGUL SYLLABLE PIEUP EO NIEUNHIEUH
    ((0x93, 0xAE), (0xBC8D)),  # HANGUL SYLLABLE PIEUP EO RIEULKIYEOK
    ((0x93, 0xAF), (0xBC8F)),  # HANGUL SYLLABLE PIEUP EO RIEULPIEUP
    ((0x93, 0xB0), (0xBC90)),  # HANGUL SYLLABLE PIEUP EO RIEULSIOS
    ((0x93, 0xB1), (0xBC91)),  # HANGUL SYLLABLE PIEUP EO RIEULTHIEUTH
    ((0x93, 0xB2), (0xBC92)),  # HANGUL SYLLABLE PIEUP EO RIEULPHIEUPH
    ((0x93, 0xB3), (0xBC93)),  # HANGUL SYLLABLE PIEUP EO RIEULHIEUH
    ((0x93, 0xB4), (0xBC96)),  # HANGUL SYLLABLE PIEUP EO PIEUPSIOS
    ((0x93, 0xB5), (0xBC98)),  # HANGUL SYLLABLE PIEUP EO SSANGSIOS
    ((0x93, 0xB6), (0xBC9B)),  # HANGUL SYLLABLE PIEUP EO CHIEUCH
    ((0x93, 0xB7), (0xBC9C)),  # HANGUL SYLLABLE PIEUP EO KHIEUKH
    ((0x93, 0xB8), (0xBC9D)),  # HANGUL SYLLABLE PIEUP EO THIEUTH
    ((0x93, 0xB9), (0xBC9E)),  # HANGUL SYLLABLE PIEUP EO PHIEUPH
    ((0x93, 0xBA), (0xBC9F)),  # HANGUL SYLLABLE PIEUP EO HIEUH
    ((0x93, 0xBB), (0xBCA2)),  # HANGUL SYLLABLE PIEUP E SSANGKIYEOK
    ((0x93, 0xBC), (0xBCA3)),  # HANGUL SYLLABLE PIEUP E KIYEOKSIOS
    ((0x93, 0xBD), (0xBCA5)),  # HANGUL SYLLABLE PIEUP E NIEUNCIEUC
    ((0x93, 0xBE), (0xBCA6)),  # HANGUL SYLLABLE PIEUP E NIEUNHIEUH
    ((0x93, 0xBF), (0xBCA9)),  # HANGUL SYLLABLE PIEUP E RIEULKIYEOK
    ((0x93, 0xC0), (0xBCAA)),  # HANGUL SYLLABLE PIEUP E RIEULMIEUM
    ((0x93, 0xC1), (0xBCAB)),  # HANGUL SYLLABLE PIEUP E RIEULPIEUP
    ((0x93, 0xC2), (0xBCAC)),  # HANGUL SYLLABLE PIEUP E RIEULSIOS
    ((0x93, 0xC3), (0xBCAD)),  # HANGUL SYLLABLE PIEUP E RIEULTHIEUTH
    ((0x93, 0xC4), (0xBCAE)),  # HANGUL SYLLABLE PIEUP E RIEULPHIEUPH
    ((0x93, 0xC5), (0xBCAF)),  # HANGUL SYLLABLE PIEUP E RIEULHIEUH
    ((0x93, 0xC6), (0xBCB2)),  # HANGUL SYLLABLE PIEUP E PIEUPSIOS
    ((0x93, 0xC7), (0xBCB6)),  # HANGUL SYLLABLE PIEUP E CIEUC
    ((0x93, 0xC8), (0xBCB7)),  # HANGUL SYLLABLE PIEUP E CHIEUCH
    ((0x93, 0xC9), (0xBCB8)),  # HANGUL SYLLABLE PIEUP E KHIEUKH
    ((0x93, 0xCA), (0xBCB9)),  # HANGUL SYLLABLE PIEUP E THIEUTH
    ((0x93, 0xCB), (0xBCBA)),  # HANGUL SYLLABLE PIEUP E PHIEUPH
    ((0x93, 0xCC), (0xBCBB)),  # HANGUL SYLLABLE PIEUP E HIEUH
    ((0x93, 0xCD), (0xBCBE)),  # HANGUL SYLLABLE PIEUP YEO SSANGKIYEOK
    ((0x93, 0xCE), (0xBCBF)),  # HANGUL SYLLABLE PIEUP YEO KIYEOKSIOS
    ((0x93, 0xCF), (0xBCC1)),  # HANGUL SYLLABLE PIEUP YEO NIEUNCIEUC
    ((0x93, 0xD0), (0xBCC2)),  # HANGUL SYLLABLE PIEUP YEO NIEUNHIEUH
    ((0x93, 0xD1), (0xBCC3)),  # HANGUL SYLLABLE PIEUP YEO TIKEUT
    ((0x93, 0xD2), (0xBCC5)),  # HANGUL SYLLABLE PIEUP YEO RIEULKIYEOK
    ((0x93, 0xD3), (0xBCC6)),  # HANGUL SYLLABLE PIEUP YEO RIEULMIEUM
    ((0x93, 0xD4), (0xBCC7)),  # HANGUL SYLLABLE PIEUP YEO RIEULPIEUP
    ((0x93, 0xD5), (0xBCC8)),  # HANGUL SYLLABLE PIEUP YEO RIEULSIOS
    ((0x93, 0xD6), (0xBCC9)),  # HANGUL SYLLABLE PIEUP YEO RIEULTHIEUTH
    ((0x93, 0xD7), (0xBCCA)),  # HANGUL SYLLABLE PIEUP YEO RIEULPHIEUPH
    ((0x93, 0xD8), (0xBCCB)),  # HANGUL SYLLABLE PIEUP YEO RIEULHIEUH
    ((0x93, 0xD9), (0xBCCC)),  # HANGUL SYLLABLE PIEUP YEO MIEUM
    ((0x93, 0xDA), (0xBCCE)),  # HANGUL SYLLABLE PIEUP YEO PIEUPSIOS
    ((0x93, 0xDB), (0xBCD2)),  # HANGUL SYLLABLE PIEUP YEO CIEUC
    ((0x93, 0xDC), (0xBCD3)),  # HANGUL SYLLABLE PIEUP YEO CHIEUCH
    ((0x93, 0xDD), (0xBCD4)),  # HANGUL SYLLABLE PIEUP YEO KHIEUKH
    ((0x93, 0xDE), (0xBCD6)),  # HANGUL SYLLABLE PIEUP YEO PHIEUPH
    ((0x93, 0xDF), (0xBCD7)),  # HANGUL SYLLABLE PIEUP YEO HIEUH
    ((0x93, 0xE0), (0xBCD9)),  # HANGUL SYLLABLE PIEUP YE KIYEOK
    ((0x93, 0xE1), (0xBCDA)),  # HANGUL SYLLABLE PIEUP YE SSANGKIYEOK
    ((0x93, 0xE2), (0xBCDB)),  # HANGUL SYLLABLE PIEUP YE KIYEOKSIOS
    ((0x93, 0xE3), (0xBCDD)),  # HANGUL SYLLABLE PIEUP YE NIEUNCIEUC
    ((0x93, 0xE4), (0xBCDE)),  # HANGUL SYLLABLE PIEUP YE NIEUNHIEUH
    ((0x93, 0xE5), (0xBCDF)),  # HANGUL SYLLABLE PIEUP YE TIKEUT
    ((0x93, 0xE6), (0xBCE0)),  # HANGUL SYLLABLE PIEUP YE RIEUL
    ((0x93, 0xE7), (0xBCE1)),  # HANGUL SYLLABLE PIEUP YE RIEULKIYEOK
    ((0x93, 0xE8), (0xBCE2)),  # HANGUL SYLLABLE PIEUP YE RIEULMIEUM
    ((0x93, 0xE9), (0xBCE3)),  # HANGUL SYLLABLE PIEUP YE RIEULPIEUP
    ((0x93, 0xEA), (0xBCE4)),  # HANGUL SYLLABLE PIEUP YE RIEULSIOS
    ((0x93, 0xEB), (0xBCE5)),  # HANGUL SYLLABLE PIEUP YE RIEULTHIEUTH
    ((0x93, 0xEC), (0xBCE6)),  # HANGUL SYLLABLE PIEUP YE RIEULPHIEUPH
    ((0x93, 0xED), (0xBCE7)),  # HANGUL SYLLABLE PIEUP YE RIEULHIEUH
    ((0x93, 0xEE), (0xBCE8)),  # HANGUL SYLLABLE PIEUP YE MIEUM
    ((0x93, 0xEF), (0xBCE9)),  # HANGUL SYLLABLE PIEUP YE PIEUP
    ((0x93, 0xF0), (0xBCEA)),  # HANGUL SYLLABLE PIEUP YE PIEUPSIOS
    ((0x93, 0xF1), (0xBCEB)),  # HANGUL SYLLABLE PIEUP YE SIOS
    ((0x93, 0xF2), (0xBCEC)),  # HANGUL SYLLABLE PIEUP YE SSANGSIOS
    ((0x93, 0xF3), (0xBCED)),  # HANGUL SYLLABLE PIEUP YE IEUNG
    ((0x93, 0xF4), (0xBCEE)),  # HANGUL SYLLABLE PIEUP YE CIEUC
    ((0x93, 0xF5), (0xBCEF)),  # HANGUL SYLLABLE PIEUP YE CHIEUCH
    ((0x93, 0xF6), (0xBCF0)),  # HANGUL SYLLABLE PIEUP YE KHIEUKH
    ((0x93, 0xF7), (0xBCF1)),  # HANGUL SYLLABLE PIEUP YE THIEUTH
    ((0x93, 0xF8), (0xBCF2)),  # HANGUL SYLLABLE PIEUP YE PHIEUPH
    ((0x93, 0xF9), (0xBCF3)),  # HANGUL SYLLABLE PIEUP YE HIEUH
    ((0x93, 0xFA), (0xBCF7)),  # HANGUL SYLLABLE PIEUP O KIYEOKSIOS
    ((0x93, 0xFB), (0xBCF9)),  # HANGUL SYLLABLE PIEUP O NIEUNCIEUC
    ((0x93, 0xFC), (0xBCFA)),  # HANGUL SYLLABLE PIEUP O NIEUNHIEUH
    ((0x93, 0xFD), (0xBCFB)),  # HANGUL SYLLABLE PIEUP O TIKEUT
    ((0x93, 0xFE), (0xBCFD)),  # HANGUL SYLLABLE PIEUP O RIEULKIYEOK
    ((0x94, 0x41), (0xBCFE)),  # HANGUL SYLLABLE PIEUP O RIEULMIEUM
    ((0x94, 0x42), (0xBCFF)),  # HANGUL SYLLABLE PIEUP O RIEULPIEUP
    ((0x94, 0x43), (0xBD00)),  # HANGUL SYLLABLE PIEUP O RIEULSIOS
    ((0x94, 0x44), (0xBD01)),  # HANGUL SYLLABLE PIEUP O RIEULTHIEUTH
    ((0x94, 0x45), (0xBD02)),  # HANGUL SYLLABLE PIEUP O RIEULPHIEUPH
    ((0x94, 0x46), (0xBD03)),  # HANGUL SYLLABLE PIEUP O RIEULHIEUH
    ((0x94, 0x47), (0xBD06)),  # HANGUL SYLLABLE PIEUP O PIEUPSIOS
    ((0x94, 0x48), (0xBD08)),  # HANGUL SYLLABLE PIEUP O SSANGSIOS
    ((0x94, 0x49), (0xBD0A)),  # HANGUL SYLLABLE PIEUP O CIEUC
    ((0x94, 0x4A), (0xBD0B)),  # HANGUL SYLLABLE PIEUP O CHIEUCH
    ((0x94, 0x4B), (0xBD0C)),  # HANGUL SYLLABLE PIEUP O KHIEUKH
    ((0x94, 0x4C), (0xBD0D)),  # HANGUL SYLLABLE PIEUP O THIEUTH
    ((0x94, 0x4D), (0xBD0E)),  # HANGUL SYLLABLE PIEUP O PHIEUPH
    ((0x94, 0x4E), (0xBD0F)),  # HANGUL SYLLABLE PIEUP O HIEUH
    ((0x94, 0x4F), (0xBD11)),  # HANGUL SYLLABLE PIEUP WA KIYEOK
    ((0x94, 0x50), (0xBD12)),  # HANGUL SYLLABLE PIEUP WA SSANGKIYEOK
    ((0x94, 0x51), (0xBD13)),  # HANGUL SYLLABLE PIEUP WA KIYEOKSIOS
    ((0x94, 0x52), (0xBD15)),  # HANGUL SYLLABLE PIEUP WA NIEUNCIEUC
    ((0x94, 0x53), (0xBD16)),  # HANGUL SYLLABLE PIEUP WA NIEUNHIEUH
    ((0x94, 0x54), (0xBD17)),  # HANGUL SYLLABLE PIEUP WA TIKEUT
    ((0x94, 0x55), (0xBD18)),  # HANGUL SYLLABLE PIEUP WA RIEUL
    ((0x94, 0x56), (0xBD19)),  # HANGUL SYLLABLE PIEUP WA RIEULKIYEOK
    ((0x94, 0x57), (0xBD1A)),  # HANGUL SYLLABLE PIEUP WA RIEULMIEUM
    ((0x94, 0x58), (0xBD1B)),  # HANGUL SYLLABLE PIEUP WA RIEULPIEUP
    ((0x94, 0x59), (0xBD1C)),  # HANGUL SYLLABLE PIEUP WA RIEULSIOS
    ((0x94, 0x5A), (0xBD1D)),  # HANGUL SYLLABLE PIEUP WA RIEULTHIEUTH
    ((0x94, 0x61), (0xBD1E)),  # HANGUL SYLLABLE PIEUP WA RIEULPHIEUPH
    ((0x94, 0x62), (0xBD1F)),  # HANGUL SYLLABLE PIEUP WA RIEULHIEUH
    ((0x94, 0x63), (0xBD20)),  # HANGUL SYLLABLE PIEUP WA MIEUM
    ((0x94, 0x64), (0xBD21)),  # HANGUL SYLLABLE PIEUP WA PIEUP
    ((0x94, 0x65), (0xBD22)),  # HANGUL SYLLABLE PIEUP WA PIEUPSIOS
    ((0x94, 0x66), (0xBD23)),  # HANGUL SYLLABLE PIEUP WA SIOS
    ((0x94, 0x67), (0xBD25)),  # HANGUL SYLLABLE PIEUP WA IEUNG
    ((0x94, 0x68), (0xBD26)),  # HANGUL SYLLABLE PIEUP WA CIEUC
    ((0x94, 0x69), (0xBD27)),  # HANGUL SYLLABLE PIEUP WA CHIEUCH
    ((0x94, 0x6A), (0xBD28)),  # HANGUL SYLLABLE PIEUP WA KHIEUKH
    ((0x94, 0x6B), (0xBD29)),  # HANGUL SYLLABLE PIEUP WA THIEUTH
    ((0x94, 0x6C), (0xBD2A)),  # HANGUL SYLLABLE PIEUP WA PHIEUPH
    ((0x94, 0x6D), (0xBD2B)),  # HANGUL SYLLABLE PIEUP WA HIEUH
    ((0x94, 0x6E), (0xBD2D)),  # HANGUL SYLLABLE PIEUP WAE KIYEOK
    ((0x94, 0x6F), (0xBD2E)),  # HANGUL SYLLABLE PIEUP WAE SSANGKIYEOK
    ((0x94, 0x70), (0xBD2F)),  # HANGUL SYLLABLE PIEUP WAE KIYEOKSIOS
    ((0x94, 0x71), (0xBD30)),  # HANGUL SYLLABLE PIEUP WAE NIEUN
    ((0x94, 0x72), (0xBD31)),  # HANGUL SYLLABLE PIEUP WAE NIEUNCIEUC
    ((0x94, 0x73), (0xBD32)),  # HANGUL SYLLABLE PIEUP WAE NIEUNHIEUH
    ((0x94, 0x74), (0xBD33)),  # HANGUL SYLLABLE PIEUP WAE TIKEUT
    ((0x94, 0x75), (0xBD34)),  # HANGUL SYLLABLE PIEUP WAE RIEUL
    ((0x94, 0x76), (0xBD35)),  # HANGUL SYLLABLE PIEUP WAE RIEULKIYEOK
    ((0x94, 0x77), (0xBD36)),  # HANGUL SYLLABLE PIEUP WAE RIEULMIEUM
    ((0x94, 0x78), (0xBD37)),  # HANGUL SYLLABLE PIEUP WAE RIEULPIEUP
    ((0x94, 0x79), (0xBD38)),  # HANGUL SYLLABLE PIEUP WAE RIEULSIOS
    ((0x94, 0x7A), (0xBD39)),  # HANGUL SYLLABLE PIEUP WAE RIEULTHIEUTH
    ((0x94, 0x81), (0xBD3A)),  # HANGUL SYLLABLE PIEUP WAE RIEULPHIEUPH
    ((0x94, 0x82), (0xBD3B)),  # HANGUL SYLLABLE PIEUP WAE RIEULHIEUH
    ((0x94, 0x83), (0xBD3C)),  # HANGUL SYLLABLE PIEUP WAE MIEUM
    ((0x94, 0x84), (0xBD3D)),  # HANGUL SYLLABLE PIEUP WAE PIEUP
    ((0x94, 0x85), (0xBD3E)),  # HANGUL SYLLABLE PIEUP WAE PIEUPSIOS
    ((0x94, 0x86), (0xBD3F)),  # HANGUL SYLLABLE PIEUP WAE SIOS
    ((0x94, 0x87), (0xBD41)),  # HANGUL SYLLABLE PIEUP WAE IEUNG
    ((0x94, 0x88), (0xBD42)),  # HANGUL SYLLABLE PIEUP WAE CIEUC
    ((0x94, 0x89), (0xBD43)),  # HANGUL SYLLABLE PIEUP WAE CHIEUCH
    ((0x94, 0x8A), (0xBD44)),  # HANGUL SYLLABLE PIEUP WAE KHIEUKH
    ((0x94, 0x8B), (0xBD45)),  # HANGUL SYLLABLE PIEUP WAE THIEUTH
    ((0x94, 0x8C), (0xBD46)),  # HANGUL SYLLABLE PIEUP WAE PHIEUPH
    ((0x94, 0x8D), (0xBD47)),  # HANGUL SYLLABLE PIEUP WAE HIEUH
    ((0x94, 0x8E), (0xBD4A)),  # HANGUL SYLLABLE PIEUP OE SSANGKIYEOK
    ((0x94, 0x8F), (0xBD4B)),  # HANGUL SYLLABLE PIEUP OE KIYEOKSIOS
    ((0x94, 0x90), (0xBD4D)),  # HANGUL SYLLABLE PIEUP OE NIEUNCIEUC
    ((0x94, 0x91), (0xBD4E)),  # HANGUL SYLLABLE PIEUP OE NIEUNHIEUH
    ((0x94, 0x92), (0xBD4F)),  # HANGUL SYLLABLE PIEUP OE TIKEUT
    ((0x94, 0x93), (0xBD51)),  # HANGUL SYLLABLE PIEUP OE RIEULKIYEOK
    ((0x94, 0x94), (0xBD52)),  # HANGUL SYLLABLE PIEUP OE RIEULMIEUM
    ((0x94, 0x95), (0xBD53)),  # HANGUL SYLLABLE PIEUP OE RIEULPIEUP
    ((0x94, 0x96), (0xBD54)),  # HANGUL SYLLABLE PIEUP OE RIEULSIOS
    ((0x94, 0x97), (0xBD55)),  # HANGUL SYLLABLE PIEUP OE RIEULTHIEUTH
    ((0x94, 0x98), (0xBD56)),  # HANGUL SYLLABLE PIEUP OE RIEULPHIEUPH
    ((0x94, 0x99), (0xBD57)),  # HANGUL SYLLABLE PIEUP OE RIEULHIEUH
    ((0x94, 0x9A), (0xBD5A)),  # HANGUL SYLLABLE PIEUP OE PIEUPSIOS
    ((0x94, 0x9B), (0xBD5B)),  # HANGUL SYLLABLE PIEUP OE SIOS
    ((0x94, 0x9C), (0xBD5C)),  # HANGUL SYLLABLE PIEUP OE SSANGSIOS
    ((0x94, 0x9D), (0xBD5D)),  # HANGUL SYLLABLE PIEUP OE IEUNG
    ((0x94, 0x9E), (0xBD5E)),  # HANGUL SYLLABLE PIEUP OE CIEUC
    ((0x94, 0x9F), (0xBD5F)),  # HANGUL SYLLABLE PIEUP OE CHIEUCH
    ((0x94, 0xA0), (0xBD60)),  # HANGUL SYLLABLE PIEUP OE KHIEUKH
    ((0x94, 0xA1), (0xBD61)),  # HANGUL SYLLABLE PIEUP OE THIEUTH
    ((0x94, 0xA2), (0xBD62)),  # HANGUL SYLLABLE PIEUP OE PHIEUPH
    ((0x94, 0xA3), (0xBD63)),  # HANGUL SYLLABLE PIEUP OE HIEUH
    ((0x94, 0xA4), (0xBD65)),  # HANGUL SYLLABLE PIEUP YO KIYEOK
    ((0x94, 0xA5), (0xBD66)),  # HANGUL SYLLABLE PIEUP YO SSANGKIYEOK
    ((0x94, 0xA6), (0xBD67)),  # HANGUL SYLLABLE PIEUP YO KIYEOKSIOS
    ((0x94, 0xA7), (0xBD69)),  # HANGUL SYLLABLE PIEUP YO NIEUNCIEUC
    ((0x94, 0xA8), (0xBD6A)),  # HANGUL SYLLABLE PIEUP YO NIEUNHIEUH
    ((0x94, 0xA9), (0xBD6B)),  # HANGUL SYLLABLE PIEUP YO TIKEUT
    ((0x94, 0xAA), (0xBD6C)),  # HANGUL SYLLABLE PIEUP YO RIEUL
    ((0x94, 0xAB), (0xBD6D)),  # HANGUL SYLLABLE PIEUP YO RIEULKIYEOK
    ((0x94, 0xAC), (0xBD6E)),  # HANGUL SYLLABLE PIEUP YO RIEULMIEUM
    ((0x94, 0xAD), (0xBD6F)),  # HANGUL SYLLABLE PIEUP YO RIEULPIEUP
    ((0x94, 0xAE), (0xBD70)),  # HANGUL SYLLABLE PIEUP YO RIEULSIOS
    ((0x94, 0xAF), (0xBD71)),  # HANGUL SYLLABLE PIEUP YO RIEULTHIEUTH
    ((0x94, 0xB0), (0xBD72)),  # HANGUL SYLLABLE PIEUP YO RIEULPHIEUPH
    ((0x94, 0xB1), (0xBD73)),  # HANGUL SYLLABLE PIEUP YO RIEULHIEUH
    ((0x94, 0xB2), (0xBD74)),  # HANGUL SYLLABLE PIEUP YO MIEUM
    ((0x94, 0xB3), (0xBD75)),  # HANGUL SYLLABLE PIEUP YO PIEUP
    ((0x94, 0xB4), (0xBD76)),  # HANGUL SYLLABLE PIEUP YO PIEUPSIOS
    ((0x94, 0xB5), (0xBD77)),  # HANGUL SYLLABLE PIEUP YO SIOS
    ((0x94, 0xB6), (0xBD78)),  # HANGUL SYLLABLE PIEUP YO SSANGSIOS
    ((0x94, 0xB7), (0xBD79)),  # HANGUL SYLLABLE PIEUP YO IEUNG
    ((0x94, 0xB8), (0xBD7A)),  # HANGUL SYLLABLE PIEUP YO CIEUC
    ((0x94, 0xB9), (0xBD7B)),  # HANGUL SYLLABLE PIEUP YO CHIEUCH
    ((0x94, 0xBA), (0xBD7C)),  # HANGUL SYLLABLE PIEUP YO KHIEUKH
    ((0x94, 0xBB), (0xBD7D)),  # HANGUL SYLLABLE PIEUP YO THIEUTH
    ((0x94, 0xBC), (0xBD7E)),  # HANGUL SYLLABLE PIEUP YO PHIEUPH
    ((0x94, 0xBD), (0xBD7F)),  # HANGUL SYLLABLE PIEUP YO HIEUH
    ((0x94, 0xBE), (0xBD82)),  # HANGUL SYLLABLE PIEUP U SSANGKIYEOK
    ((0x94, 0xBF), (0xBD83)),  # HANGUL SYLLABLE PIEUP U KIYEOKSIOS
    ((0x94, 0xC0), (0xBD85)),  # HANGUL SYLLABLE PIEUP U NIEUNCIEUC
    ((0x94, 0xC1), (0xBD86)),  # HANGUL SYLLABLE PIEUP U NIEUNHIEUH
    ((0x94, 0xC2), (0xBD8B)),  # HANGUL SYLLABLE PIEUP U RIEULPIEUP
    ((0x94, 0xC3), (0xBD8C)),  # HANGUL SYLLABLE PIEUP U RIEULSIOS
    ((0x94, 0xC4), (0xBD8D)),  # HANGUL SYLLABLE PIEUP U RIEULTHIEUTH
    ((0x94, 0xC5), (0xBD8E)),  # HANGUL SYLLABLE PIEUP U RIEULPHIEUPH
    ((0x94, 0xC6), (0xBD8F)),  # HANGUL SYLLABLE PIEUP U RIEULHIEUH
    ((0x94, 0xC7), (0xBD92)),  # HANGUL SYLLABLE PIEUP U PIEUPSIOS
    ((0x94, 0xC8), (0xBD94)),  # HANGUL SYLLABLE PIEUP U SSANGSIOS
    ((0x94, 0xC9), (0xBD96)),  # HANGUL SYLLABLE PIEUP U CIEUC
    ((0x94, 0xCA), (0xBD97)),  # HANGUL SYLLABLE PIEUP U CHIEUCH
    ((0x94, 0xCB), (0xBD98)),  # HANGUL SYLLABLE PIEUP U KHIEUKH
    ((0x94, 0xCC), (0xBD9B)),  # HANGUL SYLLABLE PIEUP U HIEUH
    ((0x94, 0xCD), (0xBD9D)),  # HANGUL SYLLABLE PIEUP WEO KIYEOK
    ((0x94, 0xCE), (0xBD9E)),  # HANGUL SYLLABLE PIEUP WEO SSANGKIYEOK
    ((0x94, 0xCF), (0xBD9F)),  # HANGUL SYLLABLE PIEUP WEO KIYEOKSIOS
    ((0x94, 0xD0), (0xBDA0)),  # HANGUL SYLLABLE PIEUP WEO NIEUN
    ((0x94, 0xD1), (0xBDA1)),  # HANGUL SYLLABLE PIEUP WEO NIEUNCIEUC
    ((0x94, 0xD2), (0xBDA2)),  # HANGUL SYLLABLE PIEUP WEO NIEUNHIEUH
    ((0x94, 0xD3), (0xBDA3)),  # HANGUL SYLLABLE PIEUP WEO TIKEUT
    ((0x94, 0xD4), (0xBDA5)),  # HANGUL SYLLABLE PIEUP WEO RIEULKIYEOK
    ((0x94, 0xD5), (0xBDA6)),  # HANGUL SYLLABLE PIEUP WEO RIEULMIEUM
    ((0x94, 0xD6), (0xBDA7)),  # HANGUL SYLLABLE PIEUP WEO RIEULPIEUP
    ((0x94, 0xD7), (0xBDA8)),  # HANGUL SYLLABLE PIEUP WEO RIEULSIOS
    ((0x94, 0xD8), (0xBDA9)),  # HANGUL SYLLABLE PIEUP WEO RIEULTHIEUTH
    ((0x94, 0xD9), (0xBDAA)),  # HANGUL SYLLABLE PIEUP WEO RIEULPHIEUPH
    ((0x94, 0xDA), (0xBDAB)),  # HANGUL SYLLABLE PIEUP WEO RIEULHIEUH
    ((0x94, 0xDB), (0xBDAC)),  # HANGUL SYLLABLE PIEUP WEO MIEUM
    ((0x94, 0xDC), (0xBDAD)),  # HANGUL SYLLABLE PIEUP WEO PIEUP
    ((0x94, 0xDD), (0xBDAE)),  # HANGUL SYLLABLE PIEUP WEO PIEUPSIOS
    ((0x94, 0xDE), (0xBDAF)),  # HANGUL SYLLABLE PIEUP WEO SIOS
    ((0x94, 0xDF), (0xBDB1)),  # HANGUL SYLLABLE PIEUP WEO IEUNG
    ((0x94, 0xE0), (0xBDB2)),  # HANGUL SYLLABLE PIEUP WEO CIEUC
    ((0x94, 0xE1), (0xBDB3)),  # HANGUL SYLLABLE PIEUP WEO CHIEUCH
    ((0x94, 0xE2), (0xBDB4)),  # HANGUL SYLLABLE PIEUP WEO KHIEUKH
    ((0x94, 0xE3), (0xBDB5)),  # HANGUL SYLLABLE PIEUP WEO THIEUTH
    ((0x94, 0xE4), (0xBDB6)),  # HANGUL SYLLABLE PIEUP WEO PHIEUPH
    ((0x94, 0xE5), (0xBDB7)),  # HANGUL SYLLABLE PIEUP WEO HIEUH
    ((0x94, 0xE6), (0xBDB9)),  # HANGUL SYLLABLE PIEUP WE KIYEOK
    ((0x94, 0xE7), (0xBDBA)),  # HANGUL SYLLABLE PIEUP WE SSANGKIYEOK
    ((0x94, 0xE8), (0xBDBB)),  # HANGUL SYLLABLE PIEUP WE KIYEOKSIOS
    ((0x94, 0xE9), (0xBDBC)),  # HANGUL SYLLABLE PIEUP WE NIEUN
    ((0x94, 0xEA), (0xBDBD)),  # HANGUL SYLLABLE PIEUP WE NIEUNCIEUC
    ((0x94, 0xEB), (0xBDBE)),  # HANGUL SYLLABLE PIEUP WE NIEUNHIEUH
    ((0x94, 0xEC), (0xBDBF)),  # HANGUL SYLLABLE PIEUP WE TIKEUT
    ((0x94, 0xED), (0xBDC0)),  # HANGUL SYLLABLE PIEUP WE RIEUL
    ((0x94, 0xEE), (0xBDC1)),  # HANGUL SYLLABLE PIEUP WE RIEULKIYEOK
    ((0x94, 0xEF), (0xBDC2)),  # HANGUL SYLLABLE PIEUP WE RIEULMIEUM
    ((0x94, 0xF0), (0xBDC3)),  # HANGUL SYLLABLE PIEUP WE RIEULPIEUP
    ((0x94, 0xF1), (0xBDC4)),  # HANGUL SYLLABLE PIEUP WE RIEULSIOS
    ((0x94, 0xF2), (0xBDC5)),  # HANGUL SYLLABLE PIEUP WE RIEULTHIEUTH
    ((0x94, 0xF3), (0xBDC6)),  # HANGUL SYLLABLE PIEUP WE RIEULPHIEUPH
    ((0x94, 0xF4), (0xBDC7)),  # HANGUL SYLLABLE PIEUP WE RIEULHIEUH
    ((0x94, 0xF5), (0xBDC8)),  # HANGUL SYLLABLE PIEUP WE MIEUM
    ((0x94, 0xF6), (0xBDC9)),  # HANGUL SYLLABLE PIEUP WE PIEUP
    ((0x94, 0xF7), (0xBDCA)),  # HANGUL SYLLABLE PIEUP WE PIEUPSIOS
    ((0x94, 0xF8), (0xBDCB)),  # HANGUL SYLLABLE PIEUP WE SIOS
    ((0x94, 0xF9), (0xBDCC)),  # HANGUL SYLLABLE PIEUP WE SSANGSIOS
    ((0x94, 0xFA), (0xBDCD)),  # HANGUL SYLLABLE PIEUP WE IEUNG
    ((0x94, 0xFB), (0xBDCE)),  # HANGUL SYLLABLE PIEUP WE CIEUC
    ((0x94, 0xFC), (0xBDCF)),  # HANGUL SYLLABLE PIEUP WE CHIEUCH
    ((0x94, 0xFD), (0xBDD0)),  # HANGUL SYLLABLE PIEUP WE KHIEUKH
    ((0x94, 0xFE), (0xBDD1)),  # HANGUL SYLLABLE PIEUP WE THIEUTH
    ((0x95, 0x41), (0xBDD2)),  # HANGUL SYLLABLE PIEUP WE PHIEUPH
    ((0x95, 0x42), (0xBDD3)),  # HANGUL SYLLABLE PIEUP WE HIEUH
    ((0x95, 0x43), (0xBDD6)),  # HANGUL SYLLABLE PIEUP WI SSANGKIYEOK
    ((0x95, 0x44), (0xBDD7)),  # HANGUL SYLLABLE PIEUP WI KIYEOKSIOS
    ((0x95, 0x45), (0xBDD9)),  # HANGUL SYLLABLE PIEUP WI NIEUNCIEUC
    ((0x95, 0x46), (0xBDDA)),  # HANGUL SYLLABLE PIEUP WI NIEUNHIEUH
    ((0x95, 0x47), (0xBDDB)),  # HANGUL SYLLABLE PIEUP WI TIKEUT
    ((0x95, 0x48), (0xBDDD)),  # HANGUL SYLLABLE PIEUP WI RIEULKIYEOK
    ((0x95, 0x49), (0xBDDE)),  # HANGUL SYLLABLE PIEUP WI RIEULMIEUM
    ((0x95, 0x4A), (0xBDDF)),  # HANGUL SYLLABLE PIEUP WI RIEULPIEUP
    ((0x95, 0x4B), (0xBDE0)),  # HANGUL SYLLABLE PIEUP WI RIEULSIOS
    ((0x95, 0x4C), (0xBDE1)),  # HANGUL SYLLABLE PIEUP WI RIEULTHIEUTH
    ((0x95, 0x4D), (0xBDE2)),  # HANGUL SYLLABLE PIEUP WI RIEULPHIEUPH
    ((0x95, 0x4E), (0xBDE3)),  # HANGUL SYLLABLE PIEUP WI RIEULHIEUH
    ((0x95, 0x4F), (0xBDE4)),  # HANGUL SYLLABLE PIEUP WI MIEUM
    ((0x95, 0x50), (0xBDE5)),  # HANGUL SYLLABLE PIEUP WI PIEUP
    ((0x95, 0x51), (0xBDE6)),  # HANGUL SYLLABLE PIEUP WI PIEUPSIOS
    ((0x95, 0x52), (0xBDE7)),  # HANGUL SYLLABLE PIEUP WI SIOS
    ((0x95, 0x53), (0xBDE8)),  # HANGUL SYLLABLE PIEUP WI SSANGSIOS
    ((0x95, 0x54), (0xBDEA)),  # HANGUL SYLLABLE PIEUP WI CIEUC
    ((0x95, 0x55), (0xBDEB)),  # HANGUL SYLLABLE PIEUP WI CHIEUCH
    ((0x95, 0x56), (0xBDEC)),  # HANGUL SYLLABLE PIEUP WI KHIEUKH
    ((0x95, 0x57), (0xBDED)),  # HANGUL SYLLABLE PIEUP WI THIEUTH
    ((0x95, 0x58), (0xBDEE)),  # HANGUL SYLLABLE PIEUP WI PHIEUPH
    ((0x95, 0x59), (0xBDEF)),  # HANGUL SYLLABLE PIEUP WI HIEUH
    ((0x95, 0x5A), (0xBDF1)),  # HANGUL SYLLABLE PIEUP YU KIYEOK
    ((0x95, 0x61), (0xBDF2)),  # HANGUL SYLLABLE PIEUP YU SSANGKIYEOK
    ((0x95, 0x62), (0xBDF3)),  # HANGUL SYLLABLE PIEUP YU KIYEOKSIOS
    ((0x95, 0x63), (0xBDF5)),  # HANGUL SYLLABLE PIEUP YU NIEUNCIEUC
    ((0x95, 0x64), (0xBDF6)),  # HANGUL SYLLABLE PIEUP YU NIEUNHIEUH
    ((0x95, 0x65), (0xBDF7)),  # HANGUL SYLLABLE PIEUP YU TIKEUT
    ((0x95, 0x66), (0xBDF9)),  # HANGUL SYLLABLE PIEUP YU RIEULKIYEOK
    ((0x95, 0x67), (0xBDFA)),  # HANGUL SYLLABLE PIEUP YU RIEULMIEUM
    ((0x95, 0x68), (0xBDFB)),  # HANGUL SYLLABLE PIEUP YU RIEULPIEUP
    ((0x95, 0x69), (0xBDFC)),  # HANGUL SYLLABLE PIEUP YU RIEULSIOS
    ((0x95, 0x6A), (0xBDFD)),  # HANGUL SYLLABLE PIEUP YU RIEULTHIEUTH
    ((0x95, 0x6B), (0xBDFE)),  # HANGUL SYLLABLE PIEUP YU RIEULPHIEUPH
    ((0x95, 0x6C), (0xBDFF)),  # HANGUL SYLLABLE PIEUP YU RIEULHIEUH
    ((0x95, 0x6D), (0xBE01)),  # HANGUL SYLLABLE PIEUP YU PIEUP
    ((0x95, 0x6E), (0xBE02)),  # HANGUL SYLLABLE PIEUP YU PIEUPSIOS
    ((0x95, 0x6F), (0xBE04)),  # HANGUL SYLLABLE PIEUP YU SSANGSIOS
    ((0x95, 0x70), (0xBE06)),  # HANGUL SYLLABLE PIEUP YU CIEUC
    ((0x95, 0x71), (0xBE07)),  # HANGUL SYLLABLE PIEUP YU CHIEUCH
    ((0x95, 0x72), (0xBE08)),  # HANGUL SYLLABLE PIEUP YU KHIEUKH
    ((0x95, 0x73), (0xBE09)),  # HANGUL SYLLABLE PIEUP YU THIEUTH
    ((0x95, 0x74), (0xBE0A)),  # HANGUL SYLLABLE PIEUP YU PHIEUPH
    ((0x95, 0x75), (0xBE0B)),  # HANGUL SYLLABLE PIEUP YU HIEUH
    ((0x95, 0x76), (0xBE0E)),  # HANGUL SYLLABLE PIEUP EU SSANGKIYEOK
    ((0x95, 0x77), (0xBE0F)),  # HANGUL SYLLABLE PIEUP EU KIYEOKSIOS
    ((0x95, 0x78), (0xBE11)),  # HANGUL SYLLABLE PIEUP EU NIEUNCIEUC
    ((0x95, 0x79), (0xBE12)),  # HANGUL SYLLABLE PIEUP EU NIEUNHIEUH
    ((0x95, 0x7A), (0xBE13)),  # HANGUL SYLLABLE PIEUP EU TIKEUT
    ((0x95, 0x81), (0xBE15)),  # HANGUL SYLLABLE PIEUP EU RIEULKIYEOK
    ((0x95, 0x82), (0xBE16)),  # HANGUL SYLLABLE PIEUP EU RIEULMIEUM
    ((0x95, 0x83), (0xBE17)),  # HANGUL SYLLABLE PIEUP EU RIEULPIEUP
    ((0x95, 0x84), (0xBE18)),  # HANGUL SYLLABLE PIEUP EU RIEULSIOS
    ((0x95, 0x85), (0xBE19)),  # HANGUL SYLLABLE PIEUP EU RIEULTHIEUTH
    ((0x95, 0x86), (0xBE1A)),  # HANGUL SYLLABLE PIEUP EU RIEULPHIEUPH
    ((0x95, 0x87), (0xBE1B)),  # HANGUL SYLLABLE PIEUP EU RIEULHIEUH
    ((0x95, 0x88), (0xBE1E)),  # HANGUL SYLLABLE PIEUP EU PIEUPSIOS
    ((0x95, 0x89), (0xBE20)),  # HANGUL SYLLABLE PIEUP EU SSANGSIOS
    ((0x95, 0x8A), (0xBE21)),  # HANGUL SYLLABLE PIEUP EU IEUNG
    ((0x95, 0x8B), (0xBE22)),  # HANGUL SYLLABLE PIEUP EU CIEUC
    ((0x95, 0x8C), (0xBE23)),  # HANGUL SYLLABLE PIEUP EU CHIEUCH
    ((0x95, 0x8D), (0xBE24)),  # HANGUL SYLLABLE PIEUP EU KHIEUKH
    ((0x95, 0x8E), (0xBE25)),  # HANGUL SYLLABLE PIEUP EU THIEUTH
    ((0x95, 0x8F), (0xBE26)),  # HANGUL SYLLABLE PIEUP EU PHIEUPH
    ((0x95, 0x90), (0xBE27)),  # HANGUL SYLLABLE PIEUP EU HIEUH
    ((0x95, 0x91), (0xBE28)),  # HANGUL SYLLABLE PIEUP YI
    ((0x95, 0x92), (0xBE29)),  # HANGUL SYLLABLE PIEUP YI KIYEOK
    ((0x95, 0x93), (0xBE2A)),  # HANGUL SYLLABLE PIEUP YI SSANGKIYEOK
    ((0x95, 0x94), (0xBE2B)),  # HANGUL SYLLABLE PIEUP YI KIYEOKSIOS
    ((0x95, 0x95), (0xBE2C)),  # HANGUL SYLLABLE PIEUP YI NIEUN
    ((0x95, 0x96), (0xBE2D)),  # HANGUL SYLLABLE PIEUP YI NIEUNCIEUC
    ((0x95, 0x97), (0xBE2E)),  # HANGUL SYLLABLE PIEUP YI NIEUNHIEUH
    ((0x95, 0x98), (0xBE2F)),  # HANGUL SYLLABLE PIEUP YI TIKEUT
    ((0x95, 0x99), (0xBE30)),  # HANGUL SYLLABLE PIEUP YI RIEUL
    ((0x95, 0x9A), (0xBE31)),  # HANGUL SYLLABLE PIEUP YI RIEULKIYEOK
    ((0x95, 0x9B), (0xBE32)),  # HANGUL SYLLABLE PIEUP YI RIEULMIEUM
    ((0x95, 0x9C), (0xBE33)),  # HANGUL SYLLABLE PIEUP YI RIEULPIEUP
    ((0x95, 0x9D), (0xBE34)),  # HANGUL SYLLABLE PIEUP YI RIEULSIOS
    ((0x95, 0x9E), (0xBE35)),  # HANGUL SYLLABLE PIEUP YI RIEULTHIEUTH
    ((0x95, 0x9F), (0xBE36)),  # HANGUL SYLLABLE PIEUP YI RIEULPHIEUPH
    ((0x95, 0xA0), (0xBE37)),  # HANGUL SYLLABLE PIEUP YI RIEULHIEUH
    ((0x95, 0xA1), (0xBE38)),  # HANGUL SYLLABLE PIEUP YI MIEUM
    ((0x95, 0xA2), (0xBE39)),  # HANGUL SYLLABLE PIEUP YI PIEUP
    ((0x95, 0xA3), (0xBE3A)),  # HANGUL SYLLABLE PIEUP YI PIEUPSIOS
    ((0x95, 0xA4), (0xBE3B)),  # HANGUL SYLLABLE PIEUP YI SIOS
    ((0x95, 0xA5), (0xBE3C)),  # HANGUL SYLLABLE PIEUP YI SSANGSIOS
    ((0x95, 0xA6), (0xBE3D)),  # HANGUL SYLLABLE PIEUP YI IEUNG
    ((0x95, 0xA7), (0xBE3E)),  # HANGUL SYLLABLE PIEUP YI CIEUC
    ((0x95, 0xA8), (0xBE3F)),  # HANGUL SYLLABLE PIEUP YI CHIEUCH
    ((0x95, 0xA9), (0xBE40)),  # HANGUL SYLLABLE PIEUP YI KHIEUKH
    ((0x95, 0xAA), (0xBE41)),  # HANGUL SYLLABLE PIEUP YI THIEUTH
    ((0x95, 0xAB), (0xBE42)),  # HANGUL SYLLABLE PIEUP YI PHIEUPH
    ((0x95, 0xAC), (0xBE43)),  # HANGUL SYLLABLE PIEUP YI HIEUH
    ((0x95, 0xAD), (0xBE46)),  # HANGUL SYLLABLE PIEUP I SSANGKIYEOK
    ((0x95, 0xAE), (0xBE47)),  # HANGUL SYLLABLE PIEUP I KIYEOKSIOS
    ((0x95, 0xAF), (0xBE49)),  # HANGUL SYLLABLE PIEUP I NIEUNCIEUC
    ((0x95, 0xB0), (0xBE4A)),  # HANGUL SYLLABLE PIEUP I NIEUNHIEUH
    ((0x95, 0xB1), (0xBE4B)),  # HANGUL SYLLABLE PIEUP I TIKEUT
    ((0x95, 0xB2), (0xBE4D)),  # HANGUL SYLLABLE PIEUP I RIEULKIYEOK
    ((0x95, 0xB3), (0xBE4F)),  # HANGUL SYLLABLE PIEUP I RIEULPIEUP
    ((0x95, 0xB4), (0xBE50)),  # HANGUL SYLLABLE PIEUP I RIEULSIOS
    ((0x95, 0xB5), (0xBE51)),  # HANGUL SYLLABLE PIEUP I RIEULTHIEUTH
    ((0x95, 0xB6), (0xBE52)),  # HANGUL SYLLABLE PIEUP I RIEULPHIEUPH
    ((0x95, 0xB7), (0xBE53)),  # HANGUL SYLLABLE PIEUP I RIEULHIEUH
    ((0x95, 0xB8), (0xBE56)),  # HANGUL SYLLABLE PIEUP I PIEUPSIOS
    ((0x95, 0xB9), (0xBE58)),  # HANGUL SYLLABLE PIEUP I SSANGSIOS
    ((0x95, 0xBA), (0xBE5C)),  # HANGUL SYLLABLE PIEUP I KHIEUKH
    ((0x95, 0xBB), (0xBE5D)),  # HANGUL SYLLABLE PIEUP I THIEUTH
    ((0x95, 0xBC), (0xBE5E)),  # HANGUL SYLLABLE PIEUP I PHIEUPH
    ((0x95, 0xBD), (0xBE5F)),  # HANGUL SYLLABLE PIEUP I HIEUH
    ((0x95, 0xBE), (0xBE62)),  # HANGUL SYLLABLE SSANGPIEUP A SSANGKIYEOK
    ((0x95, 0xBF), (0xBE63)),  # HANGUL SYLLABLE SSANGPIEUP A KIYEOKSIOS
    ((0x95, 0xC0), (0xBE65)),  # HANGUL SYLLABLE SSANGPIEUP A NIEUNCIEUC
    ((0x95, 0xC1), (0xBE66)),  # HANGUL SYLLABLE SSANGPIEUP A NIEUNHIEUH
    ((0x95, 0xC2), (0xBE67)),  # HANGUL SYLLABLE SSANGPIEUP A TIKEUT
    ((0x95, 0xC3), (0xBE69)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULKIYEOK
    ((0x95, 0xC4), (0xBE6B)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULPIEUP
    ((0x95, 0xC5), (0xBE6C)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULSIOS
    ((0x95, 0xC6), (0xBE6D)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULTHIEUTH
    ((0x95, 0xC7), (0xBE6E)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULPHIEUPH
    ((0x95, 0xC8), (0xBE6F)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULHIEUH
    ((0x95, 0xC9), (0xBE72)),  # HANGUL SYLLABLE SSANGPIEUP A PIEUPSIOS
    ((0x95, 0xCA), (0xBE76)),  # HANGUL SYLLABLE SSANGPIEUP A CIEUC
    ((0x95, 0xCB), (0xBE77)),  # HANGUL SYLLABLE SSANGPIEUP A CHIEUCH
    ((0x95, 0xCC), (0xBE78)),  # HANGUL SYLLABLE SSANGPIEUP A KHIEUKH
    ((0x95, 0xCD), (0xBE79)),  # HANGUL SYLLABLE SSANGPIEUP A THIEUTH
    ((0x95, 0xCE), (0xBE7A)),  # HANGUL SYLLABLE SSANGPIEUP A PHIEUPH
    ((0x95, 0xCF), (0xBE7E)),  # HANGUL SYLLABLE SSANGPIEUP AE SSANGKIYEOK
    ((0x95, 0xD0), (0xBE7F)),  # HANGUL SYLLABLE SSANGPIEUP AE KIYEOKSIOS
    ((0x95, 0xD1), (0xBE81)),  # HANGUL SYLLABLE SSANGPIEUP AE NIEUNCIEUC
    ((0x95, 0xD2), (0xBE82)),  # HANGUL SYLLABLE SSANGPIEUP AE NIEUNHIEUH
    ((0x95, 0xD3), (0xBE83)),  # HANGUL SYLLABLE SSANGPIEUP AE TIKEUT
    ((0x95, 0xD4), (0xBE85)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULKIYEOK
    ((0x95, 0xD5), (0xBE86)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULMIEUM
    ((0x95, 0xD6), (0xBE87)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULPIEUP
    ((0x95, 0xD7), (0xBE88)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULSIOS
    ((0x95, 0xD8), (0xBE89)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULTHIEUTH
    ((0x95, 0xD9), (0xBE8A)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULPHIEUPH
    ((0x95, 0xDA), (0xBE8B)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEULHIEUH
    ((0x95, 0xDB), (0xBE8E)),  # HANGUL SYLLABLE SSANGPIEUP AE PIEUPSIOS
    ((0x95, 0xDC), (0xBE92)),  # HANGUL SYLLABLE SSANGPIEUP AE CIEUC
    ((0x95, 0xDD), (0xBE93)),  # HANGUL SYLLABLE SSANGPIEUP AE CHIEUCH
    ((0x95, 0xDE), (0xBE94)),  # HANGUL SYLLABLE SSANGPIEUP AE KHIEUKH
    ((0x95, 0xDF), (0xBE95)),  # HANGUL SYLLABLE SSANGPIEUP AE THIEUTH
    ((0x95, 0xE0), (0xBE96)),  # HANGUL SYLLABLE SSANGPIEUP AE PHIEUPH
    ((0x95, 0xE1), (0xBE97)),  # HANGUL SYLLABLE SSANGPIEUP AE HIEUH
    ((0x95, 0xE2), (0xBE9A)),  # HANGUL SYLLABLE SSANGPIEUP YA SSANGKIYEOK
    ((0x95, 0xE3), (0xBE9B)),  # HANGUL SYLLABLE SSANGPIEUP YA KIYEOKSIOS
    ((0x95, 0xE4), (0xBE9C)),  # HANGUL SYLLABLE SSANGPIEUP YA NIEUN
    ((0x95, 0xE5), (0xBE9D)),  # HANGUL SYLLABLE SSANGPIEUP YA NIEUNCIEUC
    ((0x95, 0xE6), (0xBE9E)),  # HANGUL SYLLABLE SSANGPIEUP YA NIEUNHIEUH
    ((0x95, 0xE7), (0xBE9F)),  # HANGUL SYLLABLE SSANGPIEUP YA TIKEUT
    ((0x95, 0xE8), (0xBEA0)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEUL
    ((0x95, 0xE9), (0xBEA1)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULKIYEOK
    ((0x95, 0xEA), (0xBEA2)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULMIEUM
    ((0x95, 0xEB), (0xBEA3)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULPIEUP
    ((0x95, 0xEC), (0xBEA4)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULSIOS
    ((0x95, 0xED), (0xBEA5)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULTHIEUTH
    ((0x95, 0xEE), (0xBEA6)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULPHIEUPH
    ((0x95, 0xEF), (0xBEA7)),  # HANGUL SYLLABLE SSANGPIEUP YA RIEULHIEUH
    ((0x95, 0xF0), (0xBEA9)),  # HANGUL SYLLABLE SSANGPIEUP YA PIEUP
    ((0x95, 0xF1), (0xBEAA)),  # HANGUL SYLLABLE SSANGPIEUP YA PIEUPSIOS
    ((0x95, 0xF2), (0xBEAB)),  # HANGUL SYLLABLE SSANGPIEUP YA SIOS
    ((0x95, 0xF3), (0xBEAC)),  # HANGUL SYLLABLE SSANGPIEUP YA SSANGSIOS
    ((0x95, 0xF4), (0xBEAD)),  # HANGUL SYLLABLE SSANGPIEUP YA IEUNG
    ((0x95, 0xF5), (0xBEAE)),  # HANGUL SYLLABLE SSANGPIEUP YA CIEUC
    ((0x95, 0xF6), (0xBEAF)),  # HANGUL SYLLABLE SSANGPIEUP YA CHIEUCH
    ((0x95, 0xF7), (0xBEB0)),  # HANGUL SYLLABLE SSANGPIEUP YA KHIEUKH
    ((0x95, 0xF8), (0xBEB1)),  # HANGUL SYLLABLE SSANGPIEUP YA THIEUTH
    ((0x95, 0xF9), (0xBEB2)),  # HANGUL SYLLABLE SSANGPIEUP YA PHIEUPH
    ((0x95, 0xFA), (0xBEB3)),  # HANGUL SYLLABLE SSANGPIEUP YA HIEUH
    ((0x95, 0xFB), (0xBEB4)),  # HANGUL SYLLABLE SSANGPIEUP YAE
    ((0x95, 0xFC), (0xBEB5)),  # HANGUL SYLLABLE SSANGPIEUP YAE KIYEOK
    ((0x95, 0xFD), (0xBEB6)),  # HANGUL SYLLABLE SSANGPIEUP YAE SSANGKIYEOK
    ((0x95, 0xFE), (0xBEB7)),  # HANGUL SYLLABLE SSANGPIEUP YAE KIYEOKSIOS
    ((0x96, 0x41), (0xBEB8)),  # HANGUL SYLLABLE SSANGPIEUP YAE NIEUN
    ((0x96, 0x42), (0xBEB9)),  # HANGUL SYLLABLE SSANGPIEUP YAE NIEUNCIEUC
    ((0x96, 0x43), (0xBEBA)),  # HANGUL SYLLABLE SSANGPIEUP YAE NIEUNHIEUH
    ((0x96, 0x44), (0xBEBB)),  # HANGUL SYLLABLE SSANGPIEUP YAE TIKEUT
    ((0x96, 0x45), (0xBEBC)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEUL
    ((0x96, 0x46), (0xBEBD)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULKIYEOK
    ((0x96, 0x47), (0xBEBE)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULMIEUM
    ((0x96, 0x48), (0xBEBF)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULPIEUP
    ((0x96, 0x49), (0xBEC0)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULSIOS
    ((0x96, 0x4A), (0xBEC1)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULTHIEUTH
    ((0x96, 0x4B), (0xBEC2)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULPHIEUPH
    ((0x96, 0x4C), (0xBEC3)),  # HANGUL SYLLABLE SSANGPIEUP YAE RIEULHIEUH
    ((0x96, 0x4D), (0xBEC4)),  # HANGUL SYLLABLE SSANGPIEUP YAE MIEUM
    ((0x96, 0x4E), (0xBEC5)),  # HANGUL SYLLABLE SSANGPIEUP YAE PIEUP
    ((0x96, 0x4F), (0xBEC6)),  # HANGUL SYLLABLE SSANGPIEUP YAE PIEUPSIOS
    ((0x96, 0x50), (0xBEC7)),  # HANGUL SYLLABLE SSANGPIEUP YAE SIOS
    ((0x96, 0x51), (0xBEC8)),  # HANGUL SYLLABLE SSANGPIEUP YAE SSANGSIOS
    ((0x96, 0x52), (0xBEC9)),  # HANGUL SYLLABLE SSANGPIEUP YAE IEUNG
    ((0x96, 0x53), (0xBECA)),  # HANGUL SYLLABLE SSANGPIEUP YAE CIEUC
    ((0x96, 0x54), (0xBECB)),  # HANGUL SYLLABLE SSANGPIEUP YAE CHIEUCH
    ((0x96, 0x55), (0xBECC)),  # HANGUL SYLLABLE SSANGPIEUP YAE KHIEUKH
    ((0x96, 0x56), (0xBECD)),  # HANGUL SYLLABLE SSANGPIEUP YAE THIEUTH
    ((0x96, 0x57), (0xBECE)),  # HANGUL SYLLABLE SSANGPIEUP YAE PHIEUPH
    ((0x96, 0x58), (0xBECF)),  # HANGUL SYLLABLE SSANGPIEUP YAE HIEUH
    ((0x96, 0x59), (0xBED2)),  # HANGUL SYLLABLE SSANGPIEUP EO SSANGKIYEOK
    ((0x96, 0x5A), (0xBED3)),  # HANGUL SYLLABLE SSANGPIEUP EO KIYEOKSIOS
    ((0x96, 0x61), (0xBED5)),  # HANGUL SYLLABLE SSANGPIEUP EO NIEUNCIEUC
    ((0x96, 0x62), (0xBED6)),  # HANGUL SYLLABLE SSANGPIEUP EO NIEUNHIEUH
    ((0x96, 0x63), (0xBED9)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULKIYEOK
    ((0x96, 0x64), (0xBEDA)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULMIEUM
    ((0x96, 0x65), (0xBEDB)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULPIEUP
    ((0x96, 0x66), (0xBEDC)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULSIOS
    ((0x96, 0x67), (0xBEDD)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULTHIEUTH
    ((0x96, 0x68), (0xBEDE)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULPHIEUPH
    ((0x96, 0x69), (0xBEDF)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEULHIEUH
    ((0x96, 0x6A), (0xBEE1)),  # HANGUL SYLLABLE SSANGPIEUP EO PIEUP
    ((0x96, 0x6B), (0xBEE2)),  # HANGUL SYLLABLE SSANGPIEUP EO PIEUPSIOS
    ((0x96, 0x6C), (0xBEE6)),  # HANGUL SYLLABLE SSANGPIEUP EO CIEUC
    ((0x96, 0x6D), (0xBEE7)),  # HANGUL SYLLABLE SSANGPIEUP EO CHIEUCH
    ((0x96, 0x6E), (0xBEE8)),  # HANGUL SYLLABLE SSANGPIEUP EO KHIEUKH
    ((0x96, 0x6F), (0xBEE9)),  # HANGUL SYLLABLE SSANGPIEUP EO THIEUTH
    ((0x96, 0x70), (0xBEEA)),  # HANGUL SYLLABLE SSANGPIEUP EO PHIEUPH
    ((0x96, 0x71), (0xBEEB)),  # HANGUL SYLLABLE SSANGPIEUP EO HIEUH
    ((0x96, 0x72), (0xBEED)),  # HANGUL SYLLABLE SSANGPIEUP E KIYEOK
    ((0x96, 0x73), (0xBEEE)),  # HANGUL SYLLABLE SSANGPIEUP E SSANGKIYEOK
    ((0x96, 0x74), (0xBEEF)),  # HANGUL SYLLABLE SSANGPIEUP E KIYEOKSIOS
    ((0x96, 0x75), (0xBEF0)),  # HANGUL SYLLABLE SSANGPIEUP E NIEUN
    ((0x96, 0x76), (0xBEF1)),  # HANGUL SYLLABLE SSANGPIEUP E NIEUNCIEUC
    ((0x96, 0x77), (0xBEF2)),  # HANGUL SYLLABLE SSANGPIEUP E NIEUNHIEUH
    ((0x96, 0x78), (0xBEF3)),  # HANGUL SYLLABLE SSANGPIEUP E TIKEUT
    ((0x96, 0x79), (0xBEF4)),  # HANGUL SYLLABLE SSANGPIEUP E RIEUL
    ((0x96, 0x7A), (0xBEF5)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULKIYEOK
    ((0x96, 0x81), (0xBEF6)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULMIEUM
    ((0x96, 0x82), (0xBEF7)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULPIEUP
    ((0x96, 0x83), (0xBEF8)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULSIOS
    ((0x96, 0x84), (0xBEF9)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULTHIEUTH
    ((0x96, 0x85), (0xBEFA)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULPHIEUPH
    ((0x96, 0x86), (0xBEFB)),  # HANGUL SYLLABLE SSANGPIEUP E RIEULHIEUH
    ((0x96, 0x87), (0xBEFC)),  # HANGUL SYLLABLE SSANGPIEUP E MIEUM
    ((0x96, 0x88), (0xBEFD)),  # HANGUL SYLLABLE SSANGPIEUP E PIEUP
    ((0x96, 0x89), (0xBEFE)),  # HANGUL SYLLABLE SSANGPIEUP E PIEUPSIOS
    ((0x96, 0x8A), (0xBEFF)),  # HANGUL SYLLABLE SSANGPIEUP E SIOS
    ((0x96, 0x8B), (0xBF00)),  # HANGUL SYLLABLE SSANGPIEUP E SSANGSIOS
    ((0x96, 0x8C), (0xBF02)),  # HANGUL SYLLABLE SSANGPIEUP E CIEUC
    ((0x96, 0x8D), (0xBF03)),  # HANGUL SYLLABLE SSANGPIEUP E CHIEUCH
    ((0x96, 0x8E), (0xBF04)),  # HANGUL SYLLABLE SSANGPIEUP E KHIEUKH
    ((0x96, 0x8F), (0xBF05)),  # HANGUL SYLLABLE SSANGPIEUP E THIEUTH
    ((0x96, 0x90), (0xBF06)),  # HANGUL SYLLABLE SSANGPIEUP E PHIEUPH
    ((0x96, 0x91), (0xBF07)),  # HANGUL SYLLABLE SSANGPIEUP E HIEUH
    ((0x96, 0x92), (0xBF0A)),  # HANGUL SYLLABLE SSANGPIEUP YEO SSANGKIYEOK
    ((0x96, 0x93), (0xBF0B)),  # HANGUL SYLLABLE SSANGPIEUP YEO KIYEOKSIOS
    ((0x96, 0x94), (0xBF0C)),  # HANGUL SYLLABLE SSANGPIEUP YEO NIEUN
    ((0x96, 0x95), (0xBF0D)),  # HANGUL SYLLABLE SSANGPIEUP YEO NIEUNCIEUC
    ((0x96, 0x96), (0xBF0E)),  # HANGUL SYLLABLE SSANGPIEUP YEO NIEUNHIEUH
    ((0x96, 0x97), (0xBF0F)),  # HANGUL SYLLABLE SSANGPIEUP YEO TIKEUT
    ((0x96, 0x98), (0xBF10)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEUL
    ((0x96, 0x99), (0xBF11)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULKIYEOK
    ((0x96, 0x9A), (0xBF12)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULMIEUM
    ((0x96, 0x9B), (0xBF13)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULPIEUP
    ((0x96, 0x9C), (0xBF14)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULSIOS
    ((0x96, 0x9D), (0xBF15)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULTHIEUTH
    ((0x96, 0x9E), (0xBF16)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULPHIEUPH
    ((0x96, 0x9F), (0xBF17)),  # HANGUL SYLLABLE SSANGPIEUP YEO RIEULHIEUH
    ((0x96, 0xA0), (0xBF1A)),  # HANGUL SYLLABLE SSANGPIEUP YEO PIEUPSIOS
    ((0x96, 0xA1), (0xBF1E)),  # HANGUL SYLLABLE SSANGPIEUP YEO CIEUC
    ((0x96, 0xA2), (0xBF1F)),  # HANGUL SYLLABLE SSANGPIEUP YEO CHIEUCH
    ((0x96, 0xA3), (0xBF20)),  # HANGUL SYLLABLE SSANGPIEUP YEO KHIEUKH
    ((0x96, 0xA4), (0xBF21)),  # HANGUL SYLLABLE SSANGPIEUP YEO THIEUTH
    ((0x96, 0xA5), (0xBF22)),  # HANGUL SYLLABLE SSANGPIEUP YEO PHIEUPH
    ((0x96, 0xA6), (0xBF23)),  # HANGUL SYLLABLE SSANGPIEUP YEO HIEUH
    ((0x96, 0xA7), (0xBF24)),  # HANGUL SYLLABLE SSANGPIEUP YE
    ((0x96, 0xA8), (0xBF25)),  # HANGUL SYLLABLE SSANGPIEUP YE KIYEOK
    ((0x96, 0xA9), (0xBF26)),  # HANGUL SYLLABLE SSANGPIEUP YE SSANGKIYEOK
    ((0x96, 0xAA), (0xBF27)),  # HANGUL SYLLABLE SSANGPIEUP YE KIYEOKSIOS
    ((0x96, 0xAB), (0xBF28)),  # HANGUL SYLLABLE SSANGPIEUP YE NIEUN
    ((0x96, 0xAC), (0xBF29)),  # HANGUL SYLLABLE SSANGPIEUP YE NIEUNCIEUC
    ((0x96, 0xAD), (0xBF2A)),  # HANGUL SYLLABLE SSANGPIEUP YE NIEUNHIEUH
    ((0x96, 0xAE), (0xBF2B)),  # HANGUL SYLLABLE SSANGPIEUP YE TIKEUT
    ((0x96, 0xAF), (0xBF2C)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEUL
    ((0x96, 0xB0), (0xBF2D)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULKIYEOK
    ((0x96, 0xB1), (0xBF2E)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULMIEUM
    ((0x96, 0xB2), (0xBF2F)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULPIEUP
    ((0x96, 0xB3), (0xBF30)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULSIOS
    ((0x96, 0xB4), (0xBF31)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULTHIEUTH
    ((0x96, 0xB5), (0xBF32)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULPHIEUPH
    ((0x96, 0xB6), (0xBF33)),  # HANGUL SYLLABLE SSANGPIEUP YE RIEULHIEUH
    ((0x96, 0xB7), (0xBF34)),  # HANGUL SYLLABLE SSANGPIEUP YE MIEUM
    ((0x96, 0xB8), (0xBF35)),  # HANGUL SYLLABLE SSANGPIEUP YE PIEUP
    ((0x96, 0xB9), (0xBF36)),  # HANGUL SYLLABLE SSANGPIEUP YE PIEUPSIOS
    ((0x96, 0xBA), (0xBF37)),  # HANGUL SYLLABLE SSANGPIEUP YE SIOS
    ((0x96, 0xBB), (0xBF38)),  # HANGUL SYLLABLE SSANGPIEUP YE SSANGSIOS
    ((0x96, 0xBC), (0xBF39)),  # HANGUL SYLLABLE SSANGPIEUP YE IEUNG
    ((0x96, 0xBD), (0xBF3A)),  # HANGUL SYLLABLE SSANGPIEUP YE CIEUC
    ((0x96, 0xBE), (0xBF3B)),  # HANGUL SYLLABLE SSANGPIEUP YE CHIEUCH
    ((0x96, 0xBF), (0xBF3C)),  # HANGUL SYLLABLE SSANGPIEUP YE KHIEUKH
    ((0x96, 0xC0), (0xBF3D)),  # HANGUL SYLLABLE SSANGPIEUP YE THIEUTH
    ((0x96, 0xC1), (0xBF3E)),  # HANGUL SYLLABLE SSANGPIEUP YE PHIEUPH
    ((0x96, 0xC2), (0xBF3F)),  # HANGUL SYLLABLE SSANGPIEUP YE HIEUH
    ((0x96, 0xC3), (0xBF42)),  # HANGUL SYLLABLE SSANGPIEUP O SSANGKIYEOK
    ((0x96, 0xC4), (0xBF43)),  # HANGUL SYLLABLE SSANGPIEUP O KIYEOKSIOS
    ((0x96, 0xC5), (0xBF45)),  # HANGUL SYLLABLE SSANGPIEUP O NIEUNCIEUC
    ((0x96, 0xC6), (0xBF46)),  # HANGUL SYLLABLE SSANGPIEUP O NIEUNHIEUH
    ((0x96, 0xC7), (0xBF47)),  # HANGUL SYLLABLE SSANGPIEUP O TIKEUT
    ((0x96, 0xC8), (0xBF49)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULKIYEOK
    ((0x96, 0xC9), (0xBF4A)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULMIEUM
    ((0x96, 0xCA), (0xBF4B)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULPIEUP
    ((0x96, 0xCB), (0xBF4C)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULSIOS
    ((0x96, 0xCC), (0xBF4D)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULTHIEUTH
    ((0x96, 0xCD), (0xBF4E)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULPHIEUPH
    ((0x96, 0xCE), (0xBF4F)),  # HANGUL SYLLABLE SSANGPIEUP O RIEULHIEUH
    ((0x96, 0xCF), (0xBF52)),  # HANGUL SYLLABLE SSANGPIEUP O PIEUPSIOS
    ((0x96, 0xD0), (0xBF53)),  # HANGUL SYLLABLE SSANGPIEUP O SIOS
    ((0x96, 0xD1), (0xBF54)),  # HANGUL SYLLABLE SSANGPIEUP O SSANGSIOS
    ((0x96, 0xD2), (0xBF56)),  # HANGUL SYLLABLE SSANGPIEUP O CIEUC
    ((0x96, 0xD3), (0xBF57)),  # HANGUL SYLLABLE SSANGPIEUP O CHIEUCH
    ((0x96, 0xD4), (0xBF58)),  # HANGUL SYLLABLE SSANGPIEUP O KHIEUKH
    ((0x96, 0xD5), (0xBF59)),  # HANGUL SYLLABLE SSANGPIEUP O THIEUTH
    ((0x96, 0xD6), (0xBF5A)),  # HANGUL SYLLABLE SSANGPIEUP O PHIEUPH
    ((0x96, 0xD7), (0xBF5B)),  # HANGUL SYLLABLE SSANGPIEUP O HIEUH
    ((0x96, 0xD8), (0xBF5C)),  # HANGUL SYLLABLE SSANGPIEUP WA
    ((0x96, 0xD9), (0xBF5D)),  # HANGUL SYLLABLE SSANGPIEUP WA KIYEOK
    ((0x96, 0xDA), (0xBF5E)),  # HANGUL SYLLABLE SSANGPIEUP WA SSANGKIYEOK
    ((0x96, 0xDB), (0xBF5F)),  # HANGUL SYLLABLE SSANGPIEUP WA KIYEOKSIOS
    ((0x96, 0xDC), (0xBF60)),  # HANGUL SYLLABLE SSANGPIEUP WA NIEUN
    ((0x96, 0xDD), (0xBF61)),  # HANGUL SYLLABLE SSANGPIEUP WA NIEUNCIEUC
    ((0x96, 0xDE), (0xBF62)),  # HANGUL SYLLABLE SSANGPIEUP WA NIEUNHIEUH
    ((0x96, 0xDF), (0xBF63)),  # HANGUL SYLLABLE SSANGPIEUP WA TIKEUT
    ((0x96, 0xE0), (0xBF64)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEUL
    ((0x96, 0xE1), (0xBF65)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULKIYEOK
    ((0x96, 0xE2), (0xBF66)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULMIEUM
    ((0x96, 0xE3), (0xBF67)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULPIEUP
    ((0x96, 0xE4), (0xBF68)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULSIOS
    ((0x96, 0xE5), (0xBF69)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULTHIEUTH
    ((0x96, 0xE6), (0xBF6A)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULPHIEUPH
    ((0x96, 0xE7), (0xBF6B)),  # HANGUL SYLLABLE SSANGPIEUP WA RIEULHIEUH
    ((0x96, 0xE8), (0xBF6C)),  # HANGUL SYLLABLE SSANGPIEUP WA MIEUM
    ((0x96, 0xE9), (0xBF6D)),  # HANGUL SYLLABLE SSANGPIEUP WA PIEUP
    ((0x96, 0xEA), (0xBF6E)),  # HANGUL SYLLABLE SSANGPIEUP WA PIEUPSIOS
    ((0x96, 0xEB), (0xBF6F)),  # HANGUL SYLLABLE SSANGPIEUP WA SIOS
    ((0x96, 0xEC), (0xBF70)),  # HANGUL SYLLABLE SSANGPIEUP WA SSANGSIOS
    ((0x96, 0xED), (0xBF71)),  # HANGUL SYLLABLE SSANGPIEUP WA IEUNG
    ((0x96, 0xEE), (0xBF72)),  # HANGUL SYLLABLE SSANGPIEUP WA CIEUC
    ((0x96, 0xEF), (0xBF73)),  # HANGUL SYLLABLE SSANGPIEUP WA CHIEUCH
    ((0x96, 0xF0), (0xBF74)),  # HANGUL SYLLABLE SSANGPIEUP WA KHIEUKH
    ((0x96, 0xF1), (0xBF75)),  # HANGUL SYLLABLE SSANGPIEUP WA THIEUTH
    ((0x96, 0xF2), (0xBF76)),  # HANGUL SYLLABLE SSANGPIEUP WA PHIEUPH
    ((0x96, 0xF3), (0xBF77)),  # HANGUL SYLLABLE SSANGPIEUP WA HIEUH
    ((0x96, 0xF4), (0xBF78)),  # HANGUL SYLLABLE SSANGPIEUP WAE
    ((0x96, 0xF5), (0xBF79)),  # HANGUL SYLLABLE SSANGPIEUP WAE KIYEOK
    ((0x96, 0xF6), (0xBF7A)),  # HANGUL SYLLABLE SSANGPIEUP WAE SSANGKIYEOK
    ((0x96, 0xF7), (0xBF7B)),  # HANGUL SYLLABLE SSANGPIEUP WAE KIYEOKSIOS
    ((0x96, 0xF8), (0xBF7C)),  # HANGUL SYLLABLE SSANGPIEUP WAE NIEUN
    ((0x96, 0xF9), (0xBF7D)),  # HANGUL SYLLABLE SSANGPIEUP WAE NIEUNCIEUC
    ((0x96, 0xFA), (0xBF7E)),  # HANGUL SYLLABLE SSANGPIEUP WAE NIEUNHIEUH
    ((0x96, 0xFB), (0xBF7F)),  # HANGUL SYLLABLE SSANGPIEUP WAE TIKEUT
    ((0x96, 0xFC), (0xBF80)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEUL
    ((0x96, 0xFD), (0xBF81)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULKIYEOK
    ((0x96, 0xFE), (0xBF82)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULMIEUM
    ((0x97, 0x41), (0xBF83)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULPIEUP
    ((0x97, 0x42), (0xBF84)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULSIOS
    ((0x97, 0x43), (0xBF85)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULTHIEUTH
    ((0x97, 0x44), (0xBF86)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULPHIEUPH
    ((0x97, 0x45), (0xBF87)),  # HANGUL SYLLABLE SSANGPIEUP WAE RIEULHIEUH
    ((0x97, 0x46), (0xBF88)),  # HANGUL SYLLABLE SSANGPIEUP WAE MIEUM
    ((0x97, 0x47), (0xBF89)),  # HANGUL SYLLABLE SSANGPIEUP WAE PIEUP
    ((0x97, 0x48), (0xBF8A)),  # HANGUL SYLLABLE SSANGPIEUP WAE PIEUPSIOS
    ((0x97, 0x49), (0xBF8B)),  # HANGUL SYLLABLE SSANGPIEUP WAE SIOS
    ((0x97, 0x4A), (0xBF8C)),  # HANGUL SYLLABLE SSANGPIEUP WAE SSANGSIOS
    ((0x97, 0x4B), (0xBF8D)),  # HANGUL SYLLABLE SSANGPIEUP WAE IEUNG
    ((0x97, 0x4C), (0xBF8E)),  # HANGUL SYLLABLE SSANGPIEUP WAE CIEUC
    ((0x97, 0x4D), (0xBF8F)),  # HANGUL SYLLABLE SSANGPIEUP WAE CHIEUCH
    ((0x97, 0x4E), (0xBF90)),  # HANGUL SYLLABLE SSANGPIEUP WAE KHIEUKH
    ((0x97, 0x4F), (0xBF91)),  # HANGUL SYLLABLE SSANGPIEUP WAE THIEUTH
    ((0x97, 0x50), (0xBF92)),  # HANGUL SYLLABLE SSANGPIEUP WAE PHIEUPH
    ((0x97, 0x51), (0xBF93)),  # HANGUL SYLLABLE SSANGPIEUP WAE HIEUH
    ((0x97, 0x52), (0xBF95)),  # HANGUL SYLLABLE SSANGPIEUP OE KIYEOK
    ((0x97, 0x53), (0xBF96)),  # HANGUL SYLLABLE SSANGPIEUP OE SSANGKIYEOK
    ((0x97, 0x54), (0xBF97)),  # HANGUL SYLLABLE SSANGPIEUP OE KIYEOKSIOS
    ((0x97, 0x55), (0xBF98)),  # HANGUL SYLLABLE SSANGPIEUP OE NIEUN
    ((0x97, 0x56), (0xBF99)),  # HANGUL SYLLABLE SSANGPIEUP OE NIEUNCIEUC
    ((0x97, 0x57), (0xBF9A)),  # HANGUL SYLLABLE SSANGPIEUP OE NIEUNHIEUH
    ((0x97, 0x58), (0xBF9B)),  # HANGUL SYLLABLE SSANGPIEUP OE TIKEUT
    ((0x97, 0x59), (0xBF9C)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEUL
    ((0x97, 0x5A), (0xBF9D)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULKIYEOK
    ((0x97, 0x61), (0xBF9E)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULMIEUM
    ((0x97, 0x62), (0xBF9F)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULPIEUP
    ((0x97, 0x63), (0xBFA0)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULSIOS
    ((0x97, 0x64), (0xBFA1)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULTHIEUTH
    ((0x97, 0x65), (0xBFA2)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULPHIEUPH
    ((0x97, 0x66), (0xBFA3)),  # HANGUL SYLLABLE SSANGPIEUP OE RIEULHIEUH
    ((0x97, 0x67), (0xBFA4)),  # HANGUL SYLLABLE SSANGPIEUP OE MIEUM
    ((0x97, 0x68), (0xBFA5)),  # HANGUL SYLLABLE SSANGPIEUP OE PIEUP
    ((0x97, 0x69), (0xBFA6)),  # HANGUL SYLLABLE SSANGPIEUP OE PIEUPSIOS
    ((0x97, 0x6A), (0xBFA7)),  # HANGUL SYLLABLE SSANGPIEUP OE SIOS
    ((0x97, 0x6B), (0xBFA8)),  # HANGUL SYLLABLE SSANGPIEUP OE SSANGSIOS
    ((0x97, 0x6C), (0xBFA9)),  # HANGUL SYLLABLE SSANGPIEUP OE IEUNG
    ((0x97, 0x6D), (0xBFAA)),  # HANGUL SYLLABLE SSANGPIEUP OE CIEUC
    ((0x97, 0x6E), (0xBFAB)),  # HANGUL SYLLABLE SSANGPIEUP OE CHIEUCH
    ((0x97, 0x6F), (0xBFAC)),  # HANGUL SYLLABLE SSANGPIEUP OE KHIEUKH
    ((0x97, 0x70), (0xBFAD)),  # HANGUL SYLLABLE SSANGPIEUP OE THIEUTH
    ((0x97, 0x71), (0xBFAE)),  # HANGUL SYLLABLE SSANGPIEUP OE PHIEUPH
    ((0x97, 0x72), (0xBFAF)),  # HANGUL SYLLABLE SSANGPIEUP OE HIEUH
    ((0x97, 0x73), (0xBFB1)),  # HANGUL SYLLABLE SSANGPIEUP YO KIYEOK
    ((0x97, 0x74), (0xBFB2)),  # HANGUL SYLLABLE SSANGPIEUP YO SSANGKIYEOK
    ((0x97, 0x75), (0xBFB3)),  # HANGUL SYLLABLE SSANGPIEUP YO KIYEOKSIOS
    ((0x97, 0x76), (0xBFB4)),  # HANGUL SYLLABLE SSANGPIEUP YO NIEUN
    ((0x97, 0x77), (0xBFB5)),  # HANGUL SYLLABLE SSANGPIEUP YO NIEUNCIEUC
    ((0x97, 0x78), (0xBFB6)),  # HANGUL SYLLABLE SSANGPIEUP YO NIEUNHIEUH
    ((0x97, 0x79), (0xBFB7)),  # HANGUL SYLLABLE SSANGPIEUP YO TIKEUT
    ((0x97, 0x7A), (0xBFB8)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEUL
    ((0x97, 0x81), (0xBFB9)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULKIYEOK
    ((0x97, 0x82), (0xBFBA)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULMIEUM
    ((0x97, 0x83), (0xBFBB)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULPIEUP
    ((0x97, 0x84), (0xBFBC)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULSIOS
    ((0x97, 0x85), (0xBFBD)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULTHIEUTH
    ((0x97, 0x86), (0xBFBE)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULPHIEUPH
    ((0x97, 0x87), (0xBFBF)),  # HANGUL SYLLABLE SSANGPIEUP YO RIEULHIEUH
    ((0x97, 0x88), (0xBFC0)),  # HANGUL SYLLABLE SSANGPIEUP YO MIEUM
    ((0x97, 0x89), (0xBFC1)),  # HANGUL SYLLABLE SSANGPIEUP YO PIEUP
    ((0x97, 0x8A), (0xBFC2)),  # HANGUL SYLLABLE SSANGPIEUP YO PIEUPSIOS
    ((0x97, 0x8B), (0xBFC3)),  # HANGUL SYLLABLE SSANGPIEUP YO SIOS
    ((0x97, 0x8C), (0xBFC4)),  # HANGUL SYLLABLE SSANGPIEUP YO SSANGSIOS
    ((0x97, 0x8D), (0xBFC6)),  # HANGUL SYLLABLE SSANGPIEUP YO CIEUC
    ((0x97, 0x8E), (0xBFC7)),  # HANGUL SYLLABLE SSANGPIEUP YO CHIEUCH
    ((0x97, 0x8F), (0xBFC8)),  # HANGUL SYLLABLE SSANGPIEUP YO KHIEUKH
    ((0x97, 0x90), (0xBFC9)),  # HANGUL SYLLABLE SSANGPIEUP YO THIEUTH
    ((0x97, 0x91), (0xBFCA)),  # HANGUL SYLLABLE SSANGPIEUP YO PHIEUPH
    ((0x97, 0x92), (0xBFCB)),  # HANGUL SYLLABLE SSANGPIEUP YO HIEUH
    ((0x97, 0x93), (0xBFCE)),  # HANGUL SYLLABLE SSANGPIEUP U SSANGKIYEOK
    ((0x97, 0x94), (0xBFCF)),  # HANGUL SYLLABLE SSANGPIEUP U KIYEOKSIOS
    ((0x97, 0x95), (0xBFD1)),  # HANGUL SYLLABLE SSANGPIEUP U NIEUNCIEUC
    ((0x97, 0x96), (0xBFD2)),  # HANGUL SYLLABLE SSANGPIEUP U NIEUNHIEUH
    ((0x97, 0x97), (0xBFD3)),  # HANGUL SYLLABLE SSANGPIEUP U TIKEUT
    ((0x97, 0x98), (0xBFD5)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULKIYEOK
    ((0x97, 0x99), (0xBFD6)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULMIEUM
    ((0x97, 0x9A), (0xBFD7)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULPIEUP
    ((0x97, 0x9B), (0xBFD8)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULSIOS
    ((0x97, 0x9C), (0xBFD9)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULTHIEUTH
    ((0x97, 0x9D), (0xBFDA)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULPHIEUPH
    ((0x97, 0x9E), (0xBFDB)),  # HANGUL SYLLABLE SSANGPIEUP U RIEULHIEUH
    ((0x97, 0x9F), (0xBFDD)),  # HANGUL SYLLABLE SSANGPIEUP U PIEUP
    ((0x97, 0xA0), (0xBFDE)),  # HANGUL SYLLABLE SSANGPIEUP U PIEUPSIOS
    ((0x97, 0xA1), (0xBFE0)),  # HANGUL SYLLABLE SSANGPIEUP U SSANGSIOS
    ((0x97, 0xA2), (0xBFE2)),  # HANGUL SYLLABLE SSANGPIEUP U CIEUC
    ((0x97, 0xA3), (0xBFE3)),  # HANGUL SYLLABLE SSANGPIEUP U CHIEUCH
    ((0x97, 0xA4), (0xBFE4)),  # HANGUL SYLLABLE SSANGPIEUP U KHIEUKH
    ((0x97, 0xA5), (0xBFE5)),  # HANGUL SYLLABLE SSANGPIEUP U THIEUTH
    ((0x97, 0xA6), (0xBFE6)),  # HANGUL SYLLABLE SSANGPIEUP U PHIEUPH
    ((0x97, 0xA7), (0xBFE7)),  # HANGUL SYLLABLE SSANGPIEUP U HIEUH
    ((0x97, 0xA8), (0xBFE8)),  # HANGUL SYLLABLE SSANGPIEUP WEO
    ((0x97, 0xA9), (0xBFE9)),  # HANGUL SYLLABLE SSANGPIEUP WEO KIYEOK
    ((0x97, 0xAA), (0xBFEA)),  # HANGUL SYLLABLE SSANGPIEUP WEO SSANGKIYEOK
    ((0x97, 0xAB), (0xBFEB)),  # HANGUL SYLLABLE SSANGPIEUP WEO KIYEOKSIOS
    ((0x97, 0xAC), (0xBFEC)),  # HANGUL SYLLABLE SSANGPIEUP WEO NIEUN
    ((0x97, 0xAD), (0xBFED)),  # HANGUL SYLLABLE SSANGPIEUP WEO NIEUNCIEUC
    ((0x97, 0xAE), (0xBFEE)),  # HANGUL SYLLABLE SSANGPIEUP WEO NIEUNHIEUH
    ((0x97, 0xAF), (0xBFEF)),  # HANGUL SYLLABLE SSANGPIEUP WEO TIKEUT
    ((0x97, 0xB0), (0xBFF0)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEUL
    ((0x97, 0xB1), (0xBFF1)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULKIYEOK
    ((0x97, 0xB2), (0xBFF2)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULMIEUM
    ((0x97, 0xB3), (0xBFF3)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULPIEUP
    ((0x97, 0xB4), (0xBFF4)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULSIOS
    ((0x97, 0xB5), (0xBFF5)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULTHIEUTH
    ((0x97, 0xB6), (0xBFF6)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULPHIEUPH
    ((0x97, 0xB7), (0xBFF7)),  # HANGUL SYLLABLE SSANGPIEUP WEO RIEULHIEUH
    ((0x97, 0xB8), (0xBFF8)),  # HANGUL SYLLABLE SSANGPIEUP WEO MIEUM
    ((0x97, 0xB9), (0xBFF9)),  # HANGUL SYLLABLE SSANGPIEUP WEO PIEUP
    ((0x97, 0xBA), (0xBFFA)),  # HANGUL SYLLABLE SSANGPIEUP WEO PIEUPSIOS
    ((0x97, 0xBB), (0xBFFB)),  # HANGUL SYLLABLE SSANGPIEUP WEO SIOS
    ((0x97, 0xBC), (0xBFFC)),  # HANGUL SYLLABLE SSANGPIEUP WEO SSANGSIOS
    ((0x97, 0xBD), (0xBFFD)),  # HANGUL SYLLABLE SSANGPIEUP WEO IEUNG
    ((0x97, 0xBE), (0xBFFE)),  # HANGUL SYLLABLE SSANGPIEUP WEO CIEUC
    ((0x97, 0xBF), (0xBFFF)),  # HANGUL SYLLABLE SSANGPIEUP WEO CHIEUCH
    ((0x97, 0xC0), (0xC000)),  # HANGUL SYLLABLE SSANGPIEUP WEO KHIEUKH
    ((0x97, 0xC1), (0xC001)),  # HANGUL SYLLABLE SSANGPIEUP WEO THIEUTH
    ((0x97, 0xC2), (0xC002)),  # HANGUL SYLLABLE SSANGPIEUP WEO PHIEUPH
    ((0x97, 0xC3), (0xC003)),  # HANGUL SYLLABLE SSANGPIEUP WEO HIEUH
    ((0x97, 0xC4), (0xC004)),  # HANGUL SYLLABLE SSANGPIEUP WE
    ((0x97, 0xC5), (0xC005)),  # HANGUL SYLLABLE SSANGPIEUP WE KIYEOK
    ((0x97, 0xC6), (0xC006)),  # HANGUL SYLLABLE SSANGPIEUP WE SSANGKIYEOK
    ((0x97, 0xC7), (0xC007)),  # HANGUL SYLLABLE SSANGPIEUP WE KIYEOKSIOS
    ((0x97, 0xC8), (0xC008)),  # HANGUL SYLLABLE SSANGPIEUP WE NIEUN
    ((0x97, 0xC9), (0xC009)),  # HANGUL SYLLABLE SSANGPIEUP WE NIEUNCIEUC
    ((0x97, 0xCA), (0xC00A)),  # HANGUL SYLLABLE SSANGPIEUP WE NIEUNHIEUH
    ((0x97, 0xCB), (0xC00B)),  # HANGUL SYLLABLE SSANGPIEUP WE TIKEUT
    ((0x97, 0xCC), (0xC00C)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEUL
    ((0x97, 0xCD), (0xC00D)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULKIYEOK
    ((0x97, 0xCE), (0xC00E)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULMIEUM
    ((0x97, 0xCF), (0xC00F)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULPIEUP
    ((0x97, 0xD0), (0xC010)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULSIOS
    ((0x97, 0xD1), (0xC011)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULTHIEUTH
    ((0x97, 0xD2), (0xC012)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULPHIEUPH
    ((0x97, 0xD3), (0xC013)),  # HANGUL SYLLABLE SSANGPIEUP WE RIEULHIEUH
    ((0x97, 0xD4), (0xC014)),  # HANGUL SYLLABLE SSANGPIEUP WE MIEUM
    ((0x97, 0xD5), (0xC015)),  # HANGUL SYLLABLE SSANGPIEUP WE PIEUP
    ((0x97, 0xD6), (0xC016)),  # HANGUL SYLLABLE SSANGPIEUP WE PIEUPSIOS
    ((0x97, 0xD7), (0xC017)),  # HANGUL SYLLABLE SSANGPIEUP WE SIOS
    ((0x97, 0xD8), (0xC018)),  # HANGUL SYLLABLE SSANGPIEUP WE SSANGSIOS
    ((0x97, 0xD9), (0xC019)),  # HANGUL SYLLABLE SSANGPIEUP WE IEUNG
    ((0x97, 0xDA), (0xC01A)),  # HANGUL SYLLABLE SSANGPIEUP WE CIEUC
    ((0x97, 0xDB), (0xC01B)),  # HANGUL SYLLABLE SSANGPIEUP WE CHIEUCH
    ((0x97, 0xDC), (0xC01C)),  # HANGUL SYLLABLE SSANGPIEUP WE KHIEUKH
    ((0x97, 0xDD), (0xC01D)),  # HANGUL SYLLABLE SSANGPIEUP WE THIEUTH
    ((0x97, 0xDE), (0xC01E)),  # HANGUL SYLLABLE SSANGPIEUP WE PHIEUPH
    ((0x97, 0xDF), (0xC01F)),  # HANGUL SYLLABLE SSANGPIEUP WE HIEUH
    ((0x97, 0xE0), (0xC020)),  # HANGUL SYLLABLE SSANGPIEUP WI
    ((0x97, 0xE1), (0xC021)),  # HANGUL SYLLABLE SSANGPIEUP WI KIYEOK
    ((0x97, 0xE2), (0xC022)),  # HANGUL SYLLABLE SSANGPIEUP WI SSANGKIYEOK
    ((0x97, 0xE3), (0xC023)),  # HANGUL SYLLABLE SSANGPIEUP WI KIYEOKSIOS
    ((0x97, 0xE4), (0xC024)),  # HANGUL SYLLABLE SSANGPIEUP WI NIEUN
    ((0x97, 0xE5), (0xC025)),  # HANGUL SYLLABLE SSANGPIEUP WI NIEUNCIEUC
    ((0x97, 0xE6), (0xC026)),  # HANGUL SYLLABLE SSANGPIEUP WI NIEUNHIEUH
    ((0x97, 0xE7), (0xC027)),  # HANGUL SYLLABLE SSANGPIEUP WI TIKEUT
    ((0x97, 0xE8), (0xC028)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEUL
    ((0x97, 0xE9), (0xC029)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULKIYEOK
    ((0x97, 0xEA), (0xC02A)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULMIEUM
    ((0x97, 0xEB), (0xC02B)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULPIEUP
    ((0x97, 0xEC), (0xC02C)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULSIOS
    ((0x97, 0xED), (0xC02D)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULTHIEUTH
    ((0x97, 0xEE), (0xC02E)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULPHIEUPH
    ((0x97, 0xEF), (0xC02F)),  # HANGUL SYLLABLE SSANGPIEUP WI RIEULHIEUH
    ((0x97, 0xF0), (0xC030)),  # HANGUL SYLLABLE SSANGPIEUP WI MIEUM
    ((0x97, 0xF1), (0xC031)),  # HANGUL SYLLABLE SSANGPIEUP WI PIEUP
    ((0x97, 0xF2), (0xC032)),  # HANGUL SYLLABLE SSANGPIEUP WI PIEUPSIOS
    ((0x97, 0xF3), (0xC033)),  # HANGUL SYLLABLE SSANGPIEUP WI SIOS
    ((0x97, 0xF4), (0xC034)),  # HANGUL SYLLABLE SSANGPIEUP WI SSANGSIOS
    ((0x97, 0xF5), (0xC035)),  # HANGUL SYLLABLE SSANGPIEUP WI IEUNG
    ((0x97, 0xF6), (0xC036)),  # HANGUL SYLLABLE SSANGPIEUP WI CIEUC
    ((0x97, 0xF7), (0xC037)),  # HANGUL SYLLABLE SSANGPIEUP WI CHIEUCH
    ((0x97, 0xF8), (0xC038)),  # HANGUL SYLLABLE SSANGPIEUP WI KHIEUKH
    ((0x97, 0xF9), (0xC039)),  # HANGUL SYLLABLE SSANGPIEUP WI THIEUTH
    ((0x97, 0xFA), (0xC03A)),  # HANGUL SYLLABLE SSANGPIEUP WI PHIEUPH
    ((0x97, 0xFB), (0xC03B)),  # HANGUL SYLLABLE SSANGPIEUP WI HIEUH
    ((0x97, 0xFC), (0xC03D)),  # HANGUL SYLLABLE SSANGPIEUP YU KIYEOK
    ((0x97, 0xFD), (0xC03E)),  # HANGUL SYLLABLE SSANGPIEUP YU SSANGKIYEOK
    ((0x97, 0xFE), (0xC03F)),  # HANGUL SYLLABLE SSANGPIEUP YU KIYEOKSIOS
    ((0x98, 0x41), (0xC040)),  # HANGUL SYLLABLE SSANGPIEUP YU NIEUN
    ((0x98, 0x42), (0xC041)),  # HANGUL SYLLABLE SSANGPIEUP YU NIEUNCIEUC
    ((0x98, 0x43), (0xC042)),  # HANGUL SYLLABLE SSANGPIEUP YU NIEUNHIEUH
    ((0x98, 0x44), (0xC043)),  # HANGUL SYLLABLE SSANGPIEUP YU TIKEUT
    ((0x98, 0x45), (0xC044)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEUL
    ((0x98, 0x46), (0xC045)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULKIYEOK
    ((0x98, 0x47), (0xC046)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULMIEUM
    ((0x98, 0x48), (0xC047)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULPIEUP
    ((0x98, 0x49), (0xC048)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULSIOS
    ((0x98, 0x4A), (0xC049)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULTHIEUTH
    ((0x98, 0x4B), (0xC04A)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULPHIEUPH
    ((0x98, 0x4C), (0xC04B)),  # HANGUL SYLLABLE SSANGPIEUP YU RIEULHIEUH
    ((0x98, 0x4D), (0xC04C)),  # HANGUL SYLLABLE SSANGPIEUP YU MIEUM
    ((0x98, 0x4E), (0xC04D)),  # HANGUL SYLLABLE SSANGPIEUP YU PIEUP
    ((0x98, 0x4F), (0xC04E)),  # HANGUL SYLLABLE SSANGPIEUP YU PIEUPSIOS
    ((0x98, 0x50), (0xC04F)),  # HANGUL SYLLABLE SSANGPIEUP YU SIOS
    ((0x98, 0x51), (0xC050)),  # HANGUL SYLLABLE SSANGPIEUP YU SSANGSIOS
    ((0x98, 0x52), (0xC052)),  # HANGUL SYLLABLE SSANGPIEUP YU CIEUC
    ((0x98, 0x53), (0xC053)),  # HANGUL SYLLABLE SSANGPIEUP YU CHIEUCH
    ((0x98, 0x54), (0xC054)),  # HANGUL SYLLABLE SSANGPIEUP YU KHIEUKH
    ((0x98, 0x55), (0xC055)),  # HANGUL SYLLABLE SSANGPIEUP YU THIEUTH
    ((0x98, 0x56), (0xC056)),  # HANGUL SYLLABLE SSANGPIEUP YU PHIEUPH
    ((0x98, 0x57), (0xC057)),  # HANGUL SYLLABLE SSANGPIEUP YU HIEUH
    ((0x98, 0x58), (0xC059)),  # HANGUL SYLLABLE SSANGPIEUP EU KIYEOK
    ((0x98, 0x59), (0xC05A)),  # HANGUL SYLLABLE SSANGPIEUP EU SSANGKIYEOK
    ((0x98, 0x5A), (0xC05B)),  # HANGUL SYLLABLE SSANGPIEUP EU KIYEOKSIOS
    ((0x98, 0x61), (0xC05D)),  # HANGUL SYLLABLE SSANGPIEUP EU NIEUNCIEUC
    ((0x98, 0x62), (0xC05E)),  # HANGUL SYLLABLE SSANGPIEUP EU NIEUNHIEUH
    ((0x98, 0x63), (0xC05F)),  # HANGUL SYLLABLE SSANGPIEUP EU TIKEUT
    ((0x98, 0x64), (0xC061)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULKIYEOK
    ((0x98, 0x65), (0xC062)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULMIEUM
    ((0x98, 0x66), (0xC063)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULPIEUP
    ((0x98, 0x67), (0xC064)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULSIOS
    ((0x98, 0x68), (0xC065)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULTHIEUTH
    ((0x98, 0x69), (0xC066)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULPHIEUPH
    ((0x98, 0x6A), (0xC067)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEULHIEUH
    ((0x98, 0x6B), (0xC06A)),  # HANGUL SYLLABLE SSANGPIEUP EU PIEUPSIOS
    ((0x98, 0x6C), (0xC06B)),  # HANGUL SYLLABLE SSANGPIEUP EU SIOS
    ((0x98, 0x6D), (0xC06C)),  # HANGUL SYLLABLE SSANGPIEUP EU SSANGSIOS
    ((0x98, 0x6E), (0xC06D)),  # HANGUL SYLLABLE SSANGPIEUP EU IEUNG
    ((0x98, 0x6F), (0xC06E)),  # HANGUL SYLLABLE SSANGPIEUP EU CIEUC
    ((0x98, 0x70), (0xC06F)),  # HANGUL SYLLABLE SSANGPIEUP EU CHIEUCH
    ((0x98, 0x71), (0xC070)),  # HANGUL SYLLABLE SSANGPIEUP EU KHIEUKH
    ((0x98, 0x72), (0xC071)),  # HANGUL SYLLABLE SSANGPIEUP EU THIEUTH
    ((0x98, 0x73), (0xC072)),  # HANGUL SYLLABLE SSANGPIEUP EU PHIEUPH
    ((0x98, 0x74), (0xC073)),  # HANGUL SYLLABLE SSANGPIEUP EU HIEUH
    ((0x98, 0x75), (0xC074)),  # HANGUL SYLLABLE SSANGPIEUP YI
    ((0x98, 0x76), (0xC075)),  # HANGUL SYLLABLE SSANGPIEUP YI KIYEOK
    ((0x98, 0x77), (0xC076)),  # HANGUL SYLLABLE SSANGPIEUP YI SSANGKIYEOK
    ((0x98, 0x78), (0xC077)),  # HANGUL SYLLABLE SSANGPIEUP YI KIYEOKSIOS
    ((0x98, 0x79), (0xC078)),  # HANGUL SYLLABLE SSANGPIEUP YI NIEUN
    ((0x98, 0x7A), (0xC079)),  # HANGUL SYLLABLE SSANGPIEUP YI NIEUNCIEUC
    ((0x98, 0x81), (0xC07A)),  # HANGUL SYLLABLE SSANGPIEUP YI NIEUNHIEUH
    ((0x98, 0x82), (0xC07B)),  # HANGUL SYLLABLE SSANGPIEUP YI TIKEUT
    ((0x98, 0x83), (0xC07C)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEUL
    ((0x98, 0x84), (0xC07D)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULKIYEOK
    ((0x98, 0x85), (0xC07E)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULMIEUM
    ((0x98, 0x86), (0xC07F)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULPIEUP
    ((0x98, 0x87), (0xC080)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULSIOS
    ((0x98, 0x88), (0xC081)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULTHIEUTH
    ((0x98, 0x89), (0xC082)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULPHIEUPH
    ((0x98, 0x8A), (0xC083)),  # HANGUL SYLLABLE SSANGPIEUP YI RIEULHIEUH
    ((0x98, 0x8B), (0xC084)),  # HANGUL SYLLABLE SSANGPIEUP YI MIEUM
    ((0x98, 0x8C), (0xC085)),  # HANGUL SYLLABLE SSANGPIEUP YI PIEUP
    ((0x98, 0x8D), (0xC086)),  # HANGUL SYLLABLE SSANGPIEUP YI PIEUPSIOS
    ((0x98, 0x8E), (0xC087)),  # HANGUL SYLLABLE SSANGPIEUP YI SIOS
    ((0x98, 0x8F), (0xC088)),  # HANGUL SYLLABLE SSANGPIEUP YI SSANGSIOS
    ((0x98, 0x90), (0xC089)),  # HANGUL SYLLABLE SSANGPIEUP YI IEUNG
    ((0x98, 0x91), (0xC08A)),  # HANGUL SYLLABLE SSANGPIEUP YI CIEUC
    ((0x98, 0x92), (0xC08B)),  # HANGUL SYLLABLE SSANGPIEUP YI CHIEUCH
    ((0x98, 0x93), (0xC08C)),  # HANGUL SYLLABLE SSANGPIEUP YI KHIEUKH
    ((0x98, 0x94), (0xC08D)),  # HANGUL SYLLABLE SSANGPIEUP YI THIEUTH
    ((0x98, 0x95), (0xC08E)),  # HANGUL SYLLABLE SSANGPIEUP YI PHIEUPH
    ((0x98, 0x96), (0xC08F)),  # HANGUL SYLLABLE SSANGPIEUP YI HIEUH
    ((0x98, 0x97), (0xC092)),  # HANGUL SYLLABLE SSANGPIEUP I SSANGKIYEOK
    ((0x98, 0x98), (0xC093)),  # HANGUL SYLLABLE SSANGPIEUP I KIYEOKSIOS
    ((0x98, 0x99), (0xC095)),  # HANGUL SYLLABLE SSANGPIEUP I NIEUNCIEUC
    ((0x98, 0x9A), (0xC096)),  # HANGUL SYLLABLE SSANGPIEUP I NIEUNHIEUH
    ((0x98, 0x9B), (0xC097)),  # HANGUL SYLLABLE SSANGPIEUP I TIKEUT
    ((0x98, 0x9C), (0xC099)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULKIYEOK
    ((0x98, 0x9D), (0xC09A)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULMIEUM
    ((0x98, 0x9E), (0xC09B)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULPIEUP
    ((0x98, 0x9F), (0xC09C)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULSIOS
    ((0x98, 0xA0), (0xC09D)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULTHIEUTH
    ((0x98, 0xA1), (0xC09E)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULPHIEUPH
    ((0x98, 0xA2), (0xC09F)),  # HANGUL SYLLABLE SSANGPIEUP I RIEULHIEUH
    ((0x98, 0xA3), (0xC0A2)),  # HANGUL SYLLABLE SSANGPIEUP I PIEUPSIOS
    ((0x98, 0xA4), (0xC0A4)),  # HANGUL SYLLABLE SSANGPIEUP I SSANGSIOS
    ((0x98, 0xA5), (0xC0A6)),  # HANGUL SYLLABLE SSANGPIEUP I CIEUC
    ((0x98, 0xA6), (0xC0A7)),  # HANGUL SYLLABLE SSANGPIEUP I CHIEUCH
    ((0x98, 0xA7), (0xC0A8)),  # HANGUL SYLLABLE SSANGPIEUP I KHIEUKH
    ((0x98, 0xA8), (0xC0A9)),  # HANGUL SYLLABLE SSANGPIEUP I THIEUTH
    ((0x98, 0xA9), (0xC0AA)),  # HANGUL SYLLABLE SSANGPIEUP I PHIEUPH
    ((0x98, 0xAA), (0xC0AB)),  # HANGUL SYLLABLE SSANGPIEUP I HIEUH
    ((0x98, 0xAB), (0xC0AE)),  # HANGUL SYLLABLE SIOS A SSANGKIYEOK
    ((0x98, 0xAC), (0xC0B1)),  # HANGUL SYLLABLE SIOS A NIEUNCIEUC
    ((0x98, 0xAD), (0xC0B2)),  # HANGUL SYLLABLE SIOS A NIEUNHIEUH
    ((0x98, 0xAE), (0xC0B7)),  # HANGUL SYLLABLE SIOS A RIEULPIEUP
    ((0x98, 0xAF), (0xC0B8)),  # HANGUL SYLLABLE SIOS A RIEULSIOS
    ((0x98, 0xB0), (0xC0B9)),  # HANGUL SYLLABLE SIOS A RIEULTHIEUTH
    ((0x98, 0xB1), (0xC0BA)),  # HANGUL SYLLABLE SIOS A RIEULPHIEUPH
    ((0x98, 0xB2), (0xC0BB)),  # HANGUL SYLLABLE SIOS A RIEULHIEUH
    ((0x98, 0xB3), (0xC0BE)),  # HANGUL SYLLABLE SIOS A PIEUPSIOS
    ((0x98, 0xB4), (0xC0C2)),  # HANGUL SYLLABLE SIOS A CIEUC
    ((0x98, 0xB5), (0xC0C3)),  # HANGUL SYLLABLE SIOS A CHIEUCH
    ((0x98, 0xB6), (0xC0C4)),  # HANGUL SYLLABLE SIOS A KHIEUKH
    ((0x98, 0xB7), (0xC0C6)),  # HANGUL SYLLABLE SIOS A PHIEUPH
    ((0x98, 0xB8), (0xC0C7)),  # HANGUL SYLLABLE SIOS A HIEUH
    ((0x98, 0xB9), (0xC0CA)),  # HANGUL SYLLABLE SIOS AE SSANGKIYEOK
    ((0x98, 0xBA), (0xC0CB)),  # HANGUL SYLLABLE SIOS AE KIYEOKSIOS
    ((0x98, 0xBB), (0xC0CD)),  # HANGUL SYLLABLE SIOS AE NIEUNCIEUC
    ((0x98, 0xBC), (0xC0CE)),  # HANGUL SYLLABLE SIOS AE NIEUNHIEUH
    ((0x98, 0xBD), (0xC0CF)),  # HANGUL SYLLABLE SIOS AE TIKEUT
    ((0x98, 0xBE), (0xC0D1)),  # HANGUL SYLLABLE SIOS AE RIEULKIYEOK
    ((0x98, 0xBF), (0xC0D2)),  # HANGUL SYLLABLE SIOS AE RIEULMIEUM
    ((0x98, 0xC0), (0xC0D3)),  # HANGUL SYLLABLE SIOS AE RIEULPIEUP
    ((0x98, 0xC1), (0xC0D4)),  # HANGUL SYLLABLE SIOS AE RIEULSIOS
    ((0x98, 0xC2), (0xC0D5)),  # HANGUL SYLLABLE SIOS AE RIEULTHIEUTH
    ((0x98, 0xC3), (0xC0D6)),  # HANGUL SYLLABLE SIOS AE RIEULPHIEUPH
    ((0x98, 0xC4), (0xC0D7)),  # HANGUL SYLLABLE SIOS AE RIEULHIEUH
    ((0x98, 0xC5), (0xC0DA)),  # HANGUL SYLLABLE SIOS AE PIEUPSIOS
    ((0x98, 0xC6), (0xC0DE)),  # HANGUL SYLLABLE SIOS AE CIEUC
    ((0x98, 0xC7), (0xC0DF)),  # HANGUL SYLLABLE SIOS AE CHIEUCH
    ((0x98, 0xC8), (0xC0E0)),  # HANGUL SYLLABLE SIOS AE KHIEUKH
    ((0x98, 0xC9), (0xC0E1)),  # HANGUL SYLLABLE SIOS AE THIEUTH
    ((0x98, 0xCA), (0xC0E2)),  # HANGUL SYLLABLE SIOS AE PHIEUPH
    ((0x98, 0xCB), (0xC0E3)),  # HANGUL SYLLABLE SIOS AE HIEUH
    ((0x98, 0xCC), (0xC0E6)),  # HANGUL SYLLABLE SIOS YA SSANGKIYEOK
    ((0x98, 0xCD), (0xC0E7)),  # HANGUL SYLLABLE SIOS YA KIYEOKSIOS
    ((0x98, 0xCE), (0xC0E9)),  # HANGUL SYLLABLE SIOS YA NIEUNCIEUC
    ((0x98, 0xCF), (0xC0EA)),  # HANGUL SYLLABLE SIOS YA NIEUNHIEUH
    ((0x98, 0xD0), (0xC0EB)),  # HANGUL SYLLABLE SIOS YA TIKEUT
    ((0x98, 0xD1), (0xC0ED)),  # HANGUL SYLLABLE SIOS YA RIEULKIYEOK
    ((0x98, 0xD2), (0xC0EE)),  # HANGUL SYLLABLE SIOS YA RIEULMIEUM
    ((0x98, 0xD3), (0xC0EF)),  # HANGUL SYLLABLE SIOS YA RIEULPIEUP
    ((0x98, 0xD4), (0xC0F0)),  # HANGUL SYLLABLE SIOS YA RIEULSIOS
    ((0x98, 0xD5), (0xC0F1)),  # HANGUL SYLLABLE SIOS YA RIEULTHIEUTH
    ((0x98, 0xD6), (0xC0F2)),  # HANGUL SYLLABLE SIOS YA RIEULPHIEUPH
    ((0x98, 0xD7), (0xC0F3)),  # HANGUL SYLLABLE SIOS YA RIEULHIEUH
    ((0x98, 0xD8), (0xC0F6)),  # HANGUL SYLLABLE SIOS YA PIEUPSIOS
    ((0x98, 0xD9), (0xC0F8)),  # HANGUL SYLLABLE SIOS YA SSANGSIOS
    ((0x98, 0xDA), (0xC0FA)),  # HANGUL SYLLABLE SIOS YA CIEUC
    ((0x98, 0xDB), (0xC0FB)),  # HANGUL SYLLABLE SIOS YA CHIEUCH
    ((0x98, 0xDC), (0xC0FC)),  # HANGUL SYLLABLE SIOS YA KHIEUKH
    ((0x98, 0xDD), (0xC0FD)),  # HANGUL SYLLABLE SIOS YA THIEUTH
    ((0x98, 0xDE), (0xC0FE)),  # HANGUL SYLLABLE SIOS YA PHIEUPH
    ((0x98, 0xDF), (0xC0FF)),  # HANGUL SYLLABLE SIOS YA HIEUH
    ((0x98, 0xE0), (0xC101)),  # HANGUL SYLLABLE SIOS YAE KIYEOK
    ((0x98, 0xE1), (0xC102)),  # HANGUL SYLLABLE SIOS YAE SSANGKIYEOK
    ((0x98, 0xE2), (0xC103)),  # HANGUL SYLLABLE SIOS YAE KIYEOKSIOS
    ((0x98, 0xE3), (0xC105)),  # HANGUL SYLLABLE SIOS YAE NIEUNCIEUC
    ((0x98, 0xE4), (0xC106)),  # HANGUL SYLLABLE SIOS YAE NIEUNHIEUH
    ((0x98, 0xE5), (0xC107)),  # HANGUL SYLLABLE SIOS YAE TIKEUT
    ((0x98, 0xE6), (0xC109)),  # HANGUL SYLLABLE SIOS YAE RIEULKIYEOK
    ((0x98, 0xE7), (0xC10A)),  # HANGUL SYLLABLE SIOS YAE RIEULMIEUM
    ((0x98, 0xE8), (0xC10B)),  # HANGUL SYLLABLE SIOS YAE RIEULPIEUP
    ((0x98, 0xE9), (0xC10C)),  # HANGUL SYLLABLE SIOS YAE RIEULSIOS
    ((0x98, 0xEA), (0xC10D)),  # HANGUL SYLLABLE SIOS YAE RIEULTHIEUTH
    ((0x98, 0xEB), (0xC10E)),  # HANGUL SYLLABLE SIOS YAE RIEULPHIEUPH
    ((0x98, 0xEC), (0xC10F)),  # HANGUL SYLLABLE SIOS YAE RIEULHIEUH
    ((0x98, 0xED), (0xC111)),  # HANGUL SYLLABLE SIOS YAE PIEUP
    ((0x98, 0xEE), (0xC112)),  # HANGUL SYLLABLE SIOS YAE PIEUPSIOS
    ((0x98, 0xEF), (0xC113)),  # HANGUL SYLLABLE SIOS YAE SIOS
    ((0x98, 0xF0), (0xC114)),  # HANGUL SYLLABLE SIOS YAE SSANGSIOS
    ((0x98, 0xF1), (0xC116)),  # HANGUL SYLLABLE SIOS YAE CIEUC
    ((0x98, 0xF2), (0xC117)),  # HANGUL SYLLABLE SIOS YAE CHIEUCH
    ((0x98, 0xF3), (0xC118)),  # HANGUL SYLLABLE SIOS YAE KHIEUKH
    ((0x98, 0xF4), (0xC119)),  # HANGUL SYLLABLE SIOS YAE THIEUTH
    ((0x98, 0xF5), (0xC11A)),  # HANGUL SYLLABLE SIOS YAE PHIEUPH
    ((0x98, 0xF6), (0xC11B)),  # HANGUL SYLLABLE SIOS YAE HIEUH
    ((0x98, 0xF7), (0xC121)),  # HANGUL SYLLABLE SIOS EO NIEUNCIEUC
    ((0x98, 0xF8), (0xC122)),  # HANGUL SYLLABLE SIOS EO NIEUNHIEUH
    ((0x98, 0xF9), (0xC125)),  # HANGUL SYLLABLE SIOS EO RIEULKIYEOK
    ((0x98, 0xFA), (0xC128)),  # HANGUL SYLLABLE SIOS EO RIEULSIOS
    ((0x98, 0xFB), (0xC129)),  # HANGUL SYLLABLE SIOS EO RIEULTHIEUTH
    ((0x98, 0xFC), (0xC12A)),  # HANGUL SYLLABLE SIOS EO RIEULPHIEUPH
    ((0x98, 0xFD), (0xC12B)),  # HANGUL SYLLABLE SIOS EO RIEULHIEUH
    ((0x98, 0xFE), (0xC12E)),  # HANGUL SYLLABLE SIOS EO PIEUPSIOS
    ((0x99, 0x41), (0xC132)),  # HANGUL SYLLABLE SIOS EO CIEUC
    ((0x99, 0x42), (0xC133)),  # HANGUL SYLLABLE SIOS EO CHIEUCH
    ((0x99, 0x43), (0xC134)),  # HANGUL SYLLABLE SIOS EO KHIEUKH
    ((0x99, 0x44), (0xC135)),  # HANGUL SYLLABLE SIOS EO THIEUTH
    ((0x99, 0x45), (0xC137)),  # HANGUL SYLLABLE SIOS EO HIEUH
    ((0x99, 0x46), (0xC13A)),  # HANGUL SYLLABLE SIOS E SSANGKIYEOK
    ((0x99, 0x47), (0xC13B)),  # HANGUL SYLLABLE SIOS E KIYEOKSIOS
    ((0x99, 0x48), (0xC13D)),  # HANGUL SYLLABLE SIOS E NIEUNCIEUC
    ((0x99, 0x49), (0xC13E)),  # HANGUL SYLLABLE SIOS E NIEUNHIEUH
    ((0x99, 0x4A), (0xC13F)),  # HANGUL SYLLABLE SIOS E TIKEUT
    ((0x99, 0x4B), (0xC141)),  # HANGUL SYLLABLE SIOS E RIEULKIYEOK
    ((0x99, 0x4C), (0xC142)),  # HANGUL SYLLABLE SIOS E RIEULMIEUM
    ((0x99, 0x4D), (0xC143)),  # HANGUL SYLLABLE SIOS E RIEULPIEUP
    ((0x99, 0x4E), (0xC144)),  # HANGUL SYLLABLE SIOS E RIEULSIOS
    ((0x99, 0x4F), (0xC145)),  # HANGUL SYLLABLE SIOS E RIEULTHIEUTH
    ((0x99, 0x50), (0xC146)),  # HANGUL SYLLABLE SIOS E RIEULPHIEUPH
    ((0x99, 0x51), (0xC147)),  # HANGUL SYLLABLE SIOS E RIEULHIEUH
    ((0x99, 0x52), (0xC14A)),  # HANGUL SYLLABLE SIOS E PIEUPSIOS
    ((0x99, 0x53), (0xC14E)),  # HANGUL SYLLABLE SIOS E CIEUC
    ((0x99, 0x54), (0xC14F)),  # HANGUL SYLLABLE SIOS E CHIEUCH
    ((0x99, 0x55), (0xC150)),  # HANGUL SYLLABLE SIOS E KHIEUKH
    ((0x99, 0x56), (0xC151)),  # HANGUL SYLLABLE SIOS E THIEUTH
    ((0x99, 0x57), (0xC152)),  # HANGUL SYLLABLE SIOS E PHIEUPH
    ((0x99, 0x58), (0xC153)),  # HANGUL SYLLABLE SIOS E HIEUH
    ((0x99, 0x59), (0xC156)),  # HANGUL SYLLABLE SIOS YEO SSANGKIYEOK
    ((0x99, 0x5A), (0xC157)),  # HANGUL SYLLABLE SIOS YEO KIYEOKSIOS
    ((0x99, 0x61), (0xC159)),  # HANGUL SYLLABLE SIOS YEO NIEUNCIEUC
    ((0x99, 0x62), (0xC15A)),  # HANGUL SYLLABLE SIOS YEO NIEUNHIEUH
    ((0x99, 0x63), (0xC15B)),  # HANGUL SYLLABLE SIOS YEO TIKEUT
    ((0x99, 0x64), (0xC15D)),  # HANGUL SYLLABLE SIOS YEO RIEULKIYEOK
    ((0x99, 0x65), (0xC15E)),  # HANGUL SYLLABLE SIOS YEO RIEULMIEUM
    ((0x99, 0x66), (0xC15F)),  # HANGUL SYLLABLE SIOS YEO RIEULPIEUP
    ((0x99, 0x67), (0xC160)),  # HANGUL SYLLABLE SIOS YEO RIEULSIOS
    ((0x99, 0x68), (0xC161)),  # HANGUL SYLLABLE SIOS YEO RIEULTHIEUTH
    ((0x99, 0x69), (0xC162)),  # HANGUL SYLLABLE SIOS YEO RIEULPHIEUPH
    ((0x99, 0x6A), (0xC163)),  # HANGUL SYLLABLE SIOS YEO RIEULHIEUH
    ((0x99, 0x6B), (0xC166)),  # HANGUL SYLLABLE SIOS YEO PIEUPSIOS
    ((0x99, 0x6C), (0xC16A)),  # HANGUL SYLLABLE SIOS YEO CIEUC
    ((0x99, 0x6D), (0xC16B)),  # HANGUL SYLLABLE SIOS YEO CHIEUCH
    ((0x99, 0x6E), (0xC16C)),  # HANGUL SYLLABLE SIOS YEO KHIEUKH
    ((0x99, 0x6F), (0xC16D)),  # HANGUL SYLLABLE SIOS YEO THIEUTH
    ((0x99, 0x70), (0xC16E)),  # HANGUL SYLLABLE SIOS YEO PHIEUPH
    ((0x99, 0x71), (0xC16F)),  # HANGUL SYLLABLE SIOS YEO HIEUH
    ((0x99, 0x72), (0xC171)),  # HANGUL SYLLABLE SIOS YE KIYEOK
    ((0x99, 0x73), (0xC172)),  # HANGUL SYLLABLE SIOS YE SSANGKIYEOK
    ((0x99, 0x74), (0xC173)),  # HANGUL SYLLABLE SIOS YE KIYEOKSIOS
    ((0x99, 0x75), (0xC175)),  # HANGUL SYLLABLE SIOS YE NIEUNCIEUC
    ((0x99, 0x76), (0xC176)),  # HANGUL SYLLABLE SIOS YE NIEUNHIEUH
    ((0x99, 0x77), (0xC177)),  # HANGUL SYLLABLE SIOS YE TIKEUT
    ((0x99, 0x78), (0xC179)),  # HANGUL SYLLABLE SIOS YE RIEULKIYEOK
    ((0x99, 0x79), (0xC17A)),  # HANGUL SYLLABLE SIOS YE RIEULMIEUM
    ((0x99, 0x7A), (0xC17B)),  # HANGUL SYLLABLE SIOS YE RIEULPIEUP
    ((0x99, 0x81), (0xC17C)),  # HANGUL SYLLABLE SIOS YE RIEULSIOS
    ((0x99, 0x82), (0xC17D)),  # HANGUL SYLLABLE SIOS YE RIEULTHIEUTH
    ((0x99, 0x83), (0xC17E)),  # HANGUL SYLLABLE SIOS YE RIEULPHIEUPH
    ((0x99, 0x84), (0xC17F)),  # HANGUL SYLLABLE SIOS YE RIEULHIEUH
    ((0x99, 0x85), (0xC180)),  # HANGUL SYLLABLE SIOS YE MIEUM
    ((0x99, 0x86), (0xC181)),  # HANGUL SYLLABLE SIOS YE PIEUP
    ((0x99, 0x87), (0xC182)),  # HANGUL SYLLABLE SIOS YE PIEUPSIOS
    ((0x99, 0x88), (0xC183)),  # HANGUL SYLLABLE SIOS YE SIOS
    ((0x99, 0x89), (0xC184)),  # HANGUL SYLLABLE SIOS YE SSANGSIOS
    ((0x99, 0x8A), (0xC186)),  # HANGUL SYLLABLE SIOS YE CIEUC
    ((0x99, 0x8B), (0xC187)),  # HANGUL SYLLABLE SIOS YE CHIEUCH
    ((0x99, 0x8C), (0xC188)),  # HANGUL SYLLABLE SIOS YE KHIEUKH
    ((0x99, 0x8D), (0xC189)),  # HANGUL SYLLABLE SIOS YE THIEUTH
    ((0x99, 0x8E), (0xC18A)),  # HANGUL SYLLABLE SIOS YE PHIEUPH
    ((0x99, 0x8F), (0xC18B)),  # HANGUL SYLLABLE SIOS YE HIEUH
    ((0x99, 0x90), (0xC18F)),  # HANGUL SYLLABLE SIOS O KIYEOKSIOS
    ((0x99, 0x91), (0xC191)),  # HANGUL SYLLABLE SIOS O NIEUNCIEUC
    ((0x99, 0x92), (0xC192)),  # HANGUL SYLLABLE SIOS O NIEUNHIEUH
    ((0x99, 0x93), (0xC193)),  # HANGUL SYLLABLE SIOS O TIKEUT
    ((0x99, 0x94), (0xC195)),  # HANGUL SYLLABLE SIOS O RIEULKIYEOK
    ((0x99, 0x95), (0xC197)),  # HANGUL SYLLABLE SIOS O RIEULPIEUP
    ((0x99, 0x96), (0xC198)),  # HANGUL SYLLABLE SIOS O RIEULSIOS
    ((0x99, 0x97), (0xC199)),  # HANGUL SYLLABLE SIOS O RIEULTHIEUTH
    ((0x99, 0x98), (0xC19A)),  # HANGUL SYLLABLE SIOS O RIEULPHIEUPH
    ((0x99, 0x99), (0xC19B)),  # HANGUL SYLLABLE SIOS O RIEULHIEUH
    ((0x99, 0x9A), (0xC19E)),  # HANGUL SYLLABLE SIOS O PIEUPSIOS
    ((0x99, 0x9B), (0xC1A0)),  # HANGUL SYLLABLE SIOS O SSANGSIOS
    ((0x99, 0x9C), (0xC1A2)),  # HANGUL SYLLABLE SIOS O CIEUC
    ((0x99, 0x9D), (0xC1A3)),  # HANGUL SYLLABLE SIOS O CHIEUCH
    ((0x99, 0x9E), (0xC1A4)),  # HANGUL SYLLABLE SIOS O KHIEUKH
    ((0x99, 0x9F), (0xC1A6)),  # HANGUL SYLLABLE SIOS O PHIEUPH
    ((0x99, 0xA0), (0xC1A7)),  # HANGUL SYLLABLE SIOS O HIEUH
    ((0x99, 0xA1), (0xC1AA)),  # HANGUL SYLLABLE SIOS WA SSANGKIYEOK
    ((0x99, 0xA2), (0xC1AB)),  # HANGUL SYLLABLE SIOS WA KIYEOKSIOS
    ((0x99, 0xA3), (0xC1AD)),  # HANGUL SYLLABLE SIOS WA NIEUNCIEUC
    ((0x99, 0xA4), (0xC1AE)),  # HANGUL SYLLABLE SIOS WA NIEUNHIEUH
    ((0x99, 0xA5), (0xC1AF)),  # HANGUL SYLLABLE SIOS WA TIKEUT
    ((0x99, 0xA6), (0xC1B1)),  # HANGUL SYLLABLE SIOS WA RIEULKIYEOK
    ((0x99, 0xA7), (0xC1B2)),  # HANGUL SYLLABLE SIOS WA RIEULMIEUM
    ((0x99, 0xA8), (0xC1B3)),  # HANGUL SYLLABLE SIOS WA RIEULPIEUP
    ((0x99, 0xA9), (0xC1B4)),  # HANGUL SYLLABLE SIOS WA RIEULSIOS
    ((0x99, 0xAA), (0xC1B5)),  # HANGUL SYLLABLE SIOS WA RIEULTHIEUTH
    ((0x99, 0xAB), (0xC1B6)),  # HANGUL SYLLABLE SIOS WA RIEULPHIEUPH
    ((0x99, 0xAC), (0xC1B7)),  # HANGUL SYLLABLE SIOS WA RIEULHIEUH
    ((0x99, 0xAD), (0xC1B8)),  # HANGUL SYLLABLE SIOS WA MIEUM
    ((0x99, 0xAE), (0xC1B9)),  # HANGUL SYLLABLE SIOS WA PIEUP
    ((0x99, 0xAF), (0xC1BA)),  # HANGUL SYLLABLE SIOS WA PIEUPSIOS
    ((0x99, 0xB0), (0xC1BB)),  # HANGUL SYLLABLE SIOS WA SIOS
    ((0x99, 0xB1), (0xC1BC)),  # HANGUL SYLLABLE SIOS WA SSANGSIOS
    ((0x99, 0xB2), (0xC1BE)),  # HANGUL SYLLABLE SIOS WA CIEUC
    ((0x99, 0xB3), (0xC1BF)),  # HANGUL SYLLABLE SIOS WA CHIEUCH
    ((0x99, 0xB4), (0xC1C0)),  # HANGUL SYLLABLE SIOS WA KHIEUKH
    ((0x99, 0xB5), (0xC1C1)),  # HANGUL SYLLABLE SIOS WA THIEUTH
    ((0x99, 0xB6), (0xC1C2)),  # HANGUL SYLLABLE SIOS WA PHIEUPH
    ((0x99, 0xB7), (0xC1C3)),  # HANGUL SYLLABLE SIOS WA HIEUH
    ((0x99, 0xB8), (0xC1C5)),  # HANGUL SYLLABLE SIOS WAE KIYEOK
    ((0x99, 0xB9), (0xC1C6)),  # HANGUL SYLLABLE SIOS WAE SSANGKIYEOK
    ((0x99, 0xBA), (0xC1C7)),  # HANGUL SYLLABLE SIOS WAE KIYEOKSIOS
    ((0x99, 0xBB), (0xC1C9)),  # HANGUL SYLLABLE SIOS WAE NIEUNCIEUC
    ((0x99, 0xBC), (0xC1CA)),  # HANGUL SYLLABLE SIOS WAE NIEUNHIEUH
    ((0x99, 0xBD), (0xC1CB)),  # HANGUL SYLLABLE SIOS WAE TIKEUT
    ((0x99, 0xBE), (0xC1CD)),  # HANGUL SYLLABLE SIOS WAE RIEULKIYEOK
    ((0x99, 0xBF), (0xC1CE)),  # HANGUL SYLLABLE SIOS WAE RIEULMIEUM
    ((0x99, 0xC0), (0xC1CF)),  # HANGUL SYLLABLE SIOS WAE RIEULPIEUP
    ((0x99, 0xC1), (0xC1D0)),  # HANGUL SYLLABLE SIOS WAE RIEULSIOS
    ((0x99, 0xC2), (0xC1D1)),  # HANGUL SYLLABLE SIOS WAE RIEULTHIEUTH
    ((0x99, 0xC3), (0xC1D2)),  # HANGUL SYLLABLE SIOS WAE RIEULPHIEUPH
    ((0x99, 0xC4), (0xC1D3)),  # HANGUL SYLLABLE SIOS WAE RIEULHIEUH
    ((0x99, 0xC5), (0xC1D5)),  # HANGUL SYLLABLE SIOS WAE PIEUP
    ((0x99, 0xC6), (0xC1D6)),  # HANGUL SYLLABLE SIOS WAE PIEUPSIOS
    ((0x99, 0xC7), (0xC1D9)),  # HANGUL SYLLABLE SIOS WAE IEUNG
    ((0x99, 0xC8), (0xC1DA)),  # HANGUL SYLLABLE SIOS WAE CIEUC
    ((0x99, 0xC9), (0xC1DB)),  # HANGUL SYLLABLE SIOS WAE CHIEUCH
    ((0x99, 0xCA), (0xC1DC)),  # HANGUL SYLLABLE SIOS WAE KHIEUKH
    ((0x99, 0xCB), (0xC1DD)),  # HANGUL SYLLABLE SIOS WAE THIEUTH
    ((0x99, 0xCC), (0xC1DE)),  # HANGUL SYLLABLE SIOS WAE PHIEUPH
    ((0x99, 0xCD), (0xC1DF)),  # HANGUL SYLLABLE SIOS WAE HIEUH
    ((0x99, 0xCE), (0xC1E1)),  # HANGUL SYLLABLE SIOS OE KIYEOK
    ((0x99, 0xCF), (0xC1E2)),  # HANGUL SYLLABLE SIOS OE SSANGKIYEOK
    ((0x99, 0xD0), (0xC1E3)),  # HANGUL SYLLABLE SIOS OE KIYEOKSIOS
    ((0x99, 0xD1), (0xC1E5)),  # HANGUL SYLLABLE SIOS OE NIEUNCIEUC
    ((0x99, 0xD2), (0xC1E6)),  # HANGUL SYLLABLE SIOS OE NIEUNHIEUH
    ((0x99, 0xD3), (0xC1E7)),  # HANGUL SYLLABLE SIOS OE TIKEUT
    ((0x99, 0xD4), (0xC1E9)),  # HANGUL SYLLABLE SIOS OE RIEULKIYEOK
    ((0x99, 0xD5), (0xC1EA)),  # HANGUL SYLLABLE SIOS OE RIEULMIEUM
    ((0x99, 0xD6), (0xC1EB)),  # HANGUL SYLLABLE SIOS OE RIEULPIEUP
    ((0x99, 0xD7), (0xC1EC)),  # HANGUL SYLLABLE SIOS OE RIEULSIOS
    ((0x99, 0xD8), (0xC1ED)),  # HANGUL SYLLABLE SIOS OE RIEULTHIEUTH
    ((0x99, 0xD9), (0xC1EE)),  # HANGUL SYLLABLE SIOS OE RIEULPHIEUPH
    ((0x99, 0xDA), (0xC1EF)),  # HANGUL SYLLABLE SIOS OE RIEULHIEUH
    ((0x99, 0xDB), (0xC1F2)),  # HANGUL SYLLABLE SIOS OE PIEUPSIOS
    ((0x99, 0xDC), (0xC1F4)),  # HANGUL SYLLABLE SIOS OE SSANGSIOS
    ((0x99, 0xDD), (0xC1F5)),  # HANGUL SYLLABLE SIOS OE IEUNG
    ((0x99, 0xDE), (0xC1F6)),  # HANGUL SYLLABLE SIOS OE CIEUC
    ((0x99, 0xDF), (0xC1F7)),  # HANGUL SYLLABLE SIOS OE CHIEUCH
    ((0x99, 0xE0), (0xC1F8)),  # HANGUL SYLLABLE SIOS OE KHIEUKH
    ((0x99, 0xE1), (0xC1F9)),  # HANGUL SYLLABLE SIOS OE THIEUTH
    ((0x99, 0xE2), (0xC1FA)),  # HANGUL SYLLABLE SIOS OE PHIEUPH
    ((0x99, 0xE3), (0xC1FB)),  # HANGUL SYLLABLE SIOS OE HIEUH
    ((0x99, 0xE4), (0xC1FE)),  # HANGUL SYLLABLE SIOS YO SSANGKIYEOK
    ((0x99, 0xE5), (0xC1FF)),  # HANGUL SYLLABLE SIOS YO KIYEOKSIOS
    ((0x99, 0xE6), (0xC201)),  # HANGUL SYLLABLE SIOS YO NIEUNCIEUC
    ((0x99, 0xE7), (0xC202)),  # HANGUL SYLLABLE SIOS YO NIEUNHIEUH
    ((0x99, 0xE8), (0xC203)),  # HANGUL SYLLABLE SIOS YO TIKEUT
    ((0x99, 0xE9), (0xC205)),  # HANGUL SYLLABLE SIOS YO RIEULKIYEOK
    ((0x99, 0xEA), (0xC206)),  # HANGUL SYLLABLE SIOS YO RIEULMIEUM
    ((0x99, 0xEB), (0xC207)),  # HANGUL SYLLABLE SIOS YO RIEULPIEUP
    ((0x99, 0xEC), (0xC208)),  # HANGUL SYLLABLE SIOS YO RIEULSIOS
    ((0x99, 0xED), (0xC209)),  # HANGUL SYLLABLE SIOS YO RIEULTHIEUTH
    ((0x99, 0xEE), (0xC20A)),  # HANGUL SYLLABLE SIOS YO RIEULPHIEUPH
    ((0x99, 0xEF), (0xC20B)),  # HANGUL SYLLABLE SIOS YO RIEULHIEUH
    ((0x99, 0xF0), (0xC20E)),  # HANGUL SYLLABLE SIOS YO PIEUPSIOS
    ((0x99, 0xF1), (0xC210)),  # HANGUL SYLLABLE SIOS YO SSANGSIOS
    ((0x99, 0xF2), (0xC212)),  # HANGUL SYLLABLE SIOS YO CIEUC
    ((0x99, 0xF3), (0xC213)),  # HANGUL SYLLABLE SIOS YO CHIEUCH
    ((0x99, 0xF4), (0xC214)),  # HANGUL SYLLABLE SIOS YO KHIEUKH
    ((0x99, 0xF5), (0xC215)),  # HANGUL SYLLABLE SIOS YO THIEUTH
    ((0x99, 0xF6), (0xC216)),  # HANGUL SYLLABLE SIOS YO PHIEUPH
    ((0x99, 0xF7), (0xC217)),  # HANGUL SYLLABLE SIOS YO HIEUH
    ((0x99, 0xF8), (0xC21A)),  # HANGUL SYLLABLE SIOS U SSANGKIYEOK
    ((0x99, 0xF9), (0xC21B)),  # HANGUL SYLLABLE SIOS U KIYEOKSIOS
    ((0x99, 0xFA), (0xC21D)),  # HANGUL SYLLABLE SIOS U NIEUNCIEUC
    ((0x99, 0xFB), (0xC21E)),  # HANGUL SYLLABLE SIOS U NIEUNHIEUH
    ((0x99, 0xFC), (0xC221)),  # HANGUL SYLLABLE SIOS U RIEULKIYEOK
    ((0x99, 0xFD), (0xC222)),  # HANGUL SYLLABLE SIOS U RIEULMIEUM
    ((0x99, 0xFE), (0xC223)),  # HANGUL SYLLABLE SIOS U RIEULPIEUP
    ((0x9A, 0x41), (0xC224)),  # HANGUL SYLLABLE SIOS U RIEULSIOS
    ((0x9A, 0x42), (0xC225)),  # HANGUL SYLLABLE SIOS U RIEULTHIEUTH
    ((0x9A, 0x43), (0xC226)),  # HANGUL SYLLABLE SIOS U RIEULPHIEUPH
    ((0x9A, 0x44), (0xC227)),  # HANGUL SYLLABLE SIOS U RIEULHIEUH
    ((0x9A, 0x45), (0xC22A)),  # HANGUL SYLLABLE SIOS U PIEUPSIOS
    ((0x9A, 0x46), (0xC22C)),  # HANGUL SYLLABLE SIOS U SSANGSIOS
    ((0x9A, 0x47), (0xC22E)),  # HANGUL SYLLABLE SIOS U CIEUC
    ((0x9A, 0x48), (0xC230)),  # HANGUL SYLLABLE SIOS U KHIEUKH
    ((0x9A, 0x49), (0xC233)),  # HANGUL SYLLABLE SIOS U HIEUH
    ((0x9A, 0x4A), (0xC235)),  # HANGUL SYLLABLE SIOS WEO KIYEOK
    ((0x9A, 0x4B), (0xC236)),  # HANGUL SYLLABLE SIOS WEO SSANGKIYEOK
    ((0x9A, 0x4C), (0xC237)),  # HANGUL SYLLABLE SIOS WEO KIYEOKSIOS
    ((0x9A, 0x4D), (0xC238)),  # HANGUL SYLLABLE SIOS WEO NIEUN
    ((0x9A, 0x4E), (0xC239)),  # HANGUL SYLLABLE SIOS WEO NIEUNCIEUC
    ((0x9A, 0x4F), (0xC23A)),  # HANGUL SYLLABLE SIOS WEO NIEUNHIEUH
    ((0x9A, 0x50), (0xC23B)),  # HANGUL SYLLABLE SIOS WEO TIKEUT
    ((0x9A, 0x51), (0xC23C)),  # HANGUL SYLLABLE SIOS WEO RIEUL
    ((0x9A, 0x52), (0xC23D)),  # HANGUL SYLLABLE SIOS WEO RIEULKIYEOK
    ((0x9A, 0x53), (0xC23E)),  # HANGUL SYLLABLE SIOS WEO RIEULMIEUM
    ((0x9A, 0x54), (0xC23F)),  # HANGUL SYLLABLE SIOS WEO RIEULPIEUP
    ((0x9A, 0x55), (0xC240)),  # HANGUL SYLLABLE SIOS WEO RIEULSIOS
    ((0x9A, 0x56), (0xC241)),  # HANGUL SYLLABLE SIOS WEO RIEULTHIEUTH
    ((0x9A, 0x57), (0xC242)),  # HANGUL SYLLABLE SIOS WEO RIEULPHIEUPH
    ((0x9A, 0x58), (0xC243)),  # HANGUL SYLLABLE SIOS WEO RIEULHIEUH
    ((0x9A, 0x59), (0xC244)),  # HANGUL SYLLABLE SIOS WEO MIEUM
    ((0x9A, 0x5A), (0xC245)),  # HANGUL SYLLABLE SIOS WEO PIEUP
    ((0x9A, 0x61), (0xC246)),  # HANGUL SYLLABLE SIOS WEO PIEUPSIOS
    ((0x9A, 0x62), (0xC247)),  # HANGUL SYLLABLE SIOS WEO SIOS
    ((0x9A, 0x63), (0xC249)),  # HANGUL SYLLABLE SIOS WEO IEUNG
    ((0x9A, 0x64), (0xC24A)),  # HANGUL SYLLABLE SIOS WEO CIEUC
    ((0x9A, 0x65), (0xC24B)),  # HANGUL SYLLABLE SIOS WEO CHIEUCH
    ((0x9A, 0x66), (0xC24C)),  # HANGUL SYLLABLE SIOS WEO KHIEUKH
    ((0x9A, 0x67), (0xC24D)),  # HANGUL SYLLABLE SIOS WEO THIEUTH
    ((0x9A, 0x68), (0xC24E)),  # HANGUL SYLLABLE SIOS WEO PHIEUPH
    ((0x9A, 0x69), (0xC24F)),  # HANGUL SYLLABLE SIOS WEO HIEUH
    ((0x9A, 0x6A), (0xC252)),  # HANGUL SYLLABLE SIOS WE SSANGKIYEOK
    ((0x9A, 0x6B), (0xC253)),  # HANGUL SYLLABLE SIOS WE KIYEOKSIOS
    ((0x9A, 0x6C), (0xC255)),  # HANGUL SYLLABLE SIOS WE NIEUNCIEUC
    ((0x9A, 0x6D), (0xC256)),  # HANGUL SYLLABLE SIOS WE NIEUNHIEUH
    ((0x9A, 0x6E), (0xC257)),  # HANGUL SYLLABLE SIOS WE TIKEUT
    ((0x9A, 0x6F), (0xC259)),  # HANGUL SYLLABLE SIOS WE RIEULKIYEOK
    ((0x9A, 0x70), (0xC25A)),  # HANGUL SYLLABLE SIOS WE RIEULMIEUM
    ((0x9A, 0x71), (0xC25B)),  # HANGUL SYLLABLE SIOS WE RIEULPIEUP
    ((0x9A, 0x72), (0xC25C)),  # HANGUL SYLLABLE SIOS WE RIEULSIOS
    ((0x9A, 0x73), (0xC25D)),  # HANGUL SYLLABLE SIOS WE RIEULTHIEUTH
    ((0x9A, 0x74), (0xC25E)),  # HANGUL SYLLABLE SIOS WE RIEULPHIEUPH
    ((0x9A, 0x75), (0xC25F)),  # HANGUL SYLLABLE SIOS WE RIEULHIEUH
    ((0x9A, 0x76), (0xC261)),  # HANGUL SYLLABLE SIOS WE PIEUP
    ((0x9A, 0x77), (0xC262)),  # HANGUL SYLLABLE SIOS WE PIEUPSIOS
    ((0x9A, 0x78), (0xC263)),  # HANGUL SYLLABLE SIOS WE SIOS
    ((0x9A, 0x79), (0xC264)),  # HANGUL SYLLABLE SIOS WE SSANGSIOS
    ((0x9A, 0x7A), (0xC266)),  # HANGUL SYLLABLE SIOS WE CIEUC
    ((0x9A, 0x81), (0xC267)),  # HANGUL SYLLABLE SIOS WE CHIEUCH
    ((0x9A, 0x82), (0xC268)),  # HANGUL SYLLABLE SIOS WE KHIEUKH
    ((0x9A, 0x83), (0xC269)),  # HANGUL SYLLABLE SIOS WE THIEUTH
    ((0x9A, 0x84), (0xC26A)),  # HANGUL SYLLABLE SIOS WE PHIEUPH
    ((0x9A, 0x85), (0xC26B)),  # HANGUL SYLLABLE SIOS WE HIEUH
    ((0x9A, 0x86), (0xC26E)),  # HANGUL SYLLABLE SIOS WI SSANGKIYEOK
    ((0x9A, 0x87), (0xC26F)),  # HANGUL SYLLABLE SIOS WI KIYEOKSIOS
    ((0x9A, 0x88), (0xC271)),  # HANGUL SYLLABLE SIOS WI NIEUNCIEUC
    ((0x9A, 0x89), (0xC272)),  # HANGUL SYLLABLE SIOS WI NIEUNHIEUH
    ((0x9A, 0x8A), (0xC273)),  # HANGUL SYLLABLE SIOS WI TIKEUT
    ((0x9A, 0x8B), (0xC275)),  # HANGUL SYLLABLE SIOS WI RIEULKIYEOK
    ((0x9A, 0x8C), (0xC276)),  # HANGUL SYLLABLE SIOS WI RIEULMIEUM
    ((0x9A, 0x8D), (0xC277)),  # HANGUL SYLLABLE SIOS WI RIEULPIEUP
    ((0x9A, 0x8E), (0xC278)),  # HANGUL SYLLABLE SIOS WI RIEULSIOS
    ((0x9A, 0x8F), (0xC279)),  # HANGUL SYLLABLE SIOS WI RIEULTHIEUTH
    ((0x9A, 0x90), (0xC27A)),  # HANGUL SYLLABLE SIOS WI RIEULPHIEUPH
    ((0x9A, 0x91), (0xC27B)),  # HANGUL SYLLABLE SIOS WI RIEULHIEUH
    ((0x9A, 0x92), (0xC27E)),  # HANGUL SYLLABLE SIOS WI PIEUPSIOS
    ((0x9A, 0x93), (0xC280)),  # HANGUL SYLLABLE SIOS WI SSANGSIOS
    ((0x9A, 0x94), (0xC282)),  # HANGUL SYLLABLE SIOS WI CIEUC
    ((0x9A, 0x95), (0xC283)),  # HANGUL SYLLABLE SIOS WI CHIEUCH
    ((0x9A, 0x96), (0xC284)),  # HANGUL SYLLABLE SIOS WI KHIEUKH
    ((0x9A, 0x97), (0xC285)),  # HANGUL SYLLABLE SIOS WI THIEUTH
    ((0x9A, 0x98), (0xC286)),  # HANGUL SYLLABLE SIOS WI PHIEUPH
    ((0x9A, 0x99), (0xC287)),  # HANGUL SYLLABLE SIOS WI HIEUH
    ((0x9A, 0x9A), (0xC28A)),  # HANGUL SYLLABLE SIOS YU SSANGKIYEOK
    ((0x9A, 0x9B), (0xC28B)),  # HANGUL SYLLABLE SIOS YU KIYEOKSIOS
    ((0x9A, 0x9C), (0xC28C)),  # HANGUL SYLLABLE SIOS YU NIEUN
    ((0x9A, 0x9D), (0xC28D)),  # HANGUL SYLLABLE SIOS YU NIEUNCIEUC
    ((0x9A, 0x9E), (0xC28E)),  # HANGUL SYLLABLE SIOS YU NIEUNHIEUH
    ((0x9A, 0x9F), (0xC28F)),  # HANGUL SYLLABLE SIOS YU TIKEUT
    ((0x9A, 0xA0), (0xC291)),  # HANGUL SYLLABLE SIOS YU RIEULKIYEOK
    ((0x9A, 0xA1), (0xC292)),  # HANGUL SYLLABLE SIOS YU RIEULMIEUM
    ((0x9A, 0xA2), (0xC293)),  # HANGUL SYLLABLE SIOS YU RIEULPIEUP
    ((0x9A, 0xA3), (0xC294)),  # HANGUL SYLLABLE SIOS YU RIEULSIOS
    ((0x9A, 0xA4), (0xC295)),  # HANGUL SYLLABLE SIOS YU RIEULTHIEUTH
    ((0x9A, 0xA5), (0xC296)),  # HANGUL SYLLABLE SIOS YU RIEULPHIEUPH
    ((0x9A, 0xA6), (0xC297)),  # HANGUL SYLLABLE SIOS YU RIEULHIEUH
    ((0x9A, 0xA7), (0xC299)),  # HANGUL SYLLABLE SIOS YU PIEUP
    ((0x9A, 0xA8), (0xC29A)),  # HANGUL SYLLABLE SIOS YU PIEUPSIOS
    ((0x9A, 0xA9), (0xC29C)),  # HANGUL SYLLABLE SIOS YU SSANGSIOS
    ((0x9A, 0xAA), (0xC29E)),  # HANGUL SYLLABLE SIOS YU CIEUC
    ((0x9A, 0xAB), (0xC29F)),  # HANGUL SYLLABLE SIOS YU CHIEUCH
    ((0x9A, 0xAC), (0xC2A0)),  # HANGUL SYLLABLE SIOS YU KHIEUKH
    ((0x9A, 0xAD), (0xC2A1)),  # HANGUL SYLLABLE SIOS YU THIEUTH
    ((0x9A, 0xAE), (0xC2A2)),  # HANGUL SYLLABLE SIOS YU PHIEUPH
    ((0x9A, 0xAF), (0xC2A3)),  # HANGUL SYLLABLE SIOS YU HIEUH
    ((0x9A, 0xB0), (0xC2A6)),  # HANGUL SYLLABLE SIOS EU SSANGKIYEOK
    ((0x9A, 0xB1), (0xC2A7)),  # HANGUL SYLLABLE SIOS EU KIYEOKSIOS
    ((0x9A, 0xB2), (0xC2A9)),  # HANGUL SYLLABLE SIOS EU NIEUNCIEUC
    ((0x9A, 0xB3), (0xC2AA)),  # HANGUL SYLLABLE SIOS EU NIEUNHIEUH
    ((0x9A, 0xB4), (0xC2AB)),  # HANGUL SYLLABLE SIOS EU TIKEUT
    ((0x9A, 0xB5), (0xC2AE)),  # HANGUL SYLLABLE SIOS EU RIEULMIEUM
    ((0x9A, 0xB6), (0xC2AF)),  # HANGUL SYLLABLE SIOS EU RIEULPIEUP
    ((0x9A, 0xB7), (0xC2B0)),  # HANGUL SYLLABLE SIOS EU RIEULSIOS
    ((0x9A, 0xB8), (0xC2B1)),  # HANGUL SYLLABLE SIOS EU RIEULTHIEUTH
    ((0x9A, 0xB9), (0xC2B2)),  # HANGUL SYLLABLE SIOS EU RIEULPHIEUPH
    ((0x9A, 0xBA), (0xC2B3)),  # HANGUL SYLLABLE SIOS EU RIEULHIEUH
    ((0x9A, 0xBB), (0xC2B6)),  # HANGUL SYLLABLE SIOS EU PIEUPSIOS
    ((0x9A, 0xBC), (0xC2B8)),  # HANGUL SYLLABLE SIOS EU SSANGSIOS
    ((0x9A, 0xBD), (0xC2BA)),  # HANGUL SYLLABLE SIOS EU CIEUC
    ((0x9A, 0xBE), (0xC2BB)),  # HANGUL SYLLABLE SIOS EU CHIEUCH
    ((0x9A, 0xBF), (0xC2BC)),  # HANGUL SYLLABLE SIOS EU KHIEUKH
    ((0x9A, 0xC0), (0xC2BD)),  # HANGUL SYLLABLE SIOS EU THIEUTH
    ((0x9A, 0xC1), (0xC2BE)),  # HANGUL SYLLABLE SIOS EU PHIEUPH
    ((0x9A, 0xC2), (0xC2BF)),  # HANGUL SYLLABLE SIOS EU HIEUH
    ((0x9A, 0xC3), (0xC2C0)),  # HANGUL SYLLABLE SIOS YI
    ((0x9A, 0xC4), (0xC2C1)),  # HANGUL SYLLABLE SIOS YI KIYEOK
    ((0x9A, 0xC5), (0xC2C2)),  # HANGUL SYLLABLE SIOS YI SSANGKIYEOK
    ((0x9A, 0xC6), (0xC2C3)),  # HANGUL SYLLABLE SIOS YI KIYEOKSIOS
    ((0x9A, 0xC7), (0xC2C4)),  # HANGUL SYLLABLE SIOS YI NIEUN
    ((0x9A, 0xC8), (0xC2C5)),  # HANGUL SYLLABLE SIOS YI NIEUNCIEUC
    ((0x9A, 0xC9), (0xC2C6)),  # HANGUL SYLLABLE SIOS YI NIEUNHIEUH
    ((0x9A, 0xCA), (0xC2C7)),  # HANGUL SYLLABLE SIOS YI TIKEUT
    ((0x9A, 0xCB), (0xC2C8)),  # HANGUL SYLLABLE SIOS YI RIEUL
    ((0x9A, 0xCC), (0xC2C9)),  # HANGUL SYLLABLE SIOS YI RIEULKIYEOK
    ((0x9A, 0xCD), (0xC2CA)),  # HANGUL SYLLABLE SIOS YI RIEULMIEUM
    ((0x9A, 0xCE), (0xC2CB)),  # HANGUL SYLLABLE SIOS YI RIEULPIEUP
    ((0x9A, 0xCF), (0xC2CC)),  # HANGUL SYLLABLE SIOS YI RIEULSIOS
    ((0x9A, 0xD0), (0xC2CD)),  # HANGUL SYLLABLE SIOS YI RIEULTHIEUTH
    ((0x9A, 0xD1), (0xC2CE)),  # HANGUL SYLLABLE SIOS YI RIEULPHIEUPH
    ((0x9A, 0xD2), (0xC2CF)),  # HANGUL SYLLABLE SIOS YI RIEULHIEUH
    ((0x9A, 0xD3), (0xC2D0)),  # HANGUL SYLLABLE SIOS YI MIEUM
    ((0x9A, 0xD4), (0xC2D1)),  # HANGUL SYLLABLE SIOS YI PIEUP
    ((0x9A, 0xD5), (0xC2D2)),  # HANGUL SYLLABLE SIOS YI PIEUPSIOS
    ((0x9A, 0xD6), (0xC2D3)),  # HANGUL SYLLABLE SIOS YI SIOS
    ((0x9A, 0xD7), (0xC2D4)),  # HANGUL SYLLABLE SIOS YI SSANGSIOS
    ((0x9A, 0xD8), (0xC2D5)),  # HANGUL SYLLABLE SIOS YI IEUNG
    ((0x9A, 0xD9), (0xC2D6)),  # HANGUL SYLLABLE SIOS YI CIEUC
    ((0x9A, 0xDA), (0xC2D7)),  # HANGUL SYLLABLE SIOS YI CHIEUCH
    ((0x9A, 0xDB), (0xC2D8)),  # HANGUL SYLLABLE SIOS YI KHIEUKH
    ((0x9A, 0xDC), (0xC2D9)),  # HANGUL SYLLABLE SIOS YI THIEUTH
    ((0x9A, 0xDD), (0xC2DA)),  # HANGUL SYLLABLE SIOS YI PHIEUPH
    ((0x9A, 0xDE), (0xC2DB)),  # HANGUL SYLLABLE SIOS YI HIEUH
    ((0x9A, 0xDF), (0xC2DE)),  # HANGUL SYLLABLE SIOS I SSANGKIYEOK
    ((0x9A, 0xE0), (0xC2DF)),  # HANGUL SYLLABLE SIOS I KIYEOKSIOS
    ((0x9A, 0xE1), (0xC2E1)),  # HANGUL SYLLABLE SIOS I NIEUNCIEUC
    ((0x9A, 0xE2), (0xC2E2)),  # HANGUL SYLLABLE SIOS I NIEUNHIEUH
    ((0x9A, 0xE3), (0xC2E5)),  # HANGUL SYLLABLE SIOS I RIEULKIYEOK
    ((0x9A, 0xE4), (0xC2E6)),  # HANGUL SYLLABLE SIOS I RIEULMIEUM
    ((0x9A, 0xE5), (0xC2E7)),  # HANGUL SYLLABLE SIOS I RIEULPIEUP
    ((0x9A, 0xE6), (0xC2E8)),  # HANGUL SYLLABLE SIOS I RIEULSIOS
    ((0x9A, 0xE7), (0xC2E9)),  # HANGUL SYLLABLE SIOS I RIEULTHIEUTH
    ((0x9A, 0xE8), (0xC2EA)),  # HANGUL SYLLABLE SIOS I RIEULPHIEUPH
    ((0x9A, 0xE9), (0xC2EE)),  # HANGUL SYLLABLE SIOS I PIEUPSIOS
    ((0x9A, 0xEA), (0xC2F0)),  # HANGUL SYLLABLE SIOS I SSANGSIOS
    ((0x9A, 0xEB), (0xC2F2)),  # HANGUL SYLLABLE SIOS I CIEUC
    ((0x9A, 0xEC), (0xC2F3)),  # HANGUL SYLLABLE SIOS I CHIEUCH
    ((0x9A, 0xED), (0xC2F4)),  # HANGUL SYLLABLE SIOS I KHIEUKH
    ((0x9A, 0xEE), (0xC2F5)),  # HANGUL SYLLABLE SIOS I THIEUTH
    ((0x9A, 0xEF), (0xC2F7)),  # HANGUL SYLLABLE SIOS I HIEUH
    ((0x9A, 0xF0), (0xC2FA)),  # HANGUL SYLLABLE SSANGSIOS A SSANGKIYEOK
    ((0x9A, 0xF1), (0xC2FD)),  # HANGUL SYLLABLE SSANGSIOS A NIEUNCIEUC
    ((0x9A, 0xF2), (0xC2FE)),  # HANGUL SYLLABLE SSANGSIOS A NIEUNHIEUH
    ((0x9A, 0xF3), (0xC2FF)),  # HANGUL SYLLABLE SSANGSIOS A TIKEUT
    ((0x9A, 0xF4), (0xC301)),  # HANGUL SYLLABLE SSANGSIOS A RIEULKIYEOK
    ((0x9A, 0xF5), (0xC302)),  # HANGUL SYLLABLE SSANGSIOS A RIEULMIEUM
    ((0x9A, 0xF6), (0xC303)),  # HANGUL SYLLABLE SSANGSIOS A RIEULPIEUP
    ((0x9A, 0xF7), (0xC304)),  # HANGUL SYLLABLE SSANGSIOS A RIEULSIOS
    ((0x9A, 0xF8), (0xC305)),  # HANGUL SYLLABLE SSANGSIOS A RIEULTHIEUTH
    ((0x9A, 0xF9), (0xC306)),  # HANGUL SYLLABLE SSANGSIOS A RIEULPHIEUPH
    ((0x9A, 0xFA), (0xC307)),  # HANGUL SYLLABLE SSANGSIOS A RIEULHIEUH
    ((0x9A, 0xFB), (0xC30A)),  # HANGUL SYLLABLE SSANGSIOS A PIEUPSIOS
    ((0x9A, 0xFC), (0xC30B)),  # HANGUL SYLLABLE SSANGSIOS A SIOS
    ((0x9A, 0xFD), (0xC30E)),  # HANGUL SYLLABLE SSANGSIOS A CIEUC
    ((0x9A, 0xFE), (0xC30F)),  # HANGUL SYLLABLE SSANGSIOS A CHIEUCH
    ((0x9B, 0x41), (0xC310)),  # HANGUL SYLLABLE SSANGSIOS A KHIEUKH
    ((0x9B, 0x42), (0xC311)),  # HANGUL SYLLABLE SSANGSIOS A THIEUTH
    ((0x9B, 0x43), (0xC312)),  # HANGUL SYLLABLE SSANGSIOS A PHIEUPH
    ((0x9B, 0x44), (0xC316)),  # HANGUL SYLLABLE SSANGSIOS AE SSANGKIYEOK
    ((0x9B, 0x45), (0xC317)),  # HANGUL SYLLABLE SSANGSIOS AE KIYEOKSIOS
    ((0x9B, 0x46), (0xC319)),  # HANGUL SYLLABLE SSANGSIOS AE NIEUNCIEUC
    ((0x9B, 0x47), (0xC31A)),  # HANGUL SYLLABLE SSANGSIOS AE NIEUNHIEUH
    ((0x9B, 0x48), (0xC31B)),  # HANGUL SYLLABLE SSANGSIOS AE TIKEUT
    ((0x9B, 0x49), (0xC31D)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULKIYEOK
    ((0x9B, 0x4A), (0xC31E)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULMIEUM
    ((0x9B, 0x4B), (0xC31F)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULPIEUP
    ((0x9B, 0x4C), (0xC320)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULSIOS
    ((0x9B, 0x4D), (0xC321)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULTHIEUTH
    ((0x9B, 0x4E), (0xC322)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULPHIEUPH
    ((0x9B, 0x4F), (0xC323)),  # HANGUL SYLLABLE SSANGSIOS AE RIEULHIEUH
    ((0x9B, 0x50), (0xC326)),  # HANGUL SYLLABLE SSANGSIOS AE PIEUPSIOS
    ((0x9B, 0x51), (0xC327)),  # HANGUL SYLLABLE SSANGSIOS AE SIOS
    ((0x9B, 0x52), (0xC32A)),  # HANGUL SYLLABLE SSANGSIOS AE CIEUC
    ((0x9B, 0x53), (0xC32B)),  # HANGUL SYLLABLE SSANGSIOS AE CHIEUCH
    ((0x9B, 0x54), (0xC32C)),  # HANGUL SYLLABLE SSANGSIOS AE KHIEUKH
    ((0x9B, 0x55), (0xC32D)),  # HANGUL SYLLABLE SSANGSIOS AE THIEUTH
    ((0x9B, 0x56), (0xC32E)),  # HANGUL SYLLABLE SSANGSIOS AE PHIEUPH
    ((0x9B, 0x57), (0xC32F)),  # HANGUL SYLLABLE SSANGSIOS AE HIEUH
    ((0x9B, 0x58), (0xC330)),  # HANGUL SYLLABLE SSANGSIOS YA
    ((0x9B, 0x59), (0xC331)),  # HANGUL SYLLABLE SSANGSIOS YA KIYEOK
    ((0x9B, 0x5A), (0xC332)),  # HANGUL SYLLABLE SSANGSIOS YA SSANGKIYEOK
    ((0x9B, 0x61), (0xC333)),  # HANGUL SYLLABLE SSANGSIOS YA KIYEOKSIOS
    ((0x9B, 0x62), (0xC334)),  # HANGUL SYLLABLE SSANGSIOS YA NIEUN
    ((0x9B, 0x63), (0xC335)),  # HANGUL SYLLABLE SSANGSIOS YA NIEUNCIEUC
    ((0x9B, 0x64), (0xC336)),  # HANGUL SYLLABLE SSANGSIOS YA NIEUNHIEUH
    ((0x9B, 0x65), (0xC337)),  # HANGUL SYLLABLE SSANGSIOS YA TIKEUT
    ((0x9B, 0x66), (0xC338)),  # HANGUL SYLLABLE SSANGSIOS YA RIEUL
    ((0x9B, 0x67), (0xC339)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULKIYEOK
    ((0x9B, 0x68), (0xC33A)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULMIEUM
    ((0x9B, 0x69), (0xC33B)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULPIEUP
    ((0x9B, 0x6A), (0xC33C)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULSIOS
    ((0x9B, 0x6B), (0xC33D)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULTHIEUTH
    ((0x9B, 0x6C), (0xC33E)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULPHIEUPH
    ((0x9B, 0x6D), (0xC33F)),  # HANGUL SYLLABLE SSANGSIOS YA RIEULHIEUH
    ((0x9B, 0x6E), (0xC340)),  # HANGUL SYLLABLE SSANGSIOS YA MIEUM
    ((0x9B, 0x6F), (0xC341)),  # HANGUL SYLLABLE SSANGSIOS YA PIEUP
    ((0x9B, 0x70), (0xC342)),  # HANGUL SYLLABLE SSANGSIOS YA PIEUPSIOS
    ((0x9B, 0x71), (0xC343)),  # HANGUL SYLLABLE SSANGSIOS YA SIOS
    ((0x9B, 0x72), (0xC344)),  # HANGUL SYLLABLE SSANGSIOS YA SSANGSIOS
    ((0x9B, 0x73), (0xC346)),  # HANGUL SYLLABLE SSANGSIOS YA CIEUC
    ((0x9B, 0x74), (0xC347)),  # HANGUL SYLLABLE SSANGSIOS YA CHIEUCH
    ((0x9B, 0x75), (0xC348)),  # HANGUL SYLLABLE SSANGSIOS YA KHIEUKH
    ((0x9B, 0x76), (0xC349)),  # HANGUL SYLLABLE SSANGSIOS YA THIEUTH
    ((0x9B, 0x77), (0xC34A)),  # HANGUL SYLLABLE SSANGSIOS YA PHIEUPH
    ((0x9B, 0x78), (0xC34B)),  # HANGUL SYLLABLE SSANGSIOS YA HIEUH
    ((0x9B, 0x79), (0xC34C)),  # HANGUL SYLLABLE SSANGSIOS YAE
    ((0x9B, 0x7A), (0xC34D)),  # HANGUL SYLLABLE SSANGSIOS YAE KIYEOK
    ((0x9B, 0x81), (0xC34E)),  # HANGUL SYLLABLE SSANGSIOS YAE SSANGKIYEOK
    ((0x9B, 0x82), (0xC34F)),  # HANGUL SYLLABLE SSANGSIOS YAE KIYEOKSIOS
    ((0x9B, 0x83), (0xC350)),  # HANGUL SYLLABLE SSANGSIOS YAE NIEUN
    ((0x9B, 0x84), (0xC351)),  # HANGUL SYLLABLE SSANGSIOS YAE NIEUNCIEUC
    ((0x9B, 0x85), (0xC352)),  # HANGUL SYLLABLE SSANGSIOS YAE NIEUNHIEUH
    ((0x9B, 0x86), (0xC353)),  # HANGUL SYLLABLE SSANGSIOS YAE TIKEUT
    ((0x9B, 0x87), (0xC354)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEUL
    ((0x9B, 0x88), (0xC355)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULKIYEOK
    ((0x9B, 0x89), (0xC356)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULMIEUM
    ((0x9B, 0x8A), (0xC357)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULPIEUP
    ((0x9B, 0x8B), (0xC358)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULSIOS
    ((0x9B, 0x8C), (0xC359)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULTHIEUTH
    ((0x9B, 0x8D), (0xC35A)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULPHIEUPH
    ((0x9B, 0x8E), (0xC35B)),  # HANGUL SYLLABLE SSANGSIOS YAE RIEULHIEUH
    ((0x9B, 0x8F), (0xC35C)),  # HANGUL SYLLABLE SSANGSIOS YAE MIEUM
    ((0x9B, 0x90), (0xC35D)),  # HANGUL SYLLABLE SSANGSIOS YAE PIEUP
    ((0x9B, 0x91), (0xC35E)),  # HANGUL SYLLABLE SSANGSIOS YAE PIEUPSIOS
    ((0x9B, 0x92), (0xC35F)),  # HANGUL SYLLABLE SSANGSIOS YAE SIOS
    ((0x9B, 0x93), (0xC360)),  # HANGUL SYLLABLE SSANGSIOS YAE SSANGSIOS
    ((0x9B, 0x94), (0xC361)),  # HANGUL SYLLABLE SSANGSIOS YAE IEUNG
    ((0x9B, 0x95), (0xC362)),  # HANGUL SYLLABLE SSANGSIOS YAE CIEUC
    ((0x9B, 0x96), (0xC363)),  # HANGUL SYLLABLE SSANGSIOS YAE CHIEUCH
    ((0x9B, 0x97), (0xC364)),  # HANGUL SYLLABLE SSANGSIOS YAE KHIEUKH
    ((0x9B, 0x98), (0xC365)),  # HANGUL SYLLABLE SSANGSIOS YAE THIEUTH
    ((0x9B, 0x99), (0xC366)),  # HANGUL SYLLABLE SSANGSIOS YAE PHIEUPH
    ((0x9B, 0x9A), (0xC367)),  # HANGUL SYLLABLE SSANGSIOS YAE HIEUH
    ((0x9B, 0x9B), (0xC36A)),  # HANGUL SYLLABLE SSANGSIOS EO SSANGKIYEOK
    ((0x9B, 0x9C), (0xC36B)),  # HANGUL SYLLABLE SSANGSIOS EO KIYEOKSIOS
    ((0x9B, 0x9D), (0xC36D)),  # HANGUL SYLLABLE SSANGSIOS EO NIEUNCIEUC
    ((0x9B, 0x9E), (0xC36E)),  # HANGUL SYLLABLE SSANGSIOS EO NIEUNHIEUH
    ((0x9B, 0x9F), (0xC36F)),  # HANGUL SYLLABLE SSANGSIOS EO TIKEUT
    ((0x9B, 0xA0), (0xC371)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULKIYEOK
    ((0x9B, 0xA1), (0xC373)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULPIEUP
    ((0x9B, 0xA2), (0xC374)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULSIOS
    ((0x9B, 0xA3), (0xC375)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULTHIEUTH
    ((0x9B, 0xA4), (0xC376)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULPHIEUPH
    ((0x9B, 0xA5), (0xC377)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULHIEUH
    ((0x9B, 0xA6), (0xC37A)),  # HANGUL SYLLABLE SSANGSIOS EO PIEUPSIOS
    ((0x9B, 0xA7), (0xC37B)),  # HANGUL SYLLABLE SSANGSIOS EO SIOS
    ((0x9B, 0xA8), (0xC37E)),  # HANGUL SYLLABLE SSANGSIOS EO CIEUC
    ((0x9B, 0xA9), (0xC37F)),  # HANGUL SYLLABLE SSANGSIOS EO CHIEUCH
    ((0x9B, 0xAA), (0xC380)),  # HANGUL SYLLABLE SSANGSIOS EO KHIEUKH
    ((0x9B, 0xAB), (0xC381)),  # HANGUL SYLLABLE SSANGSIOS EO THIEUTH
    ((0x9B, 0xAC), (0xC382)),  # HANGUL SYLLABLE SSANGSIOS EO PHIEUPH
    ((0x9B, 0xAD), (0xC383)),  # HANGUL SYLLABLE SSANGSIOS EO HIEUH
    ((0x9B, 0xAE), (0xC385)),  # HANGUL SYLLABLE SSANGSIOS E KIYEOK
    ((0x9B, 0xAF), (0xC386)),  # HANGUL SYLLABLE SSANGSIOS E SSANGKIYEOK
    ((0x9B, 0xB0), (0xC387)),  # HANGUL SYLLABLE SSANGSIOS E KIYEOKSIOS
    ((0x9B, 0xB1), (0xC389)),  # HANGUL SYLLABLE SSANGSIOS E NIEUNCIEUC
    ((0x9B, 0xB2), (0xC38A)),  # HANGUL SYLLABLE SSANGSIOS E NIEUNHIEUH
    ((0x9B, 0xB3), (0xC38B)),  # HANGUL SYLLABLE SSANGSIOS E TIKEUT
    ((0x9B, 0xB4), (0xC38D)),  # HANGUL SYLLABLE SSANGSIOS E RIEULKIYEOK
    ((0x9B, 0xB5), (0xC38E)),  # HANGUL SYLLABLE SSANGSIOS E RIEULMIEUM
    ((0x9B, 0xB6), (0xC38F)),  # HANGUL SYLLABLE SSANGSIOS E RIEULPIEUP
    ((0x9B, 0xB7), (0xC390)),  # HANGUL SYLLABLE SSANGSIOS E RIEULSIOS
    ((0x9B, 0xB8), (0xC391)),  # HANGUL SYLLABLE SSANGSIOS E RIEULTHIEUTH
    ((0x9B, 0xB9), (0xC392)),  # HANGUL SYLLABLE SSANGSIOS E RIEULPHIEUPH
    ((0x9B, 0xBA), (0xC393)),  # HANGUL SYLLABLE SSANGSIOS E RIEULHIEUH
    ((0x9B, 0xBB), (0xC394)),  # HANGUL SYLLABLE SSANGSIOS E MIEUM
    ((0x9B, 0xBC), (0xC395)),  # HANGUL SYLLABLE SSANGSIOS E PIEUP
    ((0x9B, 0xBD), (0xC396)),  # HANGUL SYLLABLE SSANGSIOS E PIEUPSIOS
    ((0x9B, 0xBE), (0xC397)),  # HANGUL SYLLABLE SSANGSIOS E SIOS
    ((0x9B, 0xBF), (0xC398)),  # HANGUL SYLLABLE SSANGSIOS E SSANGSIOS
    ((0x9B, 0xC0), (0xC399)),  # HANGUL SYLLABLE SSANGSIOS E IEUNG
    ((0x9B, 0xC1), (0xC39A)),  # HANGUL SYLLABLE SSANGSIOS E CIEUC
    ((0x9B, 0xC2), (0xC39B)),  # HANGUL SYLLABLE SSANGSIOS E CHIEUCH
    ((0x9B, 0xC3), (0xC39C)),  # HANGUL SYLLABLE SSANGSIOS E KHIEUKH
    ((0x9B, 0xC4), (0xC39D)),  # HANGUL SYLLABLE SSANGSIOS E THIEUTH
    ((0x9B, 0xC5), (0xC39E)),  # HANGUL SYLLABLE SSANGSIOS E PHIEUPH
    ((0x9B, 0xC6), (0xC39F)),  # HANGUL SYLLABLE SSANGSIOS E HIEUH
    ((0x9B, 0xC7), (0xC3A0)),  # HANGUL SYLLABLE SSANGSIOS YEO
    ((0x9B, 0xC8), (0xC3A1)),  # HANGUL SYLLABLE SSANGSIOS YEO KIYEOK
    ((0x9B, 0xC9), (0xC3A2)),  # HANGUL SYLLABLE SSANGSIOS YEO SSANGKIYEOK
    ((0x9B, 0xCA), (0xC3A3)),  # HANGUL SYLLABLE SSANGSIOS YEO KIYEOKSIOS
    ((0x9B, 0xCB), (0xC3A4)),  # HANGUL SYLLABLE SSANGSIOS YEO NIEUN
    ((0x9B, 0xCC), (0xC3A5)),  # HANGUL SYLLABLE SSANGSIOS YEO NIEUNCIEUC
    ((0x9B, 0xCD), (0xC3A6)),  # HANGUL SYLLABLE SSANGSIOS YEO NIEUNHIEUH
    ((0x9B, 0xCE), (0xC3A7)),  # HANGUL SYLLABLE SSANGSIOS YEO TIKEUT
    ((0x9B, 0xCF), (0xC3A8)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEUL
    ((0x9B, 0xD0), (0xC3A9)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULKIYEOK
    ((0x9B, 0xD1), (0xC3AA)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULMIEUM
    ((0x9B, 0xD2), (0xC3AB)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULPIEUP
    ((0x9B, 0xD3), (0xC3AC)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULSIOS
    ((0x9B, 0xD4), (0xC3AD)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULTHIEUTH
    ((0x9B, 0xD5), (0xC3AE)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULPHIEUPH
    ((0x9B, 0xD6), (0xC3AF)),  # HANGUL SYLLABLE SSANGSIOS YEO RIEULHIEUH
    ((0x9B, 0xD7), (0xC3B0)),  # HANGUL SYLLABLE SSANGSIOS YEO MIEUM
    ((0x9B, 0xD8), (0xC3B1)),  # HANGUL SYLLABLE SSANGSIOS YEO PIEUP
    ((0x9B, 0xD9), (0xC3B2)),  # HANGUL SYLLABLE SSANGSIOS YEO PIEUPSIOS
    ((0x9B, 0xDA), (0xC3B3)),  # HANGUL SYLLABLE SSANGSIOS YEO SIOS
    ((0x9B, 0xDB), (0xC3B4)),  # HANGUL SYLLABLE SSANGSIOS YEO SSANGSIOS
    ((0x9B, 0xDC), (0xC3B5)),  # HANGUL SYLLABLE SSANGSIOS YEO IEUNG
    ((0x9B, 0xDD), (0xC3B6)),  # HANGUL SYLLABLE SSANGSIOS YEO CIEUC
    ((0x9B, 0xDE), (0xC3B7)),  # HANGUL SYLLABLE SSANGSIOS YEO CHIEUCH
    ((0x9B, 0xDF), (0xC3B8)),  # HANGUL SYLLABLE SSANGSIOS YEO KHIEUKH
    ((0x9B, 0xE0), (0xC3B9)),  # HANGUL SYLLABLE SSANGSIOS YEO THIEUTH
    ((0x9B, 0xE1), (0xC3BA)),  # HANGUL SYLLABLE SSANGSIOS YEO PHIEUPH
    ((0x9B, 0xE2), (0xC3BB)),  # HANGUL SYLLABLE SSANGSIOS YEO HIEUH
    ((0x9B, 0xE3), (0xC3BC)),  # HANGUL SYLLABLE SSANGSIOS YE
    ((0x9B, 0xE4), (0xC3BD)),  # HANGUL SYLLABLE SSANGSIOS YE KIYEOK
    ((0x9B, 0xE5), (0xC3BE)),  # HANGUL SYLLABLE SSANGSIOS YE SSANGKIYEOK
    ((0x9B, 0xE6), (0xC3BF)),  # HANGUL SYLLABLE SSANGSIOS YE KIYEOKSIOS
    ((0x9B, 0xE7), (0xC3C1)),  # HANGUL SYLLABLE SSANGSIOS YE NIEUNCIEUC
    ((0x9B, 0xE8), (0xC3C2)),  # HANGUL SYLLABLE SSANGSIOS YE NIEUNHIEUH
    ((0x9B, 0xE9), (0xC3C3)),  # HANGUL SYLLABLE SSANGSIOS YE TIKEUT
    ((0x9B, 0xEA), (0xC3C4)),  # HANGUL SYLLABLE SSANGSIOS YE RIEUL
    ((0x9B, 0xEB), (0xC3C5)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULKIYEOK
    ((0x9B, 0xEC), (0xC3C6)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULMIEUM
    ((0x9B, 0xED), (0xC3C7)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULPIEUP
    ((0x9B, 0xEE), (0xC3C8)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULSIOS
    ((0x9B, 0xEF), (0xC3C9)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULTHIEUTH
    ((0x9B, 0xF0), (0xC3CA)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULPHIEUPH
    ((0x9B, 0xF1), (0xC3CB)),  # HANGUL SYLLABLE SSANGSIOS YE RIEULHIEUH
    ((0x9B, 0xF2), (0xC3CC)),  # HANGUL SYLLABLE SSANGSIOS YE MIEUM
    ((0x9B, 0xF3), (0xC3CD)),  # HANGUL SYLLABLE SSANGSIOS YE PIEUP
    ((0x9B, 0xF4), (0xC3CE)),  # HANGUL SYLLABLE SSANGSIOS YE PIEUPSIOS
    ((0x9B, 0xF5), (0xC3CF)),  # HANGUL SYLLABLE SSANGSIOS YE SIOS
    ((0x9B, 0xF6), (0xC3D0)),  # HANGUL SYLLABLE SSANGSIOS YE SSANGSIOS
    ((0x9B, 0xF7), (0xC3D1)),  # HANGUL SYLLABLE SSANGSIOS YE IEUNG
    ((0x9B, 0xF8), (0xC3D2)),  # HANGUL SYLLABLE SSANGSIOS YE CIEUC
    ((0x9B, 0xF9), (0xC3D3)),  # HANGUL SYLLABLE SSANGSIOS YE CHIEUCH
    ((0x9B, 0xFA), (0xC3D4)),  # HANGUL SYLLABLE SSANGSIOS YE KHIEUKH
    ((0x9B, 0xFB), (0xC3D5)),  # HANGUL SYLLABLE SSANGSIOS YE THIEUTH
    ((0x9B, 0xFC), (0xC3D6)),  # HANGUL SYLLABLE SSANGSIOS YE PHIEUPH
    ((0x9B, 0xFD), (0xC3D7)),  # HANGUL SYLLABLE SSANGSIOS YE HIEUH
    ((0x9B, 0xFE), (0xC3DA)),  # HANGUL SYLLABLE SSANGSIOS O SSANGKIYEOK
    ((0x9C, 0x41), (0xC3DB)),  # HANGUL SYLLABLE SSANGSIOS O KIYEOKSIOS
    ((0x9C, 0x42), (0xC3DD)),  # HANGUL SYLLABLE SSANGSIOS O NIEUNCIEUC
    ((0x9C, 0x43), (0xC3DE)),  # HANGUL SYLLABLE SSANGSIOS O NIEUNHIEUH
    ((0x9C, 0x44), (0xC3E1)),  # HANGUL SYLLABLE SSANGSIOS O RIEULKIYEOK
    ((0x9C, 0x45), (0xC3E3)),  # HANGUL SYLLABLE SSANGSIOS O RIEULPIEUP
    ((0x9C, 0x46), (0xC3E4)),  # HANGUL SYLLABLE SSANGSIOS O RIEULSIOS
    ((0x9C, 0x47), (0xC3E5)),  # HANGUL SYLLABLE SSANGSIOS O RIEULTHIEUTH
    ((0x9C, 0x48), (0xC3E6)),  # HANGUL SYLLABLE SSANGSIOS O RIEULPHIEUPH
    ((0x9C, 0x49), (0xC3E7)),  # HANGUL SYLLABLE SSANGSIOS O RIEULHIEUH
    ((0x9C, 0x4A), (0xC3EA)),  # HANGUL SYLLABLE SSANGSIOS O PIEUPSIOS
    ((0x9C, 0x4B), (0xC3EB)),  # HANGUL SYLLABLE SSANGSIOS O SIOS
    ((0x9C, 0x4C), (0xC3EC)),  # HANGUL SYLLABLE SSANGSIOS O SSANGSIOS
    ((0x9C, 0x4D), (0xC3EE)),  # HANGUL SYLLABLE SSANGSIOS O CIEUC
    ((0x9C, 0x4E), (0xC3EF)),  # HANGUL SYLLABLE SSANGSIOS O CHIEUCH
    ((0x9C, 0x4F), (0xC3F0)),  # HANGUL SYLLABLE SSANGSIOS O KHIEUKH
    ((0x9C, 0x50), (0xC3F1)),  # HANGUL SYLLABLE SSANGSIOS O THIEUTH
    ((0x9C, 0x51), (0xC3F2)),  # HANGUL SYLLABLE SSANGSIOS O PHIEUPH
    ((0x9C, 0x52), (0xC3F3)),  # HANGUL SYLLABLE SSANGSIOS O HIEUH
    ((0x9C, 0x53), (0xC3F6)),  # HANGUL SYLLABLE SSANGSIOS WA SSANGKIYEOK
    ((0x9C, 0x54), (0xC3F7)),  # HANGUL SYLLABLE SSANGSIOS WA KIYEOKSIOS
    ((0x9C, 0x55), (0xC3F9)),  # HANGUL SYLLABLE SSANGSIOS WA NIEUNCIEUC
    ((0x9C, 0x56), (0xC3FA)),  # HANGUL SYLLABLE SSANGSIOS WA NIEUNHIEUH
    ((0x9C, 0x57), (0xC3FB)),  # HANGUL SYLLABLE SSANGSIOS WA TIKEUT
    ((0x9C, 0x58), (0xC3FC)),  # HANGUL SYLLABLE SSANGSIOS WA RIEUL
    ((0x9C, 0x59), (0xC3FD)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULKIYEOK
    ((0x9C, 0x5A), (0xC3FE)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULMIEUM
    ((0x9C, 0x61), (0xC3FF)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULPIEUP
    ((0x9C, 0x62), (0xC400)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULSIOS
    ((0x9C, 0x63), (0xC401)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULTHIEUTH
    ((0x9C, 0x64), (0xC402)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULPHIEUPH
    ((0x9C, 0x65), (0xC403)),  # HANGUL SYLLABLE SSANGSIOS WA RIEULHIEUH
    ((0x9C, 0x66), (0xC404)),  # HANGUL SYLLABLE SSANGSIOS WA MIEUM
    ((0x9C, 0x67), (0xC405)),  # HANGUL SYLLABLE SSANGSIOS WA PIEUP
    ((0x9C, 0x68), (0xC406)),  # HANGUL SYLLABLE SSANGSIOS WA PIEUPSIOS
    ((0x9C, 0x69), (0xC407)),  # HANGUL SYLLABLE SSANGSIOS WA SIOS
    ((0x9C, 0x6A), (0xC409)),  # HANGUL SYLLABLE SSANGSIOS WA IEUNG
    ((0x9C, 0x6B), (0xC40A)),  # HANGUL SYLLABLE SSANGSIOS WA CIEUC
    ((0x9C, 0x6C), (0xC40B)),  # HANGUL SYLLABLE SSANGSIOS WA CHIEUCH
    ((0x9C, 0x6D), (0xC40C)),  # HANGUL SYLLABLE SSANGSIOS WA KHIEUKH
    ((0x9C, 0x6E), (0xC40D)),  # HANGUL SYLLABLE SSANGSIOS WA THIEUTH
    ((0x9C, 0x6F), (0xC40E)),  # HANGUL SYLLABLE SSANGSIOS WA PHIEUPH
    ((0x9C, 0x70), (0xC40F)),  # HANGUL SYLLABLE SSANGSIOS WA HIEUH
    ((0x9C, 0x71), (0xC411)),  # HANGUL SYLLABLE SSANGSIOS WAE KIYEOK
    ((0x9C, 0x72), (0xC412)),  # HANGUL SYLLABLE SSANGSIOS WAE SSANGKIYEOK
    ((0x9C, 0x73), (0xC413)),  # HANGUL SYLLABLE SSANGSIOS WAE KIYEOKSIOS
    ((0x9C, 0x74), (0xC414)),  # HANGUL SYLLABLE SSANGSIOS WAE NIEUN
    ((0x9C, 0x75), (0xC415)),  # HANGUL SYLLABLE SSANGSIOS WAE NIEUNCIEUC
    ((0x9C, 0x76), (0xC416)),  # HANGUL SYLLABLE SSANGSIOS WAE NIEUNHIEUH
    ((0x9C, 0x77), (0xC417)),  # HANGUL SYLLABLE SSANGSIOS WAE TIKEUT
    ((0x9C, 0x78), (0xC418)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEUL
    ((0x9C, 0x79), (0xC419)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULKIYEOK
    ((0x9C, 0x7A), (0xC41A)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULMIEUM
    ((0x9C, 0x81), (0xC41B)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULPIEUP
    ((0x9C, 0x82), (0xC41C)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULSIOS
    ((0x9C, 0x83), (0xC41D)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULTHIEUTH
    ((0x9C, 0x84), (0xC41E)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULPHIEUPH
    ((0x9C, 0x85), (0xC41F)),  # HANGUL SYLLABLE SSANGSIOS WAE RIEULHIEUH
    ((0x9C, 0x86), (0xC420)),  # HANGUL SYLLABLE SSANGSIOS WAE MIEUM
    ((0x9C, 0x87), (0xC421)),  # HANGUL SYLLABLE SSANGSIOS WAE PIEUP
    ((0x9C, 0x88), (0xC422)),  # HANGUL SYLLABLE SSANGSIOS WAE PIEUPSIOS
    ((0x9C, 0x89), (0xC423)),  # HANGUL SYLLABLE SSANGSIOS WAE SIOS
    ((0x9C, 0x8A), (0xC425)),  # HANGUL SYLLABLE SSANGSIOS WAE IEUNG
    ((0x9C, 0x8B), (0xC426)),  # HANGUL SYLLABLE SSANGSIOS WAE CIEUC
    ((0x9C, 0x8C), (0xC427)),  # HANGUL SYLLABLE SSANGSIOS WAE CHIEUCH
    ((0x9C, 0x8D), (0xC428)),  # HANGUL SYLLABLE SSANGSIOS WAE KHIEUKH
    ((0x9C, 0x8E), (0xC429)),  # HANGUL SYLLABLE SSANGSIOS WAE THIEUTH
    ((0x9C, 0x8F), (0xC42A)),  # HANGUL SYLLABLE SSANGSIOS WAE PHIEUPH
    ((0x9C, 0x90), (0xC42B)),  # HANGUL SYLLABLE SSANGSIOS WAE HIEUH
    ((0x9C, 0x91), (0xC42D)),  # HANGUL SYLLABLE SSANGSIOS OE KIYEOK
    ((0x9C, 0x92), (0xC42E)),  # HANGUL SYLLABLE SSANGSIOS OE SSANGKIYEOK
    ((0x9C, 0x93), (0xC42F)),  # HANGUL SYLLABLE SSANGSIOS OE KIYEOKSIOS
    ((0x9C, 0x94), (0xC431)),  # HANGUL SYLLABLE SSANGSIOS OE NIEUNCIEUC
    ((0x9C, 0x95), (0xC432)),  # HANGUL SYLLABLE SSANGSIOS OE NIEUNHIEUH
    ((0x9C, 0x96), (0xC433)),  # HANGUL SYLLABLE SSANGSIOS OE TIKEUT
    ((0x9C, 0x97), (0xC435)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULKIYEOK
    ((0x9C, 0x98), (0xC436)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULMIEUM
    ((0x9C, 0x99), (0xC437)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULPIEUP
    ((0x9C, 0x9A), (0xC438)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULSIOS
    ((0x9C, 0x9B), (0xC439)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULTHIEUTH
    ((0x9C, 0x9C), (0xC43A)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULPHIEUPH
    ((0x9C, 0x9D), (0xC43B)),  # HANGUL SYLLABLE SSANGSIOS OE RIEULHIEUH
    ((0x9C, 0x9E), (0xC43E)),  # HANGUL SYLLABLE SSANGSIOS OE PIEUPSIOS
    ((0x9C, 0x9F), (0xC43F)),  # HANGUL SYLLABLE SSANGSIOS OE SIOS
    ((0x9C, 0xA0), (0xC440)),  # HANGUL SYLLABLE SSANGSIOS OE SSANGSIOS
    ((0x9C, 0xA1), (0xC441)),  # HANGUL SYLLABLE SSANGSIOS OE IEUNG
    ((0x9C, 0xA2), (0xC442)),  # HANGUL SYLLABLE SSANGSIOS OE CIEUC
    ((0x9C, 0xA3), (0xC443)),  # HANGUL SYLLABLE SSANGSIOS OE CHIEUCH
    ((0x9C, 0xA4), (0xC444)),  # HANGUL SYLLABLE SSANGSIOS OE KHIEUKH
    ((0x9C, 0xA5), (0xC445)),  # HANGUL SYLLABLE SSANGSIOS OE THIEUTH
    ((0x9C, 0xA6), (0xC446)),  # HANGUL SYLLABLE SSANGSIOS OE PHIEUPH
    ((0x9C, 0xA7), (0xC447)),  # HANGUL SYLLABLE SSANGSIOS OE HIEUH
    ((0x9C, 0xA8), (0xC449)),  # HANGUL SYLLABLE SSANGSIOS YO KIYEOK
    ((0x9C, 0xA9), (0xC44A)),  # HANGUL SYLLABLE SSANGSIOS YO SSANGKIYEOK
    ((0x9C, 0xAA), (0xC44B)),  # HANGUL SYLLABLE SSANGSIOS YO KIYEOKSIOS
    ((0x9C, 0xAB), (0xC44C)),  # HANGUL SYLLABLE SSANGSIOS YO NIEUN
    ((0x9C, 0xAC), (0xC44D)),  # HANGUL SYLLABLE SSANGSIOS YO NIEUNCIEUC
    ((0x9C, 0xAD), (0xC44E)),  # HANGUL SYLLABLE SSANGSIOS YO NIEUNHIEUH
    ((0x9C, 0xAE), (0xC44F)),  # HANGUL SYLLABLE SSANGSIOS YO TIKEUT
    ((0x9C, 0xAF), (0xC450)),  # HANGUL SYLLABLE SSANGSIOS YO RIEUL
    ((0x9C, 0xB0), (0xC451)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULKIYEOK
    ((0x9C, 0xB1), (0xC452)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULMIEUM
    ((0x9C, 0xB2), (0xC453)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULPIEUP
    ((0x9C, 0xB3), (0xC454)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULSIOS
    ((0x9C, 0xB4), (0xC455)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULTHIEUTH
    ((0x9C, 0xB5), (0xC456)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULPHIEUPH
    ((0x9C, 0xB6), (0xC457)),  # HANGUL SYLLABLE SSANGSIOS YO RIEULHIEUH
    ((0x9C, 0xB7), (0xC458)),  # HANGUL SYLLABLE SSANGSIOS YO MIEUM
    ((0x9C, 0xB8), (0xC459)),  # HANGUL SYLLABLE SSANGSIOS YO PIEUP
    ((0x9C, 0xB9), (0xC45A)),  # HANGUL SYLLABLE SSANGSIOS YO PIEUPSIOS
    ((0x9C, 0xBA), (0xC45B)),  # HANGUL SYLLABLE SSANGSIOS YO SIOS
    ((0x9C, 0xBB), (0xC45C)),  # HANGUL SYLLABLE SSANGSIOS YO SSANGSIOS
    ((0x9C, 0xBC), (0xC45D)),  # HANGUL SYLLABLE SSANGSIOS YO IEUNG
    ((0x9C, 0xBD), (0xC45E)),  # HANGUL SYLLABLE SSANGSIOS YO CIEUC
    ((0x9C, 0xBE), (0xC45F)),  # HANGUL SYLLABLE SSANGSIOS YO CHIEUCH
    ((0x9C, 0xBF), (0xC460)),  # HANGUL SYLLABLE SSANGSIOS YO KHIEUKH
    ((0x9C, 0xC0), (0xC461)),  # HANGUL SYLLABLE SSANGSIOS YO THIEUTH
    ((0x9C, 0xC1), (0xC462)),  # HANGUL SYLLABLE SSANGSIOS YO PHIEUPH
    ((0x9C, 0xC2), (0xC463)),  # HANGUL SYLLABLE SSANGSIOS YO HIEUH
    ((0x9C, 0xC3), (0xC466)),  # HANGUL SYLLABLE SSANGSIOS U SSANGKIYEOK
    ((0x9C, 0xC4), (0xC467)),  # HANGUL SYLLABLE SSANGSIOS U KIYEOKSIOS
    ((0x9C, 0xC5), (0xC469)),  # HANGUL SYLLABLE SSANGSIOS U NIEUNCIEUC
    ((0x9C, 0xC6), (0xC46A)),  # HANGUL SYLLABLE SSANGSIOS U NIEUNHIEUH
    ((0x9C, 0xC7), (0xC46B)),  # HANGUL SYLLABLE SSANGSIOS U TIKEUT
    ((0x9C, 0xC8), (0xC46D)),  # HANGUL SYLLABLE SSANGSIOS U RIEULKIYEOK
    ((0x9C, 0xC9), (0xC46E)),  # HANGUL SYLLABLE SSANGSIOS U RIEULMIEUM
    ((0x9C, 0xCA), (0xC46F)),  # HANGUL SYLLABLE SSANGSIOS U RIEULPIEUP
    ((0x9C, 0xCB), (0xC470)),  # HANGUL SYLLABLE SSANGSIOS U RIEULSIOS
    ((0x9C, 0xCC), (0xC471)),  # HANGUL SYLLABLE SSANGSIOS U RIEULTHIEUTH
    ((0x9C, 0xCD), (0xC472)),  # HANGUL SYLLABLE SSANGSIOS U RIEULPHIEUPH
    ((0x9C, 0xCE), (0xC473)),  # HANGUL SYLLABLE SSANGSIOS U RIEULHIEUH
    ((0x9C, 0xCF), (0xC476)),  # HANGUL SYLLABLE SSANGSIOS U PIEUPSIOS
    ((0x9C, 0xD0), (0xC477)),  # HANGUL SYLLABLE SSANGSIOS U SIOS
    ((0x9C, 0xD1), (0xC478)),  # HANGUL SYLLABLE SSANGSIOS U SSANGSIOS
    ((0x9C, 0xD2), (0xC47A)),  # HANGUL SYLLABLE SSANGSIOS U CIEUC
    ((0x9C, 0xD3), (0xC47B)),  # HANGUL SYLLABLE SSANGSIOS U CHIEUCH
    ((0x9C, 0xD4), (0xC47C)),  # HANGUL SYLLABLE SSANGSIOS U KHIEUKH
    ((0x9C, 0xD5), (0xC47D)),  # HANGUL SYLLABLE SSANGSIOS U THIEUTH
    ((0x9C, 0xD6), (0xC47E)),  # HANGUL SYLLABLE SSANGSIOS U PHIEUPH
    ((0x9C, 0xD7), (0xC47F)),  # HANGUL SYLLABLE SSANGSIOS U HIEUH
    ((0x9C, 0xD8), (0xC481)),  # HANGUL SYLLABLE SSANGSIOS WEO KIYEOK
    ((0x9C, 0xD9), (0xC482)),  # HANGUL SYLLABLE SSANGSIOS WEO SSANGKIYEOK
    ((0x9C, 0xDA), (0xC483)),  # HANGUL SYLLABLE SSANGSIOS WEO KIYEOKSIOS
    ((0x9C, 0xDB), (0xC484)),  # HANGUL SYLLABLE SSANGSIOS WEO NIEUN
    ((0x9C, 0xDC), (0xC485)),  # HANGUL SYLLABLE SSANGSIOS WEO NIEUNCIEUC
    ((0x9C, 0xDD), (0xC486)),  # HANGUL SYLLABLE SSANGSIOS WEO NIEUNHIEUH
    ((0x9C, 0xDE), (0xC487)),  # HANGUL SYLLABLE SSANGSIOS WEO TIKEUT
    ((0x9C, 0xDF), (0xC488)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEUL
    ((0x9C, 0xE0), (0xC489)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULKIYEOK
    ((0x9C, 0xE1), (0xC48A)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULMIEUM
    ((0x9C, 0xE2), (0xC48B)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULPIEUP
    ((0x9C, 0xE3), (0xC48C)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULSIOS
    ((0x9C, 0xE4), (0xC48D)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULTHIEUTH
    ((0x9C, 0xE5), (0xC48E)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULPHIEUPH
    ((0x9C, 0xE6), (0xC48F)),  # HANGUL SYLLABLE SSANGSIOS WEO RIEULHIEUH
    ((0x9C, 0xE7), (0xC490)),  # HANGUL SYLLABLE SSANGSIOS WEO MIEUM
    ((0x9C, 0xE8), (0xC491)),  # HANGUL SYLLABLE SSANGSIOS WEO PIEUP
    ((0x9C, 0xE9), (0xC492)),  # HANGUL SYLLABLE SSANGSIOS WEO PIEUPSIOS
    ((0x9C, 0xEA), (0xC493)),  # HANGUL SYLLABLE SSANGSIOS WEO SIOS
    ((0x9C, 0xEB), (0xC495)),  # HANGUL SYLLABLE SSANGSIOS WEO IEUNG
    ((0x9C, 0xEC), (0xC496)),  # HANGUL SYLLABLE SSANGSIOS WEO CIEUC
    ((0x9C, 0xED), (0xC497)),  # HANGUL SYLLABLE SSANGSIOS WEO CHIEUCH
    ((0x9C, 0xEE), (0xC498)),  # HANGUL SYLLABLE SSANGSIOS WEO KHIEUKH
    ((0x9C, 0xEF), (0xC499)),  # HANGUL SYLLABLE SSANGSIOS WEO THIEUTH
    ((0x9C, 0xF0), (0xC49A)),  # HANGUL SYLLABLE SSANGSIOS WEO PHIEUPH
    ((0x9C, 0xF1), (0xC49B)),  # HANGUL SYLLABLE SSANGSIOS WEO HIEUH
    ((0x9C, 0xF2), (0xC49D)),  # HANGUL SYLLABLE SSANGSIOS WE KIYEOK
    ((0x9C, 0xF3), (0xC49E)),  # HANGUL SYLLABLE SSANGSIOS WE SSANGKIYEOK
    ((0x9C, 0xF4), (0xC49F)),  # HANGUL SYLLABLE SSANGSIOS WE KIYEOKSIOS
    ((0x9C, 0xF5), (0xC4A0)),  # HANGUL SYLLABLE SSANGSIOS WE NIEUN
    ((0x9C, 0xF6), (0xC4A1)),  # HANGUL SYLLABLE SSANGSIOS WE NIEUNCIEUC
    ((0x9C, 0xF7), (0xC4A2)),  # HANGUL SYLLABLE SSANGSIOS WE NIEUNHIEUH
    ((0x9C, 0xF8), (0xC4A3)),  # HANGUL SYLLABLE SSANGSIOS WE TIKEUT
    ((0x9C, 0xF9), (0xC4A4)),  # HANGUL SYLLABLE SSANGSIOS WE RIEUL
    ((0x9C, 0xFA), (0xC4A5)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULKIYEOK
    ((0x9C, 0xFB), (0xC4A6)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULMIEUM
    ((0x9C, 0xFC), (0xC4A7)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULPIEUP
    ((0x9C, 0xFD), (0xC4A8)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULSIOS
    ((0x9C, 0xFE), (0xC4A9)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULTHIEUTH
    ((0x9D, 0x41), (0xC4AA)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULPHIEUPH
    ((0x9D, 0x42), (0xC4AB)),  # HANGUL SYLLABLE SSANGSIOS WE RIEULHIEUH
    ((0x9D, 0x43), (0xC4AC)),  # HANGUL SYLLABLE SSANGSIOS WE MIEUM
    ((0x9D, 0x44), (0xC4AD)),  # HANGUL SYLLABLE SSANGSIOS WE PIEUP
    ((0x9D, 0x45), (0xC4AE)),  # HANGUL SYLLABLE SSANGSIOS WE PIEUPSIOS
    ((0x9D, 0x46), (0xC4AF)),  # HANGUL SYLLABLE SSANGSIOS WE SIOS
    ((0x9D, 0x47), (0xC4B0)),  # HANGUL SYLLABLE SSANGSIOS WE SSANGSIOS
    ((0x9D, 0x48), (0xC4B1)),  # HANGUL SYLLABLE SSANGSIOS WE IEUNG
    ((0x9D, 0x49), (0xC4B2)),  # HANGUL SYLLABLE SSANGSIOS WE CIEUC
    ((0x9D, 0x4A), (0xC4B3)),  # HANGUL SYLLABLE SSANGSIOS WE CHIEUCH
    ((0x9D, 0x4B), (0xC4B4)),  # HANGUL SYLLABLE SSANGSIOS WE KHIEUKH
    ((0x9D, 0x4C), (0xC4B5)),  # HANGUL SYLLABLE SSANGSIOS WE THIEUTH
    ((0x9D, 0x4D), (0xC4B6)),  # HANGUL SYLLABLE SSANGSIOS WE PHIEUPH
    ((0x9D, 0x4E), (0xC4B7)),  # HANGUL SYLLABLE SSANGSIOS WE HIEUH
    ((0x9D, 0x4F), (0xC4B9)),  # HANGUL SYLLABLE SSANGSIOS WI KIYEOK
    ((0x9D, 0x50), (0xC4BA)),  # HANGUL SYLLABLE SSANGSIOS WI SSANGKIYEOK
    ((0x9D, 0x51), (0xC4BB)),  # HANGUL SYLLABLE SSANGSIOS WI KIYEOKSIOS
    ((0x9D, 0x52), (0xC4BD)),  # HANGUL SYLLABLE SSANGSIOS WI NIEUNCIEUC
    ((0x9D, 0x53), (0xC4BE)),  # HANGUL SYLLABLE SSANGSIOS WI NIEUNHIEUH
    ((0x9D, 0x54), (0xC4BF)),  # HANGUL SYLLABLE SSANGSIOS WI TIKEUT
    ((0x9D, 0x55), (0xC4C0)),  # HANGUL SYLLABLE SSANGSIOS WI RIEUL
    ((0x9D, 0x56), (0xC4C1)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULKIYEOK
    ((0x9D, 0x57), (0xC4C2)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULMIEUM
    ((0x9D, 0x58), (0xC4C3)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULPIEUP
    ((0x9D, 0x59), (0xC4C4)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULSIOS
    ((0x9D, 0x5A), (0xC4C5)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULTHIEUTH
    ((0x9D, 0x61), (0xC4C6)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULPHIEUPH
    ((0x9D, 0x62), (0xC4C7)),  # HANGUL SYLLABLE SSANGSIOS WI RIEULHIEUH
    ((0x9D, 0x63), (0xC4C8)),  # HANGUL SYLLABLE SSANGSIOS WI MIEUM
    ((0x9D, 0x64), (0xC4C9)),  # HANGUL SYLLABLE SSANGSIOS WI PIEUP
    ((0x9D, 0x65), (0xC4CA)),  # HANGUL SYLLABLE SSANGSIOS WI PIEUPSIOS
    ((0x9D, 0x66), (0xC4CB)),  # HANGUL SYLLABLE SSANGSIOS WI SIOS
    ((0x9D, 0x67), (0xC4CC)),  # HANGUL SYLLABLE SSANGSIOS WI SSANGSIOS
    ((0x9D, 0x68), (0xC4CD)),  # HANGUL SYLLABLE SSANGSIOS WI IEUNG
    ((0x9D, 0x69), (0xC4CE)),  # HANGUL SYLLABLE SSANGSIOS WI CIEUC
    ((0x9D, 0x6A), (0xC4CF)),  # HANGUL SYLLABLE SSANGSIOS WI CHIEUCH
    ((0x9D, 0x6B), (0xC4D0)),  # HANGUL SYLLABLE SSANGSIOS WI KHIEUKH
    ((0x9D, 0x6C), (0xC4D1)),  # HANGUL SYLLABLE SSANGSIOS WI THIEUTH
    ((0x9D, 0x6D), (0xC4D2)),  # HANGUL SYLLABLE SSANGSIOS WI PHIEUPH
    ((0x9D, 0x6E), (0xC4D3)),  # HANGUL SYLLABLE SSANGSIOS WI HIEUH
    ((0x9D, 0x6F), (0xC4D4)),  # HANGUL SYLLABLE SSANGSIOS YU
    ((0x9D, 0x70), (0xC4D5)),  # HANGUL SYLLABLE SSANGSIOS YU KIYEOK
    ((0x9D, 0x71), (0xC4D6)),  # HANGUL SYLLABLE SSANGSIOS YU SSANGKIYEOK
    ((0x9D, 0x72), (0xC4D7)),  # HANGUL SYLLABLE SSANGSIOS YU KIYEOKSIOS
    ((0x9D, 0x73), (0xC4D8)),  # HANGUL SYLLABLE SSANGSIOS YU NIEUN
    ((0x9D, 0x74), (0xC4D9)),  # HANGUL SYLLABLE SSANGSIOS YU NIEUNCIEUC
    ((0x9D, 0x75), (0xC4DA)),  # HANGUL SYLLABLE SSANGSIOS YU NIEUNHIEUH
    ((0x9D, 0x76), (0xC4DB)),  # HANGUL SYLLABLE SSANGSIOS YU TIKEUT
    ((0x9D, 0x77), (0xC4DC)),  # HANGUL SYLLABLE SSANGSIOS YU RIEUL
    ((0x9D, 0x78), (0xC4DD)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULKIYEOK
    ((0x9D, 0x79), (0xC4DE)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULMIEUM
    ((0x9D, 0x7A), (0xC4DF)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULPIEUP
    ((0x9D, 0x81), (0xC4E0)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULSIOS
    ((0x9D, 0x82), (0xC4E1)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULTHIEUTH
    ((0x9D, 0x83), (0xC4E2)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULPHIEUPH
    ((0x9D, 0x84), (0xC4E3)),  # HANGUL SYLLABLE SSANGSIOS YU RIEULHIEUH
    ((0x9D, 0x85), (0xC4E4)),  # HANGUL SYLLABLE SSANGSIOS YU MIEUM
    ((0x9D, 0x86), (0xC4E5)),  # HANGUL SYLLABLE SSANGSIOS YU PIEUP
    ((0x9D, 0x87), (0xC4E6)),  # HANGUL SYLLABLE SSANGSIOS YU PIEUPSIOS
    ((0x9D, 0x88), (0xC4E7)),  # HANGUL SYLLABLE SSANGSIOS YU SIOS
    ((0x9D, 0x89), (0xC4E8)),  # HANGUL SYLLABLE SSANGSIOS YU SSANGSIOS
    ((0x9D, 0x8A), (0xC4EA)),  # HANGUL SYLLABLE SSANGSIOS YU CIEUC
    ((0x9D, 0x8B), (0xC4EB)),  # HANGUL SYLLABLE SSANGSIOS YU CHIEUCH
    ((0x9D, 0x8C), (0xC4EC)),  # HANGUL SYLLABLE SSANGSIOS YU KHIEUKH
    ((0x9D, 0x8D), (0xC4ED)),  # HANGUL SYLLABLE SSANGSIOS YU THIEUTH
    ((0x9D, 0x8E), (0xC4EE)),  # HANGUL SYLLABLE SSANGSIOS YU PHIEUPH
    ((0x9D, 0x8F), (0xC4EF)),  # HANGUL SYLLABLE SSANGSIOS YU HIEUH
    ((0x9D, 0x90), (0xC4F2)),  # HANGUL SYLLABLE SSANGSIOS EU SSANGKIYEOK
    ((0x9D, 0x91), (0xC4F3)),  # HANGUL SYLLABLE SSANGSIOS EU KIYEOKSIOS
    ((0x9D, 0x92), (0xC4F5)),  # HANGUL SYLLABLE SSANGSIOS EU NIEUNCIEUC
    ((0x9D, 0x93), (0xC4F6)),  # HANGUL SYLLABLE SSANGSIOS EU NIEUNHIEUH
    ((0x9D, 0x94), (0xC4F7)),  # HANGUL SYLLABLE SSANGSIOS EU TIKEUT
    ((0x9D, 0x95), (0xC4F9)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULKIYEOK
    ((0x9D, 0x96), (0xC4FB)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULPIEUP
    ((0x9D, 0x97), (0xC4FC)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULSIOS
    ((0x9D, 0x98), (0xC4FD)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULTHIEUTH
    ((0x9D, 0x99), (0xC4FE)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULPHIEUPH
    ((0x9D, 0x9A), (0xC502)),  # HANGUL SYLLABLE SSANGSIOS EU PIEUPSIOS
    ((0x9D, 0x9B), (0xC503)),  # HANGUL SYLLABLE SSANGSIOS EU SIOS
    ((0x9D, 0x9C), (0xC504)),  # HANGUL SYLLABLE SSANGSIOS EU SSANGSIOS
    ((0x9D, 0x9D), (0xC505)),  # HANGUL SYLLABLE SSANGSIOS EU IEUNG
    ((0x9D, 0x9E), (0xC506)),  # HANGUL SYLLABLE SSANGSIOS EU CIEUC
    ((0x9D, 0x9F), (0xC507)),  # HANGUL SYLLABLE SSANGSIOS EU CHIEUCH
    ((0x9D, 0xA0), (0xC508)),  # HANGUL SYLLABLE SSANGSIOS EU KHIEUKH
    ((0x9D, 0xA1), (0xC509)),  # HANGUL SYLLABLE SSANGSIOS EU THIEUTH
    ((0x9D, 0xA2), (0xC50A)),  # HANGUL SYLLABLE SSANGSIOS EU PHIEUPH
    ((0x9D, 0xA3), (0xC50B)),  # HANGUL SYLLABLE SSANGSIOS EU HIEUH
    ((0x9D, 0xA4), (0xC50D)),  # HANGUL SYLLABLE SSANGSIOS YI KIYEOK
    ((0x9D, 0xA5), (0xC50E)),  # HANGUL SYLLABLE SSANGSIOS YI SSANGKIYEOK
    ((0x9D, 0xA6), (0xC50F)),  # HANGUL SYLLABLE SSANGSIOS YI KIYEOKSIOS
    ((0x9D, 0xA7), (0xC511)),  # HANGUL SYLLABLE SSANGSIOS YI NIEUNCIEUC
    ((0x9D, 0xA8), (0xC512)),  # HANGUL SYLLABLE SSANGSIOS YI NIEUNHIEUH
    ((0x9D, 0xA9), (0xC513)),  # HANGUL SYLLABLE SSANGSIOS YI TIKEUT
    ((0x9D, 0xAA), (0xC515)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULKIYEOK
    ((0x9D, 0xAB), (0xC516)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULMIEUM
    ((0x9D, 0xAC), (0xC517)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULPIEUP
    ((0x9D, 0xAD), (0xC518)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULSIOS
    ((0x9D, 0xAE), (0xC519)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULTHIEUTH
    ((0x9D, 0xAF), (0xC51A)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULPHIEUPH
    ((0x9D, 0xB0), (0xC51B)),  # HANGUL SYLLABLE SSANGSIOS YI RIEULHIEUH
    ((0x9D, 0xB1), (0xC51D)),  # HANGUL SYLLABLE SSANGSIOS YI PIEUP
    ((0x9D, 0xB2), (0xC51E)),  # HANGUL SYLLABLE SSANGSIOS YI PIEUPSIOS
    ((0x9D, 0xB3), (0xC51F)),  # HANGUL SYLLABLE SSANGSIOS YI SIOS
    ((0x9D, 0xB4), (0xC520)),  # HANGUL SYLLABLE SSANGSIOS YI SSANGSIOS
    ((0x9D, 0xB5), (0xC521)),  # HANGUL SYLLABLE SSANGSIOS YI IEUNG
    ((0x9D, 0xB6), (0xC522)),  # HANGUL SYLLABLE SSANGSIOS YI CIEUC
    ((0x9D, 0xB7), (0xC523)),  # HANGUL SYLLABLE SSANGSIOS YI CHIEUCH
    ((0x9D, 0xB8), (0xC524)),  # HANGUL SYLLABLE SSANGSIOS YI KHIEUKH
    ((0x9D, 0xB9), (0xC525)),  # HANGUL SYLLABLE SSANGSIOS YI THIEUTH
    ((0x9D, 0xBA), (0xC526)),  # HANGUL SYLLABLE SSANGSIOS YI PHIEUPH
    ((0x9D, 0xBB), (0xC527)),  # HANGUL SYLLABLE SSANGSIOS YI HIEUH
    ((0x9D, 0xBC), (0xC52A)),  # HANGUL SYLLABLE SSANGSIOS I SSANGKIYEOK
    ((0x9D, 0xBD), (0xC52B)),  # HANGUL SYLLABLE SSANGSIOS I KIYEOKSIOS
    ((0x9D, 0xBE), (0xC52D)),  # HANGUL SYLLABLE SSANGSIOS I NIEUNCIEUC
    ((0x9D, 0xBF), (0xC52E)),  # HANGUL SYLLABLE SSANGSIOS I NIEUNHIEUH
    ((0x9D, 0xC0), (0xC52F)),  # HANGUL SYLLABLE SSANGSIOS I TIKEUT
    ((0x9D, 0xC1), (0xC531)),  # HANGUL SYLLABLE SSANGSIOS I RIEULKIYEOK
    ((0x9D, 0xC2), (0xC532)),  # HANGUL SYLLABLE SSANGSIOS I RIEULMIEUM
    ((0x9D, 0xC3), (0xC533)),  # HANGUL SYLLABLE SSANGSIOS I RIEULPIEUP
    ((0x9D, 0xC4), (0xC534)),  # HANGUL SYLLABLE SSANGSIOS I RIEULSIOS
    ((0x9D, 0xC5), (0xC535)),  # HANGUL SYLLABLE SSANGSIOS I RIEULTHIEUTH
    ((0x9D, 0xC6), (0xC536)),  # HANGUL SYLLABLE SSANGSIOS I RIEULPHIEUPH
    ((0x9D, 0xC7), (0xC537)),  # HANGUL SYLLABLE SSANGSIOS I RIEULHIEUH
    ((0x9D, 0xC8), (0xC53A)),  # HANGUL SYLLABLE SSANGSIOS I PIEUPSIOS
    ((0x9D, 0xC9), (0xC53C)),  # HANGUL SYLLABLE SSANGSIOS I SSANGSIOS
    ((0x9D, 0xCA), (0xC53E)),  # HANGUL SYLLABLE SSANGSIOS I CIEUC
    ((0x9D, 0xCB), (0xC53F)),  # HANGUL SYLLABLE SSANGSIOS I CHIEUCH
    ((0x9D, 0xCC), (0xC540)),  # HANGUL SYLLABLE SSANGSIOS I KHIEUKH
    ((0x9D, 0xCD), (0xC541)),  # HANGUL SYLLABLE SSANGSIOS I THIEUTH
    ((0x9D, 0xCE), (0xC542)),  # HANGUL SYLLABLE SSANGSIOS I PHIEUPH
    ((0x9D, 0xCF), (0xC543)),  # HANGUL SYLLABLE SSANGSIOS I HIEUH
    ((0x9D, 0xD0), (0xC546)),  # HANGUL SYLLABLE IEUNG A SSANGKIYEOK
    ((0x9D, 0xD1), (0xC547)),  # HANGUL SYLLABLE IEUNG A KIYEOKSIOS
    ((0x9D, 0xD2), (0xC54B)),  # HANGUL SYLLABLE IEUNG A TIKEUT
    ((0x9D, 0xD3), (0xC54F)),  # HANGUL SYLLABLE IEUNG A RIEULPIEUP
    ((0x9D, 0xD4), (0xC550)),  # HANGUL SYLLABLE IEUNG A RIEULSIOS
    ((0x9D, 0xD5), (0xC551)),  # HANGUL SYLLABLE IEUNG A RIEULTHIEUTH
    ((0x9D, 0xD6), (0xC552)),  # HANGUL SYLLABLE IEUNG A RIEULPHIEUPH
    ((0x9D, 0xD7), (0xC556)),  # HANGUL SYLLABLE IEUNG A PIEUPSIOS
    ((0x9D, 0xD8), (0xC55A)),  # HANGUL SYLLABLE IEUNG A CIEUC
    ((0x9D, 0xD9), (0xC55B)),  # HANGUL SYLLABLE IEUNG A CHIEUCH
    ((0x9D, 0xDA), (0xC55C)),  # HANGUL SYLLABLE IEUNG A KHIEUKH
    ((0x9D, 0xDB), (0xC55F)),  # HANGUL SYLLABLE IEUNG A HIEUH
    ((0x9D, 0xDC), (0xC562)),  # HANGUL SYLLABLE IEUNG AE SSANGKIYEOK
    ((0x9D, 0xDD), (0xC563)),  # HANGUL SYLLABLE IEUNG AE KIYEOKSIOS
    ((0x9D, 0xDE), (0xC565)),  # HANGUL SYLLABLE IEUNG AE NIEUNCIEUC
    ((0x9D, 0xDF), (0xC566)),  # HANGUL SYLLABLE IEUNG AE NIEUNHIEUH
    ((0x9D, 0xE0), (0xC567)),  # HANGUL SYLLABLE IEUNG AE TIKEUT
    ((0x9D, 0xE1), (0xC569)),  # HANGUL SYLLABLE IEUNG AE RIEULKIYEOK
    ((0x9D, 0xE2), (0xC56A)),  # HANGUL SYLLABLE IEUNG AE RIEULMIEUM
    ((0x9D, 0xE3), (0xC56B)),  # HANGUL SYLLABLE IEUNG AE RIEULPIEUP
    ((0x9D, 0xE4), (0xC56C)),  # HANGUL SYLLABLE IEUNG AE RIEULSIOS
    ((0x9D, 0xE5), (0xC56D)),  # HANGUL SYLLABLE IEUNG AE RIEULTHIEUTH
    ((0x9D, 0xE6), (0xC56E)),  # HANGUL SYLLABLE IEUNG AE RIEULPHIEUPH
    ((0x9D, 0xE7), (0xC56F)),  # HANGUL SYLLABLE IEUNG AE RIEULHIEUH
    ((0x9D, 0xE8), (0xC572)),  # HANGUL SYLLABLE IEUNG AE PIEUPSIOS
    ((0x9D, 0xE9), (0xC576)),  # HANGUL SYLLABLE IEUNG AE CIEUC
    ((0x9D, 0xEA), (0xC577)),  # HANGUL SYLLABLE IEUNG AE CHIEUCH
    ((0x9D, 0xEB), (0xC578)),  # HANGUL SYLLABLE IEUNG AE KHIEUKH
    ((0x9D, 0xEC), (0xC579)),  # HANGUL SYLLABLE IEUNG AE THIEUTH
    ((0x9D, 0xED), (0xC57A)),  # HANGUL SYLLABLE IEUNG AE PHIEUPH
    ((0x9D, 0xEE), (0xC57B)),  # HANGUL SYLLABLE IEUNG AE HIEUH
    ((0x9D, 0xEF), (0xC57E)),  # HANGUL SYLLABLE IEUNG YA SSANGKIYEOK
    ((0x9D, 0xF0), (0xC57F)),  # HANGUL SYLLABLE IEUNG YA KIYEOKSIOS
    ((0x9D, 0xF1), (0xC581)),  # HANGUL SYLLABLE IEUNG YA NIEUNCIEUC
    ((0x9D, 0xF2), (0xC582)),  # HANGUL SYLLABLE IEUNG YA NIEUNHIEUH
    ((0x9D, 0xF3), (0xC583)),  # HANGUL SYLLABLE IEUNG YA TIKEUT
    ((0x9D, 0xF4), (0xC585)),  # HANGUL SYLLABLE IEUNG YA RIEULKIYEOK
    ((0x9D, 0xF5), (0xC586)),  # HANGUL SYLLABLE IEUNG YA RIEULMIEUM
    ((0x9D, 0xF6), (0xC588)),  # HANGUL SYLLABLE IEUNG YA RIEULSIOS
    ((0x9D, 0xF7), (0xC589)),  # HANGUL SYLLABLE IEUNG YA RIEULTHIEUTH
    ((0x9D, 0xF8), (0xC58A)),  # HANGUL SYLLABLE IEUNG YA RIEULPHIEUPH
    ((0x9D, 0xF9), (0xC58B)),  # HANGUL SYLLABLE IEUNG YA RIEULHIEUH
    ((0x9D, 0xFA), (0xC58E)),  # HANGUL SYLLABLE IEUNG YA PIEUPSIOS
    ((0x9D, 0xFB), (0xC590)),  # HANGUL SYLLABLE IEUNG YA SSANGSIOS
    ((0x9D, 0xFC), (0xC592)),  # HANGUL SYLLABLE IEUNG YA CIEUC
    ((0x9D, 0xFD), (0xC593)),  # HANGUL SYLLABLE IEUNG YA CHIEUCH
    ((0x9D, 0xFE), (0xC594)),  # HANGUL SYLLABLE IEUNG YA KHIEUKH
    ((0x9E, 0x41), (0xC596)),  # HANGUL SYLLABLE IEUNG YA PHIEUPH
    ((0x9E, 0x42), (0xC599)),  # HANGUL SYLLABLE IEUNG YAE KIYEOK
    ((0x9E, 0x43), (0xC59A)),  # HANGUL SYLLABLE IEUNG YAE SSANGKIYEOK
    ((0x9E, 0x44), (0xC59B)),  # HANGUL SYLLABLE IEUNG YAE KIYEOKSIOS
    ((0x9E, 0x45), (0xC59D)),  # HANGUL SYLLABLE IEUNG YAE NIEUNCIEUC
    ((0x9E, 0x46), (0xC59E)),  # HANGUL SYLLABLE IEUNG YAE NIEUNHIEUH
    ((0x9E, 0x47), (0xC59F)),  # HANGUL SYLLABLE IEUNG YAE TIKEUT
    ((0x9E, 0x48), (0xC5A1)),  # HANGUL SYLLABLE IEUNG YAE RIEULKIYEOK
    ((0x9E, 0x49), (0xC5A2)),  # HANGUL SYLLABLE IEUNG YAE RIEULMIEUM
    ((0x9E, 0x4A), (0xC5A3)),  # HANGUL SYLLABLE IEUNG YAE RIEULPIEUP
    ((0x9E, 0x4B), (0xC5A4)),  # HANGUL SYLLABLE IEUNG YAE RIEULSIOS
    ((0x9E, 0x4C), (0xC5A5)),  # HANGUL SYLLABLE IEUNG YAE RIEULTHIEUTH
    ((0x9E, 0x4D), (0xC5A6)),  # HANGUL SYLLABLE IEUNG YAE RIEULPHIEUPH
    ((0x9E, 0x4E), (0xC5A7)),  # HANGUL SYLLABLE IEUNG YAE RIEULHIEUH
    ((0x9E, 0x4F), (0xC5A8)),  # HANGUL SYLLABLE IEUNG YAE MIEUM
    ((0x9E, 0x50), (0xC5AA)),  # HANGUL SYLLABLE IEUNG YAE PIEUPSIOS
    ((0x9E, 0x51), (0xC5AB)),  # HANGUL SYLLABLE IEUNG YAE SIOS
    ((0x9E, 0x52), (0xC5AC)),  # HANGUL SYLLABLE IEUNG YAE SSANGSIOS
    ((0x9E, 0x53), (0xC5AD)),  # HANGUL SYLLABLE IEUNG YAE IEUNG
    ((0x9E, 0x54), (0xC5AE)),  # HANGUL SYLLABLE IEUNG YAE CIEUC
    ((0x9E, 0x55), (0xC5AF)),  # HANGUL SYLLABLE IEUNG YAE CHIEUCH
    ((0x9E, 0x56), (0xC5B0)),  # HANGUL SYLLABLE IEUNG YAE KHIEUKH
    ((0x9E, 0x57), (0xC5B1)),  # HANGUL SYLLABLE IEUNG YAE THIEUTH
    ((0x9E, 0x58), (0xC5B2)),  # HANGUL SYLLABLE IEUNG YAE PHIEUPH
    ((0x9E, 0x59), (0xC5B3)),  # HANGUL SYLLABLE IEUNG YAE HIEUH
    ((0x9E, 0x5A), (0xC5B6)),  # HANGUL SYLLABLE IEUNG EO SSANGKIYEOK
    ((0x9E, 0x61), (0xC5B7)),  # HANGUL SYLLABLE IEUNG EO KIYEOKSIOS
    ((0x9E, 0x62), (0xC5BA)),  # HANGUL SYLLABLE IEUNG EO NIEUNHIEUH
    ((0x9E, 0x63), (0xC5BF)),  # HANGUL SYLLABLE IEUNG EO RIEULPIEUP
    ((0x9E, 0x64), (0xC5C0)),  # HANGUL SYLLABLE IEUNG EO RIEULSIOS
    ((0x9E, 0x65), (0xC5C1)),  # HANGUL SYLLABLE IEUNG EO RIEULTHIEUTH
    ((0x9E, 0x66), (0xC5C2)),  # HANGUL SYLLABLE IEUNG EO RIEULPHIEUPH
    ((0x9E, 0x67), (0xC5C3)),  # HANGUL SYLLABLE IEUNG EO RIEULHIEUH
    ((0x9E, 0x68), (0xC5CB)),  # HANGUL SYLLABLE IEUNG EO CHIEUCH
    ((0x9E, 0x69), (0xC5CD)),  # HANGUL SYLLABLE IEUNG EO THIEUTH
    ((0x9E, 0x6A), (0xC5CF)),  # HANGUL SYLLABLE IEUNG EO HIEUH
    ((0x9E, 0x6B), (0xC5D2)),  # HANGUL SYLLABLE IEUNG E SSANGKIYEOK
    ((0x9E, 0x6C), (0xC5D3)),  # HANGUL SYLLABLE IEUNG E KIYEOKSIOS
    ((0x9E, 0x6D), (0xC5D5)),  # HANGUL SYLLABLE IEUNG E NIEUNCIEUC
    ((0x9E, 0x6E), (0xC5D6)),  # HANGUL SYLLABLE IEUNG E NIEUNHIEUH
    ((0x9E, 0x6F), (0xC5D7)),  # HANGUL SYLLABLE IEUNG E TIKEUT
    ((0x9E, 0x70), (0xC5D9)),  # HANGUL SYLLABLE IEUNG E RIEULKIYEOK
    ((0x9E, 0x71), (0xC5DA)),  # HANGUL SYLLABLE IEUNG E RIEULMIEUM
    ((0x9E, 0x72), (0xC5DB)),  # HANGUL SYLLABLE IEUNG E RIEULPIEUP
    ((0x9E, 0x73), (0xC5DC)),  # HANGUL SYLLABLE IEUNG E RIEULSIOS
    ((0x9E, 0x74), (0xC5DD)),  # HANGUL SYLLABLE IEUNG E RIEULTHIEUTH
    ((0x9E, 0x75), (0xC5DE)),  # HANGUL SYLLABLE IEUNG E RIEULPHIEUPH
    ((0x9E, 0x76), (0xC5DF)),  # HANGUL SYLLABLE IEUNG E RIEULHIEUH
    ((0x9E, 0x77), (0xC5E2)),  # HANGUL SYLLABLE IEUNG E PIEUPSIOS
    ((0x9E, 0x78), (0xC5E4)),  # HANGUL SYLLABLE IEUNG E SSANGSIOS
    ((0x9E, 0x79), (0xC5E6)),  # HANGUL SYLLABLE IEUNG E CIEUC
    ((0x9E, 0x7A), (0xC5E7)),  # HANGUL SYLLABLE IEUNG E CHIEUCH
    ((0x9E, 0x81), (0xC5E8)),  # HANGUL SYLLABLE IEUNG E KHIEUKH
    ((0x9E, 0x82), (0xC5E9)),  # HANGUL SYLLABLE IEUNG E THIEUTH
    ((0x9E, 0x83), (0xC5EA)),  # HANGUL SYLLABLE IEUNG E PHIEUPH
    ((0x9E, 0x84), (0xC5EB)),  # HANGUL SYLLABLE IEUNG E HIEUH
    ((0x9E, 0x85), (0xC5EF)),  # HANGUL SYLLABLE IEUNG YEO KIYEOKSIOS
    ((0x9E, 0x86), (0xC5F1)),  # HANGUL SYLLABLE IEUNG YEO NIEUNCIEUC
    ((0x9E, 0x87), (0xC5F2)),  # HANGUL SYLLABLE IEUNG YEO NIEUNHIEUH
    ((0x9E, 0x88), (0xC5F3)),  # HANGUL SYLLABLE IEUNG YEO TIKEUT
    ((0x9E, 0x89), (0xC5F5)),  # HANGUL SYLLABLE IEUNG YEO RIEULKIYEOK
    ((0x9E, 0x8A), (0xC5F8)),  # HANGUL SYLLABLE IEUNG YEO RIEULSIOS
    ((0x9E, 0x8B), (0xC5F9)),  # HANGUL SYLLABLE IEUNG YEO RIEULTHIEUTH
    ((0x9E, 0x8C), (0xC5FA)),  # HANGUL SYLLABLE IEUNG YEO RIEULPHIEUPH
    ((0x9E, 0x8D), (0xC5FB)),  # HANGUL SYLLABLE IEUNG YEO RIEULHIEUH
    ((0x9E, 0x8E), (0xC602)),  # HANGUL SYLLABLE IEUNG YEO CIEUC
    ((0x9E, 0x8F), (0xC603)),  # HANGUL SYLLABLE IEUNG YEO CHIEUCH
    ((0x9E, 0x90), (0xC604)),  # HANGUL SYLLABLE IEUNG YEO KHIEUKH
    ((0x9E, 0x91), (0xC609)),  # HANGUL SYLLABLE IEUNG YE KIYEOK
    ((0x9E, 0x92), (0xC60A)),  # HANGUL SYLLABLE IEUNG YE SSANGKIYEOK
    ((0x9E, 0x93), (0xC60B)),  # HANGUL SYLLABLE IEUNG YE KIYEOKSIOS
    ((0x9E, 0x94), (0xC60D)),  # HANGUL SYLLABLE IEUNG YE NIEUNCIEUC
    ((0x9E, 0x95), (0xC60E)),  # HANGUL SYLLABLE IEUNG YE NIEUNHIEUH
    ((0x9E, 0x96), (0xC60F)),  # HANGUL SYLLABLE IEUNG YE TIKEUT
    ((0x9E, 0x97), (0xC611)),  # HANGUL SYLLABLE IEUNG YE RIEULKIYEOK
    ((0x9E, 0x98), (0xC612)),  # HANGUL SYLLABLE IEUNG YE RIEULMIEUM
    ((0x9E, 0x99), (0xC613)),  # HANGUL SYLLABLE IEUNG YE RIEULPIEUP
    ((0x9E, 0x9A), (0xC614)),  # HANGUL SYLLABLE IEUNG YE RIEULSIOS
    ((0x9E, 0x9B), (0xC615)),  # HANGUL SYLLABLE IEUNG YE RIEULTHIEUTH
    ((0x9E, 0x9C), (0xC616)),  # HANGUL SYLLABLE IEUNG YE RIEULPHIEUPH
    ((0x9E, 0x9D), (0xC617)),  # HANGUL SYLLABLE IEUNG YE RIEULHIEUH
    ((0x9E, 0x9E), (0xC61A)),  # HANGUL SYLLABLE IEUNG YE PIEUPSIOS
    ((0x9E, 0x9F), (0xC61D)),  # HANGUL SYLLABLE IEUNG YE IEUNG
    ((0x9E, 0xA0), (0xC61E)),  # HANGUL SYLLABLE IEUNG YE CIEUC
    ((0x9E, 0xA1), (0xC61F)),  # HANGUL SYLLABLE IEUNG YE CHIEUCH
    ((0x9E, 0xA2), (0xC620)),  # HANGUL SYLLABLE IEUNG YE KHIEUKH
    ((0x9E, 0xA3), (0xC621)),  # HANGUL SYLLABLE IEUNG YE THIEUTH
    ((0x9E, 0xA4), (0xC622)),  # HANGUL SYLLABLE IEUNG YE PHIEUPH
    ((0x9E, 0xA5), (0xC623)),  # HANGUL SYLLABLE IEUNG YE HIEUH
    ((0x9E, 0xA6), (0xC626)),  # HANGUL SYLLABLE IEUNG O SSANGKIYEOK
    ((0x9E, 0xA7), (0xC627)),  # HANGUL SYLLABLE IEUNG O KIYEOKSIOS
    ((0x9E, 0xA8), (0xC629)),  # HANGUL SYLLABLE IEUNG O NIEUNCIEUC
    ((0x9E, 0xA9), (0xC62A)),  # HANGUL SYLLABLE IEUNG O NIEUNHIEUH
    ((0x9E, 0xAA), (0xC62B)),  # HANGUL SYLLABLE IEUNG O TIKEUT
    ((0x9E, 0xAB), (0xC62F)),  # HANGUL SYLLABLE IEUNG O RIEULPIEUP
    ((0x9E, 0xAC), (0xC631)),  # HANGUL SYLLABLE IEUNG O RIEULTHIEUTH
    ((0x9E, 0xAD), (0xC632)),  # HANGUL SYLLABLE IEUNG O RIEULPHIEUPH
    ((0x9E, 0xAE), (0xC636)),  # HANGUL SYLLABLE IEUNG O PIEUPSIOS
    ((0x9E, 0xAF), (0xC638)),  # HANGUL SYLLABLE IEUNG O SSANGSIOS
    ((0x9E, 0xB0), (0xC63A)),  # HANGUL SYLLABLE IEUNG O CIEUC
    ((0x9E, 0xB1), (0xC63C)),  # HANGUL SYLLABLE IEUNG O KHIEUKH
    ((0x9E, 0xB2), (0xC63D)),  # HANGUL SYLLABLE IEUNG O THIEUTH
    ((0x9E, 0xB3), (0xC63E)),  # HANGUL SYLLABLE IEUNG O PHIEUPH
    ((0x9E, 0xB4), (0xC63F)),  # HANGUL SYLLABLE IEUNG O HIEUH
    ((0x9E, 0xB5), (0xC642)),  # HANGUL SYLLABLE IEUNG WA SSANGKIYEOK
    ((0x9E, 0xB6), (0xC643)),  # HANGUL SYLLABLE IEUNG WA KIYEOKSIOS
    ((0x9E, 0xB7), (0xC645)),  # HANGUL SYLLABLE IEUNG WA NIEUNCIEUC
    ((0x9E, 0xB8), (0xC646)),  # HANGUL SYLLABLE IEUNG WA NIEUNHIEUH
    ((0x9E, 0xB9), (0xC647)),  # HANGUL SYLLABLE IEUNG WA TIKEUT
    ((0x9E, 0xBA), (0xC649)),  # HANGUL SYLLABLE IEUNG WA RIEULKIYEOK
    ((0x9E, 0xBB), (0xC64A)),  # HANGUL SYLLABLE IEUNG WA RIEULMIEUM
    ((0x9E, 0xBC), (0xC64B)),  # HANGUL SYLLABLE IEUNG WA RIEULPIEUP
    ((0x9E, 0xBD), (0xC64C)),  # HANGUL SYLLABLE IEUNG WA RIEULSIOS
    ((0x9E, 0xBE), (0xC64D)),  # HANGUL SYLLABLE IEUNG WA RIEULTHIEUTH
    ((0x9E, 0xBF), (0xC64E)),  # HANGUL SYLLABLE IEUNG WA RIEULPHIEUPH
    ((0x9E, 0xC0), (0xC64F)),  # HANGUL SYLLABLE IEUNG WA RIEULHIEUH
    ((0x9E, 0xC1), (0xC652)),  # HANGUL SYLLABLE IEUNG WA PIEUPSIOS
    ((0x9E, 0xC2), (0xC656)),  # HANGUL SYLLABLE IEUNG WA CIEUC
    ((0x9E, 0xC3), (0xC657)),  # HANGUL SYLLABLE IEUNG WA CHIEUCH
    ((0x9E, 0xC4), (0xC658)),  # HANGUL SYLLABLE IEUNG WA KHIEUKH
    ((0x9E, 0xC5), (0xC659)),  # HANGUL SYLLABLE IEUNG WA THIEUTH
    ((0x9E, 0xC6), (0xC65A)),  # HANGUL SYLLABLE IEUNG WA PHIEUPH
    ((0x9E, 0xC7), (0xC65B)),  # HANGUL SYLLABLE IEUNG WA HIEUH
    ((0x9E, 0xC8), (0xC65E)),  # HANGUL SYLLABLE IEUNG WAE SSANGKIYEOK
    ((0x9E, 0xC9), (0xC65F)),  # HANGUL SYLLABLE IEUNG WAE KIYEOKSIOS
    ((0x9E, 0xCA), (0xC661)),  # HANGUL SYLLABLE IEUNG WAE NIEUNCIEUC
    ((0x9E, 0xCB), (0xC662)),  # HANGUL SYLLABLE IEUNG WAE NIEUNHIEUH
    ((0x9E, 0xCC), (0xC663)),  # HANGUL SYLLABLE IEUNG WAE TIKEUT
    ((0x9E, 0xCD), (0xC664)),  # HANGUL SYLLABLE IEUNG WAE RIEUL
    ((0x9E, 0xCE), (0xC665)),  # HANGUL SYLLABLE IEUNG WAE RIEULKIYEOK
    ((0x9E, 0xCF), (0xC666)),  # HANGUL SYLLABLE IEUNG WAE RIEULMIEUM
    ((0x9E, 0xD0), (0xC667)),  # HANGUL SYLLABLE IEUNG WAE RIEULPIEUP
    ((0x9E, 0xD1), (0xC668)),  # HANGUL SYLLABLE IEUNG WAE RIEULSIOS
    ((0x9E, 0xD2), (0xC669)),  # HANGUL SYLLABLE IEUNG WAE RIEULTHIEUTH
    ((0x9E, 0xD3), (0xC66A)),  # HANGUL SYLLABLE IEUNG WAE RIEULPHIEUPH
    ((0x9E, 0xD4), (0xC66B)),  # HANGUL SYLLABLE IEUNG WAE RIEULHIEUH
    ((0x9E, 0xD5), (0xC66D)),  # HANGUL SYLLABLE IEUNG WAE PIEUP
    ((0x9E, 0xD6), (0xC66E)),  # HANGUL SYLLABLE IEUNG WAE PIEUPSIOS
    ((0x9E, 0xD7), (0xC670)),  # HANGUL SYLLABLE IEUNG WAE SSANGSIOS
    ((0x9E, 0xD8), (0xC672)),  # HANGUL SYLLABLE IEUNG WAE CIEUC
    ((0x9E, 0xD9), (0xC673)),  # HANGUL SYLLABLE IEUNG WAE CHIEUCH
    ((0x9E, 0xDA), (0xC674)),  # HANGUL SYLLABLE IEUNG WAE KHIEUKH
    ((0x9E, 0xDB), (0xC675)),  # HANGUL SYLLABLE IEUNG WAE THIEUTH
    ((0x9E, 0xDC), (0xC676)),  # HANGUL SYLLABLE IEUNG WAE PHIEUPH
    ((0x9E, 0xDD), (0xC677)),  # HANGUL SYLLABLE IEUNG WAE HIEUH
    ((0x9E, 0xDE), (0xC67A)),  # HANGUL SYLLABLE IEUNG OE SSANGKIYEOK
    ((0x9E, 0xDF), (0xC67B)),  # HANGUL SYLLABLE IEUNG OE KIYEOKSIOS
    ((0x9E, 0xE0), (0xC67D)),  # HANGUL SYLLABLE IEUNG OE NIEUNCIEUC
    ((0x9E, 0xE1), (0xC67E)),  # HANGUL SYLLABLE IEUNG OE NIEUNHIEUH
    ((0x9E, 0xE2), (0xC67F)),  # HANGUL SYLLABLE IEUNG OE TIKEUT
    ((0x9E, 0xE3), (0xC681)),  # HANGUL SYLLABLE IEUNG OE RIEULKIYEOK
    ((0x9E, 0xE4), (0xC682)),  # HANGUL SYLLABLE IEUNG OE RIEULMIEUM
    ((0x9E, 0xE5), (0xC683)),  # HANGUL SYLLABLE IEUNG OE RIEULPIEUP
    ((0x9E, 0xE6), (0xC684)),  # HANGUL SYLLABLE IEUNG OE RIEULSIOS
    ((0x9E, 0xE7), (0xC685)),  # HANGUL SYLLABLE IEUNG OE RIEULTHIEUTH
    ((0x9E, 0xE8), (0xC686)),  # HANGUL SYLLABLE IEUNG OE RIEULPHIEUPH
    ((0x9E, 0xE9), (0xC687)),  # HANGUL SYLLABLE IEUNG OE RIEULHIEUH
    ((0x9E, 0xEA), (0xC68A)),  # HANGUL SYLLABLE IEUNG OE PIEUPSIOS
    ((0x9E, 0xEB), (0xC68C)),  # HANGUL SYLLABLE IEUNG OE SSANGSIOS
    ((0x9E, 0xEC), (0xC68E)),  # HANGUL SYLLABLE IEUNG OE CIEUC
    ((0x9E, 0xED), (0xC68F)),  # HANGUL SYLLABLE IEUNG OE CHIEUCH
    ((0x9E, 0xEE), (0xC690)),  # HANGUL SYLLABLE IEUNG OE KHIEUKH
    ((0x9E, 0xEF), (0xC691)),  # HANGUL SYLLABLE IEUNG OE THIEUTH
    ((0x9E, 0xF0), (0xC692)),  # HANGUL SYLLABLE IEUNG OE PHIEUPH
    ((0x9E, 0xF1), (0xC693)),  # HANGUL SYLLABLE IEUNG OE HIEUH
    ((0x9E, 0xF2), (0xC696)),  # HANGUL SYLLABLE IEUNG YO SSANGKIYEOK
    ((0x9E, 0xF3), (0xC697)),  # HANGUL SYLLABLE IEUNG YO KIYEOKSIOS
    ((0x9E, 0xF4), (0xC699)),  # HANGUL SYLLABLE IEUNG YO NIEUNCIEUC
    ((0x9E, 0xF5), (0xC69A)),  # HANGUL SYLLABLE IEUNG YO NIEUNHIEUH
    ((0x9E, 0xF6), (0xC69B)),  # HANGUL SYLLABLE IEUNG YO TIKEUT
    ((0x9E, 0xF7), (0xC69D)),  # HANGUL SYLLABLE IEUNG YO RIEULKIYEOK
    ((0x9E, 0xF8), (0xC69E)),  # HANGUL SYLLABLE IEUNG YO RIEULMIEUM
    ((0x9E, 0xF9), (0xC69F)),  # HANGUL SYLLABLE IEUNG YO RIEULPIEUP
    ((0x9E, 0xFA), (0xC6A0)),  # HANGUL SYLLABLE IEUNG YO RIEULSIOS
    ((0x9E, 0xFB), (0xC6A1)),  # HANGUL SYLLABLE IEUNG YO RIEULTHIEUTH
    ((0x9E, 0xFC), (0xC6A2)),  # HANGUL SYLLABLE IEUNG YO RIEULPHIEUPH
    ((0x9E, 0xFD), (0xC6A3)),  # HANGUL SYLLABLE IEUNG YO RIEULHIEUH
    ((0x9E, 0xFE), (0xC6A6)),  # HANGUL SYLLABLE IEUNG YO PIEUPSIOS
    ((0x9F, 0x41), (0xC6A8)),  # HANGUL SYLLABLE IEUNG YO SSANGSIOS
    ((0x9F, 0x42), (0xC6AA)),  # HANGUL SYLLABLE IEUNG YO CIEUC
    ((0x9F, 0x43), (0xC6AB)),  # HANGUL SYLLABLE IEUNG YO CHIEUCH
    ((0x9F, 0x44), (0xC6AC)),  # HANGUL SYLLABLE IEUNG YO KHIEUKH
    ((0x9F, 0x45), (0xC6AD)),  # HANGUL SYLLABLE IEUNG YO THIEUTH
    ((0x9F, 0x46), (0xC6AE)),  # HANGUL SYLLABLE IEUNG YO PHIEUPH
    ((0x9F, 0x47), (0xC6AF)),  # HANGUL SYLLABLE IEUNG YO HIEUH
    ((0x9F, 0x48), (0xC6B2)),  # HANGUL SYLLABLE IEUNG U SSANGKIYEOK
    ((0x9F, 0x49), (0xC6B3)),  # HANGUL SYLLABLE IEUNG U KIYEOKSIOS
    ((0x9F, 0x4A), (0xC6B5)),  # HANGUL SYLLABLE IEUNG U NIEUNCIEUC
    ((0x9F, 0x4B), (0xC6B6)),  # HANGUL SYLLABLE IEUNG U NIEUNHIEUH
    ((0x9F, 0x4C), (0xC6B7)),  # HANGUL SYLLABLE IEUNG U TIKEUT
    ((0x9F, 0x4D), (0xC6BB)),  # HANGUL SYLLABLE IEUNG U RIEULPIEUP
    ((0x9F, 0x4E), (0xC6BC)),  # HANGUL SYLLABLE IEUNG U RIEULSIOS
    ((0x9F, 0x4F), (0xC6BD)),  # HANGUL SYLLABLE IEUNG U RIEULTHIEUTH
    ((0x9F, 0x50), (0xC6BE)),  # HANGUL SYLLABLE IEUNG U RIEULPHIEUPH
    ((0x9F, 0x51), (0xC6BF)),  # HANGUL SYLLABLE IEUNG U RIEULHIEUH
    ((0x9F, 0x52), (0xC6C2)),  # HANGUL SYLLABLE IEUNG U PIEUPSIOS
    ((0x9F, 0x53), (0xC6C4)),  # HANGUL SYLLABLE IEUNG U SSANGSIOS
    ((0x9F, 0x54), (0xC6C6)),  # HANGUL SYLLABLE IEUNG U CIEUC
    ((0x9F, 0x55), (0xC6C7)),  # HANGUL SYLLABLE IEUNG U CHIEUCH
    ((0x9F, 0x56), (0xC6C8)),  # HANGUL SYLLABLE IEUNG U KHIEUKH
    ((0x9F, 0x57), (0xC6C9)),  # HANGUL SYLLABLE IEUNG U THIEUTH
    ((0x9F, 0x58), (0xC6CA)),  # HANGUL SYLLABLE IEUNG U PHIEUPH
    ((0x9F, 0x59), (0xC6CB)),  # HANGUL SYLLABLE IEUNG U HIEUH
    ((0x9F, 0x5A), (0xC6CE)),  # HANGUL SYLLABLE IEUNG WEO SSANGKIYEOK
    ((0x9F, 0x61), (0xC6CF)),  # HANGUL SYLLABLE IEUNG WEO KIYEOKSIOS
    ((0x9F, 0x62), (0xC6D1)),  # HANGUL SYLLABLE IEUNG WEO NIEUNCIEUC
    ((0x9F, 0x63), (0xC6D2)),  # HANGUL SYLLABLE IEUNG WEO NIEUNHIEUH
    ((0x9F, 0x64), (0xC6D3)),  # HANGUL SYLLABLE IEUNG WEO TIKEUT
    ((0x9F, 0x65), (0xC6D5)),  # HANGUL SYLLABLE IEUNG WEO RIEULKIYEOK
    ((0x9F, 0x66), (0xC6D6)),  # HANGUL SYLLABLE IEUNG WEO RIEULMIEUM
    ((0x9F, 0x67), (0xC6D7)),  # HANGUL SYLLABLE IEUNG WEO RIEULPIEUP
    ((0x9F, 0x68), (0xC6D8)),  # HANGUL SYLLABLE IEUNG WEO RIEULSIOS
    ((0x9F, 0x69), (0xC6D9)),  # HANGUL SYLLABLE IEUNG WEO RIEULTHIEUTH
    ((0x9F, 0x6A), (0xC6DA)),  # HANGUL SYLLABLE IEUNG WEO RIEULPHIEUPH
    ((0x9F, 0x6B), (0xC6DB)),  # HANGUL SYLLABLE IEUNG WEO RIEULHIEUH
    ((0x9F, 0x6C), (0xC6DE)),  # HANGUL SYLLABLE IEUNG WEO PIEUPSIOS
    ((0x9F, 0x6D), (0xC6DF)),  # HANGUL SYLLABLE IEUNG WEO SIOS
    ((0x9F, 0x6E), (0xC6E2)),  # HANGUL SYLLABLE IEUNG WEO CIEUC
    ((0x9F, 0x6F), (0xC6E3)),  # HANGUL SYLLABLE IEUNG WEO CHIEUCH
    ((0x9F, 0x70), (0xC6E4)),  # HANGUL SYLLABLE IEUNG WEO KHIEUKH
    ((0x9F, 0x71), (0xC6E5)),  # HANGUL SYLLABLE IEUNG WEO THIEUTH
    ((0x9F, 0x72), (0xC6E6)),  # HANGUL SYLLABLE IEUNG WEO PHIEUPH
    ((0x9F, 0x73), (0xC6E7)),  # HANGUL SYLLABLE IEUNG WEO HIEUH
    ((0x9F, 0x74), (0xC6EA)),  # HANGUL SYLLABLE IEUNG WE SSANGKIYEOK
    ((0x9F, 0x75), (0xC6EB)),  # HANGUL SYLLABLE IEUNG WE KIYEOKSIOS
    ((0x9F, 0x76), (0xC6ED)),  # HANGUL SYLLABLE IEUNG WE NIEUNCIEUC
    ((0x9F, 0x77), (0xC6EE)),  # HANGUL SYLLABLE IEUNG WE NIEUNHIEUH
    ((0x9F, 0x78), (0xC6EF)),  # HANGUL SYLLABLE IEUNG WE TIKEUT
    ((0x9F, 0x79), (0xC6F1)),  # HANGUL SYLLABLE IEUNG WE RIEULKIYEOK
    ((0x9F, 0x7A), (0xC6F2)),  # HANGUL SYLLABLE IEUNG WE RIEULMIEUM
    ((0x9F, 0x81), (0xC6F3)),  # HANGUL SYLLABLE IEUNG WE RIEULPIEUP
    ((0x9F, 0x82), (0xC6F4)),  # HANGUL SYLLABLE IEUNG WE RIEULSIOS
    ((0x9F, 0x83), (0xC6F5)),  # HANGUL SYLLABLE IEUNG WE RIEULTHIEUTH
    ((0x9F, 0x84), (0xC6F6)),  # HANGUL SYLLABLE IEUNG WE RIEULPHIEUPH
    ((0x9F, 0x85), (0xC6F7)),  # HANGUL SYLLABLE IEUNG WE RIEULHIEUH
    ((0x9F, 0x86), (0xC6FA)),  # HANGUL SYLLABLE IEUNG WE PIEUPSIOS
    ((0x9F, 0x87), (0xC6FB)),  # HANGUL SYLLABLE IEUNG WE SIOS
    ((0x9F, 0x88), (0xC6FC)),  # HANGUL SYLLABLE IEUNG WE SSANGSIOS
    ((0x9F, 0x89), (0xC6FE)),  # HANGUL SYLLABLE IEUNG WE CIEUC
    ((0x9F, 0x8A), (0xC6FF)),  # HANGUL SYLLABLE IEUNG WE CHIEUCH
    ((0x9F, 0x8B), (0xC700)),  # HANGUL SYLLABLE IEUNG WE KHIEUKH
    ((0x9F, 0x8C), (0xC701)),  # HANGUL SYLLABLE IEUNG WE THIEUTH
    ((0x9F, 0x8D), (0xC702)),  # HANGUL SYLLABLE IEUNG WE PHIEUPH
    ((0x9F, 0x8E), (0xC703)),  # HANGUL SYLLABLE IEUNG WE HIEUH
    ((0x9F, 0x8F), (0xC706)),  # HANGUL SYLLABLE IEUNG WI SSANGKIYEOK
    ((0x9F, 0x90), (0xC707)),  # HANGUL SYLLABLE IEUNG WI KIYEOKSIOS
    ((0x9F, 0x91), (0xC709)),  # HANGUL SYLLABLE IEUNG WI NIEUNCIEUC
    ((0x9F, 0x92), (0xC70A)),  # HANGUL SYLLABLE IEUNG WI NIEUNHIEUH
    ((0x9F, 0x93), (0xC70B)),  # HANGUL SYLLABLE IEUNG WI TIKEUT
    ((0x9F, 0x94), (0xC70D)),  # HANGUL SYLLABLE IEUNG WI RIEULKIYEOK
    ((0x9F, 0x95), (0xC70E)),  # HANGUL SYLLABLE IEUNG WI RIEULMIEUM
    ((0x9F, 0x96), (0xC70F)),  # HANGUL SYLLABLE IEUNG WI RIEULPIEUP
    ((0x9F, 0x97), (0xC710)),  # HANGUL SYLLABLE IEUNG WI RIEULSIOS
    ((0x9F, 0x98), (0xC711)),  # HANGUL SYLLABLE IEUNG WI RIEULTHIEUTH
    ((0x9F, 0x99), (0xC712)),  # HANGUL SYLLABLE IEUNG WI RIEULPHIEUPH
    ((0x9F, 0x9A), (0xC713)),  # HANGUL SYLLABLE IEUNG WI RIEULHIEUH
    ((0x9F, 0x9B), (0xC716)),  # HANGUL SYLLABLE IEUNG WI PIEUPSIOS
    ((0x9F, 0x9C), (0xC718)),  # HANGUL SYLLABLE IEUNG WI SSANGSIOS
    ((0x9F, 0x9D), (0xC71A)),  # HANGUL SYLLABLE IEUNG WI CIEUC
    ((0x9F, 0x9E), (0xC71B)),  # HANGUL SYLLABLE IEUNG WI CHIEUCH
    ((0x9F, 0x9F), (0xC71C)),  # HANGUL SYLLABLE IEUNG WI KHIEUKH
    ((0x9F, 0xA0), (0xC71D)),  # HANGUL SYLLABLE IEUNG WI THIEUTH
    ((0x9F, 0xA1), (0xC71E)),  # HANGUL SYLLABLE IEUNG WI PHIEUPH
    ((0x9F, 0xA2), (0xC71F)),  # HANGUL SYLLABLE IEUNG WI HIEUH
    ((0x9F, 0xA3), (0xC722)),  # HANGUL SYLLABLE IEUNG YU SSANGKIYEOK
    ((0x9F, 0xA4), (0xC723)),  # HANGUL SYLLABLE IEUNG YU KIYEOKSIOS
    ((0x9F, 0xA5), (0xC725)),  # HANGUL SYLLABLE IEUNG YU NIEUNCIEUC
    ((0x9F, 0xA6), (0xC726)),  # HANGUL SYLLABLE IEUNG YU NIEUNHIEUH
    ((0x9F, 0xA7), (0xC727)),  # HANGUL SYLLABLE IEUNG YU TIKEUT
    ((0x9F, 0xA8), (0xC729)),  # HANGUL SYLLABLE IEUNG YU RIEULKIYEOK
    ((0x9F, 0xA9), (0xC72A)),  # HANGUL SYLLABLE IEUNG YU RIEULMIEUM
    ((0x9F, 0xAA), (0xC72B)),  # HANGUL SYLLABLE IEUNG YU RIEULPIEUP
    ((0x9F, 0xAB), (0xC72C)),  # HANGUL SYLLABLE IEUNG YU RIEULSIOS
    ((0x9F, 0xAC), (0xC72D)),  # HANGUL SYLLABLE IEUNG YU RIEULTHIEUTH
    ((0x9F, 0xAD), (0xC72E)),  # HANGUL SYLLABLE IEUNG YU RIEULPHIEUPH
    ((0x9F, 0xAE), (0xC72F)),  # HANGUL SYLLABLE IEUNG YU RIEULHIEUH
    ((0x9F, 0xAF), (0xC732)),  # HANGUL SYLLABLE IEUNG YU PIEUPSIOS
    ((0x9F, 0xB0), (0xC734)),  # HANGUL SYLLABLE IEUNG YU SSANGSIOS
    ((0x9F, 0xB1), (0xC736)),  # HANGUL SYLLABLE IEUNG YU CIEUC
    ((0x9F, 0xB2), (0xC738)),  # HANGUL SYLLABLE IEUNG YU KHIEUKH
    ((0x9F, 0xB3), (0xC739)),  # HANGUL SYLLABLE IEUNG YU THIEUTH
    ((0x9F, 0xB4), (0xC73A)),  # HANGUL SYLLABLE IEUNG YU PHIEUPH
    ((0x9F, 0xB5), (0xC73B)),  # HANGUL SYLLABLE IEUNG YU HIEUH
    ((0x9F, 0xB6), (0xC73E)),  # HANGUL SYLLABLE IEUNG EU SSANGKIYEOK
    ((0x9F, 0xB7), (0xC73F)),  # HANGUL SYLLABLE IEUNG EU KIYEOKSIOS
    ((0x9F, 0xB8), (0xC741)),  # HANGUL SYLLABLE IEUNG EU NIEUNCIEUC
    ((0x9F, 0xB9), (0xC742)),  # HANGUL SYLLABLE IEUNG EU NIEUNHIEUH
    ((0x9F, 0xBA), (0xC743)),  # HANGUL SYLLABLE IEUNG EU TIKEUT
    ((0x9F, 0xBB), (0xC745)),  # HANGUL SYLLABLE IEUNG EU RIEULKIYEOK
    ((0x9F, 0xBC), (0xC746)),  # HANGUL SYLLABLE IEUNG EU RIEULMIEUM
    ((0x9F, 0xBD), (0xC747)),  # HANGUL SYLLABLE IEUNG EU RIEULPIEUP
    ((0x9F, 0xBE), (0xC748)),  # HANGUL SYLLABLE IEUNG EU RIEULSIOS
    ((0x9F, 0xBF), (0xC749)),  # HANGUL SYLLABLE IEUNG EU RIEULTHIEUTH
    ((0x9F, 0xC0), (0xC74B)),  # HANGUL SYLLABLE IEUNG EU RIEULHIEUH
    ((0x9F, 0xC1), (0xC74E)),  # HANGUL SYLLABLE IEUNG EU PIEUPSIOS
    ((0x9F, 0xC2), (0xC750)),  # HANGUL SYLLABLE IEUNG EU SSANGSIOS
    ((0x9F, 0xC3), (0xC759)),  # HANGUL SYLLABLE IEUNG YI KIYEOK
    ((0x9F, 0xC4), (0xC75A)),  # HANGUL SYLLABLE IEUNG YI SSANGKIYEOK
    ((0x9F, 0xC5), (0xC75B)),  # HANGUL SYLLABLE IEUNG YI KIYEOKSIOS
    ((0x9F, 0xC6), (0xC75D)),  # HANGUL SYLLABLE IEUNG YI NIEUNCIEUC
    ((0x9F, 0xC7), (0xC75E)),  # HANGUL SYLLABLE IEUNG YI NIEUNHIEUH
    ((0x9F, 0xC8), (0xC75F)),  # HANGUL SYLLABLE IEUNG YI TIKEUT
    ((0x9F, 0xC9), (0xC761)),  # HANGUL SYLLABLE IEUNG YI RIEULKIYEOK
    ((0x9F, 0xCA), (0xC762)),  # HANGUL SYLLABLE IEUNG YI RIEULMIEUM
    ((0x9F, 0xCB), (0xC763)),  # HANGUL SYLLABLE IEUNG YI RIEULPIEUP
    ((0x9F, 0xCC), (0xC764)),  # HANGUL SYLLABLE IEUNG YI RIEULSIOS
    ((0x9F, 0xCD), (0xC765)),  # HANGUL SYLLABLE IEUNG YI RIEULTHIEUTH
    ((0x9F, 0xCE), (0xC766)),  # HANGUL SYLLABLE IEUNG YI RIEULPHIEUPH
    ((0x9F, 0xCF), (0xC767)),  # HANGUL SYLLABLE IEUNG YI RIEULHIEUH
    ((0x9F, 0xD0), (0xC769)),  # HANGUL SYLLABLE IEUNG YI PIEUP
    ((0x9F, 0xD1), (0xC76A)),  # HANGUL SYLLABLE IEUNG YI PIEUPSIOS
    ((0x9F, 0xD2), (0xC76C)),  # HANGUL SYLLABLE IEUNG YI SSANGSIOS
    ((0x9F, 0xD3), (0xC76D)),  # HANGUL SYLLABLE IEUNG YI IEUNG
    ((0x9F, 0xD4), (0xC76E)),  # HANGUL SYLLABLE IEUNG YI CIEUC
    ((0x9F, 0xD5), (0xC76F)),  # HANGUL SYLLABLE IEUNG YI CHIEUCH
    ((0x9F, 0xD6), (0xC770)),  # HANGUL SYLLABLE IEUNG YI KHIEUKH
    ((0x9F, 0xD7), (0xC771)),  # HANGUL SYLLABLE IEUNG YI THIEUTH
    ((0x9F, 0xD8), (0xC772)),  # HANGUL SYLLABLE IEUNG YI PHIEUPH
    ((0x9F, 0xD9), (0xC773)),  # HANGUL SYLLABLE IEUNG YI HIEUH
    ((0x9F, 0xDA), (0xC776)),  # HANGUL SYLLABLE IEUNG I SSANGKIYEOK
    ((0x9F, 0xDB), (0xC777)),  # HANGUL SYLLABLE IEUNG I KIYEOKSIOS
    ((0x9F, 0xDC), (0xC779)),  # HANGUL SYLLABLE IEUNG I NIEUNCIEUC
    ((0x9F, 0xDD), (0xC77A)),  # HANGUL SYLLABLE IEUNG I NIEUNHIEUH
    ((0x9F, 0xDE), (0xC77B)),  # HANGUL SYLLABLE IEUNG I TIKEUT
    ((0x9F, 0xDF), (0xC77F)),  # HANGUL SYLLABLE IEUNG I RIEULPIEUP
    ((0x9F, 0xE0), (0xC780)),  # HANGUL SYLLABLE IEUNG I RIEULSIOS
    ((0x9F, 0xE1), (0xC781)),  # HANGUL SYLLABLE IEUNG I RIEULTHIEUTH
    ((0x9F, 0xE2), (0xC782)),  # HANGUL SYLLABLE IEUNG I RIEULPHIEUPH
    ((0x9F, 0xE3), (0xC786)),  # HANGUL SYLLABLE IEUNG I PIEUPSIOS
    ((0x9F, 0xE4), (0xC78B)),  # HANGUL SYLLABLE IEUNG I CHIEUCH
    ((0x9F, 0xE5), (0xC78C)),  # HANGUL SYLLABLE IEUNG I KHIEUKH
    ((0x9F, 0xE6), (0xC78D)),  # HANGUL SYLLABLE IEUNG I THIEUTH
    ((0x9F, 0xE7), (0xC78F)),  # HANGUL SYLLABLE IEUNG I HIEUH
    ((0x9F, 0xE8), (0xC792)),  # HANGUL SYLLABLE CIEUC A SSANGKIYEOK
    ((0x9F, 0xE9), (0xC793)),  # HANGUL SYLLABLE CIEUC A KIYEOKSIOS
    ((0x9F, 0xEA), (0xC795)),  # HANGUL SYLLABLE CIEUC A NIEUNCIEUC
    ((0x9F, 0xEB), (0xC799)),  # HANGUL SYLLABLE CIEUC A RIEULKIYEOK
    ((0x9F, 0xEC), (0xC79B)),  # HANGUL SYLLABLE CIEUC A RIEULPIEUP
    ((0x9F, 0xED), (0xC79C)),  # HANGUL SYLLABLE CIEUC A RIEULSIOS
    ((0x9F, 0xEE), (0xC79D)),  # HANGUL SYLLABLE CIEUC A RIEULTHIEUTH
    ((0x9F, 0xEF), (0xC79E)),  # HANGUL SYLLABLE CIEUC A RIEULPHIEUPH
    ((0x9F, 0xF0), (0xC79F)),  # HANGUL SYLLABLE CIEUC A RIEULHIEUH
    ((0x9F, 0xF1), (0xC7A2)),  # HANGUL SYLLABLE CIEUC A PIEUPSIOS
    ((0x9F, 0xF2), (0xC7A7)),  # HANGUL SYLLABLE CIEUC A CHIEUCH
    ((0x9F, 0xF3), (0xC7A8)),  # HANGUL SYLLABLE CIEUC A KHIEUKH
    ((0x9F, 0xF4), (0xC7A9)),  # HANGUL SYLLABLE CIEUC A THIEUTH
    ((0x9F, 0xF5), (0xC7AA)),  # HANGUL SYLLABLE CIEUC A PHIEUPH
    ((0x9F, 0xF6), (0xC7AB)),  # HANGUL SYLLABLE CIEUC A HIEUH
    ((0x9F, 0xF7), (0xC7AE)),  # HANGUL SYLLABLE CIEUC AE SSANGKIYEOK
    ((0x9F, 0xF8), (0xC7AF)),  # HANGUL SYLLABLE CIEUC AE KIYEOKSIOS
    ((0x9F, 0xF9), (0xC7B1)),  # HANGUL SYLLABLE CIEUC AE NIEUNCIEUC
    ((0x9F, 0xFA), (0xC7B2)),  # HANGUL SYLLABLE CIEUC AE NIEUNHIEUH
    ((0x9F, 0xFB), (0xC7B3)),  # HANGUL SYLLABLE CIEUC AE TIKEUT
    ((0x9F, 0xFC), (0xC7B5)),  # HANGUL SYLLABLE CIEUC AE RIEULKIYEOK
    ((0x9F, 0xFD), (0xC7B6)),  # HANGUL SYLLABLE CIEUC AE RIEULMIEUM
    ((0x9F, 0xFE), (0xC7B7)),  # HANGUL SYLLABLE CIEUC AE RIEULPIEUP
    ((0xA0, 0x41), (0xC7B8)),  # HANGUL SYLLABLE CIEUC AE RIEULSIOS
    ((0xA0, 0x42), (0xC7B9)),  # HANGUL SYLLABLE CIEUC AE RIEULTHIEUTH
    ((0xA0, 0x43), (0xC7BA)),  # HANGUL SYLLABLE CIEUC AE RIEULPHIEUPH
    ((0xA0, 0x44), (0xC7BB)),  # HANGUL SYLLABLE CIEUC AE RIEULHIEUH
    ((0xA0, 0x45), (0xC7BE)),  # HANGUL SYLLABLE CIEUC AE PIEUPSIOS
    ((0xA0, 0x46), (0xC7C2)),  # HANGUL SYLLABLE CIEUC AE CIEUC
    ((0xA0, 0x47), (0xC7C3)),  # HANGUL SYLLABLE CIEUC AE CHIEUCH
    ((0xA0, 0x48), (0xC7C4)),  # HANGUL SYLLABLE CIEUC AE KHIEUKH
    ((0xA0, 0x49), (0xC7C5)),  # HANGUL SYLLABLE CIEUC AE THIEUTH
    ((0xA0, 0x4A), (0xC7C6)),  # HANGUL SYLLABLE CIEUC AE PHIEUPH
    ((0xA0, 0x4B), (0xC7C7)),  # HANGUL SYLLABLE CIEUC AE HIEUH
    ((0xA0, 0x4C), (0xC7CA)),  # HANGUL SYLLABLE CIEUC YA SSANGKIYEOK
    ((0xA0, 0x4D), (0xC7CB)),  # HANGUL SYLLABLE CIEUC YA KIYEOKSIOS
    ((0xA0, 0x4E), (0xC7CD)),  # HANGUL SYLLABLE CIEUC YA NIEUNCIEUC
    ((0xA0, 0x4F), (0xC7CF)),  # HANGUL SYLLABLE CIEUC YA TIKEUT
    ((0xA0, 0x50), (0xC7D1)),  # HANGUL SYLLABLE CIEUC YA RIEULKIYEOK
    ((0xA0, 0x51), (0xC7D2)),  # HANGUL SYLLABLE CIEUC YA RIEULMIEUM
    ((0xA0, 0x52), (0xC7D3)),  # HANGUL SYLLABLE CIEUC YA RIEULPIEUP
    ((0xA0, 0x53), (0xC7D4)),  # HANGUL SYLLABLE CIEUC YA RIEULSIOS
    ((0xA0, 0x54), (0xC7D5)),  # HANGUL SYLLABLE CIEUC YA RIEULTHIEUTH
    ((0xA0, 0x55), (0xC7D6)),  # HANGUL SYLLABLE CIEUC YA RIEULPHIEUPH
    ((0xA0, 0x56), (0xC7D7)),  # HANGUL SYLLABLE CIEUC YA RIEULHIEUH
    ((0xA0, 0x57), (0xC7D9)),  # HANGUL SYLLABLE CIEUC YA PIEUP
    ((0xA0, 0x58), (0xC7DA)),  # HANGUL SYLLABLE CIEUC YA PIEUPSIOS
    ((0xA0, 0x59), (0xC7DB)),  # HANGUL SYLLABLE CIEUC YA SIOS
    ((0xA0, 0x5A), (0xC7DC)),  # HANGUL SYLLABLE CIEUC YA SSANGSIOS
    ((0xA0, 0x61), (0xC7DE)),  # HANGUL SYLLABLE CIEUC YA CIEUC
    ((0xA0, 0x62), (0xC7DF)),  # HANGUL SYLLABLE CIEUC YA CHIEUCH
    ((0xA0, 0x63), (0xC7E0)),  # HANGUL SYLLABLE CIEUC YA KHIEUKH
    ((0xA0, 0x64), (0xC7E1)),  # HANGUL SYLLABLE CIEUC YA THIEUTH
    ((0xA0, 0x65), (0xC7E2)),  # HANGUL SYLLABLE CIEUC YA PHIEUPH
    ((0xA0, 0x66), (0xC7E3)),  # HANGUL SYLLABLE CIEUC YA HIEUH
    ((0xA0, 0x67), (0xC7E5)),  # HANGUL SYLLABLE CIEUC YAE KIYEOK
    ((0xA0, 0x68), (0xC7E6)),  # HANGUL SYLLABLE CIEUC YAE SSANGKIYEOK
    ((0xA0, 0x69), (0xC7E7)),  # HANGUL SYLLABLE CIEUC YAE KIYEOKSIOS
    ((0xA0, 0x6A), (0xC7E9)),  # HANGUL SYLLABLE CIEUC YAE NIEUNCIEUC
    ((0xA0, 0x6B), (0xC7EA)),  # HANGUL SYLLABLE CIEUC YAE NIEUNHIEUH
    ((0xA0, 0x6C), (0xC7EB)),  # HANGUL SYLLABLE CIEUC YAE TIKEUT
    ((0xA0, 0x6D), (0xC7ED)),  # HANGUL SYLLABLE CIEUC YAE RIEULKIYEOK
    ((0xA0, 0x6E), (0xC7EE)),  # HANGUL SYLLABLE CIEUC YAE RIEULMIEUM
    ((0xA0, 0x6F), (0xC7EF)),  # HANGUL SYLLABLE CIEUC YAE RIEULPIEUP
    ((0xA0, 0x70), (0xC7F0)),  # HANGUL SYLLABLE CIEUC YAE RIEULSIOS
    ((0xA0, 0x71), (0xC7F1)),  # HANGUL SYLLABLE CIEUC YAE RIEULTHIEUTH
    ((0xA0, 0x72), (0xC7F2)),  # HANGUL SYLLABLE CIEUC YAE RIEULPHIEUPH
    ((0xA0, 0x73), (0xC7F3)),  # HANGUL SYLLABLE CIEUC YAE RIEULHIEUH
    ((0xA0, 0x74), (0xC7F4)),  # HANGUL SYLLABLE CIEUC YAE MIEUM
    ((0xA0, 0x75), (0xC7F5)),  # HANGUL SYLLABLE CIEUC YAE PIEUP
    ((0xA0, 0x76), (0xC7F6)),  # HANGUL SYLLABLE CIEUC YAE PIEUPSIOS
    ((0xA0, 0x77), (0xC7F7)),  # HANGUL SYLLABLE CIEUC YAE SIOS
    ((0xA0, 0x78), (0xC7F8)),  # HANGUL SYLLABLE CIEUC YAE SSANGSIOS
    ((0xA0, 0x79), (0xC7F9)),  # HANGUL SYLLABLE CIEUC YAE IEUNG
    ((0xA0, 0x7A), (0xC7FA)),  # HANGUL SYLLABLE CIEUC YAE CIEUC
    ((0xA0, 0x81), (0xC7FB)),  # HANGUL SYLLABLE CIEUC YAE CHIEUCH
    ((0xA0, 0x82), (0xC7FC)),  # HANGUL SYLLABLE CIEUC YAE KHIEUKH
    ((0xA0, 0x83), (0xC7FD)),  # HANGUL SYLLABLE CIEUC YAE THIEUTH
    ((0xA0, 0x84), (0xC7FE)),  # HANGUL SYLLABLE CIEUC YAE PHIEUPH
    ((0xA0, 0x85), (0xC7FF)),  # HANGUL SYLLABLE CIEUC YAE HIEUH
    ((0xA0, 0x86), (0xC802)),  # HANGUL SYLLABLE CIEUC EO SSANGKIYEOK
    ((0xA0, 0x87), (0xC803)),  # HANGUL SYLLABLE CIEUC EO KIYEOKSIOS
    ((0xA0, 0x88), (0xC805)),  # HANGUL SYLLABLE CIEUC EO NIEUNCIEUC
    ((0xA0, 0x89), (0xC806)),  # HANGUL SYLLABLE CIEUC EO NIEUNHIEUH
    ((0xA0, 0x8A), (0xC807)),  # HANGUL SYLLABLE CIEUC EO TIKEUT
    ((0xA0, 0x8B), (0xC809)),  # HANGUL SYLLABLE CIEUC EO RIEULKIYEOK
    ((0xA0, 0x8C), (0xC80B)),  # HANGUL SYLLABLE CIEUC EO RIEULPIEUP
    ((0xA0, 0x8D), (0xC80C)),  # HANGUL SYLLABLE CIEUC EO RIEULSIOS
    ((0xA0, 0x8E), (0xC80D)),  # HANGUL SYLLABLE CIEUC EO RIEULTHIEUTH
    ((0xA0, 0x8F), (0xC80E)),  # HANGUL SYLLABLE CIEUC EO RIEULPHIEUPH
    ((0xA0, 0x90), (0xC80F)),  # HANGUL SYLLABLE CIEUC EO RIEULHIEUH
    ((0xA0, 0x91), (0xC812)),  # HANGUL SYLLABLE CIEUC EO PIEUPSIOS
    ((0xA0, 0x92), (0xC814)),  # HANGUL SYLLABLE CIEUC EO SSANGSIOS
    ((0xA0, 0x93), (0xC817)),  # HANGUL SYLLABLE CIEUC EO CHIEUCH
    ((0xA0, 0x94), (0xC818)),  # HANGUL SYLLABLE CIEUC EO KHIEUKH
    ((0xA0, 0x95), (0xC819)),  # HANGUL SYLLABLE CIEUC EO THIEUTH
    ((0xA0, 0x96), (0xC81A)),  # HANGUL SYLLABLE CIEUC EO PHIEUPH
    ((0xA0, 0x97), (0xC81B)),  # HANGUL SYLLABLE CIEUC EO HIEUH
    ((0xA0, 0x98), (0xC81E)),  # HANGUL SYLLABLE CIEUC E SSANGKIYEOK
    ((0xA0, 0x99), (0xC81F)),  # HANGUL SYLLABLE CIEUC E KIYEOKSIOS
    ((0xA0, 0x9A), (0xC821)),  # HANGUL SYLLABLE CIEUC E NIEUNCIEUC
    ((0xA0, 0x9B), (0xC822)),  # HANGUL SYLLABLE CIEUC E NIEUNHIEUH
    ((0xA0, 0x9C), (0xC823)),  # HANGUL SYLLABLE CIEUC E TIKEUT
    ((0xA0, 0x9D), (0xC825)),  # HANGUL SYLLABLE CIEUC E RIEULKIYEOK
    ((0xA0, 0x9E), (0xC826)),  # HANGUL SYLLABLE CIEUC E RIEULMIEUM
    ((0xA0, 0x9F), (0xC827)),  # HANGUL SYLLABLE CIEUC E RIEULPIEUP
    ((0xA0, 0xA0), (0xC828)),  # HANGUL SYLLABLE CIEUC E RIEULSIOS
    ((0xA0, 0xA1), (0xC829)),  # HANGUL SYLLABLE CIEUC E RIEULTHIEUTH
    ((0xA0, 0xA2), (0xC82A)),  # HANGUL SYLLABLE CIEUC E RIEULPHIEUPH
    ((0xA0, 0xA3), (0xC82B)),  # HANGUL SYLLABLE CIEUC E RIEULHIEUH
    ((0xA0, 0xA4), (0xC82E)),  # HANGUL SYLLABLE CIEUC E PIEUPSIOS
    ((0xA0, 0xA5), (0xC830)),  # HANGUL SYLLABLE CIEUC E SSANGSIOS
    ((0xA0, 0xA6), (0xC832)),  # HANGUL SYLLABLE CIEUC E CIEUC
    ((0xA0, 0xA7), (0xC833)),  # HANGUL SYLLABLE CIEUC E CHIEUCH
    ((0xA0, 0xA8), (0xC834)),  # HANGUL SYLLABLE CIEUC E KHIEUKH
    ((0xA0, 0xA9), (0xC835)),  # HANGUL SYLLABLE CIEUC E THIEUTH
    ((0xA0, 0xAA), (0xC836)),  # HANGUL SYLLABLE CIEUC E PHIEUPH
    ((0xA0, 0xAB), (0xC837)),  # HANGUL SYLLABLE CIEUC E HIEUH
    ((0xA0, 0xAC), (0xC839)),  # HANGUL SYLLABLE CIEUC YEO KIYEOK
    ((0xA0, 0xAD), (0xC83A)),  # HANGUL SYLLABLE CIEUC YEO SSANGKIYEOK
    ((0xA0, 0xAE), (0xC83B)),  # HANGUL SYLLABLE CIEUC YEO KIYEOKSIOS
    ((0xA0, 0xAF), (0xC83D)),  # HANGUL SYLLABLE CIEUC YEO NIEUNCIEUC
    ((0xA0, 0xB0), (0xC83E)),  # HANGUL SYLLABLE CIEUC YEO NIEUNHIEUH
    ((0xA0, 0xB1), (0xC83F)),  # HANGUL SYLLABLE CIEUC YEO TIKEUT
    ((0xA0, 0xB2), (0xC841)),  # HANGUL SYLLABLE CIEUC YEO RIEULKIYEOK
    ((0xA0, 0xB3), (0xC842)),  # HANGUL SYLLABLE CIEUC YEO RIEULMIEUM
    ((0xA0, 0xB4), (0xC843)),  # HANGUL SYLLABLE CIEUC YEO RIEULPIEUP
    ((0xA0, 0xB5), (0xC844)),  # HANGUL SYLLABLE CIEUC YEO RIEULSIOS
    ((0xA0, 0xB6), (0xC845)),  # HANGUL SYLLABLE CIEUC YEO RIEULTHIEUTH
    ((0xA0, 0xB7), (0xC846)),  # HANGUL SYLLABLE CIEUC YEO RIEULPHIEUPH
    ((0xA0, 0xB8), (0xC847)),  # HANGUL SYLLABLE CIEUC YEO RIEULHIEUH
    ((0xA0, 0xB9), (0xC84A)),  # HANGUL SYLLABLE CIEUC YEO PIEUPSIOS
    ((0xA0, 0xBA), (0xC84B)),  # HANGUL SYLLABLE CIEUC YEO SIOS
    ((0xA0, 0xBB), (0xC84E)),  # HANGUL SYLLABLE CIEUC YEO CIEUC
    ((0xA0, 0xBC), (0xC84F)),  # HANGUL SYLLABLE CIEUC YEO CHIEUCH
    ((0xA0, 0xBD), (0xC850)),  # HANGUL SYLLABLE CIEUC YEO KHIEUKH
    ((0xA0, 0xBE), (0xC851)),  # HANGUL SYLLABLE CIEUC YEO THIEUTH
    ((0xA0, 0xBF), (0xC852)),  # HANGUL SYLLABLE CIEUC YEO PHIEUPH
    ((0xA0, 0xC0), (0xC853)),  # HANGUL SYLLABLE CIEUC YEO HIEUH
    ((0xA0, 0xC1), (0xC855)),  # HANGUL SYLLABLE CIEUC YE KIYEOK
    ((0xA0, 0xC2), (0xC856)),  # HANGUL SYLLABLE CIEUC YE SSANGKIYEOK
    ((0xA0, 0xC3), (0xC857)),  # HANGUL SYLLABLE CIEUC YE KIYEOKSIOS
    ((0xA0, 0xC4), (0xC858)),  # HANGUL SYLLABLE CIEUC YE NIEUN
    ((0xA0, 0xC5), (0xC859)),  # HANGUL SYLLABLE CIEUC YE NIEUNCIEUC
    ((0xA0, 0xC6), (0xC85A)),  # HANGUL SYLLABLE CIEUC YE NIEUNHIEUH
    ((0xA0, 0xC7), (0xC85B)),  # HANGUL SYLLABLE CIEUC YE TIKEUT
    ((0xA0, 0xC8), (0xC85C)),  # HANGUL SYLLABLE CIEUC YE RIEUL
    ((0xA0, 0xC9), (0xC85D)),  # HANGUL SYLLABLE CIEUC YE RIEULKIYEOK
    ((0xA0, 0xCA), (0xC85E)),  # HANGUL SYLLABLE CIEUC YE RIEULMIEUM
    ((0xA0, 0xCB), (0xC85F)),  # HANGUL SYLLABLE CIEUC YE RIEULPIEUP
    ((0xA0, 0xCC), (0xC860)),  # HANGUL SYLLABLE CIEUC YE RIEULSIOS
    ((0xA0, 0xCD), (0xC861)),  # HANGUL SYLLABLE CIEUC YE RIEULTHIEUTH
    ((0xA0, 0xCE), (0xC862)),  # HANGUL SYLLABLE CIEUC YE RIEULPHIEUPH
    ((0xA0, 0xCF), (0xC863)),  # HANGUL SYLLABLE CIEUC YE RIEULHIEUH
    ((0xA0, 0xD0), (0xC864)),  # HANGUL SYLLABLE CIEUC YE MIEUM
    ((0xA0, 0xD1), (0xC865)),  # HANGUL SYLLABLE CIEUC YE PIEUP
    ((0xA0, 0xD2), (0xC866)),  # HANGUL SYLLABLE CIEUC YE PIEUPSIOS
    ((0xA0, 0xD3), (0xC867)),  # HANGUL SYLLABLE CIEUC YE SIOS
    ((0xA0, 0xD4), (0xC868)),  # HANGUL SYLLABLE CIEUC YE SSANGSIOS
    ((0xA0, 0xD5), (0xC869)),  # HANGUL SYLLABLE CIEUC YE IEUNG
    ((0xA0, 0xD6), (0xC86A)),  # HANGUL SYLLABLE CIEUC YE CIEUC
    ((0xA0, 0xD7), (0xC86B)),  # HANGUL SYLLABLE CIEUC YE CHIEUCH
    ((0xA0, 0xD8), (0xC86C)),  # HANGUL SYLLABLE CIEUC YE KHIEUKH
    ((0xA0, 0xD9), (0xC86D)),  # HANGUL SYLLABLE CIEUC YE THIEUTH
    ((0xA0, 0xDA), (0xC86E)),  # HANGUL SYLLABLE CIEUC YE PHIEUPH
    ((0xA0, 0xDB), (0xC86F)),  # HANGUL SYLLABLE CIEUC YE HIEUH
    ((0xA0, 0xDC), (0xC872)),  # HANGUL SYLLABLE CIEUC O SSANGKIYEOK
    ((0xA0, 0xDD), (0xC873)),  # HANGUL SYLLABLE CIEUC O KIYEOKSIOS
    ((0xA0, 0xDE), (0xC875)),  # HANGUL SYLLABLE CIEUC O NIEUNCIEUC
    ((0xA0, 0xDF), (0xC876)),  # HANGUL SYLLABLE CIEUC O NIEUNHIEUH
    ((0xA0, 0xE0), (0xC877)),  # HANGUL SYLLABLE CIEUC O TIKEUT
    ((0xA0, 0xE1), (0xC879)),  # HANGUL SYLLABLE CIEUC O RIEULKIYEOK
    ((0xA0, 0xE2), (0xC87B)),  # HANGUL SYLLABLE CIEUC O RIEULPIEUP
    ((0xA0, 0xE3), (0xC87C)),  # HANGUL SYLLABLE CIEUC O RIEULSIOS
    ((0xA0, 0xE4), (0xC87D)),  # HANGUL SYLLABLE CIEUC O RIEULTHIEUTH
    ((0xA0, 0xE5), (0xC87E)),  # HANGUL SYLLABLE CIEUC O RIEULPHIEUPH
    ((0xA0, 0xE6), (0xC87F)),  # HANGUL SYLLABLE CIEUC O RIEULHIEUH
    ((0xA0, 0xE7), (0xC882)),  # HANGUL SYLLABLE CIEUC O PIEUPSIOS
    ((0xA0, 0xE8), (0xC884)),  # HANGUL SYLLABLE CIEUC O SSANGSIOS
    ((0xA0, 0xE9), (0xC888)),  # HANGUL SYLLABLE CIEUC O KHIEUKH
    ((0xA0, 0xEA), (0xC889)),  # HANGUL SYLLABLE CIEUC O THIEUTH
    ((0xA0, 0xEB), (0xC88A)),  # HANGUL SYLLABLE CIEUC O PHIEUPH
    ((0xA0, 0xEC), (0xC88E)),  # HANGUL SYLLABLE CIEUC WA SSANGKIYEOK
    ((0xA0, 0xED), (0xC88F)),  # HANGUL SYLLABLE CIEUC WA KIYEOKSIOS
    ((0xA0, 0xEE), (0xC890)),  # HANGUL SYLLABLE CIEUC WA NIEUN
    ((0xA0, 0xEF), (0xC891)),  # HANGUL SYLLABLE CIEUC WA NIEUNCIEUC
    ((0xA0, 0xF0), (0xC892)),  # HANGUL SYLLABLE CIEUC WA NIEUNHIEUH
    ((0xA0, 0xF1), (0xC893)),  # HANGUL SYLLABLE CIEUC WA TIKEUT
    ((0xA0, 0xF2), (0xC895)),  # HANGUL SYLLABLE CIEUC WA RIEULKIYEOK
    ((0xA0, 0xF3), (0xC896)),  # HANGUL SYLLABLE CIEUC WA RIEULMIEUM
    ((0xA0, 0xF4), (0xC897)),  # HANGUL SYLLABLE CIEUC WA RIEULPIEUP
    ((0xA0, 0xF5), (0xC898)),  # HANGUL SYLLABLE CIEUC WA RIEULSIOS
    ((0xA0, 0xF6), (0xC899)),  # HANGUL SYLLABLE CIEUC WA RIEULTHIEUTH
    ((0xA0, 0xF7), (0xC89A)),  # HANGUL SYLLABLE CIEUC WA RIEULPHIEUPH
    ((0xA0, 0xF8), (0xC89B)),  # HANGUL SYLLABLE CIEUC WA RIEULHIEUH
    ((0xA0, 0xF9), (0xC89C)),  # HANGUL SYLLABLE CIEUC WA MIEUM
    ((0xA0, 0xFA), (0xC89E)),  # HANGUL SYLLABLE CIEUC WA PIEUPSIOS
    ((0xA0, 0xFB), (0xC8A0)),  # HANGUL SYLLABLE CIEUC WA SSANGSIOS
    ((0xA0, 0xFC), (0xC8A2)),  # HANGUL SYLLABLE CIEUC WA CIEUC
    ((0xA0, 0xFD), (0xC8A3)),  # HANGUL SYLLABLE CIEUC WA CHIEUCH
    ((0xA0, 0xFE), (0xC8A4)),  # HANGUL SYLLABLE CIEUC WA KHIEUKH
    ((0xA1, 0x41), (0xC8A5)),  # HANGUL SYLLABLE CIEUC WA THIEUTH
    ((0xA1, 0x42), (0xC8A6)),  # HANGUL SYLLABLE CIEUC WA PHIEUPH
    ((0xA1, 0x43), (0xC8A7)),  # HANGUL SYLLABLE CIEUC WA HIEUH
    ((0xA1, 0x44), (0xC8A9)),  # HANGUL SYLLABLE CIEUC WAE KIYEOK
    ((0xA1, 0x45), (0xC8AA)),  # HANGUL SYLLABLE CIEUC WAE SSANGKIYEOK
    ((0xA1, 0x46), (0xC8AB)),  # HANGUL SYLLABLE CIEUC WAE KIYEOKSIOS
    ((0xA1, 0x47), (0xC8AC)),  # HANGUL SYLLABLE CIEUC WAE NIEUN
    ((0xA1, 0x48), (0xC8AD)),  # HANGUL SYLLABLE CIEUC WAE NIEUNCIEUC
    ((0xA1, 0x49), (0xC8AE)),  # HANGUL SYLLABLE CIEUC WAE NIEUNHIEUH
    ((0xA1, 0x4A), (0xC8AF)),  # HANGUL SYLLABLE CIEUC WAE TIKEUT
    ((0xA1, 0x4B), (0xC8B0)),  # HANGUL SYLLABLE CIEUC WAE RIEUL
    ((0xA1, 0x4C), (0xC8B1)),  # HANGUL SYLLABLE CIEUC WAE RIEULKIYEOK
    ((0xA1, 0x4D), (0xC8B2)),  # HANGUL SYLLABLE CIEUC WAE RIEULMIEUM
    ((0xA1, 0x4E), (0xC8B3)),  # HANGUL SYLLABLE CIEUC WAE RIEULPIEUP
    ((0xA1, 0x4F), (0xC8B4)),  # HANGUL SYLLABLE CIEUC WAE RIEULSIOS
    ((0xA1, 0x50), (0xC8B5)),  # HANGUL SYLLABLE CIEUC WAE RIEULTHIEUTH
    ((0xA1, 0x51), (0xC8B6)),  # HANGUL SYLLABLE CIEUC WAE RIEULPHIEUPH
    ((0xA1, 0x52), (0xC8B7)),  # HANGUL SYLLABLE CIEUC WAE RIEULHIEUH
    ((0xA1, 0x53), (0xC8B8)),  # HANGUL SYLLABLE CIEUC WAE MIEUM
    ((0xA1, 0x54), (0xC8B9)),  # HANGUL SYLLABLE CIEUC WAE PIEUP
    ((0xA1, 0x55), (0xC8BA)),  # HANGUL SYLLABLE CIEUC WAE PIEUPSIOS
    ((0xA1, 0x56), (0xC8BB)),  # HANGUL SYLLABLE CIEUC WAE SIOS
    ((0xA1, 0x57), (0xC8BE)),  # HANGUL SYLLABLE CIEUC WAE CIEUC
    ((0xA1, 0x58), (0xC8BF)),  # HANGUL SYLLABLE CIEUC WAE CHIEUCH
    ((0xA1, 0x59), (0xC8C0)),  # HANGUL SYLLABLE CIEUC WAE KHIEUKH
    ((0xA1, 0x5A), (0xC8C1)),  # HANGUL SYLLABLE CIEUC WAE THIEUTH
    ((0xA1, 0x61), (0xC8C2)),  # HANGUL SYLLABLE CIEUC WAE PHIEUPH
    ((0xA1, 0x62), (0xC8C3)),  # HANGUL SYLLABLE CIEUC WAE HIEUH
    ((0xA1, 0x63), (0xC8C5)),  # HANGUL SYLLABLE CIEUC OE KIYEOK
    ((0xA1, 0x64), (0xC8C6)),  # HANGUL SYLLABLE CIEUC OE SSANGKIYEOK
    ((0xA1, 0x65), (0xC8C7)),  # HANGUL SYLLABLE CIEUC OE KIYEOKSIOS
    ((0xA1, 0x66), (0xC8C9)),  # HANGUL SYLLABLE CIEUC OE NIEUNCIEUC
    ((0xA1, 0x67), (0xC8CA)),  # HANGUL SYLLABLE CIEUC OE NIEUNHIEUH
    ((0xA1, 0x68), (0xC8CB)),  # HANGUL SYLLABLE CIEUC OE TIKEUT
    ((0xA1, 0x69), (0xC8CD)),  # HANGUL SYLLABLE CIEUC OE RIEULKIYEOK
    ((0xA1, 0x6A), (0xC8CE)),  # HANGUL SYLLABLE CIEUC OE RIEULMIEUM
    ((0xA1, 0x6B), (0xC8CF)),  # HANGUL SYLLABLE CIEUC OE RIEULPIEUP
    ((0xA1, 0x6C), (0xC8D0)),  # HANGUL SYLLABLE CIEUC OE RIEULSIOS
    ((0xA1, 0x6D), (0xC8D1)),  # HANGUL SYLLABLE CIEUC OE RIEULTHIEUTH
    ((0xA1, 0x6E), (0xC8D2)),  # HANGUL SYLLABLE CIEUC OE RIEULPHIEUPH
    ((0xA1, 0x6F), (0xC8D3)),  # HANGUL SYLLABLE CIEUC OE RIEULHIEUH
    ((0xA1, 0x70), (0xC8D6)),  # HANGUL SYLLABLE CIEUC OE PIEUPSIOS
    ((0xA1, 0x71), (0xC8D8)),  # HANGUL SYLLABLE CIEUC OE SSANGSIOS
    ((0xA1, 0x72), (0xC8DA)),  # HANGUL SYLLABLE CIEUC OE CIEUC
    ((0xA1, 0x73), (0xC8DB)),  # HANGUL SYLLABLE CIEUC OE CHIEUCH
    ((0xA1, 0x74), (0xC8DC)),  # HANGUL SYLLABLE CIEUC OE KHIEUKH
    ((0xA1, 0x75), (0xC8DD)),  # HANGUL SYLLABLE CIEUC OE THIEUTH
    ((0xA1, 0x76), (0xC8DE)),  # HANGUL SYLLABLE CIEUC OE PHIEUPH
    ((0xA1, 0x77), (0xC8DF)),  # HANGUL SYLLABLE CIEUC OE HIEUH
    ((0xA1, 0x78), (0xC8E2)),  # HANGUL SYLLABLE CIEUC YO SSANGKIYEOK
    ((0xA1, 0x79), (0xC8E3)),  # HANGUL SYLLABLE CIEUC YO KIYEOKSIOS
    ((0xA1, 0x7A), (0xC8E5)),  # HANGUL SYLLABLE CIEUC YO NIEUNCIEUC
    ((0xA1, 0x81), (0xC8E6)),  # HANGUL SYLLABLE CIEUC YO NIEUNHIEUH
    ((0xA1, 0x82), (0xC8E7)),  # HANGUL SYLLABLE CIEUC YO TIKEUT
    ((0xA1, 0x83), (0xC8E8)),  # HANGUL SYLLABLE CIEUC YO RIEUL
    ((0xA1, 0x84), (0xC8E9)),  # HANGUL SYLLABLE CIEUC YO RIEULKIYEOK
    ((0xA1, 0x85), (0xC8EA)),  # HANGUL SYLLABLE CIEUC YO RIEULMIEUM
    ((0xA1, 0x86), (0xC8EB)),  # HANGUL SYLLABLE CIEUC YO RIEULPIEUP
    ((0xA1, 0x87), (0xC8EC)),  # HANGUL SYLLABLE CIEUC YO RIEULSIOS
    ((0xA1, 0x88), (0xC8ED)),  # HANGUL SYLLABLE CIEUC YO RIEULTHIEUTH
    ((0xA1, 0x89), (0xC8EE)),  # HANGUL SYLLABLE CIEUC YO RIEULPHIEUPH
    ((0xA1, 0x8A), (0xC8EF)),  # HANGUL SYLLABLE CIEUC YO RIEULHIEUH
    ((0xA1, 0x8B), (0xC8F0)),  # HANGUL SYLLABLE CIEUC YO MIEUM
    ((0xA1, 0x8C), (0xC8F1)),  # HANGUL SYLLABLE CIEUC YO PIEUP
    ((0xA1, 0x8D), (0xC8F2)),  # HANGUL SYLLABLE CIEUC YO PIEUPSIOS
    ((0xA1, 0x8E), (0xC8F3)),  # HANGUL SYLLABLE CIEUC YO SIOS
    ((0xA1, 0x8F), (0xC8F4)),  # HANGUL SYLLABLE CIEUC YO SSANGSIOS
    ((0xA1, 0x90), (0xC8F6)),  # HANGUL SYLLABLE CIEUC YO CIEUC
    ((0xA1, 0x91), (0xC8F7)),  # HANGUL SYLLABLE CIEUC YO CHIEUCH
    ((0xA1, 0x92), (0xC8F8)),  # HANGUL SYLLABLE CIEUC YO KHIEUKH
    ((0xA1, 0x93), (0xC8F9)),  # HANGUL SYLLABLE CIEUC YO THIEUTH
    ((0xA1, 0x94), (0xC8FA)),  # HANGUL SYLLABLE CIEUC YO PHIEUPH
    ((0xA1, 0x95), (0xC8FB)),  # HANGUL SYLLABLE CIEUC YO HIEUH
    ((0xA1, 0x96), (0xC8FE)),  # HANGUL SYLLABLE CIEUC U SSANGKIYEOK
    ((0xA1, 0x97), (0xC8FF)),  # HANGUL SYLLABLE CIEUC U KIYEOKSIOS
    ((0xA1, 0x98), (0xC901)),  # HANGUL SYLLABLE CIEUC U NIEUNCIEUC
    ((0xA1, 0x99), (0xC902)),  # HANGUL SYLLABLE CIEUC U NIEUNHIEUH
    ((0xA1, 0x9A), (0xC903)),  # HANGUL SYLLABLE CIEUC U TIKEUT
    ((0xA1, 0x9B), (0xC907)),  # HANGUL SYLLABLE CIEUC U RIEULPIEUP
    ((0xA1, 0x9C), (0xC908)),  # HANGUL SYLLABLE CIEUC U RIEULSIOS
    ((0xA1, 0x9D), (0xC909)),  # HANGUL SYLLABLE CIEUC U RIEULTHIEUTH
    ((0xA1, 0x9E), (0xC90A)),  # HANGUL SYLLABLE CIEUC U RIEULPHIEUPH
    ((0xA1, 0x9F), (0xC90B)),  # HANGUL SYLLABLE CIEUC U RIEULHIEUH
    ((0xA1, 0xA0), (0xC90E)),  # HANGUL SYLLABLE CIEUC U PIEUPSIOS
    ((0xA1, 0xA1), (0x3000)),  # IDEOGRAPHIC SPACE
    ((0xA1, 0xA2), (0x3001)),  # IDEOGRAPHIC COMMA
    ((0xA1, 0xA3), (0x3002)),  # IDEOGRAPHIC FULL STOP
    ((0xA1, 0xA4), (0x00B7)),  # MIDDLE DOT
    ((0xA1, 0xA5), (0x2025)),  # TWO DOT LEADER
    ((0xA1, 0xA6), (0x2026)),  # HORIZONTAL ELLIPSIS
    ((0xA1, 0xA7), (0x00A8)),  # DIAERESIS
    ((0xA1, 0xA8), (0x3003)),  # DITTO MARK
    ((0xA1, 0xA9), (0x00AD)),  # SOFT HYPHEN
    ((0xA1, 0xAA), (0x2015)),  # HORIZONTAL BAR
    ((0xA1, 0xAB), (0x2225)),  # PARALLEL TO
    ((0xA1, 0xAC), (0xFF3C)),  # FULLWIDTH REVERSE SOLIDUS
    ((0xA1, 0xAD), (0x223C)),  # TILDE OPERATOR
    ((0xA1, 0xAE), (0x2018)),  # LEFT SINGLE QUOTATION MARK
    ((0xA1, 0xAF), (0x2019)),  # RIGHT SINGLE QUOTATION MARK
    ((0xA1, 0xB0), (0x201C)),  # LEFT DOUBLE QUOTATION MARK
    ((0xA1, 0xB1), (0x201D)),  # RIGHT DOUBLE QUOTATION MARK
    ((0xA1, 0xB2), (0x3014)),  # LEFT TORTOISE SHELL BRACKET
    ((0xA1, 0xB3), (0x3015)),  # RIGHT TORTOISE SHELL BRACKET
    ((0xA1, 0xB4), (0x3008)),  # LEFT ANGLE BRACKET
    ((0xA1, 0xB5), (0x3009)),  # RIGHT ANGLE BRACKET
    ((0xA1, 0xB6), (0x300A)),  # LEFT DOUBLE ANGLE BRACKET
    ((0xA1, 0xB7), (0x300B)),  # RIGHT DOUBLE ANGLE BRACKET
    ((0xA1, 0xB8), (0x300C)),  # LEFT CORNER BRACKET
    ((0xA1, 0xB9), (0x300D)),  # RIGHT CORNER BRACKET
    ((0xA1, 0xBA), (0x300E)),  # LEFT WHITE CORNER BRACKET
    ((0xA1, 0xBB), (0x300F)),  # RIGHT WHITE CORNER BRACKET
    ((0xA1, 0xBC), (0x3010)),  # LEFT BLACK LENTICULAR BRACKET
    ((0xA1, 0xBD), (0x3011)),  # RIGHT BLACK LENTICULAR BRACKET
    ((0xA1, 0xBE), (0x00B1)),  # PLUS-MINUS SIGN
    ((0xA1, 0xBF), (0x00D7)),  # MULTIPLICATION SIGN
    ((0xA1, 0xC0), (0x00F7)),  # DIVISION SIGN
    ((0xA1, 0xC1), (0x2260)),  # NOT EQUAL TO
    ((0xA1, 0xC2), (0x2264)),  # LESS-THAN OR EQUAL TO
    ((0xA1, 0xC3), (0x2265)),  # GREATER-THAN OR EQUAL TO
    ((0xA1, 0xC4), (0x221E)),  # INFINITY
    ((0xA1, 0xC5), (0x2234)),  # THEREFORE
    ((0xA1, 0xC6), (0x00B0)),  # DEGREE SIGN
    ((0xA1, 0xC7), (0x2032)),  # PRIME
    ((0xA1, 0xC8), (0x2033)),  # DOUBLE PRIME
    ((0xA1, 0xC9), (0x2103)),  # DEGREE CELSIUS
    ((0xA1, 0xCA), (0x212B)),  # ANGSTROM SIGN
    ((0xA1, 0xCB), (0xFFE0)),  # FULLWIDTH CENT SIGN
    ((0xA1, 0xCC), (0xFFE1)),  # FULLWIDTH POUND SIGN
    ((0xA1, 0xCD), (0xFFE5)),  # FULLWIDTH YEN SIGN
    ((0xA1, 0xCE), (0x2642)),  # MALE SIGN
    ((0xA1, 0xCF), (0x2640)),  # FEMALE SIGN
    ((0xA1, 0xD0), (0x2220)),  # ANGLE
    ((0xA1, 0xD1), (0x22A5)),  # UP TACK
    ((0xA1, 0xD2), (0x2312)),  # ARC
    ((0xA1, 0xD3), (0x2202)),  # PARTIAL DIFFERENTIAL
    ((0xA1, 0xD4), (0x2207)),  # NABLA
    ((0xA1, 0xD5), (0x2261)),  # IDENTICAL TO
    ((0xA1, 0xD6), (0x2252)),  # APPROXIMATELY EQUAL TO OR THE IMAGE OF
    ((0xA1, 0xD7), (0x00A7)),  # SECTION SIGN
    ((0xA1, 0xD8), (0x203B)),  # REFERENCE MARK
    ((0xA1, 0xD9), (0x2606)),  # WHITE STAR
    ((0xA1, 0xDA), (0x2605)),  # BLACK STAR
    ((0xA1, 0xDB), (0x25CB)),  # WHITE CIRCLE
    ((0xA1, 0xDC), (0x25CF)),  # BLACK CIRCLE
    ((0xA1, 0xDD), (0x25CE)),  # BULLSEYE
    ((0xA1, 0xDE), (0x25C7)),  # WHITE DIAMOND
    ((0xA1, 0xDF), (0x25C6)),  # BLACK DIAMOND
    ((0xA1, 0xE0), (0x25A1)),  # WHITE SQUARE
    ((0xA1, 0xE1), (0x25A0)),  # BLACK SQUARE
    ((0xA1, 0xE2), (0x25B3)),  # WHITE UP-POINTING TRIANGLE
    ((0xA1, 0xE3), (0x25B2)),  # BLACK UP-POINTING TRIANGLE
    ((0xA1, 0xE4), (0x25BD)),  # WHITE DOWN-POINTING TRIANGLE
    ((0xA1, 0xE5), (0x25BC)),  # BLACK DOWN-POINTING TRIANGLE
    ((0xA1, 0xE6), (0x2192)),  # RIGHTWARDS ARROW
    ((0xA1, 0xE7), (0x2190)),  # LEFTWARDS ARROW
    ((0xA1, 0xE8), (0x2191)),  # UPWARDS ARROW
    ((0xA1, 0xE9), (0x2193)),  # DOWNWARDS ARROW
    ((0xA1, 0xEA), (0x2194)),  # LEFT RIGHT ARROW
    ((0xA1, 0xEB), (0x3013)),  # GETA MARK
    ((0xA1, 0xEC), (0x226A)),  # MUCH LESS-THAN
    ((0xA1, 0xED), (0x226B)),  # MUCH GREATER-THAN
    ((0xA1, 0xEE), (0x221A)),  # SQUARE ROOT
    ((0xA1, 0xEF), (0x223D)),  # REVERSED TILDE
    ((0xA1, 0xF0), (0x221D)),  # PROPORTIONAL TO
    ((0xA1, 0xF1), (0x2235)),  # BECAUSE
    ((0xA1, 0xF2), (0x222B)),  # INTEGRAL
    ((0xA1, 0xF3), (0x222C)),  # DOUBLE INTEGRAL
    ((0xA1, 0xF4), (0x2208)),  # ELEMENT OF
    ((0xA1, 0xF5), (0x220B)),  # CONTAINS AS MEMBER
    ((0xA1, 0xF6), (0x2286)),  # SUBSET OF OR EQUAL TO
    ((0xA1, 0xF7), (0x2287)),  # SUPERSET OF OR EQUAL TO
    ((0xA1, 0xF8), (0x2282)),  # SUBSET OF
    ((0xA1, 0xF9), (0x2283)),  # SUPERSET OF
    ((0xA1, 0xFA), (0x222A)),  # UNION
    ((0xA1, 0xFB), (0x2229)),  # INTERSECTION
    ((0xA1, 0xFC), (0x2227)),  # LOGICAL AND
    ((0xA1, 0xFD), (0x2228)),  # LOGICAL OR
    ((0xA1, 0xFE), (0xFFE2)),  # FULLWIDTH NOT SIGN
    ((0xA2, 0x41), (0xC910)),  # HANGUL SYLLABLE CIEUC U SSANGSIOS
    ((0xA2, 0x42), (0xC912)),  # HANGUL SYLLABLE CIEUC U CIEUC
    ((0xA2, 0x43), (0xC913)),  # HANGUL SYLLABLE CIEUC U CHIEUCH
    ((0xA2, 0x44), (0xC914)),  # HANGUL SYLLABLE CIEUC U KHIEUKH
    ((0xA2, 0x45), (0xC915)),  # HANGUL SYLLABLE CIEUC U THIEUTH
    ((0xA2, 0x46), (0xC916)),  # HANGUL SYLLABLE CIEUC U PHIEUPH
    ((0xA2, 0x47), (0xC917)),  # HANGUL SYLLABLE CIEUC U HIEUH
    ((0xA2, 0x48), (0xC919)),  # HANGUL SYLLABLE CIEUC WEO KIYEOK
    ((0xA2, 0x49), (0xC91A)),  # HANGUL SYLLABLE CIEUC WEO SSANGKIYEOK
    ((0xA2, 0x4A), (0xC91B)),  # HANGUL SYLLABLE CIEUC WEO KIYEOKSIOS
    ((0xA2, 0x4B), (0xC91C)),  # HANGUL SYLLABLE CIEUC WEO NIEUN
    ((0xA2, 0x4C), (0xC91D)),  # HANGUL SYLLABLE CIEUC WEO NIEUNCIEUC
    ((0xA2, 0x4D), (0xC91E)),  # HANGUL SYLLABLE CIEUC WEO NIEUNHIEUH
    ((0xA2, 0x4E), (0xC91F)),  # HANGUL SYLLABLE CIEUC WEO TIKEUT
    ((0xA2, 0x4F), (0xC920)),  # HANGUL SYLLABLE CIEUC WEO RIEUL
    ((0xA2, 0x50), (0xC921)),  # HANGUL SYLLABLE CIEUC WEO RIEULKIYEOK
    ((0xA2, 0x51), (0xC922)),  # HANGUL SYLLABLE CIEUC WEO RIEULMIEUM
    ((0xA2, 0x52), (0xC923)),  # HANGUL SYLLABLE CIEUC WEO RIEULPIEUP
    ((0xA2, 0x53), (0xC924)),  # HANGUL SYLLABLE CIEUC WEO RIEULSIOS
    ((0xA2, 0x54), (0xC925)),  # HANGUL SYLLABLE CIEUC WEO RIEULTHIEUTH
    ((0xA2, 0x55), (0xC926)),  # HANGUL SYLLABLE CIEUC WEO RIEULPHIEUPH
    ((0xA2, 0x56), (0xC927)),  # HANGUL SYLLABLE CIEUC WEO RIEULHIEUH
    ((0xA2, 0x57), (0xC928)),  # HANGUL SYLLABLE CIEUC WEO MIEUM
    ((0xA2, 0x58), (0xC929)),  # HANGUL SYLLABLE CIEUC WEO PIEUP
    ((0xA2, 0x59), (0xC92A)),  # HANGUL SYLLABLE CIEUC WEO PIEUPSIOS
    ((0xA2, 0x5A), (0xC92B)),  # HANGUL SYLLABLE CIEUC WEO SIOS
    ((0xA2, 0x61), (0xC92D)),  # HANGUL SYLLABLE CIEUC WEO IEUNG
    ((0xA2, 0x62), (0xC92E)),  # HANGUL SYLLABLE CIEUC WEO CIEUC
    ((0xA2, 0x63), (0xC92F)),  # HANGUL SYLLABLE CIEUC WEO CHIEUCH
    ((0xA2, 0x64), (0xC930)),  # HANGUL SYLLABLE CIEUC WEO KHIEUKH
    ((0xA2, 0x65), (0xC931)),  # HANGUL SYLLABLE CIEUC WEO THIEUTH
    ((0xA2, 0x66), (0xC932)),  # HANGUL SYLLABLE CIEUC WEO PHIEUPH
    ((0xA2, 0x67), (0xC933)),  # HANGUL SYLLABLE CIEUC WEO HIEUH
    ((0xA2, 0x68), (0xC935)),  # HANGUL SYLLABLE CIEUC WE KIYEOK
    ((0xA2, 0x69), (0xC936)),  # HANGUL SYLLABLE CIEUC WE SSANGKIYEOK
    ((0xA2, 0x6A), (0xC937)),  # HANGUL SYLLABLE CIEUC WE KIYEOKSIOS
    ((0xA2, 0x6B), (0xC938)),  # HANGUL SYLLABLE CIEUC WE NIEUN
    ((0xA2, 0x6C), (0xC939)),  # HANGUL SYLLABLE CIEUC WE NIEUNCIEUC
    ((0xA2, 0x6D), (0xC93A)),  # HANGUL SYLLABLE CIEUC WE NIEUNHIEUH
    ((0xA2, 0x6E), (0xC93B)),  # HANGUL SYLLABLE CIEUC WE TIKEUT
    ((0xA2, 0x6F), (0xC93C)),  # HANGUL SYLLABLE CIEUC WE RIEUL
    ((0xA2, 0x70), (0xC93D)),  # HANGUL SYLLABLE CIEUC WE RIEULKIYEOK
    ((0xA2, 0x71), (0xC93E)),  # HANGUL SYLLABLE CIEUC WE RIEULMIEUM
    ((0xA2, 0x72), (0xC93F)),  # HANGUL SYLLABLE CIEUC WE RIEULPIEUP
    ((0xA2, 0x73), (0xC940)),  # HANGUL SYLLABLE CIEUC WE RIEULSIOS
    ((0xA2, 0x74), (0xC941)),  # HANGUL SYLLABLE CIEUC WE RIEULTHIEUTH
    ((0xA2, 0x75), (0xC942)),  # HANGUL SYLLABLE CIEUC WE RIEULPHIEUPH
    ((0xA2, 0x76), (0xC943)),  # HANGUL SYLLABLE CIEUC WE RIEULHIEUH
    ((0xA2, 0x77), (0xC944)),  # HANGUL SYLLABLE CIEUC WE MIEUM
    ((0xA2, 0x78), (0xC945)),  # HANGUL SYLLABLE CIEUC WE PIEUP
    ((0xA2, 0x79), (0xC946)),  # HANGUL SYLLABLE CIEUC WE PIEUPSIOS
    ((0xA2, 0x7A), (0xC947)),  # HANGUL SYLLABLE CIEUC WE SIOS
    ((0xA2, 0x81), (0xC948)),  # HANGUL SYLLABLE CIEUC WE SSANGSIOS
    ((0xA2, 0x82), (0xC949)),  # HANGUL SYLLABLE CIEUC WE IEUNG
    ((0xA2, 0x83), (0xC94A)),  # HANGUL SYLLABLE CIEUC WE CIEUC
    ((0xA2, 0x84), (0xC94B)),  # HANGUL SYLLABLE CIEUC WE CHIEUCH
    ((0xA2, 0x85), (0xC94C)),  # HANGUL SYLLABLE CIEUC WE KHIEUKH
    ((0xA2, 0x86), (0xC94D)),  # HANGUL SYLLABLE CIEUC WE THIEUTH
    ((0xA2, 0x87), (0xC94E)),  # HANGUL SYLLABLE CIEUC WE PHIEUPH
    ((0xA2, 0x88), (0xC94F)),  # HANGUL SYLLABLE CIEUC WE HIEUH
    ((0xA2, 0x89), (0xC952)),  # HANGUL SYLLABLE CIEUC WI SSANGKIYEOK
    ((0xA2, 0x8A), (0xC953)),  # HANGUL SYLLABLE CIEUC WI KIYEOKSIOS
    ((0xA2, 0x8B), (0xC955)),  # HANGUL SYLLABLE CIEUC WI NIEUNCIEUC
    ((0xA2, 0x8C), (0xC956)),  # HANGUL SYLLABLE CIEUC WI NIEUNHIEUH
    ((0xA2, 0x8D), (0xC957)),  # HANGUL SYLLABLE CIEUC WI TIKEUT
    ((0xA2, 0x8E), (0xC959)),  # HANGUL SYLLABLE CIEUC WI RIEULKIYEOK
    ((0xA2, 0x8F), (0xC95A)),  # HANGUL SYLLABLE CIEUC WI RIEULMIEUM
    ((0xA2, 0x90), (0xC95B)),  # HANGUL SYLLABLE CIEUC WI RIEULPIEUP
    ((0xA2, 0x91), (0xC95C)),  # HANGUL SYLLABLE CIEUC WI RIEULSIOS
    ((0xA2, 0x92), (0xC95D)),  # HANGUL SYLLABLE CIEUC WI RIEULTHIEUTH
    ((0xA2, 0x93), (0xC95E)),  # HANGUL SYLLABLE CIEUC WI RIEULPHIEUPH
    ((0xA2, 0x94), (0xC95F)),  # HANGUL SYLLABLE CIEUC WI RIEULHIEUH
    ((0xA2, 0x95), (0xC962)),  # HANGUL SYLLABLE CIEUC WI PIEUPSIOS
    ((0xA2, 0x96), (0xC964)),  # HANGUL SYLLABLE CIEUC WI SSANGSIOS
    ((0xA2, 0x97), (0xC965)),  # HANGUL SYLLABLE CIEUC WI IEUNG
    ((0xA2, 0x98), (0xC966)),  # HANGUL SYLLABLE CIEUC WI CIEUC
    ((0xA2, 0x99), (0xC967)),  # HANGUL SYLLABLE CIEUC WI CHIEUCH
    ((0xA2, 0x9A), (0xC968)),  # HANGUL SYLLABLE CIEUC WI KHIEUKH
    ((0xA2, 0x9B), (0xC969)),  # HANGUL SYLLABLE CIEUC WI THIEUTH
    ((0xA2, 0x9C), (0xC96A)),  # HANGUL SYLLABLE CIEUC WI PHIEUPH
    ((0xA2, 0x9D), (0xC96B)),  # HANGUL SYLLABLE CIEUC WI HIEUH
    ((0xA2, 0x9E), (0xC96D)),  # HANGUL SYLLABLE CIEUC YU KIYEOK
    ((0xA2, 0x9F), (0xC96E)),  # HANGUL SYLLABLE CIEUC YU SSANGKIYEOK
    ((0xA2, 0xA0), (0xC96F)),  # HANGUL SYLLABLE CIEUC YU KIYEOKSIOS
    ((0xA2, 0xA1), (0x21D2)),  # RIGHTWARDS DOUBLE ARROW
    ((0xA2, 0xA2), (0x21D4)),  # LEFT RIGHT DOUBLE ARROW
    ((0xA2, 0xA3), (0x2200)),  # FOR ALL
    ((0xA2, 0xA4), (0x2203)),  # THERE EXISTS
    ((0xA2, 0xA5), (0x00B4)),  # ACUTE ACCENT
    ((0xA2, 0xA6), (0xFF5E)),  # FULLWIDTH TILDE
    ((0xA2, 0xA7), (0x02C7)),  # CARON
    ((0xA2, 0xA8), (0x02D8)),  # BREVE
    ((0xA2, 0xA9), (0x02DD)),  # DOUBLE ACUTE ACCENT
    ((0xA2, 0xAA), (0x02DA)),  # RING ABOVE
    ((0xA2, 0xAB), (0x02D9)),  # DOT ABOVE
    ((0xA2, 0xAC), (0x00B8)),  # CEDILLA
    ((0xA2, 0xAD), (0x02DB)),  # OGONEK
    ((0xA2, 0xAE), (0x00A1)),  # INVERTED EXCLAMATION MARK
    ((0xA2, 0xAF), (0x00BF)),  # INVERTED QUESTION MARK
    ((0xA2, 0xB0), (0x02D0)),  # MODIFIER LETTER TRIANGULAR COLON
    ((0xA2, 0xB1), (0x222E)),  # CONTOUR INTEGRAL
    ((0xA2, 0xB2), (0x2211)),  # N-ARY SUMMATION
    ((0xA2, 0xB3), (0x220F)),  # N-ARY PRODUCT
    ((0xA2, 0xB4), (0x00A4)),  # CURRENCY SIGN
    ((0xA2, 0xB5), (0x2109)),  # DEGREE FAHRENHEIT
    ((0xA2, 0xB6), (0x2030)),  # PER MILLE SIGN
    ((0xA2, 0xB7), (0x25C1)),  # WHITE LEFT-POINTING TRIANGLE
    ((0xA2, 0xB8), (0x25C0)),  # BLACK LEFT-POINTING TRIANGLE
    ((0xA2, 0xB9), (0x25B7)),  # WHITE RIGHT-POINTING TRIANGLE
    ((0xA2, 0xBA), (0x25B6)),  # BLACK RIGHT-POINTING TRIANGLE
    ((0xA2, 0xBB), (0x2664)),  # WHITE SPADE SUIT
    ((0xA2, 0xBC), (0x2660)),  # BLACK SPADE SUIT
    ((0xA2, 0xBD), (0x2661)),  # WHITE HEART SUIT
    ((0xA2, 0xBE), (0x2665)),  # BLACK HEART SUIT
    ((0xA2, 0xBF), (0x2667)),  # WHITE CLUB SUIT
    ((0xA2, 0xC0), (0x2663)),  # BLACK CLUB SUIT
    ((0xA2, 0xC1), (0x2299)),  # CIRCLED DOT OPERATOR
    ((0xA2, 0xC2), (0x25C8)),  # WHITE DIAMOND CONTAINING BLACK SMALL DIAMOND
    ((0xA2, 0xC3), (0x25A3)),  # WHITE SQUARE CONTAINING BLACK SMALL SQUARE
    ((0xA2, 0xC4), (0x25D0)),  # CIRCLE WITH LEFT HALF BLACK
    ((0xA2, 0xC5), (0x25D1)),  # CIRCLE WITH RIGHT HALF BLACK
    ((0xA2, 0xC6), (0x2592)),  # MEDIUM SHADE
    ((0xA2, 0xC7), (0x25A4)),  # SQUARE WITH HORIZONTAL FILL
    ((0xA2, 0xC8), (0x25A5)),  # SQUARE WITH VERTICAL FILL
    ((0xA2, 0xC9), (0x25A8)),  # SQUARE WITH UPPER RIGHT TO LOWER LEFT FILL
    ((0xA2, 0xCA), (0x25A7)),  # SQUARE WITH UPPER LEFT TO LOWER RIGHT FILL
    ((0xA2, 0xCB), (0x25A6)),  # SQUARE WITH ORTHOGONAL CROSSHATCH FILL
    ((0xA2, 0xCC), (0x25A9)),  # SQUARE WITH DIAGONAL CROSSHATCH FILL
    ((0xA2, 0xCD), (0x2668)),  # HOT SPRINGS
    ((0xA2, 0xCE), (0x260F)),  # WHITE TELEPHONE
    ((0xA2, 0xCF), (0x260E)),  # BLACK TELEPHONE
    ((0xA2, 0xD0), (0x261C)),  # WHITE LEFT POINTING INDEX
    ((0xA2, 0xD1), (0x261E)),  # WHITE RIGHT POINTING INDEX
    ((0xA2, 0xD2), (0x00B6)),  # PILCROW SIGN
    ((0xA2, 0xD3), (0x2020)),  # DAGGER
    ((0xA2, 0xD4), (0x2021)),  # DOUBLE DAGGER
    ((0xA2, 0xD5), (0x2195)),  # UP DOWN ARROW
    ((0xA2, 0xD6), (0x2197)),  # NORTH EAST ARROW
    ((0xA2, 0xD7), (0x2199)),  # SOUTH WEST ARROW
    ((0xA2, 0xD8), (0x2196)),  # NORTH WEST ARROW
    ((0xA2, 0xD9), (0x2198)),  # SOUTH EAST ARROW
    ((0xA2, 0xDA), (0x266D)),  # MUSIC FLAT SIGN
    ((0xA2, 0xDB), (0x2669)),  # QUARTER NOTE
    ((0xA2, 0xDC), (0x266A)),  # EIGHTH NOTE
    ((0xA2, 0xDD), (0x266C)),  # BEAMED SIXTEENTH NOTES
    ((0xA2, 0xDE), (0x327F)),  # KOREAN STANDARD SYMBOL
    ((0xA2, 0xDF), (0x321C)),  # PARENTHESIZED HANGUL CIEUC U
    ((0xA2, 0xE0), (0x2116)),  # NUMERO SIGN
    ((0xA2, 0xE1), (0x33C7)),  # SQUARE CO
    ((0xA2, 0xE2), (0x2122)),  # TRADE MARK SIGN
    ((0xA2, 0xE3), (0x33C2)),  # SQUARE AM
    ((0xA2, 0xE4), (0x33D8)),  # SQUARE PM
    ((0xA2, 0xE5), (0x2121)),  # TELEPHONE SIGN
    ((0xA2, 0xE6), (0x20AC)),  # EURO SIGN
    ((0xA2, 0xE7), (0x00AE)),  # REGISTERED SIGN
    ((0xA3, 0x41), (0xC971)),  # HANGUL SYLLABLE CIEUC YU NIEUNCIEUC
    ((0xA3, 0x42), (0xC972)),  # HANGUL SYLLABLE CIEUC YU NIEUNHIEUH
    ((0xA3, 0x43), (0xC973)),  # HANGUL SYLLABLE CIEUC YU TIKEUT
    ((0xA3, 0x44), (0xC975)),  # HANGUL SYLLABLE CIEUC YU RIEULKIYEOK
    ((0xA3, 0x45), (0xC976)),  # HANGUL SYLLABLE CIEUC YU RIEULMIEUM
    ((0xA3, 0x46), (0xC977)),  # HANGUL SYLLABLE CIEUC YU RIEULPIEUP
    ((0xA3, 0x47), (0xC978)),  # HANGUL SYLLABLE CIEUC YU RIEULSIOS
    ((0xA3, 0x48), (0xC979)),  # HANGUL SYLLABLE CIEUC YU RIEULTHIEUTH
    ((0xA3, 0x49), (0xC97A)),  # HANGUL SYLLABLE CIEUC YU RIEULPHIEUPH
    ((0xA3, 0x4A), (0xC97B)),  # HANGUL SYLLABLE CIEUC YU RIEULHIEUH
    ((0xA3, 0x4B), (0xC97D)),  # HANGUL SYLLABLE CIEUC YU PIEUP
    ((0xA3, 0x4C), (0xC97E)),  # HANGUL SYLLABLE CIEUC YU PIEUPSIOS
    ((0xA3, 0x4D), (0xC97F)),  # HANGUL SYLLABLE CIEUC YU SIOS
    ((0xA3, 0x4E), (0xC980)),  # HANGUL SYLLABLE CIEUC YU SSANGSIOS
    ((0xA3, 0x4F), (0xC981)),  # HANGUL SYLLABLE CIEUC YU IEUNG
    ((0xA3, 0x50), (0xC982)),  # HANGUL SYLLABLE CIEUC YU CIEUC
    ((0xA3, 0x51), (0xC983)),  # HANGUL SYLLABLE CIEUC YU CHIEUCH
    ((0xA3, 0x52), (0xC984)),  # HANGUL SYLLABLE CIEUC YU KHIEUKH
    ((0xA3, 0x53), (0xC985)),  # HANGUL SYLLABLE CIEUC YU THIEUTH
    ((0xA3, 0x54), (0xC986)),  # HANGUL SYLLABLE CIEUC YU PHIEUPH
    ((0xA3, 0x55), (0xC987)),  # HANGUL SYLLABLE CIEUC YU HIEUH
    ((0xA3, 0x56), (0xC98A)),  # HANGUL SYLLABLE CIEUC EU SSANGKIYEOK
    ((0xA3, 0x57), (0xC98B)),  # HANGUL SYLLABLE CIEUC EU KIYEOKSIOS
    ((0xA3, 0x58), (0xC98D)),  # HANGUL SYLLABLE CIEUC EU NIEUNCIEUC
    ((0xA3, 0x59), (0xC98E)),  # HANGUL SYLLABLE CIEUC EU NIEUNHIEUH
    ((0xA3, 0x5A), (0xC98F)),  # HANGUL SYLLABLE CIEUC EU TIKEUT
    ((0xA3, 0x61), (0xC991)),  # HANGUL SYLLABLE CIEUC EU RIEULKIYEOK
    ((0xA3, 0x62), (0xC992)),  # HANGUL SYLLABLE CIEUC EU RIEULMIEUM
    ((0xA3, 0x63), (0xC993)),  # HANGUL SYLLABLE CIEUC EU RIEULPIEUP
    ((0xA3, 0x64), (0xC994)),  # HANGUL SYLLABLE CIEUC EU RIEULSIOS
    ((0xA3, 0x65), (0xC995)),  # HANGUL SYLLABLE CIEUC EU RIEULTHIEUTH
    ((0xA3, 0x66), (0xC996)),  # HANGUL SYLLABLE CIEUC EU RIEULPHIEUPH
    ((0xA3, 0x67), (0xC997)),  # HANGUL SYLLABLE CIEUC EU RIEULHIEUH
    ((0xA3, 0x68), (0xC99A)),  # HANGUL SYLLABLE CIEUC EU PIEUPSIOS
    ((0xA3, 0x69), (0xC99C)),  # HANGUL SYLLABLE CIEUC EU SSANGSIOS
    ((0xA3, 0x6A), (0xC99E)),  # HANGUL SYLLABLE CIEUC EU CIEUC
    ((0xA3, 0x6B), (0xC99F)),  # HANGUL SYLLABLE CIEUC EU CHIEUCH
    ((0xA3, 0x6C), (0xC9A0)),  # HANGUL SYLLABLE CIEUC EU KHIEUKH
    ((0xA3, 0x6D), (0xC9A1)),  # HANGUL SYLLABLE CIEUC EU THIEUTH
    ((0xA3, 0x6E), (0xC9A2)),  # HANGUL SYLLABLE CIEUC EU PHIEUPH
    ((0xA3, 0x6F), (0xC9A3)),  # HANGUL SYLLABLE CIEUC EU HIEUH
    ((0xA3, 0x70), (0xC9A4)),  # HANGUL SYLLABLE CIEUC YI
    ((0xA3, 0x71), (0xC9A5)),  # HANGUL SYLLABLE CIEUC YI KIYEOK
    ((0xA3, 0x72), (0xC9A6)),  # HANGUL SYLLABLE CIEUC YI SSANGKIYEOK
    ((0xA3, 0x73), (0xC9A7)),  # HANGUL SYLLABLE CIEUC YI KIYEOKSIOS
    ((0xA3, 0x74), (0xC9A8)),  # HANGUL SYLLABLE CIEUC YI NIEUN
    ((0xA3, 0x75), (0xC9A9)),  # HANGUL SYLLABLE CIEUC YI NIEUNCIEUC
    ((0xA3, 0x76), (0xC9AA)),  # HANGUL SYLLABLE CIEUC YI NIEUNHIEUH
    ((0xA3, 0x77), (0xC9AB)),  # HANGUL SYLLABLE CIEUC YI TIKEUT
    ((0xA3, 0x78), (0xC9AC)),  # HANGUL SYLLABLE CIEUC YI RIEUL
    ((0xA3, 0x79), (0xC9AD)),  # HANGUL SYLLABLE CIEUC YI RIEULKIYEOK
    ((0xA3, 0x7A), (0xC9AE)),  # HANGUL SYLLABLE CIEUC YI RIEULMIEUM
    ((0xA3, 0x81), (0xC9AF)),  # HANGUL SYLLABLE CIEUC YI RIEULPIEUP
    ((0xA3, 0x82), (0xC9B0)),  # HANGUL SYLLABLE CIEUC YI RIEULSIOS
    ((0xA3, 0x83), (0xC9B1)),  # HANGUL SYLLABLE CIEUC YI RIEULTHIEUTH
    ((0xA3, 0x84), (0xC9B2)),  # HANGUL SYLLABLE CIEUC YI RIEULPHIEUPH
    ((0xA3, 0x85), (0xC9B3)),  # HANGUL SYLLABLE CIEUC YI RIEULHIEUH
    ((0xA3, 0x86), (0xC9B4)),  # HANGUL SYLLABLE CIEUC YI MIEUM
    ((0xA3, 0x87), (0xC9B5)),  # HANGUL SYLLABLE CIEUC YI PIEUP
    ((0xA3, 0x88), (0xC9B6)),  # HANGUL SYLLABLE CIEUC YI PIEUPSIOS
    ((0xA3, 0x89), (0xC9B7)),  # HANGUL SYLLABLE CIEUC YI SIOS
    ((0xA3, 0x8A), (0xC9B8)),  # HANGUL SYLLABLE CIEUC YI SSANGSIOS
    ((0xA3, 0x8B), (0xC9B9)),  # HANGUL SYLLABLE CIEUC YI IEUNG
    ((0xA3, 0x8C), (0xC9BA)),  # HANGUL SYLLABLE CIEUC YI CIEUC
    ((0xA3, 0x8D), (0xC9BB)),  # HANGUL SYLLABLE CIEUC YI CHIEUCH
    ((0xA3, 0x8E), (0xC9BC)),  # HANGUL SYLLABLE CIEUC YI KHIEUKH
    ((0xA3, 0x8F), (0xC9BD)),  # HANGUL SYLLABLE CIEUC YI THIEUTH
    ((0xA3, 0x90), (0xC9BE)),  # HANGUL SYLLABLE CIEUC YI PHIEUPH
    ((0xA3, 0x91), (0xC9BF)),  # HANGUL SYLLABLE CIEUC YI HIEUH
    ((0xA3, 0x92), (0xC9C2)),  # HANGUL SYLLABLE CIEUC I SSANGKIYEOK
    ((0xA3, 0x93), (0xC9C3)),  # HANGUL SYLLABLE CIEUC I KIYEOKSIOS
    ((0xA3, 0x94), (0xC9C5)),  # HANGUL SYLLABLE CIEUC I NIEUNCIEUC
    ((0xA3, 0x95), (0xC9C6)),  # HANGUL SYLLABLE CIEUC I NIEUNHIEUH
    ((0xA3, 0x96), (0xC9C9)),  # HANGUL SYLLABLE CIEUC I RIEULKIYEOK
    ((0xA3, 0x97), (0xC9CB)),  # HANGUL SYLLABLE CIEUC I RIEULPIEUP
    ((0xA3, 0x98), (0xC9CC)),  # HANGUL SYLLABLE CIEUC I RIEULSIOS
    ((0xA3, 0x99), (0xC9CD)),  # HANGUL SYLLABLE CIEUC I RIEULTHIEUTH
    ((0xA3, 0x9A), (0xC9CE)),  # HANGUL SYLLABLE CIEUC I RIEULPHIEUPH
    ((0xA3, 0x9B), (0xC9CF)),  # HANGUL SYLLABLE CIEUC I RIEULHIEUH
    ((0xA3, 0x9C), (0xC9D2)),  # HANGUL SYLLABLE CIEUC I PIEUPSIOS
    ((0xA3, 0x9D), (0xC9D4)),  # HANGUL SYLLABLE CIEUC I SSANGSIOS
    ((0xA3, 0x9E), (0xC9D7)),  # HANGUL SYLLABLE CIEUC I CHIEUCH
    ((0xA3, 0x9F), (0xC9D8)),  # HANGUL SYLLABLE CIEUC I KHIEUKH
    ((0xA3, 0xA0), (0xC9DB)),  # HANGUL SYLLABLE CIEUC I HIEUH
    ((0xA3, 0xA1), (0xFF01)),  # FULLWIDTH EXCLAMATION MARK
    ((0xA3, 0xA2), (0xFF02)),  # FULLWIDTH QUOTATION MARK
    ((0xA3, 0xA3), (0xFF03)),  # FULLWIDTH NUMBER SIGN
    ((0xA3, 0xA4), (0xFF04)),  # FULLWIDTH DOLLAR SIGN
    ((0xA3, 0xA5), (0xFF05)),  # FULLWIDTH PERCENT SIGN
    ((0xA3, 0xA6), (0xFF06)),  # FULLWIDTH AMPERSAND
    ((0xA3, 0xA7), (0xFF07)),  # FULLWIDTH APOSTROPHE
    ((0xA3, 0xA8), (0xFF08)),  # FULLWIDTH LEFT PARENTHESIS
    ((0xA3, 0xA9), (0xFF09)),  # FULLWIDTH RIGHT PARENTHESIS
    ((0xA3, 0xAA), (0xFF0A)),  # FULLWIDTH ASTERISK
    ((0xA3, 0xAB), (0xFF0B)),  # FULLWIDTH PLUS SIGN
    ((0xA3, 0xAC), (0xFF0C)),  # FULLWIDTH COMMA
    ((0xA3, 0xAD), (0xFF0D)),  # FULLWIDTH HYPHEN-MINUS
    ((0xA3, 0xAE), (0xFF0E)),  # FULLWIDTH FULL STOP
    ((0xA3, 0xAF), (0xFF0F)),  # FULLWIDTH SOLIDUS
    ((0xA3, 0xB0), (0xFF10)),  # FULLWIDTH DIGIT ZERO
    ((0xA3, 0xB1), (0xFF11)),  # FULLWIDTH DIGIT ONE
    ((0xA3, 0xB2), (0xFF12)),  # FULLWIDTH DIGIT TWO
    ((0xA3, 0xB3), (0xFF13)),  # FULLWIDTH DIGIT THREE
    ((0xA3, 0xB4), (0xFF14)),  # FULLWIDTH DIGIT FOUR
    ((0xA3, 0xB5), (0xFF15)),  # FULLWIDTH DIGIT FIVE
    ((0xA3, 0xB6), (0xFF16)),  # FULLWIDTH DIGIT SIX
    ((0xA3, 0xB7), (0xFF17)),  # FULLWIDTH DIGIT SEVEN
    ((0xA3, 0xB8), (0xFF18)),  # FULLWIDTH DIGIT EIGHT
    ((0xA3, 0xB9), (0xFF19)),  # FULLWIDTH DIGIT NINE
    ((0xA3, 0xBA), (0xFF1A)),  # FULLWIDTH COLON
    ((0xA3, 0xBB), (0xFF1B)),  # FULLWIDTH SEMICOLON
    ((0xA3, 0xBC), (0xFF1C)),  # FULLWIDTH LESS-THAN SIGN
    ((0xA3, 0xBD), (0xFF1D)),  # FULLWIDTH EQUALS SIGN
    ((0xA3, 0xBE), (0xFF1E)),  # FULLWIDTH GREATER-THAN SIGN
    ((0xA3, 0xBF), (0xFF1F)),  # FULLWIDTH QUESTION MARK
    ((0xA3, 0xC0), (0xFF20)),  # FULLWIDTH COMMERCIAL AT
    ((0xA3, 0xC1), (0xFF21)),  # FULLWIDTH LATIN CAPITAL LETTER A
    ((0xA3, 0xC2), (0xFF22)),  # FULLWIDTH LATIN CAPITAL LETTER B
    ((0xA3, 0xC3), (0xFF23)),  # FULLWIDTH LATIN CAPITAL LETTER C
    ((0xA3, 0xC4), (0xFF24)),  # FULLWIDTH LATIN CAPITAL LETTER D
    ((0xA3, 0xC5), (0xFF25)),  # FULLWIDTH LATIN CAPITAL LETTER E
    ((0xA3, 0xC6), (0xFF26)),  # FULLWIDTH LATIN CAPITAL LETTER F
    ((0xA3, 0xC7), (0xFF27)),  # FULLWIDTH LATIN CAPITAL LETTER G
    ((0xA3, 0xC8), (0xFF28)),  # FULLWIDTH LATIN CAPITAL LETTER H
    ((0xA3, 0xC9), (0xFF29)),  # FULLWIDTH LATIN CAPITAL LETTER I
    ((0xA3, 0xCA), (0xFF2A)),  # FULLWIDTH LATIN CAPITAL LETTER J
    ((0xA3, 0xCB), (0xFF2B)),  # FULLWIDTH LATIN CAPITAL LETTER K
    ((0xA3, 0xCC), (0xFF2C)),  # FULLWIDTH LATIN CAPITAL LETTER L
    ((0xA3, 0xCD), (0xFF2D)),  # FULLWIDTH LATIN CAPITAL LETTER M
    ((0xA3, 0xCE), (0xFF2E)),  # FULLWIDTH LATIN CAPITAL LETTER N
    ((0xA3, 0xCF), (0xFF2F)),  # FULLWIDTH LATIN CAPITAL LETTER O
    ((0xA3, 0xD0), (0xFF30)),  # FULLWIDTH LATIN CAPITAL LETTER P
    ((0xA3, 0xD1), (0xFF31)),  # FULLWIDTH LATIN CAPITAL LETTER Q
    ((0xA3, 0xD2), (0xFF32)),  # FULLWIDTH LATIN CAPITAL LETTER R
    ((0xA3, 0xD3), (0xFF33)),  # FULLWIDTH LATIN CAPITAL LETTER S
    ((0xA3, 0xD4), (0xFF34)),  # FULLWIDTH LATIN CAPITAL LETTER T
    ((0xA3, 0xD5), (0xFF35)),  # FULLWIDTH LATIN CAPITAL LETTER U
    ((0xA3, 0xD6), (0xFF36)),  # FULLWIDTH LATIN CAPITAL LETTER V
    ((0xA3, 0xD7), (0xFF37)),  # FULLWIDTH LATIN CAPITAL LETTER W
    ((0xA3, 0xD8), (0xFF38)),  # FULLWIDTH LATIN CAPITAL LETTER X
    ((0xA3, 0xD9), (0xFF39)),  # FULLWIDTH LATIN CAPITAL LETTER Y
    ((0xA3, 0xDA), (0xFF3A)),  # FULLWIDTH LATIN CAPITAL LETTER Z
    ((0xA3, 0xDB), (0xFF3B)),  # FULLWIDTH LEFT SQUARE BRACKET
    ((0xA3, 0xDC), (0xFFE6)),  # FULLWIDTH WON SIGN
    ((0xA3, 0xDD), (0xFF3D)),  # FULLWIDTH RIGHT SQUARE BRACKET
    ((0xA3, 0xDE), (0xFF3E)),  # FULLWIDTH CIRCUMFLEX ACCENT
    ((0xA3, 0xDF), (0xFF3F)),  # FULLWIDTH LOW LINE
    ((0xA3, 0xE0), (0xFF40)),  # FULLWIDTH GRAVE ACCENT
    ((0xA3, 0xE1), (0xFF41)),  # FULLWIDTH LATIN SMALL LETTER A
    ((0xA3, 0xE2), (0xFF42)),  # FULLWIDTH LATIN SMALL LETTER B
    ((0xA3, 0xE3), (0xFF43)),  # FULLWIDTH LATIN SMALL LETTER C
    ((0xA3, 0xE4), (0xFF44)),  # FULLWIDTH LATIN SMALL LETTER D
    ((0xA3, 0xE5), (0xFF45)),  # FULLWIDTH LATIN SMALL LETTER E
    ((0xA3, 0xE6), (0xFF46)),  # FULLWIDTH LATIN SMALL LETTER F
    ((0xA3, 0xE7), (0xFF47)),  # FULLWIDTH LATIN SMALL LETTER G
    ((0xA3, 0xE8), (0xFF48)),  # FULLWIDTH LATIN SMALL LETTER H
    ((0xA3, 0xE9), (0xFF49)),  # FULLWIDTH LATIN SMALL LETTER I
    ((0xA3, 0xEA), (0xFF4A)),  # FULLWIDTH LATIN SMALL LETTER J
    ((0xA3, 0xEB), (0xFF4B)),  # FULLWIDTH LATIN SMALL LETTER K
    ((0xA3, 0xEC), (0xFF4C)),  # FULLWIDTH LATIN SMALL LETTER L
    ((0xA3, 0xED), (0xFF4D)),  # FULLWIDTH LATIN SMALL LETTER M
    ((0xA3, 0xEE), (0xFF4E)),  # FULLWIDTH LATIN SMALL LETTER N
    ((0xA3, 0xEF), (0xFF4F)),  # FULLWIDTH LATIN SMALL LETTER O
    ((0xA3, 0xF0), (0xFF50)),  # FULLWIDTH LATIN SMALL LETTER P
    ((0xA3, 0xF1), (0xFF51)),  # FULLWIDTH LATIN SMALL LETTER Q
    ((0xA3, 0xF2), (0xFF52)),  # FULLWIDTH LATIN SMALL LETTER R
    ((0xA3, 0xF3), (0xFF53)),  # FULLWIDTH LATIN SMALL LETTER S
    ((0xA3, 0xF4), (0xFF54)),  # FULLWIDTH LATIN SMALL LETTER T
    ((0xA3, 0xF5), (0xFF55)),  # FULLWIDTH LATIN SMALL LETTER U
    ((0xA3, 0xF6), (0xFF56)),  # FULLWIDTH LATIN SMALL LETTER V
    ((0xA3, 0xF7), (0xFF57)),  # FULLWIDTH LATIN SMALL LETTER W
    ((0xA3, 0xF8), (0xFF58)),  # FULLWIDTH LATIN SMALL LETTER X
    ((0xA3, 0xF9), (0xFF59)),  # FULLWIDTH LATIN SMALL LETTER Y
    ((0xA3, 0xFA), (0xFF5A)),  # FULLWIDTH LATIN SMALL LETTER Z
    ((0xA3, 0xFB), (0xFF5B)),  # FULLWIDTH LEFT CURLY BRACKET
    ((0xA3, 0xFC), (0xFF5C)),  # FULLWIDTH VERTICAL LINE
    ((0xA3, 0xFD), (0xFF5D)),  # FULLWIDTH RIGHT CURLY BRACKET
    ((0xA3, 0xFE), (0xFFE3)),  # FULLWIDTH MACRON
    ((0xA4, 0x41), (0xC9DE)),  # HANGUL SYLLABLE SSANGCIEUC A SSANGKIYEOK
    ((0xA4, 0x42), (0xC9DF)),  # HANGUL SYLLABLE SSANGCIEUC A KIYEOKSIOS
    ((0xA4, 0x43), (0xC9E1)),  # HANGUL SYLLABLE SSANGCIEUC A NIEUNCIEUC
    ((0xA4, 0x44), (0xC9E3)),  # HANGUL SYLLABLE SSANGCIEUC A TIKEUT
    ((0xA4, 0x45), (0xC9E5)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULKIYEOK
    ((0xA4, 0x46), (0xC9E6)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULMIEUM
    ((0xA4, 0x47), (0xC9E8)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULSIOS
    ((0xA4, 0x48), (0xC9E9)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULTHIEUTH
    ((0xA4, 0x49), (0xC9EA)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULPHIEUPH
    ((0xA4, 0x4A), (0xC9EB)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULHIEUH
    ((0xA4, 0x4B), (0xC9EE)),  # HANGUL SYLLABLE SSANGCIEUC A PIEUPSIOS
    ((0xA4, 0x4C), (0xC9F2)),  # HANGUL SYLLABLE SSANGCIEUC A CIEUC
    ((0xA4, 0x4D), (0xC9F3)),  # HANGUL SYLLABLE SSANGCIEUC A CHIEUCH
    ((0xA4, 0x4E), (0xC9F4)),  # HANGUL SYLLABLE SSANGCIEUC A KHIEUKH
    ((0xA4, 0x4F), (0xC9F5)),  # HANGUL SYLLABLE SSANGCIEUC A THIEUTH
    ((0xA4, 0x50), (0xC9F6)),  # HANGUL SYLLABLE SSANGCIEUC A PHIEUPH
    ((0xA4, 0x51), (0xC9F7)),  # HANGUL SYLLABLE SSANGCIEUC A HIEUH
    ((0xA4, 0x52), (0xC9FA)),  # HANGUL SYLLABLE SSANGCIEUC AE SSANGKIYEOK
    ((0xA4, 0x53), (0xC9FB)),  # HANGUL SYLLABLE SSANGCIEUC AE KIYEOKSIOS
    ((0xA4, 0x54), (0xC9FD)),  # HANGUL SYLLABLE SSANGCIEUC AE NIEUNCIEUC
    ((0xA4, 0x55), (0xC9FE)),  # HANGUL SYLLABLE SSANGCIEUC AE NIEUNHIEUH
    ((0xA4, 0x56), (0xC9FF)),  # HANGUL SYLLABLE SSANGCIEUC AE TIKEUT
    ((0xA4, 0x57), (0xCA01)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULKIYEOK
    ((0xA4, 0x58), (0xCA02)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULMIEUM
    ((0xA4, 0x59), (0xCA03)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULPIEUP
    ((0xA4, 0x5A), (0xCA04)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULSIOS
    ((0xA4, 0x61), (0xCA05)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULTHIEUTH
    ((0xA4, 0x62), (0xCA06)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULPHIEUPH
    ((0xA4, 0x63), (0xCA07)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEULHIEUH
    ((0xA4, 0x64), (0xCA0A)),  # HANGUL SYLLABLE SSANGCIEUC AE PIEUPSIOS
    ((0xA4, 0x65), (0xCA0E)),  # HANGUL SYLLABLE SSANGCIEUC AE CIEUC
    ((0xA4, 0x66), (0xCA0F)),  # HANGUL SYLLABLE SSANGCIEUC AE CHIEUCH
    ((0xA4, 0x67), (0xCA10)),  # HANGUL SYLLABLE SSANGCIEUC AE KHIEUKH
    ((0xA4, 0x68), (0xCA11)),  # HANGUL SYLLABLE SSANGCIEUC AE THIEUTH
    ((0xA4, 0x69), (0xCA12)),  # HANGUL SYLLABLE SSANGCIEUC AE PHIEUPH
    ((0xA4, 0x6A), (0xCA13)),  # HANGUL SYLLABLE SSANGCIEUC AE HIEUH
    ((0xA4, 0x6B), (0xCA15)),  # HANGUL SYLLABLE SSANGCIEUC YA KIYEOK
    ((0xA4, 0x6C), (0xCA16)),  # HANGUL SYLLABLE SSANGCIEUC YA SSANGKIYEOK
    ((0xA4, 0x6D), (0xCA17)),  # HANGUL SYLLABLE SSANGCIEUC YA KIYEOKSIOS
    ((0xA4, 0x6E), (0xCA19)),  # HANGUL SYLLABLE SSANGCIEUC YA NIEUNCIEUC
    ((0xA4, 0x6F), (0xCA1A)),  # HANGUL SYLLABLE SSANGCIEUC YA NIEUNHIEUH
    ((0xA4, 0x70), (0xCA1B)),  # HANGUL SYLLABLE SSANGCIEUC YA TIKEUT
    ((0xA4, 0x71), (0xCA1C)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEUL
    ((0xA4, 0x72), (0xCA1D)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULKIYEOK
    ((0xA4, 0x73), (0xCA1E)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULMIEUM
    ((0xA4, 0x74), (0xCA1F)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULPIEUP
    ((0xA4, 0x75), (0xCA20)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULSIOS
    ((0xA4, 0x76), (0xCA21)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULTHIEUTH
    ((0xA4, 0x77), (0xCA22)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULPHIEUPH
    ((0xA4, 0x78), (0xCA23)),  # HANGUL SYLLABLE SSANGCIEUC YA RIEULHIEUH
    ((0xA4, 0x79), (0xCA24)),  # HANGUL SYLLABLE SSANGCIEUC YA MIEUM
    ((0xA4, 0x7A), (0xCA25)),  # HANGUL SYLLABLE SSANGCIEUC YA PIEUP
    ((0xA4, 0x81), (0xCA26)),  # HANGUL SYLLABLE SSANGCIEUC YA PIEUPSIOS
    ((0xA4, 0x82), (0xCA27)),  # HANGUL SYLLABLE SSANGCIEUC YA SIOS
    ((0xA4, 0x83), (0xCA28)),  # HANGUL SYLLABLE SSANGCIEUC YA SSANGSIOS
    ((0xA4, 0x84), (0xCA2A)),  # HANGUL SYLLABLE SSANGCIEUC YA CIEUC
    ((0xA4, 0x85), (0xCA2B)),  # HANGUL SYLLABLE SSANGCIEUC YA CHIEUCH
    ((0xA4, 0x86), (0xCA2C)),  # HANGUL SYLLABLE SSANGCIEUC YA KHIEUKH
    ((0xA4, 0x87), (0xCA2D)),  # HANGUL SYLLABLE SSANGCIEUC YA THIEUTH
    ((0xA4, 0x88), (0xCA2E)),  # HANGUL SYLLABLE SSANGCIEUC YA PHIEUPH
    ((0xA4, 0x89), (0xCA2F)),  # HANGUL SYLLABLE SSANGCIEUC YA HIEUH
    ((0xA4, 0x8A), (0xCA30)),  # HANGUL SYLLABLE SSANGCIEUC YAE
    ((0xA4, 0x8B), (0xCA31)),  # HANGUL SYLLABLE SSANGCIEUC YAE KIYEOK
    ((0xA4, 0x8C), (0xCA32)),  # HANGUL SYLLABLE SSANGCIEUC YAE SSANGKIYEOK
    ((0xA4, 0x8D), (0xCA33)),  # HANGUL SYLLABLE SSANGCIEUC YAE KIYEOKSIOS
    ((0xA4, 0x8E), (0xCA34)),  # HANGUL SYLLABLE SSANGCIEUC YAE NIEUN
    ((0xA4, 0x8F), (0xCA35)),  # HANGUL SYLLABLE SSANGCIEUC YAE NIEUNCIEUC
    ((0xA4, 0x90), (0xCA36)),  # HANGUL SYLLABLE SSANGCIEUC YAE NIEUNHIEUH
    ((0xA4, 0x91), (0xCA37)),  # HANGUL SYLLABLE SSANGCIEUC YAE TIKEUT
    ((0xA4, 0x92), (0xCA38)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEUL
    ((0xA4, 0x93), (0xCA39)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULKIYEOK
    ((0xA4, 0x94), (0xCA3A)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULMIEUM
    ((0xA4, 0x95), (0xCA3B)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULPIEUP
    ((0xA4, 0x96), (0xCA3C)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULSIOS
    ((0xA4, 0x97), (0xCA3D)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULTHIEUTH
    ((0xA4, 0x98), (0xCA3E)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULPHIEUPH
    ((0xA4, 0x99), (0xCA3F)),  # HANGUL SYLLABLE SSANGCIEUC YAE RIEULHIEUH
    ((0xA4, 0x9A), (0xCA40)),  # HANGUL SYLLABLE SSANGCIEUC YAE MIEUM
    ((0xA4, 0x9B), (0xCA41)),  # HANGUL SYLLABLE SSANGCIEUC YAE PIEUP
    ((0xA4, 0x9C), (0xCA42)),  # HANGUL SYLLABLE SSANGCIEUC YAE PIEUPSIOS
    ((0xA4, 0x9D), (0xCA43)),  # HANGUL SYLLABLE SSANGCIEUC YAE SIOS
    ((0xA4, 0x9E), (0xCA44)),  # HANGUL SYLLABLE SSANGCIEUC YAE SSANGSIOS
    ((0xA4, 0x9F), (0xCA45)),  # HANGUL SYLLABLE SSANGCIEUC YAE IEUNG
    ((0xA4, 0xA0), (0xCA46)),  # HANGUL SYLLABLE SSANGCIEUC YAE CIEUC
    ((0xA4, 0xA1), (0x3131)),  # HANGUL LETTER KIYEOK
    ((0xA4, 0xA2), (0x3132)),  # HANGUL LETTER SSANGKIYEOK
    ((0xA4, 0xA3), (0x3133)),  # HANGUL LETTER KIYEOK-SIOS
    ((0xA4, 0xA4), (0x3134)),  # HANGUL LETTER NIEUN
    ((0xA4, 0xA5), (0x3135)),  # HANGUL LETTER NIEUN-CIEUC
    ((0xA4, 0xA6), (0x3136)),  # HANGUL LETTER NIEUN-HIEUH
    ((0xA4, 0xA7), (0x3137)),  # HANGUL LETTER TIKEUT
    ((0xA4, 0xA8), (0x3138)),  # HANGUL LETTER SSANGTIKEUT
    ((0xA4, 0xA9), (0x3139)),  # HANGUL LETTER RIEUL
    ((0xA4, 0xAA), (0x313A)),  # HANGUL LETTER RIEUL-KIYEOK
    ((0xA4, 0xAB), (0x313B)),  # HANGUL LETTER RIEUL-MIEUM
    ((0xA4, 0xAC), (0x313C)),  # HANGUL LETTER RIEUL-PIEUP
    ((0xA4, 0xAD), (0x313D)),  # HANGUL LETTER RIEUL-SIOS
    ((0xA4, 0xAE), (0x313E)),  # HANGUL LETTER RIEUL-THIEUTH
    ((0xA4, 0xAF), (0x313F)),  # HANGUL LETTER RIEUL-PHIEUPH
    ((0xA4, 0xB0), (0x3140)),  # HANGUL LETTER RIEUL-HIEUH
    ((0xA4, 0xB1), (0x3141)),  # HANGUL LETTER MIEUM
    ((0xA4, 0xB2), (0x3142)),  # HANGUL LETTER PIEUP
    ((0xA4, 0xB3), (0x3143)),  # HANGUL LETTER SSANGPIEUP
    ((0xA4, 0xB4), (0x3144)),  # HANGUL LETTER PIEUP-SIOS
    ((0xA4, 0xB5), (0x3145)),  # HANGUL LETTER SIOS
    ((0xA4, 0xB6), (0x3146)),  # HANGUL LETTER SSANGSIOS
    ((0xA4, 0xB7), (0x3147)),  # HANGUL LETTER IEUNG
    ((0xA4, 0xB8), (0x3148)),  # HANGUL LETTER CIEUC
    ((0xA4, 0xB9), (0x3149)),  # HANGUL LETTER SSANGCIEUC
    ((0xA4, 0xBA), (0x314A)),  # HANGUL LETTER CHIEUCH
    ((0xA4, 0xBB), (0x314B)),  # HANGUL LETTER KHIEUKH
    ((0xA4, 0xBC), (0x314C)),  # HANGUL LETTER THIEUTH
    ((0xA4, 0xBD), (0x314D)),  # HANGUL LETTER PHIEUPH
    ((0xA4, 0xBE), (0x314E)),  # HANGUL LETTER HIEUH
    ((0xA4, 0xBF), (0x314F)),  # HANGUL LETTER A
    ((0xA4, 0xC0), (0x3150)),  # HANGUL LETTER AE
    ((0xA4, 0xC1), (0x3151)),  # HANGUL LETTER YA
    ((0xA4, 0xC2), (0x3152)),  # HANGUL LETTER YAE
    ((0xA4, 0xC3), (0x3153)),  # HANGUL LETTER EO
    ((0xA4, 0xC4), (0x3154)),  # HANGUL LETTER E
    ((0xA4, 0xC5), (0x3155)),  # HANGUL LETTER YEO
    ((0xA4, 0xC6), (0x3156)),  # HANGUL LETTER YE
    ((0xA4, 0xC7), (0x3157)),  # HANGUL LETTER O
    ((0xA4, 0xC8), (0x3158)),  # HANGUL LETTER WA
    ((0xA4, 0xC9), (0x3159)),  # HANGUL LETTER WAE
    ((0xA4, 0xCA), (0x315A)),  # HANGUL LETTER OE
    ((0xA4, 0xCB), (0x315B)),  # HANGUL LETTER YO
    ((0xA4, 0xCC), (0x315C)),  # HANGUL LETTER U
    ((0xA4, 0xCD), (0x315D)),  # HANGUL LETTER WEO
    ((0xA4, 0xCE), (0x315E)),  # HANGUL LETTER WE
    ((0xA4, 0xCF), (0x315F)),  # HANGUL LETTER WI
    ((0xA4, 0xD0), (0x3160)),  # HANGUL LETTER YU
    ((0xA4, 0xD1), (0x3161)),  # HANGUL LETTER EU
    ((0xA4, 0xD2), (0x3162)),  # HANGUL LETTER YI
    ((0xA4, 0xD3), (0x3163)),  # HANGUL LETTER I
    ((0xA4, 0xD4), (0x3164)),  # HANGUL FILLER
    ((0xA4, 0xD5), (0x3165)),  # HANGUL LETTER SSANGNIEUN
    ((0xA4, 0xD6), (0x3166)),  # HANGUL LETTER NIEUN-TIKEUT
    ((0xA4, 0xD7), (0x3167)),  # HANGUL LETTER NIEUN-SIOS
    ((0xA4, 0xD8), (0x3168)),  # HANGUL LETTER NIEUN-PANSIOS
    ((0xA4, 0xD9), (0x3169)),  # HANGUL LETTER RIEUL-KIYEOK-SIOS
    ((0xA4, 0xDA), (0x316A)),  # HANGUL LETTER RIEUL-TIKEUT
    ((0xA4, 0xDB), (0x316B)),  # HANGUL LETTER RIEUL-PIEUP-SIOS
    ((0xA4, 0xDC), (0x316C)),  # HANGUL LETTER RIEUL-PANSIOS
    ((0xA4, 0xDD), (0x316D)),  # HANGUL LETTER RIEUL-YEORINHIEUH
    ((0xA4, 0xDE), (0x316E)),  # HANGUL LETTER MIEUM-PIEUP
    ((0xA4, 0xDF), (0x316F)),  # HANGUL LETTER MIEUM-SIOS
    ((0xA4, 0xE0), (0x3170)),  # HANGUL LETTER MIEUM-PANSIOS
    ((0xA4, 0xE1), (0x3171)),  # HANGUL LETTER KAPYEOUNMIEUM
    ((0xA4, 0xE2), (0x3172)),  # HANGUL LETTER PIEUP-KIYEOK
    ((0xA4, 0xE3), (0x3173)),  # HANGUL LETTER PIEUP-TIKEUT
    ((0xA4, 0xE4), (0x3174)),  # HANGUL LETTER PIEUP-SIOS-KIYEOK
    ((0xA4, 0xE5), (0x3175)),  # HANGUL LETTER PIEUP-SIOS-TIKEUT
    ((0xA4, 0xE6), (0x3176)),  # HANGUL LETTER PIEUP-CIEUC
    ((0xA4, 0xE7), (0x3177)),  # HANGUL LETTER PIEUP-THIEUTH
    ((0xA4, 0xE8), (0x3178)),  # HANGUL LETTER KAPYEOUNPIEUP
    ((0xA4, 0xE9), (0x3179)),  # HANGUL LETTER KAPYEOUNSSANGPIEUP
    ((0xA4, 0xEA), (0x317A)),  # HANGUL LETTER SIOS-KIYEOK
    ((0xA4, 0xEB), (0x317B)),  # HANGUL LETTER SIOS-NIEUN
    ((0xA4, 0xEC), (0x317C)),  # HANGUL LETTER SIOS-TIKEUT
    ((0xA4, 0xED), (0x317D)),  # HANGUL LETTER SIOS-PIEUP
    ((0xA4, 0xEE), (0x317E)),  # HANGUL LETTER SIOS-CIEUC
    ((0xA4, 0xEF), (0x317F)),  # HANGUL LETTER PANSIOS
    ((0xA4, 0xF0), (0x3180)),  # HANGUL LETTER SSANGIEUNG
    ((0xA4, 0xF1), (0x3181)),  # HANGUL LETTER YESIEUNG
    ((0xA4, 0xF2), (0x3182)),  # HANGUL LETTER YESIEUNG-SIOS
    ((0xA4, 0xF3), (0x3183)),  # HANGUL LETTER YESIEUNG-PANSIOS
    ((0xA4, 0xF4), (0x3184)),  # HANGUL LETTER KAPYEOUNPHIEUPH
    ((0xA4, 0xF5), (0x3185)),  # HANGUL LETTER SSANGHIEUH
    ((0xA4, 0xF6), (0x3186)),  # HANGUL LETTER YEORINHIEUH
    ((0xA4, 0xF7), (0x3187)),  # HANGUL LETTER YO-YA
    ((0xA4, 0xF8), (0x3188)),  # HANGUL LETTER YO-YAE
    ((0xA4, 0xF9), (0x3189)),  # HANGUL LETTER YO-I
    ((0xA4, 0xFA), (0x318A)),  # HANGUL LETTER YU-YEO
    ((0xA4, 0xFB), (0x318B)),  # HANGUL LETTER YU-YE
    ((0xA4, 0xFC), (0x318C)),  # HANGUL LETTER YU-I
    ((0xA4, 0xFD), (0x318D)),  # HANGUL LETTER ARAEA
    ((0xA4, 0xFE), (0x318E)),  # HANGUL LETTER ARAEAE
    ((0xA5, 0x41), (0xCA47)),  # HANGUL SYLLABLE SSANGCIEUC YAE CHIEUCH
    ((0xA5, 0x42), (0xCA48)),  # HANGUL SYLLABLE SSANGCIEUC YAE KHIEUKH
    ((0xA5, 0x43), (0xCA49)),  # HANGUL SYLLABLE SSANGCIEUC YAE THIEUTH
    ((0xA5, 0x44), (0xCA4A)),  # HANGUL SYLLABLE SSANGCIEUC YAE PHIEUPH
    ((0xA5, 0x45), (0xCA4B)),  # HANGUL SYLLABLE SSANGCIEUC YAE HIEUH
    ((0xA5, 0x46), (0xCA4E)),  # HANGUL SYLLABLE SSANGCIEUC EO SSANGKIYEOK
    ((0xA5, 0x47), (0xCA4F)),  # HANGUL SYLLABLE SSANGCIEUC EO KIYEOKSIOS
    ((0xA5, 0x48), (0xCA51)),  # HANGUL SYLLABLE SSANGCIEUC EO NIEUNCIEUC
    ((0xA5, 0x49), (0xCA52)),  # HANGUL SYLLABLE SSANGCIEUC EO NIEUNHIEUH
    ((0xA5, 0x4A), (0xCA53)),  # HANGUL SYLLABLE SSANGCIEUC EO TIKEUT
    ((0xA5, 0x4B), (0xCA55)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULKIYEOK
    ((0xA5, 0x4C), (0xCA56)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULMIEUM
    ((0xA5, 0x4D), (0xCA57)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULPIEUP
    ((0xA5, 0x4E), (0xCA58)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULSIOS
    ((0xA5, 0x4F), (0xCA59)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULTHIEUTH
    ((0xA5, 0x50), (0xCA5A)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULPHIEUPH
    ((0xA5, 0x51), (0xCA5B)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEULHIEUH
    ((0xA5, 0x52), (0xCA5E)),  # HANGUL SYLLABLE SSANGCIEUC EO PIEUPSIOS
    ((0xA5, 0x53), (0xCA62)),  # HANGUL SYLLABLE SSANGCIEUC EO CIEUC
    ((0xA5, 0x54), (0xCA63)),  # HANGUL SYLLABLE SSANGCIEUC EO CHIEUCH
    ((0xA5, 0x55), (0xCA64)),  # HANGUL SYLLABLE SSANGCIEUC EO KHIEUKH
    ((0xA5, 0x56), (0xCA65)),  # HANGUL SYLLABLE SSANGCIEUC EO THIEUTH
    ((0xA5, 0x57), (0xCA66)),  # HANGUL SYLLABLE SSANGCIEUC EO PHIEUPH
    ((0xA5, 0x58), (0xCA67)),  # HANGUL SYLLABLE SSANGCIEUC EO HIEUH
    ((0xA5, 0x59), (0xCA69)),  # HANGUL SYLLABLE SSANGCIEUC E KIYEOK
    ((0xA5, 0x5A), (0xCA6A)),  # HANGUL SYLLABLE SSANGCIEUC E SSANGKIYEOK
    ((0xA5, 0x61), (0xCA6B)),  # HANGUL SYLLABLE SSANGCIEUC E KIYEOKSIOS
    ((0xA5, 0x62), (0xCA6C)),  # HANGUL SYLLABLE SSANGCIEUC E NIEUN
    ((0xA5, 0x63), (0xCA6D)),  # HANGUL SYLLABLE SSANGCIEUC E NIEUNCIEUC
    ((0xA5, 0x64), (0xCA6E)),  # HANGUL SYLLABLE SSANGCIEUC E NIEUNHIEUH
    ((0xA5, 0x65), (0xCA6F)),  # HANGUL SYLLABLE SSANGCIEUC E TIKEUT
    ((0xA5, 0x66), (0xCA70)),  # HANGUL SYLLABLE SSANGCIEUC E RIEUL
    ((0xA5, 0x67), (0xCA71)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULKIYEOK
    ((0xA5, 0x68), (0xCA72)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULMIEUM
    ((0xA5, 0x69), (0xCA73)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULPIEUP
    ((0xA5, 0x6A), (0xCA74)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULSIOS
    ((0xA5, 0x6B), (0xCA75)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULTHIEUTH
    ((0xA5, 0x6C), (0xCA76)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULPHIEUPH
    ((0xA5, 0x6D), (0xCA77)),  # HANGUL SYLLABLE SSANGCIEUC E RIEULHIEUH
    ((0xA5, 0x6E), (0xCA78)),  # HANGUL SYLLABLE SSANGCIEUC E MIEUM
    ((0xA5, 0x6F), (0xCA79)),  # HANGUL SYLLABLE SSANGCIEUC E PIEUP
    ((0xA5, 0x70), (0xCA7A)),  # HANGUL SYLLABLE SSANGCIEUC E PIEUPSIOS
    ((0xA5, 0x71), (0xCA7B)),  # HANGUL SYLLABLE SSANGCIEUC E SIOS
    ((0xA5, 0x72), (0xCA7C)),  # HANGUL SYLLABLE SSANGCIEUC E SSANGSIOS
    ((0xA5, 0x73), (0xCA7E)),  # HANGUL SYLLABLE SSANGCIEUC E CIEUC
    ((0xA5, 0x74), (0xCA7F)),  # HANGUL SYLLABLE SSANGCIEUC E CHIEUCH
    ((0xA5, 0x75), (0xCA80)),  # HANGUL SYLLABLE SSANGCIEUC E KHIEUKH
    ((0xA5, 0x76), (0xCA81)),  # HANGUL SYLLABLE SSANGCIEUC E THIEUTH
    ((0xA5, 0x77), (0xCA82)),  # HANGUL SYLLABLE SSANGCIEUC E PHIEUPH
    ((0xA5, 0x78), (0xCA83)),  # HANGUL SYLLABLE SSANGCIEUC E HIEUH
    ((0xA5, 0x79), (0xCA85)),  # HANGUL SYLLABLE SSANGCIEUC YEO KIYEOK
    ((0xA5, 0x7A), (0xCA86)),  # HANGUL SYLLABLE SSANGCIEUC YEO SSANGKIYEOK
    ((0xA5, 0x81), (0xCA87)),  # HANGUL SYLLABLE SSANGCIEUC YEO KIYEOKSIOS
    ((0xA5, 0x82), (0xCA88)),  # HANGUL SYLLABLE SSANGCIEUC YEO NIEUN
    ((0xA5, 0x83), (0xCA89)),  # HANGUL SYLLABLE SSANGCIEUC YEO NIEUNCIEUC
    ((0xA5, 0x84), (0xCA8A)),  # HANGUL SYLLABLE SSANGCIEUC YEO NIEUNHIEUH
    ((0xA5, 0x85), (0xCA8B)),  # HANGUL SYLLABLE SSANGCIEUC YEO TIKEUT
    ((0xA5, 0x86), (0xCA8C)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEUL
    ((0xA5, 0x87), (0xCA8D)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULKIYEOK
    ((0xA5, 0x88), (0xCA8E)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULMIEUM
    ((0xA5, 0x89), (0xCA8F)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULPIEUP
    ((0xA5, 0x8A), (0xCA90)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULSIOS
    ((0xA5, 0x8B), (0xCA91)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULTHIEUTH
    ((0xA5, 0x8C), (0xCA92)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULPHIEUPH
    ((0xA5, 0x8D), (0xCA93)),  # HANGUL SYLLABLE SSANGCIEUC YEO RIEULHIEUH
    ((0xA5, 0x8E), (0xCA94)),  # HANGUL SYLLABLE SSANGCIEUC YEO MIEUM
    ((0xA5, 0x8F), (0xCA95)),  # HANGUL SYLLABLE SSANGCIEUC YEO PIEUP
    ((0xA5, 0x90), (0xCA96)),  # HANGUL SYLLABLE SSANGCIEUC YEO PIEUPSIOS
    ((0xA5, 0x91), (0xCA97)),  # HANGUL SYLLABLE SSANGCIEUC YEO SIOS
    ((0xA5, 0x92), (0xCA99)),  # HANGUL SYLLABLE SSANGCIEUC YEO IEUNG
    ((0xA5, 0x93), (0xCA9A)),  # HANGUL SYLLABLE SSANGCIEUC YEO CIEUC
    ((0xA5, 0x94), (0xCA9B)),  # HANGUL SYLLABLE SSANGCIEUC YEO CHIEUCH
    ((0xA5, 0x95), (0xCA9C)),  # HANGUL SYLLABLE SSANGCIEUC YEO KHIEUKH
    ((0xA5, 0x96), (0xCA9D)),  # HANGUL SYLLABLE SSANGCIEUC YEO THIEUTH
    ((0xA5, 0x97), (0xCA9E)),  # HANGUL SYLLABLE SSANGCIEUC YEO PHIEUPH
    ((0xA5, 0x98), (0xCA9F)),  # HANGUL SYLLABLE SSANGCIEUC YEO HIEUH
    ((0xA5, 0x99), (0xCAA0)),  # HANGUL SYLLABLE SSANGCIEUC YE
    ((0xA5, 0x9A), (0xCAA1)),  # HANGUL SYLLABLE SSANGCIEUC YE KIYEOK
    ((0xA5, 0x9B), (0xCAA2)),  # HANGUL SYLLABLE SSANGCIEUC YE SSANGKIYEOK
    ((0xA5, 0x9C), (0xCAA3)),  # HANGUL SYLLABLE SSANGCIEUC YE KIYEOKSIOS
    ((0xA5, 0x9D), (0xCAA4)),  # HANGUL SYLLABLE SSANGCIEUC YE NIEUN
    ((0xA5, 0x9E), (0xCAA5)),  # HANGUL SYLLABLE SSANGCIEUC YE NIEUNCIEUC
    ((0xA5, 0x9F), (0xCAA6)),  # HANGUL SYLLABLE SSANGCIEUC YE NIEUNHIEUH
    ((0xA5, 0xA0), (0xCAA7)),  # HANGUL SYLLABLE SSANGCIEUC YE TIKEUT
    ((0xA5, 0xA1), (0x2170)),  # SMALL ROMAN NUMERAL ONE
    ((0xA5, 0xA2), (0x2171)),  # SMALL ROMAN NUMERAL TWO
    ((0xA5, 0xA3), (0x2172)),  # SMALL ROMAN NUMERAL THREE
    ((0xA5, 0xA4), (0x2173)),  # SMALL ROMAN NUMERAL FOUR
    ((0xA5, 0xA5), (0x2174)),  # SMALL ROMAN NUMERAL FIVE
    ((0xA5, 0xA6), (0x2175)),  # SMALL ROMAN NUMERAL SIX
    ((0xA5, 0xA7), (0x2176)),  # SMALL ROMAN NUMERAL SEVEN
    ((0xA5, 0xA8), (0x2177)),  # SMALL ROMAN NUMERAL EIGHT
    ((0xA5, 0xA9), (0x2178)),  # SMALL ROMAN NUMERAL NINE
    ((0xA5, 0xAA), (0x2179)),  # SMALL ROMAN NUMERAL TEN
    ((0xA5, 0xB0), (0x2160)),  # ROMAN NUMERAL ONE
    ((0xA5, 0xB1), (0x2161)),  # ROMAN NUMERAL TWO
    ((0xA5, 0xB2), (0x2162)),  # ROMAN NUMERAL THREE
    ((0xA5, 0xB3), (0x2163)),  # ROMAN NUMERAL FOUR
    ((0xA5, 0xB4), (0x2164)),  # ROMAN NUMERAL FIVE
    ((0xA5, 0xB5), (0x2165)),  # ROMAN NUMERAL SIX
    ((0xA5, 0xB6), (0x2166)),  # ROMAN NUMERAL SEVEN
    ((0xA5, 0xB7), (0x2167)),  # ROMAN NUMERAL EIGHT
    ((0xA5, 0xB8), (0x2168)),  # ROMAN NUMERAL NINE
    ((0xA5, 0xB9), (0x2169)),  # ROMAN NUMERAL TEN
    ((0xA5, 0xC1), (0x0391)),  # GREEK CAPITAL LETTER ALPHA
    ((0xA5, 0xC2), (0x0392)),  # GREEK CAPITAL LETTER BETA
    ((0xA5, 0xC3), (0x0393)),  # GREEK CAPITAL LETTER GAMMA
    ((0xA5, 0xC4), (0x0394)),  # GREEK CAPITAL LETTER DELTA
    ((0xA5, 0xC5), (0x0395)),  # GREEK CAPITAL LETTER EPSILON
    ((0xA5, 0xC6), (0x0396)),  # GREEK CAPITAL LETTER ZETA
    ((0xA5, 0xC7), (0x0397)),  # GREEK CAPITAL LETTER ETA
    ((0xA5, 0xC8), (0x0398)),  # GREEK CAPITAL LETTER THETA
    ((0xA5, 0xC9), (0x0399)),  # GREEK CAPITAL LETTER IOTA
    ((0xA5, 0xCA), (0x039A)),  # GREEK CAPITAL LETTER KAPPA
    ((0xA5, 0xCB), (0x039B)),  # GREEK CAPITAL LETTER LAMDA
    ((0xA5, 0xCC), (0x039C)),  # GREEK CAPITAL LETTER MU
    ((0xA5, 0xCD), (0x039D)),  # GREEK CAPITAL LETTER NU
    ((0xA5, 0xCE), (0x039E)),  # GREEK CAPITAL LETTER XI
    ((0xA5, 0xCF), (0x039F)),  # GREEK CAPITAL LETTER OMICRON
    ((0xA5, 0xD0), (0x03A0)),  # GREEK CAPITAL LETTER PI
    ((0xA5, 0xD1), (0x03A1)),  # GREEK CAPITAL LETTER RHO
    ((0xA5, 0xD2), (0x03A3)),  # GREEK CAPITAL LETTER SIGMA
    ((0xA5, 0xD3), (0x03A4)),  # GREEK CAPITAL LETTER TAU
    ((0xA5, 0xD4), (0x03A5)),  # GREEK CAPITAL LETTER UPSILON
    ((0xA5, 0xD5), (0x03A6)),  # GREEK CAPITAL LETTER PHI
    ((0xA5, 0xD6), (0x03A7)),  # GREEK CAPITAL LETTER CHI
    ((0xA5, 0xD7), (0x03A8)),  # GREEK CAPITAL LETTER PSI
    ((0xA5, 0xD8), (0x03A9)),  # GREEK CAPITAL LETTER OMEGA
    ((0xA5, 0xE1), (0x03B1)),  # GREEK SMALL LETTER ALPHA
    ((0xA5, 0xE2), (0x03B2)),  # GREEK SMALL LETTER BETA
    ((0xA5, 0xE3), (0x03B3)),  # GREEK SMALL LETTER GAMMA
    ((0xA5, 0xE4), (0x03B4)),  # GREEK SMALL LETTER DELTA
    ((0xA5, 0xE5), (0x03B5)),  # GREEK SMALL LETTER EPSILON
    ((0xA5, 0xE6), (0x03B6)),  # GREEK SMALL LETTER ZETA
    ((0xA5, 0xE7), (0x03B7)),  # GREEK SMALL LETTER ETA
    ((0xA5, 0xE8), (0x03B8)),  # GREEK SMALL LETTER THETA
    ((0xA5, 0xE9), (0x03B9)),  # GREEK SMALL LETTER IOTA
    ((0xA5, 0xEA), (0x03BA)),  # GREEK SMALL LETTER KAPPA
    ((0xA5, 0xEB), (0x03BB)),  # GREEK SMALL LETTER LAMDA
    ((0xA5, 0xEC), (0x03BC)),  # GREEK SMALL LETTER MU
    ((0xA5, 0xED), (0x03BD)),  # GREEK SMALL LETTER NU
    ((0xA5, 0xEE), (0x03BE)),  # GREEK SMALL LETTER XI
    ((0xA5, 0xEF), (0x03BF)),  # GREEK SMALL LETTER OMICRON
    ((0xA5, 0xF0), (0x03C0)),  # GREEK SMALL LETTER PI
    ((0xA5, 0xF1), (0x03C1)),  # GREEK SMALL LETTER RHO
    ((0xA5, 0xF2), (0x03C3)),  # GREEK SMALL LETTER SIGMA
    ((0xA5, 0xF3), (0x03C4)),  # GREEK SMALL LETTER TAU
    ((0xA5, 0xF4), (0x03C5)),  # GREEK SMALL LETTER UPSILON
    ((0xA5, 0xF5), (0x03C6)),  # GREEK SMALL LETTER PHI
    ((0xA5, 0xF6), (0x03C7)),  # GREEK SMALL LETTER CHI
    ((0xA5, 0xF7), (0x03C8)),  # GREEK SMALL LETTER PSI
    ((0xA5, 0xF8), (0x03C9)),  # GREEK SMALL LETTER OMEGA
    ((0xA6, 0x41), (0xCAA8)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEUL
    ((0xA6, 0x42), (0xCAA9)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULKIYEOK
    ((0xA6, 0x43), (0xCAAA)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULMIEUM
    ((0xA6, 0x44), (0xCAAB)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULPIEUP
    ((0xA6, 0x45), (0xCAAC)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULSIOS
    ((0xA6, 0x46), (0xCAAD)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULTHIEUTH
    ((0xA6, 0x47), (0xCAAE)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULPHIEUPH
    ((0xA6, 0x48), (0xCAAF)),  # HANGUL SYLLABLE SSANGCIEUC YE RIEULHIEUH
    ((0xA6, 0x49), (0xCAB0)),  # HANGUL SYLLABLE SSANGCIEUC YE MIEUM
    ((0xA6, 0x4A), (0xCAB1)),  # HANGUL SYLLABLE SSANGCIEUC YE PIEUP
    ((0xA6, 0x4B), (0xCAB2)),  # HANGUL SYLLABLE SSANGCIEUC YE PIEUPSIOS
    ((0xA6, 0x4C), (0xCAB3)),  # HANGUL SYLLABLE SSANGCIEUC YE SIOS
    ((0xA6, 0x4D), (0xCAB4)),  # HANGUL SYLLABLE SSANGCIEUC YE SSANGSIOS
    ((0xA6, 0x4E), (0xCAB5)),  # HANGUL SYLLABLE SSANGCIEUC YE IEUNG
    ((0xA6, 0x4F), (0xCAB6)),  # HANGUL SYLLABLE SSANGCIEUC YE CIEUC
    ((0xA6, 0x50), (0xCAB7)),  # HANGUL SYLLABLE SSANGCIEUC YE CHIEUCH
    ((0xA6, 0x51), (0xCAB8)),  # HANGUL SYLLABLE SSANGCIEUC YE KHIEUKH
    ((0xA6, 0x52), (0xCAB9)),  # HANGUL SYLLABLE SSANGCIEUC YE THIEUTH
    ((0xA6, 0x53), (0xCABA)),  # HANGUL SYLLABLE SSANGCIEUC YE PHIEUPH
    ((0xA6, 0x54), (0xCABB)),  # HANGUL SYLLABLE SSANGCIEUC YE HIEUH
    ((0xA6, 0x55), (0xCABE)),  # HANGUL SYLLABLE SSANGCIEUC O SSANGKIYEOK
    ((0xA6, 0x56), (0xCABF)),  # HANGUL SYLLABLE SSANGCIEUC O KIYEOKSIOS
    ((0xA6, 0x57), (0xCAC1)),  # HANGUL SYLLABLE SSANGCIEUC O NIEUNCIEUC
    ((0xA6, 0x58), (0xCAC2)),  # HANGUL SYLLABLE SSANGCIEUC O NIEUNHIEUH
    ((0xA6, 0x59), (0xCAC3)),  # HANGUL SYLLABLE SSANGCIEUC O TIKEUT
    ((0xA6, 0x5A), (0xCAC5)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULKIYEOK
    ((0xA6, 0x61), (0xCAC6)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULMIEUM
    ((0xA6, 0x62), (0xCAC7)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULPIEUP
    ((0xA6, 0x63), (0xCAC8)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULSIOS
    ((0xA6, 0x64), (0xCAC9)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULTHIEUTH
    ((0xA6, 0x65), (0xCACA)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULPHIEUPH
    ((0xA6, 0x66), (0xCACB)),  # HANGUL SYLLABLE SSANGCIEUC O RIEULHIEUH
    ((0xA6, 0x67), (0xCACE)),  # HANGUL SYLLABLE SSANGCIEUC O PIEUPSIOS
    ((0xA6, 0x68), (0xCAD0)),  # HANGUL SYLLABLE SSANGCIEUC O SSANGSIOS
    ((0xA6, 0x69), (0xCAD2)),  # HANGUL SYLLABLE SSANGCIEUC O CIEUC
    ((0xA6, 0x6A), (0xCAD4)),  # HANGUL SYLLABLE SSANGCIEUC O KHIEUKH
    ((0xA6, 0x6B), (0xCAD5)),  # HANGUL SYLLABLE SSANGCIEUC O THIEUTH
    ((0xA6, 0x6C), (0xCAD6)),  # HANGUL SYLLABLE SSANGCIEUC O PHIEUPH
    ((0xA6, 0x6D), (0xCAD7)),  # HANGUL SYLLABLE SSANGCIEUC O HIEUH
    ((0xA6, 0x6E), (0xCADA)),  # HANGUL SYLLABLE SSANGCIEUC WA SSANGKIYEOK
    ((0xA6, 0x6F), (0xCADB)),  # HANGUL SYLLABLE SSANGCIEUC WA KIYEOKSIOS
    ((0xA6, 0x70), (0xCADC)),  # HANGUL SYLLABLE SSANGCIEUC WA NIEUN
    ((0xA6, 0x71), (0xCADD)),  # HANGUL SYLLABLE SSANGCIEUC WA NIEUNCIEUC
    ((0xA6, 0x72), (0xCADE)),  # HANGUL SYLLABLE SSANGCIEUC WA NIEUNHIEUH
    ((0xA6, 0x73), (0xCADF)),  # HANGUL SYLLABLE SSANGCIEUC WA TIKEUT
    ((0xA6, 0x74), (0xCAE1)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULKIYEOK
    ((0xA6, 0x75), (0xCAE2)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULMIEUM
    ((0xA6, 0x76), (0xCAE3)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULPIEUP
    ((0xA6, 0x77), (0xCAE4)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULSIOS
    ((0xA6, 0x78), (0xCAE5)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULTHIEUTH
    ((0xA6, 0x79), (0xCAE6)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULPHIEUPH
    ((0xA6, 0x7A), (0xCAE7)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEULHIEUH
    ((0xA6, 0x81), (0xCAE8)),  # HANGUL SYLLABLE SSANGCIEUC WA MIEUM
    ((0xA6, 0x82), (0xCAE9)),  # HANGUL SYLLABLE SSANGCIEUC WA PIEUP
    ((0xA6, 0x83), (0xCAEA)),  # HANGUL SYLLABLE SSANGCIEUC WA PIEUPSIOS
    ((0xA6, 0x84), (0xCAEB)),  # HANGUL SYLLABLE SSANGCIEUC WA SIOS
    ((0xA6, 0x85), (0xCAED)),  # HANGUL SYLLABLE SSANGCIEUC WA IEUNG
    ((0xA6, 0x86), (0xCAEE)),  # HANGUL SYLLABLE SSANGCIEUC WA CIEUC
    ((0xA6, 0x87), (0xCAEF)),  # HANGUL SYLLABLE SSANGCIEUC WA CHIEUCH
    ((0xA6, 0x88), (0xCAF0)),  # HANGUL SYLLABLE SSANGCIEUC WA KHIEUKH
    ((0xA6, 0x89), (0xCAF1)),  # HANGUL SYLLABLE SSANGCIEUC WA THIEUTH
    ((0xA6, 0x8A), (0xCAF2)),  # HANGUL SYLLABLE SSANGCIEUC WA PHIEUPH
    ((0xA6, 0x8B), (0xCAF3)),  # HANGUL SYLLABLE SSANGCIEUC WA HIEUH
    ((0xA6, 0x8C), (0xCAF5)),  # HANGUL SYLLABLE SSANGCIEUC WAE KIYEOK
    ((0xA6, 0x8D), (0xCAF6)),  # HANGUL SYLLABLE SSANGCIEUC WAE SSANGKIYEOK
    ((0xA6, 0x8E), (0xCAF7)),  # HANGUL SYLLABLE SSANGCIEUC WAE KIYEOKSIOS
    ((0xA6, 0x8F), (0xCAF8)),  # HANGUL SYLLABLE SSANGCIEUC WAE NIEUN
    ((0xA6, 0x90), (0xCAF9)),  # HANGUL SYLLABLE SSANGCIEUC WAE NIEUNCIEUC
    ((0xA6, 0x91), (0xCAFA)),  # HANGUL SYLLABLE SSANGCIEUC WAE NIEUNHIEUH
    ((0xA6, 0x92), (0xCAFB)),  # HANGUL SYLLABLE SSANGCIEUC WAE TIKEUT
    ((0xA6, 0x93), (0xCAFC)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEUL
    ((0xA6, 0x94), (0xCAFD)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULKIYEOK
    ((0xA6, 0x95), (0xCAFE)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULMIEUM
    ((0xA6, 0x96), (0xCAFF)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULPIEUP
    ((0xA6, 0x97), (0xCB00)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULSIOS
    ((0xA6, 0x98), (0xCB01)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULTHIEUTH
    ((0xA6, 0x99), (0xCB02)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULPHIEUPH
    ((0xA6, 0x9A), (0xCB03)),  # HANGUL SYLLABLE SSANGCIEUC WAE RIEULHIEUH
    ((0xA6, 0x9B), (0xCB04)),  # HANGUL SYLLABLE SSANGCIEUC WAE MIEUM
    ((0xA6, 0x9C), (0xCB05)),  # HANGUL SYLLABLE SSANGCIEUC WAE PIEUP
    ((0xA6, 0x9D), (0xCB06)),  # HANGUL SYLLABLE SSANGCIEUC WAE PIEUPSIOS
    ((0xA6, 0x9E), (0xCB07)),  # HANGUL SYLLABLE SSANGCIEUC WAE SIOS
    ((0xA6, 0x9F), (0xCB09)),  # HANGUL SYLLABLE SSANGCIEUC WAE IEUNG
    ((0xA6, 0xA0), (0xCB0A)),  # HANGUL SYLLABLE SSANGCIEUC WAE CIEUC
    ((0xA6, 0xA1), (0x2500)),  # BOX DRAWINGS LIGHT HORIZONTAL
    ((0xA6, 0xA2), (0x2502)),  # BOX DRAWINGS LIGHT VERTICAL
    ((0xA6, 0xA3), (0x250C)),  # BOX DRAWINGS LIGHT DOWN AND RIGHT
    ((0xA6, 0xA4), (0x2510)),  # BOX DRAWINGS LIGHT DOWN AND LEFT
    ((0xA6, 0xA5), (0x2518)),  # BOX DRAWINGS LIGHT UP AND LEFT
    ((0xA6, 0xA6), (0x2514)),  # BOX DRAWINGS LIGHT UP AND RIGHT
    ((0xA6, 0xA7), (0x251C)),  # BOX DRAWINGS LIGHT VERTICAL AND RIGHT
    ((0xA6, 0xA8), (0x252C)),  # BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
    ((0xA6, 0xA9), (0x2524)),  # BOX DRAWINGS LIGHT VERTICAL AND LEFT
    ((0xA6, 0xAA), (0x2534)),  # BOX DRAWINGS LIGHT UP AND HORIZONTAL
    ((0xA6, 0xAB), (0x253C)),  # BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
    ((0xA6, 0xAC), (0x2501)),  # BOX DRAWINGS HEAVY HORIZONTAL
    ((0xA6, 0xAD), (0x2503)),  # BOX DRAWINGS HEAVY VERTICAL
    ((0xA6, 0xAE), (0x250F)),  # BOX DRAWINGS HEAVY DOWN AND RIGHT
    ((0xA6, 0xAF), (0x2513)),  # BOX DRAWINGS HEAVY DOWN AND LEFT
    ((0xA6, 0xB0), (0x251B)),  # BOX DRAWINGS HEAVY UP AND LEFT
    ((0xA6, 0xB1), (0x2517)),  # BOX DRAWINGS HEAVY UP AND RIGHT
    ((0xA6, 0xB2), (0x2523)),  # BOX DRAWINGS HEAVY VERTICAL AND RIGHT
    ((0xA6, 0xB3), (0x2533)),  # BOX DRAWINGS HEAVY DOWN AND HORIZONTAL
    ((0xA6, 0xB4), (0x252B)),  # BOX DRAWINGS HEAVY VERTICAL AND LEFT
    ((0xA6, 0xB5), (0x253B)),  # BOX DRAWINGS HEAVY UP AND HORIZONTAL
    ((0xA6, 0xB6), (0x254B)),  # BOX DRAWINGS HEAVY VERTICAL AND HORIZONTAL
    ((0xA6, 0xB7), (0x2520)),  # BOX DRAWINGS VERTICAL HEAVY AND RIGHT LIGHT
    ((0xA6, 0xB8), (0x252F)),  # BOX DRAWINGS DOWN LIGHT AND HORIZONTAL HEAVY
    ((0xA6, 0xB9), (0x2528)),  # BOX DRAWINGS VERTICAL HEAVY AND LEFT LIGHT
    ((0xA6, 0xBA), (0x2537)),  # BOX DRAWINGS UP LIGHT AND HORIZONTAL HEAVY
    (
        (0xA6, 0xBB),
        (0x253F),
    ),  # BOX DRAWINGS VERTICAL LIGHT AND HORIZONTAL HEAVY
    ((0xA6, 0xBC), (0x251D)),  # BOX DRAWINGS VERTICAL LIGHT AND RIGHT HEAVY
    ((0xA6, 0xBD), (0x2530)),  # BOX DRAWINGS DOWN HEAVY AND HORIZONTAL LIGHT
    ((0xA6, 0xBE), (0x2525)),  # BOX DRAWINGS VERTICAL LIGHT AND LEFT HEAVY
    ((0xA6, 0xBF), (0x2538)),  # BOX DRAWINGS UP HEAVY AND HORIZONTAL LIGHT
    (
        (0xA6, 0xC0),
        (0x2542),
    ),  # BOX DRAWINGS VERTICAL HEAVY AND HORIZONTAL LIGHT
    ((0xA6, 0xC1), (0x2512)),  # BOX DRAWINGS DOWN HEAVY AND LEFT LIGHT
    ((0xA6, 0xC2), (0x2511)),  # BOX DRAWINGS DOWN LIGHT AND LEFT HEAVY
    ((0xA6, 0xC3), (0x251A)),  # BOX DRAWINGS UP HEAVY AND LEFT LIGHT
    ((0xA6, 0xC4), (0x2519)),  # BOX DRAWINGS UP LIGHT AND LEFT HEAVY
    ((0xA6, 0xC5), (0x2516)),  # BOX DRAWINGS UP HEAVY AND RIGHT LIGHT
    ((0xA6, 0xC6), (0x2515)),  # BOX DRAWINGS UP LIGHT AND RIGHT HEAVY
    ((0xA6, 0xC7), (0x250E)),  # BOX DRAWINGS DOWN HEAVY AND RIGHT LIGHT
    ((0xA6, 0xC8), (0x250D)),  # BOX DRAWINGS DOWN LIGHT AND RIGHT HEAVY
    ((0xA6, 0xC9), (0x251E)),  # BOX DRAWINGS UP HEAVY AND RIGHT DOWN LIGHT
    ((0xA6, 0xCA), (0x251F)),  # BOX DRAWINGS DOWN HEAVY AND RIGHT UP LIGHT
    ((0xA6, 0xCB), (0x2521)),  # BOX DRAWINGS DOWN LIGHT AND RIGHT UP HEAVY
    ((0xA6, 0xCC), (0x2522)),  # BOX DRAWINGS UP LIGHT AND RIGHT DOWN HEAVY
    ((0xA6, 0xCD), (0x2526)),  # BOX DRAWINGS UP HEAVY AND LEFT DOWN LIGHT
    ((0xA6, 0xCE), (0x2527)),  # BOX DRAWINGS DOWN HEAVY AND LEFT UP LIGHT
    ((0xA6, 0xCF), (0x2529)),  # BOX DRAWINGS DOWN LIGHT AND LEFT UP HEAVY
    ((0xA6, 0xD0), (0x252A)),  # BOX DRAWINGS UP LIGHT AND LEFT DOWN HEAVY
    ((0xA6, 0xD1), (0x252D)),  # BOX DRAWINGS LEFT HEAVY AND RIGHT DOWN LIGHT
    ((0xA6, 0xD2), (0x252E)),  # BOX DRAWINGS RIGHT HEAVY AND LEFT DOWN LIGHT
    ((0xA6, 0xD3), (0x2531)),  # BOX DRAWINGS RIGHT LIGHT AND LEFT DOWN HEAVY
    ((0xA6, 0xD4), (0x2532)),  # BOX DRAWINGS LEFT LIGHT AND RIGHT DOWN HEAVY
    ((0xA6, 0xD5), (0x2535)),  # BOX DRAWINGS LEFT HEAVY AND RIGHT UP LIGHT
    ((0xA6, 0xD6), (0x2536)),  # BOX DRAWINGS RIGHT HEAVY AND LEFT UP LIGHT
    ((0xA6, 0xD7), (0x2539)),  # BOX DRAWINGS RIGHT LIGHT AND LEFT UP HEAVY
    ((0xA6, 0xD8), (0x253A)),  # BOX DRAWINGS LEFT LIGHT AND RIGHT UP HEAVY
    (
        (0xA6, 0xD9),
        (0x253D),
    ),  # BOX DRAWINGS LEFT HEAVY AND RIGHT VERTICAL LIGHT
    (
        (0xA6, 0xDA),
        (0x253E),
    ),  # BOX DRAWINGS RIGHT HEAVY AND LEFT VERTICAL LIGHT
    (
        (0xA6, 0xDB),
        (0x2540),
    ),  # BOX DRAWINGS UP HEAVY AND DOWN HORIZONTAL LIGHT
    (
        (0xA6, 0xDC),
        (0x2541),
    ),  # BOX DRAWINGS DOWN HEAVY AND UP HORIZONTAL LIGHT
    (
        (0xA6, 0xDD),
        (0x2543),
    ),  # BOX DRAWINGS LEFT UP HEAVY AND RIGHT DOWN LIGHT
    (
        (0xA6, 0xDE),
        (0x2544),
    ),  # BOX DRAWINGS RIGHT UP HEAVY AND LEFT DOWN LIGHT
    (
        (0xA6, 0xDF),
        (0x2545),
    ),  # BOX DRAWINGS LEFT DOWN HEAVY AND RIGHT UP LIGHT
    (
        (0xA6, 0xE0),
        (0x2546),
    ),  # BOX DRAWINGS RIGHT DOWN HEAVY AND LEFT UP LIGHT
    (
        (0xA6, 0xE1),
        (0x2547),
    ),  # BOX DRAWINGS DOWN LIGHT AND UP HORIZONTAL HEAVY
    (
        (0xA6, 0xE2),
        (0x2548),
    ),  # BOX DRAWINGS UP LIGHT AND DOWN HORIZONTAL HEAVY
    (
        (0xA6, 0xE3),
        (0x2549),
    ),  # BOX DRAWINGS RIGHT LIGHT AND LEFT VERTICAL HEAVY
    (
        (0xA6, 0xE4),
        (0x254A),
    ),  # BOX DRAWINGS LEFT LIGHT AND RIGHT VERTICAL HEAVY
    ((0xA7, 0x41), (0xCB0B)),  # HANGUL SYLLABLE SSANGCIEUC WAE CHIEUCH
    ((0xA7, 0x42), (0xCB0C)),  # HANGUL SYLLABLE SSANGCIEUC WAE KHIEUKH
    ((0xA7, 0x43), (0xCB0D)),  # HANGUL SYLLABLE SSANGCIEUC WAE THIEUTH
    ((0xA7, 0x44), (0xCB0E)),  # HANGUL SYLLABLE SSANGCIEUC WAE PHIEUPH
    ((0xA7, 0x45), (0xCB0F)),  # HANGUL SYLLABLE SSANGCIEUC WAE HIEUH
    ((0xA7, 0x46), (0xCB11)),  # HANGUL SYLLABLE SSANGCIEUC OE KIYEOK
    ((0xA7, 0x47), (0xCB12)),  # HANGUL SYLLABLE SSANGCIEUC OE SSANGKIYEOK
    ((0xA7, 0x48), (0xCB13)),  # HANGUL SYLLABLE SSANGCIEUC OE KIYEOKSIOS
    ((0xA7, 0x49), (0xCB15)),  # HANGUL SYLLABLE SSANGCIEUC OE NIEUNCIEUC
    ((0xA7, 0x4A), (0xCB16)),  # HANGUL SYLLABLE SSANGCIEUC OE NIEUNHIEUH
    ((0xA7, 0x4B), (0xCB17)),  # HANGUL SYLLABLE SSANGCIEUC OE TIKEUT
    ((0xA7, 0x4C), (0xCB19)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULKIYEOK
    ((0xA7, 0x4D), (0xCB1A)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULMIEUM
    ((0xA7, 0x4E), (0xCB1B)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULPIEUP
    ((0xA7, 0x4F), (0xCB1C)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULSIOS
    ((0xA7, 0x50), (0xCB1D)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULTHIEUTH
    ((0xA7, 0x51), (0xCB1E)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULPHIEUPH
    ((0xA7, 0x52), (0xCB1F)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEULHIEUH
    ((0xA7, 0x53), (0xCB22)),  # HANGUL SYLLABLE SSANGCIEUC OE PIEUPSIOS
    ((0xA7, 0x54), (0xCB23)),  # HANGUL SYLLABLE SSANGCIEUC OE SIOS
    ((0xA7, 0x55), (0xCB24)),  # HANGUL SYLLABLE SSANGCIEUC OE SSANGSIOS
    ((0xA7, 0x56), (0xCB25)),  # HANGUL SYLLABLE SSANGCIEUC OE IEUNG
    ((0xA7, 0x57), (0xCB26)),  # HANGUL SYLLABLE SSANGCIEUC OE CIEUC
    ((0xA7, 0x58), (0xCB27)),  # HANGUL SYLLABLE SSANGCIEUC OE CHIEUCH
    ((0xA7, 0x59), (0xCB28)),  # HANGUL SYLLABLE SSANGCIEUC OE KHIEUKH
    ((0xA7, 0x5A), (0xCB29)),  # HANGUL SYLLABLE SSANGCIEUC OE THIEUTH
    ((0xA7, 0x61), (0xCB2A)),  # HANGUL SYLLABLE SSANGCIEUC OE PHIEUPH
    ((0xA7, 0x62), (0xCB2B)),  # HANGUL SYLLABLE SSANGCIEUC OE HIEUH
    ((0xA7, 0x63), (0xCB2C)),  # HANGUL SYLLABLE SSANGCIEUC YO
    ((0xA7, 0x64), (0xCB2D)),  # HANGUL SYLLABLE SSANGCIEUC YO KIYEOK
    ((0xA7, 0x65), (0xCB2E)),  # HANGUL SYLLABLE SSANGCIEUC YO SSANGKIYEOK
    ((0xA7, 0x66), (0xCB2F)),  # HANGUL SYLLABLE SSANGCIEUC YO KIYEOKSIOS
    ((0xA7, 0x67), (0xCB30)),  # HANGUL SYLLABLE SSANGCIEUC YO NIEUN
    ((0xA7, 0x68), (0xCB31)),  # HANGUL SYLLABLE SSANGCIEUC YO NIEUNCIEUC
    ((0xA7, 0x69), (0xCB32)),  # HANGUL SYLLABLE SSANGCIEUC YO NIEUNHIEUH
    ((0xA7, 0x6A), (0xCB33)),  # HANGUL SYLLABLE SSANGCIEUC YO TIKEUT
    ((0xA7, 0x6B), (0xCB34)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEUL
    ((0xA7, 0x6C), (0xCB35)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULKIYEOK
    ((0xA7, 0x6D), (0xCB36)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULMIEUM
    ((0xA7, 0x6E), (0xCB37)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULPIEUP
    ((0xA7, 0x6F), (0xCB38)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULSIOS
    ((0xA7, 0x70), (0xCB39)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULTHIEUTH
    ((0xA7, 0x71), (0xCB3A)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULPHIEUPH
    ((0xA7, 0x72), (0xCB3B)),  # HANGUL SYLLABLE SSANGCIEUC YO RIEULHIEUH
    ((0xA7, 0x73), (0xCB3C)),  # HANGUL SYLLABLE SSANGCIEUC YO MIEUM
    ((0xA7, 0x74), (0xCB3D)),  # HANGUL SYLLABLE SSANGCIEUC YO PIEUP
    ((0xA7, 0x75), (0xCB3E)),  # HANGUL SYLLABLE SSANGCIEUC YO PIEUPSIOS
    ((0xA7, 0x76), (0xCB3F)),  # HANGUL SYLLABLE SSANGCIEUC YO SIOS
    ((0xA7, 0x77), (0xCB40)),  # HANGUL SYLLABLE SSANGCIEUC YO SSANGSIOS
    ((0xA7, 0x78), (0xCB42)),  # HANGUL SYLLABLE SSANGCIEUC YO CIEUC
    ((0xA7, 0x79), (0xCB43)),  # HANGUL SYLLABLE SSANGCIEUC YO CHIEUCH
    ((0xA7, 0x7A), (0xCB44)),  # HANGUL SYLLABLE SSANGCIEUC YO KHIEUKH
    ((0xA7, 0x81), (0xCB45)),  # HANGUL SYLLABLE SSANGCIEUC YO THIEUTH
    ((0xA7, 0x82), (0xCB46)),  # HANGUL SYLLABLE SSANGCIEUC YO PHIEUPH
    ((0xA7, 0x83), (0xCB47)),  # HANGUL SYLLABLE SSANGCIEUC YO HIEUH
    ((0xA7, 0x84), (0xCB4A)),  # HANGUL SYLLABLE SSANGCIEUC U SSANGKIYEOK
    ((0xA7, 0x85), (0xCB4B)),  # HANGUL SYLLABLE SSANGCIEUC U KIYEOKSIOS
    ((0xA7, 0x86), (0xCB4D)),  # HANGUL SYLLABLE SSANGCIEUC U NIEUNCIEUC
    ((0xA7, 0x87), (0xCB4E)),  # HANGUL SYLLABLE SSANGCIEUC U NIEUNHIEUH
    ((0xA7, 0x88), (0xCB4F)),  # HANGUL SYLLABLE SSANGCIEUC U TIKEUT
    ((0xA7, 0x89), (0xCB51)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULKIYEOK
    ((0xA7, 0x8A), (0xCB52)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULMIEUM
    ((0xA7, 0x8B), (0xCB53)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULPIEUP
    ((0xA7, 0x8C), (0xCB54)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULSIOS
    ((0xA7, 0x8D), (0xCB55)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULTHIEUTH
    ((0xA7, 0x8E), (0xCB56)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULPHIEUPH
    ((0xA7, 0x8F), (0xCB57)),  # HANGUL SYLLABLE SSANGCIEUC U RIEULHIEUH
    ((0xA7, 0x90), (0xCB5A)),  # HANGUL SYLLABLE SSANGCIEUC U PIEUPSIOS
    ((0xA7, 0x91), (0xCB5B)),  # HANGUL SYLLABLE SSANGCIEUC U SIOS
    ((0xA7, 0x92), (0xCB5C)),  # HANGUL SYLLABLE SSANGCIEUC U SSANGSIOS
    ((0xA7, 0x93), (0xCB5E)),  # HANGUL SYLLABLE SSANGCIEUC U CIEUC
    ((0xA7, 0x94), (0xCB5F)),  # HANGUL SYLLABLE SSANGCIEUC U CHIEUCH
    ((0xA7, 0x95), (0xCB60)),  # HANGUL SYLLABLE SSANGCIEUC U KHIEUKH
    ((0xA7, 0x96), (0xCB61)),  # HANGUL SYLLABLE SSANGCIEUC U THIEUTH
    ((0xA7, 0x97), (0xCB62)),  # HANGUL SYLLABLE SSANGCIEUC U PHIEUPH
    ((0xA7, 0x98), (0xCB63)),  # HANGUL SYLLABLE SSANGCIEUC U HIEUH
    ((0xA7, 0x99), (0xCB65)),  # HANGUL SYLLABLE SSANGCIEUC WEO KIYEOK
    ((0xA7, 0x9A), (0xCB66)),  # HANGUL SYLLABLE SSANGCIEUC WEO SSANGKIYEOK
    ((0xA7, 0x9B), (0xCB67)),  # HANGUL SYLLABLE SSANGCIEUC WEO KIYEOKSIOS
    ((0xA7, 0x9C), (0xCB68)),  # HANGUL SYLLABLE SSANGCIEUC WEO NIEUN
    ((0xA7, 0x9D), (0xCB69)),  # HANGUL SYLLABLE SSANGCIEUC WEO NIEUNCIEUC
    ((0xA7, 0x9E), (0xCB6A)),  # HANGUL SYLLABLE SSANGCIEUC WEO NIEUNHIEUH
    ((0xA7, 0x9F), (0xCB6B)),  # HANGUL SYLLABLE SSANGCIEUC WEO TIKEUT
    ((0xA7, 0xA0), (0xCB6C)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEUL
    ((0xA7, 0xA1), (0x3395)),  # SQUARE MU L
    ((0xA7, 0xA2), (0x3396)),  # SQUARE ML
    ((0xA7, 0xA3), (0x3397)),  # SQUARE DL
    ((0xA7, 0xA4), (0x2113)),  # SCRIPT SMALL L
    ((0xA7, 0xA5), (0x3398)),  # SQUARE KL
    ((0xA7, 0xA6), (0x33C4)),  # SQUARE CC
    ((0xA7, 0xA7), (0x33A3)),  # SQUARE MM CUBED
    ((0xA7, 0xA8), (0x33A4)),  # SQUARE CM CUBED
    ((0xA7, 0xA9), (0x33A5)),  # SQUARE M CUBED
    ((0xA7, 0xAA), (0x33A6)),  # SQUARE KM CUBED
    ((0xA7, 0xAB), (0x3399)),  # SQUARE FM
    ((0xA7, 0xAC), (0x339A)),  # SQUARE NM
    ((0xA7, 0xAD), (0x339B)),  # SQUARE MU M
    ((0xA7, 0xAE), (0x339C)),  # SQUARE MM
    ((0xA7, 0xAF), (0x339D)),  # SQUARE CM
    ((0xA7, 0xB0), (0x339E)),  # SQUARE KM
    ((0xA7, 0xB1), (0x339F)),  # SQUARE MM SQUARED
    ((0xA7, 0xB2), (0x33A0)),  # SQUARE CM SQUARED
    ((0xA7, 0xB3), (0x33A1)),  # SQUARE M SQUARED
    ((0xA7, 0xB4), (0x33A2)),  # SQUARE KM SQUARED
    ((0xA7, 0xB5), (0x33CA)),  # SQUARE HA
    ((0xA7, 0xB6), (0x338D)),  # SQUARE MU G
    ((0xA7, 0xB7), (0x338E)),  # SQUARE MG
    ((0xA7, 0xB8), (0x338F)),  # SQUARE KG
    ((0xA7, 0xB9), (0x33CF)),  # SQUARE KT
    ((0xA7, 0xBA), (0x3388)),  # SQUARE CAL
    ((0xA7, 0xBB), (0x3389)),  # SQUARE KCAL
    ((0xA7, 0xBC), (0x33C8)),  # SQUARE DB
    ((0xA7, 0xBD), (0x33A7)),  # SQUARE M OVER S
    ((0xA7, 0xBE), (0x33A8)),  # SQUARE M OVER S SQUARED
    ((0xA7, 0xBF), (0x33B0)),  # SQUARE PS
    ((0xA7, 0xC0), (0x33B1)),  # SQUARE NS
    ((0xA7, 0xC1), (0x33B2)),  # SQUARE MU S
    ((0xA7, 0xC2), (0x33B3)),  # SQUARE MS
    ((0xA7, 0xC3), (0x33B4)),  # SQUARE PV
    ((0xA7, 0xC4), (0x33B5)),  # SQUARE NV
    ((0xA7, 0xC5), (0x33B6)),  # SQUARE MU V
    ((0xA7, 0xC6), (0x33B7)),  # SQUARE MV
    ((0xA7, 0xC7), (0x33B8)),  # SQUARE KV
    ((0xA7, 0xC8), (0x33B9)),  # SQUARE MV MEGA
    ((0xA7, 0xC9), (0x3380)),  # SQUARE PA AMPS
    ((0xA7, 0xCA), (0x3381)),  # SQUARE NA
    ((0xA7, 0xCB), (0x3382)),  # SQUARE MU A
    ((0xA7, 0xCC), (0x3383)),  # SQUARE MA
    ((0xA7, 0xCD), (0x3384)),  # SQUARE KA
    ((0xA7, 0xCE), (0x33BA)),  # SQUARE PW
    ((0xA7, 0xCF), (0x33BB)),  # SQUARE NW
    ((0xA7, 0xD0), (0x33BC)),  # SQUARE MU W
    ((0xA7, 0xD1), (0x33BD)),  # SQUARE MW
    ((0xA7, 0xD2), (0x33BE)),  # SQUARE KW
    ((0xA7, 0xD3), (0x33BF)),  # SQUARE MW MEGA
    ((0xA7, 0xD4), (0x3390)),  # SQUARE HZ
    ((0xA7, 0xD5), (0x3391)),  # SQUARE KHZ
    ((0xA7, 0xD6), (0x3392)),  # SQUARE MHZ
    ((0xA7, 0xD7), (0x3393)),  # SQUARE GHZ
    ((0xA7, 0xD8), (0x3394)),  # SQUARE THZ
    ((0xA7, 0xD9), (0x2126)),  # OHM SIGN
    ((0xA7, 0xDA), (0x33C0)),  # SQUARE K OHM
    ((0xA7, 0xDB), (0x33C1)),  # SQUARE M OHM
    ((0xA7, 0xDC), (0x338A)),  # SQUARE PF
    ((0xA7, 0xDD), (0x338B)),  # SQUARE NF
    ((0xA7, 0xDE), (0x338C)),  # SQUARE MU F
    ((0xA7, 0xDF), (0x33D6)),  # SQUARE MOL
    ((0xA7, 0xE0), (0x33C5)),  # SQUARE CD
    ((0xA7, 0xE1), (0x33AD)),  # SQUARE RAD
    ((0xA7, 0xE2), (0x33AE)),  # SQUARE RAD OVER S
    ((0xA7, 0xE3), (0x33AF)),  # SQUARE RAD OVER S SQUARED
    ((0xA7, 0xE4), (0x33DB)),  # SQUARE SR
    ((0xA7, 0xE5), (0x33A9)),  # SQUARE PA
    ((0xA7, 0xE6), (0x33AA)),  # SQUARE KPA
    ((0xA7, 0xE7), (0x33AB)),  # SQUARE MPA
    ((0xA7, 0xE8), (0x33AC)),  # SQUARE GPA
    ((0xA7, 0xE9), (0x33DD)),  # SQUARE WB
    ((0xA7, 0xEA), (0x33D0)),  # SQUARE LM
    ((0xA7, 0xEB), (0x33D3)),  # SQUARE LX
    ((0xA7, 0xEC), (0x33C3)),  # SQUARE BQ
    ((0xA7, 0xED), (0x33C9)),  # SQUARE GY
    ((0xA7, 0xEE), (0x33DC)),  # SQUARE SV
    ((0xA7, 0xEF), (0x33C6)),  # SQUARE C OVER KG
    ((0xA8, 0x41), (0xCB6D)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULKIYEOK
    ((0xA8, 0x42), (0xCB6E)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULMIEUM
    ((0xA8, 0x43), (0xCB6F)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULPIEUP
    ((0xA8, 0x44), (0xCB70)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULSIOS
    ((0xA8, 0x45), (0xCB71)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULTHIEUTH
    ((0xA8, 0x46), (0xCB72)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULPHIEUPH
    ((0xA8, 0x47), (0xCB73)),  # HANGUL SYLLABLE SSANGCIEUC WEO RIEULHIEUH
    ((0xA8, 0x48), (0xCB74)),  # HANGUL SYLLABLE SSANGCIEUC WEO MIEUM
    ((0xA8, 0x49), (0xCB75)),  # HANGUL SYLLABLE SSANGCIEUC WEO PIEUP
    ((0xA8, 0x4A), (0xCB76)),  # HANGUL SYLLABLE SSANGCIEUC WEO PIEUPSIOS
    ((0xA8, 0x4B), (0xCB77)),  # HANGUL SYLLABLE SSANGCIEUC WEO SIOS
    ((0xA8, 0x4C), (0xCB7A)),  # HANGUL SYLLABLE SSANGCIEUC WEO CIEUC
    ((0xA8, 0x4D), (0xCB7B)),  # HANGUL SYLLABLE SSANGCIEUC WEO CHIEUCH
    ((0xA8, 0x4E), (0xCB7C)),  # HANGUL SYLLABLE SSANGCIEUC WEO KHIEUKH
    ((0xA8, 0x4F), (0xCB7D)),  # HANGUL SYLLABLE SSANGCIEUC WEO THIEUTH
    ((0xA8, 0x50), (0xCB7E)),  # HANGUL SYLLABLE SSANGCIEUC WEO PHIEUPH
    ((0xA8, 0x51), (0xCB7F)),  # HANGUL SYLLABLE SSANGCIEUC WEO HIEUH
    ((0xA8, 0x52), (0xCB80)),  # HANGUL SYLLABLE SSANGCIEUC WE
    ((0xA8, 0x53), (0xCB81)),  # HANGUL SYLLABLE SSANGCIEUC WE KIYEOK
    ((0xA8, 0x54), (0xCB82)),  # HANGUL SYLLABLE SSANGCIEUC WE SSANGKIYEOK
    ((0xA8, 0x55), (0xCB83)),  # HANGUL SYLLABLE SSANGCIEUC WE KIYEOKSIOS
    ((0xA8, 0x56), (0xCB84)),  # HANGUL SYLLABLE SSANGCIEUC WE NIEUN
    ((0xA8, 0x57), (0xCB85)),  # HANGUL SYLLABLE SSANGCIEUC WE NIEUNCIEUC
    ((0xA8, 0x58), (0xCB86)),  # HANGUL SYLLABLE SSANGCIEUC WE NIEUNHIEUH
    ((0xA8, 0x59), (0xCB87)),  # HANGUL SYLLABLE SSANGCIEUC WE TIKEUT
    ((0xA8, 0x5A), (0xCB88)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEUL
    ((0xA8, 0x61), (0xCB89)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULKIYEOK
    ((0xA8, 0x62), (0xCB8A)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULMIEUM
    ((0xA8, 0x63), (0xCB8B)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULPIEUP
    ((0xA8, 0x64), (0xCB8C)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULSIOS
    ((0xA8, 0x65), (0xCB8D)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULTHIEUTH
    ((0xA8, 0x66), (0xCB8E)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULPHIEUPH
    ((0xA8, 0x67), (0xCB8F)),  # HANGUL SYLLABLE SSANGCIEUC WE RIEULHIEUH
    ((0xA8, 0x68), (0xCB90)),  # HANGUL SYLLABLE SSANGCIEUC WE MIEUM
    ((0xA8, 0x69), (0xCB91)),  # HANGUL SYLLABLE SSANGCIEUC WE PIEUP
    ((0xA8, 0x6A), (0xCB92)),  # HANGUL SYLLABLE SSANGCIEUC WE PIEUPSIOS
    ((0xA8, 0x6B), (0xCB93)),  # HANGUL SYLLABLE SSANGCIEUC WE SIOS
    ((0xA8, 0x6C), (0xCB94)),  # HANGUL SYLLABLE SSANGCIEUC WE SSANGSIOS
    ((0xA8, 0x6D), (0xCB95)),  # HANGUL SYLLABLE SSANGCIEUC WE IEUNG
    ((0xA8, 0x6E), (0xCB96)),  # HANGUL SYLLABLE SSANGCIEUC WE CIEUC
    ((0xA8, 0x6F), (0xCB97)),  # HANGUL SYLLABLE SSANGCIEUC WE CHIEUCH
    ((0xA8, 0x70), (0xCB98)),  # HANGUL SYLLABLE SSANGCIEUC WE KHIEUKH
    ((0xA8, 0x71), (0xCB99)),  # HANGUL SYLLABLE SSANGCIEUC WE THIEUTH
    ((0xA8, 0x72), (0xCB9A)),  # HANGUL SYLLABLE SSANGCIEUC WE PHIEUPH
    ((0xA8, 0x73), (0xCB9B)),  # HANGUL SYLLABLE SSANGCIEUC WE HIEUH
    ((0xA8, 0x74), (0xCB9D)),  # HANGUL SYLLABLE SSANGCIEUC WI KIYEOK
    ((0xA8, 0x75), (0xCB9E)),  # HANGUL SYLLABLE SSANGCIEUC WI SSANGKIYEOK
    ((0xA8, 0x76), (0xCB9F)),  # HANGUL SYLLABLE SSANGCIEUC WI KIYEOKSIOS
    ((0xA8, 0x77), (0xCBA0)),  # HANGUL SYLLABLE SSANGCIEUC WI NIEUN
    ((0xA8, 0x78), (0xCBA1)),  # HANGUL SYLLABLE SSANGCIEUC WI NIEUNCIEUC
    ((0xA8, 0x79), (0xCBA2)),  # HANGUL SYLLABLE SSANGCIEUC WI NIEUNHIEUH
    ((0xA8, 0x7A), (0xCBA3)),  # HANGUL SYLLABLE SSANGCIEUC WI TIKEUT
    ((0xA8, 0x81), (0xCBA4)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEUL
    ((0xA8, 0x82), (0xCBA5)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULKIYEOK
    ((0xA8, 0x83), (0xCBA6)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULMIEUM
    ((0xA8, 0x84), (0xCBA7)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULPIEUP
    ((0xA8, 0x85), (0xCBA8)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULSIOS
    ((0xA8, 0x86), (0xCBA9)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULTHIEUTH
    ((0xA8, 0x87), (0xCBAA)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULPHIEUPH
    ((0xA8, 0x88), (0xCBAB)),  # HANGUL SYLLABLE SSANGCIEUC WI RIEULHIEUH
    ((0xA8, 0x89), (0xCBAC)),  # HANGUL SYLLABLE SSANGCIEUC WI MIEUM
    ((0xA8, 0x8A), (0xCBAD)),  # HANGUL SYLLABLE SSANGCIEUC WI PIEUP
    ((0xA8, 0x8B), (0xCBAE)),  # HANGUL SYLLABLE SSANGCIEUC WI PIEUPSIOS
    ((0xA8, 0x8C), (0xCBAF)),  # HANGUL SYLLABLE SSANGCIEUC WI SIOS
    ((0xA8, 0x8D), (0xCBB0)),  # HANGUL SYLLABLE SSANGCIEUC WI SSANGSIOS
    ((0xA8, 0x8E), (0xCBB1)),  # HANGUL SYLLABLE SSANGCIEUC WI IEUNG
    ((0xA8, 0x8F), (0xCBB2)),  # HANGUL SYLLABLE SSANGCIEUC WI CIEUC
    ((0xA8, 0x90), (0xCBB3)),  # HANGUL SYLLABLE SSANGCIEUC WI CHIEUCH
    ((0xA8, 0x91), (0xCBB4)),  # HANGUL SYLLABLE SSANGCIEUC WI KHIEUKH
    ((0xA8, 0x92), (0xCBB5)),  # HANGUL SYLLABLE SSANGCIEUC WI THIEUTH
    ((0xA8, 0x93), (0xCBB6)),  # HANGUL SYLLABLE SSANGCIEUC WI PHIEUPH
    ((0xA8, 0x94), (0xCBB7)),  # HANGUL SYLLABLE SSANGCIEUC WI HIEUH
    ((0xA8, 0x95), (0xCBB9)),  # HANGUL SYLLABLE SSANGCIEUC YU KIYEOK
    ((0xA8, 0x96), (0xCBBA)),  # HANGUL SYLLABLE SSANGCIEUC YU SSANGKIYEOK
    ((0xA8, 0x97), (0xCBBB)),  # HANGUL SYLLABLE SSANGCIEUC YU KIYEOKSIOS
    ((0xA8, 0x98), (0xCBBC)),  # HANGUL SYLLABLE SSANGCIEUC YU NIEUN
    ((0xA8, 0x99), (0xCBBD)),  # HANGUL SYLLABLE SSANGCIEUC YU NIEUNCIEUC
    ((0xA8, 0x9A), (0xCBBE)),  # HANGUL SYLLABLE SSANGCIEUC YU NIEUNHIEUH
    ((0xA8, 0x9B), (0xCBBF)),  # HANGUL SYLLABLE SSANGCIEUC YU TIKEUT
    ((0xA8, 0x9C), (0xCBC0)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEUL
    ((0xA8, 0x9D), (0xCBC1)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULKIYEOK
    ((0xA8, 0x9E), (0xCBC2)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULMIEUM
    ((0xA8, 0x9F), (0xCBC3)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULPIEUP
    ((0xA8, 0xA0), (0xCBC4)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULSIOS
    ((0xA8, 0xA1), (0x00C6)),  # LATIN CAPITAL LETTER AE
    ((0xA8, 0xA2), (0x00D0)),  # LATIN CAPITAL LETTER ETH
    ((0xA8, 0xA3), (0x00AA)),  # FEMININE ORDINAL INDICATOR
    ((0xA8, 0xA4), (0x0126)),  # LATIN CAPITAL LETTER H WITH STROKE
    ((0xA8, 0xA6), (0x0132)),  # LATIN CAPITAL LIGATURE IJ
    ((0xA8, 0xA8), (0x013F)),  # LATIN CAPITAL LETTER L WITH MIDDLE DOT
    ((0xA8, 0xA9), (0x0141)),  # LATIN CAPITAL LETTER L WITH STROKE
    ((0xA8, 0xAA), (0x00D8)),  # LATIN CAPITAL LETTER O WITH STROKE
    ((0xA8, 0xAB), (0x0152)),  # LATIN CAPITAL LIGATURE OE
    ((0xA8, 0xAC), (0x00BA)),  # MASCULINE ORDINAL INDICATOR
    ((0xA8, 0xAD), (0x00DE)),  # LATIN CAPITAL LETTER THORN
    ((0xA8, 0xAE), (0x0166)),  # LATIN CAPITAL LETTER T WITH STROKE
    ((0xA8, 0xAF), (0x014A)),  # LATIN CAPITAL LETTER ENG
    ((0xA8, 0xB1), (0x3260)),  # CIRCLED HANGUL KIYEOK
    ((0xA8, 0xB2), (0x3261)),  # CIRCLED HANGUL NIEUN
    ((0xA8, 0xB3), (0x3262)),  # CIRCLED HANGUL TIKEUT
    ((0xA8, 0xB4), (0x3263)),  # CIRCLED HANGUL RIEUL
    ((0xA8, 0xB5), (0x3264)),  # CIRCLED HANGUL MIEUM
    ((0xA8, 0xB6), (0x3265)),  # CIRCLED HANGUL PIEUP
    ((0xA8, 0xB7), (0x3266)),  # CIRCLED HANGUL SIOS
    ((0xA8, 0xB8), (0x3267)),  # CIRCLED HANGUL IEUNG
    ((0xA8, 0xB9), (0x3268)),  # CIRCLED HANGUL CIEUC
    ((0xA8, 0xBA), (0x3269)),  # CIRCLED HANGUL CHIEUCH
    ((0xA8, 0xBB), (0x326A)),  # CIRCLED HANGUL KHIEUKH
    ((0xA8, 0xBC), (0x326B)),  # CIRCLED HANGUL THIEUTH
    ((0xA8, 0xBD), (0x326C)),  # CIRCLED HANGUL PHIEUPH
    ((0xA8, 0xBE), (0x326D)),  # CIRCLED HANGUL HIEUH
    ((0xA8, 0xBF), (0x326E)),  # CIRCLED HANGUL KIYEOK A
    ((0xA8, 0xC0), (0x326F)),  # CIRCLED HANGUL NIEUN A
    ((0xA8, 0xC1), (0x3270)),  # CIRCLED HANGUL TIKEUT A
    ((0xA8, 0xC2), (0x3271)),  # CIRCLED HANGUL RIEUL A
    ((0xA8, 0xC3), (0x3272)),  # CIRCLED HANGUL MIEUM A
    ((0xA8, 0xC4), (0x3273)),  # CIRCLED HANGUL PIEUP A
    ((0xA8, 0xC5), (0x3274)),  # CIRCLED HANGUL SIOS A
    ((0xA8, 0xC6), (0x3275)),  # CIRCLED HANGUL IEUNG A
    ((0xA8, 0xC7), (0x3276)),  # CIRCLED HANGUL CIEUC A
    ((0xA8, 0xC8), (0x3277)),  # CIRCLED HANGUL CHIEUCH A
    ((0xA8, 0xC9), (0x3278)),  # CIRCLED HANGUL KHIEUKH A
    ((0xA8, 0xCA), (0x3279)),  # CIRCLED HANGUL THIEUTH A
    ((0xA8, 0xCB), (0x327A)),  # CIRCLED HANGUL PHIEUPH A
    ((0xA8, 0xCC), (0x327B)),  # CIRCLED HANGUL HIEUH A
    ((0xA8, 0xCD), (0x24D0)),  # CIRCLED LATIN SMALL LETTER A
    ((0xA8, 0xCE), (0x24D1)),  # CIRCLED LATIN SMALL LETTER B
    ((0xA8, 0xCF), (0x24D2)),  # CIRCLED LATIN SMALL LETTER C
    ((0xA8, 0xD0), (0x24D3)),  # CIRCLED LATIN SMALL LETTER D
    ((0xA8, 0xD1), (0x24D4)),  # CIRCLED LATIN SMALL LETTER E
    ((0xA8, 0xD2), (0x24D5)),  # CIRCLED LATIN SMALL LETTER F
    ((0xA8, 0xD3), (0x24D6)),  # CIRCLED LATIN SMALL LETTER G
    ((0xA8, 0xD4), (0x24D7)),  # CIRCLED LATIN SMALL LETTER H
    ((0xA8, 0xD5), (0x24D8)),  # CIRCLED LATIN SMALL LETTER I
    ((0xA8, 0xD6), (0x24D9)),  # CIRCLED LATIN SMALL LETTER J
    ((0xA8, 0xD7), (0x24DA)),  # CIRCLED LATIN SMALL LETTER K
    ((0xA8, 0xD8), (0x24DB)),  # CIRCLED LATIN SMALL LETTER L
    ((0xA8, 0xD9), (0x24DC)),  # CIRCLED LATIN SMALL LETTER M
    ((0xA8, 0xDA), (0x24DD)),  # CIRCLED LATIN SMALL LETTER N
    ((0xA8, 0xDB), (0x24DE)),  # CIRCLED LATIN SMALL LETTER O
    ((0xA8, 0xDC), (0x24DF)),  # CIRCLED LATIN SMALL LETTER P
    ((0xA8, 0xDD), (0x24E0)),  # CIRCLED LATIN SMALL LETTER Q
    ((0xA8, 0xDE), (0x24E1)),  # CIRCLED LATIN SMALL LETTER R
    ((0xA8, 0xDF), (0x24E2)),  # CIRCLED LATIN SMALL LETTER S
    ((0xA8, 0xE0), (0x24E3)),  # CIRCLED LATIN SMALL LETTER T
    ((0xA8, 0xE1), (0x24E4)),  # CIRCLED LATIN SMALL LETTER U
    ((0xA8, 0xE2), (0x24E5)),  # CIRCLED LATIN SMALL LETTER V
    ((0xA8, 0xE3), (0x24E6)),  # CIRCLED LATIN SMALL LETTER W
    ((0xA8, 0xE4), (0x24E7)),  # CIRCLED LATIN SMALL LETTER X
    ((0xA8, 0xE5), (0x24E8)),  # CIRCLED LATIN SMALL LETTER Y
    ((0xA8, 0xE6), (0x24E9)),  # CIRCLED LATIN SMALL LETTER Z
    ((0xA8, 0xE7), (0x2460)),  # CIRCLED DIGIT ONE
    ((0xA8, 0xE8), (0x2461)),  # CIRCLED DIGIT TWO
    ((0xA8, 0xE9), (0x2462)),  # CIRCLED DIGIT THREE
    ((0xA8, 0xEA), (0x2463)),  # CIRCLED DIGIT FOUR
    ((0xA8, 0xEB), (0x2464)),  # CIRCLED DIGIT FIVE
    ((0xA8, 0xEC), (0x2465)),  # CIRCLED DIGIT SIX
    ((0xA8, 0xED), (0x2466)),  # CIRCLED DIGIT SEVEN
    ((0xA8, 0xEE), (0x2467)),  # CIRCLED DIGIT EIGHT
    ((0xA8, 0xEF), (0x2468)),  # CIRCLED DIGIT NINE
    ((0xA8, 0xF0), (0x2469)),  # CIRCLED NUMBER TEN
    ((0xA8, 0xF1), (0x246A)),  # CIRCLED NUMBER ELEVEN
    ((0xA8, 0xF2), (0x246B)),  # CIRCLED NUMBER TWELVE
    ((0xA8, 0xF3), (0x246C)),  # CIRCLED NUMBER THIRTEEN
    ((0xA8, 0xF4), (0x246D)),  # CIRCLED NUMBER FOURTEEN
    ((0xA8, 0xF5), (0x246E)),  # CIRCLED NUMBER FIFTEEN
    ((0xA8, 0xF6), (0x00BD)),  # VULGAR FRACTION ONE HALF
    ((0xA8, 0xF7), (0x2153)),  # VULGAR FRACTION ONE THIRD
    ((0xA8, 0xF8), (0x2154)),  # VULGAR FRACTION TWO THIRDS
    ((0xA8, 0xF9), (0x00BC)),  # VULGAR FRACTION ONE QUARTER
    ((0xA8, 0xFA), (0x00BE)),  # VULGAR FRACTION THREE QUARTERS
    ((0xA8, 0xFB), (0x215B)),  # VULGAR FRACTION ONE EIGHTH
    ((0xA8, 0xFC), (0x215C)),  # VULGAR FRACTION THREE EIGHTHS
    ((0xA8, 0xFD), (0x215D)),  # VULGAR FRACTION FIVE EIGHTHS
    ((0xA8, 0xFE), (0x215E)),  # VULGAR FRACTION SEVEN EIGHTHS
    ((0xA9, 0x41), (0xCBC5)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULTHIEUTH
    ((0xA9, 0x42), (0xCBC6)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULPHIEUPH
    ((0xA9, 0x43), (0xCBC7)),  # HANGUL SYLLABLE SSANGCIEUC YU RIEULHIEUH
    ((0xA9, 0x44), (0xCBC8)),  # HANGUL SYLLABLE SSANGCIEUC YU MIEUM
    ((0xA9, 0x45), (0xCBC9)),  # HANGUL SYLLABLE SSANGCIEUC YU PIEUP
    ((0xA9, 0x46), (0xCBCA)),  # HANGUL SYLLABLE SSANGCIEUC YU PIEUPSIOS
    ((0xA9, 0x47), (0xCBCB)),  # HANGUL SYLLABLE SSANGCIEUC YU SIOS
    ((0xA9, 0x48), (0xCBCC)),  # HANGUL SYLLABLE SSANGCIEUC YU SSANGSIOS
    ((0xA9, 0x49), (0xCBCD)),  # HANGUL SYLLABLE SSANGCIEUC YU IEUNG
    ((0xA9, 0x4A), (0xCBCE)),  # HANGUL SYLLABLE SSANGCIEUC YU CIEUC
    ((0xA9, 0x4B), (0xCBCF)),  # HANGUL SYLLABLE SSANGCIEUC YU CHIEUCH
    ((0xA9, 0x4C), (0xCBD0)),  # HANGUL SYLLABLE SSANGCIEUC YU KHIEUKH
    ((0xA9, 0x4D), (0xCBD1)),  # HANGUL SYLLABLE SSANGCIEUC YU THIEUTH
    ((0xA9, 0x4E), (0xCBD2)),  # HANGUL SYLLABLE SSANGCIEUC YU PHIEUPH
    ((0xA9, 0x4F), (0xCBD3)),  # HANGUL SYLLABLE SSANGCIEUC YU HIEUH
    ((0xA9, 0x50), (0xCBD5)),  # HANGUL SYLLABLE SSANGCIEUC EU KIYEOK
    ((0xA9, 0x51), (0xCBD6)),  # HANGUL SYLLABLE SSANGCIEUC EU SSANGKIYEOK
    ((0xA9, 0x52), (0xCBD7)),  # HANGUL SYLLABLE SSANGCIEUC EU KIYEOKSIOS
    ((0xA9, 0x53), (0xCBD8)),  # HANGUL SYLLABLE SSANGCIEUC EU NIEUN
    ((0xA9, 0x54), (0xCBD9)),  # HANGUL SYLLABLE SSANGCIEUC EU NIEUNCIEUC
    ((0xA9, 0x55), (0xCBDA)),  # HANGUL SYLLABLE SSANGCIEUC EU NIEUNHIEUH
    ((0xA9, 0x56), (0xCBDB)),  # HANGUL SYLLABLE SSANGCIEUC EU TIKEUT
    ((0xA9, 0x57), (0xCBDC)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEUL
    ((0xA9, 0x58), (0xCBDD)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULKIYEOK
    ((0xA9, 0x59), (0xCBDE)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULMIEUM
    ((0xA9, 0x5A), (0xCBDF)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULPIEUP
    ((0xA9, 0x61), (0xCBE0)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULSIOS
    ((0xA9, 0x62), (0xCBE1)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULTHIEUTH
    ((0xA9, 0x63), (0xCBE2)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULPHIEUPH
    ((0xA9, 0x64), (0xCBE3)),  # HANGUL SYLLABLE SSANGCIEUC EU RIEULHIEUH
    ((0xA9, 0x65), (0xCBE5)),  # HANGUL SYLLABLE SSANGCIEUC EU PIEUP
    ((0xA9, 0x66), (0xCBE6)),  # HANGUL SYLLABLE SSANGCIEUC EU PIEUPSIOS
    ((0xA9, 0x67), (0xCBE8)),  # HANGUL SYLLABLE SSANGCIEUC EU SSANGSIOS
    ((0xA9, 0x68), (0xCBEA)),  # HANGUL SYLLABLE SSANGCIEUC EU CIEUC
    ((0xA9, 0x69), (0xCBEB)),  # HANGUL SYLLABLE SSANGCIEUC EU CHIEUCH
    ((0xA9, 0x6A), (0xCBEC)),  # HANGUL SYLLABLE SSANGCIEUC EU KHIEUKH
    ((0xA9, 0x6B), (0xCBED)),  # HANGUL SYLLABLE SSANGCIEUC EU THIEUTH
    ((0xA9, 0x6C), (0xCBEE)),  # HANGUL SYLLABLE SSANGCIEUC EU PHIEUPH
    ((0xA9, 0x6D), (0xCBEF)),  # HANGUL SYLLABLE SSANGCIEUC EU HIEUH
    ((0xA9, 0x6E), (0xCBF0)),  # HANGUL SYLLABLE SSANGCIEUC YI
    ((0xA9, 0x6F), (0xCBF1)),  # HANGUL SYLLABLE SSANGCIEUC YI KIYEOK
    ((0xA9, 0x70), (0xCBF2)),  # HANGUL SYLLABLE SSANGCIEUC YI SSANGKIYEOK
    ((0xA9, 0x71), (0xCBF3)),  # HANGUL SYLLABLE SSANGCIEUC YI KIYEOKSIOS
    ((0xA9, 0x72), (0xCBF4)),  # HANGUL SYLLABLE SSANGCIEUC YI NIEUN
    ((0xA9, 0x73), (0xCBF5)),  # HANGUL SYLLABLE SSANGCIEUC YI NIEUNCIEUC
    ((0xA9, 0x74), (0xCBF6)),  # HANGUL SYLLABLE SSANGCIEUC YI NIEUNHIEUH
    ((0xA9, 0x75), (0xCBF7)),  # HANGUL SYLLABLE SSANGCIEUC YI TIKEUT
    ((0xA9, 0x76), (0xCBF8)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEUL
    ((0xA9, 0x77), (0xCBF9)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULKIYEOK
    ((0xA9, 0x78), (0xCBFA)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULMIEUM
    ((0xA9, 0x79), (0xCBFB)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULPIEUP
    ((0xA9, 0x7A), (0xCBFC)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULSIOS
    ((0xA9, 0x81), (0xCBFD)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULTHIEUTH
    ((0xA9, 0x82), (0xCBFE)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULPHIEUPH
    ((0xA9, 0x83), (0xCBFF)),  # HANGUL SYLLABLE SSANGCIEUC YI RIEULHIEUH
    ((0xA9, 0x84), (0xCC00)),  # HANGUL SYLLABLE SSANGCIEUC YI MIEUM
    ((0xA9, 0x85), (0xCC01)),  # HANGUL SYLLABLE SSANGCIEUC YI PIEUP
    ((0xA9, 0x86), (0xCC02)),  # HANGUL SYLLABLE SSANGCIEUC YI PIEUPSIOS
    ((0xA9, 0x87), (0xCC03)),  # HANGUL SYLLABLE SSANGCIEUC YI SIOS
    ((0xA9, 0x88), (0xCC04)),  # HANGUL SYLLABLE SSANGCIEUC YI SSANGSIOS
    ((0xA9, 0x89), (0xCC05)),  # HANGUL SYLLABLE SSANGCIEUC YI IEUNG
    ((0xA9, 0x8A), (0xCC06)),  # HANGUL SYLLABLE SSANGCIEUC YI CIEUC
    ((0xA9, 0x8B), (0xCC07)),  # HANGUL SYLLABLE SSANGCIEUC YI CHIEUCH
    ((0xA9, 0x8C), (0xCC08)),  # HANGUL SYLLABLE SSANGCIEUC YI KHIEUKH
    ((0xA9, 0x8D), (0xCC09)),  # HANGUL SYLLABLE SSANGCIEUC YI THIEUTH
    ((0xA9, 0x8E), (0xCC0A)),  # HANGUL SYLLABLE SSANGCIEUC YI PHIEUPH
    ((0xA9, 0x8F), (0xCC0B)),  # HANGUL SYLLABLE SSANGCIEUC YI HIEUH
    ((0xA9, 0x90), (0xCC0E)),  # HANGUL SYLLABLE SSANGCIEUC I SSANGKIYEOK
    ((0xA9, 0x91), (0xCC0F)),  # HANGUL SYLLABLE SSANGCIEUC I KIYEOKSIOS
    ((0xA9, 0x92), (0xCC11)),  # HANGUL SYLLABLE SSANGCIEUC I NIEUNCIEUC
    ((0xA9, 0x93), (0xCC12)),  # HANGUL SYLLABLE SSANGCIEUC I NIEUNHIEUH
    ((0xA9, 0x94), (0xCC13)),  # HANGUL SYLLABLE SSANGCIEUC I TIKEUT
    ((0xA9, 0x95), (0xCC15)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULKIYEOK
    ((0xA9, 0x96), (0xCC16)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULMIEUM
    ((0xA9, 0x97), (0xCC17)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULPIEUP
    ((0xA9, 0x98), (0xCC18)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULSIOS
    ((0xA9, 0x99), (0xCC19)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULTHIEUTH
    ((0xA9, 0x9A), (0xCC1A)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULPHIEUPH
    ((0xA9, 0x9B), (0xCC1B)),  # HANGUL SYLLABLE SSANGCIEUC I RIEULHIEUH
    ((0xA9, 0x9C), (0xCC1E)),  # HANGUL SYLLABLE SSANGCIEUC I PIEUPSIOS
    ((0xA9, 0x9D), (0xCC1F)),  # HANGUL SYLLABLE SSANGCIEUC I SIOS
    ((0xA9, 0x9E), (0xCC20)),  # HANGUL SYLLABLE SSANGCIEUC I SSANGSIOS
    ((0xA9, 0x9F), (0xCC23)),  # HANGUL SYLLABLE SSANGCIEUC I CHIEUCH
    ((0xA9, 0xA0), (0xCC24)),  # HANGUL SYLLABLE SSANGCIEUC I KHIEUKH
    ((0xA9, 0xA1), (0x00E6)),  # LATIN SMALL LETTER AE
    ((0xA9, 0xA2), (0x0111)),  # LATIN SMALL LETTER D WITH STROKE
    ((0xA9, 0xA3), (0x00F0)),  # LATIN SMALL LETTER ETH
    ((0xA9, 0xA4), (0x0127)),  # LATIN SMALL LETTER H WITH STROKE
    ((0xA9, 0xA5), (0x0131)),  # LATIN SMALL LETTER DOTLESS I
    ((0xA9, 0xA6), (0x0133)),  # LATIN SMALL LIGATURE IJ
    ((0xA9, 0xA7), (0x0138)),  # LATIN SMALL LETTER KRA
    ((0xA9, 0xA8), (0x0140)),  # LATIN SMALL LETTER L WITH MIDDLE DOT
    ((0xA9, 0xA9), (0x0142)),  # LATIN SMALL LETTER L WITH STROKE
    ((0xA9, 0xAA), (0x00F8)),  # LATIN SMALL LETTER O WITH STROKE
    ((0xA9, 0xAB), (0x0153)),  # LATIN SMALL LIGATURE OE
    ((0xA9, 0xAC), (0x00DF)),  # LATIN SMALL LETTER SHARP S
    ((0xA9, 0xAD), (0x00FE)),  # LATIN SMALL LETTER THORN
    ((0xA9, 0xAE), (0x0167)),  # LATIN SMALL LETTER T WITH STROKE
    ((0xA9, 0xAF), (0x014B)),  # LATIN SMALL LETTER ENG
    ((0xA9, 0xB0), (0x0149)),  # LATIN SMALL LETTER N PRECEDED BY APOSTROPHE
    ((0xA9, 0xB1), (0x3200)),  # PARENTHESIZED HANGUL KIYEOK
    ((0xA9, 0xB2), (0x3201)),  # PARENTHESIZED HANGUL NIEUN
    ((0xA9, 0xB3), (0x3202)),  # PARENTHESIZED HANGUL TIKEUT
    ((0xA9, 0xB4), (0x3203)),  # PARENTHESIZED HANGUL RIEUL
    ((0xA9, 0xB5), (0x3204)),  # PARENTHESIZED HANGUL MIEUM
    ((0xA9, 0xB6), (0x3205)),  # PARENTHESIZED HANGUL PIEUP
    ((0xA9, 0xB7), (0x3206)),  # PARENTHESIZED HANGUL SIOS
    ((0xA9, 0xB8), (0x3207)),  # PARENTHESIZED HANGUL IEUNG
    ((0xA9, 0xB9), (0x3208)),  # PARENTHESIZED HANGUL CIEUC
    ((0xA9, 0xBA), (0x3209)),  # PARENTHESIZED HANGUL CHIEUCH
    ((0xA9, 0xBB), (0x320A)),  # PARENTHESIZED HANGUL KHIEUKH
    ((0xA9, 0xBC), (0x320B)),  # PARENTHESIZED HANGUL THIEUTH
    ((0xA9, 0xBD), (0x320C)),  # PARENTHESIZED HANGUL PHIEUPH
    ((0xA9, 0xBE), (0x320D)),  # PARENTHESIZED HANGUL HIEUH
    ((0xA9, 0xBF), (0x320E)),  # PARENTHESIZED HANGUL KIYEOK A
    ((0xA9, 0xC0), (0x320F)),  # PARENTHESIZED HANGUL NIEUN A
    ((0xA9, 0xC1), (0x3210)),  # PARENTHESIZED HANGUL TIKEUT A
    ((0xA9, 0xC2), (0x3211)),  # PARENTHESIZED HANGUL RIEUL A
    ((0xA9, 0xC3), (0x3212)),  # PARENTHESIZED HANGUL MIEUM A
    ((0xA9, 0xC4), (0x3213)),  # PARENTHESIZED HANGUL PIEUP A
    ((0xA9, 0xC5), (0x3214)),  # PARENTHESIZED HANGUL SIOS A
    ((0xA9, 0xC6), (0x3215)),  # PARENTHESIZED HANGUL IEUNG A
    ((0xA9, 0xC7), (0x3216)),  # PARENTHESIZED HANGUL CIEUC A
    ((0xA9, 0xC8), (0x3217)),  # PARENTHESIZED HANGUL CHIEUCH A
    ((0xA9, 0xC9), (0x3218)),  # PARENTHESIZED HANGUL KHIEUKH A
    ((0xA9, 0xCA), (0x3219)),  # PARENTHESIZED HANGUL THIEUTH A
    ((0xA9, 0xCB), (0x321A)),  # PARENTHESIZED HANGUL PHIEUPH A
    ((0xA9, 0xCC), (0x321B)),  # PARENTHESIZED HANGUL HIEUH A
    ((0xA9, 0xCD), (0x249C)),  # PARENTHESIZED LATIN SMALL LETTER A
    ((0xA9, 0xCE), (0x249D)),  # PARENTHESIZED LATIN SMALL LETTER B
    ((0xA9, 0xCF), (0x249E)),  # PARENTHESIZED LATIN SMALL LETTER C
    ((0xA9, 0xD0), (0x249F)),  # PARENTHESIZED LATIN SMALL LETTER D
    ((0xA9, 0xD1), (0x24A0)),  # PARENTHESIZED LATIN SMALL LETTER E
    ((0xA9, 0xD2), (0x24A1)),  # PARENTHESIZED LATIN SMALL LETTER F
    ((0xA9, 0xD3), (0x24A2)),  # PARENTHESIZED LATIN SMALL LETTER G
    ((0xA9, 0xD4), (0x24A3)),  # PARENTHESIZED LATIN SMALL LETTER H
    ((0xA9, 0xD5), (0x24A4)),  # PARENTHESIZED LATIN SMALL LETTER I
    ((0xA9, 0xD6), (0x24A5)),  # PARENTHESIZED LATIN SMALL LETTER J
    ((0xA9, 0xD7), (0x24A6)),  # PARENTHESIZED LATIN SMALL LETTER K
    ((0xA9, 0xD8), (0x24A7)),  # PARENTHESIZED LATIN SMALL LETTER L
    ((0xA9, 0xD9), (0x24A8)),  # PARENTHESIZED LATIN SMALL LETTER M
    ((0xA9, 0xDA), (0x24A9)),  # PARENTHESIZED LATIN SMALL LETTER N
    ((0xA9, 0xDB), (0x24AA)),  # PARENTHESIZED LATIN SMALL LETTER O
    ((0xA9, 0xDC), (0x24AB)),  # PARENTHESIZED LATIN SMALL LETTER P
    ((0xA9, 0xDD), (0x24AC)),  # PARENTHESIZED LATIN SMALL LETTER Q
    ((0xA9, 0xDE), (0x24AD)),  # PARENTHESIZED LATIN SMALL LETTER R
    ((0xA9, 0xDF), (0x24AE)),  # PARENTHESIZED LATIN SMALL LETTER S
    ((0xA9, 0xE0), (0x24AF)),  # PARENTHESIZED LATIN SMALL LETTER T
    ((0xA9, 0xE1), (0x24B0)),  # PARENTHESIZED LATIN SMALL LETTER U
    ((0xA9, 0xE2), (0x24B1)),  # PARENTHESIZED LATIN SMALL LETTER V
    ((0xA9, 0xE3), (0x24B2)),  # PARENTHESIZED LATIN SMALL LETTER W
    ((0xA9, 0xE4), (0x24B3)),  # PARENTHESIZED LATIN SMALL LETTER X
    ((0xA9, 0xE5), (0x24B4)),  # PARENTHESIZED LATIN SMALL LETTER Y
    ((0xA9, 0xE6), (0x24B5)),  # PARENTHESIZED LATIN SMALL LETTER Z
    ((0xA9, 0xE7), (0x2474)),  # PARENTHESIZED DIGIT ONE
    ((0xA9, 0xE8), (0x2475)),  # PARENTHESIZED DIGIT TWO
    ((0xA9, 0xE9), (0x2476)),  # PARENTHESIZED DIGIT THREE
    ((0xA9, 0xEA), (0x2477)),  # PARENTHESIZED DIGIT FOUR
    ((0xA9, 0xEB), (0x2478)),  # PARENTHESIZED DIGIT FIVE
    ((0xA9, 0xEC), (0x2479)),  # PARENTHESIZED DIGIT SIX
    ((0xA9, 0xED), (0x247A)),  # PARENTHESIZED DIGIT SEVEN
    ((0xA9, 0xEE), (0x247B)),  # PARENTHESIZED DIGIT EIGHT
    ((0xA9, 0xEF), (0x247C)),  # PARENTHESIZED DIGIT NINE
    ((0xA9, 0xF0), (0x247D)),  # PARENTHESIZED NUMBER TEN
    ((0xA9, 0xF1), (0x247E)),  # PARENTHESIZED NUMBER ELEVEN
    ((0xA9, 0xF2), (0x247F)),  # PARENTHESIZED NUMBER TWELVE
    ((0xA9, 0xF3), (0x2480)),  # PARENTHESIZED NUMBER THIRTEEN
    ((0xA9, 0xF4), (0x2481)),  # PARENTHESIZED NUMBER FOURTEEN
    ((0xA9, 0xF5), (0x2482)),  # PARENTHESIZED NUMBER FIFTEEN
    ((0xA9, 0xF6), (0x00B9)),  # SUPERSCRIPT ONE
    ((0xA9, 0xF7), (0x00B2)),  # SUPERSCRIPT TWO
    ((0xA9, 0xF8), (0x00B3)),  # SUPERSCRIPT THREE
    ((0xA9, 0xF9), (0x2074)),  # SUPERSCRIPT FOUR
    ((0xA9, 0xFA), (0x207F)),  # SUPERSCRIPT LATIN SMALL LETTER N
    ((0xA9, 0xFB), (0x2081)),  # SUBSCRIPT ONE
    ((0xA9, 0xFC), (0x2082)),  # SUBSCRIPT TWO
    ((0xA9, 0xFD), (0x2083)),  # SUBSCRIPT THREE
    ((0xA9, 0xFE), (0x2084)),  # SUBSCRIPT FOUR
    ((0xAA, 0x41), (0xCC25)),  # HANGUL SYLLABLE SSANGCIEUC I THIEUTH
    ((0xAA, 0x42), (0xCC26)),  # HANGUL SYLLABLE SSANGCIEUC I PHIEUPH
    ((0xAA, 0x43), (0xCC2A)),  # HANGUL SYLLABLE CHIEUCH A SSANGKIYEOK
    ((0xAA, 0x44), (0xCC2B)),  # HANGUL SYLLABLE CHIEUCH A KIYEOKSIOS
    ((0xAA, 0x45), (0xCC2D)),  # HANGUL SYLLABLE CHIEUCH A NIEUNCIEUC
    ((0xAA, 0x46), (0xCC2F)),  # HANGUL SYLLABLE CHIEUCH A TIKEUT
    ((0xAA, 0x47), (0xCC31)),  # HANGUL SYLLABLE CHIEUCH A RIEULKIYEOK
    ((0xAA, 0x48), (0xCC32)),  # HANGUL SYLLABLE CHIEUCH A RIEULMIEUM
    ((0xAA, 0x49), (0xCC33)),  # HANGUL SYLLABLE CHIEUCH A RIEULPIEUP
    ((0xAA, 0x4A), (0xCC34)),  # HANGUL SYLLABLE CHIEUCH A RIEULSIOS
    ((0xAA, 0x4B), (0xCC35)),  # HANGUL SYLLABLE CHIEUCH A RIEULTHIEUTH
    ((0xAA, 0x4C), (0xCC36)),  # HANGUL SYLLABLE CHIEUCH A RIEULPHIEUPH
    ((0xAA, 0x4D), (0xCC37)),  # HANGUL SYLLABLE CHIEUCH A RIEULHIEUH
    ((0xAA, 0x4E), (0xCC3A)),  # HANGUL SYLLABLE CHIEUCH A PIEUPSIOS
    ((0xAA, 0x4F), (0xCC3F)),  # HANGUL SYLLABLE CHIEUCH A CHIEUCH
    ((0xAA, 0x50), (0xCC40)),  # HANGUL SYLLABLE CHIEUCH A KHIEUKH
    ((0xAA, 0x51), (0xCC41)),  # HANGUL SYLLABLE CHIEUCH A THIEUTH
    ((0xAA, 0x52), (0xCC42)),  # HANGUL SYLLABLE CHIEUCH A PHIEUPH
    ((0xAA, 0x53), (0xCC43)),  # HANGUL SYLLABLE CHIEUCH A HIEUH
    ((0xAA, 0x54), (0xCC46)),  # HANGUL SYLLABLE CHIEUCH AE SSANGKIYEOK
    ((0xAA, 0x55), (0xCC47)),  # HANGUL SYLLABLE CHIEUCH AE KIYEOKSIOS
    ((0xAA, 0x56), (0xCC49)),  # HANGUL SYLLABLE CHIEUCH AE NIEUNCIEUC
    ((0xAA, 0x57), (0xCC4A)),  # HANGUL SYLLABLE CHIEUCH AE NIEUNHIEUH
    ((0xAA, 0x58), (0xCC4B)),  # HANGUL SYLLABLE CHIEUCH AE TIKEUT
    ((0xAA, 0x59), (0xCC4D)),  # HANGUL SYLLABLE CHIEUCH AE RIEULKIYEOK
    ((0xAA, 0x5A), (0xCC4E)),  # HANGUL SYLLABLE CHIEUCH AE RIEULMIEUM
    ((0xAA, 0x61), (0xCC4F)),  # HANGUL SYLLABLE CHIEUCH AE RIEULPIEUP
    ((0xAA, 0x62), (0xCC50)),  # HANGUL SYLLABLE CHIEUCH AE RIEULSIOS
    ((0xAA, 0x63), (0xCC51)),  # HANGUL SYLLABLE CHIEUCH AE RIEULTHIEUTH
    ((0xAA, 0x64), (0xCC52)),  # HANGUL SYLLABLE CHIEUCH AE RIEULPHIEUPH
    ((0xAA, 0x65), (0xCC53)),  # HANGUL SYLLABLE CHIEUCH AE RIEULHIEUH
    ((0xAA, 0x66), (0xCC56)),  # HANGUL SYLLABLE CHIEUCH AE PIEUPSIOS
    ((0xAA, 0x67), (0xCC5A)),  # HANGUL SYLLABLE CHIEUCH AE CIEUC
    ((0xAA, 0x68), (0xCC5B)),  # HANGUL SYLLABLE CHIEUCH AE CHIEUCH
    ((0xAA, 0x69), (0xCC5C)),  # HANGUL SYLLABLE CHIEUCH AE KHIEUKH
    ((0xAA, 0x6A), (0xCC5D)),  # HANGUL SYLLABLE CHIEUCH AE THIEUTH
    ((0xAA, 0x6B), (0xCC5E)),  # HANGUL SYLLABLE CHIEUCH AE PHIEUPH
    ((0xAA, 0x6C), (0xCC5F)),  # HANGUL SYLLABLE CHIEUCH AE HIEUH
    ((0xAA, 0x6D), (0xCC61)),  # HANGUL SYLLABLE CHIEUCH YA KIYEOK
    ((0xAA, 0x6E), (0xCC62)),  # HANGUL SYLLABLE CHIEUCH YA SSANGKIYEOK
    ((0xAA, 0x6F), (0xCC63)),  # HANGUL SYLLABLE CHIEUCH YA KIYEOKSIOS
    ((0xAA, 0x70), (0xCC65)),  # HANGUL SYLLABLE CHIEUCH YA NIEUNCIEUC
    ((0xAA, 0x71), (0xCC67)),  # HANGUL SYLLABLE CHIEUCH YA TIKEUT
    ((0xAA, 0x72), (0xCC69)),  # HANGUL SYLLABLE CHIEUCH YA RIEULKIYEOK
    ((0xAA, 0x73), (0xCC6A)),  # HANGUL SYLLABLE CHIEUCH YA RIEULMIEUM
    ((0xAA, 0x74), (0xCC6B)),  # HANGUL SYLLABLE CHIEUCH YA RIEULPIEUP
    ((0xAA, 0x75), (0xCC6C)),  # HANGUL SYLLABLE CHIEUCH YA RIEULSIOS
    ((0xAA, 0x76), (0xCC6D)),  # HANGUL SYLLABLE CHIEUCH YA RIEULTHIEUTH
    ((0xAA, 0x77), (0xCC6E)),  # HANGUL SYLLABLE CHIEUCH YA RIEULPHIEUPH
    ((0xAA, 0x78), (0xCC6F)),  # HANGUL SYLLABLE CHIEUCH YA RIEULHIEUH
    ((0xAA, 0x79), (0xCC71)),  # HANGUL SYLLABLE CHIEUCH YA PIEUP
    ((0xAA, 0x7A), (0xCC72)),  # HANGUL SYLLABLE CHIEUCH YA PIEUPSIOS
    ((0xAA, 0x81), (0xCC73)),  # HANGUL SYLLABLE CHIEUCH YA SIOS
    ((0xAA, 0x82), (0xCC74)),  # HANGUL SYLLABLE CHIEUCH YA SSANGSIOS
    ((0xAA, 0x83), (0xCC76)),  # HANGUL SYLLABLE CHIEUCH YA CIEUC
    ((0xAA, 0x84), (0xCC77)),  # HANGUL SYLLABLE CHIEUCH YA CHIEUCH
    ((0xAA, 0x85), (0xCC78)),  # HANGUL SYLLABLE CHIEUCH YA KHIEUKH
    ((0xAA, 0x86), (0xCC79)),  # HANGUL SYLLABLE CHIEUCH YA THIEUTH
    ((0xAA, 0x87), (0xCC7A)),  # HANGUL SYLLABLE CHIEUCH YA PHIEUPH
    ((0xAA, 0x88), (0xCC7B)),  # HANGUL SYLLABLE CHIEUCH YA HIEUH
    ((0xAA, 0x89), (0xCC7C)),  # HANGUL SYLLABLE CHIEUCH YAE
    ((0xAA, 0x8A), (0xCC7D)),  # HANGUL SYLLABLE CHIEUCH YAE KIYEOK
    ((0xAA, 0x8B), (0xCC7E)),  # HANGUL SYLLABLE CHIEUCH YAE SSANGKIYEOK
    ((0xAA, 0x8C), (0xCC7F)),  # HANGUL SYLLABLE CHIEUCH YAE KIYEOKSIOS
    ((0xAA, 0x8D), (0xCC80)),  # HANGUL SYLLABLE CHIEUCH YAE NIEUN
    ((0xAA, 0x8E), (0xCC81)),  # HANGUL SYLLABLE CHIEUCH YAE NIEUNCIEUC
    ((0xAA, 0x8F), (0xCC82)),  # HANGUL SYLLABLE CHIEUCH YAE NIEUNHIEUH
    ((0xAA, 0x90), (0xCC83)),  # HANGUL SYLLABLE CHIEUCH YAE TIKEUT
    ((0xAA, 0x91), (0xCC84)),  # HANGUL SYLLABLE CHIEUCH YAE RIEUL
    ((0xAA, 0x92), (0xCC85)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULKIYEOK
    ((0xAA, 0x93), (0xCC86)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULMIEUM
    ((0xAA, 0x94), (0xCC87)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULPIEUP
    ((0xAA, 0x95), (0xCC88)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULSIOS
    ((0xAA, 0x96), (0xCC89)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULTHIEUTH
    ((0xAA, 0x97), (0xCC8A)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULPHIEUPH
    ((0xAA, 0x98), (0xCC8B)),  # HANGUL SYLLABLE CHIEUCH YAE RIEULHIEUH
    ((0xAA, 0x99), (0xCC8C)),  # HANGUL SYLLABLE CHIEUCH YAE MIEUM
    ((0xAA, 0x9A), (0xCC8D)),  # HANGUL SYLLABLE CHIEUCH YAE PIEUP
    ((0xAA, 0x9B), (0xCC8E)),  # HANGUL SYLLABLE CHIEUCH YAE PIEUPSIOS
    ((0xAA, 0x9C), (0xCC8F)),  # HANGUL SYLLABLE CHIEUCH YAE SIOS
    ((0xAA, 0x9D), (0xCC90)),  # HANGUL SYLLABLE CHIEUCH YAE SSANGSIOS
    ((0xAA, 0x9E), (0xCC91)),  # HANGUL SYLLABLE CHIEUCH YAE IEUNG
    ((0xAA, 0x9F), (0xCC92)),  # HANGUL SYLLABLE CHIEUCH YAE CIEUC
    ((0xAA, 0xA0), (0xCC93)),  # HANGUL SYLLABLE CHIEUCH YAE CHIEUCH
    ((0xAA, 0xA1), (0x3041)),  # HIRAGANA LETTER SMALL A
    ((0xAA, 0xA2), (0x3042)),  # HIRAGANA LETTER A
    ((0xAA, 0xA3), (0x3043)),  # HIRAGANA LETTER SMALL I
    ((0xAA, 0xA4), (0x3044)),  # HIRAGANA LETTER I
    ((0xAA, 0xA5), (0x3045)),  # HIRAGANA LETTER SMALL U
    ((0xAA, 0xA6), (0x3046)),  # HIRAGANA LETTER U
    ((0xAA, 0xA7), (0x3047)),  # HIRAGANA LETTER SMALL E
    ((0xAA, 0xA8), (0x3048)),  # HIRAGANA LETTER E
    ((0xAA, 0xA9), (0x3049)),  # HIRAGANA LETTER SMALL O
    ((0xAA, 0xAA), (0x304A)),  # HIRAGANA LETTER O
    ((0xAA, 0xAB), (0x304B)),  # HIRAGANA LETTER KA
    ((0xAA, 0xAC), (0x304C)),  # HIRAGANA LETTER GA
    ((0xAA, 0xAD), (0x304D)),  # HIRAGANA LETTER KI
    ((0xAA, 0xAE), (0x304E)),  # HIRAGANA LETTER GI
    ((0xAA, 0xAF), (0x304F)),  # HIRAGANA LETTER KU
    ((0xAA, 0xB0), (0x3050)),  # HIRAGANA LETTER GU
    ((0xAA, 0xB1), (0x3051)),  # HIRAGANA LETTER KE
    ((0xAA, 0xB2), (0x3052)),  # HIRAGANA LETTER GE
    ((0xAA, 0xB3), (0x3053)),  # HIRAGANA LETTER KO
    ((0xAA, 0xB4), (0x3054)),  # HIRAGANA LETTER GO
    ((0xAA, 0xB5), (0x3055)),  # HIRAGANA LETTER SA
    ((0xAA, 0xB6), (0x3056)),  # HIRAGANA LETTER ZA
    ((0xAA, 0xB7), (0x3057)),  # HIRAGANA LETTER SI
    ((0xAA, 0xB8), (0x3058)),  # HIRAGANA LETTER ZI
    ((0xAA, 0xB9), (0x3059)),  # HIRAGANA LETTER SU
    ((0xAA, 0xBA), (0x305A)),  # HIRAGANA LETTER ZU
    ((0xAA, 0xBB), (0x305B)),  # HIRAGANA LETTER SE
    ((0xAA, 0xBC), (0x305C)),  # HIRAGANA LETTER ZE
    ((0xAA, 0xBD), (0x305D)),  # HIRAGANA LETTER SO
    ((0xAA, 0xBE), (0x305E)),  # HIRAGANA LETTER ZO
    ((0xAA, 0xBF), (0x305F)),  # HIRAGANA LETTER TA
    ((0xAA, 0xC0), (0x3060)),  # HIRAGANA LETTER DA
    ((0xAA, 0xC1), (0x3061)),  # HIRAGANA LETTER TI
    ((0xAA, 0xC2), (0x3062)),  # HIRAGANA LETTER DI
    ((0xAA, 0xC3), (0x3063)),  # HIRAGANA LETTER SMALL TU
    ((0xAA, 0xC4), (0x3064)),  # HIRAGANA LETTER TU
    ((0xAA, 0xC5), (0x3065)),  # HIRAGANA LETTER DU
    ((0xAA, 0xC6), (0x3066)),  # HIRAGANA LETTER TE
    ((0xAA, 0xC7), (0x3067)),  # HIRAGANA LETTER DE
    ((0xAA, 0xC8), (0x3068)),  # HIRAGANA LETTER TO
    ((0xAA, 0xC9), (0x3069)),  # HIRAGANA LETTER DO
    ((0xAA, 0xCA), (0x306A)),  # HIRAGANA LETTER NA
    ((0xAA, 0xCB), (0x306B)),  # HIRAGANA LETTER NI
    ((0xAA, 0xCC), (0x306C)),  # HIRAGANA LETTER NU
    ((0xAA, 0xCD), (0x306D)),  # HIRAGANA LETTER NE
    ((0xAA, 0xCE), (0x306E)),  # HIRAGANA LETTER NO
    ((0xAA, 0xCF), (0x306F)),  # HIRAGANA LETTER HA
    ((0xAA, 0xD0), (0x3070)),  # HIRAGANA LETTER BA
    ((0xAA, 0xD1), (0x3071)),  # HIRAGANA LETTER PA
    ((0xAA, 0xD2), (0x3072)),  # HIRAGANA LETTER HI
    ((0xAA, 0xD3), (0x3073)),  # HIRAGANA LETTER BI
    ((0xAA, 0xD4), (0x3074)),  # HIRAGANA LETTER PI
    ((0xAA, 0xD5), (0x3075)),  # HIRAGANA LETTER HU
    ((0xAA, 0xD6), (0x3076)),  # HIRAGANA LETTER BU
    ((0xAA, 0xD7), (0x3077)),  # HIRAGANA LETTER PU
    ((0xAA, 0xD8), (0x3078)),  # HIRAGANA LETTER HE
    ((0xAA, 0xD9), (0x3079)),  # HIRAGANA LETTER BE
    ((0xAA, 0xDA), (0x307A)),  # HIRAGANA LETTER PE
    ((0xAA, 0xDB), (0x307B)),  # HIRAGANA LETTER HO
    ((0xAA, 0xDC), (0x307C)),  # HIRAGANA LETTER BO
    ((0xAA, 0xDD), (0x307D)),  # HIRAGANA LETTER PO
    ((0xAA, 0xDE), (0x307E)),  # HIRAGANA LETTER MA
    ((0xAA, 0xDF), (0x307F)),  # HIRAGANA LETTER MI
    ((0xAA, 0xE0), (0x3080)),  # HIRAGANA LETTER MU
    ((0xAA, 0xE1), (0x3081)),  # HIRAGANA LETTER ME
    ((0xAA, 0xE2), (0x3082)),  # HIRAGANA LETTER MO
    ((0xAA, 0xE3), (0x3083)),  # HIRAGANA LETTER SMALL YA
    ((0xAA, 0xE4), (0x3084)),  # HIRAGANA LETTER YA
    ((0xAA, 0xE5), (0x3085)),  # HIRAGANA LETTER SMALL YU
    ((0xAA, 0xE6), (0x3086)),  # HIRAGANA LETTER YU
    ((0xAA, 0xE7), (0x3087)),  # HIRAGANA LETTER SMALL YO
    ((0xAA, 0xE8), (0x3088)),  # HIRAGANA LETTER YO
    ((0xAA, 0xE9), (0x3089)),  # HIRAGANA LETTER RA
    ((0xAA, 0xEA), (0x308A)),  # HIRAGANA LETTER RI
    ((0xAA, 0xEB), (0x308B)),  # HIRAGANA LETTER RU
    ((0xAA, 0xEC), (0x308C)),  # HIRAGANA LETTER RE
    ((0xAA, 0xED), (0x308D)),  # HIRAGANA LETTER RO
    ((0xAA, 0xEE), (0x308E)),  # HIRAGANA LETTER SMALL WA
    ((0xAA, 0xEF), (0x308F)),  # HIRAGANA LETTER WA
    ((0xAA, 0xF0), (0x3090)),  # HIRAGANA LETTER WI
    ((0xAA, 0xF1), (0x3091)),  # HIRAGANA LETTER WE
    ((0xAA, 0xF2), (0x3092)),  # HIRAGANA LETTER WO
    ((0xAA, 0xF3), (0x3093)),  # HIRAGANA LETTER N
    ((0xAB, 0x41), (0xCC94)),  # HANGUL SYLLABLE CHIEUCH YAE KHIEUKH
    ((0xAB, 0x42), (0xCC95)),  # HANGUL SYLLABLE CHIEUCH YAE THIEUTH
    ((0xAB, 0x43), (0xCC96)),  # HANGUL SYLLABLE CHIEUCH YAE PHIEUPH
    ((0xAB, 0x44), (0xCC97)),  # HANGUL SYLLABLE CHIEUCH YAE HIEUH
    ((0xAB, 0x45), (0xCC9A)),  # HANGUL SYLLABLE CHIEUCH EO SSANGKIYEOK
    ((0xAB, 0x46), (0xCC9B)),  # HANGUL SYLLABLE CHIEUCH EO KIYEOKSIOS
    ((0xAB, 0x47), (0xCC9D)),  # HANGUL SYLLABLE CHIEUCH EO NIEUNCIEUC
    ((0xAB, 0x48), (0xCC9E)),  # HANGUL SYLLABLE CHIEUCH EO NIEUNHIEUH
    ((0xAB, 0x49), (0xCC9F)),  # HANGUL SYLLABLE CHIEUCH EO TIKEUT
    ((0xAB, 0x4A), (0xCCA1)),  # HANGUL SYLLABLE CHIEUCH EO RIEULKIYEOK
    ((0xAB, 0x4B), (0xCCA2)),  # HANGUL SYLLABLE CHIEUCH EO RIEULMIEUM
    ((0xAB, 0x4C), (0xCCA3)),  # HANGUL SYLLABLE CHIEUCH EO RIEULPIEUP
    ((0xAB, 0x4D), (0xCCA4)),  # HANGUL SYLLABLE CHIEUCH EO RIEULSIOS
    ((0xAB, 0x4E), (0xCCA5)),  # HANGUL SYLLABLE CHIEUCH EO RIEULTHIEUTH
    ((0xAB, 0x4F), (0xCCA6)),  # HANGUL SYLLABLE CHIEUCH EO RIEULPHIEUPH
    ((0xAB, 0x50), (0xCCA7)),  # HANGUL SYLLABLE CHIEUCH EO RIEULHIEUH
    ((0xAB, 0x51), (0xCCAA)),  # HANGUL SYLLABLE CHIEUCH EO PIEUPSIOS
    ((0xAB, 0x52), (0xCCAE)),  # HANGUL SYLLABLE CHIEUCH EO CIEUC
    ((0xAB, 0x53), (0xCCAF)),  # HANGUL SYLLABLE CHIEUCH EO CHIEUCH
    ((0xAB, 0x54), (0xCCB0)),  # HANGUL SYLLABLE CHIEUCH EO KHIEUKH
    ((0xAB, 0x55), (0xCCB1)),  # HANGUL SYLLABLE CHIEUCH EO THIEUTH
    ((0xAB, 0x56), (0xCCB2)),  # HANGUL SYLLABLE CHIEUCH EO PHIEUPH
    ((0xAB, 0x57), (0xCCB3)),  # HANGUL SYLLABLE CHIEUCH EO HIEUH
    ((0xAB, 0x58), (0xCCB6)),  # HANGUL SYLLABLE CHIEUCH E SSANGKIYEOK
    ((0xAB, 0x59), (0xCCB7)),  # HANGUL SYLLABLE CHIEUCH E KIYEOKSIOS
    ((0xAB, 0x5A), (0xCCB9)),  # HANGUL SYLLABLE CHIEUCH E NIEUNCIEUC
    ((0xAB, 0x61), (0xCCBA)),  # HANGUL SYLLABLE CHIEUCH E NIEUNHIEUH
    ((0xAB, 0x62), (0xCCBB)),  # HANGUL SYLLABLE CHIEUCH E TIKEUT
    ((0xAB, 0x63), (0xCCBD)),  # HANGUL SYLLABLE CHIEUCH E RIEULKIYEOK
    ((0xAB, 0x64), (0xCCBE)),  # HANGUL SYLLABLE CHIEUCH E RIEULMIEUM
    ((0xAB, 0x65), (0xCCBF)),  # HANGUL SYLLABLE CHIEUCH E RIEULPIEUP
    ((0xAB, 0x66), (0xCCC0)),  # HANGUL SYLLABLE CHIEUCH E RIEULSIOS
    ((0xAB, 0x67), (0xCCC1)),  # HANGUL SYLLABLE CHIEUCH E RIEULTHIEUTH
    ((0xAB, 0x68), (0xCCC2)),  # HANGUL SYLLABLE CHIEUCH E RIEULPHIEUPH
    ((0xAB, 0x69), (0xCCC3)),  # HANGUL SYLLABLE CHIEUCH E RIEULHIEUH
    ((0xAB, 0x6A), (0xCCC6)),  # HANGUL SYLLABLE CHIEUCH E PIEUPSIOS
    ((0xAB, 0x6B), (0xCCC8)),  # HANGUL SYLLABLE CHIEUCH E SSANGSIOS
    ((0xAB, 0x6C), (0xCCCA)),  # HANGUL SYLLABLE CHIEUCH E CIEUC
    ((0xAB, 0x6D), (0xCCCB)),  # HANGUL SYLLABLE CHIEUCH E CHIEUCH
    ((0xAB, 0x6E), (0xCCCC)),  # HANGUL SYLLABLE CHIEUCH E KHIEUKH
    ((0xAB, 0x6F), (0xCCCD)),  # HANGUL SYLLABLE CHIEUCH E THIEUTH
    ((0xAB, 0x70), (0xCCCE)),  # HANGUL SYLLABLE CHIEUCH E PHIEUPH
    ((0xAB, 0x71), (0xCCCF)),  # HANGUL SYLLABLE CHIEUCH E HIEUH
    ((0xAB, 0x72), (0xCCD1)),  # HANGUL SYLLABLE CHIEUCH YEO KIYEOK
    ((0xAB, 0x73), (0xCCD2)),  # HANGUL SYLLABLE CHIEUCH YEO SSANGKIYEOK
    ((0xAB, 0x74), (0xCCD3)),  # HANGUL SYLLABLE CHIEUCH YEO KIYEOKSIOS
    ((0xAB, 0x75), (0xCCD5)),  # HANGUL SYLLABLE CHIEUCH YEO NIEUNCIEUC
    ((0xAB, 0x76), (0xCCD6)),  # HANGUL SYLLABLE CHIEUCH YEO NIEUNHIEUH
    ((0xAB, 0x77), (0xCCD7)),  # HANGUL SYLLABLE CHIEUCH YEO TIKEUT
    ((0xAB, 0x78), (0xCCD8)),  # HANGUL SYLLABLE CHIEUCH YEO RIEUL
    ((0xAB, 0x79), (0xCCD9)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULKIYEOK
    ((0xAB, 0x7A), (0xCCDA)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULMIEUM
    ((0xAB, 0x81), (0xCCDB)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULPIEUP
    ((0xAB, 0x82), (0xCCDC)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULSIOS
    ((0xAB, 0x83), (0xCCDD)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULTHIEUTH
    ((0xAB, 0x84), (0xCCDE)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULPHIEUPH
    ((0xAB, 0x85), (0xCCDF)),  # HANGUL SYLLABLE CHIEUCH YEO RIEULHIEUH
    ((0xAB, 0x86), (0xCCE0)),  # HANGUL SYLLABLE CHIEUCH YEO MIEUM
    ((0xAB, 0x87), (0xCCE1)),  # HANGUL SYLLABLE CHIEUCH YEO PIEUP
    ((0xAB, 0x88), (0xCCE2)),  # HANGUL SYLLABLE CHIEUCH YEO PIEUPSIOS
    ((0xAB, 0x89), (0xCCE3)),  # HANGUL SYLLABLE CHIEUCH YEO SIOS
    ((0xAB, 0x8A), (0xCCE5)),  # HANGUL SYLLABLE CHIEUCH YEO IEUNG
    ((0xAB, 0x8B), (0xCCE6)),  # HANGUL SYLLABLE CHIEUCH YEO CIEUC
    ((0xAB, 0x8C), (0xCCE7)),  # HANGUL SYLLABLE CHIEUCH YEO CHIEUCH
    ((0xAB, 0x8D), (0xCCE8)),  # HANGUL SYLLABLE CHIEUCH YEO KHIEUKH
    ((0xAB, 0x8E), (0xCCE9)),  # HANGUL SYLLABLE CHIEUCH YEO THIEUTH
    ((0xAB, 0x8F), (0xCCEA)),  # HANGUL SYLLABLE CHIEUCH YEO PHIEUPH
    ((0xAB, 0x90), (0xCCEB)),  # HANGUL SYLLABLE CHIEUCH YEO HIEUH
    ((0xAB, 0x91), (0xCCED)),  # HANGUL SYLLABLE CHIEUCH YE KIYEOK
    ((0xAB, 0x92), (0xCCEE)),  # HANGUL SYLLABLE CHIEUCH YE SSANGKIYEOK
    ((0xAB, 0x93), (0xCCEF)),  # HANGUL SYLLABLE CHIEUCH YE KIYEOKSIOS
    ((0xAB, 0x94), (0xCCF1)),  # HANGUL SYLLABLE CHIEUCH YE NIEUNCIEUC
    ((0xAB, 0x95), (0xCCF2)),  # HANGUL SYLLABLE CHIEUCH YE NIEUNHIEUH
    ((0xAB, 0x96), (0xCCF3)),  # HANGUL SYLLABLE CHIEUCH YE TIKEUT
    ((0xAB, 0x97), (0xCCF4)),  # HANGUL SYLLABLE CHIEUCH YE RIEUL
    ((0xAB, 0x98), (0xCCF5)),  # HANGUL SYLLABLE CHIEUCH YE RIEULKIYEOK
    ((0xAB, 0x99), (0xCCF6)),  # HANGUL SYLLABLE CHIEUCH YE RIEULMIEUM
    ((0xAB, 0x9A), (0xCCF7)),  # HANGUL SYLLABLE CHIEUCH YE RIEULPIEUP
    ((0xAB, 0x9B), (0xCCF8)),  # HANGUL SYLLABLE CHIEUCH YE RIEULSIOS
    ((0xAB, 0x9C), (0xCCF9)),  # HANGUL SYLLABLE CHIEUCH YE RIEULTHIEUTH
    ((0xAB, 0x9D), (0xCCFA)),  # HANGUL SYLLABLE CHIEUCH YE RIEULPHIEUPH
    ((0xAB, 0x9E), (0xCCFB)),  # HANGUL SYLLABLE CHIEUCH YE RIEULHIEUH
    ((0xAB, 0x9F), (0xCCFC)),  # HANGUL SYLLABLE CHIEUCH YE MIEUM
    ((0xAB, 0xA0), (0xCCFD)),  # HANGUL SYLLABLE CHIEUCH YE PIEUP
    ((0xAB, 0xA1), (0x30A1)),  # KATAKANA LETTER SMALL A
    ((0xAB, 0xA2), (0x30A2)),  # KATAKANA LETTER A
    ((0xAB, 0xA3), (0x30A3)),  # KATAKANA LETTER SMALL I
    ((0xAB, 0xA4), (0x30A4)),  # KATAKANA LETTER I
    ((0xAB, 0xA5), (0x30A5)),  # KATAKANA LETTER SMALL U
    ((0xAB, 0xA6), (0x30A6)),  # KATAKANA LETTER U
    ((0xAB, 0xA7), (0x30A7)),  # KATAKANA LETTER SMALL E
    ((0xAB, 0xA8), (0x30A8)),  # KATAKANA LETTER E
    ((0xAB, 0xA9), (0x30A9)),  # KATAKANA LETTER SMALL O
    ((0xAB, 0xAA), (0x30AA)),  # KATAKANA LETTER O
    ((0xAB, 0xAB), (0x30AB)),  # KATAKANA LETTER KA
    ((0xAB, 0xAC), (0x30AC)),  # KATAKANA LETTER GA
    ((0xAB, 0xAD), (0x30AD)),  # KATAKANA LETTER KI
    ((0xAB, 0xAE), (0x30AE)),  # KATAKANA LETTER GI
    ((0xAB, 0xAF), (0x30AF)),  # KATAKANA LETTER KU
    ((0xAB, 0xB0), (0x30B0)),  # KATAKANA LETTER GU
    ((0xAB, 0xB1), (0x30B1)),  # KATAKANA LETTER KE
    ((0xAB, 0xB2), (0x30B2)),  # KATAKANA LETTER GE
    ((0xAB, 0xB3), (0x30B3)),  # KATAKANA LETTER KO
    ((0xAB, 0xB4), (0x30B4)),  # KATAKANA LETTER GO
    ((0xAB, 0xB5), (0x30B5)),  # KATAKANA LETTER SA
    ((0xAB, 0xB6), (0x30B6)),  # KATAKANA LETTER ZA
    ((0xAB, 0xB7), (0x30B7)),  # KATAKANA LETTER SI
    ((0xAB, 0xB8), (0x30B8)),  # KATAKANA LETTER ZI
    ((0xAB, 0xB9), (0x30B9)),  # KATAKANA LETTER SU
    ((0xAB, 0xBA), (0x30BA)),  # KATAKANA LETTER ZU
    ((0xAB, 0xBB), (0x30BB)),  # KATAKANA LETTER SE
    ((0xAB, 0xBC), (0x30BC)),  # KATAKANA LETTER ZE
    ((0xAB, 0xBD), (0x30BD)),  # KATAKANA LETTER SO
    ((0xAB, 0xBE), (0x30BE)),  # KATAKANA LETTER ZO
    ((0xAB, 0xBF), (0x30BF)),  # KATAKANA LETTER TA
    ((0xAB, 0xC0), (0x30C0)),  # KATAKANA LETTER DA
    ((0xAB, 0xC1), (0x30C1)),  # KATAKANA LETTER TI
    ((0xAB, 0xC2), (0x30C2)),  # KATAKANA LETTER DI
    ((0xAB, 0xC3), (0x30C3)),  # KATAKANA LETTER SMALL TU
    ((0xAB, 0xC4), (0x30C4)),  # KATAKANA LETTER TU
    ((0xAB, 0xC5), (0x30C5)),  # KATAKANA LETTER DU
    ((0xAB, 0xC6), (0x30C6)),  # KATAKANA LETTER TE
    ((0xAB, 0xC7), (0x30C7)),  # KATAKANA LETTER DE
    ((0xAB, 0xC8), (0x30C8)),  # KATAKANA LETTER TO
    ((0xAB, 0xC9), (0x30C9)),  # KATAKANA LETTER DO
    ((0xAB, 0xCA), (0x30CA)),  # KATAKANA LETTER NA
    ((0xAB, 0xCB), (0x30CB)),  # KATAKANA LETTER NI
    ((0xAB, 0xCC), (0x30CC)),  # KATAKANA LETTER NU
    ((0xAB, 0xCD), (0x30CD)),  # KATAKANA LETTER NE
    ((0xAB, 0xCE), (0x30CE)),  # KATAKANA LETTER NO
    ((0xAB, 0xCF), (0x30CF)),  # KATAKANA LETTER HA
    ((0xAB, 0xD0), (0x30D0)),  # KATAKANA LETTER BA
    ((0xAB, 0xD1), (0x30D1)),  # KATAKANA LETTER PA
    ((0xAB, 0xD2), (0x30D2)),  # KATAKANA LETTER HI
    ((0xAB, 0xD3), (0x30D3)),  # KATAKANA LETTER BI
    ((0xAB, 0xD4), (0x30D4)),  # KATAKANA LETTER PI
    ((0xAB, 0xD5), (0x30D5)),  # KATAKANA LETTER HU
    ((0xAB, 0xD6), (0x30D6)),  # KATAKANA LETTER BU
    ((0xAB, 0xD7), (0x30D7)),  # KATAKANA LETTER PU
    ((0xAB, 0xD8), (0x30D8)),  # KATAKANA LETTER HE
    ((0xAB, 0xD9), (0x30D9)),  # KATAKANA LETTER BE
    ((0xAB, 0xDA), (0x30DA)),  # KATAKANA LETTER PE
    ((0xAB, 0xDB), (0x30DB)),  # KATAKANA LETTER HO
    ((0xAB, 0xDC), (0x30DC)),  # KATAKANA LETTER BO
    ((0xAB, 0xDD), (0x30DD)),  # KATAKANA LETTER PO
    ((0xAB, 0xDE), (0x30DE)),  # KATAKANA LETTER MA
    ((0xAB, 0xDF), (0x30DF)),  # KATAKANA LETTER MI
    ((0xAB, 0xE0), (0x30E0)),  # KATAKANA LETTER MU
    ((0xAB, 0xE1), (0x30E1)),  # KATAKANA LETTER ME
    ((0xAB, 0xE2), (0x30E2)),  # KATAKANA LETTER MO
    ((0xAB, 0xE3), (0x30E3)),  # KATAKANA LETTER SMALL YA
    ((0xAB, 0xE4), (0x30E4)),  # KATAKANA LETTER YA
    ((0xAB, 0xE5), (0x30E5)),  # KATAKANA LETTER SMALL YU
    ((0xAB, 0xE6), (0x30E6)),  # KATAKANA LETTER YU
    ((0xAB, 0xE7), (0x30E7)),  # KATAKANA LETTER SMALL YO
    ((0xAB, 0xE8), (0x30E8)),  # KATAKANA LETTER YO
    ((0xAB, 0xE9), (0x30E9)),  # KATAKANA LETTER RA
    ((0xAB, 0xEA), (0x30EA)),  # KATAKANA LETTER RI
    ((0xAB, 0xEB), (0x30EB)),  # KATAKANA LETTER RU
    ((0xAB, 0xEC), (0x30EC)),  # KATAKANA LETTER RE
    ((0xAB, 0xED), (0x30ED)),  # KATAKANA LETTER RO
    ((0xAB, 0xEE), (0x30EE)),  # KATAKANA LETTER SMALL WA
    ((0xAB, 0xEF), (0x30EF)),  # KATAKANA LETTER WA
    ((0xAB, 0xF0), (0x30F0)),  # KATAKANA LETTER WI
    ((0xAB, 0xF1), (0x30F1)),  # KATAKANA LETTER WE
    ((0xAB, 0xF2), (0x30F2)),  # KATAKANA LETTER WO
    ((0xAB, 0xF3), (0x30F3)),  # KATAKANA LETTER N
    ((0xAB, 0xF4), (0x30F4)),  # KATAKANA LETTER VU
    ((0xAB, 0xF5), (0x30F5)),  # KATAKANA LETTER SMALL KA
    ((0xAB, 0xF6), (0x30F6)),  # KATAKANA LETTER SMALL KE
    ((0xAC, 0x41), (0xCCFE)),  # HANGUL SYLLABLE CHIEUCH YE PIEUPSIOS
    ((0xAC, 0x42), (0xCCFF)),  # HANGUL SYLLABLE CHIEUCH YE SIOS
    ((0xAC, 0x43), (0xCD00)),  # HANGUL SYLLABLE CHIEUCH YE SSANGSIOS
    ((0xAC, 0x44), (0xCD02)),  # HANGUL SYLLABLE CHIEUCH YE CIEUC
    ((0xAC, 0x45), (0xCD03)),  # HANGUL SYLLABLE CHIEUCH YE CHIEUCH
    ((0xAC, 0x46), (0xCD04)),  # HANGUL SYLLABLE CHIEUCH YE KHIEUKH
    ((0xAC, 0x47), (0xCD05)),  # HANGUL SYLLABLE CHIEUCH YE THIEUTH
    ((0xAC, 0x48), (0xCD06)),  # HANGUL SYLLABLE CHIEUCH YE PHIEUPH
    ((0xAC, 0x49), (0xCD07)),  # HANGUL SYLLABLE CHIEUCH YE HIEUH
    ((0xAC, 0x4A), (0xCD0A)),  # HANGUL SYLLABLE CHIEUCH O SSANGKIYEOK
    ((0xAC, 0x4B), (0xCD0B)),  # HANGUL SYLLABLE CHIEUCH O KIYEOKSIOS
    ((0xAC, 0x4C), (0xCD0D)),  # HANGUL SYLLABLE CHIEUCH O NIEUNCIEUC
    ((0xAC, 0x4D), (0xCD0E)),  # HANGUL SYLLABLE CHIEUCH O NIEUNHIEUH
    ((0xAC, 0x4E), (0xCD0F)),  # HANGUL SYLLABLE CHIEUCH O TIKEUT
    ((0xAC, 0x4F), (0xCD11)),  # HANGUL SYLLABLE CHIEUCH O RIEULKIYEOK
    ((0xAC, 0x50), (0xCD12)),  # HANGUL SYLLABLE CHIEUCH O RIEULMIEUM
    ((0xAC, 0x51), (0xCD13)),  # HANGUL SYLLABLE CHIEUCH O RIEULPIEUP
    ((0xAC, 0x52), (0xCD14)),  # HANGUL SYLLABLE CHIEUCH O RIEULSIOS
    ((0xAC, 0x53), (0xCD15)),  # HANGUL SYLLABLE CHIEUCH O RIEULTHIEUTH
    ((0xAC, 0x54), (0xCD16)),  # HANGUL SYLLABLE CHIEUCH O RIEULPHIEUPH
    ((0xAC, 0x55), (0xCD17)),  # HANGUL SYLLABLE CHIEUCH O RIEULHIEUH
    ((0xAC, 0x56), (0xCD1A)),  # HANGUL SYLLABLE CHIEUCH O PIEUPSIOS
    ((0xAC, 0x57), (0xCD1C)),  # HANGUL SYLLABLE CHIEUCH O SSANGSIOS
    ((0xAC, 0x58), (0xCD1E)),  # HANGUL SYLLABLE CHIEUCH O CIEUC
    ((0xAC, 0x59), (0xCD1F)),  # HANGUL SYLLABLE CHIEUCH O CHIEUCH
    ((0xAC, 0x5A), (0xCD20)),  # HANGUL SYLLABLE CHIEUCH O KHIEUKH
    ((0xAC, 0x61), (0xCD21)),  # HANGUL SYLLABLE CHIEUCH O THIEUTH
    ((0xAC, 0x62), (0xCD22)),  # HANGUL SYLLABLE CHIEUCH O PHIEUPH
    ((0xAC, 0x63), (0xCD23)),  # HANGUL SYLLABLE CHIEUCH O HIEUH
    ((0xAC, 0x64), (0xCD25)),  # HANGUL SYLLABLE CHIEUCH WA KIYEOK
    ((0xAC, 0x65), (0xCD26)),  # HANGUL SYLLABLE CHIEUCH WA SSANGKIYEOK
    ((0xAC, 0x66), (0xCD27)),  # HANGUL SYLLABLE CHIEUCH WA KIYEOKSIOS
    ((0xAC, 0x67), (0xCD29)),  # HANGUL SYLLABLE CHIEUCH WA NIEUNCIEUC
    ((0xAC, 0x68), (0xCD2A)),  # HANGUL SYLLABLE CHIEUCH WA NIEUNHIEUH
    ((0xAC, 0x69), (0xCD2B)),  # HANGUL SYLLABLE CHIEUCH WA TIKEUT
    ((0xAC, 0x6A), (0xCD2D)),  # HANGUL SYLLABLE CHIEUCH WA RIEULKIYEOK
    ((0xAC, 0x6B), (0xCD2E)),  # HANGUL SYLLABLE CHIEUCH WA RIEULMIEUM
    ((0xAC, 0x6C), (0xCD2F)),  # HANGUL SYLLABLE CHIEUCH WA RIEULPIEUP
    ((0xAC, 0x6D), (0xCD30)),  # HANGUL SYLLABLE CHIEUCH WA RIEULSIOS
    ((0xAC, 0x6E), (0xCD31)),  # HANGUL SYLLABLE CHIEUCH WA RIEULTHIEUTH
    ((0xAC, 0x6F), (0xCD32)),  # HANGUL SYLLABLE CHIEUCH WA RIEULPHIEUPH
    ((0xAC, 0x70), (0xCD33)),  # HANGUL SYLLABLE CHIEUCH WA RIEULHIEUH
    ((0xAC, 0x71), (0xCD34)),  # HANGUL SYLLABLE CHIEUCH WA MIEUM
    ((0xAC, 0x72), (0xCD35)),  # HANGUL SYLLABLE CHIEUCH WA PIEUP
    ((0xAC, 0x73), (0xCD36)),  # HANGUL SYLLABLE CHIEUCH WA PIEUPSIOS
    ((0xAC, 0x74), (0xCD37)),  # HANGUL SYLLABLE CHIEUCH WA SIOS
    ((0xAC, 0x75), (0xCD38)),  # HANGUL SYLLABLE CHIEUCH WA SSANGSIOS
    ((0xAC, 0x76), (0xCD3A)),  # HANGUL SYLLABLE CHIEUCH WA CIEUC
    ((0xAC, 0x77), (0xCD3B)),  # HANGUL SYLLABLE CHIEUCH WA CHIEUCH
    ((0xAC, 0x78), (0xCD3C)),  # HANGUL SYLLABLE CHIEUCH WA KHIEUKH
    ((0xAC, 0x79), (0xCD3D)),  # HANGUL SYLLABLE CHIEUCH WA THIEUTH
    ((0xAC, 0x7A), (0xCD3E)),  # HANGUL SYLLABLE CHIEUCH WA PHIEUPH
    ((0xAC, 0x81), (0xCD3F)),  # HANGUL SYLLABLE CHIEUCH WA HIEUH
    ((0xAC, 0x82), (0xCD40)),  # HANGUL SYLLABLE CHIEUCH WAE
    ((0xAC, 0x83), (0xCD41)),  # HANGUL SYLLABLE CHIEUCH WAE KIYEOK
    ((0xAC, 0x84), (0xCD42)),  # HANGUL SYLLABLE CHIEUCH WAE SSANGKIYEOK
    ((0xAC, 0x85), (0xCD43)),  # HANGUL SYLLABLE CHIEUCH WAE KIYEOKSIOS
    ((0xAC, 0x86), (0xCD44)),  # HANGUL SYLLABLE CHIEUCH WAE NIEUN
    ((0xAC, 0x87), (0xCD45)),  # HANGUL SYLLABLE CHIEUCH WAE NIEUNCIEUC
    ((0xAC, 0x88), (0xCD46)),  # HANGUL SYLLABLE CHIEUCH WAE NIEUNHIEUH
    ((0xAC, 0x89), (0xCD47)),  # HANGUL SYLLABLE CHIEUCH WAE TIKEUT
    ((0xAC, 0x8A), (0xCD48)),  # HANGUL SYLLABLE CHIEUCH WAE RIEUL
    ((0xAC, 0x8B), (0xCD49)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULKIYEOK
    ((0xAC, 0x8C), (0xCD4A)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULMIEUM
    ((0xAC, 0x8D), (0xCD4B)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULPIEUP
    ((0xAC, 0x8E), (0xCD4C)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULSIOS
    ((0xAC, 0x8F), (0xCD4D)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULTHIEUTH
    ((0xAC, 0x90), (0xCD4E)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULPHIEUPH
    ((0xAC, 0x91), (0xCD4F)),  # HANGUL SYLLABLE CHIEUCH WAE RIEULHIEUH
    ((0xAC, 0x92), (0xCD50)),  # HANGUL SYLLABLE CHIEUCH WAE MIEUM
    ((0xAC, 0x93), (0xCD51)),  # HANGUL SYLLABLE CHIEUCH WAE PIEUP
    ((0xAC, 0x94), (0xCD52)),  # HANGUL SYLLABLE CHIEUCH WAE PIEUPSIOS
    ((0xAC, 0x95), (0xCD53)),  # HANGUL SYLLABLE CHIEUCH WAE SIOS
    ((0xAC, 0x96), (0xCD54)),  # HANGUL SYLLABLE CHIEUCH WAE SSANGSIOS
    ((0xAC, 0x97), (0xCD55)),  # HANGUL SYLLABLE CHIEUCH WAE IEUNG
    ((0xAC, 0x98), (0xCD56)),  # HANGUL SYLLABLE CHIEUCH WAE CIEUC
    ((0xAC, 0x99), (0xCD57)),  # HANGUL SYLLABLE CHIEUCH WAE CHIEUCH
    ((0xAC, 0x9A), (0xCD58)),  # HANGUL SYLLABLE CHIEUCH WAE KHIEUKH
    ((0xAC, 0x9B), (0xCD59)),  # HANGUL SYLLABLE CHIEUCH WAE THIEUTH
    ((0xAC, 0x9C), (0xCD5A)),  # HANGUL SYLLABLE CHIEUCH WAE PHIEUPH
    ((0xAC, 0x9D), (0xCD5B)),  # HANGUL SYLLABLE CHIEUCH WAE HIEUH
    ((0xAC, 0x9E), (0xCD5D)),  # HANGUL SYLLABLE CHIEUCH OE KIYEOK
    ((0xAC, 0x9F), (0xCD5E)),  # HANGUL SYLLABLE CHIEUCH OE SSANGKIYEOK
    ((0xAC, 0xA0), (0xCD5F)),  # HANGUL SYLLABLE CHIEUCH OE KIYEOKSIOS
    ((0xAC, 0xA1), (0x0410)),  # CYRILLIC CAPITAL LETTER A
    ((0xAC, 0xA2), (0x0411)),  # CYRILLIC CAPITAL LETTER BE
    ((0xAC, 0xA3), (0x0412)),  # CYRILLIC CAPITAL LETTER VE
    ((0xAC, 0xA4), (0x0413)),  # CYRILLIC CAPITAL LETTER GHE
    ((0xAC, 0xA5), (0x0414)),  # CYRILLIC CAPITAL LETTER DE
    ((0xAC, 0xA6), (0x0415)),  # CYRILLIC CAPITAL LETTER IE
    ((0xAC, 0xA7), (0x0401)),  # CYRILLIC CAPITAL LETTER IO
    ((0xAC, 0xA8), (0x0416)),  # CYRILLIC CAPITAL LETTER ZHE
    ((0xAC, 0xA9), (0x0417)),  # CYRILLIC CAPITAL LETTER ZE
    ((0xAC, 0xAA), (0x0418)),  # CYRILLIC CAPITAL LETTER I
    ((0xAC, 0xAB), (0x0419)),  # CYRILLIC CAPITAL LETTER SHORT I
    ((0xAC, 0xAC), (0x041A)),  # CYRILLIC CAPITAL LETTER KA
    ((0xAC, 0xAD), (0x041B)),  # CYRILLIC CAPITAL LETTER EL
    ((0xAC, 0xAE), (0x041C)),  # CYRILLIC CAPITAL LETTER EM
    ((0xAC, 0xAF), (0x041D)),  # CYRILLIC CAPITAL LETTER EN
    ((0xAC, 0xB0), (0x041E)),  # CYRILLIC CAPITAL LETTER O
    ((0xAC, 0xB1), (0x041F)),  # CYRILLIC CAPITAL LETTER PE
    ((0xAC, 0xB2), (0x0420)),  # CYRILLIC CAPITAL LETTER ER
    ((0xAC, 0xB3), (0x0421)),  # CYRILLIC CAPITAL LETTER ES
    ((0xAC, 0xB4), (0x0422)),  # CYRILLIC CAPITAL LETTER TE
    ((0xAC, 0xB5), (0x0423)),  # CYRILLIC CAPITAL LETTER U
    ((0xAC, 0xB6), (0x0424)),  # CYRILLIC CAPITAL LETTER EF
    ((0xAC, 0xB7), (0x0425)),  # CYRILLIC CAPITAL LETTER HA
    ((0xAC, 0xB8), (0x0426)),  # CYRILLIC CAPITAL LETTER TSE
    ((0xAC, 0xB9), (0x0427)),  # CYRILLIC CAPITAL LETTER CHE
    ((0xAC, 0xBA), (0x0428)),  # CYRILLIC CAPITAL LETTER SHA
    ((0xAC, 0xBB), (0x0429)),  # CYRILLIC CAPITAL LETTER SHCHA
    ((0xAC, 0xBC), (0x042A)),  # CYRILLIC CAPITAL LETTER HARD SIGN
    ((0xAC, 0xBD), (0x042B)),  # CYRILLIC CAPITAL LETTER YERU
    ((0xAC, 0xBE), (0x042C)),  # CYRILLIC CAPITAL LETTER SOFT SIGN
    ((0xAC, 0xBF), (0x042D)),  # CYRILLIC CAPITAL LETTER E
    ((0xAC, 0xC0), (0x042E)),  # CYRILLIC CAPITAL LETTER YU
    ((0xAC, 0xC1), (0x042F)),  # CYRILLIC CAPITAL LETTER YA
    ((0xAC, 0xD1), (0x0430)),  # CYRILLIC SMALL LETTER A
    ((0xAC, 0xD2), (0x0431)),  # CYRILLIC SMALL LETTER BE
    ((0xAC, 0xD3), (0x0432)),  # CYRILLIC SMALL LETTER VE
    ((0xAC, 0xD4), (0x0433)),  # CYRILLIC SMALL LETTER GHE
    ((0xAC, 0xD5), (0x0434)),  # CYRILLIC SMALL LETTER DE
    ((0xAC, 0xD6), (0x0435)),  # CYRILLIC SMALL LETTER IE
    ((0xAC, 0xD7), (0x0451)),  # CYRILLIC SMALL LETTER IO
    ((0xAC, 0xD8), (0x0436)),  # CYRILLIC SMALL LETTER ZHE
    ((0xAC, 0xD9), (0x0437)),  # CYRILLIC SMALL LETTER ZE
    ((0xAC, 0xDA), (0x0438)),  # CYRILLIC SMALL LETTER I
    ((0xAC, 0xDB), (0x0439)),  # CYRILLIC SMALL LETTER SHORT I
    ((0xAC, 0xDC), (0x043A)),  # CYRILLIC SMALL LETTER KA
    ((0xAC, 0xDD), (0x043B)),  # CYRILLIC SMALL LETTER EL
    ((0xAC, 0xDE), (0x043C)),  # CYRILLIC SMALL LETTER EM
    ((0xAC, 0xDF), (0x043D)),  # CYRILLIC SMALL LETTER EN
    ((0xAC, 0xE0), (0x043E)),  # CYRILLIC SMALL LETTER O
    ((0xAC, 0xE1), (0x043F)),  # CYRILLIC SMALL LETTER PE
    ((0xAC, 0xE2), (0x0440)),  # CYRILLIC SMALL LETTER ER
    ((0xAC, 0xE3), (0x0441)),  # CYRILLIC SMALL LETTER ES
    ((0xAC, 0xE4), (0x0442)),  # CYRILLIC SMALL LETTER TE
    ((0xAC, 0xE5), (0x0443)),  # CYRILLIC SMALL LETTER U
    ((0xAC, 0xE6), (0x0444)),  # CYRILLIC SMALL LETTER EF
    ((0xAC, 0xE7), (0x0445)),  # CYRILLIC SMALL LETTER HA
    ((0xAC, 0xE8), (0x0446)),  # CYRILLIC SMALL LETTER TSE
    ((0xAC, 0xE9), (0x0447)),  # CYRILLIC SMALL LETTER CHE
    ((0xAC, 0xEA), (0x0448)),  # CYRILLIC SMALL LETTER SHA
    ((0xAC, 0xEB), (0x0449)),  # CYRILLIC SMALL LETTER SHCHA
    ((0xAC, 0xEC), (0x044A)),  # CYRILLIC SMALL LETTER HARD SIGN
    ((0xAC, 0xED), (0x044B)),  # CYRILLIC SMALL LETTER YERU
    ((0xAC, 0xEE), (0x044C)),  # CYRILLIC SMALL LETTER SOFT SIGN
    ((0xAC, 0xEF), (0x044D)),  # CYRILLIC SMALL LETTER E
    ((0xAC, 0xF0), (0x044E)),  # CYRILLIC SMALL LETTER YU
    ((0xAC, 0xF1), (0x044F)),  # CYRILLIC SMALL LETTER YA
    ((0xAD, 0x41), (0xCD61)),  # HANGUL SYLLABLE CHIEUCH OE NIEUNCIEUC
    ((0xAD, 0x42), (0xCD62)),  # HANGUL SYLLABLE CHIEUCH OE NIEUNHIEUH
    ((0xAD, 0x43), (0xCD63)),  # HANGUL SYLLABLE CHIEUCH OE TIKEUT
    ((0xAD, 0x44), (0xCD65)),  # HANGUL SYLLABLE CHIEUCH OE RIEULKIYEOK
    ((0xAD, 0x45), (0xCD66)),  # HANGUL SYLLABLE CHIEUCH OE RIEULMIEUM
    ((0xAD, 0x46), (0xCD67)),  # HANGUL SYLLABLE CHIEUCH OE RIEULPIEUP
    ((0xAD, 0x47), (0xCD68)),  # HANGUL SYLLABLE CHIEUCH OE RIEULSIOS
    ((0xAD, 0x48), (0xCD69)),  # HANGUL SYLLABLE CHIEUCH OE RIEULTHIEUTH
    ((0xAD, 0x49), (0xCD6A)),  # HANGUL SYLLABLE CHIEUCH OE RIEULPHIEUPH
    ((0xAD, 0x4A), (0xCD6B)),  # HANGUL SYLLABLE CHIEUCH OE RIEULHIEUH
    ((0xAD, 0x4B), (0xCD6E)),  # HANGUL SYLLABLE CHIEUCH OE PIEUPSIOS
    ((0xAD, 0x4C), (0xCD70)),  # HANGUL SYLLABLE CHIEUCH OE SSANGSIOS
    ((0xAD, 0x4D), (0xCD72)),  # HANGUL SYLLABLE CHIEUCH OE CIEUC
    ((0xAD, 0x4E), (0xCD73)),  # HANGUL SYLLABLE CHIEUCH OE CHIEUCH
    ((0xAD, 0x4F), (0xCD74)),  # HANGUL SYLLABLE CHIEUCH OE KHIEUKH
    ((0xAD, 0x50), (0xCD75)),  # HANGUL SYLLABLE CHIEUCH OE THIEUTH
    ((0xAD, 0x51), (0xCD76)),  # HANGUL SYLLABLE CHIEUCH OE PHIEUPH
    ((0xAD, 0x52), (0xCD77)),  # HANGUL SYLLABLE CHIEUCH OE HIEUH
    ((0xAD, 0x53), (0xCD79)),  # HANGUL SYLLABLE CHIEUCH YO KIYEOK
    ((0xAD, 0x54), (0xCD7A)),  # HANGUL SYLLABLE CHIEUCH YO SSANGKIYEOK
    ((0xAD, 0x55), (0xCD7B)),  # HANGUL SYLLABLE CHIEUCH YO KIYEOKSIOS
    ((0xAD, 0x56), (0xCD7C)),  # HANGUL SYLLABLE CHIEUCH YO NIEUN
    ((0xAD, 0x57), (0xCD7D)),  # HANGUL SYLLABLE CHIEUCH YO NIEUNCIEUC
    ((0xAD, 0x58), (0xCD7E)),  # HANGUL SYLLABLE CHIEUCH YO NIEUNHIEUH
    ((0xAD, 0x59), (0xCD7F)),  # HANGUL SYLLABLE CHIEUCH YO TIKEUT
    ((0xAD, 0x5A), (0xCD80)),  # HANGUL SYLLABLE CHIEUCH YO RIEUL
    ((0xAD, 0x61), (0xCD81)),  # HANGUL SYLLABLE CHIEUCH YO RIEULKIYEOK
    ((0xAD, 0x62), (0xCD82)),  # HANGUL SYLLABLE CHIEUCH YO RIEULMIEUM
    ((0xAD, 0x63), (0xCD83)),  # HANGUL SYLLABLE CHIEUCH YO RIEULPIEUP
    ((0xAD, 0x64), (0xCD84)),  # HANGUL SYLLABLE CHIEUCH YO RIEULSIOS
    ((0xAD, 0x65), (0xCD85)),  # HANGUL SYLLABLE CHIEUCH YO RIEULTHIEUTH
    ((0xAD, 0x66), (0xCD86)),  # HANGUL SYLLABLE CHIEUCH YO RIEULPHIEUPH
    ((0xAD, 0x67), (0xCD87)),  # HANGUL SYLLABLE CHIEUCH YO RIEULHIEUH
    ((0xAD, 0x68), (0xCD89)),  # HANGUL SYLLABLE CHIEUCH YO PIEUP
    ((0xAD, 0x69), (0xCD8A)),  # HANGUL SYLLABLE CHIEUCH YO PIEUPSIOS
    ((0xAD, 0x6A), (0xCD8B)),  # HANGUL SYLLABLE CHIEUCH YO SIOS
    ((0xAD, 0x6B), (0xCD8C)),  # HANGUL SYLLABLE CHIEUCH YO SSANGSIOS
    ((0xAD, 0x6C), (0xCD8D)),  # HANGUL SYLLABLE CHIEUCH YO IEUNG
    ((0xAD, 0x6D), (0xCD8E)),  # HANGUL SYLLABLE CHIEUCH YO CIEUC
    ((0xAD, 0x6E), (0xCD8F)),  # HANGUL SYLLABLE CHIEUCH YO CHIEUCH
    ((0xAD, 0x6F), (0xCD90)),  # HANGUL SYLLABLE CHIEUCH YO KHIEUKH
    ((0xAD, 0x70), (0xCD91)),  # HANGUL SYLLABLE CHIEUCH YO THIEUTH
    ((0xAD, 0x71), (0xCD92)),  # HANGUL SYLLABLE CHIEUCH YO PHIEUPH
    ((0xAD, 0x72), (0xCD93)),  # HANGUL SYLLABLE CHIEUCH YO HIEUH
    ((0xAD, 0x73), (0xCD96)),  # HANGUL SYLLABLE CHIEUCH U SSANGKIYEOK
    ((0xAD, 0x74), (0xCD97)),  # HANGUL SYLLABLE CHIEUCH U KIYEOKSIOS
    ((0xAD, 0x75), (0xCD99)),  # HANGUL SYLLABLE CHIEUCH U NIEUNCIEUC
    ((0xAD, 0x76), (0xCD9A)),  # HANGUL SYLLABLE CHIEUCH U NIEUNHIEUH
    ((0xAD, 0x77), (0xCD9B)),  # HANGUL SYLLABLE CHIEUCH U TIKEUT
    ((0xAD, 0x78), (0xCD9D)),  # HANGUL SYLLABLE CHIEUCH U RIEULKIYEOK
    ((0xAD, 0x79), (0xCD9E)),  # HANGUL SYLLABLE CHIEUCH U RIEULMIEUM
    ((0xAD, 0x7A), (0xCD9F)),  # HANGUL SYLLABLE CHIEUCH U RIEULPIEUP
    ((0xAD, 0x81), (0xCDA0)),  # HANGUL SYLLABLE CHIEUCH U RIEULSIOS
    ((0xAD, 0x82), (0xCDA1)),  # HANGUL SYLLABLE CHIEUCH U RIEULTHIEUTH
    ((0xAD, 0x83), (0xCDA2)),  # HANGUL SYLLABLE CHIEUCH U RIEULPHIEUPH
    ((0xAD, 0x84), (0xCDA3)),  # HANGUL SYLLABLE CHIEUCH U RIEULHIEUH
    ((0xAD, 0x85), (0xCDA6)),  # HANGUL SYLLABLE CHIEUCH U PIEUPSIOS
    ((0xAD, 0x86), (0xCDA8)),  # HANGUL SYLLABLE CHIEUCH U SSANGSIOS
    ((0xAD, 0x87), (0xCDAA)),  # HANGUL SYLLABLE CHIEUCH U CIEUC
    ((0xAD, 0x88), (0xCDAB)),  # HANGUL SYLLABLE CHIEUCH U CHIEUCH
    ((0xAD, 0x89), (0xCDAC)),  # HANGUL SYLLABLE CHIEUCH U KHIEUKH
    ((0xAD, 0x8A), (0xCDAD)),  # HANGUL SYLLABLE CHIEUCH U THIEUTH
    ((0xAD, 0x8B), (0xCDAE)),  # HANGUL SYLLABLE CHIEUCH U PHIEUPH
    ((0xAD, 0x8C), (0xCDAF)),  # HANGUL SYLLABLE CHIEUCH U HIEUH
    ((0xAD, 0x8D), (0xCDB1)),  # HANGUL SYLLABLE CHIEUCH WEO KIYEOK
    ((0xAD, 0x8E), (0xCDB2)),  # HANGUL SYLLABLE CHIEUCH WEO SSANGKIYEOK
    ((0xAD, 0x8F), (0xCDB3)),  # HANGUL SYLLABLE CHIEUCH WEO KIYEOKSIOS
    ((0xAD, 0x90), (0xCDB4)),  # HANGUL SYLLABLE CHIEUCH WEO NIEUN
    ((0xAD, 0x91), (0xCDB5)),  # HANGUL SYLLABLE CHIEUCH WEO NIEUNCIEUC
    ((0xAD, 0x92), (0xCDB6)),  # HANGUL SYLLABLE CHIEUCH WEO NIEUNHIEUH
    ((0xAD, 0x93), (0xCDB7)),  # HANGUL SYLLABLE CHIEUCH WEO TIKEUT
    ((0xAD, 0x94), (0xCDB8)),  # HANGUL SYLLABLE CHIEUCH WEO RIEUL
    ((0xAD, 0x95), (0xCDB9)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULKIYEOK
    ((0xAD, 0x96), (0xCDBA)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULMIEUM
    ((0xAD, 0x97), (0xCDBB)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULPIEUP
    ((0xAD, 0x98), (0xCDBC)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULSIOS
    ((0xAD, 0x99), (0xCDBD)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULTHIEUTH
    ((0xAD, 0x9A), (0xCDBE)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULPHIEUPH
    ((0xAD, 0x9B), (0xCDBF)),  # HANGUL SYLLABLE CHIEUCH WEO RIEULHIEUH
    ((0xAD, 0x9C), (0xCDC0)),  # HANGUL SYLLABLE CHIEUCH WEO MIEUM
    ((0xAD, 0x9D), (0xCDC1)),  # HANGUL SYLLABLE CHIEUCH WEO PIEUP
    ((0xAD, 0x9E), (0xCDC2)),  # HANGUL SYLLABLE CHIEUCH WEO PIEUPSIOS
    ((0xAD, 0x9F), (0xCDC3)),  # HANGUL SYLLABLE CHIEUCH WEO SIOS
    ((0xAD, 0xA0), (0xCDC5)),  # HANGUL SYLLABLE CHIEUCH WEO IEUNG
    ((0xAE, 0x41), (0xCDC6)),  # HANGUL SYLLABLE CHIEUCH WEO CIEUC
    ((0xAE, 0x42), (0xCDC7)),  # HANGUL SYLLABLE CHIEUCH WEO CHIEUCH
    ((0xAE, 0x43), (0xCDC8)),  # HANGUL SYLLABLE CHIEUCH WEO KHIEUKH
    ((0xAE, 0x44), (0xCDC9)),  # HANGUL SYLLABLE CHIEUCH WEO THIEUTH
    ((0xAE, 0x45), (0xCDCA)),  # HANGUL SYLLABLE CHIEUCH WEO PHIEUPH
    ((0xAE, 0x46), (0xCDCB)),  # HANGUL SYLLABLE CHIEUCH WEO HIEUH
    ((0xAE, 0x47), (0xCDCD)),  # HANGUL SYLLABLE CHIEUCH WE KIYEOK
    ((0xAE, 0x48), (0xCDCE)),  # HANGUL SYLLABLE CHIEUCH WE SSANGKIYEOK
    ((0xAE, 0x49), (0xCDCF)),  # HANGUL SYLLABLE CHIEUCH WE KIYEOKSIOS
    ((0xAE, 0x4A), (0xCDD1)),  # HANGUL SYLLABLE CHIEUCH WE NIEUNCIEUC
    ((0xAE, 0x4B), (0xCDD2)),  # HANGUL SYLLABLE CHIEUCH WE NIEUNHIEUH
    ((0xAE, 0x4C), (0xCDD3)),  # HANGUL SYLLABLE CHIEUCH WE TIKEUT
    ((0xAE, 0x4D), (0xCDD4)),  # HANGUL SYLLABLE CHIEUCH WE RIEUL
    ((0xAE, 0x4E), (0xCDD5)),  # HANGUL SYLLABLE CHIEUCH WE RIEULKIYEOK
    ((0xAE, 0x4F), (0xCDD6)),  # HANGUL SYLLABLE CHIEUCH WE RIEULMIEUM
    ((0xAE, 0x50), (0xCDD7)),  # HANGUL SYLLABLE CHIEUCH WE RIEULPIEUP
    ((0xAE, 0x51), (0xCDD8)),  # HANGUL SYLLABLE CHIEUCH WE RIEULSIOS
    ((0xAE, 0x52), (0xCDD9)),  # HANGUL SYLLABLE CHIEUCH WE RIEULTHIEUTH
    ((0xAE, 0x53), (0xCDDA)),  # HANGUL SYLLABLE CHIEUCH WE RIEULPHIEUPH
    ((0xAE, 0x54), (0xCDDB)),  # HANGUL SYLLABLE CHIEUCH WE RIEULHIEUH
    ((0xAE, 0x55), (0xCDDC)),  # HANGUL SYLLABLE CHIEUCH WE MIEUM
    ((0xAE, 0x56), (0xCDDD)),  # HANGUL SYLLABLE CHIEUCH WE PIEUP
    ((0xAE, 0x57), (0xCDDE)),  # HANGUL SYLLABLE CHIEUCH WE PIEUPSIOS
    ((0xAE, 0x58), (0xCDDF)),  # HANGUL SYLLABLE CHIEUCH WE SIOS
    ((0xAE, 0x59), (0xCDE0)),  # HANGUL SYLLABLE CHIEUCH WE SSANGSIOS
    ((0xAE, 0x5A), (0xCDE1)),  # HANGUL SYLLABLE CHIEUCH WE IEUNG
    ((0xAE, 0x61), (0xCDE2)),  # HANGUL SYLLABLE CHIEUCH WE CIEUC
    ((0xAE, 0x62), (0xCDE3)),  # HANGUL SYLLABLE CHIEUCH WE CHIEUCH
    ((0xAE, 0x63), (0xCDE4)),  # HANGUL SYLLABLE CHIEUCH WE KHIEUKH
    ((0xAE, 0x64), (0xCDE5)),  # HANGUL SYLLABLE CHIEUCH WE THIEUTH
    ((0xAE, 0x65), (0xCDE6)),  # HANGUL SYLLABLE CHIEUCH WE PHIEUPH
    ((0xAE, 0x66), (0xCDE7)),  # HANGUL SYLLABLE CHIEUCH WE HIEUH
    ((0xAE, 0x67), (0xCDE9)),  # HANGUL SYLLABLE CHIEUCH WI KIYEOK
    ((0xAE, 0x68), (0xCDEA)),  # HANGUL SYLLABLE CHIEUCH WI SSANGKIYEOK
    ((0xAE, 0x69), (0xCDEB)),  # HANGUL SYLLABLE CHIEUCH WI KIYEOKSIOS
    ((0xAE, 0x6A), (0xCDED)),  # HANGUL SYLLABLE CHIEUCH WI NIEUNCIEUC
    ((0xAE, 0x6B), (0xCDEE)),  # HANGUL SYLLABLE CHIEUCH WI NIEUNHIEUH
    ((0xAE, 0x6C), (0xCDEF)),  # HANGUL SYLLABLE CHIEUCH WI TIKEUT
    ((0xAE, 0x6D), (0xCDF1)),  # HANGUL SYLLABLE CHIEUCH WI RIEULKIYEOK
    ((0xAE, 0x6E), (0xCDF2)),  # HANGUL SYLLABLE CHIEUCH WI RIEULMIEUM
    ((0xAE, 0x6F), (0xCDF3)),  # HANGUL SYLLABLE CHIEUCH WI RIEULPIEUP
    ((0xAE, 0x70), (0xCDF4)),  # HANGUL SYLLABLE CHIEUCH WI RIEULSIOS
    ((0xAE, 0x71), (0xCDF5)),  # HANGUL SYLLABLE CHIEUCH WI RIEULTHIEUTH
    ((0xAE, 0x72), (0xCDF6)),  # HANGUL SYLLABLE CHIEUCH WI RIEULPHIEUPH
    ((0xAE, 0x73), (0xCDF7)),  # HANGUL SYLLABLE CHIEUCH WI RIEULHIEUH
    ((0xAE, 0x74), (0xCDFA)),  # HANGUL SYLLABLE CHIEUCH WI PIEUPSIOS
    ((0xAE, 0x75), (0xCDFC)),  # HANGUL SYLLABLE CHIEUCH WI SSANGSIOS
    ((0xAE, 0x76), (0xCDFE)),  # HANGUL SYLLABLE CHIEUCH WI CIEUC
    ((0xAE, 0x77), (0xCDFF)),  # HANGUL SYLLABLE CHIEUCH WI CHIEUCH
    ((0xAE, 0x78), (0xCE00)),  # HANGUL SYLLABLE CHIEUCH WI KHIEUKH
    ((0xAE, 0x79), (0xCE01)),  # HANGUL SYLLABLE CHIEUCH WI THIEUTH
    ((0xAE, 0x7A), (0xCE02)),  # HANGUL SYLLABLE CHIEUCH WI PHIEUPH
    ((0xAE, 0x81), (0xCE03)),  # HANGUL SYLLABLE CHIEUCH WI HIEUH
    ((0xAE, 0x82), (0xCE05)),  # HANGUL SYLLABLE CHIEUCH YU KIYEOK
    ((0xAE, 0x83), (0xCE06)),  # HANGUL SYLLABLE CHIEUCH YU SSANGKIYEOK
    ((0xAE, 0x84), (0xCE07)),  # HANGUL SYLLABLE CHIEUCH YU KIYEOKSIOS
    ((0xAE, 0x85), (0xCE09)),  # HANGUL SYLLABLE CHIEUCH YU NIEUNCIEUC
    ((0xAE, 0x86), (0xCE0A)),  # HANGUL SYLLABLE CHIEUCH YU NIEUNHIEUH
    ((0xAE, 0x87), (0xCE0B)),  # HANGUL SYLLABLE CHIEUCH YU TIKEUT
    ((0xAE, 0x88), (0xCE0D)),  # HANGUL SYLLABLE CHIEUCH YU RIEULKIYEOK
    ((0xAE, 0x89), (0xCE0E)),  # HANGUL SYLLABLE CHIEUCH YU RIEULMIEUM
    ((0xAE, 0x8A), (0xCE0F)),  # HANGUL SYLLABLE CHIEUCH YU RIEULPIEUP
    ((0xAE, 0x8B), (0xCE10)),  # HANGUL SYLLABLE CHIEUCH YU RIEULSIOS
    ((0xAE, 0x8C), (0xCE11)),  # HANGUL SYLLABLE CHIEUCH YU RIEULTHIEUTH
    ((0xAE, 0x8D), (0xCE12)),  # HANGUL SYLLABLE CHIEUCH YU RIEULPHIEUPH
    ((0xAE, 0x8E), (0xCE13)),  # HANGUL SYLLABLE CHIEUCH YU RIEULHIEUH
    ((0xAE, 0x8F), (0xCE15)),  # HANGUL SYLLABLE CHIEUCH YU PIEUP
    ((0xAE, 0x90), (0xCE16)),  # HANGUL SYLLABLE CHIEUCH YU PIEUPSIOS
    ((0xAE, 0x91), (0xCE17)),  # HANGUL SYLLABLE CHIEUCH YU SIOS
    ((0xAE, 0x92), (0xCE18)),  # HANGUL SYLLABLE CHIEUCH YU SSANGSIOS
    ((0xAE, 0x93), (0xCE1A)),  # HANGUL SYLLABLE CHIEUCH YU CIEUC
    ((0xAE, 0x94), (0xCE1B)),  # HANGUL SYLLABLE CHIEUCH YU CHIEUCH
    ((0xAE, 0x95), (0xCE1C)),  # HANGUL SYLLABLE CHIEUCH YU KHIEUKH
    ((0xAE, 0x96), (0xCE1D)),  # HANGUL SYLLABLE CHIEUCH YU THIEUTH
    ((0xAE, 0x97), (0xCE1E)),  # HANGUL SYLLABLE CHIEUCH YU PHIEUPH
    ((0xAE, 0x98), (0xCE1F)),  # HANGUL SYLLABLE CHIEUCH YU HIEUH
    ((0xAE, 0x99), (0xCE22)),  # HANGUL SYLLABLE CHIEUCH EU SSANGKIYEOK
    ((0xAE, 0x9A), (0xCE23)),  # HANGUL SYLLABLE CHIEUCH EU KIYEOKSIOS
    ((0xAE, 0x9B), (0xCE25)),  # HANGUL SYLLABLE CHIEUCH EU NIEUNCIEUC
    ((0xAE, 0x9C), (0xCE26)),  # HANGUL SYLLABLE CHIEUCH EU NIEUNHIEUH
    ((0xAE, 0x9D), (0xCE27)),  # HANGUL SYLLABLE CHIEUCH EU TIKEUT
    ((0xAE, 0x9E), (0xCE29)),  # HANGUL SYLLABLE CHIEUCH EU RIEULKIYEOK
    ((0xAE, 0x9F), (0xCE2A)),  # HANGUL SYLLABLE CHIEUCH EU RIEULMIEUM
    ((0xAE, 0xA0), (0xCE2B)),  # HANGUL SYLLABLE CHIEUCH EU RIEULPIEUP
    ((0xAF, 0x41), (0xCE2C)),  # HANGUL SYLLABLE CHIEUCH EU RIEULSIOS
    ((0xAF, 0x42), (0xCE2D)),  # HANGUL SYLLABLE CHIEUCH EU RIEULTHIEUTH
    ((0xAF, 0x43), (0xCE2E)),  # HANGUL SYLLABLE CHIEUCH EU RIEULPHIEUPH
    ((0xAF, 0x44), (0xCE2F)),  # HANGUL SYLLABLE CHIEUCH EU RIEULHIEUH
    ((0xAF, 0x45), (0xCE32)),  # HANGUL SYLLABLE CHIEUCH EU PIEUPSIOS
    ((0xAF, 0x46), (0xCE34)),  # HANGUL SYLLABLE CHIEUCH EU SSANGSIOS
    ((0xAF, 0x47), (0xCE36)),  # HANGUL SYLLABLE CHIEUCH EU CIEUC
    ((0xAF, 0x48), (0xCE37)),  # HANGUL SYLLABLE CHIEUCH EU CHIEUCH
    ((0xAF, 0x49), (0xCE38)),  # HANGUL SYLLABLE CHIEUCH EU KHIEUKH
    ((0xAF, 0x4A), (0xCE39)),  # HANGUL SYLLABLE CHIEUCH EU THIEUTH
    ((0xAF, 0x4B), (0xCE3A)),  # HANGUL SYLLABLE CHIEUCH EU PHIEUPH
    ((0xAF, 0x4C), (0xCE3B)),  # HANGUL SYLLABLE CHIEUCH EU HIEUH
    ((0xAF, 0x4D), (0xCE3C)),  # HANGUL SYLLABLE CHIEUCH YI
    ((0xAF, 0x4E), (0xCE3D)),  # HANGUL SYLLABLE CHIEUCH YI KIYEOK
    ((0xAF, 0x4F), (0xCE3E)),  # HANGUL SYLLABLE CHIEUCH YI SSANGKIYEOK
    ((0xAF, 0x50), (0xCE3F)),  # HANGUL SYLLABLE CHIEUCH YI KIYEOKSIOS
    ((0xAF, 0x51), (0xCE40)),  # HANGUL SYLLABLE CHIEUCH YI NIEUN
    ((0xAF, 0x52), (0xCE41)),  # HANGUL SYLLABLE CHIEUCH YI NIEUNCIEUC
    ((0xAF, 0x53), (0xCE42)),  # HANGUL SYLLABLE CHIEUCH YI NIEUNHIEUH
    ((0xAF, 0x54), (0xCE43)),  # HANGUL SYLLABLE CHIEUCH YI TIKEUT
    ((0xAF, 0x55), (0xCE44)),  # HANGUL SYLLABLE CHIEUCH YI RIEUL
    ((0xAF, 0x56), (0xCE45)),  # HANGUL SYLLABLE CHIEUCH YI RIEULKIYEOK
    ((0xAF, 0x57), (0xCE46)),  # HANGUL SYLLABLE CHIEUCH YI RIEULMIEUM
    ((0xAF, 0x58), (0xCE47)),  # HANGUL SYLLABLE CHIEUCH YI RIEULPIEUP
    ((0xAF, 0x59), (0xCE48)),  # HANGUL SYLLABLE CHIEUCH YI RIEULSIOS
    ((0xAF, 0x5A), (0xCE49)),  # HANGUL SYLLABLE CHIEUCH YI RIEULTHIEUTH
    ((0xAF, 0x61), (0xCE4A)),  # HANGUL SYLLABLE CHIEUCH YI RIEULPHIEUPH
    ((0xAF, 0x62), (0xCE4B)),  # HANGUL SYLLABLE CHIEUCH YI RIEULHIEUH
    ((0xAF, 0x63), (0xCE4C)),  # HANGUL SYLLABLE CHIEUCH YI MIEUM
    ((0xAF, 0x64), (0xCE4D)),  # HANGUL SYLLABLE CHIEUCH YI PIEUP
    ((0xAF, 0x65), (0xCE4E)),  # HANGUL SYLLABLE CHIEUCH YI PIEUPSIOS
    ((0xAF, 0x66), (0xCE4F)),  # HANGUL SYLLABLE CHIEUCH YI SIOS
    ((0xAF, 0x67), (0xCE50)),  # HANGUL SYLLABLE CHIEUCH YI SSANGSIOS
    ((0xAF, 0x68), (0xCE51)),  # HANGUL SYLLABLE CHIEUCH YI IEUNG
    ((0xAF, 0x69), (0xCE52)),  # HANGUL SYLLABLE CHIEUCH YI CIEUC
    ((0xAF, 0x6A), (0xCE53)),  # HANGUL SYLLABLE CHIEUCH YI CHIEUCH
    ((0xAF, 0x6B), (0xCE54)),  # HANGUL SYLLABLE CHIEUCH YI KHIEUKH
    ((0xAF, 0x6C), (0xCE55)),  # HANGUL SYLLABLE CHIEUCH YI THIEUTH
    ((0xAF, 0x6D), (0xCE56)),  # HANGUL SYLLABLE CHIEUCH YI PHIEUPH
    ((0xAF, 0x6E), (0xCE57)),  # HANGUL SYLLABLE CHIEUCH YI HIEUH
    ((0xAF, 0x6F), (0xCE5A)),  # HANGUL SYLLABLE CHIEUCH I SSANGKIYEOK
    ((0xAF, 0x70), (0xCE5B)),  # HANGUL SYLLABLE CHIEUCH I KIYEOKSIOS
    ((0xAF, 0x71), (0xCE5D)),  # HANGUL SYLLABLE CHIEUCH I NIEUNCIEUC
    ((0xAF, 0x72), (0xCE5E)),  # HANGUL SYLLABLE CHIEUCH I NIEUNHIEUH
    ((0xAF, 0x73), (0xCE62)),  # HANGUL SYLLABLE CHIEUCH I RIEULMIEUM
    ((0xAF, 0x74), (0xCE63)),  # HANGUL SYLLABLE CHIEUCH I RIEULPIEUP
    ((0xAF, 0x75), (0xCE64)),  # HANGUL SYLLABLE CHIEUCH I RIEULSIOS
    ((0xAF, 0x76), (0xCE65)),  # HANGUL SYLLABLE CHIEUCH I RIEULTHIEUTH
    ((0xAF, 0x77), (0xCE66)),  # HANGUL SYLLABLE CHIEUCH I RIEULPHIEUPH
    ((0xAF, 0x78), (0xCE67)),  # HANGUL SYLLABLE CHIEUCH I RIEULHIEUH
    ((0xAF, 0x79), (0xCE6A)),  # HANGUL SYLLABLE CHIEUCH I PIEUPSIOS
    ((0xAF, 0x7A), (0xCE6C)),  # HANGUL SYLLABLE CHIEUCH I SSANGSIOS
    ((0xAF, 0x81), (0xCE6E)),  # HANGUL SYLLABLE CHIEUCH I CIEUC
    ((0xAF, 0x82), (0xCE6F)),  # HANGUL SYLLABLE CHIEUCH I CHIEUCH
    ((0xAF, 0x83), (0xCE70)),  # HANGUL SYLLABLE CHIEUCH I KHIEUKH
    ((0xAF, 0x84), (0xCE71)),  # HANGUL SYLLABLE CHIEUCH I THIEUTH
    ((0xAF, 0x85), (0xCE72)),  # HANGUL SYLLABLE CHIEUCH I PHIEUPH
    ((0xAF, 0x86), (0xCE73)),  # HANGUL SYLLABLE CHIEUCH I HIEUH
    ((0xAF, 0x87), (0xCE76)),  # HANGUL SYLLABLE KHIEUKH A SSANGKIYEOK
    ((0xAF, 0x88), (0xCE77)),  # HANGUL SYLLABLE KHIEUKH A KIYEOKSIOS
    ((0xAF, 0x89), (0xCE79)),  # HANGUL SYLLABLE KHIEUKH A NIEUNCIEUC
    ((0xAF, 0x8A), (0xCE7A)),  # HANGUL SYLLABLE KHIEUKH A NIEUNHIEUH
    ((0xAF, 0x8B), (0xCE7B)),  # HANGUL SYLLABLE KHIEUKH A TIKEUT
    ((0xAF, 0x8C), (0xCE7D)),  # HANGUL SYLLABLE KHIEUKH A RIEULKIYEOK
    ((0xAF, 0x8D), (0xCE7E)),  # HANGUL SYLLABLE KHIEUKH A RIEULMIEUM
    ((0xAF, 0x8E), (0xCE7F)),  # HANGUL SYLLABLE KHIEUKH A RIEULPIEUP
    ((0xAF, 0x8F), (0xCE80)),  # HANGUL SYLLABLE KHIEUKH A RIEULSIOS
    ((0xAF, 0x90), (0xCE81)),  # HANGUL SYLLABLE KHIEUKH A RIEULTHIEUTH
    ((0xAF, 0x91), (0xCE82)),  # HANGUL SYLLABLE KHIEUKH A RIEULPHIEUPH
    ((0xAF, 0x92), (0xCE83)),  # HANGUL SYLLABLE KHIEUKH A RIEULHIEUH
    ((0xAF, 0x93), (0xCE86)),  # HANGUL SYLLABLE KHIEUKH A PIEUPSIOS
    ((0xAF, 0x94), (0xCE88)),  # HANGUL SYLLABLE KHIEUKH A SSANGSIOS
    ((0xAF, 0x95), (0xCE8A)),  # HANGUL SYLLABLE KHIEUKH A CIEUC
    ((0xAF, 0x96), (0xCE8B)),  # HANGUL SYLLABLE KHIEUKH A CHIEUCH
    ((0xAF, 0x97), (0xCE8C)),  # HANGUL SYLLABLE KHIEUKH A KHIEUKH
    ((0xAF, 0x98), (0xCE8D)),  # HANGUL SYLLABLE KHIEUKH A THIEUTH
    ((0xAF, 0x99), (0xCE8E)),  # HANGUL SYLLABLE KHIEUKH A PHIEUPH
    ((0xAF, 0x9A), (0xCE8F)),  # HANGUL SYLLABLE KHIEUKH A HIEUH
    ((0xAF, 0x9B), (0xCE92)),  # HANGUL SYLLABLE KHIEUKH AE SSANGKIYEOK
    ((0xAF, 0x9C), (0xCE93)),  # HANGUL SYLLABLE KHIEUKH AE KIYEOKSIOS
    ((0xAF, 0x9D), (0xCE95)),  # HANGUL SYLLABLE KHIEUKH AE NIEUNCIEUC
    ((0xAF, 0x9E), (0xCE96)),  # HANGUL SYLLABLE KHIEUKH AE NIEUNHIEUH
    ((0xAF, 0x9F), (0xCE97)),  # HANGUL SYLLABLE KHIEUKH AE TIKEUT
    ((0xAF, 0xA0), (0xCE99)),  # HANGUL SYLLABLE KHIEUKH AE RIEULKIYEOK
    ((0xB0, 0x41), (0xCE9A)),  # HANGUL SYLLABLE KHIEUKH AE RIEULMIEUM
    ((0xB0, 0x42), (0xCE9B)),  # HANGUL SYLLABLE KHIEUKH AE RIEULPIEUP
    ((0xB0, 0x43), (0xCE9C)),  # HANGUL SYLLABLE KHIEUKH AE RIEULSIOS
    ((0xB0, 0x44), (0xCE9D)),  # HANGUL SYLLABLE KHIEUKH AE RIEULTHIEUTH
    ((0xB0, 0x45), (0xCE9E)),  # HANGUL SYLLABLE KHIEUKH AE RIEULPHIEUPH
    ((0xB0, 0x46), (0xCE9F)),  # HANGUL SYLLABLE KHIEUKH AE RIEULHIEUH
    ((0xB0, 0x47), (0xCEA2)),  # HANGUL SYLLABLE KHIEUKH AE PIEUPSIOS
    ((0xB0, 0x48), (0xCEA6)),  # HANGUL SYLLABLE KHIEUKH AE CIEUC
    ((0xB0, 0x49), (0xCEA7)),  # HANGUL SYLLABLE KHIEUKH AE CHIEUCH
    ((0xB0, 0x4A), (0xCEA8)),  # HANGUL SYLLABLE KHIEUKH AE KHIEUKH
    ((0xB0, 0x4B), (0xCEA9)),  # HANGUL SYLLABLE KHIEUKH AE THIEUTH
    ((0xB0, 0x4C), (0xCEAA)),  # HANGUL SYLLABLE KHIEUKH AE PHIEUPH
    ((0xB0, 0x4D), (0xCEAB)),  # HANGUL SYLLABLE KHIEUKH AE HIEUH
    ((0xB0, 0x4E), (0xCEAE)),  # HANGUL SYLLABLE KHIEUKH YA SSANGKIYEOK
    ((0xB0, 0x4F), (0xCEAF)),  # HANGUL SYLLABLE KHIEUKH YA KIYEOKSIOS
    ((0xB0, 0x50), (0xCEB0)),  # HANGUL SYLLABLE KHIEUKH YA NIEUN
    ((0xB0, 0x51), (0xCEB1)),  # HANGUL SYLLABLE KHIEUKH YA NIEUNCIEUC
    ((0xB0, 0x52), (0xCEB2)),  # HANGUL SYLLABLE KHIEUKH YA NIEUNHIEUH
    ((0xB0, 0x53), (0xCEB3)),  # HANGUL SYLLABLE KHIEUKH YA TIKEUT
    ((0xB0, 0x54), (0xCEB4)),  # HANGUL SYLLABLE KHIEUKH YA RIEUL
    ((0xB0, 0x55), (0xCEB5)),  # HANGUL SYLLABLE KHIEUKH YA RIEULKIYEOK
    ((0xB0, 0x56), (0xCEB6)),  # HANGUL SYLLABLE KHIEUKH YA RIEULMIEUM
    ((0xB0, 0x57), (0xCEB7)),  # HANGUL SYLLABLE KHIEUKH YA RIEULPIEUP
    ((0xB0, 0x58), (0xCEB8)),  # HANGUL SYLLABLE KHIEUKH YA RIEULSIOS
    ((0xB0, 0x59), (0xCEB9)),  # HANGUL SYLLABLE KHIEUKH YA RIEULTHIEUTH
    ((0xB0, 0x5A), (0xCEBA)),  # HANGUL SYLLABLE KHIEUKH YA RIEULPHIEUPH
    ((0xB0, 0x61), (0xCEBB)),  # HANGUL SYLLABLE KHIEUKH YA RIEULHIEUH
    ((0xB0, 0x62), (0xCEBC)),  # HANGUL SYLLABLE KHIEUKH YA MIEUM
    ((0xB0, 0x63), (0xCEBD)),  # HANGUL SYLLABLE KHIEUKH YA PIEUP
    ((0xB0, 0x64), (0xCEBE)),  # HANGUL SYLLABLE KHIEUKH YA PIEUPSIOS
    ((0xB0, 0x65), (0xCEBF)),  # HANGUL SYLLABLE KHIEUKH YA SIOS
    ((0xB0, 0x66), (0xCEC0)),  # HANGUL SYLLABLE KHIEUKH YA SSANGSIOS
    ((0xB0, 0x67), (0xCEC2)),  # HANGUL SYLLABLE KHIEUKH YA CIEUC
    ((0xB0, 0x68), (0xCEC3)),  # HANGUL SYLLABLE KHIEUKH YA CHIEUCH
    ((0xB0, 0x69), (0xCEC4)),  # HANGUL SYLLABLE KHIEUKH YA KHIEUKH
    ((0xB0, 0x6A), (0xCEC5)),  # HANGUL SYLLABLE KHIEUKH YA THIEUTH
    ((0xB0, 0x6B), (0xCEC6)),  # HANGUL SYLLABLE KHIEUKH YA PHIEUPH
    ((0xB0, 0x6C), (0xCEC7)),  # HANGUL SYLLABLE KHIEUKH YA HIEUH
    ((0xB0, 0x6D), (0xCEC8)),  # HANGUL SYLLABLE KHIEUKH YAE
    ((0xB0, 0x6E), (0xCEC9)),  # HANGUL SYLLABLE KHIEUKH YAE KIYEOK
    ((0xB0, 0x6F), (0xCECA)),  # HANGUL SYLLABLE KHIEUKH YAE SSANGKIYEOK
    ((0xB0, 0x70), (0xCECB)),  # HANGUL SYLLABLE KHIEUKH YAE KIYEOKSIOS
    ((0xB0, 0x71), (0xCECC)),  # HANGUL SYLLABLE KHIEUKH YAE NIEUN
    ((0xB0, 0x72), (0xCECD)),  # HANGUL SYLLABLE KHIEUKH YAE NIEUNCIEUC
    ((0xB0, 0x73), (0xCECE)),  # HANGUL SYLLABLE KHIEUKH YAE NIEUNHIEUH
    ((0xB0, 0x74), (0xCECF)),  # HANGUL SYLLABLE KHIEUKH YAE TIKEUT
    ((0xB0, 0x75), (0xCED0)),  # HANGUL SYLLABLE KHIEUKH YAE RIEUL
    ((0xB0, 0x76), (0xCED1)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULKIYEOK
    ((0xB0, 0x77), (0xCED2)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULMIEUM
    ((0xB0, 0x78), (0xCED3)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULPIEUP
    ((0xB0, 0x79), (0xCED4)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULSIOS
    ((0xB0, 0x7A), (0xCED5)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULTHIEUTH
    ((0xB0, 0x81), (0xCED6)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULPHIEUPH
    ((0xB0, 0x82), (0xCED7)),  # HANGUL SYLLABLE KHIEUKH YAE RIEULHIEUH
    ((0xB0, 0x83), (0xCED8)),  # HANGUL SYLLABLE KHIEUKH YAE MIEUM
    ((0xB0, 0x84), (0xCED9)),  # HANGUL SYLLABLE KHIEUKH YAE PIEUP
    ((0xB0, 0x85), (0xCEDA)),  # HANGUL SYLLABLE KHIEUKH YAE PIEUPSIOS
    ((0xB0, 0x86), (0xCEDB)),  # HANGUL SYLLABLE KHIEUKH YAE SIOS
    ((0xB0, 0x87), (0xCEDC)),  # HANGUL SYLLABLE KHIEUKH YAE SSANGSIOS
    ((0xB0, 0x88), (0xCEDD)),  # HANGUL SYLLABLE KHIEUKH YAE IEUNG
    ((0xB0, 0x89), (0xCEDE)),  # HANGUL SYLLABLE KHIEUKH YAE CIEUC
    ((0xB0, 0x8A), (0xCEDF)),  # HANGUL SYLLABLE KHIEUKH YAE CHIEUCH
    ((0xB0, 0x8B), (0xCEE0)),  # HANGUL SYLLABLE KHIEUKH YAE KHIEUKH
    ((0xB0, 0x8C), (0xCEE1)),  # HANGUL SYLLABLE KHIEUKH YAE THIEUTH
    ((0xB0, 0x8D), (0xCEE2)),  # HANGUL SYLLABLE KHIEUKH YAE PHIEUPH
    ((0xB0, 0x8E), (0xCEE3)),  # HANGUL SYLLABLE KHIEUKH YAE HIEUH
    ((0xB0, 0x8F), (0xCEE6)),  # HANGUL SYLLABLE KHIEUKH EO SSANGKIYEOK
    ((0xB0, 0x90), (0xCEE7)),  # HANGUL SYLLABLE KHIEUKH EO KIYEOKSIOS
    ((0xB0, 0x91), (0xCEE9)),  # HANGUL SYLLABLE KHIEUKH EO NIEUNCIEUC
    ((0xB0, 0x92), (0xCEEA)),  # HANGUL SYLLABLE KHIEUKH EO NIEUNHIEUH
    ((0xB0, 0x93), (0xCEED)),  # HANGUL SYLLABLE KHIEUKH EO RIEULKIYEOK
    ((0xB0, 0x94), (0xCEEE)),  # HANGUL SYLLABLE KHIEUKH EO RIEULMIEUM
    ((0xB0, 0x95), (0xCEEF)),  # HANGUL SYLLABLE KHIEUKH EO RIEULPIEUP
    ((0xB0, 0x96), (0xCEF0)),  # HANGUL SYLLABLE KHIEUKH EO RIEULSIOS
    ((0xB0, 0x97), (0xCEF1)),  # HANGUL SYLLABLE KHIEUKH EO RIEULTHIEUTH
    ((0xB0, 0x98), (0xCEF2)),  # HANGUL SYLLABLE KHIEUKH EO RIEULPHIEUPH
    ((0xB0, 0x99), (0xCEF3)),  # HANGUL SYLLABLE KHIEUKH EO RIEULHIEUH
    ((0xB0, 0x9A), (0xCEF6)),  # HANGUL SYLLABLE KHIEUKH EO PIEUPSIOS
    ((0xB0, 0x9B), (0xCEFA)),  # HANGUL SYLLABLE KHIEUKH EO CIEUC
    ((0xB0, 0x9C), (0xCEFB)),  # HANGUL SYLLABLE KHIEUKH EO CHIEUCH
    ((0xB0, 0x9D), (0xCEFC)),  # HANGUL SYLLABLE KHIEUKH EO KHIEUKH
    ((0xB0, 0x9E), (0xCEFD)),  # HANGUL SYLLABLE KHIEUKH EO THIEUTH
    ((0xB0, 0x9F), (0xCEFE)),  # HANGUL SYLLABLE KHIEUKH EO PHIEUPH
    ((0xB0, 0xA0), (0xCEFF)),  # HANGUL SYLLABLE KHIEUKH EO HIEUH
    ((0xB0, 0xA1), (0xAC00)),  # HANGUL SYLLABLE KIYEOK A
    ((0xB0, 0xA2), (0xAC01)),  # HANGUL SYLLABLE KIYEOK A KIYEOK
    ((0xB0, 0xA3), (0xAC04)),  # HANGUL SYLLABLE KIYEOK A NIEUN
    ((0xB0, 0xA4), (0xAC07)),  # HANGUL SYLLABLE KIYEOK A TIKEUT
    ((0xB0, 0xA5), (0xAC08)),  # HANGUL SYLLABLE KIYEOK A RIEUL
    ((0xB0, 0xA6), (0xAC09)),  # HANGUL SYLLABLE KIYEOK A RIEULKIYEOK
    ((0xB0, 0xA7), (0xAC0A)),  # HANGUL SYLLABLE KIYEOK A RIEULMIEUM
    ((0xB0, 0xA8), (0xAC10)),  # HANGUL SYLLABLE KIYEOK A MIEUM
    ((0xB0, 0xA9), (0xAC11)),  # HANGUL SYLLABLE KIYEOK A PIEUP
    ((0xB0, 0xAA), (0xAC12)),  # HANGUL SYLLABLE KIYEOK A PIEUPSIOS
    ((0xB0, 0xAB), (0xAC13)),  # HANGUL SYLLABLE KIYEOK A SIOS
    ((0xB0, 0xAC), (0xAC14)),  # HANGUL SYLLABLE KIYEOK A SSANGSIOS
    ((0xB0, 0xAD), (0xAC15)),  # HANGUL SYLLABLE KIYEOK A IEUNG
    ((0xB0, 0xAE), (0xAC16)),  # HANGUL SYLLABLE KIYEOK A CIEUC
    ((0xB0, 0xAF), (0xAC17)),  # HANGUL SYLLABLE KIYEOK A CHIEUCH
    ((0xB0, 0xB0), (0xAC19)),  # HANGUL SYLLABLE KIYEOK A THIEUTH
    ((0xB0, 0xB1), (0xAC1A)),  # HANGUL SYLLABLE KIYEOK A PHIEUPH
    ((0xB0, 0xB2), (0xAC1B)),  # HANGUL SYLLABLE KIYEOK A HIEUH
    ((0xB0, 0xB3), (0xAC1C)),  # HANGUL SYLLABLE KIYEOK AE
    ((0xB0, 0xB4), (0xAC1D)),  # HANGUL SYLLABLE KIYEOK AE KIYEOK
    ((0xB0, 0xB5), (0xAC20)),  # HANGUL SYLLABLE KIYEOK AE NIEUN
    ((0xB0, 0xB6), (0xAC24)),  # HANGUL SYLLABLE KIYEOK AE RIEUL
    ((0xB0, 0xB7), (0xAC2C)),  # HANGUL SYLLABLE KIYEOK AE MIEUM
    ((0xB0, 0xB8), (0xAC2D)),  # HANGUL SYLLABLE KIYEOK AE PIEUP
    ((0xB0, 0xB9), (0xAC2F)),  # HANGUL SYLLABLE KIYEOK AE SIOS
    ((0xB0, 0xBA), (0xAC30)),  # HANGUL SYLLABLE KIYEOK AE SSANGSIOS
    ((0xB0, 0xBB), (0xAC31)),  # HANGUL SYLLABLE KIYEOK AE IEUNG
    ((0xB0, 0xBC), (0xAC38)),  # HANGUL SYLLABLE KIYEOK YA
    ((0xB0, 0xBD), (0xAC39)),  # HANGUL SYLLABLE KIYEOK YA KIYEOK
    ((0xB0, 0xBE), (0xAC3C)),  # HANGUL SYLLABLE KIYEOK YA NIEUN
    ((0xB0, 0xBF), (0xAC40)),  # HANGUL SYLLABLE KIYEOK YA RIEUL
    ((0xB0, 0xC0), (0xAC4B)),  # HANGUL SYLLABLE KIYEOK YA SIOS
    ((0xB0, 0xC1), (0xAC4D)),  # HANGUL SYLLABLE KIYEOK YA IEUNG
    ((0xB0, 0xC2), (0xAC54)),  # HANGUL SYLLABLE KIYEOK YAE
    ((0xB0, 0xC3), (0xAC58)),  # HANGUL SYLLABLE KIYEOK YAE NIEUN
    ((0xB0, 0xC4), (0xAC5C)),  # HANGUL SYLLABLE KIYEOK YAE RIEUL
    ((0xB0, 0xC5), (0xAC70)),  # HANGUL SYLLABLE KIYEOK EO
    ((0xB0, 0xC6), (0xAC71)),  # HANGUL SYLLABLE KIYEOK EO KIYEOK
    ((0xB0, 0xC7), (0xAC74)),  # HANGUL SYLLABLE KIYEOK EO NIEUN
    ((0xB0, 0xC8), (0xAC77)),  # HANGUL SYLLABLE KIYEOK EO TIKEUT
    ((0xB0, 0xC9), (0xAC78)),  # HANGUL SYLLABLE KIYEOK EO RIEUL
    ((0xB0, 0xCA), (0xAC7A)),  # HANGUL SYLLABLE KIYEOK EO RIEULMIEUM
    ((0xB0, 0xCB), (0xAC80)),  # HANGUL SYLLABLE KIYEOK EO MIEUM
    ((0xB0, 0xCC), (0xAC81)),  # HANGUL SYLLABLE KIYEOK EO PIEUP
    ((0xB0, 0xCD), (0xAC83)),  # HANGUL SYLLABLE KIYEOK EO SIOS
    ((0xB0, 0xCE), (0xAC84)),  # HANGUL SYLLABLE KIYEOK EO SSANGSIOS
    ((0xB0, 0xCF), (0xAC85)),  # HANGUL SYLLABLE KIYEOK EO IEUNG
    ((0xB0, 0xD0), (0xAC86)),  # HANGUL SYLLABLE KIYEOK EO CIEUC
    ((0xB0, 0xD1), (0xAC89)),  # HANGUL SYLLABLE KIYEOK EO THIEUTH
    ((0xB0, 0xD2), (0xAC8A)),  # HANGUL SYLLABLE KIYEOK EO PHIEUPH
    ((0xB0, 0xD3), (0xAC8B)),  # HANGUL SYLLABLE KIYEOK EO HIEUH
    ((0xB0, 0xD4), (0xAC8C)),  # HANGUL SYLLABLE KIYEOK E
    ((0xB0, 0xD5), (0xAC90)),  # HANGUL SYLLABLE KIYEOK E NIEUN
    ((0xB0, 0xD6), (0xAC94)),  # HANGUL SYLLABLE KIYEOK E RIEUL
    ((0xB0, 0xD7), (0xAC9C)),  # HANGUL SYLLABLE KIYEOK E MIEUM
    ((0xB0, 0xD8), (0xAC9D)),  # HANGUL SYLLABLE KIYEOK E PIEUP
    ((0xB0, 0xD9), (0xAC9F)),  # HANGUL SYLLABLE KIYEOK E SIOS
    ((0xB0, 0xDA), (0xACA0)),  # HANGUL SYLLABLE KIYEOK E SSANGSIOS
    ((0xB0, 0xDB), (0xACA1)),  # HANGUL SYLLABLE KIYEOK E IEUNG
    ((0xB0, 0xDC), (0xACA8)),  # HANGUL SYLLABLE KIYEOK YEO
    ((0xB0, 0xDD), (0xACA9)),  # HANGUL SYLLABLE KIYEOK YEO KIYEOK
    ((0xB0, 0xDE), (0xACAA)),  # HANGUL SYLLABLE KIYEOK YEO SSANGKIYEOK
    ((0xB0, 0xDF), (0xACAC)),  # HANGUL SYLLABLE KIYEOK YEO NIEUN
    ((0xB0, 0xE0), (0xACAF)),  # HANGUL SYLLABLE KIYEOK YEO TIKEUT
    ((0xB0, 0xE1), (0xACB0)),  # HANGUL SYLLABLE KIYEOK YEO RIEUL
    ((0xB0, 0xE2), (0xACB8)),  # HANGUL SYLLABLE KIYEOK YEO MIEUM
    ((0xB0, 0xE3), (0xACB9)),  # HANGUL SYLLABLE KIYEOK YEO PIEUP
    ((0xB0, 0xE4), (0xACBB)),  # HANGUL SYLLABLE KIYEOK YEO SIOS
    ((0xB0, 0xE5), (0xACBC)),  # HANGUL SYLLABLE KIYEOK YEO SSANGSIOS
    ((0xB0, 0xE6), (0xACBD)),  # HANGUL SYLLABLE KIYEOK YEO IEUNG
    ((0xB0, 0xE7), (0xACC1)),  # HANGUL SYLLABLE KIYEOK YEO THIEUTH
    ((0xB0, 0xE8), (0xACC4)),  # HANGUL SYLLABLE KIYEOK YE
    ((0xB0, 0xE9), (0xACC8)),  # HANGUL SYLLABLE KIYEOK YE NIEUN
    ((0xB0, 0xEA), (0xACCC)),  # HANGUL SYLLABLE KIYEOK YE RIEUL
    ((0xB0, 0xEB), (0xACD5)),  # HANGUL SYLLABLE KIYEOK YE PIEUP
    ((0xB0, 0xEC), (0xACD7)),  # HANGUL SYLLABLE KIYEOK YE SIOS
    ((0xB0, 0xED), (0xACE0)),  # HANGUL SYLLABLE KIYEOK O
    ((0xB0, 0xEE), (0xACE1)),  # HANGUL SYLLABLE KIYEOK O KIYEOK
    ((0xB0, 0xEF), (0xACE4)),  # HANGUL SYLLABLE KIYEOK O NIEUN
    ((0xB0, 0xF0), (0xACE7)),  # HANGUL SYLLABLE KIYEOK O TIKEUT
    ((0xB0, 0xF1), (0xACE8)),  # HANGUL SYLLABLE KIYEOK O RIEUL
    ((0xB0, 0xF2), (0xACEA)),  # HANGUL SYLLABLE KIYEOK O RIEULMIEUM
    ((0xB0, 0xF3), (0xACEC)),  # HANGUL SYLLABLE KIYEOK O RIEULSIOS
    ((0xB0, 0xF4), (0xACEF)),  # HANGUL SYLLABLE KIYEOK O RIEULHIEUH
    ((0xB0, 0xF5), (0xACF0)),  # HANGUL SYLLABLE KIYEOK O MIEUM
    ((0xB0, 0xF6), (0xACF1)),  # HANGUL SYLLABLE KIYEOK O PIEUP
    ((0xB0, 0xF7), (0xACF3)),  # HANGUL SYLLABLE KIYEOK O SIOS
    ((0xB0, 0xF8), (0xACF5)),  # HANGUL SYLLABLE KIYEOK O IEUNG
    ((0xB0, 0xF9), (0xACF6)),  # HANGUL SYLLABLE KIYEOK O CIEUC
    ((0xB0, 0xFA), (0xACFC)),  # HANGUL SYLLABLE KIYEOK WA
    ((0xB0, 0xFB), (0xACFD)),  # HANGUL SYLLABLE KIYEOK WA KIYEOK
    ((0xB0, 0xFC), (0xAD00)),  # HANGUL SYLLABLE KIYEOK WA NIEUN
    ((0xB0, 0xFD), (0xAD04)),  # HANGUL SYLLABLE KIYEOK WA RIEUL
    ((0xB0, 0xFE), (0xAD06)),  # HANGUL SYLLABLE KIYEOK WA RIEULMIEUM
    ((0xB1, 0x41), (0xCF02)),  # HANGUL SYLLABLE KHIEUKH E SSANGKIYEOK
    ((0xB1, 0x42), (0xCF03)),  # HANGUL SYLLABLE KHIEUKH E KIYEOKSIOS
    ((0xB1, 0x43), (0xCF05)),  # HANGUL SYLLABLE KHIEUKH E NIEUNCIEUC
    ((0xB1, 0x44), (0xCF06)),  # HANGUL SYLLABLE KHIEUKH E NIEUNHIEUH
    ((0xB1, 0x45), (0xCF07)),  # HANGUL SYLLABLE KHIEUKH E TIKEUT
    ((0xB1, 0x46), (0xCF09)),  # HANGUL SYLLABLE KHIEUKH E RIEULKIYEOK
    ((0xB1, 0x47), (0xCF0A)),  # HANGUL SYLLABLE KHIEUKH E RIEULMIEUM
    ((0xB1, 0x48), (0xCF0B)),  # HANGUL SYLLABLE KHIEUKH E RIEULPIEUP
    ((0xB1, 0x49), (0xCF0C)),  # HANGUL SYLLABLE KHIEUKH E RIEULSIOS
    ((0xB1, 0x4A), (0xCF0D)),  # HANGUL SYLLABLE KHIEUKH E RIEULTHIEUTH
    ((0xB1, 0x4B), (0xCF0E)),  # HANGUL SYLLABLE KHIEUKH E RIEULPHIEUPH
    ((0xB1, 0x4C), (0xCF0F)),  # HANGUL SYLLABLE KHIEUKH E RIEULHIEUH
    ((0xB1, 0x4D), (0xCF12)),  # HANGUL SYLLABLE KHIEUKH E PIEUPSIOS
    ((0xB1, 0x4E), (0xCF14)),  # HANGUL SYLLABLE KHIEUKH E SSANGSIOS
    ((0xB1, 0x4F), (0xCF16)),  # HANGUL SYLLABLE KHIEUKH E CIEUC
    ((0xB1, 0x50), (0xCF17)),  # HANGUL SYLLABLE KHIEUKH E CHIEUCH
    ((0xB1, 0x51), (0xCF18)),  # HANGUL SYLLABLE KHIEUKH E KHIEUKH
    ((0xB1, 0x52), (0xCF19)),  # HANGUL SYLLABLE KHIEUKH E THIEUTH
    ((0xB1, 0x53), (0xCF1A)),  # HANGUL SYLLABLE KHIEUKH E PHIEUPH
    ((0xB1, 0x54), (0xCF1B)),  # HANGUL SYLLABLE KHIEUKH E HIEUH
    ((0xB1, 0x55), (0xCF1D)),  # HANGUL SYLLABLE KHIEUKH YEO KIYEOK
    ((0xB1, 0x56), (0xCF1E)),  # HANGUL SYLLABLE KHIEUKH YEO SSANGKIYEOK
    ((0xB1, 0x57), (0xCF1F)),  # HANGUL SYLLABLE KHIEUKH YEO KIYEOKSIOS
    ((0xB1, 0x58), (0xCF21)),  # HANGUL SYLLABLE KHIEUKH YEO NIEUNCIEUC
    ((0xB1, 0x59), (0xCF22)),  # HANGUL SYLLABLE KHIEUKH YEO NIEUNHIEUH
    ((0xB1, 0x5A), (0xCF23)),  # HANGUL SYLLABLE KHIEUKH YEO TIKEUT
    ((0xB1, 0x61), (0xCF25)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULKIYEOK
    ((0xB1, 0x62), (0xCF26)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULMIEUM
    ((0xB1, 0x63), (0xCF27)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULPIEUP
    ((0xB1, 0x64), (0xCF28)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULSIOS
    ((0xB1, 0x65), (0xCF29)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULTHIEUTH
    ((0xB1, 0x66), (0xCF2A)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULPHIEUPH
    ((0xB1, 0x67), (0xCF2B)),  # HANGUL SYLLABLE KHIEUKH YEO RIEULHIEUH
    ((0xB1, 0x68), (0xCF2E)),  # HANGUL SYLLABLE KHIEUKH YEO PIEUPSIOS
    ((0xB1, 0x69), (0xCF32)),  # HANGUL SYLLABLE KHIEUKH YEO CIEUC
    ((0xB1, 0x6A), (0xCF33)),  # HANGUL SYLLABLE KHIEUKH YEO CHIEUCH
    ((0xB1, 0x6B), (0xCF34)),  # HANGUL SYLLABLE KHIEUKH YEO KHIEUKH
    ((0xB1, 0x6C), (0xCF35)),  # HANGUL SYLLABLE KHIEUKH YEO THIEUTH
    ((0xB1, 0x6D), (0xCF36)),  # HANGUL SYLLABLE KHIEUKH YEO PHIEUPH
    ((0xB1, 0x6E), (0xCF37)),  # HANGUL SYLLABLE KHIEUKH YEO HIEUH
    ((0xB1, 0x6F), (0xCF39)),  # HANGUL SYLLABLE KHIEUKH YE KIYEOK
    ((0xB1, 0x70), (0xCF3A)),  # HANGUL SYLLABLE KHIEUKH YE SSANGKIYEOK
    ((0xB1, 0x71), (0xCF3B)),  # HANGUL SYLLABLE KHIEUKH YE KIYEOKSIOS
    ((0xB1, 0x72), (0xCF3C)),  # HANGUL SYLLABLE KHIEUKH YE NIEUN
    ((0xB1, 0x73), (0xCF3D)),  # HANGUL SYLLABLE KHIEUKH YE NIEUNCIEUC
    ((0xB1, 0x74), (0xCF3E)),  # HANGUL SYLLABLE KHIEUKH YE NIEUNHIEUH
    ((0xB1, 0x75), (0xCF3F)),  # HANGUL SYLLABLE KHIEUKH YE TIKEUT
    ((0xB1, 0x76), (0xCF40)),  # HANGUL SYLLABLE KHIEUKH YE RIEUL
    ((0xB1, 0x77), (0xCF41)),  # HANGUL SYLLABLE KHIEUKH YE RIEULKIYEOK
    ((0xB1, 0x78), (0xCF42)),  # HANGUL SYLLABLE KHIEUKH YE RIEULMIEUM
    ((0xB1, 0x79), (0xCF43)),  # HANGUL SYLLABLE KHIEUKH YE RIEULPIEUP
    ((0xB1, 0x7A), (0xCF44)),  # HANGUL SYLLABLE KHIEUKH YE RIEULSIOS
    ((0xB1, 0x81), (0xCF45)),  # HANGUL SYLLABLE KHIEUKH YE RIEULTHIEUTH
    ((0xB1, 0x82), (0xCF46)),  # HANGUL SYLLABLE KHIEUKH YE RIEULPHIEUPH
    ((0xB1, 0x83), (0xCF47)),  # HANGUL SYLLABLE KHIEUKH YE RIEULHIEUH
    ((0xB1, 0x84), (0xCF48)),  # HANGUL SYLLABLE KHIEUKH YE MIEUM
    ((0xB1, 0x85), (0xCF49)),  # HANGUL SYLLABLE KHIEUKH YE PIEUP
    ((0xB1, 0x86), (0xCF4A)),  # HANGUL SYLLABLE KHIEUKH YE PIEUPSIOS
    ((0xB1, 0x87), (0xCF4B)),  # HANGUL SYLLABLE KHIEUKH YE SIOS
    ((0xB1, 0x88), (0xCF4C)),  # HANGUL SYLLABLE KHIEUKH YE SSANGSIOS
    ((0xB1, 0x89), (0xCF4D)),  # HANGUL SYLLABLE KHIEUKH YE IEUNG
    ((0xB1, 0x8A), (0xCF4E)),  # HANGUL SYLLABLE KHIEUKH YE CIEUC
    ((0xB1, 0x8B), (0xCF4F)),  # HANGUL SYLLABLE KHIEUKH YE CHIEUCH
    ((0xB1, 0x8C), (0xCF50)),  # HANGUL SYLLABLE KHIEUKH YE KHIEUKH
    ((0xB1, 0x8D), (0xCF51)),  # HANGUL SYLLABLE KHIEUKH YE THIEUTH
    ((0xB1, 0x8E), (0xCF52)),  # HANGUL SYLLABLE KHIEUKH YE PHIEUPH
    ((0xB1, 0x8F), (0xCF53)),  # HANGUL SYLLABLE KHIEUKH YE HIEUH
    ((0xB1, 0x90), (0xCF56)),  # HANGUL SYLLABLE KHIEUKH O SSANGKIYEOK
    ((0xB1, 0x91), (0xCF57)),  # HANGUL SYLLABLE KHIEUKH O KIYEOKSIOS
    ((0xB1, 0x92), (0xCF59)),  # HANGUL SYLLABLE KHIEUKH O NIEUNCIEUC
    ((0xB1, 0x93), (0xCF5A)),  # HANGUL SYLLABLE KHIEUKH O NIEUNHIEUH
    ((0xB1, 0x94), (0xCF5B)),  # HANGUL SYLLABLE KHIEUKH O TIKEUT
    ((0xB1, 0x95), (0xCF5D)),  # HANGUL SYLLABLE KHIEUKH O RIEULKIYEOK
    ((0xB1, 0x96), (0xCF5E)),  # HANGUL SYLLABLE KHIEUKH O RIEULMIEUM
    ((0xB1, 0x97), (0xCF5F)),  # HANGUL SYLLABLE KHIEUKH O RIEULPIEUP
    ((0xB1, 0x98), (0xCF60)),  # HANGUL SYLLABLE KHIEUKH O RIEULSIOS
    ((0xB1, 0x99), (0xCF61)),  # HANGUL SYLLABLE KHIEUKH O RIEULTHIEUTH
    ((0xB1, 0x9A), (0xCF62)),  # HANGUL SYLLABLE KHIEUKH O RIEULPHIEUPH
    ((0xB1, 0x9B), (0xCF63)),  # HANGUL SYLLABLE KHIEUKH O RIEULHIEUH
    ((0xB1, 0x9C), (0xCF66)),  # HANGUL SYLLABLE KHIEUKH O PIEUPSIOS
    ((0xB1, 0x9D), (0xCF68)),  # HANGUL SYLLABLE KHIEUKH O SSANGSIOS
    ((0xB1, 0x9E), (0xCF6A)),  # HANGUL SYLLABLE KHIEUKH O CIEUC
    ((0xB1, 0x9F), (0xCF6B)),  # HANGUL SYLLABLE KHIEUKH O CHIEUCH
    ((0xB1, 0xA0), (0xCF6C)),  # HANGUL SYLLABLE KHIEUKH O KHIEUKH
    ((0xB1, 0xA1), (0xAD0C)),  # HANGUL SYLLABLE KIYEOK WA MIEUM
    ((0xB1, 0xA2), (0xAD0D)),  # HANGUL SYLLABLE KIYEOK WA PIEUP
    ((0xB1, 0xA3), (0xAD0F)),  # HANGUL SYLLABLE KIYEOK WA SIOS
    ((0xB1, 0xA4), (0xAD11)),  # HANGUL SYLLABLE KIYEOK WA IEUNG
    ((0xB1, 0xA5), (0xAD18)),  # HANGUL SYLLABLE KIYEOK WAE
    ((0xB1, 0xA6), (0xAD1C)),  # HANGUL SYLLABLE KIYEOK WAE NIEUN
    ((0xB1, 0xA7), (0xAD20)),  # HANGUL SYLLABLE KIYEOK WAE RIEUL
    ((0xB1, 0xA8), (0xAD29)),  # HANGUL SYLLABLE KIYEOK WAE PIEUP
    ((0xB1, 0xA9), (0xAD2C)),  # HANGUL SYLLABLE KIYEOK WAE SSANGSIOS
    ((0xB1, 0xAA), (0xAD2D)),  # HANGUL SYLLABLE KIYEOK WAE IEUNG
    ((0xB1, 0xAB), (0xAD34)),  # HANGUL SYLLABLE KIYEOK OE
    ((0xB1, 0xAC), (0xAD35)),  # HANGUL SYLLABLE KIYEOK OE KIYEOK
    ((0xB1, 0xAD), (0xAD38)),  # HANGUL SYLLABLE KIYEOK OE NIEUN
    ((0xB1, 0xAE), (0xAD3C)),  # HANGUL SYLLABLE KIYEOK OE RIEUL
    ((0xB1, 0xAF), (0xAD44)),  # HANGUL SYLLABLE KIYEOK OE MIEUM
    ((0xB1, 0xB0), (0xAD45)),  # HANGUL SYLLABLE KIYEOK OE PIEUP
    ((0xB1, 0xB1), (0xAD47)),  # HANGUL SYLLABLE KIYEOK OE SIOS
    ((0xB1, 0xB2), (0xAD49)),  # HANGUL SYLLABLE KIYEOK OE IEUNG
    ((0xB1, 0xB3), (0xAD50)),  # HANGUL SYLLABLE KIYEOK YO
    ((0xB1, 0xB4), (0xAD54)),  # HANGUL SYLLABLE KIYEOK YO NIEUN
    ((0xB1, 0xB5), (0xAD58)),  # HANGUL SYLLABLE KIYEOK YO RIEUL
    ((0xB1, 0xB6), (0xAD61)),  # HANGUL SYLLABLE KIYEOK YO PIEUP
    ((0xB1, 0xB7), (0xAD63)),  # HANGUL SYLLABLE KIYEOK YO SIOS
    ((0xB1, 0xB8), (0xAD6C)),  # HANGUL SYLLABLE KIYEOK U
    ((0xB1, 0xB9), (0xAD6D)),  # HANGUL SYLLABLE KIYEOK U KIYEOK
    ((0xB1, 0xBA), (0xAD70)),  # HANGUL SYLLABLE KIYEOK U NIEUN
    ((0xB1, 0xBB), (0xAD73)),  # HANGUL SYLLABLE KIYEOK U TIKEUT
    ((0xB1, 0xBC), (0xAD74)),  # HANGUL SYLLABLE KIYEOK U RIEUL
    ((0xB1, 0xBD), (0xAD75)),  # HANGUL SYLLABLE KIYEOK U RIEULKIYEOK
    ((0xB1, 0xBE), (0xAD76)),  # HANGUL SYLLABLE KIYEOK U RIEULMIEUM
    ((0xB1, 0xBF), (0xAD7B)),  # HANGUL SYLLABLE KIYEOK U RIEULHIEUH
    ((0xB1, 0xC0), (0xAD7C)),  # HANGUL SYLLABLE KIYEOK U MIEUM
    ((0xB1, 0xC1), (0xAD7D)),  # HANGUL SYLLABLE KIYEOK U PIEUP
    ((0xB1, 0xC2), (0xAD7F)),  # HANGUL SYLLABLE KIYEOK U SIOS
    ((0xB1, 0xC3), (0xAD81)),  # HANGUL SYLLABLE KIYEOK U IEUNG
    ((0xB1, 0xC4), (0xAD82)),  # HANGUL SYLLABLE KIYEOK U CIEUC
    ((0xB1, 0xC5), (0xAD88)),  # HANGUL SYLLABLE KIYEOK WEO
    ((0xB1, 0xC6), (0xAD89)),  # HANGUL SYLLABLE KIYEOK WEO KIYEOK
    ((0xB1, 0xC7), (0xAD8C)),  # HANGUL SYLLABLE KIYEOK WEO NIEUN
    ((0xB1, 0xC8), (0xAD90)),  # HANGUL SYLLABLE KIYEOK WEO RIEUL
    ((0xB1, 0xC9), (0xAD9C)),  # HANGUL SYLLABLE KIYEOK WEO SSANGSIOS
    ((0xB1, 0xCA), (0xAD9D)),  # HANGUL SYLLABLE KIYEOK WEO IEUNG
    ((0xB1, 0xCB), (0xADA4)),  # HANGUL SYLLABLE KIYEOK WE
    ((0xB1, 0xCC), (0xADB7)),  # HANGUL SYLLABLE KIYEOK WE SIOS
    ((0xB1, 0xCD), (0xADC0)),  # HANGUL SYLLABLE KIYEOK WI
    ((0xB1, 0xCE), (0xADC1)),  # HANGUL SYLLABLE KIYEOK WI KIYEOK
    ((0xB1, 0xCF), (0xADC4)),  # HANGUL SYLLABLE KIYEOK WI NIEUN
    ((0xB1, 0xD0), (0xADC8)),  # HANGUL SYLLABLE KIYEOK WI RIEUL
    ((0xB1, 0xD1), (0xADD0)),  # HANGUL SYLLABLE KIYEOK WI MIEUM
    ((0xB1, 0xD2), (0xADD1)),  # HANGUL SYLLABLE KIYEOK WI PIEUP
    ((0xB1, 0xD3), (0xADD3)),  # HANGUL SYLLABLE KIYEOK WI SIOS
    ((0xB1, 0xD4), (0xADDC)),  # HANGUL SYLLABLE KIYEOK YU
    ((0xB1, 0xD5), (0xADE0)),  # HANGUL SYLLABLE KIYEOK YU NIEUN
    ((0xB1, 0xD6), (0xADE4)),  # HANGUL SYLLABLE KIYEOK YU RIEUL
    ((0xB1, 0xD7), (0xADF8)),  # HANGUL SYLLABLE KIYEOK EU
    ((0xB1, 0xD8), (0xADF9)),  # HANGUL SYLLABLE KIYEOK EU KIYEOK
    ((0xB1, 0xD9), (0xADFC)),  # HANGUL SYLLABLE KIYEOK EU NIEUN
    ((0xB1, 0xDA), (0xADFF)),  # HANGUL SYLLABLE KIYEOK EU TIKEUT
    ((0xB1, 0xDB), (0xAE00)),  # HANGUL SYLLABLE KIYEOK EU RIEUL
    ((0xB1, 0xDC), (0xAE01)),  # HANGUL SYLLABLE KIYEOK EU RIEULKIYEOK
    ((0xB1, 0xDD), (0xAE08)),  # HANGUL SYLLABLE KIYEOK EU MIEUM
    ((0xB1, 0xDE), (0xAE09)),  # HANGUL SYLLABLE KIYEOK EU PIEUP
    ((0xB1, 0xDF), (0xAE0B)),  # HANGUL SYLLABLE KIYEOK EU SIOS
    ((0xB1, 0xE0), (0xAE0D)),  # HANGUL SYLLABLE KIYEOK EU IEUNG
    ((0xB1, 0xE1), (0xAE14)),  # HANGUL SYLLABLE KIYEOK YI
    ((0xB1, 0xE2), (0xAE30)),  # HANGUL SYLLABLE KIYEOK I
    ((0xB1, 0xE3), (0xAE31)),  # HANGUL SYLLABLE KIYEOK I KIYEOK
    ((0xB1, 0xE4), (0xAE34)),  # HANGUL SYLLABLE KIYEOK I NIEUN
    ((0xB1, 0xE5), (0xAE37)),  # HANGUL SYLLABLE KIYEOK I TIKEUT
    ((0xB1, 0xE6), (0xAE38)),  # HANGUL SYLLABLE KIYEOK I RIEUL
    ((0xB1, 0xE7), (0xAE3A)),  # HANGUL SYLLABLE KIYEOK I RIEULMIEUM
    ((0xB1, 0xE8), (0xAE40)),  # HANGUL SYLLABLE KIYEOK I MIEUM
    ((0xB1, 0xE9), (0xAE41)),  # HANGUL SYLLABLE KIYEOK I PIEUP
    ((0xB1, 0xEA), (0xAE43)),  # HANGUL SYLLABLE KIYEOK I SIOS
    ((0xB1, 0xEB), (0xAE45)),  # HANGUL SYLLABLE KIYEOK I IEUNG
    ((0xB1, 0xEC), (0xAE46)),  # HANGUL SYLLABLE KIYEOK I CIEUC
    ((0xB1, 0xED), (0xAE4A)),  # HANGUL SYLLABLE KIYEOK I PHIEUPH
    ((0xB1, 0xEE), (0xAE4C)),  # HANGUL SYLLABLE SSANGKIYEOK A
    ((0xB1, 0xEF), (0xAE4D)),  # HANGUL SYLLABLE SSANGKIYEOK A KIYEOK
    ((0xB1, 0xF0), (0xAE4E)),  # HANGUL SYLLABLE SSANGKIYEOK A SSANGKIYEOK
    ((0xB1, 0xF1), (0xAE50)),  # HANGUL SYLLABLE SSANGKIYEOK A NIEUN
    ((0xB1, 0xF2), (0xAE54)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEUL
    ((0xB1, 0xF3), (0xAE56)),  # HANGUL SYLLABLE SSANGKIYEOK A RIEULMIEUM
    ((0xB1, 0xF4), (0xAE5C)),  # HANGUL SYLLABLE SSANGKIYEOK A MIEUM
    ((0xB1, 0xF5), (0xAE5D)),  # HANGUL SYLLABLE SSANGKIYEOK A PIEUP
    ((0xB1, 0xF6), (0xAE5F)),  # HANGUL SYLLABLE SSANGKIYEOK A SIOS
    ((0xB1, 0xF7), (0xAE60)),  # HANGUL SYLLABLE SSANGKIYEOK A SSANGSIOS
    ((0xB1, 0xF8), (0xAE61)),  # HANGUL SYLLABLE SSANGKIYEOK A IEUNG
    ((0xB1, 0xF9), (0xAE65)),  # HANGUL SYLLABLE SSANGKIYEOK A THIEUTH
    ((0xB1, 0xFA), (0xAE68)),  # HANGUL SYLLABLE SSANGKIYEOK AE
    ((0xB1, 0xFB), (0xAE69)),  # HANGUL SYLLABLE SSANGKIYEOK AE KIYEOK
    ((0xB1, 0xFC), (0xAE6C)),  # HANGUL SYLLABLE SSANGKIYEOK AE NIEUN
    ((0xB1, 0xFD), (0xAE70)),  # HANGUL SYLLABLE SSANGKIYEOK AE RIEUL
    ((0xB1, 0xFE), (0xAE78)),  # HANGUL SYLLABLE SSANGKIYEOK AE MIEUM
    ((0xB2, 0x41), (0xCF6D)),  # HANGUL SYLLABLE KHIEUKH O THIEUTH
    ((0xB2, 0x42), (0xCF6E)),  # HANGUL SYLLABLE KHIEUKH O PHIEUPH
    ((0xB2, 0x43), (0xCF6F)),  # HANGUL SYLLABLE KHIEUKH O HIEUH
    ((0xB2, 0x44), (0xCF72)),  # HANGUL SYLLABLE KHIEUKH WA SSANGKIYEOK
    ((0xB2, 0x45), (0xCF73)),  # HANGUL SYLLABLE KHIEUKH WA KIYEOKSIOS
    ((0xB2, 0x46), (0xCF75)),  # HANGUL SYLLABLE KHIEUKH WA NIEUNCIEUC
    ((0xB2, 0x47), (0xCF76)),  # HANGUL SYLLABLE KHIEUKH WA NIEUNHIEUH
    ((0xB2, 0x48), (0xCF77)),  # HANGUL SYLLABLE KHIEUKH WA TIKEUT
    ((0xB2, 0x49), (0xCF79)),  # HANGUL SYLLABLE KHIEUKH WA RIEULKIYEOK
    ((0xB2, 0x4A), (0xCF7A)),  # HANGUL SYLLABLE KHIEUKH WA RIEULMIEUM
    ((0xB2, 0x4B), (0xCF7B)),  # HANGUL SYLLABLE KHIEUKH WA RIEULPIEUP
    ((0xB2, 0x4C), (0xCF7C)),  # HANGUL SYLLABLE KHIEUKH WA RIEULSIOS
    ((0xB2, 0x4D), (0xCF7D)),  # HANGUL SYLLABLE KHIEUKH WA RIEULTHIEUTH
    ((0xB2, 0x4E), (0xCF7E)),  # HANGUL SYLLABLE KHIEUKH WA RIEULPHIEUPH
    ((0xB2, 0x4F), (0xCF7F)),  # HANGUL SYLLABLE KHIEUKH WA RIEULHIEUH
    ((0xB2, 0x50), (0xCF81)),  # HANGUL SYLLABLE KHIEUKH WA PIEUP
    ((0xB2, 0x51), (0xCF82)),  # HANGUL SYLLABLE KHIEUKH WA PIEUPSIOS
    ((0xB2, 0x52), (0xCF83)),  # HANGUL SYLLABLE KHIEUKH WA SIOS
    ((0xB2, 0x53), (0xCF84)),  # HANGUL SYLLABLE KHIEUKH WA SSANGSIOS
    ((0xB2, 0x54), (0xCF86)),  # HANGUL SYLLABLE KHIEUKH WA CIEUC
    ((0xB2, 0x55), (0xCF87)),  # HANGUL SYLLABLE KHIEUKH WA CHIEUCH
    ((0xB2, 0x56), (0xCF88)),  # HANGUL SYLLABLE KHIEUKH WA KHIEUKH
    ((0xB2, 0x57), (0xCF89)),  # HANGUL SYLLABLE KHIEUKH WA THIEUTH
    ((0xB2, 0x58), (0xCF8A)),  # HANGUL SYLLABLE KHIEUKH WA PHIEUPH
    ((0xB2, 0x59), (0xCF8B)),  # HANGUL SYLLABLE KHIEUKH WA HIEUH
    ((0xB2, 0x5A), (0xCF8D)),  # HANGUL SYLLABLE KHIEUKH WAE KIYEOK
    ((0xB2, 0x61), (0xCF8E)),  # HANGUL SYLLABLE KHIEUKH WAE SSANGKIYEOK
    ((0xB2, 0x62), (0xCF8F)),  # HANGUL SYLLABLE KHIEUKH WAE KIYEOKSIOS
    ((0xB2, 0x63), (0xCF90)),  # HANGUL SYLLABLE KHIEUKH WAE NIEUN
    ((0xB2, 0x64), (0xCF91)),  # HANGUL SYLLABLE KHIEUKH WAE NIEUNCIEUC
    ((0xB2, 0x65), (0xCF92)),  # HANGUL SYLLABLE KHIEUKH WAE NIEUNHIEUH
    ((0xB2, 0x66), (0xCF93)),  # HANGUL SYLLABLE KHIEUKH WAE TIKEUT
    ((0xB2, 0x67), (0xCF94)),  # HANGUL SYLLABLE KHIEUKH WAE RIEUL
    ((0xB2, 0x68), (0xCF95)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULKIYEOK
    ((0xB2, 0x69), (0xCF96)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULMIEUM
    ((0xB2, 0x6A), (0xCF97)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULPIEUP
    ((0xB2, 0x6B), (0xCF98)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULSIOS
    ((0xB2, 0x6C), (0xCF99)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULTHIEUTH
    ((0xB2, 0x6D), (0xCF9A)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULPHIEUPH
    ((0xB2, 0x6E), (0xCF9B)),  # HANGUL SYLLABLE KHIEUKH WAE RIEULHIEUH
    ((0xB2, 0x6F), (0xCF9C)),  # HANGUL SYLLABLE KHIEUKH WAE MIEUM
    ((0xB2, 0x70), (0xCF9D)),  # HANGUL SYLLABLE KHIEUKH WAE PIEUP
    ((0xB2, 0x71), (0xCF9E)),  # HANGUL SYLLABLE KHIEUKH WAE PIEUPSIOS
    ((0xB2, 0x72), (0xCF9F)),  # HANGUL SYLLABLE KHIEUKH WAE SIOS
    ((0xB2, 0x73), (0xCFA0)),  # HANGUL SYLLABLE KHIEUKH WAE SSANGSIOS
    ((0xB2, 0x74), (0xCFA2)),  # HANGUL SYLLABLE KHIEUKH WAE CIEUC
    ((0xB2, 0x75), (0xCFA3)),  # HANGUL SYLLABLE KHIEUKH WAE CHIEUCH
    ((0xB2, 0x76), (0xCFA4)),  # HANGUL SYLLABLE KHIEUKH WAE KHIEUKH
    ((0xB2, 0x77), (0xCFA5)),  # HANGUL SYLLABLE KHIEUKH WAE THIEUTH
    ((0xB2, 0x78), (0xCFA6)),  # HANGUL SYLLABLE KHIEUKH WAE PHIEUPH
    ((0xB2, 0x79), (0xCFA7)),  # HANGUL SYLLABLE KHIEUKH WAE HIEUH
    ((0xB2, 0x7A), (0xCFA9)),  # HANGUL SYLLABLE KHIEUKH OE KIYEOK
    ((0xB2, 0x81), (0xCFAA)),  # HANGUL SYLLABLE KHIEUKH OE SSANGKIYEOK
    ((0xB2, 0x82), (0xCFAB)),  # HANGUL SYLLABLE KHIEUKH OE KIYEOKSIOS
    ((0xB2, 0x83), (0xCFAC)),  # HANGUL SYLLABLE KHIEUKH OE NIEUN
    ((0xB2, 0x84), (0xCFAD)),  # HANGUL SYLLABLE KHIEUKH OE NIEUNCIEUC
    ((0xB2, 0x85), (0xCFAE)),  # HANGUL SYLLABLE KHIEUKH OE NIEUNHIEUH
    ((0xB2, 0x86), (0xCFAF)),  # HANGUL SYLLABLE KHIEUKH OE TIKEUT
    ((0xB2, 0x87), (0xCFB1)),  # HANGUL SYLLABLE KHIEUKH OE RIEULKIYEOK
    ((0xB2, 0x88), (0xCFB2)),  # HANGUL SYLLABLE KHIEUKH OE RIEULMIEUM
    ((0xB2, 0x89), (0xCFB3)),  # HANGUL SYLLABLE KHIEUKH OE RIEULPIEUP
    ((0xB2, 0x8A), (0xCFB4)),  # HANGUL SYLLABLE KHIEUKH OE RIEULSIOS
    ((0xB2, 0x8B), (0xCFB5)),  # HANGUL SYLLABLE KHIEUKH OE RIEULTHIEUTH
    ((0xB2, 0x8C), (0xCFB6)),  # HANGUL SYLLABLE KHIEUKH OE RIEULPHIEUPH
    ((0xB2, 0x8D), (0xCFB7)),  # HANGUL SYLLABLE KHIEUKH OE RIEULHIEUH
    ((0xB2, 0x8E), (0xCFB8)),  # HANGUL SYLLABLE KHIEUKH OE MIEUM
    ((0xB2, 0x8F), (0xCFB9)),  # HANGUL SYLLABLE KHIEUKH OE PIEUP
    ((0xB2, 0x90), (0xCFBA)),  # HANGUL SYLLABLE KHIEUKH OE PIEUPSIOS
    ((0xB2, 0x91), (0xCFBB)),  # HANGUL SYLLABLE KHIEUKH OE SIOS
    ((0xB2, 0x92), (0xCFBC)),  # HANGUL SYLLABLE KHIEUKH OE SSANGSIOS
    ((0xB2, 0x93), (0xCFBD)),  # HANGUL SYLLABLE KHIEUKH OE IEUNG
    ((0xB2, 0x94), (0xCFBE)),  # HANGUL SYLLABLE KHIEUKH OE CIEUC
    ((0xB2, 0x95), (0xCFBF)),  # HANGUL SYLLABLE KHIEUKH OE CHIEUCH
    ((0xB2, 0x96), (0xCFC0)),  # HANGUL SYLLABLE KHIEUKH OE KHIEUKH
    ((0xB2, 0x97), (0xCFC1)),  # HANGUL SYLLABLE KHIEUKH OE THIEUTH
    ((0xB2, 0x98), (0xCFC2)),  # HANGUL SYLLABLE KHIEUKH OE PHIEUPH
    ((0xB2, 0x99), (0xCFC3)),  # HANGUL SYLLABLE KHIEUKH OE HIEUH
    ((0xB2, 0x9A), (0xCFC5)),  # HANGUL SYLLABLE KHIEUKH YO KIYEOK
    ((0xB2, 0x9B), (0xCFC6)),  # HANGUL SYLLABLE KHIEUKH YO SSANGKIYEOK
    ((0xB2, 0x9C), (0xCFC7)),  # HANGUL SYLLABLE KHIEUKH YO KIYEOKSIOS
    ((0xB2, 0x9D), (0xCFC8)),  # HANGUL SYLLABLE KHIEUKH YO NIEUN
    ((0xB2, 0x9E), (0xCFC9)),  # HANGUL SYLLABLE KHIEUKH YO NIEUNCIEUC
    ((0xB2, 0x9F), (0xCFCA)),  # HANGUL SYLLABLE KHIEUKH YO NIEUNHIEUH
    ((0xB2, 0xA0), (0xCFCB)),  # HANGUL SYLLABLE KHIEUKH YO TIKEUT
    ((0xB2, 0xA1), (0xAE79)),  # HANGUL SYLLABLE SSANGKIYEOK AE PIEUP
    ((0xB2, 0xA2), (0xAE7B)),  # HANGUL SYLLABLE SSANGKIYEOK AE SIOS
    ((0xB2, 0xA3), (0xAE7C)),  # HANGUL SYLLABLE SSANGKIYEOK AE SSANGSIOS
    ((0xB2, 0xA4), (0xAE7D)),  # HANGUL SYLLABLE SSANGKIYEOK AE IEUNG
    ((0xB2, 0xA5), (0xAE84)),  # HANGUL SYLLABLE SSANGKIYEOK YA
    ((0xB2, 0xA6), (0xAE85)),  # HANGUL SYLLABLE SSANGKIYEOK YA KIYEOK
    ((0xB2, 0xA7), (0xAE8C)),  # HANGUL SYLLABLE SSANGKIYEOK YA RIEUL
    ((0xB2, 0xA8), (0xAEBC)),  # HANGUL SYLLABLE SSANGKIYEOK EO
    ((0xB2, 0xA9), (0xAEBD)),  # HANGUL SYLLABLE SSANGKIYEOK EO KIYEOK
    ((0xB2, 0xAA), (0xAEBE)),  # HANGUL SYLLABLE SSANGKIYEOK EO SSANGKIYEOK
    ((0xB2, 0xAB), (0xAEC0)),  # HANGUL SYLLABLE SSANGKIYEOK EO NIEUN
    ((0xB2, 0xAC), (0xAEC4)),  # HANGUL SYLLABLE SSANGKIYEOK EO RIEUL
    ((0xB2, 0xAD), (0xAECC)),  # HANGUL SYLLABLE SSANGKIYEOK EO MIEUM
    ((0xB2, 0xAE), (0xAECD)),  # HANGUL SYLLABLE SSANGKIYEOK EO PIEUP
    ((0xB2, 0xAF), (0xAECF)),  # HANGUL SYLLABLE SSANGKIYEOK EO SIOS
    ((0xB2, 0xB0), (0xAED0)),  # HANGUL SYLLABLE SSANGKIYEOK EO SSANGSIOS
    ((0xB2, 0xB1), (0xAED1)),  # HANGUL SYLLABLE SSANGKIYEOK EO IEUNG
    ((0xB2, 0xB2), (0xAED8)),  # HANGUL SYLLABLE SSANGKIYEOK E
    ((0xB2, 0xB3), (0xAED9)),  # HANGUL SYLLABLE SSANGKIYEOK E KIYEOK
    ((0xB2, 0xB4), (0xAEDC)),  # HANGUL SYLLABLE SSANGKIYEOK E NIEUN
    ((0xB2, 0xB5), (0xAEE8)),  # HANGUL SYLLABLE SSANGKIYEOK E MIEUM
    ((0xB2, 0xB6), (0xAEEB)),  # HANGUL SYLLABLE SSANGKIYEOK E SIOS
    ((0xB2, 0xB7), (0xAEED)),  # HANGUL SYLLABLE SSANGKIYEOK E IEUNG
    ((0xB2, 0xB8), (0xAEF4)),  # HANGUL SYLLABLE SSANGKIYEOK YEO
    ((0xB2, 0xB9), (0xAEF8)),  # HANGUL SYLLABLE SSANGKIYEOK YEO NIEUN
    ((0xB2, 0xBA), (0xAEFC)),  # HANGUL SYLLABLE SSANGKIYEOK YEO RIEUL
    ((0xB2, 0xBB), (0xAF07)),  # HANGUL SYLLABLE SSANGKIYEOK YEO SIOS
    ((0xB2, 0xBC), (0xAF08)),  # HANGUL SYLLABLE SSANGKIYEOK YEO SSANGSIOS
    ((0xB2, 0xBD), (0xAF0D)),  # HANGUL SYLLABLE SSANGKIYEOK YEO THIEUTH
    ((0xB2, 0xBE), (0xAF10)),  # HANGUL SYLLABLE SSANGKIYEOK YE
    ((0xB2, 0xBF), (0xAF2C)),  # HANGUL SYLLABLE SSANGKIYEOK O
    ((0xB2, 0xC0), (0xAF2D)),  # HANGUL SYLLABLE SSANGKIYEOK O KIYEOK
    ((0xB2, 0xC1), (0xAF30)),  # HANGUL SYLLABLE SSANGKIYEOK O NIEUN
    ((0xB2, 0xC2), (0xAF32)),  # HANGUL SYLLABLE SSANGKIYEOK O NIEUNHIEUH
    ((0xB2, 0xC3), (0xAF34)),  # HANGUL SYLLABLE SSANGKIYEOK O RIEUL
    ((0xB2, 0xC4), (0xAF3C)),  # HANGUL SYLLABLE SSANGKIYEOK O MIEUM
    ((0xB2, 0xC5), (0xAF3D)),  # HANGUL SYLLABLE SSANGKIYEOK O PIEUP
    ((0xB2, 0xC6), (0xAF3F)),  # HANGUL SYLLABLE SSANGKIYEOK O SIOS
    ((0xB2, 0xC7), (0xAF41)),  # HANGUL SYLLABLE SSANGKIYEOK O IEUNG
    ((0xB2, 0xC8), (0xAF42)),  # HANGUL SYLLABLE SSANGKIYEOK O CIEUC
    ((0xB2, 0xC9), (0xAF43)),  # HANGUL SYLLABLE SSANGKIYEOK O CHIEUCH
    ((0xB2, 0xCA), (0xAF48)),  # HANGUL SYLLABLE SSANGKIYEOK WA
    ((0xB2, 0xCB), (0xAF49)),  # HANGUL SYLLABLE SSANGKIYEOK WA KIYEOK
    ((0xB2, 0xCC), (0xAF50)),  # HANGUL SYLLABLE SSANGKIYEOK WA RIEUL
    ((0xB2, 0xCD), (0xAF5C)),  # HANGUL SYLLABLE SSANGKIYEOK WA SSANGSIOS
    ((0xB2, 0xCE), (0xAF5D)),  # HANGUL SYLLABLE SSANGKIYEOK WA IEUNG
    ((0xB2, 0xCF), (0xAF64)),  # HANGUL SYLLABLE SSANGKIYEOK WAE
    ((0xB2, 0xD0), (0xAF65)),  # HANGUL SYLLABLE SSANGKIYEOK WAE KIYEOK
    ((0xB2, 0xD1), (0xAF79)),  # HANGUL SYLLABLE SSANGKIYEOK WAE IEUNG
    ((0xB2, 0xD2), (0xAF80)),  # HANGUL SYLLABLE SSANGKIYEOK OE
    ((0xB2, 0xD3), (0xAF84)),  # HANGUL SYLLABLE SSANGKIYEOK OE NIEUN
    ((0xB2, 0xD4), (0xAF88)),  # HANGUL SYLLABLE SSANGKIYEOK OE RIEUL
    ((0xB2, 0xD5), (0xAF90)),  # HANGUL SYLLABLE SSANGKIYEOK OE MIEUM
    ((0xB2, 0xD6), (0xAF91)),  # HANGUL SYLLABLE SSANGKIYEOK OE PIEUP
    ((0xB2, 0xD7), (0xAF95)),  # HANGUL SYLLABLE SSANGKIYEOK OE IEUNG
    ((0xB2, 0xD8), (0xAF9C)),  # HANGUL SYLLABLE SSANGKIYEOK YO
    ((0xB2, 0xD9), (0xAFB8)),  # HANGUL SYLLABLE SSANGKIYEOK U
    ((0xB2, 0xDA), (0xAFB9)),  # HANGUL SYLLABLE SSANGKIYEOK U KIYEOK
    ((0xB2, 0xDB), (0xAFBC)),  # HANGUL SYLLABLE SSANGKIYEOK U NIEUN
    ((0xB2, 0xDC), (0xAFC0)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEUL
    ((0xB2, 0xDD), (0xAFC7)),  # HANGUL SYLLABLE SSANGKIYEOK U RIEULHIEUH
    ((0xB2, 0xDE), (0xAFC8)),  # HANGUL SYLLABLE SSANGKIYEOK U MIEUM
    ((0xB2, 0xDF), (0xAFC9)),  # HANGUL SYLLABLE SSANGKIYEOK U PIEUP
    ((0xB2, 0xE0), (0xAFCB)),  # HANGUL SYLLABLE SSANGKIYEOK U SIOS
    ((0xB2, 0xE1), (0xAFCD)),  # HANGUL SYLLABLE SSANGKIYEOK U IEUNG
    ((0xB2, 0xE2), (0xAFCE)),  # HANGUL SYLLABLE SSANGKIYEOK U CIEUC
    ((0xB2, 0xE3), (0xAFD4)),  # HANGUL SYLLABLE SSANGKIYEOK WEO
    ((0xB2, 0xE4), (0xAFDC)),  # HANGUL SYLLABLE SSANGKIYEOK WEO RIEUL
    ((0xB2, 0xE5), (0xAFE8)),  # HANGUL SYLLABLE SSANGKIYEOK WEO SSANGSIOS
    ((0xB2, 0xE6), (0xAFE9)),  # HANGUL SYLLABLE SSANGKIYEOK WEO IEUNG
    ((0xB2, 0xE7), (0xAFF0)),  # HANGUL SYLLABLE SSANGKIYEOK WE
    ((0xB2, 0xE8), (0xAFF1)),  # HANGUL SYLLABLE SSANGKIYEOK WE KIYEOK
    ((0xB2, 0xE9), (0xAFF4)),  # HANGUL SYLLABLE SSANGKIYEOK WE NIEUN
    ((0xB2, 0xEA), (0xAFF8)),  # HANGUL SYLLABLE SSANGKIYEOK WE RIEUL
    ((0xB2, 0xEB), (0xB000)),  # HANGUL SYLLABLE SSANGKIYEOK WE MIEUM
    ((0xB2, 0xEC), (0xB001)),  # HANGUL SYLLABLE SSANGKIYEOK WE PIEUP
    ((0xB2, 0xED), (0xB004)),  # HANGUL SYLLABLE SSANGKIYEOK WE SSANGSIOS
    ((0xB2, 0xEE), (0xB00C)),  # HANGUL SYLLABLE SSANGKIYEOK WI
    ((0xB2, 0xEF), (0xB010)),  # HANGUL SYLLABLE SSANGKIYEOK WI NIEUN
    ((0xB2, 0xF0), (0xB014)),  # HANGUL SYLLABLE SSANGKIYEOK WI RIEUL
    ((0xB2, 0xF1), (0xB01C)),  # HANGUL SYLLABLE SSANGKIYEOK WI MIEUM
    ((0xB2, 0xF2), (0xB01D)),  # HANGUL SYLLABLE SSANGKIYEOK WI PIEUP
    ((0xB2, 0xF3), (0xB028)),  # HANGUL SYLLABLE SSANGKIYEOK YU
    ((0xB2, 0xF4), (0xB044)),  # HANGUL SYLLABLE SSANGKIYEOK EU
    ((0xB2, 0xF5), (0xB045)),  # HANGUL SYLLABLE SSANGKIYEOK EU KIYEOK
    ((0xB2, 0xF6), (0xB048)),  # HANGUL SYLLABLE SSANGKIYEOK EU NIEUN
    ((0xB2, 0xF7), (0xB04A)),  # HANGUL SYLLABLE SSANGKIYEOK EU NIEUNHIEUH
    ((0xB2, 0xF8), (0xB04C)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEUL
    ((0xB2, 0xF9), (0xB04E)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULMIEUM
    ((0xB2, 0xFA), (0xB053)),  # HANGUL SYLLABLE SSANGKIYEOK EU RIEULHIEUH
    ((0xB2, 0xFB), (0xB054)),  # HANGUL SYLLABLE SSANGKIYEOK EU MIEUM
    ((0xB2, 0xFC), (0xB055)),  # HANGUL SYLLABLE SSANGKIYEOK EU PIEUP
    ((0xB2, 0xFD), (0xB057)),  # HANGUL SYLLABLE SSANGKIYEOK EU SIOS
    ((0xB2, 0xFE), (0xB059)),  # HANGUL SYLLABLE SSANGKIYEOK EU IEUNG
    ((0xB3, 0x41), (0xCFCC)),  # HANGUL SYLLABLE KHIEUKH YO RIEUL
    ((0xB3, 0x42), (0xCFCD)),  # HANGUL SYLLABLE KHIEUKH YO RIEULKIYEOK
    ((0xB3, 0x43), (0xCFCE)),  # HANGUL SYLLABLE KHIEUKH YO RIEULMIEUM
    ((0xB3, 0x44), (0xCFCF)),  # HANGUL SYLLABLE KHIEUKH YO RIEULPIEUP
    ((0xB3, 0x45), (0xCFD0)),  # HANGUL SYLLABLE KHIEUKH YO RIEULSIOS
    ((0xB3, 0x46), (0xCFD1)),  # HANGUL SYLLABLE KHIEUKH YO RIEULTHIEUTH
    ((0xB3, 0x47), (0xCFD2)),  # HANGUL SYLLABLE KHIEUKH YO RIEULPHIEUPH
    ((0xB3, 0x48), (0xCFD3)),  # HANGUL SYLLABLE KHIEUKH YO RIEULHIEUH
    ((0xB3, 0x49), (0xCFD4)),  # HANGUL SYLLABLE KHIEUKH YO MIEUM
    ((0xB3, 0x4A), (0xCFD5)),  # HANGUL SYLLABLE KHIEUKH YO PIEUP
    ((0xB3, 0x4B), (0xCFD6)),  # HANGUL SYLLABLE KHIEUKH YO PIEUPSIOS
    ((0xB3, 0x4C), (0xCFD7)),  # HANGUL SYLLABLE KHIEUKH YO SIOS
    ((0xB3, 0x4D), (0xCFD8)),  # HANGUL SYLLABLE KHIEUKH YO SSANGSIOS
    ((0xB3, 0x4E), (0xCFD9)),  # HANGUL SYLLABLE KHIEUKH YO IEUNG
    ((0xB3, 0x4F), (0xCFDA)),  # HANGUL SYLLABLE KHIEUKH YO CIEUC
    ((0xB3, 0x50), (0xCFDB)),  # HANGUL SYLLABLE KHIEUKH YO CHIEUCH
    ((0xB3, 0x51), (0xCFDC)),  # HANGUL SYLLABLE KHIEUKH YO KHIEUKH
    ((0xB3, 0x52), (0xCFDD)),  # HANGUL SYLLABLE KHIEUKH YO THIEUTH
    ((0xB3, 0x53), (0xCFDE)),  # HANGUL SYLLABLE KHIEUKH YO PHIEUPH
    ((0xB3, 0x54), (0xCFDF)),  # HANGUL SYLLABLE KHIEUKH YO HIEUH
    ((0xB3, 0x55), (0xCFE2)),  # HANGUL SYLLABLE KHIEUKH U SSANGKIYEOK
    ((0xB3, 0x56), (0xCFE3)),  # HANGUL SYLLABLE KHIEUKH U KIYEOKSIOS
    ((0xB3, 0x57), (0xCFE5)),  # HANGUL SYLLABLE KHIEUKH U NIEUNCIEUC
    ((0xB3, 0x58), (0xCFE6)),  # HANGUL SYLLABLE KHIEUKH U NIEUNHIEUH
    ((0xB3, 0x59), (0xCFE7)),  # HANGUL SYLLABLE KHIEUKH U TIKEUT
    ((0xB3, 0x5A), (0xCFE9)),  # HANGUL SYLLABLE KHIEUKH U RIEULKIYEOK
    ((0xB3, 0x61), (0xCFEA)),  # HANGUL SYLLABLE KHIEUKH U RIEULMIEUM
    ((0xB3, 0x62), (0xCFEB)),  # HANGUL SYLLABLE KHIEUKH U RIEULPIEUP
    ((0xB3, 0x63), (0xCFEC)),  # HANGUL SYLLABLE KHIEUKH U RIEULSIOS
    ((0xB3, 0x64), (0xCFED)),  # HANGUL SYLLABLE KHIEUKH U RIEULTHIEUTH
    ((0xB3, 0x65), (0xCFEE)),  # HANGUL SYLLABLE KHIEUKH U RIEULPHIEUPH
    ((0xB3, 0x66), (0xCFEF)),  # HANGUL SYLLABLE KHIEUKH U RIEULHIEUH
    ((0xB3, 0x67), (0xCFF2)),  # HANGUL SYLLABLE KHIEUKH U PIEUPSIOS
    ((0xB3, 0x68), (0xCFF4)),  # HANGUL SYLLABLE KHIEUKH U SSANGSIOS
    ((0xB3, 0x69), (0xCFF6)),  # HANGUL SYLLABLE KHIEUKH U CIEUC
    ((0xB3, 0x6A), (0xCFF7)),  # HANGUL SYLLABLE KHIEUKH U CHIEUCH
    ((0xB3, 0x6B), (0xCFF8)),  # HANGUL SYLLABLE KHIEUKH U KHIEUKH
    ((0xB3, 0x6C), (0xCFF9)),  # HANGUL SYLLABLE KHIEUKH U THIEUTH
    ((0xB3, 0x6D), (0xCFFA)),  # HANGUL SYLLABLE KHIEUKH U PHIEUPH
    ((0xB3, 0x6E), (0xCFFB)),  # HANGUL SYLLABLE KHIEUKH U HIEUH
    ((0xB3, 0x6F), (0xCFFD)),  # HANGUL SYLLABLE KHIEUKH WEO KIYEOK
    ((0xB3, 0x70), (0xCFFE)),  # HANGUL SYLLABLE KHIEUKH WEO SSANGKIYEOK
    ((0xB3, 0x71), (0xCFFF)),  # HANGUL SYLLABLE KHIEUKH WEO KIYEOKSIOS
    ((0xB3, 0x72), (0xD001)),  # HANGUL SYLLABLE KHIEUKH WEO NIEUNCIEUC
    ((0xB3, 0x73), (0xD002)),  # HANGUL SYLLABLE KHIEUKH WEO NIEUNHIEUH
    ((0xB3, 0x74), (0xD003)),  # HANGUL SYLLABLE KHIEUKH WEO TIKEUT
    ((0xB3, 0x75), (0xD005)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULKIYEOK
    ((0xB3, 0x76), (0xD006)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULMIEUM
    ((0xB3, 0x77), (0xD007)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULPIEUP
    ((0xB3, 0x78), (0xD008)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULSIOS
    ((0xB3, 0x79), (0xD009)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULTHIEUTH
    ((0xB3, 0x7A), (0xD00A)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULPHIEUPH
    ((0xB3, 0x81), (0xD00B)),  # HANGUL SYLLABLE KHIEUKH WEO RIEULHIEUH
    ((0xB3, 0x82), (0xD00C)),  # HANGUL SYLLABLE KHIEUKH WEO MIEUM
    ((0xB3, 0x83), (0xD00D)),  # HANGUL SYLLABLE KHIEUKH WEO PIEUP
    ((0xB3, 0x84), (0xD00E)),  # HANGUL SYLLABLE KHIEUKH WEO PIEUPSIOS
    ((0xB3, 0x85), (0xD00F)),  # HANGUL SYLLABLE KHIEUKH WEO SIOS
    ((0xB3, 0x86), (0xD010)),  # HANGUL SYLLABLE KHIEUKH WEO SSANGSIOS
    ((0xB3, 0x87), (0xD012)),  # HANGUL SYLLABLE KHIEUKH WEO CIEUC
    ((0xB3, 0x88), (0xD013)),  # HANGUL SYLLABLE KHIEUKH WEO CHIEUCH
    ((0xB3, 0x89), (0xD014)),  # HANGUL SYLLABLE KHIEUKH WEO KHIEUKH
    ((0xB3, 0x8A), (0xD015)),  # HANGUL SYLLABLE KHIEUKH WEO THIEUTH
    ((0xB3, 0x8B), (0xD016)),  # HANGUL SYLLABLE KHIEUKH WEO PHIEUPH
    ((0xB3, 0x8C), (0xD017)),  # HANGUL SYLLABLE KHIEUKH WEO HIEUH
    ((0xB3, 0x8D), (0xD019)),  # HANGUL SYLLABLE KHIEUKH WE KIYEOK
    ((0xB3, 0x8E), (0xD01A)),  # HANGUL SYLLABLE KHIEUKH WE SSANGKIYEOK
    ((0xB3, 0x8F), (0xD01B)),  # HANGUL SYLLABLE KHIEUKH WE KIYEOKSIOS
    ((0xB3, 0x90), (0xD01C)),  # HANGUL SYLLABLE KHIEUKH WE NIEUN
    ((0xB3, 0x91), (0xD01D)),  # HANGUL SYLLABLE KHIEUKH WE NIEUNCIEUC
    ((0xB3, 0x92), (0xD01E)),  # HANGUL SYLLABLE KHIEUKH WE NIEUNHIEUH
    ((0xB3, 0x93), (0xD01F)),  # HANGUL SYLLABLE KHIEUKH WE TIKEUT
    ((0xB3, 0x94), (0xD020)),  # HANGUL SYLLABLE KHIEUKH WE RIEUL
    ((0xB3, 0x95), (0xD021)),  # HANGUL SYLLABLE KHIEUKH WE RIEULKIYEOK
    ((0xB3, 0x96), (0xD022)),  # HANGUL SYLLABLE KHIEUKH WE RIEULMIEUM
    ((0xB3, 0x97), (0xD023)),  # HANGUL SYLLABLE KHIEUKH WE RIEULPIEUP
    ((0xB3, 0x98), (0xD024)),  # HANGUL SYLLABLE KHIEUKH WE RIEULSIOS
    ((0xB3, 0x99), (0xD025)),  # HANGUL SYLLABLE KHIEUKH WE RIEULTHIEUTH
    ((0xB3, 0x9A), (0xD026)),  # HANGUL SYLLABLE KHIEUKH WE RIEULPHIEUPH
    ((0xB3, 0x9B), (0xD027)),  # HANGUL SYLLABLE KHIEUKH WE RIEULHIEUH
    ((0xB3, 0x9C), (0xD028)),  # HANGUL SYLLABLE KHIEUKH WE MIEUM
    ((0xB3, 0x9D), (0xD029)),  # HANGUL SYLLABLE KHIEUKH WE PIEUP
    ((0xB3, 0x9E), (0xD02A)),  # HANGUL SYLLABLE KHIEUKH WE PIEUPSIOS
    ((0xB3, 0x9F), (0xD02B)),  # HANGUL SYLLABLE KHIEUKH WE SIOS
    ((0xB3, 0xA0), (0xD02C)),  # HANGUL SYLLABLE KHIEUKH WE SSANGSIOS
    ((0xB3, 0xA1), (0xB05D)),  # HANGUL SYLLABLE SSANGKIYEOK EU THIEUTH
    ((0xB3, 0xA2), (0xB07C)),  # HANGUL SYLLABLE SSANGKIYEOK I
    ((0xB3, 0xA3), (0xB07D)),  # HANGUL SYLLABLE SSANGKIYEOK I KIYEOK
    ((0xB3, 0xA4), (0xB080)),  # HANGUL SYLLABLE SSANGKIYEOK I NIEUN
    ((0xB3, 0xA5), (0xB084)),  # HANGUL SYLLABLE SSANGKIYEOK I RIEUL
    ((0xB3, 0xA6), (0xB08C)),  # HANGUL SYLLABLE SSANGKIYEOK I MIEUM
    ((0xB3, 0xA7), (0xB08D)),  # HANGUL SYLLABLE SSANGKIYEOK I PIEUP
    ((0xB3, 0xA8), (0xB08F)),  # HANGUL SYLLABLE SSANGKIYEOK I SIOS
    ((0xB3, 0xA9), (0xB091)),  # HANGUL SYLLABLE SSANGKIYEOK I IEUNG
    ((0xB3, 0xAA), (0xB098)),  # HANGUL SYLLABLE NIEUN A
    ((0xB3, 0xAB), (0xB099)),  # HANGUL SYLLABLE NIEUN A KIYEOK
    ((0xB3, 0xAC), (0xB09A)),  # HANGUL SYLLABLE NIEUN A SSANGKIYEOK
    ((0xB3, 0xAD), (0xB09C)),  # HANGUL SYLLABLE NIEUN A NIEUN
    ((0xB3, 0xAE), (0xB09F)),  # HANGUL SYLLABLE NIEUN A TIKEUT
    ((0xB3, 0xAF), (0xB0A0)),  # HANGUL SYLLABLE NIEUN A RIEUL
    ((0xB3, 0xB0), (0xB0A1)),  # HANGUL SYLLABLE NIEUN A RIEULKIYEOK
    ((0xB3, 0xB1), (0xB0A2)),  # HANGUL SYLLABLE NIEUN A RIEULMIEUM
    ((0xB3, 0xB2), (0xB0A8)),  # HANGUL SYLLABLE NIEUN A MIEUM
    ((0xB3, 0xB3), (0xB0A9)),  # HANGUL SYLLABLE NIEUN A PIEUP
    ((0xB3, 0xB4), (0xB0AB)),  # HANGUL SYLLABLE NIEUN A SIOS
    ((0xB3, 0xB5), (0xB0AC)),  # HANGUL SYLLABLE NIEUN A SSANGSIOS
    ((0xB3, 0xB6), (0xB0AD)),  # HANGUL SYLLABLE NIEUN A IEUNG
    ((0xB3, 0xB7), (0xB0AE)),  # HANGUL SYLLABLE NIEUN A CIEUC
    ((0xB3, 0xB8), (0xB0AF)),  # HANGUL SYLLABLE NIEUN A CHIEUCH
    ((0xB3, 0xB9), (0xB0B1)),  # HANGUL SYLLABLE NIEUN A THIEUTH
    ((0xB3, 0xBA), (0xB0B3)),  # HANGUL SYLLABLE NIEUN A HIEUH
    ((0xB3, 0xBB), (0xB0B4)),  # HANGUL SYLLABLE NIEUN AE
    ((0xB3, 0xBC), (0xB0B5)),  # HANGUL SYLLABLE NIEUN AE KIYEOK
    ((0xB3, 0xBD), (0xB0B8)),  # HANGUL SYLLABLE NIEUN AE NIEUN
    ((0xB3, 0xBE), (0xB0BC)),  # HANGUL SYLLABLE NIEUN AE RIEUL
    ((0xB3, 0xBF), (0xB0C4)),  # HANGUL SYLLABLE NIEUN AE MIEUM
    ((0xB3, 0xC0), (0xB0C5)),  # HANGUL SYLLABLE NIEUN AE PIEUP
    ((0xB3, 0xC1), (0xB0C7)),  # HANGUL SYLLABLE NIEUN AE SIOS
    ((0xB3, 0xC2), (0xB0C8)),  # HANGUL SYLLABLE NIEUN AE SSANGSIOS
    ((0xB3, 0xC3), (0xB0C9)),  # HANGUL SYLLABLE NIEUN AE IEUNG
    ((0xB3, 0xC4), (0xB0D0)),  # HANGUL SYLLABLE NIEUN YA
    ((0xB3, 0xC5), (0xB0D1)),  # HANGUL SYLLABLE NIEUN YA KIYEOK
    ((0xB3, 0xC6), (0xB0D4)),  # HANGUL SYLLABLE NIEUN YA NIEUN
    ((0xB3, 0xC7), (0xB0D8)),  # HANGUL SYLLABLE NIEUN YA RIEUL
    ((0xB3, 0xC8), (0xB0E0)),  # HANGUL SYLLABLE NIEUN YA MIEUM
    ((0xB3, 0xC9), (0xB0E5)),  # HANGUL SYLLABLE NIEUN YA IEUNG
    ((0xB3, 0xCA), (0xB108)),  # HANGUL SYLLABLE NIEUN EO
    ((0xB3, 0xCB), (0xB109)),  # HANGUL SYLLABLE NIEUN EO KIYEOK
    ((0xB3, 0xCC), (0xB10B)),  # HANGUL SYLLABLE NIEUN EO KIYEOKSIOS
    ((0xB3, 0xCD), (0xB10C)),  # HANGUL SYLLABLE NIEUN EO NIEUN
    ((0xB3, 0xCE), (0xB110)),  # HANGUL SYLLABLE NIEUN EO RIEUL
    ((0xB3, 0xCF), (0xB112)),  # HANGUL SYLLABLE NIEUN EO RIEULMIEUM
    ((0xB3, 0xD0), (0xB113)),  # HANGUL SYLLABLE NIEUN EO RIEULPIEUP
    ((0xB3, 0xD1), (0xB118)),  # HANGUL SYLLABLE NIEUN EO MIEUM
    ((0xB3, 0xD2), (0xB119)),  # HANGUL SYLLABLE NIEUN EO PIEUP
    ((0xB3, 0xD3), (0xB11B)),  # HANGUL SYLLABLE NIEUN EO SIOS
    ((0xB3, 0xD4), (0xB11C)),  # HANGUL SYLLABLE NIEUN EO SSANGSIOS
    ((0xB3, 0xD5), (0xB11D)),  # HANGUL SYLLABLE NIEUN EO IEUNG
    ((0xB3, 0xD6), (0xB123)),  # HANGUL SYLLABLE NIEUN EO HIEUH
    ((0xB3, 0xD7), (0xB124)),  # HANGUL SYLLABLE NIEUN E
    ((0xB3, 0xD8), (0xB125)),  # HANGUL SYLLABLE NIEUN E KIYEOK
    ((0xB3, 0xD9), (0xB128)),  # HANGUL SYLLABLE NIEUN E NIEUN
    ((0xB3, 0xDA), (0xB12C)),  # HANGUL SYLLABLE NIEUN E RIEUL
    ((0xB3, 0xDB), (0xB134)),  # HANGUL SYLLABLE NIEUN E MIEUM
    ((0xB3, 0xDC), (0xB135)),  # HANGUL SYLLABLE NIEUN E PIEUP
    ((0xB3, 0xDD), (0xB137)),  # HANGUL SYLLABLE NIEUN E SIOS
    ((0xB3, 0xDE), (0xB138)),  # HANGUL SYLLABLE NIEUN E SSANGSIOS
    ((0xB3, 0xDF), (0xB139)),  # HANGUL SYLLABLE NIEUN E IEUNG
    ((0xB3, 0xE0), (0xB140)),  # HANGUL SYLLABLE NIEUN YEO
    ((0xB3, 0xE1), (0xB141)),  # HANGUL SYLLABLE NIEUN YEO KIYEOK
    ((0xB3, 0xE2), (0xB144)),  # HANGUL SYLLABLE NIEUN YEO NIEUN
    ((0xB3, 0xE3), (0xB148)),  # HANGUL SYLLABLE NIEUN YEO RIEUL
    ((0xB3, 0xE4), (0xB150)),  # HANGUL SYLLABLE NIEUN YEO MIEUM
    ((0xB3, 0xE5), (0xB151)),  # HANGUL SYLLABLE NIEUN YEO PIEUP
    ((0xB3, 0xE6), (0xB154)),  # HANGUL SYLLABLE NIEUN YEO SSANGSIOS
    ((0xB3, 0xE7), (0xB155)),  # HANGUL SYLLABLE NIEUN YEO IEUNG
    ((0xB3, 0xE8), (0xB158)),  # HANGUL SYLLABLE NIEUN YEO KHIEUKH
    ((0xB3, 0xE9), (0xB15C)),  # HANGUL SYLLABLE NIEUN YE
    ((0xB3, 0xEA), (0xB160)),  # HANGUL SYLLABLE NIEUN YE NIEUN
    ((0xB3, 0xEB), (0xB178)),  # HANGUL SYLLABLE NIEUN O
    ((0xB3, 0xEC), (0xB179)),  # HANGUL SYLLABLE NIEUN O KIYEOK
    ((0xB3, 0xED), (0xB17C)),  # HANGUL SYLLABLE NIEUN O NIEUN
    ((0xB3, 0xEE), (0xB180)),  # HANGUL SYLLABLE NIEUN O RIEUL
    ((0xB3, 0xEF), (0xB182)),  # HANGUL SYLLABLE NIEUN O RIEULMIEUM
    ((0xB3, 0xF0), (0xB188)),  # HANGUL SYLLABLE NIEUN O MIEUM
    ((0xB3, 0xF1), (0xB189)),  # HANGUL SYLLABLE NIEUN O PIEUP
    ((0xB3, 0xF2), (0xB18B)),  # HANGUL SYLLABLE NIEUN O SIOS
    ((0xB3, 0xF3), (0xB18D)),  # HANGUL SYLLABLE NIEUN O IEUNG
    ((0xB3, 0xF4), (0xB192)),  # HANGUL SYLLABLE NIEUN O PHIEUPH
    ((0xB3, 0xF5), (0xB193)),  # HANGUL SYLLABLE NIEUN O HIEUH
    ((0xB3, 0xF6), (0xB194)),  # HANGUL SYLLABLE NIEUN WA
    ((0xB3, 0xF7), (0xB198)),  # HANGUL SYLLABLE NIEUN WA NIEUN
    ((0xB3, 0xF8), (0xB19C)),  # HANGUL SYLLABLE NIEUN WA RIEUL
    ((0xB3, 0xF9), (0xB1A8)),  # HANGUL SYLLABLE NIEUN WA SSANGSIOS
    ((0xB3, 0xFA), (0xB1CC)),  # HANGUL SYLLABLE NIEUN OE
    ((0xB3, 0xFB), (0xB1D0)),  # HANGUL SYLLABLE NIEUN OE NIEUN
    ((0xB3, 0xFC), (0xB1D4)),  # HANGUL SYLLABLE NIEUN OE RIEUL
    ((0xB3, 0xFD), (0xB1DC)),  # HANGUL SYLLABLE NIEUN OE MIEUM
    ((0xB3, 0xFE), (0xB1DD)),  # HANGUL SYLLABLE NIEUN OE PIEUP
    ((0xB4, 0x41), (0xD02E)),  # HANGUL SYLLABLE KHIEUKH WE CIEUC
    ((0xB4, 0x42), (0xD02F)),  # HANGUL SYLLABLE KHIEUKH WE CHIEUCH
    ((0xB4, 0x43), (0xD030)),  # HANGUL SYLLABLE KHIEUKH WE KHIEUKH
    ((0xB4, 0x44), (0xD031)),  # HANGUL SYLLABLE KHIEUKH WE THIEUTH
    ((0xB4, 0x45), (0xD032)),  # HANGUL SYLLABLE KHIEUKH WE PHIEUPH
    ((0xB4, 0x46), (0xD033)),  # HANGUL SYLLABLE KHIEUKH WE HIEUH
    ((0xB4, 0x47), (0xD036)),  # HANGUL SYLLABLE KHIEUKH WI SSANGKIYEOK
    ((0xB4, 0x48), (0xD037)),  # HANGUL SYLLABLE KHIEUKH WI KIYEOKSIOS
    ((0xB4, 0x49), (0xD039)),  # HANGUL SYLLABLE KHIEUKH WI NIEUNCIEUC
    ((0xB4, 0x4A), (0xD03A)),  # HANGUL SYLLABLE KHIEUKH WI NIEUNHIEUH
    ((0xB4, 0x4B), (0xD03B)),  # HANGUL SYLLABLE KHIEUKH WI TIKEUT
    ((0xB4, 0x4C), (0xD03D)),  # HANGUL SYLLABLE KHIEUKH WI RIEULKIYEOK
    ((0xB4, 0x4D), (0xD03E)),  # HANGUL SYLLABLE KHIEUKH WI RIEULMIEUM
    ((0xB4, 0x4E), (0xD03F)),  # HANGUL SYLLABLE KHIEUKH WI RIEULPIEUP
    ((0xB4, 0x4F), (0xD040)),  # HANGUL SYLLABLE KHIEUKH WI RIEULSIOS
    ((0xB4, 0x50), (0xD041)),  # HANGUL SYLLABLE KHIEUKH WI RIEULTHIEUTH
    ((0xB4, 0x51), (0xD042)),  # HANGUL SYLLABLE KHIEUKH WI RIEULPHIEUPH
    ((0xB4, 0x52), (0xD043)),  # HANGUL SYLLABLE KHIEUKH WI RIEULHIEUH
    ((0xB4, 0x53), (0xD046)),  # HANGUL SYLLABLE KHIEUKH WI PIEUPSIOS
    ((0xB4, 0x54), (0xD048)),  # HANGUL SYLLABLE KHIEUKH WI SSANGSIOS
    ((0xB4, 0x55), (0xD04A)),  # HANGUL SYLLABLE KHIEUKH WI CIEUC
    ((0xB4, 0x56), (0xD04B)),  # HANGUL SYLLABLE KHIEUKH WI CHIEUCH
    ((0xB4, 0x57), (0xD04C)),  # HANGUL SYLLABLE KHIEUKH WI KHIEUKH
    ((0xB4, 0x58), (0xD04D)),  # HANGUL SYLLABLE KHIEUKH WI THIEUTH
    ((0xB4, 0x59), (0xD04E)),  # HANGUL SYLLABLE KHIEUKH WI PHIEUPH
    ((0xB4, 0x5A), (0xD04F)),  # HANGUL SYLLABLE KHIEUKH WI HIEUH
    ((0xB4, 0x61), (0xD051)),  # HANGUL SYLLABLE KHIEUKH YU KIYEOK
    ((0xB4, 0x62), (0xD052)),  # HANGUL SYLLABLE KHIEUKH YU SSANGKIYEOK
    ((0xB4, 0x63), (0xD053)),  # HANGUL SYLLABLE KHIEUKH YU KIYEOKSIOS
    ((0xB4, 0x64), (0xD055)),  # HANGUL SYLLABLE KHIEUKH YU NIEUNCIEUC
    ((0xB4, 0x65), (0xD056)),  # HANGUL SYLLABLE KHIEUKH YU NIEUNHIEUH
    ((0xB4, 0x66), (0xD057)),  # HANGUL SYLLABLE KHIEUKH YU TIKEUT
    ((0xB4, 0x67), (0xD059)),  # HANGUL SYLLABLE KHIEUKH YU RIEULKIYEOK
    ((0xB4, 0x68), (0xD05A)),  # HANGUL SYLLABLE KHIEUKH YU RIEULMIEUM
    ((0xB4, 0x69), (0xD05B)),  # HANGUL SYLLABLE KHIEUKH YU RIEULPIEUP
    ((0xB4, 0x6A), (0xD05C)),  # HANGUL SYLLABLE KHIEUKH YU RIEULSIOS
    ((0xB4, 0x6B), (0xD05D)),  # HANGUL SYLLABLE KHIEUKH YU RIEULTHIEUTH
    ((0xB4, 0x6C), (0xD05E)),  # HANGUL SYLLABLE KHIEUKH YU RIEULPHIEUPH
    ((0xB4, 0x6D), (0xD05F)),  # HANGUL SYLLABLE KHIEUKH YU RIEULHIEUH
    ((0xB4, 0x6E), (0xD061)),  # HANGUL SYLLABLE KHIEUKH YU PIEUP
    ((0xB4, 0x6F), (0xD062)),  # HANGUL SYLLABLE KHIEUKH YU PIEUPSIOS
    ((0xB4, 0x70), (0xD063)),  # HANGUL SYLLABLE KHIEUKH YU SIOS
    ((0xB4, 0x71), (0xD064)),  # HANGUL SYLLABLE KHIEUKH YU SSANGSIOS
    ((0xB4, 0x72), (0xD065)),  # HANGUL SYLLABLE KHIEUKH YU IEUNG
    ((0xB4, 0x73), (0xD066)),  # HANGUL SYLLABLE KHIEUKH YU CIEUC
    ((0xB4, 0x74), (0xD067)),  # HANGUL SYLLABLE KHIEUKH YU CHIEUCH
    ((0xB4, 0x75), (0xD068)),  # HANGUL SYLLABLE KHIEUKH YU KHIEUKH
    ((0xB4, 0x76), (0xD069)),  # HANGUL SYLLABLE KHIEUKH YU THIEUTH
    ((0xB4, 0x77), (0xD06A)),  # HANGUL SYLLABLE KHIEUKH YU PHIEUPH
    ((0xB4, 0x78), (0xD06B)),  # HANGUL SYLLABLE KHIEUKH YU HIEUH
    ((0xB4, 0x79), (0xD06E)),  # HANGUL SYLLABLE KHIEUKH EU SSANGKIYEOK
    ((0xB4, 0x7A), (0xD06F)),  # HANGUL SYLLABLE KHIEUKH EU KIYEOKSIOS
    ((0xB4, 0x81), (0xD071)),  # HANGUL SYLLABLE KHIEUKH EU NIEUNCIEUC
    ((0xB4, 0x82), (0xD072)),  # HANGUL SYLLABLE KHIEUKH EU NIEUNHIEUH
    ((0xB4, 0x83), (0xD073)),  # HANGUL SYLLABLE KHIEUKH EU TIKEUT
    ((0xB4, 0x84), (0xD075)),  # HANGUL SYLLABLE KHIEUKH EU RIEULKIYEOK
    ((0xB4, 0x85), (0xD076)),  # HANGUL SYLLABLE KHIEUKH EU RIEULMIEUM
    ((0xB4, 0x86), (0xD077)),  # HANGUL SYLLABLE KHIEUKH EU RIEULPIEUP
    ((0xB4, 0x87), (0xD078)),  # HANGUL SYLLABLE KHIEUKH EU RIEULSIOS
    ((0xB4, 0x88), (0xD079)),  # HANGUL SYLLABLE KHIEUKH EU RIEULTHIEUTH
    ((0xB4, 0x89), (0xD07A)),  # HANGUL SYLLABLE KHIEUKH EU RIEULPHIEUPH
    ((0xB4, 0x8A), (0xD07B)),  # HANGUL SYLLABLE KHIEUKH EU RIEULHIEUH
    ((0xB4, 0x8B), (0xD07E)),  # HANGUL SYLLABLE KHIEUKH EU PIEUPSIOS
    ((0xB4, 0x8C), (0xD07F)),  # HANGUL SYLLABLE KHIEUKH EU SIOS
    ((0xB4, 0x8D), (0xD080)),  # HANGUL SYLLABLE KHIEUKH EU SSANGSIOS
    ((0xB4, 0x8E), (0xD082)),  # HANGUL SYLLABLE KHIEUKH EU CIEUC
    ((0xB4, 0x8F), (0xD083)),  # HANGUL SYLLABLE KHIEUKH EU CHIEUCH
    ((0xB4, 0x90), (0xD084)),  # HANGUL SYLLABLE KHIEUKH EU KHIEUKH
    ((0xB4, 0x91), (0xD085)),  # HANGUL SYLLABLE KHIEUKH EU THIEUTH
    ((0xB4, 0x92), (0xD086)),  # HANGUL SYLLABLE KHIEUKH EU PHIEUPH
    ((0xB4, 0x93), (0xD087)),  # HANGUL SYLLABLE KHIEUKH EU HIEUH
    ((0xB4, 0x94), (0xD088)),  # HANGUL SYLLABLE KHIEUKH YI
    ((0xB4, 0x95), (0xD089)),  # HANGUL SYLLABLE KHIEUKH YI KIYEOK
    ((0xB4, 0x96), (0xD08A)),  # HANGUL SYLLABLE KHIEUKH YI SSANGKIYEOK
    ((0xB4, 0x97), (0xD08B)),  # HANGUL SYLLABLE KHIEUKH YI KIYEOKSIOS
    ((0xB4, 0x98), (0xD08C)),  # HANGUL SYLLABLE KHIEUKH YI NIEUN
    ((0xB4, 0x99), (0xD08D)),  # HANGUL SYLLABLE KHIEUKH YI NIEUNCIEUC
    ((0xB4, 0x9A), (0xD08E)),  # HANGUL SYLLABLE KHIEUKH YI NIEUNHIEUH
    ((0xB4, 0x9B), (0xD08F)),  # HANGUL SYLLABLE KHIEUKH YI TIKEUT
    ((0xB4, 0x9C), (0xD090)),  # HANGUL SYLLABLE KHIEUKH YI RIEUL
    ((0xB4, 0x9D), (0xD091)),  # HANGUL SYLLABLE KHIEUKH YI RIEULKIYEOK
    ((0xB4, 0x9E), (0xD092)),  # HANGUL SYLLABLE KHIEUKH YI RIEULMIEUM
    ((0xB4, 0x9F), (0xD093)),  # HANGUL SYLLABLE KHIEUKH YI RIEULPIEUP
    ((0xB4, 0xA0), (0xD094)),  # HANGUL SYLLABLE KHIEUKH YI RIEULSIOS
    ((0xB4, 0xA1), (0xB1DF)),  # HANGUL SYLLABLE NIEUN OE SIOS
    ((0xB4, 0xA2), (0xB1E8)),  # HANGUL SYLLABLE NIEUN YO
    ((0xB4, 0xA3), (0xB1E9)),  # HANGUL SYLLABLE NIEUN YO KIYEOK
    ((0xB4, 0xA4), (0xB1EC)),  # HANGUL SYLLABLE NIEUN YO NIEUN
    ((0xB4, 0xA5), (0xB1F0)),  # HANGUL SYLLABLE NIEUN YO RIEUL
    ((0xB4, 0xA6), (0xB1F9)),  # HANGUL SYLLABLE NIEUN YO PIEUP
    ((0xB4, 0xA7), (0xB1FB)),  # HANGUL SYLLABLE NIEUN YO SIOS
    ((0xB4, 0xA8), (0xB1FD)),  # HANGUL SYLLABLE NIEUN YO IEUNG
    ((0xB4, 0xA9), (0xB204)),  # HANGUL SYLLABLE NIEUN U
    ((0xB4, 0xAA), (0xB205)),  # HANGUL SYLLABLE NIEUN U KIYEOK
    ((0xB4, 0xAB), (0xB208)),  # HANGUL SYLLABLE NIEUN U NIEUN
    ((0xB4, 0xAC), (0xB20B)),  # HANGUL SYLLABLE NIEUN U TIKEUT
    ((0xB4, 0xAD), (0xB20C)),  # HANGUL SYLLABLE NIEUN U RIEUL
    ((0xB4, 0xAE), (0xB214)),  # HANGUL SYLLABLE NIEUN U MIEUM
    ((0xB4, 0xAF), (0xB215)),  # HANGUL SYLLABLE NIEUN U PIEUP
    ((0xB4, 0xB0), (0xB217)),  # HANGUL SYLLABLE NIEUN U SIOS
    ((0xB4, 0xB1), (0xB219)),  # HANGUL SYLLABLE NIEUN U IEUNG
    ((0xB4, 0xB2), (0xB220)),  # HANGUL SYLLABLE NIEUN WEO
    ((0xB4, 0xB3), (0xB234)),  # HANGUL SYLLABLE NIEUN WEO SSANGSIOS
    ((0xB4, 0xB4), (0xB23C)),  # HANGUL SYLLABLE NIEUN WE
    ((0xB4, 0xB5), (0xB258)),  # HANGUL SYLLABLE NIEUN WI
    ((0xB4, 0xB6), (0xB25C)),  # HANGUL SYLLABLE NIEUN WI NIEUN
    ((0xB4, 0xB7), (0xB260)),  # HANGUL SYLLABLE NIEUN WI RIEUL
    ((0xB4, 0xB8), (0xB268)),  # HANGUL SYLLABLE NIEUN WI MIEUM
    ((0xB4, 0xB9), (0xB269)),  # HANGUL SYLLABLE NIEUN WI PIEUP
    ((0xB4, 0xBA), (0xB274)),  # HANGUL SYLLABLE NIEUN YU
    ((0xB4, 0xBB), (0xB275)),  # HANGUL SYLLABLE NIEUN YU KIYEOK
    ((0xB4, 0xBC), (0xB27C)),  # HANGUL SYLLABLE NIEUN YU RIEUL
    ((0xB4, 0xBD), (0xB284)),  # HANGUL SYLLABLE NIEUN YU MIEUM
    ((0xB4, 0xBE), (0xB285)),  # HANGUL SYLLABLE NIEUN YU PIEUP
    ((0xB4, 0xBF), (0xB289)),  # HANGUL SYLLABLE NIEUN YU IEUNG
    ((0xB4, 0xC0), (0xB290)),  # HANGUL SYLLABLE NIEUN EU
    ((0xB4, 0xC1), (0xB291)),  # HANGUL SYLLABLE NIEUN EU KIYEOK
    ((0xB4, 0xC2), (0xB294)),  # HANGUL SYLLABLE NIEUN EU NIEUN
    ((0xB4, 0xC3), (0xB298)),  # HANGUL SYLLABLE NIEUN EU RIEUL
    ((0xB4, 0xC4), (0xB299)),  # HANGUL SYLLABLE NIEUN EU RIEULKIYEOK
    ((0xB4, 0xC5), (0xB29A)),  # HANGUL SYLLABLE NIEUN EU RIEULMIEUM
    ((0xB4, 0xC6), (0xB2A0)),  # HANGUL SYLLABLE NIEUN EU MIEUM
    ((0xB4, 0xC7), (0xB2A1)),  # HANGUL SYLLABLE NIEUN EU PIEUP
    ((0xB4, 0xC8), (0xB2A3)),  # HANGUL SYLLABLE NIEUN EU SIOS
    ((0xB4, 0xC9), (0xB2A5)),  # HANGUL SYLLABLE NIEUN EU IEUNG
    ((0xB4, 0xCA), (0xB2A6)),  # HANGUL SYLLABLE NIEUN EU CIEUC
    ((0xB4, 0xCB), (0xB2AA)),  # HANGUL SYLLABLE NIEUN EU PHIEUPH
    ((0xB4, 0xCC), (0xB2AC)),  # HANGUL SYLLABLE NIEUN YI
    ((0xB4, 0xCD), (0xB2B0)),  # HANGUL SYLLABLE NIEUN YI NIEUN
    ((0xB4, 0xCE), (0xB2B4)),  # HANGUL SYLLABLE NIEUN YI RIEUL
    ((0xB4, 0xCF), (0xB2C8)),  # HANGUL SYLLABLE NIEUN I
    ((0xB4, 0xD0), (0xB2C9)),  # HANGUL SYLLABLE NIEUN I KIYEOK
    ((0xB4, 0xD1), (0xB2CC)),  # HANGUL SYLLABLE NIEUN I NIEUN
    ((0xB4, 0xD2), (0xB2D0)),  # HANGUL SYLLABLE NIEUN I RIEUL
    ((0xB4, 0xD3), (0xB2D2)),  # HANGUL SYLLABLE NIEUN I RIEULMIEUM
    ((0xB4, 0xD4), (0xB2D8)),  # HANGUL SYLLABLE NIEUN I MIEUM
    ((0xB4, 0xD5), (0xB2D9)),  # HANGUL SYLLABLE NIEUN I PIEUP
    ((0xB4, 0xD6), (0xB2DB)),  # HANGUL SYLLABLE NIEUN I SIOS
    ((0xB4, 0xD7), (0xB2DD)),  # HANGUL SYLLABLE NIEUN I IEUNG
    ((0xB4, 0xD8), (0xB2E2)),  # HANGUL SYLLABLE NIEUN I PHIEUPH
    ((0xB4, 0xD9), (0xB2E4)),  # HANGUL SYLLABLE TIKEUT A
    ((0xB4, 0xDA), (0xB2E5)),  # HANGUL SYLLABLE TIKEUT A KIYEOK
    ((0xB4, 0xDB), (0xB2E6)),  # HANGUL SYLLABLE TIKEUT A SSANGKIYEOK
    ((0xB4, 0xDC), (0xB2E8)),  # HANGUL SYLLABLE TIKEUT A NIEUN
    ((0xB4, 0xDD), (0xB2EB)),  # HANGUL SYLLABLE TIKEUT A TIKEUT
    ((0xB4, 0xDE), (0xB2EC)),  # HANGUL SYLLABLE TIKEUT A RIEUL
    ((0xB4, 0xDF), (0xB2ED)),  # HANGUL SYLLABLE TIKEUT A RIEULKIYEOK
    ((0xB4, 0xE0), (0xB2EE)),  # HANGUL SYLLABLE TIKEUT A RIEULMIEUM
    ((0xB4, 0xE1), (0xB2EF)),  # HANGUL SYLLABLE TIKEUT A RIEULPIEUP
    ((0xB4, 0xE2), (0xB2F3)),  # HANGUL SYLLABLE TIKEUT A RIEULHIEUH
    ((0xB4, 0xE3), (0xB2F4)),  # HANGUL SYLLABLE TIKEUT A MIEUM
    ((0xB4, 0xE4), (0xB2F5)),  # HANGUL SYLLABLE TIKEUT A PIEUP
    ((0xB4, 0xE5), (0xB2F7)),  # HANGUL SYLLABLE TIKEUT A SIOS
    ((0xB4, 0xE6), (0xB2F8)),  # HANGUL SYLLABLE TIKEUT A SSANGSIOS
    ((0xB4, 0xE7), (0xB2F9)),  # HANGUL SYLLABLE TIKEUT A IEUNG
    ((0xB4, 0xE8), (0xB2FA)),  # HANGUL SYLLABLE TIKEUT A CIEUC
    ((0xB4, 0xE9), (0xB2FB)),  # HANGUL SYLLABLE TIKEUT A CHIEUCH
    ((0xB4, 0xEA), (0xB2FF)),  # HANGUL SYLLABLE TIKEUT A HIEUH
    ((0xB4, 0xEB), (0xB300)),  # HANGUL SYLLABLE TIKEUT AE
    ((0xB4, 0xEC), (0xB301)),  # HANGUL SYLLABLE TIKEUT AE KIYEOK
    ((0xB4, 0xED), (0xB304)),  # HANGUL SYLLABLE TIKEUT AE NIEUN
    ((0xB4, 0xEE), (0xB308)),  # HANGUL SYLLABLE TIKEUT AE RIEUL
    ((0xB4, 0xEF), (0xB310)),  # HANGUL SYLLABLE TIKEUT AE MIEUM
    ((0xB4, 0xF0), (0xB311)),  # HANGUL SYLLABLE TIKEUT AE PIEUP
    ((0xB4, 0xF1), (0xB313)),  # HANGUL SYLLABLE TIKEUT AE SIOS
    ((0xB4, 0xF2), (0xB314)),  # HANGUL SYLLABLE TIKEUT AE SSANGSIOS
    ((0xB4, 0xF3), (0xB315)),  # HANGUL SYLLABLE TIKEUT AE IEUNG
    ((0xB4, 0xF4), (0xB31C)),  # HANGUL SYLLABLE TIKEUT YA
    ((0xB4, 0xF5), (0xB354)),  # HANGUL SYLLABLE TIKEUT EO
    ((0xB4, 0xF6), (0xB355)),  # HANGUL SYLLABLE TIKEUT EO KIYEOK
    ((0xB4, 0xF7), (0xB356)),  # HANGUL SYLLABLE TIKEUT EO SSANGKIYEOK
    ((0xB4, 0xF8), (0xB358)),  # HANGUL SYLLABLE TIKEUT EO NIEUN
    ((0xB4, 0xF9), (0xB35B)),  # HANGUL SYLLABLE TIKEUT EO TIKEUT
    ((0xB4, 0xFA), (0xB35C)),  # HANGUL SYLLABLE TIKEUT EO RIEUL
    ((0xB4, 0xFB), (0xB35E)),  # HANGUL SYLLABLE TIKEUT EO RIEULMIEUM
    ((0xB4, 0xFC), (0xB35F)),  # HANGUL SYLLABLE TIKEUT EO RIEULPIEUP
    ((0xB4, 0xFD), (0xB364)),  # HANGUL SYLLABLE TIKEUT EO MIEUM
    ((0xB4, 0xFE), (0xB365)),  # HANGUL SYLLABLE TIKEUT EO PIEUP
    ((0xB5, 0x41), (0xD095)),  # HANGUL SYLLABLE KHIEUKH YI RIEULTHIEUTH
    ((0xB5, 0x42), (0xD096)),  # HANGUL SYLLABLE KHIEUKH YI RIEULPHIEUPH
    ((0xB5, 0x43), (0xD097)),  # HANGUL SYLLABLE KHIEUKH YI RIEULHIEUH
    ((0xB5, 0x44), (0xD098)),  # HANGUL SYLLABLE KHIEUKH YI MIEUM
    ((0xB5, 0x45), (0xD099)),  # HANGUL SYLLABLE KHIEUKH YI PIEUP
    ((0xB5, 0x46), (0xD09A)),  # HANGUL SYLLABLE KHIEUKH YI PIEUPSIOS
    ((0xB5, 0x47), (0xD09B)),  # HANGUL SYLLABLE KHIEUKH YI SIOS
    ((0xB5, 0x48), (0xD09C)),  # HANGUL SYLLABLE KHIEUKH YI SSANGSIOS
    ((0xB5, 0x49), (0xD09D)),  # HANGUL SYLLABLE KHIEUKH YI IEUNG
    ((0xB5, 0x4A), (0xD09E)),  # HANGUL SYLLABLE KHIEUKH YI CIEUC
    ((0xB5, 0x4B), (0xD09F)),  # HANGUL SYLLABLE KHIEUKH YI CHIEUCH
    ((0xB5, 0x4C), (0xD0A0)),  # HANGUL SYLLABLE KHIEUKH YI KHIEUKH
    ((0xB5, 0x4D), (0xD0A1)),  # HANGUL SYLLABLE KHIEUKH YI THIEUTH
    ((0xB5, 0x4E), (0xD0A2)),  # HANGUL SYLLABLE KHIEUKH YI PHIEUPH
    ((0xB5, 0x4F), (0xD0A3)),  # HANGUL SYLLABLE KHIEUKH YI HIEUH
    ((0xB5, 0x50), (0xD0A6)),  # HANGUL SYLLABLE KHIEUKH I SSANGKIYEOK
    ((0xB5, 0x51), (0xD0A7)),  # HANGUL SYLLABLE KHIEUKH I KIYEOKSIOS
    ((0xB5, 0x52), (0xD0A9)),  # HANGUL SYLLABLE KHIEUKH I NIEUNCIEUC
    ((0xB5, 0x53), (0xD0AA)),  # HANGUL SYLLABLE KHIEUKH I NIEUNHIEUH
    ((0xB5, 0x54), (0xD0AB)),  # HANGUL SYLLABLE KHIEUKH I TIKEUT
    ((0xB5, 0x55), (0xD0AD)),  # HANGUL SYLLABLE KHIEUKH I RIEULKIYEOK
    ((0xB5, 0x56), (0xD0AE)),  # HANGUL SYLLABLE KHIEUKH I RIEULMIEUM
    ((0xB5, 0x57), (0xD0AF)),  # HANGUL SYLLABLE KHIEUKH I RIEULPIEUP
    ((0xB5, 0x58), (0xD0B0)),  # HANGUL SYLLABLE KHIEUKH I RIEULSIOS
    ((0xB5, 0x59), (0xD0B1)),  # HANGUL SYLLABLE KHIEUKH I RIEULTHIEUTH
    ((0xB5, 0x5A), (0xD0B2)),  # HANGUL SYLLABLE KHIEUKH I RIEULPHIEUPH
    ((0xB5, 0x61), (0xD0B3)),  # HANGUL SYLLABLE KHIEUKH I RIEULHIEUH
    ((0xB5, 0x62), (0xD0B6)),  # HANGUL SYLLABLE KHIEUKH I PIEUPSIOS
    ((0xB5, 0x63), (0xD0B8)),  # HANGUL SYLLABLE KHIEUKH I SSANGSIOS
    ((0xB5, 0x64), (0xD0BA)),  # HANGUL SYLLABLE KHIEUKH I CIEUC
    ((0xB5, 0x65), (0xD0BB)),  # HANGUL SYLLABLE KHIEUKH I CHIEUCH
    ((0xB5, 0x66), (0xD0BC)),  # HANGUL SYLLABLE KHIEUKH I KHIEUKH
    ((0xB5, 0x67), (0xD0BD)),  # HANGUL SYLLABLE KHIEUKH I THIEUTH
    ((0xB5, 0x68), (0xD0BE)),  # HANGUL SYLLABLE KHIEUKH I PHIEUPH
    ((0xB5, 0x69), (0xD0BF)),  # HANGUL SYLLABLE KHIEUKH I HIEUH
    ((0xB5, 0x6A), (0xD0C2)),  # HANGUL SYLLABLE THIEUTH A SSANGKIYEOK
    ((0xB5, 0x6B), (0xD0C3)),  # HANGUL SYLLABLE THIEUTH A KIYEOKSIOS
    ((0xB5, 0x6C), (0xD0C5)),  # HANGUL SYLLABLE THIEUTH A NIEUNCIEUC
    ((0xB5, 0x6D), (0xD0C6)),  # HANGUL SYLLABLE THIEUTH A NIEUNHIEUH
    ((0xB5, 0x6E), (0xD0C7)),  # HANGUL SYLLABLE THIEUTH A TIKEUT
    ((0xB5, 0x6F), (0xD0CA)),  # HANGUL SYLLABLE THIEUTH A RIEULMIEUM
    ((0xB5, 0x70), (0xD0CB)),  # HANGUL SYLLABLE THIEUTH A RIEULPIEUP
    ((0xB5, 0x71), (0xD0CC)),  # HANGUL SYLLABLE THIEUTH A RIEULSIOS
    ((0xB5, 0x72), (0xD0CD)),  # HANGUL SYLLABLE THIEUTH A RIEULTHIEUTH
    ((0xB5, 0x73), (0xD0CE)),  # HANGUL SYLLABLE THIEUTH A RIEULPHIEUPH
    ((0xB5, 0x74), (0xD0CF)),  # HANGUL SYLLABLE THIEUTH A RIEULHIEUH
    ((0xB5, 0x75), (0xD0D2)),  # HANGUL SYLLABLE THIEUTH A PIEUPSIOS
    ((0xB5, 0x76), (0xD0D6)),  # HANGUL SYLLABLE THIEUTH A CIEUC
    ((0xB5, 0x77), (0xD0D7)),  # HANGUL SYLLABLE THIEUTH A CHIEUCH
    ((0xB5, 0x78), (0xD0D8)),  # HANGUL SYLLABLE THIEUTH A KHIEUKH
    ((0xB5, 0x79), (0xD0D9)),  # HANGUL SYLLABLE THIEUTH A THIEUTH
    ((0xB5, 0x7A), (0xD0DA)),  # HANGUL SYLLABLE THIEUTH A PHIEUPH
    ((0xB5, 0x81), (0xD0DB)),  # HANGUL SYLLABLE THIEUTH A HIEUH
    ((0xB5, 0x82), (0xD0DE)),  # HANGUL SYLLABLE THIEUTH AE SSANGKIYEOK
    ((0xB5, 0x83), (0xD0DF)),  # HANGUL SYLLABLE THIEUTH AE KIYEOKSIOS
    ((0xB5, 0x84), (0xD0E1)),  # HANGUL SYLLABLE THIEUTH AE NIEUNCIEUC
    ((0xB5, 0x85), (0xD0E2)),  # HANGUL SYLLABLE THIEUTH AE NIEUNHIEUH
    ((0xB5, 0x86), (0xD0E3)),  # HANGUL SYLLABLE THIEUTH AE TIKEUT
    ((0xB5, 0x87), (0xD0E5)),  # HANGUL SYLLABLE THIEUTH AE RIEULKIYEOK
    ((0xB5, 0x88), (0xD0E6)),  # HANGUL SYLLABLE THIEUTH AE RIEULMIEUM
    ((0xB5, 0x89), (0xD0E7)),  # HANGUL SYLLABLE THIEUTH AE RIEULPIEUP
    ((0xB5, 0x8A), (0xD0E8)),  # HANGUL SYLLABLE THIEUTH AE RIEULSIOS
    ((0xB5, 0x8B), (0xD0E9)),  # HANGUL SYLLABLE THIEUTH AE RIEULTHIEUTH
    ((0xB5, 0x8C), (0xD0EA)),  # HANGUL SYLLABLE THIEUTH AE RIEULPHIEUPH
    ((0xB5, 0x8D), (0xD0EB)),  # HANGUL SYLLABLE THIEUTH AE RIEULHIEUH
    ((0xB5, 0x8E), (0xD0EE)),  # HANGUL SYLLABLE THIEUTH AE PIEUPSIOS
    ((0xB5, 0x8F), (0xD0F2)),  # HANGUL SYLLABLE THIEUTH AE CIEUC
    ((0xB5, 0x90), (0xD0F3)),  # HANGUL SYLLABLE THIEUTH AE CHIEUCH
    ((0xB5, 0x91), (0xD0F4)),  # HANGUL SYLLABLE THIEUTH AE KHIEUKH
    ((0xB5, 0x92), (0xD0F5)),  # HANGUL SYLLABLE THIEUTH AE THIEUTH
    ((0xB5, 0x93), (0xD0F6)),  # HANGUL SYLLABLE THIEUTH AE PHIEUPH
    ((0xB5, 0x94), (0xD0F7)),  # HANGUL SYLLABLE THIEUTH AE HIEUH
    ((0xB5, 0x95), (0xD0F9)),  # HANGUL SYLLABLE THIEUTH YA KIYEOK
    ((0xB5, 0x96), (0xD0FA)),  # HANGUL SYLLABLE THIEUTH YA SSANGKIYEOK
    ((0xB5, 0x97), (0xD0FB)),  # HANGUL SYLLABLE THIEUTH YA KIYEOKSIOS
    ((0xB5, 0x98), (0xD0FC)),  # HANGUL SYLLABLE THIEUTH YA NIEUN
    ((0xB5, 0x99), (0xD0FD)),  # HANGUL SYLLABLE THIEUTH YA NIEUNCIEUC
    ((0xB5, 0x9A), (0xD0FE)),  # HANGUL SYLLABLE THIEUTH YA NIEUNHIEUH
    ((0xB5, 0x9B), (0xD0FF)),  # HANGUL SYLLABLE THIEUTH YA TIKEUT
    ((0xB5, 0x9C), (0xD100)),  # HANGUL SYLLABLE THIEUTH YA RIEUL
    ((0xB5, 0x9D), (0xD101)),  # HANGUL SYLLABLE THIEUTH YA RIEULKIYEOK
    ((0xB5, 0x9E), (0xD102)),  # HANGUL SYLLABLE THIEUTH YA RIEULMIEUM
    ((0xB5, 0x9F), (0xD103)),  # HANGUL SYLLABLE THIEUTH YA RIEULPIEUP
    ((0xB5, 0xA0), (0xD104)),  # HANGUL SYLLABLE THIEUTH YA RIEULSIOS
    ((0xB5, 0xA1), (0xB367)),  # HANGUL SYLLABLE TIKEUT EO SIOS
    ((0xB5, 0xA2), (0xB369)),  # HANGUL SYLLABLE TIKEUT EO IEUNG
    ((0xB5, 0xA3), (0xB36B)),  # HANGUL SYLLABLE TIKEUT EO CHIEUCH
    ((0xB5, 0xA4), (0xB36E)),  # HANGUL SYLLABLE TIKEUT EO PHIEUPH
    ((0xB5, 0xA5), (0xB370)),  # HANGUL SYLLABLE TIKEUT E
    ((0xB5, 0xA6), (0xB371)),  # HANGUL SYLLABLE TIKEUT E KIYEOK
    ((0xB5, 0xA7), (0xB374)),  # HANGUL SYLLABLE TIKEUT E NIEUN
    ((0xB5, 0xA8), (0xB378)),  # HANGUL SYLLABLE TIKEUT E RIEUL
    ((0xB5, 0xA9), (0xB380)),  # HANGUL SYLLABLE TIKEUT E MIEUM
    ((0xB5, 0xAA), (0xB381)),  # HANGUL SYLLABLE TIKEUT E PIEUP
    ((0xB5, 0xAB), (0xB383)),  # HANGUL SYLLABLE TIKEUT E SIOS
    ((0xB5, 0xAC), (0xB384)),  # HANGUL SYLLABLE TIKEUT E SSANGSIOS
    ((0xB5, 0xAD), (0xB385)),  # HANGUL SYLLABLE TIKEUT E IEUNG
    ((0xB5, 0xAE), (0xB38C)),  # HANGUL SYLLABLE TIKEUT YEO
    ((0xB5, 0xAF), (0xB390)),  # HANGUL SYLLABLE TIKEUT YEO NIEUN
    ((0xB5, 0xB0), (0xB394)),  # HANGUL SYLLABLE TIKEUT YEO RIEUL
    ((0xB5, 0xB1), (0xB3A0)),  # HANGUL SYLLABLE TIKEUT YEO SSANGSIOS
    ((0xB5, 0xB2), (0xB3A1)),  # HANGUL SYLLABLE TIKEUT YEO IEUNG
    ((0xB5, 0xB3), (0xB3A8)),  # HANGUL SYLLABLE TIKEUT YE
    ((0xB5, 0xB4), (0xB3AC)),  # HANGUL SYLLABLE TIKEUT YE NIEUN
    ((0xB5, 0xB5), (0xB3C4)),  # HANGUL SYLLABLE TIKEUT O
    ((0xB5, 0xB6), (0xB3C5)),  # HANGUL SYLLABLE TIKEUT O KIYEOK
    ((0xB5, 0xB7), (0xB3C8)),  # HANGUL SYLLABLE TIKEUT O NIEUN
    ((0xB5, 0xB8), (0xB3CB)),  # HANGUL SYLLABLE TIKEUT O TIKEUT
    ((0xB5, 0xB9), (0xB3CC)),  # HANGUL SYLLABLE TIKEUT O RIEUL
    ((0xB5, 0xBA), (0xB3CE)),  # HANGUL SYLLABLE TIKEUT O RIEULMIEUM
    ((0xB5, 0xBB), (0xB3D0)),  # HANGUL SYLLABLE TIKEUT O RIEULSIOS
    ((0xB5, 0xBC), (0xB3D4)),  # HANGUL SYLLABLE TIKEUT O MIEUM
    ((0xB5, 0xBD), (0xB3D5)),  # HANGUL SYLLABLE TIKEUT O PIEUP
    ((0xB5, 0xBE), (0xB3D7)),  # HANGUL SYLLABLE TIKEUT O SIOS
    ((0xB5, 0xBF), (0xB3D9)),  # HANGUL SYLLABLE TIKEUT O IEUNG
    ((0xB5, 0xC0), (0xB3DB)),  # HANGUL SYLLABLE TIKEUT O CHIEUCH
    ((0xB5, 0xC1), (0xB3DD)),  # HANGUL SYLLABLE TIKEUT O THIEUTH
    ((0xB5, 0xC2), (0xB3E0)),  # HANGUL SYLLABLE TIKEUT WA
    ((0xB5, 0xC3), (0xB3E4)),  # HANGUL SYLLABLE TIKEUT WA NIEUN
    ((0xB5, 0xC4), (0xB3E8)),  # HANGUL SYLLABLE TIKEUT WA RIEUL
    ((0xB5, 0xC5), (0xB3FC)),  # HANGUL SYLLABLE TIKEUT WAE
    ((0xB5, 0xC6), (0xB410)),  # HANGUL SYLLABLE TIKEUT WAE SSANGSIOS
    ((0xB5, 0xC7), (0xB418)),  # HANGUL SYLLABLE TIKEUT OE
    ((0xB5, 0xC8), (0xB41C)),  # HANGUL SYLLABLE TIKEUT OE NIEUN
    ((0xB5, 0xC9), (0xB420)),  # HANGUL SYLLABLE TIKEUT OE RIEUL
    ((0xB5, 0xCA), (0xB428)),  # HANGUL SYLLABLE TIKEUT OE MIEUM
    ((0xB5, 0xCB), (0xB429)),  # HANGUL SYLLABLE TIKEUT OE PIEUP
    ((0xB5, 0xCC), (0xB42B)),  # HANGUL SYLLABLE TIKEUT OE SIOS
    ((0xB5, 0xCD), (0xB434)),  # HANGUL SYLLABLE TIKEUT YO
    ((0xB5, 0xCE), (0xB450)),  # HANGUL SYLLABLE TIKEUT U
    ((0xB5, 0xCF), (0xB451)),  # HANGUL SYLLABLE TIKEUT U KIYEOK
    ((0xB5, 0xD0), (0xB454)),  # HANGUL SYLLABLE TIKEUT U NIEUN
    ((0xB5, 0xD1), (0xB458)),  # HANGUL SYLLABLE TIKEUT U RIEUL
    ((0xB5, 0xD2), (0xB460)),  # HANGUL SYLLABLE TIKEUT U MIEUM
    ((0xB5, 0xD3), (0xB461)),  # HANGUL SYLLABLE TIKEUT U PIEUP
    ((0xB5, 0xD4), (0xB463)),  # HANGUL SYLLABLE TIKEUT U SIOS
    ((0xB5, 0xD5), (0xB465)),  # HANGUL SYLLABLE TIKEUT U IEUNG
    ((0xB5, 0xD6), (0xB46C)),  # HANGUL SYLLABLE TIKEUT WEO
    ((0xB5, 0xD7), (0xB480)),  # HANGUL SYLLABLE TIKEUT WEO SSANGSIOS
    ((0xB5, 0xD8), (0xB488)),  # HANGUL SYLLABLE TIKEUT WE
    ((0xB5, 0xD9), (0xB49D)),  # HANGUL SYLLABLE TIKEUT WE IEUNG
    ((0xB5, 0xDA), (0xB4A4)),  # HANGUL SYLLABLE TIKEUT WI
    ((0xB5, 0xDB), (0xB4A8)),  # HANGUL SYLLABLE TIKEUT WI NIEUN
    ((0xB5, 0xDC), (0xB4AC)),  # HANGUL SYLLABLE TIKEUT WI RIEUL
    ((0xB5, 0xDD), (0xB4B5)),  # HANGUL SYLLABLE TIKEUT WI PIEUP
    ((0xB5, 0xDE), (0xB4B7)),  # HANGUL SYLLABLE TIKEUT WI SIOS
    ((0xB5, 0xDF), (0xB4B9)),  # HANGUL SYLLABLE TIKEUT WI IEUNG
    ((0xB5, 0xE0), (0xB4C0)),  # HANGUL SYLLABLE TIKEUT YU
    ((0xB5, 0xE1), (0xB4C4)),  # HANGUL SYLLABLE TIKEUT YU NIEUN
    ((0xB5, 0xE2), (0xB4C8)),  # HANGUL SYLLABLE TIKEUT YU RIEUL
    ((0xB5, 0xE3), (0xB4D0)),  # HANGUL SYLLABLE TIKEUT YU MIEUM
    ((0xB5, 0xE4), (0xB4D5)),  # HANGUL SYLLABLE TIKEUT YU IEUNG
    ((0xB5, 0xE5), (0xB4DC)),  # HANGUL SYLLABLE TIKEUT EU
    ((0xB5, 0xE6), (0xB4DD)),  # HANGUL SYLLABLE TIKEUT EU KIYEOK
    ((0xB5, 0xE7), (0xB4E0)),  # HANGUL SYLLABLE TIKEUT EU NIEUN
    ((0xB5, 0xE8), (0xB4E3)),  # HANGUL SYLLABLE TIKEUT EU TIKEUT
    ((0xB5, 0xE9), (0xB4E4)),  # HANGUL SYLLABLE TIKEUT EU RIEUL
    ((0xB5, 0xEA), (0xB4E6)),  # HANGUL SYLLABLE TIKEUT EU RIEULMIEUM
    ((0xB5, 0xEB), (0xB4EC)),  # HANGUL SYLLABLE TIKEUT EU MIEUM
    ((0xB5, 0xEC), (0xB4ED)),  # HANGUL SYLLABLE TIKEUT EU PIEUP
    ((0xB5, 0xED), (0xB4EF)),  # HANGUL SYLLABLE TIKEUT EU SIOS
    ((0xB5, 0xEE), (0xB4F1)),  # HANGUL SYLLABLE TIKEUT EU IEUNG
    ((0xB5, 0xEF), (0xB4F8)),  # HANGUL SYLLABLE TIKEUT YI
    ((0xB5, 0xF0), (0xB514)),  # HANGUL SYLLABLE TIKEUT I
    ((0xB5, 0xF1), (0xB515)),  # HANGUL SYLLABLE TIKEUT I KIYEOK
    ((0xB5, 0xF2), (0xB518)),  # HANGUL SYLLABLE TIKEUT I NIEUN
    ((0xB5, 0xF3), (0xB51B)),  # HANGUL SYLLABLE TIKEUT I TIKEUT
    ((0xB5, 0xF4), (0xB51C)),  # HANGUL SYLLABLE TIKEUT I RIEUL
    ((0xB5, 0xF5), (0xB524)),  # HANGUL SYLLABLE TIKEUT I MIEUM
    ((0xB5, 0xF6), (0xB525)),  # HANGUL SYLLABLE TIKEUT I PIEUP
    ((0xB5, 0xF7), (0xB527)),  # HANGUL SYLLABLE TIKEUT I SIOS
    ((0xB5, 0xF8), (0xB528)),  # HANGUL SYLLABLE TIKEUT I SSANGSIOS
    ((0xB5, 0xF9), (0xB529)),  # HANGUL SYLLABLE TIKEUT I IEUNG
    ((0xB5, 0xFA), (0xB52A)),  # HANGUL SYLLABLE TIKEUT I CIEUC
    ((0xB5, 0xFB), (0xB530)),  # HANGUL SYLLABLE SSANGTIKEUT A
    ((0xB5, 0xFC), (0xB531)),  # HANGUL SYLLABLE SSANGTIKEUT A KIYEOK
    ((0xB5, 0xFD), (0xB534)),  # HANGUL SYLLABLE SSANGTIKEUT A NIEUN
    ((0xB5, 0xFE), (0xB538)),  # HANGUL SYLLABLE SSANGTIKEUT A RIEUL
    ((0xB6, 0x41), (0xD105)),  # HANGUL SYLLABLE THIEUTH YA RIEULTHIEUTH
    ((0xB6, 0x42), (0xD106)),  # HANGUL SYLLABLE THIEUTH YA RIEULPHIEUPH
    ((0xB6, 0x43), (0xD107)),  # HANGUL SYLLABLE THIEUTH YA RIEULHIEUH
    ((0xB6, 0x44), (0xD108)),  # HANGUL SYLLABLE THIEUTH YA MIEUM
    ((0xB6, 0x45), (0xD109)),  # HANGUL SYLLABLE THIEUTH YA PIEUP
    ((0xB6, 0x46), (0xD10A)),  # HANGUL SYLLABLE THIEUTH YA PIEUPSIOS
    ((0xB6, 0x47), (0xD10B)),  # HANGUL SYLLABLE THIEUTH YA SIOS
    ((0xB6, 0x48), (0xD10C)),  # HANGUL SYLLABLE THIEUTH YA SSANGSIOS
    ((0xB6, 0x49), (0xD10E)),  # HANGUL SYLLABLE THIEUTH YA CIEUC
    ((0xB6, 0x4A), (0xD10F)),  # HANGUL SYLLABLE THIEUTH YA CHIEUCH
    ((0xB6, 0x4B), (0xD110)),  # HANGUL SYLLABLE THIEUTH YA KHIEUKH
    ((0xB6, 0x4C), (0xD111)),  # HANGUL SYLLABLE THIEUTH YA THIEUTH
    ((0xB6, 0x4D), (0xD112)),  # HANGUL SYLLABLE THIEUTH YA PHIEUPH
    ((0xB6, 0x4E), (0xD113)),  # HANGUL SYLLABLE THIEUTH YA HIEUH
    ((0xB6, 0x4F), (0xD114)),  # HANGUL SYLLABLE THIEUTH YAE
    ((0xB6, 0x50), (0xD115)),  # HANGUL SYLLABLE THIEUTH YAE KIYEOK
    ((0xB6, 0x51), (0xD116)),  # HANGUL SYLLABLE THIEUTH YAE SSANGKIYEOK
    ((0xB6, 0x52), (0xD117)),  # HANGUL SYLLABLE THIEUTH YAE KIYEOKSIOS
    ((0xB6, 0x53), (0xD118)),  # HANGUL SYLLABLE THIEUTH YAE NIEUN
    ((0xB6, 0x54), (0xD119)),  # HANGUL SYLLABLE THIEUTH YAE NIEUNCIEUC
    ((0xB6, 0x55), (0xD11A)),  # HANGUL SYLLABLE THIEUTH YAE NIEUNHIEUH
    ((0xB6, 0x56), (0xD11B)),  # HANGUL SYLLABLE THIEUTH YAE TIKEUT
    ((0xB6, 0x57), (0xD11C)),  # HANGUL SYLLABLE THIEUTH YAE RIEUL
    ((0xB6, 0x58), (0xD11D)),  # HANGUL SYLLABLE THIEUTH YAE RIEULKIYEOK
    ((0xB6, 0x59), (0xD11E)),  # HANGUL SYLLABLE THIEUTH YAE RIEULMIEUM
    ((0xB6, 0x5A), (0xD11F)),  # HANGUL SYLLABLE THIEUTH YAE RIEULPIEUP
    ((0xB6, 0x61), (0xD120)),  # HANGUL SYLLABLE THIEUTH YAE RIEULSIOS
    ((0xB6, 0x62), (0xD121)),  # HANGUL SYLLABLE THIEUTH YAE RIEULTHIEUTH
    ((0xB6, 0x63), (0xD122)),  # HANGUL SYLLABLE THIEUTH YAE RIEULPHIEUPH
    ((0xB6, 0x64), (0xD123)),  # HANGUL SYLLABLE THIEUTH YAE RIEULHIEUH
    ((0xB6, 0x65), (0xD124)),  # HANGUL SYLLABLE THIEUTH YAE MIEUM
    ((0xB6, 0x66), (0xD125)),  # HANGUL SYLLABLE THIEUTH YAE PIEUP
    ((0xB6, 0x67), (0xD126)),  # HANGUL SYLLABLE THIEUTH YAE PIEUPSIOS
    ((0xB6, 0x68), (0xD127)),  # HANGUL SYLLABLE THIEUTH YAE SIOS
    ((0xB6, 0x69), (0xD128)),  # HANGUL SYLLABLE THIEUTH YAE SSANGSIOS
    ((0xB6, 0x6A), (0xD129)),  # HANGUL SYLLABLE THIEUTH YAE IEUNG
    ((0xB6, 0x6B), (0xD12A)),  # HANGUL SYLLABLE THIEUTH YAE CIEUC
    ((0xB6, 0x6C), (0xD12B)),  # HANGUL SYLLABLE THIEUTH YAE CHIEUCH
    ((0xB6, 0x6D), (0xD12C)),  # HANGUL SYLLABLE THIEUTH YAE KHIEUKH
    ((0xB6, 0x6E), (0xD12D)),  # HANGUL SYLLABLE THIEUTH YAE THIEUTH
    ((0xB6, 0x6F), (0xD12E)),  # HANGUL SYLLABLE THIEUTH YAE PHIEUPH
    ((0xB6, 0x70), (0xD12F)),  # HANGUL SYLLABLE THIEUTH YAE HIEUH
    ((0xB6, 0x71), (0xD132)),  # HANGUL SYLLABLE THIEUTH EO SSANGKIYEOK
    ((0xB6, 0x72), (0xD133)),  # HANGUL SYLLABLE THIEUTH EO KIYEOKSIOS
    ((0xB6, 0x73), (0xD135)),  # HANGUL SYLLABLE THIEUTH EO NIEUNCIEUC
    ((0xB6, 0x74), (0xD136)),  # HANGUL SYLLABLE THIEUTH EO NIEUNHIEUH
    ((0xB6, 0x75), (0xD137)),  # HANGUL SYLLABLE THIEUTH EO TIKEUT
    ((0xB6, 0x76), (0xD139)),  # HANGUL SYLLABLE THIEUTH EO RIEULKIYEOK
    ((0xB6, 0x77), (0xD13B)),  # HANGUL SYLLABLE THIEUTH EO RIEULPIEUP
    ((0xB6, 0x78), (0xD13C)),  # HANGUL SYLLABLE THIEUTH EO RIEULSIOS
    ((0xB6, 0x79), (0xD13D)),  # HANGUL SYLLABLE THIEUTH EO RIEULTHIEUTH
    ((0xB6, 0x7A), (0xD13E)),  # HANGUL SYLLABLE THIEUTH EO RIEULPHIEUPH
    ((0xB6, 0x81), (0xD13F)),  # HANGUL SYLLABLE THIEUTH EO RIEULHIEUH
    ((0xB6, 0x82), (0xD142)),  # HANGUL SYLLABLE THIEUTH EO PIEUPSIOS
    ((0xB6, 0x83), (0xD146)),  # HANGUL SYLLABLE THIEUTH EO CIEUC
    ((0xB6, 0x84), (0xD147)),  # HANGUL SYLLABLE THIEUTH EO CHIEUCH
    ((0xB6, 0x85), (0xD148)),  # HANGUL SYLLABLE THIEUTH EO KHIEUKH
    ((0xB6, 0x86), (0xD149)),  # HANGUL SYLLABLE THIEUTH EO THIEUTH
    ((0xB6, 0x87), (0xD14A)),  # HANGUL SYLLABLE THIEUTH EO PHIEUPH
    ((0xB6, 0x88), (0xD14B)),  # HANGUL SYLLABLE THIEUTH EO HIEUH
    ((0xB6, 0x89), (0xD14E)),  # HANGUL SYLLABLE THIEUTH E SSANGKIYEOK
    ((0xB6, 0x8A), (0xD14F)),  # HANGUL SYLLABLE THIEUTH E KIYEOKSIOS
    ((0xB6, 0x8B), (0xD151)),  # HANGUL SYLLABLE THIEUTH E NIEUNCIEUC
    ((0xB6, 0x8C), (0xD152)),  # HANGUL SYLLABLE THIEUTH E NIEUNHIEUH
    ((0xB6, 0x8D), (0xD153)),  # HANGUL SYLLABLE THIEUTH E TIKEUT
    ((0xB6, 0x8E), (0xD155)),  # HANGUL SYLLABLE THIEUTH E RIEULKIYEOK
    ((0xB6, 0x8F), (0xD156)),  # HANGUL SYLLABLE THIEUTH E RIEULMIEUM
    ((0xB6, 0x90), (0xD157)),  # HANGUL SYLLABLE THIEUTH E RIEULPIEUP
    ((0xB6, 0x91), (0xD158)),  # HANGUL SYLLABLE THIEUTH E RIEULSIOS
    ((0xB6, 0x92), (0xD159)),  # HANGUL SYLLABLE THIEUTH E RIEULTHIEUTH
    ((0xB6, 0x93), (0xD15A)),  # HANGUL SYLLABLE THIEUTH E RIEULPHIEUPH
    ((0xB6, 0x94), (0xD15B)),  # HANGUL SYLLABLE THIEUTH E RIEULHIEUH
    ((0xB6, 0x95), (0xD15E)),  # HANGUL SYLLABLE THIEUTH E PIEUPSIOS
    ((0xB6, 0x96), (0xD160)),  # HANGUL SYLLABLE THIEUTH E SSANGSIOS
    ((0xB6, 0x97), (0xD162)),  # HANGUL SYLLABLE THIEUTH E CIEUC
    ((0xB6, 0x98), (0xD163)),  # HANGUL SYLLABLE THIEUTH E CHIEUCH
    ((0xB6, 0x99), (0xD164)),  # HANGUL SYLLABLE THIEUTH E KHIEUKH
    ((0xB6, 0x9A), (0xD165)),  # HANGUL SYLLABLE THIEUTH E THIEUTH
    ((0xB6, 0x9B), (0xD166)),  # HANGUL SYLLABLE THIEUTH E PHIEUPH
    ((0xB6, 0x9C), (0xD167)),  # HANGUL SYLLABLE THIEUTH E HIEUH
    ((0xB6, 0x9D), (0xD169)),  # HANGUL SYLLABLE THIEUTH YEO KIYEOK
    ((0xB6, 0x9E), (0xD16A)),  # HANGUL SYLLABLE THIEUTH YEO SSANGKIYEOK
    ((0xB6, 0x9F), (0xD16B)),  # HANGUL SYLLABLE THIEUTH YEO KIYEOKSIOS
    ((0xB6, 0xA0), (0xD16D)),  # HANGUL SYLLABLE THIEUTH YEO NIEUNCIEUC
    ((0xB6, 0xA1), (0xB540)),  # HANGUL SYLLABLE SSANGTIKEUT A MIEUM
    ((0xB6, 0xA2), (0xB541)),  # HANGUL SYLLABLE SSANGTIKEUT A PIEUP
    ((0xB6, 0xA3), (0xB543)),  # HANGUL SYLLABLE SSANGTIKEUT A SIOS
    ((0xB6, 0xA4), (0xB544)),  # HANGUL SYLLABLE SSANGTIKEUT A SSANGSIOS
    ((0xB6, 0xA5), (0xB545)),  # HANGUL SYLLABLE SSANGTIKEUT A IEUNG
    ((0xB6, 0xA6), (0xB54B)),  # HANGUL SYLLABLE SSANGTIKEUT A HIEUH
    ((0xB6, 0xA7), (0xB54C)),  # HANGUL SYLLABLE SSANGTIKEUT AE
    ((0xB6, 0xA8), (0xB54D)),  # HANGUL SYLLABLE SSANGTIKEUT AE KIYEOK
    ((0xB6, 0xA9), (0xB550)),  # HANGUL SYLLABLE SSANGTIKEUT AE NIEUN
    ((0xB6, 0xAA), (0xB554)),  # HANGUL SYLLABLE SSANGTIKEUT AE RIEUL
    ((0xB6, 0xAB), (0xB55C)),  # HANGUL SYLLABLE SSANGTIKEUT AE MIEUM
    ((0xB6, 0xAC), (0xB55D)),  # HANGUL SYLLABLE SSANGTIKEUT AE PIEUP
    ((0xB6, 0xAD), (0xB55F)),  # HANGUL SYLLABLE SSANGTIKEUT AE SIOS
    ((0xB6, 0xAE), (0xB560)),  # HANGUL SYLLABLE SSANGTIKEUT AE SSANGSIOS
    ((0xB6, 0xAF), (0xB561)),  # HANGUL SYLLABLE SSANGTIKEUT AE IEUNG
    ((0xB6, 0xB0), (0xB5A0)),  # HANGUL SYLLABLE SSANGTIKEUT EO
    ((0xB6, 0xB1), (0xB5A1)),  # HANGUL SYLLABLE SSANGTIKEUT EO KIYEOK
    ((0xB6, 0xB2), (0xB5A4)),  # HANGUL SYLLABLE SSANGTIKEUT EO NIEUN
    ((0xB6, 0xB3), (0xB5A8)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEUL
    ((0xB6, 0xB4), (0xB5AA)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULMIEUM
    ((0xB6, 0xB5), (0xB5AB)),  # HANGUL SYLLABLE SSANGTIKEUT EO RIEULPIEUP
    ((0xB6, 0xB6), (0xB5B0)),  # HANGUL SYLLABLE SSANGTIKEUT EO MIEUM
    ((0xB6, 0xB7), (0xB5B1)),  # HANGUL SYLLABLE SSANGTIKEUT EO PIEUP
    ((0xB6, 0xB8), (0xB5B3)),  # HANGUL SYLLABLE SSANGTIKEUT EO SIOS
    ((0xB6, 0xB9), (0xB5B4)),  # HANGUL SYLLABLE SSANGTIKEUT EO SSANGSIOS
    ((0xB6, 0xBA), (0xB5B5)),  # HANGUL SYLLABLE SSANGTIKEUT EO IEUNG
    ((0xB6, 0xBB), (0xB5BB)),  # HANGUL SYLLABLE SSANGTIKEUT EO HIEUH
    ((0xB6, 0xBC), (0xB5BC)),  # HANGUL SYLLABLE SSANGTIKEUT E
    ((0xB6, 0xBD), (0xB5BD)),  # HANGUL SYLLABLE SSANGTIKEUT E KIYEOK
    ((0xB6, 0xBE), (0xB5C0)),  # HANGUL SYLLABLE SSANGTIKEUT E NIEUN
    ((0xB6, 0xBF), (0xB5C4)),  # HANGUL SYLLABLE SSANGTIKEUT E RIEUL
    ((0xB6, 0xC0), (0xB5CC)),  # HANGUL SYLLABLE SSANGTIKEUT E MIEUM
    ((0xB6, 0xC1), (0xB5CD)),  # HANGUL SYLLABLE SSANGTIKEUT E PIEUP
    ((0xB6, 0xC2), (0xB5CF)),  # HANGUL SYLLABLE SSANGTIKEUT E SIOS
    ((0xB6, 0xC3), (0xB5D0)),  # HANGUL SYLLABLE SSANGTIKEUT E SSANGSIOS
    ((0xB6, 0xC4), (0xB5D1)),  # HANGUL SYLLABLE SSANGTIKEUT E IEUNG
    ((0xB6, 0xC5), (0xB5D8)),  # HANGUL SYLLABLE SSANGTIKEUT YEO
    ((0xB6, 0xC6), (0xB5EC)),  # HANGUL SYLLABLE SSANGTIKEUT YEO SSANGSIOS
    ((0xB6, 0xC7), (0xB610)),  # HANGUL SYLLABLE SSANGTIKEUT O
    ((0xB6, 0xC8), (0xB611)),  # HANGUL SYLLABLE SSANGTIKEUT O KIYEOK
    ((0xB6, 0xC9), (0xB614)),  # HANGUL SYLLABLE SSANGTIKEUT O NIEUN
    ((0xB6, 0xCA), (0xB618)),  # HANGUL SYLLABLE SSANGTIKEUT O RIEUL
    ((0xB6, 0xCB), (0xB625)),  # HANGUL SYLLABLE SSANGTIKEUT O IEUNG
    ((0xB6, 0xCC), (0xB62C)),  # HANGUL SYLLABLE SSANGTIKEUT WA
    ((0xB6, 0xCD), (0xB634)),  # HANGUL SYLLABLE SSANGTIKEUT WA RIEUL
    ((0xB6, 0xCE), (0xB648)),  # HANGUL SYLLABLE SSANGTIKEUT WAE
    ((0xB6, 0xCF), (0xB664)),  # HANGUL SYLLABLE SSANGTIKEUT OE
    ((0xB6, 0xD0), (0xB668)),  # HANGUL SYLLABLE SSANGTIKEUT OE NIEUN
    ((0xB6, 0xD1), (0xB69C)),  # HANGUL SYLLABLE SSANGTIKEUT U
    ((0xB6, 0xD2), (0xB69D)),  # HANGUL SYLLABLE SSANGTIKEUT U KIYEOK
    ((0xB6, 0xD3), (0xB6A0)),  # HANGUL SYLLABLE SSANGTIKEUT U NIEUN
    ((0xB6, 0xD4), (0xB6A4)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEUL
    ((0xB6, 0xD5), (0xB6AB)),  # HANGUL SYLLABLE SSANGTIKEUT U RIEULHIEUH
    ((0xB6, 0xD6), (0xB6AC)),  # HANGUL SYLLABLE SSANGTIKEUT U MIEUM
    ((0xB6, 0xD7), (0xB6B1)),  # HANGUL SYLLABLE SSANGTIKEUT U IEUNG
    ((0xB6, 0xD8), (0xB6D4)),  # HANGUL SYLLABLE SSANGTIKEUT WE
    ((0xB6, 0xD9), (0xB6F0)),  # HANGUL SYLLABLE SSANGTIKEUT WI
    ((0xB6, 0xDA), (0xB6F4)),  # HANGUL SYLLABLE SSANGTIKEUT WI NIEUN
    ((0xB6, 0xDB), (0xB6F8)),  # HANGUL SYLLABLE SSANGTIKEUT WI RIEUL
    ((0xB6, 0xDC), (0xB700)),  # HANGUL SYLLABLE SSANGTIKEUT WI MIEUM
    ((0xB6, 0xDD), (0xB701)),  # HANGUL SYLLABLE SSANGTIKEUT WI PIEUP
    ((0xB6, 0xDE), (0xB705)),  # HANGUL SYLLABLE SSANGTIKEUT WI IEUNG
    ((0xB6, 0xDF), (0xB728)),  # HANGUL SYLLABLE SSANGTIKEUT EU
    ((0xB6, 0xE0), (0xB729)),  # HANGUL SYLLABLE SSANGTIKEUT EU KIYEOK
    ((0xB6, 0xE1), (0xB72C)),  # HANGUL SYLLABLE SSANGTIKEUT EU NIEUN
    ((0xB6, 0xE2), (0xB72F)),  # HANGUL SYLLABLE SSANGTIKEUT EU TIKEUT
    ((0xB6, 0xE3), (0xB730)),  # HANGUL SYLLABLE SSANGTIKEUT EU RIEUL
    ((0xB6, 0xE4), (0xB738)),  # HANGUL SYLLABLE SSANGTIKEUT EU MIEUM
    ((0xB6, 0xE5), (0xB739)),  # HANGUL SYLLABLE SSANGTIKEUT EU PIEUP
    ((0xB6, 0xE6), (0xB73B)),  # HANGUL SYLLABLE SSANGTIKEUT EU SIOS
    ((0xB6, 0xE7), (0xB744)),  # HANGUL SYLLABLE SSANGTIKEUT YI
    ((0xB6, 0xE8), (0xB748)),  # HANGUL SYLLABLE SSANGTIKEUT YI NIEUN
    ((0xB6, 0xE9), (0xB74C)),  # HANGUL SYLLABLE SSANGTIKEUT YI RIEUL
    ((0xB6, 0xEA), (0xB754)),  # HANGUL SYLLABLE SSANGTIKEUT YI MIEUM
    ((0xB6, 0xEB), (0xB755)),  # HANGUL SYLLABLE SSANGTIKEUT YI PIEUP
    ((0xB6, 0xEC), (0xB760)),  # HANGUL SYLLABLE SSANGTIKEUT I
    ((0xB6, 0xED), (0xB764)),  # HANGUL SYLLABLE SSANGTIKEUT I NIEUN
    ((0xB6, 0xEE), (0xB768)),  # HANGUL SYLLABLE SSANGTIKEUT I RIEUL
    ((0xB6, 0xEF), (0xB770)),  # HANGUL SYLLABLE SSANGTIKEUT I MIEUM
    ((0xB6, 0xF0), (0xB771)),  # HANGUL SYLLABLE SSANGTIKEUT I PIEUP
    ((0xB6, 0xF1), (0xB773)),  # HANGUL SYLLABLE SSANGTIKEUT I SIOS
    ((0xB6, 0xF2), (0xB775)),  # HANGUL SYLLABLE SSANGTIKEUT I IEUNG
    ((0xB6, 0xF3), (0xB77C)),  # HANGUL SYLLABLE RIEUL A
    ((0xB6, 0xF4), (0xB77D)),  # HANGUL SYLLABLE RIEUL A KIYEOK
    ((0xB6, 0xF5), (0xB780)),  # HANGUL SYLLABLE RIEUL A NIEUN
    ((0xB6, 0xF6), (0xB784)),  # HANGUL SYLLABLE RIEUL A RIEUL
    ((0xB6, 0xF7), (0xB78C)),  # HANGUL SYLLABLE RIEUL A MIEUM
    ((0xB6, 0xF8), (0xB78D)),  # HANGUL SYLLABLE RIEUL A PIEUP
    ((0xB6, 0xF9), (0xB78F)),  # HANGUL SYLLABLE RIEUL A SIOS
    ((0xB6, 0xFA), (0xB790)),  # HANGUL SYLLABLE RIEUL A SSANGSIOS
    ((0xB6, 0xFB), (0xB791)),  # HANGUL SYLLABLE RIEUL A IEUNG
    ((0xB6, 0xFC), (0xB792)),  # HANGUL SYLLABLE RIEUL A CIEUC
    ((0xB6, 0xFD), (0xB796)),  # HANGUL SYLLABLE RIEUL A PHIEUPH
    ((0xB6, 0xFE), (0xB797)),  # HANGUL SYLLABLE RIEUL A HIEUH
    ((0xB7, 0x41), (0xD16E)),  # HANGUL SYLLABLE THIEUTH YEO NIEUNHIEUH
    ((0xB7, 0x42), (0xD16F)),  # HANGUL SYLLABLE THIEUTH YEO TIKEUT
    ((0xB7, 0x43), (0xD170)),  # HANGUL SYLLABLE THIEUTH YEO RIEUL
    ((0xB7, 0x44), (0xD171)),  # HANGUL SYLLABLE THIEUTH YEO RIEULKIYEOK
    ((0xB7, 0x45), (0xD172)),  # HANGUL SYLLABLE THIEUTH YEO RIEULMIEUM
    ((0xB7, 0x46), (0xD173)),  # HANGUL SYLLABLE THIEUTH YEO RIEULPIEUP
    ((0xB7, 0x47), (0xD174)),  # HANGUL SYLLABLE THIEUTH YEO RIEULSIOS
    ((0xB7, 0x48), (0xD175)),  # HANGUL SYLLABLE THIEUTH YEO RIEULTHIEUTH
    ((0xB7, 0x49), (0xD176)),  # HANGUL SYLLABLE THIEUTH YEO RIEULPHIEUPH
    ((0xB7, 0x4A), (0xD177)),  # HANGUL SYLLABLE THIEUTH YEO RIEULHIEUH
    ((0xB7, 0x4B), (0xD178)),  # HANGUL SYLLABLE THIEUTH YEO MIEUM
    ((0xB7, 0x4C), (0xD179)),  # HANGUL SYLLABLE THIEUTH YEO PIEUP
    ((0xB7, 0x4D), (0xD17A)),  # HANGUL SYLLABLE THIEUTH YEO PIEUPSIOS
    ((0xB7, 0x4E), (0xD17B)),  # HANGUL SYLLABLE THIEUTH YEO SIOS
    ((0xB7, 0x4F), (0xD17D)),  # HANGUL SYLLABLE THIEUTH YEO IEUNG
    ((0xB7, 0x50), (0xD17E)),  # HANGUL SYLLABLE THIEUTH YEO CIEUC
    ((0xB7, 0x51), (0xD17F)),  # HANGUL SYLLABLE THIEUTH YEO CHIEUCH
    ((0xB7, 0x52), (0xD180)),  # HANGUL SYLLABLE THIEUTH YEO KHIEUKH
    ((0xB7, 0x53), (0xD181)),  # HANGUL SYLLABLE THIEUTH YEO THIEUTH
    ((0xB7, 0x54), (0xD182)),  # HANGUL SYLLABLE THIEUTH YEO PHIEUPH
    ((0xB7, 0x55), (0xD183)),  # HANGUL SYLLABLE THIEUTH YEO HIEUH
    ((0xB7, 0x56), (0xD185)),  # HANGUL SYLLABLE THIEUTH YE KIYEOK
    ((0xB7, 0x57), (0xD186)),  # HANGUL SYLLABLE THIEUTH YE SSANGKIYEOK
    ((0xB7, 0x58), (0xD187)),  # HANGUL SYLLABLE THIEUTH YE KIYEOKSIOS
    ((0xB7, 0x59), (0xD189)),  # HANGUL SYLLABLE THIEUTH YE NIEUNCIEUC
    ((0xB7, 0x5A), (0xD18A)),  # HANGUL SYLLABLE THIEUTH YE NIEUNHIEUH
    ((0xB7, 0x61), (0xD18B)),  # HANGUL SYLLABLE THIEUTH YE TIKEUT
    ((0xB7, 0x62), (0xD18C)),  # HANGUL SYLLABLE THIEUTH YE RIEUL
    ((0xB7, 0x63), (0xD18D)),  # HANGUL SYLLABLE THIEUTH YE RIEULKIYEOK
    ((0xB7, 0x64), (0xD18E)),  # HANGUL SYLLABLE THIEUTH YE RIEULMIEUM
    ((0xB7, 0x65), (0xD18F)),  # HANGUL SYLLABLE THIEUTH YE RIEULPIEUP
    ((0xB7, 0x66), (0xD190)),  # HANGUL SYLLABLE THIEUTH YE RIEULSIOS
    ((0xB7, 0x67), (0xD191)),  # HANGUL SYLLABLE THIEUTH YE RIEULTHIEUTH
    ((0xB7, 0x68), (0xD192)),  # HANGUL SYLLABLE THIEUTH YE RIEULPHIEUPH
    ((0xB7, 0x69), (0xD193)),  # HANGUL SYLLABLE THIEUTH YE RIEULHIEUH
    ((0xB7, 0x6A), (0xD194)),  # HANGUL SYLLABLE THIEUTH YE MIEUM
    ((0xB7, 0x6B), (0xD195)),  # HANGUL SYLLABLE THIEUTH YE PIEUP
    ((0xB7, 0x6C), (0xD196)),  # HANGUL SYLLABLE THIEUTH YE PIEUPSIOS
    ((0xB7, 0x6D), (0xD197)),  # HANGUL SYLLABLE THIEUTH YE SIOS
    ((0xB7, 0x6E), (0xD198)),  # HANGUL SYLLABLE THIEUTH YE SSANGSIOS
    ((0xB7, 0x6F), (0xD199)),  # HANGUL SYLLABLE THIEUTH YE IEUNG
    ((0xB7, 0x70), (0xD19A)),  # HANGUL SYLLABLE THIEUTH YE CIEUC
    ((0xB7, 0x71), (0xD19B)),  # HANGUL SYLLABLE THIEUTH YE CHIEUCH
    ((0xB7, 0x72), (0xD19C)),  # HANGUL SYLLABLE THIEUTH YE KHIEUKH
    ((0xB7, 0x73), (0xD19D)),  # HANGUL SYLLABLE THIEUTH YE THIEUTH
    ((0xB7, 0x74), (0xD19E)),  # HANGUL SYLLABLE THIEUTH YE PHIEUPH
    ((0xB7, 0x75), (0xD19F)),  # HANGUL SYLLABLE THIEUTH YE HIEUH
    ((0xB7, 0x76), (0xD1A2)),  # HANGUL SYLLABLE THIEUTH O SSANGKIYEOK
    ((0xB7, 0x77), (0xD1A3)),  # HANGUL SYLLABLE THIEUTH O KIYEOKSIOS
    ((0xB7, 0x78), (0xD1A5)),  # HANGUL SYLLABLE THIEUTH O NIEUNCIEUC
    ((0xB7, 0x79), (0xD1A6)),  # HANGUL SYLLABLE THIEUTH O NIEUNHIEUH
    ((0xB7, 0x7A), (0xD1A7)),  # HANGUL SYLLABLE THIEUTH O TIKEUT
    ((0xB7, 0x81), (0xD1A9)),  # HANGUL SYLLABLE THIEUTH O RIEULKIYEOK
    ((0xB7, 0x82), (0xD1AA)),  # HANGUL SYLLABLE THIEUTH O RIEULMIEUM
    ((0xB7, 0x83), (0xD1AB)),  # HANGUL SYLLABLE THIEUTH O RIEULPIEUP
    ((0xB7, 0x84), (0xD1AC)),  # HANGUL SYLLABLE THIEUTH O RIEULSIOS
    ((0xB7, 0x85), (0xD1AD)),  # HANGUL SYLLABLE THIEUTH O RIEULTHIEUTH
    ((0xB7, 0x86), (0xD1AE)),  # HANGUL SYLLABLE THIEUTH O RIEULPHIEUPH
    ((0xB7, 0x87), (0xD1AF)),  # HANGUL SYLLABLE THIEUTH O RIEULHIEUH
    ((0xB7, 0x88), (0xD1B2)),  # HANGUL SYLLABLE THIEUTH O PIEUPSIOS
    ((0xB7, 0x89), (0xD1B4)),  # HANGUL SYLLABLE THIEUTH O SSANGSIOS
    ((0xB7, 0x8A), (0xD1B6)),  # HANGUL SYLLABLE THIEUTH O CIEUC
    ((0xB7, 0x8B), (0xD1B7)),  # HANGUL SYLLABLE THIEUTH O CHIEUCH
    ((0xB7, 0x8C), (0xD1B8)),  # HANGUL SYLLABLE THIEUTH O KHIEUKH
    ((0xB7, 0x8D), (0xD1B9)),  # HANGUL SYLLABLE THIEUTH O THIEUTH
    ((0xB7, 0x8E), (0xD1BB)),  # HANGUL SYLLABLE THIEUTH O HIEUH
    ((0xB7, 0x8F), (0xD1BD)),  # HANGUL SYLLABLE THIEUTH WA KIYEOK
    ((0xB7, 0x90), (0xD1BE)),  # HANGUL SYLLABLE THIEUTH WA SSANGKIYEOK
    ((0xB7, 0x91), (0xD1BF)),  # HANGUL SYLLABLE THIEUTH WA KIYEOKSIOS
    ((0xB7, 0x92), (0xD1C1)),  # HANGUL SYLLABLE THIEUTH WA NIEUNCIEUC
    ((0xB7, 0x93), (0xD1C2)),  # HANGUL SYLLABLE THIEUTH WA NIEUNHIEUH
    ((0xB7, 0x94), (0xD1C3)),  # HANGUL SYLLABLE THIEUTH WA TIKEUT
    ((0xB7, 0x95), (0xD1C4)),  # HANGUL SYLLABLE THIEUTH WA RIEUL
    ((0xB7, 0x96), (0xD1C5)),  # HANGUL SYLLABLE THIEUTH WA RIEULKIYEOK
    ((0xB7, 0x97), (0xD1C6)),  # HANGUL SYLLABLE THIEUTH WA RIEULMIEUM
    ((0xB7, 0x98), (0xD1C7)),  # HANGUL SYLLABLE THIEUTH WA RIEULPIEUP
    ((0xB7, 0x99), (0xD1C8)),  # HANGUL SYLLABLE THIEUTH WA RIEULSIOS
    ((0xB7, 0x9A), (0xD1C9)),  # HANGUL SYLLABLE THIEUTH WA RIEULTHIEUTH
    ((0xB7, 0x9B), (0xD1CA)),  # HANGUL SYLLABLE THIEUTH WA RIEULPHIEUPH
    ((0xB7, 0x9C), (0xD1CB)),  # HANGUL SYLLABLE THIEUTH WA RIEULHIEUH
    ((0xB7, 0x9D), (0xD1CC)),  # HANGUL SYLLABLE THIEUTH WA MIEUM
    ((0xB7, 0x9E), (0xD1CD)),  # HANGUL SYLLABLE THIEUTH WA PIEUP
    ((0xB7, 0x9F), (0xD1CE)),  # HANGUL SYLLABLE THIEUTH WA PIEUPSIOS
    ((0xB7, 0xA0), (0xD1CF)),  # HANGUL SYLLABLE THIEUTH WA SIOS
    ((0xB7, 0xA1), (0xB798)),  # HANGUL SYLLABLE RIEUL AE
    ((0xB7, 0xA2), (0xB799)),  # HANGUL SYLLABLE RIEUL AE KIYEOK
    ((0xB7, 0xA3), (0xB79C)),  # HANGUL SYLLABLE RIEUL AE NIEUN
    ((0xB7, 0xA4), (0xB7A0)),  # HANGUL SYLLABLE RIEUL AE RIEUL
    ((0xB7, 0xA5), (0xB7A8)),  # HANGUL SYLLABLE RIEUL AE MIEUM
    ((0xB7, 0xA6), (0xB7A9)),  # HANGUL SYLLABLE RIEUL AE PIEUP
    ((0xB7, 0xA7), (0xB7AB)),  # HANGUL SYLLABLE RIEUL AE SIOS
    ((0xB7, 0xA8), (0xB7AC)),  # HANGUL SYLLABLE RIEUL AE SSANGSIOS
    ((0xB7, 0xA9), (0xB7AD)),  # HANGUL SYLLABLE RIEUL AE IEUNG
    ((0xB7, 0xAA), (0xB7B4)),  # HANGUL SYLLABLE RIEUL YA
    ((0xB7, 0xAB), (0xB7B5)),  # HANGUL SYLLABLE RIEUL YA KIYEOK
    ((0xB7, 0xAC), (0xB7B8)),  # HANGUL SYLLABLE RIEUL YA NIEUN
    ((0xB7, 0xAD), (0xB7C7)),  # HANGUL SYLLABLE RIEUL YA SIOS
    ((0xB7, 0xAE), (0xB7C9)),  # HANGUL SYLLABLE RIEUL YA IEUNG
    ((0xB7, 0xAF), (0xB7EC)),  # HANGUL SYLLABLE RIEUL EO
    ((0xB7, 0xB0), (0xB7ED)),  # HANGUL SYLLABLE RIEUL EO KIYEOK
    ((0xB7, 0xB1), (0xB7F0)),  # HANGUL SYLLABLE RIEUL EO NIEUN
    ((0xB7, 0xB2), (0xB7F4)),  # HANGUL SYLLABLE RIEUL EO RIEUL
    ((0xB7, 0xB3), (0xB7FC)),  # HANGUL SYLLABLE RIEUL EO MIEUM
    ((0xB7, 0xB4), (0xB7FD)),  # HANGUL SYLLABLE RIEUL EO PIEUP
    ((0xB7, 0xB5), (0xB7FF)),  # HANGUL SYLLABLE RIEUL EO SIOS
    ((0xB7, 0xB6), (0xB800)),  # HANGUL SYLLABLE RIEUL EO SSANGSIOS
    ((0xB7, 0xB7), (0xB801)),  # HANGUL SYLLABLE RIEUL EO IEUNG
    ((0xB7, 0xB8), (0xB807)),  # HANGUL SYLLABLE RIEUL EO HIEUH
    ((0xB7, 0xB9), (0xB808)),  # HANGUL SYLLABLE RIEUL E
    ((0xB7, 0xBA), (0xB809)),  # HANGUL SYLLABLE RIEUL E KIYEOK
    ((0xB7, 0xBB), (0xB80C)),  # HANGUL SYLLABLE RIEUL E NIEUN
    ((0xB7, 0xBC), (0xB810)),  # HANGUL SYLLABLE RIEUL E RIEUL
    ((0xB7, 0xBD), (0xB818)),  # HANGUL SYLLABLE RIEUL E MIEUM
    ((0xB7, 0xBE), (0xB819)),  # HANGUL SYLLABLE RIEUL E PIEUP
    ((0xB7, 0xBF), (0xB81B)),  # HANGUL SYLLABLE RIEUL E SIOS
    ((0xB7, 0xC0), (0xB81D)),  # HANGUL SYLLABLE RIEUL E IEUNG
    ((0xB7, 0xC1), (0xB824)),  # HANGUL SYLLABLE RIEUL YEO
    ((0xB7, 0xC2), (0xB825)),  # HANGUL SYLLABLE RIEUL YEO KIYEOK
    ((0xB7, 0xC3), (0xB828)),  # HANGUL SYLLABLE RIEUL YEO NIEUN
    ((0xB7, 0xC4), (0xB82C)),  # HANGUL SYLLABLE RIEUL YEO RIEUL
    ((0xB7, 0xC5), (0xB834)),  # HANGUL SYLLABLE RIEUL YEO MIEUM
    ((0xB7, 0xC6), (0xB835)),  # HANGUL SYLLABLE RIEUL YEO PIEUP
    ((0xB7, 0xC7), (0xB837)),  # HANGUL SYLLABLE RIEUL YEO SIOS
    ((0xB7, 0xC8), (0xB838)),  # HANGUL SYLLABLE RIEUL YEO SSANGSIOS
    ((0xB7, 0xC9), (0xB839)),  # HANGUL SYLLABLE RIEUL YEO IEUNG
    ((0xB7, 0xCA), (0xB840)),  # HANGUL SYLLABLE RIEUL YE
    ((0xB7, 0xCB), (0xB844)),  # HANGUL SYLLABLE RIEUL YE NIEUN
    ((0xB7, 0xCC), (0xB851)),  # HANGUL SYLLABLE RIEUL YE PIEUP
    ((0xB7, 0xCD), (0xB853)),  # HANGUL SYLLABLE RIEUL YE SIOS
    ((0xB7, 0xCE), (0xB85C)),  # HANGUL SYLLABLE RIEUL O
    ((0xB7, 0xCF), (0xB85D)),  # HANGUL SYLLABLE RIEUL O KIYEOK
    ((0xB7, 0xD0), (0xB860)),  # HANGUL SYLLABLE RIEUL O NIEUN
    ((0xB7, 0xD1), (0xB864)),  # HANGUL SYLLABLE RIEUL O RIEUL
    ((0xB7, 0xD2), (0xB86C)),  # HANGUL SYLLABLE RIEUL O MIEUM
    ((0xB7, 0xD3), (0xB86D)),  # HANGUL SYLLABLE RIEUL O PIEUP
    ((0xB7, 0xD4), (0xB86F)),  # HANGUL SYLLABLE RIEUL O SIOS
    ((0xB7, 0xD5), (0xB871)),  # HANGUL SYLLABLE RIEUL O IEUNG
    ((0xB7, 0xD6), (0xB878)),  # HANGUL SYLLABLE RIEUL WA
    ((0xB7, 0xD7), (0xB87C)),  # HANGUL SYLLABLE RIEUL WA NIEUN
    ((0xB7, 0xD8), (0xB88D)),  # HANGUL SYLLABLE RIEUL WA IEUNG
    ((0xB7, 0xD9), (0xB8A8)),  # HANGUL SYLLABLE RIEUL WAE SSANGSIOS
    ((0xB7, 0xDA), (0xB8B0)),  # HANGUL SYLLABLE RIEUL OE
    ((0xB7, 0xDB), (0xB8B4)),  # HANGUL SYLLABLE RIEUL OE NIEUN
    ((0xB7, 0xDC), (0xB8B8)),  # HANGUL SYLLABLE RIEUL OE RIEUL
    ((0xB7, 0xDD), (0xB8C0)),  # HANGUL SYLLABLE RIEUL OE MIEUM
    ((0xB7, 0xDE), (0xB8C1)),  # HANGUL SYLLABLE RIEUL OE PIEUP
    ((0xB7, 0xDF), (0xB8C3)),  # HANGUL SYLLABLE RIEUL OE SIOS
    ((0xB7, 0xE0), (0xB8C5)),  # HANGUL SYLLABLE RIEUL OE IEUNG
    ((0xB7, 0xE1), (0xB8CC)),  # HANGUL SYLLABLE RIEUL YO
    ((0xB7, 0xE2), (0xB8D0)),  # HANGUL SYLLABLE RIEUL YO NIEUN
    ((0xB7, 0xE3), (0xB8D4)),  # HANGUL SYLLABLE RIEUL YO RIEUL
    ((0xB7, 0xE4), (0xB8DD)),  # HANGUL SYLLABLE RIEUL YO PIEUP
    ((0xB7, 0xE5), (0xB8DF)),  # HANGUL SYLLABLE RIEUL YO SIOS
    ((0xB7, 0xE6), (0xB8E1)),  # HANGUL SYLLABLE RIEUL YO IEUNG
    ((0xB7, 0xE7), (0xB8E8)),  # HANGUL SYLLABLE RIEUL U
    ((0xB7, 0xE8), (0xB8E9)),  # HANGUL SYLLABLE RIEUL U KIYEOK
    ((0xB7, 0xE9), (0xB8EC)),  # HANGUL SYLLABLE RIEUL U NIEUN
    ((0xB7, 0xEA), (0xB8F0)),  # HANGUL SYLLABLE RIEUL U RIEUL
    ((0xB7, 0xEB), (0xB8F8)),  # HANGUL SYLLABLE RIEUL U MIEUM
    ((0xB7, 0xEC), (0xB8F9)),  # HANGUL SYLLABLE RIEUL U PIEUP
    ((0xB7, 0xED), (0xB8FB)),  # HANGUL SYLLABLE RIEUL U SIOS
    ((0xB7, 0xEE), (0xB8FD)),  # HANGUL SYLLABLE RIEUL U IEUNG
    ((0xB7, 0xEF), (0xB904)),  # HANGUL SYLLABLE RIEUL WEO
    ((0xB7, 0xF0), (0xB918)),  # HANGUL SYLLABLE RIEUL WEO SSANGSIOS
    ((0xB7, 0xF1), (0xB920)),  # HANGUL SYLLABLE RIEUL WE
    ((0xB7, 0xF2), (0xB93C)),  # HANGUL SYLLABLE RIEUL WI
    ((0xB7, 0xF3), (0xB93D)),  # HANGUL SYLLABLE RIEUL WI KIYEOK
    ((0xB7, 0xF4), (0xB940)),  # HANGUL SYLLABLE RIEUL WI NIEUN
    ((0xB7, 0xF5), (0xB944)),  # HANGUL SYLLABLE RIEUL WI RIEUL
    ((0xB7, 0xF6), (0xB94C)),  # HANGUL SYLLABLE RIEUL WI MIEUM
    ((0xB7, 0xF7), (0xB94F)),  # HANGUL SYLLABLE RIEUL WI SIOS
    ((0xB7, 0xF8), (0xB951)),  # HANGUL SYLLABLE RIEUL WI IEUNG
    ((0xB7, 0xF9), (0xB958)),  # HANGUL SYLLABLE RIEUL YU
    ((0xB7, 0xFA), (0xB959)),  # HANGUL SYLLABLE RIEUL YU KIYEOK
    ((0xB7, 0xFB), (0xB95C)),  # HANGUL SYLLABLE RIEUL YU NIEUN
    ((0xB7, 0xFC), (0xB960)),  # HANGUL SYLLABLE RIEUL YU RIEUL
    ((0xB7, 0xFD), (0xB968)),  # HANGUL SYLLABLE RIEUL YU MIEUM
    ((0xB7, 0xFE), (0xB969)),  # HANGUL SYLLABLE RIEUL YU PIEUP
    ((0xB8, 0x41), (0xD1D0)),  # HANGUL SYLLABLE THIEUTH WA SSANGSIOS
    ((0xB8, 0x42), (0xD1D1)),  # HANGUL SYLLABLE THIEUTH WA IEUNG
    ((0xB8, 0x43), (0xD1D2)),  # HANGUL SYLLABLE THIEUTH WA CIEUC
    ((0xB8, 0x44), (0xD1D3)),  # HANGUL SYLLABLE THIEUTH WA CHIEUCH
    ((0xB8, 0x45), (0xD1D4)),  # HANGUL SYLLABLE THIEUTH WA KHIEUKH
    ((0xB8, 0x46), (0xD1D5)),  # HANGUL SYLLABLE THIEUTH WA THIEUTH
    ((0xB8, 0x47), (0xD1D6)),  # HANGUL SYLLABLE THIEUTH WA PHIEUPH
    ((0xB8, 0x48), (0xD1D7)),  # HANGUL SYLLABLE THIEUTH WA HIEUH
    ((0xB8, 0x49), (0xD1D9)),  # HANGUL SYLLABLE THIEUTH WAE KIYEOK
    ((0xB8, 0x4A), (0xD1DA)),  # HANGUL SYLLABLE THIEUTH WAE SSANGKIYEOK
    ((0xB8, 0x4B), (0xD1DB)),  # HANGUL SYLLABLE THIEUTH WAE KIYEOKSIOS
    ((0xB8, 0x4C), (0xD1DC)),  # HANGUL SYLLABLE THIEUTH WAE NIEUN
    ((0xB8, 0x4D), (0xD1DD)),  # HANGUL SYLLABLE THIEUTH WAE NIEUNCIEUC
    ((0xB8, 0x4E), (0xD1DE)),  # HANGUL SYLLABLE THIEUTH WAE NIEUNHIEUH
    ((0xB8, 0x4F), (0xD1DF)),  # HANGUL SYLLABLE THIEUTH WAE TIKEUT
    ((0xB8, 0x50), (0xD1E0)),  # HANGUL SYLLABLE THIEUTH WAE RIEUL
    ((0xB8, 0x51), (0xD1E1)),  # HANGUL SYLLABLE THIEUTH WAE RIEULKIYEOK
    ((0xB8, 0x52), (0xD1E2)),  # HANGUL SYLLABLE THIEUTH WAE RIEULMIEUM
    ((0xB8, 0x53), (0xD1E3)),  # HANGUL SYLLABLE THIEUTH WAE RIEULPIEUP
    ((0xB8, 0x54), (0xD1E4)),  # HANGUL SYLLABLE THIEUTH WAE RIEULSIOS
    ((0xB8, 0x55), (0xD1E5)),  # HANGUL SYLLABLE THIEUTH WAE RIEULTHIEUTH
    ((0xB8, 0x56), (0xD1E6)),  # HANGUL SYLLABLE THIEUTH WAE RIEULPHIEUPH
    ((0xB8, 0x57), (0xD1E7)),  # HANGUL SYLLABLE THIEUTH WAE RIEULHIEUH
    ((0xB8, 0x58), (0xD1E8)),  # HANGUL SYLLABLE THIEUTH WAE MIEUM
    ((0xB8, 0x59), (0xD1E9)),  # HANGUL SYLLABLE THIEUTH WAE PIEUP
    ((0xB8, 0x5A), (0xD1EA)),  # HANGUL SYLLABLE THIEUTH WAE PIEUPSIOS
    ((0xB8, 0x61), (0xD1EB)),  # HANGUL SYLLABLE THIEUTH WAE SIOS
    ((0xB8, 0x62), (0xD1EC)),  # HANGUL SYLLABLE THIEUTH WAE SSANGSIOS
    ((0xB8, 0x63), (0xD1ED)),  # HANGUL SYLLABLE THIEUTH WAE IEUNG
    ((0xB8, 0x64), (0xD1EE)),  # HANGUL SYLLABLE THIEUTH WAE CIEUC
    ((0xB8, 0x65), (0xD1EF)),  # HANGUL SYLLABLE THIEUTH WAE CHIEUCH
    ((0xB8, 0x66), (0xD1F0)),  # HANGUL SYLLABLE THIEUTH WAE KHIEUKH
    ((0xB8, 0x67), (0xD1F1)),  # HANGUL SYLLABLE THIEUTH WAE THIEUTH
    ((0xB8, 0x68), (0xD1F2)),  # HANGUL SYLLABLE THIEUTH WAE PHIEUPH
    ((0xB8, 0x69), (0xD1F3)),  # HANGUL SYLLABLE THIEUTH WAE HIEUH
    ((0xB8, 0x6A), (0xD1F5)),  # HANGUL SYLLABLE THIEUTH OE KIYEOK
    ((0xB8, 0x6B), (0xD1F6)),  # HANGUL SYLLABLE THIEUTH OE SSANGKIYEOK
    ((0xB8, 0x6C), (0xD1F7)),  # HANGUL SYLLABLE THIEUTH OE KIYEOKSIOS
    ((0xB8, 0x6D), (0xD1F9)),  # HANGUL SYLLABLE THIEUTH OE NIEUNCIEUC
    ((0xB8, 0x6E), (0xD1FA)),  # HANGUL SYLLABLE THIEUTH OE NIEUNHIEUH
    ((0xB8, 0x6F), (0xD1FB)),  # HANGUL SYLLABLE THIEUTH OE TIKEUT
    ((0xB8, 0x70), (0xD1FC)),  # HANGUL SYLLABLE THIEUTH OE RIEUL
    ((0xB8, 0x71), (0xD1FD)),  # HANGUL SYLLABLE THIEUTH OE RIEULKIYEOK
    ((0xB8, 0x72), (0xD1FE)),  # HANGUL SYLLABLE THIEUTH OE RIEULMIEUM
    ((0xB8, 0x73), (0xD1FF)),  # HANGUL SYLLABLE THIEUTH OE RIEULPIEUP
    ((0xB8, 0x74), (0xD200)),  # HANGUL SYLLABLE THIEUTH OE RIEULSIOS
    ((0xB8, 0x75), (0xD201)),  # HANGUL SYLLABLE THIEUTH OE RIEULTHIEUTH
    ((0xB8, 0x76), (0xD202)),  # HANGUL SYLLABLE THIEUTH OE RIEULPHIEUPH
    ((0xB8, 0x77), (0xD203)),  # HANGUL SYLLABLE THIEUTH OE RIEULHIEUH
    ((0xB8, 0x78), (0xD204)),  # HANGUL SYLLABLE THIEUTH OE MIEUM
    ((0xB8, 0x79), (0xD205)),  # HANGUL SYLLABLE THIEUTH OE PIEUP
    ((0xB8, 0x7A), (0xD206)),  # HANGUL SYLLABLE THIEUTH OE PIEUPSIOS
    ((0xB8, 0x81), (0xD208)),  # HANGUL SYLLABLE THIEUTH OE SSANGSIOS
    ((0xB8, 0x82), (0xD20A)),  # HANGUL SYLLABLE THIEUTH OE CIEUC
    ((0xB8, 0x83), (0xD20B)),  # HANGUL SYLLABLE THIEUTH OE CHIEUCH
    ((0xB8, 0x84), (0xD20C)),  # HANGUL SYLLABLE THIEUTH OE KHIEUKH
    ((0xB8, 0x85), (0xD20D)),  # HANGUL SYLLABLE THIEUTH OE THIEUTH
    ((0xB8, 0x86), (0xD20E)),  # HANGUL SYLLABLE THIEUTH OE PHIEUPH
    ((0xB8, 0x87), (0xD20F)),  # HANGUL SYLLABLE THIEUTH OE HIEUH
    ((0xB8, 0x88), (0xD211)),  # HANGUL SYLLABLE THIEUTH YO KIYEOK
    ((0xB8, 0x89), (0xD212)),  # HANGUL SYLLABLE THIEUTH YO SSANGKIYEOK
    ((0xB8, 0x8A), (0xD213)),  # HANGUL SYLLABLE THIEUTH YO KIYEOKSIOS
    ((0xB8, 0x8B), (0xD214)),  # HANGUL SYLLABLE THIEUTH YO NIEUN
    ((0xB8, 0x8C), (0xD215)),  # HANGUL SYLLABLE THIEUTH YO NIEUNCIEUC
    ((0xB8, 0x8D), (0xD216)),  # HANGUL SYLLABLE THIEUTH YO NIEUNHIEUH
    ((0xB8, 0x8E), (0xD217)),  # HANGUL SYLLABLE THIEUTH YO TIKEUT
    ((0xB8, 0x8F), (0xD218)),  # HANGUL SYLLABLE THIEUTH YO RIEUL
    ((0xB8, 0x90), (0xD219)),  # HANGUL SYLLABLE THIEUTH YO RIEULKIYEOK
    ((0xB8, 0x91), (0xD21A)),  # HANGUL SYLLABLE THIEUTH YO RIEULMIEUM
    ((0xB8, 0x92), (0xD21B)),  # HANGUL SYLLABLE THIEUTH YO RIEULPIEUP
    ((0xB8, 0x93), (0xD21C)),  # HANGUL SYLLABLE THIEUTH YO RIEULSIOS
    ((0xB8, 0x94), (0xD21D)),  # HANGUL SYLLABLE THIEUTH YO RIEULTHIEUTH
    ((0xB8, 0x95), (0xD21E)),  # HANGUL SYLLABLE THIEUTH YO RIEULPHIEUPH
    ((0xB8, 0x96), (0xD21F)),  # HANGUL SYLLABLE THIEUTH YO RIEULHIEUH
    ((0xB8, 0x97), (0xD220)),  # HANGUL SYLLABLE THIEUTH YO MIEUM
    ((0xB8, 0x98), (0xD221)),  # HANGUL SYLLABLE THIEUTH YO PIEUP
    ((0xB8, 0x99), (0xD222)),  # HANGUL SYLLABLE THIEUTH YO PIEUPSIOS
    ((0xB8, 0x9A), (0xD223)),  # HANGUL SYLLABLE THIEUTH YO SIOS
    ((0xB8, 0x9B), (0xD224)),  # HANGUL SYLLABLE THIEUTH YO SSANGSIOS
    ((0xB8, 0x9C), (0xD225)),  # HANGUL SYLLABLE THIEUTH YO IEUNG
    ((0xB8, 0x9D), (0xD226)),  # HANGUL SYLLABLE THIEUTH YO CIEUC
    ((0xB8, 0x9E), (0xD227)),  # HANGUL SYLLABLE THIEUTH YO CHIEUCH
    ((0xB8, 0x9F), (0xD228)),  # HANGUL SYLLABLE THIEUTH YO KHIEUKH
    ((0xB8, 0xA0), (0xD229)),  # HANGUL SYLLABLE THIEUTH YO THIEUTH
    ((0xB8, 0xA1), (0xB96B)),  # HANGUL SYLLABLE RIEUL YU SIOS
    ((0xB8, 0xA2), (0xB96D)),  # HANGUL SYLLABLE RIEUL YU IEUNG
    ((0xB8, 0xA3), (0xB974)),  # HANGUL SYLLABLE RIEUL EU
    ((0xB8, 0xA4), (0xB975)),  # HANGUL SYLLABLE RIEUL EU KIYEOK
    ((0xB8, 0xA5), (0xB978)),  # HANGUL SYLLABLE RIEUL EU NIEUN
    ((0xB8, 0xA6), (0xB97C)),  # HANGUL SYLLABLE RIEUL EU RIEUL
    ((0xB8, 0xA7), (0xB984)),  # HANGUL SYLLABLE RIEUL EU MIEUM
    ((0xB8, 0xA8), (0xB985)),  # HANGUL SYLLABLE RIEUL EU PIEUP
    ((0xB8, 0xA9), (0xB987)),  # HANGUL SYLLABLE RIEUL EU SIOS
    ((0xB8, 0xAA), (0xB989)),  # HANGUL SYLLABLE RIEUL EU IEUNG
    ((0xB8, 0xAB), (0xB98A)),  # HANGUL SYLLABLE RIEUL EU CIEUC
    ((0xB8, 0xAC), (0xB98D)),  # HANGUL SYLLABLE RIEUL EU THIEUTH
    ((0xB8, 0xAD), (0xB98E)),  # HANGUL SYLLABLE RIEUL EU PHIEUPH
    ((0xB8, 0xAE), (0xB9AC)),  # HANGUL SYLLABLE RIEUL I
    ((0xB8, 0xAF), (0xB9AD)),  # HANGUL SYLLABLE RIEUL I KIYEOK
    ((0xB8, 0xB0), (0xB9B0)),  # HANGUL SYLLABLE RIEUL I NIEUN
    ((0xB8, 0xB1), (0xB9B4)),  # HANGUL SYLLABLE RIEUL I RIEUL
    ((0xB8, 0xB2), (0xB9BC)),  # HANGUL SYLLABLE RIEUL I MIEUM
    ((0xB8, 0xB3), (0xB9BD)),  # HANGUL SYLLABLE RIEUL I PIEUP
    ((0xB8, 0xB4), (0xB9BF)),  # HANGUL SYLLABLE RIEUL I SIOS
    ((0xB8, 0xB5), (0xB9C1)),  # HANGUL SYLLABLE RIEUL I IEUNG
    ((0xB8, 0xB6), (0xB9C8)),  # HANGUL SYLLABLE MIEUM A
    ((0xB8, 0xB7), (0xB9C9)),  # HANGUL SYLLABLE MIEUM A KIYEOK
    ((0xB8, 0xB8), (0xB9CC)),  # HANGUL SYLLABLE MIEUM A NIEUN
    ((0xB8, 0xB9), (0xB9CE)),  # HANGUL SYLLABLE MIEUM A NIEUNHIEUH
    ((0xB8, 0xBA), (0xB9CF)),  # HANGUL SYLLABLE MIEUM A TIKEUT
    ((0xB8, 0xBB), (0xB9D0)),  # HANGUL SYLLABLE MIEUM A RIEUL
    ((0xB8, 0xBC), (0xB9D1)),  # HANGUL SYLLABLE MIEUM A RIEULKIYEOK
    ((0xB8, 0xBD), (0xB9D2)),  # HANGUL SYLLABLE MIEUM A RIEULMIEUM
    ((0xB8, 0xBE), (0xB9D8)),  # HANGUL SYLLABLE MIEUM A MIEUM
    ((0xB8, 0xBF), (0xB9D9)),  # HANGUL SYLLABLE MIEUM A PIEUP
    ((0xB8, 0xC0), (0xB9DB)),  # HANGUL SYLLABLE MIEUM A SIOS
    ((0xB8, 0xC1), (0xB9DD)),  # HANGUL SYLLABLE MIEUM A IEUNG
    ((0xB8, 0xC2), (0xB9DE)),  # HANGUL SYLLABLE MIEUM A CIEUC
    ((0xB8, 0xC3), (0xB9E1)),  # HANGUL SYLLABLE MIEUM A THIEUTH
    ((0xB8, 0xC4), (0xB9E3)),  # HANGUL SYLLABLE MIEUM A HIEUH
    ((0xB8, 0xC5), (0xB9E4)),  # HANGUL SYLLABLE MIEUM AE
    ((0xB8, 0xC6), (0xB9E5)),  # HANGUL SYLLABLE MIEUM AE KIYEOK
    ((0xB8, 0xC7), (0xB9E8)),  # HANGUL SYLLABLE MIEUM AE NIEUN
    ((0xB8, 0xC8), (0xB9EC)),  # HANGUL SYLLABLE MIEUM AE RIEUL
    ((0xB8, 0xC9), (0xB9F4)),  # HANGUL SYLLABLE MIEUM AE MIEUM
    ((0xB8, 0xCA), (0xB9F5)),  # HANGUL SYLLABLE MIEUM AE PIEUP
    ((0xB8, 0xCB), (0xB9F7)),  # HANGUL SYLLABLE MIEUM AE SIOS
    ((0xB8, 0xCC), (0xB9F8)),  # HANGUL SYLLABLE MIEUM AE SSANGSIOS
    ((0xB8, 0xCD), (0xB9F9)),  # HANGUL SYLLABLE MIEUM AE IEUNG
    ((0xB8, 0xCE), (0xB9FA)),  # HANGUL SYLLABLE MIEUM AE CIEUC
    ((0xB8, 0xCF), (0xBA00)),  # HANGUL SYLLABLE MIEUM YA
    ((0xB8, 0xD0), (0xBA01)),  # HANGUL SYLLABLE MIEUM YA KIYEOK
    ((0xB8, 0xD1), (0xBA08)),  # HANGUL SYLLABLE MIEUM YA RIEUL
    ((0xB8, 0xD2), (0xBA15)),  # HANGUL SYLLABLE MIEUM YA IEUNG
    ((0xB8, 0xD3), (0xBA38)),  # HANGUL SYLLABLE MIEUM EO
    ((0xB8, 0xD4), (0xBA39)),  # HANGUL SYLLABLE MIEUM EO KIYEOK
    ((0xB8, 0xD5), (0xBA3C)),  # HANGUL SYLLABLE MIEUM EO NIEUN
    ((0xB8, 0xD6), (0xBA40)),  # HANGUL SYLLABLE MIEUM EO RIEUL
    ((0xB8, 0xD7), (0xBA42)),  # HANGUL SYLLABLE MIEUM EO RIEULMIEUM
    ((0xB8, 0xD8), (0xBA48)),  # HANGUL SYLLABLE MIEUM EO MIEUM
    ((0xB8, 0xD9), (0xBA49)),  # HANGUL SYLLABLE MIEUM EO PIEUP
    ((0xB8, 0xDA), (0xBA4B)),  # HANGUL SYLLABLE MIEUM EO SIOS
    ((0xB8, 0xDB), (0xBA4D)),  # HANGUL SYLLABLE MIEUM EO IEUNG
    ((0xB8, 0xDC), (0xBA4E)),  # HANGUL SYLLABLE MIEUM EO CIEUC
    ((0xB8, 0xDD), (0xBA53)),  # HANGUL SYLLABLE MIEUM EO HIEUH
    ((0xB8, 0xDE), (0xBA54)),  # HANGUL SYLLABLE MIEUM E
    ((0xB8, 0xDF), (0xBA55)),  # HANGUL SYLLABLE MIEUM E KIYEOK
    ((0xB8, 0xE0), (0xBA58)),  # HANGUL SYLLABLE MIEUM E NIEUN
    ((0xB8, 0xE1), (0xBA5C)),  # HANGUL SYLLABLE MIEUM E RIEUL
    ((0xB8, 0xE2), (0xBA64)),  # HANGUL SYLLABLE MIEUM E MIEUM
    ((0xB8, 0xE3), (0xBA65)),  # HANGUL SYLLABLE MIEUM E PIEUP
    ((0xB8, 0xE4), (0xBA67)),  # HANGUL SYLLABLE MIEUM E SIOS
    ((0xB8, 0xE5), (0xBA68)),  # HANGUL SYLLABLE MIEUM E SSANGSIOS
    ((0xB8, 0xE6), (0xBA69)),  # HANGUL SYLLABLE MIEUM E IEUNG
    ((0xB8, 0xE7), (0xBA70)),  # HANGUL SYLLABLE MIEUM YEO
    ((0xB8, 0xE8), (0xBA71)),  # HANGUL SYLLABLE MIEUM YEO KIYEOK
    ((0xB8, 0xE9), (0xBA74)),  # HANGUL SYLLABLE MIEUM YEO NIEUN
    ((0xB8, 0xEA), (0xBA78)),  # HANGUL SYLLABLE MIEUM YEO RIEUL
    ((0xB8, 0xEB), (0xBA83)),  # HANGUL SYLLABLE MIEUM YEO SIOS
    ((0xB8, 0xEC), (0xBA84)),  # HANGUL SYLLABLE MIEUM YEO SSANGSIOS
    ((0xB8, 0xED), (0xBA85)),  # HANGUL SYLLABLE MIEUM YEO IEUNG
    ((0xB8, 0xEE), (0xBA87)),  # HANGUL SYLLABLE MIEUM YEO CHIEUCH
    ((0xB8, 0xEF), (0xBA8C)),  # HANGUL SYLLABLE MIEUM YE
    ((0xB8, 0xF0), (0xBAA8)),  # HANGUL SYLLABLE MIEUM O
    ((0xB8, 0xF1), (0xBAA9)),  # HANGUL SYLLABLE MIEUM O KIYEOK
    ((0xB8, 0xF2), (0xBAAB)),  # HANGUL SYLLABLE MIEUM O KIYEOKSIOS
    ((0xB8, 0xF3), (0xBAAC)),  # HANGUL SYLLABLE MIEUM O NIEUN
    ((0xB8, 0xF4), (0xBAB0)),  # HANGUL SYLLABLE MIEUM O RIEUL
    ((0xB8, 0xF5), (0xBAB2)),  # HANGUL SYLLABLE MIEUM O RIEULMIEUM
    ((0xB8, 0xF6), (0xBAB8)),  # HANGUL SYLLABLE MIEUM O MIEUM
    ((0xB8, 0xF7), (0xBAB9)),  # HANGUL SYLLABLE MIEUM O PIEUP
    ((0xB8, 0xF8), (0xBABB)),  # HANGUL SYLLABLE MIEUM O SIOS
    ((0xB8, 0xF9), (0xBABD)),  # HANGUL SYLLABLE MIEUM O IEUNG
    ((0xB8, 0xFA), (0xBAC4)),  # HANGUL SYLLABLE MIEUM WA
    ((0xB8, 0xFB), (0xBAC8)),  # HANGUL SYLLABLE MIEUM WA NIEUN
    ((0xB8, 0xFC), (0xBAD8)),  # HANGUL SYLLABLE MIEUM WA SSANGSIOS
    ((0xB8, 0xFD), (0xBAD9)),  # HANGUL SYLLABLE MIEUM WA IEUNG
    ((0xB8, 0xFE), (0xBAFC)),  # HANGUL SYLLABLE MIEUM OE
    ((0xB9, 0x41), (0xD22A)),  # HANGUL SYLLABLE THIEUTH YO PHIEUPH
    ((0xB9, 0x42), (0xD22B)),  # HANGUL SYLLABLE THIEUTH YO HIEUH
    ((0xB9, 0x43), (0xD22E)),  # HANGUL SYLLABLE THIEUTH U SSANGKIYEOK
    ((0xB9, 0x44), (0xD22F)),  # HANGUL SYLLABLE THIEUTH U KIYEOKSIOS
    ((0xB9, 0x45), (0xD231)),  # HANGUL SYLLABLE THIEUTH U NIEUNCIEUC
    ((0xB9, 0x46), (0xD232)),  # HANGUL SYLLABLE THIEUTH U NIEUNHIEUH
    ((0xB9, 0x47), (0xD233)),  # HANGUL SYLLABLE THIEUTH U TIKEUT
    ((0xB9, 0x48), (0xD235)),  # HANGUL SYLLABLE THIEUTH U RIEULKIYEOK
    ((0xB9, 0x49), (0xD236)),  # HANGUL SYLLABLE THIEUTH U RIEULMIEUM
    ((0xB9, 0x4A), (0xD237)),  # HANGUL SYLLABLE THIEUTH U RIEULPIEUP
    ((0xB9, 0x4B), (0xD238)),  # HANGUL SYLLABLE THIEUTH U RIEULSIOS
    ((0xB9, 0x4C), (0xD239)),  # HANGUL SYLLABLE THIEUTH U RIEULTHIEUTH
    ((0xB9, 0x4D), (0xD23A)),  # HANGUL SYLLABLE THIEUTH U RIEULPHIEUPH
    ((0xB9, 0x4E), (0xD23B)),  # HANGUL SYLLABLE THIEUTH U RIEULHIEUH
    ((0xB9, 0x4F), (0xD23E)),  # HANGUL SYLLABLE THIEUTH U PIEUPSIOS
    ((0xB9, 0x50), (0xD240)),  # HANGUL SYLLABLE THIEUTH U SSANGSIOS
    ((0xB9, 0x51), (0xD242)),  # HANGUL SYLLABLE THIEUTH U CIEUC
    ((0xB9, 0x52), (0xD243)),  # HANGUL SYLLABLE THIEUTH U CHIEUCH
    ((0xB9, 0x53), (0xD244)),  # HANGUL SYLLABLE THIEUTH U KHIEUKH
    ((0xB9, 0x54), (0xD245)),  # HANGUL SYLLABLE THIEUTH U THIEUTH
    ((0xB9, 0x55), (0xD246)),  # HANGUL SYLLABLE THIEUTH U PHIEUPH
    ((0xB9, 0x56), (0xD247)),  # HANGUL SYLLABLE THIEUTH U HIEUH
    ((0xB9, 0x57), (0xD249)),  # HANGUL SYLLABLE THIEUTH WEO KIYEOK
    ((0xB9, 0x58), (0xD24A)),  # HANGUL SYLLABLE THIEUTH WEO SSANGKIYEOK
    ((0xB9, 0x59), (0xD24B)),  # HANGUL SYLLABLE THIEUTH WEO KIYEOKSIOS
    ((0xB9, 0x5A), (0xD24C)),  # HANGUL SYLLABLE THIEUTH WEO NIEUN
    ((0xB9, 0x61), (0xD24D)),  # HANGUL SYLLABLE THIEUTH WEO NIEUNCIEUC
    ((0xB9, 0x62), (0xD24E)),  # HANGUL SYLLABLE THIEUTH WEO NIEUNHIEUH
    ((0xB9, 0x63), (0xD24F)),  # HANGUL SYLLABLE THIEUTH WEO TIKEUT
    ((0xB9, 0x64), (0xD250)),  # HANGUL SYLLABLE THIEUTH WEO RIEUL
    ((0xB9, 0x65), (0xD251)),  # HANGUL SYLLABLE THIEUTH WEO RIEULKIYEOK
    ((0xB9, 0x66), (0xD252)),  # HANGUL SYLLABLE THIEUTH WEO RIEULMIEUM
    ((0xB9, 0x67), (0xD253)),  # HANGUL SYLLABLE THIEUTH WEO RIEULPIEUP
    ((0xB9, 0x68), (0xD254)),  # HANGUL SYLLABLE THIEUTH WEO RIEULSIOS
    ((0xB9, 0x69), (0xD255)),  # HANGUL SYLLABLE THIEUTH WEO RIEULTHIEUTH
    ((0xB9, 0x6A), (0xD256)),  # HANGUL SYLLABLE THIEUTH WEO RIEULPHIEUPH
    ((0xB9, 0x6B), (0xD257)),  # HANGUL SYLLABLE THIEUTH WEO RIEULHIEUH
    ((0xB9, 0x6C), (0xD258)),  # HANGUL SYLLABLE THIEUTH WEO MIEUM
    ((0xB9, 0x6D), (0xD259)),  # HANGUL SYLLABLE THIEUTH WEO PIEUP
    ((0xB9, 0x6E), (0xD25A)),  # HANGUL SYLLABLE THIEUTH WEO PIEUPSIOS
    ((0xB9, 0x6F), (0xD25B)),  # HANGUL SYLLABLE THIEUTH WEO SIOS
    ((0xB9, 0x70), (0xD25D)),  # HANGUL SYLLABLE THIEUTH WEO IEUNG
    ((0xB9, 0x71), (0xD25E)),  # HANGUL SYLLABLE THIEUTH WEO CIEUC
    ((0xB9, 0x72), (0xD25F)),  # HANGUL SYLLABLE THIEUTH WEO CHIEUCH
    ((0xB9, 0x73), (0xD260)),  # HANGUL SYLLABLE THIEUTH WEO KHIEUKH
    ((0xB9, 0x74), (0xD261)),  # HANGUL SYLLABLE THIEUTH WEO THIEUTH
    ((0xB9, 0x75), (0xD262)),  # HANGUL SYLLABLE THIEUTH WEO PHIEUPH
    ((0xB9, 0x76), (0xD263)),  # HANGUL SYLLABLE THIEUTH WEO HIEUH
    ((0xB9, 0x77), (0xD265)),  # HANGUL SYLLABLE THIEUTH WE KIYEOK
    ((0xB9, 0x78), (0xD266)),  # HANGUL SYLLABLE THIEUTH WE SSANGKIYEOK
    ((0xB9, 0x79), (0xD267)),  # HANGUL SYLLABLE THIEUTH WE KIYEOKSIOS
    ((0xB9, 0x7A), (0xD268)),  # HANGUL SYLLABLE THIEUTH WE NIEUN
    ((0xB9, 0x81), (0xD269)),  # HANGUL SYLLABLE THIEUTH WE NIEUNCIEUC
    ((0xB9, 0x82), (0xD26A)),  # HANGUL SYLLABLE THIEUTH WE NIEUNHIEUH
    ((0xB9, 0x83), (0xD26B)),  # HANGUL SYLLABLE THIEUTH WE TIKEUT
    ((0xB9, 0x84), (0xD26C)),  # HANGUL SYLLABLE THIEUTH WE RIEUL
    ((0xB9, 0x85), (0xD26D)),  # HANGUL SYLLABLE THIEUTH WE RIEULKIYEOK
    ((0xB9, 0x86), (0xD26E)),  # HANGUL SYLLABLE THIEUTH WE RIEULMIEUM
    ((0xB9, 0x87), (0xD26F)),  # HANGUL SYLLABLE THIEUTH WE RIEULPIEUP
    ((0xB9, 0x88), (0xD270)),  # HANGUL SYLLABLE THIEUTH WE RIEULSIOS
    ((0xB9, 0x89), (0xD271)),  # HANGUL SYLLABLE THIEUTH WE RIEULTHIEUTH
    ((0xB9, 0x8A), (0xD272)),  # HANGUL SYLLABLE THIEUTH WE RIEULPHIEUPH
    ((0xB9, 0x8B), (0xD273)),  # HANGUL SYLLABLE THIEUTH WE RIEULHIEUH
    ((0xB9, 0x8C), (0xD274)),  # HANGUL SYLLABLE THIEUTH WE MIEUM
    ((0xB9, 0x8D), (0xD275)),  # HANGUL SYLLABLE THIEUTH WE PIEUP
    ((0xB9, 0x8E), (0xD276)),  # HANGUL SYLLABLE THIEUTH WE PIEUPSIOS
    ((0xB9, 0x8F), (0xD277)),  # HANGUL SYLLABLE THIEUTH WE SIOS
    ((0xB9, 0x90), (0xD278)),  # HANGUL SYLLABLE THIEUTH WE SSANGSIOS
    ((0xB9, 0x91), (0xD279)),  # HANGUL SYLLABLE THIEUTH WE IEUNG
    ((0xB9, 0x92), (0xD27A)),  # HANGUL SYLLABLE THIEUTH WE CIEUC
    ((0xB9, 0x93), (0xD27B)),  # HANGUL SYLLABLE THIEUTH WE CHIEUCH
    ((0xB9, 0x94), (0xD27C)),  # HANGUL SYLLABLE THIEUTH WE KHIEUKH
    ((0xB9, 0x95), (0xD27D)),  # HANGUL SYLLABLE THIEUTH WE THIEUTH
    ((0xB9, 0x96), (0xD27E)),  # HANGUL SYLLABLE THIEUTH WE PHIEUPH
    ((0xB9, 0x97), (0xD27F)),  # HANGUL SYLLABLE THIEUTH WE HIEUH
    ((0xB9, 0x98), (0xD282)),  # HANGUL SYLLABLE THIEUTH WI SSANGKIYEOK
    ((0xB9, 0x99), (0xD283)),  # HANGUL SYLLABLE THIEUTH WI KIYEOKSIOS
    ((0xB9, 0x9A), (0xD285)),  # HANGUL SYLLABLE THIEUTH WI NIEUNCIEUC
    ((0xB9, 0x9B), (0xD286)),  # HANGUL SYLLABLE THIEUTH WI NIEUNHIEUH
    ((0xB9, 0x9C), (0xD287)),  # HANGUL SYLLABLE THIEUTH WI TIKEUT
    ((0xB9, 0x9D), (0xD289)),  # HANGUL SYLLABLE THIEUTH WI RIEULKIYEOK
    ((0xB9, 0x9E), (0xD28A)),  # HANGUL SYLLABLE THIEUTH WI RIEULMIEUM
    ((0xB9, 0x9F), (0xD28B)),  # HANGUL SYLLABLE THIEUTH WI RIEULPIEUP
    ((0xB9, 0xA0), (0xD28C)),  # HANGUL SYLLABLE THIEUTH WI RIEULSIOS
    ((0xB9, 0xA1), (0xBB00)),  # HANGUL SYLLABLE MIEUM OE NIEUN
    ((0xB9, 0xA2), (0xBB04)),  # HANGUL SYLLABLE MIEUM OE RIEUL
    ((0xB9, 0xA3), (0xBB0D)),  # HANGUL SYLLABLE MIEUM OE PIEUP
    ((0xB9, 0xA4), (0xBB0F)),  # HANGUL SYLLABLE MIEUM OE SIOS
    ((0xB9, 0xA5), (0xBB11)),  # HANGUL SYLLABLE MIEUM OE IEUNG
    ((0xB9, 0xA6), (0xBB18)),  # HANGUL SYLLABLE MIEUM YO
    ((0xB9, 0xA7), (0xBB1C)),  # HANGUL SYLLABLE MIEUM YO NIEUN
    ((0xB9, 0xA8), (0xBB20)),  # HANGUL SYLLABLE MIEUM YO RIEUL
    ((0xB9, 0xA9), (0xBB29)),  # HANGUL SYLLABLE MIEUM YO PIEUP
    ((0xB9, 0xAA), (0xBB2B)),  # HANGUL SYLLABLE MIEUM YO SIOS
    ((0xB9, 0xAB), (0xBB34)),  # HANGUL SYLLABLE MIEUM U
    ((0xB9, 0xAC), (0xBB35)),  # HANGUL SYLLABLE MIEUM U KIYEOK
    ((0xB9, 0xAD), (0xBB36)),  # HANGUL SYLLABLE MIEUM U SSANGKIYEOK
    ((0xB9, 0xAE), (0xBB38)),  # HANGUL SYLLABLE MIEUM U NIEUN
    ((0xB9, 0xAF), (0xBB3B)),  # HANGUL SYLLABLE MIEUM U TIKEUT
    ((0xB9, 0xB0), (0xBB3C)),  # HANGUL SYLLABLE MIEUM U RIEUL
    ((0xB9, 0xB1), (0xBB3D)),  # HANGUL SYLLABLE MIEUM U RIEULKIYEOK
    ((0xB9, 0xB2), (0xBB3E)),  # HANGUL SYLLABLE MIEUM U RIEULMIEUM
    ((0xB9, 0xB3), (0xBB44)),  # HANGUL SYLLABLE MIEUM U MIEUM
    ((0xB9, 0xB4), (0xBB45)),  # HANGUL SYLLABLE MIEUM U PIEUP
    ((0xB9, 0xB5), (0xBB47)),  # HANGUL SYLLABLE MIEUM U SIOS
    ((0xB9, 0xB6), (0xBB49)),  # HANGUL SYLLABLE MIEUM U IEUNG
    ((0xB9, 0xB7), (0xBB4D)),  # HANGUL SYLLABLE MIEUM U THIEUTH
    ((0xB9, 0xB8), (0xBB4F)),  # HANGUL SYLLABLE MIEUM U HIEUH
    ((0xB9, 0xB9), (0xBB50)),  # HANGUL SYLLABLE MIEUM WEO
    ((0xB9, 0xBA), (0xBB54)),  # HANGUL SYLLABLE MIEUM WEO NIEUN
    ((0xB9, 0xBB), (0xBB58)),  # HANGUL SYLLABLE MIEUM WEO RIEUL
    ((0xB9, 0xBC), (0xBB61)),  # HANGUL SYLLABLE MIEUM WEO PIEUP
    ((0xB9, 0xBD), (0xBB63)),  # HANGUL SYLLABLE MIEUM WEO SIOS
    ((0xB9, 0xBE), (0xBB6C)),  # HANGUL SYLLABLE MIEUM WE
    ((0xB9, 0xBF), (0xBB88)),  # HANGUL SYLLABLE MIEUM WI
    ((0xB9, 0xC0), (0xBB8C)),  # HANGUL SYLLABLE MIEUM WI NIEUN
    ((0xB9, 0xC1), (0xBB90)),  # HANGUL SYLLABLE MIEUM WI RIEUL
    ((0xB9, 0xC2), (0xBBA4)),  # HANGUL SYLLABLE MIEUM YU
    ((0xB9, 0xC3), (0xBBA8)),  # HANGUL SYLLABLE MIEUM YU NIEUN
    ((0xB9, 0xC4), (0xBBAC)),  # HANGUL SYLLABLE MIEUM YU RIEUL
    ((0xB9, 0xC5), (0xBBB4)),  # HANGUL SYLLABLE MIEUM YU MIEUM
    ((0xB9, 0xC6), (0xBBB7)),  # HANGUL SYLLABLE MIEUM YU SIOS
    ((0xB9, 0xC7), (0xBBC0)),  # HANGUL SYLLABLE MIEUM EU
    ((0xB9, 0xC8), (0xBBC4)),  # HANGUL SYLLABLE MIEUM EU NIEUN
    ((0xB9, 0xC9), (0xBBC8)),  # HANGUL SYLLABLE MIEUM EU RIEUL
    ((0xB9, 0xCA), (0xBBD0)),  # HANGUL SYLLABLE MIEUM EU MIEUM
    ((0xB9, 0xCB), (0xBBD3)),  # HANGUL SYLLABLE MIEUM EU SIOS
    ((0xB9, 0xCC), (0xBBF8)),  # HANGUL SYLLABLE MIEUM I
    ((0xB9, 0xCD), (0xBBF9)),  # HANGUL SYLLABLE MIEUM I KIYEOK
    ((0xB9, 0xCE), (0xBBFC)),  # HANGUL SYLLABLE MIEUM I NIEUN
    ((0xB9, 0xCF), (0xBBFF)),  # HANGUL SYLLABLE MIEUM I TIKEUT
    ((0xB9, 0xD0), (0xBC00)),  # HANGUL SYLLABLE MIEUM I RIEUL
    ((0xB9, 0xD1), (0xBC02)),  # HANGUL SYLLABLE MIEUM I RIEULMIEUM
    ((0xB9, 0xD2), (0xBC08)),  # HANGUL SYLLABLE MIEUM I MIEUM
    ((0xB9, 0xD3), (0xBC09)),  # HANGUL SYLLABLE MIEUM I PIEUP
    ((0xB9, 0xD4), (0xBC0B)),  # HANGUL SYLLABLE MIEUM I SIOS
    ((0xB9, 0xD5), (0xBC0C)),  # HANGUL SYLLABLE MIEUM I SSANGSIOS
    ((0xB9, 0xD6), (0xBC0D)),  # HANGUL SYLLABLE MIEUM I IEUNG
    ((0xB9, 0xD7), (0xBC0F)),  # HANGUL SYLLABLE MIEUM I CHIEUCH
    ((0xB9, 0xD8), (0xBC11)),  # HANGUL SYLLABLE MIEUM I THIEUTH
    ((0xB9, 0xD9), (0xBC14)),  # HANGUL SYLLABLE PIEUP A
    ((0xB9, 0xDA), (0xBC15)),  # HANGUL SYLLABLE PIEUP A KIYEOK
    ((0xB9, 0xDB), (0xBC16)),  # HANGUL SYLLABLE PIEUP A SSANGKIYEOK
    ((0xB9, 0xDC), (0xBC17)),  # HANGUL SYLLABLE PIEUP A KIYEOKSIOS
    ((0xB9, 0xDD), (0xBC18)),  # HANGUL SYLLABLE PIEUP A NIEUN
    ((0xB9, 0xDE), (0xBC1B)),  # HANGUL SYLLABLE PIEUP A TIKEUT
    ((0xB9, 0xDF), (0xBC1C)),  # HANGUL SYLLABLE PIEUP A RIEUL
    ((0xB9, 0xE0), (0xBC1D)),  # HANGUL SYLLABLE PIEUP A RIEULKIYEOK
    ((0xB9, 0xE1), (0xBC1E)),  # HANGUL SYLLABLE PIEUP A RIEULMIEUM
    ((0xB9, 0xE2), (0xBC1F)),  # HANGUL SYLLABLE PIEUP A RIEULPIEUP
    ((0xB9, 0xE3), (0xBC24)),  # HANGUL SYLLABLE PIEUP A MIEUM
    ((0xB9, 0xE4), (0xBC25)),  # HANGUL SYLLABLE PIEUP A PIEUP
    ((0xB9, 0xE5), (0xBC27)),  # HANGUL SYLLABLE PIEUP A SIOS
    ((0xB9, 0xE6), (0xBC29)),  # HANGUL SYLLABLE PIEUP A IEUNG
    ((0xB9, 0xE7), (0xBC2D)),  # HANGUL SYLLABLE PIEUP A THIEUTH
    ((0xB9, 0xE8), (0xBC30)),  # HANGUL SYLLABLE PIEUP AE
    ((0xB9, 0xE9), (0xBC31)),  # HANGUL SYLLABLE PIEUP AE KIYEOK
    ((0xB9, 0xEA), (0xBC34)),  # HANGUL SYLLABLE PIEUP AE NIEUN
    ((0xB9, 0xEB), (0xBC38)),  # HANGUL SYLLABLE PIEUP AE RIEUL
    ((0xB9, 0xEC), (0xBC40)),  # HANGUL SYLLABLE PIEUP AE MIEUM
    ((0xB9, 0xED), (0xBC41)),  # HANGUL SYLLABLE PIEUP AE PIEUP
    ((0xB9, 0xEE), (0xBC43)),  # HANGUL SYLLABLE PIEUP AE SIOS
    ((0xB9, 0xEF), (0xBC44)),  # HANGUL SYLLABLE PIEUP AE SSANGSIOS
    ((0xB9, 0xF0), (0xBC45)),  # HANGUL SYLLABLE PIEUP AE IEUNG
    ((0xB9, 0xF1), (0xBC49)),  # HANGUL SYLLABLE PIEUP AE THIEUTH
    ((0xB9, 0xF2), (0xBC4C)),  # HANGUL SYLLABLE PIEUP YA
    ((0xB9, 0xF3), (0xBC4D)),  # HANGUL SYLLABLE PIEUP YA KIYEOK
    ((0xB9, 0xF4), (0xBC50)),  # HANGUL SYLLABLE PIEUP YA NIEUN
    ((0xB9, 0xF5), (0xBC5D)),  # HANGUL SYLLABLE PIEUP YA PIEUP
    ((0xB9, 0xF6), (0xBC84)),  # HANGUL SYLLABLE PIEUP EO
    ((0xB9, 0xF7), (0xBC85)),  # HANGUL SYLLABLE PIEUP EO KIYEOK
    ((0xB9, 0xF8), (0xBC88)),  # HANGUL SYLLABLE PIEUP EO NIEUN
    ((0xB9, 0xF9), (0xBC8B)),  # HANGUL SYLLABLE PIEUP EO TIKEUT
    ((0xB9, 0xFA), (0xBC8C)),  # HANGUL SYLLABLE PIEUP EO RIEUL
    ((0xB9, 0xFB), (0xBC8E)),  # HANGUL SYLLABLE PIEUP EO RIEULMIEUM
    ((0xB9, 0xFC), (0xBC94)),  # HANGUL SYLLABLE PIEUP EO MIEUM
    ((0xB9, 0xFD), (0xBC95)),  # HANGUL SYLLABLE PIEUP EO PIEUP
    ((0xB9, 0xFE), (0xBC97)),  # HANGUL SYLLABLE PIEUP EO SIOS
    ((0xBA, 0x41), (0xD28D)),  # HANGUL SYLLABLE THIEUTH WI RIEULTHIEUTH
    ((0xBA, 0x42), (0xD28E)),  # HANGUL SYLLABLE THIEUTH WI RIEULPHIEUPH
    ((0xBA, 0x43), (0xD28F)),  # HANGUL SYLLABLE THIEUTH WI RIEULHIEUH
    ((0xBA, 0x44), (0xD292)),  # HANGUL SYLLABLE THIEUTH WI PIEUPSIOS
    ((0xBA, 0x45), (0xD293)),  # HANGUL SYLLABLE THIEUTH WI SIOS
    ((0xBA, 0x46), (0xD294)),  # HANGUL SYLLABLE THIEUTH WI SSANGSIOS
    ((0xBA, 0x47), (0xD296)),  # HANGUL SYLLABLE THIEUTH WI CIEUC
    ((0xBA, 0x48), (0xD297)),  # HANGUL SYLLABLE THIEUTH WI CHIEUCH
    ((0xBA, 0x49), (0xD298)),  # HANGUL SYLLABLE THIEUTH WI KHIEUKH
    ((0xBA, 0x4A), (0xD299)),  # HANGUL SYLLABLE THIEUTH WI THIEUTH
    ((0xBA, 0x4B), (0xD29A)),  # HANGUL SYLLABLE THIEUTH WI PHIEUPH
    ((0xBA, 0x4C), (0xD29B)),  # HANGUL SYLLABLE THIEUTH WI HIEUH
    ((0xBA, 0x4D), (0xD29D)),  # HANGUL SYLLABLE THIEUTH YU KIYEOK
    ((0xBA, 0x4E), (0xD29E)),  # HANGUL SYLLABLE THIEUTH YU SSANGKIYEOK
    ((0xBA, 0x4F), (0xD29F)),  # HANGUL SYLLABLE THIEUTH YU KIYEOKSIOS
    ((0xBA, 0x50), (0xD2A1)),  # HANGUL SYLLABLE THIEUTH YU NIEUNCIEUC
    ((0xBA, 0x51), (0xD2A2)),  # HANGUL SYLLABLE THIEUTH YU NIEUNHIEUH
    ((0xBA, 0x52), (0xD2A3)),  # HANGUL SYLLABLE THIEUTH YU TIKEUT
    ((0xBA, 0x53), (0xD2A5)),  # HANGUL SYLLABLE THIEUTH YU RIEULKIYEOK
    ((0xBA, 0x54), (0xD2A6)),  # HANGUL SYLLABLE THIEUTH YU RIEULMIEUM
    ((0xBA, 0x55), (0xD2A7)),  # HANGUL SYLLABLE THIEUTH YU RIEULPIEUP
    ((0xBA, 0x56), (0xD2A8)),  # HANGUL SYLLABLE THIEUTH YU RIEULSIOS
    ((0xBA, 0x57), (0xD2A9)),  # HANGUL SYLLABLE THIEUTH YU RIEULTHIEUTH
    ((0xBA, 0x58), (0xD2AA)),  # HANGUL SYLLABLE THIEUTH YU RIEULPHIEUPH
    ((0xBA, 0x59), (0xD2AB)),  # HANGUL SYLLABLE THIEUTH YU RIEULHIEUH
    ((0xBA, 0x5A), (0xD2AD)),  # HANGUL SYLLABLE THIEUTH YU PIEUP
    ((0xBA, 0x61), (0xD2AE)),  # HANGUL SYLLABLE THIEUTH YU PIEUPSIOS
    ((0xBA, 0x62), (0xD2AF)),  # HANGUL SYLLABLE THIEUTH YU SIOS
    ((0xBA, 0x63), (0xD2B0)),  # HANGUL SYLLABLE THIEUTH YU SSANGSIOS
    ((0xBA, 0x64), (0xD2B2)),  # HANGUL SYLLABLE THIEUTH YU CIEUC
    ((0xBA, 0x65), (0xD2B3)),  # HANGUL SYLLABLE THIEUTH YU CHIEUCH
    ((0xBA, 0x66), (0xD2B4)),  # HANGUL SYLLABLE THIEUTH YU KHIEUKH
    ((0xBA, 0x67), (0xD2B5)),  # HANGUL SYLLABLE THIEUTH YU THIEUTH
    ((0xBA, 0x68), (0xD2B6)),  # HANGUL SYLLABLE THIEUTH YU PHIEUPH
    ((0xBA, 0x69), (0xD2B7)),  # HANGUL SYLLABLE THIEUTH YU HIEUH
    ((0xBA, 0x6A), (0xD2BA)),  # HANGUL SYLLABLE THIEUTH EU SSANGKIYEOK
    ((0xBA, 0x6B), (0xD2BB)),  # HANGUL SYLLABLE THIEUTH EU KIYEOKSIOS
    ((0xBA, 0x6C), (0xD2BD)),  # HANGUL SYLLABLE THIEUTH EU NIEUNCIEUC
    ((0xBA, 0x6D), (0xD2BE)),  # HANGUL SYLLABLE THIEUTH EU NIEUNHIEUH
    ((0xBA, 0x6E), (0xD2C1)),  # HANGUL SYLLABLE THIEUTH EU RIEULKIYEOK
    ((0xBA, 0x6F), (0xD2C3)),  # HANGUL SYLLABLE THIEUTH EU RIEULPIEUP
    ((0xBA, 0x70), (0xD2C4)),  # HANGUL SYLLABLE THIEUTH EU RIEULSIOS
    ((0xBA, 0x71), (0xD2C5)),  # HANGUL SYLLABLE THIEUTH EU RIEULTHIEUTH
    ((0xBA, 0x72), (0xD2C6)),  # HANGUL SYLLABLE THIEUTH EU RIEULPHIEUPH
    ((0xBA, 0x73), (0xD2C7)),  # HANGUL SYLLABLE THIEUTH EU RIEULHIEUH
    ((0xBA, 0x74), (0xD2CA)),  # HANGUL SYLLABLE THIEUTH EU PIEUPSIOS
    ((0xBA, 0x75), (0xD2CC)),  # HANGUL SYLLABLE THIEUTH EU SSANGSIOS
    ((0xBA, 0x76), (0xD2CD)),  # HANGUL SYLLABLE THIEUTH EU IEUNG
    ((0xBA, 0x77), (0xD2CE)),  # HANGUL SYLLABLE THIEUTH EU CIEUC
    ((0xBA, 0x78), (0xD2CF)),  # HANGUL SYLLABLE THIEUTH EU CHIEUCH
    ((0xBA, 0x79), (0xD2D0)),  # HANGUL SYLLABLE THIEUTH EU KHIEUKH
    ((0xBA, 0x7A), (0xD2D1)),  # HANGUL SYLLABLE THIEUTH EU THIEUTH
    ((0xBA, 0x81), (0xD2D2)),  # HANGUL SYLLABLE THIEUTH EU PHIEUPH
    ((0xBA, 0x82), (0xD2D3)),  # HANGUL SYLLABLE THIEUTH EU HIEUH
    ((0xBA, 0x83), (0xD2D5)),  # HANGUL SYLLABLE THIEUTH YI KIYEOK
    ((0xBA, 0x84), (0xD2D6)),  # HANGUL SYLLABLE THIEUTH YI SSANGKIYEOK
    ((0xBA, 0x85), (0xD2D7)),  # HANGUL SYLLABLE THIEUTH YI KIYEOKSIOS
    ((0xBA, 0x86), (0xD2D9)),  # HANGUL SYLLABLE THIEUTH YI NIEUNCIEUC
    ((0xBA, 0x87), (0xD2DA)),  # HANGUL SYLLABLE THIEUTH YI NIEUNHIEUH
    ((0xBA, 0x88), (0xD2DB)),  # HANGUL SYLLABLE THIEUTH YI TIKEUT
    ((0xBA, 0x89), (0xD2DD)),  # HANGUL SYLLABLE THIEUTH YI RIEULKIYEOK
    ((0xBA, 0x8A), (0xD2DE)),  # HANGUL SYLLABLE THIEUTH YI RIEULMIEUM
    ((0xBA, 0x8B), (0xD2DF)),  # HANGUL SYLLABLE THIEUTH YI RIEULPIEUP
    ((0xBA, 0x8C), (0xD2E0)),  # HANGUL SYLLABLE THIEUTH YI RIEULSIOS
    ((0xBA, 0x8D), (0xD2E1)),  # HANGUL SYLLABLE THIEUTH YI RIEULTHIEUTH
    ((0xBA, 0x8E), (0xD2E2)),  # HANGUL SYLLABLE THIEUTH YI RIEULPHIEUPH
    ((0xBA, 0x8F), (0xD2E3)),  # HANGUL SYLLABLE THIEUTH YI RIEULHIEUH
    ((0xBA, 0x90), (0xD2E6)),  # HANGUL SYLLABLE THIEUTH YI PIEUPSIOS
    ((0xBA, 0x91), (0xD2E7)),  # HANGUL SYLLABLE THIEUTH YI SIOS
    ((0xBA, 0x92), (0xD2E8)),  # HANGUL SYLLABLE THIEUTH YI SSANGSIOS
    ((0xBA, 0x93), (0xD2E9)),  # HANGUL SYLLABLE THIEUTH YI IEUNG
    ((0xBA, 0x94), (0xD2EA)),  # HANGUL SYLLABLE THIEUTH YI CIEUC
    ((0xBA, 0x95), (0xD2EB)),  # HANGUL SYLLABLE THIEUTH YI CHIEUCH
    ((0xBA, 0x96), (0xD2EC)),  # HANGUL SYLLABLE THIEUTH YI KHIEUKH
    ((0xBA, 0x97), (0xD2ED)),  # HANGUL SYLLABLE THIEUTH YI THIEUTH
    ((0xBA, 0x98), (0xD2EE)),  # HANGUL SYLLABLE THIEUTH YI PHIEUPH
    ((0xBA, 0x99), (0xD2EF)),  # HANGUL SYLLABLE THIEUTH YI HIEUH
    ((0xBA, 0x9A), (0xD2F2)),  # HANGUL SYLLABLE THIEUTH I SSANGKIYEOK
    ((0xBA, 0x9B), (0xD2F3)),  # HANGUL SYLLABLE THIEUTH I KIYEOKSIOS
    ((0xBA, 0x9C), (0xD2F5)),  # HANGUL SYLLABLE THIEUTH I NIEUNCIEUC
    ((0xBA, 0x9D), (0xD2F6)),  # HANGUL SYLLABLE THIEUTH I NIEUNHIEUH
    ((0xBA, 0x9E), (0xD2F7)),  # HANGUL SYLLABLE THIEUTH I TIKEUT
    ((0xBA, 0x9F), (0xD2F9)),  # HANGUL SYLLABLE THIEUTH I RIEULKIYEOK
    ((0xBA, 0xA0), (0xD2FA)),  # HANGUL SYLLABLE THIEUTH I RIEULMIEUM
    ((0xBA, 0xA1), (0xBC99)),  # HANGUL SYLLABLE PIEUP EO IEUNG
    ((0xBA, 0xA2), (0xBC9A)),  # HANGUL SYLLABLE PIEUP EO CIEUC
    ((0xBA, 0xA3), (0xBCA0)),  # HANGUL SYLLABLE PIEUP E
    ((0xBA, 0xA4), (0xBCA1)),  # HANGUL SYLLABLE PIEUP E KIYEOK
    ((0xBA, 0xA5), (0xBCA4)),  # HANGUL SYLLABLE PIEUP E NIEUN
    ((0xBA, 0xA6), (0xBCA7)),  # HANGUL SYLLABLE PIEUP E TIKEUT
    ((0xBA, 0xA7), (0xBCA8)),  # HANGUL SYLLABLE PIEUP E RIEUL
    ((0xBA, 0xA8), (0xBCB0)),  # HANGUL SYLLABLE PIEUP E MIEUM
    ((0xBA, 0xA9), (0xBCB1)),  # HANGUL SYLLABLE PIEUP E PIEUP
    ((0xBA, 0xAA), (0xBCB3)),  # HANGUL SYLLABLE PIEUP E SIOS
    ((0xBA, 0xAB), (0xBCB4)),  # HANGUL SYLLABLE PIEUP E SSANGSIOS
    ((0xBA, 0xAC), (0xBCB5)),  # HANGUL SYLLABLE PIEUP E IEUNG
    ((0xBA, 0xAD), (0xBCBC)),  # HANGUL SYLLABLE PIEUP YEO
    ((0xBA, 0xAE), (0xBCBD)),  # HANGUL SYLLABLE PIEUP YEO KIYEOK
    ((0xBA, 0xAF), (0xBCC0)),  # HANGUL SYLLABLE PIEUP YEO NIEUN
    ((0xBA, 0xB0), (0xBCC4)),  # HANGUL SYLLABLE PIEUP YEO RIEUL
    ((0xBA, 0xB1), (0xBCCD)),  # HANGUL SYLLABLE PIEUP YEO PIEUP
    ((0xBA, 0xB2), (0xBCCF)),  # HANGUL SYLLABLE PIEUP YEO SIOS
    ((0xBA, 0xB3), (0xBCD0)),  # HANGUL SYLLABLE PIEUP YEO SSANGSIOS
    ((0xBA, 0xB4), (0xBCD1)),  # HANGUL SYLLABLE PIEUP YEO IEUNG
    ((0xBA, 0xB5), (0xBCD5)),  # HANGUL SYLLABLE PIEUP YEO THIEUTH
    ((0xBA, 0xB6), (0xBCD8)),  # HANGUL SYLLABLE PIEUP YE
    ((0xBA, 0xB7), (0xBCDC)),  # HANGUL SYLLABLE PIEUP YE NIEUN
    ((0xBA, 0xB8), (0xBCF4)),  # HANGUL SYLLABLE PIEUP O
    ((0xBA, 0xB9), (0xBCF5)),  # HANGUL SYLLABLE PIEUP O KIYEOK
    ((0xBA, 0xBA), (0xBCF6)),  # HANGUL SYLLABLE PIEUP O SSANGKIYEOK
    ((0xBA, 0xBB), (0xBCF8)),  # HANGUL SYLLABLE PIEUP O NIEUN
    ((0xBA, 0xBC), (0xBCFC)),  # HANGUL SYLLABLE PIEUP O RIEUL
    ((0xBA, 0xBD), (0xBD04)),  # HANGUL SYLLABLE PIEUP O MIEUM
    ((0xBA, 0xBE), (0xBD05)),  # HANGUL SYLLABLE PIEUP O PIEUP
    ((0xBA, 0xBF), (0xBD07)),  # HANGUL SYLLABLE PIEUP O SIOS
    ((0xBA, 0xC0), (0xBD09)),  # HANGUL SYLLABLE PIEUP O IEUNG
    ((0xBA, 0xC1), (0xBD10)),  # HANGUL SYLLABLE PIEUP WA
    ((0xBA, 0xC2), (0xBD14)),  # HANGUL SYLLABLE PIEUP WA NIEUN
    ((0xBA, 0xC3), (0xBD24)),  # HANGUL SYLLABLE PIEUP WA SSANGSIOS
    ((0xBA, 0xC4), (0xBD2C)),  # HANGUL SYLLABLE PIEUP WAE
    ((0xBA, 0xC5), (0xBD40)),  # HANGUL SYLLABLE PIEUP WAE SSANGSIOS
    ((0xBA, 0xC6), (0xBD48)),  # HANGUL SYLLABLE PIEUP OE
    ((0xBA, 0xC7), (0xBD49)),  # HANGUL SYLLABLE PIEUP OE KIYEOK
    ((0xBA, 0xC8), (0xBD4C)),  # HANGUL SYLLABLE PIEUP OE NIEUN
    ((0xBA, 0xC9), (0xBD50)),  # HANGUL SYLLABLE PIEUP OE RIEUL
    ((0xBA, 0xCA), (0xBD58)),  # HANGUL SYLLABLE PIEUP OE MIEUM
    ((0xBA, 0xCB), (0xBD59)),  # HANGUL SYLLABLE PIEUP OE PIEUP
    ((0xBA, 0xCC), (0xBD64)),  # HANGUL SYLLABLE PIEUP YO
    ((0xBA, 0xCD), (0xBD68)),  # HANGUL SYLLABLE PIEUP YO NIEUN
    ((0xBA, 0xCE), (0xBD80)),  # HANGUL SYLLABLE PIEUP U
    ((0xBA, 0xCF), (0xBD81)),  # HANGUL SYLLABLE PIEUP U KIYEOK
    ((0xBA, 0xD0), (0xBD84)),  # HANGUL SYLLABLE PIEUP U NIEUN
    ((0xBA, 0xD1), (0xBD87)),  # HANGUL SYLLABLE PIEUP U TIKEUT
    ((0xBA, 0xD2), (0xBD88)),  # HANGUL SYLLABLE PIEUP U RIEUL
    ((0xBA, 0xD3), (0xBD89)),  # HANGUL SYLLABLE PIEUP U RIEULKIYEOK
    ((0xBA, 0xD4), (0xBD8A)),  # HANGUL SYLLABLE PIEUP U RIEULMIEUM
    ((0xBA, 0xD5), (0xBD90)),  # HANGUL SYLLABLE PIEUP U MIEUM
    ((0xBA, 0xD6), (0xBD91)),  # HANGUL SYLLABLE PIEUP U PIEUP
    ((0xBA, 0xD7), (0xBD93)),  # HANGUL SYLLABLE PIEUP U SIOS
    ((0xBA, 0xD8), (0xBD95)),  # HANGUL SYLLABLE PIEUP U IEUNG
    ((0xBA, 0xD9), (0xBD99)),  # HANGUL SYLLABLE PIEUP U THIEUTH
    ((0xBA, 0xDA), (0xBD9A)),  # HANGUL SYLLABLE PIEUP U PHIEUPH
    ((0xBA, 0xDB), (0xBD9C)),  # HANGUL SYLLABLE PIEUP WEO
    ((0xBA, 0xDC), (0xBDA4)),  # HANGUL SYLLABLE PIEUP WEO RIEUL
    ((0xBA, 0xDD), (0xBDB0)),  # HANGUL SYLLABLE PIEUP WEO SSANGSIOS
    ((0xBA, 0xDE), (0xBDB8)),  # HANGUL SYLLABLE PIEUP WE
    ((0xBA, 0xDF), (0xBDD4)),  # HANGUL SYLLABLE PIEUP WI
    ((0xBA, 0xE0), (0xBDD5)),  # HANGUL SYLLABLE PIEUP WI KIYEOK
    ((0xBA, 0xE1), (0xBDD8)),  # HANGUL SYLLABLE PIEUP WI NIEUN
    ((0xBA, 0xE2), (0xBDDC)),  # HANGUL SYLLABLE PIEUP WI RIEUL
    ((0xBA, 0xE3), (0xBDE9)),  # HANGUL SYLLABLE PIEUP WI IEUNG
    ((0xBA, 0xE4), (0xBDF0)),  # HANGUL SYLLABLE PIEUP YU
    ((0xBA, 0xE5), (0xBDF4)),  # HANGUL SYLLABLE PIEUP YU NIEUN
    ((0xBA, 0xE6), (0xBDF8)),  # HANGUL SYLLABLE PIEUP YU RIEUL
    ((0xBA, 0xE7), (0xBE00)),  # HANGUL SYLLABLE PIEUP YU MIEUM
    ((0xBA, 0xE8), (0xBE03)),  # HANGUL SYLLABLE PIEUP YU SIOS
    ((0xBA, 0xE9), (0xBE05)),  # HANGUL SYLLABLE PIEUP YU IEUNG
    ((0xBA, 0xEA), (0xBE0C)),  # HANGUL SYLLABLE PIEUP EU
    ((0xBA, 0xEB), (0xBE0D)),  # HANGUL SYLLABLE PIEUP EU KIYEOK
    ((0xBA, 0xEC), (0xBE10)),  # HANGUL SYLLABLE PIEUP EU NIEUN
    ((0xBA, 0xED), (0xBE14)),  # HANGUL SYLLABLE PIEUP EU RIEUL
    ((0xBA, 0xEE), (0xBE1C)),  # HANGUL SYLLABLE PIEUP EU MIEUM
    ((0xBA, 0xEF), (0xBE1D)),  # HANGUL SYLLABLE PIEUP EU PIEUP
    ((0xBA, 0xF0), (0xBE1F)),  # HANGUL SYLLABLE PIEUP EU SIOS
    ((0xBA, 0xF1), (0xBE44)),  # HANGUL SYLLABLE PIEUP I
    ((0xBA, 0xF2), (0xBE45)),  # HANGUL SYLLABLE PIEUP I KIYEOK
    ((0xBA, 0xF3), (0xBE48)),  # HANGUL SYLLABLE PIEUP I NIEUN
    ((0xBA, 0xF4), (0xBE4C)),  # HANGUL SYLLABLE PIEUP I RIEUL
    ((0xBA, 0xF5), (0xBE4E)),  # HANGUL SYLLABLE PIEUP I RIEULMIEUM
    ((0xBA, 0xF6), (0xBE54)),  # HANGUL SYLLABLE PIEUP I MIEUM
    ((0xBA, 0xF7), (0xBE55)),  # HANGUL SYLLABLE PIEUP I PIEUP
    ((0xBA, 0xF8), (0xBE57)),  # HANGUL SYLLABLE PIEUP I SIOS
    ((0xBA, 0xF9), (0xBE59)),  # HANGUL SYLLABLE PIEUP I IEUNG
    ((0xBA, 0xFA), (0xBE5A)),  # HANGUL SYLLABLE PIEUP I CIEUC
    ((0xBA, 0xFB), (0xBE5B)),  # HANGUL SYLLABLE PIEUP I CHIEUCH
    ((0xBA, 0xFC), (0xBE60)),  # HANGUL SYLLABLE SSANGPIEUP A
    ((0xBA, 0xFD), (0xBE61)),  # HANGUL SYLLABLE SSANGPIEUP A KIYEOK
    ((0xBA, 0xFE), (0xBE64)),  # HANGUL SYLLABLE SSANGPIEUP A NIEUN
    ((0xBB, 0x41), (0xD2FB)),  # HANGUL SYLLABLE THIEUTH I RIEULPIEUP
    ((0xBB, 0x42), (0xD2FC)),  # HANGUL SYLLABLE THIEUTH I RIEULSIOS
    ((0xBB, 0x43), (0xD2FD)),  # HANGUL SYLLABLE THIEUTH I RIEULTHIEUTH
    ((0xBB, 0x44), (0xD2FE)),  # HANGUL SYLLABLE THIEUTH I RIEULPHIEUPH
    ((0xBB, 0x45), (0xD2FF)),  # HANGUL SYLLABLE THIEUTH I RIEULHIEUH
    ((0xBB, 0x46), (0xD302)),  # HANGUL SYLLABLE THIEUTH I PIEUPSIOS
    ((0xBB, 0x47), (0xD304)),  # HANGUL SYLLABLE THIEUTH I SSANGSIOS
    ((0xBB, 0x48), (0xD306)),  # HANGUL SYLLABLE THIEUTH I CIEUC
    ((0xBB, 0x49), (0xD307)),  # HANGUL SYLLABLE THIEUTH I CHIEUCH
    ((0xBB, 0x4A), (0xD308)),  # HANGUL SYLLABLE THIEUTH I KHIEUKH
    ((0xBB, 0x4B), (0xD309)),  # HANGUL SYLLABLE THIEUTH I THIEUTH
    ((0xBB, 0x4C), (0xD30A)),  # HANGUL SYLLABLE THIEUTH I PHIEUPH
    ((0xBB, 0x4D), (0xD30B)),  # HANGUL SYLLABLE THIEUTH I HIEUH
    ((0xBB, 0x4E), (0xD30F)),  # HANGUL SYLLABLE PHIEUPH A KIYEOKSIOS
    ((0xBB, 0x4F), (0xD311)),  # HANGUL SYLLABLE PHIEUPH A NIEUNCIEUC
    ((0xBB, 0x50), (0xD312)),  # HANGUL SYLLABLE PHIEUPH A NIEUNHIEUH
    ((0xBB, 0x51), (0xD313)),  # HANGUL SYLLABLE PHIEUPH A TIKEUT
    ((0xBB, 0x52), (0xD315)),  # HANGUL SYLLABLE PHIEUPH A RIEULKIYEOK
    ((0xBB, 0x53), (0xD317)),  # HANGUL SYLLABLE PHIEUPH A RIEULPIEUP
    ((0xBB, 0x54), (0xD318)),  # HANGUL SYLLABLE PHIEUPH A RIEULSIOS
    ((0xBB, 0x55), (0xD319)),  # HANGUL SYLLABLE PHIEUPH A RIEULTHIEUTH
    ((0xBB, 0x56), (0xD31A)),  # HANGUL SYLLABLE PHIEUPH A RIEULPHIEUPH
    ((0xBB, 0x57), (0xD31B)),  # HANGUL SYLLABLE PHIEUPH A RIEULHIEUH
    ((0xBB, 0x58), (0xD31E)),  # HANGUL SYLLABLE PHIEUPH A PIEUPSIOS
    ((0xBB, 0x59), (0xD322)),  # HANGUL SYLLABLE PHIEUPH A CIEUC
    ((0xBB, 0x5A), (0xD323)),  # HANGUL SYLLABLE PHIEUPH A CHIEUCH
    ((0xBB, 0x61), (0xD324)),  # HANGUL SYLLABLE PHIEUPH A KHIEUKH
    ((0xBB, 0x62), (0xD326)),  # HANGUL SYLLABLE PHIEUPH A PHIEUPH
    ((0xBB, 0x63), (0xD327)),  # HANGUL SYLLABLE PHIEUPH A HIEUH
    ((0xBB, 0x64), (0xD32A)),  # HANGUL SYLLABLE PHIEUPH AE SSANGKIYEOK
    ((0xBB, 0x65), (0xD32B)),  # HANGUL SYLLABLE PHIEUPH AE KIYEOKSIOS
    ((0xBB, 0x66), (0xD32D)),  # HANGUL SYLLABLE PHIEUPH AE NIEUNCIEUC
    ((0xBB, 0x67), (0xD32E)),  # HANGUL SYLLABLE PHIEUPH AE NIEUNHIEUH
    ((0xBB, 0x68), (0xD32F)),  # HANGUL SYLLABLE PHIEUPH AE TIKEUT
    ((0xBB, 0x69), (0xD331)),  # HANGUL SYLLABLE PHIEUPH AE RIEULKIYEOK
    ((0xBB, 0x6A), (0xD332)),  # HANGUL SYLLABLE PHIEUPH AE RIEULMIEUM
    ((0xBB, 0x6B), (0xD333)),  # HANGUL SYLLABLE PHIEUPH AE RIEULPIEUP
    ((0xBB, 0x6C), (0xD334)),  # HANGUL SYLLABLE PHIEUPH AE RIEULSIOS
    ((0xBB, 0x6D), (0xD335)),  # HANGUL SYLLABLE PHIEUPH AE RIEULTHIEUTH
    ((0xBB, 0x6E), (0xD336)),  # HANGUL SYLLABLE PHIEUPH AE RIEULPHIEUPH
    ((0xBB, 0x6F), (0xD337)),  # HANGUL SYLLABLE PHIEUPH AE RIEULHIEUH
    ((0xBB, 0x70), (0xD33A)),  # HANGUL SYLLABLE PHIEUPH AE PIEUPSIOS
    ((0xBB, 0x71), (0xD33E)),  # HANGUL SYLLABLE PHIEUPH AE CIEUC
    ((0xBB, 0x72), (0xD33F)),  # HANGUL SYLLABLE PHIEUPH AE CHIEUCH
    ((0xBB, 0x73), (0xD340)),  # HANGUL SYLLABLE PHIEUPH AE KHIEUKH
    ((0xBB, 0x74), (0xD341)),  # HANGUL SYLLABLE PHIEUPH AE THIEUTH
    ((0xBB, 0x75), (0xD342)),  # HANGUL SYLLABLE PHIEUPH AE PHIEUPH
    ((0xBB, 0x76), (0xD343)),  # HANGUL SYLLABLE PHIEUPH AE HIEUH
    ((0xBB, 0x77), (0xD346)),  # HANGUL SYLLABLE PHIEUPH YA SSANGKIYEOK
    ((0xBB, 0x78), (0xD347)),  # HANGUL SYLLABLE PHIEUPH YA KIYEOKSIOS
    ((0xBB, 0x79), (0xD348)),  # HANGUL SYLLABLE PHIEUPH YA NIEUN
    ((0xBB, 0x7A), (0xD349)),  # HANGUL SYLLABLE PHIEUPH YA NIEUNCIEUC
    ((0xBB, 0x81), (0xD34A)),  # HANGUL SYLLABLE PHIEUPH YA NIEUNHIEUH
    ((0xBB, 0x82), (0xD34B)),  # HANGUL SYLLABLE PHIEUPH YA TIKEUT
    ((0xBB, 0x83), (0xD34C)),  # HANGUL SYLLABLE PHIEUPH YA RIEUL
    ((0xBB, 0x84), (0xD34D)),  # HANGUL SYLLABLE PHIEUPH YA RIEULKIYEOK
    ((0xBB, 0x85), (0xD34E)),  # HANGUL SYLLABLE PHIEUPH YA RIEULMIEUM
    ((0xBB, 0x86), (0xD34F)),  # HANGUL SYLLABLE PHIEUPH YA RIEULPIEUP
    ((0xBB, 0x87), (0xD350)),  # HANGUL SYLLABLE PHIEUPH YA RIEULSIOS
    ((0xBB, 0x88), (0xD351)),  # HANGUL SYLLABLE PHIEUPH YA RIEULTHIEUTH
    ((0xBB, 0x89), (0xD352)),  # HANGUL SYLLABLE PHIEUPH YA RIEULPHIEUPH
    ((0xBB, 0x8A), (0xD353)),  # HANGUL SYLLABLE PHIEUPH YA RIEULHIEUH
    ((0xBB, 0x8B), (0xD354)),  # HANGUL SYLLABLE PHIEUPH YA MIEUM
    ((0xBB, 0x8C), (0xD355)),  # HANGUL SYLLABLE PHIEUPH YA PIEUP
    ((0xBB, 0x8D), (0xD356)),  # HANGUL SYLLABLE PHIEUPH YA PIEUPSIOS
    ((0xBB, 0x8E), (0xD357)),  # HANGUL SYLLABLE PHIEUPH YA SIOS
    ((0xBB, 0x8F), (0xD358)),  # HANGUL SYLLABLE PHIEUPH YA SSANGSIOS
    ((0xBB, 0x90), (0xD359)),  # HANGUL SYLLABLE PHIEUPH YA IEUNG
    ((0xBB, 0x91), (0xD35A)),  # HANGUL SYLLABLE PHIEUPH YA CIEUC
    ((0xBB, 0x92), (0xD35B)),  # HANGUL SYLLABLE PHIEUPH YA CHIEUCH
    ((0xBB, 0x93), (0xD35C)),  # HANGUL SYLLABLE PHIEUPH YA KHIEUKH
    ((0xBB, 0x94), (0xD35D)),  # HANGUL SYLLABLE PHIEUPH YA THIEUTH
    ((0xBB, 0x95), (0xD35E)),  # HANGUL SYLLABLE PHIEUPH YA PHIEUPH
    ((0xBB, 0x96), (0xD35F)),  # HANGUL SYLLABLE PHIEUPH YA HIEUH
    ((0xBB, 0x97), (0xD360)),  # HANGUL SYLLABLE PHIEUPH YAE
    ((0xBB, 0x98), (0xD361)),  # HANGUL SYLLABLE PHIEUPH YAE KIYEOK
    ((0xBB, 0x99), (0xD362)),  # HANGUL SYLLABLE PHIEUPH YAE SSANGKIYEOK
    ((0xBB, 0x9A), (0xD363)),  # HANGUL SYLLABLE PHIEUPH YAE KIYEOKSIOS
    ((0xBB, 0x9B), (0xD364)),  # HANGUL SYLLABLE PHIEUPH YAE NIEUN
    ((0xBB, 0x9C), (0xD365)),  # HANGUL SYLLABLE PHIEUPH YAE NIEUNCIEUC
    ((0xBB, 0x9D), (0xD366)),  # HANGUL SYLLABLE PHIEUPH YAE NIEUNHIEUH
    ((0xBB, 0x9E), (0xD367)),  # HANGUL SYLLABLE PHIEUPH YAE TIKEUT
    ((0xBB, 0x9F), (0xD368)),  # HANGUL SYLLABLE PHIEUPH YAE RIEUL
    ((0xBB, 0xA0), (0xD369)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULKIYEOK
    ((0xBB, 0xA1), (0xBE68)),  # HANGUL SYLLABLE SSANGPIEUP A RIEUL
    ((0xBB, 0xA2), (0xBE6A)),  # HANGUL SYLLABLE SSANGPIEUP A RIEULMIEUM
    ((0xBB, 0xA3), (0xBE70)),  # HANGUL SYLLABLE SSANGPIEUP A MIEUM
    ((0xBB, 0xA4), (0xBE71)),  # HANGUL SYLLABLE SSANGPIEUP A PIEUP
    ((0xBB, 0xA5), (0xBE73)),  # HANGUL SYLLABLE SSANGPIEUP A SIOS
    ((0xBB, 0xA6), (0xBE74)),  # HANGUL SYLLABLE SSANGPIEUP A SSANGSIOS
    ((0xBB, 0xA7), (0xBE75)),  # HANGUL SYLLABLE SSANGPIEUP A IEUNG
    ((0xBB, 0xA8), (0xBE7B)),  # HANGUL SYLLABLE SSANGPIEUP A HIEUH
    ((0xBB, 0xA9), (0xBE7C)),  # HANGUL SYLLABLE SSANGPIEUP AE
    ((0xBB, 0xAA), (0xBE7D)),  # HANGUL SYLLABLE SSANGPIEUP AE KIYEOK
    ((0xBB, 0xAB), (0xBE80)),  # HANGUL SYLLABLE SSANGPIEUP AE NIEUN
    ((0xBB, 0xAC), (0xBE84)),  # HANGUL SYLLABLE SSANGPIEUP AE RIEUL
    ((0xBB, 0xAD), (0xBE8C)),  # HANGUL SYLLABLE SSANGPIEUP AE MIEUM
    ((0xBB, 0xAE), (0xBE8D)),  # HANGUL SYLLABLE SSANGPIEUP AE PIEUP
    ((0xBB, 0xAF), (0xBE8F)),  # HANGUL SYLLABLE SSANGPIEUP AE SIOS
    ((0xBB, 0xB0), (0xBE90)),  # HANGUL SYLLABLE SSANGPIEUP AE SSANGSIOS
    ((0xBB, 0xB1), (0xBE91)),  # HANGUL SYLLABLE SSANGPIEUP AE IEUNG
    ((0xBB, 0xB2), (0xBE98)),  # HANGUL SYLLABLE SSANGPIEUP YA
    ((0xBB, 0xB3), (0xBE99)),  # HANGUL SYLLABLE SSANGPIEUP YA KIYEOK
    ((0xBB, 0xB4), (0xBEA8)),  # HANGUL SYLLABLE SSANGPIEUP YA MIEUM
    ((0xBB, 0xB5), (0xBED0)),  # HANGUL SYLLABLE SSANGPIEUP EO
    ((0xBB, 0xB6), (0xBED1)),  # HANGUL SYLLABLE SSANGPIEUP EO KIYEOK
    ((0xBB, 0xB7), (0xBED4)),  # HANGUL SYLLABLE SSANGPIEUP EO NIEUN
    ((0xBB, 0xB8), (0xBED7)),  # HANGUL SYLLABLE SSANGPIEUP EO TIKEUT
    ((0xBB, 0xB9), (0xBED8)),  # HANGUL SYLLABLE SSANGPIEUP EO RIEUL
    ((0xBB, 0xBA), (0xBEE0)),  # HANGUL SYLLABLE SSANGPIEUP EO MIEUM
    ((0xBB, 0xBB), (0xBEE3)),  # HANGUL SYLLABLE SSANGPIEUP EO SIOS
    ((0xBB, 0xBC), (0xBEE4)),  # HANGUL SYLLABLE SSANGPIEUP EO SSANGSIOS
    ((0xBB, 0xBD), (0xBEE5)),  # HANGUL SYLLABLE SSANGPIEUP EO IEUNG
    ((0xBB, 0xBE), (0xBEEC)),  # HANGUL SYLLABLE SSANGPIEUP E
    ((0xBB, 0xBF), (0xBF01)),  # HANGUL SYLLABLE SSANGPIEUP E IEUNG
    ((0xBB, 0xC0), (0xBF08)),  # HANGUL SYLLABLE SSANGPIEUP YEO
    ((0xBB, 0xC1), (0xBF09)),  # HANGUL SYLLABLE SSANGPIEUP YEO KIYEOK
    ((0xBB, 0xC2), (0xBF18)),  # HANGUL SYLLABLE SSANGPIEUP YEO MIEUM
    ((0xBB, 0xC3), (0xBF19)),  # HANGUL SYLLABLE SSANGPIEUP YEO PIEUP
    ((0xBB, 0xC4), (0xBF1B)),  # HANGUL SYLLABLE SSANGPIEUP YEO SIOS
    ((0xBB, 0xC5), (0xBF1C)),  # HANGUL SYLLABLE SSANGPIEUP YEO SSANGSIOS
    ((0xBB, 0xC6), (0xBF1D)),  # HANGUL SYLLABLE SSANGPIEUP YEO IEUNG
    ((0xBB, 0xC7), (0xBF40)),  # HANGUL SYLLABLE SSANGPIEUP O
    ((0xBB, 0xC8), (0xBF41)),  # HANGUL SYLLABLE SSANGPIEUP O KIYEOK
    ((0xBB, 0xC9), (0xBF44)),  # HANGUL SYLLABLE SSANGPIEUP O NIEUN
    ((0xBB, 0xCA), (0xBF48)),  # HANGUL SYLLABLE SSANGPIEUP O RIEUL
    ((0xBB, 0xCB), (0xBF50)),  # HANGUL SYLLABLE SSANGPIEUP O MIEUM
    ((0xBB, 0xCC), (0xBF51)),  # HANGUL SYLLABLE SSANGPIEUP O PIEUP
    ((0xBB, 0xCD), (0xBF55)),  # HANGUL SYLLABLE SSANGPIEUP O IEUNG
    ((0xBB, 0xCE), (0xBF94)),  # HANGUL SYLLABLE SSANGPIEUP OE
    ((0xBB, 0xCF), (0xBFB0)),  # HANGUL SYLLABLE SSANGPIEUP YO
    ((0xBB, 0xD0), (0xBFC5)),  # HANGUL SYLLABLE SSANGPIEUP YO IEUNG
    ((0xBB, 0xD1), (0xBFCC)),  # HANGUL SYLLABLE SSANGPIEUP U
    ((0xBB, 0xD2), (0xBFCD)),  # HANGUL SYLLABLE SSANGPIEUP U KIYEOK
    ((0xBB, 0xD3), (0xBFD0)),  # HANGUL SYLLABLE SSANGPIEUP U NIEUN
    ((0xBB, 0xD4), (0xBFD4)),  # HANGUL SYLLABLE SSANGPIEUP U RIEUL
    ((0xBB, 0xD5), (0xBFDC)),  # HANGUL SYLLABLE SSANGPIEUP U MIEUM
    ((0xBB, 0xD6), (0xBFDF)),  # HANGUL SYLLABLE SSANGPIEUP U SIOS
    ((0xBB, 0xD7), (0xBFE1)),  # HANGUL SYLLABLE SSANGPIEUP U IEUNG
    ((0xBB, 0xD8), (0xC03C)),  # HANGUL SYLLABLE SSANGPIEUP YU
    ((0xBB, 0xD9), (0xC051)),  # HANGUL SYLLABLE SSANGPIEUP YU IEUNG
    ((0xBB, 0xDA), (0xC058)),  # HANGUL SYLLABLE SSANGPIEUP EU
    ((0xBB, 0xDB), (0xC05C)),  # HANGUL SYLLABLE SSANGPIEUP EU NIEUN
    ((0xBB, 0xDC), (0xC060)),  # HANGUL SYLLABLE SSANGPIEUP EU RIEUL
    ((0xBB, 0xDD), (0xC068)),  # HANGUL SYLLABLE SSANGPIEUP EU MIEUM
    ((0xBB, 0xDE), (0xC069)),  # HANGUL SYLLABLE SSANGPIEUP EU PIEUP
    ((0xBB, 0xDF), (0xC090)),  # HANGUL SYLLABLE SSANGPIEUP I
    ((0xBB, 0xE0), (0xC091)),  # HANGUL SYLLABLE SSANGPIEUP I KIYEOK
    ((0xBB, 0xE1), (0xC094)),  # HANGUL SYLLABLE SSANGPIEUP I NIEUN
    ((0xBB, 0xE2), (0xC098)),  # HANGUL SYLLABLE SSANGPIEUP I RIEUL
    ((0xBB, 0xE3), (0xC0A0)),  # HANGUL SYLLABLE SSANGPIEUP I MIEUM
    ((0xBB, 0xE4), (0xC0A1)),  # HANGUL SYLLABLE SSANGPIEUP I PIEUP
    ((0xBB, 0xE5), (0xC0A3)),  # HANGUL SYLLABLE SSANGPIEUP I SIOS
    ((0xBB, 0xE6), (0xC0A5)),  # HANGUL SYLLABLE SSANGPIEUP I IEUNG
    ((0xBB, 0xE7), (0xC0AC)),  # HANGUL SYLLABLE SIOS A
    ((0xBB, 0xE8), (0xC0AD)),  # HANGUL SYLLABLE SIOS A KIYEOK
    ((0xBB, 0xE9), (0xC0AF)),  # HANGUL SYLLABLE SIOS A KIYEOKSIOS
    ((0xBB, 0xEA), (0xC0B0)),  # HANGUL SYLLABLE SIOS A NIEUN
    ((0xBB, 0xEB), (0xC0B3)),  # HANGUL SYLLABLE SIOS A TIKEUT
    ((0xBB, 0xEC), (0xC0B4)),  # HANGUL SYLLABLE SIOS A RIEUL
    ((0xBB, 0xED), (0xC0B5)),  # HANGUL SYLLABLE SIOS A RIEULKIYEOK
    ((0xBB, 0xEE), (0xC0B6)),  # HANGUL SYLLABLE SIOS A RIEULMIEUM
    ((0xBB, 0xEF), (0xC0BC)),  # HANGUL SYLLABLE SIOS A MIEUM
    ((0xBB, 0xF0), (0xC0BD)),  # HANGUL SYLLABLE SIOS A PIEUP
    ((0xBB, 0xF1), (0xC0BF)),  # HANGUL SYLLABLE SIOS A SIOS
    ((0xBB, 0xF2), (0xC0C0)),  # HANGUL SYLLABLE SIOS A SSANGSIOS
    ((0xBB, 0xF3), (0xC0C1)),  # HANGUL SYLLABLE SIOS A IEUNG
    ((0xBB, 0xF4), (0xC0C5)),  # HANGUL SYLLABLE SIOS A THIEUTH
    ((0xBB, 0xF5), (0xC0C8)),  # HANGUL SYLLABLE SIOS AE
    ((0xBB, 0xF6), (0xC0C9)),  # HANGUL SYLLABLE SIOS AE KIYEOK
    ((0xBB, 0xF7), (0xC0CC)),  # HANGUL SYLLABLE SIOS AE NIEUN
    ((0xBB, 0xF8), (0xC0D0)),  # HANGUL SYLLABLE SIOS AE RIEUL
    ((0xBB, 0xF9), (0xC0D8)),  # HANGUL SYLLABLE SIOS AE MIEUM
    ((0xBB, 0xFA), (0xC0D9)),  # HANGUL SYLLABLE SIOS AE PIEUP
    ((0xBB, 0xFB), (0xC0DB)),  # HANGUL SYLLABLE SIOS AE SIOS
    ((0xBB, 0xFC), (0xC0DC)),  # HANGUL SYLLABLE SIOS AE SSANGSIOS
    ((0xBB, 0xFD), (0xC0DD)),  # HANGUL SYLLABLE SIOS AE IEUNG
    ((0xBB, 0xFE), (0xC0E4)),  # HANGUL SYLLABLE SIOS YA
    ((0xBC, 0x41), (0xD36A)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULMIEUM
    ((0xBC, 0x42), (0xD36B)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULPIEUP
    ((0xBC, 0x43), (0xD36C)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULSIOS
    ((0xBC, 0x44), (0xD36D)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULTHIEUTH
    ((0xBC, 0x45), (0xD36E)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULPHIEUPH
    ((0xBC, 0x46), (0xD36F)),  # HANGUL SYLLABLE PHIEUPH YAE RIEULHIEUH
    ((0xBC, 0x47), (0xD370)),  # HANGUL SYLLABLE PHIEUPH YAE MIEUM
    ((0xBC, 0x48), (0xD371)),  # HANGUL SYLLABLE PHIEUPH YAE PIEUP
    ((0xBC, 0x49), (0xD372)),  # HANGUL SYLLABLE PHIEUPH YAE PIEUPSIOS
    ((0xBC, 0x4A), (0xD373)),  # HANGUL SYLLABLE PHIEUPH YAE SIOS
    ((0xBC, 0x4B), (0xD374)),  # HANGUL SYLLABLE PHIEUPH YAE SSANGSIOS
    ((0xBC, 0x4C), (0xD375)),  # HANGUL SYLLABLE PHIEUPH YAE IEUNG
    ((0xBC, 0x4D), (0xD376)),  # HANGUL SYLLABLE PHIEUPH YAE CIEUC
    ((0xBC, 0x4E), (0xD377)),  # HANGUL SYLLABLE PHIEUPH YAE CHIEUCH
    ((0xBC, 0x4F), (0xD378)),  # HANGUL SYLLABLE PHIEUPH YAE KHIEUKH
    ((0xBC, 0x50), (0xD379)),  # HANGUL SYLLABLE PHIEUPH YAE THIEUTH
    ((0xBC, 0x51), (0xD37A)),  # HANGUL SYLLABLE PHIEUPH YAE PHIEUPH
    ((0xBC, 0x52), (0xD37B)),  # HANGUL SYLLABLE PHIEUPH YAE HIEUH
    ((0xBC, 0x53), (0xD37E)),  # HANGUL SYLLABLE PHIEUPH EO SSANGKIYEOK
    ((0xBC, 0x54), (0xD37F)),  # HANGUL SYLLABLE PHIEUPH EO KIYEOKSIOS
    ((0xBC, 0x55), (0xD381)),  # HANGUL SYLLABLE PHIEUPH EO NIEUNCIEUC
    ((0xBC, 0x56), (0xD382)),  # HANGUL SYLLABLE PHIEUPH EO NIEUNHIEUH
    ((0xBC, 0x57), (0xD383)),  # HANGUL SYLLABLE PHIEUPH EO TIKEUT
    ((0xBC, 0x58), (0xD385)),  # HANGUL SYLLABLE PHIEUPH EO RIEULKIYEOK
    ((0xBC, 0x59), (0xD386)),  # HANGUL SYLLABLE PHIEUPH EO RIEULMIEUM
    ((0xBC, 0x5A), (0xD387)),  # HANGUL SYLLABLE PHIEUPH EO RIEULPIEUP
    ((0xBC, 0x61), (0xD388)),  # HANGUL SYLLABLE PHIEUPH EO RIEULSIOS
    ((0xBC, 0x62), (0xD389)),  # HANGUL SYLLABLE PHIEUPH EO RIEULTHIEUTH
    ((0xBC, 0x63), (0xD38A)),  # HANGUL SYLLABLE PHIEUPH EO RIEULPHIEUPH
    ((0xBC, 0x64), (0xD38B)),  # HANGUL SYLLABLE PHIEUPH EO RIEULHIEUH
    ((0xBC, 0x65), (0xD38E)),  # HANGUL SYLLABLE PHIEUPH EO PIEUPSIOS
    ((0xBC, 0x66), (0xD392)),  # HANGUL SYLLABLE PHIEUPH EO CIEUC
    ((0xBC, 0x67), (0xD393)),  # HANGUL SYLLABLE PHIEUPH EO CHIEUCH
    ((0xBC, 0x68), (0xD394)),  # HANGUL SYLLABLE PHIEUPH EO KHIEUKH
    ((0xBC, 0x69), (0xD395)),  # HANGUL SYLLABLE PHIEUPH EO THIEUTH
    ((0xBC, 0x6A), (0xD396)),  # HANGUL SYLLABLE PHIEUPH EO PHIEUPH
    ((0xBC, 0x6B), (0xD397)),  # HANGUL SYLLABLE PHIEUPH EO HIEUH
    ((0xBC, 0x6C), (0xD39A)),  # HANGUL SYLLABLE PHIEUPH E SSANGKIYEOK
    ((0xBC, 0x6D), (0xD39B)),  # HANGUL SYLLABLE PHIEUPH E KIYEOKSIOS
    ((0xBC, 0x6E), (0xD39D)),  # HANGUL SYLLABLE PHIEUPH E NIEUNCIEUC
    ((0xBC, 0x6F), (0xD39E)),  # HANGUL SYLLABLE PHIEUPH E NIEUNHIEUH
    ((0xBC, 0x70), (0xD39F)),  # HANGUL SYLLABLE PHIEUPH E TIKEUT
    ((0xBC, 0x71), (0xD3A1)),  # HANGUL SYLLABLE PHIEUPH E RIEULKIYEOK
    ((0xBC, 0x72), (0xD3A2)),  # HANGUL SYLLABLE PHIEUPH E RIEULMIEUM
    ((0xBC, 0x73), (0xD3A3)),  # HANGUL SYLLABLE PHIEUPH E RIEULPIEUP
    ((0xBC, 0x74), (0xD3A4)),  # HANGUL SYLLABLE PHIEUPH E RIEULSIOS
    ((0xBC, 0x75), (0xD3A5)),  # HANGUL SYLLABLE PHIEUPH E RIEULTHIEUTH
    ((0xBC, 0x76), (0xD3A6)),  # HANGUL SYLLABLE PHIEUPH E RIEULPHIEUPH
    ((0xBC, 0x77), (0xD3A7)),  # HANGUL SYLLABLE PHIEUPH E RIEULHIEUH
    ((0xBC, 0x78), (0xD3AA)),  # HANGUL SYLLABLE PHIEUPH E PIEUPSIOS
    ((0xBC, 0x79), (0xD3AC)),  # HANGUL SYLLABLE PHIEUPH E SSANGSIOS
    ((0xBC, 0x7A), (0xD3AE)),  # HANGUL SYLLABLE PHIEUPH E CIEUC
    ((0xBC, 0x81), (0xD3AF)),  # HANGUL SYLLABLE PHIEUPH E CHIEUCH
    ((0xBC, 0x82), (0xD3B0)),  # HANGUL SYLLABLE PHIEUPH E KHIEUKH
    ((0xBC, 0x83), (0xD3B1)),  # HANGUL SYLLABLE PHIEUPH E THIEUTH
    ((0xBC, 0x84), (0xD3B2)),  # HANGUL SYLLABLE PHIEUPH E PHIEUPH
    ((0xBC, 0x85), (0xD3B3)),  # HANGUL SYLLABLE PHIEUPH E HIEUH
    ((0xBC, 0x86), (0xD3B5)),  # HANGUL SYLLABLE PHIEUPH YEO KIYEOK
    ((0xBC, 0x87), (0xD3B6)),  # HANGUL SYLLABLE PHIEUPH YEO SSANGKIYEOK
    ((0xBC, 0x88), (0xD3B7)),  # HANGUL SYLLABLE PHIEUPH YEO KIYEOKSIOS
    ((0xBC, 0x89), (0xD3B9)),  # HANGUL SYLLABLE PHIEUPH YEO NIEUNCIEUC
    ((0xBC, 0x8A), (0xD3BA)),  # HANGUL SYLLABLE PHIEUPH YEO NIEUNHIEUH
    ((0xBC, 0x8B), (0xD3BB)),  # HANGUL SYLLABLE PHIEUPH YEO TIKEUT
    ((0xBC, 0x8C), (0xD3BD)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULKIYEOK
    ((0xBC, 0x8D), (0xD3BE)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULMIEUM
    ((0xBC, 0x8E), (0xD3BF)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULPIEUP
    ((0xBC, 0x8F), (0xD3C0)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULSIOS
    ((0xBC, 0x90), (0xD3C1)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULTHIEUTH
    ((0xBC, 0x91), (0xD3C2)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULPHIEUPH
    ((0xBC, 0x92), (0xD3C3)),  # HANGUL SYLLABLE PHIEUPH YEO RIEULHIEUH
    ((0xBC, 0x93), (0xD3C6)),  # HANGUL SYLLABLE PHIEUPH YEO PIEUPSIOS
    ((0xBC, 0x94), (0xD3C7)),  # HANGUL SYLLABLE PHIEUPH YEO SIOS
    ((0xBC, 0x95), (0xD3CA)),  # HANGUL SYLLABLE PHIEUPH YEO CIEUC
    ((0xBC, 0x96), (0xD3CB)),  # HANGUL SYLLABLE PHIEUPH YEO CHIEUCH
    ((0xBC, 0x97), (0xD3CC)),  # HANGUL SYLLABLE PHIEUPH YEO KHIEUKH
    ((0xBC, 0x98), (0xD3CD)),  # HANGUL SYLLABLE PHIEUPH YEO THIEUTH
    ((0xBC, 0x99), (0xD3CE)),  # HANGUL SYLLABLE PHIEUPH YEO PHIEUPH
    ((0xBC, 0x9A), (0xD3CF)),  # HANGUL SYLLABLE PHIEUPH YEO HIEUH
    ((0xBC, 0x9B), (0xD3D1)),  # HANGUL SYLLABLE PHIEUPH YE KIYEOK
    ((0xBC, 0x9C), (0xD3D2)),  # HANGUL SYLLABLE PHIEUPH YE SSANGKIYEOK
    ((0xBC, 0x9D), (0xD3D3)),  # HANGUL SYLLABLE PHIEUPH YE KIYEOKSIOS
    ((0xBC, 0x9E), (0xD3D4)),  # HANGUL SYLLABLE PHIEUPH YE NIEUN
    ((0xBC, 0x9F), (0xD3D5)),  # HANGUL SYLLABLE PHIEUPH YE NIEUNCIEUC
    ((0xBC, 0xA0), (0xD3D6)),  # HANGUL SYLLABLE PHIEUPH YE NIEUNHIEUH
    ((0xBC, 0xA1), (0xC0E5)),  # HANGUL SYLLABLE SIOS YA KIYEOK
    ((0xBC, 0xA2), (0xC0E8)),  # HANGUL SYLLABLE SIOS YA NIEUN
    ((0xBC, 0xA3), (0xC0EC)),  # HANGUL SYLLABLE SIOS YA RIEUL
    ((0xBC, 0xA4), (0xC0F4)),  # HANGUL SYLLABLE SIOS YA MIEUM
    ((0xBC, 0xA5), (0xC0F5)),  # HANGUL SYLLABLE SIOS YA PIEUP
    ((0xBC, 0xA6), (0xC0F7)),  # HANGUL SYLLABLE SIOS YA SIOS
    ((0xBC, 0xA7), (0xC0F9)),  # HANGUL SYLLABLE SIOS YA IEUNG
    ((0xBC, 0xA8), (0xC100)),  # HANGUL SYLLABLE SIOS YAE
    ((0xBC, 0xA9), (0xC104)),  # HANGUL SYLLABLE SIOS YAE NIEUN
    ((0xBC, 0xAA), (0xC108)),  # HANGUL SYLLABLE SIOS YAE RIEUL
    ((0xBC, 0xAB), (0xC110)),  # HANGUL SYLLABLE SIOS YAE MIEUM
    ((0xBC, 0xAC), (0xC115)),  # HANGUL SYLLABLE SIOS YAE IEUNG
    ((0xBC, 0xAD), (0xC11C)),  # HANGUL SYLLABLE SIOS EO
    ((0xBC, 0xAE), (0xC11D)),  # HANGUL SYLLABLE SIOS EO KIYEOK
    ((0xBC, 0xAF), (0xC11E)),  # HANGUL SYLLABLE SIOS EO SSANGKIYEOK
    ((0xBC, 0xB0), (0xC11F)),  # HANGUL SYLLABLE SIOS EO KIYEOKSIOS
    ((0xBC, 0xB1), (0xC120)),  # HANGUL SYLLABLE SIOS EO NIEUN
    ((0xBC, 0xB2), (0xC123)),  # HANGUL SYLLABLE SIOS EO TIKEUT
    ((0xBC, 0xB3), (0xC124)),  # HANGUL SYLLABLE SIOS EO RIEUL
    ((0xBC, 0xB4), (0xC126)),  # HANGUL SYLLABLE SIOS EO RIEULMIEUM
    ((0xBC, 0xB5), (0xC127)),  # HANGUL SYLLABLE SIOS EO RIEULPIEUP
    ((0xBC, 0xB6), (0xC12C)),  # HANGUL SYLLABLE SIOS EO MIEUM
    ((0xBC, 0xB7), (0xC12D)),  # HANGUL SYLLABLE SIOS EO PIEUP
    ((0xBC, 0xB8), (0xC12F)),  # HANGUL SYLLABLE SIOS EO SIOS
    ((0xBC, 0xB9), (0xC130)),  # HANGUL SYLLABLE SIOS EO SSANGSIOS
    ((0xBC, 0xBA), (0xC131)),  # HANGUL SYLLABLE SIOS EO IEUNG
    ((0xBC, 0xBB), (0xC136)),  # HANGUL SYLLABLE SIOS EO PHIEUPH
    ((0xBC, 0xBC), (0xC138)),  # HANGUL SYLLABLE SIOS E
    ((0xBC, 0xBD), (0xC139)),  # HANGUL SYLLABLE SIOS E KIYEOK
    ((0xBC, 0xBE), (0xC13C)),  # HANGUL SYLLABLE SIOS E NIEUN
    ((0xBC, 0xBF), (0xC140)),  # HANGUL SYLLABLE SIOS E RIEUL
    ((0xBC, 0xC0), (0xC148)),  # HANGUL SYLLABLE SIOS E MIEUM
    ((0xBC, 0xC1), (0xC149)),  # HANGUL SYLLABLE SIOS E PIEUP
    ((0xBC, 0xC2), (0xC14B)),  # HANGUL SYLLABLE SIOS E SIOS
    ((0xBC, 0xC3), (0xC14C)),  # HANGUL SYLLABLE SIOS E SSANGSIOS
    ((0xBC, 0xC4), (0xC14D)),  # HANGUL SYLLABLE SIOS E IEUNG
    ((0xBC, 0xC5), (0xC154)),  # HANGUL SYLLABLE SIOS YEO
    ((0xBC, 0xC6), (0xC155)),  # HANGUL SYLLABLE SIOS YEO KIYEOK
    ((0xBC, 0xC7), (0xC158)),  # HANGUL SYLLABLE SIOS YEO NIEUN
    ((0xBC, 0xC8), (0xC15C)),  # HANGUL SYLLABLE SIOS YEO RIEUL
    ((0xBC, 0xC9), (0xC164)),  # HANGUL SYLLABLE SIOS YEO MIEUM
    ((0xBC, 0xCA), (0xC165)),  # HANGUL SYLLABLE SIOS YEO PIEUP
    ((0xBC, 0xCB), (0xC167)),  # HANGUL SYLLABLE SIOS YEO SIOS
    ((0xBC, 0xCC), (0xC168)),  # HANGUL SYLLABLE SIOS YEO SSANGSIOS
    ((0xBC, 0xCD), (0xC169)),  # HANGUL SYLLABLE SIOS YEO IEUNG
    ((0xBC, 0xCE), (0xC170)),  # HANGUL SYLLABLE SIOS YE
    ((0xBC, 0xCF), (0xC174)),  # HANGUL SYLLABLE SIOS YE NIEUN
    ((0xBC, 0xD0), (0xC178)),  # HANGUL SYLLABLE SIOS YE RIEUL
    ((0xBC, 0xD1), (0xC185)),  # HANGUL SYLLABLE SIOS YE IEUNG
    ((0xBC, 0xD2), (0xC18C)),  # HANGUL SYLLABLE SIOS O
    ((0xBC, 0xD3), (0xC18D)),  # HANGUL SYLLABLE SIOS O KIYEOK
    ((0xBC, 0xD4), (0xC18E)),  # HANGUL SYLLABLE SIOS O SSANGKIYEOK
    ((0xBC, 0xD5), (0xC190)),  # HANGUL SYLLABLE SIOS O NIEUN
    ((0xBC, 0xD6), (0xC194)),  # HANGUL SYLLABLE SIOS O RIEUL
    ((0xBC, 0xD7), (0xC196)),  # HANGUL SYLLABLE SIOS O RIEULMIEUM
    ((0xBC, 0xD8), (0xC19C)),  # HANGUL SYLLABLE SIOS O MIEUM
    ((0xBC, 0xD9), (0xC19D)),  # HANGUL SYLLABLE SIOS O PIEUP
    ((0xBC, 0xDA), (0xC19F)),  # HANGUL SYLLABLE SIOS O SIOS
    ((0xBC, 0xDB), (0xC1A1)),  # HANGUL SYLLABLE SIOS O IEUNG
    ((0xBC, 0xDC), (0xC1A5)),  # HANGUL SYLLABLE SIOS O THIEUTH
    ((0xBC, 0xDD), (0xC1A8)),  # HANGUL SYLLABLE SIOS WA
    ((0xBC, 0xDE), (0xC1A9)),  # HANGUL SYLLABLE SIOS WA KIYEOK
    ((0xBC, 0xDF), (0xC1AC)),  # HANGUL SYLLABLE SIOS WA NIEUN
    ((0xBC, 0xE0), (0xC1B0)),  # HANGUL SYLLABLE SIOS WA RIEUL
    ((0xBC, 0xE1), (0xC1BD)),  # HANGUL SYLLABLE SIOS WA IEUNG
    ((0xBC, 0xE2), (0xC1C4)),  # HANGUL SYLLABLE SIOS WAE
    ((0xBC, 0xE3), (0xC1C8)),  # HANGUL SYLLABLE SIOS WAE NIEUN
    ((0xBC, 0xE4), (0xC1CC)),  # HANGUL SYLLABLE SIOS WAE RIEUL
    ((0xBC, 0xE5), (0xC1D4)),  # HANGUL SYLLABLE SIOS WAE MIEUM
    ((0xBC, 0xE6), (0xC1D7)),  # HANGUL SYLLABLE SIOS WAE SIOS
    ((0xBC, 0xE7), (0xC1D8)),  # HANGUL SYLLABLE SIOS WAE SSANGSIOS
    ((0xBC, 0xE8), (0xC1E0)),  # HANGUL SYLLABLE SIOS OE
    ((0xBC, 0xE9), (0xC1E4)),  # HANGUL SYLLABLE SIOS OE NIEUN
    ((0xBC, 0xEA), (0xC1E8)),  # HANGUL SYLLABLE SIOS OE RIEUL
    ((0xBC, 0xEB), (0xC1F0)),  # HANGUL SYLLABLE SIOS OE MIEUM
    ((0xBC, 0xEC), (0xC1F1)),  # HANGUL SYLLABLE SIOS OE PIEUP
    ((0xBC, 0xED), (0xC1F3)),  # HANGUL SYLLABLE SIOS OE SIOS
    ((0xBC, 0xEE), (0xC1FC)),  # HANGUL SYLLABLE SIOS YO
    ((0xBC, 0xEF), (0xC1FD)),  # HANGUL SYLLABLE SIOS YO KIYEOK
    ((0xBC, 0xF0), (0xC200)),  # HANGUL SYLLABLE SIOS YO NIEUN
    ((0xBC, 0xF1), (0xC204)),  # HANGUL SYLLABLE SIOS YO RIEUL
    ((0xBC, 0xF2), (0xC20C)),  # HANGUL SYLLABLE SIOS YO MIEUM
    ((0xBC, 0xF3), (0xC20D)),  # HANGUL SYLLABLE SIOS YO PIEUP
    ((0xBC, 0xF4), (0xC20F)),  # HANGUL SYLLABLE SIOS YO SIOS
    ((0xBC, 0xF5), (0xC211)),  # HANGUL SYLLABLE SIOS YO IEUNG
    ((0xBC, 0xF6), (0xC218)),  # HANGUL SYLLABLE SIOS U
    ((0xBC, 0xF7), (0xC219)),  # HANGUL SYLLABLE SIOS U KIYEOK
    ((0xBC, 0xF8), (0xC21C)),  # HANGUL SYLLABLE SIOS U NIEUN
    ((0xBC, 0xF9), (0xC21F)),  # HANGUL SYLLABLE SIOS U TIKEUT
    ((0xBC, 0xFA), (0xC220)),  # HANGUL SYLLABLE SIOS U RIEUL
    ((0xBC, 0xFB), (0xC228)),  # HANGUL SYLLABLE SIOS U MIEUM
    ((0xBC, 0xFC), (0xC229)),  # HANGUL SYLLABLE SIOS U PIEUP
    ((0xBC, 0xFD), (0xC22B)),  # HANGUL SYLLABLE SIOS U SIOS
    ((0xBC, 0xFE), (0xC22D)),  # HANGUL SYLLABLE SIOS U IEUNG
    ((0xBD, 0x41), (0xD3D7)),  # HANGUL SYLLABLE PHIEUPH YE TIKEUT
    ((0xBD, 0x42), (0xD3D9)),  # HANGUL SYLLABLE PHIEUPH YE RIEULKIYEOK
    ((0xBD, 0x43), (0xD3DA)),  # HANGUL SYLLABLE PHIEUPH YE RIEULMIEUM
    ((0xBD, 0x44), (0xD3DB)),  # HANGUL SYLLABLE PHIEUPH YE RIEULPIEUP
    ((0xBD, 0x45), (0xD3DC)),  # HANGUL SYLLABLE PHIEUPH YE RIEULSIOS
    ((0xBD, 0x46), (0xD3DD)),  # HANGUL SYLLABLE PHIEUPH YE RIEULTHIEUTH
    ((0xBD, 0x47), (0xD3DE)),  # HANGUL SYLLABLE PHIEUPH YE RIEULPHIEUPH
    ((0xBD, 0x48), (0xD3DF)),  # HANGUL SYLLABLE PHIEUPH YE RIEULHIEUH
    ((0xBD, 0x49), (0xD3E0)),  # HANGUL SYLLABLE PHIEUPH YE MIEUM
    ((0xBD, 0x4A), (0xD3E2)),  # HANGUL SYLLABLE PHIEUPH YE PIEUPSIOS
    ((0xBD, 0x4B), (0xD3E4)),  # HANGUL SYLLABLE PHIEUPH YE SSANGSIOS
    ((0xBD, 0x4C), (0xD3E5)),  # HANGUL SYLLABLE PHIEUPH YE IEUNG
    ((0xBD, 0x4D), (0xD3E6)),  # HANGUL SYLLABLE PHIEUPH YE CIEUC
    ((0xBD, 0x4E), (0xD3E7)),  # HANGUL SYLLABLE PHIEUPH YE CHIEUCH
    ((0xBD, 0x4F), (0xD3E8)),  # HANGUL SYLLABLE PHIEUPH YE KHIEUKH
    ((0xBD, 0x50), (0xD3E9)),  # HANGUL SYLLABLE PHIEUPH YE THIEUTH
    ((0xBD, 0x51), (0xD3EA)),  # HANGUL SYLLABLE PHIEUPH YE PHIEUPH
    ((0xBD, 0x52), (0xD3EB)),  # HANGUL SYLLABLE PHIEUPH YE HIEUH
    ((0xBD, 0x53), (0xD3EE)),  # HANGUL SYLLABLE PHIEUPH O SSANGKIYEOK
    ((0xBD, 0x54), (0xD3EF)),  # HANGUL SYLLABLE PHIEUPH O KIYEOKSIOS
    ((0xBD, 0x55), (0xD3F1)),  # HANGUL SYLLABLE PHIEUPH O NIEUNCIEUC
    ((0xBD, 0x56), (0xD3F2)),  # HANGUL SYLLABLE PHIEUPH O NIEUNHIEUH
    ((0xBD, 0x57), (0xD3F3)),  # HANGUL SYLLABLE PHIEUPH O TIKEUT
    ((0xBD, 0x58), (0xD3F5)),  # HANGUL SYLLABLE PHIEUPH O RIEULKIYEOK
    ((0xBD, 0x59), (0xD3F6)),  # HANGUL SYLLABLE PHIEUPH O RIEULMIEUM
    ((0xBD, 0x5A), (0xD3F7)),  # HANGUL SYLLABLE PHIEUPH O RIEULPIEUP
    ((0xBD, 0x61), (0xD3F8)),  # HANGUL SYLLABLE PHIEUPH O RIEULSIOS
    ((0xBD, 0x62), (0xD3F9)),  # HANGUL SYLLABLE PHIEUPH O RIEULTHIEUTH
    ((0xBD, 0x63), (0xD3FA)),  # HANGUL SYLLABLE PHIEUPH O RIEULPHIEUPH
    ((0xBD, 0x64), (0xD3FB)),  # HANGUL SYLLABLE PHIEUPH O RIEULHIEUH
    ((0xBD, 0x65), (0xD3FE)),  # HANGUL SYLLABLE PHIEUPH O PIEUPSIOS
    ((0xBD, 0x66), (0xD400)),  # HANGUL SYLLABLE PHIEUPH O SSANGSIOS
    ((0xBD, 0x67), (0xD402)),  # HANGUL SYLLABLE PHIEUPH O CIEUC
    ((0xBD, 0x68), (0xD403)),  # HANGUL SYLLABLE PHIEUPH O CHIEUCH
    ((0xBD, 0x69), (0xD404)),  # HANGUL SYLLABLE PHIEUPH O KHIEUKH
    ((0xBD, 0x6A), (0xD405)),  # HANGUL SYLLABLE PHIEUPH O THIEUTH
    ((0xBD, 0x6B), (0xD406)),  # HANGUL SYLLABLE PHIEUPH O PHIEUPH
    ((0xBD, 0x6C), (0xD407)),  # HANGUL SYLLABLE PHIEUPH O HIEUH
    ((0xBD, 0x6D), (0xD409)),  # HANGUL SYLLABLE PHIEUPH WA KIYEOK
    ((0xBD, 0x6E), (0xD40A)),  # HANGUL SYLLABLE PHIEUPH WA SSANGKIYEOK
    ((0xBD, 0x6F), (0xD40B)),  # HANGUL SYLLABLE PHIEUPH WA KIYEOKSIOS
    ((0xBD, 0x70), (0xD40C)),  # HANGUL SYLLABLE PHIEUPH WA NIEUN
    ((0xBD, 0x71), (0xD40D)),  # HANGUL SYLLABLE PHIEUPH WA NIEUNCIEUC
    ((0xBD, 0x72), (0xD40E)),  # HANGUL SYLLABLE PHIEUPH WA NIEUNHIEUH
    ((0xBD, 0x73), (0xD40F)),  # HANGUL SYLLABLE PHIEUPH WA TIKEUT
    ((0xBD, 0x74), (0xD410)),  # HANGUL SYLLABLE PHIEUPH WA RIEUL
    ((0xBD, 0x75), (0xD411)),  # HANGUL SYLLABLE PHIEUPH WA RIEULKIYEOK
    ((0xBD, 0x76), (0xD412)),  # HANGUL SYLLABLE PHIEUPH WA RIEULMIEUM
    ((0xBD, 0x77), (0xD413)),  # HANGUL SYLLABLE PHIEUPH WA RIEULPIEUP
    ((0xBD, 0x78), (0xD414)),  # HANGUL SYLLABLE PHIEUPH WA RIEULSIOS
    ((0xBD, 0x79), (0xD415)),  # HANGUL SYLLABLE PHIEUPH WA RIEULTHIEUTH
    ((0xBD, 0x7A), (0xD416)),  # HANGUL SYLLABLE PHIEUPH WA RIEULPHIEUPH
    ((0xBD, 0x81), (0xD417)),  # HANGUL SYLLABLE PHIEUPH WA RIEULHIEUH
    ((0xBD, 0x82), (0xD418)),  # HANGUL SYLLABLE PHIEUPH WA MIEUM
    ((0xBD, 0x83), (0xD419)),  # HANGUL SYLLABLE PHIEUPH WA PIEUP
    ((0xBD, 0x84), (0xD41A)),  # HANGUL SYLLABLE PHIEUPH WA PIEUPSIOS
    ((0xBD, 0x85), (0xD41B)),  # HANGUL SYLLABLE PHIEUPH WA SIOS
    ((0xBD, 0x86), (0xD41C)),  # HANGUL SYLLABLE PHIEUPH WA SSANGSIOS
    ((0xBD, 0x87), (0xD41E)),  # HANGUL SYLLABLE PHIEUPH WA CIEUC
    ((0xBD, 0x88), (0xD41F)),  # HANGUL SYLLABLE PHIEUPH WA CHIEUCH
    ((0xBD, 0x89), (0xD420)),  # HANGUL SYLLABLE PHIEUPH WA KHIEUKH
    ((0xBD, 0x8A), (0xD421)),  # HANGUL SYLLABLE PHIEUPH WA THIEUTH
    ((0xBD, 0x8B), (0xD422)),  # HANGUL SYLLABLE PHIEUPH WA PHIEUPH
    ((0xBD, 0x8C), (0xD423)),  # HANGUL SYLLABLE PHIEUPH WA HIEUH
    ((0xBD, 0x8D), (0xD424)),  # HANGUL SYLLABLE PHIEUPH WAE
    ((0xBD, 0x8E), (0xD425)),  # HANGUL SYLLABLE PHIEUPH WAE KIYEOK
    ((0xBD, 0x8F), (0xD426)),  # HANGUL SYLLABLE PHIEUPH WAE SSANGKIYEOK
    ((0xBD, 0x90), (0xD427)),  # HANGUL SYLLABLE PHIEUPH WAE KIYEOKSIOS
    ((0xBD, 0x91), (0xD428)),  # HANGUL SYLLABLE PHIEUPH WAE NIEUN
    ((0xBD, 0x92), (0xD429)),  # HANGUL SYLLABLE PHIEUPH WAE NIEUNCIEUC
    ((0xBD, 0x93), (0xD42A)),  # HANGUL SYLLABLE PHIEUPH WAE NIEUNHIEUH
    ((0xBD, 0x94), (0xD42B)),  # HANGUL SYLLABLE PHIEUPH WAE TIKEUT
    ((0xBD, 0x95), (0xD42C)),  # HANGUL SYLLABLE PHIEUPH WAE RIEUL
    ((0xBD, 0x96), (0xD42D)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULKIYEOK
    ((0xBD, 0x97), (0xD42E)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULMIEUM
    ((0xBD, 0x98), (0xD42F)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULPIEUP
    ((0xBD, 0x99), (0xD430)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULSIOS
    ((0xBD, 0x9A), (0xD431)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULTHIEUTH
    ((0xBD, 0x9B), (0xD432)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULPHIEUPH
    ((0xBD, 0x9C), (0xD433)),  # HANGUL SYLLABLE PHIEUPH WAE RIEULHIEUH
    ((0xBD, 0x9D), (0xD434)),  # HANGUL SYLLABLE PHIEUPH WAE MIEUM
    ((0xBD, 0x9E), (0xD435)),  # HANGUL SYLLABLE PHIEUPH WAE PIEUP
    ((0xBD, 0x9F), (0xD436)),  # HANGUL SYLLABLE PHIEUPH WAE PIEUPSIOS
    ((0xBD, 0xA0), (0xD437)),  # HANGUL SYLLABLE PHIEUPH WAE SIOS
    ((0xBD, 0xA1), (0xC22F)),  # HANGUL SYLLABLE SIOS U CHIEUCH
    ((0xBD, 0xA2), (0xC231)),  # HANGUL SYLLABLE SIOS U THIEUTH
    ((0xBD, 0xA3), (0xC232)),  # HANGUL SYLLABLE SIOS U PHIEUPH
    ((0xBD, 0xA4), (0xC234)),  # HANGUL SYLLABLE SIOS WEO
    ((0xBD, 0xA5), (0xC248)),  # HANGUL SYLLABLE SIOS WEO SSANGSIOS
    ((0xBD, 0xA6), (0xC250)),  # HANGUL SYLLABLE SIOS WE
    ((0xBD, 0xA7), (0xC251)),  # HANGUL SYLLABLE SIOS WE KIYEOK
    ((0xBD, 0xA8), (0xC254)),  # HANGUL SYLLABLE SIOS WE NIEUN
    ((0xBD, 0xA9), (0xC258)),  # HANGUL SYLLABLE SIOS WE RIEUL
    ((0xBD, 0xAA), (0xC260)),  # HANGUL SYLLABLE SIOS WE MIEUM
    ((0xBD, 0xAB), (0xC265)),  # HANGUL SYLLABLE SIOS WE IEUNG
    ((0xBD, 0xAC), (0xC26C)),  # HANGUL SYLLABLE SIOS WI
    ((0xBD, 0xAD), (0xC26D)),  # HANGUL SYLLABLE SIOS WI KIYEOK
    ((0xBD, 0xAE), (0xC270)),  # HANGUL SYLLABLE SIOS WI NIEUN
    ((0xBD, 0xAF), (0xC274)),  # HANGUL SYLLABLE SIOS WI RIEUL
    ((0xBD, 0xB0), (0xC27C)),  # HANGUL SYLLABLE SIOS WI MIEUM
    ((0xBD, 0xB1), (0xC27D)),  # HANGUL SYLLABLE SIOS WI PIEUP
    ((0xBD, 0xB2), (0xC27F)),  # HANGUL SYLLABLE SIOS WI SIOS
    ((0xBD, 0xB3), (0xC281)),  # HANGUL SYLLABLE SIOS WI IEUNG
    ((0xBD, 0xB4), (0xC288)),  # HANGUL SYLLABLE SIOS YU
    ((0xBD, 0xB5), (0xC289)),  # HANGUL SYLLABLE SIOS YU KIYEOK
    ((0xBD, 0xB6), (0xC290)),  # HANGUL SYLLABLE SIOS YU RIEUL
    ((0xBD, 0xB7), (0xC298)),  # HANGUL SYLLABLE SIOS YU MIEUM
    ((0xBD, 0xB8), (0xC29B)),  # HANGUL SYLLABLE SIOS YU SIOS
    ((0xBD, 0xB9), (0xC29D)),  # HANGUL SYLLABLE SIOS YU IEUNG
    ((0xBD, 0xBA), (0xC2A4)),  # HANGUL SYLLABLE SIOS EU
    ((0xBD, 0xBB), (0xC2A5)),  # HANGUL SYLLABLE SIOS EU KIYEOK
    ((0xBD, 0xBC), (0xC2A8)),  # HANGUL SYLLABLE SIOS EU NIEUN
    ((0xBD, 0xBD), (0xC2AC)),  # HANGUL SYLLABLE SIOS EU RIEUL
    ((0xBD, 0xBE), (0xC2AD)),  # HANGUL SYLLABLE SIOS EU RIEULKIYEOK
    ((0xBD, 0xBF), (0xC2B4)),  # HANGUL SYLLABLE SIOS EU MIEUM
    ((0xBD, 0xC0), (0xC2B5)),  # HANGUL SYLLABLE SIOS EU PIEUP
    ((0xBD, 0xC1), (0xC2B7)),  # HANGUL SYLLABLE SIOS EU SIOS
    ((0xBD, 0xC2), (0xC2B9)),  # HANGUL SYLLABLE SIOS EU IEUNG
    ((0xBD, 0xC3), (0xC2DC)),  # HANGUL SYLLABLE SIOS I
    ((0xBD, 0xC4), (0xC2DD)),  # HANGUL SYLLABLE SIOS I KIYEOK
    ((0xBD, 0xC5), (0xC2E0)),  # HANGUL SYLLABLE SIOS I NIEUN
    ((0xBD, 0xC6), (0xC2E3)),  # HANGUL SYLLABLE SIOS I TIKEUT
    ((0xBD, 0xC7), (0xC2E4)),  # HANGUL SYLLABLE SIOS I RIEUL
    ((0xBD, 0xC8), (0xC2EB)),  # HANGUL SYLLABLE SIOS I RIEULHIEUH
    ((0xBD, 0xC9), (0xC2EC)),  # HANGUL SYLLABLE SIOS I MIEUM
    ((0xBD, 0xCA), (0xC2ED)),  # HANGUL SYLLABLE SIOS I PIEUP
    ((0xBD, 0xCB), (0xC2EF)),  # HANGUL SYLLABLE SIOS I SIOS
    ((0xBD, 0xCC), (0xC2F1)),  # HANGUL SYLLABLE SIOS I IEUNG
    ((0xBD, 0xCD), (0xC2F6)),  # HANGUL SYLLABLE SIOS I PHIEUPH
    ((0xBD, 0xCE), (0xC2F8)),  # HANGUL SYLLABLE SSANGSIOS A
    ((0xBD, 0xCF), (0xC2F9)),  # HANGUL SYLLABLE SSANGSIOS A KIYEOK
    ((0xBD, 0xD0), (0xC2FB)),  # HANGUL SYLLABLE SSANGSIOS A KIYEOKSIOS
    ((0xBD, 0xD1), (0xC2FC)),  # HANGUL SYLLABLE SSANGSIOS A NIEUN
    ((0xBD, 0xD2), (0xC300)),  # HANGUL SYLLABLE SSANGSIOS A RIEUL
    ((0xBD, 0xD3), (0xC308)),  # HANGUL SYLLABLE SSANGSIOS A MIEUM
    ((0xBD, 0xD4), (0xC309)),  # HANGUL SYLLABLE SSANGSIOS A PIEUP
    ((0xBD, 0xD5), (0xC30C)),  # HANGUL SYLLABLE SSANGSIOS A SSANGSIOS
    ((0xBD, 0xD6), (0xC30D)),  # HANGUL SYLLABLE SSANGSIOS A IEUNG
    ((0xBD, 0xD7), (0xC313)),  # HANGUL SYLLABLE SSANGSIOS A HIEUH
    ((0xBD, 0xD8), (0xC314)),  # HANGUL SYLLABLE SSANGSIOS AE
    ((0xBD, 0xD9), (0xC315)),  # HANGUL SYLLABLE SSANGSIOS AE KIYEOK
    ((0xBD, 0xDA), (0xC318)),  # HANGUL SYLLABLE SSANGSIOS AE NIEUN
    ((0xBD, 0xDB), (0xC31C)),  # HANGUL SYLLABLE SSANGSIOS AE RIEUL
    ((0xBD, 0xDC), (0xC324)),  # HANGUL SYLLABLE SSANGSIOS AE MIEUM
    ((0xBD, 0xDD), (0xC325)),  # HANGUL SYLLABLE SSANGSIOS AE PIEUP
    ((0xBD, 0xDE), (0xC328)),  # HANGUL SYLLABLE SSANGSIOS AE SSANGSIOS
    ((0xBD, 0xDF), (0xC329)),  # HANGUL SYLLABLE SSANGSIOS AE IEUNG
    ((0xBD, 0xE0), (0xC345)),  # HANGUL SYLLABLE SSANGSIOS YA IEUNG
    ((0xBD, 0xE1), (0xC368)),  # HANGUL SYLLABLE SSANGSIOS EO
    ((0xBD, 0xE2), (0xC369)),  # HANGUL SYLLABLE SSANGSIOS EO KIYEOK
    ((0xBD, 0xE3), (0xC36C)),  # HANGUL SYLLABLE SSANGSIOS EO NIEUN
    ((0xBD, 0xE4), (0xC370)),  # HANGUL SYLLABLE SSANGSIOS EO RIEUL
    ((0xBD, 0xE5), (0xC372)),  # HANGUL SYLLABLE SSANGSIOS EO RIEULMIEUM
    ((0xBD, 0xE6), (0xC378)),  # HANGUL SYLLABLE SSANGSIOS EO MIEUM
    ((0xBD, 0xE7), (0xC379)),  # HANGUL SYLLABLE SSANGSIOS EO PIEUP
    ((0xBD, 0xE8), (0xC37C)),  # HANGUL SYLLABLE SSANGSIOS EO SSANGSIOS
    ((0xBD, 0xE9), (0xC37D)),  # HANGUL SYLLABLE SSANGSIOS EO IEUNG
    ((0xBD, 0xEA), (0xC384)),  # HANGUL SYLLABLE SSANGSIOS E
    ((0xBD, 0xEB), (0xC388)),  # HANGUL SYLLABLE SSANGSIOS E NIEUN
    ((0xBD, 0xEC), (0xC38C)),  # HANGUL SYLLABLE SSANGSIOS E RIEUL
    ((0xBD, 0xED), (0xC3C0)),  # HANGUL SYLLABLE SSANGSIOS YE NIEUN
    ((0xBD, 0xEE), (0xC3D8)),  # HANGUL SYLLABLE SSANGSIOS O
    ((0xBD, 0xEF), (0xC3D9)),  # HANGUL SYLLABLE SSANGSIOS O KIYEOK
    ((0xBD, 0xF0), (0xC3DC)),  # HANGUL SYLLABLE SSANGSIOS O NIEUN
    ((0xBD, 0xF1), (0xC3DF)),  # HANGUL SYLLABLE SSANGSIOS O TIKEUT
    ((0xBD, 0xF2), (0xC3E0)),  # HANGUL SYLLABLE SSANGSIOS O RIEUL
    ((0xBD, 0xF3), (0xC3E2)),  # HANGUL SYLLABLE SSANGSIOS O RIEULMIEUM
    ((0xBD, 0xF4), (0xC3E8)),  # HANGUL SYLLABLE SSANGSIOS O MIEUM
    ((0xBD, 0xF5), (0xC3E9)),  # HANGUL SYLLABLE SSANGSIOS O PIEUP
    ((0xBD, 0xF6), (0xC3ED)),  # HANGUL SYLLABLE SSANGSIOS O IEUNG
    ((0xBD, 0xF7), (0xC3F4)),  # HANGUL SYLLABLE SSANGSIOS WA
    ((0xBD, 0xF8), (0xC3F5)),  # HANGUL SYLLABLE SSANGSIOS WA KIYEOK
    ((0xBD, 0xF9), (0xC3F8)),  # HANGUL SYLLABLE SSANGSIOS WA NIEUN
    ((0xBD, 0xFA), (0xC408)),  # HANGUL SYLLABLE SSANGSIOS WA SSANGSIOS
    ((0xBD, 0xFB), (0xC410)),  # HANGUL SYLLABLE SSANGSIOS WAE
    ((0xBD, 0xFC), (0xC424)),  # HANGUL SYLLABLE SSANGSIOS WAE SSANGSIOS
    ((0xBD, 0xFD), (0xC42C)),  # HANGUL SYLLABLE SSANGSIOS OE
    ((0xBD, 0xFE), (0xC430)),  # HANGUL SYLLABLE SSANGSIOS OE NIEUN
    ((0xBE, 0x41), (0xD438)),  # HANGUL SYLLABLE PHIEUPH WAE SSANGSIOS
    ((0xBE, 0x42), (0xD439)),  # HANGUL SYLLABLE PHIEUPH WAE IEUNG
    ((0xBE, 0x43), (0xD43A)),  # HANGUL SYLLABLE PHIEUPH WAE CIEUC
    ((0xBE, 0x44), (0xD43B)),  # HANGUL SYLLABLE PHIEUPH WAE CHIEUCH
    ((0xBE, 0x45), (0xD43C)),  # HANGUL SYLLABLE PHIEUPH WAE KHIEUKH
    ((0xBE, 0x46), (0xD43D)),  # HANGUL SYLLABLE PHIEUPH WAE THIEUTH
    ((0xBE, 0x47), (0xD43E)),  # HANGUL SYLLABLE PHIEUPH WAE PHIEUPH
    ((0xBE, 0x48), (0xD43F)),  # HANGUL SYLLABLE PHIEUPH WAE HIEUH
    ((0xBE, 0x49), (0xD441)),  # HANGUL SYLLABLE PHIEUPH OE KIYEOK
    ((0xBE, 0x4A), (0xD442)),  # HANGUL SYLLABLE PHIEUPH OE SSANGKIYEOK
    ((0xBE, 0x4B), (0xD443)),  # HANGUL SYLLABLE PHIEUPH OE KIYEOKSIOS
    ((0xBE, 0x4C), (0xD445)),  # HANGUL SYLLABLE PHIEUPH OE NIEUNCIEUC
    ((0xBE, 0x4D), (0xD446)),  # HANGUL SYLLABLE PHIEUPH OE NIEUNHIEUH
    ((0xBE, 0x4E), (0xD447)),  # HANGUL SYLLABLE PHIEUPH OE TIKEUT
    ((0xBE, 0x4F), (0xD448)),  # HANGUL SYLLABLE PHIEUPH OE RIEUL
    ((0xBE, 0x50), (0xD449)),  # HANGUL SYLLABLE PHIEUPH OE RIEULKIYEOK
    ((0xBE, 0x51), (0xD44A)),  # HANGUL SYLLABLE PHIEUPH OE RIEULMIEUM
    ((0xBE, 0x52), (0xD44B)),  # HANGUL SYLLABLE PHIEUPH OE RIEULPIEUP
    ((0xBE, 0x53), (0xD44C)),  # HANGUL SYLLABLE PHIEUPH OE RIEULSIOS
    ((0xBE, 0x54), (0xD44D)),  # HANGUL SYLLABLE PHIEUPH OE RIEULTHIEUTH
    ((0xBE, 0x55), (0xD44E)),  # HANGUL SYLLABLE PHIEUPH OE RIEULPHIEUPH
    ((0xBE, 0x56), (0xD44F)),  # HANGUL SYLLABLE PHIEUPH OE RIEULHIEUH
    ((0xBE, 0x57), (0xD450)),  # HANGUL SYLLABLE PHIEUPH OE MIEUM
    ((0xBE, 0x58), (0xD451)),  # HANGUL SYLLABLE PHIEUPH OE PIEUP
    ((0xBE, 0x59), (0xD452)),  # HANGUL SYLLABLE PHIEUPH OE PIEUPSIOS
    ((0xBE, 0x5A), (0xD453)),  # HANGUL SYLLABLE PHIEUPH OE SIOS
    ((0xBE, 0x61), (0xD454)),  # HANGUL SYLLABLE PHIEUPH OE SSANGSIOS
    ((0xBE, 0x62), (0xD455)),  # HANGUL SYLLABLE PHIEUPH OE IEUNG
    ((0xBE, 0x63), (0xD456)),  # HANGUL SYLLABLE PHIEUPH OE CIEUC
    ((0xBE, 0x64), (0xD457)),  # HANGUL SYLLABLE PHIEUPH OE CHIEUCH
    ((0xBE, 0x65), (0xD458)),  # HANGUL SYLLABLE PHIEUPH OE KHIEUKH
    ((0xBE, 0x66), (0xD459)),  # HANGUL SYLLABLE PHIEUPH OE THIEUTH
    ((0xBE, 0x67), (0xD45A)),  # HANGUL SYLLABLE PHIEUPH OE PHIEUPH
    ((0xBE, 0x68), (0xD45B)),  # HANGUL SYLLABLE PHIEUPH OE HIEUH
    ((0xBE, 0x69), (0xD45D)),  # HANGUL SYLLABLE PHIEUPH YO KIYEOK
    ((0xBE, 0x6A), (0xD45E)),  # HANGUL SYLLABLE PHIEUPH YO SSANGKIYEOK
    ((0xBE, 0x6B), (0xD45F)),  # HANGUL SYLLABLE PHIEUPH YO KIYEOKSIOS
    ((0xBE, 0x6C), (0xD461)),  # HANGUL SYLLABLE PHIEUPH YO NIEUNCIEUC
    ((0xBE, 0x6D), (0xD462)),  # HANGUL SYLLABLE PHIEUPH YO NIEUNHIEUH
    ((0xBE, 0x6E), (0xD463)),  # HANGUL SYLLABLE PHIEUPH YO TIKEUT
    ((0xBE, 0x6F), (0xD465)),  # HANGUL SYLLABLE PHIEUPH YO RIEULKIYEOK
    ((0xBE, 0x70), (0xD466)),  # HANGUL SYLLABLE PHIEUPH YO RIEULMIEUM
    ((0xBE, 0x71), (0xD467)),  # HANGUL SYLLABLE PHIEUPH YO RIEULPIEUP
    ((0xBE, 0x72), (0xD468)),  # HANGUL SYLLABLE PHIEUPH YO RIEULSIOS
    ((0xBE, 0x73), (0xD469)),  # HANGUL SYLLABLE PHIEUPH YO RIEULTHIEUTH
    ((0xBE, 0x74), (0xD46A)),  # HANGUL SYLLABLE PHIEUPH YO RIEULPHIEUPH
    ((0xBE, 0x75), (0xD46B)),  # HANGUL SYLLABLE PHIEUPH YO RIEULHIEUH
    ((0xBE, 0x76), (0xD46C)),  # HANGUL SYLLABLE PHIEUPH YO MIEUM
    ((0xBE, 0x77), (0xD46E)),  # HANGUL SYLLABLE PHIEUPH YO PIEUPSIOS
    ((0xBE, 0x78), (0xD470)),  # HANGUL SYLLABLE PHIEUPH YO SSANGSIOS
    ((0xBE, 0x79), (0xD471)),  # HANGUL SYLLABLE PHIEUPH YO IEUNG
    ((0xBE, 0x7A), (0xD472)),  # HANGUL SYLLABLE PHIEUPH YO CIEUC
    ((0xBE, 0x81), (0xD473)),  # HANGUL SYLLABLE PHIEUPH YO CHIEUCH
    ((0xBE, 0x82), (0xD474)),  # HANGUL SYLLABLE PHIEUPH YO KHIEUKH
    ((0xBE, 0x83), (0xD475)),  # HANGUL SYLLABLE PHIEUPH YO THIEUTH
    ((0xBE, 0x84), (0xD476)),  # HANGUL SYLLABLE PHIEUPH YO PHIEUPH
    ((0xBE, 0x85), (0xD477)),  # HANGUL SYLLABLE PHIEUPH YO HIEUH
    ((0xBE, 0x86), (0xD47A)),  # HANGUL SYLLABLE PHIEUPH U SSANGKIYEOK
    ((0xBE, 0x87), (0xD47B)),  # HANGUL SYLLABLE PHIEUPH U KIYEOKSIOS
    ((0xBE, 0x88), (0xD47D)),  # HANGUL SYLLABLE PHIEUPH U NIEUNCIEUC
    ((0xBE, 0x89), (0xD47E)),  # HANGUL SYLLABLE PHIEUPH U NIEUNHIEUH
    ((0xBE, 0x8A), (0xD481)),  # HANGUL SYLLABLE PHIEUPH U RIEULKIYEOK
    ((0xBE, 0x8B), (0xD483)),  # HANGUL SYLLABLE PHIEUPH U RIEULPIEUP
    ((0xBE, 0x8C), (0xD484)),  # HANGUL SYLLABLE PHIEUPH U RIEULSIOS
    ((0xBE, 0x8D), (0xD485)),  # HANGUL SYLLABLE PHIEUPH U RIEULTHIEUTH
    ((0xBE, 0x8E), (0xD486)),  # HANGUL SYLLABLE PHIEUPH U RIEULPHIEUPH
    ((0xBE, 0x8F), (0xD487)),  # HANGUL SYLLABLE PHIEUPH U RIEULHIEUH
    ((0xBE, 0x90), (0xD48A)),  # HANGUL SYLLABLE PHIEUPH U PIEUPSIOS
    ((0xBE, 0x91), (0xD48C)),  # HANGUL SYLLABLE PHIEUPH U SSANGSIOS
    ((0xBE, 0x92), (0xD48E)),  # HANGUL SYLLABLE PHIEUPH U CIEUC
    ((0xBE, 0x93), (0xD48F)),  # HANGUL SYLLABLE PHIEUPH U CHIEUCH
    ((0xBE, 0x94), (0xD490)),  # HANGUL SYLLABLE PHIEUPH U KHIEUKH
    ((0xBE, 0x95), (0xD491)),  # HANGUL SYLLABLE PHIEUPH U THIEUTH
    ((0xBE, 0x96), (0xD492)),  # HANGUL SYLLABLE PHIEUPH U PHIEUPH
    ((0xBE, 0x97), (0xD493)),  # HANGUL SYLLABLE PHIEUPH U HIEUH
    ((0xBE, 0x98), (0xD495)),  # HANGUL SYLLABLE PHIEUPH WEO KIYEOK
    ((0xBE, 0x99), (0xD496)),  # HANGUL SYLLABLE PHIEUPH WEO SSANGKIYEOK
    ((0xBE, 0x9A), (0xD497)),  # HANGUL SYLLABLE PHIEUPH WEO KIYEOKSIOS
    ((0xBE, 0x9B), (0xD498)),  # HANGUL SYLLABLE PHIEUPH WEO NIEUN
    ((0xBE, 0x9C), (0xD499)),  # HANGUL SYLLABLE PHIEUPH WEO NIEUNCIEUC
    ((0xBE, 0x9D), (0xD49A)),  # HANGUL SYLLABLE PHIEUPH WEO NIEUNHIEUH
    ((0xBE, 0x9E), (0xD49B)),  # HANGUL SYLLABLE PHIEUPH WEO TIKEUT
    ((0xBE, 0x9F), (0xD49C)),  # HANGUL SYLLABLE PHIEUPH WEO RIEUL
    ((0xBE, 0xA0), (0xD49D)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULKIYEOK
    ((0xBE, 0xA1), (0xC434)),  # HANGUL SYLLABLE SSANGSIOS OE RIEUL
    ((0xBE, 0xA2), (0xC43C)),  # HANGUL SYLLABLE SSANGSIOS OE MIEUM
    ((0xBE, 0xA3), (0xC43D)),  # HANGUL SYLLABLE SSANGSIOS OE PIEUP
    ((0xBE, 0xA4), (0xC448)),  # HANGUL SYLLABLE SSANGSIOS YO
    ((0xBE, 0xA5), (0xC464)),  # HANGUL SYLLABLE SSANGSIOS U
    ((0xBE, 0xA6), (0xC465)),  # HANGUL SYLLABLE SSANGSIOS U KIYEOK
    ((0xBE, 0xA7), (0xC468)),  # HANGUL SYLLABLE SSANGSIOS U NIEUN
    ((0xBE, 0xA8), (0xC46C)),  # HANGUL SYLLABLE SSANGSIOS U RIEUL
    ((0xBE, 0xA9), (0xC474)),  # HANGUL SYLLABLE SSANGSIOS U MIEUM
    ((0xBE, 0xAA), (0xC475)),  # HANGUL SYLLABLE SSANGSIOS U PIEUP
    ((0xBE, 0xAB), (0xC479)),  # HANGUL SYLLABLE SSANGSIOS U IEUNG
    ((0xBE, 0xAC), (0xC480)),  # HANGUL SYLLABLE SSANGSIOS WEO
    ((0xBE, 0xAD), (0xC494)),  # HANGUL SYLLABLE SSANGSIOS WEO SSANGSIOS
    ((0xBE, 0xAE), (0xC49C)),  # HANGUL SYLLABLE SSANGSIOS WE
    ((0xBE, 0xAF), (0xC4B8)),  # HANGUL SYLLABLE SSANGSIOS WI
    ((0xBE, 0xB0), (0xC4BC)),  # HANGUL SYLLABLE SSANGSIOS WI NIEUN
    ((0xBE, 0xB1), (0xC4E9)),  # HANGUL SYLLABLE SSANGSIOS YU IEUNG
    ((0xBE, 0xB2), (0xC4F0)),  # HANGUL SYLLABLE SSANGSIOS EU
    ((0xBE, 0xB3), (0xC4F1)),  # HANGUL SYLLABLE SSANGSIOS EU KIYEOK
    ((0xBE, 0xB4), (0xC4F4)),  # HANGUL SYLLABLE SSANGSIOS EU NIEUN
    ((0xBE, 0xB5), (0xC4F8)),  # HANGUL SYLLABLE SSANGSIOS EU RIEUL
    ((0xBE, 0xB6), (0xC4FA)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULMIEUM
    ((0xBE, 0xB7), (0xC4FF)),  # HANGUL SYLLABLE SSANGSIOS EU RIEULHIEUH
    ((0xBE, 0xB8), (0xC500)),  # HANGUL SYLLABLE SSANGSIOS EU MIEUM
    ((0xBE, 0xB9), (0xC501)),  # HANGUL SYLLABLE SSANGSIOS EU PIEUP
    ((0xBE, 0xBA), (0xC50C)),  # HANGUL SYLLABLE SSANGSIOS YI
    ((0xBE, 0xBB), (0xC510)),  # HANGUL SYLLABLE SSANGSIOS YI NIEUN
    ((0xBE, 0xBC), (0xC514)),  # HANGUL SYLLABLE SSANGSIOS YI RIEUL
    ((0xBE, 0xBD), (0xC51C)),  # HANGUL SYLLABLE SSANGSIOS YI MIEUM
    ((0xBE, 0xBE), (0xC528)),  # HANGUL SYLLABLE SSANGSIOS I
    ((0xBE, 0xBF), (0xC529)),  # HANGUL SYLLABLE SSANGSIOS I KIYEOK
    ((0xBE, 0xC0), (0xC52C)),  # HANGUL SYLLABLE SSANGSIOS I NIEUN
    ((0xBE, 0xC1), (0xC530)),  # HANGUL SYLLABLE SSANGSIOS I RIEUL
    ((0xBE, 0xC2), (0xC538)),  # HANGUL SYLLABLE SSANGSIOS I MIEUM
    ((0xBE, 0xC3), (0xC539)),  # HANGUL SYLLABLE SSANGSIOS I PIEUP
    ((0xBE, 0xC4), (0xC53B)),  # HANGUL SYLLABLE SSANGSIOS I SIOS
    ((0xBE, 0xC5), (0xC53D)),  # HANGUL SYLLABLE SSANGSIOS I IEUNG
    ((0xBE, 0xC6), (0xC544)),  # HANGUL SYLLABLE IEUNG A
    ((0xBE, 0xC7), (0xC545)),  # HANGUL SYLLABLE IEUNG A KIYEOK
    ((0xBE, 0xC8), (0xC548)),  # HANGUL SYLLABLE IEUNG A NIEUN
    ((0xBE, 0xC9), (0xC549)),  # HANGUL SYLLABLE IEUNG A NIEUNCIEUC
    ((0xBE, 0xCA), (0xC54A)),  # HANGUL SYLLABLE IEUNG A NIEUNHIEUH
    ((0xBE, 0xCB), (0xC54C)),  # HANGUL SYLLABLE IEUNG A RIEUL
    ((0xBE, 0xCC), (0xC54D)),  # HANGUL SYLLABLE IEUNG A RIEULKIYEOK
    ((0xBE, 0xCD), (0xC54E)),  # HANGUL SYLLABLE IEUNG A RIEULMIEUM
    ((0xBE, 0xCE), (0xC553)),  # HANGUL SYLLABLE IEUNG A RIEULHIEUH
    ((0xBE, 0xCF), (0xC554)),  # HANGUL SYLLABLE IEUNG A MIEUM
    ((0xBE, 0xD0), (0xC555)),  # HANGUL SYLLABLE IEUNG A PIEUP
    ((0xBE, 0xD1), (0xC557)),  # HANGUL SYLLABLE IEUNG A SIOS
    ((0xBE, 0xD2), (0xC558)),  # HANGUL SYLLABLE IEUNG A SSANGSIOS
    ((0xBE, 0xD3), (0xC559)),  # HANGUL SYLLABLE IEUNG A IEUNG
    ((0xBE, 0xD4), (0xC55D)),  # HANGUL SYLLABLE IEUNG A THIEUTH
    ((0xBE, 0xD5), (0xC55E)),  # HANGUL SYLLABLE IEUNG A PHIEUPH
    ((0xBE, 0xD6), (0xC560)),  # HANGUL SYLLABLE IEUNG AE
    ((0xBE, 0xD7), (0xC561)),  # HANGUL SYLLABLE IEUNG AE KIYEOK
    ((0xBE, 0xD8), (0xC564)),  # HANGUL SYLLABLE IEUNG AE NIEUN
    ((0xBE, 0xD9), (0xC568)),  # HANGUL SYLLABLE IEUNG AE RIEUL
    ((0xBE, 0xDA), (0xC570)),  # HANGUL SYLLABLE IEUNG AE MIEUM
    ((0xBE, 0xDB), (0xC571)),  # HANGUL SYLLABLE IEUNG AE PIEUP
    ((0xBE, 0xDC), (0xC573)),  # HANGUL SYLLABLE IEUNG AE SIOS
    ((0xBE, 0xDD), (0xC574)),  # HANGUL SYLLABLE IEUNG AE SSANGSIOS
    ((0xBE, 0xDE), (0xC575)),  # HANGUL SYLLABLE IEUNG AE IEUNG
    ((0xBE, 0xDF), (0xC57C)),  # HANGUL SYLLABLE IEUNG YA
    ((0xBE, 0xE0), (0xC57D)),  # HANGUL SYLLABLE IEUNG YA KIYEOK
    ((0xBE, 0xE1), (0xC580)),  # HANGUL SYLLABLE IEUNG YA NIEUN
    ((0xBE, 0xE2), (0xC584)),  # HANGUL SYLLABLE IEUNG YA RIEUL
    ((0xBE, 0xE3), (0xC587)),  # HANGUL SYLLABLE IEUNG YA RIEULPIEUP
    ((0xBE, 0xE4), (0xC58C)),  # HANGUL SYLLABLE IEUNG YA MIEUM
    ((0xBE, 0xE5), (0xC58D)),  # HANGUL SYLLABLE IEUNG YA PIEUP
    ((0xBE, 0xE6), (0xC58F)),  # HANGUL SYLLABLE IEUNG YA SIOS
    ((0xBE, 0xE7), (0xC591)),  # HANGUL SYLLABLE IEUNG YA IEUNG
    ((0xBE, 0xE8), (0xC595)),  # HANGUL SYLLABLE IEUNG YA THIEUTH
    ((0xBE, 0xE9), (0xC597)),  # HANGUL SYLLABLE IEUNG YA HIEUH
    ((0xBE, 0xEA), (0xC598)),  # HANGUL SYLLABLE IEUNG YAE
    ((0xBE, 0xEB), (0xC59C)),  # HANGUL SYLLABLE IEUNG YAE NIEUN
    ((0xBE, 0xEC), (0xC5A0)),  # HANGUL SYLLABLE IEUNG YAE RIEUL
    ((0xBE, 0xED), (0xC5A9)),  # HANGUL SYLLABLE IEUNG YAE PIEUP
    ((0xBE, 0xEE), (0xC5B4)),  # HANGUL SYLLABLE IEUNG EO
    ((0xBE, 0xEF), (0xC5B5)),  # HANGUL SYLLABLE IEUNG EO KIYEOK
    ((0xBE, 0xF0), (0xC5B8)),  # HANGUL SYLLABLE IEUNG EO NIEUN
    ((0xBE, 0xF1), (0xC5B9)),  # HANGUL SYLLABLE IEUNG EO NIEUNCIEUC
    ((0xBE, 0xF2), (0xC5BB)),  # HANGUL SYLLABLE IEUNG EO TIKEUT
    ((0xBE, 0xF3), (0xC5BC)),  # HANGUL SYLLABLE IEUNG EO RIEUL
    ((0xBE, 0xF4), (0xC5BD)),  # HANGUL SYLLABLE IEUNG EO RIEULKIYEOK
    ((0xBE, 0xF5), (0xC5BE)),  # HANGUL SYLLABLE IEUNG EO RIEULMIEUM
    ((0xBE, 0xF6), (0xC5C4)),  # HANGUL SYLLABLE IEUNG EO MIEUM
    ((0xBE, 0xF7), (0xC5C5)),  # HANGUL SYLLABLE IEUNG EO PIEUP
    ((0xBE, 0xF8), (0xC5C6)),  # HANGUL SYLLABLE IEUNG EO PIEUPSIOS
    ((0xBE, 0xF9), (0xC5C7)),  # HANGUL SYLLABLE IEUNG EO SIOS
    ((0xBE, 0xFA), (0xC5C8)),  # HANGUL SYLLABLE IEUNG EO SSANGSIOS
    ((0xBE, 0xFB), (0xC5C9)),  # HANGUL SYLLABLE IEUNG EO IEUNG
    ((0xBE, 0xFC), (0xC5CA)),  # HANGUL SYLLABLE IEUNG EO CIEUC
    ((0xBE, 0xFD), (0xC5CC)),  # HANGUL SYLLABLE IEUNG EO KHIEUKH
    ((0xBE, 0xFE), (0xC5CE)),  # HANGUL SYLLABLE IEUNG EO PHIEUPH
    ((0xBF, 0x41), (0xD49E)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULMIEUM
    ((0xBF, 0x42), (0xD49F)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULPIEUP
    ((0xBF, 0x43), (0xD4A0)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULSIOS
    ((0xBF, 0x44), (0xD4A1)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULTHIEUTH
    ((0xBF, 0x45), (0xD4A2)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULPHIEUPH
    ((0xBF, 0x46), (0xD4A3)),  # HANGUL SYLLABLE PHIEUPH WEO RIEULHIEUH
    ((0xBF, 0x47), (0xD4A4)),  # HANGUL SYLLABLE PHIEUPH WEO MIEUM
    ((0xBF, 0x48), (0xD4A5)),  # HANGUL SYLLABLE PHIEUPH WEO PIEUP
    ((0xBF, 0x49), (0xD4A6)),  # HANGUL SYLLABLE PHIEUPH WEO PIEUPSIOS
    ((0xBF, 0x4A), (0xD4A7)),  # HANGUL SYLLABLE PHIEUPH WEO SIOS
    ((0xBF, 0x4B), (0xD4A8)),  # HANGUL SYLLABLE PHIEUPH WEO SSANGSIOS
    ((0xBF, 0x4C), (0xD4AA)),  # HANGUL SYLLABLE PHIEUPH WEO CIEUC
    ((0xBF, 0x4D), (0xD4AB)),  # HANGUL SYLLABLE PHIEUPH WEO CHIEUCH
    ((0xBF, 0x4E), (0xD4AC)),  # HANGUL SYLLABLE PHIEUPH WEO KHIEUKH
    ((0xBF, 0x4F), (0xD4AD)),  # HANGUL SYLLABLE PHIEUPH WEO THIEUTH
    ((0xBF, 0x50), (0xD4AE)),  # HANGUL SYLLABLE PHIEUPH WEO PHIEUPH
    ((0xBF, 0x51), (0xD4AF)),  # HANGUL SYLLABLE PHIEUPH WEO HIEUH
    ((0xBF, 0x52), (0xD4B0)),  # HANGUL SYLLABLE PHIEUPH WE
    ((0xBF, 0x53), (0xD4B1)),  # HANGUL SYLLABLE PHIEUPH WE KIYEOK
    ((0xBF, 0x54), (0xD4B2)),  # HANGUL SYLLABLE PHIEUPH WE SSANGKIYEOK
    ((0xBF, 0x55), (0xD4B3)),  # HANGUL SYLLABLE PHIEUPH WE KIYEOKSIOS
    ((0xBF, 0x56), (0xD4B4)),  # HANGUL SYLLABLE PHIEUPH WE NIEUN
    ((0xBF, 0x57), (0xD4B5)),  # HANGUL SYLLABLE PHIEUPH WE NIEUNCIEUC
    ((0xBF, 0x58), (0xD4B6)),  # HANGUL SYLLABLE PHIEUPH WE NIEUNHIEUH
    ((0xBF, 0x59), (0xD4B7)),  # HANGUL SYLLABLE PHIEUPH WE TIKEUT
    ((0xBF, 0x5A), (0xD4B8)),  # HANGUL SYLLABLE PHIEUPH WE RIEUL
    ((0xBF, 0x61), (0xD4B9)),  # HANGUL SYLLABLE PHIEUPH WE RIEULKIYEOK
    ((0xBF, 0x62), (0xD4BA)),  # HANGUL SYLLABLE PHIEUPH WE RIEULMIEUM
    ((0xBF, 0x63), (0xD4BB)),  # HANGUL SYLLABLE PHIEUPH WE RIEULPIEUP
    ((0xBF, 0x64), (0xD4BC)),  # HANGUL SYLLABLE PHIEUPH WE RIEULSIOS
    ((0xBF, 0x65), (0xD4BD)),  # HANGUL SYLLABLE PHIEUPH WE RIEULTHIEUTH
    ((0xBF, 0x66), (0xD4BE)),  # HANGUL SYLLABLE PHIEUPH WE RIEULPHIEUPH
    ((0xBF, 0x67), (0xD4BF)),  # HANGUL SYLLABLE PHIEUPH WE RIEULHIEUH
    ((0xBF, 0x68), (0xD4C0)),  # HANGUL SYLLABLE PHIEUPH WE MIEUM
    ((0xBF, 0x69), (0xD4C1)),  # HANGUL SYLLABLE PHIEUPH WE PIEUP
    ((0xBF, 0x6A), (0xD4C2)),  # HANGUL SYLLABLE PHIEUPH WE PIEUPSIOS
    ((0xBF, 0x6B), (0xD4C3)),  # HANGUL SYLLABLE PHIEUPH WE SIOS
    ((0xBF, 0x6C), (0xD4C4)),  # HANGUL SYLLABLE PHIEUPH WE SSANGSIOS
    ((0xBF, 0x6D), (0xD4C5)),  # HANGUL SYLLABLE PHIEUPH WE IEUNG
    ((0xBF, 0x6E), (0xD4C6)),  # HANGUL SYLLABLE PHIEUPH WE CIEUC
    ((0xBF, 0x6F), (0xD4C7)),  # HANGUL SYLLABLE PHIEUPH WE CHIEUCH
    ((0xBF, 0x70), (0xD4C8)),  # HANGUL SYLLABLE PHIEUPH WE KHIEUKH
    ((0xBF, 0x71), (0xD4C9)),  # HANGUL SYLLABLE PHIEUPH WE THIEUTH
    ((0xBF, 0x72), (0xD4CA)),  # HANGUL SYLLABLE PHIEUPH WE PHIEUPH
    ((0xBF, 0x73), (0xD4CB)),  # HANGUL SYLLABLE PHIEUPH WE HIEUH
    ((0xBF, 0x74), (0xD4CD)),  # HANGUL SYLLABLE PHIEUPH WI KIYEOK
    ((0xBF, 0x75), (0xD4CE)),  # HANGUL SYLLABLE PHIEUPH WI SSANGKIYEOK
    ((0xBF, 0x76), (0xD4CF)),  # HANGUL SYLLABLE PHIEUPH WI KIYEOKSIOS
    ((0xBF, 0x77), (0xD4D1)),  # HANGUL SYLLABLE PHIEUPH WI NIEUNCIEUC
    ((0xBF, 0x78), (0xD4D2)),  # HANGUL SYLLABLE PHIEUPH WI NIEUNHIEUH
    ((0xBF, 0x79), (0xD4D3)),  # HANGUL SYLLABLE PHIEUPH WI TIKEUT
    ((0xBF, 0x7A), (0xD4D5)),  # HANGUL SYLLABLE PHIEUPH WI RIEULKIYEOK
    ((0xBF, 0x81), (0xD4D6)),  # HANGUL SYLLABLE PHIEUPH WI RIEULMIEUM
    ((0xBF, 0x82), (0xD4D7)),  # HANGUL SYLLABLE PHIEUPH WI RIEULPIEUP
    ((0xBF, 0x83), (0xD4D8)),  # HANGUL SYLLABLE PHIEUPH WI RIEULSIOS
    ((0xBF, 0x84), (0xD4D9)),  # HANGUL SYLLABLE PHIEUPH WI RIEULTHIEUTH
    ((0xBF, 0x85), (0xD4DA)),  # HANGUL SYLLABLE PHIEUPH WI RIEULPHIEUPH
    ((0xBF, 0x86), (0xD4DB)),  # HANGUL SYLLABLE PHIEUPH WI RIEULHIEUH
    ((0xBF, 0x87), (0xD4DD)),  # HANGUL SYLLABLE PHIEUPH WI PIEUP
    ((0xBF, 0x88), (0xD4DE)),  # HANGUL SYLLABLE PHIEUPH WI PIEUPSIOS
    ((0xBF, 0x89), (0xD4E0)),  # HANGUL SYLLABLE PHIEUPH WI SSANGSIOS
    ((0xBF, 0x8A), (0xD4E1)),  # HANGUL SYLLABLE PHIEUPH WI IEUNG
    ((0xBF, 0x8B), (0xD4E2)),  # HANGUL SYLLABLE PHIEUPH WI CIEUC
    ((0xBF, 0x8C), (0xD4E3)),  # HANGUL SYLLABLE PHIEUPH WI CHIEUCH
    ((0xBF, 0x8D), (0xD4E4)),  # HANGUL SYLLABLE PHIEUPH WI KHIEUKH
    ((0xBF, 0x8E), (0xD4E5)),  # HANGUL SYLLABLE PHIEUPH WI THIEUTH
    ((0xBF, 0x8F), (0xD4E6)),  # HANGUL SYLLABLE PHIEUPH WI PHIEUPH
    ((0xBF, 0x90), (0xD4E7)),  # HANGUL SYLLABLE PHIEUPH WI HIEUH
    ((0xBF, 0x91), (0xD4E9)),  # HANGUL SYLLABLE PHIEUPH YU KIYEOK
    ((0xBF, 0x92), (0xD4EA)),  # HANGUL SYLLABLE PHIEUPH YU SSANGKIYEOK
    ((0xBF, 0x93), (0xD4EB)),  # HANGUL SYLLABLE PHIEUPH YU KIYEOKSIOS
    ((0xBF, 0x94), (0xD4ED)),  # HANGUL SYLLABLE PHIEUPH YU NIEUNCIEUC
    ((0xBF, 0x95), (0xD4EE)),  # HANGUL SYLLABLE PHIEUPH YU NIEUNHIEUH
    ((0xBF, 0x96), (0xD4EF)),  # HANGUL SYLLABLE PHIEUPH YU TIKEUT
    ((0xBF, 0x97), (0xD4F1)),  # HANGUL SYLLABLE PHIEUPH YU RIEULKIYEOK
    ((0xBF, 0x98), (0xD4F2)),  # HANGUL SYLLABLE PHIEUPH YU RIEULMIEUM
    ((0xBF, 0x99), (0xD4F3)),  # HANGUL SYLLABLE PHIEUPH YU RIEULPIEUP
    ((0xBF, 0x9A), (0xD4F4)),  # HANGUL SYLLABLE PHIEUPH YU RIEULSIOS
    ((0xBF, 0x9B), (0xD4F5)),  # HANGUL SYLLABLE PHIEUPH YU RIEULTHIEUTH
    ((0xBF, 0x9C), (0xD4F6)),  # HANGUL SYLLABLE PHIEUPH YU RIEULPHIEUPH
    ((0xBF, 0x9D), (0xD4F7)),  # HANGUL SYLLABLE PHIEUPH YU RIEULHIEUH
    ((0xBF, 0x9E), (0xD4F9)),  # HANGUL SYLLABLE PHIEUPH YU PIEUP
    ((0xBF, 0x9F), (0xD4FA)),  # HANGUL SYLLABLE PHIEUPH YU PIEUPSIOS
    ((0xBF, 0xA0), (0xD4FC)),  # HANGUL SYLLABLE PHIEUPH YU SSANGSIOS
    ((0xBF, 0xA1), (0xC5D0)),  # HANGUL SYLLABLE IEUNG E
    ((0xBF, 0xA2), (0xC5D1)),  # HANGUL SYLLABLE IEUNG E KIYEOK
    ((0xBF, 0xA3), (0xC5D4)),  # HANGUL SYLLABLE IEUNG E NIEUN
    ((0xBF, 0xA4), (0xC5D8)),  # HANGUL SYLLABLE IEUNG E RIEUL
    ((0xBF, 0xA5), (0xC5E0)),  # HANGUL SYLLABLE IEUNG E MIEUM
    ((0xBF, 0xA6), (0xC5E1)),  # HANGUL SYLLABLE IEUNG E PIEUP
    ((0xBF, 0xA7), (0xC5E3)),  # HANGUL SYLLABLE IEUNG E SIOS
    ((0xBF, 0xA8), (0xC5E5)),  # HANGUL SYLLABLE IEUNG E IEUNG
    ((0xBF, 0xA9), (0xC5EC)),  # HANGUL SYLLABLE IEUNG YEO
    ((0xBF, 0xAA), (0xC5ED)),  # HANGUL SYLLABLE IEUNG YEO KIYEOK
    ((0xBF, 0xAB), (0xC5EE)),  # HANGUL SYLLABLE IEUNG YEO SSANGKIYEOK
    ((0xBF, 0xAC), (0xC5F0)),  # HANGUL SYLLABLE IEUNG YEO NIEUN
    ((0xBF, 0xAD), (0xC5F4)),  # HANGUL SYLLABLE IEUNG YEO RIEUL
    ((0xBF, 0xAE), (0xC5F6)),  # HANGUL SYLLABLE IEUNG YEO RIEULMIEUM
    ((0xBF, 0xAF), (0xC5F7)),  # HANGUL SYLLABLE IEUNG YEO RIEULPIEUP
    ((0xBF, 0xB0), (0xC5FC)),  # HANGUL SYLLABLE IEUNG YEO MIEUM
    ((0xBF, 0xB1), (0xC5FD)),  # HANGUL SYLLABLE IEUNG YEO PIEUP
    ((0xBF, 0xB2), (0xC5FE)),  # HANGUL SYLLABLE IEUNG YEO PIEUPSIOS
    ((0xBF, 0xB3), (0xC5FF)),  # HANGUL SYLLABLE IEUNG YEO SIOS
    ((0xBF, 0xB4), (0xC600)),  # HANGUL SYLLABLE IEUNG YEO SSANGSIOS
    ((0xBF, 0xB5), (0xC601)),  # HANGUL SYLLABLE IEUNG YEO IEUNG
    ((0xBF, 0xB6), (0xC605)),  # HANGUL SYLLABLE IEUNG YEO THIEUTH
    ((0xBF, 0xB7), (0xC606)),  # HANGUL SYLLABLE IEUNG YEO PHIEUPH
    ((0xBF, 0xB8), (0xC607)),  # HANGUL SYLLABLE IEUNG YEO HIEUH
    ((0xBF, 0xB9), (0xC608)),  # HANGUL SYLLABLE IEUNG YE
    ((0xBF, 0xBA), (0xC60C)),  # HANGUL SYLLABLE IEUNG YE NIEUN
    ((0xBF, 0xBB), (0xC610)),  # HANGUL SYLLABLE IEUNG YE RIEUL
    ((0xBF, 0xBC), (0xC618)),  # HANGUL SYLLABLE IEUNG YE MIEUM
    ((0xBF, 0xBD), (0xC619)),  # HANGUL SYLLABLE IEUNG YE PIEUP
    ((0xBF, 0xBE), (0xC61B)),  # HANGUL SYLLABLE IEUNG YE SIOS
    ((0xBF, 0xBF), (0xC61C)),  # HANGUL SYLLABLE IEUNG YE SSANGSIOS
    ((0xBF, 0xC0), (0xC624)),  # HANGUL SYLLABLE IEUNG O
    ((0xBF, 0xC1), (0xC625)),  # HANGUL SYLLABLE IEUNG O KIYEOK
    ((0xBF, 0xC2), (0xC628)),  # HANGUL SYLLABLE IEUNG O NIEUN
    ((0xBF, 0xC3), (0xC62C)),  # HANGUL SYLLABLE IEUNG O RIEUL
    ((0xBF, 0xC4), (0xC62D)),  # HANGUL SYLLABLE IEUNG O RIEULKIYEOK
    ((0xBF, 0xC5), (0xC62E)),  # HANGUL SYLLABLE IEUNG O RIEULMIEUM
    ((0xBF, 0xC6), (0xC630)),  # HANGUL SYLLABLE IEUNG O RIEULSIOS
    ((0xBF, 0xC7), (0xC633)),  # HANGUL SYLLABLE IEUNG O RIEULHIEUH
    ((0xBF, 0xC8), (0xC634)),  # HANGUL SYLLABLE IEUNG O MIEUM
    ((0xBF, 0xC9), (0xC635)),  # HANGUL SYLLABLE IEUNG O PIEUP
    ((0xBF, 0xCA), (0xC637)),  # HANGUL SYLLABLE IEUNG O SIOS
    ((0xBF, 0xCB), (0xC639)),  # HANGUL SYLLABLE IEUNG O IEUNG
    ((0xBF, 0xCC), (0xC63B)),  # HANGUL SYLLABLE IEUNG O CHIEUCH
    ((0xBF, 0xCD), (0xC640)),  # HANGUL SYLLABLE IEUNG WA
    ((0xBF, 0xCE), (0xC641)),  # HANGUL SYLLABLE IEUNG WA KIYEOK
    ((0xBF, 0xCF), (0xC644)),  # HANGUL SYLLABLE IEUNG WA NIEUN
    ((0xBF, 0xD0), (0xC648)),  # HANGUL SYLLABLE IEUNG WA RIEUL
    ((0xBF, 0xD1), (0xC650)),  # HANGUL SYLLABLE IEUNG WA MIEUM
    ((0xBF, 0xD2), (0xC651)),  # HANGUL SYLLABLE IEUNG WA PIEUP
    ((0xBF, 0xD3), (0xC653)),  # HANGUL SYLLABLE IEUNG WA SIOS
    ((0xBF, 0xD4), (0xC654)),  # HANGUL SYLLABLE IEUNG WA SSANGSIOS
    ((0xBF, 0xD5), (0xC655)),  # HANGUL SYLLABLE IEUNG WA IEUNG
    ((0xBF, 0xD6), (0xC65C)),  # HANGUL SYLLABLE IEUNG WAE
    ((0xBF, 0xD7), (0xC65D)),  # HANGUL SYLLABLE IEUNG WAE KIYEOK
    ((0xBF, 0xD8), (0xC660)),  # HANGUL SYLLABLE IEUNG WAE NIEUN
    ((0xBF, 0xD9), (0xC66C)),  # HANGUL SYLLABLE IEUNG WAE MIEUM
    ((0xBF, 0xDA), (0xC66F)),  # HANGUL SYLLABLE IEUNG WAE SIOS
    ((0xBF, 0xDB), (0xC671)),  # HANGUL SYLLABLE IEUNG WAE IEUNG
    ((0xBF, 0xDC), (0xC678)),  # HANGUL SYLLABLE IEUNG OE
    ((0xBF, 0xDD), (0xC679)),  # HANGUL SYLLABLE IEUNG OE KIYEOK
    ((0xBF, 0xDE), (0xC67C)),  # HANGUL SYLLABLE IEUNG OE NIEUN
    ((0xBF, 0xDF), (0xC680)),  # HANGUL SYLLABLE IEUNG OE RIEUL
    ((0xBF, 0xE0), (0xC688)),  # HANGUL SYLLABLE IEUNG OE MIEUM
    ((0xBF, 0xE1), (0xC689)),  # HANGUL SYLLABLE IEUNG OE PIEUP
    ((0xBF, 0xE2), (0xC68B)),  # HANGUL SYLLABLE IEUNG OE SIOS
    ((0xBF, 0xE3), (0xC68D)),  # HANGUL SYLLABLE IEUNG OE IEUNG
    ((0xBF, 0xE4), (0xC694)),  # HANGUL SYLLABLE IEUNG YO
    ((0xBF, 0xE5), (0xC695)),  # HANGUL SYLLABLE IEUNG YO KIYEOK
    ((0xBF, 0xE6), (0xC698)),  # HANGUL SYLLABLE IEUNG YO NIEUN
    ((0xBF, 0xE7), (0xC69C)),  # HANGUL SYLLABLE IEUNG YO RIEUL
    ((0xBF, 0xE8), (0xC6A4)),  # HANGUL SYLLABLE IEUNG YO MIEUM
    ((0xBF, 0xE9), (0xC6A5)),  # HANGUL SYLLABLE IEUNG YO PIEUP
    ((0xBF, 0xEA), (0xC6A7)),  # HANGUL SYLLABLE IEUNG YO SIOS
    ((0xBF, 0xEB), (0xC6A9)),  # HANGUL SYLLABLE IEUNG YO IEUNG
    ((0xBF, 0xEC), (0xC6B0)),  # HANGUL SYLLABLE IEUNG U
    ((0xBF, 0xED), (0xC6B1)),  # HANGUL SYLLABLE IEUNG U KIYEOK
    ((0xBF, 0xEE), (0xC6B4)),  # HANGUL SYLLABLE IEUNG U NIEUN
    ((0xBF, 0xEF), (0xC6B8)),  # HANGUL SYLLABLE IEUNG U RIEUL
    ((0xBF, 0xF0), (0xC6B9)),  # HANGUL SYLLABLE IEUNG U RIEULKIYEOK
    ((0xBF, 0xF1), (0xC6BA)),  # HANGUL SYLLABLE IEUNG U RIEULMIEUM
    ((0xBF, 0xF2), (0xC6C0)),  # HANGUL SYLLABLE IEUNG U MIEUM
    ((0xBF, 0xF3), (0xC6C1)),  # HANGUL SYLLABLE IEUNG U PIEUP
    ((0xBF, 0xF4), (0xC6C3)),  # HANGUL SYLLABLE IEUNG U SIOS
    ((0xBF, 0xF5), (0xC6C5)),  # HANGUL SYLLABLE IEUNG U IEUNG
    ((0xBF, 0xF6), (0xC6CC)),  # HANGUL SYLLABLE IEUNG WEO
    ((0xBF, 0xF7), (0xC6CD)),  # HANGUL SYLLABLE IEUNG WEO KIYEOK
    ((0xBF, 0xF8), (0xC6D0)),  # HANGUL SYLLABLE IEUNG WEO NIEUN
    ((0xBF, 0xF9), (0xC6D4)),  # HANGUL SYLLABLE IEUNG WEO RIEUL
    ((0xBF, 0xFA), (0xC6DC)),  # HANGUL SYLLABLE IEUNG WEO MIEUM
    ((0xBF, 0xFB), (0xC6DD)),  # HANGUL SYLLABLE IEUNG WEO PIEUP
    ((0xBF, 0xFC), (0xC6E0)),  # HANGUL SYLLABLE IEUNG WEO SSANGSIOS
    ((0xBF, 0xFD), (0xC6E1)),  # HANGUL SYLLABLE IEUNG WEO IEUNG
    ((0xBF, 0xFE), (0xC6E8)),  # HANGUL SYLLABLE IEUNG WE
    ((0xC0, 0x41), (0xD4FE)),  # HANGUL SYLLABLE PHIEUPH YU CIEUC
    ((0xC0, 0x42), (0xD4FF)),  # HANGUL SYLLABLE PHIEUPH YU CHIEUCH
    ((0xC0, 0x43), (0xD500)),  # HANGUL SYLLABLE PHIEUPH YU KHIEUKH
    ((0xC0, 0x44), (0xD501)),  # HANGUL SYLLABLE PHIEUPH YU THIEUTH
    ((0xC0, 0x45), (0xD502)),  # HANGUL SYLLABLE PHIEUPH YU PHIEUPH
    ((0xC0, 0x46), (0xD503)),  # HANGUL SYLLABLE PHIEUPH YU HIEUH
    ((0xC0, 0x47), (0xD505)),  # HANGUL SYLLABLE PHIEUPH EU KIYEOK
    ((0xC0, 0x48), (0xD506)),  # HANGUL SYLLABLE PHIEUPH EU SSANGKIYEOK
    ((0xC0, 0x49), (0xD507)),  # HANGUL SYLLABLE PHIEUPH EU KIYEOKSIOS
    ((0xC0, 0x4A), (0xD509)),  # HANGUL SYLLABLE PHIEUPH EU NIEUNCIEUC
    ((0xC0, 0x4B), (0xD50A)),  # HANGUL SYLLABLE PHIEUPH EU NIEUNHIEUH
    ((0xC0, 0x4C), (0xD50B)),  # HANGUL SYLLABLE PHIEUPH EU TIKEUT
    ((0xC0, 0x4D), (0xD50D)),  # HANGUL SYLLABLE PHIEUPH EU RIEULKIYEOK
    ((0xC0, 0x4E), (0xD50E)),  # HANGUL SYLLABLE PHIEUPH EU RIEULMIEUM
    ((0xC0, 0x4F), (0xD50F)),  # HANGUL SYLLABLE PHIEUPH EU RIEULPIEUP
    ((0xC0, 0x50), (0xD510)),  # HANGUL SYLLABLE PHIEUPH EU RIEULSIOS
    ((0xC0, 0x51), (0xD511)),  # HANGUL SYLLABLE PHIEUPH EU RIEULTHIEUTH
    ((0xC0, 0x52), (0xD512)),  # HANGUL SYLLABLE PHIEUPH EU RIEULPHIEUPH
    ((0xC0, 0x53), (0xD513)),  # HANGUL SYLLABLE PHIEUPH EU RIEULHIEUH
    ((0xC0, 0x54), (0xD516)),  # HANGUL SYLLABLE PHIEUPH EU PIEUPSIOS
    ((0xC0, 0x55), (0xD518)),  # HANGUL SYLLABLE PHIEUPH EU SSANGSIOS
    ((0xC0, 0x56), (0xD519)),  # HANGUL SYLLABLE PHIEUPH EU IEUNG
    ((0xC0, 0x57), (0xD51A)),  # HANGUL SYLLABLE PHIEUPH EU CIEUC
    ((0xC0, 0x58), (0xD51B)),  # HANGUL SYLLABLE PHIEUPH EU CHIEUCH
    ((0xC0, 0x59), (0xD51C)),  # HANGUL SYLLABLE PHIEUPH EU KHIEUKH
    ((0xC0, 0x5A), (0xD51D)),  # HANGUL SYLLABLE PHIEUPH EU THIEUTH
    ((0xC0, 0x61), (0xD51E)),  # HANGUL SYLLABLE PHIEUPH EU PHIEUPH
    ((0xC0, 0x62), (0xD51F)),  # HANGUL SYLLABLE PHIEUPH EU HIEUH
    ((0xC0, 0x63), (0xD520)),  # HANGUL SYLLABLE PHIEUPH YI
    ((0xC0, 0x64), (0xD521)),  # HANGUL SYLLABLE PHIEUPH YI KIYEOK
    ((0xC0, 0x65), (0xD522)),  # HANGUL SYLLABLE PHIEUPH YI SSANGKIYEOK
    ((0xC0, 0x66), (0xD523)),  # HANGUL SYLLABLE PHIEUPH YI KIYEOKSIOS
    ((0xC0, 0x67), (0xD524)),  # HANGUL SYLLABLE PHIEUPH YI NIEUN
    ((0xC0, 0x68), (0xD525)),  # HANGUL SYLLABLE PHIEUPH YI NIEUNCIEUC
    ((0xC0, 0x69), (0xD526)),  # HANGUL SYLLABLE PHIEUPH YI NIEUNHIEUH
    ((0xC0, 0x6A), (0xD527)),  # HANGUL SYLLABLE PHIEUPH YI TIKEUT
    ((0xC0, 0x6B), (0xD528)),  # HANGUL SYLLABLE PHIEUPH YI RIEUL
    ((0xC0, 0x6C), (0xD529)),  # HANGUL SYLLABLE PHIEUPH YI RIEULKIYEOK
    ((0xC0, 0x6D), (0xD52A)),  # HANGUL SYLLABLE PHIEUPH YI RIEULMIEUM
    ((0xC0, 0x6E), (0xD52B)),  # HANGUL SYLLABLE PHIEUPH YI RIEULPIEUP
    ((0xC0, 0x6F), (0xD52C)),  # HANGUL SYLLABLE PHIEUPH YI RIEULSIOS
    ((0xC0, 0x70), (0xD52D)),  # HANGUL SYLLABLE PHIEUPH YI RIEULTHIEUTH
    ((0xC0, 0x71), (0xD52E)),  # HANGUL SYLLABLE PHIEUPH YI RIEULPHIEUPH
    ((0xC0, 0x72), (0xD52F)),  # HANGUL SYLLABLE PHIEUPH YI RIEULHIEUH
    ((0xC0, 0x73), (0xD530)),  # HANGUL SYLLABLE PHIEUPH YI MIEUM
    ((0xC0, 0x74), (0xD531)),  # HANGUL SYLLABLE PHIEUPH YI PIEUP
    ((0xC0, 0x75), (0xD532)),  # HANGUL SYLLABLE PHIEUPH YI PIEUPSIOS
    ((0xC0, 0x76), (0xD533)),  # HANGUL SYLLABLE PHIEUPH YI SIOS
    ((0xC0, 0x77), (0xD534)),  # HANGUL SYLLABLE PHIEUPH YI SSANGSIOS
    ((0xC0, 0x78), (0xD535)),  # HANGUL SYLLABLE PHIEUPH YI IEUNG
    ((0xC0, 0x79), (0xD536)),  # HANGUL SYLLABLE PHIEUPH YI CIEUC
    ((0xC0, 0x7A), (0xD537)),  # HANGUL SYLLABLE PHIEUPH YI CHIEUCH
    ((0xC0, 0x81), (0xD538)),  # HANGUL SYLLABLE PHIEUPH YI KHIEUKH
    ((0xC0, 0x82), (0xD539)),  # HANGUL SYLLABLE PHIEUPH YI THIEUTH
    ((0xC0, 0x83), (0xD53A)),  # HANGUL SYLLABLE PHIEUPH YI PHIEUPH
    ((0xC0, 0x84), (0xD53B)),  # HANGUL SYLLABLE PHIEUPH YI HIEUH
    ((0xC0, 0x85), (0xD53E)),  # HANGUL SYLLABLE PHIEUPH I SSANGKIYEOK
    ((0xC0, 0x86), (0xD53F)),  # HANGUL SYLLABLE PHIEUPH I KIYEOKSIOS
    ((0xC0, 0x87), (0xD541)),  # HANGUL SYLLABLE PHIEUPH I NIEUNCIEUC
    ((0xC0, 0x88), (0xD542)),  # HANGUL SYLLABLE PHIEUPH I NIEUNHIEUH
    ((0xC0, 0x89), (0xD543)),  # HANGUL SYLLABLE PHIEUPH I TIKEUT
    ((0xC0, 0x8A), (0xD545)),  # HANGUL SYLLABLE PHIEUPH I RIEULKIYEOK
    ((0xC0, 0x8B), (0xD546)),  # HANGUL SYLLABLE PHIEUPH I RIEULMIEUM
    ((0xC0, 0x8C), (0xD547)),  # HANGUL SYLLABLE PHIEUPH I RIEULPIEUP
    ((0xC0, 0x8D), (0xD548)),  # HANGUL SYLLABLE PHIEUPH I RIEULSIOS
    ((0xC0, 0x8E), (0xD549)),  # HANGUL SYLLABLE PHIEUPH I RIEULTHIEUTH
    ((0xC0, 0x8F), (0xD54A)),  # HANGUL SYLLABLE PHIEUPH I RIEULPHIEUPH
    ((0xC0, 0x90), (0xD54B)),  # HANGUL SYLLABLE PHIEUPH I RIEULHIEUH
    ((0xC0, 0x91), (0xD54E)),  # HANGUL SYLLABLE PHIEUPH I PIEUPSIOS
    ((0xC0, 0x92), (0xD550)),  # HANGUL SYLLABLE PHIEUPH I SSANGSIOS
    ((0xC0, 0x93), (0xD552)),  # HANGUL SYLLABLE PHIEUPH I CIEUC
    ((0xC0, 0x94), (0xD553)),  # HANGUL SYLLABLE PHIEUPH I CHIEUCH
    ((0xC0, 0x95), (0xD554)),  # HANGUL SYLLABLE PHIEUPH I KHIEUKH
    ((0xC0, 0x96), (0xD555)),  # HANGUL SYLLABLE PHIEUPH I THIEUTH
    ((0xC0, 0x97), (0xD556)),  # HANGUL SYLLABLE PHIEUPH I PHIEUPH
    ((0xC0, 0x98), (0xD557)),  # HANGUL SYLLABLE PHIEUPH I HIEUH
    ((0xC0, 0x99), (0xD55A)),  # HANGUL SYLLABLE HIEUH A SSANGKIYEOK
    ((0xC0, 0x9A), (0xD55B)),  # HANGUL SYLLABLE HIEUH A KIYEOKSIOS
    ((0xC0, 0x9B), (0xD55D)),  # HANGUL SYLLABLE HIEUH A NIEUNCIEUC
    ((0xC0, 0x9C), (0xD55E)),  # HANGUL SYLLABLE HIEUH A NIEUNHIEUH
    ((0xC0, 0x9D), (0xD55F)),  # HANGUL SYLLABLE HIEUH A TIKEUT
    ((0xC0, 0x9E), (0xD561)),  # HANGUL SYLLABLE HIEUH A RIEULKIYEOK
    ((0xC0, 0x9F), (0xD562)),  # HANGUL SYLLABLE HIEUH A RIEULMIEUM
    ((0xC0, 0xA0), (0xD563)),  # HANGUL SYLLABLE HIEUH A RIEULPIEUP
    ((0xC0, 0xA1), (0xC6E9)),  # HANGUL SYLLABLE IEUNG WE KIYEOK
    ((0xC0, 0xA2), (0xC6EC)),  # HANGUL SYLLABLE IEUNG WE NIEUN
    ((0xC0, 0xA3), (0xC6F0)),  # HANGUL SYLLABLE IEUNG WE RIEUL
    ((0xC0, 0xA4), (0xC6F8)),  # HANGUL SYLLABLE IEUNG WE MIEUM
    ((0xC0, 0xA5), (0xC6F9)),  # HANGUL SYLLABLE IEUNG WE PIEUP
    ((0xC0, 0xA6), (0xC6FD)),  # HANGUL SYLLABLE IEUNG WE IEUNG
    ((0xC0, 0xA7), (0xC704)),  # HANGUL SYLLABLE IEUNG WI
    ((0xC0, 0xA8), (0xC705)),  # HANGUL SYLLABLE IEUNG WI KIYEOK
    ((0xC0, 0xA9), (0xC708)),  # HANGUL SYLLABLE IEUNG WI NIEUN
    ((0xC0, 0xAA), (0xC70C)),  # HANGUL SYLLABLE IEUNG WI RIEUL
    ((0xC0, 0xAB), (0xC714)),  # HANGUL SYLLABLE IEUNG WI MIEUM
    ((0xC0, 0xAC), (0xC715)),  # HANGUL SYLLABLE IEUNG WI PIEUP
    ((0xC0, 0xAD), (0xC717)),  # HANGUL SYLLABLE IEUNG WI SIOS
    ((0xC0, 0xAE), (0xC719)),  # HANGUL SYLLABLE IEUNG WI IEUNG
    ((0xC0, 0xAF), (0xC720)),  # HANGUL SYLLABLE IEUNG YU
    ((0xC0, 0xB0), (0xC721)),  # HANGUL SYLLABLE IEUNG YU KIYEOK
    ((0xC0, 0xB1), (0xC724)),  # HANGUL SYLLABLE IEUNG YU NIEUN
    ((0xC0, 0xB2), (0xC728)),  # HANGUL SYLLABLE IEUNG YU RIEUL
    ((0xC0, 0xB3), (0xC730)),  # HANGUL SYLLABLE IEUNG YU MIEUM
    ((0xC0, 0xB4), (0xC731)),  # HANGUL SYLLABLE IEUNG YU PIEUP
    ((0xC0, 0xB5), (0xC733)),  # HANGUL SYLLABLE IEUNG YU SIOS
    ((0xC0, 0xB6), (0xC735)),  # HANGUL SYLLABLE IEUNG YU IEUNG
    ((0xC0, 0xB7), (0xC737)),  # HANGUL SYLLABLE IEUNG YU CHIEUCH
    ((0xC0, 0xB8), (0xC73C)),  # HANGUL SYLLABLE IEUNG EU
    ((0xC0, 0xB9), (0xC73D)),  # HANGUL SYLLABLE IEUNG EU KIYEOK
    ((0xC0, 0xBA), (0xC740)),  # HANGUL SYLLABLE IEUNG EU NIEUN
    ((0xC0, 0xBB), (0xC744)),  # HANGUL SYLLABLE IEUNG EU RIEUL
    ((0xC0, 0xBC), (0xC74A)),  # HANGUL SYLLABLE IEUNG EU RIEULPHIEUPH
    ((0xC0, 0xBD), (0xC74C)),  # HANGUL SYLLABLE IEUNG EU MIEUM
    ((0xC0, 0xBE), (0xC74D)),  # HANGUL SYLLABLE IEUNG EU PIEUP
    ((0xC0, 0xBF), (0xC74F)),  # HANGUL SYLLABLE IEUNG EU SIOS
    ((0xC0, 0xC0), (0xC751)),  # HANGUL SYLLABLE IEUNG EU IEUNG
    ((0xC0, 0xC1), (0xC752)),  # HANGUL SYLLABLE IEUNG EU CIEUC
    ((0xC0, 0xC2), (0xC753)),  # HANGUL SYLLABLE IEUNG EU CHIEUCH
    ((0xC0, 0xC3), (0xC754)),  # HANGUL SYLLABLE IEUNG EU KHIEUKH
    ((0xC0, 0xC4), (0xC755)),  # HANGUL SYLLABLE IEUNG EU THIEUTH
    ((0xC0, 0xC5), (0xC756)),  # HANGUL SYLLABLE IEUNG EU PHIEUPH
    ((0xC0, 0xC6), (0xC757)),  # HANGUL SYLLABLE IEUNG EU HIEUH
    ((0xC0, 0xC7), (0xC758)),  # HANGUL SYLLABLE IEUNG YI
    ((0xC0, 0xC8), (0xC75C)),  # HANGUL SYLLABLE IEUNG YI NIEUN
    ((0xC0, 0xC9), (0xC760)),  # HANGUL SYLLABLE IEUNG YI RIEUL
    ((0xC0, 0xCA), (0xC768)),  # HANGUL SYLLABLE IEUNG YI MIEUM
    ((0xC0, 0xCB), (0xC76B)),  # HANGUL SYLLABLE IEUNG YI SIOS
    ((0xC0, 0xCC), (0xC774)),  # HANGUL SYLLABLE IEUNG I
    ((0xC0, 0xCD), (0xC775)),  # HANGUL SYLLABLE IEUNG I KIYEOK
    ((0xC0, 0xCE), (0xC778)),  # HANGUL SYLLABLE IEUNG I NIEUN
    ((0xC0, 0xCF), (0xC77C)),  # HANGUL SYLLABLE IEUNG I RIEUL
    ((0xC0, 0xD0), (0xC77D)),  # HANGUL SYLLABLE IEUNG I RIEULKIYEOK
    ((0xC0, 0xD1), (0xC77E)),  # HANGUL SYLLABLE IEUNG I RIEULMIEUM
    ((0xC0, 0xD2), (0xC783)),  # HANGUL SYLLABLE IEUNG I RIEULHIEUH
    ((0xC0, 0xD3), (0xC784)),  # HANGUL SYLLABLE IEUNG I MIEUM
    ((0xC0, 0xD4), (0xC785)),  # HANGUL SYLLABLE IEUNG I PIEUP
    ((0xC0, 0xD5), (0xC787)),  # HANGUL SYLLABLE IEUNG I SIOS
    ((0xC0, 0xD6), (0xC788)),  # HANGUL SYLLABLE IEUNG I SSANGSIOS
    ((0xC0, 0xD7), (0xC789)),  # HANGUL SYLLABLE IEUNG I IEUNG
    ((0xC0, 0xD8), (0xC78A)),  # HANGUL SYLLABLE IEUNG I CIEUC
    ((0xC0, 0xD9), (0xC78E)),  # HANGUL SYLLABLE IEUNG I PHIEUPH
    ((0xC0, 0xDA), (0xC790)),  # HANGUL SYLLABLE CIEUC A
    ((0xC0, 0xDB), (0xC791)),  # HANGUL SYLLABLE CIEUC A KIYEOK
    ((0xC0, 0xDC), (0xC794)),  # HANGUL SYLLABLE CIEUC A NIEUN
    ((0xC0, 0xDD), (0xC796)),  # HANGUL SYLLABLE CIEUC A NIEUNHIEUH
    ((0xC0, 0xDE), (0xC797)),  # HANGUL SYLLABLE CIEUC A TIKEUT
    ((0xC0, 0xDF), (0xC798)),  # HANGUL SYLLABLE CIEUC A RIEUL
    ((0xC0, 0xE0), (0xC79A)),  # HANGUL SYLLABLE CIEUC A RIEULMIEUM
    ((0xC0, 0xE1), (0xC7A0)),  # HANGUL SYLLABLE CIEUC A MIEUM
    ((0xC0, 0xE2), (0xC7A1)),  # HANGUL SYLLABLE CIEUC A PIEUP
    ((0xC0, 0xE3), (0xC7A3)),  # HANGUL SYLLABLE CIEUC A SIOS
    ((0xC0, 0xE4), (0xC7A4)),  # HANGUL SYLLABLE CIEUC A SSANGSIOS
    ((0xC0, 0xE5), (0xC7A5)),  # HANGUL SYLLABLE CIEUC A IEUNG
    ((0xC0, 0xE6), (0xC7A6)),  # HANGUL SYLLABLE CIEUC A CIEUC
    ((0xC0, 0xE7), (0xC7AC)),  # HANGUL SYLLABLE CIEUC AE
    ((0xC0, 0xE8), (0xC7AD)),  # HANGUL SYLLABLE CIEUC AE KIYEOK
    ((0xC0, 0xE9), (0xC7B0)),  # HANGUL SYLLABLE CIEUC AE NIEUN
    ((0xC0, 0xEA), (0xC7B4)),  # HANGUL SYLLABLE CIEUC AE RIEUL
    ((0xC0, 0xEB), (0xC7BC)),  # HANGUL SYLLABLE CIEUC AE MIEUM
    ((0xC0, 0xEC), (0xC7BD)),  # HANGUL SYLLABLE CIEUC AE PIEUP
    ((0xC0, 0xED), (0xC7BF)),  # HANGUL SYLLABLE CIEUC AE SIOS
    ((0xC0, 0xEE), (0xC7C0)),  # HANGUL SYLLABLE CIEUC AE SSANGSIOS
    ((0xC0, 0xEF), (0xC7C1)),  # HANGUL SYLLABLE CIEUC AE IEUNG
    ((0xC0, 0xF0), (0xC7C8)),  # HANGUL SYLLABLE CIEUC YA
    ((0xC0, 0xF1), (0xC7C9)),  # HANGUL SYLLABLE CIEUC YA KIYEOK
    ((0xC0, 0xF2), (0xC7CC)),  # HANGUL SYLLABLE CIEUC YA NIEUN
    ((0xC0, 0xF3), (0xC7CE)),  # HANGUL SYLLABLE CIEUC YA NIEUNHIEUH
    ((0xC0, 0xF4), (0xC7D0)),  # HANGUL SYLLABLE CIEUC YA RIEUL
    ((0xC0, 0xF5), (0xC7D8)),  # HANGUL SYLLABLE CIEUC YA MIEUM
    ((0xC0, 0xF6), (0xC7DD)),  # HANGUL SYLLABLE CIEUC YA IEUNG
    ((0xC0, 0xF7), (0xC7E4)),  # HANGUL SYLLABLE CIEUC YAE
    ((0xC0, 0xF8), (0xC7E8)),  # HANGUL SYLLABLE CIEUC YAE NIEUN
    ((0xC0, 0xF9), (0xC7EC)),  # HANGUL SYLLABLE CIEUC YAE RIEUL
    ((0xC0, 0xFA), (0xC800)),  # HANGUL SYLLABLE CIEUC EO
    ((0xC0, 0xFB), (0xC801)),  # HANGUL SYLLABLE CIEUC EO KIYEOK
    ((0xC0, 0xFC), (0xC804)),  # HANGUL SYLLABLE CIEUC EO NIEUN
    ((0xC0, 0xFD), (0xC808)),  # HANGUL SYLLABLE CIEUC EO RIEUL
    ((0xC0, 0xFE), (0xC80A)),  # HANGUL SYLLABLE CIEUC EO RIEULMIEUM
    ((0xC1, 0x41), (0xD564)),  # HANGUL SYLLABLE HIEUH A RIEULSIOS
    ((0xC1, 0x42), (0xD566)),  # HANGUL SYLLABLE HIEUH A RIEULPHIEUPH
    ((0xC1, 0x43), (0xD567)),  # HANGUL SYLLABLE HIEUH A RIEULHIEUH
    ((0xC1, 0x44), (0xD56A)),  # HANGUL SYLLABLE HIEUH A PIEUPSIOS
    ((0xC1, 0x45), (0xD56C)),  # HANGUL SYLLABLE HIEUH A SSANGSIOS
    ((0xC1, 0x46), (0xD56E)),  # HANGUL SYLLABLE HIEUH A CIEUC
    ((0xC1, 0x47), (0xD56F)),  # HANGUL SYLLABLE HIEUH A CHIEUCH
    ((0xC1, 0x48), (0xD570)),  # HANGUL SYLLABLE HIEUH A KHIEUKH
    ((0xC1, 0x49), (0xD571)),  # HANGUL SYLLABLE HIEUH A THIEUTH
    ((0xC1, 0x4A), (0xD572)),  # HANGUL SYLLABLE HIEUH A PHIEUPH
    ((0xC1, 0x4B), (0xD573)),  # HANGUL SYLLABLE HIEUH A HIEUH
    ((0xC1, 0x4C), (0xD576)),  # HANGUL SYLLABLE HIEUH AE SSANGKIYEOK
    ((0xC1, 0x4D), (0xD577)),  # HANGUL SYLLABLE HIEUH AE KIYEOKSIOS
    ((0xC1, 0x4E), (0xD579)),  # HANGUL SYLLABLE HIEUH AE NIEUNCIEUC
    ((0xC1, 0x4F), (0xD57A)),  # HANGUL SYLLABLE HIEUH AE NIEUNHIEUH
    ((0xC1, 0x50), (0xD57B)),  # HANGUL SYLLABLE HIEUH AE TIKEUT
    ((0xC1, 0x51), (0xD57D)),  # HANGUL SYLLABLE HIEUH AE RIEULKIYEOK
    ((0xC1, 0x52), (0xD57E)),  # HANGUL SYLLABLE HIEUH AE RIEULMIEUM
    ((0xC1, 0x53), (0xD57F)),  # HANGUL SYLLABLE HIEUH AE RIEULPIEUP
    ((0xC1, 0x54), (0xD580)),  # HANGUL SYLLABLE HIEUH AE RIEULSIOS
    ((0xC1, 0x55), (0xD581)),  # HANGUL SYLLABLE HIEUH AE RIEULTHIEUTH
    ((0xC1, 0x56), (0xD582)),  # HANGUL SYLLABLE HIEUH AE RIEULPHIEUPH
    ((0xC1, 0x57), (0xD583)),  # HANGUL SYLLABLE HIEUH AE RIEULHIEUH
    ((0xC1, 0x58), (0xD586)),  # HANGUL SYLLABLE HIEUH AE PIEUPSIOS
    ((0xC1, 0x59), (0xD58A)),  # HANGUL SYLLABLE HIEUH AE CIEUC
    ((0xC1, 0x5A), (0xD58B)),  # HANGUL SYLLABLE HIEUH AE CHIEUCH
    ((0xC1, 0x61), (0xD58C)),  # HANGUL SYLLABLE HIEUH AE KHIEUKH
    ((0xC1, 0x62), (0xD58D)),  # HANGUL SYLLABLE HIEUH AE THIEUTH
    ((0xC1, 0x63), (0xD58E)),  # HANGUL SYLLABLE HIEUH AE PHIEUPH
    ((0xC1, 0x64), (0xD58F)),  # HANGUL SYLLABLE HIEUH AE HIEUH
    ((0xC1, 0x65), (0xD591)),  # HANGUL SYLLABLE HIEUH YA KIYEOK
    ((0xC1, 0x66), (0xD592)),  # HANGUL SYLLABLE HIEUH YA SSANGKIYEOK
    ((0xC1, 0x67), (0xD593)),  # HANGUL SYLLABLE HIEUH YA KIYEOKSIOS
    ((0xC1, 0x68), (0xD594)),  # HANGUL SYLLABLE HIEUH YA NIEUN
    ((0xC1, 0x69), (0xD595)),  # HANGUL SYLLABLE HIEUH YA NIEUNCIEUC
    ((0xC1, 0x6A), (0xD596)),  # HANGUL SYLLABLE HIEUH YA NIEUNHIEUH
    ((0xC1, 0x6B), (0xD597)),  # HANGUL SYLLABLE HIEUH YA TIKEUT
    ((0xC1, 0x6C), (0xD598)),  # HANGUL SYLLABLE HIEUH YA RIEUL
    ((0xC1, 0x6D), (0xD599)),  # HANGUL SYLLABLE HIEUH YA RIEULKIYEOK
    ((0xC1, 0x6E), (0xD59A)),  # HANGUL SYLLABLE HIEUH YA RIEULMIEUM
    ((0xC1, 0x6F), (0xD59B)),  # HANGUL SYLLABLE HIEUH YA RIEULPIEUP
    ((0xC1, 0x70), (0xD59C)),  # HANGUL SYLLABLE HIEUH YA RIEULSIOS
    ((0xC1, 0x71), (0xD59D)),  # HANGUL SYLLABLE HIEUH YA RIEULTHIEUTH
    ((0xC1, 0x72), (0xD59E)),  # HANGUL SYLLABLE HIEUH YA RIEULPHIEUPH
    ((0xC1, 0x73), (0xD59F)),  # HANGUL SYLLABLE HIEUH YA RIEULHIEUH
    ((0xC1, 0x74), (0xD5A0)),  # HANGUL SYLLABLE HIEUH YA MIEUM
    ((0xC1, 0x75), (0xD5A1)),  # HANGUL SYLLABLE HIEUH YA PIEUP
    ((0xC1, 0x76), (0xD5A2)),  # HANGUL SYLLABLE HIEUH YA PIEUPSIOS
    ((0xC1, 0x77), (0xD5A3)),  # HANGUL SYLLABLE HIEUH YA SIOS
    ((0xC1, 0x78), (0xD5A4)),  # HANGUL SYLLABLE HIEUH YA SSANGSIOS
    ((0xC1, 0x79), (0xD5A6)),  # HANGUL SYLLABLE HIEUH YA CIEUC
    ((0xC1, 0x7A), (0xD5A7)),  # HANGUL SYLLABLE HIEUH YA CHIEUCH
    ((0xC1, 0x81), (0xD5A8)),  # HANGUL SYLLABLE HIEUH YA KHIEUKH
    ((0xC1, 0x82), (0xD5A9)),  # HANGUL SYLLABLE HIEUH YA THIEUTH
    ((0xC1, 0x83), (0xD5AA)),  # HANGUL SYLLABLE HIEUH YA PHIEUPH
    ((0xC1, 0x84), (0xD5AB)),  # HANGUL SYLLABLE HIEUH YA HIEUH
    ((0xC1, 0x85), (0xD5AC)),  # HANGUL SYLLABLE HIEUH YAE
    ((0xC1, 0x86), (0xD5AD)),  # HANGUL SYLLABLE HIEUH YAE KIYEOK
    ((0xC1, 0x87), (0xD5AE)),  # HANGUL SYLLABLE HIEUH YAE SSANGKIYEOK
    ((0xC1, 0x88), (0xD5AF)),  # HANGUL SYLLABLE HIEUH YAE KIYEOKSIOS
    ((0xC1, 0x89), (0xD5B0)),  # HANGUL SYLLABLE HIEUH YAE NIEUN
    ((0xC1, 0x8A), (0xD5B1)),  # HANGUL SYLLABLE HIEUH YAE NIEUNCIEUC
    ((0xC1, 0x8B), (0xD5B2)),  # HANGUL SYLLABLE HIEUH YAE NIEUNHIEUH
    ((0xC1, 0x8C), (0xD5B3)),  # HANGUL SYLLABLE HIEUH YAE TIKEUT
    ((0xC1, 0x8D), (0xD5B4)),  # HANGUL SYLLABLE HIEUH YAE RIEUL
    ((0xC1, 0x8E), (0xD5B5)),  # HANGUL SYLLABLE HIEUH YAE RIEULKIYEOK
    ((0xC1, 0x8F), (0xD5B6)),  # HANGUL SYLLABLE HIEUH YAE RIEULMIEUM
    ((0xC1, 0x90), (0xD5B7)),  # HANGUL SYLLABLE HIEUH YAE RIEULPIEUP
    ((0xC1, 0x91), (0xD5B8)),  # HANGUL SYLLABLE HIEUH YAE RIEULSIOS
    ((0xC1, 0x92), (0xD5B9)),  # HANGUL SYLLABLE HIEUH YAE RIEULTHIEUTH
    ((0xC1, 0x93), (0xD5BA)),  # HANGUL SYLLABLE HIEUH YAE RIEULPHIEUPH
    ((0xC1, 0x94), (0xD5BB)),  # HANGUL SYLLABLE HIEUH YAE RIEULHIEUH
    ((0xC1, 0x95), (0xD5BC)),  # HANGUL SYLLABLE HIEUH YAE MIEUM
    ((0xC1, 0x96), (0xD5BD)),  # HANGUL SYLLABLE HIEUH YAE PIEUP
    ((0xC1, 0x97), (0xD5BE)),  # HANGUL SYLLABLE HIEUH YAE PIEUPSIOS
    ((0xC1, 0x98), (0xD5BF)),  # HANGUL SYLLABLE HIEUH YAE SIOS
    ((0xC1, 0x99), (0xD5C0)),  # HANGUL SYLLABLE HIEUH YAE SSANGSIOS
    ((0xC1, 0x9A), (0xD5C1)),  # HANGUL SYLLABLE HIEUH YAE IEUNG
    ((0xC1, 0x9B), (0xD5C2)),  # HANGUL SYLLABLE HIEUH YAE CIEUC
    ((0xC1, 0x9C), (0xD5C3)),  # HANGUL SYLLABLE HIEUH YAE CHIEUCH
    ((0xC1, 0x9D), (0xD5C4)),  # HANGUL SYLLABLE HIEUH YAE KHIEUKH
    ((0xC1, 0x9E), (0xD5C5)),  # HANGUL SYLLABLE HIEUH YAE THIEUTH
    ((0xC1, 0x9F), (0xD5C6)),  # HANGUL SYLLABLE HIEUH YAE PHIEUPH
    ((0xC1, 0xA0), (0xD5C7)),  # HANGUL SYLLABLE HIEUH YAE HIEUH
    ((0xC1, 0xA1), (0xC810)),  # HANGUL SYLLABLE CIEUC EO MIEUM
    ((0xC1, 0xA2), (0xC811)),  # HANGUL SYLLABLE CIEUC EO PIEUP
    ((0xC1, 0xA3), (0xC813)),  # HANGUL SYLLABLE CIEUC EO SIOS
    ((0xC1, 0xA4), (0xC815)),  # HANGUL SYLLABLE CIEUC EO IEUNG
    ((0xC1, 0xA5), (0xC816)),  # HANGUL SYLLABLE CIEUC EO CIEUC
    ((0xC1, 0xA6), (0xC81C)),  # HANGUL SYLLABLE CIEUC E
    ((0xC1, 0xA7), (0xC81D)),  # HANGUL SYLLABLE CIEUC E KIYEOK
    ((0xC1, 0xA8), (0xC820)),  # HANGUL SYLLABLE CIEUC E NIEUN
    ((0xC1, 0xA9), (0xC824)),  # HANGUL SYLLABLE CIEUC E RIEUL
    ((0xC1, 0xAA), (0xC82C)),  # HANGUL SYLLABLE CIEUC E MIEUM
    ((0xC1, 0xAB), (0xC82D)),  # HANGUL SYLLABLE CIEUC E PIEUP
    ((0xC1, 0xAC), (0xC82F)),  # HANGUL SYLLABLE CIEUC E SIOS
    ((0xC1, 0xAD), (0xC831)),  # HANGUL SYLLABLE CIEUC E IEUNG
    ((0xC1, 0xAE), (0xC838)),  # HANGUL SYLLABLE CIEUC YEO
    ((0xC1, 0xAF), (0xC83C)),  # HANGUL SYLLABLE CIEUC YEO NIEUN
    ((0xC1, 0xB0), (0xC840)),  # HANGUL SYLLABLE CIEUC YEO RIEUL
    ((0xC1, 0xB1), (0xC848)),  # HANGUL SYLLABLE CIEUC YEO MIEUM
    ((0xC1, 0xB2), (0xC849)),  # HANGUL SYLLABLE CIEUC YEO PIEUP
    ((0xC1, 0xB3), (0xC84C)),  # HANGUL SYLLABLE CIEUC YEO SSANGSIOS
    ((0xC1, 0xB4), (0xC84D)),  # HANGUL SYLLABLE CIEUC YEO IEUNG
    ((0xC1, 0xB5), (0xC854)),  # HANGUL SYLLABLE CIEUC YE
    ((0xC1, 0xB6), (0xC870)),  # HANGUL SYLLABLE CIEUC O
    ((0xC1, 0xB7), (0xC871)),  # HANGUL SYLLABLE CIEUC O KIYEOK
    ((0xC1, 0xB8), (0xC874)),  # HANGUL SYLLABLE CIEUC O NIEUN
    ((0xC1, 0xB9), (0xC878)),  # HANGUL SYLLABLE CIEUC O RIEUL
    ((0xC1, 0xBA), (0xC87A)),  # HANGUL SYLLABLE CIEUC O RIEULMIEUM
    ((0xC1, 0xBB), (0xC880)),  # HANGUL SYLLABLE CIEUC O MIEUM
    ((0xC1, 0xBC), (0xC881)),  # HANGUL SYLLABLE CIEUC O PIEUP
    ((0xC1, 0xBD), (0xC883)),  # HANGUL SYLLABLE CIEUC O SIOS
    ((0xC1, 0xBE), (0xC885)),  # HANGUL SYLLABLE CIEUC O IEUNG
    ((0xC1, 0xBF), (0xC886)),  # HANGUL SYLLABLE CIEUC O CIEUC
    ((0xC1, 0xC0), (0xC887)),  # HANGUL SYLLABLE CIEUC O CHIEUCH
    ((0xC1, 0xC1), (0xC88B)),  # HANGUL SYLLABLE CIEUC O HIEUH
    ((0xC1, 0xC2), (0xC88C)),  # HANGUL SYLLABLE CIEUC WA
    ((0xC1, 0xC3), (0xC88D)),  # HANGUL SYLLABLE CIEUC WA KIYEOK
    ((0xC1, 0xC4), (0xC894)),  # HANGUL SYLLABLE CIEUC WA RIEUL
    ((0xC1, 0xC5), (0xC89D)),  # HANGUL SYLLABLE CIEUC WA PIEUP
    ((0xC1, 0xC6), (0xC89F)),  # HANGUL SYLLABLE CIEUC WA SIOS
    ((0xC1, 0xC7), (0xC8A1)),  # HANGUL SYLLABLE CIEUC WA IEUNG
    ((0xC1, 0xC8), (0xC8A8)),  # HANGUL SYLLABLE CIEUC WAE
    ((0xC1, 0xC9), (0xC8BC)),  # HANGUL SYLLABLE CIEUC WAE SSANGSIOS
    ((0xC1, 0xCA), (0xC8BD)),  # HANGUL SYLLABLE CIEUC WAE IEUNG
    ((0xC1, 0xCB), (0xC8C4)),  # HANGUL SYLLABLE CIEUC OE
    ((0xC1, 0xCC), (0xC8C8)),  # HANGUL SYLLABLE CIEUC OE NIEUN
    ((0xC1, 0xCD), (0xC8CC)),  # HANGUL SYLLABLE CIEUC OE RIEUL
    ((0xC1, 0xCE), (0xC8D4)),  # HANGUL SYLLABLE CIEUC OE MIEUM
    ((0xC1, 0xCF), (0xC8D5)),  # HANGUL SYLLABLE CIEUC OE PIEUP
    ((0xC1, 0xD0), (0xC8D7)),  # HANGUL SYLLABLE CIEUC OE SIOS
    ((0xC1, 0xD1), (0xC8D9)),  # HANGUL SYLLABLE CIEUC OE IEUNG
    ((0xC1, 0xD2), (0xC8E0)),  # HANGUL SYLLABLE CIEUC YO
    ((0xC1, 0xD3), (0xC8E1)),  # HANGUL SYLLABLE CIEUC YO KIYEOK
    ((0xC1, 0xD4), (0xC8E4)),  # HANGUL SYLLABLE CIEUC YO NIEUN
    ((0xC1, 0xD5), (0xC8F5)),  # HANGUL SYLLABLE CIEUC YO IEUNG
    ((0xC1, 0xD6), (0xC8FC)),  # HANGUL SYLLABLE CIEUC U
    ((0xC1, 0xD7), (0xC8FD)),  # HANGUL SYLLABLE CIEUC U KIYEOK
    ((0xC1, 0xD8), (0xC900)),  # HANGUL SYLLABLE CIEUC U NIEUN
    ((0xC1, 0xD9), (0xC904)),  # HANGUL SYLLABLE CIEUC U RIEUL
    ((0xC1, 0xDA), (0xC905)),  # HANGUL SYLLABLE CIEUC U RIEULKIYEOK
    ((0xC1, 0xDB), (0xC906)),  # HANGUL SYLLABLE CIEUC U RIEULMIEUM
    ((0xC1, 0xDC), (0xC90C)),  # HANGUL SYLLABLE CIEUC U MIEUM
    ((0xC1, 0xDD), (0xC90D)),  # HANGUL SYLLABLE CIEUC U PIEUP
    ((0xC1, 0xDE), (0xC90F)),  # HANGUL SYLLABLE CIEUC U SIOS
    ((0xC1, 0xDF), (0xC911)),  # HANGUL SYLLABLE CIEUC U IEUNG
    ((0xC1, 0xE0), (0xC918)),  # HANGUL SYLLABLE CIEUC WEO
    ((0xC1, 0xE1), (0xC92C)),  # HANGUL SYLLABLE CIEUC WEO SSANGSIOS
    ((0xC1, 0xE2), (0xC934)),  # HANGUL SYLLABLE CIEUC WE
    ((0xC1, 0xE3), (0xC950)),  # HANGUL SYLLABLE CIEUC WI
    ((0xC1, 0xE4), (0xC951)),  # HANGUL SYLLABLE CIEUC WI KIYEOK
    ((0xC1, 0xE5), (0xC954)),  # HANGUL SYLLABLE CIEUC WI NIEUN
    ((0xC1, 0xE6), (0xC958)),  # HANGUL SYLLABLE CIEUC WI RIEUL
    ((0xC1, 0xE7), (0xC960)),  # HANGUL SYLLABLE CIEUC WI MIEUM
    ((0xC1, 0xE8), (0xC961)),  # HANGUL SYLLABLE CIEUC WI PIEUP
    ((0xC1, 0xE9), (0xC963)),  # HANGUL SYLLABLE CIEUC WI SIOS
    ((0xC1, 0xEA), (0xC96C)),  # HANGUL SYLLABLE CIEUC YU
    ((0xC1, 0xEB), (0xC970)),  # HANGUL SYLLABLE CIEUC YU NIEUN
    ((0xC1, 0xEC), (0xC974)),  # HANGUL SYLLABLE CIEUC YU RIEUL
    ((0xC1, 0xED), (0xC97C)),  # HANGUL SYLLABLE CIEUC YU MIEUM
    ((0xC1, 0xEE), (0xC988)),  # HANGUL SYLLABLE CIEUC EU
    ((0xC1, 0xEF), (0xC989)),  # HANGUL SYLLABLE CIEUC EU KIYEOK
    ((0xC1, 0xF0), (0xC98C)),  # HANGUL SYLLABLE CIEUC EU NIEUN
    ((0xC1, 0xF1), (0xC990)),  # HANGUL SYLLABLE CIEUC EU RIEUL
    ((0xC1, 0xF2), (0xC998)),  # HANGUL SYLLABLE CIEUC EU MIEUM
    ((0xC1, 0xF3), (0xC999)),  # HANGUL SYLLABLE CIEUC EU PIEUP
    ((0xC1, 0xF4), (0xC99B)),  # HANGUL SYLLABLE CIEUC EU SIOS
    ((0xC1, 0xF5), (0xC99D)),  # HANGUL SYLLABLE CIEUC EU IEUNG
    ((0xC1, 0xF6), (0xC9C0)),  # HANGUL SYLLABLE CIEUC I
    ((0xC1, 0xF7), (0xC9C1)),  # HANGUL SYLLABLE CIEUC I KIYEOK
    ((0xC1, 0xF8), (0xC9C4)),  # HANGUL SYLLABLE CIEUC I NIEUN
    ((0xC1, 0xF9), (0xC9C7)),  # HANGUL SYLLABLE CIEUC I TIKEUT
    ((0xC1, 0xFA), (0xC9C8)),  # HANGUL SYLLABLE CIEUC I RIEUL
    ((0xC1, 0xFB), (0xC9CA)),  # HANGUL SYLLABLE CIEUC I RIEULMIEUM
    ((0xC1, 0xFC), (0xC9D0)),  # HANGUL SYLLABLE CIEUC I MIEUM
    ((0xC1, 0xFD), (0xC9D1)),  # HANGUL SYLLABLE CIEUC I PIEUP
    ((0xC1, 0xFE), (0xC9D3)),  # HANGUL SYLLABLE CIEUC I SIOS
    ((0xC2, 0x41), (0xD5CA)),  # HANGUL SYLLABLE HIEUH EO SSANGKIYEOK
    ((0xC2, 0x42), (0xD5CB)),  # HANGUL SYLLABLE HIEUH EO KIYEOKSIOS
    ((0xC2, 0x43), (0xD5CD)),  # HANGUL SYLLABLE HIEUH EO NIEUNCIEUC
    ((0xC2, 0x44), (0xD5CE)),  # HANGUL SYLLABLE HIEUH EO NIEUNHIEUH
    ((0xC2, 0x45), (0xD5CF)),  # HANGUL SYLLABLE HIEUH EO TIKEUT
    ((0xC2, 0x46), (0xD5D1)),  # HANGUL SYLLABLE HIEUH EO RIEULKIYEOK
    ((0xC2, 0x47), (0xD5D3)),  # HANGUL SYLLABLE HIEUH EO RIEULPIEUP
    ((0xC2, 0x48), (0xD5D4)),  # HANGUL SYLLABLE HIEUH EO RIEULSIOS
    ((0xC2, 0x49), (0xD5D5)),  # HANGUL SYLLABLE HIEUH EO RIEULTHIEUTH
    ((0xC2, 0x4A), (0xD5D6)),  # HANGUL SYLLABLE HIEUH EO RIEULPHIEUPH
    ((0xC2, 0x4B), (0xD5D7)),  # HANGUL SYLLABLE HIEUH EO RIEULHIEUH
    ((0xC2, 0x4C), (0xD5DA)),  # HANGUL SYLLABLE HIEUH EO PIEUPSIOS
    ((0xC2, 0x4D), (0xD5DC)),  # HANGUL SYLLABLE HIEUH EO SSANGSIOS
    ((0xC2, 0x4E), (0xD5DE)),  # HANGUL SYLLABLE HIEUH EO CIEUC
    ((0xC2, 0x4F), (0xD5DF)),  # HANGUL SYLLABLE HIEUH EO CHIEUCH
    ((0xC2, 0x50), (0xD5E0)),  # HANGUL SYLLABLE HIEUH EO KHIEUKH
    ((0xC2, 0x51), (0xD5E1)),  # HANGUL SYLLABLE HIEUH EO THIEUTH
    ((0xC2, 0x52), (0xD5E2)),  # HANGUL SYLLABLE HIEUH EO PHIEUPH
    ((0xC2, 0x53), (0xD5E3)),  # HANGUL SYLLABLE HIEUH EO HIEUH
    ((0xC2, 0x54), (0xD5E6)),  # HANGUL SYLLABLE HIEUH E SSANGKIYEOK
    ((0xC2, 0x55), (0xD5E7)),  # HANGUL SYLLABLE HIEUH E KIYEOKSIOS
    ((0xC2, 0x56), (0xD5E9)),  # HANGUL SYLLABLE HIEUH E NIEUNCIEUC
    ((0xC2, 0x57), (0xD5EA)),  # HANGUL SYLLABLE HIEUH E NIEUNHIEUH
    ((0xC2, 0x58), (0xD5EB)),  # HANGUL SYLLABLE HIEUH E TIKEUT
    ((0xC2, 0x59), (0xD5ED)),  # HANGUL SYLLABLE HIEUH E RIEULKIYEOK
    ((0xC2, 0x5A), (0xD5EE)),  # HANGUL SYLLABLE HIEUH E RIEULMIEUM
    ((0xC2, 0x61), (0xD5EF)),  # HANGUL SYLLABLE HIEUH E RIEULPIEUP
    ((0xC2, 0x62), (0xD5F0)),  # HANGUL SYLLABLE HIEUH E RIEULSIOS
    ((0xC2, 0x63), (0xD5F1)),  # HANGUL SYLLABLE HIEUH E RIEULTHIEUTH
    ((0xC2, 0x64), (0xD5F2)),  # HANGUL SYLLABLE HIEUH E RIEULPHIEUPH
    ((0xC2, 0x65), (0xD5F3)),  # HANGUL SYLLABLE HIEUH E RIEULHIEUH
    ((0xC2, 0x66), (0xD5F6)),  # HANGUL SYLLABLE HIEUH E PIEUPSIOS
    ((0xC2, 0x67), (0xD5F8)),  # HANGUL SYLLABLE HIEUH E SSANGSIOS
    ((0xC2, 0x68), (0xD5FA)),  # HANGUL SYLLABLE HIEUH E CIEUC
    ((0xC2, 0x69), (0xD5FB)),  # HANGUL SYLLABLE HIEUH E CHIEUCH
    ((0xC2, 0x6A), (0xD5FC)),  # HANGUL SYLLABLE HIEUH E KHIEUKH
    ((0xC2, 0x6B), (0xD5FD)),  # HANGUL SYLLABLE HIEUH E THIEUTH
    ((0xC2, 0x6C), (0xD5FE)),  # HANGUL SYLLABLE HIEUH E PHIEUPH
    ((0xC2, 0x6D), (0xD5FF)),  # HANGUL SYLLABLE HIEUH E HIEUH
    ((0xC2, 0x6E), (0xD602)),  # HANGUL SYLLABLE HIEUH YEO SSANGKIYEOK
    ((0xC2, 0x6F), (0xD603)),  # HANGUL SYLLABLE HIEUH YEO KIYEOKSIOS
    ((0xC2, 0x70), (0xD605)),  # HANGUL SYLLABLE HIEUH YEO NIEUNCIEUC
    ((0xC2, 0x71), (0xD606)),  # HANGUL SYLLABLE HIEUH YEO NIEUNHIEUH
    ((0xC2, 0x72), (0xD607)),  # HANGUL SYLLABLE HIEUH YEO TIKEUT
    ((0xC2, 0x73), (0xD609)),  # HANGUL SYLLABLE HIEUH YEO RIEULKIYEOK
    ((0xC2, 0x74), (0xD60A)),  # HANGUL SYLLABLE HIEUH YEO RIEULMIEUM
    ((0xC2, 0x75), (0xD60B)),  # HANGUL SYLLABLE HIEUH YEO RIEULPIEUP
    ((0xC2, 0x76), (0xD60C)),  # HANGUL SYLLABLE HIEUH YEO RIEULSIOS
    ((0xC2, 0x77), (0xD60D)),  # HANGUL SYLLABLE HIEUH YEO RIEULTHIEUTH
    ((0xC2, 0x78), (0xD60E)),  # HANGUL SYLLABLE HIEUH YEO RIEULPHIEUPH
    ((0xC2, 0x79), (0xD60F)),  # HANGUL SYLLABLE HIEUH YEO RIEULHIEUH
    ((0xC2, 0x7A), (0xD612)),  # HANGUL SYLLABLE HIEUH YEO PIEUPSIOS
    ((0xC2, 0x81), (0xD616)),  # HANGUL SYLLABLE HIEUH YEO CIEUC
    ((0xC2, 0x82), (0xD617)),  # HANGUL SYLLABLE HIEUH YEO CHIEUCH
    ((0xC2, 0x83), (0xD618)),  # HANGUL SYLLABLE HIEUH YEO KHIEUKH
    ((0xC2, 0x84), (0xD619)),  # HANGUL SYLLABLE HIEUH YEO THIEUTH
    ((0xC2, 0x85), (0xD61A)),  # HANGUL SYLLABLE HIEUH YEO PHIEUPH
    ((0xC2, 0x86), (0xD61B)),  # HANGUL SYLLABLE HIEUH YEO HIEUH
    ((0xC2, 0x87), (0xD61D)),  # HANGUL SYLLABLE HIEUH YE KIYEOK
    ((0xC2, 0x88), (0xD61E)),  # HANGUL SYLLABLE HIEUH YE SSANGKIYEOK
    ((0xC2, 0x89), (0xD61F)),  # HANGUL SYLLABLE HIEUH YE KIYEOKSIOS
    ((0xC2, 0x8A), (0xD621)),  # HANGUL SYLLABLE HIEUH YE NIEUNCIEUC
    ((0xC2, 0x8B), (0xD622)),  # HANGUL SYLLABLE HIEUH YE NIEUNHIEUH
    ((0xC2, 0x8C), (0xD623)),  # HANGUL SYLLABLE HIEUH YE TIKEUT
    ((0xC2, 0x8D), (0xD625)),  # HANGUL SYLLABLE HIEUH YE RIEULKIYEOK
    ((0xC2, 0x8E), (0xD626)),  # HANGUL SYLLABLE HIEUH YE RIEULMIEUM
    ((0xC2, 0x8F), (0xD627)),  # HANGUL SYLLABLE HIEUH YE RIEULPIEUP
    ((0xC2, 0x90), (0xD628)),  # HANGUL SYLLABLE HIEUH YE RIEULSIOS
    ((0xC2, 0x91), (0xD629)),  # HANGUL SYLLABLE HIEUH YE RIEULTHIEUTH
    ((0xC2, 0x92), (0xD62A)),  # HANGUL SYLLABLE HIEUH YE RIEULPHIEUPH
    ((0xC2, 0x93), (0xD62B)),  # HANGUL SYLLABLE HIEUH YE RIEULHIEUH
    ((0xC2, 0x94), (0xD62C)),  # HANGUL SYLLABLE HIEUH YE MIEUM
    ((0xC2, 0x95), (0xD62E)),  # HANGUL SYLLABLE HIEUH YE PIEUPSIOS
    ((0xC2, 0x96), (0xD62F)),  # HANGUL SYLLABLE HIEUH YE SIOS
    ((0xC2, 0x97), (0xD630)),  # HANGUL SYLLABLE HIEUH YE SSANGSIOS
    ((0xC2, 0x98), (0xD631)),  # HANGUL SYLLABLE HIEUH YE IEUNG
    ((0xC2, 0x99), (0xD632)),  # HANGUL SYLLABLE HIEUH YE CIEUC
    ((0xC2, 0x9A), (0xD633)),  # HANGUL SYLLABLE HIEUH YE CHIEUCH
    ((0xC2, 0x9B), (0xD634)),  # HANGUL SYLLABLE HIEUH YE KHIEUKH
    ((0xC2, 0x9C), (0xD635)),  # HANGUL SYLLABLE HIEUH YE THIEUTH
    ((0xC2, 0x9D), (0xD636)),  # HANGUL SYLLABLE HIEUH YE PHIEUPH
    ((0xC2, 0x9E), (0xD637)),  # HANGUL SYLLABLE HIEUH YE HIEUH
    ((0xC2, 0x9F), (0xD63A)),  # HANGUL SYLLABLE HIEUH O SSANGKIYEOK
    ((0xC2, 0xA0), (0xD63B)),  # HANGUL SYLLABLE HIEUH O KIYEOKSIOS
    ((0xC2, 0xA1), (0xC9D5)),  # HANGUL SYLLABLE CIEUC I IEUNG
    ((0xC2, 0xA2), (0xC9D6)),  # HANGUL SYLLABLE CIEUC I CIEUC
    ((0xC2, 0xA3), (0xC9D9)),  # HANGUL SYLLABLE CIEUC I THIEUTH
    ((0xC2, 0xA4), (0xC9DA)),  # HANGUL SYLLABLE CIEUC I PHIEUPH
    ((0xC2, 0xA5), (0xC9DC)),  # HANGUL SYLLABLE SSANGCIEUC A
    ((0xC2, 0xA6), (0xC9DD)),  # HANGUL SYLLABLE SSANGCIEUC A KIYEOK
    ((0xC2, 0xA7), (0xC9E0)),  # HANGUL SYLLABLE SSANGCIEUC A NIEUN
    ((0xC2, 0xA8), (0xC9E2)),  # HANGUL SYLLABLE SSANGCIEUC A NIEUNHIEUH
    ((0xC2, 0xA9), (0xC9E4)),  # HANGUL SYLLABLE SSANGCIEUC A RIEUL
    ((0xC2, 0xAA), (0xC9E7)),  # HANGUL SYLLABLE SSANGCIEUC A RIEULPIEUP
    ((0xC2, 0xAB), (0xC9EC)),  # HANGUL SYLLABLE SSANGCIEUC A MIEUM
    ((0xC2, 0xAC), (0xC9ED)),  # HANGUL SYLLABLE SSANGCIEUC A PIEUP
    ((0xC2, 0xAD), (0xC9EF)),  # HANGUL SYLLABLE SSANGCIEUC A SIOS
    ((0xC2, 0xAE), (0xC9F0)),  # HANGUL SYLLABLE SSANGCIEUC A SSANGSIOS
    ((0xC2, 0xAF), (0xC9F1)),  # HANGUL SYLLABLE SSANGCIEUC A IEUNG
    ((0xC2, 0xB0), (0xC9F8)),  # HANGUL SYLLABLE SSANGCIEUC AE
    ((0xC2, 0xB1), (0xC9F9)),  # HANGUL SYLLABLE SSANGCIEUC AE KIYEOK
    ((0xC2, 0xB2), (0xC9FC)),  # HANGUL SYLLABLE SSANGCIEUC AE NIEUN
    ((0xC2, 0xB3), (0xCA00)),  # HANGUL SYLLABLE SSANGCIEUC AE RIEUL
    ((0xC2, 0xB4), (0xCA08)),  # HANGUL SYLLABLE SSANGCIEUC AE MIEUM
    ((0xC2, 0xB5), (0xCA09)),  # HANGUL SYLLABLE SSANGCIEUC AE PIEUP
    ((0xC2, 0xB6), (0xCA0B)),  # HANGUL SYLLABLE SSANGCIEUC AE SIOS
    ((0xC2, 0xB7), (0xCA0C)),  # HANGUL SYLLABLE SSANGCIEUC AE SSANGSIOS
    ((0xC2, 0xB8), (0xCA0D)),  # HANGUL SYLLABLE SSANGCIEUC AE IEUNG
    ((0xC2, 0xB9), (0xCA14)),  # HANGUL SYLLABLE SSANGCIEUC YA
    ((0xC2, 0xBA), (0xCA18)),  # HANGUL SYLLABLE SSANGCIEUC YA NIEUN
    ((0xC2, 0xBB), (0xCA29)),  # HANGUL SYLLABLE SSANGCIEUC YA IEUNG
    ((0xC2, 0xBC), (0xCA4C)),  # HANGUL SYLLABLE SSANGCIEUC EO
    ((0xC2, 0xBD), (0xCA4D)),  # HANGUL SYLLABLE SSANGCIEUC EO KIYEOK
    ((0xC2, 0xBE), (0xCA50)),  # HANGUL SYLLABLE SSANGCIEUC EO NIEUN
    ((0xC2, 0xBF), (0xCA54)),  # HANGUL SYLLABLE SSANGCIEUC EO RIEUL
    ((0xC2, 0xC0), (0xCA5C)),  # HANGUL SYLLABLE SSANGCIEUC EO MIEUM
    ((0xC2, 0xC1), (0xCA5D)),  # HANGUL SYLLABLE SSANGCIEUC EO PIEUP
    ((0xC2, 0xC2), (0xCA5F)),  # HANGUL SYLLABLE SSANGCIEUC EO SIOS
    ((0xC2, 0xC3), (0xCA60)),  # HANGUL SYLLABLE SSANGCIEUC EO SSANGSIOS
    ((0xC2, 0xC4), (0xCA61)),  # HANGUL SYLLABLE SSANGCIEUC EO IEUNG
    ((0xC2, 0xC5), (0xCA68)),  # HANGUL SYLLABLE SSANGCIEUC E
    ((0xC2, 0xC6), (0xCA7D)),  # HANGUL SYLLABLE SSANGCIEUC E IEUNG
    ((0xC2, 0xC7), (0xCA84)),  # HANGUL SYLLABLE SSANGCIEUC YEO
    ((0xC2, 0xC8), (0xCA98)),  # HANGUL SYLLABLE SSANGCIEUC YEO SSANGSIOS
    ((0xC2, 0xC9), (0xCABC)),  # HANGUL SYLLABLE SSANGCIEUC O
    ((0xC2, 0xCA), (0xCABD)),  # HANGUL SYLLABLE SSANGCIEUC O KIYEOK
    ((0xC2, 0xCB), (0xCAC0)),  # HANGUL SYLLABLE SSANGCIEUC O NIEUN
    ((0xC2, 0xCC), (0xCAC4)),  # HANGUL SYLLABLE SSANGCIEUC O RIEUL
    ((0xC2, 0xCD), (0xCACC)),  # HANGUL SYLLABLE SSANGCIEUC O MIEUM
    ((0xC2, 0xCE), (0xCACD)),  # HANGUL SYLLABLE SSANGCIEUC O PIEUP
    ((0xC2, 0xCF), (0xCACF)),  # HANGUL SYLLABLE SSANGCIEUC O SIOS
    ((0xC2, 0xD0), (0xCAD1)),  # HANGUL SYLLABLE SSANGCIEUC O IEUNG
    ((0xC2, 0xD1), (0xCAD3)),  # HANGUL SYLLABLE SSANGCIEUC O CHIEUCH
    ((0xC2, 0xD2), (0xCAD8)),  # HANGUL SYLLABLE SSANGCIEUC WA
    ((0xC2, 0xD3), (0xCAD9)),  # HANGUL SYLLABLE SSANGCIEUC WA KIYEOK
    ((0xC2, 0xD4), (0xCAE0)),  # HANGUL SYLLABLE SSANGCIEUC WA RIEUL
    ((0xC2, 0xD5), (0xCAEC)),  # HANGUL SYLLABLE SSANGCIEUC WA SSANGSIOS
    ((0xC2, 0xD6), (0xCAF4)),  # HANGUL SYLLABLE SSANGCIEUC WAE
    ((0xC2, 0xD7), (0xCB08)),  # HANGUL SYLLABLE SSANGCIEUC WAE SSANGSIOS
    ((0xC2, 0xD8), (0xCB10)),  # HANGUL SYLLABLE SSANGCIEUC OE
    ((0xC2, 0xD9), (0xCB14)),  # HANGUL SYLLABLE SSANGCIEUC OE NIEUN
    ((0xC2, 0xDA), (0xCB18)),  # HANGUL SYLLABLE SSANGCIEUC OE RIEUL
    ((0xC2, 0xDB), (0xCB20)),  # HANGUL SYLLABLE SSANGCIEUC OE MIEUM
    ((0xC2, 0xDC), (0xCB21)),  # HANGUL SYLLABLE SSANGCIEUC OE PIEUP
    ((0xC2, 0xDD), (0xCB41)),  # HANGUL SYLLABLE SSANGCIEUC YO IEUNG
    ((0xC2, 0xDE), (0xCB48)),  # HANGUL SYLLABLE SSANGCIEUC U
    ((0xC2, 0xDF), (0xCB49)),  # HANGUL SYLLABLE SSANGCIEUC U KIYEOK
    ((0xC2, 0xE0), (0xCB4C)),  # HANGUL SYLLABLE SSANGCIEUC U NIEUN
    ((0xC2, 0xE1), (0xCB50)),  # HANGUL SYLLABLE SSANGCIEUC U RIEUL
    ((0xC2, 0xE2), (0xCB58)),  # HANGUL SYLLABLE SSANGCIEUC U MIEUM
    ((0xC2, 0xE3), (0xCB59)),  # HANGUL SYLLABLE SSANGCIEUC U PIEUP
    ((0xC2, 0xE4), (0xCB5D)),  # HANGUL SYLLABLE SSANGCIEUC U IEUNG
    ((0xC2, 0xE5), (0xCB64)),  # HANGUL SYLLABLE SSANGCIEUC WEO
    ((0xC2, 0xE6), (0xCB78)),  # HANGUL SYLLABLE SSANGCIEUC WEO SSANGSIOS
    ((0xC2, 0xE7), (0xCB79)),  # HANGUL SYLLABLE SSANGCIEUC WEO IEUNG
    ((0xC2, 0xE8), (0xCB9C)),  # HANGUL SYLLABLE SSANGCIEUC WI
    ((0xC2, 0xE9), (0xCBB8)),  # HANGUL SYLLABLE SSANGCIEUC YU
    ((0xC2, 0xEA), (0xCBD4)),  # HANGUL SYLLABLE SSANGCIEUC EU
    ((0xC2, 0xEB), (0xCBE4)),  # HANGUL SYLLABLE SSANGCIEUC EU MIEUM
    ((0xC2, 0xEC), (0xCBE7)),  # HANGUL SYLLABLE SSANGCIEUC EU SIOS
    ((0xC2, 0xED), (0xCBE9)),  # HANGUL SYLLABLE SSANGCIEUC EU IEUNG
    ((0xC2, 0xEE), (0xCC0C)),  # HANGUL SYLLABLE SSANGCIEUC I
    ((0xC2, 0xEF), (0xCC0D)),  # HANGUL SYLLABLE SSANGCIEUC I KIYEOK
    ((0xC2, 0xF0), (0xCC10)),  # HANGUL SYLLABLE SSANGCIEUC I NIEUN
    ((0xC2, 0xF1), (0xCC14)),  # HANGUL SYLLABLE SSANGCIEUC I RIEUL
    ((0xC2, 0xF2), (0xCC1C)),  # HANGUL SYLLABLE SSANGCIEUC I MIEUM
    ((0xC2, 0xF3), (0xCC1D)),  # HANGUL SYLLABLE SSANGCIEUC I PIEUP
    ((0xC2, 0xF4), (0xCC21)),  # HANGUL SYLLABLE SSANGCIEUC I IEUNG
    ((0xC2, 0xF5), (0xCC22)),  # HANGUL SYLLABLE SSANGCIEUC I CIEUC
    ((0xC2, 0xF6), (0xCC27)),  # HANGUL SYLLABLE SSANGCIEUC I HIEUH
    ((0xC2, 0xF7), (0xCC28)),  # HANGUL SYLLABLE CHIEUCH A
    ((0xC2, 0xF8), (0xCC29)),  # HANGUL SYLLABLE CHIEUCH A KIYEOK
    ((0xC2, 0xF9), (0xCC2C)),  # HANGUL SYLLABLE CHIEUCH A NIEUN
    ((0xC2, 0xFA), (0xCC2E)),  # HANGUL SYLLABLE CHIEUCH A NIEUNHIEUH
    ((0xC2, 0xFB), (0xCC30)),  # HANGUL SYLLABLE CHIEUCH A RIEUL
    ((0xC2, 0xFC), (0xCC38)),  # HANGUL SYLLABLE CHIEUCH A MIEUM
    ((0xC2, 0xFD), (0xCC39)),  # HANGUL SYLLABLE CHIEUCH A PIEUP
    ((0xC2, 0xFE), (0xCC3B)),  # HANGUL SYLLABLE CHIEUCH A SIOS
    ((0xC3, 0x41), (0xD63D)),  # HANGUL SYLLABLE HIEUH O NIEUNCIEUC
    ((0xC3, 0x42), (0xD63E)),  # HANGUL SYLLABLE HIEUH O NIEUNHIEUH
    ((0xC3, 0x43), (0xD63F)),  # HANGUL SYLLABLE HIEUH O TIKEUT
    ((0xC3, 0x44), (0xD641)),  # HANGUL SYLLABLE HIEUH O RIEULKIYEOK
    ((0xC3, 0x45), (0xD642)),  # HANGUL SYLLABLE HIEUH O RIEULMIEUM
    ((0xC3, 0x46), (0xD643)),  # HANGUL SYLLABLE HIEUH O RIEULPIEUP
    ((0xC3, 0x47), (0xD644)),  # HANGUL SYLLABLE HIEUH O RIEULSIOS
    ((0xC3, 0x48), (0xD646)),  # HANGUL SYLLABLE HIEUH O RIEULPHIEUPH
    ((0xC3, 0x49), (0xD647)),  # HANGUL SYLLABLE HIEUH O RIEULHIEUH
    ((0xC3, 0x4A), (0xD64A)),  # HANGUL SYLLABLE HIEUH O PIEUPSIOS
    ((0xC3, 0x4B), (0xD64C)),  # HANGUL SYLLABLE HIEUH O SSANGSIOS
    ((0xC3, 0x4C), (0xD64E)),  # HANGUL SYLLABLE HIEUH O CIEUC
    ((0xC3, 0x4D), (0xD64F)),  # HANGUL SYLLABLE HIEUH O CHIEUCH
    ((0xC3, 0x4E), (0xD650)),  # HANGUL SYLLABLE HIEUH O KHIEUKH
    ((0xC3, 0x4F), (0xD652)),  # HANGUL SYLLABLE HIEUH O PHIEUPH
    ((0xC3, 0x50), (0xD653)),  # HANGUL SYLLABLE HIEUH O HIEUH
    ((0xC3, 0x51), (0xD656)),  # HANGUL SYLLABLE HIEUH WA SSANGKIYEOK
    ((0xC3, 0x52), (0xD657)),  # HANGUL SYLLABLE HIEUH WA KIYEOKSIOS
    ((0xC3, 0x53), (0xD659)),  # HANGUL SYLLABLE HIEUH WA NIEUNCIEUC
    ((0xC3, 0x54), (0xD65A)),  # HANGUL SYLLABLE HIEUH WA NIEUNHIEUH
    ((0xC3, 0x55), (0xD65B)),  # HANGUL SYLLABLE HIEUH WA TIKEUT
    ((0xC3, 0x56), (0xD65D)),  # HANGUL SYLLABLE HIEUH WA RIEULKIYEOK
    ((0xC3, 0x57), (0xD65E)),  # HANGUL SYLLABLE HIEUH WA RIEULMIEUM
    ((0xC3, 0x58), (0xD65F)),  # HANGUL SYLLABLE HIEUH WA RIEULPIEUP
    ((0xC3, 0x59), (0xD660)),  # HANGUL SYLLABLE HIEUH WA RIEULSIOS
    ((0xC3, 0x5A), (0xD661)),  # HANGUL SYLLABLE HIEUH WA RIEULTHIEUTH
    ((0xC3, 0x61), (0xD662)),  # HANGUL SYLLABLE HIEUH WA RIEULPHIEUPH
    ((0xC3, 0x62), (0xD663)),  # HANGUL SYLLABLE HIEUH WA RIEULHIEUH
    ((0xC3, 0x63), (0xD664)),  # HANGUL SYLLABLE HIEUH WA MIEUM
    ((0xC3, 0x64), (0xD665)),  # HANGUL SYLLABLE HIEUH WA PIEUP
    ((0xC3, 0x65), (0xD666)),  # HANGUL SYLLABLE HIEUH WA PIEUPSIOS
    ((0xC3, 0x66), (0xD668)),  # HANGUL SYLLABLE HIEUH WA SSANGSIOS
    ((0xC3, 0x67), (0xD66A)),  # HANGUL SYLLABLE HIEUH WA CIEUC
    ((0xC3, 0x68), (0xD66B)),  # HANGUL SYLLABLE HIEUH WA CHIEUCH
    ((0xC3, 0x69), (0xD66C)),  # HANGUL SYLLABLE HIEUH WA KHIEUKH
    ((0xC3, 0x6A), (0xD66D)),  # HANGUL SYLLABLE HIEUH WA THIEUTH
    ((0xC3, 0x6B), (0xD66E)),  # HANGUL SYLLABLE HIEUH WA PHIEUPH
    ((0xC3, 0x6C), (0xD66F)),  # HANGUL SYLLABLE HIEUH WA HIEUH
    ((0xC3, 0x6D), (0xD672)),  # HANGUL SYLLABLE HIEUH WAE SSANGKIYEOK
    ((0xC3, 0x6E), (0xD673)),  # HANGUL SYLLABLE HIEUH WAE KIYEOKSIOS
    ((0xC3, 0x6F), (0xD675)),  # HANGUL SYLLABLE HIEUH WAE NIEUNCIEUC
    ((0xC3, 0x70), (0xD676)),  # HANGUL SYLLABLE HIEUH WAE NIEUNHIEUH
    ((0xC3, 0x71), (0xD677)),  # HANGUL SYLLABLE HIEUH WAE TIKEUT
    ((0xC3, 0x72), (0xD678)),  # HANGUL SYLLABLE HIEUH WAE RIEUL
    ((0xC3, 0x73), (0xD679)),  # HANGUL SYLLABLE HIEUH WAE RIEULKIYEOK
    ((0xC3, 0x74), (0xD67A)),  # HANGUL SYLLABLE HIEUH WAE RIEULMIEUM
    ((0xC3, 0x75), (0xD67B)),  # HANGUL SYLLABLE HIEUH WAE RIEULPIEUP
    ((0xC3, 0x76), (0xD67C)),  # HANGUL SYLLABLE HIEUH WAE RIEULSIOS
    ((0xC3, 0x77), (0xD67D)),  # HANGUL SYLLABLE HIEUH WAE RIEULTHIEUTH
    ((0xC3, 0x78), (0xD67E)),  # HANGUL SYLLABLE HIEUH WAE RIEULPHIEUPH
    ((0xC3, 0x79), (0xD67F)),  # HANGUL SYLLABLE HIEUH WAE RIEULHIEUH
    ((0xC3, 0x7A), (0xD680)),  # HANGUL SYLLABLE HIEUH WAE MIEUM
    ((0xC3, 0x81), (0xD681)),  # HANGUL SYLLABLE HIEUH WAE PIEUP
    ((0xC3, 0x82), (0xD682)),  # HANGUL SYLLABLE HIEUH WAE PIEUPSIOS
    ((0xC3, 0x83), (0xD684)),  # HANGUL SYLLABLE HIEUH WAE SSANGSIOS
    ((0xC3, 0x84), (0xD686)),  # HANGUL SYLLABLE HIEUH WAE CIEUC
    ((0xC3, 0x85), (0xD687)),  # HANGUL SYLLABLE HIEUH WAE CHIEUCH
    ((0xC3, 0x86), (0xD688)),  # HANGUL SYLLABLE HIEUH WAE KHIEUKH
    ((0xC3, 0x87), (0xD689)),  # HANGUL SYLLABLE HIEUH WAE THIEUTH
    ((0xC3, 0x88), (0xD68A)),  # HANGUL SYLLABLE HIEUH WAE PHIEUPH
    ((0xC3, 0x89), (0xD68B)),  # HANGUL SYLLABLE HIEUH WAE HIEUH
    ((0xC3, 0x8A), (0xD68E)),  # HANGUL SYLLABLE HIEUH OE SSANGKIYEOK
    ((0xC3, 0x8B), (0xD68F)),  # HANGUL SYLLABLE HIEUH OE KIYEOKSIOS
    ((0xC3, 0x8C), (0xD691)),  # HANGUL SYLLABLE HIEUH OE NIEUNCIEUC
    ((0xC3, 0x8D), (0xD692)),  # HANGUL SYLLABLE HIEUH OE NIEUNHIEUH
    ((0xC3, 0x8E), (0xD693)),  # HANGUL SYLLABLE HIEUH OE TIKEUT
    ((0xC3, 0x8F), (0xD695)),  # HANGUL SYLLABLE HIEUH OE RIEULKIYEOK
    ((0xC3, 0x90), (0xD696)),  # HANGUL SYLLABLE HIEUH OE RIEULMIEUM
    ((0xC3, 0x91), (0xD697)),  # HANGUL SYLLABLE HIEUH OE RIEULPIEUP
    ((0xC3, 0x92), (0xD698)),  # HANGUL SYLLABLE HIEUH OE RIEULSIOS
    ((0xC3, 0x93), (0xD699)),  # HANGUL SYLLABLE HIEUH OE RIEULTHIEUTH
    ((0xC3, 0x94), (0xD69A)),  # HANGUL SYLLABLE HIEUH OE RIEULPHIEUPH
    ((0xC3, 0x95), (0xD69B)),  # HANGUL SYLLABLE HIEUH OE RIEULHIEUH
    ((0xC3, 0x96), (0xD69C)),  # HANGUL SYLLABLE HIEUH OE MIEUM
    ((0xC3, 0x97), (0xD69E)),  # HANGUL SYLLABLE HIEUH OE PIEUPSIOS
    ((0xC3, 0x98), (0xD6A0)),  # HANGUL SYLLABLE HIEUH OE SSANGSIOS
    ((0xC3, 0x99), (0xD6A2)),  # HANGUL SYLLABLE HIEUH OE CIEUC
    ((0xC3, 0x9A), (0xD6A3)),  # HANGUL SYLLABLE HIEUH OE CHIEUCH
    ((0xC3, 0x9B), (0xD6A4)),  # HANGUL SYLLABLE HIEUH OE KHIEUKH
    ((0xC3, 0x9C), (0xD6A5)),  # HANGUL SYLLABLE HIEUH OE THIEUTH
    ((0xC3, 0x9D), (0xD6A6)),  # HANGUL SYLLABLE HIEUH OE PHIEUPH
    ((0xC3, 0x9E), (0xD6A7)),  # HANGUL SYLLABLE HIEUH OE HIEUH
    ((0xC3, 0x9F), (0xD6A9)),  # HANGUL SYLLABLE HIEUH YO KIYEOK
    ((0xC3, 0xA0), (0xD6AA)),  # HANGUL SYLLABLE HIEUH YO SSANGKIYEOK
    ((0xC3, 0xA1), (0xCC3C)),  # HANGUL SYLLABLE CHIEUCH A SSANGSIOS
    ((0xC3, 0xA2), (0xCC3D)),  # HANGUL SYLLABLE CHIEUCH A IEUNG
    ((0xC3, 0xA3), (0xCC3E)),  # HANGUL SYLLABLE CHIEUCH A CIEUC
    ((0xC3, 0xA4), (0xCC44)),  # HANGUL SYLLABLE CHIEUCH AE
    ((0xC3, 0xA5), (0xCC45)),  # HANGUL SYLLABLE CHIEUCH AE KIYEOK
    ((0xC3, 0xA6), (0xCC48)),  # HANGUL SYLLABLE CHIEUCH AE NIEUN
    ((0xC3, 0xA7), (0xCC4C)),  # HANGUL SYLLABLE CHIEUCH AE RIEUL
    ((0xC3, 0xA8), (0xCC54)),  # HANGUL SYLLABLE CHIEUCH AE MIEUM
    ((0xC3, 0xA9), (0xCC55)),  # HANGUL SYLLABLE CHIEUCH AE PIEUP
    ((0xC3, 0xAA), (0xCC57)),  # HANGUL SYLLABLE CHIEUCH AE SIOS
    ((0xC3, 0xAB), (0xCC58)),  # HANGUL SYLLABLE CHIEUCH AE SSANGSIOS
    ((0xC3, 0xAC), (0xCC59)),  # HANGUL SYLLABLE CHIEUCH AE IEUNG
    ((0xC3, 0xAD), (0xCC60)),  # HANGUL SYLLABLE CHIEUCH YA
    ((0xC3, 0xAE), (0xCC64)),  # HANGUL SYLLABLE CHIEUCH YA NIEUN
    ((0xC3, 0xAF), (0xCC66)),  # HANGUL SYLLABLE CHIEUCH YA NIEUNHIEUH
    ((0xC3, 0xB0), (0xCC68)),  # HANGUL SYLLABLE CHIEUCH YA RIEUL
    ((0xC3, 0xB1), (0xCC70)),  # HANGUL SYLLABLE CHIEUCH YA MIEUM
    ((0xC3, 0xB2), (0xCC75)),  # HANGUL SYLLABLE CHIEUCH YA IEUNG
    ((0xC3, 0xB3), (0xCC98)),  # HANGUL SYLLABLE CHIEUCH EO
    ((0xC3, 0xB4), (0xCC99)),  # HANGUL SYLLABLE CHIEUCH EO KIYEOK
    ((0xC3, 0xB5), (0xCC9C)),  # HANGUL SYLLABLE CHIEUCH EO NIEUN
    ((0xC3, 0xB6), (0xCCA0)),  # HANGUL SYLLABLE CHIEUCH EO RIEUL
    ((0xC3, 0xB7), (0xCCA8)),  # HANGUL SYLLABLE CHIEUCH EO MIEUM
    ((0xC3, 0xB8), (0xCCA9)),  # HANGUL SYLLABLE CHIEUCH EO PIEUP
    ((0xC3, 0xB9), (0xCCAB)),  # HANGUL SYLLABLE CHIEUCH EO SIOS
    ((0xC3, 0xBA), (0xCCAC)),  # HANGUL SYLLABLE CHIEUCH EO SSANGSIOS
    ((0xC3, 0xBB), (0xCCAD)),  # HANGUL SYLLABLE CHIEUCH EO IEUNG
    ((0xC3, 0xBC), (0xCCB4)),  # HANGUL SYLLABLE CHIEUCH E
    ((0xC3, 0xBD), (0xCCB5)),  # HANGUL SYLLABLE CHIEUCH E KIYEOK
    ((0xC3, 0xBE), (0xCCB8)),  # HANGUL SYLLABLE CHIEUCH E NIEUN
    ((0xC3, 0xBF), (0xCCBC)),  # HANGUL SYLLABLE CHIEUCH E RIEUL
    ((0xC3, 0xC0), (0xCCC4)),  # HANGUL SYLLABLE CHIEUCH E MIEUM
    ((0xC3, 0xC1), (0xCCC5)),  # HANGUL SYLLABLE CHIEUCH E PIEUP
    ((0xC3, 0xC2), (0xCCC7)),  # HANGUL SYLLABLE CHIEUCH E SIOS
    ((0xC3, 0xC3), (0xCCC9)),  # HANGUL SYLLABLE CHIEUCH E IEUNG
    ((0xC3, 0xC4), (0xCCD0)),  # HANGUL SYLLABLE CHIEUCH YEO
    ((0xC3, 0xC5), (0xCCD4)),  # HANGUL SYLLABLE CHIEUCH YEO NIEUN
    ((0xC3, 0xC6), (0xCCE4)),  # HANGUL SYLLABLE CHIEUCH YEO SSANGSIOS
    ((0xC3, 0xC7), (0xCCEC)),  # HANGUL SYLLABLE CHIEUCH YE
    ((0xC3, 0xC8), (0xCCF0)),  # HANGUL SYLLABLE CHIEUCH YE NIEUN
    ((0xC3, 0xC9), (0xCD01)),  # HANGUL SYLLABLE CHIEUCH YE IEUNG
    ((0xC3, 0xCA), (0xCD08)),  # HANGUL SYLLABLE CHIEUCH O
    ((0xC3, 0xCB), (0xCD09)),  # HANGUL SYLLABLE CHIEUCH O KIYEOK
    ((0xC3, 0xCC), (0xCD0C)),  # HANGUL SYLLABLE CHIEUCH O NIEUN
    ((0xC3, 0xCD), (0xCD10)),  # HANGUL SYLLABLE CHIEUCH O RIEUL
    ((0xC3, 0xCE), (0xCD18)),  # HANGUL SYLLABLE CHIEUCH O MIEUM
    ((0xC3, 0xCF), (0xCD19)),  # HANGUL SYLLABLE CHIEUCH O PIEUP
    ((0xC3, 0xD0), (0xCD1B)),  # HANGUL SYLLABLE CHIEUCH O SIOS
    ((0xC3, 0xD1), (0xCD1D)),  # HANGUL SYLLABLE CHIEUCH O IEUNG
    ((0xC3, 0xD2), (0xCD24)),  # HANGUL SYLLABLE CHIEUCH WA
    ((0xC3, 0xD3), (0xCD28)),  # HANGUL SYLLABLE CHIEUCH WA NIEUN
    ((0xC3, 0xD4), (0xCD2C)),  # HANGUL SYLLABLE CHIEUCH WA RIEUL
    ((0xC3, 0xD5), (0xCD39)),  # HANGUL SYLLABLE CHIEUCH WA IEUNG
    ((0xC3, 0xD6), (0xCD5C)),  # HANGUL SYLLABLE CHIEUCH OE
    ((0xC3, 0xD7), (0xCD60)),  # HANGUL SYLLABLE CHIEUCH OE NIEUN
    ((0xC3, 0xD8), (0xCD64)),  # HANGUL SYLLABLE CHIEUCH OE RIEUL
    ((0xC3, 0xD9), (0xCD6C)),  # HANGUL SYLLABLE CHIEUCH OE MIEUM
    ((0xC3, 0xDA), (0xCD6D)),  # HANGUL SYLLABLE CHIEUCH OE PIEUP
    ((0xC3, 0xDB), (0xCD6F)),  # HANGUL SYLLABLE CHIEUCH OE SIOS
    ((0xC3, 0xDC), (0xCD71)),  # HANGUL SYLLABLE CHIEUCH OE IEUNG
    ((0xC3, 0xDD), (0xCD78)),  # HANGUL SYLLABLE CHIEUCH YO
    ((0xC3, 0xDE), (0xCD88)),  # HANGUL SYLLABLE CHIEUCH YO MIEUM
    ((0xC3, 0xDF), (0xCD94)),  # HANGUL SYLLABLE CHIEUCH U
    ((0xC3, 0xE0), (0xCD95)),  # HANGUL SYLLABLE CHIEUCH U KIYEOK
    ((0xC3, 0xE1), (0xCD98)),  # HANGUL SYLLABLE CHIEUCH U NIEUN
    ((0xC3, 0xE2), (0xCD9C)),  # HANGUL SYLLABLE CHIEUCH U RIEUL
    ((0xC3, 0xE3), (0xCDA4)),  # HANGUL SYLLABLE CHIEUCH U MIEUM
    ((0xC3, 0xE4), (0xCDA5)),  # HANGUL SYLLABLE CHIEUCH U PIEUP
    ((0xC3, 0xE5), (0xCDA7)),  # HANGUL SYLLABLE CHIEUCH U SIOS
    ((0xC3, 0xE6), (0xCDA9)),  # HANGUL SYLLABLE CHIEUCH U IEUNG
    ((0xC3, 0xE7), (0xCDB0)),  # HANGUL SYLLABLE CHIEUCH WEO
    ((0xC3, 0xE8), (0xCDC4)),  # HANGUL SYLLABLE CHIEUCH WEO SSANGSIOS
    ((0xC3, 0xE9), (0xCDCC)),  # HANGUL SYLLABLE CHIEUCH WE
    ((0xC3, 0xEA), (0xCDD0)),  # HANGUL SYLLABLE CHIEUCH WE NIEUN
    ((0xC3, 0xEB), (0xCDE8)),  # HANGUL SYLLABLE CHIEUCH WI
    ((0xC3, 0xEC), (0xCDEC)),  # HANGUL SYLLABLE CHIEUCH WI NIEUN
    ((0xC3, 0xED), (0xCDF0)),  # HANGUL SYLLABLE CHIEUCH WI RIEUL
    ((0xC3, 0xEE), (0xCDF8)),  # HANGUL SYLLABLE CHIEUCH WI MIEUM
    ((0xC3, 0xEF), (0xCDF9)),  # HANGUL SYLLABLE CHIEUCH WI PIEUP
    ((0xC3, 0xF0), (0xCDFB)),  # HANGUL SYLLABLE CHIEUCH WI SIOS
    ((0xC3, 0xF1), (0xCDFD)),  # HANGUL SYLLABLE CHIEUCH WI IEUNG
    ((0xC3, 0xF2), (0xCE04)),  # HANGUL SYLLABLE CHIEUCH YU
    ((0xC3, 0xF3), (0xCE08)),  # HANGUL SYLLABLE CHIEUCH YU NIEUN
    ((0xC3, 0xF4), (0xCE0C)),  # HANGUL SYLLABLE CHIEUCH YU RIEUL
    ((0xC3, 0xF5), (0xCE14)),  # HANGUL SYLLABLE CHIEUCH YU MIEUM
    ((0xC3, 0xF6), (0xCE19)),  # HANGUL SYLLABLE CHIEUCH YU IEUNG
    ((0xC3, 0xF7), (0xCE20)),  # HANGUL SYLLABLE CHIEUCH EU
    ((0xC3, 0xF8), (0xCE21)),  # HANGUL SYLLABLE CHIEUCH EU KIYEOK
    ((0xC3, 0xF9), (0xCE24)),  # HANGUL SYLLABLE CHIEUCH EU NIEUN
    ((0xC3, 0xFA), (0xCE28)),  # HANGUL SYLLABLE CHIEUCH EU RIEUL
    ((0xC3, 0xFB), (0xCE30)),  # HANGUL SYLLABLE CHIEUCH EU MIEUM
    ((0xC3, 0xFC), (0xCE31)),  # HANGUL SYLLABLE CHIEUCH EU PIEUP
    ((0xC3, 0xFD), (0xCE33)),  # HANGUL SYLLABLE CHIEUCH EU SIOS
    ((0xC3, 0xFE), (0xCE35)),  # HANGUL SYLLABLE CHIEUCH EU IEUNG
    ((0xC4, 0x41), (0xD6AB)),  # HANGUL SYLLABLE HIEUH YO KIYEOKSIOS
    ((0xC4, 0x42), (0xD6AD)),  # HANGUL SYLLABLE HIEUH YO NIEUNCIEUC
    ((0xC4, 0x43), (0xD6AE)),  # HANGUL SYLLABLE HIEUH YO NIEUNHIEUH
    ((0xC4, 0x44), (0xD6AF)),  # HANGUL SYLLABLE HIEUH YO TIKEUT
    ((0xC4, 0x45), (0xD6B1)),  # HANGUL SYLLABLE HIEUH YO RIEULKIYEOK
    ((0xC4, 0x46), (0xD6B2)),  # HANGUL SYLLABLE HIEUH YO RIEULMIEUM
    ((0xC4, 0x47), (0xD6B3)),  # HANGUL SYLLABLE HIEUH YO RIEULPIEUP
    ((0xC4, 0x48), (0xD6B4)),  # HANGUL SYLLABLE HIEUH YO RIEULSIOS
    ((0xC4, 0x49), (0xD6B5)),  # HANGUL SYLLABLE HIEUH YO RIEULTHIEUTH
    ((0xC4, 0x4A), (0xD6B6)),  # HANGUL SYLLABLE HIEUH YO RIEULPHIEUPH
    ((0xC4, 0x4B), (0xD6B7)),  # HANGUL SYLLABLE HIEUH YO RIEULHIEUH
    ((0xC4, 0x4C), (0xD6B8)),  # HANGUL SYLLABLE HIEUH YO MIEUM
    ((0xC4, 0x4D), (0xD6BA)),  # HANGUL SYLLABLE HIEUH YO PIEUPSIOS
    ((0xC4, 0x4E), (0xD6BC)),  # HANGUL SYLLABLE HIEUH YO SSANGSIOS
    ((0xC4, 0x4F), (0xD6BD)),  # HANGUL SYLLABLE HIEUH YO IEUNG
    ((0xC4, 0x50), (0xD6BE)),  # HANGUL SYLLABLE HIEUH YO CIEUC
    ((0xC4, 0x51), (0xD6BF)),  # HANGUL SYLLABLE HIEUH YO CHIEUCH
    ((0xC4, 0x52), (0xD6C0)),  # HANGUL SYLLABLE HIEUH YO KHIEUKH
    ((0xC4, 0x53), (0xD6C1)),  # HANGUL SYLLABLE HIEUH YO THIEUTH
    ((0xC4, 0x54), (0xD6C2)),  # HANGUL SYLLABLE HIEUH YO PHIEUPH
    ((0xC4, 0x55), (0xD6C3)),  # HANGUL SYLLABLE HIEUH YO HIEUH
    ((0xC4, 0x56), (0xD6C6)),  # HANGUL SYLLABLE HIEUH U SSANGKIYEOK
    ((0xC4, 0x57), (0xD6C7)),  # HANGUL SYLLABLE HIEUH U KIYEOKSIOS
    ((0xC4, 0x58), (0xD6C9)),  # HANGUL SYLLABLE HIEUH U NIEUNCIEUC
    ((0xC4, 0x59), (0xD6CA)),  # HANGUL SYLLABLE HIEUH U NIEUNHIEUH
    ((0xC4, 0x5A), (0xD6CB)),  # HANGUL SYLLABLE HIEUH U TIKEUT
    ((0xC4, 0x61), (0xD6CD)),  # HANGUL SYLLABLE HIEUH U RIEULKIYEOK
    ((0xC4, 0x62), (0xD6CE)),  # HANGUL SYLLABLE HIEUH U RIEULMIEUM
    ((0xC4, 0x63), (0xD6CF)),  # HANGUL SYLLABLE HIEUH U RIEULPIEUP
    ((0xC4, 0x64), (0xD6D0)),  # HANGUL SYLLABLE HIEUH U RIEULSIOS
    ((0xC4, 0x65), (0xD6D2)),  # HANGUL SYLLABLE HIEUH U RIEULPHIEUPH
    ((0xC4, 0x66), (0xD6D3)),  # HANGUL SYLLABLE HIEUH U RIEULHIEUH
    ((0xC4, 0x67), (0xD6D5)),  # HANGUL SYLLABLE HIEUH U PIEUP
    ((0xC4, 0x68), (0xD6D6)),  # HANGUL SYLLABLE HIEUH U PIEUPSIOS
    ((0xC4, 0x69), (0xD6D8)),  # HANGUL SYLLABLE HIEUH U SSANGSIOS
    ((0xC4, 0x6A), (0xD6DA)),  # HANGUL SYLLABLE HIEUH U CIEUC
    ((0xC4, 0x6B), (0xD6DB)),  # HANGUL SYLLABLE HIEUH U CHIEUCH
    ((0xC4, 0x6C), (0xD6DC)),  # HANGUL SYLLABLE HIEUH U KHIEUKH
    ((0xC4, 0x6D), (0xD6DD)),  # HANGUL SYLLABLE HIEUH U THIEUTH
    ((0xC4, 0x6E), (0xD6DE)),  # HANGUL SYLLABLE HIEUH U PHIEUPH
    ((0xC4, 0x6F), (0xD6DF)),  # HANGUL SYLLABLE HIEUH U HIEUH
    ((0xC4, 0x70), (0xD6E1)),  # HANGUL SYLLABLE HIEUH WEO KIYEOK
    ((0xC4, 0x71), (0xD6E2)),  # HANGUL SYLLABLE HIEUH WEO SSANGKIYEOK
    ((0xC4, 0x72), (0xD6E3)),  # HANGUL SYLLABLE HIEUH WEO KIYEOKSIOS
    ((0xC4, 0x73), (0xD6E5)),  # HANGUL SYLLABLE HIEUH WEO NIEUNCIEUC
    ((0xC4, 0x74), (0xD6E6)),  # HANGUL SYLLABLE HIEUH WEO NIEUNHIEUH
    ((0xC4, 0x75), (0xD6E7)),  # HANGUL SYLLABLE HIEUH WEO TIKEUT
    ((0xC4, 0x76), (0xD6E9)),  # HANGUL SYLLABLE HIEUH WEO RIEULKIYEOK
    ((0xC4, 0x77), (0xD6EA)),  # HANGUL SYLLABLE HIEUH WEO RIEULMIEUM
    ((0xC4, 0x78), (0xD6EB)),  # HANGUL SYLLABLE HIEUH WEO RIEULPIEUP
    ((0xC4, 0x79), (0xD6EC)),  # HANGUL SYLLABLE HIEUH WEO RIEULSIOS
    ((0xC4, 0x7A), (0xD6ED)),  # HANGUL SYLLABLE HIEUH WEO RIEULTHIEUTH
    ((0xC4, 0x81), (0xD6EE)),  # HANGUL SYLLABLE HIEUH WEO RIEULPHIEUPH
    ((0xC4, 0x82), (0xD6EF)),  # HANGUL SYLLABLE HIEUH WEO RIEULHIEUH
    ((0xC4, 0x83), (0xD6F1)),  # HANGUL SYLLABLE HIEUH WEO PIEUP
    ((0xC4, 0x84), (0xD6F2)),  # HANGUL SYLLABLE HIEUH WEO PIEUPSIOS
    ((0xC4, 0x85), (0xD6F3)),  # HANGUL SYLLABLE HIEUH WEO SIOS
    ((0xC4, 0x86), (0xD6F4)),  # HANGUL SYLLABLE HIEUH WEO SSANGSIOS
    ((0xC4, 0x87), (0xD6F6)),  # HANGUL SYLLABLE HIEUH WEO CIEUC
    ((0xC4, 0x88), (0xD6F7)),  # HANGUL SYLLABLE HIEUH WEO CHIEUCH
    ((0xC4, 0x89), (0xD6F8)),  # HANGUL SYLLABLE HIEUH WEO KHIEUKH
    ((0xC4, 0x8A), (0xD6F9)),  # HANGUL SYLLABLE HIEUH WEO THIEUTH
    ((0xC4, 0x8B), (0xD6FA)),  # HANGUL SYLLABLE HIEUH WEO PHIEUPH
    ((0xC4, 0x8C), (0xD6FB)),  # HANGUL SYLLABLE HIEUH WEO HIEUH
    ((0xC4, 0x8D), (0xD6FE)),  # HANGUL SYLLABLE HIEUH WE SSANGKIYEOK
    ((0xC4, 0x8E), (0xD6FF)),  # HANGUL SYLLABLE HIEUH WE KIYEOKSIOS
    ((0xC4, 0x8F), (0xD701)),  # HANGUL SYLLABLE HIEUH WE NIEUNCIEUC
    ((0xC4, 0x90), (0xD702)),  # HANGUL SYLLABLE HIEUH WE NIEUNHIEUH
    ((0xC4, 0x91), (0xD703)),  # HANGUL SYLLABLE HIEUH WE TIKEUT
    ((0xC4, 0x92), (0xD705)),  # HANGUL SYLLABLE HIEUH WE RIEULKIYEOK
    ((0xC4, 0x93), (0xD706)),  # HANGUL SYLLABLE HIEUH WE RIEULMIEUM
    ((0xC4, 0x94), (0xD707)),  # HANGUL SYLLABLE HIEUH WE RIEULPIEUP
    ((0xC4, 0x95), (0xD708)),  # HANGUL SYLLABLE HIEUH WE RIEULSIOS
    ((0xC4, 0x96), (0xD709)),  # HANGUL SYLLABLE HIEUH WE RIEULTHIEUTH
    ((0xC4, 0x97), (0xD70A)),  # HANGUL SYLLABLE HIEUH WE RIEULPHIEUPH
    ((0xC4, 0x98), (0xD70B)),  # HANGUL SYLLABLE HIEUH WE RIEULHIEUH
    ((0xC4, 0x99), (0xD70C)),  # HANGUL SYLLABLE HIEUH WE MIEUM
    ((0xC4, 0x9A), (0xD70D)),  # HANGUL SYLLABLE HIEUH WE PIEUP
    ((0xC4, 0x9B), (0xD70E)),  # HANGUL SYLLABLE HIEUH WE PIEUPSIOS
    ((0xC4, 0x9C), (0xD70F)),  # HANGUL SYLLABLE HIEUH WE SIOS
    ((0xC4, 0x9D), (0xD710)),  # HANGUL SYLLABLE HIEUH WE SSANGSIOS
    ((0xC4, 0x9E), (0xD712)),  # HANGUL SYLLABLE HIEUH WE CIEUC
    ((0xC4, 0x9F), (0xD713)),  # HANGUL SYLLABLE HIEUH WE CHIEUCH
    ((0xC4, 0xA0), (0xD714)),  # HANGUL SYLLABLE HIEUH WE KHIEUKH
    ((0xC4, 0xA1), (0xCE58)),  # HANGUL SYLLABLE CHIEUCH I
    ((0xC4, 0xA2), (0xCE59)),  # HANGUL SYLLABLE CHIEUCH I KIYEOK
    ((0xC4, 0xA3), (0xCE5C)),  # HANGUL SYLLABLE CHIEUCH I NIEUN
    ((0xC4, 0xA4), (0xCE5F)),  # HANGUL SYLLABLE CHIEUCH I TIKEUT
    ((0xC4, 0xA5), (0xCE60)),  # HANGUL SYLLABLE CHIEUCH I RIEUL
    ((0xC4, 0xA6), (0xCE61)),  # HANGUL SYLLABLE CHIEUCH I RIEULKIYEOK
    ((0xC4, 0xA7), (0xCE68)),  # HANGUL SYLLABLE CHIEUCH I MIEUM
    ((0xC4, 0xA8), (0xCE69)),  # HANGUL SYLLABLE CHIEUCH I PIEUP
    ((0xC4, 0xA9), (0xCE6B)),  # HANGUL SYLLABLE CHIEUCH I SIOS
    ((0xC4, 0xAA), (0xCE6D)),  # HANGUL SYLLABLE CHIEUCH I IEUNG
    ((0xC4, 0xAB), (0xCE74)),  # HANGUL SYLLABLE KHIEUKH A
    ((0xC4, 0xAC), (0xCE75)),  # HANGUL SYLLABLE KHIEUKH A KIYEOK
    ((0xC4, 0xAD), (0xCE78)),  # HANGUL SYLLABLE KHIEUKH A NIEUN
    ((0xC4, 0xAE), (0xCE7C)),  # HANGUL SYLLABLE KHIEUKH A RIEUL
    ((0xC4, 0xAF), (0xCE84)),  # HANGUL SYLLABLE KHIEUKH A MIEUM
    ((0xC4, 0xB0), (0xCE85)),  # HANGUL SYLLABLE KHIEUKH A PIEUP
    ((0xC4, 0xB1), (0xCE87)),  # HANGUL SYLLABLE KHIEUKH A SIOS
    ((0xC4, 0xB2), (0xCE89)),  # HANGUL SYLLABLE KHIEUKH A IEUNG
    ((0xC4, 0xB3), (0xCE90)),  # HANGUL SYLLABLE KHIEUKH AE
    ((0xC4, 0xB4), (0xCE91)),  # HANGUL SYLLABLE KHIEUKH AE KIYEOK
    ((0xC4, 0xB5), (0xCE94)),  # HANGUL SYLLABLE KHIEUKH AE NIEUN
    ((0xC4, 0xB6), (0xCE98)),  # HANGUL SYLLABLE KHIEUKH AE RIEUL
    ((0xC4, 0xB7), (0xCEA0)),  # HANGUL SYLLABLE KHIEUKH AE MIEUM
    ((0xC4, 0xB8), (0xCEA1)),  # HANGUL SYLLABLE KHIEUKH AE PIEUP
    ((0xC4, 0xB9), (0xCEA3)),  # HANGUL SYLLABLE KHIEUKH AE SIOS
    ((0xC4, 0xBA), (0xCEA4)),  # HANGUL SYLLABLE KHIEUKH AE SSANGSIOS
    ((0xC4, 0xBB), (0xCEA5)),  # HANGUL SYLLABLE KHIEUKH AE IEUNG
    ((0xC4, 0xBC), (0xCEAC)),  # HANGUL SYLLABLE KHIEUKH YA
    ((0xC4, 0xBD), (0xCEAD)),  # HANGUL SYLLABLE KHIEUKH YA KIYEOK
    ((0xC4, 0xBE), (0xCEC1)),  # HANGUL SYLLABLE KHIEUKH YA IEUNG
    ((0xC4, 0xBF), (0xCEE4)),  # HANGUL SYLLABLE KHIEUKH EO
    ((0xC4, 0xC0), (0xCEE5)),  # HANGUL SYLLABLE KHIEUKH EO KIYEOK
    ((0xC4, 0xC1), (0xCEE8)),  # HANGUL SYLLABLE KHIEUKH EO NIEUN
    ((0xC4, 0xC2), (0xCEEB)),  # HANGUL SYLLABLE KHIEUKH EO TIKEUT
    ((0xC4, 0xC3), (0xCEEC)),  # HANGUL SYLLABLE KHIEUKH EO RIEUL
    ((0xC4, 0xC4), (0xCEF4)),  # HANGUL SYLLABLE KHIEUKH EO MIEUM
    ((0xC4, 0xC5), (0xCEF5)),  # HANGUL SYLLABLE KHIEUKH EO PIEUP
    ((0xC4, 0xC6), (0xCEF7)),  # HANGUL SYLLABLE KHIEUKH EO SIOS
    ((0xC4, 0xC7), (0xCEF8)),  # HANGUL SYLLABLE KHIEUKH EO SSANGSIOS
    ((0xC4, 0xC8), (0xCEF9)),  # HANGUL SYLLABLE KHIEUKH EO IEUNG
    ((0xC4, 0xC9), (0xCF00)),  # HANGUL SYLLABLE KHIEUKH E
    ((0xC4, 0xCA), (0xCF01)),  # HANGUL SYLLABLE KHIEUKH E KIYEOK
    ((0xC4, 0xCB), (0xCF04)),  # HANGUL SYLLABLE KHIEUKH E NIEUN
    ((0xC4, 0xCC), (0xCF08)),  # HANGUL SYLLABLE KHIEUKH E RIEUL
    ((0xC4, 0xCD), (0xCF10)),  # HANGUL SYLLABLE KHIEUKH E MIEUM
    ((0xC4, 0xCE), (0xCF11)),  # HANGUL SYLLABLE KHIEUKH E PIEUP
    ((0xC4, 0xCF), (0xCF13)),  # HANGUL SYLLABLE KHIEUKH E SIOS
    ((0xC4, 0xD0), (0xCF15)),  # HANGUL SYLLABLE KHIEUKH E IEUNG
    ((0xC4, 0xD1), (0xCF1C)),  # HANGUL SYLLABLE KHIEUKH YEO
    ((0xC4, 0xD2), (0xCF20)),  # HANGUL SYLLABLE KHIEUKH YEO NIEUN
    ((0xC4, 0xD3), (0xCF24)),  # HANGUL SYLLABLE KHIEUKH YEO RIEUL
    ((0xC4, 0xD4), (0xCF2C)),  # HANGUL SYLLABLE KHIEUKH YEO MIEUM
    ((0xC4, 0xD5), (0xCF2D)),  # HANGUL SYLLABLE KHIEUKH YEO PIEUP
    ((0xC4, 0xD6), (0xCF2F)),  # HANGUL SYLLABLE KHIEUKH YEO SIOS
    ((0xC4, 0xD7), (0xCF30)),  # HANGUL SYLLABLE KHIEUKH YEO SSANGSIOS
    ((0xC4, 0xD8), (0xCF31)),  # HANGUL SYLLABLE KHIEUKH YEO IEUNG
    ((0xC4, 0xD9), (0xCF38)),  # HANGUL SYLLABLE KHIEUKH YE
    ((0xC4, 0xDA), (0xCF54)),  # HANGUL SYLLABLE KHIEUKH O
    ((0xC4, 0xDB), (0xCF55)),  # HANGUL SYLLABLE KHIEUKH O KIYEOK
    ((0xC4, 0xDC), (0xCF58)),  # HANGUL SYLLABLE KHIEUKH O NIEUN
    ((0xC4, 0xDD), (0xCF5C)),  # HANGUL SYLLABLE KHIEUKH O RIEUL
    ((0xC4, 0xDE), (0xCF64)),  # HANGUL SYLLABLE KHIEUKH O MIEUM
    ((0xC4, 0xDF), (0xCF65)),  # HANGUL SYLLABLE KHIEUKH O PIEUP
    ((0xC4, 0xE0), (0xCF67)),  # HANGUL SYLLABLE KHIEUKH O SIOS
    ((0xC4, 0xE1), (0xCF69)),  # HANGUL SYLLABLE KHIEUKH O IEUNG
    ((0xC4, 0xE2), (0xCF70)),  # HANGUL SYLLABLE KHIEUKH WA
    ((0xC4, 0xE3), (0xCF71)),  # HANGUL SYLLABLE KHIEUKH WA KIYEOK
    ((0xC4, 0xE4), (0xCF74)),  # HANGUL SYLLABLE KHIEUKH WA NIEUN
    ((0xC4, 0xE5), (0xCF78)),  # HANGUL SYLLABLE KHIEUKH WA RIEUL
    ((0xC4, 0xE6), (0xCF80)),  # HANGUL SYLLABLE KHIEUKH WA MIEUM
    ((0xC4, 0xE7), (0xCF85)),  # HANGUL SYLLABLE KHIEUKH WA IEUNG
    ((0xC4, 0xE8), (0xCF8C)),  # HANGUL SYLLABLE KHIEUKH WAE
    ((0xC4, 0xE9), (0xCFA1)),  # HANGUL SYLLABLE KHIEUKH WAE IEUNG
    ((0xC4, 0xEA), (0xCFA8)),  # HANGUL SYLLABLE KHIEUKH OE
    ((0xC4, 0xEB), (0xCFB0)),  # HANGUL SYLLABLE KHIEUKH OE RIEUL
    ((0xC4, 0xEC), (0xCFC4)),  # HANGUL SYLLABLE KHIEUKH YO
    ((0xC4, 0xED), (0xCFE0)),  # HANGUL SYLLABLE KHIEUKH U
    ((0xC4, 0xEE), (0xCFE1)),  # HANGUL SYLLABLE KHIEUKH U KIYEOK
    ((0xC4, 0xEF), (0xCFE4)),  # HANGUL SYLLABLE KHIEUKH U NIEUN
    ((0xC4, 0xF0), (0xCFE8)),  # HANGUL SYLLABLE KHIEUKH U RIEUL
    ((0xC4, 0xF1), (0xCFF0)),  # HANGUL SYLLABLE KHIEUKH U MIEUM
    ((0xC4, 0xF2), (0xCFF1)),  # HANGUL SYLLABLE KHIEUKH U PIEUP
    ((0xC4, 0xF3), (0xCFF3)),  # HANGUL SYLLABLE KHIEUKH U SIOS
    ((0xC4, 0xF4), (0xCFF5)),  # HANGUL SYLLABLE KHIEUKH U IEUNG
    ((0xC4, 0xF5), (0xCFFC)),  # HANGUL SYLLABLE KHIEUKH WEO
    ((0xC4, 0xF6), (0xD000)),  # HANGUL SYLLABLE KHIEUKH WEO NIEUN
    ((0xC4, 0xF7), (0xD004)),  # HANGUL SYLLABLE KHIEUKH WEO RIEUL
    ((0xC4, 0xF8), (0xD011)),  # HANGUL SYLLABLE KHIEUKH WEO IEUNG
    ((0xC4, 0xF9), (0xD018)),  # HANGUL SYLLABLE KHIEUKH WE
    ((0xC4, 0xFA), (0xD02D)),  # HANGUL SYLLABLE KHIEUKH WE IEUNG
    ((0xC4, 0xFB), (0xD034)),  # HANGUL SYLLABLE KHIEUKH WI
    ((0xC4, 0xFC), (0xD035)),  # HANGUL SYLLABLE KHIEUKH WI KIYEOK
    ((0xC4, 0xFD), (0xD038)),  # HANGUL SYLLABLE KHIEUKH WI NIEUN
    ((0xC4, 0xFE), (0xD03C)),  # HANGUL SYLLABLE KHIEUKH WI RIEUL
    ((0xC5, 0x41), (0xD715)),  # HANGUL SYLLABLE HIEUH WE THIEUTH
    ((0xC5, 0x42), (0xD716)),  # HANGUL SYLLABLE HIEUH WE PHIEUPH
    ((0xC5, 0x43), (0xD717)),  # HANGUL SYLLABLE HIEUH WE HIEUH
    ((0xC5, 0x44), (0xD71A)),  # HANGUL SYLLABLE HIEUH WI SSANGKIYEOK
    ((0xC5, 0x45), (0xD71B)),  # HANGUL SYLLABLE HIEUH WI KIYEOKSIOS
    ((0xC5, 0x46), (0xD71D)),  # HANGUL SYLLABLE HIEUH WI NIEUNCIEUC
    ((0xC5, 0x47), (0xD71E)),  # HANGUL SYLLABLE HIEUH WI NIEUNHIEUH
    ((0xC5, 0x48), (0xD71F)),  # HANGUL SYLLABLE HIEUH WI TIKEUT
    ((0xC5, 0x49), (0xD721)),  # HANGUL SYLLABLE HIEUH WI RIEULKIYEOK
    ((0xC5, 0x4A), (0xD722)),  # HANGUL SYLLABLE HIEUH WI RIEULMIEUM
    ((0xC5, 0x4B), (0xD723)),  # HANGUL SYLLABLE HIEUH WI RIEULPIEUP
    ((0xC5, 0x4C), (0xD724)),  # HANGUL SYLLABLE HIEUH WI RIEULSIOS
    ((0xC5, 0x4D), (0xD725)),  # HANGUL SYLLABLE HIEUH WI RIEULTHIEUTH
    ((0xC5, 0x4E), (0xD726)),  # HANGUL SYLLABLE HIEUH WI RIEULPHIEUPH
    ((0xC5, 0x4F), (0xD727)),  # HANGUL SYLLABLE HIEUH WI RIEULHIEUH
    ((0xC5, 0x50), (0xD72A)),  # HANGUL SYLLABLE HIEUH WI PIEUPSIOS
    ((0xC5, 0x51), (0xD72C)),  # HANGUL SYLLABLE HIEUH WI SSANGSIOS
    ((0xC5, 0x52), (0xD72E)),  # HANGUL SYLLABLE HIEUH WI CIEUC
    ((0xC5, 0x53), (0xD72F)),  # HANGUL SYLLABLE HIEUH WI CHIEUCH
    ((0xC5, 0x54), (0xD730)),  # HANGUL SYLLABLE HIEUH WI KHIEUKH
    ((0xC5, 0x55), (0xD731)),  # HANGUL SYLLABLE HIEUH WI THIEUTH
    ((0xC5, 0x56), (0xD732)),  # HANGUL SYLLABLE HIEUH WI PHIEUPH
    ((0xC5, 0x57), (0xD733)),  # HANGUL SYLLABLE HIEUH WI HIEUH
    ((0xC5, 0x58), (0xD736)),  # HANGUL SYLLABLE HIEUH YU SSANGKIYEOK
    ((0xC5, 0x59), (0xD737)),  # HANGUL SYLLABLE HIEUH YU KIYEOKSIOS
    ((0xC5, 0x5A), (0xD739)),  # HANGUL SYLLABLE HIEUH YU NIEUNCIEUC
    ((0xC5, 0x61), (0xD73A)),  # HANGUL SYLLABLE HIEUH YU NIEUNHIEUH
    ((0xC5, 0x62), (0xD73B)),  # HANGUL SYLLABLE HIEUH YU TIKEUT
    ((0xC5, 0x63), (0xD73D)),  # HANGUL SYLLABLE HIEUH YU RIEULKIYEOK
    ((0xC5, 0x64), (0xD73E)),  # HANGUL SYLLABLE HIEUH YU RIEULMIEUM
    ((0xC5, 0x65), (0xD73F)),  # HANGUL SYLLABLE HIEUH YU RIEULPIEUP
    ((0xC5, 0x66), (0xD740)),  # HANGUL SYLLABLE HIEUH YU RIEULSIOS
    ((0xC5, 0x67), (0xD741)),  # HANGUL SYLLABLE HIEUH YU RIEULTHIEUTH
    ((0xC5, 0x68), (0xD742)),  # HANGUL SYLLABLE HIEUH YU RIEULPHIEUPH
    ((0xC5, 0x69), (0xD743)),  # HANGUL SYLLABLE HIEUH YU RIEULHIEUH
    ((0xC5, 0x6A), (0xD745)),  # HANGUL SYLLABLE HIEUH YU PIEUP
    ((0xC5, 0x6B), (0xD746)),  # HANGUL SYLLABLE HIEUH YU PIEUPSIOS
    ((0xC5, 0x6C), (0xD748)),  # HANGUL SYLLABLE HIEUH YU SSANGSIOS
    ((0xC5, 0x6D), (0xD74A)),  # HANGUL SYLLABLE HIEUH YU CIEUC
    ((0xC5, 0x6E), (0xD74B)),  # HANGUL SYLLABLE HIEUH YU CHIEUCH
    ((0xC5, 0x6F), (0xD74C)),  # HANGUL SYLLABLE HIEUH YU KHIEUKH
    ((0xC5, 0x70), (0xD74D)),  # HANGUL SYLLABLE HIEUH YU THIEUTH
    ((0xC5, 0x71), (0xD74E)),  # HANGUL SYLLABLE HIEUH YU PHIEUPH
    ((0xC5, 0x72), (0xD74F)),  # HANGUL SYLLABLE HIEUH YU HIEUH
    ((0xC5, 0x73), (0xD752)),  # HANGUL SYLLABLE HIEUH EU SSANGKIYEOK
    ((0xC5, 0x74), (0xD753)),  # HANGUL SYLLABLE HIEUH EU KIYEOKSIOS
    ((0xC5, 0x75), (0xD755)),  # HANGUL SYLLABLE HIEUH EU NIEUNCIEUC
    ((0xC5, 0x76), (0xD75A)),  # HANGUL SYLLABLE HIEUH EU RIEULMIEUM
    ((0xC5, 0x77), (0xD75B)),  # HANGUL SYLLABLE HIEUH EU RIEULPIEUP
    ((0xC5, 0x78), (0xD75C)),  # HANGUL SYLLABLE HIEUH EU RIEULSIOS
    ((0xC5, 0x79), (0xD75D)),  # HANGUL SYLLABLE HIEUH EU RIEULTHIEUTH
    ((0xC5, 0x7A), (0xD75E)),  # HANGUL SYLLABLE HIEUH EU RIEULPHIEUPH
    ((0xC5, 0x81), (0xD75F)),  # HANGUL SYLLABLE HIEUH EU RIEULHIEUH
    ((0xC5, 0x82), (0xD762)),  # HANGUL SYLLABLE HIEUH EU PIEUPSIOS
    ((0xC5, 0x83), (0xD764)),  # HANGUL SYLLABLE HIEUH EU SSANGSIOS
    ((0xC5, 0x84), (0xD766)),  # HANGUL SYLLABLE HIEUH EU CIEUC
    ((0xC5, 0x85), (0xD767)),  # HANGUL SYLLABLE HIEUH EU CHIEUCH
    ((0xC5, 0x86), (0xD768)),  # HANGUL SYLLABLE HIEUH EU KHIEUKH
    ((0xC5, 0x87), (0xD76A)),  # HANGUL SYLLABLE HIEUH EU PHIEUPH
    ((0xC5, 0x88), (0xD76B)),  # HANGUL SYLLABLE HIEUH EU HIEUH
    ((0xC5, 0x89), (0xD76D)),  # HANGUL SYLLABLE HIEUH YI KIYEOK
    ((0xC5, 0x8A), (0xD76E)),  # HANGUL SYLLABLE HIEUH YI SSANGKIYEOK
    ((0xC5, 0x8B), (0xD76F)),  # HANGUL SYLLABLE HIEUH YI KIYEOKSIOS
    ((0xC5, 0x8C), (0xD771)),  # HANGUL SYLLABLE HIEUH YI NIEUNCIEUC
    ((0xC5, 0x8D), (0xD772)),  # HANGUL SYLLABLE HIEUH YI NIEUNHIEUH
    ((0xC5, 0x8E), (0xD773)),  # HANGUL SYLLABLE HIEUH YI TIKEUT
    ((0xC5, 0x8F), (0xD775)),  # HANGUL SYLLABLE HIEUH YI RIEULKIYEOK
    ((0xC5, 0x90), (0xD776)),  # HANGUL SYLLABLE HIEUH YI RIEULMIEUM
    ((0xC5, 0x91), (0xD777)),  # HANGUL SYLLABLE HIEUH YI RIEULPIEUP
    ((0xC5, 0x92), (0xD778)),  # HANGUL SYLLABLE HIEUH YI RIEULSIOS
    ((0xC5, 0x93), (0xD779)),  # HANGUL SYLLABLE HIEUH YI RIEULTHIEUTH
    ((0xC5, 0x94), (0xD77A)),  # HANGUL SYLLABLE HIEUH YI RIEULPHIEUPH
    ((0xC5, 0x95), (0xD77B)),  # HANGUL SYLLABLE HIEUH YI RIEULHIEUH
    ((0xC5, 0x96), (0xD77E)),  # HANGUL SYLLABLE HIEUH YI PIEUPSIOS
    ((0xC5, 0x97), (0xD77F)),  # HANGUL SYLLABLE HIEUH YI SIOS
    ((0xC5, 0x98), (0xD780)),  # HANGUL SYLLABLE HIEUH YI SSANGSIOS
    ((0xC5, 0x99), (0xD782)),  # HANGUL SYLLABLE HIEUH YI CIEUC
    ((0xC5, 0x9A), (0xD783)),  # HANGUL SYLLABLE HIEUH YI CHIEUCH
    ((0xC5, 0x9B), (0xD784)),  # HANGUL SYLLABLE HIEUH YI KHIEUKH
    ((0xC5, 0x9C), (0xD785)),  # HANGUL SYLLABLE HIEUH YI THIEUTH
    ((0xC5, 0x9D), (0xD786)),  # HANGUL SYLLABLE HIEUH YI PHIEUPH
    ((0xC5, 0x9E), (0xD787)),  # HANGUL SYLLABLE HIEUH YI HIEUH
    ((0xC5, 0x9F), (0xD78A)),  # HANGUL SYLLABLE HIEUH I SSANGKIYEOK
    ((0xC5, 0xA0), (0xD78B)),  # HANGUL SYLLABLE HIEUH I KIYEOKSIOS
    ((0xC5, 0xA1), (0xD044)),  # HANGUL SYLLABLE KHIEUKH WI MIEUM
    ((0xC5, 0xA2), (0xD045)),  # HANGUL SYLLABLE KHIEUKH WI PIEUP
    ((0xC5, 0xA3), (0xD047)),  # HANGUL SYLLABLE KHIEUKH WI SIOS
    ((0xC5, 0xA4), (0xD049)),  # HANGUL SYLLABLE KHIEUKH WI IEUNG
    ((0xC5, 0xA5), (0xD050)),  # HANGUL SYLLABLE KHIEUKH YU
    ((0xC5, 0xA6), (0xD054)),  # HANGUL SYLLABLE KHIEUKH YU NIEUN
    ((0xC5, 0xA7), (0xD058)),  # HANGUL SYLLABLE KHIEUKH YU RIEUL
    ((0xC5, 0xA8), (0xD060)),  # HANGUL SYLLABLE KHIEUKH YU MIEUM
    ((0xC5, 0xA9), (0xD06C)),  # HANGUL SYLLABLE KHIEUKH EU
    ((0xC5, 0xAA), (0xD06D)),  # HANGUL SYLLABLE KHIEUKH EU KIYEOK
    ((0xC5, 0xAB), (0xD070)),  # HANGUL SYLLABLE KHIEUKH EU NIEUN
    ((0xC5, 0xAC), (0xD074)),  # HANGUL SYLLABLE KHIEUKH EU RIEUL
    ((0xC5, 0xAD), (0xD07C)),  # HANGUL SYLLABLE KHIEUKH EU MIEUM
    ((0xC5, 0xAE), (0xD07D)),  # HANGUL SYLLABLE KHIEUKH EU PIEUP
    ((0xC5, 0xAF), (0xD081)),  # HANGUL SYLLABLE KHIEUKH EU IEUNG
    ((0xC5, 0xB0), (0xD0A4)),  # HANGUL SYLLABLE KHIEUKH I
    ((0xC5, 0xB1), (0xD0A5)),  # HANGUL SYLLABLE KHIEUKH I KIYEOK
    ((0xC5, 0xB2), (0xD0A8)),  # HANGUL SYLLABLE KHIEUKH I NIEUN
    ((0xC5, 0xB3), (0xD0AC)),  # HANGUL SYLLABLE KHIEUKH I RIEUL
    ((0xC5, 0xB4), (0xD0B4)),  # HANGUL SYLLABLE KHIEUKH I MIEUM
    ((0xC5, 0xB5), (0xD0B5)),  # HANGUL SYLLABLE KHIEUKH I PIEUP
    ((0xC5, 0xB6), (0xD0B7)),  # HANGUL SYLLABLE KHIEUKH I SIOS
    ((0xC5, 0xB7), (0xD0B9)),  # HANGUL SYLLABLE KHIEUKH I IEUNG
    ((0xC5, 0xB8), (0xD0C0)),  # HANGUL SYLLABLE THIEUTH A
    ((0xC5, 0xB9), (0xD0C1)),  # HANGUL SYLLABLE THIEUTH A KIYEOK
    ((0xC5, 0xBA), (0xD0C4)),  # HANGUL SYLLABLE THIEUTH A NIEUN
    ((0xC5, 0xBB), (0xD0C8)),  # HANGUL SYLLABLE THIEUTH A RIEUL
    ((0xC5, 0xBC), (0xD0C9)),  # HANGUL SYLLABLE THIEUTH A RIEULKIYEOK
    ((0xC5, 0xBD), (0xD0D0)),  # HANGUL SYLLABLE THIEUTH A MIEUM
    ((0xC5, 0xBE), (0xD0D1)),  # HANGUL SYLLABLE THIEUTH A PIEUP
    ((0xC5, 0xBF), (0xD0D3)),  # HANGUL SYLLABLE THIEUTH A SIOS
    ((0xC5, 0xC0), (0xD0D4)),  # HANGUL SYLLABLE THIEUTH A SSANGSIOS
    ((0xC5, 0xC1), (0xD0D5)),  # HANGUL SYLLABLE THIEUTH A IEUNG
    ((0xC5, 0xC2), (0xD0DC)),  # HANGUL SYLLABLE THIEUTH AE
    ((0xC5, 0xC3), (0xD0DD)),  # HANGUL SYLLABLE THIEUTH AE KIYEOK
    ((0xC5, 0xC4), (0xD0E0)),  # HANGUL SYLLABLE THIEUTH AE NIEUN
    ((0xC5, 0xC5), (0xD0E4)),  # HANGUL SYLLABLE THIEUTH AE RIEUL
    ((0xC5, 0xC6), (0xD0EC)),  # HANGUL SYLLABLE THIEUTH AE MIEUM
    ((0xC5, 0xC7), (0xD0ED)),  # HANGUL SYLLABLE THIEUTH AE PIEUP
    ((0xC5, 0xC8), (0xD0EF)),  # HANGUL SYLLABLE THIEUTH AE SIOS
    ((0xC5, 0xC9), (0xD0F0)),  # HANGUL SYLLABLE THIEUTH AE SSANGSIOS
    ((0xC5, 0xCA), (0xD0F1)),  # HANGUL SYLLABLE THIEUTH AE IEUNG
    ((0xC5, 0xCB), (0xD0F8)),  # HANGUL SYLLABLE THIEUTH YA
    ((0xC5, 0xCC), (0xD10D)),  # HANGUL SYLLABLE THIEUTH YA IEUNG
    ((0xC5, 0xCD), (0xD130)),  # HANGUL SYLLABLE THIEUTH EO
    ((0xC5, 0xCE), (0xD131)),  # HANGUL SYLLABLE THIEUTH EO KIYEOK
    ((0xC5, 0xCF), (0xD134)),  # HANGUL SYLLABLE THIEUTH EO NIEUN
    ((0xC5, 0xD0), (0xD138)),  # HANGUL SYLLABLE THIEUTH EO RIEUL
    ((0xC5, 0xD1), (0xD13A)),  # HANGUL SYLLABLE THIEUTH EO RIEULMIEUM
    ((0xC5, 0xD2), (0xD140)),  # HANGUL SYLLABLE THIEUTH EO MIEUM
    ((0xC5, 0xD3), (0xD141)),  # HANGUL SYLLABLE THIEUTH EO PIEUP
    ((0xC5, 0xD4), (0xD143)),  # HANGUL SYLLABLE THIEUTH EO SIOS
    ((0xC5, 0xD5), (0xD144)),  # HANGUL SYLLABLE THIEUTH EO SSANGSIOS
    ((0xC5, 0xD6), (0xD145)),  # HANGUL SYLLABLE THIEUTH EO IEUNG
    ((0xC5, 0xD7), (0xD14C)),  # HANGUL SYLLABLE THIEUTH E
    ((0xC5, 0xD8), (0xD14D)),  # HANGUL SYLLABLE THIEUTH E KIYEOK
    ((0xC5, 0xD9), (0xD150)),  # HANGUL SYLLABLE THIEUTH E NIEUN
    ((0xC5, 0xDA), (0xD154)),  # HANGUL SYLLABLE THIEUTH E RIEUL
    ((0xC5, 0xDB), (0xD15C)),  # HANGUL SYLLABLE THIEUTH E MIEUM
    ((0xC5, 0xDC), (0xD15D)),  # HANGUL SYLLABLE THIEUTH E PIEUP
    ((0xC5, 0xDD), (0xD15F)),  # HANGUL SYLLABLE THIEUTH E SIOS
    ((0xC5, 0xDE), (0xD161)),  # HANGUL SYLLABLE THIEUTH E IEUNG
    ((0xC5, 0xDF), (0xD168)),  # HANGUL SYLLABLE THIEUTH YEO
    ((0xC5, 0xE0), (0xD16C)),  # HANGUL SYLLABLE THIEUTH YEO NIEUN
    ((0xC5, 0xE1), (0xD17C)),  # HANGUL SYLLABLE THIEUTH YEO SSANGSIOS
    ((0xC5, 0xE2), (0xD184)),  # HANGUL SYLLABLE THIEUTH YE
    ((0xC5, 0xE3), (0xD188)),  # HANGUL SYLLABLE THIEUTH YE NIEUN
    ((0xC5, 0xE4), (0xD1A0)),  # HANGUL SYLLABLE THIEUTH O
    ((0xC5, 0xE5), (0xD1A1)),  # HANGUL SYLLABLE THIEUTH O KIYEOK
    ((0xC5, 0xE6), (0xD1A4)),  # HANGUL SYLLABLE THIEUTH O NIEUN
    ((0xC5, 0xE7), (0xD1A8)),  # HANGUL SYLLABLE THIEUTH O RIEUL
    ((0xC5, 0xE8), (0xD1B0)),  # HANGUL SYLLABLE THIEUTH O MIEUM
    ((0xC5, 0xE9), (0xD1B1)),  # HANGUL SYLLABLE THIEUTH O PIEUP
    ((0xC5, 0xEA), (0xD1B3)),  # HANGUL SYLLABLE THIEUTH O SIOS
    ((0xC5, 0xEB), (0xD1B5)),  # HANGUL SYLLABLE THIEUTH O IEUNG
    ((0xC5, 0xEC), (0xD1BA)),  # HANGUL SYLLABLE THIEUTH O PHIEUPH
    ((0xC5, 0xED), (0xD1BC)),  # HANGUL SYLLABLE THIEUTH WA
    ((0xC5, 0xEE), (0xD1C0)),  # HANGUL SYLLABLE THIEUTH WA NIEUN
    ((0xC5, 0xEF), (0xD1D8)),  # HANGUL SYLLABLE THIEUTH WAE
    ((0xC5, 0xF0), (0xD1F4)),  # HANGUL SYLLABLE THIEUTH OE
    ((0xC5, 0xF1), (0xD1F8)),  # HANGUL SYLLABLE THIEUTH OE NIEUN
    ((0xC5, 0xF2), (0xD207)),  # HANGUL SYLLABLE THIEUTH OE SIOS
    ((0xC5, 0xF3), (0xD209)),  # HANGUL SYLLABLE THIEUTH OE IEUNG
    ((0xC5, 0xF4), (0xD210)),  # HANGUL SYLLABLE THIEUTH YO
    ((0xC5, 0xF5), (0xD22C)),  # HANGUL SYLLABLE THIEUTH U
    ((0xC5, 0xF6), (0xD22D)),  # HANGUL SYLLABLE THIEUTH U KIYEOK
    ((0xC5, 0xF7), (0xD230)),  # HANGUL SYLLABLE THIEUTH U NIEUN
    ((0xC5, 0xF8), (0xD234)),  # HANGUL SYLLABLE THIEUTH U RIEUL
    ((0xC5, 0xF9), (0xD23C)),  # HANGUL SYLLABLE THIEUTH U MIEUM
    ((0xC5, 0xFA), (0xD23D)),  # HANGUL SYLLABLE THIEUTH U PIEUP
    ((0xC5, 0xFB), (0xD23F)),  # HANGUL SYLLABLE THIEUTH U SIOS
    ((0xC5, 0xFC), (0xD241)),  # HANGUL SYLLABLE THIEUTH U IEUNG
    ((0xC5, 0xFD), (0xD248)),  # HANGUL SYLLABLE THIEUTH WEO
    ((0xC5, 0xFE), (0xD25C)),  # HANGUL SYLLABLE THIEUTH WEO SSANGSIOS
    ((0xC6, 0x41), (0xD78D)),  # HANGUL SYLLABLE HIEUH I NIEUNCIEUC
    ((0xC6, 0x42), (0xD78E)),  # HANGUL SYLLABLE HIEUH I NIEUNHIEUH
    ((0xC6, 0x43), (0xD78F)),  # HANGUL SYLLABLE HIEUH I TIKEUT
    ((0xC6, 0x44), (0xD791)),  # HANGUL SYLLABLE HIEUH I RIEULKIYEOK
    ((0xC6, 0x45), (0xD792)),  # HANGUL SYLLABLE HIEUH I RIEULMIEUM
    ((0xC6, 0x46), (0xD793)),  # HANGUL SYLLABLE HIEUH I RIEULPIEUP
    ((0xC6, 0x47), (0xD794)),  # HANGUL SYLLABLE HIEUH I RIEULSIOS
    ((0xC6, 0x48), (0xD795)),  # HANGUL SYLLABLE HIEUH I RIEULTHIEUTH
    ((0xC6, 0x49), (0xD796)),  # HANGUL SYLLABLE HIEUH I RIEULPHIEUPH
    ((0xC6, 0x4A), (0xD797)),  # HANGUL SYLLABLE HIEUH I RIEULHIEUH
    ((0xC6, 0x4B), (0xD79A)),  # HANGUL SYLLABLE HIEUH I PIEUPSIOS
    ((0xC6, 0x4C), (0xD79C)),  # HANGUL SYLLABLE HIEUH I SSANGSIOS
    ((0xC6, 0x4D), (0xD79E)),  # HANGUL SYLLABLE HIEUH I CIEUC
    ((0xC6, 0x4E), (0xD79F)),  # HANGUL SYLLABLE HIEUH I CHIEUCH
    ((0xC6, 0x4F), (0xD7A0)),  # HANGUL SYLLABLE HIEUH I KHIEUKH
    ((0xC6, 0x50), (0xD7A1)),  # HANGUL SYLLABLE HIEUH I THIEUTH
    ((0xC6, 0x51), (0xD7A2)),  # HANGUL SYLLABLE HIEUH I PHIEUPH
    ((0xC6, 0x52), (0xD7A3)),  # HANGUL SYLLABLE HIEUH I HIEUH
    ((0xC6, 0xA1), (0xD264)),  # HANGUL SYLLABLE THIEUTH WE
    ((0xC6, 0xA2), (0xD280)),  # HANGUL SYLLABLE THIEUTH WI
    ((0xC6, 0xA3), (0xD281)),  # HANGUL SYLLABLE THIEUTH WI KIYEOK
    ((0xC6, 0xA4), (0xD284)),  # HANGUL SYLLABLE THIEUTH WI NIEUN
    ((0xC6, 0xA5), (0xD288)),  # HANGUL SYLLABLE THIEUTH WI RIEUL
    ((0xC6, 0xA6), (0xD290)),  # HANGUL SYLLABLE THIEUTH WI MIEUM
    ((0xC6, 0xA7), (0xD291)),  # HANGUL SYLLABLE THIEUTH WI PIEUP
    ((0xC6, 0xA8), (0xD295)),  # HANGUL SYLLABLE THIEUTH WI IEUNG
    ((0xC6, 0xA9), (0xD29C)),  # HANGUL SYLLABLE THIEUTH YU
    ((0xC6, 0xAA), (0xD2A0)),  # HANGUL SYLLABLE THIEUTH YU NIEUN
    ((0xC6, 0xAB), (0xD2A4)),  # HANGUL SYLLABLE THIEUTH YU RIEUL
    ((0xC6, 0xAC), (0xD2AC)),  # HANGUL SYLLABLE THIEUTH YU MIEUM
    ((0xC6, 0xAD), (0xD2B1)),  # HANGUL SYLLABLE THIEUTH YU IEUNG
    ((0xC6, 0xAE), (0xD2B8)),  # HANGUL SYLLABLE THIEUTH EU
    ((0xC6, 0xAF), (0xD2B9)),  # HANGUL SYLLABLE THIEUTH EU KIYEOK
    ((0xC6, 0xB0), (0xD2BC)),  # HANGUL SYLLABLE THIEUTH EU NIEUN
    ((0xC6, 0xB1), (0xD2BF)),  # HANGUL SYLLABLE THIEUTH EU TIKEUT
    ((0xC6, 0xB2), (0xD2C0)),  # HANGUL SYLLABLE THIEUTH EU RIEUL
    ((0xC6, 0xB3), (0xD2C2)),  # HANGUL SYLLABLE THIEUTH EU RIEULMIEUM
    ((0xC6, 0xB4), (0xD2C8)),  # HANGUL SYLLABLE THIEUTH EU MIEUM
    ((0xC6, 0xB5), (0xD2C9)),  # HANGUL SYLLABLE THIEUTH EU PIEUP
    ((0xC6, 0xB6), (0xD2CB)),  # HANGUL SYLLABLE THIEUTH EU SIOS
    ((0xC6, 0xB7), (0xD2D4)),  # HANGUL SYLLABLE THIEUTH YI
    ((0xC6, 0xB8), (0xD2D8)),  # HANGUL SYLLABLE THIEUTH YI NIEUN
    ((0xC6, 0xB9), (0xD2DC)),  # HANGUL SYLLABLE THIEUTH YI RIEUL
    ((0xC6, 0xBA), (0xD2E4)),  # HANGUL SYLLABLE THIEUTH YI MIEUM
    ((0xC6, 0xBB), (0xD2E5)),  # HANGUL SYLLABLE THIEUTH YI PIEUP
    ((0xC6, 0xBC), (0xD2F0)),  # HANGUL SYLLABLE THIEUTH I
    ((0xC6, 0xBD), (0xD2F1)),  # HANGUL SYLLABLE THIEUTH I KIYEOK
    ((0xC6, 0xBE), (0xD2F4)),  # HANGUL SYLLABLE THIEUTH I NIEUN
    ((0xC6, 0xBF), (0xD2F8)),  # HANGUL SYLLABLE THIEUTH I RIEUL
    ((0xC6, 0xC0), (0xD300)),  # HANGUL SYLLABLE THIEUTH I MIEUM
    ((0xC6, 0xC1), (0xD301)),  # HANGUL SYLLABLE THIEUTH I PIEUP
    ((0xC6, 0xC2), (0xD303)),  # HANGUL SYLLABLE THIEUTH I SIOS
    ((0xC6, 0xC3), (0xD305)),  # HANGUL SYLLABLE THIEUTH I IEUNG
    ((0xC6, 0xC4), (0xD30C)),  # HANGUL SYLLABLE PHIEUPH A
    ((0xC6, 0xC5), (0xD30D)),  # HANGUL SYLLABLE PHIEUPH A KIYEOK
    ((0xC6, 0xC6), (0xD30E)),  # HANGUL SYLLABLE PHIEUPH A SSANGKIYEOK
    ((0xC6, 0xC7), (0xD310)),  # HANGUL SYLLABLE PHIEUPH A NIEUN
    ((0xC6, 0xC8), (0xD314)),  # HANGUL SYLLABLE PHIEUPH A RIEUL
    ((0xC6, 0xC9), (0xD316)),  # HANGUL SYLLABLE PHIEUPH A RIEULMIEUM
    ((0xC6, 0xCA), (0xD31C)),  # HANGUL SYLLABLE PHIEUPH A MIEUM
    ((0xC6, 0xCB), (0xD31D)),  # HANGUL SYLLABLE PHIEUPH A PIEUP
    ((0xC6, 0xCC), (0xD31F)),  # HANGUL SYLLABLE PHIEUPH A SIOS
    ((0xC6, 0xCD), (0xD320)),  # HANGUL SYLLABLE PHIEUPH A SSANGSIOS
    ((0xC6, 0xCE), (0xD321)),  # HANGUL SYLLABLE PHIEUPH A IEUNG
    ((0xC6, 0xCF), (0xD325)),  # HANGUL SYLLABLE PHIEUPH A THIEUTH
    ((0xC6, 0xD0), (0xD328)),  # HANGUL SYLLABLE PHIEUPH AE
    ((0xC6, 0xD1), (0xD329)),  # HANGUL SYLLABLE PHIEUPH AE KIYEOK
    ((0xC6, 0xD2), (0xD32C)),  # HANGUL SYLLABLE PHIEUPH AE NIEUN
    ((0xC6, 0xD3), (0xD330)),  # HANGUL SYLLABLE PHIEUPH AE RIEUL
    ((0xC6, 0xD4), (0xD338)),  # HANGUL SYLLABLE PHIEUPH AE MIEUM
    ((0xC6, 0xD5), (0xD339)),  # HANGUL SYLLABLE PHIEUPH AE PIEUP
    ((0xC6, 0xD6), (0xD33B)),  # HANGUL SYLLABLE PHIEUPH AE SIOS
    ((0xC6, 0xD7), (0xD33C)),  # HANGUL SYLLABLE PHIEUPH AE SSANGSIOS
    ((0xC6, 0xD8), (0xD33D)),  # HANGUL SYLLABLE PHIEUPH AE IEUNG
    ((0xC6, 0xD9), (0xD344)),  # HANGUL SYLLABLE PHIEUPH YA
    ((0xC6, 0xDA), (0xD345)),  # HANGUL SYLLABLE PHIEUPH YA KIYEOK
    ((0xC6, 0xDB), (0xD37C)),  # HANGUL SYLLABLE PHIEUPH EO
    ((0xC6, 0xDC), (0xD37D)),  # HANGUL SYLLABLE PHIEUPH EO KIYEOK
    ((0xC6, 0xDD), (0xD380)),  # HANGUL SYLLABLE PHIEUPH EO NIEUN
    ((0xC6, 0xDE), (0xD384)),  # HANGUL SYLLABLE PHIEUPH EO RIEUL
    ((0xC6, 0xDF), (0xD38C)),  # HANGUL SYLLABLE PHIEUPH EO MIEUM
    ((0xC6, 0xE0), (0xD38D)),  # HANGUL SYLLABLE PHIEUPH EO PIEUP
    ((0xC6, 0xE1), (0xD38F)),  # HANGUL SYLLABLE PHIEUPH EO SIOS
    ((0xC6, 0xE2), (0xD390)),  # HANGUL SYLLABLE PHIEUPH EO SSANGSIOS
    ((0xC6, 0xE3), (0xD391)),  # HANGUL SYLLABLE PHIEUPH EO IEUNG
    ((0xC6, 0xE4), (0xD398)),  # HANGUL SYLLABLE PHIEUPH E
    ((0xC6, 0xE5), (0xD399)),  # HANGUL SYLLABLE PHIEUPH E KIYEOK
    ((0xC6, 0xE6), (0xD39C)),  # HANGUL SYLLABLE PHIEUPH E NIEUN
    ((0xC6, 0xE7), (0xD3A0)),  # HANGUL SYLLABLE PHIEUPH E RIEUL
    ((0xC6, 0xE8), (0xD3A8)),  # HANGUL SYLLABLE PHIEUPH E MIEUM
    ((0xC6, 0xE9), (0xD3A9)),  # HANGUL SYLLABLE PHIEUPH E PIEUP
    ((0xC6, 0xEA), (0xD3AB)),  # HANGUL SYLLABLE PHIEUPH E SIOS
    ((0xC6, 0xEB), (0xD3AD)),  # HANGUL SYLLABLE PHIEUPH E IEUNG
    ((0xC6, 0xEC), (0xD3B4)),  # HANGUL SYLLABLE PHIEUPH YEO
    ((0xC6, 0xED), (0xD3B8)),  # HANGUL SYLLABLE PHIEUPH YEO NIEUN
    ((0xC6, 0xEE), (0xD3BC)),  # HANGUL SYLLABLE PHIEUPH YEO RIEUL
    ((0xC6, 0xEF), (0xD3C4)),  # HANGUL SYLLABLE PHIEUPH YEO MIEUM
    ((0xC6, 0xF0), (0xD3C5)),  # HANGUL SYLLABLE PHIEUPH YEO PIEUP
    ((0xC6, 0xF1), (0xD3C8)),  # HANGUL SYLLABLE PHIEUPH YEO SSANGSIOS
    ((0xC6, 0xF2), (0xD3C9)),  # HANGUL SYLLABLE PHIEUPH YEO IEUNG
    ((0xC6, 0xF3), (0xD3D0)),  # HANGUL SYLLABLE PHIEUPH YE
    ((0xC6, 0xF4), (0xD3D8)),  # HANGUL SYLLABLE PHIEUPH YE RIEUL
    ((0xC6, 0xF5), (0xD3E1)),  # HANGUL SYLLABLE PHIEUPH YE PIEUP
    ((0xC6, 0xF6), (0xD3E3)),  # HANGUL SYLLABLE PHIEUPH YE SIOS
    ((0xC6, 0xF7), (0xD3EC)),  # HANGUL SYLLABLE PHIEUPH O
    ((0xC6, 0xF8), (0xD3ED)),  # HANGUL SYLLABLE PHIEUPH O KIYEOK
    ((0xC6, 0xF9), (0xD3F0)),  # HANGUL SYLLABLE PHIEUPH O NIEUN
    ((0xC6, 0xFA), (0xD3F4)),  # HANGUL SYLLABLE PHIEUPH O RIEUL
    ((0xC6, 0xFB), (0xD3FC)),  # HANGUL SYLLABLE PHIEUPH O MIEUM
    ((0xC6, 0xFC), (0xD3FD)),  # HANGUL SYLLABLE PHIEUPH O PIEUP
    ((0xC6, 0xFD), (0xD3FF)),  # HANGUL SYLLABLE PHIEUPH O SIOS
    ((0xC6, 0xFE), (0xD401)),  # HANGUL SYLLABLE PHIEUPH O IEUNG
    ((0xC7, 0xA1), (0xD408)),  # HANGUL SYLLABLE PHIEUPH WA
    ((0xC7, 0xA2), (0xD41D)),  # HANGUL SYLLABLE PHIEUPH WA IEUNG
    ((0xC7, 0xA3), (0xD440)),  # HANGUL SYLLABLE PHIEUPH OE
    ((0xC7, 0xA4), (0xD444)),  # HANGUL SYLLABLE PHIEUPH OE NIEUN
    ((0xC7, 0xA5), (0xD45C)),  # HANGUL SYLLABLE PHIEUPH YO
    ((0xC7, 0xA6), (0xD460)),  # HANGUL SYLLABLE PHIEUPH YO NIEUN
    ((0xC7, 0xA7), (0xD464)),  # HANGUL SYLLABLE PHIEUPH YO RIEUL
    ((0xC7, 0xA8), (0xD46D)),  # HANGUL SYLLABLE PHIEUPH YO PIEUP
    ((0xC7, 0xA9), (0xD46F)),  # HANGUL SYLLABLE PHIEUPH YO SIOS
    ((0xC7, 0xAA), (0xD478)),  # HANGUL SYLLABLE PHIEUPH U
    ((0xC7, 0xAB), (0xD479)),  # HANGUL SYLLABLE PHIEUPH U KIYEOK
    ((0xC7, 0xAC), (0xD47C)),  # HANGUL SYLLABLE PHIEUPH U NIEUN
    ((0xC7, 0xAD), (0xD47F)),  # HANGUL SYLLABLE PHIEUPH U TIKEUT
    ((0xC7, 0xAE), (0xD480)),  # HANGUL SYLLABLE PHIEUPH U RIEUL
    ((0xC7, 0xAF), (0xD482)),  # HANGUL SYLLABLE PHIEUPH U RIEULMIEUM
    ((0xC7, 0xB0), (0xD488)),  # HANGUL SYLLABLE PHIEUPH U MIEUM
    ((0xC7, 0xB1), (0xD489)),  # HANGUL SYLLABLE PHIEUPH U PIEUP
    ((0xC7, 0xB2), (0xD48B)),  # HANGUL SYLLABLE PHIEUPH U SIOS
    ((0xC7, 0xB3), (0xD48D)),  # HANGUL SYLLABLE PHIEUPH U IEUNG
    ((0xC7, 0xB4), (0xD494)),  # HANGUL SYLLABLE PHIEUPH WEO
    ((0xC7, 0xB5), (0xD4A9)),  # HANGUL SYLLABLE PHIEUPH WEO IEUNG
    ((0xC7, 0xB6), (0xD4CC)),  # HANGUL SYLLABLE PHIEUPH WI
    ((0xC7, 0xB7), (0xD4D0)),  # HANGUL SYLLABLE PHIEUPH WI NIEUN
    ((0xC7, 0xB8), (0xD4D4)),  # HANGUL SYLLABLE PHIEUPH WI RIEUL
    ((0xC7, 0xB9), (0xD4DC)),  # HANGUL SYLLABLE PHIEUPH WI MIEUM
    ((0xC7, 0xBA), (0xD4DF)),  # HANGUL SYLLABLE PHIEUPH WI SIOS
    ((0xC7, 0xBB), (0xD4E8)),  # HANGUL SYLLABLE PHIEUPH YU
    ((0xC7, 0xBC), (0xD4EC)),  # HANGUL SYLLABLE PHIEUPH YU NIEUN
    ((0xC7, 0xBD), (0xD4F0)),  # HANGUL SYLLABLE PHIEUPH YU RIEUL
    ((0xC7, 0xBE), (0xD4F8)),  # HANGUL SYLLABLE PHIEUPH YU MIEUM
    ((0xC7, 0xBF), (0xD4FB)),  # HANGUL SYLLABLE PHIEUPH YU SIOS
    ((0xC7, 0xC0), (0xD4FD)),  # HANGUL SYLLABLE PHIEUPH YU IEUNG
    ((0xC7, 0xC1), (0xD504)),  # HANGUL SYLLABLE PHIEUPH EU
    ((0xC7, 0xC2), (0xD508)),  # HANGUL SYLLABLE PHIEUPH EU NIEUN
    ((0xC7, 0xC3), (0xD50C)),  # HANGUL SYLLABLE PHIEUPH EU RIEUL
    ((0xC7, 0xC4), (0xD514)),  # HANGUL SYLLABLE PHIEUPH EU MIEUM
    ((0xC7, 0xC5), (0xD515)),  # HANGUL SYLLABLE PHIEUPH EU PIEUP
    ((0xC7, 0xC6), (0xD517)),  # HANGUL SYLLABLE PHIEUPH EU SIOS
    ((0xC7, 0xC7), (0xD53C)),  # HANGUL SYLLABLE PHIEUPH I
    ((0xC7, 0xC8), (0xD53D)),  # HANGUL SYLLABLE PHIEUPH I KIYEOK
    ((0xC7, 0xC9), (0xD540)),  # HANGUL SYLLABLE PHIEUPH I NIEUN
    ((0xC7, 0xCA), (0xD544)),  # HANGUL SYLLABLE PHIEUPH I RIEUL
    ((0xC7, 0xCB), (0xD54C)),  # HANGUL SYLLABLE PHIEUPH I MIEUM
    ((0xC7, 0xCC), (0xD54D)),  # HANGUL SYLLABLE PHIEUPH I PIEUP
    ((0xC7, 0xCD), (0xD54F)),  # HANGUL SYLLABLE PHIEUPH I SIOS
    ((0xC7, 0xCE), (0xD551)),  # HANGUL SYLLABLE PHIEUPH I IEUNG
    ((0xC7, 0xCF), (0xD558)),  # HANGUL SYLLABLE HIEUH A
    ((0xC7, 0xD0), (0xD559)),  # HANGUL SYLLABLE HIEUH A KIYEOK
    ((0xC7, 0xD1), (0xD55C)),  # HANGUL SYLLABLE HIEUH A NIEUN
    ((0xC7, 0xD2), (0xD560)),  # HANGUL SYLLABLE HIEUH A RIEUL
    ((0xC7, 0xD3), (0xD565)),  # HANGUL SYLLABLE HIEUH A RIEULTHIEUTH
    ((0xC7, 0xD4), (0xD568)),  # HANGUL SYLLABLE HIEUH A MIEUM
    ((0xC7, 0xD5), (0xD569)),  # HANGUL SYLLABLE HIEUH A PIEUP
    ((0xC7, 0xD6), (0xD56B)),  # HANGUL SYLLABLE HIEUH A SIOS
    ((0xC7, 0xD7), (0xD56D)),  # HANGUL SYLLABLE HIEUH A IEUNG
    ((0xC7, 0xD8), (0xD574)),  # HANGUL SYLLABLE HIEUH AE
    ((0xC7, 0xD9), (0xD575)),  # HANGUL SYLLABLE HIEUH AE KIYEOK
    ((0xC7, 0xDA), (0xD578)),  # HANGUL SYLLABLE HIEUH AE NIEUN
    ((0xC7, 0xDB), (0xD57C)),  # HANGUL SYLLABLE HIEUH AE RIEUL
    ((0xC7, 0xDC), (0xD584)),  # HANGUL SYLLABLE HIEUH AE MIEUM
    ((0xC7, 0xDD), (0xD585)),  # HANGUL SYLLABLE HIEUH AE PIEUP
    ((0xC7, 0xDE), (0xD587)),  # HANGUL SYLLABLE HIEUH AE SIOS
    ((0xC7, 0xDF), (0xD588)),  # HANGUL SYLLABLE HIEUH AE SSANGSIOS
    ((0xC7, 0xE0), (0xD589)),  # HANGUL SYLLABLE HIEUH AE IEUNG
    ((0xC7, 0xE1), (0xD590)),  # HANGUL SYLLABLE HIEUH YA
    ((0xC7, 0xE2), (0xD5A5)),  # HANGUL SYLLABLE HIEUH YA IEUNG
    ((0xC7, 0xE3), (0xD5C8)),  # HANGUL SYLLABLE HIEUH EO
    ((0xC7, 0xE4), (0xD5C9)),  # HANGUL SYLLABLE HIEUH EO KIYEOK
    ((0xC7, 0xE5), (0xD5CC)),  # HANGUL SYLLABLE HIEUH EO NIEUN
    ((0xC7, 0xE6), (0xD5D0)),  # HANGUL SYLLABLE HIEUH EO RIEUL
    ((0xC7, 0xE7), (0xD5D2)),  # HANGUL SYLLABLE HIEUH EO RIEULMIEUM
    ((0xC7, 0xE8), (0xD5D8)),  # HANGUL SYLLABLE HIEUH EO MIEUM
    ((0xC7, 0xE9), (0xD5D9)),  # HANGUL SYLLABLE HIEUH EO PIEUP
    ((0xC7, 0xEA), (0xD5DB)),  # HANGUL SYLLABLE HIEUH EO SIOS
    ((0xC7, 0xEB), (0xD5DD)),  # HANGUL SYLLABLE HIEUH EO IEUNG
    ((0xC7, 0xEC), (0xD5E4)),  # HANGUL SYLLABLE HIEUH E
    ((0xC7, 0xED), (0xD5E5)),  # HANGUL SYLLABLE HIEUH E KIYEOK
    ((0xC7, 0xEE), (0xD5E8)),  # HANGUL SYLLABLE HIEUH E NIEUN
    ((0xC7, 0xEF), (0xD5EC)),  # HANGUL SYLLABLE HIEUH E RIEUL
    ((0xC7, 0xF0), (0xD5F4)),  # HANGUL SYLLABLE HIEUH E MIEUM
    ((0xC7, 0xF1), (0xD5F5)),  # HANGUL SYLLABLE HIEUH E PIEUP
    ((0xC7, 0xF2), (0xD5F7)),  # HANGUL SYLLABLE HIEUH E SIOS
    ((0xC7, 0xF3), (0xD5F9)),  # HANGUL SYLLABLE HIEUH E IEUNG
    ((0xC7, 0xF4), (0xD600)),  # HANGUL SYLLABLE HIEUH YEO
    ((0xC7, 0xF5), (0xD601)),  # HANGUL SYLLABLE HIEUH YEO KIYEOK
    ((0xC7, 0xF6), (0xD604)),  # HANGUL SYLLABLE HIEUH YEO NIEUN
    ((0xC7, 0xF7), (0xD608)),  # HANGUL SYLLABLE HIEUH YEO RIEUL
    ((0xC7, 0xF8), (0xD610)),  # HANGUL SYLLABLE HIEUH YEO MIEUM
    ((0xC7, 0xF9), (0xD611)),  # HANGUL SYLLABLE HIEUH YEO PIEUP
    ((0xC7, 0xFA), (0xD613)),  # HANGUL SYLLABLE HIEUH YEO SIOS
    ((0xC7, 0xFB), (0xD614)),  # HANGUL SYLLABLE HIEUH YEO SSANGSIOS
    ((0xC7, 0xFC), (0xD615)),  # HANGUL SYLLABLE HIEUH YEO IEUNG
    ((0xC7, 0xFD), (0xD61C)),  # HANGUL SYLLABLE HIEUH YE
    ((0xC7, 0xFE), (0xD620)),  # HANGUL SYLLABLE HIEUH YE NIEUN
    ((0xC8, 0xA1), (0xD624)),  # HANGUL SYLLABLE HIEUH YE RIEUL
    ((0xC8, 0xA2), (0xD62D)),  # HANGUL SYLLABLE HIEUH YE PIEUP
    ((0xC8, 0xA3), (0xD638)),  # HANGUL SYLLABLE HIEUH O
    ((0xC8, 0xA4), (0xD639)),  # HANGUL SYLLABLE HIEUH O KIYEOK
    ((0xC8, 0xA5), (0xD63C)),  # HANGUL SYLLABLE HIEUH O NIEUN
    ((0xC8, 0xA6), (0xD640)),  # HANGUL SYLLABLE HIEUH O RIEUL
    ((0xC8, 0xA7), (0xD645)),  # HANGUL SYLLABLE HIEUH O RIEULTHIEUTH
    ((0xC8, 0xA8), (0xD648)),  # HANGUL SYLLABLE HIEUH O MIEUM
    ((0xC8, 0xA9), (0xD649)),  # HANGUL SYLLABLE HIEUH O PIEUP
    ((0xC8, 0xAA), (0xD64B)),  # HANGUL SYLLABLE HIEUH O SIOS
    ((0xC8, 0xAB), (0xD64D)),  # HANGUL SYLLABLE HIEUH O IEUNG
    ((0xC8, 0xAC), (0xD651)),  # HANGUL SYLLABLE HIEUH O THIEUTH
    ((0xC8, 0xAD), (0xD654)),  # HANGUL SYLLABLE HIEUH WA
    ((0xC8, 0xAE), (0xD655)),  # HANGUL SYLLABLE HIEUH WA KIYEOK
    ((0xC8, 0xAF), (0xD658)),  # HANGUL SYLLABLE HIEUH WA NIEUN
    ((0xC8, 0xB0), (0xD65C)),  # HANGUL SYLLABLE HIEUH WA RIEUL
    ((0xC8, 0xB1), (0xD667)),  # HANGUL SYLLABLE HIEUH WA SIOS
    ((0xC8, 0xB2), (0xD669)),  # HANGUL SYLLABLE HIEUH WA IEUNG
    ((0xC8, 0xB3), (0xD670)),  # HANGUL SYLLABLE HIEUH WAE
    ((0xC8, 0xB4), (0xD671)),  # HANGUL SYLLABLE HIEUH WAE KIYEOK
    ((0xC8, 0xB5), (0xD674)),  # HANGUL SYLLABLE HIEUH WAE NIEUN
    ((0xC8, 0xB6), (0xD683)),  # HANGUL SYLLABLE HIEUH WAE SIOS
    ((0xC8, 0xB7), (0xD685)),  # HANGUL SYLLABLE HIEUH WAE IEUNG
    ((0xC8, 0xB8), (0xD68C)),  # HANGUL SYLLABLE HIEUH OE
    ((0xC8, 0xB9), (0xD68D)),  # HANGUL SYLLABLE HIEUH OE KIYEOK
    ((0xC8, 0xBA), (0xD690)),  # HANGUL SYLLABLE HIEUH OE NIEUN
    ((0xC8, 0xBB), (0xD694)),  # HANGUL SYLLABLE HIEUH OE RIEUL
    ((0xC8, 0xBC), (0xD69D)),  # HANGUL SYLLABLE HIEUH OE PIEUP
    ((0xC8, 0xBD), (0xD69F)),  # HANGUL SYLLABLE HIEUH OE SIOS
    ((0xC8, 0xBE), (0xD6A1)),  # HANGUL SYLLABLE HIEUH OE IEUNG
    ((0xC8, 0xBF), (0xD6A8)),  # HANGUL SYLLABLE HIEUH YO
    ((0xC8, 0xC0), (0xD6AC)),  # HANGUL SYLLABLE HIEUH YO NIEUN
    ((0xC8, 0xC1), (0xD6B0)),  # HANGUL SYLLABLE HIEUH YO RIEUL
    ((0xC8, 0xC2), (0xD6B9)),  # HANGUL SYLLABLE HIEUH YO PIEUP
    ((0xC8, 0xC3), (0xD6BB)),  # HANGUL SYLLABLE HIEUH YO SIOS
    ((0xC8, 0xC4), (0xD6C4)),  # HANGUL SYLLABLE HIEUH U
    ((0xC8, 0xC5), (0xD6C5)),  # HANGUL SYLLABLE HIEUH U KIYEOK
    ((0xC8, 0xC6), (0xD6C8)),  # HANGUL SYLLABLE HIEUH U NIEUN
    ((0xC8, 0xC7), (0xD6CC)),  # HANGUL SYLLABLE HIEUH U RIEUL
    ((0xC8, 0xC8), (0xD6D1)),  # HANGUL SYLLABLE HIEUH U RIEULTHIEUTH
    ((0xC8, 0xC9), (0xD6D4)),  # HANGUL SYLLABLE HIEUH U MIEUM
    ((0xC8, 0xCA), (0xD6D7)),  # HANGUL SYLLABLE HIEUH U SIOS
    ((0xC8, 0xCB), (0xD6D9)),  # HANGUL SYLLABLE HIEUH U IEUNG
    ((0xC8, 0xCC), (0xD6E0)),  # HANGUL SYLLABLE HIEUH WEO
    ((0xC8, 0xCD), (0xD6E4)),  # HANGUL SYLLABLE HIEUH WEO NIEUN
    ((0xC8, 0xCE), (0xD6E8)),  # HANGUL SYLLABLE HIEUH WEO RIEUL
    ((0xC8, 0xCF), (0xD6F0)),  # HANGUL SYLLABLE HIEUH WEO MIEUM
    ((0xC8, 0xD0), (0xD6F5)),  # HANGUL SYLLABLE HIEUH WEO IEUNG
    ((0xC8, 0xD1), (0xD6FC)),  # HANGUL SYLLABLE HIEUH WE
    ((0xC8, 0xD2), (0xD6FD)),  # HANGUL SYLLABLE HIEUH WE KIYEOK
    ((0xC8, 0xD3), (0xD700)),  # HANGUL SYLLABLE HIEUH WE NIEUN
    ((0xC8, 0xD4), (0xD704)),  # HANGUL SYLLABLE HIEUH WE RIEUL
    ((0xC8, 0xD5), (0xD711)),  # HANGUL SYLLABLE HIEUH WE IEUNG
    ((0xC8, 0xD6), (0xD718)),  # HANGUL SYLLABLE HIEUH WI
    ((0xC8, 0xD7), (0xD719)),  # HANGUL SYLLABLE HIEUH WI KIYEOK
    ((0xC8, 0xD8), (0xD71C)),  # HANGUL SYLLABLE HIEUH WI NIEUN
    ((0xC8, 0xD9), (0xD720)),  # HANGUL SYLLABLE HIEUH WI RIEUL
    ((0xC8, 0xDA), (0xD728)),  # HANGUL SYLLABLE HIEUH WI MIEUM
    ((0xC8, 0xDB), (0xD729)),  # HANGUL SYLLABLE HIEUH WI PIEUP
    ((0xC8, 0xDC), (0xD72B)),  # HANGUL SYLLABLE HIEUH WI SIOS
    ((0xC8, 0xDD), (0xD72D)),  # HANGUL SYLLABLE HIEUH WI IEUNG
    ((0xC8, 0xDE), (0xD734)),  # HANGUL SYLLABLE HIEUH YU
    ((0xC8, 0xDF), (0xD735)),  # HANGUL SYLLABLE HIEUH YU KIYEOK
    ((0xC8, 0xE0), (0xD738)),  # HANGUL SYLLABLE HIEUH YU NIEUN
    ((0xC8, 0xE1), (0xD73C)),  # HANGUL SYLLABLE HIEUH YU RIEUL
    ((0xC8, 0xE2), (0xD744)),  # HANGUL SYLLABLE HIEUH YU MIEUM
    ((0xC8, 0xE3), (0xD747)),  # HANGUL SYLLABLE HIEUH YU SIOS
    ((0xC8, 0xE4), (0xD749)),  # HANGUL SYLLABLE HIEUH YU IEUNG
    ((0xC8, 0xE5), (0xD750)),  # HANGUL SYLLABLE HIEUH EU
    ((0xC8, 0xE6), (0xD751)),  # HANGUL SYLLABLE HIEUH EU KIYEOK
    ((0xC8, 0xE7), (0xD754)),  # HANGUL SYLLABLE HIEUH EU NIEUN
    ((0xC8, 0xE8), (0xD756)),  # HANGUL SYLLABLE HIEUH EU NIEUNHIEUH
    ((0xC8, 0xE9), (0xD757)),  # HANGUL SYLLABLE HIEUH EU TIKEUT
    ((0xC8, 0xEA), (0xD758)),  # HANGUL SYLLABLE HIEUH EU RIEUL
    ((0xC8, 0xEB), (0xD759)),  # HANGUL SYLLABLE HIEUH EU RIEULKIYEOK
    ((0xC8, 0xEC), (0xD760)),  # HANGUL SYLLABLE HIEUH EU MIEUM
    ((0xC8, 0xED), (0xD761)),  # HANGUL SYLLABLE HIEUH EU PIEUP
    ((0xC8, 0xEE), (0xD763)),  # HANGUL SYLLABLE HIEUH EU SIOS
    ((0xC8, 0xEF), (0xD765)),  # HANGUL SYLLABLE HIEUH EU IEUNG
    ((0xC8, 0xF0), (0xD769)),  # HANGUL SYLLABLE HIEUH EU THIEUTH
    ((0xC8, 0xF1), (0xD76C)),  # HANGUL SYLLABLE HIEUH YI
    ((0xC8, 0xF2), (0xD770)),  # HANGUL SYLLABLE HIEUH YI NIEUN
    ((0xC8, 0xF3), (0xD774)),  # HANGUL SYLLABLE HIEUH YI RIEUL
    ((0xC8, 0xF4), (0xD77C)),  # HANGUL SYLLABLE HIEUH YI MIEUM
    ((0xC8, 0xF5), (0xD77D)),  # HANGUL SYLLABLE HIEUH YI PIEUP
    ((0xC8, 0xF6), (0xD781)),  # HANGUL SYLLABLE HIEUH YI IEUNG
    ((0xC8, 0xF7), (0xD788)),  # HANGUL SYLLABLE HIEUH I
    ((0xC8, 0xF8), (0xD789)),  # HANGUL SYLLABLE HIEUH I KIYEOK
    ((0xC8, 0xF9), (0xD78C)),  # HANGUL SYLLABLE HIEUH I NIEUN
    ((0xC8, 0xFA), (0xD790)),  # HANGUL SYLLABLE HIEUH I RIEUL
    ((0xC8, 0xFB), (0xD798)),  # HANGUL SYLLABLE HIEUH I MIEUM
    ((0xC8, 0xFC), (0xD799)),  # HANGUL SYLLABLE HIEUH I PIEUP
    ((0xC8, 0xFD), (0xD79B)),  # HANGUL SYLLABLE HIEUH I SIOS
    ((0xC8, 0xFE), (0xD79D)),  # HANGUL SYLLABLE HIEUH I IEUNG
    ((0xCA, 0xA1), (0x4F3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA2), (0x4F73)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA3), (0x5047)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA4), (0x50F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA5), (0x52A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA6), (0x53EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA7), (0x5475)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA8), (0x54E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xA9), (0x5609)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAA), (0x5AC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAB), (0x5BB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAC), (0x6687)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAD), (0x67B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAE), (0x67B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xAF), (0x67EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB0), (0x6B4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB1), (0x73C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB2), (0x75C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB3), (0x7A3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB4), (0x82DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB5), (0x8304)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB6), (0x8857)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB7), (0x8888)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB8), (0x8A36)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xB9), (0x8CC8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBA), (0x8DCF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBB), (0x8EFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBC), (0x8FE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBD), (0x99D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBE), (0x523B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xBF), (0x5374)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC0), (0x5404)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC1), (0x606A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC2), (0x6164)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC3), (0x6BBC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC4), (0x73CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC5), (0x811A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC6), (0x89BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC7), (0x89D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC8), (0x95A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xC9), (0x4F83)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCA), (0x520A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCB), (0x58BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCC), (0x5978)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCD), (0x59E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCE), (0x5E72)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xCF), (0x5E79)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD0), (0x61C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD1), (0x63C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD2), (0x6746)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD3), (0x67EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD4), (0x687F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD5), (0x6F97)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD6), (0x764E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD7), (0x770B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD8), (0x78F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xD9), (0x7A08)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDA), (0x7AFF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDB), (0x7C21)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDC), (0x809D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDD), (0x826E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDE), (0x8271)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xDF), (0x8AEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE0), (0x9593)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE1), (0x4E6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE2), (0x559D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE3), (0x66F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE4), (0x6E34)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE5), (0x78A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE6), (0x7AED)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE7), (0x845B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE8), (0x8910)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xE9), (0x874E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xEA), (0x97A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xEB), (0x52D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xEC), (0x574E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xED), (0x582A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xEE), (0x5D4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xEF), (0x611F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF0), (0x61BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF1), (0x6221)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF2), (0x6562)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF3), (0x67D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF4), (0x6A44)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF5), (0x6E1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF6), (0x7518)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF7), (0x75B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF8), (0x76E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xF9), (0x77B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xFA), (0x7D3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xFB), (0x90AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xFC), (0x9451)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xFD), (0x9452)),  # CJK UNIFIED IDEOGRAPH
    ((0xCA, 0xFE), (0x9F95)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA1), (0x5323)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA2), (0x5CAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA3), (0x7532)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA4), (0x80DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA5), (0x9240)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA6), (0x9598)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA7), (0x525B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA8), (0x5808)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xA9), (0x59DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAA), (0x5CA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAB), (0x5D17)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAC), (0x5EB7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAD), (0x5F3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAE), (0x5F4A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xAF), (0x6177)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB0), (0x6C5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB1), (0x757A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB2), (0x7586)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB3), (0x7CE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB4), (0x7D73)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB5), (0x7DB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB6), (0x7F8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB7), (0x8154)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB8), (0x8221)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xB9), (0x8591)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBA), (0x8941)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBB), (0x8B1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBC), (0x92FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBD), (0x964D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBE), (0x9C47)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xBF), (0x4ECB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC0), (0x4EF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC1), (0x500B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC2), (0x51F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC3), (0x584F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC4), (0x6137)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC5), (0x613E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC6), (0x6168)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC7), (0x6539)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC8), (0x69EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xC9), (0x6F11)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCA), (0x75A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCB), (0x7686)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCC), (0x76D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCD), (0x7B87)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCE), (0x82A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xCF), (0x84CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD0), (0xF900)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCB, 0xD1), (0x93A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD2), (0x958B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD3), (0x5580)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD4), (0x5BA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD5), (0x5751)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD6), (0xF901)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCB, 0xD7), (0x7CB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD8), (0x7FB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xD9), (0x91B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDA), (0x5028)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDB), (0x53BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDC), (0x5C45)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDD), (0x5DE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDE), (0x62D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xDF), (0x636E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE0), (0x64DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE1), (0x64E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE2), (0x6E20)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE3), (0x70AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE4), (0x795B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE5), (0x8DDD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE6), (0x8E1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE7), (0xF902)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCB, 0xE8), (0x907D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xE9), (0x9245)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xEA), (0x92F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xEB), (0x4E7E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xEC), (0x4EF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xED), (0x5065)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xEE), (0x5DFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xEF), (0x5EFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF0), (0x6106)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF1), (0x6957)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF2), (0x8171)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF3), (0x8654)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF4), (0x8E47)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF5), (0x9375)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF6), (0x9A2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF7), (0x4E5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF8), (0x5091)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xF9), (0x6770)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xFA), (0x6840)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xFB), (0x5109)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xFC), (0x528D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xFD), (0x5292)),  # CJK UNIFIED IDEOGRAPH
    ((0xCB, 0xFE), (0x6AA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA1), (0x77BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA2), (0x9210)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA3), (0x9ED4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA4), (0x52AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA5), (0x602F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA6), (0x8FF2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA7), (0x5048)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA8), (0x61A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xA9), (0x63ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAA), (0x64CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAB), (0x683C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAC), (0x6A84)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAD), (0x6FC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAE), (0x8188)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xAF), (0x89A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB0), (0x9694)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB1), (0x5805)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB2), (0x727D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB3), (0x72AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB4), (0x7504)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB5), (0x7D79)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB6), (0x7E6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB7), (0x80A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB8), (0x898B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xB9), (0x8B74)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBA), (0x9063)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBB), (0x9D51)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBC), (0x6289)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBD), (0x6C7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBE), (0x6F54)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xBF), (0x7D50)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC0), (0x7F3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC1), (0x8A23)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC2), (0x517C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC3), (0x614A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC4), (0x7B9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC5), (0x8B19)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC6), (0x9257)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC7), (0x938C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC8), (0x4EAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xC9), (0x4FD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCA), (0x501E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCB), (0x50BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCC), (0x5106)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCD), (0x52C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCE), (0x52CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xCF), (0x537F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD0), (0x5770)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD1), (0x5883)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD2), (0x5E9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD3), (0x5F91)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD4), (0x6176)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD5), (0x61AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD6), (0x64CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD7), (0x656C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD8), (0x666F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xD9), (0x66BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDA), (0x66F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDB), (0x6897)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDC), (0x6D87)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDD), (0x7085)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDE), (0x70F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xDF), (0x749F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE0), (0x74A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE1), (0x74CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE2), (0x75D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE3), (0x786C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE4), (0x78EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE5), (0x7ADF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE6), (0x7AF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE7), (0x7D45)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE8), (0x7D93)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xE9), (0x8015)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xEA), (0x803F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xEB), (0x811B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xEC), (0x8396)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xED), (0x8B66)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xEE), (0x8F15)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xEF), (0x9015)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF0), (0x93E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF1), (0x9803)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF2), (0x9838)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF3), (0x9A5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF4), (0x9BE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF5), (0x4FC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF6), (0x5553)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF7), (0x583A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF8), (0x5951)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xF9), (0x5B63)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xFA), (0x5C46)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xFB), (0x60B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xFC), (0x6212)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xFD), (0x6842)),  # CJK UNIFIED IDEOGRAPH
    ((0xCC, 0xFE), (0x68B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA1), (0x68E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA2), (0x6EAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA3), (0x754C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA4), (0x7678)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA5), (0x78CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA6), (0x7A3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA7), (0x7CFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA8), (0x7E6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xA9), (0x7E7C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAA), (0x8A08)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAB), (0x8AA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAC), (0x8C3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAD), (0x968E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAE), (0x9DC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xAF), (0x53E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB0), (0x53E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB1), (0x544A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB2), (0x5471)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB3), (0x56FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB4), (0x59D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB5), (0x5B64)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB6), (0x5C3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB7), (0x5EAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB8), (0x62F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xB9), (0x6537)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBA), (0x6545)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBB), (0x6572)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBC), (0x66A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBD), (0x67AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBE), (0x69C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xBF), (0x6CBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC0), (0x75FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC1), (0x7690)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC2), (0x777E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC3), (0x7A3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC4), (0x7F94)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC5), (0x8003)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC6), (0x80A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC7), (0x818F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC8), (0x82E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xC9), (0x82FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCA), (0x83F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCB), (0x85C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCC), (0x8831)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCD), (0x88B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCE), (0x8AA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xCF), (0xF903)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCD, 0xD0), (0x8F9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD1), (0x932E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD2), (0x96C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD3), (0x9867)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD4), (0x9AD8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD5), (0x9F13)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD6), (0x54ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD7), (0x659B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD8), (0x66F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xD9), (0x688F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDA), (0x7A40)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDB), (0x8C37)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDC), (0x9D60)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDD), (0x56F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDE), (0x5764)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xDF), (0x5D11)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE0), (0x6606)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE1), (0x68B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE2), (0x68CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE3), (0x6EFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE4), (0x7428)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE5), (0x889E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE6), (0x9BE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE7), (0x6C68)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xE8), (0xF904)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCD, 0xE9), (0x9AA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xEA), (0x4F9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xEB), (0x516C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xEC), (0x5171)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xED), (0x529F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xEE), (0x5B54)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xEF), (0x5DE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF0), (0x6050)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF1), (0x606D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF2), (0x62F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF3), (0x63A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF4), (0x653B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF5), (0x73D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF6), (0x7A7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF7), (0x86A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF8), (0x8CA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xF9), (0x978F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xFA), (0x4E32)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xFB), (0x5BE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xFC), (0x6208)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xFD), (0x679C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCD, 0xFE), (0x74DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA1), (0x79D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA2), (0x83D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA3), (0x8A87)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA4), (0x8AB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA5), (0x8DE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA6), (0x904E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA7), (0x934B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA8), (0x9846)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xA9), (0x5ED3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xAA), (0x69E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xAB), (0x85FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xAC), (0x90ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xAD), (0xF905)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCE, 0xAE), (0x51A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xAF), (0x5B98)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB0), (0x5BEC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB1), (0x6163)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB2), (0x68FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB3), (0x6B3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB4), (0x704C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB5), (0x742F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB6), (0x74D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB7), (0x7BA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB8), (0x7F50)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xB9), (0x83C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBA), (0x89C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBB), (0x8CAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBC), (0x95DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBD), (0x9928)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBE), (0x522E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xBF), (0x605D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC0), (0x62EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC1), (0x9002)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC2), (0x4F8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC3), (0x5149)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC4), (0x5321)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC5), (0x58D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC6), (0x5EE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC7), (0x66E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC8), (0x6D38)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xC9), (0x709A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCA), (0x72C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCB), (0x73D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCC), (0x7B50)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCD), (0x80F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCE), (0x945B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xCF), (0x5366)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD0), (0x639B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD1), (0x7F6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD2), (0x4E56)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD3), (0x5080)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD4), (0x584A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD5), (0x58DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD6), (0x602A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD7), (0x6127)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD8), (0x62D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xD9), (0x69D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDA), (0x9B41)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDB), (0x5B8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDC), (0x7D18)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDD), (0x80B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDE), (0x8F5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xDF), (0x4EA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE0), (0x50D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE1), (0x54AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE2), (0x55AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE3), (0x5B0C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE4), (0x5DA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE5), (0x5DE7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE6), (0x652A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE7), (0x654E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE8), (0x6821)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xE9), (0x6A4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xEA), (0x72E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xEB), (0x768E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xEC), (0x77EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xED), (0x7D5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xEE), (0x7FF9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xEF), (0x81A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF0), (0x854E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF1), (0x86DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF2), (0x8F03)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF3), (0x8F4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF4), (0x90CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF5), (0x9903)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF6), (0x9A55)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF7), (0x9BAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF8), (0x4E18)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xF9), (0x4E45)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xFA), (0x4E5D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xFB), (0x4EC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xFC), (0x4FF1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xFD), (0x5177)),  # CJK UNIFIED IDEOGRAPH
    ((0xCE, 0xFE), (0x52FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA1), (0x5340)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA2), (0x53E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA3), (0x53E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA4), (0x548E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA5), (0x5614)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA6), (0x5775)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA7), (0x57A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA8), (0x5BC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xA9), (0x5D87)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAA), (0x5ED0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAB), (0x61FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAC), (0x62D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAD), (0x6551)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAE), (0x67B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xAF), (0x67E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB0), (0x69CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB1), (0x6B50)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB2), (0x6BC6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB3), (0x6BEC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB4), (0x6C42)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB5), (0x6E9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB6), (0x7078)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB7), (0x72D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB8), (0x7396)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xB9), (0x7403)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBA), (0x77BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBB), (0x77E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBC), (0x7A76)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBD), (0x7D7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBE), (0x8009)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xBF), (0x81FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC0), (0x8205)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC1), (0x820A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC2), (0x82DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC3), (0x8862)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC4), (0x8B33)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC5), (0x8CFC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC6), (0x8EC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC7), (0x9011)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC8), (0x90B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xC9), (0x9264)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCA), (0x92B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCB), (0x99D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCC), (0x9A45)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCD), (0x9CE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCE), (0x9DD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xCF), (0x9F9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD0), (0x570B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD1), (0x5C40)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD2), (0x83CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD3), (0x97A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD4), (0x97AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD5), (0x9EB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD6), (0x541B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD7), (0x7A98)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD8), (0x7FA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xD9), (0x88D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDA), (0x8ECD)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDB), (0x90E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDC), (0x5800)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDD), (0x5C48)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDE), (0x6398)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xDF), (0x7A9F)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE0), (0x5BAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE1), (0x5F13)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE2), (0x7A79)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE3), (0x7AAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE4), (0x828E)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE5), (0x8EAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE6), (0x5026)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE7), (0x5238)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE8), (0x52F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xE9), (0x5377)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xEA), (0x5708)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xEB), (0x62F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xEC), (0x6372)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xED), (0x6B0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xEE), (0x6DC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xEF), (0x7737)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF0), (0x53A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF1), (0x7357)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF2), (0x8568)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF3), (0x8E76)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF4), (0x95D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF5), (0x673A)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF6), (0x6AC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF7), (0x6F70)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF8), (0x8A6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xF9), (0x8ECC)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xFA), (0x994B)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xFB), (0xF906)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xCF, 0xFC), (0x6677)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xFD), (0x6B78)),  # CJK UNIFIED IDEOGRAPH
    ((0xCF, 0xFE), (0x8CB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA1), (0x9B3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA2), (0xF907)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD0, 0xA3), (0x53EB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA4), (0x572D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA5), (0x594E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA6), (0x63C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA7), (0x69FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA8), (0x73EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xA9), (0x7845)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAA), (0x7ABA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAB), (0x7AC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAC), (0x7CFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAD), (0x8475)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAE), (0x898F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xAF), (0x8D73)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB0), (0x9035)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB1), (0x95A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB2), (0x52FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB3), (0x5747)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB4), (0x7547)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB5), (0x7B60)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB6), (0x83CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB7), (0x921E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xB8), (0xF908)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD0, 0xB9), (0x6A58)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBA), (0x514B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBB), (0x524B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBC), (0x5287)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBD), (0x621F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBE), (0x68D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xBF), (0x6975)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC0), (0x9699)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC1), (0x50C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC2), (0x52A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC3), (0x52E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC4), (0x61C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC5), (0x65A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC6), (0x6839)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC7), (0x69FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC8), (0x747E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xC9), (0x7B4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCA), (0x82B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCB), (0x83EB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCC), (0x89B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCD), (0x8B39)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCE), (0x8FD1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xCF), (0x9949)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD0), (0xF909)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD0, 0xD1), (0x4ECA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD2), (0x5997)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD3), (0x64D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD4), (0x6611)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD5), (0x6A8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD6), (0x7434)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD7), (0x7981)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD8), (0x79BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xD9), (0x82A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xDA), (0x887E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xDB), (0x887F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xDC), (0x895F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xDD), (0xF90A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD0, 0xDE), (0x9326)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xDF), (0x4F0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE0), (0x53CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE1), (0x6025)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE2), (0x6271)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE3), (0x6C72)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE4), (0x7D1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE5), (0x7D66)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE6), (0x4E98)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE7), (0x5162)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE8), (0x77DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xE9), (0x80AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xEA), (0x4F01)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xEB), (0x4F0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xEC), (0x5176)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xED), (0x5180)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xEE), (0x55DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xEF), (0x5668)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF0), (0x573B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF1), (0x57FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF2), (0x57FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF3), (0x5914)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF4), (0x5947)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF5), (0x5993)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF6), (0x5BC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF7), (0x5C90)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF8), (0x5D0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xF9), (0x5DF1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xFA), (0x5E7E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xFB), (0x5FCC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xFC), (0x6280)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xFD), (0x65D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD0, 0xFE), (0x65E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA1), (0x671E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA2), (0x671F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA3), (0x675E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA4), (0x68CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA5), (0x68C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA6), (0x6A5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA7), (0x6B3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA8), (0x6C23)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xA9), (0x6C7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAA), (0x6C82)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAB), (0x6DC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAC), (0x7398)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAD), (0x7426)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAE), (0x742A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xAF), (0x7482)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB0), (0x74A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB1), (0x7578)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB2), (0x757F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB3), (0x7881)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB4), (0x78EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB5), (0x7941)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB6), (0x7947)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB7), (0x7948)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB8), (0x797A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xB9), (0x7B95)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBA), (0x7D00)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBB), (0x7DBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBC), (0x7F88)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBD), (0x8006)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBE), (0x802D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xBF), (0x808C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC0), (0x8A18)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC1), (0x8B4F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC2), (0x8C48)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC3), (0x8D77)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC4), (0x9321)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC5), (0x9324)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC6), (0x98E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC7), (0x9951)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC8), (0x9A0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xC9), (0x9A0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCA), (0x9A65)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCB), (0x9E92)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCC), (0x7DCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCD), (0x4F76)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCE), (0x5409)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xCF), (0x62EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD0), (0x6854)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD1), (0x91D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD2), (0x55AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD3), (0x513A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD4), (0xF90B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xD5), (0xF90C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xD6), (0x5A1C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD7), (0x61E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xD8), (0xF90D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xD9), (0x62CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xDA), (0x62FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xDB), (0xF90E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xDC), (0xF90F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xDD), (0xF910)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xDE), (0xF911)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xDF), (0xF912)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE0), (0xF913)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE1), (0x90A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xE2), (0xF914)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE3), (0xF915)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE4), (0xF916)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE5), (0xF917)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE6), (0xF918)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE7), (0x8AFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xE8), (0xF919)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xE9), (0xF91A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xEA), (0xF91B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xEB), (0xF91C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xEC), (0x6696)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xED), (0xF91D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xEE), (0x7156)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xEF), (0xF91E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xF0), (0xF91F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xF1), (0x96E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF2), (0xF920)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xF3), (0x634F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF4), (0x637A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF5), (0x5357)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF6), (0xF921)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xF7), (0x678F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF8), (0x6960)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xF9), (0x6E73)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xFA), (0xF922)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xFB), (0x7537)),  # CJK UNIFIED IDEOGRAPH
    ((0xD1, 0xFC), (0xF923)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xFD), (0xF924)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD1, 0xFE), (0xF925)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xA1), (0x7D0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xA2), (0xF926)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xA3), (0xF927)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xA4), (0x8872)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xA5), (0x56CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xA6), (0x5A18)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xA7), (0xF928)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xA8), (0xF929)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xA9), (0xF92A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xAA), (0xF92B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xAB), (0xF92C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xAC), (0x4E43)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xAD), (0xF92D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xAE), (0x5167)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xAF), (0x5948)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB0), (0x67F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB1), (0x8010)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB2), (0xF92E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xB3), (0x5973)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB4), (0x5E74)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB5), (0x649A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB6), (0x79CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB7), (0x5FF5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB8), (0x606C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xB9), (0x62C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xBA), (0x637B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xBB), (0x5BE7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xBC), (0x5BD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xBD), (0x52AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xBE), (0xF92F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xBF), (0x5974)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xC0), (0x5F29)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xC1), (0x6012)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xC2), (0xF930)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC3), (0xF931)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC4), (0xF932)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC5), (0x7459)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xC6), (0xF933)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC7), (0xF934)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC8), (0xF935)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xC9), (0xF936)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xCA), (0xF937)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xCB), (0xF938)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xCC), (0x99D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xCD), (0xF939)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xCE), (0xF93A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xCF), (0xF93B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD0), (0xF93C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD1), (0xF93D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD2), (0xF93E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD3), (0xF93F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD4), (0xF940)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD5), (0xF941)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD6), (0xF942)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD7), (0xF943)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xD8), (0x6FC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xD9), (0xF944)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xDA), (0xF945)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xDB), (0x81BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xDC), (0x8FB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xDD), (0x60F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xDE), (0xF946)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xDF), (0xF947)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE0), (0x8166)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xE1), (0xF948)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE2), (0xF949)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE3), (0x5C3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xE4), (0xF94A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE5), (0xF94B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE6), (0xF94C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE7), (0xF94D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE8), (0xF94E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xE9), (0xF94F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xEA), (0xF950)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xEB), (0xF951)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xEC), (0x5AE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xED), (0x8A25)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xEE), (0x677B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xEF), (0x7D10)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xF0), (0xF952)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF1), (0xF953)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF2), (0xF954)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF3), (0xF955)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF4), (0xF956)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF5), (0xF957)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF6), (0x80FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xF7), (0xF958)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF8), (0xF959)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD2, 0xF9), (0x5C3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xFA), (0x6CE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xFB), (0x533F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xFC), (0x6EBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xFD), (0x591A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD2, 0xFE), (0x8336)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA1), (0x4E39)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA2), (0x4EB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA3), (0x4F46)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA4), (0x55AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA5), (0x5718)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA6), (0x58C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA7), (0x5F56)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA8), (0x65B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xA9), (0x65E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAA), (0x6A80)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAB), (0x6BB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAC), (0x6E4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAD), (0x77ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAE), (0x7AEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xAF), (0x7C1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB0), (0x7DDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB1), (0x86CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB2), (0x8892)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB3), (0x9132)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB4), (0x935B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB5), (0x64BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB6), (0x6FBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB7), (0x737A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB8), (0x75B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xB9), (0x9054)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBA), (0x5556)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBB), (0x574D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBC), (0x61BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBD), (0x64D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBE), (0x66C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xBF), (0x6DE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC0), (0x6E5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC1), (0x6F6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC2), (0x6FB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC3), (0x75F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC4), (0x8043)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC5), (0x81BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC6), (0x8541)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC7), (0x8983)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC8), (0x8AC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xC9), (0x8B5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCA), (0x931F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCB), (0x6C93)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCC), (0x7553)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCD), (0x7B54)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCE), (0x8E0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xCF), (0x905D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD0), (0x5510)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD1), (0x5802)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD2), (0x5858)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD3), (0x5E62)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD4), (0x6207)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD5), (0x649E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD6), (0x68E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD7), (0x7576)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD8), (0x7CD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xD9), (0x87B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDA), (0x9EE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDB), (0x4EE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDC), (0x5788)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDD), (0x576E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDE), (0x5927)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xDF), (0x5C0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE0), (0x5CB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE1), (0x5E36)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE2), (0x5F85)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE3), (0x6234)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE4), (0x64E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE5), (0x73B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE6), (0x81FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE7), (0x888B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE8), (0x8CB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xE9), (0x968A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xEA), (0x9EDB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xEB), (0x5B85)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xEC), (0x5FB7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xED), (0x60B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xEE), (0x5012)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xEF), (0x5200)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF0), (0x5230)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF1), (0x5716)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF2), (0x5835)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF3), (0x5857)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF4), (0x5C0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF5), (0x5C60)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF6), (0x5CF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF7), (0x5D8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF8), (0x5EA6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xF9), (0x5F92)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xFA), (0x60BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xFB), (0x6311)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xFC), (0x6389)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xFD), (0x6417)),  # CJK UNIFIED IDEOGRAPH
    ((0xD3, 0xFE), (0x6843)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA1), (0x68F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA2), (0x6AC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA3), (0x6DD8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA4), (0x6E21)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA5), (0x6ED4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA6), (0x6FE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA7), (0x71FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA8), (0x76DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xA9), (0x7779)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAA), (0x79B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAB), (0x7A3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAC), (0x8404)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAD), (0x89A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAE), (0x8CED)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xAF), (0x8DF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB0), (0x8E48)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB1), (0x9003)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB2), (0x9014)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB3), (0x9053)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB4), (0x90FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB5), (0x934D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB6), (0x9676)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB7), (0x97DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB8), (0x6BD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xB9), (0x7006)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBA), (0x7258)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBB), (0x72A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBC), (0x7368)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBD), (0x7763)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBE), (0x79BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xBF), (0x7BE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC0), (0x7E9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC1), (0x8B80)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC2), (0x58A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC3), (0x60C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC4), (0x6566)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC5), (0x65FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC6), (0x66BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC7), (0x6C8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC8), (0x711E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xC9), (0x71C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCA), (0x8C5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCB), (0x9813)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCC), (0x4E6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCD), (0x7A81)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCE), (0x4EDD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xCF), (0x51AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD0), (0x51CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD1), (0x52D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD2), (0x540C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD3), (0x61A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD4), (0x6771)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD5), (0x6850)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD6), (0x68DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD7), (0x6D1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD8), (0x6F7C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xD9), (0x75BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDA), (0x77B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDB), (0x7AE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDC), (0x80F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDD), (0x8463)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDE), (0x9285)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xDF), (0x515C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE0), (0x6597)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE1), (0x675C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE2), (0x6793)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE3), (0x75D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE4), (0x7AC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE5), (0x8373)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE6), (0xF95A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD4, 0xE7), (0x8C46)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE8), (0x9017)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xE9), (0x982D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xEA), (0x5C6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xEB), (0x81C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xEC), (0x829A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xED), (0x9041)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xEE), (0x906F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xEF), (0x920D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF0), (0x5F97)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF1), (0x5D9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF2), (0x6A59)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF3), (0x71C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF4), (0x767B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF5), (0x7B49)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF6), (0x85E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF7), (0x8B04)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF8), (0x9127)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xF9), (0x9A30)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xFA), (0x5587)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xFB), (0x61F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xFC), (0xF95B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD4, 0xFD), (0x7669)),  # CJK UNIFIED IDEOGRAPH
    ((0xD4, 0xFE), (0x7F85)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA1), (0x863F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA2), (0x87BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA3), (0x88F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA4), (0x908F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA5), (0xF95C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD5, 0xA6), (0x6D1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA7), (0x70D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA8), (0x73DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xA9), (0x7D61)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xAA), (0x843D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xAB), (0xF95D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD5, 0xAC), (0x916A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xAD), (0x99F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xAE), (0xF95E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD5, 0xAF), (0x4E82)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB0), (0x5375)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB1), (0x6B04)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB2), (0x6B12)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB3), (0x703E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB4), (0x721B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB5), (0x862D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB6), (0x9E1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB7), (0x524C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB8), (0x8FA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xB9), (0x5D50)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBA), (0x64E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBB), (0x652C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBC), (0x6B16)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBD), (0x6FEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBE), (0x7C43)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xBF), (0x7E9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC0), (0x85CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC1), (0x8964)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC2), (0x89BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC3), (0x62C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC4), (0x81D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC5), (0x881F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC6), (0x5ECA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC7), (0x6717)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC8), (0x6D6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xC9), (0x72FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCA), (0x7405)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCB), (0x746F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCC), (0x8782)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCD), (0x90DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCE), (0x4F86)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xCF), (0x5D0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD0), (0x5FA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD1), (0x840A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD2), (0x51B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD3), (0x63A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD4), (0x7565)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD5), (0x4EAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD6), (0x5006)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD7), (0x5169)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD8), (0x51C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xD9), (0x6881)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDA), (0x6A11)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDB), (0x7CAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDC), (0x7CB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDD), (0x7CE7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDE), (0x826F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xDF), (0x8AD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE0), (0x8F1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE1), (0x91CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE2), (0x4FB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE3), (0x5137)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE4), (0x52F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE5), (0x5442)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE6), (0x5EEC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE7), (0x616E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE8), (0x623E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xE9), (0x65C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xEA), (0x6ADA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xEB), (0x6FFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xEC), (0x792A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xED), (0x85DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xEE), (0x8823)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xEF), (0x95AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF0), (0x9A62)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF1), (0x9A6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF2), (0x9E97)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF3), (0x9ECE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF4), (0x529B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF5), (0x66C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF6), (0x6B77)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF7), (0x701D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF8), (0x792B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xF9), (0x8F62)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xFA), (0x9742)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xFB), (0x6190)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xFC), (0x6200)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xFD), (0x6523)),  # CJK UNIFIED IDEOGRAPH
    ((0xD5, 0xFE), (0x6F23)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA1), (0x7149)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA2), (0x7489)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA3), (0x7DF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA4), (0x806F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA5), (0x84EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA6), (0x8F26)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA7), (0x9023)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA8), (0x934A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xA9), (0x51BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAA), (0x5217)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAB), (0x52A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAC), (0x6D0C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAD), (0x70C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAE), (0x88C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xAF), (0x5EC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB0), (0x6582)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB1), (0x6BAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB2), (0x6FC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB3), (0x7C3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB4), (0x7375)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB5), (0x4EE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB6), (0x4F36)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB7), (0x56F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xB8), (0xF95F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD6, 0xB9), (0x5CBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBA), (0x5DBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBB), (0x601C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBC), (0x73B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBD), (0x7B2D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBE), (0x7F9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xBF), (0x7FCE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC0), (0x8046)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC1), (0x901E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC2), (0x9234)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC3), (0x96F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC4), (0x9748)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC5), (0x9818)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC6), (0x9F61)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC7), (0x4F8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC8), (0x6FA7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xC9), (0x79AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xCA), (0x91B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xCB), (0x96B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xCC), (0x52DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xCD), (0xF960)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD6, 0xCE), (0x6488)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xCF), (0x64C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD0), (0x6AD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD1), (0x6F5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD2), (0x7018)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD3), (0x7210)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD4), (0x76E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD5), (0x8001)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD6), (0x8606)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD7), (0x865C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD8), (0x8DEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xD9), (0x8F05)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDA), (0x9732)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDB), (0x9B6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDC), (0x9DFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDD), (0x9E75)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDE), (0x788C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xDF), (0x797F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE0), (0x7DA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE1), (0x83C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE2), (0x9304)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE3), (0x9E7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE4), (0x9E93)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE5), (0x8AD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE6), (0x58DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE7), (0x5F04)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE8), (0x6727)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xE9), (0x7027)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xEA), (0x74CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xEB), (0x7C60)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xEC), (0x807E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xED), (0x5121)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xEE), (0x7028)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xEF), (0x7262)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF0), (0x78CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF1), (0x8CC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF2), (0x8CDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF3), (0x8CF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF4), (0x96F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF5), (0x4E86)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF6), (0x50DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF7), (0x5BEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF8), (0x5ED6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xF9), (0x6599)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xFA), (0x71CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xFB), (0x7642)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xFC), (0x77AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xFD), (0x804A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD6, 0xFE), (0x84FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA1), (0x907C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA2), (0x9B27)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA3), (0x9F8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA4), (0x58D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA5), (0x5A41)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA6), (0x5C62)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA7), (0x6A13)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA8), (0x6DDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xA9), (0x6F0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAA), (0x763B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAB), (0x7D2F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAC), (0x7E37)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAD), (0x851E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAE), (0x8938)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xAF), (0x93E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB0), (0x964B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB1), (0x5289)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB2), (0x65D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB3), (0x67F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB4), (0x69B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB5), (0x6D41)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB6), (0x6E9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB7), (0x700F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB8), (0x7409)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xB9), (0x7460)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBA), (0x7559)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBB), (0x7624)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBC), (0x786B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBD), (0x8B2C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBE), (0x985E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xBF), (0x516D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC0), (0x622E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC1), (0x9678)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC2), (0x4F96)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC3), (0x502B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC4), (0x5D19)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC5), (0x6DEA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC6), (0x7DB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC7), (0x8F2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC8), (0x5F8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xC9), (0x6144)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xCA), (0x6817)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xCB), (0xF961)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD7, 0xCC), (0x9686)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xCD), (0x52D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xCE), (0x808B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xCF), (0x51DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD0), (0x51CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD1), (0x695E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD2), (0x7A1C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD3), (0x7DBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD4), (0x83F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD5), (0x9675)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD6), (0x4FDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD7), (0x5229)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD8), (0x5398)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xD9), (0x540F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDA), (0x550E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDB), (0x5C65)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDC), (0x60A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDD), (0x674E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDE), (0x68A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xDF), (0x6D6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE0), (0x7281)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE1), (0x72F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE2), (0x7406)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE3), (0x7483)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE4), (0xF962)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xD7, 0xE5), (0x75E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE6), (0x7C6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE7), (0x7F79)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE8), (0x7FB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xE9), (0x8389)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xEA), (0x88CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xEB), (0x88E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xEC), (0x91CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xED), (0x91D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xEE), (0x96E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xEF), (0x9BC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF0), (0x541D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF1), (0x6F7E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF2), (0x71D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF3), (0x7498)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF4), (0x85FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF5), (0x8EAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF6), (0x96A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF7), (0x9C57)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF8), (0x9E9F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xF9), (0x6797)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xFA), (0x6DCB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xFB), (0x7433)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xFC), (0x81E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xFD), (0x9716)),  # CJK UNIFIED IDEOGRAPH
    ((0xD7, 0xFE), (0x782C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA1), (0x7ACB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA2), (0x7B20)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA3), (0x7C92)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA4), (0x6469)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA5), (0x746A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA6), (0x75F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA7), (0x78BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA8), (0x78E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xA9), (0x99AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAA), (0x9B54)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAB), (0x9EBB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAC), (0x5BDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAD), (0x5E55)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAE), (0x6F20)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xAF), (0x819C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB0), (0x83AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB1), (0x9088)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB2), (0x4E07)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB3), (0x534D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB4), (0x5A29)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB5), (0x5DD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB6), (0x5F4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB7), (0x6162)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB8), (0x633D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xB9), (0x6669)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBA), (0x66FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBB), (0x6EFF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBC), (0x6F2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBD), (0x7063)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBE), (0x779E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xBF), (0x842C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC0), (0x8513)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC1), (0x883B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC2), (0x8F13)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC3), (0x9945)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC4), (0x9C3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC5), (0x551C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC6), (0x62B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC7), (0x672B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC8), (0x6CAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xC9), (0x8309)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCA), (0x896A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCB), (0x977A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCC), (0x4EA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCD), (0x5984)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCE), (0x5FD8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xCF), (0x5FD9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD0), (0x671B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD1), (0x7DB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD2), (0x7F54)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD3), (0x8292)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD4), (0x832B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD5), (0x83BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD6), (0x8F1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD7), (0x9099)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD8), (0x57CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xD9), (0x59B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDA), (0x5A92)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDB), (0x5BD0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDC), (0x6627)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDD), (0x679A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDE), (0x6885)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xDF), (0x6BCF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE0), (0x7164)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE1), (0x7F75)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE2), (0x8CB7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE3), (0x8CE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE4), (0x9081)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE5), (0x9B45)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE6), (0x8108)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE7), (0x8C8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE8), (0x964C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xE9), (0x9A40)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xEA), (0x9EA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xEB), (0x5B5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xEC), (0x6C13)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xED), (0x731B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xEE), (0x76F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xEF), (0x76DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF0), (0x840C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF1), (0x51AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF2), (0x8993)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF3), (0x514D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF4), (0x5195)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF5), (0x52C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF6), (0x68C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF7), (0x6C94)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF8), (0x7704)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xF9), (0x7720)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xFA), (0x7DBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xFB), (0x7DEC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xFC), (0x9762)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xFD), (0x9EB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD8, 0xFE), (0x6EC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA1), (0x8511)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA2), (0x51A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA3), (0x540D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA4), (0x547D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA5), (0x660E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA6), (0x669D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA7), (0x6927)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA8), (0x6E9F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xA9), (0x76BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAA), (0x7791)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAB), (0x8317)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAC), (0x84C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAD), (0x879F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAE), (0x9169)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xAF), (0x9298)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB0), (0x9CF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB1), (0x8882)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB2), (0x4FAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB3), (0x5192)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB4), (0x52DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB5), (0x59C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB6), (0x5E3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB7), (0x6155)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB8), (0x6478)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xB9), (0x6479)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBA), (0x66AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBB), (0x67D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBC), (0x6A21)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBD), (0x6BCD)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBE), (0x6BDB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xBF), (0x725F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC0), (0x7261)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC1), (0x7441)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC2), (0x7738)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC3), (0x77DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC4), (0x8017)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC5), (0x82BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC6), (0x8305)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC7), (0x8B00)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC8), (0x8B28)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xC9), (0x8C8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCA), (0x6728)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCB), (0x6C90)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCC), (0x7267)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCD), (0x76EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCE), (0x7766)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xCF), (0x7A46)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD0), (0x9DA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD1), (0x6B7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD2), (0x6C92)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD3), (0x5922)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD4), (0x6726)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD5), (0x8499)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD6), (0x536F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD7), (0x5893)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD8), (0x5999)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xD9), (0x5EDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDA), (0x63CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDB), (0x6634)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDC), (0x6773)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDD), (0x6E3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDE), (0x732B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xDF), (0x7AD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE0), (0x82D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE1), (0x9328)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE2), (0x52D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE3), (0x5DEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE4), (0x61AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE5), (0x61CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE6), (0x620A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE7), (0x62C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE8), (0x64AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xE9), (0x65E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xEA), (0x6959)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xEB), (0x6B66)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xEC), (0x6BCB)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xED), (0x7121)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xEE), (0x73F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xEF), (0x755D)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF0), (0x7E46)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF1), (0x821E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF2), (0x8302)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF3), (0x856A)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF4), (0x8AA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF5), (0x8CBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF6), (0x9727)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF7), (0x9D61)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF8), (0x58A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xF9), (0x9ED8)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xFA), (0x5011)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xFB), (0x520E)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xFC), (0x543B)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xFD), (0x554F)),  # CJK UNIFIED IDEOGRAPH
    ((0xD9, 0xFE), (0x6587)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA1), (0x6C76)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA2), (0x7D0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA3), (0x7D0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA4), (0x805E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA5), (0x868A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA6), (0x9580)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA7), (0x96EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA8), (0x52FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xA9), (0x6C95)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAA), (0x7269)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAB), (0x5473)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAC), (0x5A9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAD), (0x5C3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAE), (0x5D4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xAF), (0x5F4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB0), (0x5FAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB1), (0x672A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB2), (0x68B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB3), (0x6963)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB4), (0x6E3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB5), (0x6E44)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB6), (0x7709)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB7), (0x7C73)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB8), (0x7F8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xB9), (0x8587)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBA), (0x8B0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBB), (0x8FF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBC), (0x9761)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBD), (0x9EF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBE), (0x5CB7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xBF), (0x60B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC0), (0x610D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC1), (0x61AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC2), (0x654F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC3), (0x65FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC4), (0x65FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC5), (0x6C11)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC6), (0x6CEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC7), (0x739F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC8), (0x73C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xC9), (0x7DE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCA), (0x9594)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCB), (0x5BC6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCC), (0x871C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCD), (0x8B10)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCE), (0x525D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xCF), (0x535A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD0), (0x62CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD1), (0x640F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD2), (0x64B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD3), (0x6734)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD4), (0x6A38)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD5), (0x6CCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD6), (0x73C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD7), (0x749E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD8), (0x7B94)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xD9), (0x7C95)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDA), (0x7E1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDB), (0x818A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDC), (0x8236)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDD), (0x8584)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDE), (0x8FEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xDF), (0x96F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE0), (0x99C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE1), (0x4F34)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE2), (0x534A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE3), (0x53CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE4), (0x53DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE5), (0x62CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE6), (0x642C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE7), (0x6500)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE8), (0x6591)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xE9), (0x69C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xEA), (0x6CEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xEB), (0x6F58)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xEC), (0x73ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xED), (0x7554)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xEE), (0x7622)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xEF), (0x76E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF0), (0x76FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF1), (0x78D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF2), (0x78FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF3), (0x792C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF4), (0x7D46)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF5), (0x822C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF6), (0x87E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF7), (0x8FD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF8), (0x9812)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xF9), (0x98EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xFA), (0x52C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xFB), (0x62D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xFC), (0x64A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xFD), (0x6E24)),  # CJK UNIFIED IDEOGRAPH
    ((0xDA, 0xFE), (0x6F51)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA1), (0x767C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA2), (0x8DCB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA3), (0x91B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA4), (0x9262)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA5), (0x9AEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA6), (0x9B43)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA7), (0x5023)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA8), (0x508D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xA9), (0x574A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAA), (0x59A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAB), (0x5C28)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAC), (0x5E47)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAD), (0x5F77)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAE), (0x623F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xAF), (0x653E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB0), (0x65B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB1), (0x65C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB2), (0x6609)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB3), (0x678B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB4), (0x699C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB5), (0x6EC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB6), (0x78C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB7), (0x7D21)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB8), (0x80AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xB9), (0x8180)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBA), (0x822B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBB), (0x82B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBC), (0x84A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBD), (0x868C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBE), (0x8A2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xBF), (0x8B17)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC0), (0x90A6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC1), (0x9632)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC2), (0x9F90)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC3), (0x500D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC4), (0x4FF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC5), (0xF963)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDB, 0xC6), (0x57F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC7), (0x5F98)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC8), (0x62DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xC9), (0x6392)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCA), (0x676F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCB), (0x6E43)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCC), (0x7119)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCD), (0x76C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCE), (0x80CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xCF), (0x80DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD0), (0x88F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD1), (0x88F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD2), (0x8919)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD3), (0x8CE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD4), (0x8F29)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD5), (0x914D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD6), (0x966A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD7), (0x4F2F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD8), (0x4F70)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xD9), (0x5E1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDA), (0x67CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDB), (0x6822)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDC), (0x767D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDD), (0x767E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDE), (0x9B44)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xDF), (0x5E61)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE0), (0x6A0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE1), (0x7169)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE2), (0x71D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE3), (0x756A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE4), (0xF964)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDB, 0xE5), (0x7E41)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE6), (0x8543)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE7), (0x85E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE8), (0x98DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xE9), (0x4F10)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xEA), (0x7B4F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xEB), (0x7F70)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xEC), (0x95A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xED), (0x51E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xEE), (0x5E06)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xEF), (0x68B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF0), (0x6C3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF1), (0x6C4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF2), (0x6CDB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF3), (0x72AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF4), (0x7BC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF5), (0x8303)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF6), (0x6CD5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF7), (0x743A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF8), (0x50FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xF9), (0x5288)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xFA), (0x58C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xFB), (0x64D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xFC), (0x6A97)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xFD), (0x74A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDB, 0xFE), (0x7656)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA1), (0x78A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA2), (0x8617)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA3), (0x95E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA4), (0x9739)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA5), (0xF965)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDC, 0xA6), (0x535E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA7), (0x5F01)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA8), (0x8B8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xA9), (0x8FA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAA), (0x8FAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAB), (0x908A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAC), (0x5225)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAD), (0x77A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAE), (0x9C49)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xAF), (0x9F08)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB0), (0x4E19)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB1), (0x5002)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB2), (0x5175)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB3), (0x5C5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB4), (0x5E77)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB5), (0x661E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB6), (0x663A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB7), (0x67C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB8), (0x68C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xB9), (0x70B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBA), (0x7501)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBB), (0x75C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBC), (0x79C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBD), (0x7ADD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBE), (0x8F27)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xBF), (0x9920)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC0), (0x9A08)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC1), (0x4FDD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC2), (0x5821)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC3), (0x5831)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC4), (0x5BF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC5), (0x666E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC6), (0x6B65)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC7), (0x6D11)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC8), (0x6E7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xC9), (0x6F7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCA), (0x73E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCB), (0x752B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCC), (0x83E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCD), (0x88DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCE), (0x8913)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xCF), (0x8B5C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD0), (0x8F14)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD1), (0x4F0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD2), (0x50D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD3), (0x5310)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD4), (0x535C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD5), (0x5B93)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD6), (0x5FA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD7), (0x670D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD8), (0x798F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xD9), (0x8179)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDA), (0x832F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDB), (0x8514)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDC), (0x8907)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDD), (0x8986)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDE), (0x8F39)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xDF), (0x8F3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE0), (0x99A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE1), (0x9C12)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE2), (0x672C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE3), (0x4E76)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE4), (0x4FF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE5), (0x5949)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE6), (0x5C01)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE7), (0x5CEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE8), (0x5CF0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xE9), (0x6367)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xEA), (0x68D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xEB), (0x70FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xEC), (0x71A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xED), (0x742B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xEE), (0x7E2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xEF), (0x84EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF0), (0x8702)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF1), (0x9022)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF2), (0x92D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF3), (0x9CF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF4), (0x4E0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF5), (0x4ED8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF6), (0x4FEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF7), (0x5085)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF8), (0x5256)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xF9), (0x526F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xFA), (0x5426)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xFB), (0x5490)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xFC), (0x57E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xFD), (0x592B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDC, 0xFE), (0x5A66)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA1), (0x5B5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA2), (0x5B75)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA3), (0x5BCC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA4), (0x5E9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA5), (0xF966)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDD, 0xA6), (0x6276)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA7), (0x6577)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA8), (0x65A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xA9), (0x6D6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAA), (0x6EA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAB), (0x7236)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAC), (0x7B26)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAD), (0x7C3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAE), (0x7F36)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xAF), (0x8150)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB0), (0x8151)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB1), (0x819A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB2), (0x8240)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB3), (0x8299)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB4), (0x83A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB5), (0x8A03)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB6), (0x8CA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB7), (0x8CE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB8), (0x8CFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xB9), (0x8D74)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBA), (0x8DBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBB), (0x90E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBC), (0x91DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBD), (0x961C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBE), (0x9644)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xBF), (0x99D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC0), (0x9CE7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC1), (0x5317)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC2), (0x5206)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC3), (0x5429)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC4), (0x5674)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC5), (0x58B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC6), (0x5954)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC7), (0x596E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC8), (0x5FFF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xC9), (0x61A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCA), (0x626E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCB), (0x6610)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCC), (0x6C7E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCD), (0x711A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCE), (0x76C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xCF), (0x7C89)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD0), (0x7CDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD1), (0x7D1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD2), (0x82AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD3), (0x8CC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD4), (0x96F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD5), (0xF967)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDD, 0xD6), (0x4F5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD7), (0x5F17)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD8), (0x5F7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xD9), (0x62C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDA), (0x5D29)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDB), (0x670B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDC), (0x68DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDD), (0x787C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDE), (0x7E43)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xDF), (0x9D6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE0), (0x4E15)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE1), (0x5099)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE2), (0x5315)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE3), (0x532A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE4), (0x5351)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE5), (0x5983)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE6), (0x5A62)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE7), (0x5E87)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE8), (0x60B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xE9), (0x618A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xEA), (0x6249)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xEB), (0x6279)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xEC), (0x6590)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xED), (0x6787)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xEE), (0x69A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xEF), (0x6BD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF0), (0x6BD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF1), (0x6BD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF2), (0x6BD8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF3), (0x6CB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF4), (0xF968)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDD, 0xF5), (0x7435)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF6), (0x75FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF7), (0x7812)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF8), (0x7891)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xF9), (0x79D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xFA), (0x79D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xFB), (0x7C83)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xFC), (0x7DCB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xFD), (0x7FE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDD, 0xFE), (0x80A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA1), (0x813E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA2), (0x81C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA3), (0x83F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA4), (0x871A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA5), (0x88E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA6), (0x8AB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA7), (0x8B6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA8), (0x8CBB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xA9), (0x9119)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAA), (0x975E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAB), (0x98DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAC), (0x9F3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAD), (0x56AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAE), (0x5B2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xAF), (0x5F6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB0), (0x658C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB1), (0x6AB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB2), (0x6BAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB3), (0x6D5C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB4), (0x6FF1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB5), (0x7015)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB6), (0x725D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB7), (0x73AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB8), (0x8CA7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xB9), (0x8CD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBA), (0x983B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBB), (0x6191)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBC), (0x6C37)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBD), (0x8058)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBE), (0x9A01)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xBF), (0x4E4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC0), (0x4E8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC1), (0x4E9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC2), (0x4ED5)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC3), (0x4F3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC4), (0x4F3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC5), (0x4F7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC6), (0x4FDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC7), (0x50FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC8), (0x53F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xC9), (0x53F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCA), (0x5506)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCB), (0x55E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCC), (0x56DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCD), (0x58EB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCE), (0x5962)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xCF), (0x5A11)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD0), (0x5BEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD1), (0x5BFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD2), (0x5C04)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD3), (0x5DF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD4), (0x5E2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD5), (0x5F99)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD6), (0x601D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD7), (0x6368)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD8), (0x659C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xD9), (0x65AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDA), (0x67F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDB), (0x67FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDC), (0x68AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDD), (0x6B7B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDE), (0x6C99)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xDF), (0x6CD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE0), (0x6E23)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE1), (0x7009)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE2), (0x7345)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE3), (0x7802)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE4), (0x793E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE5), (0x7940)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE6), (0x7960)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE7), (0x79C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE8), (0x7BE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xE9), (0x7D17)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xEA), (0x7D72)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xEB), (0x8086)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xEC), (0x820D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xED), (0x838E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xEE), (0x84D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xEF), (0x86C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF0), (0x88DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF1), (0x8A50)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF2), (0x8A5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF3), (0x8B1D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF4), (0x8CDC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF5), (0x8D66)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF6), (0x8FAD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF7), (0x90AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF8), (0x98FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xF9), (0x99DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xFA), (0x9E9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xFB), (0x524A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xFC), (0xF969)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDE, 0xFD), (0x6714)),  # CJK UNIFIED IDEOGRAPH
    ((0xDE, 0xFE), (0xF96A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDF, 0xA1), (0x5098)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA2), (0x522A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA3), (0x5C71)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA4), (0x6563)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA5), (0x6C55)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA6), (0x73CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA7), (0x7523)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA8), (0x759D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xA9), (0x7B97)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAA), (0x849C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAB), (0x9178)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAC), (0x9730)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAD), (0x4E77)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAE), (0x6492)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xAF), (0x6BBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB0), (0x715E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB1), (0x85A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB2), (0x4E09)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB3), (0xF96B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDF, 0xB4), (0x6749)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB5), (0x68EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB6), (0x6E17)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB7), (0x829F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB8), (0x8518)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xB9), (0x886B)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBA), (0x63F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBB), (0x6F81)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBC), (0x9212)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBD), (0x98AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBE), (0x4E0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xBF), (0x50B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC0), (0x50CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC1), (0x511F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC2), (0x5546)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC3), (0x55AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC4), (0x5617)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC5), (0x5B40)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC6), (0x5C19)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC7), (0x5CE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC8), (0x5E38)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xC9), (0x5E8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCA), (0x5EA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCB), (0x5EC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCC), (0x60F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCD), (0x6851)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCE), (0x6A61)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xCF), (0x6E58)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD0), (0x723D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD1), (0x7240)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD2), (0x72C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD3), (0x76F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD4), (0x7965)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD5), (0x7BB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD6), (0x7FD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD7), (0x88F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD8), (0x89F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xD9), (0x8A73)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDA), (0x8C61)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDB), (0x8CDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDC), (0x971C)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDD), (0x585E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDE), (0x74BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xDF), (0x8CFD)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE0), (0x55C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE1), (0xF96C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDF, 0xE2), (0x7A61)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE3), (0x7D22)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE4), (0x8272)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE5), (0x7272)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE6), (0x751F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE7), (0x7525)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xE8), (0xF96D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xDF, 0xE9), (0x7B19)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xEA), (0x5885)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xEB), (0x58FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xEC), (0x5DBC)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xED), (0x5E8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xEE), (0x5EB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xEF), (0x5F90)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF0), (0x6055)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF1), (0x6292)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF2), (0x637F)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF3), (0x654D)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF4), (0x6691)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF5), (0x66D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF6), (0x66F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF7), (0x6816)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF8), (0x68F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xF9), (0x7280)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xFA), (0x745E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xFB), (0x7B6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xFC), (0x7D6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xFD), (0x7DD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xDF, 0xFE), (0x7F72)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA1), (0x80E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA2), (0x8212)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA3), (0x85AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA4), (0x897F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA5), (0x8A93)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA6), (0x901D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA7), (0x92E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA8), (0x9ECD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xA9), (0x9F20)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAA), (0x5915)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAB), (0x596D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAC), (0x5E2D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAD), (0x60DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAE), (0x6614)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xAF), (0x6673)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB0), (0x6790)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB1), (0x6C50)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB2), (0x6DC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB3), (0x6F5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB4), (0x77F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB5), (0x78A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB6), (0x84C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB7), (0x91CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB8), (0x932B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xB9), (0x4ED9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBA), (0x50CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBB), (0x5148)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBC), (0x5584)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBD), (0x5B0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBE), (0x5BA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xBF), (0x6247)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC0), (0x657E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC1), (0x65CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC2), (0x6E32)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC3), (0x717D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC4), (0x7401)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC5), (0x7444)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC6), (0x7487)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC7), (0x74BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC8), (0x766C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xC9), (0x79AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCA), (0x7DDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCB), (0x7E55)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCC), (0x7FA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCD), (0x817A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCE), (0x81B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xCF), (0x8239)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD0), (0x861A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD1), (0x87EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD2), (0x8A75)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD3), (0x8DE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD4), (0x9078)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD5), (0x9291)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD6), (0x9425)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD7), (0x994D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD8), (0x9BAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xD9), (0x5368)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDA), (0x5C51)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDB), (0x6954)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDC), (0x6CC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDD), (0x6D29)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDE), (0x6E2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xDF), (0x820C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE0), (0x859B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE1), (0x893B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE2), (0x8A2D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE3), (0x8AAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE4), (0x96EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE5), (0x9F67)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE6), (0x5261)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE7), (0x66B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE8), (0x6BB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xE9), (0x7E96)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xEA), (0x87FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xEB), (0x8D0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xEC), (0x9583)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xED), (0x965D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xEE), (0x651D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xEF), (0x6D89)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF0), (0x71EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF1), (0xF96E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE0, 0xF2), (0x57CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF3), (0x59D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF4), (0x5BAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF5), (0x6027)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF6), (0x60FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF7), (0x6210)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF8), (0x661F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xF9), (0x665F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xFA), (0x7329)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xFB), (0x73F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xFC), (0x76DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xFD), (0x7701)),  # CJK UNIFIED IDEOGRAPH
    ((0xE0, 0xFE), (0x7B6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA1), (0x8056)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA2), (0x8072)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA3), (0x8165)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA4), (0x8AA0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA5), (0x9192)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA6), (0x4E16)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA7), (0x52E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA8), (0x6B72)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xA9), (0x6D17)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xAA), (0x7A05)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xAB), (0x7B39)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xAC), (0x7D30)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xAD), (0xF96F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE1, 0xAE), (0x8CB0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xAF), (0x53EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB0), (0x562F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB1), (0x5851)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB2), (0x5BB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB3), (0x5C0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB4), (0x5C11)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB5), (0x5DE2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB6), (0x6240)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB7), (0x6383)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB8), (0x6414)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xB9), (0x662D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBA), (0x68B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBB), (0x6CBC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBC), (0x6D88)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBD), (0x6EAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBE), (0x701F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xBF), (0x70A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC0), (0x71D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC1), (0x7526)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC2), (0x758F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC3), (0x758E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC4), (0x7619)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC5), (0x7B11)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC6), (0x7BE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC7), (0x7C2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC8), (0x7D20)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xC9), (0x7D39)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCA), (0x852C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCB), (0x856D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCC), (0x8607)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCD), (0x8A34)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCE), (0x900D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xCF), (0x9061)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD0), (0x90B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD1), (0x92B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD2), (0x97F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD3), (0x9A37)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD4), (0x4FD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD5), (0x5C6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD6), (0x675F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD7), (0x6D91)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD8), (0x7C9F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xD9), (0x7E8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDA), (0x8B16)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDB), (0x8D16)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDC), (0x901F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDD), (0x5B6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDE), (0x5DFD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xDF), (0x640D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE0), (0x84C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE1), (0x905C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE2), (0x98E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE3), (0x7387)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE4), (0x5B8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE5), (0x609A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE6), (0x677E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE7), (0x6DDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE8), (0x8A1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xE9), (0x8AA6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xEA), (0x9001)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xEB), (0x980C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xEC), (0x5237)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xED), (0xF970)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE1, 0xEE), (0x7051)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xEF), (0x788E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF0), (0x9396)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF1), (0x8870)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF2), (0x91D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF3), (0x4FEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF4), (0x53D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF5), (0x55FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF6), (0x56DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF7), (0x5782)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF8), (0x58FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xF9), (0x5AC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xFA), (0x5B88)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xFB), (0x5CAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xFC), (0x5CC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xFD), (0x5E25)),  # CJK UNIFIED IDEOGRAPH
    ((0xE1, 0xFE), (0x6101)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA1), (0x620D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA2), (0x624B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA3), (0x6388)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA4), (0x641C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA5), (0x6536)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA6), (0x6578)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA7), (0x6A39)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA8), (0x6B8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xA9), (0x6C34)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAA), (0x6D19)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAB), (0x6F31)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAC), (0x71E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAD), (0x72E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAE), (0x7378)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xAF), (0x7407)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB0), (0x74B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB1), (0x7626)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB2), (0x7761)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB3), (0x79C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB4), (0x7A57)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB5), (0x7AEA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB6), (0x7CB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB7), (0x7D8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB8), (0x7DAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xB9), (0x7E61)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBA), (0x7F9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBB), (0x8129)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBC), (0x8331)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBD), (0x8490)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBE), (0x84DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xBF), (0x85EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC0), (0x8896)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC1), (0x8AB0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC2), (0x8B90)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC3), (0x8F38)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC4), (0x9042)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC5), (0x9083)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC6), (0x916C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC7), (0x9296)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC8), (0x92B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xC9), (0x968B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCA), (0x96A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCB), (0x96A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCC), (0x96D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCD), (0x9700)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCE), (0x9808)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xCF), (0x9996)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD0), (0x9AD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD1), (0x9B1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD2), (0x53D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD3), (0x587E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD4), (0x5919)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD5), (0x5B70)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD6), (0x5BBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD7), (0x6DD1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD8), (0x6F5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xD9), (0x719F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDA), (0x7421)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDB), (0x74B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDC), (0x8085)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDD), (0x83FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDE), (0x5DE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xDF), (0x5F87)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE0), (0x5FAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE1), (0x6042)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE2), (0x65EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE3), (0x6812)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE4), (0x696F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE5), (0x6A53)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE6), (0x6B89)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE7), (0x6D35)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE8), (0x6DF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xE9), (0x73E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xEA), (0x76FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xEB), (0x77AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xEC), (0x7B4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xED), (0x7D14)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xEE), (0x8123)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xEF), (0x821C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF0), (0x8340)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF1), (0x84F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF2), (0x8563)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF3), (0x8A62)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF4), (0x8AC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF5), (0x9187)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF6), (0x931E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF7), (0x9806)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF8), (0x99B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xF9), (0x620C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xFA), (0x8853)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xFB), (0x8FF0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xFC), (0x9265)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xFD), (0x5D07)),  # CJK UNIFIED IDEOGRAPH
    ((0xE2, 0xFE), (0x5D27)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA1), (0x5D69)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA2), (0x745F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA3), (0x819D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA4), (0x8768)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA5), (0x6FD5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA6), (0x62FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA7), (0x7FD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA8), (0x8936)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xA9), (0x8972)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAA), (0x4E1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAB), (0x4E58)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAC), (0x50E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAD), (0x52DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAE), (0x5347)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xAF), (0x627F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB0), (0x6607)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB1), (0x7E69)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB2), (0x8805)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB3), (0x965E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB4), (0x4F8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB5), (0x5319)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB6), (0x5636)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB7), (0x59CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB8), (0x5AA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xB9), (0x5C38)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBA), (0x5C4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBB), (0x5C4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBC), (0x5E02)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBD), (0x5F11)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBE), (0x6043)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xBF), (0x65BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC0), (0x662F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC1), (0x6642)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC2), (0x67BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC3), (0x67F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC4), (0x731C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC5), (0x77E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC6), (0x793A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC7), (0x7FC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC8), (0x8494)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xC9), (0x84CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCA), (0x8996)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCB), (0x8A66)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCC), (0x8A69)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCD), (0x8AE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCE), (0x8C55)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xCF), (0x8C7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD0), (0x57F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD1), (0x5BD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD2), (0x5F0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD3), (0x606F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD4), (0x62ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD5), (0x690D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD6), (0x6B96)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD7), (0x6E5C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD8), (0x7184)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xD9), (0x7BD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDA), (0x8755)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDB), (0x8B58)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDC), (0x8EFE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDD), (0x98DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDE), (0x98FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xDF), (0x4F38)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE0), (0x4F81)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE1), (0x4FE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE2), (0x547B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE3), (0x5A20)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE4), (0x5BB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE5), (0x613C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE6), (0x65B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE7), (0x6668)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE8), (0x71FC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xE9), (0x7533)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xEA), (0x795E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xEB), (0x7D33)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xEC), (0x814E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xED), (0x81E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xEE), (0x8398)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xEF), (0x85AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF0), (0x85CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF1), (0x8703)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF2), (0x8A0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF3), (0x8EAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF4), (0x8F9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF5), (0xF971)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE3, 0xF6), (0x8FC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF7), (0x5931)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF8), (0x5BA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xF9), (0x5BE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xFA), (0x6089)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xFB), (0x5BE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xFC), (0x5C0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xFD), (0x5FC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE3, 0xFE), (0x6C81)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA1), (0xF972)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE4, 0xA2), (0x6DF1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA3), (0x700B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA4), (0x751A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA5), (0x82AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA6), (0x8AF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA7), (0x4EC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA8), (0x5341)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xA9), (0xF973)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE4, 0xAA), (0x96D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xAB), (0x6C0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xAC), (0x4E9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xAD), (0x4FC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xAE), (0x5152)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xAF), (0x555E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB0), (0x5A25)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB1), (0x5CE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB2), (0x6211)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB3), (0x7259)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB4), (0x82BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB5), (0x83AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB6), (0x86FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB7), (0x8859)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB8), (0x8A1D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xB9), (0x963F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBA), (0x96C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBB), (0x9913)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBC), (0x9D09)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBD), (0x9D5D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBE), (0x580A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xBF), (0x5CB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC0), (0x5DBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC1), (0x5E44)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC2), (0x60E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC3), (0x6115)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC4), (0x63E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC5), (0x6A02)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC6), (0x6E25)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC7), (0x9102)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC8), (0x9354)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xC9), (0x984E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCA), (0x9C10)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCB), (0x9F77)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCC), (0x5B89)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCD), (0x5CB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCE), (0x6309)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xCF), (0x664F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD0), (0x6848)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD1), (0x773C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD2), (0x96C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD3), (0x978D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD4), (0x9854)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD5), (0x9B9F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD6), (0x65A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD7), (0x8B01)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD8), (0x8ECB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xD9), (0x95BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDA), (0x5535)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDB), (0x5CA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDC), (0x5DD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDD), (0x5EB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDE), (0x6697)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xDF), (0x764C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE0), (0x83F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE1), (0x95C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE2), (0x58D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE3), (0x62BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE4), (0x72CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE5), (0x9D28)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE6), (0x4EF0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE7), (0x592E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE8), (0x600F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xE9), (0x663B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xEA), (0x6B83)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xEB), (0x79E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xEC), (0x9D26)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xED), (0x5393)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xEE), (0x54C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xEF), (0x57C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF0), (0x5D16)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF1), (0x611B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF2), (0x66D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF3), (0x6DAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF4), (0x788D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF5), (0x827E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF6), (0x9698)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF7), (0x9744)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF8), (0x5384)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xF9), (0x627C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xFA), (0x6396)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xFB), (0x6DB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xFC), (0x7E0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xFD), (0x814B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE4, 0xFE), (0x984D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA1), (0x6AFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA2), (0x7F4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA3), (0x9DAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA4), (0x9E1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA5), (0x4E5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA6), (0x503B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA7), (0x51B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA8), (0x591C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xA9), (0x60F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xAA), (0x63F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xAB), (0x6930)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xAC), (0x723A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xAD), (0x8036)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xAE), (0xF974)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xAF), (0x91CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB0), (0x5F31)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB1), (0xF975)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xB2), (0xF976)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xB3), (0x7D04)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB4), (0x82E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB5), (0x846F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB6), (0x84BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB7), (0x85E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB8), (0x8E8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xB9), (0xF977)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xBA), (0x4F6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xBB), (0xF978)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xBC), (0xF979)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xBD), (0x58E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xBE), (0x5B43)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xBF), (0x6059)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC0), (0x63DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC1), (0x6518)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC2), (0x656D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC3), (0x6698)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC4), (0xF97A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xC5), (0x694A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC6), (0x6A23)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC7), (0x6D0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC8), (0x7001)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xC9), (0x716C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xCA), (0x75D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xCB), (0x760D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xCC), (0x79B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xCD), (0x7A70)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xCE), (0xF97B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xCF), (0x7F8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD0), (0xF97C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xD1), (0x8944)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD2), (0xF97D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xD3), (0x8B93)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD4), (0x91C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD5), (0x967D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD6), (0xF97E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xD7), (0x990A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD8), (0x5704)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xD9), (0x5FA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDA), (0x65BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDB), (0x6F01)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDC), (0x7600)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDD), (0x79A6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDE), (0x8A9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xDF), (0x99AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE0), (0x9B5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE1), (0x9F6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE2), (0x5104)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE3), (0x61B6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE4), (0x6291)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE5), (0x6A8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE6), (0x81C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE7), (0x5043)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE8), (0x5830)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xE9), (0x5F66)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xEA), (0x7109)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xEB), (0x8A00)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xEC), (0x8AFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xED), (0x5B7C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xEE), (0x8616)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xEF), (0x4FFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF0), (0x513C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF1), (0x56B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF2), (0x5944)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF3), (0x63A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF4), (0x6DF9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF5), (0x5DAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF6), (0x696D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF7), (0x5186)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF8), (0x4E88)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xF9), (0x4F59)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xFA), (0xF97F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xFB), (0xF980)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xFC), (0xF981)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE5, 0xFD), (0x5982)),  # CJK UNIFIED IDEOGRAPH
    ((0xE5, 0xFE), (0xF982)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xA1), (0xF983)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xA2), (0x6B5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xA3), (0x6C5D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xA4), (0xF984)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xA5), (0x74B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xA6), (0x7916)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xA7), (0xF985)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xA8), (0x8207)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xA9), (0x8245)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xAA), (0x8339)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xAB), (0x8F3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xAC), (0x8F5D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xAD), (0xF986)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xAE), (0x9918)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xAF), (0xF987)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB0), (0xF988)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB1), (0xF989)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB2), (0x4EA6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xB3), (0xF98A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB4), (0x57DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xB5), (0x5F79)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xB6), (0x6613)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xB7), (0xF98B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB8), (0xF98C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xB9), (0x75AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xBA), (0x7E79)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xBB), (0x8B6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xBC), (0xF98D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xBD), (0x9006)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xBE), (0x9A5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xBF), (0x56A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC0), (0x5827)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC1), (0x59F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC2), (0x5A1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC3), (0x5BB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC4), (0xF98E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xC5), (0x5EF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC6), (0xF98F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xC7), (0xF990)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xC8), (0x6350)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xC9), (0x633B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xCA), (0xF991)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xCB), (0x693D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xCC), (0x6C87)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xCD), (0x6CBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xCE), (0x6D8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xCF), (0x6D93)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD0), (0x6DF5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD1), (0x6F14)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD2), (0xF992)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xD3), (0x70DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD4), (0x7136)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD5), (0x7159)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD6), (0xF993)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xD7), (0x71C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD8), (0x71D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xD9), (0xF994)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xDA), (0x784F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xDB), (0x786F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xDC), (0xF995)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xDD), (0x7B75)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xDE), (0x7DE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xDF), (0xF996)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE0), (0x7E2F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xE1), (0xF997)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE2), (0x884D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xE3), (0x8EDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xE4), (0xF998)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE5), (0xF999)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE6), (0xF99A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE7), (0x925B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xE8), (0xF99B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xE9), (0x9CF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xEA), (0xF99C)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xEB), (0xF99D)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xEC), (0xF99E)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xED), (0x6085)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xEE), (0x6D85)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xEF), (0xF99F)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF0), (0x71B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xF1), (0xF9A0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF2), (0xF9A1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF3), (0x95B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xF4), (0x53AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xF5), (0xF9A2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF6), (0xF9A3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF7), (0xF9A4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xF8), (0x67D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xF9), (0xF9A5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE6, 0xFA), (0x708E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xFB), (0x7130)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xFC), (0x7430)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xFD), (0x8276)),  # CJK UNIFIED IDEOGRAPH
    ((0xE6, 0xFE), (0x82D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA1), (0xF9A6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xA2), (0x95BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA3), (0x9AE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA4), (0x9E7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA5), (0x66C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA6), (0xF9A7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xA7), (0x71C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA8), (0x8449)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xA9), (0xF9A8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xAA), (0xF9A9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xAB), (0x584B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xAC), (0xF9AA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xAD), (0xF9AB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xAE), (0x5DB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xAF), (0x5F71)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB0), (0xF9AC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xB1), (0x6620)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB2), (0x668E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB3), (0x6979)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB4), (0x69AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB5), (0x6C38)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB6), (0x6CF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB7), (0x6E36)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB8), (0x6F41)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xB9), (0x6FDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBA), (0x701B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBB), (0x702F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBC), (0x7150)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBD), (0x71DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBE), (0x7370)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xBF), (0xF9AD)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xC0), (0x745B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC1), (0xF9AE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xC2), (0x74D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC3), (0x76C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC4), (0x7A4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC5), (0x7E93)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC6), (0xF9AF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xC7), (0xF9B0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xC8), (0x82F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xC9), (0x8A60)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xCA), (0x8FCE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xCB), (0xF9B1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xCC), (0x9348)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xCD), (0xF9B2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xCE), (0x9719)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xCF), (0xF9B3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xD0), (0xF9B4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xD1), (0x4E42)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD2), (0x502A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD3), (0xF9B5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xD4), (0x5208)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD5), (0x53E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD6), (0x66F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD7), (0x6C6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD8), (0x6FCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xD9), (0x730A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDA), (0x777F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDB), (0x7A62)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDC), (0x82AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDD), (0x85DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDE), (0x8602)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xDF), (0xF9B6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xE0), (0x88D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE1), (0x8A63)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE2), (0x8B7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE3), (0x8C6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE4), (0xF9B7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xE5), (0x92B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE6), (0xF9B8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xE7), (0x9713)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE8), (0x9810)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xE9), (0x4E94)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xEA), (0x4F0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xEB), (0x4FC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xEC), (0x50B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xED), (0x5348)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xEE), (0x543E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xEF), (0x5433)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF0), (0x55DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF1), (0x5862)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF2), (0x58BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF3), (0x5967)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF4), (0x5A1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF5), (0x5BE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF6), (0x609F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF7), (0xF9B9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE7, 0xF8), (0x61CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xF9), (0x6556)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xFA), (0x65FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xFB), (0x6664)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xFC), (0x68A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xFD), (0x6C5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE7, 0xFE), (0x6FB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA1), (0x70CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA2), (0x71AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA3), (0x7352)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA4), (0x7B7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA5), (0x8708)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA6), (0x8AA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA7), (0x9C32)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA8), (0x9F07)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xA9), (0x5C4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAA), (0x6C83)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAB), (0x7344)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAC), (0x7389)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAD), (0x923A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAE), (0x6EAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xAF), (0x7465)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB0), (0x761F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB1), (0x7A69)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB2), (0x7E15)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB3), (0x860A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB4), (0x5140)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB5), (0x58C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB6), (0x64C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB7), (0x74EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB8), (0x7515)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xB9), (0x7670)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBA), (0x7FC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBB), (0x9095)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBC), (0x96CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBD), (0x9954)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBE), (0x6E26)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xBF), (0x74E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC0), (0x7AA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC1), (0x7AAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC2), (0x81E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC3), (0x86D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC4), (0x8778)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC5), (0x8A1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC6), (0x5A49)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC7), (0x5B8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC8), (0x5B9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xC9), (0x68A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCA), (0x6900)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCB), (0x6D63)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCC), (0x73A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCD), (0x7413)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCE), (0x742C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xCF), (0x7897)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD0), (0x7DE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD1), (0x7FEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD2), (0x8118)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD3), (0x8155)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD4), (0x839E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD5), (0x8C4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD6), (0x962E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD7), (0x9811)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD8), (0x66F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xD9), (0x5F80)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDA), (0x65FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDB), (0x6789)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDC), (0x6C6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDD), (0x738B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDE), (0x502D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xDF), (0x5A03)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE0), (0x6B6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE1), (0x77EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE2), (0x5916)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE3), (0x5D6C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE4), (0x5DCD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE5), (0x7325)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE6), (0x754F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xE7), (0xF9BA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xE8), (0xF9BB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xE9), (0x50E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xEA), (0x51F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xEB), (0x582F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xEC), (0x592D)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xED), (0x5996)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xEE), (0x59DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xEF), (0x5BE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF0), (0xF9BC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xF1), (0xF9BD)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xF2), (0x5DA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF3), (0x62D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF4), (0x6416)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF5), (0x6493)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF6), (0x64FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF7), (0xF9BE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xF8), (0x66DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xF9), (0xF9BF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xFA), (0x6A48)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xFB), (0xF9C0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE8, 0xFC), (0x71FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xFD), (0x7464)),  # CJK UNIFIED IDEOGRAPH
    ((0xE8, 0xFE), (0xF9C1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE9, 0xA1), (0x7A88)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA2), (0x7AAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA3), (0x7E47)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA4), (0x7E5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA5), (0x8000)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA6), (0x8170)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA7), (0xF9C2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE9, 0xA8), (0x87EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xA9), (0x8981)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xAA), (0x8B20)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xAB), (0x9059)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xAC), (0xF9C3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE9, 0xAD), (0x9080)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xAE), (0x9952)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xAF), (0x617E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB0), (0x6B32)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB1), (0x6D74)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB2), (0x7E1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB3), (0x8925)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB4), (0x8FB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB5), (0x4FD1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB6), (0x50AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB7), (0x5197)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB8), (0x52C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xB9), (0x57C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBA), (0x5889)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBB), (0x5BB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBC), (0x5EB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBD), (0x6142)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBE), (0x6995)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xBF), (0x6D8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC0), (0x6E67)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC1), (0x6EB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC2), (0x7194)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC3), (0x7462)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC4), (0x7528)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC5), (0x752C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC6), (0x8073)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC7), (0x8338)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC8), (0x84C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xC9), (0x8E0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xCA), (0x9394)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xCB), (0x93DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xCC), (0xF9C4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE9, 0xCD), (0x4E8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xCE), (0x4F51)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xCF), (0x5076)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD0), (0x512A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD1), (0x53C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD2), (0x53CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD3), (0x53F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD4), (0x5B87)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD5), (0x5BD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD6), (0x5C24)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD7), (0x611A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD8), (0x6182)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xD9), (0x65F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDA), (0x725B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDB), (0x7397)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDC), (0x7440)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDD), (0x76C2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDE), (0x7950)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xDF), (0x7991)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE0), (0x79B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE1), (0x7D06)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE2), (0x7FBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE3), (0x828B)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE4), (0x85D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE5), (0x865E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE6), (0x8FC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE7), (0x9047)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE8), (0x90F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xE9), (0x91EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xEA), (0x9685)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xEB), (0x96E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xEC), (0x96E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xED), (0x52D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xEE), (0x5F67)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xEF), (0x65ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF0), (0x6631)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF1), (0x682F)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF2), (0x715C)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF3), (0x7A36)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF4), (0x90C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF5), (0x980A)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF6), (0x4E91)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF7), (0xF9C5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xE9, 0xF8), (0x6A52)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xF9), (0x6B9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xFA), (0x6F90)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xFB), (0x7189)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xFC), (0x8018)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xFD), (0x82B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xE9, 0xFE), (0x8553)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA1), (0x904B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA2), (0x9695)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA3), (0x96F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA4), (0x97FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA5), (0x851A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA6), (0x9B31)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA7), (0x4E90)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA8), (0x718A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xA9), (0x96C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAA), (0x5143)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAB), (0x539F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAC), (0x54E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAD), (0x5713)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAE), (0x5712)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xAF), (0x57A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB0), (0x5A9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB1), (0x5AC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB2), (0x5BC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB3), (0x6028)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB4), (0x613F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB5), (0x63F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB6), (0x6C85)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB7), (0x6D39)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB8), (0x6E72)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xB9), (0x6E90)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBA), (0x7230)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBB), (0x733F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBC), (0x7457)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBD), (0x82D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBE), (0x8881)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xBF), (0x8F45)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC0), (0x9060)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC1), (0xF9C6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEA, 0xC2), (0x9662)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC3), (0x9858)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC4), (0x9D1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC5), (0x6708)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC6), (0x8D8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC7), (0x925E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC8), (0x4F4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xC9), (0x5049)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCA), (0x50DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCB), (0x5371)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCC), (0x570D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCD), (0x59D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCE), (0x5A01)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xCF), (0x5C09)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD0), (0x6170)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD1), (0x6690)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD2), (0x6E2D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD3), (0x7232)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD4), (0x744B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD5), (0x7DEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD6), (0x80C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD7), (0x840E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD8), (0x8466)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xD9), (0x853F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDA), (0x875F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDB), (0x885B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDC), (0x8918)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDD), (0x8B02)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDE), (0x9055)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xDF), (0x97CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE0), (0x9B4F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE1), (0x4E73)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE2), (0x4F91)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE3), (0x5112)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE4), (0x516A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE5), (0xF9C7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEA, 0xE6), (0x552F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE7), (0x55A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE8), (0x5B7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xE9), (0x5BA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xEA), (0x5E7C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xEB), (0x5E7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xEC), (0x5EBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xED), (0x60A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xEE), (0x60DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xEF), (0x6108)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF0), (0x6109)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF1), (0x63C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF2), (0x6538)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF3), (0x6709)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF4), (0xF9C8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEA, 0xF5), (0x67D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF6), (0x67DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF7), (0xF9C9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEA, 0xF8), (0x6961)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xF9), (0x6962)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xFA), (0x6CB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xFB), (0x6D27)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xFC), (0xF9CA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEA, 0xFD), (0x6E38)),  # CJK UNIFIED IDEOGRAPH
    ((0xEA, 0xFE), (0xF9CB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xA1), (0x6FE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA2), (0x7336)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA3), (0x7337)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA4), (0xF9CC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xA5), (0x745C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA6), (0x7531)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA7), (0xF9CD)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xA8), (0x7652)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xA9), (0xF9CE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xAA), (0xF9CF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xAB), (0x7DAD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xAC), (0x81FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xAD), (0x8438)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xAE), (0x88D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xAF), (0x8A98)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB0), (0x8ADB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB1), (0x8AED)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB2), (0x8E30)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB3), (0x8E42)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB4), (0x904A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB5), (0x903E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB6), (0x907A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB7), (0x9149)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB8), (0x91C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xB9), (0x936E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xBA), (0xF9D0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xBB), (0xF9D1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xBC), (0x5809)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xBD), (0xF9D2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xBE), (0x6BD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xBF), (0x8089)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC0), (0x80B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC1), (0xF9D3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xC2), (0xF9D4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xC3), (0x5141)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC4), (0x596B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC5), (0x5C39)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC6), (0xF9D5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xC7), (0xF9D6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xC8), (0x6F64)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xC9), (0x73A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xCA), (0x80E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xCB), (0x8D07)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xCC), (0xF9D7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xCD), (0x9217)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xCE), (0x958F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xCF), (0xF9D8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xD0), (0xF9D9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xD1), (0xF9DA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xD2), (0xF9DB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xD3), (0x807F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xD4), (0x620E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xD5), (0x701C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xD6), (0x7D68)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xD7), (0x878D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xD8), (0xF9DC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEB, 0xD9), (0x57A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDA), (0x6069)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDB), (0x6147)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDC), (0x6BB7)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDD), (0x8ABE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDE), (0x9280)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xDF), (0x96B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE0), (0x4E59)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE1), (0x541F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE2), (0x6DEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE3), (0x852D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE4), (0x9670)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE5), (0x97F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE6), (0x98EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE7), (0x63D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE8), (0x6CE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xE9), (0x9091)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xEA), (0x51DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xEB), (0x61C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xEC), (0x81BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xED), (0x9DF9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xEE), (0x4F9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xEF), (0x501A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF0), (0x5100)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF1), (0x5B9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF2), (0x610F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF3), (0x61FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF4), (0x64EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF5), (0x6905)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF6), (0x6BC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF7), (0x7591)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF8), (0x77E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xF9), (0x7FA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xFA), (0x8264)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xFB), (0x858F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xFC), (0x87FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xFD), (0x8863)),  # CJK UNIFIED IDEOGRAPH
    ((0xEB, 0xFE), (0x8ABC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA1), (0x8B70)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA2), (0x91AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA3), (0x4E8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA4), (0x4EE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA5), (0x4F0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA6), (0xF9DD)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xA7), (0xF9DE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xA8), (0x5937)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xA9), (0x59E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xAA), (0xF9DF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xAB), (0x5DF2)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xAC), (0x5F1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xAD), (0x5F5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xAE), (0x6021)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xAF), (0xF9E0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB0), (0xF9E1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB1), (0xF9E2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB2), (0xF9E3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB3), (0x723E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xB4), (0x73E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xB5), (0xF9E4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB6), (0x7570)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xB7), (0x75CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xB8), (0xF9E5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xB9), (0x79FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xBA), (0xF9E6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xBB), (0x800C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xBC), (0x8033)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xBD), (0x8084)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xBE), (0x82E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xBF), (0x8351)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC0), (0xF9E7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xC1), (0xF9E8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xC2), (0x8CBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC3), (0x8CB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC4), (0x9087)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC5), (0xF9E9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xC6), (0xF9EA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xC7), (0x98F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC8), (0x990C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xC9), (0xF9EB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xCA), (0xF9EC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xCB), (0x7037)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xCC), (0x76CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xCD), (0x7FCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xCE), (0x7FCC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xCF), (0x7FFC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD0), (0x8B1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD1), (0x4EBA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD2), (0x4EC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD3), (0x5203)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD4), (0x5370)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD5), (0xF9ED)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xD6), (0x54BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD7), (0x56E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD8), (0x59FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xD9), (0x5BC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xDA), (0x5F15)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xDB), (0x5FCD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xDC), (0x6E6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xDD), (0xF9EE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xDE), (0xF9EF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xDF), (0x7D6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE0), (0x8335)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE1), (0xF9F0)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xE2), (0x8693)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE3), (0x8A8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE4), (0xF9F1)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xE5), (0x976D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE6), (0x9777)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xE7), (0xF9F2)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xE8), (0xF9F3)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xE9), (0x4E00)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xEA), (0x4F5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xEB), (0x4F7E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xEC), (0x58F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xED), (0x65E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xEE), (0x6EA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xEF), (0x9038)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF0), (0x93B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF1), (0x99B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF2), (0x4EFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF3), (0x58EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF4), (0x598A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF5), (0x59D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF6), (0x6041)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xF7), (0xF9F4)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xF8), (0xF9F5)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xF9), (0x7A14)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xFA), (0xF9F6)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEC, 0xFB), (0x834F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xFC), (0x8CC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xFD), (0x5165)),  # CJK UNIFIED IDEOGRAPH
    ((0xEC, 0xFE), (0x5344)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA1), (0xF9F7)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xED, 0xA2), (0xF9F8)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xED, 0xA3), (0xF9F9)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xED, 0xA4), (0x4ECD)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA5), (0x5269)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA6), (0x5B55)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA7), (0x82BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA8), (0x4ED4)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xA9), (0x523A)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAA), (0x54A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAB), (0x59C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAC), (0x59FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAD), (0x5B50)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAE), (0x5B57)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xAF), (0x5B5C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB0), (0x6063)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB1), (0x6148)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB2), (0x6ECB)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB3), (0x7099)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB4), (0x716E)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB5), (0x7386)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB6), (0x74F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB7), (0x75B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB8), (0x78C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xB9), (0x7D2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBA), (0x8005)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBB), (0x81EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBC), (0x8328)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBD), (0x8517)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBE), (0x85C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xBF), (0x8AEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC0), (0x8CC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC1), (0x96CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC2), (0x4F5C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC3), (0x52FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC4), (0x56BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC5), (0x65AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC6), (0x6628)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC7), (0x707C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC8), (0x70B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xC9), (0x7235)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCA), (0x7DBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCB), (0x828D)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCC), (0x914C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCD), (0x96C0)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCE), (0x9D72)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xCF), (0x5B71)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD0), (0x68E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD1), (0x6B98)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD2), (0x6F7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD3), (0x76DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD4), (0x5C91)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD5), (0x66AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD6), (0x6F5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD7), (0x7BB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD8), (0x7C2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xD9), (0x8836)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDA), (0x96DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDB), (0x4E08)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDC), (0x4ED7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDD), (0x5320)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDE), (0x5834)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xDF), (0x58BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE0), (0x58EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE1), (0x596C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE2), (0x5C07)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE3), (0x5E33)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE4), (0x5E84)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE5), (0x5F35)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE6), (0x638C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE7), (0x66B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE8), (0x6756)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xE9), (0x6A1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xEA), (0x6AA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xEB), (0x6B0C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xEC), (0x6F3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xED), (0x7246)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xEE), (0xF9FA)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xED, 0xEF), (0x7350)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF0), (0x748B)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF1), (0x7AE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF2), (0x7CA7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF3), (0x8178)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF4), (0x81DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF5), (0x81E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF6), (0x838A)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF7), (0x846C)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF8), (0x8523)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xF9), (0x8594)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xFA), (0x85CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xFB), (0x88DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xFC), (0x8D13)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xFD), (0x91AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xED, 0xFE), (0x9577)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA1), (0x969C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA2), (0x518D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA3), (0x54C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA4), (0x5728)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA5), (0x5BB0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA6), (0x624D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA7), (0x6750)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA8), (0x683D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xA9), (0x6893)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAA), (0x6E3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAB), (0x6ED3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAC), (0x707D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAD), (0x7E21)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAE), (0x88C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xAF), (0x8CA1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB0), (0x8F09)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB1), (0x9F4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB2), (0x9F4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB3), (0x722D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB4), (0x7B8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB5), (0x8ACD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB6), (0x931A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB7), (0x4F47)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB8), (0x4F4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xB9), (0x5132)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBA), (0x5480)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBB), (0x59D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBC), (0x5E95)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBD), (0x62B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBE), (0x6775)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xBF), (0x696E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC0), (0x6A17)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC1), (0x6CAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC2), (0x6E1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC3), (0x72D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC4), (0x732A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC5), (0x75BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC6), (0x7BB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC7), (0x7D35)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC8), (0x82E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xC9), (0x83F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCA), (0x8457)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCB), (0x85F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCC), (0x8A5B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCD), (0x8CAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCE), (0x8E87)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xCF), (0x9019)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD0), (0x90B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD1), (0x96CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD2), (0x9F5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD3), (0x52E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD4), (0x540A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD5), (0x5AE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD6), (0x5BC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD7), (0x6458)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD8), (0x6575)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xD9), (0x6EF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xDA), (0x72C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xDB), (0xF9FB)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xEE, 0xDC), (0x7684)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xDD), (0x7A4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xDE), (0x7B1B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xDF), (0x7C4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE0), (0x7E3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE1), (0x7FDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE2), (0x837B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE3), (0x8B2B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE4), (0x8CCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE5), (0x8D64)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE6), (0x8DE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE7), (0x8E5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE8), (0x8FEA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xE9), (0x8FF9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xEA), (0x9069)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xEB), (0x93D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xEC), (0x4F43)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xED), (0x4F7A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xEE), (0x50B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xEF), (0x5168)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF0), (0x5178)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF1), (0x524D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF2), (0x526A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF3), (0x5861)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF4), (0x587C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF5), (0x5960)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF6), (0x5C08)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF7), (0x5C55)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF8), (0x5EDB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xF9), (0x609B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xFA), (0x6230)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xFB), (0x6813)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xFC), (0x6BBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xFD), (0x6C08)),  # CJK UNIFIED IDEOGRAPH
    ((0xEE, 0xFE), (0x6FB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA1), (0x714E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA2), (0x7420)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA3), (0x7530)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA4), (0x7538)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA5), (0x7551)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA6), (0x7672)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA7), (0x7B4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA8), (0x7B8B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xA9), (0x7BAD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAA), (0x7BC6)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAB), (0x7E8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAC), (0x8A6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAD), (0x8F3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAE), (0x8F49)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xAF), (0x923F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB0), (0x9293)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB1), (0x9322)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB2), (0x942B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB3), (0x96FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB4), (0x985A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB5), (0x986B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB6), (0x991E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB7), (0x5207)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB8), (0x622A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xB9), (0x6298)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBA), (0x6D59)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBB), (0x7664)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBC), (0x7ACA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBD), (0x7BC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBE), (0x7D76)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xBF), (0x5360)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC0), (0x5CBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC1), (0x5E97)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC2), (0x6F38)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC3), (0x70B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC4), (0x7C98)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC5), (0x9711)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC6), (0x9B8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC7), (0x9EDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC8), (0x63A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xC9), (0x647A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCA), (0x8776)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCB), (0x4E01)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCC), (0x4E95)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCD), (0x4EAD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCE), (0x505C)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xCF), (0x5075)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD0), (0x5448)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD1), (0x59C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD2), (0x5B9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD3), (0x5E40)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD4), (0x5EAD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD5), (0x5EF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD6), (0x5F81)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD7), (0x60C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD8), (0x633A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xD9), (0x653F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDA), (0x6574)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDB), (0x65CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDC), (0x6676)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDD), (0x6678)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDE), (0x67FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xDF), (0x6968)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE0), (0x6A89)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE1), (0x6B63)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE2), (0x6C40)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE3), (0x6DC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE4), (0x6DE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE5), (0x6E1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE6), (0x6E5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE7), (0x701E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE8), (0x70A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xE9), (0x738E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xEA), (0x73FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xEB), (0x753A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xEC), (0x775B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xED), (0x7887)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xEE), (0x798E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xEF), (0x7A0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF0), (0x7A7D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF1), (0x7CBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF2), (0x7D8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF3), (0x8247)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF4), (0x8A02)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF5), (0x8AEA)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF6), (0x8C9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF7), (0x912D)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF8), (0x914A)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xF9), (0x91D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xFA), (0x9266)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xFB), (0x92CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xFC), (0x9320)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xFD), (0x9706)),  # CJK UNIFIED IDEOGRAPH
    ((0xEF, 0xFE), (0x9756)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA1), (0x975C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA2), (0x9802)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA3), (0x9F0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA4), (0x5236)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA5), (0x5291)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA6), (0x557C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA7), (0x5824)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA8), (0x5E1D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xA9), (0x5F1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAA), (0x608C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAB), (0x63D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAC), (0x68AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAD), (0x6FDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAE), (0x796D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xAF), (0x7B2C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB0), (0x81CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB1), (0x85BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB2), (0x88FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB3), (0x8AF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB4), (0x8E44)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB5), (0x918D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB6), (0x9664)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB7), (0x969B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB8), (0x973D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xB9), (0x984C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBA), (0x9F4A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBB), (0x4FCE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBC), (0x5146)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBD), (0x51CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBE), (0x52A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xBF), (0x5632)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC0), (0x5F14)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC1), (0x5F6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC2), (0x63AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC3), (0x64CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC4), (0x65E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC5), (0x6641)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC6), (0x66FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC7), (0x66F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC8), (0x671D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xC9), (0x689D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCA), (0x68D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCB), (0x69FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCC), (0x6F15)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCD), (0x6F6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCE), (0x7167)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xCF), (0x71E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD0), (0x722A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD1), (0x74AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD2), (0x773A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD3), (0x7956)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD4), (0x795A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD5), (0x79DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD6), (0x7A20)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD7), (0x7A95)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD8), (0x7C97)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xD9), (0x7CDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDA), (0x7D44)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDB), (0x7E70)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDC), (0x8087)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDD), (0x85FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDE), (0x86A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xDF), (0x8A54)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE0), (0x8ABF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE1), (0x8D99)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE2), (0x8E81)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE3), (0x9020)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE4), (0x906D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE5), (0x91E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE6), (0x963B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE7), (0x96D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE8), (0x9CE5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xE9), (0x65CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xEA), (0x7C07)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xEB), (0x8DB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xEC), (0x93C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xED), (0x5B58)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xEE), (0x5C0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xEF), (0x5352)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF0), (0x62D9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF1), (0x731D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF2), (0x5027)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF3), (0x5B97)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF4), (0x5F9E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF5), (0x60B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF6), (0x616B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF7), (0x68D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF8), (0x6DD9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xF9), (0x742E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xFA), (0x7A2E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xFB), (0x7D42)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xFC), (0x7D9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xFD), (0x7E31)),  # CJK UNIFIED IDEOGRAPH
    ((0xF0, 0xFE), (0x816B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA1), (0x8E2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA2), (0x8E35)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA3), (0x937E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA4), (0x9418)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA5), (0x4F50)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA6), (0x5750)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA7), (0x5DE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA8), (0x5EA7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xA9), (0x632B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAA), (0x7F6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAB), (0x4E3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAC), (0x4F4F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAD), (0x4F8F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAE), (0x505A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xAF), (0x59DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB0), (0x80C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB1), (0x546A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB2), (0x5468)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB3), (0x55FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB4), (0x594F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB5), (0x5B99)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB6), (0x5DDE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB7), (0x5EDA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB8), (0x665D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xB9), (0x6731)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBA), (0x67F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBB), (0x682A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBC), (0x6CE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBD), (0x6D32)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBE), (0x6E4A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xBF), (0x6F8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC0), (0x70B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC1), (0x73E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC2), (0x7587)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC3), (0x7C4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC4), (0x7D02)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC5), (0x7D2C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC6), (0x7DA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC7), (0x821F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC8), (0x86DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xC9), (0x8A3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCA), (0x8A85)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCB), (0x8D70)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCC), (0x8E8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCD), (0x8F33)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCE), (0x9031)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xCF), (0x914E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD0), (0x9152)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD1), (0x9444)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD2), (0x99D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD3), (0x7AF9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD4), (0x7CA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD5), (0x4FCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD6), (0x5101)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD7), (0x51C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD8), (0x57C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xD9), (0x5BEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDA), (0x5CFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDB), (0x6659)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDC), (0x6A3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDD), (0x6D5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDE), (0x6E96)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xDF), (0x6FEC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE0), (0x710C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE1), (0x756F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE2), (0x7AE3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE3), (0x8822)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE4), (0x9021)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE5), (0x9075)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE6), (0x96CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE7), (0x99FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE8), (0x8301)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xE9), (0x4E2D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xEA), (0x4EF2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xEB), (0x8846)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xEC), (0x91CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xED), (0x537D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xEE), (0x6ADB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xEF), (0x696B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF0), (0x6C41)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF1), (0x847A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF2), (0x589E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF3), (0x618E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF4), (0x66FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF5), (0x62EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF6), (0x70DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF7), (0x7511)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF8), (0x75C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xF9), (0x7E52)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xFA), (0x84B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xFB), (0x8B49)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xFC), (0x8D08)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xFD), (0x4E4B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF1, 0xFE), (0x53EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA1), (0x54AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA2), (0x5730)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA3), (0x5740)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA4), (0x5FD7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA5), (0x6301)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA6), (0x6307)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA7), (0x646F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA8), (0x652F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xA9), (0x65E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAA), (0x667A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAB), (0x679D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAC), (0x67B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAD), (0x6B62)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAE), (0x6C60)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xAF), (0x6C9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB0), (0x6F2C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB1), (0x77E5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB2), (0x7825)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB3), (0x7949)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB4), (0x7957)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB5), (0x7D19)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB6), (0x80A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB7), (0x8102)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB8), (0x81F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xB9), (0x829D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xBA), (0x82B7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xBB), (0x8718)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xBC), (0x8A8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xBD), (0xF9FC)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF2, 0xBE), (0x8D04)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xBF), (0x8DBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC0), (0x9072)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC1), (0x76F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC2), (0x7A19)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC3), (0x7A37)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC4), (0x7E54)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC5), (0x8077)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC6), (0x5507)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC7), (0x55D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC8), (0x5875)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xC9), (0x632F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCA), (0x6422)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCB), (0x6649)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCC), (0x664B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCD), (0x686D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCE), (0x699B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xCF), (0x6B84)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD0), (0x6D25)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD1), (0x6EB1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD2), (0x73CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD3), (0x7468)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD4), (0x74A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD5), (0x755B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD6), (0x75B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD7), (0x76E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD8), (0x771E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xD9), (0x778B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDA), (0x79E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDB), (0x7E09)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDC), (0x7E1D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDD), (0x81FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDE), (0x852F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xDF), (0x8897)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE0), (0x8A3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE1), (0x8CD1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE2), (0x8EEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE3), (0x8FB0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE4), (0x9032)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE5), (0x93AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE6), (0x9663)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE7), (0x9673)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE8), (0x9707)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xE9), (0x4F84)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xEA), (0x53F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xEB), (0x59EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xEC), (0x5AC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xED), (0x5E19)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xEE), (0x684E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xEF), (0x74C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF0), (0x75BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF1), (0x79E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF2), (0x7A92)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF3), (0x81A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF4), (0x86ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF5), (0x8CEA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF6), (0x8DCC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF7), (0x8FED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF8), (0x659F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xF9), (0x6715)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xFA), (0xF9FD)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF2, 0xFB), (0x57F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xFC), (0x6F57)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xFD), (0x7DDD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF2, 0xFE), (0x8F2F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA1), (0x93F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA2), (0x96C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA3), (0x5FB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA4), (0x61F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA5), (0x6F84)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA6), (0x4E14)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA7), (0x4F98)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA8), (0x501F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xA9), (0x53C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAA), (0x55DF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAB), (0x5D6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAC), (0x5DEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAD), (0x6B21)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAE), (0x6B64)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xAF), (0x78CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB0), (0x7B9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB1), (0xF9FE)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF3, 0xB2), (0x8E49)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB3), (0x8ECA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB4), (0x906E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB5), (0x6349)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB6), (0x643E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB7), (0x7740)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB8), (0x7A84)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xB9), (0x932F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBA), (0x947F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBB), (0x9F6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBC), (0x64B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBD), (0x6FAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBE), (0x71E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xBF), (0x74A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC0), (0x74DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC1), (0x7AC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC2), (0x7C12)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC3), (0x7E82)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC4), (0x7CB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC5), (0x7E98)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC6), (0x8B9A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC7), (0x8D0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC8), (0x947D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xC9), (0x9910)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCA), (0x994C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCB), (0x5239)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCC), (0x5BDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCD), (0x64E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCE), (0x672D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xCF), (0x7D2E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD0), (0x50ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD1), (0x53C3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD2), (0x5879)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD3), (0x6158)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD4), (0x6159)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD5), (0x61FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD6), (0x65AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD7), (0x7AD9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD8), (0x8B92)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xD9), (0x8B96)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDA), (0x5009)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDB), (0x5021)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDC), (0x5275)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDD), (0x5531)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDE), (0x5A3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xDF), (0x5EE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE0), (0x5F70)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE1), (0x6134)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE2), (0x655E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE3), (0x660C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE4), (0x6636)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE5), (0x66A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE6), (0x69CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE7), (0x6EC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE8), (0x6F32)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xE9), (0x7316)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xEA), (0x7621)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xEB), (0x7A93)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xEC), (0x8139)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xED), (0x8259)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xEE), (0x83D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xEF), (0x84BC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF0), (0x50B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF1), (0x57F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF2), (0x5BC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF3), (0x5BE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF4), (0x5F69)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF5), (0x63A1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF6), (0x7826)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF7), (0x7DB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF8), (0x83DC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xF9), (0x8521)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xFA), (0x91C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xFB), (0x91F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xFC), (0x518A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xFD), (0x67F5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF3, 0xFE), (0x7B56)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA1), (0x8CAC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA2), (0x51C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA3), (0x59BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA4), (0x60BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA5), (0x8655)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA6), (0x501C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA7), (0xF9FF)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF4, 0xA8), (0x5254)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xA9), (0x5C3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAA), (0x617D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAB), (0x621A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAC), (0x62D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAD), (0x64F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAE), (0x65A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xAF), (0x6ECC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB0), (0x7620)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB1), (0x810A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB2), (0x8E60)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB3), (0x965F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB4), (0x96BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB5), (0x4EDF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB6), (0x5343)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB7), (0x5598)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB8), (0x5929)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xB9), (0x5DDD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBA), (0x64C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBB), (0x6CC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBC), (0x6DFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBD), (0x7394)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBE), (0x7A7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xBF), (0x821B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC0), (0x85A6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC1), (0x8CE4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC2), (0x8E10)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC3), (0x9077)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC4), (0x91E7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC5), (0x95E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC6), (0x9621)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC7), (0x97C6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC8), (0x51F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xC9), (0x54F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCA), (0x5586)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCB), (0x5FB9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCC), (0x64A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCD), (0x6F88)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCE), (0x7DB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xCF), (0x8F1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD0), (0x8F4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD1), (0x9435)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD2), (0x50C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD3), (0x5C16)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD4), (0x6CBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD5), (0x6DFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD6), (0x751B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD7), (0x77BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD8), (0x7C3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xD9), (0x7C64)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDA), (0x8A79)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDB), (0x8AC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDC), (0x581E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDD), (0x59BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDE), (0x5E16)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xDF), (0x6377)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE0), (0x7252)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE1), (0x758A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE2), (0x776B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE3), (0x8ADC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE4), (0x8CBC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE5), (0x8F12)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE6), (0x5EF3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE7), (0x6674)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE8), (0x6DF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xE9), (0x807D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xEA), (0x83C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xEB), (0x8ACB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xEC), (0x9751)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xED), (0x9BD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xEE), (0xFA00)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF4, 0xEF), (0x5243)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF0), (0x66FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF1), (0x6D95)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF2), (0x6EEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF3), (0x7DE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF4), (0x8AE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF5), (0x902E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF6), (0x905E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF7), (0x9AD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF8), (0x521D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xF9), (0x527F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xFA), (0x54E8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xFB), (0x6194)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xFC), (0x6284)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xFD), (0x62DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF4, 0xFE), (0x68A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA1), (0x6912)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA2), (0x695A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA3), (0x6A35)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA4), (0x7092)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA5), (0x7126)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA6), (0x785D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA7), (0x7901)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA8), (0x790E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xA9), (0x79D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAA), (0x7A0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAB), (0x8096)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAC), (0x8278)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAD), (0x82D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAE), (0x8349)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xAF), (0x8549)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB0), (0x8C82)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB1), (0x8D85)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB2), (0x9162)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB3), (0x918B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB4), (0x91AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB5), (0x4FC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB6), (0x56D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB7), (0x71ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB8), (0x77D7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xB9), (0x8700)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBA), (0x89F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBB), (0x5BF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBC), (0x5FD6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBD), (0x6751)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBE), (0x90A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xBF), (0x53E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC0), (0x585A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC1), (0x5BF5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC2), (0x60A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC3), (0x6181)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC4), (0x6460)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC5), (0x7E3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC6), (0x8070)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC7), (0x8525)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC8), (0x9283)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xC9), (0x64AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCA), (0x50AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCB), (0x5D14)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCC), (0x6700)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCD), (0x589C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCE), (0x62BD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xCF), (0x63A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD0), (0x690E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD1), (0x6978)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD2), (0x6A1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD3), (0x6E6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD4), (0x76BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD5), (0x79CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD6), (0x82BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD7), (0x8429)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD8), (0x8ACF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xD9), (0x8DA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDA), (0x8FFD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDB), (0x9112)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDC), (0x914B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDD), (0x919C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDE), (0x9310)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xDF), (0x9318)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE0), (0x939A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE1), (0x96DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE2), (0x9A36)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE3), (0x9C0D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE4), (0x4E11)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE5), (0x755C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE6), (0x795D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE7), (0x7AFA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE8), (0x7B51)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xE9), (0x7BC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xEA), (0x7E2E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xEB), (0x84C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xEC), (0x8E59)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xED), (0x8E74)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xEE), (0x8EF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xEF), (0x9010)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF0), (0x6625)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF1), (0x693F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF2), (0x7443)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF3), (0x51FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF4), (0x672E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF5), (0x9EDC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF6), (0x5145)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF7), (0x5FE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF8), (0x6C96)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xF9), (0x87F2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xFA), (0x885D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xFB), (0x8877)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xFC), (0x60B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xFD), (0x81B5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF5, 0xFE), (0x8403)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA1), (0x8D05)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA2), (0x53D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA3), (0x5439)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA4), (0x5634)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA5), (0x5A36)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA6), (0x5C31)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA7), (0x708A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA8), (0x7FE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xA9), (0x805A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAA), (0x8106)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAB), (0x81ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAC), (0x8DA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAD), (0x9189)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAE), (0x9A5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xAF), (0x9DF2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB0), (0x5074)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB1), (0x4EC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB2), (0x53A0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB3), (0x60FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB4), (0x6E2C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB5), (0x5C64)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB6), (0x4F88)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB7), (0x5024)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB8), (0x55E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xB9), (0x5CD9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBA), (0x5E5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBB), (0x6065)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBC), (0x6894)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBD), (0x6CBB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBE), (0x6DC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xBF), (0x71BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC0), (0x75D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC1), (0x75F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC2), (0x7661)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC3), (0x7A1A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC4), (0x7A49)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC5), (0x7DC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC6), (0x7DFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC7), (0x7F6E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC8), (0x81F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xC9), (0x86A9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCA), (0x8F1C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCB), (0x96C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCC), (0x99B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCD), (0x9F52)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCE), (0x5247)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xCF), (0x52C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD0), (0x98ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD1), (0x89AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD2), (0x4E03)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD3), (0x67D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD4), (0x6F06)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD5), (0x4FB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD6), (0x5BE2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD7), (0x6795)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD8), (0x6C88)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xD9), (0x6D78)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDA), (0x741B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDB), (0x7827)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDC), (0x91DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDD), (0x937C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDE), (0x87C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xDF), (0x79E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE0), (0x7A31)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE1), (0x5FEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE2), (0x4ED6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE3), (0x54A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE4), (0x553E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE5), (0x58AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE6), (0x59A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE7), (0x60F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE8), (0x6253)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xE9), (0x62D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xEA), (0x6736)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xEB), (0x6955)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xEC), (0x8235)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xED), (0x9640)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xEE), (0x99B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xEF), (0x99DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF0), (0x502C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF1), (0x5353)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF2), (0x5544)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF3), (0x577C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF4), (0xFA01)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF6, 0xF5), (0x6258)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF6), (0xFA02)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF6, 0xF7), (0x64E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF8), (0x666B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xF9), (0x67DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xFA), (0x6FC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xFB), (0x6FEF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xFC), (0x7422)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xFD), (0x7438)),  # CJK UNIFIED IDEOGRAPH
    ((0xF6, 0xFE), (0x8A17)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA1), (0x9438)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA2), (0x5451)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA3), (0x5606)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA4), (0x5766)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA5), (0x5F48)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA6), (0x619A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA7), (0x6B4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA8), (0x7058)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xA9), (0x70AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAA), (0x7DBB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAB), (0x8A95)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAC), (0x596A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAD), (0x812B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAE), (0x63A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xAF), (0x7708)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB0), (0x803D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB1), (0x8CAA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB2), (0x5854)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB3), (0x642D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB4), (0x69BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB5), (0x5B95)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB6), (0x5E11)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB7), (0x6E6F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xB8), (0xFA03)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF7, 0xB9), (0x8569)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBA), (0x514C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBB), (0x53F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBC), (0x592A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBD), (0x6020)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBE), (0x614B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xBF), (0x6B86)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC0), (0x6C70)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC1), (0x6CF0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC2), (0x7B1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC3), (0x80CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC4), (0x82D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC5), (0x8DC6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC6), (0x90B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC7), (0x98B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xC8), (0xFA04)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF7, 0xC9), (0x64C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCA), (0x6FA4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCB), (0x6491)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCC), (0x6504)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCD), (0x514E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCE), (0x5410)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xCF), (0x571F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD0), (0x8A0E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD1), (0x615F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD2), (0x6876)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD3), (0xFA05)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF7, 0xD4), (0x75DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD5), (0x7B52)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD6), (0x7D71)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD7), (0x901A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD8), (0x5806)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xD9), (0x69CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDA), (0x817F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDB), (0x892A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDC), (0x9000)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDD), (0x9839)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDE), (0x5078)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xDF), (0x5957)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE0), (0x59AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE1), (0x6295)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE2), (0x900F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE3), (0x9B2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE4), (0x615D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE5), (0x7279)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE6), (0x95D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE7), (0x5761)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE8), (0x5A46)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xE9), (0x5DF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xEA), (0x628A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xEB), (0x64AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xEC), (0x64FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xED), (0x6777)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xEE), (0x6CE2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xEF), (0x6D3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF0), (0x722C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF1), (0x7436)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF2), (0x7834)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF3), (0x7F77)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF4), (0x82AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF5), (0x8DDB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF6), (0x9817)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF7), (0x5224)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF8), (0x5742)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xF9), (0x677F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xFA), (0x7248)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xFB), (0x74E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xFC), (0x8CA9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xFD), (0x8FA6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF7, 0xFE), (0x9211)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA1), (0x962A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA2), (0x516B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA3), (0x53ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA4), (0x634C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA5), (0x4F69)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA6), (0x5504)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA7), (0x6096)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA8), (0x6557)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xA9), (0x6C9B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAA), (0x6D7F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAB), (0x724C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAC), (0x72FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAD), (0x7A17)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAE), (0x8987)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xAF), (0x8C9D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB0), (0x5F6D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB1), (0x6F8E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB2), (0x70F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB3), (0x81A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB4), (0x610E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB5), (0x4FBF)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB6), (0x504F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB7), (0x6241)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB8), (0x7247)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xB9), (0x7BC7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBA), (0x7DE8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBB), (0x7FE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBC), (0x904D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBD), (0x97AD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBE), (0x9A19)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xBF), (0x8CB6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC0), (0x576A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC1), (0x5E73)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC2), (0x67B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC3), (0x840D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC4), (0x8A55)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC5), (0x5420)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC6), (0x5B16)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC7), (0x5E63)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC8), (0x5EE2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xC9), (0x5F0A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCA), (0x6583)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCB), (0x80BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCC), (0x853D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCD), (0x9589)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCE), (0x965B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xCF), (0x4F48)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD0), (0x5305)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD1), (0x530D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD2), (0x530F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD3), (0x5486)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD4), (0x54FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD5), (0x5703)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD6), (0x5E03)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD7), (0x6016)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD8), (0x629B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xD9), (0x62B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xDA), (0x6355)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xDB), (0xFA06)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF8, 0xDC), (0x6CE1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xDD), (0x6D66)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xDE), (0x75B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xDF), (0x7832)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE0), (0x80DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE1), (0x812F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE2), (0x82DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE3), (0x8461)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE4), (0x84B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE5), (0x888D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE6), (0x8912)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE7), (0x900B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE8), (0x92EA)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xE9), (0x98FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xEA), (0x9B91)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xEB), (0x5E45)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xEC), (0x66B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xED), (0x66DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xEE), (0x7011)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xEF), (0x7206)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF0), (0xFA07)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xF8, 0xF1), (0x4FF5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF2), (0x527D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF3), (0x5F6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF4), (0x6153)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF5), (0x6753)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF6), (0x6A19)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF7), (0x6F02)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF8), (0x74E2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xF9), (0x7968)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xFA), (0x8868)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xFB), (0x8C79)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xFC), (0x98C7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xFD), (0x98C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF8, 0xFE), (0x9A43)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA1), (0x54C1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA2), (0x7A1F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA3), (0x6953)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA4), (0x8AF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA5), (0x8C4A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA6), (0x98A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA7), (0x99AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA8), (0x5F7C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xA9), (0x62AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAA), (0x75B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAB), (0x76AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAC), (0x88AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAD), (0x907F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAE), (0x9642)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xAF), (0x5339)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB0), (0x5F3C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB1), (0x5FC5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB2), (0x6CCC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB3), (0x73CC)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB4), (0x7562)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB5), (0x758B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB6), (0x7B46)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB7), (0x82FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB8), (0x999D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xB9), (0x4E4F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBA), (0x903C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBB), (0x4E0B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBC), (0x4F55)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBD), (0x53A6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBE), (0x590F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xBF), (0x5EC8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC0), (0x6630)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC1), (0x6CB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC2), (0x7455)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC3), (0x8377)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC4), (0x8766)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC5), (0x8CC0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC6), (0x9050)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC7), (0x971E)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC8), (0x9C15)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xC9), (0x58D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCA), (0x5B78)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCB), (0x8650)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCC), (0x8B14)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCD), (0x9DB4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCE), (0x5BD2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xCF), (0x6068)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD0), (0x608D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD1), (0x65F1)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD2), (0x6C57)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD3), (0x6F22)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD4), (0x6FA3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD5), (0x701A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD6), (0x7F55)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD7), (0x7FF0)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD8), (0x9591)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xD9), (0x9592)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDA), (0x9650)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDB), (0x97D3)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDC), (0x5272)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDD), (0x8F44)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDE), (0x51FD)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xDF), (0x542B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE0), (0x54B8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE1), (0x5563)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE2), (0x558A)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE3), (0x6ABB)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE4), (0x6DB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE5), (0x7DD8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE6), (0x8266)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE7), (0x929C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE8), (0x9677)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xE9), (0x9E79)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xEA), (0x5408)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xEB), (0x54C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xEC), (0x76D2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xED), (0x86E4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xEE), (0x95A4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xEF), (0x95D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF0), (0x965C)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF1), (0x4EA2)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF2), (0x4F09)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF3), (0x59EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF4), (0x5AE6)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF5), (0x5DF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF6), (0x6052)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF7), (0x6297)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF8), (0x676D)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xF9), (0x6841)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xFA), (0x6C86)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xFB), (0x6E2F)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xFC), (0x7F38)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xFD), (0x809B)),  # CJK UNIFIED IDEOGRAPH
    ((0xF9, 0xFE), (0x822A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA1), (0xFA08)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xFA, 0xA2), (0xFA09)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xFA, 0xA3), (0x9805)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA4), (0x4EA5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA5), (0x5055)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA6), (0x54B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA7), (0x5793)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA8), (0x595A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xA9), (0x5B69)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAA), (0x5BB3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAB), (0x61C8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAC), (0x6977)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAD), (0x6D77)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAE), (0x7023)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xAF), (0x87F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB0), (0x89E3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB1), (0x8A72)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB2), (0x8AE7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB3), (0x9082)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB4), (0x99ED)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB5), (0x9AB8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB6), (0x52BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB7), (0x6838)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB8), (0x5016)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xB9), (0x5E78)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBA), (0x674F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBB), (0x8347)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBC), (0x884C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBD), (0x4EAB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBE), (0x5411)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xBF), (0x56AE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC0), (0x73E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC1), (0x9115)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC2), (0x97FF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC3), (0x9909)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC4), (0x9957)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC5), (0x9999)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC6), (0x5653)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC7), (0x589F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC8), (0x865B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xC9), (0x8A31)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCA), (0x61B2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCB), (0x6AF6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCC), (0x737B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCD), (0x8ED2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCE), (0x6B47)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xCF), (0x96AA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD0), (0x9A57)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD1), (0x5955)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD2), (0x7200)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD3), (0x8D6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD4), (0x9769)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD5), (0x4FD4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD6), (0x5CF4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD7), (0x5F26)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD8), (0x61F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xD9), (0x665B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDA), (0x6CEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDB), (0x70AB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDC), (0x7384)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDD), (0x73B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDE), (0x73FE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xDF), (0x7729)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE0), (0x774D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE1), (0x7D43)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE2), (0x7D62)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE3), (0x7E23)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE4), (0x8237)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE5), (0x8852)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE6), (0xFA0A)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xFA, 0xE7), (0x8CE2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE8), (0x9249)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xE9), (0x986F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xEA), (0x5B51)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xEB), (0x7A74)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xEC), (0x8840)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xED), (0x9801)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xEE), (0x5ACC)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xEF), (0x4FE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF0), (0x5354)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF1), (0x593E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF2), (0x5CFD)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF3), (0x633E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF4), (0x6D79)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF5), (0x72F9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF6), (0x8105)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF7), (0x8107)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF8), (0x83A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xF9), (0x92CF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xFA), (0x9830)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xFB), (0x4EA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xFC), (0x5144)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xFD), (0x5211)),  # CJK UNIFIED IDEOGRAPH
    ((0xFA, 0xFE), (0x578B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA1), (0x5F62)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA2), (0x6CC2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA3), (0x6ECE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA4), (0x7005)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA5), (0x7050)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA6), (0x70AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA7), (0x7192)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA8), (0x73E9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xA9), (0x7469)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAA), (0x834A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAB), (0x87A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAC), (0x8861)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAD), (0x9008)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAE), (0x90A2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xAF), (0x93A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB0), (0x99A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB1), (0x516E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB2), (0x5F57)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB3), (0x60E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB4), (0x6167)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB5), (0x66B3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB6), (0x8559)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB7), (0x8E4A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB8), (0x91AF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xB9), (0x978B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBA), (0x4E4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBB), (0x4E92)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBC), (0x547C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBD), (0x58D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBE), (0x58FA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xBF), (0x597D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC0), (0x5CB5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC1), (0x5F27)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC2), (0x6236)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC3), (0x6248)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC4), (0x660A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC5), (0x6667)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC6), (0x6BEB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC7), (0x6D69)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC8), (0x6DCF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xC9), (0x6E56)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCA), (0x6EF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCB), (0x6F94)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCC), (0x6FE0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCD), (0x6FE9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCE), (0x705D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xCF), (0x72D0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD0), (0x7425)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD1), (0x745A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD2), (0x74E0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD3), (0x7693)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD4), (0x795C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD5), (0x7CCA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD6), (0x7E1E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD7), (0x80E1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD8), (0x82A6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xD9), (0x846B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDA), (0x84BF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDB), (0x864E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDC), (0x865F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDD), (0x8774)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDE), (0x8B77)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xDF), (0x8C6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE0), (0x93AC)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE1), (0x9800)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE2), (0x9865)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE3), (0x60D1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE4), (0x6216)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE5), (0x9177)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE6), (0x5A5A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE7), (0x660F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE8), (0x6DF7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xE9), (0x6E3E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xEA), (0x743F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xEB), (0x9B42)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xEC), (0x5FFD)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xED), (0x60DA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xEE), (0x7B0F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xEF), (0x54C4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF0), (0x5F18)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF1), (0x6C5E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF2), (0x6CD3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF3), (0x6D2A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF4), (0x70D8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF5), (0x7D05)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF6), (0x8679)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF7), (0x8A0C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF8), (0x9D3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xF9), (0x5316)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xFA), (0x548C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xFB), (0x5B05)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xFC), (0x6A3A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xFD), (0x706B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFB, 0xFE), (0x7575)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA1), (0x798D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA2), (0x79BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA3), (0x82B1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA4), (0x83EF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA5), (0x8A71)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA6), (0x8B41)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA7), (0x8CA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA8), (0x9774)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xA9), (0xFA0B)),  # CJK COMPATIBILITY IDEOGRAPH
    ((0xFC, 0xAA), (0x64F4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xAB), (0x652B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xAC), (0x78BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xAD), (0x78BB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xAE), (0x7A6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xAF), (0x4E38)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB0), (0x559A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB1), (0x5950)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB2), (0x5BA6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB3), (0x5E7B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB4), (0x60A3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB5), (0x63DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB6), (0x6B61)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB7), (0x6665)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB8), (0x6853)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xB9), (0x6E19)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBA), (0x7165)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBB), (0x74B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBC), (0x7D08)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBD), (0x9084)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBE), (0x9A69)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xBF), (0x9C25)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC0), (0x6D3B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC1), (0x6ED1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC2), (0x733E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC3), (0x8C41)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC4), (0x95CA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC5), (0x51F0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC6), (0x5E4C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC7), (0x5FA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC8), (0x604D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xC9), (0x60F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCA), (0x6130)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCB), (0x614C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCC), (0x6643)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCD), (0x6644)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCE), (0x69A5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xCF), (0x6CC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD0), (0x6E5F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD1), (0x6EC9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD2), (0x6F62)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD3), (0x714C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD4), (0x749C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD5), (0x7687)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD6), (0x7BC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD7), (0x7C27)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD8), (0x8352)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xD9), (0x8757)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDA), (0x9051)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDB), (0x968D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDC), (0x9EC3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDD), (0x532F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDE), (0x56DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xDF), (0x5EFB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE0), (0x5F8A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE1), (0x6062)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE2), (0x6094)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE3), (0x61F7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE4), (0x6666)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE5), (0x6703)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE6), (0x6A9C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE7), (0x6DEE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE8), (0x6FAE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xE9), (0x7070)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xEA), (0x736A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xEB), (0x7E6A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xEC), (0x81BE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xED), (0x8334)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xEE), (0x86D4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xEF), (0x8AA8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF0), (0x8CC4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF1), (0x5283)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF2), (0x7372)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF3), (0x5B96)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF4), (0x6A6B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF5), (0x9404)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF6), (0x54EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF7), (0x5686)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF8), (0x5B5D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xF9), (0x6548)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xFA), (0x6585)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xFB), (0x66C9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xFC), (0x689F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xFD), (0x6D8D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFC, 0xFE), (0x6DC6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA1), (0x723B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA2), (0x80B4)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA3), (0x9175)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA4), (0x9A4D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA5), (0x4FAF)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA6), (0x5019)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA7), (0x539A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA8), (0x540E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xA9), (0x543C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAA), (0x5589)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAB), (0x55C5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAC), (0x5E3F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAD), (0x5F8C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAE), (0x673D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xAF), (0x7166)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB0), (0x73DD)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB1), (0x9005)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB2), (0x52DB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB3), (0x52F3)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB4), (0x5864)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB5), (0x58CE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB6), (0x7104)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB7), (0x718F)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB8), (0x71FB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xB9), (0x85B0)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBA), (0x8A13)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBB), (0x6688)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBC), (0x85A8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBD), (0x55A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBE), (0x6684)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xBF), (0x714A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC0), (0x8431)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC1), (0x5349)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC2), (0x5599)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC3), (0x6BC1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC4), (0x5F59)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC5), (0x5FBD)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC6), (0x63EE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC7), (0x6689)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC8), (0x7147)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xC9), (0x8AF1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCA), (0x8F1D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCB), (0x9EBE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCC), (0x4F11)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCD), (0x643A)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCE), (0x70CB)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xCF), (0x7566)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD0), (0x8667)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD1), (0x6064)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD2), (0x8B4E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD3), (0x9DF8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD4), (0x5147)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD5), (0x51F6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD6), (0x5308)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD7), (0x6D36)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD8), (0x80F8)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xD9), (0x9ED1)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDA), (0x6615)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDB), (0x6B23)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDC), (0x7098)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDD), (0x75D5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDE), (0x5403)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xDF), (0x5C79)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE0), (0x7D07)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE1), (0x8A16)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE2), (0x6B20)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE3), (0x6B3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE4), (0x6B46)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE5), (0x5438)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE6), (0x6070)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE7), (0x6D3D)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE8), (0x7FD5)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xE9), (0x8208)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xEA), (0x50D6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xEB), (0x51DE)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xEC), (0x559C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xED), (0x566B)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xEE), (0x56CD)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xEF), (0x59EC)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF0), (0x5B09)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF1), (0x5E0C)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF2), (0x6199)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF3), (0x6198)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF4), (0x6231)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF5), (0x665E)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF6), (0x66E6)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF7), (0x7199)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF8), (0x71B9)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xF9), (0x71BA)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xFA), (0x72A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xFB), (0x79A7)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xFC), (0x7A00)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xFD), (0x7FB2)),  # CJK UNIFIED IDEOGRAPH
    ((0xFD, 0xFE), (0x8A70)),  # CJK UNIFIED IDEOGRAPH
]
